# 优化生成文件名的效率

## 新旧代码对比

旧代码在执行 `getFileName` 方法时，每次都会在其内部创建一个函数表达式：

```js
function getFileName () {
  // ...
  const generateFileName = (rule) => {...}
  // ...
}
```

新代码把这个函数表达式改为在函数外的函数声明，直接调用：

```js
function generateFileName (rule, cfg) {
  // ...
}

function getFileName () {
  // ...
  generateFileName(rule, cfg)
  // ...
}
```

在新代码里，`generateFileName` 已经事先声明好了，在 `getFileName` 里面直接调用即可。

这样能提高多少效率呢？

## 测试

抓取 `初音ミク` tag 1000 页，约 60000 个作品，共 123317 条数据。

遍历生成文件名：

```js
console.time('filename')
for (const data of store.result) {
  fileName.getFileName(data)
}
console.timeEnd('filename')
```

多次执行此代码。

旧代码花费的时间平均在 2061 ms 左右：

```
1996.89892578125 ms
2080.978759765625 ms
2128.549072265625 ms
2034.5361328125 ms
2069.7451171875 ms
```

新代码花费的时间平均在 1864 ms 左右：

```
1919.650146484375 ms
1847.028076171875 ms
1852.77880859375 ms
1865.02099609375 ms
1837.669189453125 ms
```

遍历 12 万条数据生成文件名，新代码比旧代码节约了 200 ms。大概节约了 10% 的时间。

考虑到生成文件名时的运算比较多，这次的改动只是其中的一小部分，所以提升 10% 的效率也可以令人满意了。

当然，生成文件名本来花费的时间也就不多，这点优化聊胜于无。
