# 优化转换动图的性能

## 两个测试样本

样本1：（体积稍小）https://www.pixiv.net/artworks/99831960

zip 源文件体积：7.36MB

样本2：（体积较大）https://www.pixiv.net/artworks/99829091

zip 源文件体积：20.50MB

## 缩短提取图片数据的时间

以前下载器使用 zip.js 库从 zip 文件里提取图片数据。它返回的数据是 base64URL 字符串。

zip.js 库是一个泛用库，它不能针对 Pixiv 的压缩包进行优化。

我在制作预览动图的功能时，实现了从 zip 文件里提取图片数据的方法，这是专门对 Pixiv 的压缩包编写的，所以速度更快。

我用自己编写的方法替换了 zip.js，节约了提取图片所需的时间。

旧版本的提取图片所需时间：（多次测试取大概的平均值）
- 样本1：1000 ms
- 样本2：1900 ms

优化之后：
- 样本1：170 ms
- 样本2：800 ms

## 缩短动图转换为 webm 的时间

之前在编码 webm 时，我传递给编码器的数据是绘制了图片的 Canvas 对象。但是这会导致编码器做一些重复工作，浪费了时间。

现在我把图像转换为 webp 格式的 DataURL，然后传递给编码器。对于现在使用的编码器来说，这样是最快的。

结合上面“缩短提取图片数据的时间”的优化，转换动图所需的时间有明显的减少。

转换时间指的是：从动图的 zip 文件下载完成之后开始，截止到动图转换完毕的时间。

旧版本的转换时间：（掐秒表，多次测试后的平均值）
- 样本1：5 s
- 样本2：29 s

优化之后：
- 样本1：3.6 s
- 样本2：22 s

## 减少了动图转换为 webm 的内存使用量

这里仅测试样本 2，因为它的体积比较大。

之前在编码 webm 时，由于编码器做了一些重复工作，内存使用量会略多一些。

而且旧版本有个很大的问题，在编码时间过去大约一半时（十几秒之后），会有数秒钟的时间段，内存占用会迅速多出一个 GB，令人震惊。等这个动图下载完毕后，内存才会回落。

新版本没有这个问题，也就是内存使用量不会突然大幅增加。在整个转换过程中，内存的增加量只有 200 MB 左右。

旧版本瞬间占用大量内存的问题，无疑会影响系统整体性能，也会影响这个网页的稳定性。有些用户在下载大量动图时，页面偶尔会崩溃，新版本里这样的异常情况应该会明显减少。

## 优化了动图转换为 apng 的代码

仅仅优化了代码。没有明显的性能提升。

## 缩短动图转换为 gif 的时间

优化了转换为 gif 的代码，并且转换时间也缩短了一些。

旧版本的转换时间：
- 样本1：6.6 s
- 样本2：42 s

优化之后：
- 样本1：5.5 s
- 样本2：39 s

