# 对移动端浏览器进行优化

下载器是基于 pixiv 的桌面端网站开发的，一些功能并没有对移动端页面进行优化。现在打算优化一下。

## 通过 UA 判断是否处于移动端模式

Kiwi 浏览器移动端的 UA 是这样的：

```
Mozilla/5.0 (Linux; Android 10; MI 8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Mobile Safari/537.36
```

切换为桌面版网站时是这样的：

```
Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36
```

两个主要区别：
1. 移动端的操作系统是 Android，桌面端是 Linux
2. 移动端有 Mobile 标志，桌面端没有

可以通过 `navigator.userAgent.includes('Mobile')` 判断是否是移动端，并且 Chrome 浏览器在切换为移动端调试时也有这个标记，例如：

```
Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Mobile Safari/537.36
```

## 优化下载器一些面板的宽度

之前下载器面板宽度在竖屏时通常是超出屏幕宽度的，难以使用。

现在适配了宽度。包括主面板、消息提示面板、输出面板、轻提示。

## 识别主题颜色

移动端的主题颜色是 body 上的 class，如果含有 dark 就是夜间模式，如果没有 dark 就是白色模式。

现在进行了适配。

## 在作品页面内，点击收藏按钮、点赞按钮时下载作品

之前在移动端页面点击收藏按钮、点赞按钮时，下载器无法监听到对应事件，这是因为下载器压根没有获取到对应元素。

现在我在 `WorkToolBar` 类里添加 `getElementsOnMobile` 方法，处理移动端的工具栏按钮，使这两个功能正常了。

## 快速收藏按钮及联动效果

修复了在移动端快速收藏按钮功能不正常的问题，以及导致其他相关按钮样式或功能异常的问题。

## 点击作品缩略图里的收藏按钮时下载作品

移动端的作品缩略图选择器以及收藏按钮选择器都与桌面端不同。现在适配了，所以此功能已经修复。

## 在缩略图上显示下载按钮

之前移动端的“在缩略图右上角显示下载按钮”功能无效，因为按钮是当鼠标移动到缩略图上时才会显示。但是移动端没有鼠标移动事件，导致按钮始终不会显示。

现在为每个作品都添加了一个始终显示的缩略图，解决了此问题。

## 手动选择作品

之前手动选择作品在移动端存在异常，某些作品的缩略图元素可能无法选择，而且点击到作品缩略图时会进入作品，导致页面跳转，这使得此功能几乎无法使用。

现在修复了此问题，可以选择所有作品了。

至于页面跳转的问题，之前是对作品缩略图绑定 `click` 事件，配合 `ev.preventDefault()`，但是阻止默认事件无效。

现在改为使用 `touchend` 事件，可以阻止跳转了。

不过还存在一些小问题，比如选择了一些作品，然后跳转页面，新页面里含有已选择的作品。这些作品应该能够被识别并且显示已选择的对号图标，但是现在有时不会显示。不过这个问题不太大，我也懒得修。

## 获取 token

移动端获取 token 的字符串前缀不同，导致之前获取不到 token，现在修复。

## 提示 Kiwi 浏览器不能创建文件夹的 bug

当下载器识别到是移动端时，会在下载时显示这个提示。

目前我不能精准识别 Kiwi 浏览器，所以其他浏览器的移动端页面可能也会看到这个提示。

## 高亮已关注的用户

之前在移动端不能高亮已关注的用户，原因是此功能需要先获取登录用户的 id，移动端由于网页代码不同没获取到，现在修复。

## 显示更大的缩略图导致右侧一列被隐藏

在移动端的很多页面都是 2 个作品横排，有些页面是 3 个作品横排，已经够大了，所以不需要显示更大的缩略图了。

正好此功能会导致显示出现问题，所以在移动端禁用了此功能。

## 其他小问题


