# 对首页关注的人的新作品板块的研究

下载器启用“显示更大的缩略图”之后，首页关注的人的新作品很容易产生滚动过头的问题。怎么解决这个问题呢？

## 尝试1：缩略图列表

同时最多存在 18 个作品缩略图 div。但是只会加载显示的缩略图，其他在屏幕外的则不会生成 img 标签，直到用户滚动这个区域，让这个作品能看到。

我尝试把每个缩略图 div 的宽度定死，但是没用，因为 Pixiv 一次滚动的数量似乎是固定的。

```js
const thumbDiv = document.querySelectorAll('.hkzusx')[1].querySelectorAll('.bgeGyS')
for(const div of thumbDiv){
  div.style.width='540px'
}
```

设置宽度为 540px 之后，一屏只能显示 4 个缩略图，但是一次滚动还是会滚动很多个。

## 尝试2：左侧的占位元素

.hkzusx>div.gdUpXR

```html
<div class="sc-w2rqc8-1 gdUpXR" style="width: 6968px;"></div>
```

增加它的 width，画面就会被向右推（显示出左侧的内容）

减少它的 width，画面就会被向左拉（显示出右侧的内容）

如果向右滚动过头了，应该增加它的 width，但是这不会让 Pixiv 重新生成本应在左侧的缩略图。比如一次性把 width 减少 2000px，左侧大片空白了，但是 Pixiv 不会自动处理。

## 尝试3：自己创建滚动列表

我设想了一下自己创建滚动列表，感觉这是最接近可行的方案了，但是有个功能不好实现，就是鼠标放在画师名字上之后显示出来的小卡片。

这个小卡片显示后会发出两个请求，一个请求画师详细信息，一个请求画师最近的几个作品的数据。

## API 数据来源

打开首页时，会一次性请求首页所有板块显示的作品。

https://www.pixiv.net/ajax/top/illust?mode=all&lang=zh&version=27fb89088836e5316a96ab5c2e05e588a2ab50fe

数据中的 `body.page.follow` 保存了“关注用户・好P友的作品”列表，最多有 48 个作品的 ID。它只是 ID 列表 `number[]`，每个作品具体的数据保存在 `body.thumbnails.illust`。

具体数据格式如:

```json
{
  "id": "105776811",
  "title": "なぜ体中が熱くなるのか？",
  "illustType": 0,
  "xRestrict": 1,
  "restrict": 0,
  "sl": 6,
  "url": "https://i.pximg.net/c/250x250_80_a2/img-master/img/2023/02/28/01/41/02/105776811_p0_square1200.jpg",
  "description": "",
  "tags": [
      "R-18",
      "原神",
      "神里綾華",
      "魅惑の顔",
      "潮吹き",
      "露出",
      "極上の女体"
  ],
  "userId": "16070561",
  "userName": "Go for it !",
  "width": 2304,
  "height": 3456,
  "pageCount": 2,
  "isBookmarkable": true,
  "bookmarkData": {
      "id": "19643770741",
      "private": false
  },
  "alt": "#原神 なぜ体中が熱くなるのか？ - Go for it !的插画",
  "titleCaptionTranslation": {
      "workTitle": null,
      "workCaption": null
  },
  "createDate": "2023-02-28T01:41:02+09:00",
  "updateDate": "2023-02-28T01:41:02+09:00",
  "isUnlisted": false,
  "isMasked": false,
  "aiType": 2,
  "urls": {
      "250x250": "https://i.pximg.net/c/250x250_80_a2/img-master/img/2023/02/28/01/41/02/105776811_p0_square1200.jpg",
      "360x360": "https://i.pximg.net/c/360x360_70/img-master/img/2023/02/28/01/41/02/105776811_p0_square1200.jpg",
      "540x540": "https://i.pximg.net/c/540x540_70/img-master/img/2023/02/28/01/41/02/105776811_p0_square1200.jpg"
  },
  "profileImageUrl": "https://i.pximg.net/user-profile/img/2022/11/24/13/41/35/23651462_d8cfae27f3c20a7c4bec7fa8d39a8162_50.png"
},
```

