# 页面处于后台时，转换动图的时间

2022/08/31

之前在转换动图时，如果页面被隐藏了，下载器会显示一条提示：

```
这个标签页正在转换动图。如果这个标签页被隐藏了，转换速度可能会变慢。
```

但是现在标签页处于后台时，动图转换速度似乎没有明显的降低，所以我移除了这个提示。

## 分析

之前在转换动图时，如果页面被隐藏了，转换速度会明显变慢（主要是 WebM 格式）。

我的猜测是因为定时器的存在。对于 WebM 视频来说，之前在转换过程中有第三方库的定时器 setTimeout，所以当页面处于后台时，转换速度会降低。

但是后来我直接向编码库传递转换好的 DataURL，它就不会执行定时器了。所以即使页面处于后台，转换速度也不会有明显的增加。

其实还是会增加一点，但是增加的幅度不多，不需要专门用消息框去提醒用户。

下面进行测试，转换单个动图，输出其转换花费的时间。

测试的动图作品：

https://www.pixiv.net/artworks/100875971

21.3 MB，123 张图片，每张分辨率 640*360，177 KB。

一种测试方式是始终处于该标签页，另一种是按下快速下载按钮后，就立即切换到其他页面。

每种方式都至少执行 3 次，取平均值。（测试平台是我的电脑，其他人可能根据性能不同有不同的结果）

## WebM

```js
console.time('webm')
file = await convertUgoira.webm(file, arg.result.ugoiraInfo)
console.timeEnd('webm')
```

- 前台时：6873 ms
- 后台时：7814 ms

可以看到时间增加了约 1 秒，可以接受。

## GIF

```js
console.time('gif')
file = await convertUgoira.gif(file, arg.result.ugoiraInfo)
console.timeEnd('gif')
```

- 前台时：8202 ms
- 后台时：8451 ms

时间几乎没有增加，偶尔还比在前台时更快。

## APNG

```js
console.time('apng')
file = await convertUgoira.apng(file, arg.result.ugoiraInfo)
console.timeEnd('apng')
```

- 前台时：15467 ms
- 后台时：16034 ms

时间增加了约 0.6 秒，可以接受。
