# 首页顶部滚动区域的问题

首页（https://www.pixiv.net/）顶部的第一个版块“关注用户・好P友的作品”是一个横向滚动区域。

```js
document.querySelector('section div ul')
```

## 节点结构

ul 宽度很大，约为 12000px。

里面有 15 个作品元素（div），这个数量是固定的。

默认情况下，每个作品的宽度也是固定的：

`width: 184px` + `margin-left: 24px` = `208 px`

此外还有占位用的 div，它们通过改变 width 来控制中间显示的作品。

```html
<ul>
  <!-- 开头的占位元素（一开始不存在，滚动 2 次后生成） -->
  <div style="width: xxxxpx;"></div>

  <!-- 固定有 15 个作品元素，可以动态删除并添加节点，但数量保持 15 -->
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>
  <div></div>

  <!-- 末尾的占位元素（始终存在） -->
  <div style="width: 9128px;"></div>
</ul>
```

## 滚动逻辑

### 作品的变化

这里同时显示 6 个作品，滚动时只滚动 5 个。

也就是说，列表里的显示的最后一个作品，在向右滚动后显示在左侧的第一个。

列表里第一个作品，在向左滚动后会变成显示的最后一个作品。

我推测是这样运作的：

每次滚动时，把第 6+5*n 个元素滚动到可视区域。

初始状态下显示的 6 个作品是： 1 2 3 4 5 6。

第一次滚动，第 11 个作品元素 `scrollIntoView()`。这样显示的 6 个作品是： 6 7 8 9 10 11。

第二次滚动，第 16 个作品元素 `scrollIntoView()`。这样显示的 6 个作品是： 11 12 13 14 15 16。

但是在放大缩略图之后似乎也并不总是这样。第一次滚动后并不会让第 11 个作品显示到可视区域。

----------

当第二次向右滚动时，因为默认的 15 个作品已经全部显完了，还需要显示新的作品，此时会动态修改作品列表元素。

左侧不需要的作品元素会被移除，然后在作品列表后面添加需要显示的作品。


向右滚动时，删除开头的 5 个作品元素，然后在末尾添加 5 个作品元素。（1 2 3 4 5 会被删除，原本的 6 变成了 1）

向左滚动时，删除末尾的 5 个作品元素，然后在开头添加 5 个作品元素。

作品元素变化后，现在显示的第一个作品在作品列表里是第 6 个。（前 5 个没有显示出来，被隐藏了）

### 占位元素的变化

初始状态下，在作品列表的后面有一个占位的 div。它的宽度就是两个占位符的总宽度。

当向右滚动 2 次后（默认的 15 个作品已经显示完了，还需要显示新的作品），在作品列表的前面还会生成一个占位用的 div，初始宽度为 1016（208*5-24）px。

每当第一个占位元素的宽度增加时，第二个占位元素的宽度就会相应减少。

具体来说，每次滚动 5 个作品，左侧占位元素的宽度每次增加 1040 px（第一次生成时是 1016 px），右侧占位元素的宽度则每次相应减少。

占位元素的 width 在一次滚动期间会改变多次（次数不固定），最后一个作品滚动完才会固定下来。

## 问题出现

这个版块原本是同时显示 6 个作品，每次滚动 5 个作品。

下载器放大了作品缩略图之后，缩略图的宽度变大了，而且还不固定，可能有时同时显示 5 个，有时同时显示 4 个，有时只能显示 3.5 个。总之很少像原来那样同时显示 6 个。

pixiv 滚动的逻辑没有变，它还是一次滚动 5 个作品，这个数字太大了，导致有些作品本来应该在滚动后显示的，但是现在直接被滚动到显示区域外了，用户看不到。

此外，默认情况下作品元素固定是 15 个，但是下载器放大缩略图之后，并且滚动几次之后，作品元素数量可能会有 18 个。

## 解决思路

### 1

统计每个作品的实际宽度，每次滚动后由下载器计算应该滚动多少宽度，并且处理差额。

例如实际上滚动了 1400px，但是这导致一些作品被隐藏了。下载器需要计算作品不会被隐藏的数值，然后去修改占位元素的 width。

增加左侧占位元素的宽度，会把图片列表向右推，这样就能够显示出左侧原本被隐藏的作品。

但实际上这个思路很难实现，因为后面作品的预览图是延迟加载的，加载前获取不到它的宽度。

例如当前显示到了第 9 个作品，我想计算下一次滚动后，应该显示多少图片，那么我至少需要获取第 10、11、12、13 个作品的宽度。但是它们里面没有图片，都是占位符，我无法获取它们的宽度（或者说加载缩略图之后的宽度），也就没办法计算出下次滚动后应该显示到第几个作品为止。

还有个问题是，这个思路可能需要计算被删除的元素的宽度，但是这很难做到，因为捕获到被删除的元素时，已经无法获取它的宽度了。

### 2

这个思路只是缓解问题，不能真正解决问题。

那就是每次滚动后修改占位元素的宽度，让这次滚动的距离减少。

例如向右滚动后，左侧占位符宽度增加了 1450 px。我默认会有作品被遮挡，于是我增加它的宽度到 2000 px，这样能够多显示出来左侧的一个作品。

这需要判断是向左滚动还是向右滚动。向左滚动需要减少左侧占位符的宽度，向右滚动需要增加左侧占位符的宽度。

### 3

向右滚动时，移除了多少个作品元素，那么左侧占位符的宽度就应该增加这些作品元素的宽度之和。如果它增加的值少了或是多了，就修改它的值。

这需要保存它增加前的宽度，然后加（减）被移除的作品的宽度之和。

图片是占位符的，宽度按照 208 处理。

### 结论

今天花了数个小时研究这个问题，但是无法解决。

如果修改了左侧占位符的 width，有时候情况会更糟糕，可能在一次滚动中，pixiv 会删除十个甚至更多的作品，然后再添加一些作品。

下一次滚动应该显示的作品都已经被删了，无法让预期的作品显示出来了，那么就走不通了。

### 4

我又有一个想法，滚动时的问题经常是左侧有些作品被意外隐藏了。如果检查左侧被隐藏的作品，让它们都显示出来，怎么样？

试验后发现不行:

1. 有些作品确实是应该隐藏的，不应该总是让所有作品都显示
2. pixiv 的代码还是会移除一些还没来得及显示的元素，漏图的问题还是无法解决。

## 临时代码

我在 `ShowLargerThumbnails.ts` 里尝试解决这个问题时的代码。最终这些修改没有提交，退回上一个版本了。

```ts
import { EVT } from './EVT'
import { pageType } from './PageType'
import { settings } from './setting/Settings'
import { Tools } from './Tools'
import { Utils } from './utils/Utils'

class ShowLargerThumbnails {
  constructor() {
    this.loadCssText()
    this.bindEvents()
    this.handleHomePageFirstScrollArea()
  }

  // css 内容来自 style/showLargerThumbnails.css
  private css = ''
  private readonly styleId = 'ShowLargerThumbnails'
  private enable = false

  private async loadCssText() {
    const css = await fetch(
      chrome.runtime.getURL('style/showLargerThumbnails.css')
    )
    this.css = await css.text()
    this.setCss()
  }

  private bindEvents() {
    window.addEventListener(EVT.list.settingChange, (ev: CustomEventInit) => {
      const data = ev.detail.data as any
      if (data.name === 'showLargerThumbnails') {
        this.setCss()
      }
    })

    window.addEventListener(EVT.list.pageSwitch, () => {
      this.setCss()
      this.handleHomePageFirstScrollArea()
    })
  }

  private setCss() {
    if (!this.css) {
      return
    }

    if (Tools.notEnabledShowLargerThumb()) {
      return this.removeStyle()
    }
    settings.showLargerThumbnails ? this.addStyle() : this.removeStyle()
  }

  private addStyle() {
    if (document.querySelector('#' + this.styleId)) {
      return
    }

    const el = document.createElement('style')
    el.id = this.styleId
    el.innerHTML = this.css
    document.body.append(el)
    this.enable = true
  }

  private removeStyle() {
    const el = document.querySelector('#' + this.styleId)
    el && el.remove()
    this.enable = false
  }

  // 首页顶部滚动区域的 ul 元素
  private workListUL?: HTMLUListElement
  // 监视 ul
  private obUl?: MutationObserver

  // ul 里作品列表前面的占位 div 元素
  private leftSpaceDiv?: HTMLDivElement
  // 监视占位元素
  private obLeft?: MutationObserver

  // 被移除的作品列表
  private removedWorkList: HTMLElement[] = []

  private timer?: number
  private setDivWidthTimer?: number

  private handleHomePageFirstScrollArea() {
    if (pageType.type !== pageType.list.Home) {
      this.workListUL = undefined
      this.obUl?.disconnect()
      this.leftSpaceDiv = undefined
      this.obLeft?.disconnect()
      this.change = 0
      return
    }

    this.workListUL = (document.querySelector('section div ul') as HTMLUListElement | null) || undefined
    if (!this.enable || !this.workListUL || this.workListUL.childNodes.length < 2) {
      return this.timer = window.setTimeout(() => {
        this.handleHomePageFirstScrollArea()
      }, 500)
    }

    window.clearTimeout(this.timer)

    this.setWorkWidth()
    this.obUl = new MutationObserver((mutations) => {
      for (const mutation of mutations) {
        // 当有元素被删除时
        if (mutation.removedNodes.length > 0) {
          // 重新计算作品的宽度
          window.clearTimeout(this.setDivWidthTimer)
          this.setDivWidthTimer = window.setTimeout(() => {
            this.setWorkWidth()
          }, 100);

          for (const node of mutation.removedNodes) {
            const div = node as HTMLDivElement
            // 如果删除了左侧占位符
            // 带有 width 样式的是左侧占位符
            if (div.style.width) {
              this.leftSpaceDiv = undefined
              this.obLeft?.disconnect()
              this.removedWorkList = []
              this.change = 0
              console.log('删除了左侧占位符')
              continue
            }
            // 如果不是占位符，那么就是作品缩略图元素
            this.removedWorkList.push(div)
            this.handleRemoveWork()
          }

          // getComputedStyle 对已删除的节点无效，属性值都是空字符串
        }

        // 当有元素被添加时，查找左侧占位符，并监视它的 width 值的变化
        if (mutation.addedNodes.length > 0) {
          for (const node of mutation.addedNodes) {
            const div = node as HTMLDivElement
            // 带有 width 样式的是左侧占位符
            if (div.style.width) {
              this.leftSpaceDiv = div
              this.obLeft = new MutationObserver((records) => {
                for (const record of records) {
                  if (record.type === 'attributes') {
                    // oldValue: "width: 184px;"
                    const oldWidth = Number.parseInt(record.oldValue!.replace('width: ', '') || '0')
                    const newWidth = Number.parseInt((record.target as HTMLDivElement).style.width) || 0

                    // 如果新值和 setWidth 一样，说明是本模块手动设置的值，不需要处理
                    if (newWidth === this.setWidth) {
                      return
                    }
                    this.change += newWidth - oldWidth
                  }
                }
              })
              this.obLeft.observe(this.leftSpaceDiv, {
                attributes: true,
                attributeOldValue: true
              })
            }
          }
        }
      }
    })
    this.obUl.observe(this.workListUL, {
      childList: true,
    })

  }
  private change = 0
  private setWidth = 0

  private handleRemoveWork = Utils.debounce(() => {
    if (!this.leftSpaceDiv) {
      return
    }
    let totalWidth = 0
    console.countReset('remove')
    console.log(this.removedWorkList)
    for (const node of this.removedWorkList) {
        const div = node as HTMLDivElement
        console.count('remove')
        if (div.dataset.width) {
          totalWidth += Number.parseInt(div.dataset.width)
        } else {
          // width 184 + ml 24 = 208
          totalWidth += 208
        }
    }
    const abs = Math.abs(totalWidth - this.change)
    let now = Number.parseInt(this.leftSpaceDiv!.style.width)
    console.log('now' + now)
    console.log('abs' + abs)
    if (Math.abs(this.change) > 300) {
      // 占位元素的 left 变大是向右滚动
      if (this.change > 0) {
        this.setWidth = (now + abs)
      } else {
        this.setWidth = (now - abs)
      }
      console.log('setWidth' + this.setWidth)
      this.leftSpaceDiv!.style.width = this.setWidth + 'px'
    }
    this.removedWorkList = []
    this.change = 0
  }, 200)

  // 计算作品缩略图的宽度，并保存在元素上
  private setWorkWidth() {
    if (!this.workListUL || this.removedWorkList.length === 0) {
      return
    }
    for (const node of this.workListUL.childNodes) {
      const div = node as HTMLDivElement
      // 如果有 width 属性，是占位符元素
      if (div.style.width) {
        continue
      }
      // 如果已经计算过宽度了
      if (div.dataset.width) {
        continue
      }
      // 如果没有 img 标签，说明图片是懒加载的，所以不能计算它的宽度
      const img = div.querySelector('img') as HTMLImageElement
      if (!img) {
        continue
      }
      if (img.complete) {
        div.dataset.width = getComputedStyle(div)['width']
      } else {
        img.onload = () => {
          div.dataset.width = getComputedStyle(div)['width']
        }
      }
    }
  }
}

new ShowLargerThumbnails()
```