/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ({

/***/ "./src/ts/API.ts":
/*!***********************!*\
  !*** ./src/ts/API.ts ***!
  \***********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   API: () => (/* binding */ API)
/* harmony export */ });
class API {
    // 发送 get 请求，返回 json 数据，抛出异常
    static sendGetRequest(url) {
        return new Promise((resolve, reject) => {
            fetch(url, {
                method: 'get',
                credentials: 'same-origin',
            })
                .then((response) => {
                // response.ok 的状态码范围是 200-299
                if (response.ok) {
                    return response.json();
                }
                else {
                    // 请求成功但状态码异常
                    reject({
                        status: response.status,
                        statusText: response.statusText,
                    });
                    switch (response.status) {
                        case 400:
                            return console.error('Status Code: 400（Bad Request）。服务器无法理解此请求');
                        case 401:
                            return console.error('Status Code: 401（Unauthorized）。您可能需要登录 Pixiv 账号');
                        case 403:
                            return console.error('Status Code: 403（Forbidden）。服务器拒绝了这个请求');
                        case 404:
                            return console.error('Status Code: 404（Not Found）。服务器找不到请求的资源');
                        case 500:
                            return console.error('Status Code: 500（Internal Server Error）。服务器内部错误');
                        case 503:
                            return console.error('Status Code: 503（Service Unavailable）。服务器忙或者在维护');
                        default:
                            return console.error(`请求的状态不正确，状态码：${response.status}`);
                    }
                }
            })
                .then((data) => {
                resolve(data);
            })
                .catch((error) => {
                // 请求失败
                reject(error);
            });
        });
    }
    // 获取收藏数据
    // 这个 api 返回的作品列表顺序是按收藏顺序由近期到早期排列的
    static async getBookmarkData(id, type = 'illusts', tag, offset, hide = false) {
        const url = `https://www.pixiv.net/ajax/user/${id}/${type}/bookmarks?tag=${tag}&offset=${offset}&limit=100&rest=${hide ? 'hide' : 'show'}&rdm=${Math.random()}`;
        return this.sendGetRequest(url);
    }
    // 添加收藏
    static async addBookmark(id, type, tags, hide, token) {
        const restrict = hide ? 1 : 0;
        let body = {};
        if (type === 'illusts') {
            body = {
                comment: '',
                illust_id: id,
                restrict: restrict,
                tags: tags,
            };
        }
        else {
            body = {
                comment: '',
                novel_id: id,
                restrict: restrict,
                tags: tags,
            };
        }
        return fetch(`https://www.pixiv.net/ajax/${type}/bookmarks/add`, {
            method: 'POST',
            credentials: 'same-origin',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'x-csrf-token': token,
            },
            body: JSON.stringify(body),
        });
    }
    static async deleteBookmark(bookmarkID, type, token) {
        // 注意，这里的 ID 是 bookmarkID，不是作品 ID
        const bodyStr = type === 'illusts'
            ? `bookmark_id=${bookmarkID}`
            : `del=1&book_id=${bookmarkID}`;
        return fetch(`https://www.pixiv.net/ajax/${type}/bookmarks/delete`, {
            method: 'POST',
            credentials: 'same-origin',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
                'x-csrf-token': token,
            },
            body: bodyStr,
        });
    }
    // 获取关注的用户列表
    static getFollowingList(id, rest = 'show', tag = '', offset = 0, limit = 100, lang = 'zh') {
        const url = `https://www.pixiv.net/ajax/user/${id}/following?offset=${offset}&limit=${limit}&rest=${rest}&tag=${tag}&lang=${lang}`;
        return this.sendGetRequest(url);
    }
    // 获取好 P 友列表
    static getMyPixivList(id, offset = 0, limit = 100, lang = 'zh') {
        const url = `https://www.pixiv.net/ajax/user/${id}/mypixiv?offset=${offset}&limit=${limit}&lang=${lang}`;
        return this.sendGetRequest(url);
    }
    // 获取粉丝列表
    static getFollowersList(id, offset = 0, limit = 100, lang = 'zh') {
        const url = `https://www.pixiv.net/ajax/user/${id}/followers?offset=${offset}&limit=${limit}&lang=${lang}`;
        return this.sendGetRequest(url);
    }
    // 获取用户信息
    static getUserProfile(id) {
        // full=1 在画师的作品列表页使用，获取详细信息
        // full=0 在作品页内使用，只获取少量信息
        const url = `https://www.pixiv.net/ajax/user/${id}?full=1`;
        return this.sendGetRequest(url);
    }
    // 获取用户指定类型的作品列表
    // 返回作品的 id 列表，不包含详细信息
    static async getUserWorksByType(id, type = ['illusts', 'manga', 'novels']) {
        let typeSet = new Set(type);
        let result = [];
        const url = `https://www.pixiv.net/ajax/user/${id}/profile/all`;
        let data = await this.sendGetRequest(url);
        for (const type of typeSet.values()) {
            const idList = Object.keys(data.body[type]);
            for (const id of idList) {
                result.push({
                    type,
                    id,
                });
            }
        }
        return result;
    }
    // 获取用户指定类型、并且指定 tag 的作品列表
    // 返回整个请求的结果，里面包含作品的详细信息
    // 必须带 tag 使用。不带 tag 虽然也能获得数据，但是获得的并不全，很奇怪。
    static getUserWorksByTypeWithTag(id, type, tag, offset = 0, limit = 100) {
        // https://www.pixiv.net/ajax/user/2369321/illusts/tag?tag=Fate/GrandOrder&offset=0&limit=100
        const url = `https://www.pixiv.net/ajax/user/${id}/${type}/tag?tag=${tag}&offset=${offset}&limit=${limit}`;
        return this.sendGetRequest(url);
    }
    // 获取插画 漫画 动图 的详细信息
    static getArtworkData(id, unlisted = false) {
        const url = `https://www.pixiv.net/ajax/illust/${unlisted ? 'unlisted/' : ''}${id}`;
        return this.sendGetRequest(url);
    }
    // 获取动图的元数据
    static getUgoiraMeta(id) {
        const url = `https://www.pixiv.net/ajax/illust/${id}/ugoira_meta`;
        return this.sendGetRequest(url);
    }
    // 获取小说的详细信息
    static getNovelData(id, unlisted = false) {
        const url = `https://www.pixiv.net/ajax/novel/${unlisted ? 'unlisted/' : ''}${id}`;
        return this.sendGetRequest(url);
    }
    // 获取相关作品
    static getRelatedData(id) {
        // 最后的 18 是预加载首屏的多少个作品的信息，和下载并没有关系
        const url = `https://www.pixiv.net/ajax/illust/${id}/recommend/init?limit=18`;
        return this.sendGetRequest(url);
    }
    // 获取排行榜数据
    // 排行榜数据基本是一批 50 条作品信息
    static getRankingData(option) {
        let url = `https://www.pixiv.net/ranking.php?mode=${option.mode}&p=${option.p}&format=json`;
        // 把可选项添加到 url 里
        let temp = new URL(url);
        // 下面两项需要判断有值再添加。不可以让这些字段使用空值
        if (option.worksType) {
            temp.searchParams.set('content', option.worksType);
        }
        if (option.date) {
            temp.searchParams.set('date', option.date);
        }
        url = temp.toString();
        return this.sendGetRequest(url);
    }
    // 获取收藏后的相似作品数据
    // 需要传入作品 id 和要抓取的数量。但是实际获取到的数量会比指定的数量少一些
    static getRecommenderData(id, number) {
        const url = `/rpc/recommender.php?type=illust&sample_illusts=${id}&num_recommendations=${number}`;
        return this.sendGetRequest(url);
    }
    // 获取搜索数据
    static getSearchData(word, type = 'artworks', p = 1, option = {}) {
        // 基础的 url
        let url = `https://www.pixiv.net/ajax/search/${type}/${encodeURIComponent(word)}?word=${encodeURIComponent(word)}&p=${p}`;
        // 把可选项添加到 url 里
        let temp = new URL(url);
        for (const [key, value] of Object.entries(option)) {
            if (value) {
                temp.searchParams.set(key, value);
            }
        }
        url = temp.toString();
        return this.sendGetRequest(url);
    }
    static getNovelSearchData(word, p = 1, option = {}) {
        // 基础的 url
        let url = `https://www.pixiv.net/ajax/search/novels/${encodeURIComponent(word)}?word=${encodeURIComponent(word)}&p=${p}`;
        // 把可选项添加到 url 里
        let temp = new URL(url);
        for (const [key, value] of Object.entries(option)) {
            if (value) {
                temp.searchParams.set(key, value);
            }
        }
        url = temp.toString();
        return this.sendGetRequest(url);
    }
    // 获取大家的新作品的数据
    static getNewIllustData(option) {
        const url = `https://www.pixiv.net/ajax/illust/new?lastId=${option.lastId}&limit=${option.limit}&type=${option.type}&r18=${option.r18}`;
        return this.sendGetRequest(url);
    }
    // 获取大家的新作小说的数据
    static getNewNovleData(option) {
        const url = `https://www.pixiv.net/ajax/novel/new?lastId=${option.lastId}&limit=${option.limit}&r18=${option.r18}`;
        return this.sendGetRequest(url);
    }
    // 获取关注的用户的新作品的数据
    static getBookmarkNewWorkData(type, p, tag = '', r18, lang = 'zh') {
        const url = `https://www.pixiv.net/ajax/follow_latest/${type}?p=${p}&tag=${tag}&mode=${r18 ? 'r18' : 'all'}&lang=${lang}`;
        return this.sendGetRequest(url);
    }
    // 获取小说的系列作品信息
    // 这个 api 目前一批最多只能返回 30 个作品的数据，所以可能需要多次获取
    static getNovelSeriesData(series_id, limit = 30, last_order, order_by = 'asc') {
        const url = `https://www.pixiv.net/ajax/novel/series_content/${series_id}?limit=${limit}&last_order=${last_order}&order_by=${order_by}`;
        return this.sendGetRequest(url);
    }
    // 获取系列信息
    // 这个接口的数据结构里同时有 illust （包含漫画）和 novel 系列数据
    // 恍惚记得有插画系列来着，但是没找到对应的网址，难道是记错了？
    static getSeriesData(series_id, pageNo) {
        const url = `https://www.pixiv.net/ajax/series/${series_id}?p=${pageNo}`;
        return this.sendGetRequest(url);
    }
    // 点赞
    static async addLike(id, type, token) {
        let data = {};
        if (type === 'illusts') {
            data = {
                illust_id: id,
            };
        }
        else {
            data = {
                novel_id: id,
            };
        }
        const r = await fetch(`https://www.pixiv.net/ajax/${type}/like`, {
            method: 'POST',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json; charset=utf-8',
                'x-csrf-token': token,
            },
            credentials: 'same-origin',
            body: JSON.stringify(data),
        });
        const json = (await r.json());
        return json;
    }
    static async getMuteSettings() {
        return this.sendGetRequest(`https://www.pixiv.net/ajax/mute/items?context=setting`);
    }
    /**获取系列小说的设定资料 */
    static async getNovelSeriesGlossary(seriesId) {
        return this.sendGetRequest(`https://www.pixiv.net/ajax/novel/series/${seriesId}/glossary`);
    }
    /**获取系列小说某条设定资料的详细信息 */
    static async getNovelSeriesGlossaryItem(seriesId, itemId) {
        return this.sendGetRequest(`https://www.pixiv.net/ajax/novel/series/${seriesId}/glossary/item/${itemId}`);
    }
    /**获取用户最近的几条消息 */
    static async getLatestMessage(number) {
        return this.sendGetRequest(`https://www.pixiv.net/rpc/index.php?mode=latest_message_threads2&num=${number}&offset=0`);
    }
    /**关注一个用户 */
    // recaptcha_enterprise_score_token 对于有些用户是不需要的，不过传递空值是允许的
    static async addFollowingUser(userID, token, recaptcha_enterprise_score_token) {
        return new Promise(async (resolve) => {
            const response = await fetch(`https://www.pixiv.net/bookmark_add.php`, {
                method: 'POST',
                credentials: 'same-origin',
                headers: {
                    Accept: 'application/json',
                    'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
                    'x-csrf-token': token,
                },
                body: `mode=add&type=user&user_id=${userID}&tag=&restrict=0&format=json&recaptcha_enterprise_score_token=${recaptcha_enterprise_score_token}`,
            });
            // 如果操作成功，则返回值是 []
            // 如果用户不存在，返回值是该用户主页的网页源码
            // 如果 token 错误，返回值是一个包含错误提示的 JSON 对象
            // 所以这里需要转换为 text
            await response.text();
            return resolve(response.status);
        });
    }
}



/***/ }),

/***/ "./src/ts/ArtworkThumbnail.ts":
/*!************************************!*\
  !*** ./src/ts/ArtworkThumbnail.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   artworkThumbnail: () => (/* binding */ artworkThumbnail)
/* harmony export */ });
/* harmony import */ var _WorkThumbnail__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./WorkThumbnail */ "./src/ts/WorkThumbnail.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");




// 查找图像作品的缩略图，当鼠标进入、移出时等动作触发时执行回调函数
class ArtworkThumbnail extends _WorkThumbnail__WEBPACK_IMPORTED_MODULE_0__.WorkThumbnail {
    constructor() {
        super();
        this.selectors = [];
        if (_Config__WEBPACK_IMPORTED_MODULE_3__.Config.mobile) {
            // 移动端的作品选择器就这一个
            this.selectors = ['.works-item-illust'];
        }
        else {
            this.selectors = [
                'div[width="136"]',
                'div[width="131"]',
                'div[width="288"]',
                'div[width="184"]',
                'div[width="112"]',
                'div[width="104"]',
                'div[width="90"]',
                'div[width="118"]',
                '._work',
                '._work.item',
                'li>div>div:first-child',
            ];
        }
        this.findThumbnail(document.body);
        this.createObserver(document.body);
    }
    findThumbnail(parent) {
        if (!parent.querySelectorAll) {
            return;
        }
        // 遍历所有的选择器，为找到的元素绑定事件
        // 注意：有时候一个节点里会含有多种尺寸的缩略图，为了全部查找到它们，必须遍历所有的选择器。
        // 如果在查找到某个选择器之后，不再查找剩余的选择器，就可能会遗漏一部分缩略图。
        // 但是，这有可能会导致事件的重复绑定，所以下载器添加了 dataset.mouseover 标记以减少重复绑定
        // 例如，画师主页顶部的“精选”作品会被两个选择器查找到：'li>div>div:first-child' 'div[width="288"]'
        for (const selector of this.selectors) {
            // 现在 'li>div>div:first-child' 只在投稿页面使用
            if (selector === 'li>div>div:first-child' &&
                _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Request) {
                return;
            }
            const elements = parent.querySelectorAll(selector);
            for (const el of elements) {
                const id = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.findWorkIdFromElement(el, 'illusts');
                if (_Config__WEBPACK_IMPORTED_MODULE_3__.Config.mobile) {
                    // 在移动端页面里，即使没有找到作品 id，也要执行回调函数
                    // 因为此时可能内部的 A 标签还未生成，所以会获取不到 id
                    // 而之后下载器只会监听新添加的缩略图容器，不会监听内部添加 A 标签的事件，
                    // 所以以后也不会监听到它。那么只能先为它绑定事件，
                    // 等到点击下载按钮时再尝试获取 id
                    this.bindEvents(el, id);
                }
                else {
                    // 在桌面版页面里，只有查找到作品 id 时才会执行回调函数
                    if (id) {
                        this.bindEvents(el, id);
                    }
                }
            }
        }
    }
}
const artworkThumbnail = new ArtworkThumbnail();



/***/ }),

/***/ "./src/ts/BG.ts":
/*!**********************!*\
  !*** ./src/ts/BG.ts ***!
  \**********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   bg: () => (/* binding */ bg)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./utils/IndexedDB */ "./src/ts/utils/IndexedDB.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");




class BG {
    constructor() {
        this.list = [];
        this.bgModeflagClassName = 'xzBG';
        this.bgLayerClassName = 'xzBGLayer';
        this.bgUrl = '';
        this.DBName = 'PBDBG';
        this.DBVer = 1;
        this.storeName = 'bg';
        this.keyName = 'bg';
        // 在数据库升级事件里创建表
        this.onUpdate = (db) => {
            if (!db.objectStoreNames.contains(this.storeName)) {
                db.createObjectStore(this.storeName, {
                    keyPath: 'key',
                });
            }
        };
        this.IDB = new _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_2__.IndexedDB();
        this.init();
    }
    async init() {
        this.bindEvents();
        await this.initDB();
        this.restore();
    }
    async initDB() {
        await this.IDB.open(this.DBName, this.DBVer, this.onUpdate);
    }
    createBGLayer(wrap) {
        const div = document.createElement('div');
        div.classList.add(this.bgLayerClassName);
        const el = wrap.insertAdjacentElement('afterbegin', div);
        return el;
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.selectBG, () => {
            this.selectBG();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clearBG, () => {
            this.clearBG();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'bgDisplay') {
                this.setBGAll();
            }
            if (data.name === 'bgOpacity') {
                this.setBGAll();
            }
            if (data.name === 'bgPositionY') {
                this.setBGAll();
            }
        });
    }
    async restore() {
        const data = (await this.IDB.get(this.storeName, this.keyName));
        if (!data || !data.file) {
            return;
        }
        this.bgUrl = URL.createObjectURL(data.file);
        this.preload();
    }
    async selectBG() {
        const file = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.selectFile('.jpg,.jpeg,.png,.bmp,.webp'))[0];
        this.bgUrl = URL.createObjectURL(file);
        this.preload();
        for (const o of this.list) {
            this.setBG(o);
        }
        const data = {
            key: this.keyName,
            file: file,
        };
        const test = await this.IDB.get(this.storeName, this.keyName);
        this.IDB[test ? 'put' : 'add'](this.storeName, data);
    }
    clearBG() {
        this.IDB.clear(this.storeName);
        this.bgUrl = '';
        for (const o of this.list) {
            o.bg.style.backgroundImage = 'none';
            this.setDisplay(o);
        }
    }
    // 预加载背景图片
    preload() {
        // 由于浏览器的工作原理，背景图片在未被显示之前是不会加载的，在显示时才会进行加载。这会导致背景层显示之后出现短暂的空白（因为在加载图片）。为了避免空白，需要预加载图片
        const img = new Image();
        img.src = this.bgUrl;
        img.style.display = 'none';
        document.body.append(img);
    }
    async setBG(o) {
        this.setPositionY(o);
        this.setOpacity(o);
        this.setBGURL(o);
        this.setDisplay(o);
    }
    async setBGAll() {
        for (const o of this.list) {
            this.setPositionY(o);
            this.setOpacity(o);
            this.setBGURL(o);
            this.setDisplay(o);
        }
    }
    setBGURL(o) {
        o.bg.style.backgroundImage = `url(${this.bgUrl})`;
    }
    setDisplay(o) {
        o.bg.style.display = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.bgDisplay ? 'block' : 'none';
        if (!this.bgUrl) {
            o.wrap.classList.remove(this.bgModeflagClassName);
        }
        else {
            o.wrap.classList[_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.bgDisplay ? 'add' : 'remove'](this.bgModeflagClassName);
        }
    }
    setOpacity(o) {
        o.bg.style.opacity = (o.opacity || _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.bgOpacity / 100).toString();
    }
    setPositionY(o) {
        o.bg.style.backgroundPositionY = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.bgPositionY;
    }
    // 其他模块可以调用这个方法，为一个元素添加背景层
    // 如果传入一个真值的不透明度，会始终使用传入的不透明度，忽略用户用户设置的不透明度
    useBG(wrap, opacity) {
        if (this.bgUrl) {
            this.readySet(wrap, opacity);
        }
        else {
            let timer = window.setInterval(() => {
                if (this.bgUrl) {
                    window.clearInterval(timer);
                    this.readySet(wrap);
                }
            }, 300);
        }
    }
    readySet(wrap, opacity) {
        const o = {
            wrap,
            bg: this.createBGLayer(wrap),
            opacity,
        };
        this.list.push(o);
        this.setBG(o);
    }
}
const bg = new BG();



/***/ }),

/***/ "./src/ts/BoldKeywords.ts":
/*!********************************!*\
  !*** ./src/ts/BoldKeywords.ts ***!
  \********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   BoldKeywords: () => (/* binding */ BoldKeywords)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");


class BoldKeywords {
    constructor(wrap) {
        this.className = 'showBlobKeywords';
        this.wrap = wrap;
        this.bindEvent();
        this.setClassName();
    }
    bindEvent() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'boldKeywords') {
                this.setClassName();
            }
        });
    }
    setClassName() {
        this.wrap.classList[_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.boldKeywords ? 'add' : 'remove'](this.className);
    }
}



/***/ }),

/***/ "./src/ts/Bookmark.ts":
/*!****************************!*\
  !*** ./src/ts/Bookmark.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   bookmark: () => (/* binding */ bookmark)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Log */ "./src/ts/Log.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Token__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./Token */ "./src/ts/Token.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");










// 对 API.addBookmark 进行封装
class Bookmark {
    constructor() {
        /** 接收到需要排队的任务时增加计数 */
        this.taskID = 0;
        /**叫号的号码，当 add 方法的 slowly 参数为 true 时，需要等待叫号到它才能执行 */
        this.nextTaskID = 1;
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadComplete, () => {
            if (this.taskID > this.nextTaskID) {
                const msg = _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_收藏任务尚未完成请等待');
                _Log__WEBPACK_IMPORTED_MODULE_4__.log.warning(msg);
                _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.warning(msg, {
                    position: 'center',
                });
            }
        });
        // 如果用户在离开页面时还有等待重试的收藏任务，就提示用户
        // 使用 window.onbeforeunload 事件
        // 但是这会导致 SelectWork 里的该事件出现问题，或者两个模块里都会出现问题，所以就不提示了
    }
    async getWorkData(type, id) {
        return type === 'illusts'
            ? await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(id)
            : await _API__WEBPACK_IMPORTED_MODULE_0__.API.getNovelData(id);
    }
    /**添加收藏
     *
     * 可选参数 tags：可以直接传入这个作品的 tag 列表
     *
     * 如果未传入 tags，但收藏设置要求 tags，则此方法会发送请求获取作品数据
     *
     * 可选参数 needAddTag：控制是否添加 tag。缺省时使用 settings.widthTagBoolean
     *
     * 可选参数 restrict：指示这个收藏是否为非公开收藏。false 为公开收藏，true 为非公开收藏。缺省时使用 settings.restrictBoolean
     *
     * 可选参数 slowly：未指定或 false 时，立即执行这个收藏请求。设置为 true 则会获得一个号码并等待叫号到它再执行。这是为了减少 429 错误发生的概率。当需要大批量收藏作品时应该设置为 true。
     */
    async add(id, type, tags, needAddTag, restrict, slowly) {
        return new Promise(async (resolve, reject) => {
            const _needAddTag = needAddTag === undefined ? _setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.widthTagBoolean : !!needAddTag;
            if (_needAddTag) {
                // 需要添加 tags
                if (tags === undefined) {
                    // 如果未传递 tags，则请求作品数据来获取 tags
                    const data = await this.getWorkData(type, id);
                    tags = _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.extractTags(data);
                }
            }
            else {
                // 不需要添加 tags
                tags = [];
            }
            const _restrict = restrict === undefined ? _setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.restrictBoolean : !!restrict;
            // 立即执行的情况
            if (!slowly) {
                const status = await this.sendRequest(id, type, tags, _restrict);
                return resolve(status);
            }
            // 需要排队的情况
            const NO = ++this.taskID;
            await this.waitCallMe(NO);
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_5__.setTimeoutWorker.set(async () => {
                const status = await this.sendRequest(id, type, tags, _restrict);
                this.nextTaskID++;
                return resolve(status);
            }, _Config__WEBPACK_IMPORTED_MODULE_1__.Config.slowCrawlDealy);
        });
    }
    async waitCallMe(NO) {
        return new Promise(async (resolve) => {
            if (this.nextTaskID === NO) {
                return resolve(NO);
            }
            else {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_5__.setTimeoutWorker.set(() => {
                    return resolve(this.waitCallMe(NO));
                }, 300);
            }
        });
    }
    async sendRequest(id, type, tags, hide) {
        return new Promise(async (resolve) => {
            _API__WEBPACK_IMPORTED_MODULE_0__.API.addBookmark(id, type, tags, hide, _Token__WEBPACK_IMPORTED_MODULE_8__.token.token).then(async (res) => {
                switch (res.status) {
                    case 400:
                        await _Token__WEBPACK_IMPORTED_MODULE_8__.token.reset();
                        return resolve(this.sendRequest(id, type, tags, hide));
                    case 429:
                    case 500:
                        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_添加收藏失败'), {
                            position: 'center',
                        });
                        _Log__WEBPACK_IMPORTED_MODULE_4__.log.error(`${_Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.createWorkLink(id, type === 'illusts')} ${_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_添加收藏失败')}. ${_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_错误代码')}${res.status}. ${_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_下载器会在几分钟后重试')}`);
                        window.setTimeout(() => {
                            return resolve(this.sendRequest(id, type, tags, hide));
                        }, _Config__WEBPACK_IMPORTED_MODULE_1__.Config.retryTime);
                        break;
                    default:
                        return resolve(res.status);
                }
            });
        });
    }
}
const bookmark = new Bookmark();



/***/ }),

/***/ "./src/ts/CenterPanel.ts":
/*!*******************************!*\
  !*** ./src/ts/CenterPanel.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _BG__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./BG */ "./src/ts/BG.ts");
/* harmony import */ var _OpenCenterPanel__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./OpenCenterPanel */ "./src/ts/OpenCenterPanel.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _BoldKeywords__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./BoldKeywords */ "./src/ts/BoldKeywords.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./ShowHelp */ "./src/ts/ShowHelp.ts");











// 选项卡的名称和索引
var Tabbar;
(function (Tabbar) {
    Tabbar[Tabbar["Crawl"] = 0] = "Crawl";
    Tabbar[Tabbar["Download"] = 1] = "Download";
    Tabbar[Tabbar["Other"] = 2] = "Other";
})(Tabbar || (Tabbar = {}));
// 中间面板
class CenterPanel {
    constructor() {
        this.updateActiveClass = 'updateActiveClass';
        this.TitleActiveClass = 'active';
        this.titleAnimationElClassList = ['tab1', 'tab2', 'tab3'];
        this.allLangFlag = [];
        this.addCenterPanel();
        _Theme__WEBPACK_IMPORTED_MODULE_3__.theme.register(this.centerPanel);
        _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.register(this.centerPanel);
        this.activeTab(Tabbar.Crawl);
        _BG__WEBPACK_IMPORTED_MODULE_6__.bg.useBG(this.centerPanel);
        new _BoldKeywords__WEBPACK_IMPORTED_MODULE_9__.BoldKeywords(this.centerPanel);
        this.allLangFlag = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.langTypes.map((type) => 'lang_' + type);
        this.setLangFlag();
        this.bindEvents();
    }
    // 添加中间面板
    addCenterPanel() {
        const centerPanelHTML = `
      <div class="centerWrap ${'lang_' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.type}">

      <div class="centerWrap_head">
      <div class="centerWrap_title blue">
      ${_Config__WEBPACK_IMPORTED_MODULE_4__.Config.appName}
      <div class="btns">
      <a class="has_tip centerWrap_top_btn update" data-xztip="_newver" data-xztitle="_newver" href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases/latest" target="_blank">
        <svg class="icon" aria-hidden="true">
          <use xlink:href="#icon-gengxin"></use>
        </svg>
      </a>
      <a class="has_tip centerWrap_top_btn github_icon" data-xztip="_github" data-xztitle="_github" href="https://github.com/xuejianxianzun/PixivBatchDownloader" target="_blank">
      <svg class="icon" aria-hidden="true">
        <use xlink:href="#icon-github"></use>
      </svg>
      </a>
      <a class="has_tip centerWrap_top_btn wiki_url" data-xztip="_wiki" data-xztitle="_wiki" href="https://xuejianxianzun.github.io/PBDWiki" target="_blank">
        <svg class="icon" aria-hidden="true">
          <use xlink:href="#icon-help"></use>
        </svg>
      </a>
        <button class="textButton ${!_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile && 'has_tip'} centerWrap_top_btn centerWrap_close" ${!_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile &&
            'data-xztip="_隐藏控制面板" data-xztitle="_隐藏控制面板"'}>
        <svg class="icon" aria-hidden="true">
          <use xlink:href="#icon-guanbi"></use>
        </svg>
        </button>
      </div>
      </div>
      </div>

      <div class="centerWrap_tabs tabsTitle">
        <div class="title" data-xztext="_抓取" tabindex="0"></div>
        <div class="title" data-xztext="_下载" tabindex="0"></div>
        <div class="title" data-xztext="_更多" tabindex="0"></div>
        <div class="title_active"></div>
      </div>

      <div class="centerWrap_con beautify_scrollbar">

      <slot data-name="form"></slot>

      <div class="help_bar gray1"> 
      <button class="textButton gray1" id="showDownTip" type="button" data-xztext="_常见问题"></button>
      <a class="gray1" href="https://xuejianxianzun.github.io/PBDWiki" target="_blank" data-xztext="_wiki"></a>
      <a class="gray1" href="https://discord.gg/eW9JtTK" target="_blank">Discord</a>
      <a class="gray1" href="https://chrome.google.com/webstore/detail/pixiv-fanbox-downloader/ihnfpdchjnmlehnoeffgcbakfmdjcckn" target="_blank" data-xztext="_fanboxDownloader"></a>
      <button class="textButton gray1" id="showPatronTip" type="button" data-xztext="_赞助我"></button>
      <br>
      </div>

      </div>

      </div>
      `;
        document.body.insertAdjacentHTML('beforebegin', centerPanelHTML);
        this.centerPanel = document.querySelector('.centerWrap');
        this.updateLink = this.centerPanel.querySelector('.update');
        this.allTabTitle = this.centerPanel.querySelectorAll('.tabsTitle .title');
        this.titleAnimationEl = this.centerPanel.querySelector('.title_active');
        // 设置移动端样式
        if (_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
            this.centerPanel.classList.add('mobile');
        }
    }
    setLangFlag() {
        this.allLangFlag.forEach((flag) => {
            this.centerPanel.classList.remove(flag);
        });
        this.centerPanel.classList.add('lang_' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.type);
    }
    bindEvents() {
        // 监听点击扩展图标的消息，开关中间面板
        chrome.runtime.onMessage.addListener((msg) => {
            if (msg.msg === 'click_icon') {
                this.toggle();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingInitialized, () => {
            _ShowHelp__WEBPACK_IMPORTED_MODULE_10__.showHelp.show('tipHowToUse', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_HowToUse'));
        });
        // 使用快捷键 Alt + x 切换中间面板显示隐藏
        window.addEventListener('keydown', (ev) => {
            if (ev.altKey && ev.code === 'KeyX') {
                this.toggle();
            }
        }, false);
        // 关闭按钮
        document
            .querySelector('.centerWrap_close')
            .addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('closeCenterPanel');
            if (!_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
                _ShowHelp__WEBPACK_IMPORTED_MODULE_10__.showHelp.show('tipAltXToShowControlPanel', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_快捷键ALTX显示隐藏控制面板'));
            }
        });
        // 开始抓取作品时，隐藏
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlStart, () => {
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('closeCenterPanel');
        });
        // 抓取完作品详细数据时，显示
        for (const ev of [_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete, _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resume]) {
            window.addEventListener(ev, () => {
                if (!_store_States__WEBPACK_IMPORTED_MODULE_2__.states.quickCrawl) {
                    this.show();
                }
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.openCenterPanel, () => {
            this.show();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.closeCenterPanel, () => {
            this.close();
        });
        // 显示更新按钮
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.hasNewVer, () => {
            this.updateLink.classList.add(this.updateActiveClass);
            this.updateLink.style.display = 'inline-block';
        });
        // 显示常见问题
        this.centerPanel
            .querySelector('#showDownTip')
            .addEventListener('click', () => {
            let msg = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_常见问题说明');
            if (_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
                msg =
                    msg + '<br><br>' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_Kiwi浏览器可能不能建立文件夹的bug');
            }
            _MsgBox__WEBPACK_IMPORTED_MODULE_5__.msgBox.show(msg, {
                title: _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_常见问题'),
            });
        });
        this.centerPanel
            .querySelector('#showPatronTip')
            .addEventListener('click', () => _MsgBox__WEBPACK_IMPORTED_MODULE_5__.msgBox.show(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_赞助方式提示'), {
            title: _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_赞助我'),
        }));
        this.centerPanel.addEventListener('click', (e) => {
            const ev = e || window.event;
            ev.stopPropagation();
        });
        document.addEventListener('click', () => {
            if (getComputedStyle(this.centerPanel)['display'] !== 'none') {
                _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('closeCenterPanel');
            }
        });
        // 在选项卡的标题上触发事件时，激活对应的选项卡
        let eventList = ['click', 'mouseenter'];
        if (_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
            eventList = ['touchend'];
        }
        for (let index = 0; index < this.allTabTitle.length; index++) {
            const title = this.allTabTitle[index];
            eventList.forEach((eventName) => {
                title.addEventListener(eventName, () => {
                    // 触发 mouseenter 时，如果用户设置的是通过点击来切换选项卡，则直接返回
                    // 触发 click 时无需检测，始终可以切换
                    if (eventName === 'mouseenter' && _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.switchTabBar === 'click') {
                        return;
                    }
                    this.activeTab(index);
                });
            });
            // 当标题获得焦点，并且用户按下了回车或空格键时，激活对应的选项卡
            title.addEventListener('keydown', (event) => {
                if ((event.code === 'Enter' || event.code === 'Space') &&
                    event.target === title) {
                    event.stopPropagation();
                    event.preventDefault();
                    this.activeTab(index);
                }
            });
        }
        // 当可以开始下载时，切换到“下载”选项卡
        for (const ev of [_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete, _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resume]) {
            window.addEventListener(ev, () => {
                if (_store_States__WEBPACK_IMPORTED_MODULE_2__.states.mergeNovel) {
                    return;
                }
                this.activeTab(Tabbar.Download);
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlEmpty, () => {
            this.activeTab(Tabbar.Crawl);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.langChange, () => {
            this.setLangFlag();
        });
    }
    // 设置激活的选项卡
    activeTab(no = 0) {
        // 显示选项卡的内容
        const allTabCon = this.centerPanel.querySelectorAll('.tabsContnet');
        for (let index = 0; index < allTabCon.length; index++) {
            allTabCon[index].style.display = index === no ? 'block' : 'none';
        }
        // 高亮选项卡的标题
        for (const title of this.allTabTitle) {
            title.classList.remove(this.TitleActiveClass);
        }
        this.allTabTitle[no].classList.add(this.TitleActiveClass);
        // 设置动画效果
        const useClass = this.titleAnimationElClassList[no];
        if (this.titleAnimationEl.classList.contains(useClass)) {
            return;
        }
        this.titleAnimationElClassList.forEach((str) => {
            this.titleAnimationEl.classList.remove(str);
        });
        this.titleAnimationEl.classList.add(useClass);
    }
    // 显示中间区域
    show() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_2__.states.mergeNovel) {
            return;
        }
        this.centerPanel.style.display = 'block';
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('centerPanelOpened');
    }
    // 隐藏中间区域
    close() {
        this.centerPanel.style.display = 'none';
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('centerPanelClosed');
    }
    toggle() {
        const nowDisplay = this.centerPanel.style.display;
        nowDisplay === 'block' ? this.close() : this.show();
        if (nowDisplay === 'block') {
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('closeCenterPanel');
        }
        else {
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('openCenterPanel');
        }
    }
}
new CenterPanel();


/***/ }),

/***/ "./src/ts/CheckUnsupportBrowser.ts":
/*!*****************************************!*\
  !*** ./src/ts/CheckUnsupportBrowser.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Log */ "./src/ts/Log.ts");



// 某些国产套壳浏览器不能正常使用本程序。如果检测到该浏览器，则显示提示
// 相关文档： notes/一些国产套壳浏览器使用本程序的情况.md
class CheckUnsupportBrowser {
    constructor() {
        this.rules = {
            // "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.66 Safari/537.36 SE 2.X MetaSr 1.0"
            Sougou: function () {
                return navigator.userAgent.includes(' SE ');
            },
            // "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.25 Safari/537.36 Core/1.70.3872.400 QQBrowser/10.8.4455.400"
            QQ: function () {
                return navigator.userAgent.includes('QQBrowser');
            },
            // "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3947.100 Safari/537.36 2345Explorer/10.21.0.21486"
            '2345': function () {
                return navigator.userAgent.includes('2345Explorer');
            },
            All: function () {
                // 如果这个浏览器的 Chrome 内核的版本号较低，也会显示提示
                // 为什么设置为 88：
                // 1. 下载器使用的 Manifest V2 需要的内核版本最低为 79
                // 2. Cent 浏览器的内核版本是 86，但它即使使用 V2，仍然会在转换 GIF 时出现问题，所以需要提高版本号
                // 3. 未来升级到 Manifest V3 需要的内核版本最低为 88
                const minChromeVer = 88;
                const test = navigator.userAgent.match(/Chrome\/(\d*)/);
                if (test && test[1]) {
                    const ver = Number.parseInt(test[1]);
                    if (ver < minChromeVer) {
                        return true;
                    }
                }
                return false;
            },
        };
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingInitialized, () => {
            this.check();
        });
    }
    check() {
        for (const func of Object.values(this.rules)) {
            if (func()) {
                const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_不支持的浏览器');
                _Log__WEBPACK_IMPORTED_MODULE_2__.log.error(msg);
                return;
            }
        }
        if (navigator.userAgent.includes('YaBrowser')) {
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_yandex浏览器的警告'));
        }
    }
}
new CheckUnsupportBrowser();


/***/ }),

/***/ "./src/ts/Colors.ts":
/*!**************************!*\
  !*** ./src/ts/Colors.ts ***!
  \**************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Colors: () => (/* binding */ Colors)
/* harmony export */ });
var Colors;
(function (Colors) {
    // 通用颜色
    Colors["white"] = "#fff";
    Colors["black"] = "#000";
    Colors["red"] = "#f00";
    Colors["theme"] = "#0ea8ef";
    // 带有语义的字体颜色
    Colors["textSuccess"] = "#00BD17";
    Colors["textWarning"] = "#d27e00";
    Colors["textError"] = "#f00";
    // 背景颜色
    // 稍暗，适合在颜色区域的面积较大时使用
    Colors["bgBlue"] = "#0ea8ef";
    Colors["bgGreen"] = "#14ad27";
    Colors["bgYellow"] = "#e49d00";
    Colors["bgRed"] = "#f33939";
    // 带有语义的背景颜色
    // 稍亮，适合在小区域使用
    Colors["bgBrightBlue"] = "#29b3f3";
    Colors["bgSuccess"] = "#00BD17";
    Colors["bgWarning"] = "#e49d00";
    Colors["bgError"] = "#f00";
})(Colors || (Colors = {}));



/***/ }),

/***/ "./src/ts/Config.ts":
/*!**************************!*\
  !*** ./src/ts/Config.ts ***!
  \**************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Config: () => (/* binding */ Config)
/* harmony export */ });
// 定义一些常量
// 用户不可以修改这里的配置
class Config {
}
/**使用输出面板显示内容时，如果文件数量大于这个值，就不再显示内容，而是保存到 txt 文件 */
Config.outputMax = 5000;
/**同时下载的文件数量的最大值 */
Config.downloadThreadMax = 6;
/**下载某个文件出错时，最大重试次数 */
Config.retryMax = 10;
/**作品类型所对应的字符串名称 */
Config.worksTypeName = ['Illustration', 'Manga', 'Ugoira', 'Novel'];
/**程序名 */
Config.appName = 'Powerful Pixiv Downloader';
/**下载器储存设置时使用的 key name */
Config.settingStoreName = 'xzSetting';
/**按收藏数量过滤作品时，预设的最大收藏数量 */
Config.BookmarkCountLimit = 9999999;
/**Pixiv 作品总数量上限 */
Config.worksNumberLimit = 9999999999;
/**当抓取被 pixiv 限制，返回了空数据时，等待这个时间之后再继续抓取 */
Config.retryTime = 200000;
/**慢速抓取模式下，每个抓取请求之间的间隔时间（ms） */
Config.slowCrawlDealy = 1400;
/**浏览器是否处于移动端模式 */
Config.mobile = navigator.userAgent.includes('Mobile');



/***/ }),

/***/ "./src/ts/ConvertUgoira/ConvertUgoira.ts":
/*!***********************************************!*\
  !*** ./src/ts/ConvertUgoira/ConvertUgoira.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   convertUgoira: () => (/* binding */ convertUgoira)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _ToWebM__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ToWebM */ "./src/ts/ConvertUgoira/ToWebM.ts");
/* harmony import */ var _ToGIF__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./ToGIF */ "./src/ts/ConvertUgoira/ToGIF.ts");
/* harmony import */ var _ToAPNG__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./ToAPNG */ "./src/ts/ConvertUgoira/ToAPNG.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");









// 控制动图转换
class ConvertUgoira {
    constructor() {
        this.downloading = true; // 是否在下载。如果下载停止了则不继续转换后续任务，避免浪费资源
        this._count = 0; // 统计有几个转换任务
        this.maxCount = 1; // 允许同时运行多少个转换任务
        this.msgFlag = 'tipConvertUgoira';
        this.setMaxCount();
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStart, () => {
            this.downloading = true;
            _MsgBox__WEBPACK_IMPORTED_MODULE_5__.msgBox.resetOnce(this.msgFlag);
        });
        [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadPause, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop].forEach((event) => {
            window.addEventListener(event, () => {
                this.downloading = false;
            });
        });
        // 设置发生变化时
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'convertUgoiraThread') {
                this.setMaxCount();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.convertSuccess, () => {
            this.complete();
        });
        // 如果转换动图时页面被隐藏了，则显示提示
        // document.addEventListener('visibilitychange', () => {
        //   this.checkHidden()
        // })
    }
    setMaxCount() {
        this.maxCount =
            _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.convertUgoiraThread > 0 ? _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.convertUgoiraThread : 1;
    }
    set count(num) {
        this._count = num;
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('convertChange', this._count);
        // this.checkHidden()
    }
    async start(file, info, type) {
        return new Promise(async (resolve, reject) => {
            const t = window.setInterval(async () => {
                if (this._count < this.maxCount) {
                    window.clearInterval(t);
                    if (!this.downloading) {
                        return;
                    }
                    this.count = this._count + 1;
                    // 提取每一张图片
                    const zipFileBuffer = await file.arrayBuffer();
                    const indexList = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getJPGContentIndex(zipFileBuffer);
                    const ImageBitmapList = await _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.extractImage(zipFileBuffer, indexList, 'ImageBitmap');
                    if (type === 'gif') {
                        resolve(_ToGIF__WEBPACK_IMPORTED_MODULE_3__.toGIF.convert(ImageBitmapList, info));
                    }
                    else if (type === 'png') {
                        resolve(_ToAPNG__WEBPACK_IMPORTED_MODULE_4__.toAPNG.convert(ImageBitmapList, info));
                    }
                    else {
                        // 如果没有 type 则默认使用 webm
                        resolve(_ToWebM__WEBPACK_IMPORTED_MODULE_2__.toWebM.convert(ImageBitmapList, info));
                    }
                }
            }, 200);
        });
    }
    complete() {
        this.count = this._count - 1;
    }
    // 转换成 WebM
    async webm(file, info, id) {
        const delayTooLarge = info.frames.find((item) => item.delay > 32767);
        if (delayTooLarge) {
            const msg = _Lang__WEBPACK_IMPORTED_MODULE_6__.lang.transl('_动图不能转换为WEBM视频的提示', _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.createWorkLink(id));
            _MsgBox__WEBPACK_IMPORTED_MODULE_5__.msgBox.warning(msg);
            _Log__WEBPACK_IMPORTED_MODULE_8__.log.warning(msg);
            return await this.start(file, info, 'gif');
        }
        return await this.start(file, info, 'webm');
    }
    // 转换成 GIF
    async gif(file, info, id) {
        return await this.start(file, info, 'gif');
    }
    // 转换成 APNG
    async apng(file, info, id) {
        return await this.start(file, info, 'png');
    }
    checkHidden() {
        if (this._count > 0 && document.visibilityState === 'hidden') {
            _MsgBox__WEBPACK_IMPORTED_MODULE_5__.msgBox.once(this.msgFlag, _Lang__WEBPACK_IMPORTED_MODULE_6__.lang.transl('_转换动图时页面被隐藏的提示'), 'warning');
        }
    }
}
const convertUgoira = new ConvertUgoira();



/***/ }),

/***/ "./src/ts/ConvertUgoira/ToAPNG.ts":
/*!****************************************!*\
  !*** ./src/ts/ConvertUgoira/ToAPNG.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   toAPNG: () => (/* binding */ toAPNG)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");

class ToAPNG {
    async convert(ImageBitmapList, info) {
        return new Promise(async (resolve, reject) => {
            const width = ImageBitmapList[0].width;
            const height = ImageBitmapList[0].height;
            const canvas = document.createElement('canvas');
            const ctx = canvas.getContext('2d', {
                willReadFrequently: true,
            });
            canvas.width = width;
            canvas.height = height;
            // 添加帧数据
            let arrayBuffList = [];
            ImageBitmapList.forEach((imageBitmap) => {
                ctx.drawImage(imageBitmap, 0, 0);
                // 从画布获取图像绘制后的 Uint8ClampedArray buffer
                const buff = ctx.getImageData(0, 0, width, height).data.buffer;
                arrayBuffList.push(buff);
            });
            const delayList = info.frames.map((frame) => frame.delay);
            // 编码
            // https://github.com/photopea/UPNG.js/#encoder
            const pngFile = UPNG.encode(arrayBuffList, width, height, 0, delayList);
            arrayBuffList = null;
            const blob = new Blob([pngFile], {
                type: 'image/vnd.mozilla.apng',
            });
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('convertSuccess');
            resolve(blob);
        });
    }
}
const toAPNG = new ToAPNG();



/***/ }),

/***/ "./src/ts/ConvertUgoira/ToGIF.ts":
/*!***************************************!*\
  !*** ./src/ts/ConvertUgoira/ToGIF.ts ***!
  \***************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   toGIF: () => (/* binding */ toGIF)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");

class ToGIF {
    constructor() {
        this.gifWorkerUrl = '';
        this.loadWorkerJS();
    }
    // 添加转换 GIF 的 worker 文件
    async loadWorkerJS() {
        let gifWorker = await fetch(chrome.runtime.getURL('lib/gif.worker.js'));
        const gifWorkerBolb = await gifWorker.blob();
        this.gifWorkerUrl = URL.createObjectURL(gifWorkerBolb);
    }
    // 转换成 GIF
    async convert(ImageBitmapList, info) {
        return new Promise(async (resolve, reject) => {
            // 配置 gif.js
            let gif = new GIF({
                workers: 4,
                quality: 10,
                workerScript: this.gifWorkerUrl,
            });
            // 绑定渲染完成事件
            gif.on('finished', (file) => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('convertSuccess');
                resolve(file);
            });
            const width = ImageBitmapList[0].width;
            const height = ImageBitmapList[0].height;
            const canvas = document.createElement('canvas');
            const ctx = canvas.getContext('2d');
            canvas.width = width;
            canvas.height = height;
            // 添加帧数据
            ImageBitmapList.forEach((imageBitmap, index) => {
                ctx.drawImage(imageBitmap, 0, 0);
                const ImageData = ctx.getImageData(0, 0, width, height);
                gif.addFrame(ImageData, {
                    delay: info.frames[index].delay,
                });
            });
            // 渲染 gif
            gif.render();
        });
    }
}
const toGIF = new ToGIF();



/***/ }),

/***/ "./src/ts/ConvertUgoira/ToWebM.ts":
/*!****************************************!*\
  !*** ./src/ts/ConvertUgoira/ToWebM.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   toWebM: () => (/* binding */ toWebM)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");

class ToWebM {
    async convert(ImageBitmapList, info) {
        return new Promise(async (resolve, reject) => {
            const width = ImageBitmapList[0].width;
            const height = ImageBitmapList[0].height;
            const canvas = document.createElement('canvas');
            const ctx = canvas.getContext('2d');
            canvas.width = width;
            canvas.height = height;
            // 创建视频编码器
            const encoder = new Whammy.Video();
            // 添加帧数据
            ImageBitmapList.forEach((imageBitmap, index) => {
                ctx.drawImage(imageBitmap, 0, 0);
                // 把图像转换为 webp 格式的 DataURL，这样 webm 编码器内部可以直接使用，不需要进行一些重复的操作
                // https://github.com/antimatter15/whammy#basic-usage
                const url = canvas.toDataURL('image/webp', 0.9);
                encoder.add(url, info.frames[index].delay);
            });
            // 编码视频
            encoder.compile(false, (video) => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('convertSuccess');
                resolve(video);
            });
        });
    }
}
const toWebM = new ToWebM();



/***/ }),

/***/ "./src/ts/CopyToClipboard.ts":
/*!***********************************!*\
  !*** ./src/ts/CopyToClipboard.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   CopyToClipboard: () => (/* binding */ CopyToClipboard)
/* harmony export */ });
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");


class CopyToClipboard {
    static setClipboard(text) {
        return new Promise((resolve, reject) => {
            const type = 'text/plain';
            const blob = new Blob([text], { type });
            const data = [new ClipboardItem({ [type]: blob })];
            window.navigator.clipboard.write(data).then(() => {
                _Toast__WEBPACK_IMPORTED_MODULE_1__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_已复制到剪贴板'));
                resolve();
            }, () => {
                _Toast__WEBPACK_IMPORTED_MODULE_1__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_写入剪贴板失败'));
                reject();
            });
        });
    }
}



/***/ }),

/***/ "./src/ts/DoubleWidthThumb.ts":
/*!************************************!*\
  !*** ./src/ts/DoubleWidthThumb.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _FindHorizontalImageWrap__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./FindHorizontalImageWrap */ "./src/ts/FindHorizontalImageWrap.ts");




// 如果一个作品的缩略图是横图，则把这个缩略图的容器的宽度设置为默认宽度的 2 倍
// 注意：必须开启“替换方形缩略图以显示图片比例”，“横图占用二倍宽度”的功能才能生效
class DoubleWidthThumb {
    constructor() {
        /* 双倍宽度的图片的 id（由下载器添加这个 id） */
        this.addId = 'doubleWidth';
        this.styleId = 'doubleWidthStyle';
        this.css = `#doubleWidth {
    width: 30% !important;
  }`;
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'doubleWidthThumb') {
                // 如果开启了父级设置“显示更大的缩略图”，以及这个设置，则必须开启“替换方形缩略图以显示图片比例”
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showLargerThumbnails &&
                    _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.doubleWidthThumb &&
                    !_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.replaceSquareThumb) {
                    (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.setSetting)('replaceSquareThumb', true);
                }
                this.setCss();
            }
            if (data.name === 'showLargerThumbnails') {
                this.setCss();
            }
            // 如果关闭了“替换方形缩略图以显示图片比例”，则需要关闭这个设置，因为这个设置无法生效
            if (data.name === 'replaceSquareThumb') {
                if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.replaceSquareThumb && _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.doubleWidthThumb) {
                    (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.setSetting)('doubleWidthThumb', false);
                }
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitch, () => {
            this.setCss();
        });
        // 如果一个缩略图是横图，则在它的容器上添加特定 id
        _FindHorizontalImageWrap__WEBPACK_IMPORTED_MODULE_3__.findHorizontalImageWrap.onFind((wrap) => {
            if (!wrap.id) {
                wrap.id = this.addId;
            }
        });
    }
    setCss() {
        if (_Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.notEnabledShowLargerThumb()) {
            return this.removeStyle();
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.replaceSquareThumb &&
            _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showLargerThumbnails &&
            _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.doubleWidthThumb) {
            this.addStyle();
        }
        else {
            this.removeStyle();
        }
    }
    addStyle() {
        if (document.querySelector('#' + this.styleId)) {
            return;
        }
        const el = document.createElement('style');
        el.id = this.styleId;
        el.innerHTML = this.css;
        document.body.append(el);
    }
    removeStyle() {
        const el = document.querySelector('#' + this.styleId);
        el && el.remove();
    }
}
new DoubleWidthThumb();


/***/ }),

/***/ "./src/ts/EVT.ts":
/*!***********************!*\
  !*** ./src/ts/EVT.ts ***!
  \***********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   EVT: () => (/* binding */ EVT)
/* harmony export */ });
// 管理自定义事件
class EVENT {
    constructor() {
        this.bindOnceFlagList = [];
        this.list = {
            /** 当抓取开始时触发 */
            crawlStart: 'crawlStart',
            stopCrawl: 'stopCrawl',
            /** 让下载器抓取特定的 tag，而不是自动获取当前页面的 tag（仅在 tag 搜索页面有效） */
            crawlTag: 'crawlTag',
            /** 当检查到错误的设置时触发 */
            wrongSetting: 'wrongSetting',
            /** 当获取作品的 id 列表完成时触发 */
            getIdListFinished: 'getIdListFinished',
            /** 获取了作品的 id 列表，需要下载这些 id 列表时使用 */
            crawlIdList: 'crawlIdList',
            /** 当抓取完成时触发。不管结果是否为空都会触发 */
            crawlComplete: 'crawlComplete',
            /** 当抓取结果为空时触发。触发时机晚于 crawlComplete */
            crawlEmpty: 'crawlEmpty',
            /** store 里每存储一个作品的元数据，就触发一次。如果一个元数据产生了多个结果（多图作品），只触发一次 */
            addResult: 'addResult',
            /** 当抓取完毕之后，抓取结果又发生变化时触发（比如进行多次筛选、改变设置项等，导致结果变化） */
            resultChange: 'resultChange',
            /** 当进行快速抓取时触发 */
            quickCrawl: 'quickCrawl',
            /** 抓取完毕后，可以准备开始下载时触发 */
            /** 它是一个派生事件，可以由多个其他事件触发 */
            readyDownload: 'readyDownload',
            /** 下载被取消（取消是在尚未开始下载前触发的，它不同于下载停止） */
            downloadCancel: 'downloadCancel',
            /** 开始下载时触发 */
            downloadStart: 'downloadStart',
            /** 下载状态变成暂停时触发 */
            downloadPause: 'downloadPause',
            /** 请求暂停下载 */
            requestPauseDownload: 'requestPauseDownload',
            /** 下载状态变成停止时触发 */
            downloadStop: 'downloadStop',
            /** 当文件在下载阶段下载失败时触发 */
            /** 当动图转换出错时触发 */
            downloadError: 'downloadError',
            /** 当一个文件在下载阶段被跳过时触发 */
            skipDownload: 'skipDownload',
            /** 当浏览器把一个文件保存到本地失败时触发 */
            saveFileError: 'saveFileError',
            /** 当下载的文件传递给浏览器进行保存，并且成功保存之后触发 */
            /** skipDownload 也会触发这个事件 */
            downloadSuccess: 'downloadSuccess',
            /** 下载队列里的所有文件都已经下载并保存完毕，并且没有出错的。如果有出错的，就不会触发这个事件 */
            downloadComplete: 'downloadComplete',
            /** 页面切换 */
            pageSwitch: 'pageSwitch',
            /** 页面切换，并且页面类型变化 */
            pageSwitchedTypeChange: 'pageSwitchedTypeChange',
            /** 页面切换，并且页面类型不变 */
            pageSwitchedTypeNotChange: 'pageSwitchedTypeNotChange',
            /** 程序启动时，设置初始化完毕后触发 */
            settingInitialized: 'settingInitialized',
            /** 请求重置所有设置 */
            resetSettings: 'resetSettings',
            /** 重置所有设置执行完毕 */
            resetSettingsEnd: 'resetSettingsEnd',
            /** 请求导出设置 */
            exportSettings: 'exportSettings',
            /** 请求导入设置 */
            importSettings: 'importSettings',
            /** 重新显示帮助 */
            resetHelpTip: 'resetHelpTip',
            /** 当动图转换数量发生变化时触发 */
            convertChange: 'convertChange',
            /** 当动图转换成功时触发 */
            convertSuccess: 'convertSuccess',
            /** 指示打开中间面板 */
            openCenterPanel: 'openCenterPanel',
            /** 指示关闭中间面板 */
            closeCenterPanel: 'closeCenterPanel',
            /** 中间面板已打开 */
            centerPanelOpened: 'centerPanelOpened',
            /** 中间面板已关闭 */
            centerPanelClosed: 'centerPanelClosed',
            /** 当清除多图作品时触发 */
            clearMultiple: 'clearMultiple',
            /** 当清除动图作品时触发 */
            clearUgoira: 'clearUgoira',
            /** 当手动删除作品时触发 */
            deleteWork: 'deleteWork',
            /** 当下载器在页面上创建的作品列表全部完成时触发 */
            worksUpdate: 'worksUpdate',
            /** 当需要清空下载记录时触发（只用于检测重复文件的下载记录） */
            clearDownloadRecord: 'clearDownloadRecord',
            /** 当需要导出下载记录时触发 */
            exportDownloadRecord: 'exportDownloadRecord',
            /** 当需要导入下载记录时触发 */
            importDownloadRecord: 'importDownloadRecord',
            /** 当需要清空断点续传的数据时触发 */
            clearSavedCrawl: 'clearSavedCrawl',
            /** 当从断点续传数据恢复了下载时触发 */
            resume: 'resume',
            /** 当需要导出 csv 文件时触发 */
            exportCSV: 'exportCSV',
            /** 当需要导出抓取结果时触发 */
            exportResult: 'exportResult',
            /** 当需要导入抓取结果时触发 */
            importResult: 'importResult',
            /** 当需要保存用户头像时触发 */
            saveAvatarImage: 'saveAvatarImage',
            /** 当需要保存用户头像为图标时触发 */
            saveAvatarIcon: 'saveAvatarIcon',
            /** 当需要保存用户背景图片时触发 */
            saveUserCover: 'saveUserCover',
            /** 当需要预览文件名时触发 */
            previewFileName: 'previewFileName',
            /** 当需要预览 url 时触发 */
            showURLs: 'showURLs',
            /** 当需要输出面板输出内容时触发 */
            output: 'output',
            /** 当设置表单里的设置项发生变化时触发 */
            settingChange: 'settingChange',
            /** 当下载器检测到有新版本时触发 */
            hasNewVer: 'hasNewVer',
            /** 进入批量收藏模式时触发 */
            bookmarkModeStart: 'bookmarkModeStart',
            /** 批量收藏完成时触发 */
            bookmarkModeEnd: 'bookmarkModeEnd',
            /** 需要单独显示信息时触发 */
            showMsg: 'showMsg',
            /** 需要显示冒泡提示时触发 */
            sendToast: 'sendToast',
            /** 需要清空日志区域时触发 */
            clearLog: 'clearLog',
            /** 选择背景图片 */
            selectBG: 'selectBG',
            /** 清除背景图片 */
            clearBG: 'clearBG',
            /** 点击了下载器在作品缩略图上添加的按钮时触发 */
            /** 其他按钮监听这个事件后隐藏自己，就可以避免其他按钮出现闪烁、残留的问题 */
            clickBtnOnThumb: 'clickBtnOnThumb',
            /** 显示原比例图片时触发 */
            showOriginSizeImage: 'showOriginSizeImage',
            /** 语言类型改变时触发 */
            langChange: 'langChange',
            /** 开始定时抓取时触发 */
            startTimedCrawl: 'startTimedCrawl',
            /** 请求取消定时抓取时触发*/
            cancelTimedCrawl: 'cancelTimedCrawl',
            /**当获取到页面的主题颜色时触发 */
            getPageTheme: 'getPageTheme',
            /**当下载模块向浏览器发起一个下载请求（保存文件到本地）时触发 */
            sendBrowserDownload: 'sendBrowserDownload',
            /**需要显示预览作品详细信息的面板时触发 */
            showPreviewWorkDetailPanel: 'showPreviewWorkDetailPanel',
            /**预览作品详细信息的面板关闭后触发 */
            PreviewWorkDetailPanelClosed: 'PreviewWorkDetailPanelClosed',
            // 通过鼠标滚轮事件来切换预览图
            wheelScrollSwitchPreviewImage: 'wheelScrollSwitchPreviewImage',
            // 当结束对一个作品的预览时触发（即预览图窗口消失时触发）
            previewEnd: 'previewEnd',
        };
    }
    // 只绑定某个事件一次，用于防止事件重复绑定
    // 通过 flag 确认是否是同一个事件
    // 可以执行多次，不会自动解绑
    bindOnce(flag, targetEvt, evtFun) {
        const query = this.bindOnceFlagList.includes(flag);
        if (!query) {
            this.bindOnceFlagList.push(flag);
            window.addEventListener(targetEvt, function (ev) {
                evtFun(ev);
            });
        }
    }
    fire(type, data) {
        const event = new CustomEvent(type, {
            detail: { data: data === undefined ? {} : data },
        });
        window.dispatchEvent(event);
    }
}
const EVT = new EVENT();



/***/ }),

/***/ "./src/ts/FileName.ts":
/*!****************************!*\
  !*** ./src/ts/FileName.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   fileName: () => (/* binding */ fileName)
/* harmony export */ });
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _setting_NameRuleManager__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/NameRuleManager */ "./src/ts/setting/NameRuleManager.ts");
/* harmony import */ var _SetUserName__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./SetUserName */ "./src/ts/SetUserName.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _utils_DateFormat__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./utils/DateFormat */ "./src/ts/utils/DateFormat.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");








// 生成文件名
class FileName {
    constructor() {
        // 下载器所有的动图格式后缀名
        this.ugoiraExt = ['zip', 'webm', 'gif', 'png'];
        this.addStr = '[downloader_add]';
        // 不能出现在文件名开头的一些特定字符
        this.checkStartCharList = ['/', ' '];
        this.atList = ['@', '＠'];
    }
    // 生成 {rank} 标记的值
    createRank(rank) {
        // 处理空值
        if (rank === null) {
            return '';
        }
        // string 是旧版本中使用的，以前抓取结果里的 rank 直接就是 '#1' 这样的字符串，后来改成了数字类型
        if (typeof rank === 'string') {
            return rank;
        }
        // 其他的情况则应该是期望的值（数字类型）
        return '#' + rank;
    }
    // 生成 {p_num} 标记的值
    createPNum(data) {
        // 只有插画和漫画有编号
        if (data.type === 0 || data.type === 1) {
            const index = data.index ?? _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getResultIndex(data);
            // 处理第一张图不带序号的情况
            if (index === 0 && _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.noSerialNo) {
                if (data.pageCount === 1 && _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.noSerialNoForSingleImg) {
                    return '';
                }
                if (data.pageCount > 1 && _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.noSerialNoForMultiImg) {
                    return '';
                }
            }
            const p = index.toString();
            // 处理在前面填充 0 的情况
            return _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.zeroPadding
                ? p.padStart(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.zeroPaddingLength, '0')
                : p;
        }
        else {
            // 其他类型没有编号，返回空字符串
            return '';
        }
    }
    // 生成 {id} 标记的值
    createId(data, p_num) {
        // 如果不需要添加序号，或者没有序号，则只返回数字 id
        if (p_num === '') {
            return data.idNum.toString();
        }
        // 添加序号
        return `${data.idNum}_p${p_num}`;
    }
    // 返回收藏数的简化显示
    getBKM1000(bmk) {
        if (bmk < 1000) {
            return '0+';
        }
        else {
            // 1000 以上，以 1000 为单位
            const str = bmk.toString();
            return str.slice(0, str.length - 3) + '000+';
        }
    }
    // 在文件名前面添加一层文件夹
    // appendFolder 方法会对非法字符进行处理（包括处理路径分隔符 / 这主要是因为 tags 可能含有斜线 /，需要替换）
    appendFolder(fullPath, folderName) {
        const allPart = fullPath.split('/');
        allPart.splice(allPart.length - 1, 0, _utils_Utils__WEBPACK_IMPORTED_MODULE_6__.Utils.replaceUnsafeStr(folderName));
        return allPart.join('/');
    }
    // 检查文件名开头是否含有特定字符
    checkStartChar(str) {
        for (const check of this.checkStartCharList) {
            if (str.startsWith(check)) {
                return true;
            }
        }
        return false;
    }
    // 移除文件名开头的特定字符
    removeStartChar(str) {
        while (this.checkStartChar(str)) {
            for (const check of this.checkStartCharList) {
                if (str.startsWith(check)) {
                    str = str.replace(check, '');
                }
            }
        }
        return str;
    }
    RemoveAtFromUsername(name) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.removeAtFromUsername) {
            return name;
        }
        for (const at of this.atList) {
            let index = name.indexOf(at);
            if (index > 0) {
                name = name.substring(0, index);
            }
        }
        return name;
    }
    // 传入命名规则和所有标记，生成文件名
    generateFileName(rule, cfg) {
        let result = rule;
        // 把命名规则里的标记替换成实际值
        for (const [key, val] of Object.entries(cfg)) {
            if (rule.includes(key)) {
                // 空值替换成空字符串
                let temp = val.value ?? '';
                // 如果这个值不是字符串类型则转换为字符串
                temp = typeof temp !== 'string' ? temp.toString() : temp;
                // 替换不可以作为文件名的特殊字符
                if (!val.safe) {
                    temp = _utils_Utils__WEBPACK_IMPORTED_MODULE_6__.Utils.replaceUnsafeStr(temp);
                }
                // 添加标记前缀
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.tagNameToFileName) {
                    temp = val.prefix + temp;
                }
                // 将标记替换成结果，如果有重复的标记，全部替换
                result = result.replace(new RegExp(key, 'g'), temp);
            }
        }
        // 处理文件名里的一些边界情况
        // 如果文件名开头不可用的特殊字符
        result = this.removeStartChar(result);
        // 测试用例
        // const testStr = ' / / {page_tag} / {page_title} /{id}-{user}'
        // console.log(this.removeStartChar(testStr))
        // 如果文件名的尾部是 / 则去掉
        if (result.endsWith('/')) {
            result = result.substring(0, result.length - 1);
        }
        // 处理连续的 /
        result = result.replace(/\/{2,100}/g, '/');
        return result;
    }
    /**传入一个抓取结果，生成其文件名 */
    createFileName(data) {
        // 命名规则
        let userSetName = _setting_NameRuleManager__WEBPACK_IMPORTED_MODULE_1__.nameRuleManager.rule;
        // 检查是否要使用特定的其他命名规则
        // 这是一个定制功能，所以这里设置的规则只会修改原有的文件名，而不会涉及到文件夹部分
        // 如果一个作品符合多条规则，则把多条规则合并。例如：
        // 包含[原神]，命名规则{id}_genshin
        // 包含[Loli]，命名规则{id}_loli
        // 包含[AI生成]，命名规则{id}_AI
        // 比如说有一张ai生成的原神萝莉图例子，以上三个tag都有，那么把文件命名为{id}_genshin_loli_AI
        let diffNames = [];
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.UseDifferentNameRuleIfWorkHasTagSwitch) {
            const workTags = data.tags.map((tag) => tag.toLowerCase());
            for (const item of _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.UseDifferentNameRuleIfWorkHasTagList) {
                for (const setTag of item.tags) {
                    if (workTags.includes(setTag.toLowerCase())) {
                        diffNames.push(item.rule);
                        // 一条规则里的 tag 可能会有多个存在于同一个作品的标签列表里
                        // 如果匹配到就跳过这条规则，以避免重复添加规则对应的命名规则
                        break;
                    }
                }
            }
        }
        if (diffNames.length > 0) {
            let fileName = diffNames.join('').replace(/{id}/g, '');
            fileName = '{id}' + fileName;
            const names = userSetName.split('/');
            names.splice(names.length - 1, 1, fileName);
            userSetName = names.join('/');
        }
        // 判断是否要为每个作品创建单独的文件夹
        let createFolderForEachWork = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.workDir &&
            _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.downloadCount[data.idNum] > _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.workDirFileNumber;
        let r18FolderName = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.r18Folder ? _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.r18FolderName : '';
        const allNameRule = userSetName +
            (createFolderForEachWork ? _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.workDirNameRule : '') +
            r18FolderName;
        // 1 生成所有命名标记的值
        // 对于一些较为耗时的计算，先判断用户设置的命名规则里是否使用了这个标记，如果未使用则不计算
        const p_num = this.createPNum(data);
        const cfg = {
            '{p_title}': {
                value: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.title,
                prefix: '',
                safe: false,
            },
            '{page_title}': {
                value: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.title,
                prefix: '',
                safe: false,
            },
            '{p_tag}': {
                value: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.tag,
                prefix: '',
                safe: false,
            },
            '{page_tag}': {
                value: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.tag,
                prefix: '',
                safe: false,
            },
            '{id}': {
                value: this.createId(data, p_num),
                prefix: '',
                safe: true,
            },
            '{id_num}': {
                value: data.idNum || parseInt(data.id),
                prefix: '',
                safe: true,
            },
            '{p_num}': {
                value: !allNameRule.includes('{p_num}') ? null : p_num,
                prefix: '',
                safe: true,
            },
            '{rank}': {
                value: !allNameRule.includes('{rank}')
                    ? null
                    : this.createRank(data.rank),
                prefix: '',
                safe: true,
            },
            '{title}': {
                value: data.title,
                prefix: 'title_',
                safe: false,
            },
            '{user}': {
                value: this.RemoveAtFromUsername(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.setUserNameList[data.userId] || data.user),
                prefix: 'user_',
                safe: false,
            },
            '{userid}': {
                value: data.userId,
                prefix: 'uid_',
                safe: true,
            },
            '{user_id}': {
                value: data.userId,
                prefix: 'uid_',
                safe: true,
            },
            '{px}': {
                value: !allNameRule.includes('{px}')
                    ? null
                    : data.fullWidth
                        ? data.fullWidth + 'x' + data.fullHeight
                        : '',
                prefix: '',
                safe: true,
            },
            '{tags}': {
                value: !allNameRule.includes('{tags}')
                    ? null
                    : data.tags.join(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.tagsSeparator),
                prefix: 'tags_',
                safe: false,
            },
            '{tags_translate}': {
                value: !allNameRule.includes('{tags_translate}')
                    ? null
                    : data.tagsWithTransl.join(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.tagsSeparator),
                prefix: 'tags_',
                safe: false,
            },
            '{tags_transl_only}': {
                value: !allNameRule.includes('{tags_transl_only}')
                    ? null
                    : data.tagsTranslOnly.join(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.tagsSeparator),
                prefix: 'tags_',
                safe: false,
            },
            '{bmk}': {
                value: data.bmk,
                prefix: 'bmk_',
                safe: true,
            },
            '{bmk_id}': {
                value: data.bmkId || '',
                prefix: 'bmk-id_',
                safe: true,
            },
            '{bmk_1000}': {
                value: this.getBKM1000(data.bmk),
                prefix: 'bmk_',
                safe: true,
            },
            '{like}': {
                value: data.likeCount,
                prefix: 'like_',
                safe: true,
            },
            '{view}': {
                value: data.viewCount,
                prefix: 'view_',
                safe: true,
            },
            '{date}': {
                value: !allNameRule.includes('{date}')
                    ? null
                    : _utils_DateFormat__WEBPACK_IMPORTED_MODULE_5__.DateFormat.format(data.date, _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.dateFormat),
                prefix: '',
                safe: false,
            },
            '{upload_date}': {
                value: !allNameRule.includes('{upload_date}')
                    ? null
                    : _utils_DateFormat__WEBPACK_IMPORTED_MODULE_5__.DateFormat.format(data.uploadDate, _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.dateFormat),
                prefix: '',
                safe: false,
            },
            '{task_date}': {
                value: !allNameRule.includes('{task_date}')
                    ? null
                    : _utils_DateFormat__WEBPACK_IMPORTED_MODULE_5__.DateFormat.format(_store_Store__WEBPACK_IMPORTED_MODULE_3__.store.crawlCompleteTime, _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.dateFormat),
                prefix: '',
                safe: false,
            },
            '{type}': {
                value: _Config__WEBPACK_IMPORTED_MODULE_4__.Config.worksTypeName[data.type],
                prefix: '',
                safe: true,
            },
            '{AI}': {
                value: data.aiType === 2 ? 'AI' : '',
                prefix: '',
                safe: true,
            },
            '{series_title}': {
                value: data.seriesTitle || '',
                prefix: '',
                safe: false,
            },
            '{series_order}': {
                value: data.seriesOrder === null ? '' : '#' + data.seriesOrder,
                prefix: '',
                safe: true,
            },
            '{series_id}': {
                value: data.seriesId,
                prefix: '',
                safe: true,
            },
            '{sl}': {
                value: data.sl ?? 0,
                prefix: '',
                safe: true,
            },
        };
        // 2 生成文件名
        let result = this.generateFileName(userSetName, cfg);
        // 3 根据某些设置向结果中添加新的文件夹
        // 注意：添加文件夹的顺序会影响文件夹的层级，所以不可随意更改顺序
        // 根据作品类型自动创建对应的文件夹
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByType) {
            // 根据作品类型和对应开关确定是否需要要为其建立文件夹
            const allSwitch = [
                _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByTypeIllust,
                _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByTypeManga,
                _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByTypeUgoira,
                _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByTypeNovel,
            ];
            if (allSwitch[data.type]) {
                const folder = _Config__WEBPACK_IMPORTED_MODULE_4__.Config.worksTypeName[data.type];
                result = this.appendFolder(result, folder);
            }
        }
        // 根据 sl 创建文件夹
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderBySl && data.sl !== null) {
            const folder = 'sl' + data.sl.toString();
            result = this.appendFolder(result, folder);
        }
        // 根据第一个匹配的 tag 建立文件夹
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderByTag && _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderTagList.length > 0) {
            const workTags = data.tagsWithTransl.map((val) => val.toLowerCase());
            // 循环用户输入的 tag 列表，查找作品 tag 是否含有匹配项
            // 这样用户输入的第一个匹配的 tag 就会作为文件夹名字
            // 不要循环作品 tag 列表，因为那样找到的第一个匹配项未必是用户输入的第一个
            // 例如 用户输入顺序：巨乳 欧派
            // 作品 tag 里的顺序：欧派 巨乳
            for (const tag of _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.createFolderTagList) {
                // 查找匹配的时候转换成小写
                const nowTag = tag.toLowerCase();
                if (workTags.includes(nowTag)) {
                    // 设置为文件夹名字的时候使用原 tag（不转换成小写）
                    result = this.appendFolder(result, tag);
                    break;
                }
            }
        }
        // 把 R18(G) 作品存入指定目录里
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.r18Folder && (data.xRestrict === 1 || data.xRestrict === 2)) {
            result = this.appendFolder(result, this.generateFileName(r18FolderName, cfg));
        }
        // 为每个作品创建单独的文件夹
        if (createFolderForEachWork) {
            const workDirName = this.generateFileName(_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.workDirNameRule, cfg);
            // 生成文件名。由于用户可能会添加斜线来建立多层路径，所以需要循环添加每个路径
            const allPath = workDirName.split('/');
            for (const path of allPath) {
                if (path.length > 0) {
                    result = this.appendFolder(result, path);
                }
            }
        }
        // 4 文件夹部分和文件名已经全部生成完毕，处理一些边界情况
        // 处理连续的 / 有时候两个斜线中间的字段是空值，最后就变成两个斜线挨在一起了
        result = result.replace(/\/{2,100}/g, '/');
        // 对每一层路径和文件名进行处理
        const paths = result.split('/');
        for (let i = 0; i < paths.length; i++) {
            // 去掉每层路径首尾的空格
            // 把每层路径头尾的 . 替换成全角的．因为 Chrome 不允许头尾使用 .
            paths[i] = paths[i].trim().replace(/^\./g, '．').replace(/\.$/g, '．');
            // 处理路径是 Windows 保留文件名的情况（不需要处理后缀名）
            paths[i] = _utils_Utils__WEBPACK_IMPORTED_MODULE_6__.Utils.handleWindowsReservedName(paths[i], this.addStr);
        }
        result = paths.join('/');
        // 5 生成后缀名
        // 如果是动图，那么此时根据用户设置的动图保存格式，更新其后缀名
        if (this.ugoiraExt.includes(data.ext) &&
            data.ugoiraInfo &&
            _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.imageSize !== 'thumb') {
            // 当下载图片的方形缩略图时，不修改其后缀名，因为此时下载的是作品的静态缩略图，不是动图
            data.ext = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.ugoiraSaveAs;
        }
        // 如果是小说，那么此时根据用户设置的动图保存格式，更新其后缀名
        if (data.type === 3) {
            data.ext = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.novelSaveAs;
        }
        const extResult = '.' + data.ext;
        // 6 处理不创建文件夹的情况
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.notFolderWhenOneFile && _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.length === 1) {
            // 舍弃文件夹部分，只保留文件名
            result = result.split('/').pop();
        }
        // 7 文件名长度限制
        // 不计算文件夹的长度，只计算 文件名+后缀名 部分
        // 理论上文件夹部分也可能会超长，但是实际使用中几乎不会有人这么设置，所以不处理
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.fileNameLengthLimitSwitch) {
            let limit = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.fileNameLengthLimit;
            const allPart = result.split('/');
            const lastIndex = allPart.length - 1;
            if (allPart[lastIndex].length + extResult.length > limit) {
                allPart[lastIndex] = allPart[lastIndex].substring(0, limit - extResult.length);
            }
            result = allPart.join('/');
        }
        // 8 添加后缀名
        result += extResult;
        // 9 返回结果
        return result;
    }
}
const fileName = new FileName();



/***/ }),

/***/ "./src/ts/FindHorizontalImageWrap.ts":
/*!*******************************************!*\
  !*** ./src/ts/FindHorizontalImageWrap.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   findHorizontalImageWrap: () => (/* binding */ findHorizontalImageWrap)
/* harmony export */ });
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");

// 查找横图作品的缩略图和容器
class FindHorizontalImageWrap {
    constructor() {
        // 寻找作品缩略图的容器时使用的选择器
        // 并不是所有容器都需要处理，只需要处理应用了“显示更大的缩略图”的容器
        // 有些缩略图并不会被放大，也就不用处理它们的容器
        this.wrapSelectors = ['.searchList', 'li[size="1"]'];
        this.onFindCB = [];
        this.obBody();
    }
    // 首先会动态生成 li（或者是包含很多 li 的容器元素）
    // 但是此时 li 里面没有 img 标签，而是用一个 figure 标签占位
    // 然后先为一些 li 生成里面的 img 标签（在用户主页会先给首屏显示的 li 生成 img 标签，但是在其他页面也有可能直接为所有 li 生成 img）
    // 有时候当页面滚动到下面的 li 的时候，才会生成里面的 img 标签
    // observer 可以捕获到添加的 img 标签，并且有 src 属性
    // 如果开启了下载器的替换方形缩略图功能，则捕获到的 src 是替换后的
    // 如果 img 的 src 是在缓存里的（并且没有禁用缓存），则捕获到它时就已经 complete 了
    obBody() {
        const ob = new MutationObserver((mutations) => {
            for (const mutation of mutations) {
                if (mutation.addedNodes.length > 0) {
                    for (const el of mutation.addedNodes) {
                        // 从添加的元素里寻找缩略图容器
                        const e = el;
                        const wrapList = [];
                        // 如果添加的是单个的 li，，则判断它是不是缩略图容器
                        if (e.nodeName === 'LI') {
                            if (e.getAttribute('size') === '1' ||
                                e.querySelector('div[width="184"]') ||
                                e.classList.contains('searchList')) {
                                wrapList.push(e);
                            }
                        }
                        else if (e.nodeName === 'IMG' && e.src) {
                            if (_PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.list.ArtworkSearch) {
                                // 在搜索页面里，添加的元素是 img 而不是其容器 li
                                const li = e.parentElement?.parentElement?.parentElement?.parentElement
                                    ?.parentElement?.parentElement;
                                if (li && li.nodeName === 'LI') {
                                    this.readyCheckImage(e, li);
                                }
                            }
                        }
                        else if (e.nodeType === 1) {
                            // 添加的不是 li，则试图从元素中寻找缩略图容器
                            for (const selector of this.wrapSelectors) {
                                const elList = e.querySelectorAll(selector);
                                for (const el of elList) {
                                    wrapList.push(el);
                                }
                                // 如果这个选择器查找到了元素，就不再查找下一个选择器，以免重复查找
                                if (elList.length > 0) {
                                    break;
                                }
                            }
                            // 如果前面没有找到缩略图容器，则尝试其他办法
                            // 注意，这里使用的选择器不是容器本身的选择器，而是容器的子元素，所以需要单独处理
                            // if (wrapList.length === 0) {
                            //   const elList = e.querySelectorAll('div[width="184"]')
                            //   for (const el of elList) {
                            //     if(el.parentNode?.nodeName === 'LI'){
                            //       wrapList.push(el.parentNode as HTMLLIElement)
                            //     }
                            //   }
                            // }
                        }
                        // 监视缩略图容器
                        for (const wrap of wrapList) {
                            this.obWorkWrap(wrap);
                        }
                    }
                }
            }
        });
        ob.observe(document.body, {
            childList: true,
            subtree: true,
        });
    }
    // 监视作品缩略图容器内部的 img 元素
    obWorkWrap(wrap) {
        // 已经有 img 元素的情况
        if (_PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.list.ArtworkSearch) {
            // .searchList 是下载器在搜索页面生成的元素，里面一开始就有 img 元素，所以不需要监视
            if (wrap.classList.contains('searchList') || wrap.nodeName === 'LI') {
                const img = wrap.querySelector('img');
                this.readyCheckImage(img, wrap);
                return;
            }
        }
        // 如果是动态生成 img 的情况，则需要对 wrap 使用监视器
        const ob = new MutationObserver((records) => {
            for (const record of records) {
                // 生成作品缩略图内部的 img 时，addedNodes 数组里只有 img 标签这一个元素
                if (record.addedNodes.length === 1 &&
                    record.addedNodes[0].nodeName === 'IMG') {
                    const img = record.addedNodes[0];
                    this.readyCheckImage(img, wrap, ob);
                }
            }
        });
        ob.observe(wrap, {
            childList: true,
            subtree: true,
        });
    }
    // 当图片加载完成时检查它的宽高，并取消监视
    readyCheckImage(img, wrap, ob) {
        if (img?.complete) {
            this.checkImage(img, wrap);
            ob && ob.disconnect();
        }
        else {
            img &&
                (img.onload = () => {
                    this.checkImage(img, wrap);
                    ob && ob.disconnect();
                });
        }
    }
    // 当 img 加载完成后，计算 img 是横图还是竖图
    checkImage(img, wrap) {
        if (!img.src.includes('1200.jpg')) {
            return;
        }
        if (img.naturalWidth / img.naturalHeight > 1) {
            this.find(wrap);
        }
    }
    // 注册回调函数
    onFind(cb) {
        this.onFindCB.push(cb);
    }
    // 当找到横图的容器时会执行回调函数
    find(wrap) {
        for (const cb of this.onFindCB) {
            cb(wrap);
        }
    }
}
const findHorizontalImageWrap = new FindHorizontalImageWrap();



/***/ }),

/***/ "./src/ts/HiddenBrowserDownloadBar.ts":
/*!********************************************!*\
  !*** ./src/ts/HiddenBrowserDownloadBar.ts ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");

class HiddenBrowserDownloadBar {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'hiddenBrowserDownloadBar') {
                chrome.runtime.sendMessage({
                    msg: 'setShelfEnabled',
                    value: !data.value,
                });
                // 如果这个设置为 true，则应该向后台传递 false
            }
        });
    }
}
new HiddenBrowserDownloadBar();


/***/ }),

/***/ "./src/ts/HighlightFollowingUsers.ts":
/*!*******************************************!*\
  !*** ./src/ts/HighlightFollowingUsers.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");










class HighlightFollowingUsers {
    constructor() {
        this.pageTheme = '';
        /**当前登录用户的关注用户列表 */
        this.following = [];
        /**当前登录用户的关注用户总数 */
        this.total = 0;
        this.highlightClassName = 'pbdHighlightFollowing';
        // 检查包含用户 id 的链接，并且需要以 id 结束
        // 这是因为 id 之后还有字符的链接是不需要的，例如：
        // https://www.pixiv.net/en/users/17207914/artworks
        // 下载器只匹配用户主页的链接，不匹配用户子页面的链接
        this.checkUserLinkReg = /\/users\/(\d+)$/;
        if (!_utils_Utils__WEBPACK_IMPORTED_MODULE_5__.Utils.isPixiv()) {
            return;
        }
        this.delayCheckUpdate();
        window.setTimeout(() => {
            this.startMutationObserver();
        }, 0);
        chrome.runtime.onMessage.addListener(async (msg, sender, sendResponse) => {
            if (msg.msg === 'dispathFollowingData') {
                this.receiveData(msg.data);
            }
            if (msg.msg === 'updateFollowingData') {
                const following = await this.getList();
                chrome.runtime.sendMessage({
                    msg: 'setFollowingData',
                    data: {
                        user: _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID,
                        following: following,
                        total: this.total,
                    },
                });
            }
            if (msg.msg === 'getLoggedUserID') {
                sendResponse({ loggedUserID: _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID });
            }
        });
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID) {
            chrome.runtime.sendMessage({
                msg: 'requestFollowingData',
            });
        }
        // 每当下载器获取了页面的主题颜色时
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.getPageTheme, (ev) => {
            if (ev.detail.data) {
                if (this.pageTheme !== ev.detail.data) {
                    // 当用户改变页面主题时，一些页面元素会重新生成，但是目前的代码不能监听到这个变化
                    // 所以需要来更新高亮状态
                    window.setTimeout(() => {
                        this.makeHighlight();
                    }, 0);
                }
                this.pageTheme = ev.detail.data;
                // 给 html 标签添加自定义 data 属性，这是因为原本的 html 标签在没有任何 data 属性的时候，
                // 可能是普通模式，也可能是夜间模式，所以下载器必须自行添加一个属性，
                // 才能让高亮样式在不同模式中有不同的效果
                document.documentElement.setAttribute('data-xzpagetheme', this.pageTheme);
            }
        });
        // 在作品页内，作品大图下方和右侧的作者名字变化时，监视器无法监测到变化，尤其是右侧的名字
        // 所以用定时器执行
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch, () => {
            if (!_Config__WEBPACK_IMPORTED_MODULE_9__.Config.mobile &&
                (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Artwork ||
                    _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Novel)) {
                let time = 0;
                let interval = 500;
                let timer = window.setInterval(() => {
                    time = time + interval;
                    if (time > 5000) {
                        window.clearInterval(timer);
                    }
                    const leftA = document.querySelectorAll('#root main a[href*=user]');
                    const rightA = document.querySelectorAll('#root main+aside a[href*=user]');
                    const allA = Array.from(leftA).concat(Array.from(rightA));
                    this.makeHighlight(allA);
                }, interval);
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'highlightFollowingUsers') {
                if (!data.value) {
                    this.clearHighlight();
                }
                else {
                    this.makeHighlight();
                }
            }
        });
    }
    async receiveData(list) {
        const thisUserData = list.find((data) => data.user === _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID);
        if (thisUserData) {
            this.following = thisUserData.following;
            this.total = thisUserData.total;
            this.makeHighlight();
        }
        else {
            // 恢复的数据里没有当前用户的数据，需要获取
            this.checkNeedUpdate();
        }
    }
    /**全量获取当前用户的所有关注列表 */
    async getList() {
        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_8__.lang.transl('_正在加载关注用户列表'), {
            position: 'topCenter',
        });
        // 需要获取公开关注和私密关注
        const publicList = await this.getFollowingList('show');
        const privateList = await this.getFollowingList('hide');
        const followingIDList = publicList.concat(privateList);
        return followingIDList;
    }
    /**获取公开或私密关注的用户 ID 列表 */
    async getFollowingList(rest) {
        const ids = [];
        let offset = 0;
        let total = await this.getFollowingTotal(rest);
        if (total === 0) {
            return ids;
        }
        // 每次请求 100 个关注用户的数据
        const limit = 100;
        while (ids.length < total) {
            const res = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getFollowingList(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID, rest, '', offset, limit);
            offset = offset + limit;
            for (const users of res.body.users) {
                ids.push(users.userId);
            }
            if (res.body.users.length === 0) {
                // 实际获取到的关注用户数量可能比 total 少，这是正常的
                // 例如 toal 是 3522，实际上获取到的可能是 3483 个，再往后都是空数组了
                break;
            }
        }
        return ids;
    }
    /**只请求第一页的数据，以获取 total */
    async getFollowingTotal(rest) {
        const res = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getFollowingList(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID, rest, '', 0, 24);
        return res.body.total;
    }
    getUpdateTime() {
        // 每次检查更新的最低时间间隔是 5 分钟
        // 如果用户打开了多个标签页，它们都会加载关注列表的第一页来检查数量
        // 所以间隔不宜太短
        const base = 300000;
        // 产生一个 10 分钟内的随机数
        const random = Math.random() * 600000;
        // 通常不需要担心间隔时间太大导致数据更新不及时
        // 因为多个标签页里只要有一个更新了数据，所有的标签页都会得到新数据
        return base + random;
    }
    async delayCheckUpdate() {
        window.clearTimeout(this.checkUpdateTimer);
        this.checkUpdateTimer = window.setTimeout(async () => {
            this.checkNeedUpdate();
            return this.delayCheckUpdate();
        }, this.getUpdateTime());
    }
    /**检查关注用户的数量，如果数量发生变化则执行全量更新 */
    async checkNeedUpdate() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.highlightFollowingUsers) {
            return;
        }
        // 因为本程序不区分公开和非公开关注，所以只储存总数
        let newTotal = 0;
        for (const rest of ['show', 'hide']) {
            const total = await this.getFollowingTotal(rest);
            newTotal = newTotal + total;
        }
        if (newTotal !== this.total) {
            // console.log(`关注用户总数量变化 ${this.total} -> ${newTotal}`)
            this.total = newTotal;
            chrome.runtime.sendMessage({
                msg: 'needUpdateFollowingData',
                user: _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.loggedUserID,
            });
        }
    }
    makeHighlight(aList) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.highlightFollowingUsers) {
            return;
        }
        // 这里不需要检查 this.followingList.length === 0 的情况
        // 因为可能之前的数量是 1，之后用户取消关注，变成了 0，那么下面的代码依然需要执行
        // 以把之前高亮过的元素取消高亮
        const allA = aList || document.querySelectorAll('a');
        for (const a of allA) {
            let match = false;
            if (a.href) {
                // 小说排行榜里的用户链接普遍带有 /novels 后缀，所以不要求以用户 id 结尾
                const test = a.href.match(_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.NovelRanking
                    ? /\/users\/(\d+)/
                    : this.checkUserLinkReg);
                if (test && test.length > 1) {
                    match = this.following.includes(test[1]);
                    // 要高亮的元素
                    let target = a;
                    // 如果用户链接的 a 标签包含子元素，则将 className 添加到它的某个子元素上
                    // 这是为了尽量精确的只高亮用户链接，避免高亮区域里包含其他不必要的元素
                    // 但是有些页面里不适合这样做，例如在排行榜页面里，a 标签的第一个元素是用户头像
                    // 此时如果只高亮第一个元素，那么效果就很不明显，所以就需要高亮整个 a 标签
                    // 在多数情况下，高亮第一个子元素
                    if (a.firstChild && a.firstChild.nodeType === 1) {
                        target = a.firstChild;
                    }
                    // 在某些页面里高亮最后一个子元素
                    if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.ArtworkRanking ||
                        _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.AreaRanking) {
                        if (a.lastChild && a.lastChild.nodeType === 1) {
                            target = a.lastChild;
                        }
                    }
                    target.classList[match ? 'add' : 'remove'](this.highlightClassName);
                }
            }
        }
        this.handleUserHomePage();
    }
    startMutationObserver() {
        const observer = new MutationObserver((mutations) => {
            for (const mutation of mutations) {
                if (mutation.addedNodes.length > 0) {
                    for (const addedNodes of mutation.addedNodes) {
                        if (addedNodes.nodeName === 'A') {
                            // 直接是 A 标签的情况
                            this.makeHighlight(Array.from([addedNodes]));
                        }
                        else {
                            // addedNodes 也会包含纯文本，所以需要判断 nodeType
                            // https://developer.mozilla.org/zh-CN/docs/Web/API/Node/nodeType
                            if (addedNodes.nodeType === 1) {
                                // 如果是元素，则查找里面的 A 标签
                                const allA = addedNodes.querySelectorAll('a');
                                this.makeHighlight(Array.from(allA));
                            }
                            else {
                                // 如果不是元素，而且它也不是 A 标签，则尝试查找它的父元素是不是 A 标签
                                // 这是因为在作品页内，作品大图下方和右侧的作者名字变化时，上面的代码无法监测到
                                // 但是这段代码只能检测到下方的，右侧的还是监测不到。而且可能拖累性能，所以我注释掉了
                                // const parent = mutation.target.parentElement
                                // if (parent && parent.nodeName === 'A') {
                                //   this.makeHighlight([parent as HTMLAnchorElement])
                                // }
                            }
                        }
                    }
                }
            }
        });
        // 注意：本模块最好不要监听 attributes 变化，因为本模块自己就会修改元素的 attributes
        // 监听 attributes 并进行处理可能导致一些代码重复执行，或者死循环
        observer.observe(document.body, {
            childList: true,
            subtree: true,
        });
    }
    handleUserHomePage() {
        if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.UserHome) {
            // 在用户主页里，高亮用户名（因为用户名没有超链接，需要单独处理）
            const userID = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getUserId();
            const flag = this.following.includes(userID);
            const h1 = document.querySelector('h1');
            if (h1) {
                h1.classList[flag ? 'add' : 'remove'](this.highlightClassName);
            }
            // 取消用户主页里“主页”按钮的高亮，它具有用户主页链接，但它不是用户名
            const selector = _Config__WEBPACK_IMPORTED_MODULE_9__.Config.mobile ? '.v-nav-tabs a' : 'nav a';
            const homeBtn = document.querySelector(selector);
            if (homeBtn) {
                homeBtn.classList.remove(this.highlightClassName);
            }
        }
    }
    clearHighlight() {
        const allElement = document.querySelectorAll('.' + this.highlightClassName);
        for (const el of allElement) {
            el.classList.remove(this.highlightClassName);
        }
    }
}
new HighlightFollowingUsers();


/***/ }),

/***/ "./src/ts/ImageViewer.ts":
/*!*******************************!*\
  !*** ./src/ts/ImageViewer.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ImageViewer: () => (/* binding */ ImageViewer)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Loading__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Loading */ "./src/ts/Loading.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./store/CacheWorkData */ "./src/ts/store/CacheWorkData.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./download/DownloadOnClickBookmark */ "./src/ts/download/DownloadOnClickBookmark.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/// <reference path = "./ImageViewer.d.ts" />












// 对 Viewer 进行修改以供下载器使用
// 原版是接收页面上已存在的缩略图列表，但在下载器里它需要从作品 id 获取数据，生成缩略图列表。并且需要进行一些改造
class ImageViewer {
    // new() 不会创建图片查看器，需要再手动执行 init()
    // 这是因为有的模块需要获取异步操作之后生成的元素，但是构造函数无法返回异步操作，所以使用 init() 进行包装
    constructor(cfg) {
        this.viewerUl = document.createElement('ul'); // 图片列表的 ul 元素
        this.show = false; // 当前查看器实例是否处于显示状态
        this.isOriginalSize = false; // 是否原尺寸显示图片
        this.pageCount = 1;
        this.firstImageURL = ''; // 第一张图片的 url
        // 默认配置
        this.cfg = {
            workId: _Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.getIllustId(),
            imageNumber: 2,
            imageSize: 'original',
            autoStart: false,
            showLoading: false,
        };
        this.addBtnClass = 'viewer-add-btn';
        this.cfg = Object.assign(this.cfg, cfg);
    }
    async init() {
        // 删除旧的图片查看器元素
        const oldViewerContainer = document.querySelector('.viewer-container');
        oldViewerContainer && oldViewerContainer.remove();
        const wrap = await this.createImageList();
        if (wrap) {
            this.bindHotKey();
            this.configureViewer();
        }
        return wrap;
    }
    // 事件会重复绑定，设计如此，这是因为每次绑定时的 this 是不同的，必须重新绑定。而且不会冲突
    bindHotKey() {
        // 按 F 进入/退出 1:1 查看模式
        document.addEventListener('keydown', (event) => {
            if (event.code === 'KeyF') {
                if (this.show) {
                    this.isOriginalSize = !this.isOriginalSize;
                    this.setOriginalSize();
                }
            }
        });
        // 按 Alt + B 收藏当前作品
        // 因为 Pixiv 会在按下 B 键时收藏当前作品，所以下载器不能使用 B 键。尝试阻止 Pixiv 的事件但是没有成功
        document.addEventListener('keydown', (event) => {
            if (event.altKey && event.code === 'KeyB') {
                if (this.show) {
                    this.addBookmark();
                }
            }
        });
        // 按 D 下载当前作品
        document.addEventListener('keydown', (event) => {
            if (event.code === 'KeyD') {
                if (this.show) {
                    this.download();
                }
            }
        });
        // 监听左右方向键，防止在看图时，左右方向键导致 Pixiv 切换作品
        window.addEventListener('keydown', (event) => {
            if (event.code === 'ArrowLeft' || event.code === 'ArrowRight') {
                if (this.show) {
                    // 阻止事件冒泡
                    event.stopPropagation();
                    // 控制切换到上一张或者下一张
                    // true 表示启用循环切换
                    event.code === 'ArrowLeft'
                        ? this.myViewer.prev(true)
                        : this.myViewer.next(true);
                }
            }
        }, true);
    }
    // 图片查看器需要一个图片列表元素，创建缩略图列表
    async createImageList() {
        return new Promise(async (resolve) => {
            // 获取作品数据
            if (_store_CacheWorkData__WEBPACK_IMPORTED_MODULE_8__.cacheWorkData.has(this.cfg.workId)) {
                this.workData = _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_8__.cacheWorkData.get(this.cfg.workId);
            }
            else {
                this.cfg.showLoading && (_Loading__WEBPACK_IMPORTED_MODULE_3__.loading.show = true);
                const unlisted = _PageType__WEBPACK_IMPORTED_MODULE_11__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_11__.pageType.list.Unlisted;
                const data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(this.cfg.workId, unlisted);
                this.workData = data;
                _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_8__.cacheWorkData.set(data);
                this.cfg.showLoading && (_Loading__WEBPACK_IMPORTED_MODULE_3__.loading.show = false);
            }
            const body = this.workData.body;
            // 处理插画、漫画、动图作品，不处理其他类型的作品
            if (body.illustType === 0 ||
                body.illustType === 1 ||
                body.illustType === 2) {
                // 如果图片数量达到指定值，则会创建创建缩略图，启用图片查看器
                if (body.pageCount >= this.cfg.imageNumber) {
                    this.pageCount = body.pageCount;
                    this.firstImageURL =
                        body.urls[this.cfg.imageSize] || body.urls.original;
                    // 缩略图列表的结构： div > ul > li > img
                    this.viewerWarpper = document.createElement('div');
                    this.viewerUl = document.createElement('ul');
                    this.viewerUl.classList.add('beautify_scrollbar');
                    this.viewerWarpper.appendChild(this.viewerUl);
                    this.viewerWarpper.style.display = 'none';
                    // 生成 UL 里面的缩略图列表
                    let html = [];
                    for (let index = 0; index < body.pageCount; index++) {
                        const str = `<li><img src="${_Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.convertThumbURLTo540px(body.urls.thumb.replace('p0', 'p' + index))}" data-src="${this.firstImageURL.replace('p0', 'p' + index)}"></li>`;
                        html.push(str);
                    }
                    this.viewerUl.innerHTML = html.join('');
                }
            }
            return resolve(this.viewerWarpper);
        });
    }
    // 配置图片查看器
    configureViewer() {
        // 图片查看器显示之后
        this.viewerUl.addEventListener('shown', () => {
            this.show = true;
            this.addDownloadBtn();
            this.addBookmarkBtn();
            // 如果图片数量只有 1 个，则不显示缩略图一栏
            const navbar = document.querySelector('.viewer-navbar');
            if (navbar) {
                // 控制不透明度，这样它依然会占据空间，不会导致工具栏下移
                navbar.style.opacity = this.pageCount > 1 ? '1' : '0';
            }
            // 点击 1：1 按钮时
            const oneToOne = document.querySelector('.viewer-one-to-one');
            if (oneToOne) {
                oneToOne.setAttribute('title', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_原始尺寸') + ' (F)');
                oneToOne.addEventListener('click', (ev) => {
                    // 阻止冒泡，否则放大过程中会多一次闪烁（推测可能是这个按钮原有的事件导致的，停止冒泡之后就好了）
                    ev.stopPropagation();
                    this.isOriginalSize = !this.isOriginalSize;
                    this.setOriginalSize();
                }, true);
            }
        });
        // 退出图片查看器时（可能尚未完全退出）
        this.viewerUl.addEventListener('hide', () => {
            this.show = false;
        });
        // 查看每一张图片时，如果处于 1:1 模式，就把图片缩放到 100%
        // viewed 事件是图片加载完成时触发的
        this.viewerUl.addEventListener('viewed', () => {
            if (this.isOriginalSize) {
                this.setOriginalSize();
            }
        });
        // 因为选项里的 size 是枚举类型，所以在这里也要定义一个枚举
        let ToolbarButtonSize;
        (function (ToolbarButtonSize) {
            ToolbarButtonSize["Small"] = "small";
            ToolbarButtonSize["Medium"] = "medium";
            ToolbarButtonSize["Large"] = "large";
        })(ToolbarButtonSize || (ToolbarButtonSize = {}));
        // 配置新的看图组件
        const handleToTop = this.moveToTop.bind(this);
        const pageCount = this.pageCount;
        const firstImageURL = this.firstImageURL;
        this.myViewer = new Viewer(this.viewerUl, {
            toolbar: {
                zoomIn: 0,
                zoomOut: 0,
                oneToOne: 1,
                reset: 0,
                prev: 1,
                play: {
                    show: 0,
                    size: ToolbarButtonSize.Large,
                },
                next: 1,
                rotateLeft: 0,
                rotateRight: 0,
                flipHorizontal: 0,
                flipVertical: 0,
            },
            url(image) {
                return image.dataset.src;
            },
            viewed(ev) {
                handleToTop();
                // 当图片显示完成（加载完成）后，预加载下一张图片
                let index = ev.detail.index;
                if (index < pageCount - 1) {
                    index++;
                }
                const nextImg = firstImageURL.replace('p0', 'p' + index);
                const img = new Image();
                img.src = nextImg;
            },
            // 取消一些动画，比如切换图片时，图片从小变大出现的动画
            transition: false,
            keyboard: true,
            // 不显示 title（图片名和宽高信息）
            title: false,
            // 不显示缩放比例
            tooltip: false,
        });
        // 预加载第一张图片
        const img = new Image();
        img.src = firstImageURL;
        if (this.cfg.autoStart) {
            this.myViewer.show();
        }
    }
    // 设置原始尺寸显示
    setOriginalSize() {
        if (this.isOriginalSize) {
            // 1:1 显示图片
            this.myViewer.zoomTo(1);
            this.moveToTop();
        }
        else {
            // 缩小图片以适应可视区域
            const w = this.myViewer.image.naturalWidth;
            const h = this.myViewer.image.naturalHeight;
            const vw = this.myViewer.viewerData.width * 0.9;
            const vh = this.myViewer.viewerData.height * 0.9;
            const wScale = vw / w;
            const hScale = vh / h;
            let scale = Math.min(wScale, hScale);
            if (scale >= 1) {
                return;
            }
            this.myViewer.zoomTo(scale);
            const nowTop = Number.parseInt(this.myViewer.image.style.marginTop);
            this.myViewer.move(0, vh * 0.05 - nowTop);
        }
    }
    // 如果图片的高度超出可视区域高度，则从图片的顶部开始显示
    moveToTop() {
        const img = this.myViewer.image;
        const windowHeight = window.innerHeight;
        if (img.height <= windowHeight) {
            return;
        }
        // 如果图片高度大于视口高度，让它从顶部显示
        // 目的是把图片的 marginTop 设为 0，但不能直接修改 marginTop，否则鼠标拖动图片时会抖动
        const nowTop = Number.parseInt(this.myViewer.image.style.marginTop);
        this.myViewer.move(0, 0 - nowTop);
    }
    /**在图片查看器的工具栏里添加按钮
     *
     * 元素必须具有 id 属性，用于区分
     */
    addBtn(btn) {
        // 最后的查看器元素就是最新添加的查看器
        const allContainer = document.querySelectorAll('.viewer-container');
        const last = allContainer[allContainer.length - 1];
        const test = last.querySelector('#' + btn.id);
        if (test) {
            return;
        }
        const one2one = last.querySelector('.viewer-one-to-one');
        if (one2one) {
            return one2one.insertAdjacentElement('afterend', btn);
        }
        else {
            console.error('Add btn failed');
        }
    }
    // 在图片查看器里添加下载按钮
    addDownloadBtn() {
        const li = document.createElement('li');
        li.setAttribute('role', 'button');
        li.setAttribute('title', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载') + ' (D)');
        li.classList.add(this.addBtnClass);
        li.textContent = '↓';
        li.id = 'imageViewerDownloadBtn';
        this.addBtn(li);
        li.addEventListener('click', () => {
            this.download();
        });
    }
    // 在图片查看器里添加收藏按钮
    addBookmarkBtn() {
        const btn = document.createElement('li');
        btn.setAttribute('role', 'button');
        btn.setAttribute('title', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_收藏') + ' (Alt + B)');
        btn.classList.add(this.addBtnClass);
        btn.style.fontSize = '14px';
        btn.textContent = '✩';
        btn.id = 'imageViewerBookmarkBtn';
        this.addBtn(btn);
        btn.addEventListener('click', async () => {
            // 添加收藏
            this.addBookmark();
            // 下载这个作品
            _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_10__.downloadOnClickBookmark.send(this.workData.body.illustId);
        });
    }
    async addBookmark() {
        // 显示提示
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_收藏'), {
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_9__.Colors.bgBlue,
        });
        const res = await _Bookmark__WEBPACK_IMPORTED_MODULE_7__.bookmark.add(this.cfg.workId, 'illusts', _Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.extractTags(this.workData));
        if (res === 200) {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已收藏'));
        }
    }
    // 下载当前查看的作品
    download() {
        _store_States__WEBPACK_IMPORTED_MODULE_4__.states.quickCrawl = true;
        // 发送要下载的作品 id
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('crawlIdList', [
            {
                id: this.cfg.workId,
                type: 'illusts',
            },
        ]);
        // 显示提示
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已发送下载请求'), {
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_9__.Colors.bgBlue,
        });
    }
}



/***/ }),

/***/ "./src/ts/InitPage.ts":
/*!****************************!*\
  !*** ./src/ts/InitPage.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _crawlMixedPage_InitHomePage__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./crawlMixedPage/InitHomePage */ "./src/ts/crawlMixedPage/InitHomePage.ts");
/* harmony import */ var _crawlArtworkPage_InitArtworkPage__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./crawlArtworkPage/InitArtworkPage */ "./src/ts/crawlArtworkPage/InitArtworkPage.ts");
/* harmony import */ var _crawlMixedPage_InitUserPage__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./crawlMixedPage/InitUserPage */ "./src/ts/crawlMixedPage/InitUserPage.ts");
/* harmony import */ var _crawlMixedPage_InitBookmarkLegacyPage__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./crawlMixedPage/InitBookmarkLegacyPage */ "./src/ts/crawlMixedPage/InitBookmarkLegacyPage.ts");
/* harmony import */ var _crawlMixedPage_InitBookmarkPage__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./crawlMixedPage/InitBookmarkPage */ "./src/ts/crawlMixedPage/InitBookmarkPage.ts");
/* harmony import */ var _crawlArtworkPage_InitSearchArtworkPage__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./crawlArtworkPage/InitSearchArtworkPage */ "./src/ts/crawlArtworkPage/InitSearchArtworkPage.ts");
/* harmony import */ var _crawlArtworkPage_InitAreaRankingPage__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./crawlArtworkPage/InitAreaRankingPage */ "./src/ts/crawlArtworkPage/InitAreaRankingPage.ts");
/* harmony import */ var _crawlArtworkPage_InitRankingArtworkPage__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./crawlArtworkPage/InitRankingArtworkPage */ "./src/ts/crawlArtworkPage/InitRankingArtworkPage.ts");
/* harmony import */ var _crawlArtworkPage_InitPixivisionPage__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./crawlArtworkPage/InitPixivisionPage */ "./src/ts/crawlArtworkPage/InitPixivisionPage.ts");
/* harmony import */ var _crawlArtworkPage_InitBookmarkDetailPage__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./crawlArtworkPage/InitBookmarkDetailPage */ "./src/ts/crawlArtworkPage/InitBookmarkDetailPage.ts");
/* harmony import */ var _crawlMixedPage_InitBookmarkNewPage__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ./crawlMixedPage/InitBookmarkNewPage */ "./src/ts/crawlMixedPage/InitBookmarkNewPage.ts");
/* harmony import */ var _crawlArtworkPage_InitDiscoverPage__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./crawlArtworkPage/InitDiscoverPage */ "./src/ts/crawlArtworkPage/InitDiscoverPage.ts");
/* harmony import */ var _crawlArtworkPage_InitNewArtworkPage__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ./crawlArtworkPage/InitNewArtworkPage */ "./src/ts/crawlArtworkPage/InitNewArtworkPage.ts");
/* harmony import */ var _crawlNovelPage_InitNovelPage__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ./crawlNovelPage/InitNovelPage */ "./src/ts/crawlNovelPage/InitNovelPage.ts");
/* harmony import */ var _crawlNovelPage_InitNovelSeriesPage__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./crawlNovelPage/InitNovelSeriesPage */ "./src/ts/crawlNovelPage/InitNovelSeriesPage.ts");
/* harmony import */ var _crawlNovelPage_InitSearchNovelPage__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ./crawlNovelPage/InitSearchNovelPage */ "./src/ts/crawlNovelPage/InitSearchNovelPage.ts");
/* harmony import */ var _crawlNovelPage_InitRankingNovelPage__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ./crawlNovelPage/InitRankingNovelPage */ "./src/ts/crawlNovelPage/InitRankingNovelPage.ts");
/* harmony import */ var _crawlNovelPage_InitNewNovelPage__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ./crawlNovelPage/InitNewNovelPage */ "./src/ts/crawlNovelPage/InitNewNovelPage.ts");
/* harmony import */ var _crawlArtworkPage_InitArtworkSeriesPage__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! ./crawlArtworkPage/InitArtworkSeriesPage */ "./src/ts/crawlArtworkPage/InitArtworkSeriesPage.ts");
/* harmony import */ var _crawlMixedPage_InitFollowingPage__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! ./crawlMixedPage/InitFollowingPage */ "./src/ts/crawlMixedPage/InitFollowingPage.ts");
/* harmony import */ var _crawl_InitUnsupportedPage__WEBPACK_IMPORTED_MODULE_22__ = __webpack_require__(/*! ./crawl/InitUnsupportedPage */ "./src/ts/crawl/InitUnsupportedPage.ts");
/* harmony import */ var _crawlMixedPage_InitUnlistedPage__WEBPACK_IMPORTED_MODULE_23__ = __webpack_require__(/*! ./crawlMixedPage/InitUnlistedPage */ "./src/ts/crawlMixedPage/InitUnlistedPage.ts");
// 根据页面类型来初始化抓取流程和一些特定的功能
























class InitPage {
    constructor() {
        this.initPage();
        // 页面类型变化时，初始化抓取流程
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitchedTypeChange, () => {
            setTimeout(() => {
                this.initPage();
            }, 0);
        });
    }
    initPage() {
        switch (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type) {
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Home:
                return new _crawlMixedPage_InitHomePage__WEBPACK_IMPORTED_MODULE_2__.InitHomePage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Artwork:
                return new _crawlArtworkPage_InitArtworkPage__WEBPACK_IMPORTED_MODULE_3__.InitArtworkPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.UserHome:
                return new _crawlMixedPage_InitUserPage__WEBPACK_IMPORTED_MODULE_4__.InitUserPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.BookmarkLegacy:
                return new _crawlMixedPage_InitBookmarkLegacyPage__WEBPACK_IMPORTED_MODULE_5__.InitBookmarkLegacyPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Bookmark:
                return new _crawlMixedPage_InitBookmarkPage__WEBPACK_IMPORTED_MODULE_6__.InitBookmarkPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkSearch:
                return new _crawlArtworkPage_InitSearchArtworkPage__WEBPACK_IMPORTED_MODULE_7__.InitSearchArtworkPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.AreaRanking:
                return new _crawlArtworkPage_InitAreaRankingPage__WEBPACK_IMPORTED_MODULE_8__.InitAreaRankingPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkRanking:
                return new _crawlArtworkPage_InitRankingArtworkPage__WEBPACK_IMPORTED_MODULE_9__.InitRankingArtworkPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Pixivision:
                return new _crawlArtworkPage_InitPixivisionPage__WEBPACK_IMPORTED_MODULE_10__.InitPixivisionPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.BookmarkDetail:
                return new _crawlArtworkPage_InitBookmarkDetailPage__WEBPACK_IMPORTED_MODULE_11__.InitBookmarkDetailPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NewArtworkBookmark:
                return new _crawlMixedPage_InitBookmarkNewPage__WEBPACK_IMPORTED_MODULE_12__.InitBookmarkNewPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Discover:
                return new _crawlArtworkPage_InitDiscoverPage__WEBPACK_IMPORTED_MODULE_13__.InitDiscoverPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NewArtwork:
                return new _crawlArtworkPage_InitNewArtworkPage__WEBPACK_IMPORTED_MODULE_14__.InitNewArtworkPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Novel:
                return new _crawlNovelPage_InitNovelPage__WEBPACK_IMPORTED_MODULE_15__.InitNovelPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelSeries:
                return new _crawlNovelPage_InitNovelSeriesPage__WEBPACK_IMPORTED_MODULE_16__.InitNovelSeriesPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelSearch:
                return new _crawlNovelPage_InitSearchNovelPage__WEBPACK_IMPORTED_MODULE_17__.InitSearchNovelPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelRanking:
                return new _crawlNovelPage_InitRankingNovelPage__WEBPACK_IMPORTED_MODULE_18__.InitRankingNovelPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NewNovelBookmark:
                return new _crawlMixedPage_InitBookmarkNewPage__WEBPACK_IMPORTED_MODULE_12__.InitBookmarkNewPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NewNovel:
                return new _crawlNovelPage_InitNewNovelPage__WEBPACK_IMPORTED_MODULE_19__.InitNewNovelPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkSeries:
                return new _crawlArtworkPage_InitArtworkSeriesPage__WEBPACK_IMPORTED_MODULE_20__.InitArtworkSeriesPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Following:
                return new _crawlMixedPage_InitFollowingPage__WEBPACK_IMPORTED_MODULE_21__.InitFollowingPage();
            case _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Unlisted:
                return new _crawlMixedPage_InitUnlistedPage__WEBPACK_IMPORTED_MODULE_23__.InitUnlistedPage();
            default:
                return new _crawl_InitUnsupportedPage__WEBPACK_IMPORTED_MODULE_22__.InitUnsupportedPage();
        }
    }
}
new InitPage();


/***/ }),

/***/ "./src/ts/Input.ts":
/*!*************************!*\
  !*** ./src/ts/Input.ts ***!
  \*************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Input: () => (/* binding */ Input)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");



class Input {
    /**所有选项皆是可选的 */
    constructor(option) {
        this.defultOption = {
            width: 600,
            type: 'input',
            rows: 3,
            instruction: '',
            placeholder: '',
            value: '',
            submitButtonText: _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_提交'),
        };
        this.value = '';
        this.id = '';
        this.submitted = false;
        this.cancelled = false;
        this.init(option);
    }
    init(option) {
        const _option = Object.assign(this.defultOption, option || {});
        this.value = _option.value;
        this.id = `input` + new Date().getTime();
        this.create(_option);
    }
    create(option) {
        const example = `<div class="XZInputWrap ?:mobile" id="input1691811888224">
    <p class="XZInputInstruction">instruction</p>
    <div class="XZInputContainer">
      <input type="text" class="XZInput" value="default" placeholder="tip" />
      <textarea class="XZInput" placeholder="tip">default</textarea>
      <button class="XZInputButton">Submit</button>
      <button class="XZInputButton cancel">Cancel</button>
    </div>
  </div>`;
        const wrap = document.createElement('div');
        wrap.classList.add('XZInputWrap');
        _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile && wrap.classList.add('mobile');
        wrap.id = this.id;
        // 这里设置的宽度是粗略值，后面会再修改
        wrap.style.width = option.width + 200 + 'px';
        _Theme__WEBPACK_IMPORTED_MODULE_2__.theme.register(wrap);
        if (option.instruction) {
            const p = document.createElement('p');
            p.classList.add('XZInputInstruction');
            p.innerHTML = option.instruction;
            wrap.append(p);
        }
        const container = document.createElement('div');
        container.classList.add('XZInputContainer');
        const input = document.createElement(option.type);
        input.classList.add('XZInput');
        input.setAttribute('placeholder', option.placeholder);
        input.style.flexBasis = option.width + 'px';
        if (option.type === 'input') {
            input.setAttribute('type', 'text');
            input.setAttribute('value', option.value);
        }
        else {
            input.textContent = option.value;
            input.setAttribute('rows', option.rows.toString());
        }
        container.append(input);
        const submitButton = document.createElement('button');
        submitButton.classList.add('XZInputButton');
        submitButton.textContent = option.submitButtonText;
        container.append(submitButton);
        const cancelButton = document.createElement('button');
        cancelButton.classList.add('XZInputButton', 'cancel');
        cancelButton.textContent = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_取消');
        container.append(cancelButton);
        wrap.append(container);
        // 由于 wrap 宽度要考虑按钮宽度，但按钮宽度不固定，所以要先添加到页面上，获取按钮实际宽度，再调整 wrap 宽度
        wrap.style.opacity = '0';
        document.body.append(wrap);
        // 根据按钮宽度，重设 wrap 宽度
        const submitRect = submitButton.getClientRects();
        const cancelRect = cancelButton.getClientRects();
        // 14 是按钮的 margin-left 值
        wrap.style.width =
            option.width + 14 + submitRect[0].width + 14 + cancelRect[0].width + 'px';
        wrap.style.opacity = '1';
        input.focus();
        if (option.value) {
            input.setSelectionRange(option.value.length, option.value.length);
        }
        input.addEventListener('change', () => {
            this.value = input.value;
        });
        // 按 Esc 直接移除本组件，并且不会执行 onSubmit 回调
        input.addEventListener('keydown', (ev) => {
            if (ev.code === 'Escape') {
                this.remove();
            }
        });
        submitButton.addEventListener('click', () => {
            this.submitted = true;
            this.remove();
        });
        cancelButton.addEventListener('click', () => {
            this.cancelled = true;
            this.remove();
        });
    }
    remove() {
        const wrap = document.querySelector(`#${this.id}`);
        wrap && wrap.remove();
    }
    /**当用户点击提交按钮后，返回 value。注意：可能会返回空字符串
     * 如果用户点击取消按钮，则抛出 reject
     */
    submit() {
        return new Promise((resolve, reject) => {
            window.setTimeout(() => {
                if (this.cancelled) {
                    return reject('');
                }
                if (this.submitted) {
                    return resolve(this.value);
                }
                return resolve(this.submit());
            }, 100);
        });
    }
}



/***/ }),

/***/ "./src/ts/Lang.ts":
/*!************************!*\
  !*** ./src/ts/Lang.ts ***!
  \************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   lang: () => (/* binding */ lang)
/* harmony export */ });
/* harmony import */ var _LangText__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./LangText */ "./src/ts/LangText.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");


// 语言类
class Lang {
    constructor() {
        this.langTypes = ['zh-cn', 'zh-tw', 'en', 'ja', 'ko', 'ru'];
        this.flagIndex = new Map([
            ['zh-cn', 0],
            ['zh-tw', 1],
            ['en', 2],
            ['ja', 3],
            ['ko', 4],
            ['ru', 5],
        ]);
        // 保存注册的元素
        // 在注册的元素里设置特殊的标记，让本模块可以动态更新其文本
        this.elList = [];
        this.htmlLangType = this.getHtmlLangType();
        this.type = this.htmlLangType;
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name !== 'userSetLang') {
                return;
            }
            const old = this.type;
            this.type = data.value === 'auto' ? this.htmlLangType : data.value;
            if (this.type !== old) {
                _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('langChange');
                this.elList.forEach((el) => {
                    this.handleMark(el);
                });
            }
        });
    }
    // 获取页面使用的语言，返回语言标记
    getHtmlLangType() {
        const flag = document.documentElement.lang;
        switch (flag) {
            case 'zh':
            case 'zh-CN':
            case 'zh-Hans':
                return 'zh-cn'; // 简体中文
            case 'ja':
                return 'ja'; // 日本語
            case 'zh-Hant':
            case 'zh-tw':
            case 'zh-TW':
                return 'zh-tw'; // 繁體中文
            case 'ko':
                return 'ko'; // 한국어
            default:
                return 'en'; // English
        }
    }
    // translate
    transl(name, ...arg) {
        let content = _LangText__WEBPACK_IMPORTED_MODULE_0__.langText[name][this.flagIndex.get(this.type)];
        arg.forEach((val) => (content = content.replace('{}', val)));
        return content;
    }
    register(el) {
        this.elList.push(el);
        this.handleMark(el);
    }
    // 查找元素上的标记，设置其文本和属性
    handleMark(wrap) {
        // 设置 innerHTML
        const textEl = wrap.querySelectorAll('*[data-xztext]');
        for (const el of textEl) {
            // 因为有些文本中含有 html 标签，所以这里需要使用 innerHTML 而不是 textContent
            el.innerHTML = this.transl(el.dataset.xztext);
        }
        // 元素自身存在 xztext 标记的情况
        const text = wrap.dataset.xztext;
        if (text) {
            wrap.innerHTML = this.transl(text);
        }
        // 设置带参数的 innerHTML
        const textArgsEl = wrap.querySelectorAll('*[data-xztextargs]');
        textArgsEl.forEach((el) => this.handleTextArgs(el));
        // 元素自身存在 xztextargs 标记的情况
        const textargs = wrap.dataset.xztextargs;
        if (textargs) {
            this.handleTextArgs(wrap);
        }
        // 设置 tip
        const tipEl = wrap.querySelectorAll('*[data-xztip]');
        for (const el of tipEl) {
            el.dataset.tip = this.transl(el.dataset.xztip);
        }
        // 设置 placeholder
        const placeholderEl = wrap.querySelectorAll('*[data-xzplaceholder]');
        for (const el of placeholderEl) {
            el.setAttribute('placeholder', this.transl(el.dataset.xzplaceholder));
        }
        // 设置 title
        const titleEl = wrap.querySelectorAll('*[data-xztitle]');
        for (const el of titleEl) {
            el.setAttribute('title', this.transl(el.dataset.xztitle));
        }
        // 元素自身存在 title 标记的情况
        const title = wrap.dataset.xztitle;
        if (title) {
            wrap.setAttribute('title', this.transl(title));
        }
    }
    handleTextArgs(el) {
        let args = el.dataset.xztextargs.split(',');
        const first = args.shift();
        el.innerHTML = this.transl(first, ...args);
    }
    // 需要更新已注册元素的文本时调用此方法
    updateText(el, ...args) {
        // 清空文本的情况
        if (args === undefined || args[0] === '') {
            delete el.dataset.xztext;
            delete el.dataset.xztextargs;
            el.innerHTML = '';
            return;
        }
        // 设置文本
        if (args.length === 1) {
            // 无参数文本
            el.dataset.xztext = args[0];
            el.innerHTML = this.transl(args[0]);
            delete el.dataset.xztextargs;
        }
        else {
            // 有参数文本
            el.dataset.xztextargs = args.join(',');
            const first = args.shift();
            el.innerHTML = this.transl(first, ...args);
            delete el.dataset.xztext;
        }
    }
}
const lang = new Lang();



/***/ }),

/***/ "./src/ts/LangText.ts":
/*!****************************!*\
  !*** ./src/ts/LangText.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   langText: () => (/* binding */ langText)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");

// 储存下载器使用的多语言文本
// 在属性名前面加上下划线，和文本内容做出区别
// {} 是占位符
// <br> 是换行
const langText = {
    _只下载已收藏: [
        '只下载已收藏',
        '只下載已收藏',
        'Download only bookmarked works',
        'ブックマークのみをダウンロードする',
        '북마크된 작품만 다운로드',
        'Загружайте только работы, сохраненные в закладках',
    ],
    _下载作品类型: [
        '下载作品类型',
        '下載作品類型',
        'Download work type',
        'ダウンロード作品の種類',
        '다운로드할 작품 유형',
        'Типы контента для загрузки',
    ],
    _作品类型: [
        '作品<span class="key">类型</span>',
        '作品<span class="key">類型</span>',
        '<span class="key">Type</span> of work',
        '作品の<span class="key">種類</span>',
        '작품 <span class="key">유형</span>',
        'Тип <span class="key">работы</span>',
    ],
    _不能含有tag: [
        '<span class="key">不能</span>含有标签',
        '<span class="key">不能</span>含有標籤',
        '<span class="key">Exclude</span> tag',
        'タグを除外する',
        '<span class="key">제외</span> 태그',
        '<span class="key">Исключить</span> ярлык',
    ],
    _排除tag的提示文字: [
        '您可在下载前设置要排除的标签，这样在下载时将不会下载含有这些标签的作品。不区分大小写；如需排除多个标签，请使用英文逗号分隔。请注意，要排除的标签的优先级大于要包含的标签的优先级。',
        '可在下載前設定要排除的標籤，下載時將排除含有這些標籤的作品，不區分大小寫；如需排除多個標籤，請使用半形逗號（,）分隔。請注意，要排除的標籤優先於要包含的標籤。',
        'Before downloading, you can set the tag you want to exclude. Not case sensitive; If you need to set multiple tags, you can use comma (,) separated. The excluded tag takes precedence over the included tag',
        'ダウンロード前に、除外するタグを設定できます。大文字と小文字を区別しない；複数のタグを設定する必要がある場合は、「,」で区切ってください。除外されたタグは、必要なタグよりも優先されます',
        '다운로드하기 전에 제외해야 하는 태그를 설정할 수 있습니다. 대소문자를 구분하지 않습니다. 여러 태그를 설정해야 하는 경우 쉼표(,)로 구분합니다. 제외된 태그가 포함된 태그보다 우선합니다.',
        'Перед загрузкой можно задать тег, который необходимо исключить. Не чувствителен к регистру; Если вам нужно задать несколько тегов, вы можете использовать разделение запятыми (,). Исключенный тег имеет приоритет над включенным тегом',
    ],
    _设置了排除tag之后的提示: [
        '排除标签：',
        '排除標籤：',
        'Excludes tag: ',
        '以下のタグを除外：',
        '제외 태그: ',
        'Исключающий тег: ',
    ],
    _必须含有tag: [
        '<span class="key">必须</span>含有标签',
        '<span class="key">必須</span>含有標籤',
        '<span class="key">Include</span> tag',
        '<span class="key">必要な</span>タグ',
        '<span class="key">포함</span> 태그',
        '<span class="key">Включать</span> ярлык',
    ],
    _必须tag的提示文字: [
        '您可在下载前设置作品里必须包含的标签，不区分大小写；如需包含多个标签，请使用英文逗号分隔。',
        '可在下載前設定作品裡必須包含的標籤，不區分大小寫；如需包含多個標籤，請使用半形逗號（,）分隔。',
        'Before downloading, you can set the tag that must be included. Not case sensitive; If you need to set multiple tags, you can use comma (,) separated.',
        'ダウンロードする前に、必要なタグを設定することができます。大文字と小文字を区別しない；複数のタグを設定する必要がある場合は、「,」で区切ってください。',
        '다운로드하기 전에 포함해야 하는 태그를 설정할 수 있습니다. 대소문자를 구분하지 않습니다. 여러 태그를 설정해야 하는 경우 쉼표(,)로 구분합니다.',
        'Перед загрузкой можно задать тег, который должен быть включен. Не чувствителен к регистру; Если вам нужно задать несколько тегов, вы можете использовать разделение запятыми (,).',
    ],
    _设置了必须tag之后的提示: [
        '包含标签：',
        '包含標籤：',
        'Include tag: ',
        '以下の タグ を含める：',
        '포함 태그: ',
        'Включающий тег: ',
    ],
    _图片的宽高比例: [
        '图片的宽高<span class="key">比例</span>',
        '圖片的寬高<span class="key">比例</span>',
        'Aspect <span class="key">ratio</span>',
        '画像の縦横比',
        '<span class="key">종횡비</span>',
        'Сотношение <span class="key">сторон</span>',
    ],
    _设置宽高比例Title: [
        '设置宽高比例，也可以手动输入宽高比',
        '設定寬高比，也可以手動輸入寬高比。',
        'Set the aspect ratio, or manually type the aspect ratio',
        '縦横比を設定する、手動で縦横比を入力することもできる',
        '종횡비를 설정하거나, 값을 수동으로 입력할 수 있습니다.',
        'Установите соотношение сторон или введите соотношение сторон вручную',
    ],
    _不限制: [
        '不限制',
        '不限制',
        'not limited',
        '無制限',
        '제한 없음',
        'Без лимитов',
    ],
    _横图: ['横图', '橫圖', 'Horizontal', '横長', '가로', 'Горизонтальный'],
    _竖图: ['竖图', '豎圖', 'Vertical', '縦長', '세로', 'Вертикальный'],
    _正方形: ['正方形', '正方形', 'Square', '正方形', '정사각형', 'Квадрат(1:1)'],
    _宽高比: ['宽高比', '寬高比', 'Ratio', '縦横比', '종횡비 ', 'Соотношение'],
    _设置了宽高比之后的提示: [
        '宽高比：{}',
        '寬高比：{}',
        'Aspect ratio: {}',
        '縦横比：{}',
        '종횡비: {}',
        'Соотношение сторон: {}',
    ],
    _宽高比必须是数字: [
        '宽高比必须是数字',
        '寬高比必須是數字',
        'The aspect ratio must be a number',
        '縦横比は数値でなければなりません',
        '종횡비는 숫자여야 합니다',
        'Соотношение сторон должно быть числом',
    ],
    _图片的宽高: [
        '图片的<span class="key">宽高</span>',
        '圖片的<span class="key">寬高</span>',
        '<span class="key">width</span> and height',
        '画像の幅と高さ',
        '<span class="key">너비</span> 그리고 높이',
        '<span class="key">Ширина</span> и высота',
    ],
    _筛选宽高的提示文字: [
        '请输入最小宽度和最小高度，不会下载不符合要求的图片。',
        '請輸入最小寬度和最小高度，只會下載符合要求的圖片。',
        'Please type the minimum width and minimum height. Will not download images that do not meet the requirements',
        '最小幅と最小高さを入力してください。要件を満たしていない画像はダウンロードされません。',
        '최소 너비와 최소 높이를 입력해주세요, 요구 사항을 충족하지 않는 이미지는 다운로드하지 않습니다.',
        'Введите минимальную ширину и минимальную высоту. Не соответствующие требованиям изображения, загружаться не будут',
    ],
    _本次输入的数值无效: [
        '本次输入的数值无效',
        '本次輸入的數值無效',
        'Invalid input',
        '無効な入力',
        '잘못된 입력',
        'Недопустимый ввод',
    ],
    _宽度: ['宽度', '寬度', 'Width', '幅', '너비', 'Ширина'],
    _或者: [' 或者 ', ' 或是 ', ' or ', ' または ', '또는', 'или'],
    _并且: [' 并且 ', ' 並且 ', ' and ', ' そして ', '그리고', 'и'],
    _高度: ['高度', '高度', 'height', '高さ', '높이', 'высота'],
    _抓取多少作品: [
        '抓取<span class="key">多少</span>作品',
        '擷取<span class="key">多少</span>作品',
        'How <span class="key">many</span> works to crawl',
        'クロールする作品の数',
        '긁어올 작품 <span class="key">수</span>',
        'Какое <span class="key">колличество</span> работ сканировать',
    ],
    _抓取多少页面: [
        '抓取<span class="key">多少</span>页面',
        '擷取<span class="key">多少</span>頁面',
        'How <span class="key">many</span> pages to crawl',
        'クロールするページ数',
        '긁어올 페이지 <span class="key">수</span>',
        'Какое <span class="key">колличество</span> страниц сканироватьь',
    ],
    _收藏数量: [
        '<span class="key">收藏</span>数量',
        '<span class="key">收藏</span>數量',
        'Number of <span class="key">bookmarks</span>',
        'ブックマークの数',
        '<span class="key">북마크</span> 수',
        'Колличество <span class="key">закладок</span>',
    ],
    _设置收藏数量的提示: [
        '如果作品的收藏数小于设置的数字，作品不会被下载。',
        '只會下載設定收藏數範圍內的作品。',
        'If the number of bookmarks of the work is less than the set number, the work will not be downloaded.',
        '作品のブックマークされた数が設定された数字よりも少ない場合、作品はダウンロードされません。',
        '작품의 북마크 수가 설정된 값보다 적을 경우 작품은 다운로드되지 않습니다.',
        'Если количество закладок произведения меньше заданного, произведение не будет загружено',
    ],
    _筛选收藏数的提示文字: [
        '请输入一个数字，如果作品的收藏数小于这个数字，作品不会被下载。',
        '請輸入數字，只會下載設定收藏數範圍內的作品。',
        'Please type a number. If the number of bookmarks of the work is less than this number, the work will not be downloaded.',
        '数字を入力してください。 作品のブックマークされた数がこの数字より少ない場合、作品はダウンロードされません。',
        '숫자를 입력해주세요. 작품의 북마크 수가 이 수보다 적을 경우 작품은 다운로드되지 않습니다.',
        'Пожалуйста, введите число. Если количество закладок произведения меньше этого числа, произведение не будет загружено.',
    ],
    _收藏数大于: [
        '收藏数 >= ',
        '收藏數 >= ',
        'Number of bookmarks >= ',
        'ブックマークの数 >= ',
        '북마크 수 >= ',
        'Количество закладок >= ',
    ],
    _收藏数小于: [
        '收藏数 <= ',
        '收藏數 <= ',
        'Number of bookmarks <= ',
        'ブックマークの数 <= ',
        '북마크 수 <=',
        'Количество закладок <= ',
    ],
    _本次任务已全部完成: [
        '本次任务已全部完成。',
        '本次工作已全部完成。',
        'This task has been completed.',
        'この作業は完了しました。',
        '이 작업은 완료되었습니다.',
        'Эта задача была выполнена.',
    ],
    _本次任务条件: [
        '本次任务条件: ',
        '本次工作條件：',
        'This task condition: ',
        'この作業の条件：',
        '이 작업 조건: ',
        'Это условие задачи: ',
    ],
    _参数不合法本次操作已取消: [
        '参数不合法，本次操作已取消。',
        '參數不合法，本次動作已取消。',
        'Parameter is not legal, this operation has been canceled.',
        'パラメータは有効ではありません。この操作はキャンセルされました。',
        '매개변수가 잘못되었습니다, 이 작업은 취소됩니다.',
        'Параметр не является разрешенным, операция отменена.',
    ],
    _本次操作已取消: [
        '本次操作已取消',
        '本次動作已取消',
        'This operation has been canceled',
        'この操作はキャンセルされました',
        '이 작업이 취소되었습니다.',
        'Эта операция была отменена',
    ],
    _向下获取所有作品: [
        '向下获取所有作品',
        '向下取得所有作品',
        'download all the work from this page.',
        'このページからすべての作品をダウンロードする。',
        '모든 작품 다운로드',
        'загрузить все работы с этой страницы',
    ],
    _从本页开始下载提示: [
        '从当前页面开始下载。<br>如果要限制下载的页数，请输入从 1 开始的数字。<br>1 为仅下载本页，-1 为下载所有页面。',
        '從當前頁面開始下載。<br>如果要限制下載的頁數，請輸入從 1 開始的數字。<br>1 為僅下載本頁，-1 為下載所有頁面。',
        'Download from the current page.<br>If you want to set the number of pages to download, type a number starting at 1. <br>1 is to download only this page, -1 to download all pages.',
        '現在のページからダウンロードしてください。<br>ダウンロードするページを設定する場合は、1から始まる数字を入力してください。<br>1 は現在のページのみをダウンロードする，すべてのページをダウンロードするには -1。',
        '현재 페이지에서 다운로드합니다.<br>다운로드할 페이지 수를 설정하려면 1로 시작하는 숫자를 입력해주세요.<br>1은 이 페이지만 다운로드합니다, -1은 모든 페이지를 다운로드합니다.',
        'Загрузка с текущей страницы.<br>Если вы хотите задать количество страниц для загрузки, введите число, начиная с 1. <br>1 - это загрузка только этой страницы, -1 для загрузки всех страниц.',
    ],
    _下载所有页面: [
        '下载所有页面',
        '下載所有頁面',
        'download all pages',
        'すべてのページをダウンロードする',
        '모든 페이지 다운로드',
        'загрузить все страницы',
    ],
    _下载x个相关作品: [
        '下载 {} 个相关作品',
        '下載 {} 個相關作品',
        'download {} related works.',
        '関連作品 {} 枚をダウンロードする。',
        '관련 작품 {}개를 다운로드',
        'скачать {} связанные работы',
    ],
    _下载所有相关作品: [
        '下载所有相关作品',
        '下載所有相關作品',
        'download all related works.',
        '関連作品をすべてダウンロードする。',
        '모든 관련 작품 다운로드',
        'скачать все соответствующие работы.',
    ],
    _下载推荐作品: [
        '下载推荐作品',
        '下載推薦作品',
        'download recommended works',
        'おすすめ作品をダウンロードする',
        '추천 작품 다운로드',
        'скачать рекомендуемые работы',
    ],
    _下载排行榜前x个作品: [
        '下载排行榜前 {} 个作品',
        '下載排行榜前 {} 個作品',
        'download the top {} works in the ranking list',
        'ランク前 {} 位の作品をダウンロードする。',
        '랭킹 목록 상위 {}개의 작품 다운로드',
        'загрузить лучшие {} работы в рейтинговом списке',
    ],
    _输入超过了最大值: [
        '您输入的数字超过了最大值',
        '輸入的數字超出最大值',
        'The number you set exceeds the maximum',
        '入力した番号が最大値を超えています',
        '설정하신 수가 최대값을 초과합니다',
        'Заданное вами число превышает максимальное',
    ],
    _从本页开始下载x页: [
        '从本页开始下载 {} 页',
        '從本頁開始下載 {} 頁',
        'download {} pages from this page',
        'このページから {} ページをダウンロードする',
        '이 페이지부터 {} 페이지 다운로드',
        'Начать загрузку с страниц этой {} страницы',
    ],
    _从本页开始下载x个: [
        '从本页开始下载 {} 个作品',
        '從本頁開始下載 {} 個作品',
        'Download {} works from this page.',
        'このページから {} 枚の作品をダウンロード。',
        '이 페이지부터 {}개의 작품 다운로드',
        'Загрузить {} работы с этой страницы.',
    ],
    _任务开始: [
        '任务开始',
        '工作開始',
        'Task starts',
        '作業が開始されます',
        '작업 시작',
        'Задание начинается',
    ],
    _排除作品类型: [
        '排除作品类型：',
        '排除作品類型：',
        'Excludes these types of works: ',
        'これらのタイプの作品を除外：',
        '제외된 작품 유형: ',
        'Исключает эти виды работ: ',
    ],
    _多图作品: [
        '多图作品',
        '多圖作品',
        'Multi-image works',
        '複数画像作品',
        '여러 이미지 작품',
        'Работа с несколькими изображениями',
    ],
    _多图下载设置: [
        '多图下载设置',
        '多圖下載設定',
        'Download multi-image works',
        '複数画像設定',
        '여러 이미지 작품 다운로드',
        'Загрузить работы с несколькими изображениями',
    ],
    _不下载: ['不下载', '不下載', 'No', '必要なし', '아니요', 'Нет'],
    _全部下载: ['全部下载', '全部下載', 'Yes', '全部ダウンロード', '네', 'Да'],
    _插画: [
        '插画',
        '插畫',
        'Illustrations',
        'イラスト',
        '일러스트',
        'Иллюстрации',
    ],
    _漫画: ['漫画', '漫畫', 'Manga', '漫画', '만화', 'Манга'],
    _动图: [
        '动图',
        '動圖',
        'Ugoira',
        'うごイラ',
        '움직이는 일러스트',
        'Ugoira(гиф)',
    ],
    _动图保存格式: [
        '<span class="key">动图</span>保存格式',
        '<span class="key">動圖</span>儲存格式',
        'Save the <span class="key">ugoira</span> work as',
        'うごイラの保存タイプ',
        '<span class="key">움직이는 일러스트</span> 작품 저장 형식',
        'Сохранить <span class="key">Ugoira</span> как',
    ],
    _动图保存格式title: [
        '下载动图时，可以把它转换成视频文件',
        '下載動圖時，可轉換為影片檔。',
        'When you download a ugoira work, you can convert it to a video file.',
        'うごイラをダウンロードするとき、動画に変換することができます。',
        '움직이는 일러스트 작품을 다운로드 하면, 동영상 파일로 변환할 수 있습니다.',
        'Когда вы скачиваете произведение ugoira, вы можете конвертировать его в видеофайл.',
    ],
    _webmVideo: [
        'WebM 视频',
        '影片（WebM）',
        'WebM video',
        'WebM ビデオ',
        'WebM 동영상',
        'WebM видео',
    ],
    _gif: [
        'GIF 图片',
        '圖片（GIF）',
        'GIF image',
        'GIF 画像',
        'GIF 이미지',
        'GIF изображение',
    ],
    _apng: [
        'APNG 图片',
        '圖片（APNG）',
        'APNG image',
        'APNG 画像',
        'APNG 이미지',
        'APNG изображение',
    ],
    _zipFile: [
        'Zip 文件',
        '壓縮檔（Zip）',
        'Zip file',
        'ZIP ファイル',
        'Zip 파일',
        'Zip файл',
    ],
    _当前作品个数: [
        '当前有 {} 个作品',
        '目前有 {} 個作品',
        'There are now {} works',
        '今は　{}　枚の作品があります',
        '현재 {}개의 작품이 있습니다',
        'В настоящее время существует {} работ',
    ],
    _当前有x个用户: [
        '当前有 {} 个用户',
        '目前有 {} 個使用者',
        'There are currently {} users',
        '現在 {} 人のユーザーがいます',
        '현재 {}명의 유저가 있습니다',
        'В настоящее время существует {} пользователей',
    ],
    _已抓取x个用户: [
        '已抓取 {} 个用户',
        '已擷取 {} 個使用者',
        'crawled {} users',
        'クロールされた {} ユーザー',
        '{}명의 유저를 긁어왔습니다',
        'Сканированные {} пользователи',
    ],
    _排行榜进度: [
        '已抓取本页面第{}部分',
        '已擷取本頁面第 {} 部分',
        'Part {} of this page has been crawled',
        'このページの第　{}　部がクロールされました',
        '이 페이지의 {} 부분을 긁어왔습니다',
        'Часть {} этой страницы была просмотрена',
    ],
    _新作品进度: [
        '已抓取本页面 {} 个作品',
        '已擷取本頁面 {} 個作品',
        'This page has been crawled {} works',
        'このページの {} つの作品をクロールしました',
        '이 페이지의 {}개의 작품을 긁어왔습니다',
        'На этой странице было просканированно {} работ',
    ],
    _抓取多少个作品: [
        '抓取本页面 {} 个作品',
        '擷取本頁面 {} 個作品',
        'Crawl this page {} works',
        'このページの {} つの作品をクロールします',
        '이 페이지의 {}개의 작품 긁어오기',
        'Сканировать на этой странице {} работ',
    ],
    _相关作品抓取完毕: [
        '相关作品抓取完毕。包含有{}个作品，开始获取作品信息。',
        '相關作品擷取完畢。包含有 {} 個作品，開始取得作品資訊。',
        'The related works have been crawled. Contains {} works and starts getting information about the work.',
        '関連作品はクロールされました。 {} 作品を含み、その作品に関する情報の取得を開始します。',
        '관련 작품 긁어오기 완료, {}개의 작품이 포함되어 있으며, 작품 정보 가져오기를 시작합니다',
        'Связанные работы были просканированы. Содержит {} работ и начинает получать информацию о работе(ах).',
    ],
    _排行榜任务完成: [
        '本页面抓取完毕。<br>当前有{}个作品，开始获取作品信息。',
        '本頁面擷取完畢。<br>目前有 {} 個作品，開始取得作品資訊。',
        'This page is crawled and now has {} works.<br> Start getting the works for more information.',
        'このページのクロール終了。<br>{}枚の作品があります。 作品情報の取得を開始します。',
        '이 페이지 긁어오기 완료되었습니다<br>현재 {}개의 작품이 있으며, 작품 정보 가져오기를 시작합니다',
        'Эта страница просмотрена и имеет {} работ.<br> Начинаю получать работы для получения дополнительной информации.',
    ],
    _开始获取作品信息: [
        '开始获取作品信息',
        '開始取得作品資訊',
        'Start getting work data',
        '作品情報の取得を開始します',
        '작품 정보 가져오기 시작',
        'Начинаю получать данные о работе',
    ],
    _列表页抓取进度: [
        '已抓取列表页 {} 个页面',
        '已擷取清單頁 {} 個頁面',
        'Has acquired {} list pages',
        '{} のリストページを取得しました',
        '{}개의 목록 페이지를 획득하였습니다',
        'Получено {} страниц списка',
    ],
    _列表页抓取进度2: [
        '正在抓取列表页 {}/{}',
        '正在抓取列表頁 {}/{}',
        'crawling list page {}/{}',
        'リストページの取得 {}/{}',
        '목록 페이지 긁어오는 중 {}/{}',
        'Вытаскивание списка страниц {}/{}',
    ],
    _列表页抓取完成: [
        '列表页面抓取完成',
        '清單頁面擷取完成',
        'The list page is crawled',
        'リストページがクロールされ',
        '목록 페이지 긁어오기 완료',
        'Список страниц просканирован',
    ],
    _抓取结果为零: [
        '抓取完毕，但没有找到符合筛选条件的作品。<br>请检查“抓取”相关的设置。',
        '擷取完畢，但沒有找到符合篩選條件的作品。<br>請檢查“抓取”相關的設定。',
        'Crawl complete but did not find works that match the filter criteria.<br>Please check the settings related to Crawl.',
        'クロールは終了しましたが、フィルタ条件に一致する作品が見つかりませんでした。<br>クロールに関する設定を確認してください。',
        '긁어오기가 완료되었지만 필터 조건과 일치하는 작품을 찾지 못했습니다.<br>크롤링 관련 설정을 확인하세요.',
        'Вытаскивание завершено, но не найдены работы, соответствующие критериям фильтра.<br>Пожалуйста, проверьте настройки, связанные со сканированием.',
    ],
    _当前任务尚未完成: [
        '当前任务尚未完成',
        '目前工作尚未完成',
        'The current task has not yet been completed',
        '現在の作業はまだ完了していません',
        '현재 작업이 아직 완료되지 않았습니다',
        'Текущее задание еще не выполнено',
    ],
    _当前任务尚未完成2: [
        '当前任务尚未完成，请等待完成后再下载。',
        '目前工作尚未完成，請等待完成後再下載。',
        'The current task has not yet been completed',
        '現在の作業はまだ完了していません、完了するまでお待ちください',
        '현재 작업이 아직 완료되지 않았습니다, 완료될 때까지 기다려주세요.',
        'Текущее задание еще не выполнено',
    ],
    _列表抓取完成开始获取作品页: [
        '当前列表中有{}张作品，开始获取作品信息',
        '目前清單中有 {} 張作品，開始取得作品資訊',
        'Now has {} works. Start getting the works for more information.',
        '{} 枚の作品があります。 作品情報の取得を開始します。',
        '현재 {}개의 작품이 있습니다, 작품 정보 가져오기를 시작합니다',
        'Сейчас в работе {} работ. Начните получать работы для получения дополнительной информации.',
    ],
    _开始获取作品页面: [
        '开始获取作品页面',
        '開始取得作品頁面',
        'Start getting the works page',
        '作品ページの取得を開始する',
        '작품 페이지 가져오기 시작',
        'Начинаю получать страницу с работами',
    ],
    _无权访问: [
        '无权访问 {}，跳过该作品。',
        '沒有權限存取 {}，跳過該作品。',
        'No access {}, skip.',
        '{} のアクセス権限がありません、作品を無視する。',
        '{}에 접근 권한이 없습니다, 이 작업을 건너뜁니다.',
        'Нет доступа {}, пропуск.',
    ],
    _作品页状态码0: [
        '请求的 URL 不可访问 (0)',
        '要求的 URL 無法存取 (0)',
        'The requested URL is not accessible (0)',
        '要求された URL にアクセスできません (0)',
        '요청한 URL에 접근할 수 없습니다 (0)',
        'Запрашиваемый URL недоступен (0)',
    ],
    _作品页状态码400: [
        '该作品已被删除 (400)',
        '該作品已被刪除 (400)',
        'The work has been deleted (400)',
        '作品は削除されました (400)',
        '이 작품은 삭제되었습니다 (400)',
        'Работа была удалена (400)',
    ],
    _作品页状态码401: [
        '请您登录 Pixiv 账号然后重试。(401)',
        '請您登入 Pixiv 帳號後重試。(401)',
        'Please log in to your Pixiv account and try again. (401)',
        'Pixiv アカウントにログインして、もう一度お試しください。(401)',
        'Pixiv 계정에 로그인 후 다시 시도해주세요. (401)',
        'Пожалуйста, войдите в свою учетную запись Pixiv и попробуйте еще раз. (401)',
    ],
    _作品页状态码403: [
        '无权访问请求的 URL (403)',
        '沒有權限存取要求的 URL (403)',
        'Have no access to the requested URL (403)',
        'リクエストされた URL にアクセスできない (403)',
        '요청한 URL에 접근 권한이 없습니다 (403)',
        'Нет доступа к запрашиваемому URL (403)',
    ],
    _作品页状态码404: [
        '404 not found',
        '404 not found',
        '404 not found',
        '404 not found',
        '404 not found',
        '404 not found',
    ],
    _作品页状态码429: [
        '错误代码：429（请求数量过多）。下载器会重新抓取它。',
        '錯誤程式碼：429（請求數量過多）。下載器会重新抓取它。',
        'Error code: 429 (Too many requests). The downloader will re-crawl it.',
        'エラー コード: 429 (要求が多すぎます)。ダウンローダーはそれを再クロールします。',
        '오류 코드: 429(요청이 너무 많음). 다운로더가 다시 크롤링합니다.',
        'Код ошибки: 429 (Слишком много запросов). Загрузчик будет повторять вытаскивание.',
    ],
    _错误代码: [
        '错误代码：',
        '錯誤程式碼：',
        'Error code: ',
        'エラー コード: ',
        '오류 코드: ',
        'Код ошибки: ',
    ],
    _作品页状态码500: [
        'Pixiv 拒绝返回数据 (500)。下载器会重新抓取它。',
        'Pixiv 拒絕返回資料 (500)。下載器会重新抓取它。',
        'Pixiv refuses to return data (500). The downloader will re-crawl it.',
        'ピクシブはデータの返却を拒否します (500)。ダウンローダーはそれを再クロールします。',
        'pixiv는 데이터 반환을 거부합니다 (500). 다운로더가 다시 크롤링합니다.',
        'Pixiv отказывается возвращать данные (500). Загрузчик будет повторять вытаскивание.',
    ],
    _正在抓取: [
        '正在抓取，请等待……',
        '擷取中，請稍後……',
        'Getting, please wait...',
        'クロール中、しばらくお待ちください...',
        '얻어오는 중, 잠시만 기다려주세요...',
        'Получение, пожалуйста, подождите...',
    ],
    _获取全部书签作品: [
        '获取全部书签作品，时间可能比较长，请耐心等待。',
        '取得全部書籤作品，時間可能比較長，請耐心等待。',
        'Get all bookmarked works, the time may be longer, please wait.',
        'ブックマークしたすべての作品を取得すると、時間がかかることがあります。お待ちください。',
        '북마크된 작품을 모두 가져오는 것은 시간이 오래 걸릴 수 있으니 기다려주세요.',
        'Получить все работы из закладок, это может занять время, пожалуйста, подождите',
    ],
    _抓取图片网址遇到中断: [
        '当前任务已中断!',
        '目前工作已中斷！',
        'The current task has been interrupted.',
        '現在の作業が中断されました。',
        '현재 작업이 중단되었습니다!',
        'Текущая задача была прервана.',
    ],
    _关闭: ['关闭', '關閉', 'close', 'クローズ', '닫기', 'закрыть'],
    _输出信息: [
        '输出信息',
        '輸出資訊',
        'Output information',
        '出力情報',
        '출력 정보',
        'Выходная информация',
    ],
    _复制: ['复制', '複製', 'Copy', 'コピー', '복사', 'Копировать'],
    _已复制到剪贴板: [
        '已复制到剪贴板，可直接粘贴',
        '已複製至剪貼簿，可直接貼上',
        'Has been copied to the clipboard',
        'クリップボードにコピーされました',
        '클립보드에 복사되었습니다.',
        'Скопировано в буфер обмена',
    ],
    _下载设置: [
        '下载设置',
        '下載設定',
        'Download settings',
        'ダウンロード設定',
        '다운로드 설정',
        'Настройки загрузки',
    ],
    _收起展开设置项: [
        '收起/展开设置项',
        '摺疊/展開設定項目',
        'Collapse/expand settings',
        '設定の折りたたみ/展開',
        '설정 축소/확장',
        'Свернуть/развернуть настройки',
    ],
    _github: [
        'Github 页面，欢迎 star',
        'Github 頁面，歡迎 star',
        'Github page, if you like, please star it',
        'Github のページ、star をクリックしてください',
        'Github, 유용하셨다면 Star를 주세요.',
        'Страница на Github, если вам нравится, пожалуйста, поставьте звезду',
    ],
    _wiki: ['使用手册', 'Wiki', 'Wiki', 'マニュアル', '위키', 'Вики'],
    _快捷键ALTX显示隐藏控制面板: [
        '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">X</span> 显示或隐藏控制面板。',
        '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">X</span> 顯示或隱藏控制面板。',
        'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">X</span> to show or hide the control panel.',
        'ショートカット キー <span class="blue">Alt</span> + <span class="blue">X</span> を使用して、コントロール パネルを表示または非表示にできます。',
        '단축키 <span class="blue">Alt</span> + <span class="blue">X</span>를 사용하여 제어판을 표시하거나 숨길 수 있습니다.',
        'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">X</span>, чтобы отобразить или скрыть панель управления.',
    ],
    _隐藏控制面板: [
        '隐藏控制面板（Alt + X）',
        '隱藏控制面板（Alt + X）',
        'hide control panel (Alt + X)',
        'コントロールパネルを隠す（Alt + X）',
        '제어판 숨기기 (Alt + X)',
        'скрыть панель управления (Alt + X)',
    ],
    _显示控制面板: [
        '显示控制面板 (Alt + X)',
        '顯示控制面板 (Alt + X)',
        'Show control panel (Alt + X)',
        'コントロールパネルを表示 (Alt + X)',
        '제어판 표시 (Alt + X)',
        'показать панель управления (Alt + X)',
    ],
    _共抓取到n个文件: [
        '共抓取到 {} 个文件',
        '共擷取到 {} 個檔案',
        'Crawl a total of {} files',
        '合計 {} つのファイルがあります',
        '총 {}개의 파일을 긁어왔습니다',
        'Всего просканированно {} файлов',
    ],
    _共抓取到n个作品: [
        '共抓取到 {} 个作品',
        '共擷取到 {} 個作品',
        'Crawl a total of {} works',
        '合計 {} つの作品があります',
        '총 {}개의 작품을 긁어왔습니다',
        'Всего просканированно {} работ',
    ],
    _命名规则: [
        '<span class="key">命名</span>规则',
        '<span class="key">命名</span>規則',
        '<span class="key">Naming</span> rule',
        '<span class="key">命名</span>規則',
        '<span class="key">명명</span> 규칙',
        '<span class="key">Правила</span> названий',
    ],
    _命名规则2: [
        '命名规则',
        '命名規則',
        'Naming rule',
        '命名規則',
        '명명 규칙',
        'Правила названий',
    ],
    _设置文件夹名的提示: [
        `可以使用 '<span class="key">/</span>' 建立文件夹。示例：`,
        `可以使用斜線（<span class="key">/</span>）建立資料夾。範例：`,
        `You can create a directory with '<span class="key">/</span>'. Example：`,
        `フォルダーは '<span class="key">/</span>' で作成できます。例：`,
        `'<span class="key">/</span>'을 사용하여 디렉토리를 생성할 수 있습니다.<br>예:`,
        `Вы можете создать каталог с помощью '<span class="key">/</span>'. Пример:`,
    ],
    _添加命名标记前缀: [
        '添加命名标记<span class="key">前缀</span>',
        '加入命名標記<span class="key">前綴</span>',
        'Add named tag <span class="key">prefix</span>',
        '前にタグの名前を追加',
        '명명된 태그 추가 <span class="key">접두사</span>',
        'Добавить именованный тег <span class="key">префикс</span>',
    ],
    _添加字段名称提示: [
        `例如，在用户名前面添加“user_”标记`,
        '例如，在使用者名稱前面加入「user_」標記。',
        `For example, add the 'user_' tag in front of the username`,
        'たとえば、ユーザー名の前に 「user_」タグを追加します。',
        `예: 유저명 앞에 'user_' 태그 추가`,
        `Например, добавьте тег 'user_' перед именем пользователя`,
    ],
    _命名标记id: [
        '默认文件名，如 <span class="blue">44920385_p0</span>',
        '預設檔案名稱，例如：<span class="blue">44920385_p0</span>。',
        'Default file name, for example <span class="blue">44920385_p0</span>',
        'デフォルトのファイル名，例 <span class="blue">44920385_p0</span>',
        '기본 파일명. 예: <span class="blue">44920385_p0</span>',
        'Имя файла по умолчанию, например <span class="blue">44920385_p0</span>',
    ],
    _命名标记title: [
        '作品标题',
        '作品標題',
        'Works title',
        '作品のタイトル',
        '작품 제목',
        'Название работ',
    ],
    _命名标记tags: [
        '作品的标签列表',
        '作品的標籤清單',
        'The tags of the work',
        '作品のタグ',
        '작품 태그',
        'Теги работ',
    ],
    _命名标记user: [
        '用户名字',
        '使用者名稱',
        'User name',
        'ユーザー名',
        '유저명',
        'Никнейм юзера',
    ],
    _用户id: [
        '用户 ID（数字）',
        '使用者 ID（數字）',
        'User ID (Number)',
        'ユーザー ID (Number)',
        '유저 ID (숫자)',
        'ID Юзера (Число)',
    ],
    _命名标记px: [
        '宽度和高度。例如：<span class="blue">600x900</span>',
        '寬度和高度。例如：<span class="blue">600x900</span>',
        'Width and height, e.g. <span class="blue">600x900</span>',
        '幅と高さ。例：<span class="blue">600x900</span>',
        '너비와 높이. 예: <span class="blue">600x900</span>',
        'Ширина и высота, напр. <span class="blue">600x900</span>',
    ],
    _命名标记bmk: [
        'Bookmark count，作品的收藏数。把它放在最前面可以让文件按收藏数排序。',
        'Bookmark count，作品的收藏數。將它放在最前面可以讓檔案依收藏數排序。',
        'Bookmark count, bookmarks number of works.',
        'Bookmark count，作品のボックマークの数、前に追加することでボックマーク数で并べることができます。',
        '북마크 수. 맨 앞에 두면 북마크 수별로 문서를 정렬할 수 있습니다.',
        'Количество закладок, количество произведений в закладках',
    ],
    _命名标记bmk_id: [
        'Bookmark Id。你收藏的每一个作品都会有一个 Bookmark Id。收藏的时间越晚，Bookmark Id 就越大。当你下载你的收藏时，可以使用 {bmk_id} 作为排序依据。',
        'Bookmark Id。你收藏的每一個作品都會有一個 Bookmark Id。收藏的時間越晚，Bookmark Id 就越大。當你下載你的收藏時，可以使用 {bmk_id} 作為排序依據。',
        'Bookmark Id. Every work in your bookmarks will have a Bookmark Id. The later the bookmark is added, the larger the Bookmark Id. When you download your bookmarks, you can use {bmk_id} as a sorting basis.',
        'ブックマークID。 ブックマーク内のすべての作品にはブックマークIDがあります。 ブックマークを後で追加すると、ブックマークIDが大きくなります。 ブックマークをダウンロードするときは、{bmk_id}を並べ替えの基準として使用できます。',
        '북마크 ID. 당신이 북마크하고 있는 작품마다 북마크 ID가 있습니다. 북마크 시간이 늦어질수록 북마크 ID는 커집니다. 북마크를 다운로드할때 {bmk_id}를 기준으로 정렬할 수 있습니다.',
        'Bookmark Id. Каждая работа в ваших закладках будет иметь идентификатор закладки. Чем позже добавлена закладка, тем больше Id закладки. Когда вы загружаете закладки, вы можете использовать {bmk_id} в качестве основы для сортировки.',
    ],
    _命名标记bmk_1000: [
        '作品收藏数的简化显示。例如：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
        '作品收藏數的簡化顯示。例如：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
        'Simplified number of bookmark, e.g. <span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
        '作品のボックマークの数の簡略表示。例：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
        '단순화된 북마크 수. 예: <span class="blue">0+</span>, <span class="blue">1000+</span>, <span class="blue">2000+</span>, <span class="blue">3000+</span> ……',
        'Упрощенное количество закладок, напр. <span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ......',
    ],
    _命名标记like: [
        'Like count，作品的点赞数。',
        'Like count，作品的點讚數。',
        'Like count.',
        'Like count。',
        '좋아요 수',
        'Колличество лайков',
    ],
    _命名标记view: [
        'View count，作品的浏览量。',
        'View count，作品的瀏覽量。',
        'View count.',
        'View count。',
        '조회수',
        'Колличество просмотров',
    ],
    _命名标记id_num: [
        '数字 ID，如 <span class="blue">44920385</span>',
        '數字 ID，例如：<span class="blue">44920385</span>。',
        'Number ID, for example <span class="blue">44920385</span>',
        '<span class="blue">44920385</span> などの番号 ID',
        '숫자 ID. 예: <span class="blue">44920385</span>',
        'Идентификатор номера, например <span class="blue">44920385</span>',
    ],
    _命名标记p_num: [
        '图片在作品内的序号，如 <span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span> …… 每个作品都会重新计数。',
        '圖片在作品內的序號，例如：<span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span>……每個作品都將重新計數。',
        'The serial number of the image in the work, such as <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> ... Each work will be recounted.',
        '<span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span> など、作品の画像のシリアル番号。各ピースは再集計されます。',
        '작품 안에 있는 번호. 예: <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> …… 작품마다 다시 세어봅니다.',
        'Порядковый номер изображения в работе, например, <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> .... Каждое произведение будет пересказано',
    ],
    _命名标记tags_trans: [
        '作品的标签列表，附带翻译后的标签（如果有）',
        '作品的標籤清單，包含翻譯後的標籤（如果有的話）。',
        'The tags of the work, with the translated tag (if any)',
        '作品のタグリスト、翻訳付きタグ(あれば)',
        '작품 태그, 번역된 태그 (있다면)',
        'Теги произведения, с тегом перевода (если есть)',
    ],
    _命名标记tags_transl_only: [
        '翻译后的标签列表',
        '譯後的標籤清單。',
        'Translated tags',
        '翻訳后のタグリスト',
        '번역된 태그',
        'Теги перевода',
    ],
    _命名标记date: [
        '作品的创建时间。如 <span class="blue">2019-08-29</span>。',
        '作品的建立時間。例如：<span class="blue">2019-08-29</span>。',
        'The time the creation of the work. Such as <span class="blue">2019-08-29</span>',
        '作品の作成時間。例 <span class="blue">2019-08-29</span>',
        '작품 생성 날짜. 예: <span class="blue">2019-08-29</span>',
        'Время создания произведения. Например, <span class="blue">2019-08-29</span>',
    ],
    _命名标记upload_date: [
        '作品内容最后一次被修改的时间。如 <span class="blue">2019-08-30</span>。',
        '作品內容最後一次被修改的時間。如 <span class="blue">2019-08-30</span>。',
        'The time when the content of the work was last modified. Such as <span class="blue">2019-08-30</span>.',
        '作品の内容が最後に変更された時刻。例 <span class="blue">2019-08-30</span>',
        '저작물의 내용이 마지막으로 수정된 시간입니다. 예: <span class="blue">2019-08-30</span>',
        'Время, когда содержание работы было изменено в последний раз. Например, <span class="blue">2019-08-30</span>.',
    ],
    _命名标记rank: [
        '作品在排行榜中的排名。如 <span class="blue">#1</span>、<span class="blue">#2</span> …… 只能在排行榜页面中使用。',
        '作品在排行榜中的排名。例如：<span class="blue">#1</span>、<span class="blue">#2</span>……只能在排行榜頁面中使用。',
        'The ranking of the work in the ranking pages. Such as <span class="blue">#1</span>, <span class="blue">#2</span> ... Can only be used in ranking pages.',
        '作品のランキング。例え　<span class="blue">#1</span>、<span class="blue">#2</span> …… ランキングページのみで使用できます。',
        '작품의 랭킹. 예: <span class="blue">#1</span>, <span class="blue">#2</span> …… 랭킹 페이지에서만 사용할 수 있습니다.',
        'Рейтинг работы на страницах рейтинга. Например, <span class="blue">#1</span>, <span class="blue">#2</span> ... Может использоваться только на страницах ранжирования.',
    ],
    _命名标记type: [
        '作品类型，分为：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
        '作品類型，分為：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
        'The type of work, divided into：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
        '作品分類は：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
        '작품 유형: <span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
        'Тип работы, разделенный на：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    ],
    _命名标记AI: [
        '如果作品是由 AI 生成的，则输出 <span class="blue">AI</span>',
        '如果作品是由 AI 生成的，則輸出 <span class="blue">AI</span>',
        'If the work is generated by AI, output <span class="blue">AI</span>',
        '作品がAIで生成された場合、<span class="blue">AI</span>を出力',
        '작업이 AI로 생성된 경우 <span class="blue">AI</span> 출력',
        'Если работа создана с помощью ИИ, выведите <span class="blue">AI</span>',
    ],
    _命名标记提醒: [
        '为了防止文件名重复，命名规则里一定要包含 {id} 或者 {id_num}{p_num}。<br>您可以使用多个标记；建议在不同标记之间添加分割用的字符。示例：{id}-{user_id}',
        '為了防止檔名重複，命名規則裡一定要包含 {id} 或者 {id_num}{p_num}。<br>您可以使用多個標記；建議在不同標記之間加入分隔用的字元。範例：{id}-{user_id}',
        'To prevent duplicate file names, {id} or {id_num}{p_num} must be included in the naming rules.<br>You can use multiple tags, and you can add a separate character between different tags. Example: {id}-{user_id}',
        'ファイル名の重複を防ぐために、命名規則には {id} または {id_num}{p_num} を含める必要があります。<br>複数のタグを使用することができます；異なるタグ間の分割のために文字を追加することをお勧めします。例：{id}-{user_id}',
        '파일명이 중복되지 않도록, 명명 규칙에는 {id} 또는 {id_num}{p_num}이 포함되어야 합니다.<br>여러 태그를 사용할 수 있습니다. 서로 다른 태그 사이에 구분자를 넣는 것을 권장합니다. 예: {id}-{user}',
        'Чтобы предотвратить дублирование имен файлов, {id} или {id_num}{p_num} должны быть включены в правила именования.<br>Вы можете использовать несколько тегов, и вы можете добавить отдельный символ между различными тегами. Пример: {id}-{user_id}',
    ],
    _有些标记并不总是可用的提醒: [
        '有些标记并不总是可用，有时它们可能什么都不输出。',
        '有些標記並不總是可用，有時它們可能什麼都不輸出。',
        'Some tags are not always available, and sometimes they may output nothing.',
        '一部のタグは常に使用できるとは限らず、何も出力しない場合もあります。',
        '일부 태그는 항상 사용할 수 있는 것은 아니며 때로는 아무 것도 출력하지 않을 수도 있습니다.',
        'Некоторые теги не всегда доступны, а иногда могут ничего не выводить.',
    ],
    _命名规则一定要包含id: [
        '为了防止文件名重复，命名规则里一定要包含 {id} 或者 {id_num}{p_num}',
        '為了防止檔名重複，命名規則裡一定要包含 {id} 或者 {id_num}{p_num}。',
        'To prevent duplicate file names, {id} or {id_num}{p_num} must be included in the naming rules.',
        'ファイル名の重複を防ぐために、命名規則には {id} または {id_num}{p_num} を含める必要があります。',
        '파일명이 중복되지 않도록, 명명 규칙에는 {id} 또는 {id_num}{p_num}이 포함되어야 합니다.',
        'Чтобы предотвратить дублирование имен файлов, {id} или {id_num}{p_num} должны быть включены в правила именования.',
    ],
    _文件夹标记PTag: [
        '如果页面里的作品属于同一个标签，则输出这个标签。',
        '如果頁面裡的作品屬於同一個標籤，則輸出這個標籤。',
        'If the works on the page belong to the same tag, then output this tag.',
        'ページ上の作品が同じタグに属している場合は、このタグを出力します。',
        '페이지의 작품이 같은 태그에 속하는 경우 이 태그를 출력합니다.',
        'Если работы на странице относятся к одному и тому же тегу, то выводить этот тег.',
    ],
    _命名标记seriesTitle: [
        '系列标题，只在系列页面中可用（小说系列、漫画系列）。',
        '系列標題，只在系列頁面中可用（小說系列、漫畫系列）。',
        'Series title, only available in series pages (Novel series, Manga series).',
        'シリーズタイトル，シリーズページのみ（小説連載、漫画連載）。',
        '시리즈 제목, 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
        'Название серии, доступно только на страницах серий (серия романов, серия манги).',
    ],
    _命名标记seriesOrder: [
        '作品在系列中的序号，如 <span class="blue">#1</span> <span class="blue">#2</span>。只在系列页面中可用（小说系列、漫画系列）。',
        '作品在系列中的編號，如 <span class="blue">#1</span> <span class="blue">#2</span>。只在系列頁面中可用（小說系列、漫畫系列）。',
        'The number of the work in the series, such as <span class="blue">#1</span> <span class="blue">#2</span>. only available in series pages (Novel series, Manga series).',
        'シリーズの中の作品の番号，例え <span class="blue">#1</span> <span class="blue">#2</span>。シリーズページのみ（小説連載、漫画連載）。',
        '시리즈 내 작품 번호. 예: <span class="blue">#1</span> <span class="blue">#2</span>. 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
        'Номер работы в серии, например, <span class="blue">#1</span> <span class="blue">#2</span>. доступны только на страницах серий (серия романов, серия манги).',
    ],
    _命名标记seriesId: [
        '系列 ID，只在系列页面中可用（小说系列、漫画系列）。',
        '系列 ID，只在系列頁面中可用（小說系列、漫畫系列）。',
        'Series ID, only available in series pages (Novel series, Manga series).',
        'シリーズ ID，シリーズページのみ（小説連載、漫画連載）。',
        '시리즈 ID, 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
        'Идентификатор серии, доступен только на страницах серий (серия романов, серия манги).',
    ],
    _文件夹标记PTitle: [
        '页面标题',
        '頁面標題',
        'Page title',
        'ページタイトル',
        '페이지 제목',
        'Заголовок страницы',
    ],
    _预览文件名: [
        '预览文件名',
        '預覽檔案名稱',
        'Preview file name',
        'ファイル名',
        '파일명 미리보기',
        'Имя файла предварительного просмотра',
    ],
    _下载线程: [
        '下载<span class="key">线程</span>',
        '下載<span class="key">執行緒</span>',
        'Download <span class="key">thread</span>',
        '同時ダウンロード数',
        '다운로드 <span class="key">쓰레드</span>',
        'Кол-во <span class="key">потоков</span> на загрузку',
    ],
    _下载线程的说明: [
        `你可以输入 1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} 之间的数字，设置同时下载的数量`,
        `你可以輸入 1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} 之間的數字，設定同時下載的數量。`,
        `You can type a number between 1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} to set the number of concurrent downloads`,
        `同時ダウンロード数を設定、1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} の数値を入力してください`,
        `1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} 사이의 숫자를 입력하여 동시 다운로드 수를 설정할 수 있습니다.`,
        `Вы можете ввести число между 1-${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.downloadThreadMax} , чтобы установить количество одновременных загрузок`,
    ],
    _开始下载: [
        '开始下载',
        '開始下載',
        'Start download',
        '開始',
        '다운로드 시작',
        'Начать загрузку',
    ],
    _暂停下载: [
        '暂停下载',
        '暫停下載',
        'Pause download',
        '一時停止',
        '다운로드 일시중지',
        'Приостановить загрузку',
    ],
    _停止下载: [
        '停止下载',
        '停止下載',
        'Stop download',
        '停止',
        '다운로드 정지',
        'Остановить загрузку',
    ],
    _复制url: [
        '复制 URL',
        '複製下載網址',
        'Copy URLs',
        'URL をコピー',
        'URL 복사',
        'Копировать URL',
    ],
    _当前状态: [
        '当前状态 ',
        '目前狀態：',
        'State ',
        '現在の状態 ',
        '현재 상태',
        'Текущее состояние',
    ],
    _未开始下载: [
        '未开始下载',
        '未開始下載',
        'Not yet started downloading',
        'まだダウンロードを開始していません',
        '아직 다운로드를 시작하지 않았습니다.',
        'Загрузка еще не началась',
    ],
    _下载进度: [
        '下载进度',
        '下載進度',
        'Total progress',
        '概要',
        '다운로드 진행률',
        'Полный прогресс',
    ],
    _任务进度: [
        '任务进度',
        '任務進度',
        'Task progress',
        'タスクの進行状況',
        '작업 진행',
        'прогресс',
    ],
    _常见问题: ['常见问题', '常見問題', 'Help', 'よくある質問', '도움말', 'help'],
    _uuid: [
        '如果下载后的文件名异常，请禁用其他有下载功能的浏览器扩展。<br>例如：Chrono 下载管理器、free Download Manager、Image Downloader、DownThemAll! 等。',
        '如果下載後的檔案名稱異常，請停用其他有下載功能的瀏覽器擴充功能。<br>例如：Chrono 下载管理器、free Download Manager、Image Downloader、DownThemAll! 等。',
        'If the file name after downloading is abnormal, disable other browser extensions that have download capabilities.<br>For example: Chrono Download Manager, free Download Manager, Image Downloader, DownThemAll! and more.',
        'ダウンロード後のファイル名が異常な場合は、ダウンロード機能を持つ他のブラウザ拡張機能を無効にしてください。<br>例：Chrono Download Manager, free Download Manager, Image Downloader, DownThemAll! など。',
        '다운로드 후 파일명이 이상할 경우 다운로드 기능이 있는 다른 브라우저 확장 프로그램을 비활성화해주세요.예: Chrono Download Manager, free Download Manager、Image Downloader、DownThemAll! 등.',
        'Если имя файла после загрузки ненормальное, отключите другие расширения браузера, которые имеют возможность загрузки.<br> Например: Chrono Download Manager, бесплатный менеджер загрузок, загрузчик изображений, DownThemAll! и многое другое.',
    ],
    _常见问题说明: [
        '下载的文件保存在浏览器的下载目录里。<br><br>建议在浏览器的下载设置中关闭“下载前询问每个文件的保存位置”。<br><br>如果下载后的文件名异常，请禁用其他有下载功能的浏览器扩展。<br><br>如果你的浏览器在启动时停止响应，你可以清除浏览器的下载记录。<br><br>如果你使用 V2ray、Clash 等代理软件，可以确认一下 Pixiv 的图片域名（i.pximg.net）是否走了代理，如果没走代理就在代理规则里添加这个域名。<br><br>如果你需要一个梯子（机场）,可以试试这个机场：<a href="https://v3.xiaoy666.top/#/register?code=KEA3xTT4" title="农家有风小院" target="_blank">农家有风小院</a>，价格实惠，网络稳定。先购买订阅，然后在仪表盘复制订阅链接使用。<br><br>下载器 QQ 群：499873152<br><br>在 Wiki 查看常见问题：<br><a href="https://xuejianxianzun.github.io/PBDWiki/#/zh-cn/常见问题" target="_blank">https://xuejianxianzun.github.io/PBDWiki/#/zh-cn/常见问题</a><br><br>中文教程视频：<br><a href="https://www.youtube.com/playlist?list=PLO2Mj4AiZzWEpN6x_lAG8mzeNyJzd478d" target="_blank">https://www.youtube.com/playlist?list=PLO2Mj4AiZzWEpN6x_lAG8mzeNyJzd478d</a>',
        '下載的檔案儲存在瀏覽器的下載目錄裡。<br><br>請不要在瀏覽器的下載選項裡選取「下載每個檔案前先詢問儲存位置」。<br><br>如果下載後的檔名異常，請停用其他有下載功能的瀏覽器擴充功能。<br><br>如果你的瀏覽器在啟動時停止響應，你可以清除瀏覽器的下載記錄。',
        'The downloaded file is saved in the browsers download directory. <br><br>It is recommended to turn off "Ask where to save each file before downloading" in the browser`s download settings.<br><br>If the file name after downloading is abnormal, disable other browser extensions that have download capabilities.<br><br>If your browser stops responding at startup, you can clear your browser`s download history.',
        'ダウンロードしたファイルは、ブラウザのダウンロードディレクトリに保存されます。<br><br>ブラウザのダウンロード設定で 「 ダウンロード前に各ファイルの保存場所を確認する 」 をオフにすることをお勧めします。<br><br>ダウンロード後のファイル名が異常な場合は、ダウンロード機能を持つ他のブラウザ拡張機能を無効にしてください。<br><br>起動時にブラウザーが応答しなくなった場合は、ブラウザーのダウンロード履歴を消去できます。',
        '다운로드한 파일은 브라우저의 다운로드 디렉토리에 저장됩니다.<br><br>브라우저의 다운로드 설정에서 "다운로드 전에 각 파일의 저장 위치 확인"을 끄는 것이 좋습니다.<br><br>다운로드 후 파일명이 이상할 경우 다운로드 기능이 있는 다른 브라우저 확장 프로그램을 비활성화해주세요.<br><br>시작 시 브라우저가 응답하지 않으면 브라우저의 다운로드 기록을 지울 수 있습니다.',
        'Загруженный файл сохраняется в каталоге загрузки браузеров. <br><br>Рекомендуется отключить "Спрашивать, куда сохранять каждый файл перед загрузкой" в настройках загрузки браузера.<br><br>Если имя файла после загрузки является ненормальным, отключите другие расширения браузера, которые имеют возможности загрузки.<br><br>Если ваш браузер перестает отвечать на запросы при запуске, вы можете очистить историю загрузок вашего браузера.',
    ],
    _正在下载中: [
        '正在下载中',
        '正在下載',
        'Downloading',
        'ダウンロード中',
        '다운로드 중',
        'Загрузка',
    ],
    _下载完毕: [
        '✓ 下载完毕',
        '✓ 下載完畢',
        '✓ Download complete',
        '✓ ダウンロードが完了しました',
        '✓ 다운로드 완료',
        '✓ Загрузка завершена',
    ],
    _下载完毕2: [
        '下载完毕',
        '下載完畢',
        'Download complete',
        'ダウンロードが完了しました',
        '다운로드 완료',
        'Загрузка завершена',
    ],
    _已暂停: [
        '下载已暂停',
        '下載已暫停',
        'Download is paused',
        'ダウンロードは一時停止中です',
        '다운로드 일시중지',
        'Загрузка приостановлена',
    ],
    _已停止: [
        '下载已停止',
        '下載已停止',
        'Download stopped',
        'ダウンロードが停止しました',
        '다운로드 정지',
        'Загрузка остановлена',
    ],
    _已下载: [
        '已下载',
        '已下載',
        'downloaded',
        'downloaded',
        '다운로드됨',
        'загруженно',
    ],
    _抓取完毕: [
        '抓取完毕！',
        '擷取完畢！',
        'Crawl complete!',
        'クロールが終了しました！',
        '긁어오기 완료!',
        'Вытаскивание завершено!',
    ],
    _抓取完毕2: [
        '抓取完毕',
        '擷取完畢',
        'Crawl complete',
        'クロールが終了しました',
        '긁어오기 완료',
        'Вытаскивание завершено',
    ],
    _快速下载本页: [
        '快速下载本页作品 (Alt + Q)',
        '快速下載本頁作品 (Alt + Q)',
        'Download this work quickly (Alt + Q)',
        'この作品をすばやくダウンロードする (Alt + Q)',
        '작품 빠른 다운로드 (Alt + Q)',
        'Быстро загрузить эту работу (Alt + Q)',
    ],
    _快捷键ALTQ快速下载本页作品: [
        '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">Q</span> 快速下载本页作品。',
        '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">Q</span> 快速下載本頁作品。',
        'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">Q</span> to quickly download works on this page.',
        'ショートカット キー <span class="blue">Alt</span> + <span class="blue">Q</span> を使用して、このページの作品をすばやくダウンロードできます。',
        '단축키 <span class="blue">Alt</span> + <span class="blue">Q</span>를 사용하여 이 페이지에서 작품을 빠르게 다운로드할 수 있습니다.',
        'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">Q</span> для быстрой загрузки работ на этой странице.',
    ],
    _抓取此作品: [
        '抓取此作品',
        '抓取此作品',
        'Crawl this work',
        'この作品をクロールする',
        '이 작품을 크롤링',
        'Просканировать эту работу',
    ],
    _从本页开始抓取new: [
        '从本页开始抓取新作品',
        '從本頁開始擷取新作品',
        'Crawl the new works from this page',
        'このページから新しい作品を入手する',
        '이 페이지부터 새 작품 긁어오기',
        'Просканировать новые работы с этой страницы',
    ],
    _从本页开始抓取old: [
        '从本页开始抓取旧作品',
        '從本頁開始擷取舊作品',
        'Crawl the old works from this page',
        'このページから古い作品を入手する',
        '이 페이지부터 오래된 작품 긁어오기',
        'Просканировать старые работы с этой страницы',
    ],
    _抓取推荐作品: [
        '抓取推荐作品',
        '擷取推薦作品',
        'Crawl the recommend works',
        '推奨作品をダウンロードする',
        '추천 작품 긁어오기',
        'Просканировать рекомендуемые работы',
    ],
    _抓取推荐作品Title: [
        '抓取页面底部的的推荐作品',
        '擷取頁面底部的推薦作品。',
        'Crawl the recommended works at the bottom of the page',
        'ページの下部で推奨作品をクロールします',
        '페이지 하단 추천 작품 긁어오기',
        'Просканировать рекомендованные работы внизу страницы',
    ],
    _抓取相关作品: [
        '抓取相关作品',
        '擷取相關作品',
        'Crawl the related works',
        '関連作品をダウンロードする',
        '관련 작품 긁어오기',
        'Просканировать похожие работы',
    ],
    _调整完毕: [
        '调整完毕，当前有{}个作品。',
        '調整完畢，目前有 {} 個作品。',
        'The adjustment is complete and now has {} works.',
        '調整が完了し、今、{} の作品があります。',
        '조정이 완료되어, 현재 {}개의 작품이 있습니다.',
        'Настройка завершена и теперь имеет {} работ',
    ],
    _抓取当前作品: [
        '抓取当前作品',
        '擷取目前作品',
        'Crawl the current work',
        '現在の作品をクロールする',
        '현재 작품 긁어오기',
        'Просканировать текущую работу',
    ],
    _抓取当前作品Title: [
        '抓取当前列表里的所有作品',
        '擷取目前清單裡的所有作品',
        'Crawl all the works in the current list',
        '現在のリスト内のすべての作品をクロールする',
        '현재 목록에 있는 모든 작품 긁어오기',
        'Просканировать все работы в текущем списке',
    ],
    _清除多图作品: [
        '清除多图作品',
        '清除多圖作品',
        'Remove multi-image works',
        '複数画像をクリア',
        '여러 이미지 작품 지우기',
        'Удалить работы с несколькими изображениями',
    ],
    _清除动图作品: [
        '清除动图作品',
        '清除動圖作品',
        'Remove ugoira work',
        'うごイラ作品を削除する',
        '움직이는 일러스트 작품 지우기',
        'Убрать Ugoira(gif) работы',
    ],
    _手动删除作品: [
        '手动删除作品',
        '手動刪除作品',
        'Manually delete the work',
        '作品を手動で削除する',
        '수동으로 작품 지우기',
        'Вручную удалить работу',
    ],
    _手动删除作品Title: [
        '可以在下载前手动删除不需要的作品',
        '可以在下載前手動刪除不需要的作品，點擊作品刪除。',
        'You can manually delete unwanted work before downloading',
        'ダウンロードする前に不要な作品を手動で削除することができます',
        '다운로드를 원하지 않는 작품을 수동으로 지울 수 있습니다.',
        'Вы можете вручную удалить нежелательные работы перед загрузкой',
    ],
    _退出手动删除: [
        '退出手动删除',
        '結束手動刪除',
        'Exit manually delete',
        '削除モードを終了する',
        '수동 지우기 종료',
        'Выйти из ручного удаления',
    ],
    _抓取本页作品: [
        '抓取本页作品',
        '擷取本頁作品',
        'Crawl this page works',
        'このページをクロールする',
        '이 페이지의 작품 긁어오기',
        'Просканировать работы с этой страницы',
    ],
    _抓取本页作品Title: [
        '抓取本页列表中的所有作品',
        '擷取本頁清單中的所有作品',
        'Crawl this page works',
        'このページの全ての作品をクロールする',
        '이 페이지의 모든 작품 긁어오기',
        'Просканировать работы с этой страницы',
    ],
    _抓取本排行榜作品: [
        '抓取本排行榜作品',
        '擷取本排行榜作品',
        'Crawl the works in this list',
        'このリストの作品をクロールする',
        '이 목록의 작품 긁어오기',
        'Просканировать работы из этого списка',
    ],
    _抓取本排行榜作品Title: [
        '抓取本排行榜的所有作品，包括现在尚未加载出来的。',
        '擷取本排行榜的所有作品，包括現在尚未載入出來的。',
        'Crawl all of the works in this list, including those that are not yet loaded.',
        'まだ読み込まれていないものを含めて、このリストの作品をダウンロードする',
        '아직 불러오지 않은 작품을 포함하여, 이 목록의 모든 작품을 긁어옵니다.',
        'Просмотреть все работы в этом списке, включая те, которые еще не загружены.',
    ],
    _抓取首次登场的作品: [
        '抓取首次登场作品',
        '擷取首次登場作品',
        'Crawl the debut works',
        '初登場作品をダウンロードする',
        '데뷔작 긁어오기',
        'Просканировать по дебютные работы',
    ],
    _抓取首次登场的作品Title: [
        '只下载首次登场的作品',
        '只下載首次登場的作品',
        'Download only debut works',
        '初登場作品のみダウンロードします',
        '데뷔작만 다운로드',
        'Скачать только дебютные работы',
    ],
    _抓取该页面的图片: [
        '抓取该页面的图片',
        '擷取該頁面的圖片',
        'Crawl the image of the page',
        'ページの画像をクロールする',
        '페이지의 이미지 긁어오기',
        'Просканировать по изображение страницы',
    ],
    _抓取相似图片: [
        '抓取相似图片',
        '擷取相似圖片',
        'Crawl similar works',
        '類似の作品をクロールする',
        '비슷한 작품 긁어오기',
        'Просканировать похожие работы',
    ],
    _想要获取多少个作品: [
        '您想要获取多少个作品？',
        '想要取得多少個作品？',
        'How many works do you want to download?',
        'いくつの作品をダウンロードしたいですか？',
        '몇 개의 작품을 다운로드하시겠습니까?',
        'Сколько работ вы хотите загрузить?',
    ],
    _数字提示1: [
        '-1, 或者大于 0',
        '-1，或是大於 0',
        '-1, or greater than 0',
        '-1、または 0 より大きい',
        '-1, 또는 0보다 크게',
        '-1, или больше 0',
    ],
    _下载大家的新作品: [
        '下载大家的新作品',
        '下載大家的新作品',
        'Download everyone`s new work',
        'みんなの新作をダウンロードする',
        '모두의 새 작품 다운로드',
        'Вседа загружать новые работы',
    ],
    _屏蔽设定: [
        '屏蔽設定',
        '封鎖設定',
        'Mute settings',
        'ミュート設定',
        '차단 설정',
        'Настройки защиты',
    ],
    _举报: ['举报', '回報', 'Report', '報告', '신고', 'Отчет'],
    _输入id进行抓取: [
        '输入 ID 进行抓取',
        '輸入 ID 進行擷取',
        'Type ID to crawl',
        'IDを入力してダウンロードする',
        '유형 ID 긁어오기',
        'Введите ID для вытаскивания',
    ],
    _输入id进行抓取的提示文字: [
        '请输入作品 id。如果有多个 id，则以换行分割（即每行一个id）。',
        '請輸入作品 id。如果有多個 id，則以換行分隔（即每行一個 id）。',
        'Please type the illustration id. If there is more than one id, one id per line.',
        'イラストレーターIDを入力してください。 複数の id がある場合は、1 行に 1 つの id を付けます。',
        '일러스트 작품 ID를 입력해주세요. 여러 개의 ID가 있으면 줄을 바꾸어주세요 (한 줄에 한 개의 ID).',
        'Пожалуйста, введите идентификатор иллюстрации. Если идентификаторов несколько, то по одному идентификатору на строку.',
    ],
    _输入的ID视为图像ID: [
        '因为这个标签页展示的是图像，所以输入的 ID 会被视为图像作品的 ID。',
        '因為這個標籤頁展示的是圖片，所以輸入的 ID 會被視為圖片作品的 ID。',
        'Since this tab displays images, the ID entered will be considered the ID of the image work.',
        'このタブは画像を表示するため、入力したIDが画像作品のIDとなります。',
        '이 탭에는 이미지가 표시되므로 입력한 ID가 해당 이미지 작품의 ID로 간주됩니다.',
        'Поскольку на этой вкладке отображаются изображения, введенный идентификатор будет считаться идентификатором работы с изображением.',
    ],
    _输入的ID视为小说ID: [
        '因为这个标签页展示的是小说，所以输入的 ID 会被视为小说作品的 ID。',
        '因為這個標籤頁展示的是小說，所以輸入的 ID 會被視為小說作品的 ID。',
        'Since this tab displays novels, the ID entered will be treated as the ID of the novel work.',
        'このタブは小説を表示するため、入力したIDは小説作品のIDとして扱われます。',
        '이 탭에는 소설이 표시되므로 입력한 ID는 소설 작품의 ID로 처리됩니다.',
        'Поскольку на этой вкладке отображаются романы, введенный идентификатор будет рассматриваться как идентификатор романа.',
    ],
    _开始抓取: [
        '开始抓取',
        '開始擷取',
        'Start crawl',
        'クロールを開始する',
        '긁어오기 시작',
        'Начать вытаскивание',
    ],
    _给未分类作品添加添加tag: [
        '给未分类的作品添加标签',
        '幫未分類的作品加入標籤',
        'Add tag to uncategorized work',
        '未分類の作品にタグを追加',
        '분류되지 않은 작품에 태그 추가',
        'Добавить метку к неклассифицированной работе',
    ],
    _id不合法: [
        'id不合法',
        'id 不合法',
        'id is illegal',
        'id が不正な',
        '올바르지 않은 ID',
        'Это ID неверно',
    ],
    _快速收藏: [
        '快速收藏 (Ctrl + B)',
        '快速收藏 (Ctrl + B)',
        'Quick bookmarks (Ctrl + B)',
        'クイックブックマーク (Ctrl + B)',
        '빠른 북마크 (Ctrl + B)',
        'Быстрые закладки (Ctrl + B)',
    ],
    _启用: ['启用', '啟用', 'Enable', '有効にする', '활성화', 'Включить'],
    _自动开始下载: [
        '<span class="key">自动</span>开始下载',
        '<span class="key">自動</span>開始下載',
        'Download starts <span class="key">automatically</span>',
        'ダウンロードは自動的に開始されます',
        '<span class="key">자동으로</span> 다운로드 시작',
        'Загрузка начинается <span class="key">автоматически</span>',
    ],
    _自动开始下载的提示: [
        '当“开始下载”状态可用时，自动开始下载，不需要点击下载按钮。',
        '當可下載時自動開始下載，不需要點選下載按鈕。',
        'When the &quot;Start Download &quot; status is available, the download starts automatically and no need to click the download button.',
        '「ダウンロードを開始する」ステータスが利用可能になると、ダウンロードは自動的に開始され、ダウンロードボタンをクリックする必要はありません。',
        '"다운로드 시작" 상태가 활성화되면, 다운로드가 자동으로 시작되고 다운로드 시작 버튼을 클릭할 필요가 없게 됩니다.',
        'При активации этого тумблера загрузка начнется автоматически, без необходимости нажимать кнопку загрузки',
    ],
    _转换任务提示: [
        '正在转换 {} 个文件',
        '正在轉換 {} 個檔案',
        'Converting {} files',
        '{} ファイルの変換',
        '{}개의 파일을 변환하는 중',
        'Преобразование {} файлов',
    ],
    _最近更新: [
        '最近更新',
        '最近更新',
        'What`s new',
        '最近更新する',
        '새로운 기능',
        'Что нового',
    ],
    _确定: ['确定', '確定', 'Ok', '確定', '확인', 'Ок'],
    _file404: [
        '404 错误：文件 {} 不存在。',
        '404 錯誤：檔案 {} 不存在。',
        '404 error: File {} does not exist.',
        '404 エラー：ファイル {} は存在しません。',
        '404 오류: 파일 {}이 존재하지 않습니다.',
        '404 ошибка: Файл {} не существует.',
    ],
    _文件下载失败: [
        '文件 {} 下载失败',
        '檔案 {} 下載失敗',
        'File {} download failed',
        'ファイル {} のダウンロードを失敗しました',
        '파일 {} 다운로드 실패',
        'Загрузка файла {} не удалась',
    ],
    _是否重置设置: [
        '是否重置设置？',
        '確定要重設設定嗎？',
        'Do you want to reset the settings?',
        '設定をリセットしますか？',
        '설정을 초기화하시겠습니까?',
        'Вы хотите сбросить настройки?',
    ],
    _newver: [
        '有新版本可用',
        '有新版本可更新',
        'A new version is available',
        '新しいバージョンがあります',
        '새 버전이 있습니다',
        'Доступна новая версия',
    ],
    _id范围: [
        '<span class="key">ID</span> 范围',
        '<span class="key">ID</span> 範圍',
        '<span class="key">ID</span> range',
        '<span class="key">ID</span> 範囲',
        '<span class="key">ID</span> 범위',
        '<span class="key">ID</span> диапазон',
    ],
    _设置id范围提示: [
        '您可以输入一个作品 id，抓取比它新或者比它旧的作品',
        '可以輸入一個作品 id，擷取比它新或者比它舊的作品。',
        'You can type a work id and crawl works that are newer or older than it',
        '1 つの作品 id を入力することで、それより新しいあるいは古い作品をクロールことができます',
        '작품 ID를 입력하여, 그보다 새로운 혹은 오래된 작품을 긁어올 수 있습니다.',
        'Вы можете ввести идентификатор работы и просмотреть работы, которые новее или старше его',
    ],
    _大于: ['大于', '大於', 'Bigger than', 'より大きい', '보다 큼', 'Больше чем'],
    _小于: ['小于', '小於', 'Less than', 'より小さい', '보다 작음', 'Меньше чем'],
    _投稿时间: [
        '投稿<span class="key">时间</span>',
        '投稿<span class="key">時間</span>',
        'Posting <span class="key">date</span>',
        '投稿日時',
        '게시 <span class="key">날짜</span>',
        '<span class="key">Дата</span> публикации',
    ],
    _设置投稿时间提示: [
        '您可以下载指定时间内发布的作品',
        '可以下載指定時間內發布的作品。',
        'You can download works posted in a specified period of time',
        '指定された時間内に配信された作品をダウンロードすることができます',
        '지정된 기간 내에 게시된 작품을 다운로드할 수 있습니다.',
        'Вы можете загружать работы, размещенные за определенный период времени',
    ],
    _时间范围: [
        '时间范围',
        '時間範圍',
        'Time range',
        '時間範囲',
        '시간 범위',
        'Диапазон времени',
    ],
    _必须大于0: [
        '必须大于 0',
        '必須大於 0',
        'must be greater than 0',
        '0 より大きくなければなりません',
        '0보다 커야합니다',
        'должно быть больше 0',
    ],
    _开始筛选: [
        '开始筛选',
        '開始篩選',
        'Start screening',
        'スクリーニング開始',
        '선별 시작',
        'Начать скрининг',
    ],
    _开始筛选Title: [
        '按照设置来筛选当前标签里的作品。',
        '按照設定來篩選目前標籤裡的作品。',
        'Screen the works in the current tag.',
        '現在のタグにある作品を設定によってスクリーニングする',
        '설정에 따라 현재 태그 내 작품을 선별합니다.',
        'Отобразить работы в с текущим тегом',
    ],
    _在结果中筛选: [
        '在结果中筛选',
        '在結果中篩選',
        'Screen in results',
        '結果の中からスクリーニング',
        '결과 중에서 선별',
        'Экран результатов',
    ],
    _在结果中筛选说明: [
        '您可以改变设置，并在结果中再次筛选。',
        '可以變更設定，並在結果中再次篩選。',
        'You can change the settings and screen again in the results.',
        '設定を変えて、結果の中で再びスクリーニングすることができます。',
        '설정을 변경하고, 결과를 다시 선별할 수 있습니다',
        'Вы можете изменить настройки и снова просмотреть результаты',
    ],
    _抓取筛选结果: [
        '抓取筛选结果',
        '擷取篩選結果',
        'Crawl the screening results',
        'スクリーニングの結果をクロールする',
        '선별 결과 긁어오기',
        'Просканировать результаты скрининга',
    ],
    _尚未开始筛选: [
        '尚未开始筛选',
        '尚未開始篩選',
        'Screening has not started',
        'まだスクリーニングを開始していない',
        '선별이 시작되지 않았습니다',
        'Скрининг не начался',
    ],
    _没有数据可供使用: [
        '没有数据可供使用',
        '沒有資料可供使用',
        'No data is available.',
        '使用可能なデータはない',
        '사용 가능한 데이터가 없습니다',
        'Нет данных',
    ],
    _预览搜索结果: [
        '<span class="key">预览</span>搜索页面的筛选结果',
        '<span class="key">預覽</span>搜尋頁面的篩選結果',
        '<span class="key">Preview</span> filter results on search page',
        '検索ページのフィルタ結果をプレビューします',
        '<span class="key">미리보기</span> 검색 페이지 필터 결과',
        '<span class="key">Предварительный просмотр</span> результатов фильтрации на странице поиска',
    ],
    _预览搜索结果说明: [
        '下载器可以把符合条件的作品显示在当前页面上。如果抓取结果太多导致页面崩溃，请关闭这个功能。<br>启用预览功能时，下载器不会自动开始下载。',
        '下載器可以將符合條件的作品顯示在目前頁面上。如果擷取結果太多導致頁面當掉，請關閉這個功能。<br>啟用預覽功能時，下載器不會自動開始下載。',
        'The downloader can display the qualified works on the current page. If too many crawling results cause the page to crash, turn off this feature.<br>When the preview feature is enabled, the downloader does not start downloading automatically.',
        'ローダは、該当する作品を現在のページに表示することができます。クロール結果が多すぎてページが崩れる場合は、この機能をオフにしてください。<br>プレビュー機能を有効にすると、ダウンロードは自動的に開始されません。',
        '다운로더는 조건에 맞는 작품을 현재 페이지에 표시할 수 있습니다. 긁어오기 결과가 너무 많아서 페이지가 충돌하면 이 기능을 꺼주세요.<br> 미리보기를 사용하면 다운로드가 자동으로 시작되지 않습니다.',
        'Загрузчик может отображать подходящие работы на текущей странице. Пожалуйста, отключите эту функцию, если слишком большое количество результатов просмотра приводит к сбою страницы.<br>Загрузчик не начинает автоматическую загрузку, если включена функция предварительного просмотра.',
    ],
    _目录名使用: [
        '目录名使用：',
        '資料夾名稱使用：',
        'Name: ',
        'ディレクトリ名の使用：',
        '이름: ',
        'Имя: ',
    ],
    _目录名: ['目录名', '資料夾名稱', 'Name', 'ディレクトリ名', '이름', 'Имя'],
    _启用快速收藏: [
        '启用快速收藏',
        '開啟快速收藏',
        'Enable quick bookmark',
        'クイックボックマークを有効にする',
        '빠른 북마크 활성화',
        'Включить быструю закладку',
    ],
    _启用快速收藏说明: [
        '当你点击下载器添加的收藏按钮(☆)，把作品添加到书签时，自动添加这个作品的标签。',
        '當點選下載器新增的收藏按鈕（☆），將作品加入書籤時，自動新增這個作品的標籤。',
        'When you click the favorite button (☆) added by the downloader to bookmark a work, the tag of the work is automatically added.',
        'ダウンローダーに追加されたボックマークボタン「☆」をクリックして、作品をブックマークに追加すると、自動的に作品のタグが追加されます。',
        '다운로더에 추가된 북마크 버튼(☆)을 클릭하여 북마크에 작품을 추가하면 자동으로 이 작품의 태그가 추가됩니다.',
        'Когда вы нажимаете на кнопку Закладка (☆), добавленную загрузчиком, чтобы добавить произведение в закладки, автоматически добавляется тег для этого произведения',
    ],
    _新增设置项: [
        '新增设置项',
        '新增設定項目',
        'Added setting items',
        '新たな機能を追加されました',
        '새로운 설정 항목 추가',
        'Добавить новый элемент настройки',
    ],
    _新增功能: [
        '新增功能',
        '新增功能',
        'New feature',
        '新機能',
        '새로운 기능',
        'Новая фича',
    ],
    _抓取: ['抓取', '擷取', 'Crawl', 'クロール', '긁어오기', 'Сканирование'],
    _下载: ['下载', '下載', 'Download', 'ダウンロード', '다운로드', 'Скачивание'],
    _其他: ['其他', '其他', 'Other', 'その他', '그 외', 'Другие настройки'],
    _更多: ['更多', '更多', 'More', 'もっと', '더보기', 'Больше'],
    _第一张图不带序号: [
        '第一张图不带<span class="key">序号</span>',
        '第一張圖片不包含<span class="key">序號</span>',
        'The first image without a <span class="key">serial number</span>',
        '最初のイメージの番号を削除',
        '<span class="key">일련번호</span>가 없는 첫 번째 이미지',
        'Первое изображение без <span class="key">серийного номера</span>',
    ],
    _第一张图不带序号说明: [
        '去掉每个作品第一张图的序号。例如 80036479_p0 变成 80036479',
        '去掉每個作品第一張圖的序號。例如：80036479_p0 變成 80036479。',
        'Remove the serial number of the first image of each work. For example 80036479_p0 becomes 80036479.',
        '作品ごとの最初のイメージの番号を削除します。例えば 80036479_p0 は 80036479 になります。',
        '작품마다 첫 번째 이미지의 일련번호를 지웁니다.<br>예: 80036479_p0은 80036479가 됩니다.',
        'Удалите серийный номер с первой фотографии каждой работы. Например, 80036479_p0 становится 80036479',
    ],
    _最小值: ['最小值', '最小值', 'Minimum', '最小値', '최소', 'Минимум'],
    _最大值: ['最大值', '最大值', 'Maximum', '最大値', '최대', 'Максимум'],
    _单图作品: [
        '单图作品',
        '單圖作品',
        'Single image works',
        'シングルイメージ作品',
        '단일 이미지 작품',
        'Работа с одним изображением',
    ],
    _彩色图片: [
        '彩色图片',
        '彩色圖片',
        'Color images',
        'カラーイメージ',
        '컬러 이미지',
        'Цветная картинки',
    ],
    _黑白图片: [
        '黑白图片',
        '黑白圖片',
        'Black and white images',
        '白黒イメージ',
        '흑백 이미지',
        'Черно-белые изображения',
    ],
    _不保存图片因为颜色: [
        '{} 没有被保存，因为它的颜色不符合设定。',
        '{} 並未儲存，因為它的色彩不符合設定。',
        '{} was not saved because its colors do not match the settings.',
        '{} は色が設定に合わないため、保存されていません。',
        '{} 색상이 설정과 일치하지 않아, 저장되지 않았습니다.',
        '{} не был(и) сохранен(ы), потому что его цвета не соответствуют настройкам.',
    ],
    _同时转换多少个动图: [
        '同时<span class="key">转换</span>多少个动图',
        '同時<span class="key">轉換</span>多少個動圖',
        'How many animations are <span class="key">converted</span> at the same time',
        '同時変換のうごイラの上限',
        '동시에 <span class="key">변환할</span> 움직이는 일러스트 수',
        'Сколько анимаций <span class="key">преобразуется</span> одновременно',
    ],
    _同时转换多少个动图警告: [
        '同时转换多个动图会增加资源占用。',
        '同時轉換多個動圖會增加資源占用。',
        'Converting multiple animations at the same time will increase resource consumption. ',
        '複数の動画を同時に変換すると、リソースの占有が増加します。',
        '여러 움직이는 일러스트를 동시에 변환하면 리소스가 더 많이 사용됩니다.',
        'Одновременное преобразование нескольких анимаций увеличит потребление ресурсов. ',
    ],
    _提示: ['提示', '提示', 'Tip', 'ヒント', '팁', 'Совет'],
    _提示2: ['提示', '提示', 'Tip', '？', '팁', 'Совет'],
    _fanboxDownloader: [
        'Fanbox 下载器',
        'Fanbox 下載器',
        'Fanbox Downloader',
        'Fanbox ダウンロード',
        'Fanbox 다운로더',
        'Fanbox загрузчик',
    ],
    _不保存图片因为体积: [
        '{} 没有被保存，因为它的体积不符合设定。',
        '{} 並未儲存，因為它的大小不符合設定。',
        '{} was not saved because its size do not match the settings.',
        '{} はファイルサイズが設定に合わないため、保存されていません。',
        '{} 크기가 설정에 맞지 않아, 저장되지 않았습니다.',
        '{} не был(и) сохранен(ы), потому что его размер не соответствует настройкам.',
    ],
    _文件体积限制: [
        '文件<span class="key">体积</span>限制',
        '檔案<span class="key">體積</span>限制',
        'File <span class="key">size</span> limit',
        'ファイルサイズ制限',
        '파일 <span class="key">크기</span> 제한',
        'Ограничение <span class="key">размера</span> файла',
    ],
    _不符合要求的文件不会被保存: [
        '不符合要求的文件不会被保存。',
        '不會儲存不符合要求的檔案。',
        'Files that do not meet the requirements will not be saved.',
        '設定 に合わないファイルは保存されません。',
        '요구 사항을 충족하지 않는 파일은 저장되지 않습니다.',
        'Файлы, не соответствующие требованиям, не будут сохранены',
    ],
    _小说: ['小说', '小說', 'Novel', '小説', '소설', 'Новеллы'],
    _抓取系列小说: [
        '抓取系列小说',
        '擷取系列小說',
        'Crawl series of novels',
        '小説のシリーズをクロールする',
        '시리즈 소설 긁어오기',
        'Просканировать серию новелл',
    ],
    _合并系列小说: [
        '合并系列小说',
        '合併系列小說',
        'Merge series of novels',
        'シリーズ小説の統合',
        '시리즈 소설 합치기',
        'Объединить серию новелл',
    ],
    _小说保存格式: [
        '<span class="key">小说</span>保存格式',
        '<span class="key">小說</span>儲存格式',
        'Save the <span class="key">novel</span> as',
        '<span class="key">小説</span>の保存形式',
        '<span class="key">소설</span>저장 형식',
        'Сохранить <span class="key">новеллу</span> как',
    ],
    _在小说里保存元数据: [
        '在小说里保存<span class="key">元数据</span>',
        '在小說裡儲存<span class="key">元資料</span>',
        'Save <span class="key">metadata</span> in the novel',
        '小説の中にメタデータを保存する',
        '소설 내 <span class="key">메타데이터</span> 저장',
        'Сохранить <span class="key">метаданные</span> новеллы',
    ],
    _在小说里保存元数据提示: [
        '把作者、网址等信息保存到小说里',
        '將作者、網址等資訊儲存到小說裡',
        'Save the author, URL and other information in the file',
        '作者やURLなどの情報をファイルの中に保存します。',
        '작가, URL, 그 외 정보를 소설 내에 저장합니다.',
        'Сохранить автора, URL и другую информацию в файле',
    ],
    _收藏本页面的所有作品: [
        '收藏本页面的所有作品',
        '收藏本頁面的所有作品',
        'Bookmark all works on this page',
        'この頁の全ての作品をブックマークに追加します',
        '이 페이지의 북마크된 모든 작품 다운로드',
        'Перенести в закладки все работы на этой странице',
    ],
    _输出内容太多已经为你保存到文件: [
        '因为输出内容太多，已经为您保存到文件。',
        '因為輸出內容太多，已經為你儲存到檔案。',
        'Because the output is too much, it has been saved to a file.',
        '出力内容が多いため、txt ファイルに保存しました。',
        '출력 내용이 너무 많아, 파일로 저장했습니다.',
        'Так как выход слишком большой, он был сохранен в файл',
    ],
    _不下载重复文件: [
        '不下载<span class="key">重复</span>文件',
        '不下載<span class="key">重複</span>檔案',
        'Don`t download <span class="key">duplicate</span> files',
        '重複ファイルをダウンロードしない',
        '<span class="key">중복</span>파일 다운로드하지 않기',
        'Не загружать <span class="key">дубликаты</span> файлов',
    ],
    _不下载重复文件的提示: [
        '下载器会保存自己的下载记录，以避免下载重复的文件。<br>你可以清除浏览器的下载记录，这不会影响下载器的下载记录。<br>当你清除 Cookie 及其他网站数据时，下载器的记录也会被清除。',
        '下載器會儲存自己的下載紀錄，以避免下載重複的檔案。<br>你可以清除瀏覽器的下載記錄，這不會影響下載器的下載記錄。<br>當你清除 Cookie 及其他網站資料時，下載器的記錄也會被清除。',
        `The downloader will save its download record to avoid downloading duplicate files.<br>You can clear the browser's download history, which will not affect the downloader's download record.<br>When you clear cookies and other site data, the downloader's records will also be cleared.`,
        'ダウンローダーは独自のダウンロード履歴を保存して、重複ファイルのダウンロードを回避する。<br>ブラウザのダウンロード履歴をクリアできますが、ダウンローダのダウンロード記録には影響しません。<br>cookie と他のサイトデータを削除すると、ダウンローダーの記録も削除されます。',
        '다운로더가 중복되는 파일을 다운로드하지 않도록 자신의 다운로드 기록을 저장합니다.<br>브라우저의 다운로드 기록을 지울 수 있으며 이는 다운로더의 다운로드 기록에 영향을 미치지 않습니다.<br>쿠키와 다른 사이트 데이터를 지울 때 다운로드 기록도 삭제됩니다.',
        'Загрузчик хранит собственную историю загрузок, чтобы избежать загрузки дубликатов файлов.<br> Вы можете очистить историю загрузок вашего браузера, и это не повлияет на историю загрузок загрузчика.<br> Когда вы очищаете cookies и другие данные веб-сайта, история загрузчика также будет очищена.<br>',
    ],
    _策略: [
        '策略：',
        '策略：',
        'Strategy:',
        'フィルター：',
        '전략:',
        'Стратегия',
    ],
    _严格: ['严格', '嚴格', 'Strict', '厳格', '엄격하게', 'Строгий'],
    _宽松: ['宽松', '寬鬆', 'Loose', '緩い', '느슨하게', 'Свободный'],
    _严格模式说明: [
        '判断条件：作品的 id、上传日期、文件名',
        '判斷條件：作品的 id、上傳日期、檔名',
        'Judgment conditions: id, upload date, file name of the work',
        '審査条件：作品のID、アップロード日、ファイル名',
        '판정 조건: 작품 ID, 업로드 날짜, 파일명',
        'Условия оценки: идентификатор, дата загрузки, имя файла работы',
    ],
    _宽松模式说明: [
        '判断条件：作品的 id、上传日期',
        '判斷條件：作品的 id、上傳日期',
        'Judgment conditions: id, upload date of the work',
        '審査条件：作品のID、アップロード日',
        '판정 조건: 작품 ID, 업로드 날짜',
        'Условия оценки: идентификатор, дата загрузки работы',
    ],
    _清除下载记录: [
        '清除下载记录',
        '清除下載記錄',
        'Clear download record',
        'ダウンロード記録をクリア',
        '다운로드 기록 지우기',
        'Очистить запись загрузки',
    ],
    确定要清除下载记录吗: [
        '确定要清除下载记录吗？',
        '確定要清除下載記錄嗎？',
        'Are you sure you want to clear download record?',
        'ダウンロード記録を消去してもよろしいですか?',
        '다운로드 기록을 지우시겠습니까?',
        'Вы уверены, что хотите очистить запись загрузки?',
    ],
    _下载记录已清除: [
        '下载记录已清除',
        '已清除下載紀錄',
        'Download record has been cleared',
        'ダウンロード履歴がクリアされました',
        '다운로드 기록이 비워졌습니다',
        'Запись загрузок была очищена',
    ],
    _跳过下载因为重复文件: [
        '检测到文件 {} 已经下载过，跳过此次下载',
        '偵測到檔案 {} 已經下載過，跳過此次下載。',
        'Skip downloading duplicate files {}',
        '重複ファイル {} をスキップ',
        '파일 {}이(가) 이미 다운로드되어 있어, 다운로드를 건너뜁니다',
        'Пропустить загрузку дубликатов файлов {}',
    ],
    _保存用户头像为图标: [
        '保存用户头像为图标',
        '將使用者頭貼另存為圖示檔案',
        'Save user avatar as icon',
        'プロフィール画像をアイコンとして保存',
        '아이콘으로 유저 프로필 이미지 저장',
        'Сохранить аватар пользователя как иконку',
    ],
    _保存用户头像为图标说明: [
        '把用户头像保存为 ico 文件，可以手动设置成文件夹的图标。',
        '將使用者頭貼儲存為 ico 檔案，可以手動設定成資料夾圖示。',
        'Save user avatar as icon',
        'ユーザーのプロフィール画像を ico ファイルとして保存して、フォルダーアイコンとして設定できます。',
        '유저 프로필 이미지를 ico 파일로 저장하면, 디렉토리 아이콘으로 수동 설정할 수 있습니다.',
        'Сохранить аватар пользователя как иконку',
    ],
    _正在保存抓取结果: [
        '正在保存抓取结果',
        '正在儲存擷取結果',
        'Saving crawl results',
        'クロール結果を保存しています',
        '긁어오기 결과 저장 중',
        'Сохранение результатов вытаскивания',
    ],
    _已保存抓取结果: [
        '已保存抓取结果',
        '已儲存擷取結果',
        'Crawl results saved',
        'クロール結果を保存しました',
        '긁어오기 결과가 저장되었습니다',
        'Сохранение результатов вытаскивания',
    ],
    _正在恢复抓取结果: [
        '正在恢复抓取结果',
        '正在還原擷取結果',
        'Restoring crawl results',
        'クロール結果を再開しています',
        '긁어오기 결과 복구 중',
        'Восстановление результатов вытаскивания',
    ],
    _已恢复抓取结果: [
        '已恢复抓取结果',
        '已還原擷取結果',
        'Crawl results resumed',
        'クロール結果を再開しました',
        '긁어오기 결과가 복구되었습니다',
        'Результаты вытаскивания восстановлены',
    ],
    _清空已保存的抓取结果: [
        '清空已保存的抓取结果',
        '清除已儲存的擷取結果',
        'Clear saved crawl results',
        'セーブしたクロール結果をクリアします',
        '저장된 긁어오기 결과 비우기',
        'Очистить сохраненные результаты вытаскивания',
    ],
    _数据清除完毕: [
        '数据清除完毕',
        '資料清除完畢',
        'Data cleared',
        'クリアされたデータ',
        '데이터가 비워졌습니다',
        'Данные очищены',
    ],
    _已跳过n个文件: [
        '已跳过 {} 个文件',
        '已跳過 {} 個檔案',
        '{} files skipped',
        '{} つのファイルをスキップしました',
        '{}개의 파일을 건너뛰었습니다',
        '{} файл(ов) пропущены',
    ],
    _不保存图片因为宽高: [
        '{} 没有被保存，因为它的宽高不符合设定。',
        '{} 並未儲存，因為它的寬高不符合設定。',
        '{} was not saved because its width and height do not match the settings.',
        '{} は幅と高さが設定に合わないため、保存されていません。',
        '{} 너비와 높이가 설정에 맞지 않아, 저장되지 않았습니다.',
        '{} не был(и) сохранен, потому что его ширина и высота не соответствуют настройкам.',
    ],
    _保存: ['保存', '儲存', 'Save', '保存', '저장', 'Сохранить'],
    _加载: ['加载', '載入', 'Load', 'ロード', '불러오기', 'Загрузить'],
    _保存命名规则提示: [
        '保存命名规则，最多 20 个',
        '儲存命名規則，最多 20 個',
        'Save naming rule, up to 20',
        '命名規則を保存します。最大 20 個まで',
        '명명 규칙 저장, 최대 20개',
        'Сохранить правило именования, до 20',
    ],
    _已保存命名规则: [
        '已保存命名规则',
        '已儲存命名規則',
        'Naming rule saved',
        '命名規則を保存しました',
        '명명 규칙이 저장되었습니다.',
        'Правило наименования сохранено',
    ],
    _命名: ['命名', '命名', 'Name', '命名', '이름', 'Имя'],
    _无损: ['无损', '無損', 'Lossless', 'ロスレス', '무손실', 'Без потерь'],
    _文件名长度限制: [
        '文件名<span class="key">长度</span>限制',
        '檔案名稱<span class="key">長度</span>限制',
        'File name <span class="key">length</span> limit',
        'ファイル名の長さ制限',
        '파일명 <span class="key">길이</span> 제한',
        'Лимит <span class="key">длины</span> имени файла',
    ],
    _标签分隔符号: [
        '标签<span class="key">分隔</span>符号',
        '標簽<span class="key">分隔</span>符號',
        'Tag <span class="key">separation</span> symbol',
        'タグ <span class="key">セパレーション</span>シンボル',
        '태그 <span class="key">분리</span> 기호',
        'Тег <span class="key">символ разделения</span>',
    ],
    _标签分隔符号提示: [
        '只会影响这些命名标记的结果：<span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{tags_transl_only}</span>。<br>推荐符号<span class="blue"> , # ^ & _</span>',
        '只會影響這些命名標記的結果：<span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{tags_transl_only}</span>。<br>推薦符號<span class="blue"> , # ^ & _</span>',
        'Only affects results for these named tags: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only}</span>. <br>Recommended symbols <span class="blue"> , # ^ & _</span>',
        '次の名前付きタグの結果にのみ影響します: <span class="blue">{tags}</span>、<span class="blue">{tags_translate}</span>、<span class="blue">{ tags_transl_only }</スパン>。 <br>推奨記号 <span class="blue"> , # ^ & _</span>。',
        '이러한 명명된 태그의 결과에만 영향을 미칩니다: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only }</스팬>. <br>권장 기호 <span class="blue"> , # ^ & _</span>',
        'Влияет только на результаты для следующих именованных тегов: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only </промежуток>. <br>Рекомендуемые символы <span class="blue"> , # ^ & _</span>',
    ],
    _导出csv: [
        '导出 CSV 文件',
        '匯出 CSV 檔',
        'Export CSV file',
        'CSV ファイルをエクスポート',
        'CSV 파일 내보내기',
        'Экспорт в файл CSV',
    ],
    _导出抓取结果: [
        '导出抓取结果',
        '匯出擷取結果',
        'Export results',
        'クロール結果をエクスポート',
        '결과 내보내기',
        'Экспорт результатов',
    ],
    _导入抓取结果: [
        '导入抓取结果',
        '匯入擷取結果',
        'Import results',
        'クロール結果をインポート',
        '결과 불러오기',
        'Импорт результатов',
    ],
    _导入成功: [
        '导入成功',
        '匯入成功',
        'Import successfully',
        'インポート成功',
        '가져오기 성공',
        'Импорт успешен',
    ],
    _导出成功: [
        '导出成功',
        '匯出成功',
        'Export successfully',
        'エクスポート成功',
        '내보내기 성공',
        'Импорт успешен',
    ],
    _图片尺寸: [
        '图片<span class="key">尺寸</span>',
        '圖片<span class="key">尺寸</span>',
        'Image <span class="key">size</span>',
        '画像<span class="key">サイズ</span>',
        '이미지 <span class="key">크기</span>',
        '<span class="key">Размер</span> изображения',
    ],
    _图片尺寸2: [
        '图片尺寸',
        '圖片尺寸',
        'Image size',
        '画像サイズ',
        '이미지 크기',
        'Размер изображения',
    ],
    _原图: ['原图', '原圖', 'Original', 'Original', '원본', 'Оригинал'],
    _普通: ['普通', '普通', 'Regular', 'Regular', '레귤러', 'Обычный'],
    _小图: ['小图', '小圖', 'Small', 'Small', '스몰', 'Маленький'],
    _方形缩略图: [
        '方形缩略图',
        '方形縮圖',
        'Square thumbnail',
        'Square thumbnail',
        '정사각형 썸네일',
        'Квадратная миниатюра',
    ],
    _导出: ['导出', '匯出', 'Export', 'エクスポート', '내보내기', 'Экспорт'],
    _导入: ['导入', '匯入', 'Import', 'インポート', '불러오기', 'Импорт'],
    _清除: ['清除', '清除', 'Clear', 'クリア', '비우기', 'Очистить'],
    _导入下载记录: [
        '导入下载记录',
        '匯入下載紀錄',
        'Import download record',
        'ダウンロード記録をインポート',
        '다운로드 기록 불러오기',
        'Импорт записи загрузки',
    ],
    _导出下载记录: [
        '导出下载记录',
        '匯出下載紀錄',
        'Export download record',
        'ダウンロード記録のエクスポート',
        '다운로드 기록 내보내기',
        'Экспорт записи загрузки',
    ],
    _数据较多需要花费一些时间: [
        '数据较多，需要花费一些时间',
        '資料較多，需要花費一些時間',
        'A lot of data, it will take some time',
        'データ量が多いので少し時間がかかります',
        '데이터가 많아 시간이 좀 걸립니다',
        'Много данных, это займет некоторое время',
    ],
    _完成: ['完成', '完成', 'Completed', '完了', '완료됨', 'Готово'],
    _日期格式: [
        '日期和时间<span class="key">格式</span>',
        '日期和時間<span class="key">格式</span>',
        'Date and time <span class="key">format</span>',
        '日付と時刻の書式',
        '날짜 및 시간 <span class="key">형식</span>',
        '<span class="key">Формат</span> даты и времени',
    ],
    _日期格式提示: [
        '你可以使用以下标记来设置日期和时间格式。这会影响命名规则里的 {date} 和 {upload_date} 和 {task_date}。<br>对于时间如 2021-04-30T06:40:08',
        '你可以使用以下標記來設定日期和時間格式。這會影響命名規則裡的 {date} 和 {upload_date} 和 {task_date}。<br>對於資料如：2021-04-30T06:40:08。',
        'You can use the following notation to set the date and time format. This will affect {date} and {upload_date} and {task_date} in the naming rules. <br>For time such as 2021-04-30T06:40:08',
        '以下のタグを使用して日時と時刻の書式を設定することができます。 これは命名規則の {date} と {upload_date} と {task_date} に影響します。 <br> 例：2021-04-30T06:40:08',
        '다음 표기법을 사용하여 날짜 및 시간 형식을 설정할 수 있습니다.<br>이것은 명명 규칙에 있는 {date}와 {upload_date}와 {task_date}에 영향을 미칩니다.<br>예: 2021-04-30T06:40:08',
        'Для установки формата даты и времени можно использовать следующую нотацию. Это повлияет на {date} и {upload_date} и {task_date} в правилах именования. <br>Для времени, например, 2021-04-30T06:40:08',
    ],
    _命名标记taskDate: [
        '本次任务抓取完成时的时间。例如：<span class="blue">2020-10-21</span>。',
        '本次工作擷取完成時的時間。例如：<span class="blue">2020-10-21</span>。',
        'The time when the task was crawl completed. For example: <span class="blue">2020-10-21</span>',
        'この作業のクロールが完了した時刻です。 例：<span class="blue">2020-10-21</span>',
        '긁어오기 작업을 완료한 날짜입니다. 예: <span class="blue">2020-10-21</span>',
        'Время, когда задание было выполнено. Например: <span class="blue">2020-10-21</span>',
    ],
    _自动检测: [
        '自动检测',
        '自動偵測',
        'Auto',
        '自動検出',
        '자동',
        'Авто детект',
    ],
    _公开: ['公开', '公開', 'Public', '公開', '공개', 'Публичный'],
    _不公开: ['不公开', '非公開', 'Private', '非公開', '비공개', 'Приватный'],
    _已收藏: [
        '已收藏',
        '已收藏',
        'Bookmarked',
        'ブックマークした',
        '북마크됨',
        'В закладках',
    ],
    _已收藏带参数: [
        '已收藏 {}',
        '已收藏 {}',
        'Bookmarked {}',
        'ブックマークした {}',
        '북마크된 {}',
        'В закладках {}',
    ],
    _未收藏: [
        '未收藏',
        '未收藏',
        'Not bookmarked',
        'ブックマークされていない',
        '북마크되지 않음',
        'Не в закладках',
    ],
    _下载之后收藏作品: [
        '下载之后<span class="key">收藏</span>作品',
        '下載之後<span class="key">收藏</span>作品',
        '<span class="key">Bookmark</span> works after downloading',
        'ダウンロードした作品をブックマークする',
        '다운로드 후 작품 <span class="key">북마크</span>',
        '<span class="key">Закладка</span> работает после загрузки',
    ],
    _下载之后收藏作品的提示: [
        '下载文件之后，自动收藏这个作品。',
        '下載檔案之後，自動收藏這個作品。',
        'After you download a file, the downloader will automatically bookmark the work.',
        'ダウンロード後、作品は自動的にブックマークされます。',
        '파일을 다운로드하면, 자동으로 작품을 북마크합니다.',
        'После загрузки файла загрузчик автоматически делает закладку',
    ],
    _收藏设置: [
        '下载器的<span class="key">收藏</span>按钮 (✩)',
        '下載器的<span class="key">收藏</span>按鈕 (✩)',
        `Downloader's <span class="key">bookmark</span> button (✩)`,
        'ダウンローダーの<span class="key">ブックマーク</span>ボタン (✩)',
        '다운로더의 <span class="key">북마크</span> 버튼 (☆)',
        `Кнопка <span class="key">закладок</span> загрузчика (✩)`,
    ],
    _下载器的收藏按钮默认会添加作品的标签: [
        '点击 <span class="blue">✩</span> 按钮时，下载器会收藏这个作品并且附带它的标签。',
        '點選 <span class="blue">✩</span> 按鈕時，下載器會收藏這個作品並且附帶它的標籤。',
        'When the <span class="blue">✩</span> button is clicked, the downloader bookmarks this work and attaches its tag.',
        '<span class="blue">✩</span> ボタンをクリックすると、ダウンローダはこの作品をブックマークし、タグを付けます。',
        '<span class="blue">✩</span> 버튼을 클릭하면 다운로더는 이 작품을 북마크하고 태그를 붙입니다.',
        'При нажатии кнопки <span class="blue">✩</span> загрузчик добавляет эту работу в закладки и прикрепляет свой тег.',
    ],
    _添加tag: [
        '添加标签',
        '加入標籤',
        'Add tag',
        'タグを追加',
        '태그 추가',
        'Добавить тег',
    ],
    _不添加tag: [
        '不添加标签',
        '不加入標籤',
        "Don't add tag",
        'タグなし',
        '태그 추가하지 않기',
        'Не добавлять тег',
    ],
    _用户阻止名单: [
        '用户<span class="key">阻止</span>名单',
        '使用者<span class="key">阻止</span>名單',
        'User <span class="key">block</span> list',
        'ユーザーブロックリスト',
        '유저 <span class="key">차단</span> 목록',
        '<span class="key">Блок</span> списка пользователей ',
    ],
    _用户阻止名单的说明: [
        '不下载这些用户的作品。需要输入用户 id。如果有多个用户 id，使用英文逗号,分割。',
        '不下載這些使用者的作品。需要輸入使用者 id。若有多個使用者 id，使用半形逗號（,）分隔。',
        'The works of these users will not be downloaded. Need to type the user ID. If there are multiple user ID, use comma (,) separated.',
        'これらのユーザーの作品はダウンロードしません。ユーザー ID が必要です。複数のユーザ ID は "," で区切ってください。',
        '이 유저들의 작품은 다운로드되지 않습니다. 유저 ID를 입력해야 합니다.<br>여러 유저 ID가 있는 경우 쉼표(,)로 구분합니다.',
        'Работы этих пользователей не будут загружаться. Необходимо ввести идентификатор пользователя. Если имеется несколько идентификаторов пользователя, используйте разделение запятыми (,).',
    ],
    _全部: ['全部', '全部', 'All', '全部', '전부', 'Все'],
    _任一: ['任一', '任一', 'One', '何れか', '하나만', 'Один'],
    _颜色主题: [
        '颜色<span class="key">主题</span>',
        '色彩<span class="key">主題</span>',
        'Color <span class="key">theme</span>',
        'カラーテーマ',
        '<class key="key">테마</span>',
        'Цветовая <span class="key">тема</span>',
    ],
    _管理设置: [
        '管理<span class="key">设置</span>',
        '管理<span class="key">設定</span>',
        'Manage <span class="key">settings</span>',
        '<span class="key">設定</span>の管理',
        '<span class="key">설정</span> 관리',
        'Изменение <span class="key">настроек</span>',
    ],
    _导出设置: [
        '导出设置',
        '匯出設定',
        'Export settings',
        'エクスポート設定',
        '내보내기',
        'Настройки экспорта',
    ],
    _导入设置: [
        '导入设置',
        '匯入設定',
        'Import settings',
        'インポート設定',
        '불러오기',
        'Настройки импорта',
    ],
    _重置设置: [
        '重置设置',
        '重設設定',
        'Reset settings',
        'リセット設定',
        '설정 초기화',
        'Сброс настроек',
    ],
    _日均收藏数量: [
        '日均收藏数量',
        '日均收藏數量',
        'Average number of daily bookmarks',
        '1 日の平均ブックマーク数',
        '일일 평균 북마크 수',
        'Среднее количество ежедневных закладок',
    ],
    _日均收藏数量的提示: [
        '你可以设置作品的平均每日收藏数量。满足条件的作品会被下载。',
        '您可以設定作品的平均每日收藏數量。滿足條件的作品會被下載。',
        'You can set the average daily bookmarks number of works. Works that meet the conditions will be downloaded.',
        '作品の 1 日の平均ブックマーク数を設定することができます。条件を満した作品はダウンロードされます。',
        '작품의 일일 평균 북마크 수를 설정할 수 있습니다. 조건을 만족한 작품은 다운로드됩니다.',
        'Вы можете установить среднесуточное количество закладок в работах. Работы, удовлетворяющие условиям, будут загружены.',
    ],
    _导出关注列表CSV: [
        '导出关注的用户列表（CSV）',
        '匯出關注的使用者列表（CSV）',
        'Export Followed Users List (CSV)',
        'フォローされているユーザーのリストをエクスポートする（CSV）',
        '팔로우한 사용자 목록 내보내기 (CSV)',
        'Экспорт списка отслеживаемых пользователей (CSV)',
    ],
    _导出关注列表: [
        '导出关注的用户列表（JSON）',
        '匯出關注的使用者列表（JSON）',
        'Export Followed Users List (JSON)',
        'フォローされているユーザーのリストをエクスポートする（JSON）',
        '팔로우한 사용자 목록 내보내기 (JSON)',
        'Экспорт списка отслеживаемых пользователей (JSON)',
    ],
    _批量关注用户: [
        '批量关注用户（JSON）',
        '批次關注使用者（JSON）',
        'Follow users in batches (JSON)',
        'ユーザーをバッチでフォローする（JSON）',
        '일괄적으로 사용자 팔로우 (JSON)',
        'Подписывайтесь на пользователей пакетами (JSON)',
    ],
    _导入导出关注用户列表的说明: [
        '在你或其他用户的 Following 页面里，你可以导出关注的用户列表，也可以导入列表来批量关注用户。<br>当你有多个帐户时，可以使用这个方法同步你关注的用户列表。你也可以复制其他用户的关注用户列表。',
        '在你或其他使用者的 Following 頁面裡，你可以匯出關注的使用者列表，也可以匯入列表來批次關注使用者。<br>當你有多個帳戶時，可以使用這個方法同步你關注的使用者列表。你也可以複製其他使用者的關注使用者列表。',
        "On the Following page of you or other users, you can export the list of followed users, or import the list to follow users in batches. <br>When you have multiple accounts, you can use this method to synchronize the list of users you follow. You can also copy another user's followed user list.",
        '自分または他のユーザーの [フォロー中] ページで、フォローしているユーザーのリストをエクスポートしたり、ユーザーをフォローするリストをバッチでインポートしたりできます。 <br>複数のアカウントをお持ちの場合、この方法を使用して、フォローしているユーザーのリストを同期できます。 別のユーザーのフォローしているユーザー リストをコピーすることもできます。',
        '본인 또는 다른 사용자의 팔로잉 페이지에서 팔로우한 사용자 목록을 내보내거나 목록을 가져와 사용자를 일괄적으로 팔로우할 수 있습니다. <br>계정이 여러 개인 경우 이 방법을 사용하여 팔로우하는 사용자 목록을 동기화할 수 있습니다. 다른 사용자의 팔로우된 사용자 목록을 복사할 수도 있습니다.',
        'На странице «Отслеживание» вас или других пользователей вы можете экспортировать список отслеживаемых пользователей или импортировать список для подписки на пользователей в пакетном режиме. <br>Если у вас несколько учетных записей, вы можете использовать этот метод для синхронизации списка пользователей, на которых вы подписаны. Вы также можете скопировать список отслеживаемых пользователей другого пользователя.',
    ],
    _手动选择作品: [
        '手动选择作品',
        '手動選擇作品',
        'Manually select',
        '手動で作品を選ぶ',
        '수동 선택',
        'Ручной выбор',
    ],
    _快捷键ALTS手动选择作品: [
        '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">S</span> 开始或暂停手动选择作品。<br>选择完毕之后，打开下载器面板，点击“抓取选择的作品”。',
        '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">S</span> 開始或暫停手動選擇作品。<br>選擇完畢之後，開啟下載器面板，點選“抓取選擇的作品”。',
        'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">S</span> to start or pause manual selection of works.<br>After selecting, open the downloader panel and click "Crawl selected works".',
        'ショートカット キー <span class="blue">Alt</span> + <span class="blue">S</span> を使用して、作品の手動選択を開始または一時停止できます。<br>選択後、ダウンローダパネルを開いて「選ばれた作品をクロール」をクリック。',
        '바로 가기 키 <span class="blue">Alt</span> + <span class="blue">S</span>를 사용하여 작품 수동 선택을 시작하거나 일시 중지할 수 있습니다.<br>선택한 후 다운로더 패널을 열고 "선택된 작품 긁어오기"를 클릭합니다.',
        'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">S</span>, чтобы начать или приостановить ручной выбор произведений.<br>После выбора откройте панель загрузчика и нажмите «Стащить выбранные работы».',
    ],
    _抓取选择的作品: [
        '抓取选择的作品',
        '擷取選擇的作品',
        'Crawl selected works',
        '選ばれた作品をクロール',
        '선택된 작품 긁어오기',
        'Стащить выбранные работы',
    ],
    _抓取选择的作品2: [
        '抓取选择的作品 {}',
        '擷取選擇的作品 {}',
        'Crawl selected works {}',
        '選ばれた作品をクロール {}',
        '선택된 작품 긁어오기 {}',
        'Стащить выбранные работы',
    ],
    _清空选择的作品: [
        '清空选择的作品',
        '清空選擇的作品',
        'Clear selected works',
        '選んだ作品をクリアします',
        '선택된 작품 비우기',
        'Очистить выбранные работы',
    ],
    _暂停选择: [
        '暂停选择',
        '暫停選擇',
        'Pause select',
        '選択を一時停止',
        '선택 일시중지',
        'Остановить выбора',
    ],
    _继续选择: [
        '继续选择',
        '繼續選擇',
        'Continue select',
        '選択を続ける',
        '선택 이어하기',
        'Продолжить выбор',
    ],
    _离开页面前提示选择的作品未抓取: [
        '选择的作品尚未抓取。现在离开此页面会导致你选择的作品被清空。',
        '選擇的作品尚未擷取。現在離開此頁面會導致您選擇的作品被清空。',
        'The selected work has not been crawled. Leaving this page now will cause your selected work to be cleared.',
        '選ばれた作品はまだクロールしていません。今このページを離れると、選ばれた作品がクリアされます。',
        '선택된 작품을 아직 긁어오지 않았습니다. 지금 현재 페이지를 떠나면 선택된 작품이 비워집니다.',
        'Выбранная работа не была стащена. Если вы покинете эту страницу, выбранная вами работа будет очищена.',
    ],
    _排除了所有作品类型: [
        '排除了所有作品类型',
        '排除了所有作品類型',
        'Excluded all work types',
        'すべての作品種類を除外しました',
        '모든 작품 유형 제외',
        'Исключены все типы работ',
    ],
    _为作品建立单独的文件夹: [
        '为<span class="key">每个</span>作品建立单独的文件夹',
        '為<span class="key">每個</span>作品建立單獨的資料夾',
        'Create a separate folder for <span class="key">each</span> work',
        '作品ごとに別フォルダを作成',
        '작품마다 <span class="key">별도</span>의 디렉토리 생성',
        'Создайте отдельную папку для <span class="key">каждой</span> работы',
    ],
    _为作品建立单独的文件夹说明: [
        '这里应该使用 {id_num} 代替 {id}',
        '這裡應該使用 {id_num} 代替 {id}',
        'Here {id_num} should be used instead of {id}',
        'ここでは、{id} の代わりに {id_num} を使用する必要があります',
        '여기서는 {id}대신 {id_num}을 사용해야 합니다',
        'Здесь {id_num} следует использовать вместо {id}',
    ],
    _文件数量大于: [
        '文件数量 >',
        '檔案數量 >',
        'Number of files >',
        'ファイル数 >',
        '파일 수 >',
        'Количество файлов >',
    ],
    _保存用户头像: [
        '保存用户头像',
        '儲存使用者頭貼',
        'Save user avatar',
        'ユーザーアイコンの保存',
        '유저 프로필 이미지 저장',
        'Сохранить аватар пользователя',
    ],
    _保存用户封面: [
        '保存用户封面',
        '儲存使用者封面',
        'Save user cover',
        'ユーザーカバーの保存',
        '유저 커버 저장',
        'Сохранить обложку пользователя',
    ],
    _待处理: [
        '待处理',
        '待處理',
        'Pending',
        '処理待ち',
        '처리 대기',
        'В ожидании',
    ],
    _超出最大页码: [
        '超出最大页码：',
        '超出最大頁碼：',
        'Maximum page number exceeded:',
        '最大ページ数を超えました：',
        '최대 페이지 번호 초과:',
        'Превышен максимальный номер страницы:',
    ],
    _针对特定用户屏蔽tag: [
        '针对特定用户屏蔽<span class="key">标签</span>',
        '針對特定使用者排除<span class="key">標籤</span>',
        'Block <span class="key">tags</span> for specific users',
        '特定のユーザーに対して<span class="key">タグ</span>をブロック',
        '특정 유저에 대한 차단 <span class="key">태그</span>',
        'Блокировать <span class="key">теги</span> для определенных пользователей',
    ],
    _展开收起: [
        '展开/收起',
        '展開/摺疊',
        'Expand/Collapse',
        '展開/折りたたみ',
        '확장/축소',
        'Развернуть/Свернуть',
    ],
    _展开: ['展开', '展開', 'Expand', '展開', '확장', 'Развернуть'],
    _收起: ['收起', '摺疊', 'Collapse', '折りたたみ', '축소', 'Свернуть'],
    _把r18作品存入指定的文件夹里: [
        '把 <span class="key">R-18(G)</span> 作品存入指定的文件夹里',
        '把 <span class="key">R-18(G)</span> 作品存入指定的資料夾裡',
        'Save the <span class="key">R-18(G)</span> works in the designated folder',
        '<span class="key">R-18(G)</span> の作品を指定のフォルダに入れる',
        '<span class="key">R-18(G)</span> 작품을 지정된 디렉토리에 저장',
        'Сохраните <span class="key">R-18(G)</span> работы в указанной папке',
    ],
    _必填项不能为空: [
        '必填项不能为空',
        '必填項不能為空',
        'Required fields cannot be empty',
        '必須フィールドが入力されていません',
        '필수 입력 항목은 비워둘 수 없습니다',
        'Обязательные поля не могут быть пустыми',
    ],
    _用户ID必须是数字: [
        '用户 ID 必须是数字',
        '使用者 ID 必須是數字',
        'User ID must be a number',
        'ユーザー ID は数字です',
        '유저 ID는 숫자만 허용합니다',
        'Идентификатор пользователя должен быть числом',
    ],
    _必须是数字: [
        '必须是数字',
        '必須是數字',
        'Number',
        '数字でなければなりません',
        '숫자만 허용',
        'Число',
    ],
    _tag用逗号分割: [
        '多个标签使用英文逗号,分割',
        '多個標籤使用半形逗號（,）分隔',
        'Multiple tags use comma (,) split',
        '複数のタグはカンマ「,」で区切ってください',
        '여러 태그는 쉼표(,)로 구분합니다.',
        'Для нескольких тегов используется разделение запятой (,)',
    ],
    _添加: ['添加', '新增', 'Add', '追加', '추가', 'Добавить'],
    _取消: ['取消', '取消', 'Cancel', 'キャンセル', '취소', 'Отмена'],
    _更新: ['更新', '更新', 'Update', '更新', '업데이트', 'Обновить'],
    _删除: ['删除', '刪除', 'Delete', '削除', '제거', 'Удалить'],
    _添加成功: [
        '添加成功',
        '新增成功',
        'Added successfully',
        '追加されました',
        '성공적으로 추가되었습니다.',
        'Добавлено успешно',
    ],
    _更新成功: [
        '更新成功',
        '更新成功',
        'update completed',
        '更新成功',
        '업데이트에 성공하였습니다.',
        'обновление завершено',
    ],
    _在作品缩略图上显示放大按钮: [
        '在作品缩略图上显示<span class="key">放大</span>按钮',
        '在作品縮圖上顯示<span class="key">放大</span>按鈕',
        'Show <span class="key">zoom</span> button on thumbnail',
        '作品のサムネイルに<span class="key">拡大</span>ボタンを表示',
        '썸네일에 <span class="key">확대</span> 버튼 표시',
        'Показать кнопку <span class="key">увеличить</span> на миниатюре',
    ],
    _在作品缩略图上显示下载按钮: [
        '在作品缩略图上显示<span class="key">下载</span>按钮',
        '在作品縮圖上顯示<span class="key">下載</span>按鈕',
        'Show <span class="key">download</span> button on thumbnail',
        '作品のサムネイルに<span class="key">ダウンロード</span>ボタンを表示',
        '썸네일에 <span class="key">다운로드</span> 버튼 표시',
        'Показать кнопку <span class="key">загрузить</span> на миниатюре',
    ],
    _已发送下载请求: [
        '已发送下载请求',
        '已傳送下載請求',
        'Download request sent',
        'ダウンロードリクエストを送信しました',
        '다운로드 요청 전송',
        'Запрос на скачивание отправлен',
    ],
    _HowToUse: [
        '点击页面右侧的蓝色按钮可以打开下载器面板。<br><br>下载的文件保存在浏览器的下载目录里。<br><br>建议您在浏览器的下载设置中关闭“下载前询问每个文件的保存位置”。<br><br>下载器默认开启了一些增强功能，这可能导致 Pixiv 页面样式改变。你可以在下载器的“更多”标签页中开启或关闭这些功能。',
        '點選頁面右側的藍色按鈕可以開啟下載器面板。<br><br>下載的檔案儲存在瀏覽器的下載目錄裡。<br><br>請不要在瀏覽器的下載選項裡選取「下載每個檔案前先詢問儲存位置」。<br><br>下載器預設開啟了一些增強功能，這可能導致 Pixiv 頁面樣式改變。你可以在下載器的“更多”標籤頁中開啟或關閉這些功能。',
        'Click the blue button on the right side of the page to open the downloader panel.<br><br>The downloaded file is saved in the browser`s download directory. <br><br>It is recommended to turn off "Ask where to save each file before downloading" in the browser`s download settings.<br><br>The downloader has some enhancements turned on by default, which may cause changes in the style of Pixiv pages. You can turn these features on or off in the "More" tab of the downloader.',
        'ページ右側の青いボタンをクリックすると、ダウンローダーパネルが開きます。<br><br>ダウンロードしたファイルは、ブラウザのダウンロードディレクトリに保存されます。<br><br>ブラウザのダウンロード設定で 「 ダウンロード前に各ファイルの保存場所を確認する 」 をオフにすることをお勧めします。<br><br>ダウンローダーにはデフォルトでいくつかの機能拡張が有効になっており、これにより Pixiv ページのスタイルが変更される可能性があります。 これらの機能は、ダウンローダーの「その他」タブでオンまたはオフにできます。',
        '페이지 오른쪽에 있는 파란색 버튼을 클릭하면 다운로드 패널이 열립니다.<br><br>다운로드한 파일은 브라우저의 다운로드 디렉토리에 저장됩니다.<br><br>브라우저의 다운로드 설정에서 "다운로드 전에 각 파일의 저장 위치 확인"을 끄는 것이 좋습니다.<br><br>다운로더에는 기본적으로 몇 가지 향상된 기능이 켜져 있으며 이로 인해 Pixiv 페이지 스타일이 변경될 수 있습니다. 다운로더의 "더 보기" 탭에서 이러한 기능을 켜거나 끌 수 있습니다.',
        'Нажмите синюю кнопку в правой части страницы, чтобы открыть панель загрузчика.<br><br>Загруженный файл сохраняется в каталоге загрузки браузера. <br><br>Рекомендуется отключить "Спрашивать, куда сохранять каждый файл перед загрузкой" в настройках загрузки браузера.<br><br>В загрузчике по умолчанию включены некоторые улучшения, которые могут привести к изменению стиля страниц Pixiv. Вы можете включить или отключить эти функции на вкладке «Дополнительно» загрузчика.',
    ],
    _我知道了: ['我知道了', '我知道了', 'OK', '分かりました', '확인', 'Ок'],
    _背景图片: [
        '<span class="key">背景</span>图片',
        '<span class="key">背景</span>圖片',
        '<span class="key">Background</span> image',
        '<span class="key">背景</span>画像',
        '<span class="key">배경</span> 이미지',
        '<span class="key">Фоновое</span> изображение',
    ],
    _选择文件: [
        '选择文件',
        '選擇檔案',
        'Select a file',
        'ファイルを選択',
        '파일 선택',
        'Выберите файл',
    ],
    _不透明度: [
        '不透明度',
        '不透明度',
        'Opacity',
        '不透明度',
        '투명도',
        'Непрозрачность',
    ],
    _对齐方式: [
        '对齐方式',
        '對齊方式',
        'Alignment',
        '揃え方式',
        '정렬',
        'Выравнивание',
    ],
    _顶部: ['顶部', '頂部', 'top', '上揃え', '상단', 'топ'],
    _居中: ['居中', '居中', 'center', '中央揃え', '중앙', 'центр'],
    _根据作品类型自动建立文件夹: [
        '根据作品<span class="key">类型</span>自动建立文件夹',
        '根據作品<span class="key">類型</span>自動建立資料夾',
        'Create folders based on the <span class="key">type</span> of work',
        '作品種類に応じてフォルダを自動作成',
        '작품 <span class="key">유형</span>에 따라 자동으로 디렉토리 생성',
        'Создание папок на основе <span class="key">вида</span> работы',
    ],
    _使用第一个匹配的tag建立文件夹: [
        '使用第一个匹配的<span class="key">标签</span>建立文件夹',
        '使用第一個符合的<span class="key">標籤</span>建立資料夾',
        'Create a folder with the first matched <span class="key">tag</span>',
        '最初の一致するタグにフォルダを作成',
        '첫 번째 일치하는 <span class="key">태그</span>로 디렉토리 생성',
        'Создать папку с первым совпавшим <span class="key">тегом</span>',
    ],
    _使用匹配的tag建立文件夹的说明: [
        '如果作品的标签列表里含有用户设置的标签，就会使用这个标签建立文件夹（仅限第一个）',
        '如果作品的標籤列表裡含有使用者設定的標籤，就會使用這個標籤建立資料夾（僅限第一個）',
        'If the tag list of the work contains a tag set by the user, this tag will be used to create a folder (only the first one)',
        '作品のタグリストにユーザーが設定したタグが含まれている場合、そのタグを使用してフォルダが作成されます。(最初の1つだけ)',
        '작품의 태그에 유저가 설정한 태그가 포함되어 있다면, 태그를 사용하여 디렉토리를 생성합니다. (첫 번째 태그만 해당)',
        'Если в списке тегов работы есть тег, заданный пользователем, этот тег будет использован для создания папки (только первой)',
    ],
    _全年龄: [
        '全年龄',
        '全年齡',
        'All ages',
        '全年齢',
        '전체 연령',
        'Все возраста',
    ],
    _没有符合条件的结果: [
        '没有符合条件的结果',
        '沒有符合條件的結果',
        'There are no eligible results',
        '対象となる結果はありません',
        '조건에 부합하는 결과가 없습니다',
        'Нет результатов, отвечающих требованиям',
    ],
    _收藏: ['收藏', '收藏', 'Bookmark', 'ブックマーク', '북마크', 'Закладка'],
    _已加入收藏: [
        '已加入收藏',
        '已加入收藏',
        'Bookmarked',
        'ブックマークした',
        '북마크됨',
        'В закладках',
    ],
    _全屏查看: [
        '全屏',
        '全螢幕',
        'Full screen view',
        '全画面表示',
        '전체 화면 보기',
        'Просмотр на весь экран',
    ],
    _抓取id区间: [
        '抓取 ID 区间',
        '擷取 ID 區間',
        'Crawl ID range',
        'ID 範囲をクロール',
        'ID 범위 긁어오기',
        'Стащить диапазон идентификаторов',
    ],
    _抓取id区间说明: [
        '你可以设置一个作品 ID 范围，抓取此范围内的所有作品（包含开始和结束的 id）。<br>注意：如果一次任务中产生的抓取结果数量太多，可能会导致页面崩溃。<br>如果你需要抓取很多 ID，请考虑拆分成多个任务。我建议每批抓取的 ID 数量不要超过 100,000 个。',
        '你可以設定一個作品 ID 範圍，擷取此範圍內的所有作品（包含開始和結束的 id）。<br>注意：如果一次任務中產生的擷取結果數量太多，可能會導致頁面崩潰。<br>如果你需要抓取很多 ID，請考慮拆分成多個任務。我建議每批抓取的 ID 數量不要超過 100,000 個。',
        'You can set a range of work ID and grab all works in this range (including the begin and end ID). <br>Note: If the number of crawling results in a task is too much, it may cause the page to crash.<br>If you need to crawl a lot of IDs, consider splitting it into multiple tasks. I recommend crawling no more than 100,000 IDs per batch.',
        '作品 ID の範囲を設定し、その範囲内のすべての作品をクロールすることができます。「開始 ID と終了 id を含む」<br>注意：1 つのタスクであまりにも多くのクロール結果を生成すると、ページがクラッシュする可能性があります。<br>多数の ID をクロールする必要がある場合は、複数のタスクに分割することを検討してください。 バッチごとにクロールする ID は 100,000 未満にすることをお勧めします。',
        '작품 ID 범위를 설정할 수 있습니다. 이 범위 내의 모든 작품 (시작과 끝 ID 포함).<br>참고: 작업의 긁어오기 결과가 너무 많으면 페이지가 충돌할 수 있습니다.<br>많은 ID를 크롤링해야 하는 경우 이를 여러 작업으로 분할하는 것이 좋습니다. 배치당 100,000개 이하의 ID를 크롤링하는 것이 좋습니다.',
        'Вы можете задать диапазон идентификаторов работ и захватить все работы в этом диапазоне (включая идентификаторы начала и конца). <br>Примечание: Если в задании слишком большое количество результатов стаскивания, это может привести к сбою страницы.<br>Если вам нужно сканировать большое количество идентификаторов, рассмотрите возможность разделения этого процесса на несколько задач. Я рекомендую сканировать не более 100 000 идентификаторов за пакет.',
    ],
    _抓取id区间起点: [
        '请输入开始的 ID: ',
        '請輸入開始的 ID: ',
        'Please type in the beginning ID: ',
        '開始 ID を入力してください: ',
        '시작 ID를 입력해주세요: ',
        'Пожалуйста, введите начальный идентификатор: ',
    ],
    _抓取id区间终点: [
        '请输入结束的 ID: ',
        '請輸入結束的 ID: ',
        'Please type  in the ending ID: ',
        '終了 ID を入力してください: ',
        '끝 ID를 입력해주세요: ',
        'Пожалуйста, введите конечный идентификатор: ',
    ],
    _选项卡切换方式: [
        '<span class="key">选项卡</span>切换方式',
        '<span class="key">頁籤</span>切換方式',
        'How to switch <span class="key">tabs</span>',
        'タブ切り替え方式',
        '<span class="key">탭</span> 전환 방식',
        'Как переключать <span class="key">вкладки</span>',
    ],
    _鼠标经过: [
        '鼠标经过',
        '滑鼠經過',
        'Mouse over',
        'マウスオーバー',
        '마우스 올리기',
        'Наведите мышь',
    ],
    _鼠标点击: [
        '鼠标点击',
        '滑鼠點選',
        'Mouse click',
        'マウスクリック',
        '마우스 클릭',
        'Кликнуть мышкой',
    ],
    _在序号前面填充0: [
        '在序号前面<span class="key">填充 0</span>',
        '在序號前面<span class="key">填充 0</span>',
        '<span class="key">Add 0</span> in front of the serial number',
        'シリアル番号の前に 0 を記入',
        '일련번호 앞 <span class="key">0 채우기</span>',
        '<span class="key">Добавьте 0</span> перед серийным номером',
    ],
    _在序号前面填充0的说明: [
        '这可以解决一些软件不能正确的按照文件名来排序文件的问题。',
        '這可以解決一些軟體不能正確的按照檔名來排序檔案的問題。',
        'This can solve the problem that some software cannot correctly sort files by file name.',
        'これにより、一部のソフトウェアがファイルをファイル名で正しくソートできないという問題を解決できます。',
        '이것은 일부 소프트웨어가 파일 이름별로 파일을 올바르게 정렬할 수 없는 문제를 해결할 수 있습니다.',
        'Это может решить проблему того, что некоторые программы не могут правильно сортировать файлы по имени файла.',
    ],
    _序号总长度: [
        '序号总长度',
        '序號總長度',
        'Total length of serial number',
        'シリアル番号の全長',
        '일련번호 전체 길이',
        'Общая длина серийного номера',
    ],
    _完全一致: [
        '完全一致',
        '完全一致',
        'Perfect match',
        '完全一致',
        '완전 일치',
        'Идеальное совпадение',
    ],
    _部分一致: [
        '部分一致',
        '部分一致',
        'Partial match',
        '部分一致',
        '부분 일치',
        'Частичное совпадение',
    ],
    _位置: ['位置', '位置', 'Position', '位置', '위치', 'Позиция'],
    _左: ['左', '左', 'Left', '左', '왼쪽', 'Слева'],
    _右: ['右', '右', 'Right', '右', '오른쪽', 'Справа'],
    _多图作品只下载前几张图片: [
        '多图作品只下载<span class="key">前几张</span>图片',
        '多圖作品只下載<span class="key">前幾張</span>圖片',
        'Multi-image works only download the <span class="key">first few</span> images',
        'マルチ作品は最初の何枚の画像のみをダウンロードする',
        '여러 이미지 작품은 <span class="key">처음 몇 개</span>의 이미지만 다운로드합니까',
        'При работе с несколькими изображениями загружаются только <span class="key">первые несколько</span> изображений',
    ],
    _多图作品的图片数量上限: [
        '多图作品的图片<span class="key">数量</span>上限',
        '多圖作品的圖片<span class="key">數量</span>上限',
        '<span class="key">Maximum number</span> of images for multi-image works',
        'マルチ作品の最大画像数',
        '여러 이미지 작품의 <span class="key">최대 수</span>',
        '<span class="key">Максимальное количество</span> изображений для работ с несколькими изображениями',
    ],
    _超出此限制的多图作品不会被下载: [
        '超出此限制的多图作品不会被下载',
        '不會下載超出此限制的多圖作品',
        'Multi-image works exceeding this limit will not be downloaded',
        'この制限を超えたマルチ作品はダウンロードされません',
        '이 제한을 초과하는 여러 이미지 작품은 다운로드되지 않습니다.',
        'Работы с несколькими изображениями, превышающие этот лимит, не будут загружены',
    ],
    _在搜索页面添加快捷搜索区域: [
        '在搜索页面添加快捷<span class="key">搜索</span>区域',
        '在搜尋頁面新增快速<span class="key">搜尋</span>區域',
        'Add a quick <span class="key">search</span> area on the search page',
        '検索ページにクイック検索領域を追加します',
        '검색 페이지에 빠른 <span class="key">검색</span> 영역 추가',
        'Добавить область быстрого <span class="key">поиска</span> на странице поиска',
    ],
    _保存作品的元数据: [
        '保存作品的<span class="key">元数据</span>',
        '儲存作品的<span class="key">元資料</span>',
        'Save the <span class="key">metadata</span> of the work',
        '作品のメタデータを保存する',
        '작품 <span class="key">메타데이터</span> 저장',
        'Сохранить <span class="key">метаданные</span> работы',
    ],
    _保存作品的元数据说明: [
        '为每个作品建立一个 txt 文件保存它的元数据',
        '為每個作品建立一個 txt 檔案儲存它的元資料',
        'Create a txt file for each work to save its metadata',
        '作品ごとに txt ファイルを作成して、メタデータを保存します',
        '각 작품에 대한 메타데이터를 저장하는 txt 파일 생성',
        'Создайте txt-файл для каждой работы, чтобы сохранить ее метаданные',
    ],
    _在不同的页面类型中使用不同的命名规则: [
        '在不同的页面类型中使用<span class="key">不同</span>的命名规则',
        '在不同的頁面類型中使用<span class="key">不同</span>的命名規則',
        'Use <span class="key">different</span> naming rules in different page types',
        'ページの種類によって異なる命名規則を使用',
        '페이지 유형에 따라 <span class="key">다른</span> 명명 규칙 사용',
        'Использовать <span class="key">различные</span> правила именования в разных типах страниц',
    ],
    _显示高级设置: [
        '显示<span class="key">高级</span>设置',
        '顯示<span class="key">進階</span>設定',
        'Show <span class="key">advanced</span> settings',
        '詳細設定を表示する',
        '<span class="key">고급</span> 설정 보기',
        'Показать <span class="key">расширенные</span> настройки',
    ],
    _显示高级设置说明: [
        '被隐藏的设置仍然会发挥作用',
        '被隱藏的設定仍然會發揮作用',
        'Hidden settings will still work',
        '隠していた設定がそのまま機能する',
        '숨겨진 설정은 계속 작동합니다.',
        'Скрытые настройки будут работать',
    ],
    _状态码为0的错误提示: [
        '下载时发生错误，状态码为 0，请求未成功。可能的原因：<br><br>1. 系统磁盘的剩余空间可能不足（通常是 C 盘）（建议剩余空间大于 4GB）。请尝试清理系统磁盘空间，然后重新启动浏览器，继续未完成的下载。<br><br>2. 网络错误。可能是网络代理导致的问题。如果你使用 Nginx 或者 Apache 反代理访问 pixiv，请换成梯子。<br><br>3. 可以尝试重启浏览器，或者禁用此扩展然后重新启用，并刷新这个标签页。',
        '下載時發生錯誤，狀態碼為 0，請求未成功。可能的原因：<br><br>1. 系統磁碟的剩餘空間可能不足（通常是 C 盤）（建議剩餘空間大於 4GB）。請嘗試清理系統磁碟空間，然後重新啟動瀏覽器，繼續未完成的下載。<br><br>2. 網路錯誤。可能是網路代理導致的問題。<br><br>3. 可以嘗試重啟瀏覽器，或者禁用此擴充套件然後重新啟用，並重新整理這個標籤頁。',
        'An error occurred while downloading, the status code is 0, and the request was unsuccessful. Possible reasons: <br><br>1. The remaining space of the system disk may be insufficient (usually C drive)(it is recommended that the remaining space be greater than 4GB). Please try to clear the system disk space, and then restart the browser to continue the unfinished download. <br><br>2. Network error. It may be a problem caused by a network proxy.<br><br>3. You can try to restart the browser, or disable and re-enable the extension, and refresh the tab.',
        'ダウンロード中にエラーが発生し、ステータスコードは0で、リクエストは失敗しました。 考えられる理由：<br> <br> 1。 システムディスクの残りのスペースが不足している可能性があります(通常はCドライブ)（残りのスペースは4GBを超えることをお勧めします）。 システムのディスク領域をクリアしてから、ブラウザを再起動して、未完了のダウンロードを続行してください。 <br> <br> 2。 ネットワークエラー。 ネットワークプロキシが原因の問題である可能性があります。<br><br>3. ブラウザを再起動するか、拡張機能を無効にしてから再度有効にして、タブを更新してみてください。',
        '다운로드 중 오류가 발생했으며, 상태 코드가 0이고 요청에 실패했습니다. 가능한 원인: <br><br>1. 시스템 디스크의 남은 공간이 부족할 수 있습니다(보통 C드라이브)(남은 공간은 4GB보다 큰 것이 좋습니다). 시스템 디스크 공간을 비운 다음 브라우저를 다시 시작하여 완료되지 않은 다운로드를 계속해주세요. <br><br>2. 네트워크 오류. 네트워크 프록시로 인한 문제일 수 있습니다.<br><br>3. 브라우저를 다시 시작하거나 확장 프로그램을 비활성화했다가 다시 활성화하고 탭을 새로 고칠 수 있습니다.',
        'Во время загрузки произошла ошибка, код состояния равен 0, и запрос был выполнен неудачно. Возможные причины: <br><br>1. Оставшегося места на системном диске может быть недостаточно (обычно это диск C) (рекомендуется, чтобы оставшееся место было больше 4 ГБ). Пожалуйста, попробуйте освободить место на системном диске, а затем перезапустите браузер, чтобы продолжить незаконченную загрузку. <br><br>2. Ошибка сети. Это может быть проблема, вызванная сетевым прокси-сервером.<br><br>3. Вы можете попробовать перезапустить браузер или отключить и снова включить расширение и обновить вкладку.',
    ],
    _下载完成后显示通知: [
        '下载完成后显示<span class="key">通知</span>',
        '下載完成後顯示<span class="key">通知</span>',
        'Show <span class="key">notification</span> after download is complete',
        'ダウンロードが完了した後に通知を表示する',
        '다운로드가 완료되면 <span class="key">알림</span> 표시',
        'Показать <span class="key">уведомление</span> после завершения загрузки',
    ],
    _高亮显示关键字: [
        '<span class="key">高亮</span>显示关键字',
        '<span class="key">標明</span>顯示關鍵字',
        '<span class="key">Highlight</span> keywords',
        'キーワードを強調表示',
        '<span class="key">강조</span> 키워드 표시',
        '<span class="key">Выделить</span> ключевые слова',
    ],
    _抓取标签列表: [
        '抓取标签列表',
        '擷取標籤列表',
        'Crawl a list of tags',
        'タグのリストをクロール',
        '태그 긁어오기',
        'Сканировать список тегов',
    ],
    _抓取标签列表的输入框提示: [
        '请输入你要抓取的标签列表。多个标签之间使用换行分割',
        '請輸入你要擷取的標籤列表。多個標籤之間使用換行分隔',
        'Please type the list of tags you want to crawl. Use line breaks between multiple tags',
        'クロールしたいタグのリストを入力してください。 複数のタグを改行で分割',
        '긁어올 태그를 입력해주세요. 여러 태그는 줄 바꿈 사용',
        'Пожалуйста, введите список тегов, которые вы хотите просмотреть. Используйте разрывы строк между несколькими тегами',
    ],
    _抓取标签列表的文件夹提示: [
        '在抓取标签列表时，你可以使用 {page_tag} 或者 {page_title} 标记获取当前抓取的标签，并用来建立文件夹。例如：{page_tag}/{id}',
        '在擷取標籤列表時，你可以使用 {page_tag} 或者 {page_title} 標記獲取目前擷取的標籤，並用來建立資料夾。例如：{page_tag}/{id}',
        'When crawling the tag list, you can use {page_tag} or {page_title} tags to get the tags currently crawled and use them to create folders. For example: {page_tag}/{id}',
        'タグリストをクロールする時に、 {page_tag} や {page_title}を使用すると、現在クロールされているタグを取得し、それらを使ってフォルダを作成することができます。例：{page_tag}/{id}',
        '태그를 긁어올 때 {page_tag} 또는 {page_title} 태그를 사용하여, 긁어온 태그로 디렉토리를 생성할 수 있습니다. 예: {page_tag}/{id}',
        'При сканировании списка тегов вы можете использовать теги {page_tag} или {page_title}, чтобы получить теги, которые в данный момент просматриваются, и использовать их для создания папок. Например: {page_tag}/{id}',
    ],
    _停止抓取标签列表: [
        '停止抓取标签列表',
        '停止擷取標籤列表',
        'Stop crawling the list of tags',
        'タグリストのクロールを停止',
        '태그 긁어오기 정지',
        'Прекратить сканирование списка тегов',
    ],
    _等待下载的标签: [
        '等待下载的标签',
        '等待下載的標籤',
        'Tags waiting to be downloaded',
        'ダウンロード待ちのタグ',
        '다운로드 대기 중인 태그',
        'Теги, ожидающие загрузки',
    ],
    _你确定要停止抓取吗: [
        '你确定要停止抓取吗？',
        '確定要停止擷取嗎？',
        'Are you sure you want to stop crawling?',
        '本当にクロールをやめたいのか',
        '긁어오기를 중지하시겠습니까?',
        'Ты уверен, что хочешь перестать сканировать?',
    ],
    _只能在搜索页面使用: [
        '只能在搜索页面使用',
        '只能在搜尋頁面使用',
        'Can only be used on the search page',
        '検索ページでのみ使用できます',
        '검색 페이지에서만 사용 가능',
        'Можно использовать только на странице поиска',
    ],
    _自动导出抓取结果: [
        '自动<span class="key">导出</span>抓取结果',
        '自動<span class="key">匯出</span>擷取結果',
        'Automatically <span class="key">export</span> crawl results',
        'クロール結果の自動エクスポート',
        '자동으로 긁어오기 결과 <span class="key">내보내기</span>',
        'Автоматически <span class="key">экспортировать</span> результаты сканирования',
    ],
    _抓取结果: [
        '抓取结果',
        '擷取結果',
        'Crawl results',
        'クロール結果',
        '긁어오기 결과',
        'Сканировать результаты',
    ],
    _文件格式: [
        '文件格式',
        '檔案格式',
        'File format',
        'ファイル形式',
        '파일 형식',
        'Формат файла',
    ],
    _格式: ['格式', '格式', 'Format', '形式', '형식', 'Формат'],
    _预览作品: [
        '<span class="key">预览</span>作品',
        '<span class="key">預覽</span>作品',
        '<span class="key">Preview</span> works',
        '作品のプレビュー',
        '작품 <span class="key">미리보기</span>',
        '<span class="key">Превью</span> работает',
    ],
    _点击鼠标左键可以关闭预览图: [
        '点击鼠标左键可以关闭预览图',
        '點選滑鼠左鍵可以關閉預覽圖',
        'Click the left mouse button to close the preview',
        'マウスの左クリックでプレビュー画像を閉じる',
        '마우스 왼쪽 버튼을 클릭하면 미리보기를 닫습니다',
        'Нажмите левую кнопку мыши, чтобы закрыть предварительный просмотр',
    ],
    _尺寸: ['尺寸', '尺寸', 'Size', 'サイズ', '크기', 'Размер'],
    _允许鼠标停留在预览图上: [
        '允许鼠标停留在预览图上',
        '允許滑鼠停留在預覽圖上',
        'Allow the mouse to stay on the preview image',
        'プレビュー画像の上にマウスを置くことができます',
        '마우스가 미리보기 이미지 위에서 유지되도록 허용',
        'Разрешить мыши оставаться на изображении предварительного просмотра',
    ],
    _点击预览图时下载作品: [
        '点击预览图时下载作品',
        '點選預覽圖時下載作品',
        'Download the work when you click on the preview',
        'プレビュー画像をクリックするとその作品がダウンロードされます',
        '미리보기 이미지를 클릭하면 작품 다운로드',
        'Загружать работу при нажатии на предварительный просмотр',
    ],
    _转换动图时页面被隐藏的提示: [
        '这个标签页正在转换动图。如果这个标签页被隐藏了，转换速度可能会变慢。',
        '這個標籤頁正在轉換動圖。如果這個標籤頁被隱藏了，轉換速度可能會變慢。',
        'This tab page is converting ugoira. If this tab page is hidden, the conversion speed may slow down.',
        'このタブページはうごイラを変換しています。 このタブを非表示にすると、変換速度が低下する場合があります。',
        '이 탭은 움직이는 일러스트를 변환하는 중입니다.이 탭이 숨겨지면 변환 속도가 느려질 수 있습니다.',
        'Эта страница вкладки преобразует ugoira. Если эта страница вкладки скрыта, скорость конвертации может замедлиться.',
    ],
    _原始尺寸: [
        '原始尺寸',
        '原始尺寸',
        'Original size',
        'オリジナルサイズ',
        '원본 크기',
        'Оригинальный размер',
    ],
    _增强: ['增强', '增強', 'Enhance', '強化機能', '향상', 'Улучшение'],
    _长按右键显示大图: [
        '在缩略图上长按鼠标右键时显示<span class="key">大图</span>',
        '在縮圖上長按滑鼠右鍵時顯示<span class="key">大圖</span>',
        'Long press the right mouse button on the thumbnail to display the <span class="key">large image</span>',
        'サムネイルでマウスの右ボタンを長押しすると、大きな画像が表示されます。',
        '썸네일을 마우스 오른쪽 버튼으로 클릭했을 때 <span class="key">큰 이미지</span> 표시',
        'Длительное нажатие правой кнопки мыши на миниатюре для отображения <span class="key">большого изображения</span>',
    ],
    _鼠标滚轮切换图片: [
        '预览多图作品时，可以使用鼠标滚轮切换图片。',
        '預覽多圖作品時，可以使用滑鼠滾輪切換圖片。',
        'When previewing multi-image works, you can use the mouse wheel to switch images.',
        '複数画像をプレビューする際に、マウスホイールを使って画像を切り替えることができます。',
        '여러 이미지 작품을 미리 볼 때, 마우스 휠을 사용하여 이미지를 전환할 수 있습니다.',
        'При предварительном просмотре работ с несколькими изображениями можно использовать колесико мыши для переключения изображений',
    ],
    _whatisnew: [
        `修复因为 Pixiv 的变化而导致的抓取失败的问题。`,
        `修復因為 Pixiv 的變化而導致的抓取失敗的問題。`,
        `Fixed crawl failures due to Pixiv changes.`,
        `Pixiv の変更によるクロールの失敗を修正しました。`,
        `Pixiv 변경으로 인한 크롤링 실패를 수정했습니다.`,
        'Исправлены сбои в сканировании из-за изменений в Pixiv',
    ],
    _等待时间: [
        '等待时间',
        '等待時間',
        'Waiting time',
        '待ち時間',
        '대기 시간',
        'Время ожидания',
    ],
    _格式错误: [
        '格式错误',
        '格式錯誤',
        'Format error',
        'フォーマットエラー',
        '형식 오류',
        'Ошибка форматантирования',
    ],
    _下载数量错误: [
        '下载的页数（作品）数量设置错误',
        '下載的頁數（作品）數量設定錯誤',
        'The number of downloaded pages (works) is set incorrectly',
        '下载页数（作品）设置不正确',
        '다운로드할 페이지 수(작품) 설정 오류',
        'Количество загруженных страниц (работ) установлено неверно',
    ],
    _默认下载多页: [
        '开始抓取, 如有多页，默认会下载全部。',
        '開始擷取，如有多頁，預設會下載全部。',
        'Start crawl, if there are multiple pages, the default will be downloaded.',
        'クロールを開始する、複数のページがある場合、デフォルトですべてをダウンロードされます。',
        '긁어오기를 시작합니다. 여러 페이지가 있으면 기본적으로 모두 다운로드됩니다.',
        'Начать сканирование, если есть несколько страниц, все будут загружены по умолчанию.',
    ],
    _赞助我: [
        '赞助我',
        '贊助我',
        'Sponsor me',
        '支援する',
        '후원하기',
        'Поддержать меня',
    ],
    _赞助方式提示: [
        `非常感谢您的支持！<br>
    您可以在 Patreon 上赞助我：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a><br>
    中国大陆用户可以在“爱发电”上赞助我：<br>
    <a href="https://afdian.net/@xuejianxianzun" target="_blank">https://afdian.net/@xuejianxianzun</a><br>
    也可以扫描二维码：<br>
    <a href="https://github.com/xuejianxianzun/PixivBatchDownloader#%E6%94%AF%E6%8C%81%E5%92%8C%E6%8D%90%E5%8A%A9" target="_blank">在 Github 上查看二维码</a> 或者加入 QQ 群 499873152，在群文件里查看二维码。
    `,
        `非常感謝您的支持！<br>
    您可以在 Patreon 上贊助我：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a><br>
    中國大陸使用者可以在“愛發電”上贊助我：<br>
    <a href="https://afdian.net/@xuejianxianzun" target="_blank">https://afdian.net/@xuejianxianzun</a><br>
    也可以掃描行動條碼（QR Code）：<br>
    <a href="https://github.com/xuejianxianzun/PixivBatchDownloader#%E6%94%AF%E6%8C%81%E5%92%8C%E6%8D%90%E5%8A%A9" target="_blank">在 Github 上檢視行動條碼</a>。
    `,
        `Thank you very much for your support!<br>
    You can sponsor me on Patreon: <br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
        `ご支援してくださった皆様、本当にありがとうございました。<br>
    ご支援してくださった方は、以下の Patreon で：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank"> https://www.patreon.com/xuejianxianzun </a>
    `,
        `지원해주셔서 정말 감사합니다!<br>
    Patreon에서 저를 후원해주세요<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
        `Большое вам спасибо за вашу поддержку!<br>
    Вы можете спонсировать меня на Patreon: <br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
    ],
    _替换方形缩略图以显示图片比例: [
        '替换方形<span class="key">缩略图</span>以显示图片比例',
        '替換方形<span class="key">縮圖</span>以顯示圖片比例',
        'Replace square <span class="key">thumbnails</span> to show image ratio',
        '正方形のサムネイルを置き換えて、画像のスケールを表示。',
        '이미지 종횡비를 표시하기 위해 정사각형 <span class="key">썸네일</span> 교체',
        'Замените квадратные <span class="key">миниатюры</span>, чтобы показать соотношение сторон изображения',
    ],
    _只有一个抓取结果时不建立文件夹: [
        '只有一个抓取结果时<span class="key">不建立</span>文件夹',
        '只有一個擷取結果時<span class="key">不建立</span>資料夾',
        '<span class="key">Do not create</span> a folder when there is only one crawl result',
        'クロール結果が１つのみの場合、フォルダを作成しない',
        '긁어오기 결과가 하나일 때 디렉토리 <span class="key">생성하지 않기</span>',
        'Когда есть только один результат сканирования, <span class="key">никакая</span> папка не создается',
    ],
    _搜索页面页数限制: [
        '由于 pixiv 的限制，下载器最多只能抓取到第 {} 页。',
        '由於 pixiv 的限制，下載器最多只能擷取到第 {} 頁。',
        'Due to the limitation of pixiv, the downloader can only crawl up to the {}th page.',
        'pixiv の制限により、ダウンローダーは {} ページ目までしかクロールできません。',
        'pixiv 제한으로 인해 최대 {} 페이지까지만 다운로드 받을 수 있습니다.',
        'Из-за ограничений pixiv загрузчик может сканировать только до {}-й страницы',
    ],
    _获取图片的宽高时出现错误: [
        '获取图片的宽高时出现错误：',
        '獲取圖片的寬高時出現錯誤：',
        'An error occurred while getting the width and height of the image:',
        '画像の幅と高さの取得中にエラーが発生しました：',
        '이미지의 너비를 가져오는 도중 오류가 발생했습니다:',
        'Произошла ошибка при получении ширины и высоты изображения:',
    ],
    _上限: ['上限', '上限', 'Upper limit', '上限', '상한', 'Верхний предел'],
    _预览搜索结果的数量达到上限的提示: [
        '预览搜索结果的数量已经达到上限，剩余的结果不会显示。',
        '預覽搜尋結果的數量已經達到上限，剩餘的結果不會顯示。',
        'The number of preview search results has reached the upper limit, and the remaining results will not be displayed.',
        'プレビュー検索結果の数が上限に達し、残りの結果は表示されません。',
        '미리보기 검색 결과 수가 상한에 도달하여, 남은 결과는 표시되지 않습니다.',
        'Количество результатов предварительного поиска достигло верхнего предела, и оставшиеся результаты не будут отображаться.',
    ],
    _新增命名标记: [
        '新增命名标记',
        '新增命名標記',
        'Add named tag',
        '名前付きタグを追加',
        '명명된 태그 추가',
        'Добавить именованный тег',
    ],
    _自定义用户名: [
        '自定义<span class="key">用户名</span>',
        '自訂<span class="key">使用者名稱</span>',
        'Customize <span class="key">username</span>',
        'カスタムユーザー名',
        '사용자 정의 <span class="key">유저명</span>',
        'Настроить <span class="key">имя пользователя</span>',
    ],
    _自定义用户名的说明: [
        `有些用户可能会改名，如果你想使用他原来的名字，你可以在这里手动设置他的名字。<br>
    你也可以为用户设置别名。<br>
    当你在命名规则中使用 {user} 标记时，下载器会优先使用你设置的名字。`,
        `有些使用者可能會改名，如果你想使用他原來的名字，你可以在這裡手動設定他的名字。<br>
    你也可以為使用者設定別名。<br>
    當你在命名規則中使用 {user} 標記時，下載器會優先使用你設定的名字。`,
        `Some users may change their name. If you want to use his original name, you can manually set his name here. <br>
    You can also set aliases for users. <br>
    When you use the {user} tag in the naming rule, the downloader will give priority to the name you set.`,
        `ユーザーによっては名前を変更する場合があります。元の名前を使いたい場合は、ここで名前を手動で設定することができます。<br>
    また、ユーザーの別名を設定することも可能です。<br>
    命名規則で {user} タグを使用すると、ダウンローダーは設定された名前を優先的に使用します。`,
        `일부 유저는 이름을 바꿀 수 있습니다. 만약 당신이 그의 원래 이름을 사용하고 싶다면, 당신은 여기에서 그의 이름을 수동으로 설정할 수 있습니다.<br>
    사용자의 별칭을 설정할 수도 있습니다. <br>
    명명 규칙에 {user} 태그를 사용할 때 다운로드더가 사용자 정의 유저명을 우선시합니다.`,
        `Некоторые пользователи могут изменить свое имя. Если вы хотите использовать его оригинальное имя, вы можете вручную задать его имя здесь. <br>
    Вы также можете задать псевдонимы для пользователей. <br>
    Когда вы используете тег {user} в правиле именования, загрузчик будет отдавать приоритет имени, которое вы задали.`,
    ],
    _移除用户名中的at和后续字符: [
        '移除用户名中的 <span class="key">@</span> 和后续字符',
        '移除使用者名稱中的 <span class="key">@</span> 和後續字元',
        'Remove <span class="key">@</span> and subsequent characters in username',
        'ユーザー名から <span class="key">@</span> 以降の文字を削除する',
        '유저명에서 @와 후속 문자 제거',
        'Удалить <span class="key">@</span> и последующие символы в имени пользователя',
    ],
    _移除用户名中的at和后续字符的说明: [
        '例如：Anmi@画集発売中 → Anmi',
        '例如：Anmi@画集発売中 → Anmi',
        'For example：Anmi@画集発売中 → Anmi',
        '例：Anmi@画集発売中 → Anmi',
        '예: Anmi@画集発売中 → Anmi',
        'Например: Anmi@画集発売中 → Anmi',
    ],
    _抓取被限制时返回空结果的提示: [
        'Pixiv 返回了空数据。下载器已暂停抓取，并且会在等待几分钟后继续抓取。(429)<br>这说明您的账号被 Pixiv 限制访问了，等待几分钟即可恢复正常。<br>您可以启用“减慢抓取速度”功能来减少 429 问题出现的概率。',
        'Pixiv 返回了空資料。下載器已暫停抓取，並且會在等待幾分鐘後繼續抓取。(429)<br>這說明您的賬號被 Pixiv 限制訪問了，等待幾分鐘即可恢復正常。<br>您可以啟用“減慢抓取速度”功能來減少 429 問題出現的機率。',
        'Pixiv returned empty data. The downloader has paused crawling and will resume crawling after a few minutes. (429)<br>This means that your account has been restricted by Pixiv, please wait for a few minutes for it to return to normal.<br>You can reduce the chances of 429 issues by enabling the "Slow down crawl" feature.',
        'Pixivが空のデータを返しました。 ダウンローダーはクロールを一時停止し、数分後にクロールを再開します。(429)<br>これは、あなたのアカウントが Pixiv によって制限されていることを意味します。通常の状態に戻るまで数分お待ちください。<br>"クロールを遅くする" 機能を有効にすると、429 の問題が発生する可能性を減らすことができます。',
        'Pixiv가 빈 데이터를 반환했습니다. 다운로더가 긁어오기를 일시 중지하고 몇 분 동안 기다린 후 긁어오기를 계속합니다. (429)<br>이것은 귀하의 계정이 Pixiv에 의해 제한되었음을 의미합니다. 정상으로 돌아갈 때까지 몇 분 정도 기다리십시오.<br>"천천히 크롤링" 기능을 활성화하면 429 문제 발생 가능성을 줄일 수 있습니다.',
        'Pixiv вернул пустые данные. Загрузчик приостановил загрузку и возобновит ее через несколько минут. (429)<br>Это означает, что ваша учетная запись была ограничена Pixiv, подождите несколько минут, пока она вернется в нормальное состояние.<br>Вы можете снизить вероятность возникновения ошибок 429, включив функцию «Замедлить сканирование».',
    ],
    _搜索模式: [
        '搜索模式',
        '搜尋模式',
        'Search mode',
        '検索モード',
        '검색 모드',
        'Режим поиска',
    ],
    _标签部分一致: [
        '标签（部分一致）',
        '標籤（部分一致）',
        'Tags (partial match)',
        'タグ（部分一致）',
        '태그 (부분 일치)',
        'Теги (частичное совпадение)',
    ],
    _标签完全一致: [
        '标签（完全一致）',
        '標籤（完全一致）',
        'Tags (perfect match)',
        'タグ（完全一致）',
        '태그 (완전 일치)',
        'Теги (идеальное совпадение)',
    ],
    _标题说明文字: [
        '标题、说明文字',
        '標題、說明文字',
        'Title, Caption',
        'タイトル・キャプション',
        '제목, 설명',
        'Название, Подпись',
    ],
    _正文: ['正文', '本文', 'Text', '本文', '본문', 'Текст'],
    _标签标题说明文字: [
        '标签、标题、说明文字',
        '標籤、標題、說明文字',
        'Tags, Titles, Captions',
        'タグ・タイトル・キャプション',
        '태그, 제목, 설명',
        'Теги, Заголовки, Подписи',
    ],
    _save_file_failed_tip: [
        `{} 保存失败，code：{}。下载器将会重试下载这个文件。`,
        `{} 儲存失敗，code：{}。下載器將會重試下載這個檔案。`,
        `{} save failed, code: {}. The downloader will retry to download the file.`,
        `{} 保存に失敗しました。code：{}。ダウンローダーはファイルのダウンロードを再試行します。`,
        `{} 저장에 실패했습니다. 코드: {}. 다운로드더가 파일 다운로드를 다시 시도합니다.`,
        `{} сохранение не удалось, код: {}. Загрузчик повторит попытку загрузить файл.`,
    ],
    _user_canceled_tip: [
        `{} 未保存，code：{}。`,
        `{} 未儲存，code：{}。`,
        `{} not saved, code: {}.`,
        `{} 保存されていません。code：{}。`,
        `{} 저장되지 않음, 코드: {}.`,
        `{} не сохранено, код: {}.`,
    ],
    _FILE_FAILED_tip: [
        '可能是文件名太长，或是其他原因导致文件保存失败。你可以尝试启用高级设置里的“文件名长度限制”。',
        '可能是檔名太長，或是其他原因導致檔案儲存失敗。你可以嘗試啟用高階設定裡的“檔案名稱長度限制”。',
        'Maybe the file name is too long, or other reasons cause the file to fail to save. You can try enabling "File name length limit" in advanced settings.',
        'ファイル名が長すぎるか、他の理由でファイルの保存に失敗した可能性があります。 詳細設定で「ファイル名の長さ制限」を有効にしてみてください。',
        '파일명이 너무 길거나 다른 이유로 저장에 실패한 것 같습니다. 고급 설정에서 "파일명 길이 제한"을 사용하도록 설정할 수 있습니다.',
        'Возможно, имя файла слишком длинное, или по другим причинам файл не удается сохранить. Вы можете попробовать включить "Ограничение длины имени файла". в расширенных настройках.',
    ],
    _显示摘要信息: [
        '显示摘要信息',
        '顯示摘要資訊',
        'Show summary',
        '要約情報を表示する',
        '요약 정보 표시',
        'Показать сводку',
    ],
    _显示更大的缩略图: [
        '显示<span class="key">更大</span>的缩略图',
        '顯示<span class="key">更大</span>的縮圖',
        'show <span class="key">larger</span> thumbnails',
        '大きなサムネイルを表示する',
        '<span class="key">더 큰</span> 썸네일 표시',
        'Показывать <span class="key">большие</span> миниатюры',
    ],
    _横图占用二倍宽度: [
        '横图占用二倍宽度',
        '橫圖佔用二倍寬度',
        'Horizontal image takes up double the width',
        '水平方向の画像は幅の2倍を占めます',
        '가로 그림은 두 배의 너비를 차지',
        'Горизонтальное изображение занимает вдвое большую ширину',
    ],
    _该功能默认开启: [
        '这个功能默认启用。',
        '這個功能預設啟用。',
        'This feature is enabled by default.',
        'この機能はデフォルトで有効になっています。',
        '이 기능은 기본적으로 활성화됩니다.',
        'Эта функция включена по умолчанию.',
    ],
    _默认未启用: [
        '默认未启用。',
        '預設未啟用。',
        'It is disabled by default.',
        'デフォルトでは有効になっていません。',
        '기본값이 비활성화되어 있습니다.',
        'По умолчанию не работает.',
    ],
    _你可以在更多选项卡的xx分类里找到它: [
        '你可以在“更多”选项卡 → “{}”分类里找到它。（需要先启用“显示高级设置”）',
        '你可以在“更多”選項卡 → “{}”分類裡找到它。（需要先啟用“顯示進階設定”）',
        'You can find it in the "More" tab → "{}" category. ("Show advanced settings" needs to be enabled first)',
        '[もっと]タブ→[{}]カテゴリにあります。（最初に「詳細設定を表示」を有効にする必要があります）',
        '"더보기" 탭 → "{}" 카테고리에서 찾을 수 있습니다. ("고급 설정 보기"를 먼저 활성화해야 합니다.)',
        'Вы можете найти его в разделе "Еще". вкладка → "{}" категория. ("Показать расширенные настройки" необходимо сначала включить)',
    ],
    _你可以在xx选项卡里找到它: [
        '你可以在“{}”选项卡里找到它。（需要先启用“显示高级设置”）',
        '你可以在“{}”選項卡裡找到它。（需要先啟用“顯示進階設定”）',
        'You can find it in the "{}" tab. ("Show advanced settings" needs to be enabled first)',
        '「{}」タブにあります。（最初に「詳細設定を表示」を有効にする必要があります）',
        '"{}" 탭에서 찾을 수 있습니다. ("고급 설정 보기"를 먼저 활성화해야 합니다.)',
        'Вы можете найти его на вкладке "{}". ("Показать расширенные настройки" необходимо сначала включить)',
    ],
    _使用鼠标滚轮切换作品里的图片: [
        '使用鼠标滚轮切换多图作品里的图片',
        '使用滑鼠滾輪切換多圖作品裡的圖片',
        'Use the mouse wheel to switch images in multi-image works',
        'マウスホイールを使用して、マルチイメージ作品のイメージを切り替えます',
        '마우스 휠을 사용하여 여러 이미지 작품에서 이미지 전환',
        'Используйте колесико мыши для переключения изображений в работах с несколькими изображениями',
    ],
    _这可能会阻止页面滚动: [
        '这可能会阻止页面滚动',
        '這可能會阻止頁面滾動',
        'This might stop the page from scrolling',
        'ページのスクロールを妨げる可能性があります',
        '이 기능은 페이지를 스크롤하지 못하게 할 수 있습니다.',
        'Это может остановить прокрутку страницы',
    ],
    _动图转换失败的提示: [
        '动图转换失败，id：{}',
        '動圖轉換失敗，id：{}',
        'Ugoira(animation) conversion failed, id: {}',
        'うごイラの変換に失敗しました、id：{}',
        '움직이는 일러스트 변환에 실패했습니다, ID: {}',
        'Не удалось преобразовать Ugoira(анимацию), идентификатор: {}',
    ],
    _动图不能转换为WEBM视频的提示: [
        '作品 ID {} 不能转换为 WEBM 视频，因为它的某一帧延迟大于 32767 毫秒。下载器会把它转换为 GIF 图像。',
        '作品 ID {} 不能轉換為 WEBM 影片，因為它的某一幀延遲大於 32767 毫秒。下載器會把它轉換為 GIF 影象。',
        'Work ID {} cannot be converted to WEBM video because it has a frame duration greater than 32767 ms. The downloader will convert it into a GIF image.',
        'ワークid {} は、32767ミリ秒以上のフレーム長を持つため、webm動画に変換できません。ダウンローダはそれをgif画像に変換します。',
        '작업 ID {}의 프레임 지속 시간이 32767 ms보다 크기 때문에 WEBM 비디오로 변환할 수 없습니다.다운로더가 GIF 이미지로 변환해 줍니다.',
        'Рабочий ID {} не может быть преобразован в WEBM видео, потому что он имеет длительность кадров более 32767 мс. Загрузчик преобразует его в изображение GIF.',
    ],
    _作品id无法下载带状态码: [
        '{} 无法下载，状态码：{}',
        '{} 無法下載，狀態碼：{}',
        '{} failed to download, status code: {}',
        '{} ダウンロードに失敗しました、ステータスコード：{}',
        '{} 다운로드할 수 없습니다, 상태 코드: {}',
        '{} не удалось загрузить, код состояния: {}',
    ],
    _作品总数为0: [
        '作品总数为 0，Pixiv 可能拒绝了此次抓取。请稍后重试。',
        '作品總數為 0，Pixiv 可能拒絕了此次抓取。請稍後重試。',
        'The total number of works is 0, Pixiv may have refused this crawl. Please try again later.',
        '作品の総数は 0 です。 Pixivがこのクロールを拒否した可能性があります。 後でもう一度やり直してください。',
        '총 작품 수가 0개입니다, Pixiv가 이번 긁어오기를 거부한 것으로 보입니다. 잠시 후에 다시 시도해주세요.',
        'Общее количество работ равно 0, возможно, Pixiv блокирует сканирование. Пожалуйста, повторите попытку позже.',
    ],
    _快捷键AltP: [
        '快捷键 Alt + P',
        '快捷鍵 Alt + P',
        'Hot key: Alt + P',
        'ホットキー Alt + P',
        '단축키: Alt + P',
        'Горячая клавиша: Alt + P',
    ],
    _优化预览作品功能: [
        '优化“预览作品”功能',
        '最佳化“預覽作品”功能',
        'Optimize the "Preview Works" function',
        '「作品のプレビュー」機能を最適化する',
        '"작품 미리보기" 기능 최적화',
        'Оптимизация "Предварительного просмотра работ" функция',
    ],
    _设定资料: [
        '设定资料',
        '設定資料',
        'Reference materials',
        '設定資料',
        '설정 자료',
        'Справочные материалы',
    ],
    _年龄限制: [
        '<span class="key">年龄</span>限制',
        '<span class="key">年齡</span>限制',
        '<span class="key">Age</span> limit',
        '<span class="key">年齢</span>制限',
        '<span class="key">연령</span> 제한',
        '<span class="key">Возраст</span> ограничение',
    ],
    _收藏状态: [
        '<span class="key">收藏</span>状态',
        '<span class="key">收藏</span>狀態',
        '<span class="key">Bookmark</span> status',
        '<span class="key">ブックマーク</span>ステータス',
        '<span class="key">북마크</span> 상태',
        'Статус <span class="key">закладки</span> ',
    ],
    _图片色彩: [
        '图片<span class="key">色彩</span>',
        '圖片<span class="key">色彩</span>',
        'Image <span class="key">color</span>',
        '画像の<span class="key">色</span>',
        '이미지 <span class="key">색채</span>',
        '<span class="key">Цвет</span> изображения',
    ],
    _图片数量: [
        '图片<span class="key">数量</span>',
        '圖片<span class="key">數量</span>',
        '<span class="key">Number</span> of images',
        '画像の<span class="key">数</span>',
        '이미지 <span class="key">수</span>',
        '<span class="key">Количество</span> изображений',
    ],
    _不抓取多图作品的最后一张图片: [
        '不抓取多图作品的<span class="key">最后一张</span>图片',
        '不抓取多圖作品的<span class="key">最後一張</span>圖片',
        'Do not crawl the <span class="key">last image</span> of multi-image works',
        'マルチ画像作品の<span class="key">最後の画像</span>をつかまないでください',
        '여러 이미지의 <span class="key">마지막 이미지</span> 긁어오지 않기',
        'Не сканировать по <span class="key">последние изображения</span> в много картинных работах',
    ],
    _下载小说的封面图片: [
        '下载小说的<span class="key">封面</span>图片',
        '下載小說的<span class="key">封面</span>圖片',
        'Download the <span class="key">cover</span> image of the novel',
        '小説の<span class="key">表紙画像</span>をダウンロード',
        '소설 <span class="key">커버</span> 이미지 다운로드',
        'Скачать изображение <span class="key">обложки</span> новеллы',
    ],
    _预览动图: [
        '<span class="key">预览</span>动图',
        '<span class="key">預覽</span>動圖',
        '<span class="key">Preview</span> Ugoira',
        'うごイラのプレビュー',
        '움직이는 일러스트 <span class="key">미리보기</span>',
        '<span class="key">Превью</span> Ugoira(анимации)',
    ],
    _过度访问警告警告: [
        '下载器检测到你可能收到了 pixiv 的警告消息，这通常是因为过度下载导致的。<br><strong>当你再次被警告时，你会被 Pixiv 封号。</strong><br>我建议你减少下载数量，或者使用新的账号进行下载。',
        '下載器檢測到你可能收到了 pixiv 的警告訊息，這通常是因為過度下載導致的。<br><strong>當你再次被警告時，你會被 Pixiv 封號。</strong><br>我建議你減少下載數量，或者使用新的賬號進行下載。',
        'The downloader has detected that you may have received a warning message from pixiv, usually due to excessive downloads.<br><strong>When you are warned again, you will be banned from Pixiv. </strong><br>I suggest you reduce your downloads, or use a new account to download.',
        'ダウンロードが多すぎるため、pixivから警告メッセージが届いた可能性があることをダウンローダーが検出しました。<br><strong>再度警告を受けた場合、Pixivから追放されます。 </strong><br>ダウンロード数を減らすか、新しいアカウントを使用してダウンロードすることをお勧めします。',
        '다운로더는 일반적으로 과도한 다운로드로 인해 pixiv에서 경고 메시지를 수신했을 수 있음을 감지했습니다.<br><strong>다시 경고를 받으면 Pixiv에서 차단됩니다. </strong><br>다운로드를 줄이거나 새 계정을 사용하여 다운로드하는 것이 좋습니다.',
        'Программа загрузки обнаружила, что вы могли получить предупреждающее сообщение от pixiv, обычно из-за чрезмерной загрузки.<br><strong>Когда вы снова получите предупреждение, вы будете заблокированы в Pixiv. </strong><br>Я предлагаю вам сократить количество загрузок или использовать новую учетную запись для загрузки.',
    ],
    _下载小说里的内嵌图片: [
        '下载小说里的<span class="key">内嵌</span>图片',
        '下載小說裡的<span class="key">內嵌</span>圖片',
        'Download <span class="key">embedded</span> images in novels',
        '小説に埋め込まれた画像をダウンロードする',
        '소설에서 <span class="key">인라인</span> 이미지 다운로드',
        'Загрузка <span class="key">вложенных</span> изображений в новеллах',
    ],
    _其他优化: [
        '其他优化',
        '其他最佳化',
        'Other optimizations',
        'その他の最適化',
        '기타 최적화',
        'Другие оптимизации',
    ],
    _隐藏浏览器底部的下载栏: [
        '隐藏浏览器底部的<span class="key">下载栏</span>',
        '隱藏瀏覽器底部的<span class="key">下載欄</span>',
        `Hide the <span class="key">download bar</span> at the bottom of the browser`,
        'ブラウザの下部にあるダウンロードバーを非表示にします',
        '브라우저 하단의 <span class="key">다운로드 바</span> 숨기기',
        `Скрыть <span class="key">панель загрузки</span> в нижней части браузера`,
    ],
    _没有可用的抓取结果: [
        '没有可用的抓取结果',
        '沒有可用的抓取結果',
        'No crawl results available',
        'クロール結果がありません',
        '사용 가능한 크롤링 결과가 없습니다.',
        'Результаты сканирования недоступны',
    ],
    _预览作品时按快捷键可以下载这个作品: [
        '预览作品时，按快捷键 <span class="blue">D</span> 可以下载这个作品。<br>按快捷键 <span class="blue">C</span> 仅下载当前显示的这张图片。',
        '預覽作品時，按快捷鍵 <span class="blue">D</span> 可以下載這個作品。<br>按快捷鍵 <span class="blue">C</span> 僅下載當前顯示的這張圖片。',
        'When previewing a work, press the shortcut key <span class="blue">D</span> to download the work.<br>Press the shortcut key <span class="blue">C</span> to download only the currently displayed image.',
        '作品をプレビューしているときに、ショートカット キー <span class="blue">D</span> を押すと、作品をダウンロードできます。<br>ショートカット キー <span class="blue">C</span> を押して、現在表示されている画像のみをダウンロードします。',
        '이미지를 미리 보는 동안 바로 가기 <span class="blue">D</span>를 눌러 다운로드하세요.<br>현재 표시된 이미지만 다운로드하려면 단축키 <span class="blue">C</span>를 누르십시오.',
        'При предварительном просмотре произведения нажмите клавишу <span class="blue">D</span>, чтобы загрузить произведение.<br>Нажмите клавишу быстрого доступа <span class="blue">C</span>, чтобы загрузить только отображаемое в данный момент изображение.',
    ],
    _预览作品时按快捷键C仅下载当前图片: [
        '预览作品时，按快捷键 <span class="blue">C</span> 仅下载当前显示的这张图片。',
        '預覽作品時，按快捷鍵 <span class="blue">C</span> 僅下載當前顯示的這張圖片。',
        'When previewing a work, press the shortcut key <span class="blue">C</span> to download only the currently displayed image.',
        '作品のプレビュー中に、ショートカット キー <span class="blue">C</span> を押すと、現在表示されている画像のみをダウンロードできます。',
        '작품 미리보기 시 단축키 <span class="blue">C</span>를 누르면 현재 표시된 이미지만 다운로드 됩니다.',
        'При предварительном просмотре работы нажмите клавишу быстрого доступа <span class="blue">C</span>, чтобы загрузить только отображаемое в данный момент изображение.',
    ],
    _定时抓取: [
        '定时抓取',
        '定時抓取',
        'Timed crawl',
        '時限クロール',
        '시간 제한 크롤링',
        'Сканирование по таймеру',
    ],
    _定时抓取说明: [
        '每隔一定时间，自动开始抓取和下载。',
        '每隔一定時間，自動開始抓取和下載。',
        'Automatically start crawling and downloading at regular intervals.',
        '定期的にクロールとダウンロードを自動的に開始します。',
        '정기적으로 자동으로 크롤링 및 다운로드를 시작합니다.',
        'Автоматически запускать сканирование и загрузку через регулярные промежутки времени',
    ],
    _定时抓取已启动的提示: [
        '定时抓取已启动，间隔时间：{} 分钟。<br>如果你想修改间隔时间，可以在“更多”选项卡里修改设置：定时抓取的间隔时间。',
        '定時抓取已啟動，間隔時間：{} 分鐘。<br>如果你想修改間隔時間，可以在“更多”選項卡里修改設定：定時抓取的間隔時間。',
        'Timed crawl started, interval: {} minutes.<br>If you want to modify the interval time, you can modify the settings in the "More" tab: The interval time of timed crawl.',
        '時限クロールが開始されました。間隔: {} 分。<br>間隔時間を変更したい場合は、[詳細] タブの設定 (時間指定クロールの間隔時間) を変更できます。',
        '시간 제한 크롤링이 시작되었습니다. 간격: {}분. <br>간격 시간을 수정하려면 "자세히" 탭에서 예약된 크롤링 간격 설정을 수정할 수 있습니다.',
        'Таймер сканирования запущен, интервал: {} минут.<br>Если вы хотите изменить интервал времени, вы можете изменить настройки на вкладке «Дополнительно»: Интервальное время сканирования с таймером.',
    ],
    _定时抓取的推荐用法: [
        '推荐用法：增量抓取新作品。例如在关注的用户的新作品页面里，设置抓取页数为 2，然后启动定时抓取。这样下载器可以自动下载新作品。<br>建议启用“不下载重复文件”功能，以避免下载重复的文件。',
        '推薦用法：增量抓取新作品。例如在關注的使用者的新作品頁面裡，設定抓取頁數為 2，然後啟動定時抓取。這樣下載器可以自動下載新作品。<br>建議啟用“不下載重複檔案”功能，以避免下載重複的檔案。',
        'Recommended usage: Fetch new work incrementally. For example, in the new work page of the user you follow, set the number of crawled pages to 2, and then start timing crawling. This way the downloader can automatically download new works.<br>It is recommended to enable the "Do not download duplicate files" feature to avoid downloading duplicate files.',
        '推奨される使用法: 新しい作業を段階的にフェッチします。 たとえば、フォローしているユーザーの新しい作品ページで、クロールされたページの数を 2 に設定し、クロールのタイミングを開始します。 このようにして、ダウンローダーは新しい作品を自動的にダウンロードできます。<br>重複ファイルのダウンロードを避けるために、「重複ファイルをダウンロードしない」機能を有効にすることをお勧めします。',
        '권장 사용법: 새 작업을 점진적으로 가져옵니다. 예를 들어 팔로우하는 사용자의 새 작업 페이지에서 크롤링 페이지 수를 2로 설정한 다음 타이밍 크롤링을 시작합니다. 이렇게 하면 다운로더가 자동으로 새 작품을 다운로드할 수 있습니다.<br>중복 파일 다운로드를 방지하기 위해 "중복 파일 다운로드 금지" 기능을 활성화하는 것이 좋습니다.',
        'Рекомендуемое использование: получать новую работу постепенно. Например, на новой рабочей странице пользователя, за которым вы следите, установите количество просканированных страниц равным 2, а затем запустите сканирование по времени. Таким образом, загрузчик может автоматически загружать новые работы.<br>Рекомендуется включить функцию "Не загружать дубликаты файлов", чтобы избежать загрузки дубликатов файлов.',
    ],
    _定时抓取已启动的提示2: [
        '在定时抓取时，将这个标签页静置即可。不要改变这个标签页的 URL，否则抓取结果可能不符合预期。<br><br>如果这个扩展程序自动更新了，那么这个页面将不能正常下载文件（需要刷新页面来恢复正常）。 如果你想长期执行定时抓取任务，建议安装下载器的离线版本，以免因为自动更新而导致问题。<br>你可以在这里下载离线安装包：<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
        '在定時抓取時，將這個標籤頁靜置即可。不要改變這個標籤頁的 URL，否則抓取結果可能不符合預期。<br><br>如果這個擴充套件程式自動更新了，那麼這個頁面將不能正常下載檔案（需要重新整理頁面來恢復正常）。 如果你想長期執行定時抓取任務，建議安裝下載器的離線版本，以免因為自動更新而導致問題。<br>你可以在這裡下載離線安裝包：<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
        'During timed crawling, just leave this tab alone. Do not change the URL of this tab, or the crawl results may not be as expected.<br><br>If the extension is automatically updated, the page will not be able to download files normally (refresh the page to restore normal). If you want to perform scheduled crawling tasks for a long time, it is recommended to install the offline version of the downloader to avoid problems caused by automatic updates.<br>You can download the offline installation package here: <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
        '時限クロール中は、このタブをそのままにしておきます。 このタブの URL は変更しないでください。変更すると、クロール結果が期待どおりにならない可能性があります。<br><br>拡張機能が自動的に更新されると、ページはファイルを正常にダウンロードできなくなります (ページを更新して正常に戻します)。 スケジュールされたクロール タスクを長時間実行する場合は、自動更新による問題を回避するために、ダウンローダのオフライン バージョンをインストールすることをお勧めします。<br>オフライン インストール パッケージは、次の場所からダウンロードできます。<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
        '시간 제한 크롤링 중에는 이 탭을 그대로 두십시오. 이 탭의 URL을 변경하지 마십시오. 그렇지 않으면 크롤링 결과가 예상과 다를 수 있습니다.<br><br>확장자가 자동으로 업데이트되면 페이지에서 파일을 정상적으로 다운로드할 수 없습니다(페이지를 새로고침하여 정상으로 복원). 예약된 크롤링 작업을 장기간 수행하려면 자동 업데이트로 인한 문제를 방지하기 위해 다운로더의 오프라인 버전을 설치하는 것이 좋습니다.<br>여기에서 오프라인 설치 패키지를 다운로드할 수 있습니다. <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
        'Во время сканирования по времени просто оставьте эту вкладку в покое. Не меняйте URL-адрес этой вкладки, иначе результаты сканирования могут отличаться от ожидаемых.<br><br>Если расширение автоматически обновляется, страница не сможет загружать файлы в обычном режиме (обновите страницу, чтобы восстановить нормальный режим). Если вы хотите выполнять запланированные задачи обхода в течение длительного времени, рекомендуется установить автономную версию загрузчика, чтобы избежать проблем, вызванных автоматическими обновлениями.<br>Вы можете скачать автономный установочный пакет здесь: <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Страница релизов</a>',
    ],
    _定时抓取的间隔时间: [
        '<span class="key">定时</span>抓取的间隔时间',
        '<span class="key">定時</span>抓取的間隔時間',
        'The interval time of <span class="key">timed crawl</span>',
        '<span class="key">時間指定</span>クロールの間隔時間',
        '정기 크롤링 간격 시간',
        'Интервальное время сканирования с таймером',
    ],
    _定时抓取的间隔时间2: [
        '定时抓取的间隔时间',
        '定時抓取的間隔時間',
        'The interval time of timed crawl',
        '時間指定クロールの間隔時間',
        '정기 크롤링 간격 시간',
        'Интервальное время сканирования с таймером',
    ],
    _分钟: ['分钟', '分鐘', 'Minute', '分', '분', 'Минут'],
    _定时抓取的时间超过最大值: [
        '定时抓取的间隔时间超过最大值：',
        '定時抓取的間隔時間超過最大值：',
        'The interval of timed crawl exceeds the maximum value: ',
        '時間指定クロールの間隔が最大値を超えています: ',
        '시간 지정 크롤링 간격이 최대값을 초과합니다: ',
        'Интервал сканирования по таймеру превышает максимальное значение: ',
    ],
    _定时抓取的时间最小值: [
        '定时抓取的间隔时间最小值为 1 分钟。',
        '定時抓取的間隔時間最小值為 1 分鐘。',
        'The minimum interval for timed crawls is 1 minute.',
        '時間指定クロールの最小間隔は 1 分です。',
        '시간 지정 크롤링의 최소 간격은 1분입니다.',
        'Минимальный интервал для сканирования по таймеру составляет 1 минуту.',
    ],
    _取消定时抓取: [
        '取消定时抓取',
        '取消定時抓取',
        'Cancel timed crawl',
        '時間指定クロールをキャンセル',
        '시간 지정 크롤링 취소',
        'Отменить сканирование по таймеру',
    ],
    _已取消定时抓取: [
        '已取消定时抓取',
        '已取消定時抓取',
        'Timed crawl canceled',
        '時間指定クロールがキャンセルされました',
        '예약된 크롤링이 취소되었습니다.',
        'Сканирование по таймеру отменено',
    ],
    _因为URL变化取消定时抓取任务: [
        '因为 URL 变化，定时抓取任务已被取消。',
        '因為 URL 變化，定時抓取任務已被取消。',
        'The timed crawl task has been canceled due to URL changes.',
        'URL が変更されたため、時間指定クロール タスクがキャンセルされました。',
        'URL 변경으로 인해 시간이 지정된 크롤링 작업이 취소되었습니다.',
        'Задание на сканирование по времени было отменено из-за изменений URL.',
    ],
    _开始定时抓取: [
        '开始定时抓取',
        '開始定時抓取',
        'Start timed crawling',
        '時間指定クロールを開始する',
        '시간 지정 크롤링 시작',
        'Начать сканирование по таймеру',
    ],
    _等待下一次定时抓取: [
        '等待下一次定时抓取',
        '等待下一次定時抓取',
        'Wait for the next timed crawl',
        '次回の時限クロールを待つ',
        '다음 시간 크롤링을 기다립니다.',
        'Подождите следующего сканирования с таймером',
    ],
    _当前时间: [
        '当前时间：',
        '當前時間：',
        'Current time: ',
        '現在の時刻：',
        '현재 시간: ',
        'Текущее время: ',
    ],
    _仅在部分页面中可用: [
        '仅在部分页面中可用。',
        '僅在部分頁面中可用。',
        'Only available on some pages.',
        '一部のページのみ利用可能です。',
        '일부 페이지에서만 사용할 수 있습니다.',
        'Доступно только на некоторых страницах',
    ],
    _发生错误原因: [
        '发生错误，原因：',
        '發生錯誤，原因：',
        'An error occurred due to:',
        '次の理由でエラーが発生しました:',
        '다음으로 인해 오류가 발생했습니다.',
        'Произошла ошибка по причине:',
    ],
    _扩展程序已更新: [
        '扩展程序已更新。',
        '擴充套件程式已更新。',
        'The extension has been updated.',
        '拡張機能が更新されました。',
        '확장이 업데이트되었습니다.',
        'Расширение было обновлено.',
    ],
    _未知错误: [
        '未知错误。',
        '未知錯誤。',
        'unknown mistake.',
        '未知の間違い。',
        '알 수 없는 실수.',
        'неизвестная ошибка',
    ],
    _请刷新页面: [
        '请刷新页面。',
        '請重新整理頁面。',
        'Please refresh the page.',
        'ページを更新してください。',
        '페이지를 새로고침하세요.',
        'Пожалуйста, обновите страницу.',
    ],
    _减慢抓取速度: [
        '<span class="key">减慢</span>抓取速度',
        '<span class="key">減慢</span>抓取速度',
        '<span class="key">Slow down</span> crawl',
        'クロールを<span class="key">遅くする</span>',
        '<span class="key">천천히</span> 크롤링',
        '<span class="key">Замедлить</span> сканирование',
    ],
    _减慢抓取速度的说明: [
        '减慢抓取速度可以避免在抓取时被 Pixiv 临时限制。但是会增加抓取时间。',
        '減慢抓取速度可以避免在抓取時被 Pixiv 臨時限制。但是會增加抓取時間。',
        'Slow down the crawl to avoid being temporarily restricted by Pixiv while crawling. But this will increase the crawl time.',
        'クロール中にPixivによって一時的に制限されないように、クロールを遅くします。 ただし、これによりクロール時間が長くなります。',
        '크롤링하는 동안 Pixiv에 의해 일시적으로 제한되지 않도록 크롤링 속도를 늦춥니다. 그러나 이것은 크롤링 시간을 증가시킵니다.',
        'Замедлить сканирование, чтобы избежать временного ограничения Pixiv во время сканирования. Это увеличит время сканирования.',
    ],
    _作品数量: [
        '作品数量',
        '作品數量',
        'Number of works',
        '作品数',
        '작품 수',
        'Количество работ',
    ],
    _当作品数量大于: [
        '当作品数量 >',
        '當作品數量 >',
        'When the number of works >',
        '作品数 >',
        '작품 수 >',
        'При количестве работ >',
    ],
    _慢速抓取: [
        '慢速抓取，以避免触发 429 限制',
        '慢速抓取，以避免觸發 429 限制',
        'Crawl slowly to avoid triggering 429 throttling',
        '429 スロットリングのトリガーを避けるためにゆっくりとクロールします',
        '429 스로틀링 트리거를 방지하기 위해 천천히 크롤링',
        'Медленно сканируйте, чтобы не спровоцировать 429 троттлинг.',
    ],
    _慢速执行以避免引起429错误: [
        '慢速执行，以避免引起 429 错误',
        '慢速執行，以避免引起 429 錯誤',
        'Execute slowly to avoid causing 429 errors',
        '429 エラーの発生を避けるためにゆっくり実行してください',
        '429 오류가 발생하지 않도록 천천히 실행하십시오.',
        'Выполняйте медленно, чтобы избежать ошибок 429',
    ],
    _点击收藏按钮时下载作品: [
        '点击<span class="key">收藏</span>按钮时下载作品',
        '點選<span class="key">收藏</span>按鈕時下載作品',
        'Download a work when you click the <span class="key">bookmark</span> button',
        '<span class="key">ブックマーク</span>ボタンをクリックすると作品をダウンロード',
        '<span class="key">북마크</span> 버튼 클릭 시 작품 다운로드',
        'Загрузка произведения при нажатии кнопки <span class="key">закладка</span>',
    ],
    _点击点赞按钮时下载作品: [
        '点击<span class="key">点赞</span>按钮时下载作品',
        '點選<span class="key">點贊</span>按鈕時下載作品',
        'Download a work when you click the <span class="key">like</span> button',
        '<span class="key">いいね</span> ボタンをクリックすると作品がダウンロードされます',
        '<span class="key">좋아요</span> 버튼 클릭 시 작품 다운로드',
        'Загрузка произведения при нажатии на кнопку <span class="key">лайк</span>',
    ],
    _优化性能和用户体验: [
        '优化性能和用户体验。',
        '最佳化效能和使用者體驗。',
        'Optimize performance and user experience.',
        'パフォーマンスとユーザー エクスペリエンスを最適化します。',
        '성능과 사용자 경험을 최적화합니다.',
        'Оптимизация производительности и пользовательского опыта',
    ],
    _修复bug: [
        '修复 bug',
        '修復 bug',
        'fix bugs',
        'バグを修正',
        '버그 수정',
        'Баг фикс',
    ],
    _修复已知问题: [
        '修复已知问题',
        '修復已知問題',
        'fix known issues',
        '既知の問題を修正する',
        '알려진 문제 수정',
        'исправить известные проблемы',
    ],
    _不支持的浏览器: [
        '你的浏览器不能正常使用这个扩展程序，主要原因可能是浏览器内核版本太低，或者存在兼容性问题。<br>建议您更换成最新版本的 Chrome 或 Edge 浏览器。',
        '你的瀏覽器不能正常使用這個擴充套件程式，主要原因可能是瀏覽器核心版本太低，或者存在相容性問題。<br>建議您更換成最新版本的 Chrome 或 Edge 瀏覽器。',
        'Your browser cannot use this extension properly. The main reason may be that the browser kernel version is too low, or there is a compatibility problem. <br>We recommend that you switch to the latest version of Chrome or Edge.',
        'お使いのブラウザでは、この拡張機能を正しく使用できません。 主な理由としては、ブラウザのカーネル バージョンが低すぎるか、互換性の問題がある可能性があります。 <br>最新バージョンの Chrome または Edge に切り替えることをお勧めします。',
        '브라우저에서 이 확장 프로그램을 제대로 사용할 수 없습니다. 주된 이유는 브라우저 커널 버전이 너무 낮거나 호환성 문제가 있기 때문일 수 있습니다. <br>최신 버전의 Chrome 또는 Edge로 전환하는 것이 좋습니다.',
        'Ваш браузер не может правильно использовать это расширение. Основной причиной может быть слишком низкая версия ядра браузера или проблема совместимости. <br>Мы рекомендуем вам перейти на последнюю версию Chrome или Edge.',
    ],
    _日期时间格式错误: [
        '日期时间格式错误',
        '日期時間格式錯誤',
        'wrong datetime format',
        '間違った日時形式',
        '잘못된 날짜/시간 형식',
        'неправильный формат даты',
    ],
    _添加了对此页面类型的支持: [
        '添加了对此页面类型的支持：',
        '添加了对此页面类型的支持：',
        'Added support for this page type:',
        '次のページ タイプのサポートが追加されました：',
        '이 페이지 유형에 대한 지원이 추가되었습니다：',
        'Добавлена поддержка этого типа страницы:',
    ],
    _仅可由链接浏览: [
        '仅可由链接浏览',
        '僅可由連結瀏覽',
        'URL restricted',
        'URL限定公開',
        'URL 한정 공개',
        'URL ограничен',
    ],
    _添加了俄语翻译: [
        '添加了俄语翻译',
        '添加了俄語翻譯',
        'Added Russian translation',
        'ロシア語の翻訳を追加',
        '러시아어 번역 추가',
        'Добавлен русский перевод',
    ],
    _移除本页面中所有作品的标签: [
        '移除本页面中所有作品的标签',
        '移除本頁面中所有作品的標籤',
        'Remove tags from all works on this page',
        'このページのすべての作品からタグを削除します',
        '이 페이지의 모든 작품에서 태그 제거',
        'Удалить теги со всех работ на этой странице',
    ],
    _它们会变成未分类状态: [
        '它们会变成未分类状态',
        '它們會變成未分類狀態',
        'They become uncategorized',
        'それらは未分類になります',
        '분류되지 않습니다',
        'Они становятся некатегоризированными',
    ],
    _取消收藏本页面的所有作品: [
        '取消收藏本页面中的所有作品',
        '取消收藏本頁面中的所有作品',
        'Unbookmark all works on this page',
        'このページのすべての作品のブックマークを解除',
        '이 페이지의 모든 작품에 대한 북마크 해제',
        'Удалить из избранного все работы на этой странице',
    ],
    _取消收藏所有已被删除的作品: [
        '取消收藏所有已被删除的作品',
        '取消收藏所有已被刪除的作品',
        'Unbookmark all deleted works',
        '削除した作品をすべてブックマーク解除する',
        '삭제된 모든 작품 북마크 해제',
        'Снять закладку со всех удаленных работ',
    ],
    _取消收藏所有已被删除的作品的使用说明: [
        '在你的收藏页面里，切换到下载器的“更多”标签页可以看到该功能按钮。',
        '在你的收藏頁面裡，切換到下載器的“更多”標籤頁可以看到該功能按鈕。',
        'In your bookmark page, switch to the "More" tab of the downloader to see this function button.',
        'ブックマーク ページで、ダウンローダーの「その他」タブに切り替えると、この機能ボタンが表示されます。',
        '북마크 페이지에서 다운로더의 "더보기" 탭으로 전환하면 이 기능 버튼을 볼 수 있습니다.',
        'На странице закладок перейдите на вкладку «Дополнительно» загрузчика, чтобы увидеть эту функциональную кнопку.',
    ],
    _取消收藏作品: [
        '取消收藏作品',
        '取消收藏作品',
        'Unbookmark works',
        '作品のブックマークを解除',
        '작품 북마크 해제',
        'Снять закладку с работ',
    ],
    _收藏页面里的按钮: [
        '当你在自己的收藏页面时，可以在“更多”选项卡里看到这个按钮。',
        '當你在自己的收藏頁面時，可以在“更多”選項卡里看到這個按鈕。',
        `You can see this button in the "More" tab when you're on your bookmarks page.`,
        'このボタンは、ブックマーク ページの [もっと] タブに表示されます。',
        '북마크 페이지에 있을 때 "더보기" 탭에서 이 버튼을 볼 수 있습니다.',
        'Вы можете увидеть эту кнопку на вкладке «Больше», когда находитесь на странице закладок.',
    ],
    _收藏任务尚未完成请等待: [
        '收藏作品的任务尚未全部完成，请等待',
        '收藏作品的任務尚未全部完成，請等待',
        'The task of bookmarking works has not been completed yet, please wait.',
        '作品のブックマーク作業がまだ完了していませんので、しばらくお待ちください',
        '작품을 북마크하는 작업이 아직 완료되지 않았습니다. 잠시만 기다려 주세요',
        'Задание работ по закладке еще не выполнено, пожалуйста, подождите',
    ],
    _收藏作品完毕: [
        '收藏作品完毕',
        '收藏作品完畢',
        'Bookmark works finished',
        'ブックマーク作業終了',
        '북마크 작업 완료',
        'Работа над закладками завершена',
    ],
    _添加收藏失败: [
        '添加收藏失败',
        '新增收藏失敗',
        'Failed to add bookmark',
        'ブックマークを追加できませんでした',
        '북마크 추가 실패',
        'Не удалось добавить закладку',
    ],
    _下载器会在几分钟后重试: [
        '下载器会在几分钟后重试。',
        '下載器會在幾分鐘後重試。',
        'The downloader will try again in a few minutes.',
        'ダウンローダーは数分後に再試行します。',
        '다운로더는 몇 분 후에 다시 시도합니다.',
        'Загрузчик повторит попытку через несколько минут.',
    ],
    _重试收藏: [
        '重试收藏',
        '重試收藏',
        'Retry bookmark',
        'ブックマークを再試行',
        '북마크 다시 시도',
        'Повторить закладку',
    ],
    _剩余xx个: [
        '剩余 {} 个。',
        '剩餘 {} 個。',
        '{} remaining.',
        '{} 残り。',
        '{} 남음.',
        '{} осталось.',
    ],
    _重试收藏成功: [
        '重试收藏成功。',
        '重試收藏成功。',
        'Retry bookmark successfully.',
        'ブックマークを再試行します。',
        '북마크를 다시 시도하십시오.',
        'Повторите попытку закладки.',
    ],
    _出现错误请稍后重试: [
        '出现错误，请稍后重试。',
        '出現錯誤，請稍後重試。',
        'An error occurred, please try again later.',
        'エラーが発生しました。しばらくしてからもう一度お試しください。',
        '오류가 발생했습니다. 잠시 후 다시 시도 해주세요.',
        'Произошла ошибка. Пожалуйста, повторите попытку позже.',
    ],
    _请稍后重试: [
        '请稍后重试。',
        '請稍後重試。',
        'Please try again later.',
        '後でもう一度やり直してください。',
        '잠시 후에 다시 시도해주세요.',
        'Пожалуйста, повторите попытку позже.',
    ],
    _确定要离开吗: [
        '确定要离开吗？',
        '確定要離開嗎？',
        'Are you sure you want to leave?',
        '退会してもよろしいですか？',
        '떠나시겠습니까?',
        'Вы уверены, что хотите оставить?',
    ],
    _yandex浏览器的警告: [
        `如果你在 Yandex 浏览器（Android）上使用 Powerful Pixiv Downloader，请换成 Kiwi 浏览器。<br>
    因为下载器在最近将会升级到 Manifest version 3，但是 Yandex 浏览器不支持  Manifest version 3， 所以它不能使用新版本的下载器。`,
        `如果你在 Yandex 瀏覽器（Android）上使用 Powerful Pixiv Downloader，請換成 Kiwi 瀏覽器。<br>
    因為下載器在最近將會升級到 Manifest version 3，但是 Yandex 瀏覽器不支援  Manifest version 3， 所以它不能使用新版本的下載器。`,
        `If you are using Powerful Pixiv Downloader on Yandex browser（Android）, please switch to Kiwi browser. <br>
    Because the downloader will be upgraded to Manifest version 3 in the near future, but Yandex browser does not support Manifest version 3, so it cannot use the new version of the downloader.`,
        `Yandex（Android） ブラウザで強力な Pixiv Downloader を使用している場合は、Kiwi ブラウザに切り替えてください。 <br>
    ダウンローダは近いうちにマニフェスト バージョン 3 にアップグレードされますが、Yandex ブラウザはマニフェスト バージョン 3 をサポートしていないため、新しいバージョンのダウンローダを使用することはできません。`,
        `Yandex（Android） 브라우저에서 강력한 Pixiv Downloader를 사용하는 경우 Kiwi 브라우저로 전환하십시오. <br>
    다운로더는 가까운 시일 내에 Manifest 버전 3으로 업그레이드되지만 Yandex 브라우저는 Manifest 버전 3을 지원하지 않으므로 새 버전의 다운로더를 사용할 수 없습니다.`,
        `Если вы используете Powerful Pixiv Downloader в браузере Yandex（Android）, перейдите на браузер Kiwi. <br>
    Потому что в ближайшее время загрузчик будет обновлен до Манифеста версии 3, но Yandex браузер не поддерживает Манифест версии 3, поэтому он не может использовать новую версию загрузчика.`,
    ],
    _导出日志: [
        '导出<span class="key">日志</span>',
        '匯出<span class="key">日誌</span>',
        'Export <span class="key">log</span>',
        '<span class="key">ログ</span>のエクスポート',
        '내보내기 로그',
        'Экспорт <span class="key">журнала</span>',
    ],
    _导出日志成功: [
        '✓ 导出日志',
        '✓ 匯出日誌',
        '✓ Export log',
        '✓ ログのエクスポート',
        '✓ 내보내기 로그',
        '✓ Экспорт журнала',
    ],
    _导出时机: [
        '导出时机',
        '匯出時機',
        'Export timing',
        'エクスポートのタイミング',
        '내보내기 타이밍',
        'Время экспорта',
    ],
    _日志类型: [
        '日志类型',
        '日誌型別',
        'Log type',
        'ログの種類',
        '로그 유형',
        'Тип журнала',
    ],
    _正常: ['正常', '正常', 'Normal', '普通', '정상', 'Обычный'],
    _错误: ['错误', '錯誤', 'Error', 'エラー', '오류', 'Ошибка'],
    _排除关键字: [
        '排除关键字',
        '排除關鍵字',
        'Exclude keywords',
        'キーワードを除外',
        '키워드 제외',
        'Исключить ключевые слова',
    ],
    _Chrome108版本转换WebM失败的问题: [
        '从 Chrome 108 版本开始，浏览器的一些变化导致下载器转换 WebM 视频失败。<br>现已修复转换功能。',
        '從 Chrome 108 版本開始，瀏覽器的一些變化導致下載器轉換 WebM 影片失敗。<br>現已修復轉換功能。',
        'Starting with Chrome version 108, some changes in the browser caused the downloader to fail to convert WebM videos. <br>The conversion function is now fixed.',
        'Chrome バージョン 108 以降、ブラウザーの一部の変更により、ダウンローダーが WebM ビデオの変換に失敗しました。 <br>変換機能を修正しました。',
        'Chrome 버전 108부터 브라우저의 일부 변경으로 인해 다운로더가 WebM 비디오를 변환하지 못했습니다. <br>변환 기능이 수정되었습니다.',
        'Начиная с Chrome версии 108, некоторые изменения в браузере приводили к тому, что загрузчик не мог конвертировать видео WebM. <br>Функция преобразования теперь исправлена.',
    ],
    _特定用户的多图作品不下载最后几张图片: [
        '特定用户的多图作品不下载<span class="key">最后几张</span>图片',
        '特定使用者的多圖作品不下載<span class="key">最後幾張</span>圖片',
        `Don't download the <span class="key">last few</span> images for specific user's multi-image works`,
        '特定のユーザーのマルチイメージ作品の最後のいくつかのイメージをダウンロードしないでください',
        '특정 사용자의 다중 이미지 작품에 대한 마지막 몇 개의 이미지를 다운로드하지 마십시오.',
        'Не загружайте несколько последних изображений для работы с несколькими изображениями конкретного пользователя.',
    ],
    _不下载最后几张图片: [
        '不下载最后几张图片',
        '不下載最後幾張圖片',
        'Do not download the last few images',
        '最後の数枚の画像をダウンロードしない',
        '마지막 몇 개의 이미지를 다운로드하지 마십시오',
        'Не загружайте последние несколько изображений',
    ],
    _提示0表示不生效: [
        '0 表示不生效',
        '0 表示不生效',
        '0 means no effect',
        '0 は影響なしを意味します',
        '0은 영향이 없음을 의미합니다.',
        '0 означает отсутствие эффекта',
    ],
    _如果作品含有某些标签则对这个作品使用另一种命名规则: [
        '如果作品含有某些<span class="key">特定标签</span>，则对这个作品使用另一种命名规则',
        '如果作品含有某些<span class="key">特定標籤</span>，則對這個作品使用另一種命名規則',
        'Use a different naming rule for the work if it has certain tags',
        '特定のタグがある場合は、作品に別の命名規則を使用する',
        '특정 태그가 있는 경우 작업에 다른 명명 규칙을 사용하십시오.',
        'Используйте другое правило именования для работы, если она имеет определенные теги',
    ],
    _升级到manifest_v3的提示: [
        '下载器已升级到 Manifest V3。<br>如果你在下载时遇到问题，请打开扩展管理页面，重新加载本扩展。',
        '下載器已升級到 Manifest V3。<br>如果你在下載時遇到問題，請開啟擴充套件管理頁面，重新載入本擴充套件。',
        'Downloader has been upgraded to Manifest V3. <br>If you encounter problems when downloading, please open the extension management page and reload this extension.',
        'Downloader が Manifest V3 にアップグレードされました。 <br>ダウンロード中に問題が発生した場合は、拡張機能の管理ページを開いて、この拡張機能をリロードしてください。',
        '다운로더가 Manifest V3로 업그레이드되었습니다. <br>다운로드 시 문제가 발생하면 확장 프로그램 관리 페이지를 열고 이 확장 프로그램을 새로고침하세요.',
        'Загрузчик обновлен до версии Manifest V3. <br>Если у вас возникли проблемы при загрузке, откройте страницу управления расширением и перезагрузите это расширение.',
    ],
    _AI作品: [
        '<span class="key">AI</span> 作品',
        '<span class="key">AI</span> 作品',
        '<span class="key">AI</span> works',
        '<span class="key">AI</span>が働く',
        '<span class="key">AI</span> 작동',
        '<span class="key">ИИ</span> работает',
    ],
    _AI生成: [
        'AI 生成',
        'AI 生成',
        'AI-generated',
        'AI 生成',
        'AI 생성',
        'сгенерированный ИИ',
    ],
    _非AI生成: [
        '非 AI 生成',
        '非 AI 生成',
        'Not AI-generated',
        'AI生成ではない',
        'AI 생성 아님',
        'Не сгенерировано ИИ',
    ],
    _未知: [
        '未知',
        '未知',
        'Unknown',
        '知らない',
        '알려지지 않은',
        'Неизвестный',
    ],
    _AI未知作品的说明: [
        '早期作品没有标记，无法判断',
        '早期作品沒有標記，無法判斷',
        'Early works are not marked and cannot be judged',
        '初期の作品は採点せず、審査不可',
        '초기 작품은 표시되지 않으며 평가할 수 없습니다.',
        'Ранние работы не отмечены и не могут быть оценены',
    ],
    _用户可以选择是否下载AI生成的作品: [
        '用户可以选择是否下载由 AI 生成的作品。',
        '使用者可以選擇是否下載由 AI 生成的作品。',
        'Users can choose whether to download AI-generated works.',
        'ユーザーは、AI によって生成された作品をダウンロードするかどうかを選択できます。',
        '사용자는 AI가 생성한 작품을 다운로드할지 여부를 선택할 수 있습니다.',
        'Пользователи могут выбирать, загружать ли работы, созданные ИИ.',
    ],
    _文件下载顺序: [
        '文件下载<span class="key">顺序</span>',
        '檔案下載<span class="key">順序</span>',
        'File download <span class="key">order</span>',
        'ファイルのダウンロード<span class="key">順序</span>',
        '파일 다운로드 순서',
        'Порядок загрузки файлов',
    ],
    _降序: [
        '降序',
        '降序',
        'Descending',
        '降順',
        '내림차순',
        'в порядке убывания',
    ],
    _升序: [
        '升序',
        '升序',
        'Ascending',
        '昇順',
        '오름차순',
        'возрастающий порядок',
    ],
    _排序依据: [
        '排序依据',
        '排序依據',
        'Sort by',
        'ソート基準',
        '정렬 기준',
        'Сортировать по',
    ],
    _作品ID: [
        '作品 ID',
        '作品 ID',
        'Work ID',
        '作品ID',
        'ID 아이디',
        'РРабочий идентификатор',
    ],
    _收藏时间: [
        '收藏时间',
        '收藏時間',
        'Bookmark time',
        'ブックマーク時間',
        '북마크 시간',
        'время сбора',
    ],
    _收藏数量2: [
        '收藏数量',
        '收藏數量',
        'Number of bookmarks',
        'ブックマークの数',
        '북마크 수',
        'Колличество закладок',
    ],
    _重新显示帮助: [
        '重新显示帮助',
        '重新顯示幫助',
        'Redisplay help',
        'ヘルプを再表示',
        '도움말 다시 표시',
        'Повторно отобразить справку',
    ],
    _自定义标签分隔符号的提示: [
        '现在你可以自定义文件名中使用的标签分隔符号，以替换默认的 <span class="blue">,</span>。',
        '現在你可以自定義檔名中使用的標籤分隔符號，以替換預設的 <span class="blue">,</span>。',
        'You can now customize the tag separator used in filenames to replace the default <span class="blue">,</span>',
        'ファイル名で使用されるタグ区切りをカスタマイズして、デフォルトの <span class="blue">,</span> を置き換えることができるようになりました',
        '이제 파일 이름에 사용되는 태그 구분 기호를 사용자 지정하여 기본 <span class="blue">,</span>',
        'Теперь вы можете настроить разделитель тегов, используемый в именах файлов, чтобы заменить используемый по умолчанию <span class="blue">,</span>',
    ],
    _高亮关注的用户: [
        '<span class="key">高亮</span>关注的用户',
        '<span class="key">高亮</span>關注的使用者',
        '<span class="key">Highlight</span> following users',
        'フォローしているユーザーを強調表示する',
        '다음 사용자 <span class="key">강조표시</span>',
        '<span class="key">Выделить</span> следующих пользователей',
    ],
    _高亮关注的用户的说明: [
        '你关注（Following）的用户的名字会具有黄色背景，或者显示为黄色。<br>这便于你确认自己是否关注了某个用户。',
        '你關注（Following）的使用者的名字會具有黃色背景，或者顯示為黃色。<br>這便於你確認自己是否關注了某個使用者。',
        'The names of users you are following will have a yellow background, or be displayed in yellow. <br>This is convenient for you to confirm whether you follow a certain user.',
        'フォローしているユーザーの名前は背景が黄色、または黄色で表示されます。 <br>特定のユーザーをフォローしているかどうかを確認するのに便利です。',
        '팔로우하는 사용자의 이름은 노란색 배경으로 표시되거나 노란색으로 표시됩니다. <br>특정 사용자를 팔로우하고 있는지 확인할 때 편리합니다.',
        'Имена пользователей, на которых вы подписаны, будут иметь желтый фон или отображаться желтым цветом. <br>Это удобно для вас, чтобы подтвердить, подписаны ли вы на определенного пользователя',
    ],
    _正在加载关注用户列表: [
        '正在加载关注用户列表',
        '正在載入關注使用者列表',
        'Loading list of followed users',
        'フォローしているユーザーのリストを読み込み中',
        '팔로우한 사용자 목록 로드 중',
        'Загрузка списка отслеживаемых пользователей',
    ],
    _Kiwi浏览器可能不能建立文件夹的bug: [
        '如果你使用的是 Kiwi 浏览器，它可能不会建立文件夹。这是 Kiwi 浏览器的 bug。',
        '如果你使用的是 Kiwi 瀏覽器，它可能不會建立資料夾。這是 Kiwi 瀏覽器的 bug。',
        'If you are using the Kiwi browser, it may not create the folder. This is a bug in the Kiwi browser',
        'Kiwi ブラウザを使用している場合、フォルダが作成されない場合があります。 これは Kiwi ブラウザのバグです。',
        '키위 브라우저를 사용하는 경우 폴더가 생성되지 않을 수 있습니다. 이것은 Kiwi 브라우저의 버그입니다.',
        'Если вы используете браузер Kiwi, он может не создать папку. Это баг браузера Киви.',
    ],
    _优化移动设备上的用户体验: [
        '优化移动设备上的用户体验。',
        '最佳化移動裝置上的使用者體驗。',
        'Optimize user experience on mobile devices.',
        'モバイルデバイスでのユーザーエクスペリエンスを最適化します。',
        '모바일 장치에서 사용자 경험을 최적화합니다.',
        'Оптимизируйте взаимодействие с пользователем на мобильных устройствах.',
    ],
    _导出CSV文件的提示: [
        'CSV 文件的可读性更好，但它不能用于恢复（导入）抓取结果。',
        'CSV 檔案的可讀性更好，但它不能用於恢復（匯入）抓取結果。',
        'A CSV file is more readable, but it cannot be used to restore (import) crawl results.',
        'CSV ファイルは読みやすいですが、クロール結果の復元 (インポート) には使用できません。',
        'CSV 파일은 더 읽기 쉽지만 크롤링 결과를 복원(가져오기)하는 데 사용할 수 없습니다.',
        'CSV-файл более удобен для чтения, но его нельзя использовать для восстановления (импорта) результатов сканирования.',
    ],
    _批量收藏作品时减慢速度: [
        '批量收藏作品时减慢速度，以减少 429 错误发生的概率',
        '批量收藏作品时减慢速度，以减少 429 错误发生的概率',
        'Slow down when batch bookmarking works to reduce chance of 429 errors',
        'バッチブックマークが機能すると、429 エラーの可能性を減らすために速度が低下します。',
        '429 오류 가능성을 줄이기 위해 일괄 북마크가 작동할 때 속도를 늦춥니다.',
        'Замедлите работу, когда пакетная закладка работает, чтобы уменьшить вероятность ошибки 429',
    ],
    _停止抓取: [
        '停止抓取',
        '停止擷取',
        'Stop crawling',
        'クロールをやめる',
        '크롤링 중지',
        'Остановить сканирование',
    ],
    _已停止抓取: [
        '已停止抓取',
        '已停止擷取',
        'Crawl stopped',
        'クロールを停止しました',
        '크롤링 중지됨',
        'Сканирование остановлено',
    ],
    _导入ID列表: [
        '导入 ID 列表',
        '匯入 ID 列表',
        'Import ID list',
        'インポートIDリスト',
        'ID 목록 가져오기',
        'Список идентификаторов импорта',
    ],
    _导出ID列表: [
        '获取 ID 列表完毕后导出列表，并停止抓取',
        '獲取 ID 列表完畢後匯出列表，並停止抓取',
        'Export the ID list after fetching it, and stop crawling',
        'IDリストを取得後にエクスポートし、クロールを停止します',
        'ID 목록을 가져온 후 내보내기 및 크롤링 중지',
        'Экспортируйте список идентификаторов после его извлечения и остановите сканирование.',
    ],
    _导入的用户ID数量: [
        '导入的用户 ID 数量：',
        '匯入的使用者 ID 數量：',
        'Number of user IDs imported: ',
        'インポートされたユーザー ID の数: ',
        '가져온 사용자 ID 수: ',
        'Количество импортированных идентификаторов пользователей:',
    ],
    _任务已中止: [
        '任务已中止',
        '任務已中止',
        'task aborted',
        'タスクが中止されました',
        '작업이 중단됨',
        'задача прервана',
    ],
    _新增的关注用户达到每日限制: [
        '新增的关注用户数量达到 {}， 下载器已中止任务，以免你的账号被 Pixiv 限制。<br>建议明天再执行此任务。',
        '新增的關注使用者數量達到 {}， 下載器已中止任務，以免你的賬號被 Pixiv 限制。<br>建議明天再執行此任務。',
        'The number of newly added followers has reached {}, the downloader has stopped the task to prevent your account from being restricted by Pixiv. <br>It is recommended to perform this task again tomorrow.',
        '新しく追加されたフォロワーの数が {} に達しました。あなたのアカウントが Pixiv によって制限されるのを防ぐために、ダウンローダーはタスクを停止しました。 <br>このタスクは明日もう一度実行することをお勧めします。',
        '새로 추가된 팔로워 수가 {}에 도달했습니다. 다운로더가 작업을 중지하여 Pixiv에서 귀하의 계정을 제한하지 않도록 했습니다. <br>내일 이 작업을 다시 수행하는 것이 좋습니다.',
        'Количество новых подписчиков достигло {}, загрузчик остановил задачу, чтобы предотвратить ограничение вашей учетной записи Pixiv. <br>Рекомендуется повторить это задание завтра.',
    ],
    _没有找到关注按钮的提示: [
        '跳过关注用户 {} 因为没有找到关注按钮。你可以手动关注此用户。再次执行此任务有可能解决此问题。',
        '跳過關注使用者 {} 因為沒有找到關注按鈕。你可以手動關注此使用者。再次執行此任務有可能解決此問題。',
        'Skip following user {} because no follow button was found. You can follow this user manually. Performing this task again may resolve the issue.',
        'フォロー ボタンが見つからなかったため、ユーザー {} のフォローをスキップします。このユーザーを手動でフォローできます。 このタスクを再度実行すると、問題が解決される可能性があります。',
        '팔로우 버튼을 찾을 수 없으므로 사용자 {} 팔로우를 건너뜁니다. 이 사용자를 수동으로 팔로우할 수 있습니다. 이 작업을 다시 수행하면 문제가 해결될 수 있습니다.',
        'Пропустить подписку на пользователя {}, поскольку кнопка подписки не найдена. Вы можете подписаться на этого пользователя вручную. Повторное выполнение этой задачи может решить проблему.',
    ],
    _你的账号已经被Pixiv限制: [
        '你的账号已经被 Pixiv 限制',
        '你的賬號已經被 Pixiv 限制',
        'Your account has been restricted by Pixiv',
        'あなたのアカウントはPixivによって制限されています',
        '귀하의 계정은 Pixiv에 의해 제한되었습니다.',
        'Ваша учетная запись была ограничена Pixiv',
    ],
    _模拟用户点击: [
        '下载器发送的 API 返回 400 错误（需要 recaptcha enterprise score token），切换到模拟用户点击的方式，这会使用较多的硬件资源。',
        '下載器傳送的 API 返回 400 錯誤（需要 recaptcha enterprise score token），切換到模擬使用者點選的方式，這會使用較多的硬體資源。',
        'The API sent by the downloader returns a 400 error (recaptcha enterprise score token is required), and switches to the method of simulating user clicks, which will use more hardware resources.',
        'ダウンローダーによって送信された API は 400 エラー (recaptcha enterprise score token が必要です) を返し、より多くのハードウェア リソースを使用するユーザーのクリックをシミュレートする方法に切り替わります。',
        '다운로더가 보낸 API는 400 오류(recaptcha enterprise score token 필요)를 반환하고 더 많은 하드웨어 리소스를 사용하는 사용자 클릭 시뮬레이션 방법으로 전환합니다.',
        'API, отправленный загрузчиком, возвращает ошибку 400 (требуется recaptcha enterprise score token) и переключается на метод имитации пользовательских кликов, который будет использовать больше аппаратных ресурсов.',
    ],
    _提示可以重新执行批量关注任务: [
        '如果该标签页失去响应，或者关注的用户有遗漏，请关闭标签页，再重新打开，重新执行此任务。',
        '如果該標籤頁失去響應，或者關注的使用者有遺漏，請關閉標籤頁，再重新開啟，重新執行此任務。',
        'If the tab becomes unresponsive, or if you miss a follower, close the tab, reopen it, and redo the task.',
        'タブが応答しなくなった場合、またはフォロワーを見逃した場合は、タブを閉じて再度開き、タスクをやり直してください。',
        '탭이 응답하지 않거나 팔로어를 놓친 경우 탭을 닫았다가 다시 열고 작업을 다시 실행하십시오.',
        'Если вкладка перестает отвечать на запросы или вы пропустили подписчика, закройте вкладку, снова откройте ее и повторите задачу.',
    ],
    _新增x个: [
        '新增 {} 个',
        '新增 {} 個',
        'Added {}',
        '追加した {}',
        '추가됨 {}',
        'Добавлен {}',
    ],
    _优化批量关注用户的功能: [
        '优化批量关注用户的功能',
        '最佳化批次關注使用者的功能',
        'Optimize the function of following users in batches',
        'ユーザーの一括フォロー機能を最適化',
        '사용자 일괄 팔로우 기능 최적화',
        'Оптимизируйте функцию подписки на пользователей в пакетном режиме.',
    ],
    _修复了显示更大的缩略图失效的问题: [
        '修复了“显示更大的缩略图”失效的问题',
        '修復了“顯示更大的縮圖”失效的問題',
        'Fixed an issue where "Show Larger Thumbnails" didn\'t work',
        '「大きなサムネイルを表示」が機能しない問題を修正しました',
        '"큰 축소판 보기"가 작동하지 않는 문제를 수정했습니다.',
        'Исправлена ​​ошибка, из-за которой не работал параметр «Показать увеличенные эскизы».',
    ],
    _可能发生了错误请刷新页面重试: [
        '可能发生了错误。<br>如果下载进度卡住，请刷新页面重试，或者重启浏览器。',
        '可能發生了錯誤。<br>如果下載進度卡住，請重新整理頁面重試，或者重啟瀏覽器。',
        'An error may have occurred. <br>If the download progress is stuck, please refresh the page and try again, or restart the browser.',
        'エラーが発生した可能性があります。 <br>ダウンロードの進行が進まない場合は、ページを更新して再試行するか、ブラウザを再起動してください。',
        '오류가 발생했을 수 있습니다. <br>다운로드 진행이 중단되면 페이지를 새로 고친 후 다시 시도하거나 브라우저를 다시 시작하세요.',
        'Возможно, произошла ошибка. <br>Если процесс загрузки завис, обновите страницу и повторите попытку или перезапустите браузер.',
    ],
    _在多图作品页面里显示缩略图列表: [
        '在多图作品页面里显示<span class="key">缩略图</span>列表',
        '在多圖作品頁面裡顯示<span class="key">縮圖</span>列表',
        'In the multi-image work page, display the <span class="key">thumbnail</span> list',
        '複数画像作品ページでサムネイル一覧を表示',
        '멀티 이미지 작품 페이지에서 썸네일 목록을 표시',
        'На рабочей странице с несколькими изображениями отобразите список эскизов',
    ],
    _提交: ['提交', '提交', 'Submit', '提出する', '제출하다', 'Подавать'],
    _已导出被删除的作品的ID列表: [
        '已导出被删除的作品的 ID 列表',
        '已匯出被刪除的作品的 ID 列表',
        'List of IDs of deleted works exported',
        'エクスポートされた削除作品のID一覧',
        '내보낸 삭제된 작품의 ID 목록',
        'Список идентификаторов удаленных работ экспортирован',
    ],
    _在收藏页面里提示有辅助功能可用: [
        '在你的收藏页面里，下载器的“更多”标签页里有一些功能可以帮助管理你的收藏。',
        '在你的收藏頁面裡，下載器的“更多”標籤頁裡有一些功能可以幫助管理你的收藏。',
        `On your bookmarks page, the Downloader's "More" tab has some features to help you manage your bookmarks.`,
        'ブックマーク ページのダウンローダーの [その他] タブには、ブックマークの管理に役立つ機能がいくつかあります。',
        '북마크 페이지에서 다운로더의 "더보기" 탭에는 북마크를 관리하는 데 도움이 되는 몇 가지 기능이 있습니다.',
        'На странице закладок на вкладке «Дополнительно» Downloader есть некоторые функции, которые помогут вам управлять своими закладками.',
    ],
    _预览作品的详细信息: [
        '预览作品的<span class="key">详细</span>信息',
        '預覽作品時的<span class="key">詳細</span>資料',
        'Preview the <span class="key">details</span> of the work',
        'プレビュー作品の詳細です',
        '작품의 상세한 정보를 미리보다',
        'Подробности предварительного показа',
    ],
    _预览作品的详细信息的说明: [
        '鼠标放在作品缩略图上即可查看作品数据',
        '滑鼠放在作品縮圖上即可檢視作品資料',
        'Mouse over the thumbnail of the work to view the work data',
        '作品のサムネイルにマウスをかざすだけで作品データを見ることができます',
        '마우스를 작품 썸네일 위에 놓으면 작품 데이터를 볼 수 있다',
        'Данные о работе можно увидеть с помощью мыши на сокращённом графике',
    ],
    _显示区域宽度: [
        '显示区域宽度',
        '顯示區域寬度',
        'Display area width',
        '表示領域幅です',
        '영역 너비 보이기',
        'Покажи ширину зоны',
    ],
    _写入剪贴板失败: [
        '写入剪贴板失败',
        '寫入剪貼簿失敗',
        'Writing to clipboard failed',
        'クリップボードへの書き込みに失敗しました',
        '클립보드에 쓰지 못했습니다.',
        'Запись в буфер обмена не удалась',
    ],
};



/***/ }),

/***/ "./src/ts/ListenPageSwitch.ts":
/*!************************************!*\
  !*** ./src/ts/ListenPageSwitch.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");

// 监听页面的无刷新切换
class ListenPageSwitch {
    constructor() {
        this.supportListenHistory();
        this.listenPageSwitch();
    }
    // 为监听 url 变化的事件提供支持
    supportListenHistory() {
        const s = document.createElement('script');
        const url = chrome.runtime.getURL('lib/listen_history_change.js');
        s.src = url;
        document.head.appendChild(s);
    }
    // 无刷新切换页面时派发事件
    listenPageSwitch() {
        // 点击浏览器的前进或后退按钮会触发 popstate 事件
        // 点击链接进入一个 url 不同的页面是 pushState 操作
        // 现在还没有遇到 replaceState 操作
        ;
        ['pushState', 'popstate', 'replaceState'].forEach((item) => {
            window.addEventListener(item, () => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('pageSwitch');
            });
        });
    }
}
new ListenPageSwitch();


/***/ }),

/***/ "./src/ts/Loading.ts":
/*!***************************!*\
  !*** ./src/ts/Loading.ts ***!
  \***************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   loading: () => (/* binding */ loading)
/* harmony export */ });
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");

// loading 图标
class Loading {
    constructor() {
        this.id = 'xzLoadingWrap';
        this.html = `
  <div id="xzLoadingWrap">
    <div class="iconWrap">
    <svg class="icon" aria-hidden="true">
      <use xlink:href="#icon-loading"></use>
    </svg>
    </div>
  </div>`;
        this._show = false;
    }
    set show(val) {
        this._show = val;
        this._show ? this.showEl() : this.hiddenEl();
    }
    get show() {
        return this._show;
    }
    create() {
        document.body.insertAdjacentHTML('beforeend', this.html);
        const el = document.body.querySelector('#' + this.id);
        _Theme__WEBPACK_IMPORTED_MODULE_0__.theme.register(el);
        return el;
    }
    getEl() {
        let el = document.body.querySelector('#' + this.id);
        if (el) {
            return el;
        }
        else {
            return this.create();
        }
    }
    showEl() {
        this.getEl().style.display = 'flex';
    }
    hiddenEl() {
        this.getEl().style.display = 'none';
    }
}
const loading = new Loading();



/***/ }),

/***/ "./src/ts/Log.ts":
/*!***********************!*\
  !*** ./src/ts/Log.ts ***!
  \***********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   log: () => (/* binding */ log)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_DateFormat__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./utils/DateFormat */ "./src/ts/utils/DateFormat.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");











// 日志
class Log {
    constructor() {
        this.id = 'logWrap'; // 日志区域元素的 id
        this.wrap = document.createElement('div'); // 日志容器的区域
        this.logArea = document.createElement('div'); // 日志主体区域
        this.refresh = document.createElement('span'); // 刷新时使用的元素
        this.levelColor = [
            'inherit',
            _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.textSuccess,
            _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.textWarning,
            _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.textError,
        ];
        this.max = 300;
        this.count = 0;
        this.record = [];
        this.toBottom = false; // 指示是否需要把日志滚动到底部。当有日志被添加或刷新，则为 true。滚动到底部之后复位到 false，避免一直滚动到底部。
        this.scrollToBottom();
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clearLog, () => {
            this.clear();
        });
        const clearRecordEvents = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clearLog, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop];
        clearRecordEvents.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.record = [];
            });
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete, () => {
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLog && _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogTiming === 'crawlComplete') {
                this.export();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete, () => {
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLog &&
                _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogTiming === 'downloadComplete') {
                this.export();
            }
        });
    }
    // 添加日志
    /*
    str 日志文本
    level 日志等级
    br 换行标签的个数
    keepShow 追加日志的模式，默认为 true，把这一条日志添加后不再修改。false 则是刷新显示这条消息。
  
    level 日志等级：
    0 normal
    1 success
    2 warning
    3 error
    */
    add(str, level, br, keepShow) {
        this.checkElement();
        let span = document.createElement('span');
        if (!keepShow) {
            span = this.refresh;
        }
        else {
            this.count++;
        }
        span.innerHTML = str;
        span.style.color = this.levelColor[level];
        while (br > 0) {
            span.appendChild(document.createElement('br'));
            br--;
        }
        this.logArea.appendChild(span);
        this.toBottom = true; // 需要把日志滚动到底部
        // 把持久日志保存到记录里
        if (keepShow) {
            this.record.push({ html: span.outerHTML, level });
        }
    }
    log(str, br = 1, keepShow = true) {
        this.add(str, 0, br, keepShow);
    }
    success(str, br = 1, keepShow = true) {
        this.add(str, 1, br, keepShow);
    }
    warning(str, br = 1, keepShow = true) {
        this.add(str, 2, br, keepShow);
    }
    error(str, br = 1, keepShow = true) {
        this.add(str, 3, br, keepShow);
    }
    checkElement() {
        // 如果日志区域没有被添加到页面上，则添加
        let test = document.getElementById(this.id);
        if (test === null) {
            this.wrap = document.createElement('div');
            this.wrap.id = this.id;
            this.logArea = document.createElement('div');
            this.logArea.classList.add('beautify_scrollbar', 'logContent');
            if (_Config__WEBPACK_IMPORTED_MODULE_10__.Config.mobile) {
                this.wrap.classList.add('mobile');
            }
            this.wrap.append(this.logArea);
            document.body.insertAdjacentElement('beforebegin', this.wrap);
            _Theme__WEBPACK_IMPORTED_MODULE_1__.theme.register(this.wrap);
            // 虽然可以应用背景图片，但是由于日志区域比较狭长，背景图片的视觉效果不佳，看起来比较粗糙，所以还是不应用背景图片了
            // bg.useBG(this.wrap, 0.9)
        }
        // 如果页面上的日志条数超过指定数量，则清空
        // 因为日志数量太多的话会占用很大的内存。同时显示 8000 条日志可能占用接近 1 GB 的内存
        if (this.count > this.max) {
            this.clear();
        }
    }
    /**移除日志区域 */
    remove() {
        this.count = 0;
        this.wrap.remove();
    }
    /**清空显示的日志内容 */
    clear() {
        this.count = 0;
        this.logArea.innerHTML = '';
    }
    // 因为日志区域限制了最大高度，可能会出现滚动条，这里使日志总是滚动到底部
    scrollToBottom() {
        window.setInterval(() => {
            if (this.toBottom) {
                this.logArea.scrollTop = this.logArea.scrollHeight;
                this.toBottom = false;
            }
        }, 800);
    }
    export() {
        const data = [];
        for (const record of this.record) {
            let html = '';
            if (record.level !== 3 && _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogNormal) {
                html = record.html;
            }
            if (record.level === 3 && _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogError) {
                html = record.html;
            }
            // 检查排除的关键字
            if (html && _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogExclude.length > 0) {
                let checkStr = html;
                // 如果含有作品链接，则只检查链接后面的部分。这是为了避免因作品 id 中包含要排除的关键字而导致错误的排除
                if (html.includes('<a href')) {
                    const array = html.split('</a>');
                    checkStr = array[array.length - 1];
                }
                const index = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportLogExclude.findIndex((val) => {
                    return checkStr.includes(val);
                });
                if (index === -1) {
                    data.push(html);
                }
            }
        }
        if (data.length === 0) {
            return;
        }
        const fileName = `log-${_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.replaceUnsafeStr(_Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.getPageTitle())}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.replaceUnsafeStr(_utils_DateFormat__WEBPACK_IMPORTED_MODULE_9__.DateFormat.format(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.crawlCompleteTime, _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.dateFormat))}.html`;
        const content = `<!DOCTYPE html>
<html>
<body>
<div id="logWrap">
${data.join('\n')}
</div>
</body>
</html>`;
        const blob = new Blob([content], {
            type: 'text/html',
        });
        const url = URL.createObjectURL(blob);
        _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.downloadFile(url, fileName);
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_导出日志成功');
        log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(msg, {
            position: 'topCenter',
        });
    }
}
const log = new Log();



/***/ }),

/***/ "./src/ts/MsgBox.ts":
/*!**************************!*\
  !*** ./src/ts/MsgBox.ts ***!
  \**************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   msgBox: () => (/* binding */ msgBox)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _BG__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./BG */ "./src/ts/BG.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");






// 简单的消息框
class MsgBox {
    constructor() {
        this.typeColor = {
            success: _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.textSuccess,
            warning: _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.textWarning,
            error: _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.textError,
        };
        this.onceFlags = [];
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.showMsg, (ev) => {
            const msg = ev.detail.data;
            this.create(msg);
        });
    }
    /** 在当前标签页中只会显示一次的消息
     */
    once(flag, msg, type = 'show', arg) {
        if (this.onceFlags.includes(flag)) {
            return;
        }
        this.onceFlags.push(flag);
        switch (type) {
            case 'show':
                this.show(msg, arg);
                break;
            case 'warning':
                this.warning(msg, arg);
                break;
            case 'success':
                this.success(msg, arg);
                break;
            case 'error':
                this.error(msg, arg);
                break;
            default:
                this.show(msg, arg);
                break;
        }
    }
    /**
     * 清除某个 once 标记，使其对应的消息可以再次显示
     */
    resetOnce(flag) {
        const index = this.onceFlags.findIndex((str) => str === flag);
        if (index > -1) {
            this.onceFlags.splice(index);
        }
    }
    show(msg, arg) {
        this.create(Object.assign({}, arg, { msg: msg }));
    }
    success(msg, arg) {
        this.create(Object.assign({ color: this.typeColor.success }, arg, { msg: msg }));
    }
    warning(msg, arg) {
        this.create(Object.assign({ color: this.typeColor.warning }, arg, { msg: msg }));
    }
    error(msg, arg) {
        this.create(Object.assign({ color: this.typeColor.error }, arg, { msg: msg }));
    }
    create(data) {
        const wrap = document.createElement('div');
        wrap.classList.add('xz_msg_box');
        if (_Config__WEBPACK_IMPORTED_MODULE_5__.Config.mobile) {
            wrap.classList.add('mobile');
        }
        let colorStyle = '';
        if (data.color) {
            colorStyle = `style="color:${data.color}"`;
        }
        wrap.innerHTML = `
        <p class="title" ${colorStyle}>${data.title || ''}</p>
        <p class="content" ${colorStyle}>${data.msg}</p>
        <button class="btn" type="button">${data.btn || _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_确定')}</button>
      `;
        _Theme__WEBPACK_IMPORTED_MODULE_2__.theme.register(wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.register(wrap);
        const btn = wrap.querySelector('.btn');
        if (btn) {
            wrap.addEventListener('click', (ev) => {
                ev.stopPropagation();
            });
            btn.addEventListener('click', () => {
                this.remove(wrap);
            });
            window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.closeCenterPanel, () => {
                this.remove(wrap);
            });
        }
        document.body.append(wrap);
        btn.focus();
        _BG__WEBPACK_IMPORTED_MODULE_4__.bg.useBG(wrap);
    }
    remove(el) {
        el && el.parentNode && el.parentNode.removeChild(el);
    }
}
const msgBox = new MsgBox();



/***/ }),

/***/ "./src/ts/NovelThumbnail.ts":
/*!**********************************!*\
  !*** ./src/ts/NovelThumbnail.ts ***!
  \**********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   novelThumbnail: () => (/* binding */ novelThumbnail)
/* harmony export */ });
/* harmony import */ var _WorkThumbnail__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./WorkThumbnail */ "./src/ts/WorkThumbnail.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");




// 查找小说作品的缩略图，当鼠标进入、移出时等动作触发时执行回调函数
class NovelThumbnail extends _WorkThumbnail__WEBPACK_IMPORTED_MODULE_0__.WorkThumbnail {
    constructor() {
        super();
        this.selectors = [];
        if (_Config__WEBPACK_IMPORTED_MODULE_3__.Config.mobile) {
            // 移动端的作品选择器就这一个
            this.selectors = ['.works-item-novel'];
        }
        else {
            this.selectors = [
                'li[size="1"]>div',
                'section li>div',
                'nav>div>div',
                'div.gtm-novel-work-recommend-link',
                'section ul>div',
                'section ul>li',
                'div._ranking-item',
                'div[size="496"]',
                'li',
            ];
        }
        this.findThumbnail(document.body);
        this.createObserver(document.body);
    }
    findThumbnail(parent) {
        if (!parent.querySelectorAll) {
            return;
        }
        // 遍历所有的选择器，为找到的元素绑定事件
        // 注意：有时候一个节点里会含有多种尺寸的缩略图，为了全部查找到它们，必须遍历所有的选择器。
        // 如果在查找到某个选择器之后，不再查找剩余的选择器，就可能会遗漏一部分缩略图。
        // 但是，这有可能会导致事件的重复绑定，所以下载器添加了 dataset.mouseover 标记以减少重复绑定
        for (const selector of this.selectors) {
            // 处理桌面端特殊情况中使用的选择器
            if (!_Config__WEBPACK_IMPORTED_MODULE_3__.Config.mobile) {
                // 在用户主页只使用指定的选择器，避免其他选择器导致顶部“精选”的小说作品被重复绑定事件
                if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.UserHome &&
                    selector !== 'section ul>li' &&
                    selector !== 'li[size="1"]>div') {
                    continue;
                }
                // 在小说排行榜里只使用 div._ranking-item
                if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelRanking &&
                    selector !== 'div._ranking-item') {
                    continue;
                }
                // 在小说系列页面里只使用 section ul>li
                if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelSeries &&
                    selector !== 'section ul>li') {
                    continue;
                }
                // div.gtm-novel-work-recommend-link 只能在小说页面里使用
                if (selector === 'div.gtm-novel-work-recommend-link' &&
                    _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.Novel) {
                    continue;
                }
            }
            let elements = parent.querySelectorAll(selector);
            // 处理特殊的动态添加的元素
            // 有些动态添加的元素不能被选择器选中
            // 小说系列页面里动态添加的就是 li 元素，并且这个 li 元素必须整个使用，不能再细分
            if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.NovelSeries &&
                parent.nodeName === 'LI') {
                elements = [parent];
            }
            for (const el of elements) {
                const id = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.findWorkIdFromElement(el, 'novels');
                // 在移动端页面里，此时获取的可能是 '0'
                // 依然绑定
                if (_Config__WEBPACK_IMPORTED_MODULE_3__.Config.mobile) {
                    this.bindEvents(el, id);
                }
                else {
                    // 在桌面版页面里，只有查找到作品 id 时才会执行回调函数
                    if (id) {
                        this.bindEvents(el, id);
                    }
                }
            }
        }
    }
}
const novelThumbnail = new NovelThumbnail();



/***/ }),

/***/ "./src/ts/OpenCenterPanel.ts":
/*!***********************************!*\
  !*** ./src/ts/OpenCenterPanel.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");


// 页面右侧的按钮，点击可以打开中间面板
class OpenCenterPanel {
    constructor() {
        this.btn = document.createElement('button');
        this.addBtn();
        this.show();
        this.bindEvents();
    }
    addBtn() {
        this.btn = document.createElement('button');
        this.btn.classList.add('rightButton');
        this.btn.id = 'openCenterPanelBtn';
        this.btn.setAttribute('data-xztitle', '_显示控制面板');
        this.btn.innerHTML = `<svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-dakai"></use>
</svg>`;
        document.body.append(this.btn);
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.register(this.btn);
    }
    bindEvents() {
        // 这里阻止事件冒泡是为了配合 CenterPanel 的“点击页面其他部分隐藏 CenterPanel”的效果
        this.btn.addEventListener('click', (e) => {
            const ev = e || window.event;
            ev.stopPropagation();
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('openCenterPanel');
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.centerPanelClosed, () => {
            this.show();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.centerPanelOpened, () => {
            this.hide();
        });
    }
    show() {
        this.btn.style.display = 'flex';
    }
    hide() {
        this.btn.style.display = 'none';
    }
}
new OpenCenterPanel();


/***/ }),

/***/ "./src/ts/PageType.ts":
/*!****************************!*\
  !*** ./src/ts/PageType.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   pageType: () => (/* binding */ pageType)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./utils/SecretSignal */ "./src/ts/utils/SecretSignal.ts");



// 所有页面类型及对应的数字编号
// 可以通过 pageType.list 使用
var PageName;
(function (PageName) {
    PageName[PageName["Unsupported"] = -1] = "Unsupported";
    PageName[PageName["Home"] = 0] = "Home";
    PageName[PageName["Artwork"] = 1] = "Artwork";
    PageName[PageName["UserHome"] = 2] = "UserHome";
    PageName[PageName["BookmarkLegacy"] = 3] = "BookmarkLegacy";
    PageName[PageName["Bookmark"] = 4] = "Bookmark";
    PageName[PageName["ArtworkSearch"] = 5] = "ArtworkSearch";
    PageName[PageName["AreaRanking"] = 6] = "AreaRanking";
    PageName[PageName["ArtworkRanking"] = 7] = "ArtworkRanking";
    PageName[PageName["Pixivision"] = 8] = "Pixivision";
    PageName[PageName["BookmarkDetail"] = 9] = "BookmarkDetail";
    PageName[PageName["NewArtworkBookmark"] = 10] = "NewArtworkBookmark";
    PageName[PageName["Discover"] = 11] = "Discover";
    PageName[PageName["NewArtwork"] = 12] = "NewArtwork";
    PageName[PageName["Novel"] = 13] = "Novel";
    PageName[PageName["NovelSeries"] = 14] = "NovelSeries";
    PageName[PageName["NovelSearch"] = 15] = "NovelSearch";
    PageName[PageName["NovelRanking"] = 16] = "NovelRanking";
    PageName[PageName["NewNovelBookmark"] = 17] = "NewNovelBookmark";
    PageName[PageName["NewNovel"] = 18] = "NewNovel";
    PageName[PageName["ArtworkSeries"] = 19] = "ArtworkSeries";
    PageName[PageName["Following"] = 20] = "Following";
    PageName[PageName["Request"] = 21] = "Request";
    PageName[PageName["Unlisted"] = 22] = "Unlisted";
})(PageName || (PageName = {}));
// 获取页面类型
class PageType {
    constructor() {
        // 当前页面类型
        this.type = PageName.Unsupported;
        // 所有页面类型
        this.list = PageName;
        this.type = this.getType();
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitch, () => {
            this.checkTypeChange();
        });
        _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_2__.secretSignal.register('ppdtask3', () => {
            this.openAllTestPage();
        });
    }
    getType() {
        const url = window.location.href;
        const pathname = window.location.pathname;
        if (window.location.hostname === 'www.pixiv.net' &&
            ['/', '/manga', '/novel', '/en/'].includes(pathname)) {
            return PageName.Home;
        }
        else if ((pathname.startsWith('/artworks') ||
            pathname.startsWith('/en/artworks')) &&
            /\/artworks\/\d{1,10}/.test(url)) {
            return PageName.Artwork;
        }
        else if (/\/users\/\d+/.test(url) && !url.includes('/bookmarks')) {
            if (pathname.includes('/following') ||
                pathname.includes('/mypixiv') ||
                pathname.includes('/followers')) {
                return PageName.Following;
            }
            else {
                return PageName.UserHome;
            }
        }
        else if (pathname.endsWith('bookmark.php')) {
            return PageName.BookmarkLegacy;
        }
        else if (pathname.includes('/bookmarks/')) {
            return PageName.Bookmark;
        }
        else if (url.includes('/tags/')) {
            return pathname.endsWith('/novels')
                ? PageName.NovelSearch
                : PageName.ArtworkSearch;
        }
        else if (pathname === '/ranking_area.php' && location.search !== '') {
            return PageName.AreaRanking;
        }
        else if (pathname === '/ranking.php') {
            return PageName.ArtworkRanking;
        }
        else if (url.includes('https://www.pixivision.net') &&
            url.includes('/a/')) {
            return PageName.Pixivision;
        }
        else if ((url.includes('/bookmark_add.php?id=') ||
            url.includes('/bookmark_detail.php?illust_id=')) &&
            !pathname.includes('/novel')) {
            return PageName.BookmarkDetail;
        }
        else if (url.includes('/bookmark_new_illust.php') ||
            url.includes('/bookmark_new_illust_r18.php')) {
            return PageName.NewArtworkBookmark;
        }
        else if (pathname === '/discovery' ||
            pathname.startsWith('/novel/discovery')) {
            return PageName.Discover;
        }
        else if (url.includes('/new_illust.php') ||
            url.includes('/new_illust_r18.php')) {
            return PageName.NewArtwork;
        }
        else if (pathname === '/novel/show.php') {
            return PageName.Novel;
        }
        else if (pathname.startsWith('/novel/series/')) {
            return PageName.NovelSeries;
        }
        else if (pathname === '/novel/ranking.php') {
            return PageName.NovelRanking;
        }
        else if (pathname.startsWith('/novel/bookmark_new')) {
            return PageName.NewNovelBookmark;
        }
        else if (pathname.startsWith('/novel/new')) {
            return PageName.NewNovel;
        }
        else if (pathname.startsWith('/user/') && pathname.includes('/series/')) {
            return PageName.ArtworkSeries;
        }
        else if (pathname.startsWith('/request')) {
            return PageName.Request;
        }
        else if (pathname.includes('/unlisted')) {
            return PageName.Unlisted;
        }
        else {
            // 没有匹配到可用的页面类型
            return PageName.Unsupported;
        }
    }
    // 页面切换时，检查页面类型是否变化
    checkTypeChange() {
        const old = this.type;
        this.type = this.getType();
        if (this.type !== old) {
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('pageSwitchedTypeChange', this.type);
        }
        else {
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('pageSwitchedTypeNotChange', this.type);
        }
    }
    async openAllTestPage() {
        // 列出要打开的测试页面。不包含已经不存在的页面类型和 Pixivision
        const testPageList = [
            {
                type: PageName.Unsupported,
                url: 'https://www.pixiv.net/stacc?mode=unify',
            },
            {
                type: PageName.Home,
                url: 'https://www.pixiv.net',
            },
            {
                type: PageName.Artwork,
                url: 'https://www.pixiv.net/artworks/108271116',
            },
            {
                type: PageName.UserHome,
                url: 'https://www.pixiv.net/users/89469319',
            },
            {
                type: PageName.Bookmark,
                url: 'https://www.pixiv.net/users/96661459/bookmarks/artworks',
            },
            {
                type: PageName.ArtworkSearch,
                url: 'https://www.pixiv.net/tags/%E5%8E%9F%E7%A5%9E/artworks?s_mode=s_tag',
            },
            {
                type: PageName.AreaRanking,
                url: 'https://www.pixiv.net/ranking_area.php?type=state&no=0',
            },
            {
                type: PageName.ArtworkRanking,
                url: 'https://www.pixiv.net/ranking.php',
            },
            {
                type: PageName.NewArtworkBookmark,
                url: 'https://www.pixiv.net/bookmark_new_illust.php',
            },
            {
                type: PageName.Discover,
                url: 'https://www.pixiv.net/discovery',
            },
            {
                type: PageName.NewArtwork,
                url: 'https://www.pixiv.net/new_illust.php',
            },
            {
                type: PageName.ArtworkSeries,
                url: 'https://www.pixiv.net/user/3698796/series/61267',
            },
            {
                type: PageName.Following,
                url: 'https://www.pixiv.net/users/96661459/following',
            },
            {
                type: PageName.Request,
                url: 'https://www.pixiv.net/request',
            },
            {
                type: PageName.Unlisted,
                url: 'https://www.pixiv.net/artworks/unlisted/eE3fTYaROT9IsZmep386',
            },
            {
                type: PageName.Novel,
                url: 'https://www.pixiv.net/novel/show.php?id=12771688',
            },
            {
                type: PageName.NovelSeries,
                url: 'https://www.pixiv.net/novel/series/1090654',
            },
            {
                type: PageName.NovelSearch,
                url: 'https://www.pixiv.net/tags/%E7%99%BE%E5%90%88/novels',
            },
            {
                type: PageName.NovelRanking,
                url: 'https://www.pixiv.net/novel/ranking.php?mode=daily',
            },
            {
                type: PageName.NewNovelBookmark,
                url: 'https://www.pixiv.net/novel/bookmark_new.php',
            },
            {
                type: PageName.NewNovel,
                url: 'https://www.pixiv.net/novel/new.php',
            },
        ];
        const wait = () => {
            return new Promise((resolve) => {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_1__.setTimeoutWorker.set(() => {
                    resolve();
                }, 500);
            });
        };
        for (const item of testPageList) {
            window.open(item.url);
            await wait();
        }
    }
}
const pageType = new PageType();



/***/ }),

/***/ "./src/ts/PreviewUgoira.ts":
/*!*********************************!*\
  !*** ./src/ts/PreviewUgoira.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   PreviewUgoira: () => (/* binding */ PreviewUgoira)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Log */ "./src/ts/Log.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");




// 预览动图
class PreviewUgoira {
    constructor(id, canvasWrap, prevSize, wrapWidth, wrapHeight) {
        this.prevSize = 'regular';
        /**完整的 zip 文件的字节数 */
        this.zipLength = 0;
        /** 用固定的字节数分割出多个文件片段 */
        this.rangeSize = 500000;
        /**保存每个文件片段的请求头的值
         * 字符串格式如 'bytes=0-499999'
         */
        this.rangeList = [];
        /**把分段加载的 zip 文件合并，保存到这个容器 */
        this.zipContent = new ArrayBuffer(0);
        /**所有文件片段是否都加载完毕 */
        this.loadend = false;
        /**每个 jpg 文件内容的开始位置 */
        this.jpgContentIndexList = [];
        /**每个 jpg 文件的数据。按照图片在压缩包里的顺序，储存对应的数据 */
        this.jpgFileList = [];
        // jpg 文件名的长度固定为 10 个字节 000000.jpg
        this.jpgNameLength = 10;
        this.wrapWidth = 0;
        this.wrapHeight = 0;
        this.canvas = document.createElement('canvas');
        this.canvasCon = this.canvas.getContext('2d');
        this.canvasIsAppend = false;
        this.width = 0;
        this.height = 0;
        this.destroyed = false;
        this.playIndex = 0;
        this.playDelay = 0;
        this.lastPlayTime = 0;
        this.animationID = 0;
        this.play = (timestamp) => {
            if (this.lastPlayTime === 0) {
                this.lastPlayTime = timestamp;
            }
            // 计算自上次执行之后，是否到了该执行下一次动画的时间
            if (timestamp - this.lastPlayTime >= this.playDelay) {
                // 如果要播放的图片尚未加载完成，则等到下一次动画帧再执行
                const img = this.jpgFileList[this.playIndex].img;
                if (!img.complete) {
                    return (this.animationID = window.requestAnimationFrame(this.play));
                }
                this.canvasCon.drawImage(img, 0, 0, this.width, this.height);
                // 绘制出第一张图片之后，才能显示 canvas 并隐藏之前的 img
                // 如果过早的隐藏 img 并显示 canvas，会导致闪烁（因为 img 先隐藏，此时 canvas 还没有绘制图像）
                if (this.playIndex === 0) {
                    this.canvas.style.display = 'inline-block';
                    const img = this.canvasWrap.querySelector('img');
                    if (img) {
                        img.style.display = 'none';
                    }
                }
                this.playDelay = this.jpgFileList[this.playIndex].delay;
                this.playIndex++;
                if (this.playIndex > this.jpgFileList.length - 1) {
                    this.playIndex = 0;
                }
                // 记录最后一次执行动画的时间
                this.lastPlayTime = timestamp;
            }
            this.animationID = window.requestAnimationFrame(this.play);
        };
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.previewUgoira) {
            return;
        }
        this.id = id;
        this.canvasWrap = canvasWrap;
        this.prevSize = prevSize;
        wrapWidth && (this.wrapWidth = wrapWidth);
        wrapHeight && (this.wrapHeight = wrapHeight);
        this.start();
    }
    async start() {
        // 获取这个动图的 meta 数据
        this.meta = await this.getMeta(this.id);
        // 目前只支持提取 jpg 图片
        if (this.meta.mime_type !== 'image/jpeg') {
            const msg = 'Preview ugoira error: mime type unsupport: ' + this.meta.mime_type;
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(msg);
            console.warn(msg);
            return;
        }
        // 设置要使用的 URL
        if (this.prevSize === 'regular') {
            this.zipURL = this.meta.src;
        }
        else {
            this.zipURL = this.meta.originalSrc;
        }
        // 获取动图体积
        this.zipLength = await this.getFileLength();
        // 生成区间
        this.rangeList = this.setRangeList(this.zipLength, this.rangeSize);
        for (const range of this.rangeList) {
            // 加载文件片段
            const buff = await this.loadRangeFileAsBuff(range);
            if (this.destroyed) {
                return;
            }
            // 把这个文件片段追加到文件容器里
            this.zipContent = this.appendBuff(this.zipContent, buff);
            // 判断整个压缩包是否全部加载完毕
            this.loadend = this.zipContent.byteLength === this.zipLength;
            // 提取出每个 jpg 图片的数据
            // 由于我之前使用的 zip 库无法解析不完整的 zip 文件，所以我需要自己提取 jpg 图片的数据
            this.jpgContentIndexList = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getJPGContentIndex(this.zipContent, this.jpgContentIndexList);
            this.extractJPGData(this.zipContent, this.jpgContentIndexList);
            // 设置画布的宽高
            if (this.jpgFileList.length > 0 && this.width === 0) {
                // 画布的宽高不能超过外部 wrap 的宽高
                // 本来我是打算从 wrap 宽度和动图宽度中取比较小的值
                // const size = await this.getImageSize(this.jpgFileList[0].blobURL)
                // this.width = Math.min(size.width, wrapWidth)
                // this.height = Math.min(size.height, wrapHeight)
                // 但是当预览作品的尺寸为“普通”时，动图的尺寸可能比 wrap 的尺寸小
                // 因为 wrap 显示的普通尺寸是 1200px，但是动图的普通尺寸是 600px
                // 所以我直接让画布使用 wrap 的尺寸了。如果动图比 wrap 小，就会放大到 wrap 的尺寸
                this.width =
                    this.wrapWidth || Number.parseInt(this.canvasWrap.style.width);
                this.height =
                    this.wrapHeight || Number.parseInt(this.canvasWrap.style.height);
            }
            // 检查是否应该开始播放动画
            // 如果动画的图片总量达到了 30 帧，则等到至少加载了 10 帧之后再开始播放
            // 这样可以减少在刚开始播放时，因可用帧数太少而导致画面看起来抖动（快速循环）的诡异情况
            if (this.meta.frames.length >= 30) {
                this.jpgFileList.length >= 10 && this.startPlay();
            }
            else if (this.meta.frames.length >= 20) {
                this.jpgFileList.length >= 5 && this.startPlay();
            }
            else if (this.meta.frames.length >= 10) {
                this.jpgFileList.length >= 4 && this.startPlay();
            }
            else {
                this.jpgFileList.length >= 1 && this.startPlay();
            }
        }
        // 保存整个压缩包（debug 用）
        // const newFile = new Blob([this.zipContent])
        // const url = URL.createObjectURL(newFile)
        // Utils.downloadFile(url, `${this.id}.zip`)
    }
    /**获取该作品的 meta 数据 */
    getMeta(id) {
        return new Promise(async (resolve, reject) => {
            const meta = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getUgoiraMeta(id);
            if (meta.error) {
                throw reject(meta.message);
            }
            resolve(meta.body);
        });
    }
    /** 发送 HEAD 请求，获取 zip 压缩包的体积 */
    getFileLength() {
        return new Promise(async (resolve, reject) => {
            const response = await fetch(this.zipURL, {
                method: 'head',
                credentials: 'same-origin',
            });
            const length = response.headers.get('content-length');
            if (!length) {
                throw reject('getFileLength error: get length failed');
            }
            resolve(Number.parseInt(length));
        });
    }
    /** 根据 zip 文件的体积分割出数个区间，生成对应的标记文本 */
    setRangeList(total, rangeSize) {
        const result = [];
        // total 是 length，但 start 和 end 是下标
        let start = 0;
        let end = 0;
        const max = total - 1;
        while (end < max) {
            if (start > 0) {
                start++;
            }
            end = start + rangeSize - 1;
            if (end > max) {
                end = max;
            }
            const str = `bytes=${start}-${end}`;
            result.push(str);
            start = end;
        }
        return result;
    }
    loadRangeFileAsBuff(range) {
        return new Promise(async (resolve, reject) => {
            const res = await fetch(this.zipURL, {
                method: 'get',
                headers: {
                    range: range,
                },
            });
            const buff = await res.arrayBuffer();
            resolve(buff);
        });
    }
    /**把 ArrayBuffer 追加到已存在的 ArrayBuffer 容器里  */
    appendBuff(target, newBuff) {
        const totalLength = target.byteLength + newBuff.byteLength;
        const uint8 = new Uint8Array(totalLength);
        uint8.set(new Uint8Array(target));
        uint8.set(new Uint8Array(newBuff), target.byteLength);
        return uint8.buffer;
    }
    /** 从 zip 文件里提取出所有 jpg 图片的数据 */
    extractJPGData(file, indexList) {
        indexList.forEach((number, index, array) => {
            // 如果这是最后一个标记，并且压缩包没有整个加载完成，则不提取最后一个文件的数据
            // 因为此时最后一个文件的数据很可能是破损的
            if (index === array.length - 1 && !this.loadend) {
                return;
            }
            // 如果这张图片没有被保存，才会提取它
            // 如果已经有这个图片的数据，就不再提取它，以提高性能
            if (this.jpgFileList[index] === undefined) {
                // 确定要提取的文件的起始位置
                // 从当前文件名之后开始
                const start = number;
                // 截止下一个文件名之前
                // 删除不需要的数据：
                // 30 字节的是 zip 文件的数据，虽然没有实际影响，但还是去掉
                // 10 字节的是下一个 jpg 的文件名
                let end = array[index + 1] - 30 - this.jpgNameLength;
                if (index === array.length - 1) {
                    // 如果是最后一个 jpg 文件，则截止到 zip 文件的结尾
                    // 这导致它会包含 zip 的目录数据，但是不会影响图片的显示
                    end = file.byteLength;
                }
                // slice 方法的 end 不会包含在结果里
                const buffer = file.slice(start, end);
                const blob = new Blob([buffer], {
                    type: 'image/jpeg',
                });
                const url = URL.createObjectURL(blob);
                // 下载这张图片（debug 用）
                // Utils.downloadFile(url, `${index}.jpg`)
                const img = new Image(this.width, this.height);
                img.src = url;
                this.jpgFileList[index] = {
                    img: img,
                    delay: this.meta.frames[index].delay,
                };
            }
        });
    }
    startPlay() {
        if (this.jpgFileList.length > 0 && !this.canvasIsAppend) {
            this.addCanvas();
            this.canvasIsAppend = true;
            this.animationID = window.requestAnimationFrame(this.play);
        }
    }
    addCanvas() {
        const oldCanvas = this.canvasWrap.querySelector('canvas');
        if (oldCanvas) {
            oldCanvas.remove();
        }
        this.canvas.style.display = 'none';
        this.canvasWrap.append(this.canvas);
        this.canvas.width = this.width;
        this.canvas.height = this.height;
    }
    setSize(width, height) {
        // 当用户移动鼠标时，ShowOriginSizeImage 会频繁触发 setSize
        // 如果频繁的重设尺寸，会导致动画闪烁。所以判断只在有必要时才重设尺寸
        if (width === this.width && height === this.height) {
            return;
        }
        this.width = width;
        this.height = height;
        this.canvas.width = this.width;
        this.canvas.height = this.height;
    }
    destroy() {
        this.destroyed = true;
        window.cancelAnimationFrame(this.animationID);
        this.canvas.remove();
        this.zipContent = new ArrayBuffer(0);
        this.jpgFileList = [];
        this.jpgContentIndexList = [];
    }
}



/***/ }),

/***/ "./src/ts/PreviewWork.ts":
/*!*******************************!*\
  !*** ./src/ts/PreviewWork.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _ShowOriginSizeImage__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./ShowOriginSizeImage */ "./src/ts/ShowOriginSizeImage.ts");
/* harmony import */ var _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./store/CacheWorkData */ "./src/ts/store/CacheWorkData.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _PreviewUgoira__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./PreviewUgoira */ "./src/ts/PreviewUgoira.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _utils_DateFormat__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ./utils/DateFormat */ "./src/ts/utils/DateFormat.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _PreviewWorkDetailInfo__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./PreviewWorkDetailInfo */ "./src/ts/PreviewWorkDetailInfo.ts");

















// 鼠标停留在作品的缩略图上时，预览作品
class PreviewWork {
    constructor() {
        // 预览作品的容器的元素
        this.wrapId = 'previewWorkWrap';
        this.img = document.createElement('img');
        this.border = 4; // border 占据的空间
        this.tipId = 'previewWorkTip';
        this.tipHeight = 22;
        // 保存当前鼠标经过的缩略图的数据
        this.workId = '';
        // 显示作品中的第几张图片
        this.index = 0;
        // 延迟显示预览区域的定时器
        // 鼠标进入缩略图时，本模块会立即请求作品数据，但在请求完成后不会立即加载图片，这是为了避免浪费网络资源
        this.delayShowTimer = undefined;
        // 延迟隐藏预览区域的定时器
        this.delayHiddenTimer = undefined;
        // 当用户点击预览图使预览图隐藏时，不再显示这个作品的预览图（切换作品可以解除限制）
        this.dontShowAgain = false;
        // 是否允许预览区域遮挡作品缩略图
        this.allowOverThumb = true;
        // 当前预览图是否遮挡了作品缩略图
        this.overThumb = false;
        this._show = false;
        // 当鼠标滚轮滚动时，切换显示的图片
        // 此事件必须使用节流，因为有时候鼠标滚轮短暂的滚动一下就会触发 2 次 mousewheel 事件
        this.swicthImage = _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.throttle(() => {
            const count = this.workData.body.pageCount;
            const up = this.wheelEvent.deltaY < 0;
            if (up) {
                if (this.index > 0) {
                    this.index--;
                }
                else {
                    this.index = count - 1;
                }
            }
            else {
                if (this.index < count - 1) {
                    this.index++;
                }
                else {
                    this.index = 0;
                }
            }
            this.showWrap();
        }, 100);
        this.onWheelScroll = (ev) => {
            if (this.show &&
                _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.wheelScrollSwitchImageOnPreviewWork &&
                this.workData.body.pageCount > 1) {
                ev.preventDefault();
                this.wheelEvent = ev;
                this.swicthImage();
            }
        };
        if (_Config__WEBPACK_IMPORTED_MODULE_15__.Config.mobile) {
            return;
        }
        this.createElements();
        this.bindEvents();
    }
    get show() {
        return this._show;
    }
    set show(val) {
        if (val) {
            this.workData = _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_5__.cacheWorkData.get(this.workId);
            // 如果保存的作品数据不是最后一个鼠标经过的作品，可能是请求尚未完成，此时延长等待时间
            if (!this.workData || this.workData.body.id !== this.workId) {
                this.readyShow();
            }
            else {
                // 显示作品的详细信息
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.PreviewWorkDetailInfo) {
                    _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('showPreviewWorkDetailPanel', this.workData);
                }
                this.sendUrls();
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.PreviewWork) {
                    this._show = true;
                    _ShowOriginSizeImage__WEBPACK_IMPORTED_MODULE_4__.showOriginSizeImage.hide();
                    this.showWrap();
                    window.clearTimeout(this.delayHiddenTimer);
                    if (!_Config__WEBPACK_IMPORTED_MODULE_15__.Config.mobile) {
                        _ShowHelp__WEBPACK_IMPORTED_MODULE_13__.showHelp.show('tipPressDToQuickDownload', _Lang__WEBPACK_IMPORTED_MODULE_10__.lang.transl('_预览作品时按快捷键可以下载这个作品'));
                    }
                }
            }
        }
        else {
            // 隐藏时重置一些变量
            window.clearTimeout(this.delayShowTimer);
            window.clearTimeout(this.delayHiddenTimer);
            this.overThumb = false;
            this._show = false;
            this.dontShowAgain = false;
            this.wrap.style.display = 'none';
            // 隐藏 wrap 时，把 img 的 src 设置为空
            // 这样图片会停止加载，避免浪费网络资源
            this.img.src = '';
            // 销毁预览动图的模块
            if (this.previewUgoira) {
                this.previewUgoira.destroy();
                this.previewUgoira = null;
            }
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('previewEnd');
        }
    }
    createElements() {
        this.wrap = document.createElement('div');
        this.wrap.id = this.wrapId;
        this.tip = document.createElement('div');
        this.tip.id = this.tipId;
        this.wrap.appendChild(this.tip);
        document.body.appendChild(this.wrap);
    }
    bindEvents() {
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__.artworkThumbnail.onEnter((el, id) => {
            if (this.dontShowAgain) {
                return;
            }
            // 当鼠标进入到不同作品时
            // 隐藏之前的预览图
            // 重置 index
            if (this.workId !== id) {
                this.show = false;
                this.index = 0;
            }
            this.workId = id;
            this.workEL = el;
            if (!_store_CacheWorkData__WEBPACK_IMPORTED_MODULE_5__.cacheWorkData.has(id)) {
                // 如果在缓存中没有找到这个作品的数据，则发起请求
                this.fetchWorkData();
            }
            else {
                this.workData = _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_5__.cacheWorkData.get(id);
            }
            this.readyShow();
            el.addEventListener('mousewheel', this.onWheelScroll);
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__.artworkThumbnail.onLeave((el) => {
            // 当鼠标离开作品缩略图时，有可能是因为显示了作品详细信息的面板。此时让预览图保持显示
            if (_PreviewWorkDetailInfo__WEBPACK_IMPORTED_MODULE_16__.previewWorkDetailInfo.show) {
                return;
            }
            if (this.overThumb) {
                // 如果预览图遮挡了作品缩略图，就需要延迟隐藏预览图。
                // 因为预览图显示之后，鼠标可能处于预览图上，这会触发此事件。
                // 如果不延迟隐藏，预览图就会马上消失，无法查看
                this.delayHiddenTimer = window.setTimeout(() => {
                    this.show = false;
                    el.removeEventListener('mousewheel', this.onWheelScroll);
                }, 100);
            }
            else {
                this.show = false;
                el.removeEventListener('mousewheel', this.onWheelScroll);
            }
        });
        window.addEventListener('keydown', (ev) => {
            // 可以使用 Alt + P 快捷键来启用/禁用此功能
            if (ev.altKey && ev.code === 'KeyP') {
                (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('PreviewWork', !_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.PreviewWork);
                // 显示提示信息
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.PreviewWork) {
                    const msg = 'Preview work - On';
                    _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.success(msg);
                }
                else {
                    const msg = 'Preview work - Off';
                    _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.warning(msg);
                }
            }
            // 预览作品时，可以使用快捷键 D 下载这个作品
            if (ev.code === 'KeyD' && this.show) {
                _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('crawlIdList', [
                    {
                        type: 'illusts',
                        id: this.workData.body.id,
                    },
                ]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_6__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_10__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_11__.Colors.bgBlue,
                    position: 'center',
                });
            }
            // 预览作品时，可以使用快捷键 C 仅下载当前显示的图片
            if (ev.code === 'KeyC' && this.show) {
                // 在作品页面内按 C 时，Pixiv 会把焦点定位到评论输入框里，这里阻止此行为
                ev.stopPropagation();
                if (this.workData.body.pageCount > 1) {
                    _store_Store__WEBPACK_IMPORTED_MODULE_14__.store.setDownloadOnlyPart(Number.parseInt(this.workData.body.id), [
                        this.index,
                    ]);
                }
                _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('crawlIdList', [
                    {
                        type: 'illusts',
                        id: this.workData.body.id,
                    },
                ]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_6__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_10__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_11__.Colors.bgBlue,
                    position: 'center',
                });
            }
        }, true);
        const hiddenEvtList = [
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.centerPanelOpened,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.showOriginSizeImage,
        ];
        hiddenEvtList.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.show = false;
            });
        });
        // 当作品的详情面板隐藏时，鼠标位置可能在作品缩略图之外。所以此时需要检测鼠标位置，决定是否需要隐藏预览图
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.PreviewWorkDetailPanelClosed, (ev) => {
            const data = ev.detail?.data;
            if (this.mouseInElementArea(this.workEL, data.x, data.y) === false) {
                this.show = false;
            }
        });
        this.wrap.addEventListener('mouseenter', () => {
            window.clearTimeout(this.delayHiddenTimer);
        });
        this.wrap.addEventListener('mousemove', (ev) => {
            // 鼠标在预览图上移动出缩略图区域时，隐藏预览图
            if (this.mouseInElementArea(this.workEL, ev.clientX, ev.clientY) === false) {
                this.show = false;
            }
        });
        this.wrap.addEventListener('click', (ev) => {
            this.show = false;
            // 点击预览图使预览图消失时，如果鼠标仍处于缩略图区域内，则不再显示这个作品的预览图
            // 当鼠标移出这个作品的缩略图之后取消此限制
            if (this.mouseInElementArea(this.workEL, ev.clientX, ev.clientY)) {
                this.dontShowAgain = true;
            }
        });
        this.wrap.addEventListener('mousewheel', (ev) => {
            this.overThumb && this.onWheelScroll(ev);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.wheelScrollSwitchPreviewImage, (ev) => {
            const mouseEvent = ev.detail.data;
            mouseEvent && this.onWheelScroll(mouseEvent);
        });
    }
    // 判断鼠标是否处于某个元素的范围内
    mouseInElementArea(el, x, y) {
        if (!el) {
            return false;
        }
        const rect = el.getBoundingClientRect();
        return x > rect.left && x < rect.right && y > rect.top && y < rect.bottom;
    }
    preload() {
        // 如果下载器正在下载文件，则不预加载
        if (this.show && !_store_States__WEBPACK_IMPORTED_MODULE_6__.states.downloading) {
            const count = this.workData.body.pageCount;
            if (count > this.index + 1) {
                let url = this.workData.body.urls[_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.prevWorkSize];
                url = url.replace('p0', `p${this.index + 1}`);
                let img = new Image();
                // 在预加载过程中，如果查看的图片变化了，或者不显示预览区域了，则立即中断预加载
                const nowIndex = this.index;
                const timer = window.setInterval(() => {
                    if (this.index !== nowIndex || !this.show) {
                        window.clearInterval(timer);
                        img && (img.src = '');
                        img = null;
                    }
                }, 50);
                img.onload = () => {
                    window.clearInterval(timer);
                    img && (img = null);
                };
                img.src = url;
            }
        }
    }
    async fetchWorkData() {
        const data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(this.workId);
        _store_CacheWorkData__WEBPACK_IMPORTED_MODULE_5__.cacheWorkData.set(data);
    }
    readyShow() {
        this.delayShowTimer = window.setTimeout(() => {
            this.show = true;
        }, _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.previewWorkWait);
    }
    // 通过 img 元素加载图片，获取图片的原始尺寸
    async getImageSize(url) {
        return new Promise((resolve) => {
            // 鼠标滚轮滚动时，此方法可能会在短时间内触发多次。通过 index 判断当前请求是否应该继续
            let testImg = new Image();
            testImg.src = url;
            const bindIndex = this.index;
            const timer = window.setInterval(() => {
                if (this.index !== bindIndex) {
                    // 如果要显示的图片发生了变化，则立即停止加载当前图片，避免浪费网络流量
                    window.clearInterval(timer);
                    testImg.src = '';
                    testImg = null;
                    // 本来这里应该 reject 的，但是那样就需要在 await 的地方处理这个错误
                    // 我不想处理错误，所以用 available 标记来偷懒
                    return resolve({
                        width: 0,
                        height: 0,
                        available: false,
                    });
                }
                else {
                    // 如果获取到了图片的宽高，也立即停止加载当前图片，并返回结果
                    if (testImg.naturalWidth > 0) {
                        const width = testImg.naturalWidth;
                        const height = testImg.naturalHeight;
                        window.clearInterval(timer);
                        testImg.src = '';
                        testImg = null;
                        return resolve({
                            width,
                            height,
                            available: true,
                        });
                    }
                }
            }, 50);
        });
    }
    // 显示预览 wrap
    async showWrap() {
        if (!this.workEL || !this.workData) {
            return;
        }
        const url = this.replaceUrl(this.workData.body.urls[_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.prevWorkSize]);
        const size = await this.getImageSize(url);
        // getImageSize 可能需要花费比较长的时间。有时候在 getImageSize 之前是要显示 wrap 的，但是之后鼠标移出，需要隐藏 wrap，再之后 getImageSize 才执行完毕。
        // 所以此时需要再次判断是否要显示 wrap。如果不再次判断的话，可能有时候需要隐藏预览图，但是预览图却显示出来了
        if (!size.available || !this.show) {
            return;
        }
        const w = size.width;
        const h = size.height;
        const cfg = {
            width: w,
            height: h,
            left: 0,
            top: 0,
        };
        // 每次显示图片时，都销毁旧的 img 元素，然后重新生成一个 img 元素，而不是修改之前的 img 元素的 src
        // 因为修改 src 的方式存在严重的问题：虽然 src 已经变化了，但是 img 元素显示的还是上一张图片（不管上一张图片是否加载完成）。等到新的图片完全加载完成后，img 才会变化。
        // 这会导致一些问题：
        // 1. 在新图片的加载过程中，用户无法看到加载进度。只能等到图片加载完成后瞬间完全显示出来。
        // 2. 在新图片的加载过程中，图片的宽高是新图片的宽高，但是显示的内容还是旧的图片。如果这两张图片的尺寸不一致，此时显示的（旧）图片看上去是变形的
        // 只有生成新的 img 元素，才能解决上面的问题
        this.img.src = '';
        this.img.remove();
        this.img = document.createElement('img');
        // 当图片加载完成时，预加载下一张图片
        this.img.onload = () => this.preload();
        this.img.src = url;
        this.wrap.appendChild(this.img);
        // 1. 计算图片显示的尺寸
        const rect = this.workEL.getBoundingClientRect();
        // 不显示摘要信息时，也不显示边框，所以此时把 border 设置为 0
        this.border = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showPreviewWorkTip ? 4 : 0;
        // 计算各个可用区域的尺寸，提前减去了 border、tip 等元素占据的空间
        const innerWidth = window.innerWidth - 17;
        const leftSpace = rect.left - this.border;
        const rightSpace = innerWidth - rect.right - this.border;
        const xSpace = Math.max(leftSpace, rightSpace);
        const tipHeight = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showPreviewWorkTip ? this.tipHeight : 0;
        const scrollBarHeight = window.innerHeight - document.documentElement.clientHeight;
        const ySpace = window.innerHeight - scrollBarHeight - this.border - tipHeight;
        // 宽高从图片宽高、可用区域的宽高中取最小值，使图片不会超出可视区域外
        // 竖图
        if (w < h) {
            cfg.height = Math.min(ySpace, h);
            cfg.width = (cfg.height / h) * w;
            // 此时宽度可能会超过水平方向上的可用区域，则需要再次调整宽高
            if (cfg.width > xSpace) {
                cfg.height = (xSpace / cfg.width) * cfg.height;
                cfg.width = xSpace;
            }
        }
        else if (w > h) {
            // 横图
            if (this.allowOverThumb) {
                // 如果允许预览图覆盖在作品缩略图上，则预览图的最大宽度可以等于视口宽度
                if (w > innerWidth) {
                    cfg.width = innerWidth;
                }
            }
            else {
                // 否则，预览图的宽度不可以超过图片两侧的空白区域的宽度
                cfg.width = Math.min(xSpace, w);
            }
            cfg.height = (cfg.width / w) * h;
            // 此时高度可能会超过垂直方向上的可用区域，则需要再次调整宽高
            if (cfg.height > ySpace) {
                cfg.width = (ySpace / cfg.height) * cfg.width;
                cfg.height = ySpace;
            }
        }
        else {
            // 正方形图片
            cfg.height = Math.min(ySpace, xSpace, h);
            cfg.width = cfg.height;
        }
        // 上面计算的高度是图片的高度，现在计算 wrap 的宽高，需要加上内部其他元素的高度
        cfg.height = cfg.height + tipHeight;
        // 2. 计算位置
        // 在页面可视区域内，比较缩略图左侧和右侧空间，把 wrap 显示在空间比较大的那一侧
        if (leftSpace >= rightSpace) {
            // 左侧空间大
            // 先让预览图的右侧贴着图片左侧边缘显示
            cfg.left = rect.left - cfg.width - this.border + window.scrollX;
            // 如果预览图超出可视范围，则向右移动
            if (cfg.left < 0) {
                this.overThumb = true;
                cfg.left = 0;
            }
        }
        else {
            // 右侧空间大
            // 先让预览图的左侧贴着图片右侧边缘显示
            cfg.left = rect.right + window.scrollX;
            // 如果预览图超出可视范围，则向左移动
            if (cfg.width > rightSpace) {
                this.overThumb = true;
                cfg.left = cfg.left - (cfg.left + cfg.width - innerWidth) - this.border;
            }
        }
        // 然后设置 top
        // 让 wrap 和缩略图在垂直方向上居中对齐
        cfg.top = rect.top;
        const wrapHalfHeight = (cfg.height + this.border) / 2;
        const workHalfHeight = rect.height / 2;
        cfg.top = cfg.top - wrapHalfHeight + workHalfHeight;
        // 检查 wrap 顶端是否超出了窗口可视区域
        if (cfg.top < 0) {
            cfg.top = 0;
        }
        // 检查 wrap 底部是否超出了窗口可视区域
        const bottomOver = cfg.top + cfg.height + this.border - window.innerHeight;
        if (bottomOver > 0) {
            // 如果底部超出了窗口可视区域，则计算顶部是否还有可用空间
            if (cfg.top > 0) {
                // 如果顶部还有空间可用，就尽量向上移动，但不会导致顶端超出可视区域
                cfg.top = cfg.top - Math.min(bottomOver, cfg.top) - scrollBarHeight;
            }
        }
        // 3. 设置顶部提示区域的内容
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showPreviewWorkTip) {
            const text = [];
            const body = this.workData.body;
            if (body.pageCount > 1) {
                text.push(`${this.index + 1}/${body.pageCount}`);
            }
            text.push(body.bookmarkCount.toString());
            // 加载原图时，可以获取到每张图片的真实尺寸
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.prevWorkSize === 'original') {
                text.push(`${w}x${h}`);
            }
            else {
                // 如果加载的是普通尺寸，则永远显示第一张图的原始尺寸
                // 因为此时获取不到后续图片的原始尺寸
                text.push(`${this.workData.body.width}x${this.workData.body.height}`);
            }
            text.push(_utils_DateFormat__WEBPACK_IMPORTED_MODULE_12__.DateFormat.format(body.createDate, 'YYYY/MM/DD'));
            text.push(body.title);
            text.push(body.description);
            this.tip.innerHTML = text
                .map((str) => {
                return `<span>${str}</span>`;
            })
                .join('');
            this.tip.style.display = 'block';
        }
        else {
            this.tip.style.display = 'none';
        }
        // 4. 显示 wrap
        this.img.style.height = cfg.height - tipHeight + 'px';
        const styleArray = [];
        for (const [key, value] of Object.entries(cfg)) {
            styleArray.push(`${key}:${value}px;`);
        }
        styleArray.push('display:block;');
        // 如果不显示摘要信息，覆写一些样式
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showPreviewWorkTip) {
            styleArray.push('border:none;');
            styleArray.push('box-shadow:none;');
        }
        this.wrap.setAttribute('style', styleArray.join(''));
        // 每次显示图片后，传递图片的 url
        this.sendUrls();
        // 预览动图
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.previewUgoira && this.workData.body.illustType === 2) {
            this.previewUgoira = new _PreviewUgoira__WEBPACK_IMPORTED_MODULE_8__.PreviewUgoira(this.workData.body.id, this.wrap, _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.prevWorkSize, cfg.width, cfg.height - tipHeight);
            // 需要显式传递 wrap 的宽高，特别是高度。因为需要减去顶部提示区域的高度
        }
    }
    replaceUrl(url) {
        return url.replace('p0', `p${this.index}`);
    }
    sendUrls() {
        const data = this.workData;
        if (!data) {
            return;
        }
        // 传递图片的 url，但是不传递尺寸。
        // 因为预览图片默认加载“普通”尺寸的图片，但是 showOriginSizeImage 默认显示“原图”尺寸。
        // 而且对于第一张之后的图片，加载“普通”尺寸的图片时，无法获取“原图”的尺寸。
        _ShowOriginSizeImage__WEBPACK_IMPORTED_MODULE_4__.showOriginSizeImage.setData({
            original: this.replaceUrl(data.body.urls.original),
            regular: this.replaceUrl(data.body.urls.regular),
        }, data, this.index);
    }
}
new PreviewWork();


/***/ }),

/***/ "./src/ts/PreviewWorkDetailInfo.ts":
/*!*****************************************!*\
  !*** ./src/ts/PreviewWorkDetailInfo.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   previewWorkDetailInfo: () => (/* binding */ previewWorkDetailInfo)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _CopyToClipboard__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./CopyToClipboard */ "./src/ts/CopyToClipboard.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");






// 预览作品的详细信息
// 这个模块由 PreviewWork 提供作品数据，这样可以避免一些重复代码
class PreviewWorkDetailInfo {
    constructor() {
        // 因为预览作品模块里没有保存鼠标位置，所以本模块需要自己保存鼠标位置
        this.mouseX = 0;
        this.mouseY = 0;
        this.show = false;
        // 保存当前预览的作品 ID，避免在一个预览图上多次显示这个详情面板
        // 当预览图的窗口消失时，会重置这个 ID
        this.showWorkID = '';
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.showPreviewWorkDetailPanel, (ev) => {
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.PreviewWorkDetailInfo) {
                const data = ev.detail.data;
                this.create(data);
            }
        });
        window.addEventListener('mousemove', (ev) => {
            this.mouseX = ev.clientX;
            this.mouseY = ev.clientY;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.previewEnd, () => {
            this.showWorkID = '';
        });
    }
    create(workData) {
        // 有可能会重复创建，所以需要处理一下
        if (this.show) {
            return;
        }
        if (workData.body.id === this.showWorkID) {
            return;
        }
        else {
            this.showWorkID = workData.body.id;
        }
        // 这里先把 show 状态设置为 true。实际显示出来还需要经过后面的处理
        this.show = true;
        const wrap = document.createElement('div');
        // 设置文字内容
        // 生成 tag 内容
        const tagsHTML = [];
        for (const item of workData.body.tags.tags) {
            const array = [];
            const link = `https://www.pixiv.net/tags/${item.tag}/artworks`;
            array.push(`<span class="origin"><a href="${link}" target="_blank">#${item.tag}</a></span>`);
            if (item.translation?.en) {
                array.push(`<span class="transl"><a href="${link}" target="_blank">${item.translation?.en}</a></span>`);
            }
            tagsHTML.push(`<span>${array.join('')}</span>`);
        }
        // 生成收藏数、点赞数、浏览量一栏
        const bmkHTML = [];
        const schema = [
            {
                title: 'bookmark',
                icon: `<svg viewBox="0 0 12 12" width="12" height="12"><path fill="currentColor" d="
      M9,0.75 C10.6568542,0.75 12,2.09314575 12,3.75 C12,6.68851315 10.0811423,9.22726429 6.24342696,11.3662534
      L6.24342863,11.3662564 C6.09210392,11.4505987 5.90790324,11.4505988 5.75657851,11.3662565
      C1.9188595,9.22726671 0,6.68851455 0,3.75 C1.1324993e-16,2.09314575 1.34314575,0.75 3,0.75
      C4.12649824,0.75 5.33911281,1.60202454 6,2.66822994 C6.66088719,1.60202454 7.87350176,0.75 9,0.75 Z"></path></svg>`,
                number: workData.body.bookmarkCount,
            },
            {
                title: 'like',
                icon: `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 12 12" width="12" height="12"><path fill="#858585" d="M2 6a2 2 0 110-4 2 2 0 010 4zm8 0a2 2 0 110-4 2 2 0 010 4zM2.11 8.89a1 1 0 011.415-1.415 3.5 3.5 0 004.95 0 1 1 0 011.414 1.414 5.5 5.5 0 01-7.778 0z"/></svg>`,
                number: workData.body.likeCount,
            },
            {
                title: 'view',
                icon: `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 14 12" width="12" height="12"><path fill="#858585" d="M0 6c2-3.333 4.333-5 7-5s5 1.667 7 5c-2 3.333-4.333 5-7 5S2 9.333 0 6z"/><path fill="#fff" d="M7 8.5a2.5 2.5 0 110-5 2.5 2.5 0 010 5zm0-1a1.5 1.5 0 100-3 1.5 1.5 0 000 3z"/></svg>`,
                number: workData.body.viewCount,
            },
        ];
        for (const cfg of schema) {
            bmkHTML.push(`
      <span class="item" title="${cfg.title}">
        <span>${cfg.icon}</span>
        <span>${_Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.numberToString(cfg.number)}</span>
      </span class="item">
      `);
        }
        // 生成 R-18(G) 和 AI 标记
        let r18HTML = '';
        if (workData.body.xRestrict === 1) {
            r18HTML = '<span class="r18">R-18</span>';
        }
        else if (workData.body.xRestrict === 2) {
            r18HTML = '<span class="r18">R-18G</span>';
        }
        let aiHTML = '';
        if (workData.body.aiType === 2) {
            aiHTML = '<span class="ai">AI</span>';
        }
        wrap.innerHTML = `
        <div class="content">
        <p class="flags">${r18HTML} ${aiHTML}</p>
        <p class="title">${workData.body.title}</p>
        <p class="desc">${workData.body.description}</p>
        <p class="tags">${tagsHTML.join('')}</p>
        <p class="size">${workData.body.width} x ${workData.body.height}</p>
        <p class="bmk">${bmkHTML.join('')}</p>
        <p class="date">${new Date(workData.body.uploadDate).toLocaleString()}</p>
        <p class="buttons"><button class="textButton" id="copyTXT">Copy TXT</button> <button class="textButton" id="copyJSON">Copy JSON</button></p>
        </div>
      `;
        // 按钮功能
        const copyTXT = wrap.querySelector('#copyTXT');
        copyTXT.addEventListener('click', () => {
            this.copyTXT(workData);
        });
        const copyJSON = wrap.querySelector('#copyJSON');
        copyJSON.addEventListener('click', () => {
            this.copyJSON(workData);
        });
        // 取消超链接的跳转确认，也就是把跳转链接替换为真正的链接
        const allLink = wrap.querySelectorAll('a');
        for (const a of allLink) {
            if (a.href.includes('jump.php')) {
                a.href = a.innerText;
            }
        }
        // 设置样式
        wrap.classList.add('xz_PreviewWorkDetailPanel');
        wrap.style.width = _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.PreviewDetailInfoWidth + 'px';
        wrap.addEventListener('click', () => {
            this.remove(wrap);
        });
        wrap.addEventListener('mouseleave', () => {
            this.remove(wrap);
        });
        wrap.addEventListener('mousewheel', (ev) => {
            if (this.show) {
                ev.preventDefault();
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('wheelScrollSwitchPreviewImage', ev);
            }
        });
        _Theme__WEBPACK_IMPORTED_MODULE_1__.theme.register(wrap);
        document.body.append(wrap);
        // 获取宽高，以鼠标为中心显示
        const rect = wrap.getBoundingClientRect();
        // 设置 left
        let left = this.mouseX - rect.width / 2;
        // 最小的 left 为 10，避免其紧贴左侧，看不到左边界。其他四边也同理
        if (left < 10) {
            left = 10;
        }
        // 如果面板右侧超出屏幕，则把面板向左移动
        let right = left + rect.width;
        if (right > window.innerWidth) {
            left = left + (window.innerWidth - right) - 10;
        }
        wrap.style.left = left + 'px';
        // 设置 top
        let top = this.mouseY - rect.height / 2;
        if (top < 10) {
            top = 10;
        }
        // 如果面板底部超出屏幕，则把面板向上移动
        let bottom = top + rect.height;
        if (bottom > window.innerHeight) {
            top = top + (window.innerHeight - bottom) - 10;
        }
        wrap.style.top = top + 'px';
        wrap.style.opacity = '1';
    }
    remove(el) {
        el && el.parentNode && el.parentNode.removeChild(el);
        this.show = false;
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('PreviewWorkDetailPanelClosed', {
            x: this.mouseX,
            y: this.mouseY,
        });
    }
    copyTXT(workData) {
        // 组织输出的内容
        const tags = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.extractTags(workData).map((tag) => `#${tag}`);
        const array = [];
        const body = workData.body;
        array.push(`ID\n${body.id}`);
        array.push(`URL\nhttps://www.pixiv.net/artworks/${body.id}`);
        array.push(`Original\n${body.urls?.original}`);
        array.push(`xRestrict\n${_Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getXRestrictText(body.xRestrict)}`);
        array.push(`AI\n${_Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getAITypeText(body.aiType)}`);
        array.push(`User\n${body.userName}`);
        array.push(`UserID\n${body.userId}`);
        array.push(`Title\n${body.title}`);
        if (body.description) {
            array.push(`Description\n${_utils_Utils__WEBPACK_IMPORTED_MODULE_5__.Utils.htmlToText(body.description)}`);
        }
        array.push(`Tags\n${tags.join('\n')}`);
        array.push(`Size\n${body.width} x ${body.height}`);
        array.push(`Bookmark\n${body.bookmarkCount}`);
        array.push(`Date\n${new Date(body.uploadDate).toLocaleString()}`);
        const text = array.join('\n\n');
        _CopyToClipboard__WEBPACK_IMPORTED_MODULE_4__.CopyToClipboard.setClipboard(text);
    }
    copyJSON(workData) {
        const text = JSON.stringify(workData, null, 2);
        _CopyToClipboard__WEBPACK_IMPORTED_MODULE_4__.CopyToClipboard.setClipboard(text);
    }
}
const previewWorkDetailInfo = new PreviewWorkDetailInfo();



/***/ }),

/***/ "./src/ts/RemoveWorksTagsInBookmarks.ts":
/*!**********************************************!*\
  !*** ./src/ts/RemoveWorksTagsInBookmarks.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   removeWorksTagsInBookmarks: () => (/* binding */ removeWorksTagsInBookmarks)
/* harmony export */ });
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Log */ "./src/ts/Log.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Bookmark */ "./src/ts/Bookmark.ts");





class RemoveWorksTagsInBookmarks {
    async start(list) {
        if (list.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_2__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_没有数据可供使用'));
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_没有数据可供使用'));
            return;
        }
        _store_States__WEBPACK_IMPORTED_MODULE_3__.states.busy = true;
        const total = list.length.toString();
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_当前作品个数', total));
        let number = 0;
        for (const item of list) {
            try {
                await _Bookmark__WEBPACK_IMPORTED_MODULE_4__.bookmark.add(item.workID.toString(), item.type, [], false, item.private, true);
            }
            catch (error) {
                // 处理自己收藏的作品时可能遇到错误。最常见的错误就是作品被删除了，获取作品数据时会产生 404 错误
                // 但是也可能出现其他错误，比如因为请求太多而出现 429 错误。因为 429 错误需要等待几分钟后才能重试，这里偷懒不再重试
            }
            number++;
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.log(`${number} / ${total}`, 1, false);
        }
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_移除本页面中所有作品的标签') + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_完成');
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_2__.toast.success(msg, {
            position: 'topCenter',
        });
        _store_States__WEBPACK_IMPORTED_MODULE_3__.states.busy = false;
    }
}
const removeWorksTagsInBookmarks = new RemoveWorksTagsInBookmarks();



/***/ }),

/***/ "./src/ts/ReplaceSquareThumb.ts":
/*!**************************************!*\
  !*** ./src/ts/ReplaceSquareThumb.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");




class ReplaceSquareThumb {
    constructor() {
        this.bindEvents();
        this.observer();
    }
    isDisable() {
        return window.location.pathname.startsWith('/group');
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'replaceSquareThumb') {
                if (data.value) {
                    this.replaceAllImage();
                }
            }
        });
    }
    replaceAllImage() {
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.replaceSquareThumb ||
            _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type == _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkRanking) {
            const allImage = document.querySelectorAll('img');
            allImage.forEach((img) => this.replace(img));
        }
    }
    replace(img) {
        if (!img.src || img.dataset.index || this.isDisable()) {
            return;
        }
        const src = img.src;
        if (!src.endsWith('square1200.jpg') && !src.endsWith('custom1200.jpg')) {
            if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkRanking &&
                _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.showLargerThumbnails) {
                // 排行榜里的缩略图本来就是保持了比例的，不需要替换其缩略图。
                // 但是排行榜里的缩略图原本尺寸较小，当用户启用了“显示更大的缩略图”之后，缩略图被放大后显得模糊，此时需要替换成更大尺寸的缩略图。
                // 排行榜页面的图片 URL 比较特别，末尾是 master1200，如下：
                // 'https://i.pximg.net/c/240x480/img-master/img/2022/08/01/17/59/39/100156836_p0_master1200.jpg'
                if (!src.includes('240x480')) {
                    return;
                }
            }
            else {
                return;
            }
        }
        img.src = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.convertThumbURLTo540px(src);
        img.style.objectFit = 'contain';
    }
    observer() {
        const observer = new MutationObserver((records) => {
            if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.replaceSquareThumb) {
                if (_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkRanking) {
                    // 在排行榜页面里，即使用户未启用“替换方形缩略图以显示图片比例”功能，也依然执行替换缩略图的动作
                    return;
                }
            }
            records.forEach((record) => {
                if (record.type === 'childList') {
                    record.addedNodes.forEach((node) => {
                        if (node.nodeName === 'IMG') {
                            this.replace(node);
                        }
                    });
                }
                if (record.type === 'attributes') {
                    if (record.attributeName === 'src' &&
                        record.target.nodeName === 'IMG') {
                        this.replace(record.target);
                    }
                }
            });
        });
        observer.observe(document.body, {
            childList: true,
            subtree: true,
            attributeFilter: ['src'],
        });
    }
}
new ReplaceSquareThumb();


/***/ }),

/***/ "./src/ts/RequestSponsorship.ts":
/*!**************************************!*\
  !*** ./src/ts/RequestSponsorship.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");




class RequestSponsorship {
    constructor() {
        // 30 * 24 * 60 * 60 * 1000
        this.interval = 2592000000;
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingInitialized, () => {
            // 赋予初始值
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.requestSponsorshipTime === 0) {
                (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('requestSponsorshipTime', new Date().getTime() + this.interval);
            }
            window.setTimeout(() => {
                this.check();
            }, 10000);
        });
    }
    check() {
        const now = new Date().getTime();
        if (now >= _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.requestSponsorshipTime) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.once('request sponsorship', _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_赞助方式提示'), 'show', {
                title: _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_赞助我'),
            });
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('requestSponsorshipTime', now + this.interval);
        }
    }
}
new RequestSponsorship();


/***/ }),

/***/ "./src/ts/SelectWork.ts":
/*!******************************!*\
  !*** ./src/ts/SelectWork.ts ***!
  \******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _NovelThumbnail__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./NovelThumbnail */ "./src/ts/NovelThumbnail.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");













// 手动选择作品，图片作品和小说都可以选择
class SelectWork {
    constructor() {
        this.created = false;
        this.selectorId = 'selectWorkEl';
        this.left = 0;
        this.top = 0;
        this.half = 10; // 指示器的一半宽度（用于设置位置）
        this._start = false;
        this._pause = false;
        this._tempHide = false; // 打开下载面板时临时隐藏。这个变量只会影响选择器的 display
        this.controlBtn = document.createElement('button'); // 启动、暂停、继续选择的按钮
        this.crawlBtn = document.createElement('button'); // 抓取选择的作品的按钮，并且会退出选择模式
        this.clearBtn = document.createElement('button'); // 清空选择的作品的按钮
        this.selectedWorkFlagClass = 'selectedWorkFlag'; // 给已选择的作品添加标记时使用的 class
        this.positionValue = ['relative', 'absolute', 'fixed']; // 标记元素需要父元素拥有这些定位属性
        // 不同页面里的作品列表容器的选择器可能不同，这里储存所有页面里会使用到的的选择器
        // root 是大部分页面通用的; js-mount-point-discovery 是发现页面使用的
        this.worksWrapperSelectorList = [
            '#root',
            '#js-mount-point-discovery',
        ];
        // 储存当前页面使用的选择器
        this.usedWorksWrapperSelector = this.worksWrapperSelectorList[0];
        // 储存当前页面的作品列表容器
        this.worksWrapper = document.body;
        this.ob = undefined;
        this.idList = [];
        this.sendCrawl = false; // 它用来判断抓取的是不是选择的作品。抓取选择的作品时激活此标记；当触发下一次的抓取完成事件时，表示已经抓取了选择的作品。
        this.crawled = false; // 是否已经抓取了选择的作品
        this.svg = `<svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-select"></use>
</svg>`;
        const unlisted = _PageType__WEBPACK_IMPORTED_MODULE_10__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_10__.pageType.list.Unlisted;
        if (!this.created && _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.isPixiv() && !unlisted) {
            this.created = true;
            this.selector = this.createSelectorEl();
            this.addBtn();
            this.bindEvents();
        }
    }
    get start() {
        return this._start;
    }
    set start(bool) {
        this._start = bool;
        _store_States__WEBPACK_IMPORTED_MODULE_4__.states.selectWork = bool;
        this.updateSelectorEl();
        this.updateControlBtn();
    }
    get pause() {
        return this._pause;
    }
    set pause(bool) {
        this._pause = bool;
        if (bool) {
            _store_States__WEBPACK_IMPORTED_MODULE_4__.states.selectWork = false;
        }
        this.updateSelectorEl();
        this.updateControlBtn();
    }
    get tempHide() {
        return this._tempHide;
    }
    set tempHide(bool) {
        this._tempHide = bool;
        this.updateSelectorEl();
    }
    bindEvents() {
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_8__.artworkThumbnail.onClick((el, id, ev) => {
            this.clickThumbnail(el, id, ev, 'illusts');
        });
        _NovelThumbnail__WEBPACK_IMPORTED_MODULE_9__.novelThumbnail.onClick((el, id, ev) => {
            this.clickThumbnail(el, id, ev, 'novels');
        });
        document.body.addEventListener(_Config__WEBPACK_IMPORTED_MODULE_12__.Config.mobile ? 'touchend' : 'click', (ev) => {
            this.clickElement(ev.target, ev);
        }, true);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.openCenterPanel, () => {
            this.tempHide = true;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.closeCenterPanel, () => {
            this.tempHide = false;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.crawlComplete, () => {
            if (this.sendCrawl) {
                this.sendCrawl = false;
                this.crawled = true;
            }
        });
        // 可以使用 Alt + S 快捷键来模拟点击控制按钮
        window.addEventListener('keydown', (ev) => {
            if (ev.altKey && ev.code === 'KeyS') {
                this.controlBtn.click();
            }
        });
        // 鼠标移动时保存鼠标的坐标
        window.addEventListener('mousemove', (ev) => {
            this.moveEvent(ev);
        }, true);
        // 离开页面前提示用户
        // 如果把此处的 window.onbeforeunload 换成 window.addEventListener('beforeunload') 会出现问题
        // 浏览器不会弹出询问对话框，而是直接关闭页面
        window.onbeforeunload = () => {
            // 如果存在选择的作品，并且选择的作品（全部或部分）没有被抓取，则进行提示
            if (this.idList.length > 0 && !this.crawled) {
                _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_离开页面前提示选择的作品未抓取'), {
                    btn: _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_我知道了'),
                });
                return false;
            }
        };
        // 每次页面切换之后，查找新的作品列表容器并保存
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.pageSwitch, () => {
            let worksWrapper = null;
            for (const selector of this.worksWrapperSelectorList) {
                worksWrapper = document.querySelector(selector);
                if (worksWrapper) {
                    this.usedWorksWrapperSelector = selector;
                    break;
                }
            }
            this.worksWrapper = worksWrapper || document.body;
        });
        // 每次页面切换之后，查找新显示的作品里是否有之前被选择的作品，如果有则为其添加标记
        // 因为 pixiv 的页面切换会导致作品列表变化，之前添加的标记也就没有了，需要重新添加
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.pageSwitch, () => {
            // 每次触发时都要断开之前绑定的观察器，否则会导致事件重复绑定
            // 因为 pageSwitch 事件可能会触发多次，如果不断开之前的观察器，那么每切换一次页面就会多绑定和执行一个回调
            this.ob && this.ob.disconnect();
            this.ob = new MutationObserver(_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.debounce(() => {
                this.reAddAllFlag();
            }, 300));
            this.ob.observe(this.worksWrapper, {
                childList: true,
                subtree: true,
            });
        });
    }
    clearIdList() {
        // 清空标记需要使用 id 数据，所以需要执行之后才能清空 id
        this.removeAllSelectedFlag();
        this.idList = [];
        this.updateCrawlBtn();
    }
    createSelectorEl() {
        const el = document.createElement('div');
        el.id = this.selectorId;
        document.body.appendChild(el);
        return el;
    }
    updateSelectorEl() {
        if (!this.selector) {
            return;
        }
        const show = this.canSelect() && !this.tempHide;
        this.selector.style.display = show ? 'block' : 'none';
        // 设置元素的 style 时，如果新的值和旧的值相同（例如：每次都设置 display 为 none），Chrome 会自动优化，此时不会导致节点发生变化。
        // 如果选择器处于隐藏状态，就不会更新其坐标。这样可以优化性能
        if (show) {
            this.selector.style.left = this.left - this.half + 'px';
            this.selector.style.top = this.top - this.half + 'px';
        }
    }
    addBtn() {
        this.controlBtn = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.addBtn('selectWorkBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_手动选择作品');
        this.controlBtn.setAttribute('title', 'Alt + S');
        this.updateControlBtn();
        this.clearBtn = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.addBtn('selectWorkBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgRed, '_清空选择的作品');
        this.clearBtn.style.display = 'none';
        this.clearBtn.addEventListener('click', () => {
            this.clearIdList();
            this.clearBtn.style.display = 'none';
            this.crawlBtn.style.display = 'none';
        });
        this.crawlBtn = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.addBtn('selectWorkBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取选择的作品');
        this.crawlBtn.style.display = 'none';
        this.crawlBtn.addEventListener('click', (ev) => {
            this.sendDownload();
        });
    }
    // 切换控制按钮的文字和点击事件
    updateControlBtn() {
        if (!this.start) {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.controlBtn, '_手动选择作品');
            this.controlBtn.onclick = (ev) => {
                this.startSelect(ev);
                this.clearBtn.style.display = 'block';
                if (!_Config__WEBPACK_IMPORTED_MODULE_12__.Config.mobile) {
                    _ShowHelp__WEBPACK_IMPORTED_MODULE_11__.showHelp.show('tipAltSToSelectWork', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_快捷键ALTS手动选择作品'));
                }
            };
        }
        else {
            if (!this.pause) {
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.controlBtn, '_暂停选择');
                this.controlBtn.onclick = (ev) => {
                    this.pauseSelect();
                };
            }
            else {
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.controlBtn, '_继续选择');
                this.controlBtn.onclick = (ev) => {
                    this.startSelect(ev);
                };
            }
        }
    }
    // 在选择作品的数量改变时，在抓取按钮上显示作品数量
    updateCrawlBtn() {
        this.crawlBtn.style.display = this.start ? 'block' : 'none';
        if (this.idList.length > 0) {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.crawlBtn, '_抓取选择的作品2', this.idList.length.toString());
            this.clearBtn.style.display = 'block';
        }
        else {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.crawlBtn, '_抓取选择的作品');
        }
    }
    addId(el, id, type) {
        const index = this.idList.findIndex((item) => {
            return item.id === id && item.type === type;
        });
        // 添加这个 id
        if (index === -1) {
            this.idList.push({
                id,
                type,
            });
            this.crawled = false;
            this.addSelectedFlag(el, id);
        }
        else {
            // id 已存在，则删除
            this.idList.splice(index, 1);
            this.removeSelectedFlag(id);
        }
        this.updateCrawlBtn();
    }
    clickThumbnail(el, id, ev, type) {
        if (!this.canSelect()) {
            return;
        }
        // 如果点击的元素是作品缩略图里的收藏按钮，则不选择这个作品，这样可以让收藏按钮发挥作用
        // 注意这些 nodeName 是小写的
        const target = ev.target;
        if (target && (target.nodeName === 'svg' || target.nodeName === 'path')) {
            return;
        }
        // 真实点击的元素
        // console.log(ev.target)
        // 绑定了这个事件的元素
        // console.log(ev.currentTarget)
        if (!id || id === '0') {
            id = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.findWorkIdFromElement(el, type === 'novels' ? 'novels' : 'illusts');
        }
        // 阻止默认事件，否则会进入作品页面，导致无法在当前页面继续选择
        ev.preventDefault();
        ev.stopPropagation();
        this.addId(el, id, type);
    }
    clickElement(el, ev) {
        if (!this.canSelect()) {
            return;
        }
        if (!el || el.nodeName !== 'A') {
            return;
        }
        const href = el.href;
        const artworkId = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.getIllustId(href);
        if (artworkId) {
            ev.preventDefault();
            // 如果查找到了作品 id，必须阻止冒泡，否则会执行 clickThumbnail
            ev.stopPropagation();
            this.addId(el.parentElement, artworkId, 'illusts');
            return;
        }
        const novelId = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.getNovelId(href);
        if (novelId) {
            ev.preventDefault();
            ev.stopPropagation();
            this.addId(el.parentElement, novelId, 'novels');
            return;
        }
    }
    // 监听鼠标移动
    // 鼠标移动时，由于事件触发频率很高，所以这里的代码也会执行很多次，但是这没有导致明显的性能问题，所以没有使用节流等加以限制
    moveEvent(ev) {
        this.left = ev.x;
        this.top = ev.y;
        this.updateSelectorEl();
    }
    // 按 Esc 键时暂停选择
    escEvent(ev) {
        if (ev.code === 'Escape') {
            this.pauseSelect();
        }
    }
    // 开始或继续选择
    startSelect(ev) {
        this.start = true;
        if (this.pause) {
            // 如果之前暂停了，则继续选择。不清空之前的结果
            this.pause = false;
        }
        else {
            // 如果是全新开始的选择，则清空之前的结果
            this.clearIdList();
        }
        this.bindEscEvent = this.escEvent.bind(this);
        document.addEventListener('keyup', this.bindEscEvent);
        _EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.fire('closeCenterPanel');
    }
    pauseSelect() {
        this.pause = true;
        this.bindEscEvent &&
            document.removeEventListener('keyup', this.bindEscEvent);
    }
    canSelect() {
        return this.start && !this.pause;
    }
    // 抓取选择的作品，这会自动暂停手动选择作品
    sendDownload() {
        this.pauseSelect();
        if (this.idList.length > 0) {
            // 传递 id 列表时，将其转换成一个新的数组。否则传递的是引用，外部的一些操作可能会影响内部的 id 列表
            _EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.fire('crawlIdList', Array.from(this.idList));
            this.sendCrawl = true;
            this.crawled = false;
            _store_States__WEBPACK_IMPORTED_MODULE_4__.states.quickCrawl = true;
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已发送下载请求'), {
                bgColor: _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue,
            });
        }
        else {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有数据可供使用'));
        }
    }
    // 给这个作品添加标记
    addSelectedFlag(wrap, id) {
        const i = document.createElement('i');
        i.classList.add(this.selectedWorkFlagClass);
        i.dataset.id = id;
        i.innerHTML = this.svg;
        wrap.insertAdjacentElement('afterbegin', i);
        // 如果容器没有某些定位，可能会导致下载器添加的标记的位置异常。修复此问题
        const position = window.getComputedStyle(wrap)['position'];
        if (!this.positionValue.includes(position)) {
            wrap.style.position = 'relative';
        }
    }
    // 重新添加被选择的作品上的标记
    reAddAllFlag() {
        if (this.idList.length === 0) {
            return;
        }
        for (const { id, type } of this.idList) {
            if (this.getSelectedFlag(id)) {
                // 如果这个作品的标记依旧存在，就不需要重新添加
                /**
                 * 示例：从作品列表 https://www.pixiv.net/users/18095070/illustrations
                 * 进入 tag 列表页 https://www.pixiv.net/users/18095070/illustrations/%E5%A5%B3%E3%81%AE%E5%AD%90
                 * pixiv 会复用可用的作品，所以这些作品上的标记也依然存在，不需要重新添加
                 */
                return;
            }
            let el;
            if (type === 'novels') {
                el = document.querySelector(`${this.usedWorksWrapperSelector} a[href="/novel/show.php?id=${id}"]`);
            }
            else {
                el = document.querySelector(`${this.usedWorksWrapperSelector} a[href="/artworks/${id}"]`);
            }
            if (el) {
                // 如果在当前页面查找到了选择的作品，就给它添加标记
                this.addSelectedFlag(el, id);
            }
        }
    }
    getSelectedFlag(id) {
        return document.querySelector(`.${this.selectedWorkFlagClass}[data-id='${id}']`);
    }
    // 清空指定作品的标记
    removeSelectedFlag(id) {
        const el = this.getSelectedFlag(id);
        el && el.remove();
    }
    // 清空所有标记
    removeAllSelectedFlag() {
        for (const item of this.idList) {
            this.removeSelectedFlag(item.id);
        }
    }
}
new SelectWork();


/***/ }),

/***/ "./src/ts/SetTimeoutWorker.ts":
/*!************************************!*\
  !*** ./src/ts/SetTimeoutWorker.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   setTimeoutWorker: () => (/* binding */ setTimeoutWorker)
/* harmony export */ });
class SetTimeoutWorker {
    constructor() {
        // 因为 worker 的代码很短，所以直接储存在这里，避免从网络加载导致的延迟问题
        this.workerCode = `onmessage = (ev) => {
    setTimeout(() => {
      postMessage({
        id: ev.data.id
      })
    }, ev.data.time)
  }`;
        this.list = [];
        this.timerId = 0;
        this.createWorker();
    }
    createWorker() {
        const blob = new Blob([this.workerCode]);
        this.worker = new Worker(URL.createObjectURL(blob));
        this.worker.addEventListener('message', (ev) => {
            const id = ev.data.id;
            if (this.list[id].callback !== null) {
                this.list[id].callback();
                this.clear(id);
            }
        });
    }
    set(callback, time) {
        const data = {
            id: this.timerId,
            time,
            callback,
        };
        this.list.push(data);
        this.timerId++;
        this.worker.postMessage({
            id: data.id,
            time,
        });
        return data.id;
    }
    clear(id) {
        this.list[id].callback = null;
    }
}
const setTimeoutWorker = new SetTimeoutWorker();



/***/ }),

/***/ "./src/ts/SetUserName.ts":
/*!*******************************!*\
  !*** ./src/ts/SetUserName.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");







// 为某些用户设置固定的用户名，或者别名
class SetUserName {
    constructor() {
        this.slotName = 'setUserNameSlot';
        this._addWrapShow = false;
        this.wrapHTML = `
  <span class="setUserNameWrap">

    <span class="controlBar">
    <span class="total">0</span>
      <button type="button" class="textButton expand" data-xztext="_收起"></button>
      <button type="button" class="textButton showAdd" data-xztext="_添加"></button>
    </span>

    <div class="addWrap">
      <div class="settingItem addInputWrap" >
        <div class="inputItem uid">
          <span class="label uidLabel" data-xztext="_用户id"></span>
          <input type="text" class="setinput_style1 blue addUidInput" data-xzplaceholder="_必须是数字" />
        </div>

        <div class="inputItem name">
          <span class="label nameLabel" data-xztext="_命名标记user"></span>
          <input type="text" class="setinput_style1 blue addNameInput" />
        </div>

        <div class="btns">
          <button type="button" class="textButton add" data-xztitle="_添加">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-wanchengqueding"></use>
            </svg>
          </button>

          
          <button type="button" class="textButton cancel" data-xztitle="_取消">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-guanbiquxiao"></use>
            </svg>
          </button>
        </div>
      </div>
    </div>

    <div class="listWrap">
    </div>
  </span>
  `;
        this.createWrap();
        _Theme__WEBPACK_IMPORTED_MODULE_4__.theme.register(this.wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.wrap);
        this.bindEvents();
    }
    set addWrapShow(val) {
        this._addWrapShow = val;
        if (val) {
            this.addWrap.style.display = 'block';
            this.addInputUid.focus();
        }
        else {
            this.addWrap.style.display = 'none';
            this.addInputUid.value = '';
            this.addInputName.value = '';
        }
    }
    get addWrapShow() {
        return this._addWrapShow;
    }
    // 创建列表外部的容器，静态 html
    createWrap() {
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.useSlot(this.slotName, this.wrapHTML);
        this.expandBtn = this.wrap.querySelector('.expand');
        this.showAddBtn = this.wrap.querySelector('.showAdd');
        this.totalSpan = this.wrap.querySelector('.total');
        this.addWrap = this.wrap.querySelector('.addWrap');
        this.addInputUid = this.wrap.querySelector('.addUidInput');
        this.addInputName = this.wrap.querySelector('.addNameInput');
        this.addBtn = this.wrap.querySelector('.add');
        this.cancelBtn = this.wrap.querySelector('.cancel');
        this.listWrap = this.wrap.querySelector('.listWrap');
        // 展开/折叠按钮
        this.expandBtn.addEventListener('click', () => {
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('setUserNameShow', !_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameShow);
        });
        // 切换显示添加规则的区域
        this.showAddBtn.addEventListener('click', () => {
            this.addWrapShow = !this.addWrapShow;
        });
        // 添加规则的按钮
        this.addBtn.addEventListener('click', () => {
            this.addRule(this.addInputUid.value, this.addInputName.value);
        });
        // 取消添加的按钮
        this.cancelBtn.addEventListener('click', () => {
            this.addWrapShow = false;
        });
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'setUserNameShow') {
                this.showListWrap();
            }
            if (data.name === 'setUserNameList') {
                this.createAllList();
            }
        });
    }
    showListWrap() {
        const show = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameShow;
        this.listWrap.style.display = show ? 'flex' : 'none';
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.expandBtn, show ? '_收起' : '_展开');
    }
    // 根据规则动态创建 html
    createAllList() {
        this.totalSpan.textContent = Object.keys(_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList).length.toString();
        this.listWrap.innerHTML = '';
        const df = document.createDocumentFragment();
        for (const [uid, name] of Object.entries(_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList)) {
            df.append(this.createOneList(uid, name));
        }
        this.listWrap.append(df);
    }
    // 创建规则对应的元素，并绑定事件
    createOneList(uid, name) {
        const html = `
      <div class="inputItem uid">
        <input type="text" class="setinput_style1 blue" data-uidInput="${uid}" value="${uid}" />
      </div>

      <div class="inputItem name">
        <input type="text" class="setinput_style1 blue" data-nameInput="${uid}" value="${name}" />
      </div>

      <div class="btns">
        <button type="button" class="textButton" data-updateRule="${uid}" data-xztitle="_更新">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-gengxin"></use>
          </svg>
        </button>

        <button type="button" class="textButton" data-deleteRule="${uid}" data-xztitle="_删除">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-shanchu1"></use>
          </svg>
        </button>
    </div>`;
        const element = document.createElement('div');
        element.classList.add('settingItem');
        element.dataset.key = uid;
        element.innerHTML = html;
        const updateRule = element.querySelector(`button[data-updateRule='${uid}']`);
        const deleteRule = element.querySelector(`button[data-deleteRule='${uid}']`);
        const uidInput = element.querySelector(`input[data-uidInput='${uid}']`);
        const nameInput = element.querySelector(`input[data-nameInput='${uid}']`);
        [uidInput, nameInput].forEach((el) => {
            el?.addEventListener('change', () => {
                if (el.value) {
                    this.updateRule(uid, uidInput.value, nameInput.value, false);
                }
            });
        });
        // 更新规则
        updateRule?.addEventListener('click', () => {
            this.updateRule(uid, uidInput.value, nameInput.value);
        });
        // 删除规则
        deleteRule?.addEventListener('click', () => {
            this.deleteRule(uid);
        });
        return element;
    }
    // 检查用户输入的值
    checkValue(uidInput, nameInput) {
        if (!uidInput || !nameInput) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_必填项不能为空'));
            return false;
        }
        const uid = Number.parseInt(uidInput);
        if (!uid || isNaN(uid)) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_用户ID必须是数字'));
            return false;
        }
        return {
            uidInput,
            nameInput,
        };
    }
    // 添加规则
    addRule(uid, name) {
        const check = this.checkValue(uid, name);
        if (!check) {
            return;
        }
        _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList[uid] = name;
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('setUserNameList', _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList);
        this.addWrapShow = false;
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_添加成功'));
    }
    // 更新规则
    // tip 表示是否用显示操作成功的提示。当用户点击了更新按钮时应该显示提示；输入内容变化导致的自动更新可以不显示提示
    updateRule(oldUid, uid, name, tip = true) {
        const check = this.checkValue(uid, name);
        if (!check) {
            return;
        }
        delete _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList[oldUid];
        _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList[uid] = name;
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('setUserNameList', _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList);
        this.addWrapShow = false;
        if (tip) {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_更新成功'));
        }
    }
    // 删除规则
    deleteRule(uid) {
        delete _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList[uid];
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('setUserNameList', _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.setUserNameList);
        this.removeListElement(uid);
    }
    removeListElement(uid) {
        const listElement = this.listWrap.querySelector(`.settingItem[data-key='${uid}']`);
        listElement?.remove();
    }
}
new SetUserName();


/***/ }),

/***/ "./src/ts/ShowDownloadBtnOnThumb.ts":
/*!******************************************!*\
  !*** ./src/ts/ShowDownloadBtnOnThumb.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _ShowDownloadBtnOnThumbOnDesktop__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./ShowDownloadBtnOnThumbOnDesktop */ "./src/ts/ShowDownloadBtnOnThumbOnDesktop.ts");
/* harmony import */ var _ShowDownloadBtnOnThumbOnMobile__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ShowDownloadBtnOnThumbOnMobile */ "./src/ts/ShowDownloadBtnOnThumbOnMobile.ts");



// 在图片作品的缩略图上显示下载按钮，点击按钮可以直接下载这个作品
class ShowDownloadBtnOnThumb {
    constructor() {
        _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile
            ? new _ShowDownloadBtnOnThumbOnMobile__WEBPACK_IMPORTED_MODULE_2__.ShowDownloadBtnOnThumbOnMobile()
            : new _ShowDownloadBtnOnThumbOnDesktop__WEBPACK_IMPORTED_MODULE_1__.ShowDownloadBtnOnThumbOnDesktop();
    }
}
new ShowDownloadBtnOnThumb();


/***/ }),

/***/ "./src/ts/ShowDownloadBtnOnThumbOnDesktop.ts":
/*!***************************************************!*\
  !*** ./src/ts/ShowDownloadBtnOnThumbOnDesktop.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ShowDownloadBtnOnThumbOnDesktop: () => (/* binding */ ShowDownloadBtnOnThumbOnDesktop)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");








// 在图片作品的缩略图上显示下载按钮，点击按钮会直接下载这个作品
class ShowDownloadBtnOnThumbOnDesktop {
    constructor() {
        this.btnId = 'downloadBtnOnThumb';
        this.btnSize = 32;
        this.currentWorkId = ''; // 保存触发事件的缩略图的作品 id
        this.hiddenBtnTimer = 0; // 使用定时器让按钮延迟消失。这是为了解决一些情况下按钮闪烁的问题
        this.hiddenBtnDelay = 100;
        this.doNotShowBtn = false; // 当点击了按钮后，进入此状态，此状态中不会显示按钮
        if (_Config__WEBPACK_IMPORTED_MODULE_7__.Config.mobile) {
            return;
        }
        // 在桌面端，只有一个下载按钮，当鼠标经过作品缩略图时才会显示下载按钮
        this.addBtn();
        this.bindEvents();
    }
    // 此状态是为了解决这个问题：点击了按钮之后，按钮会被隐藏，隐藏之后，鼠标下方就是图片缩略图区域，这会触发缩略图的鼠标事件，导致按钮马上就又显示了出来。所以点击按钮之后设置这个状态，在其为 true 的期间不会显示按钮。过一段时间再把它复位。复位所需的时间很短，因为只要能覆盖这段时间就可以了：从隐藏按钮开始算起，到缩略图触发鼠标事件结束。
    addBtn() {
        const btn = document.createElement('button');
        btn.id = this.btnId;
        btn.classList.add(this.btnId);
        btn.innerHTML = `
    <svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-download"></use>
</svg>`;
        this.btn = document.body.appendChild(btn);
    }
    bindEvents() {
        // 页面切换时隐藏按钮
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitch, () => {
            this.hiddenBtn();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clickBtnOnThumb, () => {
            this.hiddenBtnNow();
        });
        // 鼠标移入按钮时取消隐藏按钮
        this.btn.addEventListener('mouseenter', (ev) => {
            window.clearTimeout(this.hiddenBtnTimer);
        });
        // 鼠标移出按钮时隐藏按钮
        this.btn.addEventListener('mouseleave', () => {
            this.hiddenBtn();
        });
        // 点击按钮时发送下载任务
        this.btn.addEventListener('click', (ev) => {
            this.hiddenBtnNow();
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('clickBtnOnThumb');
            if (this.currentWorkId) {
                const IDData = {
                    type: 'illusts',
                    id: this.currentWorkId,
                };
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', [IDData]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_3__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_4__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_5__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_6__.Colors.bgBlue,
                });
            }
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__.artworkThumbnail.onEnter((el, id) => {
            this.currentWorkId = id;
            this.showBtn(el);
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__.artworkThumbnail.onLeave(() => {
            this.hiddenBtn();
        });
    }
    // 显示按钮
    showBtn(target) {
        if (this.doNotShowBtn || !_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showDownloadBtnOnThumb) {
            return;
        }
        window.clearTimeout(this.hiddenBtnTimer);
        const rect = target.getBoundingClientRect();
        this.btn.style.left =
            window.scrollX +
                rect.left +
                (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.magnifierPosition === 'left' ? 0 : rect.width - this.btnSize) +
                'px';
        let top = window.scrollY + rect.top;
        // 如果显示了放大按钮，就需要加大 top，让下载按钮显示在放大按钮下面
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.magnifier) {
            top = top + this.btnSize + 8;
        }
        this.btn.style.top = top + 'px';
        this.btn.style.display = 'flex';
    }
    // 延迟隐藏按钮
    hiddenBtn() {
        window.clearTimeout(this.hiddenBtnTimer);
        this.hiddenBtnTimer = window.setTimeout(() => {
            this.btn.style.display = 'none';
        }, this.hiddenBtnDelay);
    }
    // 立刻隐藏按钮
    hiddenBtnNow() {
        this.doNotShowBtn = true;
        window.setTimeout(() => {
            this.doNotShowBtn = false;
        }, 100);
        window.clearTimeout(this.hiddenBtnTimer);
        this.btn.style.display = 'none';
    }
}



/***/ }),

/***/ "./src/ts/ShowDownloadBtnOnThumbOnMobile.ts":
/*!**************************************************!*\
  !*** ./src/ts/ShowDownloadBtnOnThumbOnMobile.ts ***!
  \**************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ShowDownloadBtnOnThumbOnMobile: () => (/* binding */ ShowDownloadBtnOnThumbOnMobile)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");








// 在图片作品的缩略图上显示下载按钮，点击按钮会直接下载这个作品
class ShowDownloadBtnOnThumbOnMobile {
    constructor() {
        this.className = 'downloadBtnOnThumb';
        this.size = 32;
        // 在移动端，由于没有 mouseover 事件，
        // 所以只能每个作品缩略图分别添加一个下载按钮
        this.bindEvents();
    }
    bindEvents() {
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_2__.artworkThumbnail.onFound((el, id) => {
            if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showDownloadBtnOnThumb) {
                return;
            }
            const btn = this.addBtn(el);
            btn.addEventListener('click', (ev) => {
                if (!id) {
                    id = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.findWorkIdFromElement(el, 'illusts');
                }
                if (!id) {
                    return;
                }
                const IDData = {
                    type: 'illusts',
                    id: id,
                };
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', [IDData]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_3__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_4__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_5__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_6__.Colors.bgBlue,
                });
            });
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'showDownloadBtnOnThumb') {
                this.toggleShowBtns(data.value);
                this.setPageCountStyle(data.value);
            }
        });
    }
    addBtn(target) {
        const btn = document.createElement('button');
        btn.classList.add(this.className);
        btn.innerHTML = `
    <svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-download"></use>
</svg>`;
        btn.style.left = 'auto';
        btn.style.right = '0px';
        btn.style.top = '0px';
        btn.style.display = 'flex';
        target.appendChild(btn);
        return btn;
    }
    setPageCountStyle(value) {
        // 显示按钮时，让缩略图的页数文字下移到按钮下面，否则页数会被按钮遮挡
        if (value && !this.styleElement) {
            this.styleElement = document.createElement('style');
            this.styleElement.innerText = `.status-page-count-container {margin-top: ${this.size}px;}`;
            document.body.append(this.styleElement);
        }
        if (!value && this.styleElement) {
            this.styleElement.remove();
            this.styleElement = undefined;
        }
    }
    toggleShowBtns(value) {
        const btns = document.body.querySelectorAll(`.${this.className}`);
        for (const btn of btns) {
            ;
            btn.style.display = value ? 'flex' : 'none';
        }
    }
}



/***/ }),

/***/ "./src/ts/ShowHelp.ts":
/*!****************************!*\
  !*** ./src/ts/ShowHelp.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   showHelp: () => (/* binding */ showHelp)
/* harmony export */ });
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");




// 用消息框显示一次性的提示
class ShowHelp {
    show(settingKey, msg) {
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings[settingKey] === true) {
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)(settingKey, false);
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.show(msg, {
                title: _Config__WEBPACK_IMPORTED_MODULE_1__.Config.appName + ' Help',
                btn: _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_我知道了'),
            });
        }
    }
}
const showHelp = new ShowHelp();



/***/ }),

/***/ "./src/ts/ShowLargerThumbnails.ts":
/*!****************************************!*\
  !*** ./src/ts/ShowLargerThumbnails.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");





class ShowLargerThumbnails {
    constructor() {
        // css 内容来自 style/showLargerThumbnails.css
        this.css = '';
        this.styleId = 'ShowLargerThumbnails';
        this.findFriendsWrap = false;
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            return;
        }
        this.loadCssText();
        this.bindEvents();
        this.findFriendsWrapEl();
    }
    async loadCssText() {
        const css = await fetch(chrome.runtime.getURL('style/showLargerThumbnails.css'));
        this.css = await css.text();
        this.setCss();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'showLargerThumbnails') {
                this.setCss();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch, () => {
            this.setCss();
            this.findFriendsWrap = false;
        });
        window.setInterval(() => {
            this.findFriendsWrapEl();
        }, 1000);
    }
    setCss() {
        if (!this.css) {
            return;
        }
        if (_Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.notEnabledShowLargerThumb()) {
            return this.removeStyle();
        }
        _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showLargerThumbnails ? this.addStyle() : this.removeStyle();
    }
    addStyle() {
        if (document.querySelector('#' + this.styleId)) {
            return;
        }
        const el = document.createElement('style');
        el.id = this.styleId;
        el.innerHTML = this.css;
        document.body.append(el);
    }
    removeStyle() {
        const el = document.querySelector('#' + this.styleId);
        el && el.remove();
    }
    // 在首页查找“关注用户・好P友的作品”列表容器，为其添加自定义的 className
    findFriendsWrapEl() {
        if (this.findFriendsWrap || _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Home) {
            return;
        }
        const sectionList = document.querySelectorAll('section');
        if (sectionList && sectionList[1]) {
            if (sectionList[1].querySelector('ul div')) {
                sectionList[1].classList.add('homeFriendsNewWorks');
                this.findFriendsWrap = true;
            }
        }
    }
}
new ShowLargerThumbnails();


/***/ }),

/***/ "./src/ts/ShowNotification.ts":
/*!************************************!*\
  !*** ./src/ts/ShowNotification.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Tools */ "./src/ts/Tools.ts");






class ShowNotification {
    constructor() {
        this.iconURL = '';
        this.iconURL = chrome.runtime.getURL('icon/logo128.png');
        this.bindEvents();
    }
    bindEvents() {
        // 当用户开启“下载完成后显示通知”的提示时，请求权限
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'showNotificationAfterDownloadComplete' && data.value) {
                this.requstPremission();
            }
        });
        // 当下载任务完毕时，显示通知
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete, () => {
            window.setTimeout(() => {
                // 如果抓取标签列表没有完成，则不显示通知
                // 在一次抓取多个标签时，当最后一个标签下载完之后会解除 crawlTagList 状态，这时可以显示一条通知
                // 如果有等待下载的任务，则不显示通知
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.showNotificationAfterDownloadComplete &&
                    !_store_States__WEBPACK_IMPORTED_MODULE_3__.states.crawlTagList &&
                    _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.waitingIdList.length === 0) {
                    this.show(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_下载完毕2'), _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getPageTitle());
                }
            }, 0);
        });
    }
    async show(title, text) {
        await this.requstPremission();
        new Notification(title, {
            body: text,
            // 不设置 tag。如果设置了相同的 tag，那么新的通知会覆盖旧的通知，导致如果有多个页面下载完毕，用户只能看到最后一个页面的通知
            // tag: 'PowerfulPixivDownloader',
            icon: this.iconURL,
        });
    }
    requstPremission() {
        if (Notification.permission !== 'granted') {
            return Notification.requestPermission();
        }
    }
}
new ShowNotification();


/***/ }),

/***/ "./src/ts/ShowOriginSizeImage.ts":
/*!***************************************!*\
  !*** ./src/ts/ShowOriginSizeImage.ts ***!
  \***************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   showOriginSizeImage: () => (/* binding */ showOriginSizeImage)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _PreviewUgoira__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./PreviewUgoira */ "./src/ts/PreviewUgoira.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");












class ShowOriginSizeImage {
    constructor() {
        this.urls = {
            original: '',
            regular: '',
        };
        // 显示作品中的第几张图片
        this.index = 0;
        // 原比例查看图片的容器的元素
        this.wrapId = 'originSizeWrap';
        this.defaultSize = 1200;
        this.border = 4; // wrap 的 border 占据的空间
        // 不可以把 left、top 设置为负值，否则超出屏幕的区域无法查看
        // 所以通过修改 margin 来达到定位的效果
        this.style = {
            imgW: this.defaultSize,
            imgH: this.defaultSize,
            width: this.defaultSize,
            height: this.defaultSize,
            mt: 0,
            ml: 0,
        };
        this.defaultStyle = _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.deepCopy(this.style);
        this.zoomList = [
            0.1, 0.2, 0.3, 0.4, 0.5, 0.75, 1, 1.5, 2, 2.5, 3, 3.5, 4, 5,
        ];
        this.zoomIndex = 6;
        // 默认的缩放比例为 1
        this.zoom = this.zoomList[this.zoomIndex];
        this.testImg = new Image();
        this.getImageSizeTimer = 0;
        // 定义当鼠标移动 1 像素时，wrap 移动多少像素
        this.onePxMove = 10;
        this.moveX = 0;
        this.moveY = 0;
        this._show = false;
        this.showTimer = 0;
        this.rightClickBeforeShow = false;
        this.readyShow = (ev) => {
            window.clearTimeout(this.showTimer);
            // 当预览区域显示之后，在作品缩略图上长按鼠标右键，显示原尺寸图片
            // 0 左键 1 滚轮 2 右键
            if (ev.button === 2) {
                this.showTimer = window.setTimeout(() => {
                    this.rightClickBeforeShow = true;
                    this.initWrap(ev);
                }, 500);
            }
        };
        this.cancelReadyShow = (ev) => {
            window.clearTimeout(this.showTimer);
        };
        if (_Config__WEBPACK_IMPORTED_MODULE_11__.Config.mobile) {
            return;
        }
        this.createElements();
        this.bindEvents();
    }
    get show() {
        return this._show;
    }
    set show(val) {
        this._show = val;
        if (val) {
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('showOriginSizeImage');
            this.wrap.style.display = 'block';
            if (!_Config__WEBPACK_IMPORTED_MODULE_11__.Config.mobile) {
                _ShowHelp__WEBPACK_IMPORTED_MODULE_9__.showHelp.show('tipPressDToQuickDownload', _Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_预览作品时按快捷键可以下载这个作品'));
            }
            // 预览动图
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.previewUgoira && this.workData?.body.illustType === 2) {
                this.previewUgoira = new _PreviewUgoira__WEBPACK_IMPORTED_MODULE_4__.PreviewUgoira(this.workData.body.id, this.wrap, _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showOriginImageSize);
            }
        }
        else {
            this.img.src = '';
            this.wrap.style.display = 'none';
            // 销毁预览动图的模块
            if (this.previewUgoira) {
                this.previewUgoira.destroy();
                this.previewUgoira = null;
            }
        }
    }
    createElements() {
        this.wrap = document.createElement('div');
        this.wrap.id = this.wrapId;
        this.img = document.createElement('img');
        this.wrap.appendChild(this.img);
        document.documentElement.appendChild(this.wrap);
    }
    bindEvents() {
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__.artworkThumbnail.onEnter((el) => {
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showOriginImage) {
                // 这里测试在 CentBrowser（内核版本 86）中存在问题，因为 CentBrowser 里鼠标右键松开时才会触发 mousedown 事件，导致根本没法做鼠标长按的效果
                el.addEventListener('mousedown', this.readyShow);
                el.addEventListener('mouseup', this.cancelReadyShow);
            }
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__.artworkThumbnail.onLeave((el) => {
            el.removeEventListener('mousedown', this.readyShow);
            el.removeEventListener('mouseup', this.cancelReadyShow);
        });
        this.wrap.addEventListener('click', () => {
            this.show = false;
        });
        document.body.addEventListener('click', () => {
            this.show = false;
        });
        this.wrap.addEventListener('mousewheel', (ev) => {
            ev.preventDefault();
            // 向上滚 deltaY 是负数（-125），向下滚是正数（125）
            const zoomAdd = ev.deltaY < 0;
            this.zoomWrap(ev, zoomAdd);
        });
        this.wrap.addEventListener('mousemove', (ev) => {
            if (this.moveX === 0) {
                // client x y 是可视区域，不包含滚动区域
                this.moveX = ev.clientX;
                this.moveY = ev.clientY;
            }
            // 本来我对此事件进行了节流处理，但是节流的话容易显得画面不流畅。
            // 而且我试了试，不节流也不会产生太高的 CPU 负荷。所以现在不再做节流处理
            this.moveWrap(ev);
            this.moveX = ev.clientX;
            this.moveY = ev.clientY;
        });
        window.addEventListener('contextmenu', (ev) => {
            // 如果是在原图区域显示之前按下了右键，并且随后显示了原图区域，那么就屏蔽这一次右键菜单
            if (this.rightClickBeforeShow) {
                ev.preventDefault();
                this.rightClickBeforeShow = false;
                this.moveX = ev.clientX;
                this.moveY = ev.clientY;
            }
        });
        window.addEventListener('keydown', (ev) => {
            // 预览大图时，可以使用快捷键 D 下载这个作品
            if (ev.code === 'KeyD' && this.show) {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', [
                    {
                        type: 'illusts',
                        id: this.workData.body.id,
                    },
                ]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_5__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_8__.Colors.bgBlue,
                    position: 'center',
                });
            }
            // 预览作品时，可以使用快捷键 C 仅下载当前显示的图片
            if (ev.code === 'KeyC' && this.show) {
                ev.stopPropagation();
                if (this.workData.body.pageCount > 1) {
                    _store_Store__WEBPACK_IMPORTED_MODULE_10__.store.setDownloadOnlyPart(Number.parseInt(this.workData.body.id), [
                        this.index,
                    ]);
                }
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', [
                    {
                        type: 'illusts',
                        id: this.workData.body.id,
                    },
                ]);
                // 下载时不显示下载面板
                _store_States__WEBPACK_IMPORTED_MODULE_5__.states.quickCrawl = true;
                _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_已发送下载请求'), {
                    bgColor: _Colors__WEBPACK_IMPORTED_MODULE_8__.Colors.bgBlue,
                    position: 'center',
                });
            }
            // 按 Esc 键时取消预览
            if (ev.code === 'Escape' && this.show) {
                this.show = false;
                ev.stopPropagation();
            }
        }, true);
    }
    async getImage(url) {
        window.clearInterval(this.getImageSizeTimer);
        this.testImg.src = '';
        return new Promise((resolve) => {
            this.testImg = new Image();
            this.testImg.src = url;
            this.getImageSizeTimer = window.setInterval(() => {
                if (this.testImg.naturalWidth > 0) {
                    window.clearInterval(this.getImageSizeTimer);
                    return resolve(this.testImg);
                }
            }, 50);
        });
    }
    // 初次显示一个图片时，初始化 wrap 的样式
    async initWrap(ev) {
        const url = this.urls[_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.showOriginImageSize];
        if (!url) {
            return;
        }
        this.zoomIndex = 6;
        this.zoom = this.zoomList[this.zoomIndex];
        this.style = this.defaultStyle;
        // 获取图片的原始宽高
        this.img = await this.getImage(url);
        this.style.imgW = this.img.naturalWidth;
        this.style.imgH = this.img.naturalHeight;
        this.style.width = this.style.imgW;
        this.style.height = this.style.imgH;
        // 替换 img 元素
        this.wrap.querySelector('img').remove();
        this.wrap.appendChild(this.img);
        // 设置定位
        const innerWidth = window.innerWidth - 17;
        // 在水平方向上，总是居中显示
        this.style.ml = (innerWidth - this.style.width - this.border) / 2;
        // 在垂直方向上
        if (this.style.height > window.innerHeight) {
            // 如果图片高度超过了可视区域，则从顶部显示
            this.style.mt = 0;
        }
        else {
            // 否则垂直居中显示
            this.style.mt = (window.innerHeight - this.style.height - this.border) / 2;
        }
        // 计算鼠标移动 1 像素时，图片应该移动多少像素
        // 计算横向的 onePxMove
        let onePxMoveX = this.style.imgW / innerWidth;
        if (this.style.imgW > innerWidth) {
            // 如果图片宽度超出窗口可视宽度，计算鼠标左侧和右侧的图像宽度分别是多少
            const hiddenHalf = (this.style.imgW - innerWidth) / 2;
            const leftWidth = ev.clientX + hiddenHalf;
            const rightWidth = this.style.imgW - leftWidth;
            // 计算鼠标向左或向右移动 1 像素时，图片应该移动多少像素。取比较大的一个值
            onePxMoveX = Math.max(leftWidth / ev.clientX, rightWidth / (innerWidth - ev.clientX));
        }
        // 计算纵向的 onePxMove
        let onePxMoveY = this.style.imgH / window.innerHeight;
        if (this.style.imgH > window.innerHeight) {
            // 如果图片高度超出窗口可视高度，下载器会把图片从顶部显示。此时需要特殊处理 onePxMove
            // 让鼠标从当前位置向下到窗口底部时，可以完整查看整个图片
            onePxMoveY =
                (this.style.imgH - ev.clientY) / (window.innerHeight - ev.clientY);
        }
        // 比较水平方向和垂直方向的计算结果，取比较大的一个值
        // onePxMove 乘以修正系数，进行放大
        // 这样可以让用户在移动鼠标时，不需要移动到边界上就可以查看到图片的边界
        this.onePxMove = Math.max(onePxMoveX, onePxMoveY) * 1.1;
        // 设置样式，显示图片
        this.setWrapStyle();
        this.show = true;
    }
    // 以鼠标所在位置为中心点缩放
    // 例如，鼠标放在角色的眼睛上面进行缩放，在缩放之后，依然把眼睛定位到鼠标所在位置
    // 当用户滚动鼠标滚轮时，传递鼠标相对于原图区域的坐标（不包含 border）
    zoomWrap(ev, zoomAdd) {
        // 设置 zoom 等级
        const oldZoom = this.zoom;
        const oldZoomIndex = this.zoomIndex;
        let cancel = false;
        this.zoomIndex += zoomAdd ? 1 : -1;
        this.zoom = this.zoomList[this.zoomIndex];
        if (this.zoom === undefined) {
            cancel = true;
        }
        // 检查缩放后的图片的尺寸是否超出了限制
        const testWidth = this.style.imgW * this.zoom;
        const testHeight = this.style.imgH * this.zoom;
        if (Math.max(testWidth, testHeight) > 30000) {
            cancel = true;
        }
        if (cancel) {
            this.zoom = oldZoom;
            this.zoomIndex = oldZoomIndex;
            return;
        }
        // 启动缩放
        this.style.width = this.style.imgW * this.zoom;
        this.style.height = this.style.imgH * this.zoom;
        // 计算这次缩放相对于上次缩放增加的倍率（容器的尺寸会增加多少倍）
        const zoom = (this.zoom - oldZoom) / oldZoom;
        // 缩放之前，鼠标与容器顶点形成了一个矩形（0, 0, offsetX, offsetY）
        // 计算这个矩形在缩放之后，相比于缩放之前增加了多少像素
        const offsetXAdd = ev.offsetX * zoom;
        const offsetYAdd = ev.offsetY * zoom;
        // 对缩放之前的 margin 值加以修改，使缩放之前的鼠标位置的图像现在仍然位于鼠标位置
        this.style.ml = this.style.ml - offsetXAdd;
        this.style.mt = this.style.mt - offsetYAdd;
        this.setWrapStyle();
    }
    moveWrap(ev) {
        // 计算鼠标距离上次执行时，移动的距离
        const mouseMoveX = ev.clientX - this.moveX;
        const mouseMoveY = ev.clientY - this.moveY;
        // 在水平方向上应该移动多少像素
        let moveX = mouseMoveX * this.onePxMove * this.zoom;
        // 在垂直方向上应该移动多少像素
        let moveY = mouseMoveY * this.onePxMove * this.zoom;
        // 设置容差值，允许图像的边界与可视区域之间存在空隙
        // 例如，本来图片的左侧与可视区域的左侧重合时，就不应该允许图片继续向右移动了。
        // 现在设置了容差值，使图片可以继续向右移动 tolerance 像素。
        // 这样可以让用户知道已经移动到了图片的边缘，避免用户产生疑惑。
        const tolerance = 10;
        let ml;
        let mt;
        // 鼠标向左移动，wrap 向右移动，ml 增加
        if (mouseMoveX < 0) {
            // 如果 wrap 左侧还有被隐藏的部分，才允许向右移动
            if (this.style.ml < tolerance) {
                ml = this.style.ml - moveX;
            }
        }
        // 鼠标向右移动，wrap 向左移动，ml 减少
        if (mouseMoveX > 0) {
            // 如果 wrap 右侧还有被隐藏的部分，才允许向左移动
            if (this.style.ml + this.style.width > window.innerWidth - tolerance) {
                ml = this.style.ml - moveX;
            }
        }
        // 鼠标向上移动，warp 向下移动，mt 增加
        if (mouseMoveY < 0) {
            // 如果 wrap 顶部还有被隐藏的部分，才允许向下移动
            if (this.style.mt < tolerance) {
                mt = this.style.mt - moveY;
            }
        }
        // 鼠标向下移动，warp 向上移动，mt 减少
        if (mouseMoveY > 0) {
            // 如果 wrap 底部还有被隐藏的部分，才允许向上移动
            if (this.style.mt + this.style.height > window.innerHeight - tolerance) {
                mt = this.style.mt - moveY;
            }
        }
        // 设置 margin 时，需要检查容器是否处于可视区域之外。如果超出了可视区域则不赋值
        if (ml !== undefined) {
            if ((ml > 0 && ml < window.innerWidth) ||
                (ml < 0 && ml + this.style.width > tolerance)) {
                // 如果 ml 小于 0，其右边的坐标不可以小于 0
                this.style.ml = ml;
            }
        }
        if (mt !== undefined) {
            if ((mt > 0 && mt < window.innerHeight) ||
                (mt < 0 && mt + this.style.height > tolerance)) {
                // 如果 mt 小于 0，其底边的坐标不可以小于 0
                this.style.mt = mt;
            }
        }
        this.setWrapStyle();
    }
    setWrapStyle() {
        this.wrap.style.width = this.style.width + 'px';
        this.wrap.style.height = this.style.height + 'px';
        this.wrap.style.marginTop = this.style.mt + 'px';
        this.wrap.style.marginLeft = this.style.ml + 'px';
        this.previewUgoira &&
            this.previewUgoira.setSize(this.style.width, this.style.height);
    }
    setData(urls, data, index) {
        this.urls = urls;
        this.workData = data;
        this.index = index;
    }
    hide() {
        this.show = false;
    }
}
const showOriginSizeImage = new ShowOriginSizeImage();



/***/ }),

/***/ "./src/ts/ShowWhatIsNew.ts":
/*!*********************************!*\
  !*** ./src/ts/ShowWhatIsNew.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");






// 显示最近更新内容
class ShowWhatIsNew {
    constructor() {
        this.flag = '16.5.1';
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.settingInitialized, () => {
            // 消息文本要写在 settingInitialized 事件回调里，否则它们可能会被翻译成错误的语言
            let msg = `
      <span>${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_优化性能和用户体验')}</span>
      `;
            // <strong>${lang.transl('_新增功能')}:</strong>
            // <br>
            // <span class="blue">${lang.transl('abc')}</span>
            // ${lang.transl(
            //   '_你可以在更多选项卡的xx分类里找到它',
            //   lang.transl('_增强')
            // )}
            // 在更新说明的下方显示赞助提示
            msg += `
      <br>
      <br>
      ${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_赞助方式提示')}`;
            this.show(msg);
        });
    }
    show(msg) {
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.isPixiv() && _setting_Settings__WEBPACK_IMPORTED_MODULE_5__.settings.whatIsNewFlag !== this.flag) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.show(msg, {
                title: _Config__WEBPACK_IMPORTED_MODULE_1__.Config.appName + ` ${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_最近更新')}`,
                btn: _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_我知道了'),
            });
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_5__.setSetting)('whatIsNewFlag', this.flag);
        }
    }
}
new ShowWhatIsNew();


/***/ }),

/***/ "./src/ts/ShowZoomBtnOnThumb.ts":
/*!**************************************!*\
  !*** ./src/ts/ShowZoomBtnOnThumb.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _ImageViewer__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./ImageViewer */ "./src/ts/ImageViewer.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");





// 在作品缩略图上显示放大按钮，点击按钮会调用图片查看器来查看大图
class ShowZoomBtnOnThumb {
    constructor() {
        this.btnId = 'zoomBtnOnThumb';
        this.btnSize = [32, 32];
        this.hiddenBtnTimer = 0; // 使用定时器让按钮延迟消失。这是为了解决一些情况下按钮闪烁的问题
        this.hiddenBtnDelay = 100;
        this.currentWorkId = ''; // 显示放大按钮时，保存触发事件的作品 id
        this.doNotShowBtn = false; // 当点击了放大按钮后，进入此状态，此状态中不会显示放大按钮
        if (_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
            return;
        }
        this.addBtn();
        this.bindEvents();
    }
    // 此状态是为了解决这个问题：点击了放大按钮之后，按钮会被隐藏，隐藏之后，鼠标下方就是图片缩略图区域，这会触发缩略图的鼠标事件，导致放大按钮马上就又显示了出来。所以点击放大按钮之后设置这个状态，在其为 true 的期间不会显示放大按钮。过一段时间再把它复位。复位所需的时间很短，因为只要能覆盖这段时间就可以了：从隐藏放大按钮开始算起，到缩略图触发鼠标事件结束。
    addBtn() {
        const btn = document.createElement('button');
        btn.id = this.btnId;
        btn.innerHTML = `
    <svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-fangda"></use>
</svg>`;
        this.btn = document.body.appendChild(btn);
    }
    bindEvents() {
        // 页面切换时隐藏按钮
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitch, () => {
            this.hiddenBtn();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clickBtnOnThumb, () => {
            this.hiddenBtnNow();
        });
        // 页面切换时隐藏按钮
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.hiddenBtn();
        });
        // 鼠标移入按钮时取消隐藏按钮
        this.btn.addEventListener('mouseenter', (ev) => {
            window.clearTimeout(this.hiddenBtnTimer);
        });
        // 鼠标移出按钮时隐藏按钮
        this.btn.addEventListener('mouseleave', () => {
            this.hiddenBtn();
        });
        // 点击按钮时初始化图片查看器
        this.btn.addEventListener('click', (ev) => {
            this.hiddenBtnNow();
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('clickBtnOnThumb');
            if (this.currentWorkId) {
                const viewer = new _ImageViewer__WEBPACK_IMPORTED_MODULE_1__.ImageViewer({
                    workId: this.currentWorkId,
                    imageNumber: 1,
                    imageSize: _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.magnifierSize,
                    autoStart: true,
                    showLoading: true,
                });
                viewer.init();
            }
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__.artworkThumbnail.onEnter((el, id) => {
            this.currentWorkId = id;
            this.showBtn(el);
        });
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_3__.artworkThumbnail.onLeave(() => {
            this.hiddenBtn();
        });
    }
    // 显示放大按钮
    showBtn(target) {
        if (this.doNotShowBtn) {
            return;
        }
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.magnifier) {
            return;
        }
        window.clearTimeout(this.hiddenBtnTimer);
        const rect = target.getBoundingClientRect();
        this.btn.style.left =
            window.scrollX +
                rect.left +
                (_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.magnifierPosition === 'left'
                    ? 0
                    : rect.width - this.btnSize[0]) +
                'px';
        this.btn.style.top = window.scrollY + rect.top + 'px';
        this.btn.style.display = 'flex';
    }
    // 延迟隐藏放大按钮
    hiddenBtn() {
        window.clearTimeout(this.hiddenBtnTimer);
        this.hiddenBtnTimer = window.setTimeout(() => {
            this.btn.style.display = 'none';
        }, this.hiddenBtnDelay);
    }
    // 立刻隐藏放大按钮
    hiddenBtnNow() {
        this.doNotShowBtn = true;
        window.setTimeout(() => {
            this.doNotShowBtn = false;
        }, 100);
        window.clearTimeout(this.hiddenBtnTimer);
        this.btn.style.display = 'none';
    }
}
new ShowZoomBtnOnThumb();


/***/ }),

/***/ "./src/ts/Theme.ts":
/*!*************************!*\
  !*** ./src/ts/Theme.ts ***!
  \*************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   theme: () => (/* binding */ theme)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");



// 下载器的主题默认跟随页面主题。如果用户设置了下载器主题，则不再跟随页面主题
class Theme {
    constructor() {
        this.allTheme = ['white', 'dark'];
        this.defaultTheme = 'white'; // 默认主题
        this.theme = 'white'; // 保存当前使用的主题
        this.settingTheme = ''; // 保存用户设置的下载器主题
        // 主题标记以及对应的 className
        // 把需要响应主题变化的元素注册到这个组件里，元素会被添加当前主题的 className
        // 默认主题 white 是没有 className 的，其他主题通过对应的 className，在默认主题的基础上更改样式。
        this.classNameMap = new Map([
            ['white', ''],
            ['dark', 'theme-dark'],
        ]);
        this.selector = '#gtm-var-theme-kind'; // 通过这个选择器查找含有主题标记的元素
        this.timer = 0;
        // 页面上储存的主题标记，与本组件里的主题的对应关系
        this.htmlFlagMap = new Map([
            ['', 'white'],
            ['default', 'white'],
            ['dark', 'dark'],
        ]);
        this.elList = []; // 保存已注册的元素
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.isPixiv()) {
            this.bindEvents();
        }
    }
    bindEvents() {
        // 主题设置变化时修改主题
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'theme') {
                this.settingTheme = data.value;
                this.setTheme(data.value);
            }
        });
        // 使用定时器查找标记元素
        this.timer = window.setInterval(() => {
            this.findFlag();
        }, 300);
    }
    // 查找含有 pixiv 主题标记的元素，并监听其变化
    findFlag() {
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            const el = document.body;
            if (el) {
                window.clearInterval(this.timer);
                this.setTheme(this.getThemeFromHtml());
                // 监听 body 的 class 变化
                const ob = new MutationObserver(() => {
                    const flag = this.getThemeFromHtml();
                    this.setTheme(flag);
                });
                ob.observe(el, {
                    attributes: true,
                    attributeFilter: ['class'],
                });
            }
        }
        else {
            const el = document.querySelector(this.selector);
            if (el) {
                window.clearInterval(this.timer);
                this.setTheme(this.getThemeFromHtml());
                // 监听标记元素的 textContent 变化
                const ob = new MutationObserver((mutationsList) => {
                    for (const item of mutationsList) {
                        if (item.type === 'characterData') {
                            const flag = this.getThemeFromHtml();
                            this.setTheme(flag);
                            break;
                        }
                    }
                });
                ob.observe(el, {
                    characterData: true,
                    subtree: true,
                });
            }
        }
    }
    getThemeFromHtml() {
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            // 移动端需要使用不同的方法来获取主题
            const dark = document.body.classList.contains('dark');
            const pageTheme = dark ? 'dark' : 'white';
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('getPageTheme', pageTheme);
            return pageTheme;
        }
        else {
            // 桌面端
            // 从含有 pixiv 主题标记的元素里获取主题
            const el = document.querySelector(this.selector);
            if (el) {
                const pageTheme = this.htmlFlagMap.get(el.textContent);
                _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('getPageTheme', pageTheme);
                return pageTheme || this.defaultTheme;
            }
            // 根据 html 元素的背景颜色判断
            // 此方法不适用于移动端，因为移动端的 html 背景色总是 'rgba(0, 0, 0, 0)'
            // "rgb(245, 245, 245)"
            // "rgb(0, 0, 0)"
            const htmlBG = getComputedStyle(document.documentElement)['backgroundColor'];
            if (htmlBG) {
                if (htmlBG.includes('rgb(2')) {
                    return 'white';
                }
                else if (htmlBG.includes('rgb(0')) {
                    return 'dark';
                }
            }
            return this.defaultTheme;
        }
    }
    setTheme(flag) {
        // 如果用户设置了下载器主题，则始终使用下载器主题（忽略页面主题）
        if (this.allTheme.includes(this.settingTheme)) {
            flag = this.settingTheme;
        }
        let result = 'white'; // 储存根据标记所选择的主题
        // 根据标记，设置要使用的主题
        switch (flag) {
            case 'white':
                result = 'white';
                break;
            case 'dark':
                result = 'dark';
                break;
            default:
                // 如果传递的值是本模块不能识别的，包括 'auto'，就都自动获取
                result = this.getThemeFromHtml() || this.defaultTheme;
                break;
        }
        // 如果计算出的主题和当前主题不同，则执行变化
        if (result !== this.theme) {
            this.theme = result;
            for (const el of this.elList) {
                this.setClass(el);
            }
        }
    }
    // 把元素注册到本组件里
    register(el) {
        if (!_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.isPixiv()) {
            return;
        }
        this.elList.push(el);
        this.setClass(el);
    }
    // 给元素设置主题对应的 className
    setClass(el) {
        // 先清除所有主题颜色的 className
        for (const className of this.classNameMap.values()) {
            if (el.classList.contains(className)) {
                el.classList.remove(className);
            }
        }
        // 添加当前主题对应的 className
        const name = this.classNameMap.get(this.theme);
        name && el.classList.add(name);
    }
}
const theme = new Theme();



/***/ }),

/***/ "./src/ts/Tip.ts":
/*!***********************!*\
  !*** ./src/ts/Tip.ts ***!
  \***********************/
/***/ (() => {


// 给下载器的界面元素添加提示文本，当鼠标移动到元素上时会显示提示
// 如果要给某个元素添加提示，先给它添加 has_tip 的 className，然后用 data-tip 设置提示内容
class Tip {
    constructor() {
        this.addTipEl();
        this.bindEvents();
    }
    addTipEl() {
        this.tipEl = document.createElement('div');
        this.tipEl.id = 'tip';
        document.body.append(this.tipEl);
    }
    bindEvents() {
        const tips = document.querySelectorAll('.has_tip');
        for (const el of tips) {
            for (const ev of ['mouseenter', 'mouseleave']) {
                el.addEventListener(ev, (event) => {
                    const e = (event || window.event);
                    const text = el.dataset.tip;
                    this.showTip(text, {
                        type: ev === 'mouseenter' ? 1 : 0,
                        x: e.clientX,
                        y: e.clientY,
                    });
                });
            }
        }
    }
    // 显示中间面板上的提示。参数 mouse 指示鼠标是移入还是移出，并包含鼠标坐标
    showTip(text, mouse) {
        if (!text) {
            throw new Error('No tip text.');
        }
        if (mouse.type === 1) {
            this.tipEl.innerHTML = text;
            this.tipEl.style.left = mouse.x + 30 + 'px';
            this.tipEl.style.top = mouse.y - 30 + 'px';
            this.tipEl.style.display = 'block';
        }
        else if (mouse.type === 0) {
            this.tipEl.style.display = 'none';
        }
    }
}
new Tip();


/***/ }),

/***/ "./src/ts/Toast.ts":
/*!*************************!*\
  !*** ./src/ts/Toast.ts ***!
  \*************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   toast: () => (/* binding */ toast)
/* harmony export */ });
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");


// 轻提示，只显示文字和背景颜色
// 适用于无需用户进行确认的提示
class Toast {
    constructor() {
        this.defaultCfg = {
            msg: '',
            color: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.white,
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgBrightBlue,
            dealy: 1500,
            enter: 'up',
            leave: 'fade',
            position: 'mouse',
        };
        this.successCfg = {
            msg: '',
            color: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.white,
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgSuccess,
            dealy: 1500,
            enter: 'up',
            leave: 'fade',
            position: 'mouse',
        };
        this.warningCfg = {
            msg: '',
            color: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.white,
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgWarning,
            dealy: 1500,
            enter: 'up',
            leave: 'fade',
            position: 'mouse',
        };
        this.errorCfg = {
            msg: '',
            color: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.white,
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgError,
            dealy: 1500,
            enter: 'up',
            leave: 'fade',
            position: 'mouse',
        };
        this.tipClassName = 'xzToast';
        this.mousePosition = { x: 0, y: 0 };
        this.minTop = 20;
        this.once = 1; // 每一帧移动多少像素
        this.total = 20; // 移动多少像素后消失
        this.bindEvents();
    }
    bindEvents() {
        // 必须是监听 mousemove 而不是 click
        window.addEventListener('mousemove', (ev) => {
            this.mousePosition.x = ev.x;
            this.mousePosition.y = ev.y;
        });
    }
    show(msg, arg) {
        this.create(Object.assign({}, this.defaultCfg, arg, { msg: msg }));
    }
    success(msg, arg) {
        this.create(Object.assign({}, this.successCfg, arg, { msg: msg }));
    }
    warning(msg, arg) {
        this.create(Object.assign({}, this.warningCfg, arg, { msg: msg }));
    }
    error(msg, arg) {
        this.create(Object.assign({}, this.errorCfg, arg, { msg: msg }));
    }
    create(arg) {
        const span = document.createElement('span');
        span.textContent = arg.msg;
        span.style.color = arg.color;
        // 设置背景颜色，优先使用 color
        span.style.backgroundColor = arg.bgColor;
        span.style.opacity = '0'; // 先使提示完全透明
        span.classList.add(this.tipClassName);
        if (_Config__WEBPACK_IMPORTED_MODULE_1__.Config.mobile) {
            span.classList.add('mobile');
        }
        // 把提示添加到页面上
        document.body.appendChild(span);
        // 设置 left，使其居中
        // 默认的中间点是窗口的中间
        let centerPoint = window.innerWidth / 2;
        if (arg.position === 'mouse') {
            // 把中间点设置为鼠标所处的位置
            centerPoint = this.mousePosition.x;
        }
        // 设置 left
        const rect = span.getBoundingClientRect();
        let left = centerPoint - rect.width / 2;
        const minLeft = 0; // 防止提示左侧超出窗口
        const maxLeft = window.innerWidth - rect.width; // 防止提示右侧超出窗口
        if (left < minLeft) {
            left = minLeft;
        }
        if (left > maxLeft) {
            left = maxLeft;
        }
        span.style.left = left + 'px';
        // 设置 top
        let lastTop = 0;
        if (arg.position === 'topCenter') {
            lastTop = this.minTop;
        }
        if (arg.position === 'center') {
            lastTop = window.innerHeight / 2 - this.minTop;
        }
        if (arg.position === 'mouse') {
            // 跟随鼠标位置
            // top 值减去一点高度，使文字出现在鼠标上方
            let y = this.mousePosition.y - 40;
            if (y < this.minTop) {
                y = this.minTop;
            }
            lastTop = y;
        }
        // 出现动画
        if (arg.enter === 'none') {
            span.style.top = lastTop + 'px';
            span.style.opacity = '1';
        }
        else {
            this.enter(span, arg.enter, lastTop);
        }
        // 消失动画
        window.setTimeout(() => {
            if (arg.leave === 'none') {
                span.remove();
            }
            else {
                this.leave(span, arg.leave, lastTop);
            }
        }, arg.dealy);
    }
    // 提示出现的动画
    enter(el, way, lastTop) {
        const startTop = lastTop + this.total; // 初始 top 值
        const once = 2;
        const total = this.total;
        let numberOfTimes = 0; // 执行次数
        const frame = function (timestamp) {
            numberOfTimes++;
            // 计算总共上移了多少像素
            const move = once * numberOfTimes;
            // 计算不透明度
            const opacity = move / total;
            if (move <= total && opacity <= 1) {
                if (way === 'up') {
                    el.style.top = startTop - move + 'px';
                }
                el.style.opacity = opacity.toString();
                // 请求下一帧
                window.requestAnimationFrame(frame);
            }
        };
        window.requestAnimationFrame(frame);
    }
    // 提示消失的动画
    leave(el, way, lastTop) {
        const startTop = lastTop; // 初始 top 值
        const once = this.once;
        const total = this.total;
        let numberOfTimes = 0; // 执行次数
        const frame = function (timestamp) {
            numberOfTimes++;
            // 计算总共上移了多少像素
            const move = once * numberOfTimes;
            // 计算不透明度
            const opacity = 1 - move / total;
            if (move < total && opacity > 0) {
                if (way === 'up') {
                    el.style.top = startTop - move + 'px';
                }
                el.style.opacity = opacity.toString();
                // 请求下一帧
                window.requestAnimationFrame(frame);
            }
            else {
                // 动画执行完毕，删除元素
                el.remove();
            }
        };
        window.requestAnimationFrame(frame);
    }
}
const toast = new Toast();



/***/ }),

/***/ "./src/ts/Token.ts":
/*!*************************!*\
  !*** ./src/ts/Token.ts ***!
  \*************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   token: () => (/* binding */ token)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");



// 获取和保存 token
class Token {
    constructor() {
        this.tokenStore = 'xzToken';
        this.timeStore = 'xzTokenTime';
        this.updateURL = 'https://www.pixiv.net/artworks/62751951';
        this.interval = 300000; // 两次更新之间的最小时间间隔。目前设置为 5 分钟
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.isPixiv()) {
            this.token = this.getToken();
            this.updateToken();
            this.bindEvents();
        }
    }
    bindEvents() {
        // 重置设置时重新获取一次 token
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resetSettingsEnd, () => {
            this.reset();
        });
    }
    getToken() {
        const token = localStorage.getItem(this.tokenStore);
        return token ? token : '';
    }
    async updateToken() {
        const nowTime = new Date().getTime();
        const lastTimeStr = localStorage.getItem(this.timeStore);
        if (this.token &&
            lastTimeStr &&
            nowTime - Number.parseInt(lastTimeStr) < this.interval) {
            return;
        }
        // 从网页源码里获取用户 token 并储存
        return fetch(this.updateURL)
            .then((response) => {
            return response.text();
        })
            .then((data) => {
            const regExp = _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? /postKey":"(\w+)"/ : /token":"(\w+)"/;
            const result = data.match(regExp);
            if (result) {
                this.token = result[1];
            }
            if (this.token) {
                localStorage.setItem(this.tokenStore, this.token);
                localStorage.setItem(this.timeStore, new Date().getTime().toString());
            }
            else {
                console.error('UpdateToken failed: no token found!');
            }
        });
    }
    // 不论用户是否登录，都有 token，所以不能根据 token 来判断用户是否登录
    // 在桌面端，如果存在下面的字符串，则说明用户未登录：
    // "userData":null
    async reset() {
        this.token = '';
        localStorage.removeItem(this.tokenStore);
        localStorage.removeItem(this.timeStore);
        return this.updateToken();
    }
}
const token = new Token();



/***/ }),

/***/ "./src/ts/Tools.ts":
/*!*************************!*\
  !*** ./src/ts/Tools.ts ***!
  \*************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Tools: () => (/* binding */ Tools)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./utils/Utils */ "./src/ts/utils/Utils.ts");




class Tools {
    // 把结果中的动图排列到最前面
    static sortUgoiraFirst(a, b) {
        if (a.type === 2 && b.type !== 2) {
            return -1;
        }
        else if (a.type === 2 && b.type === 2) {
            return 0;
        }
        else {
            return 1;
        }
    }
    // 根据 tag 判断是否是 R-18(G) 作品
    static isR18OrR18G(tags) {
        const str = Array.isArray(tags) ? tags.toString() : tags;
        return (str.includes('R-18') ||
            str.includes('R-18G') ||
            str.includes('R18') ||
            str.includes('R18G'));
    }
    // 在不同的页面类型里，尝试从 url 中获取 tag
    static getTagFromURL(url = location.href) {
        const nowURL = new URL(url);
        // 2 用户作品列表页
        if (/\/users\/\d+/.test(url) && !url.includes('/bookmarks')) {
            // 匹配 pathname 里用户 id 之后的字符
            const test = nowURL.pathname.match(/\/users\/\d+(\/.+)/);
            if (test && test.length === 2) {
                const str = test[1];
                // 如果用户 id 之后的字符多于一个路径，则把最后一个路径作为 tag，示例
                // https://www.pixiv.net/users/2188232/illustrations/ghostblade
                const array = str.split('/');
                // ["", "illustrations", "ghostblade"]
                if (array.length > 2) {
                    return decodeURIComponent(array[array.length - 1]);
                }
            }
        }
        // 4 旧版收藏页面
        if (nowURL.pathname === '/bookmark.php') {
            if (parseInt(_utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.getURLSearchField(nowURL.href, 'untagged')) === 1) {
                // 旧版 “未分类” tag 是个特殊标记
                // https://www.pixiv.net/bookmark.php?untagged=1
                return '未分類';
            }
        }
        // 4 新版收藏页面
        if (nowURL.pathname.includes('/bookmarks/')) {
            // 新版收藏页 url，tag 在路径末端，如
            // https://www.pixiv.net/users/9460149/bookmarks/artworks/R-18
            // https://www.pixiv.net/users/9460149/bookmarks/novels/R-18
            const test = /\/bookmarks\/\w*\/(.[^\/|^\?|^&]*)/.exec(nowURL.pathname);
            if (test !== null && test.length > 1 && !!test[1]) {
                return decodeURIComponent(test[1]);
            }
        }
        // 5 搜索页面
        if (nowURL.pathname.includes('/tags/')) {
            return decodeURIComponent(nowURL.pathname.split('tags/')[1].split('/')[0]);
        }
        // 默认情况，从查询字符串里获取，如下网址
        // https://www.pixiv.net/bookmark.php?tag=R-18
        return decodeURIComponent(_utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.getURLSearchField(nowURL.href, 'tag'));
    }
    /**从 url 里获取 artworks id。如果查找不到 id 会返回空字符串 */
    static getIllustId(url) {
        if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Unlisted) {
            return _utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.getURLPathField(window.location.pathname, 'unlisted');
        }
        const str = url || window.location.href;
        let test = null;
        if (str.includes('/artworks/')) {
            // 新版 url
            test = /artworks\/(\d*\d)/.exec(str);
        }
        else if (str.includes('illust_id')) {
            // 传统 url
            test = /illust_id=(\d*\d)/.exec(str);
        }
        if (test && test.length > 1) {
            return test[1];
        }
        else {
            return '';
        }
    }
    /**从 url 里获取 novel id。如果查找不到 id 会返回空字符串 */
    // https://www.pixiv.net/novel/show.php?id=12771688
    static getNovelId(url) {
        if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Unlisted) {
            return _utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.getURLPathField(window.location.pathname, 'unlisted');
        }
        const str = url || window.location.href;
        let result = '';
        const test = str.match(/novel\/show.php\?id=(\d*)?/);
        if (test && test.length > 1) {
            result = test[1];
        }
        return result;
    }
    /**从 DOM 元素中获取作品的 id
     *
     * 如果查找不到 id 会返回空字符串
     */
    static findWorkIdFromElement(el, type = 'illusts') {
        let a;
        if (el.nodeName === 'A') {
            a = el;
        }
        else {
            if (type === 'illusts') {
                a = el.querySelector('a[href*="/artworks/"]');
            }
            else {
                a = el.querySelector('a[href*="/novel/show"]');
            }
        }
        if (!a) {
            return '';
        }
        if (type === 'illusts') {
            return this.getIllustId(a.href);
        }
        else {
            return this.getNovelId(a.href);
        }
    }
    // 获取当前页面的用户 id
    // 这是一个不够可靠的 api
    // 测试：在作品页内 https://www.pixiv.net/artworks/79399027 获取 userId ，正确结果应该是 13895186
    static getUserId() {
        const newRegExp = /\/users\/(\d+)/; // 获取 /users/ 后面连续的数字部分，也就是用户的 id
        // 列表页里从 url 中获取
        const test4 = newRegExp.exec(location.pathname);
        if (!!test4 && test4.length > 1 && !!test4[1]) {
            return test4[1];
        }
        // 获取包含用户 id 的元素，注意这些选择器可能会变，需要进行检查
        const testA = document.querySelector('aside a') || document.querySelector('nav a');
        // 第一个元素是作品页内，作品下方的作者头像区域的 a 标签
        // 第一个元素是作品页内，页面右侧作者信息区域的 a 标签
        // 第二个元素是用户主页或列表页里，“主页”按钮的 a 标签
        if (testA && testA.href) {
            const test5 = newRegExp.exec(testA.href);
            if (!!test5 && test5.length > 1 && !!test5[1]) {
                return test5[1];
            }
        }
        // 从旧版页面的 head 元素的 script 脚本内容里匹配这一部分
        // pixiv.context.user.id = "<userid>"
        const test1 = /user.id = "(\d*)"/.exec(document.head.innerHTML);
        if (test1 && test1.length > 0) {
            return test1[1];
        }
        // 从旧版页面的 head 元素的 script 脚本内容里匹配这一部分
        // pixiv.context.userId = "<userid>"
        const test2 = /userId = "(\d*)"/.exec(document.head.innerHTML);
        if (test2 && test2.length > 0) {
            return test2[1];
        }
        // 最后从 body 里匹配
        // Warning ：这有可能会匹配到错误的（其他）用户 id！
        const test3 = newRegExp.exec(document.body.innerHTML);
        if (test3) {
            return test3[1];
        }
        // 如果都没有获取到
        throw new Error('getUserId failed!');
    }
    static getLoggedUserID() {
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            const match = document.head.innerHTML.match(/'user_id', (\d*)/);
            if (match && match.length > 1) {
                return match[1];
            }
        }
        // 在新版页面里，从 head 里的 script 里匹配用户 id
        const match = document.head.innerHTML.match(/'user_id', "(\d*)"/);
        if (match && match.length > 1) {
            return match[1];
        }
        const element = document.querySelector('#qualtrics_user-id');
        if (element) {
            const text = element.textContent;
            if (text && /^\d+\d$/.test(text)) {
                return text;
            }
        }
        {
            // 在旧版页面里，从 head 里的 script 里匹配用户 id
            const match2 = document.head.innerHTML.match(/pixiv.user.id = "(\d*)"/);
            if (match2 && match2.length > 1) {
                return match2[1];
            }
        }
        {
            // 在约稿页面里，从 body 里的 script 里匹配用户 id
            const el = document.querySelector('script#gtm-datalayer');
            if (el && el.textContent) {
                const match3 = el.textContent.match(/user_id:'(\d+)'/);
                if (match3 && match3.length > 1) {
                    return match3[1];
                }
            }
        }
        return '';
    }
    // 将元素插入到 Pixiv 页面顶部
    /*
    newindex-inner 是在未登录时的用户作品列表页面使用的
    layout-body 是在未登录时的搜索页使用的
    */
    static insertToHead(el) {
        if (document.body) {
            document.body.insertAdjacentElement('afterbegin', el);
        }
        else {
            ;
            (document.querySelector('.newindex-inner') ||
                document.querySelector('.layout-body')).insertAdjacentElement('beforebegin', el);
        }
        return el;
    }
    // 寻找 slot，本程序使用的 slot 都要有 data-name 属性
    static findSlot(name) {
        const slot = document.querySelector(`slot[data-name=${name}]`);
        if (!slot) {
            throw new Error(`No such slot: ${name}`);
        }
        return slot;
    }
    // 使用指定的插槽
    static useSlot(name, element) {
        const slot = this.findSlot(name);
        if (typeof element === 'string') {
            // 插入字符串形式的元素
            // 这里不直接使用 insertAdjacentElement 是为了可以返回生成的元素
            const wrap = document.createElement('div');
            wrap.innerHTML = element;
            const el = wrap.children[0];
            slot.appendChild(el);
            return el;
        }
        else {
            // 插入 html 元素
            slot.appendChild(element);
            return element;
        }
    }
    // 清空指定的插槽
    static clearSlot(name) {
        this.findSlot(name).innerHTML = '';
    }
    // 创建下载面板上的通用按钮
    // 注意 textFlag 和 titleFlag 必须是 LangText 里存在的属性，这是为了能根据语言设置动态切换文本
    // 如果 text 和 title 是直接设置的字符串，那么不应该使用这个方法设置，而是由调用者自行设置
    static addBtn(slot, bg = '', textFlag = '', titleFlag = '') {
        const e = document.createElement('button');
        e.type = 'button';
        e.style.backgroundColor = bg;
        textFlag && e.setAttribute('data-xztext', textFlag);
        titleFlag && e.setAttribute('data-xztitle', titleFlag);
        this.useSlot(slot, e);
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.register(e);
        return e;
    }
    /**获取页面标题 */
    static getPageTitle() {
        // 删除下载器在标题上添加的状态，以及剩余文件数量的数字
        let result = document.title
            .replace(/\[(↑|→|▶|↓|║|■|✓|☑| )\]/, '')
            .replace(/^ (\d+) /, '');
        // 如果开头有空格则去掉空格
        if (result.startsWith(' ')) {
            result = result.replace(/ */, '');
        }
        return result;
    }
    // 自定义的类型保护
    /**判断 Tags 类型 */
    static isArtworkTags(data) {
        return data.translation !== undefined;
    }
    /**判断作品数据是图像作品还是小说作品 */
    static isArtworkData(data) {
        return data.body.illustType !== undefined;
    }
    /**从作品数据里提取出 tag 列表
     *
     * 可选参数 type:
     *
     * 'origin' 默认值，获取原版 tag
     *
     * 'transl' 获取翻译后的 tag。只有图片作品有翻译，小说作品的 tag 没有翻译。如果某个 tag 没有翻译，则会保存它的原版 tag
     *
     * 'both' 同时获取原版 tag 和翻译后的 tag。此时可能会有重复的值，所以返回值做了去重处理。
     */
    static extractTags(data, type = 'origin') {
        const tags = [];
        const tagsTransl = [];
        const tagArr = data.body.tags.tags;
        for (const tagData of tagArr) {
            // 添加原版 tag
            tags.push(tagData.tag);
            // 添加翻译的 tag
            // 缺省使用原标签
            let useOriginTag = true;
            if (this.isArtworkTags(tagData)) {
                // 不管是什么语种的翻译结果，都保存在 en 属性里
                if (tagData.translation && tagData.translation.en) {
                    useOriginTag = false;
                    // 如果用户在 Pixiv 的页面语言是中文，则应用优化策略
                    // 如果翻译后的标签是纯英文，则判断原标签是否含有至少一部分中文，如果是则使用原标签
                    // 这是为了解决一些中文标签被翻译成英文的问题，如 原神 被翻译为 Genshin Impact
                    // 能代(アズールレーン) Noshiro (Azur Lane) 也会使用原标签
                    // 但是如果原标签里没有中文则依然会使用翻译后的标签，如 フラミンゴ flamingo
                    if (_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.htmlLangType === 'zh-cn' || _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.htmlLangType === 'zh-tw') {
                        const allEnglish = [].every.call(tagData.translation.en, function (s) {
                            return s.charCodeAt(0) < 128;
                        });
                        if (allEnglish) {
                            useOriginTag = this.chineseRegexp.test(tagData.tag);
                        }
                    }
                }
            }
            tagsTransl.push(useOriginTag ? tagData.tag : tagData.translation.en);
        }
        if (type === 'origin') {
            return tags;
        }
        else if (type === 'transl') {
            return Array.from(new Set(tagsTransl));
        }
        else {
            return Array.from(new Set(tags.concat(tagsTransl)));
        }
    }
    /**转换 thumb url
     *
     * 因为现在 pixiv 的多图作品的数据里，thumb url 只是第一张图的，后面的图片没有 thumb url 数据
     *
     * 此方法使用另一套缩略图 url，这样所有的图片都能够获得可用的缩略图 url
     */
    // 现在的多图作品，有些是所有图片都有 250px 的缩略图，此时作品数据里的 thubm url 含有 img-master，例如：
    // https://i.pximg.net/c/250x250_80_a2/img-master/img/2021/11/28/18/30/25/94433369_p0_square1200.jpg
    // 有些作品的 thumb url 里是 custom-thumb，例如：
    // https://i.pximg.net/c/250x250_80_a2/custom-thumb/img/2021/03/03/06/29/53/88179809_p0_custom1200.jpg
    // 这种只有第一张图有缩略图，后面的图片没有缩略图。对于这种情况，将其替换成所有图片都有缩略图的 url。（即第一种 url）
    static convertArtworkThumbURL(thumbURL, no) {
        return thumbURL
            .replace('250x250_80_a2/custom-thumb', '250x250_80_a2/img-master')
            .replace('custom1200', 'square1200')
            .replace('p0', 'p' + no);
    }
    static convertThumbURLTo540px(url) {
        // 'https://i.pximg.net/c/250x250_80_a2/custom-thumb/img/2021/10/26/00/00/08/93686259_p0_custom1200.jpg'
        // 'https://i.pximg.net/c/250x250_80_a2/img-master/img/2019/06/23/17/29/27/75369283_square1200.jpg'
        // 排行榜页面的图片 URL 如：
        // 'https://i.pximg.net/c/240x480/img-master/img/2022/08/01/17/59/39/100156836_p0_master1200.jpg'
        const test = url.match(this.convertThumbURLReg);
        if (!test || !test[1]) {
            return url;
        }
        // '2021/10/26/00/00/08/93686259_p0'
        // '2019/06/23/17/29/27/75369283'
        const array = test[1].split('/');
        const id_index = array.pop();
        const datetime = array.join('/');
        return `https://i.pximg.net/c/540x540_70/img-master/img/${datetime}/${id_index}_master1200.jpg`;
    }
    // 旧版本的 Result 数据中没有 index 属性，使用此方法进行兼容性处理
    static getResultIndex(data) {
        if (data.type === 0 || data.type === 1) {
            // 插画和漫画，从 id 里取出序号
            return parseInt(/\d*$/.exec(data.id)[0]);
        }
        else {
            // 其他作品类型只有 1 个文件所以值为 0
            return 0;
        }
    }
    static isPremium() {
        // 在 body 的一个 script 标签里包含有当前用户是否是会员的信息
        // premium: 'yes'
        // premium: 'no'
        const test = document.body.innerHTML.match(/premium: '(\w+)'/);
        if (test && test.length > 1) {
            return test[1] === 'yes';
        }
        return false;
    }
    // 在小说页面，以及某些特定页面里，不启用"显示更大的缩略图"和“横图占用二倍宽度”功能
    static notEnabledShowLargerThumb() {
        if (window.location.pathname.includes('/novel') ||
            window.location.pathname.includes('/ranking_area') ||
            window.location.hostname.includes('pixivision.net')) {
            return true;
        }
        return false;
    }
    // 传入作品 id，生成作品页面的超链接
    /**
     *
     * @param id 作品 id
     * @param artwork true 图像作品； false 小说作品。默认为图像作品
     * @returns 超链接（A 标签）
     */
    static createWorkLink(id, artwork = true) {
        const idNum = typeof id === 'number' ? id : Number.parseInt(id);
        const href = `https://www.pixiv.net/${artwork ? 'i' : 'n'}/${idNum}`;
        return `<a href="${href}" target="_blank">${id}</a>`;
    }
    // 传入用户 id，生成用户页面的超链接
    /**
     *
     * @param userID 用户 id
     * @returns 超链接（A 标签）
     */
    static createUserLink(userID) {
        const idNum = typeof userID === 'number' ? userID : Number.parseInt(userID);
        const href = `https://www.pixiv.net/users/${idNum}`;
        return `<a href="${href}" target="_blank">${idNum}</a>`;
    }
    /**替换 EPUB 文本里的特殊字符和换行符 */
    // 换行符必须放在最后处理，以免其 < 符号被替换
    static replaceEPUBText(str) {
        return str
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/&lt;br/g, '<br')
            .replace(/\n/g, '<br/>');
    }
    /** 在 zip 压缩包里查找类似于 000000.jpg 的标记，返回它后面的位置的下标
     *
     * @param zipFile  Zip 文件的内容
     * @param existingIndexList  可选传入一个已存在的索引列表。如果传入，那么这个方法在搜索 zip 文件里的图片时，不会查找已有索引的部分（也就是不会重复查找文件的前半部分），只会查找没有索引的部分
     * @returns number[] 返回一个索引列表的数组
     *
     */
    static getJPGContentIndex(zipFile, existingIndexList) {
        let indexList = [];
        if (existingIndexList && existingIndexList.length > 0) {
            indexList = existingIndexList;
        }
        // 每次查找时，开始的位置
        let offset = 0;
        // 循环的次数
        let loopTimes = 0;
        while (true) {
            // 如果当前偏移量的后面有已经查找到的索引，就不必重复查找了
            // 跳过这次循环，下次直接从已有的索引后面开始查找
            if (indexList[loopTimes] !== undefined && offset < indexList[loopTimes]) {
                offset = indexList[loopTimes];
                ++loopTimes;
                continue;
            }
            let data;
            if (offset === 0) {
                // 一开始从数据开头查找
                data = new Uint8Array(zipFile);
            }
            else {
                // 每次查找之后，从上次查找结束的位置开始查找
                // 这样可以避免重复查找前面的数据
                data = new Uint8Array(zipFile, offset);
            }
            // 查找以 jpg 文件名的标记，如 000000.jpg
            // 其实文件内容里也可能会出现符合条件的数据，只是概率很小
            // 有些文件名后面有 PK 标记，如 000000.jpgPK，需要排除，因为这是 zip 的文件目录，不是图片
            const index = data.findIndex((val, index2, array) => {
                // 0 0 0 . j p g P
                if (val === 48 &&
                    array[index2 + 1] === 48 &&
                    array[index2 + 2] === 48 &&
                    array[index2 + 6] === 46 &&
                    array[index2 + 7] === 106 &&
                    array[index2 + 8] === 112 &&
                    array[index2 + 9] === 103 &&
                    array[index2 + 10] !== 80) {
                    // const enc = new TextDecoder('utf-8')
                    // const buff = data.slice(index2, index2 + 10)
                    // console.log(enc.decode(buff))
                    return true;
                }
                return false;
            });
            if (index !== -1) {
                const fileContentStart = offset + index + 10;
                indexList[loopTimes] = fileContentStart;
                offset = fileContentStart;
                ++loopTimes;
            }
            else {
                return indexList;
            }
        }
    }
    static async extractImage(zipFile, indexList, target) {
        return new Promise(async (resolve, reject) => {
            const result = [];
            let i = 0;
            for (const index of indexList) {
                // 起始位置
                const start = index;
                // 截止下一个文件名之前
                // 删除不需要的数据：
                // 30 字节的是 zip 文件添加的数据，虽然没有实际影响，但还是去掉
                // 10 字节的是下一个 jpg 的文件名
                let end = indexList[i + 1] - 30 - 10;
                if (i === indexList.length - 1) {
                    // 如果是最后一个 jpg 文件，则截止到 zip 文件的结尾
                    // 这导致它会包含 zip 的目录数据，但是不会影响图片的显示
                    end = zipFile.byteLength;
                }
                const blob = new Blob([zipFile.slice(start, end)], {
                    type: 'image/jpeg',
                });
                if (target === 'ImageBitmap') {
                    const map = await createImageBitmap(blob);
                    result.push(map);
                }
                else if (target === 'img') {
                    const url = URL.createObjectURL(blob);
                    const img = await _utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.loadImg(url);
                    result.push(img);
                }
                ++i;
            }
            resolve(result);
        });
    }
    /**根据 illustType，返回作品类型的描述字符串 */
    // 主要用于储存进 idList
    static getWorkTypeString(illustType) {
        switch (parseInt(illustType.toString())) {
            case 0:
                return 'illusts';
            case 1:
                return 'manga';
            case 2:
                return 'ugoira';
            case 3:
                return 'novels';
            default:
                return 'unknown';
        }
    }
    /**根据作品类型字符串，返回对应的数字 */
    static getWorkType(workTypeString) {
        switch (workTypeString) {
            case 'illusts':
                return 0;
            case 'manga':
                return 1;
            case 'ugoira':
                return 2;
            case 'novels':
                return 3;
            default:
                return undefined;
        }
    }
    /**根据作品类型字符串，返回对应的数字。但是这里把插画、漫画、动图均返回 -1。
     * 这是因为某些时候无法确定一个图像作品到底属于哪一类型，所以用 -1 笼统的概括
     */
    static getWorkTypeVague(workTypeString) {
        switch (workTypeString) {
            case 'illusts':
            case 'manga':
            case 'ugoira':
                return -1;
            case 'novels':
                return 3;
            default:
                return undefined;
        }
    }
    /**如果一个作品是 AI 生成的，则返回特定的字符串标记
     *
     * 这个标记就是作品页面里和标签列表显示在一起的字符串
     */
    static getAIGeneratedMark(aiType) {
        if (aiType === 2) {
            return this.AIMark.get(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.htmlLangType);
        }
        return '';
    }
    static checkUserLogin() {
        // 如果有“登录”的超链接，则是未登录状态
        // 在桌面版和移动版网页里都有效
        const loginLink = document.querySelector('a[href^="/login"]');
        if (loginLink) {
            return false;
        }
        return true;
    }
    static rangeRandom(start, end) {
        const difference = end - start;
        const num = Math.ceil(Math.random() * difference);
        return start + num;
    }
    /**格式化数字，每千位添加一个逗号，返回结果字符串 */
    static numberToString(int) {
        let stringArray = Array.from(int.toString());
        let group = [];
        let index = stringArray.length;
        while (index > 0) {
            let array = [];
            for (let times = 0; times < 3; times++) {
                index >= 0 && array.push(stringArray[--index]);
            }
            group.push(array.reverse().join(''));
        }
        const result = group.reverse().join(',');
        return result;
    }
    static getXRestrictText(number) {
        return this.xRestrictMap.get(number);
    }
    static getAITypeText(number) {
        return this.AIType[number];
    }
}
Tools.chineseRegexp = /[一-龥]/;
Tools.convertThumbURLReg = /img\/(.*)_.*1200/;
Tools.AIMark = new Map([
    ['zh-cn', 'AI生成'],
    ['zh-tw', 'AI生成'],
    ['en', 'AI-generated'],
    ['ja', 'AI生成'],
    ['ko', 'AI 생성'],
    ['ru', 'сгенерированный ИИ'],
]);
Tools.xRestrictMap = new Map([
    [0, 'AllAges'],
    [1, 'R-18'],
    [2, 'R-18G'],
]);
Tools.AIType = ['Unknown', 'No', 'Yes'];



/***/ }),

/***/ "./src/ts/UnBookmarkWorks.ts":
/*!***********************************!*\
  !*** ./src/ts/UnBookmarkWorks.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   unBookmarkWorks: () => (/* binding */ unBookmarkWorks)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./API */ "./src/ts/API.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Log */ "./src/ts/Log.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Token__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Token */ "./src/ts/Token.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");








class UnBookmarkWorks {
    async start(list) {
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_取消收藏作品'));
        if (list.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
            return;
        }
        _store_States__WEBPACK_IMPORTED_MODULE_5__.states.busy = true;
        const total = list.length;
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前作品个数', total.toString()));
        // 尚不清楚 deleteBookmark 使用的 API 是否会被计入 429 限制里
        // 当操作的作品数量大于一页（48 个作品）时，使用慢速抓取
        const slowMode = total > 48;
        let progress = 0;
        for (const item of list) {
            try {
                await this.waitSlowMode(slowMode);
                await _API__WEBPACK_IMPORTED_MODULE_0__.API.deleteBookmark(item.bookmarkID, item.type, _Token__WEBPACK_IMPORTED_MODULE_4__.token.token);
            }
            catch (error) {
                // 处理自己收藏的作品时可能遇到错误。最常见的错误就是作品被删除了，获取作品数据时会产生 404 错误
                // 对于出错的作品直接跳过，不需要对其执行任何操作
                // 不过这种作品无法被删除，执行完毕后还是会留在收藏里
            }
            progress++;
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${progress} / ${total}`, 1, false);
        }
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_取消收藏作品') + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_完成');
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.success(msg, {
            position: 'topCenter',
        });
        _store_States__WEBPACK_IMPORTED_MODULE_5__.states.busy = false;
    }
    waitSlowMode(slowMode) {
        return new Promise((resolve) => {
            if (!slowMode) {
                return resolve();
            }
            else {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_6__.setTimeoutWorker.set(() => {
                    return resolve();
                }, _Config__WEBPACK_IMPORTED_MODULE_7__.Config.slowCrawlDealy);
            }
        });
    }
}
const unBookmarkWorks = new UnBookmarkWorks();



/***/ }),

/***/ "./src/ts/WorkThumbnail.ts":
/*!*********************************!*\
  !*** ./src/ts/WorkThumbnail.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   WorkThumbnail: () => (/* binding */ WorkThumbnail)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");

// 查找作品的缩略图，当鼠标进入、移出时等动作触发时执行回调函数
class WorkThumbnail {
    constructor() {
        /**作品缩略图的选择器 */
        // 选择器的元素必须含有作品的超链接（超链接可以在这个元素上，也可以在这个元素的子元素上）
        this.selectors = [];
        this.foundCallback = [];
        this.enterCallback = [];
        this.leaveCallback = [];
        this.clickCallback = [];
        this.bookmarkBtnCallback = [];
    }
    /**查找缩略图右下角的收藏按钮 */
    findBookmarkBtn(el) {
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            // 移动端的收藏按钮不是 button，其容器是 div.bookmark
            return el.querySelector('.bookmark');
        }
        else {
            // 桌面端的缩略图容器里只有 1 个 button，就是收藏按钮。目前还没有发现有多个 button 的情况
            if (el.querySelector('button svg[width="32"]')) {
                return el.querySelector('button');
            }
            // 旧版缩略图里，缩略图元素是 div._one-click-bookmark （例如：各种排行榜页面）
            return el.querySelector('div._one-click-bookmark');
        }
    }
    /**为作品缩略图绑定事件 */
    // 注意：在移动端页面，此时获取的 id 可能是空字符串。可以在执行回调时尝试再次获取 id
    bindEvents(el, id) {
        // 如果这个缩略图元素、或者它的直接父元素、或者它的直接子元素已经有标记，就跳过它
        // mouseover 这个标记名称不可以修改，因为它在 Pixiv Previewer 里硬编码了
        // https://github.com/xuejianxianzun/PixivBatchDownloader/issues/212
        if (el.dataset.mouseover) {
            return;
        }
        if (el.parentElement && el.parentElement.dataset.mouseover) {
            return;
        }
        if (el.firstElementChild &&
            el.firstElementChild.dataset.mouseover) {
            return;
        }
        // 当对一个缩略图元素绑定事件时，在它上面添加标记
        // 添加标记的目的是为了减少事件重复绑定的情况发生
        ;
        el.dataset.mouseover = '1';
        this.foundCallback.forEach((cb) => cb(el, id));
        el.addEventListener('mouseenter', (ev) => {
            this.enterCallback.forEach((cb) => cb(el, id, ev));
        });
        el.addEventListener('mouseleave', (ev) => {
            this.leaveCallback.forEach((cb) => cb(el, ev));
        });
        el.addEventListener(_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? 'touchend' : 'click', (ev) => {
            this.clickCallback.forEach((cb) => cb(el, id, ev));
        }, false);
        // 查找作品缩略图右下角的收藏按钮
        const bmkBtn = this.findBookmarkBtn(el);
        if (!!bmkBtn) {
            bmkBtn.addEventListener(_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? 'touchend' : 'click', (ev) => {
                this.bookmarkBtnCallback.forEach((cb) => cb(el, id, bmkBtn, ev));
            });
        }
    }
    /**使用监视器，让未来添加的作品缩略图也绑定上事件 */
    createObserver(target) {
        const observer = new MutationObserver((records) => {
            for (const record of records) {
                if (record.addedNodes.length > 0) {
                    // 遍历被添加的元素
                    for (const newEl of record.addedNodes) {
                        this.findThumbnail(newEl);
                    }
                }
            }
        });
        observer.observe(target, {
            childList: true,
            subtree: true,
        });
    }
    /**添加下载器寻找到一个作品缩略图时的回调。
     * 注意：这个回调只会执行一次，因为它不是根据用户操作的事件触发的。
     *
     * 回调函数会接收到 2 个参数：
     *
     * @el 作品缩略图的元素
     *
     * @id 作品 id（在移动端页面里，此时传递的 id 可能是空字符串 ''）
     */
    onFound(cb) {
        this.foundCallback.push(cb);
    }
    /**添加鼠标进入作品缩略图时的回调。
     *
     * 回调函数会接收到 3 个参数：
     *
     * @el 作品缩略图的元素
     *
     * @id 作品 id
     *
     * @ev Event 对象
     */
    onEnter(cb) {
        this.enterCallback.push(cb);
    }
    /**添加鼠标离开作品缩略图时的回调。
     *
     * 回调函数会接收到 2 个参数：
     *
     * @el 作品缩略图的元素
     *
     * @ev Event 对象
     *
     * 没有 id 参数，因为鼠标离开时的 id 就是鼠标进入时的 id
     */
    onLeave(cb) {
        this.leaveCallback.push(cb);
    }
    /**添加鼠标点击作品缩略图时的回调。
     *
     * 回调函数会接收到 3 个参数：
     *
     * @el 作品缩略图的元素
     *
     * @id 作品 id
     *
     * @ev Event 对象
     */
    onClick(cb) {
        this.clickCallback.push(cb);
    }
    /**添加鼠标点击缩略图里的收藏按钮时的回调。
     *
     * 回调函数会接收到 4 个参数：
     *
     * @el 作品缩略图的元素
     *
     * @id 作品 id
     *
     * @btn 收藏按钮
     *
     * @ev Event 对象
     */
    onClickBookmarkBtn(cb) {
        this.bookmarkBtnCallback.push(cb);
    }
}



/***/ }),

/***/ "./src/ts/WorkToolBar.ts":
/*!*******************************!*\
  !*** ./src/ts/WorkToolBar.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   workToolBar: () => (/* binding */ workToolBar)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./PageType */ "./src/ts/PageType.ts");



// 在作品页面里，获取作品内容下方包含点赞、收藏、分享等按钮的工具栏元素
// 注意：处于自己的作品页面里的时候，是没有收藏和点赞按钮的
class WorkToolBar {
    constructor() {
        this.flag = 'xzToolbar'; // 在工具栏上添加的标记
        this.timer = 0; // 获取元素用的定时器
        this.callbackList = [];
        this.init();
        this.bindEvents();
    }
    async init() {
        this.toolbar = undefined;
        this.pixivBMKDiv = undefined;
        this.likeBtn = undefined;
        window.clearInterval(this.timer);
        if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Artwork &&
            _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Novel) {
            return;
        }
        this.timer = window.setInterval(() => {
            _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? this.getElementsOnMobile() : this.getElementsOnDesktop();
        }, 300);
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch, () => {
            this.init();
        });
    }
    async getElementsOnDesktop() {
        // 获取工具栏
        const toolbarParent = document.querySelectorAll('main > section');
        for (const el of toolbarParent) {
            const test = el.querySelector('div>section');
            if (test) {
                const toolbar = test;
                if (!toolbar.classList.contains(this.flag)) {
                    this.toolbar = toolbar;
                    toolbar.classList.add(this.flag);
                    break;
                }
            }
        }
        if (!this.toolbar) {
            return;
        }
        // 获取心形收藏按钮的 div
        const total = this.toolbar.childElementCount;
        // 心形收藏按钮是倒数第二个（从左往右数第二个）
        this.pixivBMKDiv =
            this.toolbar.childNodes[total - 2] || undefined;
        // 获取点赞按钮
        const btnList = this.toolbar.querySelectorAll('button');
        this.likeBtn = btnList[btnList.length - 1] || undefined;
        // 全部获取完毕
        if (this.pixivBMKDiv && this.likeBtn) {
            window.clearInterval(this.timer);
            this.executionCB();
        }
    }
    async getElementsOnMobile() {
        // 获取工具栏
        const toolbar = document.querySelector('.work-interactions');
        if (!toolbar) {
            return;
        }
        this.toolbar = toolbar;
        // 在移动端不要给工具栏添加自定义 class 名，因为切换页面时元素没重新生成，class 还在
        const divs = toolbar.querySelectorAll('div');
        if (divs.length !== 4) {
            return;
        }
        // 只在正常模式下（有 4 个按钮）时工作
        // 如果在自己的作品页面里，就只有 1 个分享按钮
        // 获取心形收藏按钮的 div
        this.pixivBMKDiv = divs[1];
        // 获取点赞按钮
        this.likeBtn = divs[0];
        // 全部获取完毕
        if (this.pixivBMKDiv && this.likeBtn) {
            window.clearInterval(this.timer);
            this.executionCB();
        }
    }
    /**注册回调函数
     *
     * 当 WorkToolBar 模块获取到了所有需要的元素时，会执行回调函数，并把以下元素作为参数传入：
     *
     * toolbar, pixivBMKDiv, likeBtn */
    register(cb) {
        this.callbackList.push(cb);
    }
    executionCB() {
        this.callbackList.forEach((cb) => cb(this.toolbar, this.pixivBMKDiv, this.likeBtn));
    }
}
const workToolBar = new WorkToolBar();



/***/ }),

/***/ "./src/ts/crawlArtworkPage/CrawlRecommendWorks.ts":
/*!********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/CrawlRecommendWorks.ts ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");





// 在作品页面里，点击收藏按钮后会出现推荐作品。这个模块用于抓取推荐作品
class CrawlRecommendWorks {
    constructor() {
        this.linkSelector = '.gtm-illust-recommend-thumbnail-link';
        this.found = false;
        this.IDList = [];
        this.timer = window.setInterval(() => {
            this.foundTarget();
        }, 300);
        this.bindEvents();
    }
    get checkSelector() {
        return _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? '.recommend-slidein-booster' : 'figcaption h2';
    }
    get datasetFlag() {
        return _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? 'gtmRecommendIllustId' : 'gtmValue';
    }
    foundTarget() {
        if (this.found || _PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.list.Artwork) {
            return;
        }
        const titleBar = document.querySelector(this.checkSelector);
        if (titleBar) {
            this.found = true;
            window.clearTimeout(this.timer);
            // 等待一段时间再获取作品超链接，因为立刻获取的话可能还未生成
            // 其实在 PC 端页面是可以立即获取到的，但是在移动端页面需要等待较长时间，500ms 不够用
            window.setTimeout(() => {
                this.readyCrawl();
            }, _Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile ? 1000 : 100);
        }
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch, () => {
            // 页面切换后，页面元素可能还没来得及变化，所以需要等待一段时间后再开始查找
            // 如果立即查找，那么经常会查找到已经存在的推荐列表，于是就会立即停止查找
            window.setTimeout(() => {
                this.found = false;
                this.IDList = [];
                window.clearTimeout(this.timer);
                this.timer = window.setInterval(() => {
                    this.foundTarget();
                }, 300);
            }, 600);
        });
    }
    readyCrawl() {
        const allLinks = document.querySelectorAll(this.linkSelector);
        for (const a of allLinks) {
            const id = a.dataset[this.datasetFlag];
            if (id) {
                this.IDList.push({
                    id,
                    type: 'illusts',
                });
            }
        }
        this.addDownloadBtn();
    }
    addDownloadBtn() {
        if (this.IDList.length === 0) {
            return;
        }
        const target = document.querySelector(this.checkSelector);
        if (!target) {
            return;
        }
        const btn = document.createElement('button');
        btn.textContent = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载推荐作品');
        btn.classList.add('blueTextBtn');
        btn.addEventListener('click', () => {
            _store_States__WEBPACK_IMPORTED_MODULE_4__.states.quickCrawl = true;
            // 传递 ID 列表时需要复制一份，因为如果直接传递变量，那么这个数组会在抓取之后被清空
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('crawlIdList', [...this.IDList]);
        });
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            // 在移动端页面，需要把按钮添加到第一个子元素后面，因为第一个子元素才是“推荐作品”的标题
            target.firstElementChild.insertAdjacentElement('afterend', btn);
        }
        else {
            // 修改标题的父元素样式，以便让标题和下载按钮可以并排显示
            target.parentElement.style.display = 'flex';
            target.parentElement.style.alignItems = 'center';
            target.insertAdjacentElement('afterend', btn);
        }
    }
}
new CrawlRecommendWorks();


/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitAreaRankingPage.ts":
/*!********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitAreaRankingPage.ts ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitAreaRankingPage: () => (/* binding */ InitAreaRankingPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
// 初始化地区排行榜页面






class InitAreaRankingPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    initAny() { }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取本页作品', '_抓取本页作品Title').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.hideOption([1]);
    }
    async getIdList() {
        const allPicArea = document.querySelectorAll('.ranking-item>.work_wrapper');
        for (const el of allPicArea) {
            const img = el.querySelector('._thumbnail');
            // img.dataset.type 全都是 "illust"，因此不能用来区分作品类型
            // 提取出 tag 列表
            const id = img.dataset.id;
            const tags = img.dataset.tags.split(' ');
            // 有的作品没有收藏按钮，点进去之后发现这个作品已经被删除了，只是排行榜里没有及时更新。这样的作品没有收藏按钮。
            const bookmarkBtn = el.querySelector('._one-click-bookmark');
            const bookmarked = bookmarkBtn
                ? bookmarkBtn.classList.contains('on')
                : false;
            const filterOpt = {
                id: id,
                tags: tags,
                bookmarkData: bookmarked,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_4__.filter.check(filterOpt)) {
                const id = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getIllustId(el.querySelector('a').href);
                _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.push({
                    type: 'illusts',
                    id,
                });
            }
        }
        this.getIdListFinished();
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitArtworkPage.ts":
/*!****************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitArtworkPage.ts ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitArtworkPage: () => (/* binding */ InitArtworkPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _CrawlRecommendWorks__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./CrawlRecommendWorks */ "./src/ts/crawlArtworkPage/CrawlRecommendWorks.ts");
//初始化 artwork 作品页










class InitArtworkPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.crawlDirection = 0; // 抓取方向，指示抓取新作品还是旧作品
        /*
        -1 抓取新作品
        0 不设置抓取方向
        1 抓取旧作品
        */
        this.crawlRelated = false; // 是否下载相关作品
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_从本页开始抓取new').addEventListener('click', () => {
            this.crawlDirection = -1;
            this.readyCrawl();
        });
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_从本页开始抓取old').addEventListener('click', () => {
            this.crawlDirection = 1;
            this.readyCrawl();
        });
        const downRelatedBtn = _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取相关作品');
        downRelatedBtn.addEventListener('click', () => {
            this.crawlRelated = true;
            this.readyCrawl();
        }, false);
    }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.clearSlot('otherBtns');
    }
    getWantPage() {
        // 检查下载页数的设置
        if (!this.crawlRelated) {
            const crawlAllTip = this.crawlDirection === -1
                ? _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始抓取new')
                : _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始抓取old');
            this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x个'), crawlAllTip);
        }
        else {
            // 相关作品的提示
            this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载x个相关作品'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载所有相关作品'));
        }
    }
    nextStep() {
        // 下载相关作品
        if (this.crawlRelated) {
            this.getRelatedList();
        }
        else {
            // 向前向后下载
            this.getIdList();
        }
    }
    async getIdList() {
        let type = ['illusts', 'manga'];
        let idList = await _API__WEBPACK_IMPORTED_MODULE_6__.API.getUserWorksByType(_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getUserId(), type);
        // 储存符合条件的 id
        let nowId = parseInt(_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getIllustId(window.location.href));
        idList.forEach((id) => {
            let idNum = parseInt(id.id);
            // 新作品
            if (idNum >= nowId && this.crawlDirection === -1) {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push(id);
            }
            else if (idNum <= nowId && this.crawlDirection === 1) {
                // 旧作品
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push(id);
            }
        });
        // 当设置了下载个数时，进行裁剪
        if (this.crawlNumber !== -1) {
            // 新作品 升序排列
            if (this.crawlDirection === -1) {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_8__.Utils.sortByProperty('id')).reverse();
            }
            else {
                // 旧作品 降序排列
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_8__.Utils.sortByProperty('id'));
            }
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList = _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.splice(0, this.crawlNumber);
        }
        this.getIdListFinished();
    }
    // 下载相关作品时使用
    async getRelatedList() {
        let data = await _API__WEBPACK_IMPORTED_MODULE_6__.API.getRelatedData(_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getIllustId());
        // 相关作品的列表由两部分构成，所以要组合起来
        let ids = [];
        for (const illust of data.body.illusts) {
            if (illust.isAdContainer) {
                continue;
            }
            ids.push(illust.id);
        }
        ids = ids.concat(data.body.nextIds);
        // 当设置了下载个数时，进行裁剪
        if (this.crawlNumber !== -1) {
            ids = ids.slice(0, this.crawlNumber);
        }
        for (const id of ids) {
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push({
                type: 'illusts',
                id,
            });
        }
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_相关作品抓取完毕', _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length.toString()));
        this.getIdListFinished();
    }
    resetGetIdListStatus() {
        this.crawlDirection = 0; // 解除下载方向的标记
        this.crawlRelated = false; // 解除下载相关作品的标记
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitArtworkSeriesPage.ts":
/*!**********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitArtworkSeriesPage.ts ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitArtworkSeriesPage: () => (/* binding */ InitArtworkSeriesPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
// 初始化插画/漫画的系列作品页面











class InitArtworkSeriesPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.seriesId = '';
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        this.maxCount = 100;
        _setting_Options__WEBPACK_IMPORTED_MODULE_5__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInputGreater0(this.maxCount, true);
    }
    nextStep() {
        // 设置起始页码
        const p = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLSearchField(location.href, 'p');
        this.startpageNo = parseInt(p) || 1;
        // 获取系列 id
        this.seriesId = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLPathField(window.location.pathname, 'series');
        this.getIdList();
    }
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let p = this.startpageNo + this.listPageFinished;
        const data = await _API__WEBPACK_IMPORTED_MODULE_2__.API.getSeriesData(this.seriesId, p);
        this.listPageFinished++;
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        // 保存本页面的作品的 id 列表
        const idList = [];
        for (const info of data.body.page.series) {
            idList.push(info.workId);
        }
        // data.body.page.series 里的才是本页面的作品，illust 里则不同，有时它的作品数量比页面上的更多
        // 从 illust 里查找 id 对应的数据，进行过滤
        for (const work of data.body.thumbnails.illust) {
            if (!idList.includes(work.id)) {
                continue;
            }
            if (work.isAdContainer) {
                continue;
            }
            // 过滤器进行检查
            const filterOpt = {
                aiType: work.aiType,
                id: work.id,
                tags: work.tags,
                bookmarkData: !!work.bookmarkData,
                width: work.pageCount === 1 ? work.width : 0,
                height: work.pageCount === 1 ? work.height : 0,
                workType: work.illustType,
                userId: work.userId,
                createDate: work.createDate,
                xRestrict: work.xRestrict,
            };
            // 因为这个 api 的 illust 数据可能是插画也可能是漫画，所以 type 是 unknown
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check(filterOpt)) {
                _store_Store__WEBPACK_IMPORTED_MODULE_7__.store.idList.push({
                    type: 'illusts',
                    id: work.id,
                });
            }
        }
        // 如果 data.body.page.series 为空，就是到了最后一页
        const endFlag = data.body.page.series.length === 0;
        // 抓取完毕
        if (endFlag ||
            p >= this.maxCount ||
            this.listPageFinished === this.crawlNumber) {
            _Log__WEBPACK_IMPORTED_MODULE_8__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_列表页抓取完成'));
            this.getIdListFinished();
        }
        else {
            // 继续抓取
            _Log__WEBPACK_IMPORTED_MODULE_8__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_列表页抓取进度', this.listPageFinished.toString()), 1, false);
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.listPageFinished = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitBookmarkDetailPage.ts":
/*!***********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitBookmarkDetailPage.ts ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitBookmarkDetailPage: () => (/* binding */ InitBookmarkDetailPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
// 初始化 bookmark_detail 页面






class InitBookmarkDetailPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取相似图片', '_抓取相似图片').addEventListener('click', () => {
            this.readyCrawl();
        }, false);
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_想要获取多少个作品',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInputGreater0(this.maxCount, false);
    }
    // 获取相似的作品列表
    async getIdList() {
        let data = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getRecommenderData(_Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getIllustId(), this.crawlNumber);
        for (const id of data.recommendations) {
            _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.push({
                type: 'illusts',
                id: id.toString(),
            });
        }
        this.getIdListFinished();
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitDiscoverPage.ts":
/*!*****************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitDiscoverPage.ts ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitDiscoverPage: () => (/* binding */ InitDiscoverPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
// 初始化发现页面





class InitDiscoverPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取当前作品', '_抓取当前作品Title').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.hideOption([1]);
    }
    getWantPage() { }
    getIdList() {
        // 在发现页面，直接获取页面上显示的作品，不需要获取列表页
        if (location.pathname.includes('/novel')) {
            // 小说页面
            const allWork = document.querySelectorAll('.gtm-novel-work-recommend-link');
            allWork.forEach((div) => {
                const a = div.querySelector('a');
                if (a) {
                    const id = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getNovelId(a.href);
                    _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push({
                        type: 'novels',
                        id,
                    });
                }
            });
        }
        else {
            // 插画漫画页面
            const allLink = document.querySelectorAll('div[width="184"]>a');
            // 获取已有作品的 id
            allLink.forEach((a) => {
                const id = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getIllustId(a.href);
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push({
                    type: 'illusts',
                    id,
                });
            });
        }
        this.getIdListFinished();
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitNewArtworkPage.ts":
/*!*******************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitNewArtworkPage.ts ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitNewArtworkPage: () => (/* binding */ InitNewArtworkPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化 本站的最新作品 artwork 页面













class InitNewArtworkPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.option = this.resetOption();
        this.limitMax = 20; // 每次请求的数量最大是 20
        this.fetchCount = 0; // 已请求的作品数量
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_下载大家的新作品').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_想要获取多少个作品',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInputGreater0(this.maxCount, false);
    }
    nextStep() {
        this.setSlowCrawl();
        this.initFetchURL();
        this.getIdList();
    }
    resetOption() {
        return {
            lastId: '0',
            limit: '20',
            type: '',
            r18: '',
        };
    }
    // 组织要请求的 url
    initFetchURL() {
        this.option = this.resetOption();
        if (this.crawlNumber < this.limitMax) {
            this.option.limit = this.crawlNumber.toString();
        }
        else {
            this.option.limit = this.limitMax.toString();
        }
        // 当前页面的作品类型，默认是 illust
        this.option.type =
            _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLSearchField(location.href, 'type') || 'illust';
        // 是否是 R18 模式
        this.option.r18 = (location.href.includes('_r18.php') || false).toString();
    }
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let data;
        try {
            data = await _API__WEBPACK_IMPORTED_MODULE_5__.API.getNewIllustData(this.option);
        }
        catch (error) {
            this.getIdList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let useData = data.body.illusts;
        for (const nowData of useData) {
            // 抓取够了指定的数量
            if (this.fetchCount + 1 > this.crawlNumber) {
                break;
            }
            else {
                this.fetchCount++;
            }
            // 排除广告信息
            if (nowData.isAdContainer) {
                continue;
            }
            const filterOpt = {
                aiType: nowData.aiType,
                id: nowData.id,
                width: nowData.pageCount === 1 ? nowData.width : 0,
                height: nowData.pageCount === 1 ? nowData.height : 0,
                pageCount: nowData.pageCount,
                bookmarkData: nowData.bookmarkData,
                workType: nowData.illustType,
                tags: nowData.tags,
                userId: nowData.userId,
                createDate: nowData.createDate,
                xRestrict: nowData.xRestrict,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_4__.filter.check(filterOpt)) {
                _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.idList.push({
                    type: _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getWorkTypeString(nowData.illustType),
                    id: nowData.id,
                });
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_新作品进度', this.fetchCount.toString()), 1, false);
        // 抓取完毕
        if (this.fetchCount >= this.crawlNumber ||
            this.fetchCount >= this.maxCount) {
            _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_开始获取作品页面'));
            this.getIdListFinished();
            return;
        }
        // 继续抓取
        this.option.lastId = data.body.lastId;
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.slowCrawlMode) {
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(() => {
                this.getIdList();
            }, _Config__WEBPACK_IMPORTED_MODULE_11__.Config.slowCrawlDealy);
        }
        else {
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.fetchCount = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitPixivisionPage.ts":
/*!*******************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitPixivisionPage.ts ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitPixivisionPage: () => (/* binding */ InitPixivisionPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
// 初始化 pixivision 页面






class InitPixivisionPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    addCrawlBtns() {
        const typeA = document.querySelector('a[data-gtm-action=ClickCategory]');
        const type = typeA.dataset.gtmLabel;
        if (type === 'illustration' || type === 'manga' || type === 'cosplay') {
            // 在插画、漫画、cosplay类型的页面上创建下载功能
            _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取该页面的图片').addEventListener('click', () => {
                this.readyCrawl();
            }, false);
        }
    }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.hideOption([
            1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 16, 18, 19, 21, 22, 23, 24, 26,
            27, 28, 30, 31, 33, 34, 35, 36, 37, 38, 39, 40, 42, 43, 44, 46, 47, 48,
            49, 50, 51, 54, 55, 56, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
            70, 71, 72, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87,
        ]);
    }
    nextStep() {
        this.getPixivision();
    }
    // 保存要下载的图片的信息
    addResult(id, url, ext) {
        _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.addResult({
            id: id,
            idNum: Number.parseInt(id),
            original: url,
            ext: ext,
        });
    }
    async getPixivision() {
        const a = document.querySelector('a[data-gtm-action=ClickCategory]');
        const type = a.dataset.gtmLabel;
        if (type === 'illustration') {
            // 插画页面，需要对图片进行测试获取原图 url
            const imageList = document.querySelectorAll('.am__work__main img');
            const urls = Array.from(imageList).map((el) => {
                return el.src
                    .replace('c/768x1200_80/img-master', 'img-original')
                    .replace('_master1200', '');
            });
            for (const url of urls) {
                let arr = url.split('/');
                const id = arr[arr.length - 1].split('.')[0].split('_')[0]; // 作品id，尝试提取出数字部分
                await this.testExtName(url, id);
            }
            this.crawlFinished();
        }
        else {
            // 漫画和 cosplay ，直接保存页面上的图片
            let selector = '';
            if (type === 'manga') {
                selector = '.am__work__illust';
            }
            else if (type === 'cosplay') {
                selector = '.fab__image-block__image img';
            }
            // 把图片url添加进数组
            const imageList = document.querySelectorAll(selector);
            Array.from(imageList).forEach((el) => {
                const url = el.src;
                if (url !== 'https://i.pximg.net/imgaz/upload/20170407/256097898.jpg') {
                    // 跳过Cure的logo图片
                    // 漫画页面的图片 url 如：
                    // https://i.pximg.net/c/768x1200_80/img-master/img/2017/06/19/01/08/28/63457814_p0_master1200.jpg
                    // cosplay 页面的 ur 如：
                    // https://i.pximg.net/imgaz/upload/20170808/670930758.jpg
                    const arr = url.split('/');
                    const id = arr[arr.length - 1].split('.')[0].split('_')[0]; // 作品id，尝试提取出数字部分
                    const extTest = arr[arr.length - 1].match(/\.(.*$)/); // 扩展名，不带点 .
                    let ext = 'jpg';
                    if (extTest && extTest.length > 1) {
                        ext = extTest[1];
                    }
                    this.addResult(id, url, ext);
                }
            });
            this.crawlFinished();
        }
    }
    // 通过加载图片来判断图片的后缀名。pixivision 页面直接获取的图片后缀都是 jpg 的
    async testExtName(url, id) {
        let ext = 'jpg'; // 默认为 jpg
        await _utils_Utils__WEBPACK_IMPORTED_MODULE_5__.Utils.loadImg(url).catch(() => {
            // 如果图片加载失败则把后缀改为 png
            url = url.replace('.jpg', '.png');
            ext = 'png';
        });
        this.addResult(id, url, ext);
        this.logResultNumber();
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitRankingArtworkPage.ts":
/*!***********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitRankingArtworkPage.ts ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitRankingArtworkPage: () => (/* binding */ InitRankingArtworkPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
// 初始化 artwork 排行榜页面












class InitRankingArtworkPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.pageCount = 10; // 排行榜的页数
        this.option = this.resetOption();
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取本排行榜作品', '_抓取本排行榜作品Title').addEventListener('click', () => {
            this.readyCrawl();
        });
        // 判断当前页面是否有“首次登场”标记
        const debutModes = ['daily', 'daily_r18', 'rookie', ''];
        const mode = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'mode');
        if (debutModes.includes(mode)) {
            _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取首次登场的作品', '_抓取首次登场的作品Title').addEventListener('click', () => {
                _store_States__WEBPACK_IMPORTED_MODULE_10__.states.debut = true;
                this.readyCrawl();
            });
        }
    }
    initAny() {
        // 抓取完成后，复位 debut 标记
        // 因为 debut 只在抓取阶段被过滤器使用，所以抓取完成后就可以复位
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.crawlComplete, () => {
            _store_States__WEBPACK_IMPORTED_MODULE_10__.states.debut = false;
        });
    }
    setFormOption() {
        // 个数/页数选项的提示
        this.maxCount = 500;
        _setting_Options__WEBPACK_IMPORTED_MODULE_6__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_想要获取多少个作品',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    resetOption() {
        return { mode: 'daily', p: 1, worksType: '', date: '' };
    }
    setPartNum() {
        // 设置页数。排行榜页面一页有50张作品，当页面到达底部时会加载下一页
        if (location.pathname.includes('r18g')) {
            // r18g 只有1个榜单，固定1页
            this.pageCount = 1;
        }
        else if (location.pathname.includes('_r18')) {
            // r18 模式，这里的6是最大值，有的排行榜并没有6页
            this.pageCount = 6;
        }
        else {
            // 普通模式，这里的10也是最大值。如果实际没有10页，则在检测到404页面的时候停止抓取下一页
            this.pageCount = 10;
        }
    }
    getWantPage() {
        this.listPageFinished = 0;
        // 检查下载页数的设置
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_下载排行榜前x个作品'), _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_向下获取所有作品'));
        // 如果设置的作品个数是 -1，则设置为下载所有作品
        if (this.crawlNumber === -1) {
            this.crawlNumber = 500;
        }
    }
    nextStep() {
        // 设置 option 信息
        // mode 一定要有值，其他字段不需要一定有值
        this.option = this.resetOption();
        this.option.mode = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'mode') || 'daily';
        this.option.worksType = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'content');
        this.option.date = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'date');
        this.startpageNo = 1;
        this.setPartNum();
        this.getIdList();
    }
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        this.option.p = this.startpageNo + this.listPageFinished;
        // 发起请求，获取作品列表
        let data;
        try {
            data = await _API__WEBPACK_IMPORTED_MODULE_2__.API.getRankingData(this.option);
        }
        catch (error) {
            if (error.status === 404) {
                // 如果发生了404错误，则中断抓取，直接下载已有部分。因为可能确实没有下一部分了
                console.log('404错误，直接下载已有部分');
                this.getIdListFinished();
            }
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        this.listPageFinished++;
        const contents = data.contents; // 取出作品信息列表
        for (const data of contents) {
            // 检查是否已经抓取到了指定数量的作品
            if (data.rank > this.crawlNumber) {
                return this.getIdListFinished();
            }
            const pageCount = parseInt(data.illust_page_count);
            // 目前这个数据里并没有包含收藏数量，所以在这里没办法检查收藏数量要求
            const filterOpt = {
                id: data.illust_id,
                workType: parseInt(data.illust_type),
                tags: data.tags,
                pageCount: pageCount,
                bookmarkData: data.is_bookmarked,
                width: pageCount === 1 ? data.width : 0,
                height: pageCount === 1 ? data.height : 0,
                yes_rank: data.yes_rank,
                userId: data.user_id.toString(),
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_7__.filter.check(filterOpt)) {
                _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.setRankList(data.illust_id.toString(), data.rank);
                _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.idList.push({
                    type: _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.getWorkTypeString(data.illust_type),
                    id: data.illust_id.toString(),
                });
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_9__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_排行榜进度', this.listPageFinished.toString()), 1, false);
        // 抓取完毕
        if (this.listPageFinished === this.pageCount) {
            this.getIdListFinished();
        }
        else {
            // 继续抓取
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.listPageFinished = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlArtworkPage/InitSearchArtworkPage.ts":
/*!**********************************************************!*\
  !*** ./src/ts/crawlArtworkPage/InitSearchArtworkPage.ts ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitSearchArtworkPage: () => (/* binding */ InitSearchArtworkPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _pageFunciton_DeleteWorks__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../pageFunciton/DeleteWorks */ "./src/ts/pageFunciton/DeleteWorks.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _pageFunciton_FastScreen__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../pageFunciton/FastScreen */ "./src/ts/pageFunciton/FastScreen.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../pageFunciton/BookmarkAllWorks */ "./src/ts/pageFunciton/BookmarkAllWorks.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../store/IdListWithPageNo */ "./src/ts/store/IdListWithPageNo.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ../Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _crawlMixedPage_CrawlTagList__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! ../crawlMixedPage/CrawlTagList */ "./src/ts/crawlMixedPage/CrawlTagList.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_22__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_23__ = __webpack_require__(/*! ../download/DownloadOnClickBookmark */ "./src/ts/download/DownloadOnClickBookmark.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_24__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化 artwork 搜索页

























class InitSearchArtworkPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.worksWrapSelector = '#root section ul';
        this.listClass = 'searchList';
        this.multipleClass = 'multiplePart';
        this.ugoiraClass = 'ugoiraPart';
        this.addBMKBtnClass = 'bmkBtn';
        this.bookmarkedClass = 'bookmarked';
        this.countSelector = 'section h3+div span';
        this.worksType = '';
        this.option = {};
        this.worksNoPerPage = 60; // 每个页面有多少个作品
        this.needCrawlPageCount = 0; // 需要抓取多少个列表页面
        this.sendCrawlTaskCount = 0; // 发送抓取请求之前会自增，用于计算要抓取的页码。不是请求完成后自增
        this.allOption = [
            'order',
            'type',
            'wlt',
            'wgt',
            'hlt',
            'hgt',
            'ratio',
            'tool',
            's_mode',
            'mode',
            'scd',
            'ecd',
            'blt',
            'bgt',
            'work_lang',
            'ai_type',
        ];
        this.resultMeta = []; // 每次“开始筛选”完成后，储存当时所有结果，以备“在结果中筛选”使用
        this.worksWrap = null;
        this.deleteId = 0; // 手动删除时，要删除的作品的 id
        this.causeResultChange = ['firstFewImagesSwitch', 'firstFewImages']; // 这些选项变更时，可能会导致结果改变。但是过滤器 filter 不会检查，所以需要单独检测它的变更，手动处理
        this.crawlStartBySelf = false; // 这次抓取是否是由当前页面的“开始抓取”按钮发起的
        this.previewCount = 0; // 共显示了多少个作品的预览图
        this.showPreviewLimitTip = false; // 当预览数量达到上限时显示一次提示
        // 储存预览搜索结果的元素
        this.workPreviewBuffer = document.createDocumentFragment();
        this.tipEmptyResult = _utils_Utils__WEBPACK_IMPORTED_MODULE_15__.Utils.debounce(() => {
            _Log__WEBPACK_IMPORTED_MODULE_9__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取被限制时返回空结果的提示'));
        }, 1000);
        this.onSettingChange = (event) => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList) {
                return;
            }
            const data = event.detail.data;
            if (this.causeResultChange.includes(data.name)) {
                if (_store_Store__WEBPACK_IMPORTED_MODULE_8__.store.result.length > 0) {
                    this.reAddResult();
                    _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('resultChange');
                }
            }
        };
        // 抓取完成后，保存结果的元数据，并重新添加抓取结果
        this.onCrawlFinish = () => {
            // 有些操作也会触发抓取完毕的事件，但不应该调整搜索页面的结果。
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList || _store_States__WEBPACK_IMPORTED_MODULE_14__.states.quickCrawl) {
                return;
            }
            if (!this.crawlStartBySelf) {
                return;
            }
            this.resultMeta = [..._store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta];
            // 在搜索页面抓取完毕之后，作品数据会按照收藏数量排序。所以这里需要清空之前的预览，重新生成预览
            this.clearPreview();
            this.reAddResult();
            this.showPreview();
            // 解绑创建作品元素的事件
            window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.addResult, this.createPreview);
            this.crawlStartBySelf = false;
            setTimeout(() => {
                _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('worksUpdate');
            }, 0);
        };
        // 显示抓取到的作品数量
        this.showCount = () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList || !_setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.previewResult) {
                return;
            }
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.previewResult && this.countEl) {
                const count = this.resultMeta.length || _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta.length;
                this.countEl.textContent = count.toString();
            }
        };
        // 生成抓取结果对应的作品元素
        this.createPreview = (event) => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList) {
                return;
            }
            if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.previewResult || !this.worksWrap) {
                return;
            }
            // 检查显示的预览数量是否达到上限
            if (this.previewCount >= _setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.previewResultLimit) {
                if (!this.showPreviewLimitTip) {
                    const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_预览搜索结果的数量达到上限的提示');
                    _Log__WEBPACK_IMPORTED_MODULE_9__.log.warning(msg);
                    _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.warning(msg);
                    this.showPreviewLimitTip = true;
                }
                return;
            }
            this.previewCount++;
            const data = event.detail.data;
            let r18Text = '';
            if (data.xRestrict === 1) {
                r18Text = 'R-18';
            }
            if (data.xRestrict === 2) {
                r18Text = 'R-18G';
            }
            let r18HTML = r18Text
                ? `
      <div class="r18Part">
        <div class="child">
          <div class="text">${r18Text}</div>
        </div>
      </div>`
                : '';
            let multipleHTML = '';
            if (data.pageCount > 1) {
                multipleHTML = `
        <div class="${this.multipleClass}">
          <div class="child">
            <span class="span1">
              <span class="span2">
              <svg viewBox="0 0 9 10" size="9" class="multipleSvg">
                <path d="M8,3 C8.55228475,3 9,3.44771525 9,4 L9,9 C9,9.55228475 8.55228475,10 8,10 L3,10
                C2.44771525,10 2,9.55228475 2,9 L6,9 C7.1045695,9 8,8.1045695 8,7 L8,3 Z M1,1 L6,1
                C6.55228475,1 7,1.44771525 7,2 L7,7 C7,7.55228475 6.55228475,8 6,8 L1,8 C0.44771525,8
                0,7.55228475 0,7 L0,2 C0,1.44771525 0.44771525,1 1,1 Z" transform=""></path>
                </svg>
              </span>
            </span>
            <span>${data.pageCount}</span>
          </div>  
        </div>`;
            }
            let ugoiraHTML = '';
            if (data.ugoiraInfo) {
                ugoiraHTML = `
        <svg viewBox="0 0 24 24" class="${this.ugoiraClass}" style="width: 48px; height: 48px;">
        <circle cx="12" cy="12" r="10" class="ugoiraCircle"></circle>
          <path d="M9,8.74841664 L9,15.2515834 C9,15.8038681 9.44771525,16.2515834 10,16.2515834
              C10.1782928,16.2515834 10.3533435,16.2039156 10.5070201,16.1135176 L16.0347118,12.8619342
              C16.510745,12.5819147 16.6696454,11.969013 16.3896259,11.4929799
              C16.3034179,11.3464262 16.1812655,11.2242738 16.0347118,11.1380658 L10.5070201,7.88648243
              C10.030987,7.60646294 9.41808527,7.76536339 9.13806578,8.24139652
              C9.04766776,8.39507316 9,8.57012386 9,8.74841664 Z"></path>
        </svg>`;
            }
            // 添加收藏的作品，让收藏图标变红
            const bookmarkedFlag = data.bookmarked ? this.bookmarkedClass : '';
            const html = `
    <div class="searchContent">
      <div class="searchImgArea">
        <div width="184" height="184" class="searchImgAreaContent">
          <a target="_blank" class="imgAreaLink" href="/artworks/${data.idNum}">
            <!--顶部横幅-->
            <div class="topbar">

            <!--R-18 标记-->
            ${r18HTML}

            <!--多图作品标记-->
            ${multipleHTML}
              
            </div>
            <!--图片部分-->
            <div class="imgWrap">
            <img src="${_setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.replaceSquareThumb
                ? _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.convertThumbURLTo540px(data.thumb)
                : data.thumb}" alt="${data.title}" style="object-fit: contain; object-position: center center;">
              <!-- 动图 svg -->
              ${ugoiraHTML}
              </div>
          </a>
          <!--添加显示收藏数-->
          <div class="bmkCount">${data.bmk}</div>
          <!--收藏按钮-->
          <div class="bmkBtnWrap">
            <div class="">
            <button type="button" class="${this.addBMKBtnClass}">
            <svg viewBox="0 0 1024 1024" width="32" height="32" class="bmkBtnSvg ${bookmarkedFlag}">
            <path d="M958.733019 411.348626 659.258367 353.59527 511.998465 85.535095 364.741633 353.59527 65.265958 411.348626 273.72878 634.744555 235.88794 938.463881 511.998465 808.479435 788.091594 938.463881 750.250754 634.744555Z" p-id="1106" class="path2"></path>
            <path d="M959.008 406.016l-308-47.008L512 64 372.992 359.008l-308 47.008 223.008 228-52.992 324L512 805.024l276.992 152.992-52.992-324zM512 740L304 856.992l40-235.008-179.008-182.016 242.016-32 104.992-224 104 224 240.992 34.016L680 622.976l36.992 235.008z" p-id="919"></path>
            </svg>
            </button>
            </div>
          </div>
        <!--收藏按钮结束-->
        </div>
      </div>
      <!--标题名-->
      <a target="_blank" class="titleLink" href="/artworks/${data.idNum}">${data.title}</a>
      <!--底部-->
      <div class="bottomBar">
      <!--作者信息-->
      <div class="userInfo">
          <a target="_blank" href="/users/${data.userId}">
            <div class="userName">${data.user}</div>
          </a>
        </div>
      </div>
    </div>
    `;
            // 相比 pixiv 原本的作品预览区域，这里去掉了作者头像的部分，因为抓取到的数据里没有作者头像。
            // 生成预览元素
            const li = document.createElement('li');
            li.classList.add(this.listClass);
            li.dataset.id = data.idNum.toString();
            li.innerHTML = html;
            // 绑定收藏按钮的事件
            const addBMKBtn = li.querySelector(`.${this.addBMKBtnClass}`);
            const bookmarkedClass = this.bookmarkedClass;
            addBMKBtn.addEventListener('click', function () {
                // 添加收藏
                const e = new CustomEvent('addBMK', {
                    detail: { data: { id: data.idNum, tags: data.tags, el: addBMKBtn } },
                });
                window.dispatchEvent(e);
                // 下载这个作品
                _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_23__.downloadOnClickBookmark.send(data.idNum.toString());
            });
            // 添加到缓冲中
            this.workPreviewBuffer.append(li);
        };
        // 清除多图作品
        this.clearMultiple = () => {
            this.filterResult((data) => {
                return data.pageCount <= 1;
            });
        };
        // 清除动图作品
        this.clearUgoira = () => {
            this.filterResult((data) => {
                return !data.ugoiraInfo;
            });
        };
        // 手动删除作品
        this.deleteWork = (event) => {
            const el = event.detail.data;
            this.deleteId = parseInt(el.dataset.id);
            this.filterResult((data) => {
                return data.idNum !== this.deleteId;
            });
        };
        this.addBookmark = async (event) => {
            const data = event.detail.data;
            for (const r of _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.result) {
                if (r.idNum === data.id) {
                    const res = await _Bookmark__WEBPACK_IMPORTED_MODULE_19__.bookmark.add(data.id.toString(), 'illusts', data.tags);
                    if (res === 200) {
                        // 同步数据
                        r.bookmarked = true;
                        this.resultMeta.forEach((result) => {
                            if (result.idNum === data.id) {
                                result.bookmarked = true;
                            }
                        });
                        data.el.classList.add(this.bookmarkedClass);
                    }
                    break;
                }
            }
        };
        this.crawlTag = () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList) {
                this.readyCrawl();
            }
        };
        this.init();
    }
    setFormOption() {
        const isPremium = _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.isPremium();
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: `1 - ${isPremium ? 5000 : 1000}`,
        });
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.resultMeta = [];
            this.crawlStartBySelf = true;
            window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.addResult, this.createPreview);
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
        _crawlMixedPage_CrawlTagList__WEBPACK_IMPORTED_MODULE_20__.crawlTagList.init();
        _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_在结果中筛选', '_在结果中筛选说明').addEventListener('click', () => {
            this.screenInResult();
        });
    }
    addAnyElement() {
        const deleteWorks = new _pageFunciton_DeleteWorks__WEBPACK_IMPORTED_MODULE_4__.DeleteWorks(`.${this.listClass}`);
        deleteWorks.addClearMultipleBtn(`.${this.multipleClass}`, () => {
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('clearMultiple');
        });
        deleteWorks.addClearUgoiraBtn(`.${this.ugoiraClass}`, () => {
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('clearUgoira');
        });
        deleteWorks.addManuallyDeleteBtn((el) => {
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('deleteWork', el);
        });
        // 添加收藏本页所有作品的功能
        const bookmarkAllBtn = _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_收藏本页面的所有作品');
        const bookmarkAll = new _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_13__.BookmarkAllWorks(bookmarkAllBtn);
        bookmarkAllBtn.addEventListener('click', () => {
            const listWrap = this.getWorksWrap();
            if (listWrap) {
                const list = listWrap.querySelectorAll('li');
                // 被二次筛选过滤掉的作品会被隐藏，所以批量添加收藏时，过滤掉隐藏的作品
                const showList = Array.from(list).filter((el) => {
                    return el.style.display !== 'none';
                });
                bookmarkAll.sendWorkList(showList);
            }
        });
    }
    initAny() {
        this.removeBlockOnHotBar();
        new _pageFunciton_FastScreen__WEBPACK_IMPORTED_MODULE_11__.FastScreen();
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.pageSwitchedTypeNotChange, this.removeBlockOnHotBar);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.addResult, this.showCount);
        window.addEventListener('addBMK', this.addBookmark);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.crawlComplete, this.onCrawlFinish);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.clearMultiple, this.clearMultiple);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.clearUgoira, this.clearUgoira);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.deleteWork, this.deleteWork);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.settingChange, this.onSettingChange);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.crawlTag, this.crawlTag);
        // 定期将缓冲中的预览作品元素添加到页面上
        window.setInterval(() => {
            this.showPreview();
        }, 1000);
    }
    showPreview() {
        if (this.workPreviewBuffer.firstChild && this.worksWrap) {
            this.worksWrap.appendChild(this.workPreviewBuffer);
        }
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.clearSlot('otherBtns');
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.addResult, this.showCount);
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.crawlComplete, this.onCrawlFinish);
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.settingChange, this.onSettingChange);
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.crawlTag, this.crawlTag);
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x页'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载所有页面'));
    }
    async nextStep() {
        this.setSlowCrawl();
        this.initFetchURL();
        // 计算应该抓取多少页
        const data = await this.getSearchData(1);
        // 计算总页数
        let pageCount = Math.ceil(data.total / this.worksNoPerPage);
        if (pageCount > 1000) {
            // 如果作品页数大于 1000 页，则判断当前用户是否是 pixiv 会员
            const isPremium = _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.isPremium();
            if (!isPremium) {
                // 如果用户不是会员，则最多只能抓取到 1000 页
                pageCount = 1000;
                _Log__WEBPACK_IMPORTED_MODULE_9__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索页面页数限制', pageCount.toString()));
            }
            else {
                // 如果用户是会员，最多可以抓取到 5000 页
                if (pageCount > 5000) {
                    pageCount = 5000;
                    _Log__WEBPACK_IMPORTED_MODULE_9__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索页面页数限制', pageCount.toString()));
                }
            }
        }
        // 如果当前页面的页码大于有效页码，则不进行抓取
        if (this.startpageNo > pageCount) {
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('crawlComplete');
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('crawlEmpty');
            if (data.total === 0) {
                return _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_作品总数为0'));
            }
            return _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.error(`${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_超出最大页码')} ${pageCount}`);
        }
        if (this.crawlNumber === -1 || this.crawlNumber > pageCount) {
            this.crawlNumber = pageCount;
        }
        // 计算从当前页面开始抓取的话，有多少页
        let needFetchPage = pageCount - this.startpageNo + 1;
        // 比较用户设置的页数，取较小的那个数值
        this.needCrawlPageCount = Math.min(needFetchPage, this.crawlNumber);
        if (this.needCrawlPageCount === 0) {
            return this.noResult();
        }
        this.getIdList();
        this.clearPreview();
        this.countEl = document.querySelector(this.countSelector);
    }
    // 组织要请求的 url 中的参数
    initFetchURL() {
        // 从 URL 中获取分类。可能有语言标识。
        /*
        https://www.pixiv.net/tags/Fate%2FGrandOrder/illustrations
        https://www.pixiv.net/en/tags/Fate%2FGrandOrder/illustrations
        */
        const URLType = location.pathname.split('tags/')[1].split('/')[1] ?? '';
        // 在“顶部”页面的时候是没有分类的，会是 undefined，此时使用空字符串
        switch (URLType) {
            case '':
                this.worksType = 'artworks';
                break;
            case 'illustrations':
            case 'illust_and_ugoira':
            case 'ugoira':
            case 'illust':
                this.worksType = 'illustrations';
                break;
            case 'manga':
                this.worksType = 'manga';
                break;
            default:
                this.worksType = 'artworks';
                break;
        }
        let p = _utils_Utils__WEBPACK_IMPORTED_MODULE_15__.Utils.getURLSearchField(location.href, 'p');
        this.startpageNo = parseInt(p) || 1;
        // 从页面 url 中获取可以使用的选项
        this.option = {};
        this.allOption.forEach((param) => {
            let value = _utils_Utils__WEBPACK_IMPORTED_MODULE_15__.Utils.getURLSearchField(location.href, param);
            if (value !== '') {
                this.option[param] = value;
            }
        });
        // 如果 url 里没有显式指定标签匹配模式，则使用 完全一致 模式
        // 因为在这种情况下，pixiv 默认使用的就是 完全一致
        if (!this.option.s_mode) {
            this.option.s_mode = 's_tag_full';
        }
        // 在日志里显示标签匹配模式
        _Log__WEBPACK_IMPORTED_MODULE_9__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索模式')}: ${this.tipSearchMode(this.option.s_mode)}`);
    }
    // 注意：同样的 mode，在搜索图片时和搜索小说时可能有不同的含义。所以这个方法不是通用的。
    tipSearchMode(mode) {
        switch (mode) {
            case 's_tag':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标签部分一致');
            case 's_tag_full':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标签完全一致');
            case 's_tc':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标题说明文字');
            default:
                return mode;
        }
    }
    // 获取搜索页的数据。因为有多处使用，所以进行了封装
    async getSearchData(p) {
        let data = await _API__WEBPACK_IMPORTED_MODULE_7__.API.getSearchData(_store_Store__WEBPACK_IMPORTED_MODULE_8__.store.tag, this.worksType, p, this.option);
        return data.body.illust || data.body.illustManga || data.body.manga;
    }
    delayReTry(p) {
        window.setTimeout(() => {
            this.getIdList(p);
        }, _Config__WEBPACK_IMPORTED_MODULE_22__.Config.retryTime);
        // 限制时间大约是 3 分钟，这里为了保险起见，设置了更大的延迟时间。
    }
    // 仅当出错重试时，才会传递参数 p。此时直接使用传入的 p，而不是继续让 p 增加
    async getIdList(p) {
        if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        if (p === undefined) {
            p = this.startpageNo + this.sendCrawlTaskCount;
            this.sendCrawlTaskCount++;
        }
        // 发起请求，获取列表页
        let data;
        try {
            data = await this.getSearchData(p);
            if (data.total === 0) {
                console.log(`page ${p}: total 0`);
                this.tipEmptyResult();
                return this.delayReTry(p);
            }
        }
        catch {
            return this.getIdList(p);
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        data = data.data;
        for (const nowData of data) {
            // 排除广告信息
            if (nowData.isAdContainer) {
                continue;
            }
            const filterOpt = {
                aiType: nowData.aiType,
                createDate: nowData.createDate,
                id: nowData.id,
                width: nowData.pageCount === 1 ? nowData.width : 0,
                height: nowData.pageCount === 1 ? nowData.height : 0,
                pageCount: nowData.pageCount,
                bookmarkData: nowData.bookmarkData,
                workType: nowData.illustType,
                tags: nowData.tags,
                userId: nowData.userId,
                xRestrict: nowData.xRestrict,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check(filterOpt)) {
                _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_16__.idListWithPageNo.add(_PageType__WEBPACK_IMPORTED_MODULE_21__.pageType.type, {
                    type: _Tools__WEBPACK_IMPORTED_MODULE_12__.Tools.getWorkTypeString(nowData.illustType),
                    id: nowData.id,
                }, p);
            }
        }
        this.listPageFinished++;
        _Log__WEBPACK_IMPORTED_MODULE_9__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取进度2', this.listPageFinished.toString(), this.needCrawlPageCount.toString()), 1, false);
        if (this.sendCrawlTaskCount + 1 <= this.needCrawlPageCount) {
            // 继续发送抓取任务（+1 是因为 sendCrawlTaskCount 从 0 开始）
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.slowCrawlMode) {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_24__.setTimeoutWorker.set(() => {
                    this.getIdList();
                }, _Config__WEBPACK_IMPORTED_MODULE_22__.Config.slowCrawlDealy);
            }
            else {
                this.getIdList();
            }
        }
        else {
            // 抓取任务已经全部发送
            if (this.listPageFinished === this.needCrawlPageCount) {
                // 抓取任务全部完成
                _Log__WEBPACK_IMPORTED_MODULE_9__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取完成'));
                _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_16__.idListWithPageNo.store(_PageType__WEBPACK_IMPORTED_MODULE_21__.pageType.type);
                this.getIdListFinished();
            }
        }
    }
    resetGetIdListStatus() {
        this.listPageFinished = 0;
        this.sendCrawlTaskCount = 0;
    }
    // 搜索页把下载任务按收藏数从高到低下载
    sortResult() {
        _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_15__.Utils.sortByProperty('bmk'));
        _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.result.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_15__.Utils.sortByProperty('bmk'));
    }
    // 返回包含作品列表的 ul 元素
    getWorksWrap() {
        const test = document.querySelectorAll(this.worksWrapSelector);
        if (test.length > 0) {
            if (test.length > 2) {
                // 大于 2 的情况是在搜索页的首页，或者小说页面
                return test[2];
            }
            // 在插画、漫画、artworks 页面只有两个 ul 或者一个
            return test[test.length - 1];
        }
        return null;
    }
    // 清空预览作品的列表，在开始抓取时和作品抓取完毕时使用
    clearPreview() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_10__.settings.previewResult || !this.crawlStartBySelf) {
            return;
        }
        this.worksWrap = this.getWorksWrap();
        if (this.worksWrap) {
            this.worksWrap.innerHTML = '';
        }
        // 同时重置一些变量
        this.previewCount = 0;
        this.showPreviewLimitTip = false;
        this.workPreviewBuffer = document.createDocumentFragment();
    }
    // 传递作品 id 列表，从页面上的作品列表里移除这些作品
    removeWorks(idList) {
        // #root section ul .searchList
        const listSelector = `${this.worksWrapSelector} .${this.listClass}`;
        const lists = document.querySelectorAll(listSelector);
        for (const li of lists) {
            if (li.dataset.id && idList.includes(li.dataset.id)) {
                li.style.display = 'none';
                // li.remove()
                // 推测隐藏元素可以更快的重绘好页面，因为删除元素修改了 dom 结构，花的时间可能会多一些
            }
        }
    }
    // 筛选抓取结果。传入函数，过滤符合条件的结果
    // 在抓取完成之后，所有会从结果合集中删除某些结果的操作都要经过这里
    async filterResult(callback) {
        if (this.resultMeta.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_17__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有可用的抓取结果'));
            return;
        }
        const beforeLength = this.resultMeta.length; // 储存过滤前的结果数量
        const resultMetaTemp = [];
        const resultMetaRemoved = [];
        for (const meta of this.resultMeta) {
            if (await callback(meta)) {
                resultMetaTemp.push(meta);
            }
            else {
                resultMetaRemoved.push(meta);
            }
        }
        this.resultMeta = resultMetaTemp;
        // 如果过滤后，作品元数据发生了改变则重排作品
        if (this.resultMeta.length !== beforeLength) {
            let ids = [];
            for (const result of resultMetaRemoved) {
                ids.push(result.idNum.toString());
            }
            this.removeWorks(ids);
            this.reAddResult();
        }
        _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('resultChange');
    }
    // 重新添加抓取结果，执行时机：
    // 1 作品抓取完毕之后，添加抓取到的数据
    // 2 使用“在结果中筛选”或删除作品，使得作品数据变化了，改变作品列表视图
    // 3 修改了“多图下载设置”，导致作品数据变化
    reAddResult() {
        _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.reset();
        // store.addResult 会触发 addResult 事件，让本模块生成对应作品的预览，并显示作品数量
        for (let data of this.resultMeta) {
            _store_Store__WEBPACK_IMPORTED_MODULE_8__.store.addResult(data);
        }
        // showCount 依赖 addResult 事件，但如果清空了所有结果，则不会触发 addResult 事件，所以需要手动调用它
        if (this.resultMeta.length === 0) {
            this.showCount();
        }
    }
    // 在当前结果中再次筛选，会修改第一次筛选的结果
    screenInResult() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.busy) {
            _Toast__WEBPACK_IMPORTED_MODULE_17__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前任务尚未完成'));
            return;
        }
        this.getMultipleSetting();
        this.filterResult((data) => {
            const filterOpt = {
                aiType: data.aiType,
                id: data.id,
                workType: data.type,
                pageCount: data.pageCount,
                tags: data.tags,
                bookmarkCount: data.bmk,
                bookmarkData: data.bookmarked,
                width: data.pageCount === 1 ? data.fullWidth : 0,
                height: data.pageCount === 1 ? data.fullHeight : 0,
                createDate: data.date,
                userId: data.userId,
                xRestrict: data.xRestrict,
            };
            return _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check(filterOpt);
        });
    }
    // 去除覆盖在热门作品上面的会员购买链接
    removeBlockOnHotBar() {
        // 需要重复执行，因为这个链接会生成不止一次
        window.setInterval(() => {
            if (_PageType__WEBPACK_IMPORTED_MODULE_21__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_21__.pageType.list.ArtworkSearch) {
                return;
            }
            const hotWorksLink = document.querySelector('section a[href^="/premium"]');
            if (hotWorksLink) {
                hotWorksLink.remove();
            }
        }, 300);
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/CrawlTagList.ts":
/*!***********************************************!*\
  !*** ./src/ts/crawlMixedPage/CrawlTagList.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   crawlTagList: () => (/* binding */ crawlTagList)
/* harmony export */ });
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
// 在搜索页面抓取 tag 列表，抓取完一个 tag 就立即开始下载。下载完毕后再抓取下一个 tag
// 这是个单例类，为了控制其执行时机，需要手动执行 init 方法来进行一部分初始化








class CrawlTagList {
    constructor() {
        this.bindEventFlag = false;
        this.EnablPage = [_PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.list.ArtworkSearch, _PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.list.NovelSearch];
        this._tagList = [];
        this.storeName = 'crawlTagList';
        this.wrapId = 'crawlTagListInputWrap';
        this.onDownloadComplete = () => {
            window.setTimeout(() => {
                if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.crawlTagList) {
                    this._tagList.shift();
                    this.tagList = this._tagList;
                    if (this._tagList.length === 0) {
                        _store_States__WEBPACK_IMPORTED_MODULE_4__.states.crawlTagList = false;
                        // 输出提示
                        this.showTagListWrap.innerHTML = `<span style="color:${_Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.textSuccess}">${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载完毕')}</span>`;
                        return;
                    }
                    // 继续抓取下一个标签
                    this.readyCrawl();
                }
            }, 0);
        };
    }
    init() {
        this.addCrawlBtns();
        this.addElement();
        this.bindEvents();
        this.restoreData();
    }
    get tagList() {
        return this._tagList;
    }
    set tagList(val) {
        this._tagList = val;
        this.storeData();
        this.showTagList();
        this.toggleWrap(true);
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取标签列表').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('closeCenterPanel');
            this.toggleWrap(true);
            // 跳转到页面顶部，否则用户可能看不到输入区域
            window.scrollTo(0, 0);
        });
    }
    addElement() {
        const htmlText = `<textarea
      id="crawlTagListTextArea"
      data-xzplaceholder="_抓取标签列表的输入框提示"
    ></textarea>
    <p id="crawlTagListTip" data-xztext="_抓取标签列表的文件夹提示"></p>
    <div id="crawlTagListBtnsWrap">
      <button id="crawlTagListBtn" data-xztext="_抓取标签列表"></button>
      <button id="clearTagListBtn" data-xztext="_停止抓取标签列表"></button>
    </div>
    <div id="tagListWrap">
      <p data-xztext="_等待下载的标签"></p>
      <ul id="showTagList">
      <ul>
    </div>
  `;
        // 创建输入标签列表的区域。如果页面上已存在该区域，则移除它，重新创建
        const test = document.getElementById(this.wrapId);
        if (test !== null) {
            test.remove();
        }
        const wrap = document.createElement('div');
        wrap.id = this.wrapId;
        wrap.innerHTML = htmlText;
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.insertToHead(wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.wrap);
        this.input = this.wrap.querySelector('#crawlTagListTextArea');
        this.startCrawlBtn = this.wrap.querySelector('#crawlTagListBtn');
        this.clearCrawlBtn = this.wrap.querySelector('#clearTagListBtn');
        this.showTagListWrap = this.wrap.querySelector('#showTagList');
        this.startCrawlBtn.addEventListener('click', () => {
            this.checkInput();
        });
        this.clearCrawlBtn.addEventListener('click', () => {
            this.clear();
        });
    }
    bindEvents() {
        // 防止事件重复绑定
        if (this.bindEventFlag) {
            return;
        }
        this.bindEventFlag = true;
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.downloadComplete, this.onDownloadComplete);
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.crawlEmpty, this.onDownloadComplete);
        // 当页面类型变化时，如果进入到了不支持的页面类型，则隐藏输入区域
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.pageSwitch, () => {
            if (!this.EnablPage.includes(_PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.type) &&
                this._tagList.length === 0) {
                this.toggleWrap(false);
            }
        });
    }
    checkInput() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
            return _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前任务尚未完成'));
        }
        const value = this.input.value.trim();
        if (value === '') {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有数据可供使用'));
        }
        const array = value.split('\n').filter((val) => val !== '' && val !== ' ');
        if (array.length === 0) {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有数据可供使用'));
        }
        this.tagList = array;
        this.readyCrawl();
    }
    readyCrawl() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
            return _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前任务尚未完成'));
        }
        if (!this.EnablPage.includes(_PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.type)) {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取标签列表') +
                '<br>' +
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_只能在搜索页面使用'));
        }
        _store_States__WEBPACK_IMPORTED_MODULE_4__.states.crawlTagList = true;
        const tag = this._tagList[0];
        // 修改 title，便于使用 P_title 建立文件夹
        document.title = tag;
        // 修改 url，使用当前抓取的标签替换原本 url 里的标签，便于使用 p_tag  建立文件夹
        const urlTag = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.getTagFromURL();
        const newURL = location.href.replace(encodeURIComponent(urlTag), encodeURIComponent(tag));
        history.pushState({ p_tag: tag }, tag, newURL);
        // 触发抓取事件
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlTag', tag);
    }
    // 控制 wrap 的显示，如果不传入参数，则自行切换显示/隐藏状态
    toggleWrap(flag) {
        if (flag !== undefined) {
            this.wrap.style.display = flag ? 'block' : 'none';
        }
        else {
            const now = this.wrap.style.display;
            this.wrap.style.display = now === 'block' ? 'none' : 'block';
        }
    }
    showTagList() {
        const text = this.tagList.map((val) => `<li>${val}</li>`);
        this.showTagListWrap.innerHTML = text.join('');
    }
    clear() {
        if (this.tagList.length === 0) {
            return;
        }
        const confirm = window.confirm(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_你确定要停止抓取吗'));
        if (confirm) {
            this.tagList = [];
            // states.busy 有可能是因为下载器正在抓取作品，通过刷新页面可以取消抓取。
            if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
                location.reload();
            }
        }
    }
    // 每当 tagList 状态变化时，保存 tagList 到本地存储
    storeData() {
        if (this.tagList.length === 0) {
            return localStorage.removeItem(this.storeName);
        }
        localStorage.setItem(this.storeName, JSON.stringify(this.tagList));
    }
    // 启动时从本地存储里读取 tagList 数据
    restoreData() {
        const str = localStorage.getItem(this.storeName);
        if (!str) {
            return;
        }
        const data = JSON.parse(str);
        if (str.length === 0) {
            return;
        }
        this.tagList = data;
        // 在输入框里显示需要抓取的标签列表
        this.input.value = this.tagList.map((val) => val).join('\n');
        // 不会自动开始抓取未完成的标签。这是基于以下考虑：
        // 1. 如果之前有未完成的下载，那么下载器会自动恢复下载。如果此时自动开始抓取，会造成冲突
        // 2. 如果自动开始抓取，那么用户每打开一个新的搜索页面，下载器都会自动开始抓取，影响用户正常使用
    }
}
const crawlTagList = new CrawlTagList();



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitBookmarkLegacyPage.ts":
/*!*********************************************************!*\
  !*** ./src/ts/crawlMixedPage/InitBookmarkLegacyPage.ts ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitBookmarkLegacyPage: () => (/* binding */ InitBookmarkLegacyPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
// 初始化旧版收藏页面
// 该页面类型已不复存在，这个类仅做占位之用。
// 在 PageType 里不能删除 BookmarkLegacy 枚举成员，因为有些代码里硬编码了它的枚举值 3
// 例如：settings.wantPageArr[pageType.type]
// settings.nameRuleForEachPageType[pageType.type]

class InitBookmarkLegacyPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitBookmarkNewPage.ts":
/*!******************************************************!*\
  !*** ./src/ts/crawlMixedPage/InitBookmarkNewPage.ts ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitBookmarkNewPage: () => (/* binding */ InitBookmarkNewPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化 关注的用户的新作品页面













class InitBookmarkNewPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.type = 'illust';
        this.tag = '';
        this.r18 = false;
        this.newVer = false;
        // 这次抓取任务最多可以抓取到多少个作品
        this.crawlWorksMaxNumber = 0;
        // 裁剪 API 返回的作品数据时的偏移量
        this.firstOffset = 0;
        // 总计抓取了多少个作品（被裁剪掉的不算）
        this.crawledWorksNumber = 0;
        // 每当 API 返回数据时，保存第一个作品的 id，用来判断后面的数据是否出现重复
        // 如果数据重复就停止抓取，因为后面的每一页的数据都是完全相同的
        this.firstWorkId = '';
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        this.maxCount = 100;
        _setting_Options__WEBPACK_IMPORTED_MODULE_4__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInputGreater0(this.maxCount, true);
    }
    nextStep() {
        this.setSlowCrawl();
        this.type = window.location.pathname.includes('/novel') ? 'novel' : 'illust';
        this.tag = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLSearchField(window.location.href, 'tag');
        this.r18 = location.pathname.includes('r18');
        this.newVer = !document.querySelector('h1');
        // 根据页数计算最多抓取多少个作品。新版一页 60 个作品，旧版一页 20 个作品
        this.crawlWorksMaxNumber = this.crawlNumber * (this.newVer ? 60 : 20);
        // 设置 API 里发起请求的页数
        const p = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLSearchField(location.href, 'p');
        const pageNo = parseInt(p) || 1;
        if (this.newVer) {
            // 新版页面里，由于 API 返回的就是这一页的作品，所以直接获取地址栏的页码就可以
            this.startpageNo = pageNo;
        }
        else {
            // 旧版页面里，由于 API 一页会返回相当于旧版 3 页的数据，所以不能直接使用地址栏的页码
            // 例如用户在旧版页面的第 5 页开始抓取，实质上是从第 81 个作品开始抓取。所以 API 里要从第 2 页开始抓取（第 61 - 120 个作品），并且设置偏移量为 20
            // API 里开始抓取的页数，向上取整
            this.startpageNo = Math.ceil((pageNo * 20) / 60);
            // 计算 API 返回的 60 个数据里，可以保留多少个
            const howManyLeft = this.startpageNo * 60 - (pageNo - 1) * 20;
            // 计算偏移量
            this.firstOffset = 60 - howManyLeft;
        }
        this.getIdList();
    }
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let p = this.startpageNo + this.listPageFinished;
        let data;
        try {
            data = await _API__WEBPACK_IMPORTED_MODULE_6__.API.getBookmarkNewWorkData(this.type, p, this.tag, this.r18);
        }
        catch (error) {
            this.getIdList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let worksData = data.body.thumbnails[this.type];
        // 检查数据，如果数据为空，或者和上一页的数据重复，说明已经不需要继续抓取了
        if (worksData.length === 0 || this.firstWorkId === worksData[0].id) {
            _Log__WEBPACK_IMPORTED_MODULE_8__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取完成'));
            return this.getIdListFinished();
        }
        else {
            // 如果数据没有重复，则保存第一个作品的 id
            this.firstWorkId = worksData[0].id;
        }
        // 旧版页面可能需要对 API 返回的数据进行裁剪
        if (!this.newVer) {
            // 使用偏移量移除不需要的数据（仅一次）
            if (this.firstOffset > 0) {
                worksData = worksData.slice(this.firstOffset);
                this.firstOffset = 0;
            }
            // 计算还有多少个作品需要抓取，然后裁剪数组，避免抓取结果超出预定的数量
            const needCrawl = this.crawlWorksMaxNumber - this.crawledWorksNumber;
            if (needCrawl < worksData.length) {
                worksData = worksData.slice(0, needCrawl);
            }
        }
        this.crawledWorksNumber += worksData.length;
        // 过滤作品
        // 过滤插画·漫画
        if (this.type === 'illust') {
            for (const data of worksData) {
                if (data.isAdContainer) {
                    continue;
                }
                const filterOpt = {
                    aiType: data.aiType,
                    id: data.id,
                    width: data.pageCount === 1 ? data.width : 0,
                    height: data.pageCount === 1 ? data.height : 0,
                    pageCount: data.pageCount,
                    bookmarkData: data.bookmarkData,
                    workType: data.illustType,
                    tags: data.tags,
                    userId: data.userId,
                    xRestrict: data.xRestrict,
                };
                if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_5__.filter.check(filterOpt)) {
                    _store_Store__WEBPACK_IMPORTED_MODULE_7__.store.idList.push({
                        type: _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getWorkTypeString(data.illustType),
                        id: data.id,
                    });
                }
            }
        }
        else {
            // 过滤小说
            for (const data of worksData) {
                const filterOpt = {
                    aiType: data.aiType,
                    id: data.id,
                    workType: 3,
                    tags: data.tags,
                    bookmarkCount: data.bookmarkCount,
                    bookmarkData: data.bookmarkData,
                    userId: data.userId,
                };
                if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_5__.filter.check(filterOpt)) {
                    _store_Store__WEBPACK_IMPORTED_MODULE_7__.store.idList.push({
                        type: 'novels',
                        id: data.id,
                    });
                }
            }
        }
        this.listPageFinished++;
        _Log__WEBPACK_IMPORTED_MODULE_8__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取进度', this.listPageFinished.toString()), 1, false);
        // 判断任务状态
        // 如果抓取到了指定数量的作品，或者抓取到了最后一页，或者抓取完了指定的页面数量
        if (this.crawledWorksNumber >= this.crawlWorksMaxNumber ||
            p >= this.maxCount ||
            this.listPageFinished === this.crawlNumber) {
            _Log__WEBPACK_IMPORTED_MODULE_8__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取完成'));
            this.getIdListFinished();
        }
        else {
            // 继续抓取
            if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.slowCrawlMode) {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(() => {
                    this.getIdList();
                }, _Config__WEBPACK_IMPORTED_MODULE_11__.Config.slowCrawlDealy);
            }
            else {
                this.getIdList();
            }
        }
    }
    resetGetIdListStatus() {
        this.listPageFinished = 0;
        this.firstOffset = 0;
        this.crawledWorksNumber = 0;
        this.firstWorkId = '';
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitBookmarkPage.ts":
/*!***************************************************!*\
  !*** ./src/ts/crawlMixedPage/InitBookmarkPage.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitBookmarkPage: () => (/* binding */ InitBookmarkPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Token__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Token */ "./src/ts/Token.ts");
/* harmony import */ var _pageFunciton_BookmarksAddTag__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../pageFunciton/BookmarksAddTag */ "./src/ts/pageFunciton/BookmarksAddTag.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _UnBookmarkWorks__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../UnBookmarkWorks */ "./src/ts/UnBookmarkWorks.ts");
/* harmony import */ var _RemoveWorksTagsInBookmarks__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../RemoveWorksTagsInBookmarks */ "./src/ts/RemoveWorksTagsInBookmarks.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");
// 初始化新版收藏页面




















class InitBookmarkPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.idList = []; // 储存从列表页获取到的 id
        this.bookmarkDataList = [];
        this.type = 'illusts'; // 页面是图片还是小说
        this.isHide = false; // 当前页面是否显示的是非公开收藏
        this.requsetNumber = 0; // 根据页数，计算要抓取的作品个数
        this.filteredNumber = 0; // 记录检查了多少作品（不论结果是否通过都计入）
        this.onceRequest = 100; // 每次请求多少个数量
        this.offset = 0; // 每次请求的偏移量
        // 点击不同的功能按钮时，设定抓取模式
        this.crawlMode = 'normal';
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_4__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_从本页开始下载x页'), _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_下载所有页面'));
    }
    addAnyElement() {
        // 如果不存在 token，则不添加与收藏相关的按钮
        if (!_Token__WEBPACK_IMPORTED_MODULE_8__.token.token) {
            return;
        }
        // 判断这个收藏页面是不是用户自己的页面，如果不是，则不会添加相关按钮
        const URLUserID = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLPathField(window.location.pathname, 'users');
        if (URLUserID === '' || URLUserID !== _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.loggedUserID) {
            return;
        }
        // 显示提示
        window.setTimeout(() => {
            _ShowHelp__WEBPACK_IMPORTED_MODULE_19__.showHelp.show('tipBookmarkManage', _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_在收藏页面里提示有辅助功能可用'));
        }, 1000);
        const btn = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgGreen, '_给未分类作品添加添加tag');
        new _pageFunciton_BookmarksAddTag__WEBPACK_IMPORTED_MODULE_9__.BookmarksAddTag(btn);
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgYellow, '_移除本页面中所有作品的标签').addEventListener('click', () => {
            this.removeWorksTagsOnThisPage();
        });
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgRed, '_取消收藏本页面的所有作品').addEventListener('click', () => {
            this.unBookmarkAllWorksOnThisPage();
        });
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgRed, '_取消收藏所有已被删除的作品').addEventListener('click', () => {
            this.unBookmarkAll404Works();
        });
    }
    // 移除本页面中所有作品的标签
    removeWorksTagsOnThisPage() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.busy || this.crawlMode !== 'normal') {
            _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_当前任务尚未完成'));
            return;
        }
        // 走一遍简化的抓取流程
        this.crawlMode = 'removeTags';
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_移除本页面中所有作品的标签'));
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_它们会变成未分类状态'));
        _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_移除本页面中所有作品的标签'), {
            position: 'topCenter',
        });
        _EVT__WEBPACK_IMPORTED_MODULE_18__.EVT.fire('closeCenterPanel');
        // 设置抓取页数为 1
        this.crawlNumber = 1;
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getTagFromURL();
        this.readyGetIdList();
        this.getIdList();
    }
    // 取消收藏本页面的所有作品
    unBookmarkAllWorksOnThisPage() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.busy || this.crawlMode !== 'normal') {
            _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_当前任务尚未完成'));
            return;
        }
        // 走一遍简化的抓取流程
        this.crawlMode = 'unBookmark';
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_取消收藏本页面的所有作品'));
        _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_取消收藏本页面的所有作品'), {
            position: 'topCenter',
        });
        _EVT__WEBPACK_IMPORTED_MODULE_18__.EVT.fire('closeCenterPanel');
        // 设置抓取页数为 1
        this.crawlNumber = 1;
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getTagFromURL();
        this.readyGetIdList();
        this.getIdList();
    }
    unBookmarkAll404Works() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.busy || this.crawlMode !== 'normal') {
            _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_当前任务尚未完成'));
            return;
        }
        // 走一遍简化的抓取流程
        this.crawlMode = 'unBookmark404';
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_取消收藏所有已被删除的作品'));
        _Toast__WEBPACK_IMPORTED_MODULE_15__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_取消收藏所有已被删除的作品'), {
            position: 'topCenter',
        });
        _EVT__WEBPACK_IMPORTED_MODULE_18__.EVT.fire('closeCenterPanel');
        // 设置抓取页数为 -1
        this.crawlNumber = -1;
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getTagFromURL();
        this.setSlowCrawl();
        this.readyGetIdList();
        // 抓取全部收藏
        this.offset = 0;
        this.getIdList();
    }
    nextStep() {
        this.crawlMode = 'normal';
        this.setSlowCrawl();
        this.readyGetIdList();
        this.getIdList();
    }
    readyGetIdList() {
        if (window.location.pathname.includes('/novel')) {
            this.type = 'novels';
        }
        // 每页个作品数，插画 48 个，小说 24 个
        const onceNumber = window.location.pathname.includes('/novels') ? 24 : 48;
        // 如果前面有页数，就去掉前面页数的作品数量。即：从本页开始下载
        const nowPage = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'p'); // 判断当前处于第几页，页码从 1 开始。也可能没有页码
        if (nowPage) {
            this.offset = (parseInt(nowPage) - 1) * onceNumber;
        }
        if (this.offset < 0) {
            this.offset = 0;
        }
        // 根据页数设置，计算要下载的个数
        if (this.crawlNumber === -1) {
            this.requsetNumber = _Config__WEBPACK_IMPORTED_MODULE_12__.Config.worksNumberLimit;
        }
        else {
            this.requsetNumber = onceNumber * this.crawlNumber;
        }
        // 判断是公开收藏还是非公开收藏
        // 在新旧版 url 里，rest 都是在查询字符串里的
        this.isHide = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'rest') === 'hide';
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_正在抓取'));
        if (this.crawlNumber === -1) {
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_获取全部书签作品'));
        }
    }
    // 获取用户的收藏作品列表
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let data;
        try {
            data = await _API__WEBPACK_IMPORTED_MODULE_1__.API.getBookmarkData(_Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getUserId(), this.type, _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag, this.offset, this.isHide);
        }
        catch (error) {
            this.getIdList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        if (data.body.works.length === 0 ||
            this.bookmarkDataList.length >= this.requsetNumber ||
            this.idList.length >= this.requsetNumber ||
            this.filteredNumber >= this.requsetNumber) {
            // 书签页获取完毕
            return this.afterGetIdList();
        }
        else {
            // 没有抓取完毕时，添加数据
            for (const workData of data.body.works) {
                if (this.filteredNumber >= this.requsetNumber) {
                    return this.afterGetIdList();
                }
                if (workData.bookmarkData) {
                    if (this.crawlMode === 'unBookmark' ||
                        this.crawlMode === 'removeTags' ||
                        (this.crawlMode === 'unBookmark404' &&
                            Number.parseInt(workData.userId) == 0)) {
                        this.bookmarkDataList.push({
                            workID: Number.parseInt(workData.id),
                            type: workData.illustType === undefined
                                ? 'novels'
                                : 'illusts',
                            bookmarkID: workData.bookmarkData.id,
                            private: workData.bookmarkData.private,
                        });
                    }
                }
                if (this.crawlMode === 'normal') {
                    const filterOpt = {
                        aiType: workData.aiType,
                        id: workData.id,
                        tags: workData.tags,
                        bookmarkData: workData.bookmarkData,
                        createDate: workData.createDate,
                        userId: workData.userId,
                        xRestrict: workData.xRestrict,
                    };
                    this.filteredNumber++;
                    if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_10__.filter.check(filterOpt)) {
                        this.idList.push({
                            type: workData.illustType === undefined
                                ? 'novels'
                                : _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getWorkTypeString(workData.illustType),
                            id: workData.id,
                        });
                    }
                }
            }
            this.offset += this.onceRequest;
            const length = this.crawlMode === 'normal'
                ? this.idList.length
                : this.bookmarkDataList.length;
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_当前作品个数', length.toString()), 1, false);
            // 继续抓取
            if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.slowCrawlMode) {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_14__.setTimeoutWorker.set(() => {
                    this.getIdList();
                }, _Config__WEBPACK_IMPORTED_MODULE_12__.Config.slowCrawlDealy);
            }
            else {
                this.getIdList();
            }
        }
    }
    // 获取作品 id 列表完毕之后
    afterGetIdList() {
        // 裁剪作品。但是如果
        if (this.crawlMode === 'normal') {
            // 因为书签页面一次获取 100 个作品，大于一页的数量。所以可能会抓取多了，需要删除多余的作品
            if (this.idList.length > this.requsetNumber) {
                // 删除后面部分（较早收藏的），留下近期收藏的
                this.idList.splice(this.requsetNumber, this.idList.length);
                // 书签页面的 api 没有考虑页面上的排序顺序，获取到的 id 列表始终是按收藏顺序由近期到早期排列的
            }
        }
        else {
            if (this.bookmarkDataList.length > this.requsetNumber) {
                this.bookmarkDataList.splice(this.requsetNumber, this.bookmarkDataList.length);
            }
        }
        if (this.crawlMode === 'normal') {
            // 正常抓取
            _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.concat(this.idList);
            this.getIdListFinished();
        }
        else if (this.crawlMode === 'unBookmark' ||
            this.crawlMode === 'unBookmark404') {
            // 取消收藏
            // 导出已被删除的收藏的 ID 列表
            if (this.crawlMode === 'unBookmark404' &&
                this.bookmarkDataList.length > 0) {
                const IDList = [];
                for (const item of this.bookmarkDataList) {
                    IDList.push(item.workID);
                }
                const blob = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.json2Blob(IDList);
                const url = URL.createObjectURL(blob);
                _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.downloadFile(url, '404 bookmark ID list.txt');
                _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_已导出被删除的作品的ID列表'));
            }
            const bookmarkDataList = Array.from(this.bookmarkDataList);
            this.resetGetIdListStatus();
            _UnBookmarkWorks__WEBPACK_IMPORTED_MODULE_16__.unBookmarkWorks.start(bookmarkDataList);
        }
        else if (this.crawlMode === 'removeTags') {
            // 移除本页面作品的标签
            const bookmarkDataList = Array.from(this.bookmarkDataList);
            this.resetGetIdListStatus();
            _RemoveWorksTagsInBookmarks__WEBPACK_IMPORTED_MODULE_17__.removeWorksTagsInBookmarks.start(bookmarkDataList);
        }
    }
    resetGetIdListStatus() {
        this.type = 'illusts';
        this.crawlMode = 'normal';
        this.idList = [];
        this.bookmarkDataList = [];
        this.offset = 0;
        this.requsetNumber = 0;
        this.filteredNumber = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitFollowingPage.ts":
/*!****************************************************!*\
  !*** ./src/ts/crawlMixedPage/InitFollowingPage.ts ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitFollowingPage: () => (/* binding */ InitFollowingPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _utils_CreateCSV__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/CreateCSV */ "./src/ts/utils/CreateCSV.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Token__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../Token */ "./src/ts/Token.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
// 初始化关注页面、好 P 友页面、粉丝页面


















class InitFollowingPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.baseOffset = 0; // 开始抓取时，记录初始的偏移量
        this.onceNumber = 24; // 每页 24 个画师
        this.pageType = 0; // 页面子类型
        // 0 我的关注
        // 1 我的好 P 友
        // 2 我的粉丝
        this.getUserListNo = 0; // 获取用户列表时，记录请求的次数
        this.limit = 100; // 每次请求多少个用户
        this.totalNeed = Number.MAX_SAFE_INTEGER;
        this.crawlUserID = '';
        this.rest = 'show';
        this.tag = '';
        this.userList = [];
        this.index = 0; // getIdList 时，对 userList 的索引
        this.task = 'crawl';
        this.CSVData = []; // 储存用户列表，包含 id 和用户名
        this.importFollowedUserIDs = [];
        this.homePrefix = 'https://www.pixiv.net/users/'; // 用户主页的通用链接前缀
        this.stopAddFollow = false;
        this.sendReqNumber = 0;
        this.dailyLimit = 1000; // 每天限制关注的数量，以免被封号
        this.tokenHasUpdated = false;
        this.need_recaptcha_enterprise_score_token = false;
        this.getPageType();
        this.init();
    }
    getPageType() {
        const pathname = window.location.pathname;
        if (pathname.includes('/following')) {
            this.pageType = 0;
        }
        else if (pathname.includes('/mypixiv')) {
            this.pageType = 1;
        }
        else if (pathname.includes('/followers')) {
            this.pageType = 2;
        }
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
        _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_导出关注列表CSV').addEventListener('click', () => {
            this.task = 'exportCSV';
            this.readyCrawl();
        });
        const exportButton = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_导出关注列表');
        exportButton.addEventListener('click', () => {
            this.task = 'exportJSON';
            this.readyCrawl();
        });
        exportButton.addEventListener('mouseenter', () => {
            _ShowHelp__WEBPACK_IMPORTED_MODULE_14__.showHelp.show('tipExportFollowingUserList', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导入导出关注用户列表的说明'));
        });
        const batchFollowButton = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_批量关注用户');
        batchFollowButton.addEventListener('click', async () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.busy) {
                return _Toast__WEBPACK_IMPORTED_MODULE_13__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前任务尚未完成'));
            }
            if (_store_Store__WEBPACK_IMPORTED_MODULE_5__.store.loggedUserID === '') {
                return _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_作品页状态码401'));
            }
            _EVT__WEBPACK_IMPORTED_MODULE_17__.EVT.fire('clearLog');
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_批量关注用户'));
            this.importFollowedUserIDs = await this.importUserList();
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导入的用户ID数量') + this.importFollowedUserIDs.length);
            if (this.importFollowedUserIDs.length === 0) {
                return _Log__WEBPACK_IMPORTED_MODULE_6__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_本次任务已全部完成'));
            }
            this.stopAddFollow = false;
            this.sendReqNumber = 0;
            // 导入关注列表后，需要获取关注的所有用户列表，以便在添加关注时跳过已关注的，节约时间
            this.task = 'batchFollow';
            _store_States__WEBPACK_IMPORTED_MODULE_10__.states.slowCrawlMode = true;
            _store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl = false;
            _EVT__WEBPACK_IMPORTED_MODULE_17__.EVT.fire('crawlStart');
            // 批量添加关注时，获取所有关注的用户
            this.crawlNumber = -1;
            // 把页面类型设置为 0，始终获取关注的用户列表
            this.pageType = 0;
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正在加载关注用户列表'));
            this.readyGet();
            // 始终抓取自己的关注列表，而非别人的，因为添加关注时，需要和自己的关注列表进行对比
            this.crawlUserID = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.loggedUserID;
            this.getUserList();
        });
        batchFollowButton.addEventListener('mouseenter', () => {
            _ShowHelp__WEBPACK_IMPORTED_MODULE_14__.showHelp.show('tipExportFollowingUserList', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导入导出关注用户列表的说明'));
        });
    }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x页'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载所有页面'));
    }
    nextStep() {
        this.setSlowCrawl();
        this.readyGet();
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正在抓取'));
        this.getPageType();
        this.getUserList();
    }
    readyGet() {
        this.rest = location.href.includes('rest=hide') ? 'hide' : 'show';
        this.tag = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLPathField(window.location.pathname, 'following');
        // 获取抓取开始时的页码
        const nowPage = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLSearchField(location.href, 'p');
        // 计算开始抓取时的偏移量
        if (nowPage !== '') {
            this.baseOffset = (parseInt(nowPage) - 1) * this.onceNumber;
        }
        else {
            this.baseOffset = 0;
        }
        // 要抓取多少个用户
        this.totalNeed = Number.MAX_SAFE_INTEGER;
        if (this.crawlNumber !== -1) {
            this.totalNeed = this.onceNumber * this.crawlNumber;
        }
        // 获取当前页面的用户 id
        const test = /users\/(\d*)\//.exec(location.href);
        if (test && test.length > 1) {
            this.crawlUserID = test[1];
        }
        else {
            const msg = `Get the user's own id failed`;
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg, 2);
            throw new Error(msg);
        }
    }
    // 获取用户列表
    async getUserList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getUserListComplete();
        }
        const offset = this.baseOffset + this.getUserListNo * this.limit;
        let res;
        try {
            switch (this.pageType) {
                case 0:
                    res = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getFollowingList(this.crawlUserID, this.rest, this.tag, offset);
                    break;
                case 1:
                    res = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getMyPixivList(this.crawlUserID, offset);
                    break;
                case 2:
                    res = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getFollowersList(this.crawlUserID, offset);
                    break;
            }
        }
        catch {
            this.getUserList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getUserListComplete();
        }
        const users = res.body.users;
        if (users.length === 0) {
            // 用户列表抓取完毕
            return this.getUserListComplete();
        }
        for (const userData of users) {
            this.userList.push(userData.userId);
            if (this.task === 'exportCSV') {
                this.CSVData.push({
                    userId: userData.userId,
                    userName: userData.userName,
                    homePage: this.homePrefix + userData.userId,
                    userComment: userData.userComment,
                    profileImageUrl: userData.profileImageUrl,
                });
            }
            if (this.userList.length >= this.totalNeed) {
                // 抓取到了指定数量的用户
                return this.getUserListComplete();
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前有x个用户', this.userList.length.toString()), 1, false);
        this.getUserListNo++;
        this.getUserList();
    }
    async getUserListComplete() {
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前有x个用户', this.userList.length.toString()));
        // 在批量关注用户时，抓取结果为 0 并不影响继续执行
        if (this.userList.length === 0 && this.task !== 'batchFollow') {
            return this.getIdListFinished();
        }
        if (this.task === 'exportCSV') {
            this.exportCSV();
            const msg = '✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导出关注列表CSV');
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.success(msg);
            _Toast__WEBPACK_IMPORTED_MODULE_13__.toast.success(msg);
            this.stopCrawl();
            return;
        }
        if (this.task === 'exportJSON') {
            this.exportJSON();
            const msg = '✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导出关注列表');
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.success(msg);
            _Toast__WEBPACK_IMPORTED_MODULE_13__.toast.success(msg);
            this.stopCrawl();
            return;
        }
        if (this.task === 'batchFollow') {
            await this.batchFollow();
            this.stopCrawl();
            return;
        }
        this.getIdList();
    }
    stopCrawl() {
        _store_States__WEBPACK_IMPORTED_MODULE_10__.states.slowCrawlMode = false;
        _store_States__WEBPACK_IMPORTED_MODULE_10__.states.busy = false;
        this.resetGetIdListStatus();
        _EVT__WEBPACK_IMPORTED_MODULE_17__.EVT.fire('stopCrawl');
    }
    exportCSV() {
        // 添加用户信息
        const data = this.CSVData.map((item) => {
            return Object.values(item);
        });
        // 添加用户信息的标题字段
        data.unshift(Object.keys(this.CSVData[0]));
        const csv = _utils_CreateCSV__WEBPACK_IMPORTED_MODULE_8__.createCSV.create(data);
        const csvURL = URL.createObjectURL(csv);
        const csvName = _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getPageTitle();
        _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.downloadFile(csvURL, _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.replaceUnsafeStr(csvName) + '.csv');
    }
    exportJSON() {
        const blob = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.json2Blob(this.userList);
        const url = URL.createObjectURL(blob);
        _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.downloadFile(url, `following list-toal ${this.userList.length}-from user ${_utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLPathField(window.location.pathname, 'users')}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.replaceUnsafeStr(new Date().toLocaleString())}.json`);
        URL.revokeObjectURL(url);
    }
    async importUserList() {
        return new Promise(async (resolve) => {
            const loadedJSON = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.loadJSONFile().catch((err) => {
                return _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.error(err);
            }));
            if (!loadedJSON) {
                return resolve([]);
            }
            // 要求是数组并且为 string[]
            if (!Array.isArray(loadedJSON) ||
                loadedJSON.length === 0 ||
                typeof loadedJSON[0] !== 'string') {
                _Toast__WEBPACK_IMPORTED_MODULE_13__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_格式错误'));
                return resolve([]);
            }
            return resolve(loadedJSON);
        });
    }
    logProgress(current, total, newAdded) {
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(`${current} / ${total}, ${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_新增x个', newAdded.toString())}`, 1, false);
    }
    async batchFollow() {
        return new Promise(async (resolve, reject) => {
            const taskName = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang
                .transl('_批量关注用户')
                .replace('（JSON）', '')
                .replace('(JSON)', '');
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.success(taskName);
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_慢速执行以避免引起429错误'));
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_提示可以重新执行批量关注任务'));
            let followed = 0;
            let number = 0;
            const total = this.importFollowedUserIDs.length;
            for (const userID of this.importFollowedUserIDs) {
                this.logProgress(number, total, this.sendReqNumber);
                if (this.stopAddFollow) {
                    const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_任务已中止');
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg);
                    _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.error(msg);
                    return resolve();
                }
                if (this.sendReqNumber >= this.dailyLimit) {
                    this.stopAddFollow = true;
                    const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_新增的关注用户达到每日限制', this.dailyLimit.toString());
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg);
                    _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.error(msg);
                    return resolve();
                }
                number++;
                if (this.userList.includes(userID) === false) {
                    this.sendReqNumber++;
                    await this.addFollow(userID);
                }
                else {
                    followed++;
                }
            }
            this.logProgress(number, total, this.sendReqNumber);
            _Log__WEBPACK_IMPORTED_MODULE_6__.log.success('✓ ' + taskName);
            _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.success('✓ ' + taskName);
            return resolve();
        });
    }
    clearIframe(iframe) {
        iframe.src = 'about:blank';
        iframe.remove();
        iframe = null;
        console.log('清理iframe');
        // 下载器每生成一个 iframe，Pixiv 的脚本也会创建一个 iframe，一并清除
        const allIframe = document.querySelectorAll('body>iframe');
        for (const frame of allIframe) {
            if (frame?.src.includes('criteo.com')) {
                frame.remove();
            }
        }
    }
    async addFollow(userID) {
        return new Promise(async (resolve) => {
            // 需要携带 need_recaptcha_enterprise_score_token 时，用 iframe 加载网页然后点击关注按钮
            if (this.need_recaptcha_enterprise_score_token) {
                const iframe = await this.clickFollowButton(userID);
                this.clearIframe(iframe);
                return resolve(200);
            }
            // 不需要携带 need_recaptcha_enterprise_score_token 时可以直接添加关注
            const status = await _API__WEBPACK_IMPORTED_MODULE_4__.API.addFollowingUser(userID, _Token__WEBPACK_IMPORTED_MODULE_16__.token.token);
            if (status !== 200) {
                const errorMsg = `Error: ${_Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.createUserLink(userID)} Status: ${status}`;
                if (status === 404) {
                    // 404 可能的原因：
                    // 1. token 无效
                    // 2. 该用户不存在
                    if (this.tokenHasUpdated === true) {
                        _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(errorMsg);
                    }
                    else {
                        // 404 时尝试重新获取 token，然后重试请求（仅执行一次）
                        this.tokenHasUpdated = true;
                        await _Token__WEBPACK_IMPORTED_MODULE_16__.token.reset();
                        await _API__WEBPACK_IMPORTED_MODULE_4__.API.addFollowingUser(userID, _Token__WEBPACK_IMPORTED_MODULE_16__.token.token);
                    }
                }
                else if (status === 400) {
                    // 400 是需要传递 recaptcha_enterprise_score_token 的时候，它的值为空或错误
                    // 此时发出一次错误提醒，并重试添加关注
                    this.need_recaptcha_enterprise_score_token = true;
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_模拟用户点击'));
                    const iframe = await this.clickFollowButton(userID);
                    this.clearIframe(iframe);
                    return resolve(200);
                }
                else if (status === 403) {
                    // 403 是访问权限已经被限制
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(errorMsg);
                    const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_你的账号已经被Pixiv限制');
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg);
                    _MsgBox__WEBPACK_IMPORTED_MODULE_15__.msgBox.error(msg);
                    this.stopAddFollow = true;
                    return resolve(status);
                }
                else {
                    // 其他错误
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(errorMsg);
                }
            }
            // 慢速执行
            // 关注用户的 API 也会触发 429 错误，此时获取作品数据的话会返回 429，
            // 但是关注用户的 API 依然返回 200，并且返回值也正常，但实际上关注用户的操作失败了。无法判断到底有没有关注成功
            // 所以需要限制添加的速度。我用 1400ms 依然会触发 429，所以需要使用更大的时间间隔，以确保不会触发 429
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(() => {
                return resolve(status);
            }, _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.rangeRandom(2500, 3600));
        });
    }
    fun(userID, iframe) {
        return new Promise(async (resolve) => {
            // 等待一段时间，默认操作完成。但是如果此时一些请求尚未完成，可能会被取消。所以这个时间最好稍大一点
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(() => {
                return resolve(iframe);
            }, _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.rangeRandom(2500, 3600));
            const button = iframe.contentDocument?.querySelector('button[data-click-label]');
            if (button) {
                button.click();
                console.log(userID + ' click');
            }
            else {
                const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有找到关注按钮的提示', _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.createUserLink(userID));
                _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg);
                return resolve(iframe);
            }
        });
    }
    async clickFollowButton(userID) {
        return new Promise(async (resolve, reject) => {
            const url = `https://www.pixiv.net/${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.htmlLangType === 'en' ? 'en/' : ''}users/${userID}`;
            const res = await fetch(url);
            // const text = await res.text()
            const iframe = document.createElement('iframe');
            iframe.style.display = 'none';
            document.body.append(iframe);
            // iframe.srcdoc = text
            iframe.src = url;
            // 在一定时间后，强制执行回调，不管 iframe.onload 的状态。
            // 因为有时一些广告脚本可能会加载失败，导致很久才能进入 onload。那样会等待太久。
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(async () => {
                const _iframe = await this.fun(userID, iframe);
                return resolve(_iframe);
            }, _Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.rangeRandom(2500, 3600));
        });
    }
    // 获取用户的 id 列表
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let idList = [];
        try {
            idList = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getUserWorksByType(this.userList[this.index]);
        }
        catch {
            this.getIdList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.concat(idList);
        this.index++;
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已抓取x个用户', this.index.toString())}, ${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前作品个数', _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.length.toString())}`, 1, false);
        if (this.index >= this.userList.length) {
            return this.getIdListFinished();
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_10__.states.slowCrawlMode) {
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_12__.setTimeoutWorker.set(() => {
                this.getIdList();
            }, _Config__WEBPACK_IMPORTED_MODULE_11__.Config.slowCrawlDealy);
        }
        else {
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.userList = [];
        this.task = 'crawl';
        this.CSVData = [];
        this.importFollowedUserIDs = [];
        this.getUserListNo = 0;
        this.index = 0;
    }
    sortResult() {
        // 把作品数据按 id 倒序排列，id 大的在前面，这样可以先下载最新作品，后下载早期作品
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.result.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.sortByProperty('id'));
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitHomePage.ts":
/*!***********************************************!*\
  !*** ./src/ts/crawlMixedPage/InitHomePage.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitHomePage: () => (/* binding */ InitHomePage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Input__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Input */ "./src/ts/Input.ts");
// 初始化首页
















class InitHomePage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.downIdButton = document.createElement('button');
        this.importIDListButton = document.createElement('button');
        this.type = 'illusts';
        this.init();
        this.checkPageType();
        this.idRangeTip = this.createidRangeTip();
    }
    checkPageType() {
        this.type = window.location.pathname.includes('novel')
            ? 'novels'
            : 'illusts';
    }
    addCrawlBtns() {
        this.downIdButton = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_输入id进行抓取');
        this.downIdButton.addEventListener('click', () => {
            this.inputIDList();
        });
        const crawlIdRange = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取id区间');
        crawlIdRange.addEventListener('click', () => {
            this.crawlIdRange();
        });
        this.importIDListButton = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_导入ID列表');
        this.importIDListButton.addEventListener('click', () => {
            this.importIDList();
        });
    }
    addAnyElement() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_清空已保存的抓取结果').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('clearSavedCrawl');
        });
    }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.hideOption([1]);
    }
    // 单独添加一个用于提示 id 范围的元素，因为上面的日志显示在日志区域的顶端，不便于查看
    createidRangeTip() {
        const div = document.createElement('div');
        div.classList.add('id_range_tip', 'beautify_scrollbar', 'logWrap');
        _Theme__WEBPACK_IMPORTED_MODULE_8__.theme.register(div);
        return document.body.insertAdjacentElement('beforebegin', div);
    }
    async inputIDList() {
        _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('closeCenterPanel');
        this.checkPageType();
        const input = new _Input__WEBPACK_IMPORTED_MODULE_15__.Input({
            width: 400,
            type: 'textarea',
            rows: 10,
            instruction: _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_输入id进行抓取的提示文字') +
                '<br><br>' +
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl(this.type === 'illusts'
                    ? '_输入的ID视为图像ID'
                    : '_输入的ID视为小说ID'),
            placeholder: '10000\n10001\n10002\n10003',
        });
        const value = await input.submit();
        if (!value) {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_本次操作已取消'));
        }
        // 不必去重，因为 store 存储抓取结果时会去重
        const array = value.split('\n');
        const result = [];
        for (const str of array) {
            const id = parseInt(str);
            if (isNaN(id) || id < 22 || id > _Config__WEBPACK_IMPORTED_MODULE_6__.Config.worksNumberLimit) {
                console.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_id不合法') + ': ' + str);
            }
            else {
                result.push(id.toString());
            }
        }
        this.addIdList(result);
    }
    async crawlIdRange() {
        _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('closeCenterPanel');
        this.checkPageType();
        let start = 0;
        let end = 0;
        // 接收起点
        const startInput = new _Input__WEBPACK_IMPORTED_MODULE_15__.Input({
            width: 400,
            instruction: _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取id区间说明') +
                '<br><br>' +
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl(this.type === 'illusts'
                    ? '_输入的ID视为图像ID'
                    : '_输入的ID视为小说ID') +
                '<br><br>' +
                _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取id区间起点'),
            placeholder: '100',
        });
        const startValue = await startInput.submit();
        if (startValue) {
            const num = Number.parseInt(startValue);
            if (!isNaN(num) && num >= 0) {
                start = num;
            }
            else {
                return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_参数不合法本次操作已取消'));
            }
        }
        else {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_本次操作已取消'));
        }
        // 接收终点
        const endInput = new _Input__WEBPACK_IMPORTED_MODULE_15__.Input({
            width: 400,
            instruction: _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取id区间终点'),
            placeholder: '200',
        });
        const endValue = await endInput.submit();
        if (endValue) {
            const num = Number.parseInt(endValue);
            if (!isNaN(num) && num >= start) {
                end = num;
            }
            else {
                return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_参数不合法本次操作已取消'));
            }
        }
        else {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_本次操作已取消'));
        }
        // 提示抓取范围，便于用户分批次抓取的时候查看
        const tip = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取id区间') + `: ${start} - ${end}`;
        this.idRangeTip.textContent = tip;
        this.idRangeTip.style.display = 'block';
        // 不要在这里使用 log.log ，因为之后开始抓取时，日志区域会被清空，所以用户在日志区域里看不到这个提示
        // 生成 id 列表
        const ids = [];
        while (start <= end) {
            ids.push(start.toString());
            start++;
        }
        this.addIdList(ids);
    }
    // 把 id 列表添加到 store 里，然后开始抓取
    addIdList(ids) {
        const idList = [];
        for (const id of ids) {
            idList.push({
                type: this.type,
                id: id,
            });
        }
        _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('crawlIdList', idList);
    }
    async importIDList() {
        const loadedJSON = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.loadJSONFile().catch((err) => {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_10__.msgBox.error(err);
        }));
        if (!loadedJSON) {
            return;
        }
        // 要求是数组并且要有内容
        if (!Array.isArray(loadedJSON) || !loadedJSON.length || !loadedJSON[0]) {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_格式错误'));
        }
        // 检查是否含有必须的字段（只检查了一部分）
        const keys = Object.keys(loadedJSON[0]);
        const need = ['id', 'type'];
        for (const field of need) {
            if (!keys.includes(field)) {
                return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_格式错误'));
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_12__.log.success('✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导入ID列表'));
        _store_Store__WEBPACK_IMPORTED_MODULE_11__.store.reset();
        _store_Store__WEBPACK_IMPORTED_MODULE_11__.store.idList = loadedJSON;
        this.crawlImportIDList();
    }
    crawlImportIDList() {
        _Log__WEBPACK_IMPORTED_MODULE_12__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_当前作品个数', _store_Store__WEBPACK_IMPORTED_MODULE_11__.store.idList.length.toString()));
        _Log__WEBPACK_IMPORTED_MODULE_12__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_开始获取作品信息'));
        if (_Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.checkUserLogin() === false) {
            // 如果未登录账号，则全速抓取
            _store_States__WEBPACK_IMPORTED_MODULE_13__.states.slowCrawlMode = false;
            if (_store_Store__WEBPACK_IMPORTED_MODULE_11__.store.idList.length <= this.ajaxThreadsDefault) {
                this.ajaxThread = _store_Store__WEBPACK_IMPORTED_MODULE_11__.store.idList.length;
            }
            else {
                this.ajaxThread = this.ajaxThreadsDefault;
            }
        }
        else {
            // 登录账号后，可以使用慢速抓取
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_14__.settings.slowCrawl &&
                _store_Store__WEBPACK_IMPORTED_MODULE_11__.store.idList.length > _setting_Settings__WEBPACK_IMPORTED_MODULE_14__.settings.slowCrawlOnWorksNumber) {
                _Log__WEBPACK_IMPORTED_MODULE_12__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_慢速抓取'));
                _store_States__WEBPACK_IMPORTED_MODULE_13__.states.slowCrawlMode = true;
                this.ajaxThread = 1;
            }
        }
        for (let i = 0; i < this.ajaxThread; i++) {
            this.getWorksData();
        }
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.clearSlot('otherBtns');
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitUnlistedPage.ts":
/*!***************************************************!*\
  !*** ./src/ts/crawlMixedPage/InitUnlistedPage.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitUnlistedPage: () => (/* binding */ InitUnlistedPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
//初始化 Unlisted 作品页






class InitUnlistedPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取此作品').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_2__.options.hideOption([1]);
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.clearSlot('otherBtns');
    }
    nextStep() {
        this.getIdList();
    }
    getIdList() {
        const workId = _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.getURLPathField(window.location.pathname, 'unlisted');
        const isNovel = window.location.href.includes('/novel');
        const idData = {
            type: isNovel ? 'novels' : 'illusts',
            id: workId,
        };
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList = [idData];
        this.getIdListFinished();
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/InitUserPage.ts":
/*!***********************************************!*\
  !*** ./src/ts/crawlMixedPage/InitUserPage.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitUserPage: () => (/* binding */ InitUserPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _pageFunciton_SaveAvatarIcon__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../pageFunciton/SaveAvatarIcon */ "./src/ts/pageFunciton/SaveAvatarIcon.ts");
/* harmony import */ var _pageFunciton_SaveAvatarImage__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../pageFunciton/SaveAvatarImage */ "./src/ts/pageFunciton/SaveAvatarImage.ts");
/* harmony import */ var _pageFunciton_SaveUserCover__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../pageFunciton/SaveUserCover */ "./src/ts/pageFunciton/SaveUserCover.ts");
/* harmony import */ var _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../pageFunciton/BookmarkAllWorks */ "./src/ts/pageFunciton/BookmarkAllWorks.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
// 初始化用户页面
















var ListType;
(function (ListType) {
    ListType[ListType["UserHome"] = 0] = "UserHome";
    ListType[ListType["Artworks"] = 1] = "Artworks";
    ListType[ListType["Illustrations"] = 2] = "Illustrations";
    ListType[ListType["Manga"] = 3] = "Manga";
    ListType[ListType["Novels"] = 4] = "Novels";
})(ListType || (ListType = {}));
class InitUserPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.listType = ListType.UserHome; // 当前页面应该获取哪些类型的作品
        this.onceNumber = 48; // 每页作品个数，插画是 48 个，小说是 24 个
        this.bookmarkAll = new _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_13__.BookmarkAllWorks();
        this.sendBookmarkIdList = () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.bookmarkMode) {
                // 将 id 的 type 设置为 illusts 或 novels
                const list = [];
                for (const data of _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList) {
                    if (data.type === 'novels') {
                        list.push(data);
                    }
                    else {
                        list.push({
                            type: 'illusts',
                            id: data.id,
                        });
                    }
                }
                _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList = []; // 清空这次抓取到的 id 列表
                this.bookmarkAll.sendIdList(list);
            }
        };
        this.init();
    }
    // 添加中间按钮
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    addAnyElement() {
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_保存用户头像').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('saveAvatarImage');
        });
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_保存用户头像为图标', '_保存用户头像为图标说明').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('saveAvatarIcon');
        });
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_保存用户封面').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('saveUserCover');
        });
        // 添加收藏本页所有作品的功能
        const bookmarkAllBtn = _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_收藏本页面的所有作品');
        this.bookmarkAll = new _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_13__.BookmarkAllWorks(bookmarkAllBtn);
        bookmarkAllBtn.addEventListener('click', async () => {
            // 获取该用户的所有作品的 id 列表
            // 模拟了抓取流程，以获取相同的 id 列表
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('bookmarkModeStart');
            _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag = _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getTagFromURL();
            this.crawlNumber = 1; // 设置为只抓取 1 页
            this.readyGetIdList();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.getIdListFinished, this.sendBookmarkIdList);
    }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x页'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载所有页面'));
    }
    nextStep() {
        this.readyGetIdList();
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正在抓取'));
    }
    readyGetIdList() {
        // 判断页面类型
        // 匹配 pathname 里用户 id 之后的字符
        const test = location.pathname.match(/\/users\/\d+(\/.+)/);
        if (test === null) {
            // 用户主页
            this.listType = ListType.UserHome;
        }
        else if (test.length === 2) {
            const str = test[1]; //取出用户 id 之后的字符
            if (str.includes('/artworks')) {
                // 插画和漫画列表
                this.listType = ListType.Artworks;
            }
            else if (str.includes('/illustrations')) {
                // 插画列表
                this.listType = ListType.Illustrations;
            }
            else if (str.includes('/manga')) {
                // 漫画列表
                this.listType = ListType.Manga;
            }
            else if (str.includes('/novels')) {
                // 小说列表
                this.listType = ListType.Novels;
                this.onceNumber = 24; // 如果是在小说列表页，一页只有 24 个作品
            }
        }
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag ? this.getIdListByTag() : this.getIdList();
    }
    getOffset() {
        const nowPage = _utils_Utils__WEBPACK_IMPORTED_MODULE_14__.Utils.getURLSearchField(location.href, 'p'); // 判断当前处于第几页，页码从 1 开始。也可能没有页码
        let offset = 0;
        if (nowPage) {
            offset = (parseInt(nowPage) - 1) * this.onceNumber;
        }
        if (offset < 0) {
            offset = 0;
        }
        return offset;
    }
    // 根据页数设置，计算要下载的个数
    getRequsetNumber() {
        let requsetNumber = _Config__WEBPACK_IMPORTED_MODULE_15__.Config.worksNumberLimit;
        if (this.crawlNumber !== -1) {
            requsetNumber = this.onceNumber * this.crawlNumber;
        }
        return requsetNumber;
    }
    // 获取用户某些类型的作品的 id 列表
    async getIdList() {
        let type = [];
        switch (this.listType) {
            case ListType.UserHome:
                type = ['illusts', 'manga', 'novels'];
                break;
            case ListType.Artworks:
                type = ['illusts', 'manga'];
                break;
            case ListType.Illustrations:
                type = ['illusts'];
                break;
            case ListType.Manga:
                type = ['manga'];
                break;
            case ListType.Novels:
                type = ['novels'];
                break;
        }
        let idList = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getUserWorksByType(_Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getUserId(), type);
        // 判断是否全都是小说，如果是，把每页的作品个数设置为 24 个
        const allWorkIsNovels = idList.every((data) => {
            return data.type === 'novels';
        });
        allWorkIsNovels && (this.onceNumber = 24);
        // 计算偏移量和需要保留的作品个数
        const offset = this.getOffset();
        const requsetNumber = this.getRequsetNumber();
        // 按照 id 升序排列，之后会删除不需要的部分
        idList.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_14__.Utils.sortByProperty('id')).reverse();
        // 不带 tag 获取作品时，由于 API 是一次性返回用户的所有作品，可能大于要求的数量，所以需要去掉多余的作品。
        // 删除 offset 需要去掉的部分。删除后面的 id，也就是近期作品
        idList.splice(idList.length - offset, idList.length);
        // 删除超过 requsetNumber 的作品。删除前面的 id，也就是早期作品
        if (idList.length > requsetNumber) {
            idList.splice(0, idList.length - requsetNumber);
        }
        // 储存
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.concat(idList);
        this.getIdListFinished();
    }
    // 获取用户某些类型的作品的 id 列表（附带 tag）
    async getIdListByTag() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        // 这里不用判断用户主页的情况，因为用户主页不会带 tag
        let type = 'illustmanga';
        switch (this.listType) {
            case ListType.Artworks:
                type = 'illustmanga';
                break;
            case ListType.Illustrations:
                type = 'illusts';
                break;
            case ListType.Manga:
                type = 'manga';
                break;
            case ListType.Novels:
                type = 'novels';
                break;
        }
        // 计算初始偏移量
        let offset = this.getOffset();
        // 计算需要获取多少个作品
        const requsetNumber = this.getRequsetNumber();
        // 循环请求作品，一次请求一页。假设用户的标签页面最大页数不会超过这个数字
        const maxRequest = 1000;
        for (const iterator of new Array(maxRequest)) {
            let data = await _API__WEBPACK_IMPORTED_MODULE_4__.API.getUserWorksByTypeWithTag(_Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getUserId(), type, _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.tag, offset, this.onceNumber);
            if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
                return this.getIdListFinished();
            }
            // 图片和小说返回的数据是不同的，小说没有 illustType 标记
            if (this.listType === ListType.Novels) {
                const d = data;
                d.body.works.forEach((data) => _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.push({
                    type: 'novels',
                    id: data.id,
                }));
            }
            else {
                const d = data;
                d.body.works.forEach((data) => {
                    let type = 'illusts';
                    switch (data.illustType) {
                        case 0:
                            type = 'illusts';
                            break;
                        case 1:
                            type = 'manga';
                            break;
                        case 2:
                            type = 'ugoira';
                            break;
                    }
                    _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.push({
                        type,
                        id: data.id,
                    });
                });
            }
            offset += data.body.works.length;
            // 如果已经抓取到了预定的数量
            // 或者 API 返回的作品数量不足一页的数量，则认为抓取完毕
            if (_store_Store__WEBPACK_IMPORTED_MODULE_5__.store.idList.length >= requsetNumber ||
                data.body.works.length < this.onceNumber) {
                return this.getIdListFinished();
            }
        }
    }
    resetGetIdListStatus() {
        this.listType = ListType.UserHome;
    }
    sortResult() {
        // 把作品数据按 id 倒序排列，id 大的在前面，这样可以先下载最新作品，后下载早期作品
        _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.result.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_14__.Utils.sortByProperty('id'));
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.clearSlot('otherBtns');
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.getIdListFinished, this.sendBookmarkIdList);
    }
}



/***/ }),

/***/ "./src/ts/crawlMixedPage/QuickCrawl.ts":
/*!*********************************************!*\
  !*** ./src/ts/crawlMixedPage/QuickCrawl.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");









// 快速抓取
class QuickCrawl {
    constructor() {
        this.show = true; // 是否显示
        // 指定在哪些页面类型里启用
        this.enablePageType = [
            _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Artwork,
            _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Novel,
            _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Unlisted,
        ];
        this.addBtn();
        this.setVisible();
        this.bindEvents();
    }
    addBtn() {
        // 在右侧添加快速抓取按钮
        this.btn = document.createElement('button');
        this.btn.classList.add('rightButton');
        this.btn.id = 'quickCrawlBtn';
        this.btn.setAttribute('data-xztitle', '_快速下载本页');
        this.btn.innerHTML = `<svg class="icon" aria-hidden="true">
  <use xlink:href="#icon-download"></use>
</svg>`;
        document.body.append(this.btn);
        _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.register(this.btn);
    }
    bindEvents() {
        // 点击按钮启动快速抓取
        this.btn.addEventListener('click', () => {
            this.sendDownload();
            if (!_Config__WEBPACK_IMPORTED_MODULE_1__.Config.mobile) {
                _ShowHelp__WEBPACK_IMPORTED_MODULE_5__.showHelp.show('tipAltQToQuickDownload', _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_快捷键ALTQ快速下载本页作品'));
            }
        }, false);
        // 使用快捷键 Alt + Q 启动快速抓取
        window.addEventListener('keydown', (ev) => {
            if (this.show && ev.altKey && ev.code === 'KeyQ') {
                this.sendDownload();
            }
        }, false);
        // 页面类型改变时设置按钮的显示隐藏
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.pageSwitch, () => {
            this.setVisible();
        });
    }
    sendDownload() {
        _store_States__WEBPACK_IMPORTED_MODULE_6__.states.quickCrawl = true;
        const isNovel = window.location.href.includes('/novel');
        let idData;
        if (isNovel) {
            idData = {
                type: 'novels',
                id: _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getNovelId(window.location.href),
            };
        }
        else {
            idData = {
                type: 'illusts',
                id: _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getIllustId(window.location.href),
            };
        }
        _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.fire('crawlIdList', [idData]);
        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_已发送下载请求'), {
            bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgBlue,
        });
    }
    setVisible() {
        this.show = this.enablePageType.includes(_PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.type);
        this.btn.style.display = this.show ? 'flex' : 'none';
    }
}
new QuickCrawl();


/***/ }),

/***/ "./src/ts/crawlNovelPage/GetNovelGlossarys.ts":
/*!****************************************************!*\
  !*** ./src/ts/crawlNovelPage/GetNovelGlossarys.ts ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   getNovelGlossarys: () => (/* binding */ getNovelGlossarys)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");

class GetNovelGlossarys {
    /**获取系列小说的设定资料 */
    async getGlossarys(seriesId) {
        return new Promise(async (resolve, reject) => {
            // 先获取设定资料的分类、每条设定资料的简略数据
            // 注意此时每条设定资料缺少 detail 数据（此时为 null）
            const glossaryData = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getNovelSeriesGlossary(seriesId);
            const result = glossaryData.body.categories;
            if (result.length === 0) {
                return resolve(result);
            }
            // 请求每条设定资料的详细数据
            for (const categorie of result) {
                for (const item of categorie.items) {
                    const data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getNovelSeriesGlossaryItem(item.seriesId, item.id);
                    item.detail = data.body.item.detail;
                }
            }
            return resolve(result);
        });
    }
    /**把设定资料用特定格式存储起来 */
    storeGlossaryText(data) {
        const array = [];
        for (const categorie of data) {
            array.push(categorie.name);
            array.push('\n\n');
            for (const item of categorie.items) {
                array.push(item.name);
                array.push('\n');
                array.push(item.overview);
                array.push('\n\n');
                if (item.detail) {
                    array.push(item.detail);
                    array.push('\n\n');
                }
                array.push('----------------------------------------');
                array.push('\n\n');
            }
        }
        if (array.length > 0) {
            return array.join('') + '\n\n';
        }
        return '';
    }
}
const getNovelGlossarys = new GetNovelGlossarys();



/***/ }),

/***/ "./src/ts/crawlNovelPage/InitNewNovelPage.ts":
/*!***************************************************!*\
  !*** ./src/ts/crawlNovelPage/InitNewNovelPage.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitNewNovelPage: () => (/* binding */ InitNewNovelPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化 本站的最新作品 小说页面












class InitNewNovelPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.option = this.resetOption();
        this.limitMax = 20; // 每次请求的数量最大是 20
        this.fetchCount = 0; // 已请求的作品数量
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_下载大家的新作品').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_想要获取多少个作品',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInputGreater0(this.maxCount, false);
    }
    nextStep() {
        this.setSlowCrawl();
        this.initFetchURL();
        this.getIdList();
    }
    resetOption() {
        return {
            lastId: '0',
            limit: '20',
            type: '',
            r18: '',
        };
    }
    // 组织要请求的 url
    initFetchURL() {
        this.option = this.resetOption();
        if (this.crawlNumber < this.limitMax) {
            this.option.limit = this.crawlNumber.toString();
        }
        else {
            this.option.limit = this.limitMax.toString();
        }
        // 是否是 R18 模式
        this.option.r18 = (location.href.includes('_r18.php') || false).toString();
    }
    async getIdList() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let data;
        try {
            data = await _API__WEBPACK_IMPORTED_MODULE_5__.API.getNewNovleData(this.option);
        }
        catch (error) {
            this.getIdList();
            return;
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        let useData = data.body.novels;
        for (const nowData of useData) {
            // 抓取够了指定的数量
            if (this.fetchCount + 1 > this.crawlNumber) {
                break;
            }
            else {
                this.fetchCount++;
            }
            const filterOpt = {
                aiType: nowData.aiType,
                id: nowData.id,
                bookmarkData: nowData.bookmarkData,
                bookmarkCount: nowData.bookmarkCount,
                workType: 3,
                tags: nowData.tags,
                userId: nowData.userId,
                createDate: nowData.createDate,
                xRestrict: nowData.xRestrict,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_4__.filter.check(filterOpt)) {
                _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.idList.push({
                    type: 'novels',
                    id: nowData.id,
                });
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_新作品进度', this.fetchCount.toString()), 1, false);
        // 抓取完毕
        if (this.fetchCount >= this.crawlNumber ||
            this.fetchCount >= this.maxCount) {
            _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_开始获取作品页面'));
            this.getIdListFinished();
            return;
        }
        // 继续抓取
        this.option.lastId = data.body.lastId;
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode) {
            _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_11__.setTimeoutWorker.set(() => {
                this.getIdList();
            }, _Config__WEBPACK_IMPORTED_MODULE_10__.Config.slowCrawlDealy);
        }
        else {
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.fetchCount = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlNovelPage/InitNovelPage.ts":
/*!************************************************!*\
  !*** ./src/ts/crawlNovelPage/InitNovelPage.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitNovelPage: () => (/* binding */ InitNovelPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
//初始化小说作品页








class InitNovelPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.crawlDirection = 0; // 抓取方向，指示抓取新作品还是旧作品
        this.init();
    }
    /*
    -1 抓取新作品
    0 不设置抓取方向
    1 抓取旧作品
    */
    initAny() { }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_从本页开始抓取new').addEventListener('click', () => {
            this.crawlDirection = -1;
            this.readyCrawl();
        });
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_从本页开始抓取old').addEventListener('click', () => {
            this.crawlDirection = 1;
            this.readyCrawl();
        });
    }
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.clearSlot('otherBtns');
    }
    getWantPage() {
        // 检查下载页数的设置
        const crawlAllTip = this.crawlDirection === -1
            ? _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始抓取new')
            : _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始抓取old');
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x个'), crawlAllTip);
    }
    async getIdList() {
        let type = ['novels'];
        let idList = await _API__WEBPACK_IMPORTED_MODULE_6__.API.getUserWorksByType(_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getUserId(), type);
        // 储存符合条件的 id
        let nowId = parseInt(_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getNovelId(window.location.href));
        idList.forEach((id) => {
            let idNum = parseInt(id.id);
            // 新作品
            if (idNum >= nowId && this.crawlDirection === -1) {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push(id);
            }
            else if (idNum <= nowId && this.crawlDirection === 1) {
                // 旧作品
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.push(id);
            }
        });
        // 当设置了下载个数时，进行裁剪
        if (this.crawlNumber !== -1) {
            // 新作品 升序排列
            if (this.crawlDirection === -1) {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.sortByProperty('id')).reverse();
            }
            else {
                // 旧作品 降序排列
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.sortByProperty('id'));
            }
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList = _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.splice(0, this.crawlNumber);
        }
        this.getIdListFinished();
    }
    resetGetIdListStatus() {
        this.crawlDirection = 0; // 解除下载方向的标记
    }
}



/***/ }),

/***/ "./src/ts/crawlNovelPage/InitNovelSeriesPage.ts":
/*!******************************************************!*\
  !*** ./src/ts/crawlNovelPage/InitNovelSeriesPage.ts ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitNovelSeriesPage: () => (/* binding */ InitNovelSeriesPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _GetNovelGlossarys__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./GetNovelGlossarys */ "./src/ts/crawlNovelPage/GetNovelGlossarys.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
//初始化小说系列作品页面










class InitNovelSeriesPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.seriesId = '';
        this.limit = 30;
        this.last = 0;
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取系列小说').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    addAnyElement() {
        _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_合并系列小说').addEventListener('click', () => {
            _store_States__WEBPACK_IMPORTED_MODULE_6__.states.mergeNovel = true;
            this.readyCrawl();
        });
    }
    initAny() { }
    setFormOption() {
        // 隐藏“个数/页数”选项
        _setting_Options__WEBPACK_IMPORTED_MODULE_2__.options.hideOption([1]);
    }
    getWantPage() { }
    async nextStep() {
        this.seriesId = _utils_Utils__WEBPACK_IMPORTED_MODULE_9__.Utils.getURLPathField(window.location.pathname, 'series');
        if (_store_States__WEBPACK_IMPORTED_MODULE_6__.states.mergeNovel && _setting_Settings__WEBPACK_IMPORTED_MODULE_7__.settings.saveNovelMeta) {
            const data = await _GetNovelGlossarys__WEBPACK_IMPORTED_MODULE_8__.getNovelGlossarys.getGlossarys(this.seriesId);
            _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.novelSeriesGlossary = _GetNovelGlossarys__WEBPACK_IMPORTED_MODULE_8__.getNovelGlossarys.storeGlossaryText(data);
        }
        this.getIdList();
    }
    async getIdList() {
        const seriesData = await _API__WEBPACK_IMPORTED_MODULE_5__.API.getNovelSeriesData(this.seriesId, this.limit, this.last, 'asc');
        const list = seriesData.body.page.seriesContents;
        for (const item of list) {
            _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.idList.push({
                type: 'novels',
                id: item.id,
            });
        }
        this.last += list.length;
        // 如果这一次返回的作品数量达到了每批限制，可能这次没有请求完，继续请求后续的数据
        if (list.length === this.limit) {
            this.getIdList();
        }
        else {
            this.getIdListFinished();
        }
    }
    resetGetIdListStatus() {
        this.seriesId = '';
        this.last = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlNovelPage/InitRankingNovelPage.ts":
/*!*******************************************************!*\
  !*** ./src/ts/crawlNovelPage/InitRankingNovelPage.ts ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitRankingNovelPage: () => (/* binding */ InitRankingNovelPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
// 初始化小说排行榜页面








class InitRankingNovelPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.pageUrlList = [];
        this.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_抓取本排行榜作品', '_抓取本排行榜作品Title').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    initAny() { }
    setFormOption() {
        // 个数/页数选项的提示
        this.maxCount = 100;
        _setting_Options__WEBPACK_IMPORTED_MODULE_4__.options.setWantPageTip({
            text: '_抓取多少作品',
            tip: '_想要获取多少个作品',
            rangTip: `1 - ${this.maxCount}`,
        });
    }
    getWantPage() {
        // 检查下载页数的设置
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载排行榜前x个作品'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_向下获取所有作品'));
        // 如果设置的作品个数是 -1，则设置为下载所有作品
        if (this.crawlNumber === -1) {
            this.crawlNumber = this.maxCount;
        }
    }
    getPageUrl() {
        const ul = document.querySelector('.ui-selectbox-container ul');
        if (ul) {
            const li = ul.querySelectorAll('li');
            this.maxCount = li.length * 50;
            for (const el of li) {
                this.pageUrlList.push(el.dataset.url);
            }
        }
        else {
            // 只有一页的话，没有页码部分的 ul li
            this.pageUrlList.push(location.href);
        }
    }
    nextStep() {
        this.getPageUrl();
        this.getIdList();
    }
    async getIdList() {
        let dom;
        try {
            const res = await fetch(this.pageUrlList[this.listPageFinished]);
            const text = await res.text();
            const parse = new DOMParser();
            dom = parse.parseFromString(text, 'text/html');
        }
        catch (error) {
            this.getIdList();
            return;
        }
        this.listPageFinished++;
        const rankingItem = dom.querySelectorAll('._ranking-items>div[id]');
        // 检查每个作品的信息
        for (const item of rankingItem) {
            const rank = parseInt(item.querySelector('h1').innerText);
            // 检查是否已经抓取到了指定数量的作品
            if (rank > this.crawlNumber) {
                return this.getIdListFinished();
            }
            // https://www.pixiv.net/novel/show.php?id=12831389
            const link = item.querySelector('.imgbox a').href;
            const id = parseInt(link.split('id=')[1]);
            const bmkEl = item.querySelector('.bookmark-count');
            let bmk = bmkEl ? parseInt(bmkEl.innerText) : 0;
            const tags = [];
            const tagsA = item.querySelectorAll('.tags>li>a');
            for (const a of tagsA) {
                tags.push(a.innerText.trim());
            }
            const userId = item.querySelector('img').dataset.userId;
            // 有的作品没有收藏按钮，点进去之后发现这个作品已经被删除了，只是排行榜里没有及时更新。这样的作品没有收藏按钮。
            const bookmarkBtn = item.querySelector('._one-click-bookmark');
            const bookmarked = bookmarkBtn
                ? bookmarkBtn.classList.contains('on')
                : false;
            const filterOpt = {
                id: id,
                workType: 3,
                tags: tags,
                bookmarkCount: bmk,
                bookmarkData: bookmarked,
                userId: userId,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_5__.filter.check(filterOpt)) {
                _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.setRankList(id.toString(), rank);
                _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.idList.push({
                    type: 'novels',
                    id: id.toString(),
                });
            }
        }
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_排行榜进度', this.listPageFinished.toString()), 1, false);
        // 抓取完毕
        if (_store_Store__WEBPACK_IMPORTED_MODULE_6__.store.idList.length >= this.crawlNumber ||
            this.listPageFinished === this.pageUrlList.length) {
            this.getIdListFinished();
        }
        else {
            // 继续抓取
            this.getIdList();
        }
    }
    resetGetIdListStatus() {
        this.pageUrlList = [];
        this.listPageFinished = 0;
    }
}



/***/ }),

/***/ "./src/ts/crawlNovelPage/InitSearchNovelPage.ts":
/*!******************************************************!*\
  !*** ./src/ts/crawlNovelPage/InitSearchNovelPage.ts ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitSearchNovelPage: () => (/* binding */ InitSearchNovelPage)
/* harmony export */ });
/* harmony import */ var _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../crawl/InitPageBase */ "./src/ts/crawl/InitPageBase.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _pageFunciton_FastScreen__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../pageFunciton/FastScreen */ "./src/ts/pageFunciton/FastScreen.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../pageFunciton/BookmarkAllWorks */ "./src/ts/pageFunciton/BookmarkAllWorks.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../store/IdListWithPageNo */ "./src/ts/store/IdListWithPageNo.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _crawlMixedPage_CrawlTagList__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../crawlMixedPage/CrawlTagList */ "./src/ts/crawlMixedPage/CrawlTagList.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化小说搜索页




















class InitSearchNovelPage extends _crawl_InitPageBase__WEBPACK_IMPORTED_MODULE_0__.InitPageBase {
    constructor() {
        super();
        this.worksWrapSelector = '#root section>div ul';
        this.option = {};
        this.worksNoPerPage = 24; // 每个页面有多少个作品
        this.needCrawlPageCount = 0; // 一共有有多少个列表页面
        this.sendCrawlTaskCount = 0; // 已经抓取了多少个列表页面
        this.allOption = [
            'order',
            'type',
            'wlt',
            'wgt',
            'hlt',
            'hgt',
            'ratio',
            'tool',
            's_mode',
            'mode',
            'scd',
            'ecd',
            'blt',
            'bgt',
            'tlt',
            'tgt',
            'original_only',
            'work_lang',
            'ai_type',
        ];
        this.tipEmptyResult = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.debounce(() => {
            _Log__WEBPACK_IMPORTED_MODULE_7__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_抓取被限制时返回空结果的提示'));
        }, 1000);
        this.crawlTag = () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_16__.states.crawlTagList) {
                this.readyCrawl();
            }
        };
        this.init();
        new _pageFunciton_FastScreen__WEBPACK_IMPORTED_MODULE_8__.FastScreen();
        _crawlMixedPage_CrawlTagList__WEBPACK_IMPORTED_MODULE_15__.crawlTagList.init();
    }
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
        this.addStartTimedCrawlBtn(this.readyCrawl.bind(this));
        this.addCancelTimedCrawlBtn();
    }
    getWorksWrap() {
        const test = document.querySelectorAll(this.worksWrapSelector);
        if (test.length > 0) {
            // 小说页面用这个选择器，只匹配到了一个 ul
            return test[test.length - 1];
        }
        return null;
    }
    addAnyElement() {
        // 添加收藏本页所有作品的功能
        const bookmarkAllBtn = _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.addBtn('otherBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgGreen, '_收藏本页面的所有作品');
        const bookmarkAll = new _pageFunciton_BookmarkAllWorks__WEBPACK_IMPORTED_MODULE_10__.BookmarkAllWorks(bookmarkAllBtn);
        bookmarkAllBtn.addEventListener('click', () => {
            const listWrap = this.getWorksWrap();
            if (listWrap) {
                const list = document.querySelectorAll('#root section>div ul>li');
                if (list.length > 0) {
                    bookmarkAll.sendWorkList(list, 'novels');
                }
            }
        });
    }
    setFormOption() {
        const isPremium = _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.isPremium();
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_3__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: `1 - ${isPremium ? 5000 : 1000}`,
        });
    }
    initAny() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_13__.EVT.list.crawlTag, this.crawlTag);
    }
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.clearSlot('otherBtns');
        window.removeEventListener(_EVT__WEBPACK_IMPORTED_MODULE_13__.EVT.list.crawlTag, this.crawlTag);
    }
    async nextStep() {
        this.setSlowCrawl();
        this.initFetchURL();
        // 计算应该抓取多少页
        const data = await this.getSearchData(1);
        // 计算总页数
        let pageCount = Math.ceil(data.total / this.worksNoPerPage);
        if (pageCount > 1000) {
            // 如果作品页数大于 1000 页，则判断当前用户是否是 pixiv 会员
            const isPremium = _Tools__WEBPACK_IMPORTED_MODULE_9__.Tools.isPremium();
            if (!isPremium) {
                // 如果用户不是会员，则最多只能抓取到 1000 页
                pageCount = 1000;
                _Log__WEBPACK_IMPORTED_MODULE_7__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索页面页数限制', pageCount.toString()));
            }
            else {
                // 如果用户是会员，最多可以抓取到 5000 页
                if (pageCount > 5000) {
                    pageCount = 5000;
                    _Log__WEBPACK_IMPORTED_MODULE_7__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索页面页数限制', pageCount.toString()));
                }
            }
        }
        // 如果当前页面的页码大于有效页码，则不进行抓取
        if (this.startpageNo > pageCount) {
            _EVT__WEBPACK_IMPORTED_MODULE_13__.EVT.fire('crawlComplete');
            _EVT__WEBPACK_IMPORTED_MODULE_13__.EVT.fire('crawlEmpty');
            if (data.total === 0) {
                return _MsgBox__WEBPACK_IMPORTED_MODULE_14__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_作品总数为0'));
            }
            return _MsgBox__WEBPACK_IMPORTED_MODULE_14__.msgBox.error(`${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_超出最大页码')} ${pageCount}`);
        }
        if (this.crawlNumber === -1 || this.crawlNumber > pageCount) {
            this.crawlNumber = pageCount;
        }
        // 计算从当前页面开始抓取的话，有多少页
        let needFetchPage = pageCount - this.startpageNo + 1;
        // 比较用户设置的页数，取较小的那个数值
        this.needCrawlPageCount = Math.min(needFetchPage, this.crawlNumber);
        if (this.needCrawlPageCount === 0) {
            return this.noResult();
        }
        this.getIdList();
    }
    getWantPage() {
        this.crawlNumber = this.checkWantPageInput(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_从本页开始下载x页'), _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载所有页面'));
    }
    // 获取搜索页的数据。因为有多处使用，所以进行了封装
    async getSearchData(p) {
        let data = await _API__WEBPACK_IMPORTED_MODULE_5__.API.getNovelSearchData(_store_Store__WEBPACK_IMPORTED_MODULE_6__.store.tag, p, this.option);
        return data.body.novel;
    }
    // 组织要请求的 url 中的参数
    initFetchURL() {
        let p = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, 'p');
        this.startpageNo = parseInt(p) || 1;
        // 从页面 url 中获取可以使用的选项
        this.option = {};
        this.allOption.forEach((param) => {
            let value = _utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.getURLSearchField(location.href, param);
            if (value !== '') {
                this.option[param] = value;
            }
        });
        // 抓取时始终关闭“以系列为单位显示”
        this.option.gs = '0';
        // 如果 url 里没有显式指定标签匹配模式，则使用 完全一致 模式
        // 因为在这种情况下，pixiv 默认使用的就是 完全一致
        if (!this.option.s_mode) {
            this.option.s_mode = 's_tag_full';
        }
        // 在日志里显示标签匹配模式
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_搜索模式')}: ${this.tipSearchMode(this.option.s_mode)}`);
    }
    // 注意：同样的 mode，在搜索图片时和搜索小说时可能有不同的含义。所以这个方法不是通用的。
    tipSearchMode(mode) {
        switch (mode) {
            case 's_tag_only':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标签部分一致');
            case 's_tag_full':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标签完全一致');
            case 's_tc':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正文');
            case 's_tag':
                return _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_标签标题说明文字');
            default:
                return mode;
        }
    }
    delayReTry(p) {
        window.setTimeout(() => {
            this.getIdList(p);
        }, _Config__WEBPACK_IMPORTED_MODULE_18__.Config.retryTime);
        // 限制时间大约是 3 分钟，这里为了保险起见，设置了更大的延迟时间。
    }
    // 仅当出错重试时，才会传递参数 p。此时直接使用传入的 p，而不是继续让 p 增加
    async getIdList(p) {
        if (_store_States__WEBPACK_IMPORTED_MODULE_16__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        if (p === undefined) {
            p = this.startpageNo + this.sendCrawlTaskCount;
            this.sendCrawlTaskCount++;
        }
        // 发起请求，获取列表页
        let data;
        try {
            data = await this.getSearchData(p);
            if (data.total === 0) {
                console.log(`page ${p}: total 0`);
                this.tipEmptyResult();
                return this.delayReTry(p);
            }
        }
        catch {
            return this.getIdList(p);
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_16__.states.stopCrawl) {
            return this.getIdListFinished();
        }
        data = data.data;
        for (const nowData of data) {
            const filterOpt = {
                aiType: nowData.aiType,
                createDate: nowData.createDate,
                id: nowData.id,
                bookmarkData: nowData.bookmarkData,
                bookmarkCount: nowData.bookmarkCount,
                workType: 3,
                tags: nowData.tags,
                userId: nowData.userId,
                xRestrict: nowData.xRestrict,
            };
            if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_4__.filter.check(filterOpt)) {
                _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_12__.idListWithPageNo.add(_PageType__WEBPACK_IMPORTED_MODULE_17__.pageType.type, {
                    type: 'novels',
                    id: nowData.id,
                }, p);
            }
        }
        this.listPageFinished++;
        _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取进度2', this.listPageFinished.toString(), this.needCrawlPageCount.toString()), 1, false);
        if (this.sendCrawlTaskCount + 1 <= this.needCrawlPageCount) {
            // 继续发送抓取任务（+1 是因为 sendCrawlTaskCount 从 0 开始）
            if (_store_States__WEBPACK_IMPORTED_MODULE_16__.states.slowCrawlMode) {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_19__.setTimeoutWorker.set(() => {
                    this.getIdList();
                }, _Config__WEBPACK_IMPORTED_MODULE_18__.Config.slowCrawlDealy);
            }
            else {
                this.getIdList();
            }
        }
        else {
            // 抓取任务已经全部发送
            if (this.listPageFinished === this.needCrawlPageCount) {
                // 抓取任务全部完成
                _Log__WEBPACK_IMPORTED_MODULE_7__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_列表页抓取完成'));
                _store_IdListWithPageNo__WEBPACK_IMPORTED_MODULE_12__.idListWithPageNo.store(_PageType__WEBPACK_IMPORTED_MODULE_17__.pageType.type);
                this.getIdListFinished();
            }
        }
    }
    resetGetIdListStatus() {
        this.listPageFinished = 0;
        this.sendCrawlTaskCount = 0;
    }
    // 搜索页把下载任务按收藏数从高到低下载
    sortResult() {
        _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.resultMeta.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.sortByProperty('bmk'));
        _store_Store__WEBPACK_IMPORTED_MODULE_6__.store.result.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_11__.Utils.sortByProperty('bmk'));
    }
}



/***/ }),

/***/ "./src/ts/crawl/InitPageBase.ts":
/*!**************************************!*\
  !*** ./src/ts/crawl/InitPageBase.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitPageBase: () => (/* binding */ InitPageBase)
/* harmony export */ });
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _store_SaveArtworkData__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../store/SaveArtworkData */ "./src/ts/store/SaveArtworkData.ts");
/* harmony import */ var _store_SaveNovelData__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../store/SaveNovelData */ "./src/ts/store/SaveNovelData.ts");
/* harmony import */ var _filter_Mute__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../filter/Mute */ "./src/ts/filter/Mute.ts");
/* harmony import */ var _StopCrawl__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./StopCrawl */ "./src/ts/crawl/StopCrawl.ts");
/* harmony import */ var _SelectWork__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../SelectWork */ "./src/ts/SelectWork.ts");
/* harmony import */ var _pageFunciton_DestroyManager__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../pageFunciton/DestroyManager */ "./src/ts/pageFunciton/DestroyManager.ts");
/* harmony import */ var _VipSearchOptimize__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./VipSearchOptimize */ "./src/ts/crawl/VipSearchOptimize.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_22__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _TimedCrawl__WEBPACK_IMPORTED_MODULE_23__ = __webpack_require__(/*! ./TimedCrawl */ "./src/ts/crawl/TimedCrawl.ts");
/* harmony import */ var _pageFunciton_QuickBookmark__WEBPACK_IMPORTED_MODULE_24__ = __webpack_require__(/*! ../pageFunciton/QuickBookmark */ "./src/ts/pageFunciton/QuickBookmark.ts");
/* harmony import */ var _pageFunciton_DisplayThumbnailListOnMultiImageWorkPage__WEBPACK_IMPORTED_MODULE_25__ = __webpack_require__(/*! ../pageFunciton/DisplayThumbnailListOnMultiImageWorkPage */ "./src/ts/pageFunciton/DisplayThumbnailListOnMultiImageWorkPage.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_26__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
// 初始化所有页面抓取流程的基类



























class InitPageBase {
    constructor() {
        this.crawlNumber = 0; // 要抓取的个数/页数
        this.maxCount = 1000; // 当前页面类型最多有多少个页面/作品
        this.startpageNo = 1; // 列表页开始抓取时的页码，只在 api 需要页码时使用
        this.listPageFinished = 0; // 记录一共抓取了多少个列表页
        this.ajaxThreadsDefault = 10; // 抓取作品数据时的并发请求数量默认值，也是最大值
        this.ajaxThread = this.ajaxThreadsDefault; // 抓取时的并发请求数
        this.finishedRequest = 0; // 抓取作品之后，如果 id 队列为空，则统计有几个并发线程完成了请求。当这个数量等于 ajaxThreads 时，说明所有请求都完成了
        this.log429ErrorTip = _utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.debounce(() => {
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_抓取被限制时返回空结果的提示'));
        }, 500);
    }
    // 子组件必须调用 init 方法，并且不可以修改 init 方法
    init() {
        this.setFormOption();
        this.addCrawlBtns();
        this.addAnyElement();
        this.initAny();
        // 如果在 init 方法中绑定了全局事件，并且该事件只适用于当前页面类型，那么应该在 destroy 中解绑事件。
        // 注册当前页面的 destroy 函数
        _pageFunciton_DestroyManager__WEBPACK_IMPORTED_MODULE_15__.destroyManager.register(this.destroy.bind(this));
        // 切换页面时，如果任务已经完成，则移除日志区域
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.bindOnce('clearLogAfterPageSwitch', _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.pageSwitch, () => {
            if (!_store_States__WEBPACK_IMPORTED_MODULE_9__.states.busy) {
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.remove();
            }
        });
        // 监听下载 id 列表的事件
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.bindOnce('crawlIdList', _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.crawlIdList, (ev) => {
            const idList = ev.detail.data;
            if (idList) {
                this.crawlIdList(idList);
            }
            // 通过 bindOnce 绑定的 this 是执行此代码时通过这个虚拟类生成的实例，这个 this 是不会变化的
            // 但是这个虚拟类会产生多个实例，所以这里调用 this 的方法时，要求这个方法与具体实例无关，不受实例变化影响
            // 也就是说即使页面类型变化并且生成了新的实例，调用旧实例上的这个方法也依然会正常运行
            // 如果某个方法做不到这一点, 就不要在这里调用。
            // 基于此，在这里修改 this 上的属性是不合适的，因为每个新实例都会复制这个虚拟类上的属性，它们是独立的
        });
    }
    // 设置表单里的选项。主要是设置页数，并隐藏不需要的选项。
    setFormOption() {
        // 个数/页数选项的提示
        _setting_Options__WEBPACK_IMPORTED_MODULE_7__.options.setWantPageTip({
            text: '_抓取多少页面',
            tip: '_从本页开始下载提示',
            rangTip: '_数字提示1',
        });
    }
    // 添加抓取区域的默认按钮，可以被子类覆写
    addCrawlBtns() {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_开始抓取', '_默认下载多页').addEventListener('click', () => {
            this.readyCrawl();
        });
    }
    // 添加其他任意元素（如果有）
    addAnyElement() { }
    // 初始化任意内容
    // 如果有一些代码不能归纳到 init 方法的前面几个方法里，那就放在这里
    // 通常用来初始化特有的组件、功能、事件、状态等
    initAny() { }
    // 销毁初始化页面时添加的元素和事件，恢复设置项等
    destroy() {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.clearSlot('crawlBtns');
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.clearSlot('otherBtns');
    }
    // 作品个数/页数的输入不合法
    getWantPageError() {
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('wrongSetting');
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_下载数量错误');
        _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.error(msg);
        throw new Error(msg);
    }
    // 在某些页面检查页数/个数设置
    // 可以为 -1，或者大于 0
    checkWantPageInput(crawlPartTip, crawlAllTip) {
        const want = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.wantPageArr[_PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.type];
        // 如果比 1 小，并且不是 -1，则不通过
        if ((want < 1 && want !== -1) || isNaN(want)) {
            // 比 1 小的数里，只允许 -1 , 0 也不行
            throw this.getWantPageError();
        }
        if (want >= 1) {
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(crawlPartTip.replace('{}', want.toString()));
        }
        else if (want === -1) {
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(crawlAllTip);
        }
        return want;
    }
    // 在某些页面检查页数/个数设置，要求必须大于 0
    // 参数 max 为最大值
    // 参数 page 指示单位是“页”（页面）还是“个”（作品个数）
    checkWantPageInputGreater0(max, page) {
        const want = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.wantPageArr[_PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.type];
        if (want > 0) {
            const result = Math.min(want, max);
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl(page ? '_从本页开始下载x页' : '_从本页开始下载x个', result.toString()));
            return result;
        }
        else {
            throw this.getWantPageError();
        }
    }
    // 设置要获取的作品数或页数。有些页面使用，有些页面不使用。使用时再具体定义
    getWantPage() { }
    // 获取多图作品设置。因为这个不属于过滤器 filter，所以在这里直接获取
    getMultipleSetting() {
        // 获取作品张数设置
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.firstFewImagesSwitch) {
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(`${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_多图作品只下载前几张图片')} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.firstFewImages}`);
        }
    }
    setSlowCrawl() {
        _store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.slowCrawl;
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.slowCrawl) {
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_慢速抓取'));
        }
    }
    // 准备正常进行抓取，执行一些检查
    async readyCrawl() {
        // 检查是否可以开始抓取
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.busy) {
            _Toast__WEBPACK_IMPORTED_MODULE_17__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_当前任务尚未完成'));
            return;
        }
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('clearLog');
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_开始抓取'));
        _Toast__WEBPACK_IMPORTED_MODULE_17__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_开始抓取'), {
            position: 'center',
        });
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlStart');
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.isPixiv()) {
            await _filter_Mute__WEBPACK_IMPORTED_MODULE_12__.mute.getMuteSettings();
        }
        this.getWantPage();
        this.getMultipleSetting();
        this.finishedRequest = 0;
        _store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl = false;
        // 进入第一个抓取流程
        this.nextStep();
    }
    // 基于传递的 id 列表直接开始抓取
    // 这个方法是为了让其他模块可以传递 id 列表，直接进行下载。
    // 这个类的子类没有必要使用这个方法。当子类需要直接指定 id 列表时，修改自己的 getIdList 方法即可。
    async crawlIdList(idList) {
        // 检查是否可以开始抓取
        // 如果不能抓取则把 id 列表添加到等待队列中
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.busy) {
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.waitingIdList.push(...idList);
        }
        else {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('clearLog');
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_开始抓取'));
            _Toast__WEBPACK_IMPORTED_MODULE_17__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_开始抓取'), {
                position: 'center',
            });
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlStart');
            if (_utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.isPixiv()) {
                await _filter_Mute__WEBPACK_IMPORTED_MODULE_12__.mute.getMuteSettings();
            }
            this.getMultipleSetting();
            this.finishedRequest = 0;
            _store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl = false;
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList = idList;
            this.getIdListFinished();
        }
    }
    // 当可以开始抓取时，进入下一个流程。默认情况下，开始获取作品列表。如有不同，由子类具体定义
    nextStep() {
        this.getIdList();
    }
    // 获取 id 列表，由各个子类具体定义
    getIdList() { }
    // id 列表获取完毕，开始抓取作品内容页
    async getIdListFinished() {
        _store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode = false;
        this.resetGetIdListStatus();
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('getIdListFinished');
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl || _store_States__WEBPACK_IMPORTED_MODULE_9__.states.bookmarkMode) {
            return;
        }
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length === 0) {
            return this.noResult();
        }
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_当前作品个数', _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length.toString()));
        // 导出 ID 列表，并停止抓取
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.exportIDList && _utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.isPixiv()) {
            const resultList = await _utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.json2BlobSafe(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList);
            for (const result of resultList) {
                _utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.downloadFile(result.url, `ID list-total ${result.total}-from ${_Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getPageTitle()}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.replaceUnsafeStr(new Date().toLocaleString())}.json`);
            }
            _store_States__WEBPACK_IMPORTED_MODULE_9__.states.busy = false;
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('stopCrawl');
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_导出ID列表'));
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_已停止抓取'));
            return;
        }
        // 这个 return 在这里重置任务状态，不继续抓取作品的详情了，用于调试时反复进行抓取
        // return states.busy = false
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_开始获取作品信息'));
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.slowCrawl &&
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length > _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.slowCrawlOnWorksNumber) {
            // 慢速抓取
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_慢速抓取'));
            _store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode = true;
            this.ajaxThread = 1;
        }
        else {
            // 全速抓取
            _store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode = false;
            this.ajaxThread = Math.min(this.ajaxThreadsDefault, _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length);
        }
        // 开始抓取作品数据
        for (let i = 0; i < this.ajaxThread; i++) {
            window.setTimeout(() => {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length > 0 ? this.getWorksData() : this.afterGetWorksData();
            }, 0);
        }
    }
    // 重设抓取作品列表时使用的变量或标记
    resetGetIdListStatus() { }
    // 获取作品的数据
    async getWorksData(idData) {
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
            return this.crawlFinished();
        }
        idData = idData || _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.shift();
        if (!idData) {
            return this.afterGetWorksData();
        }
        const id = idData.id;
        if (!id) {
            const msg = 'Error: work id is invalid!';
            _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.error(msg);
            throw new Error(msg);
        }
        // 在抓取之前，预先对 id 进行检查，如果不符合要求则不发送这个请求，直接跳过它
        // 现在这里能够检查 2 种设置条件：
        // 1. 检查 id 是否符合 id 范围条件
        // 2. 检查 id 的发布时间是否符合时间范围条件
        // 3. 区分图像作品和小说。注意：因为在某些情况下，下载器只能确定一个作品是图像还是小说，但不能区分它具体是图像里的哪一种类型（插画、漫画、动图），所以这里不能检查具体的图像类型，只能检查是图像还是小说
        const check = await _filter_Filter__WEBPACK_IMPORTED_MODULE_21__.filter.check({
            id,
            workTypeString: idData.type,
            workType: _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getWorkTypeVague(idData.type),
        });
        if (!check) {
            return this.afterGetWorksData();
        }
        try {
            const unlisted = _PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.list.Unlisted;
            // 这里不能使用 cacheWorkData中的缓存数据，因为某些数据（如作品的收藏状态）可能已经发生变化
            if (idData.type === 'novels') {
                const data = await _API__WEBPACK_IMPORTED_MODULE_3__.API.getNovelData(id, unlisted);
                await _store_SaveNovelData__WEBPACK_IMPORTED_MODULE_11__.saveNovelData.save(data);
                this.afterGetWorksData(data);
            }
            else {
                const data = await _API__WEBPACK_IMPORTED_MODULE_3__.API.getArtworkData(id, unlisted);
                await _store_SaveArtworkData__WEBPACK_IMPORTED_MODULE_10__.saveArtworkData.save(data);
                this.afterGetWorksData(data);
            }
        }
        catch (error) {
            // 当 API 里的网络请求的状态码异常时，会 reject，被这里捕获
            if (error.status) {
                // 请求成功，但状态码不正常
                this.logErrorStatus(error.status, idData);
                if (error.status === 500 || error.status === 429) {
                    // 如果状态码 500 或 429，获取不到作品数据，可能是被 pixiv 限制了，等待一段时间后再次发送这个请求
                    this.log429ErrorTip();
                    window.setTimeout(() => {
                        this.getWorksData(idData);
                    }, _Config__WEBPACK_IMPORTED_MODULE_22__.Config.retryTime);
                    return;
                }
                else {
                    this.afterGetWorksData();
                }
            }
            else {
                // 请求失败，没有获得服务器的返回数据，一般都是
                // TypeError: Failed to fetch
                // 此外这里也会捕获到 save 作品数据时的错误（如果有）
                console.error(error);
                // 再次发送这个请求
                window.setTimeout(() => {
                    this.getWorksData(idData);
                }, 2000);
            }
        }
    }
    // 每当获取完一个作品的信息
    async afterGetWorksData(data) {
        this.logResultNumber();
        // 抓取可能中途停止，保留抓取结果
        if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl) {
            return this.crawlFinished();
        }
        // 如果会员搜索优化策略指示停止抓取，则立即进入完成状态
        if (data && (await _VipSearchOptimize__WEBPACK_IMPORTED_MODULE_16__.vipSearchOptimize.stopCrawl(data))) {
            // 指示抓取已停止
            _store_States__WEBPACK_IMPORTED_MODULE_9__.states.stopCrawl = true;
            return this.crawlFinished();
        }
        // 在进行下一次抓取前，预先检查这个 id 是否符合过滤条件
        // 如果它不符合过滤条件，则立刻跳过它，这样也不会发送请求来获取这个作品的数据
        // 这样可以加快抓取速度
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length > 0) {
            const nextIDData = _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList[0];
            const check = await _filter_Filter__WEBPACK_IMPORTED_MODULE_21__.filter.check({
                id: nextIDData.id,
                workTypeString: nextIDData.type,
                workType: _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getWorkTypeVague(nextIDData.type),
            });
            if (!check) {
                _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.shift();
                return this.getWorksData();
            }
        }
        // 如果存在下一个作品，则继续抓取
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length > 0) {
            if (_store_States__WEBPACK_IMPORTED_MODULE_9__.states.slowCrawlMode) {
                _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_26__.setTimeoutWorker.set(() => {
                    this.getWorksData();
                }, _Config__WEBPACK_IMPORTED_MODULE_22__.Config.slowCrawlDealy);
            }
            else {
                this.getWorksData();
            }
        }
        else {
            // 没有剩余作品，统计此后有多少个完成的请求
            this.finishedRequest++;
            // 所有请求都执行完毕
            if (this.finishedRequest === this.ajaxThread) {
                this.crawlFinished();
            }
        }
    }
    // 抓取完毕
    crawlFinished() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result.length === 0) {
            return this.noResult();
        }
        _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.crawlCompleteTime = new Date();
        // 对文件进行排序
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.setFileDownloadOrder) {
            // 按照用户设置的规则进行排序
            const scheme = new Map([
                ['ID', 'id'],
                ['bookmarkCount', 'bmk'],
                ['bookmarkID', 'bmkId'],
            ]);
            let key = scheme.get(_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.downloadOrderSortBy);
            // 在搜索页面预览抓取结果时，始终按收藏数量排序
            if (_PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_20__.pageType.list.ArtworkSearch &&
                _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.previewResult) {
                key = 'bmk';
            }
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.sortByProperty(key, _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.downloadOrder));
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.resultMeta.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_19__.Utils.sortByProperty(key, _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.downloadOrder));
        }
        else {
            // 如果用户未设置排序规则，则每个页面自行处理排序逻辑
            this.sortResult();
        }
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_共抓取到n个作品', _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.resultMeta.length.toString()));
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_共抓取到n个文件', _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result.length.toString()));
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_抓取完毕'), 2);
        // 发出抓取完毕的信号
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlComplete');
        // 自动导出抓取结果
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.autoExportResult &&
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result.length > _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.autoExportResultNumber) {
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.autoExportResultCSV) {
                _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('exportCSV');
            }
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.autoExportResultJSON) {
                _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('exportResult');
            }
        }
    }
    // 网络请求状态异常时输出提示
    logErrorStatus(status, idData) {
        const isNovel = idData.type === 'novels';
        const workLink = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.createWorkLink(idData.id, !isNovel);
        switch (status) {
            case 0:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码0'));
                break;
            case 400:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码400'));
                break;
            case 401:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码401'));
                break;
            case 403:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码403'));
                break;
            case 404:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码404'));
                break;
            case 429:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码429'));
                break;
            case 500:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(workLink + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_作品页状态码500'));
                break;
            default:
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_无权访问', workLink) + `HTTP status code: ${status}`);
                break;
        }
    }
    // 每当抓取了一个作品之后，输出提示
    logResultNumber() {
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_待处理')} ${_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.idList.length}, ${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_共抓取到n个作品', _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.resultMeta.length.toString())}`, 1, false);
    }
    // 抓取结果为 0 时输出提示
    noResult() {
        // 先触发 crawlComplete，后触发 crawlEmpty。这样便于其他组件处理 crawlEmpty 这个例外情况
        // 如果触发顺序反过来，那么最后执行的都是 crawlComplete，可能会覆盖对 crawlEmpty 的处理
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlComplete');
        _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('crawlEmpty');
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_抓取结果为零');
        _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(msg, 2);
        _MsgBox__WEBPACK_IMPORTED_MODULE_18__.msgBox.error(msg);
    }
    // 抓取完成后，对结果进行排序
    sortResult() { }
    /**定时抓取的按钮 */
    addStartTimedCrawlBtn(cb) {
        _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgBlue, '_定时抓取', '_定时抓取说明').addEventListener('click', () => {
            _TimedCrawl__WEBPACK_IMPORTED_MODULE_23__.timedCrawl.start(cb);
        });
    }
    /**取消定时抓取的按钮 */
    addCancelTimedCrawlBtn() {
        const btn = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_1__.Colors.bgWarning, '_取消定时抓取');
        btn.style.display = 'none';
        btn.addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.fire('cancelTimedCrawl');
            btn.style.display = 'none';
        });
        // 启动定时抓取之后，显示取消定时抓取的按钮
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_6__.EVT.list.startTimedCrawl, () => {
            btn.style.display = 'flex';
        });
    }
}



/***/ }),

/***/ "./src/ts/crawl/InitUnsupportedPage.ts":
/*!*********************************************!*\
  !*** ./src/ts/crawl/InitUnsupportedPage.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   InitUnsupportedPage: () => (/* binding */ InitUnsupportedPage)
/* harmony export */ });
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");
/* harmony import */ var _InitPageBase__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./InitPageBase */ "./src/ts/crawl/InitPageBase.ts");


// 初始化不支持的页面类型
class InitUnsupportedPage extends _InitPageBase__WEBPACK_IMPORTED_MODULE_1__.InitPageBase {
    constructor() {
        super();
        this.init();
    }
    // 在不支持的页面类型里，不会添加专门用于当前页面的抓取按钮
    // 只会由 SelectWork 模块添加通用的“手动抓取”功能
    addCrawlBtns() { }
    setFormOption() {
        _setting_Options__WEBPACK_IMPORTED_MODULE_0__.options.hideOption([1]);
    }
}



/***/ }),

/***/ "./src/ts/crawl/StopCrawl.ts":
/*!***********************************!*\
  !*** ./src/ts/crawl/StopCrawl.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");







class StopCrawl {
    constructor() {
        this.addBtn();
        this.bindEvents();
    }
    addBtn() {
        this.btn = _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.addBtn('stopCrawl', _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgRed, '_停止抓取');
        this.hide();
        this.btn.addEventListener('click', () => {
            this.hide();
            const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已停止抓取');
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(msg);
            _Toast__WEBPACK_IMPORTED_MODULE_4__.toast.error(msg);
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('stopCrawl');
            _store_States__WEBPACK_IMPORTED_MODULE_6__.states.stopCrawl = true;
        });
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlStart, () => {
            this.show();
        });
        const hiddenEvents = [_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete, _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.stopCrawl];
        hiddenEvents.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.hide();
            });
        });
    }
    hide() {
        this.btn.style.display = 'none';
    }
    show() {
        this.btn.style.display = 'flex';
    }
}
new StopCrawl();


/***/ }),

/***/ "./src/ts/crawl/TimedCrawl.ts":
/*!************************************!*\
  !*** ./src/ts/crawl/TimedCrawl.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   timedCrawl: () => (/* binding */ timedCrawl)
/* harmony export */ });
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Input__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Input */ "./src/ts/Input.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");








class TimedCrawl {
    constructor() {
        this.time = 0;
        /**定时器可用的最大延迟时间，这里计算为分钟 */
        // https://developer.mozilla.org/zh-CN/docs/Web/API/setTimeout#%E6%9C%80%E5%A4%A7%E5%BB%B6%E6%97%B6%E5%80%BC
        // max: 2147483647 / 60 / 1000
        this.timeMinuteMax = 35791;
        /**这次抓取是否是由本模块发起的 */
        this.crawlBySelf = false;
        this.bindEvents();
    }
    /**启动定时抓取任务。
     *
     * 只能有 1 个定时抓取任务，如果重复调用此方法，后传递的回调函数会覆盖之前的回调函数。
     */
    async start(cb) {
        const input = new _Input__WEBPACK_IMPORTED_MODULE_6__.Input({
            instruction: `${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取的间隔时间2')} (${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_分钟')})`,
            value: _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.timedCrawlInterval.toString(),
            width: 500,
        });
        const value = await input.submit();
        if (!value) {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_本次操作已取消'));
        }
        const number = Number.parseInt(value);
        if (isNaN(number)) {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_参数不合法本次操作已取消'));
        }
        if (number > this.timeMinuteMax) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取的时间超过最大值') +
                this.timeMinuteMax +
                _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_分钟'));
            return;
        }
        if (number < 1) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取的时间最小值'));
            return;
        }
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.setSetting)('timedCrawlInterval', number);
        this.reset();
        this.callback = cb;
        this.execute();
        _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.fire('startTimedCrawl');
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取已启动的提示', _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.timedCrawlInterval.toString());
        _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.show(msg +
            '<br><br>' +
            _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取的推荐用法') +
            '<br><br>' +
            _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_定时抓取已启动的提示2'));
        _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(msg);
    }
    execute() {
        this.time = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.timedCrawlInterval * 60 * 1000;
        this.timer = window.setTimeout(() => {
            if (!this.callback) {
                return;
            }
            this.crawlBySelf = true;
            _store_States__WEBPACK_IMPORTED_MODULE_5__.states.quickCrawl = true;
            this.callback();
            this.execute();
        }, this.time);
    }
    reset() {
        this.callback = undefined;
        window.clearTimeout(this.timer);
        this.crawlBySelf = false;
        _store_States__WEBPACK_IMPORTED_MODULE_5__.states.quickCrawl = false;
    }
    bindEvents() {
        // 当抓取结果为空，或者下载中止、完成时复位标记
        const resetCrawlBySelf = [
            _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.crawlEmpty,
            _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.downloadComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.downloadCancel,
        ];
        for (const ev of resetCrawlBySelf) {
            window.addEventListener(ev, () => {
                window.setTimeout(() => {
                    // 需要延迟执行，在日志提示显示之后再复位状态
                    this.crawlBySelf = false;
                }, 50);
            });
        }
        // 显示一些提示
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.crawlStart, () => {
            if (!this.crawlBySelf) {
                return;
            }
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_开始定时抓取'));
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前时间') + new Date().toLocaleString());
        });
        const tipWaitNextCrawl = [_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.crawlEmpty, _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.downloadComplete];
        for (const ev of tipWaitNextCrawl) {
            window.addEventListener(ev, () => {
                window.setTimeout(() => {
                    if (this.crawlBySelf) {
                        _Log__WEBPACK_IMPORTED_MODULE_3__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前时间') + new Date().toLocaleString());
                        _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_等待下一次定时抓取'));
                    }
                }, 0);
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.cancelTimedCrawl, () => {
            this.reset();
            const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_已取消定时抓取');
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(msg);
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.success(msg);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.pageSwitch, () => {
            if (!this.callback) {
                return;
            }
            this.reset();
            const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_因为URL变化取消定时抓取任务');
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(msg);
            _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.error(msg);
        });
    }
}
const timedCrawl = new TimedCrawl();



/***/ }),

/***/ "./src/ts/crawl/VipSearchOptimize.ts":
/*!*******************************************!*\
  !*** ./src/ts/crawl/VipSearchOptimize.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   vipSearchOptimize: () => (/* binding */ vipSearchOptimize)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");






// 当 Pixiv 会员使用按热门度排序搜索时，通过检查收藏数量是否符合要求来进行优化
// 原理：当会员使用热门度排序时，Pixiv 返回的数据是按收藏数量从高到低排序的。（但不是严格一致，经常有少量作品顺序不对）
// 假如会员用户在下载器里设置了收藏数量大于 10000，那么当查找到小于 10000 收藏的作品时，就可以考虑停止抓取作品了，因为后面的作品都是收藏数量低于 10000 的了
class VipSearchOptimize {
    constructor() {
        // 在哪些页面上启用
        this.enablePageType = [_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.list.ArtworkSearch];
        // 小说搜索页面不需要优化，因为列表数据中包含了每个作品的收藏数
        // 只有会员才能使用的排序方式（按热门度排序）
        this.vipOrders = [
            'popular_d',
            'popular_male_d',
            'popular_female_d',
        ];
        // popular_d 受全站欢迎
        // popular_male_d 受男性欢迎
        // popular_female_d 受女性欢迎
        // 是否对这次抓取使用优化策略
        this.vipSearchOptimize = false;
        this.filterFailed = 0; // 连续检查失败的数量。在检查作品是否满足收藏条件时，如果满足就将此计数清零；如果不满足就自增
        this.checkNumber = 30; // 连续多少个作品未达到要求时，停止抓取。这是一个猜测值
        this.bindEvents();
    }
    // 设置 checkNumber 的原因：Pixiv 按热门度排序返回的数据其实并不是严格按照收藏数量排序的。所以设置一个数字作为处理这个情况的手段：连续多少个作品都不满足要求时，认为后续都是不满足要求的
    bindEvents() {
        // 启动抓取时设置是否启用优化策略
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.vipSearchOptimize = this.setVipOptimize();
        });
        // 抓取完毕时重置状态
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete, () => {
            this.reset();
        });
    }
    reset() {
        this.vipSearchOptimize = false;
        this.filterFailed = 0;
    }
    // 指示是否停止抓取作品
    async stopCrawl(data) {
        // 如果未启用会员搜索优化，或者没有设置收藏数量要求，则不停止抓取
        if (!this.vipSearchOptimize || !_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.BMKNumSwitch) {
            return false;
        }
        // 连续多少个作品没有达到要求，则停止抓取
        if (this.filterFailed >= this.checkNumber) {
            return true;
        }
        // 判断收藏数量是否不符合要求
        // createDate 用于计算日均收藏数量，必须传递
        const check = await _filter_Filter__WEBPACK_IMPORTED_MODULE_3__.filter.check({
            bookmarkCount: data.body.bookmarkCount,
            createDate: data.body.createDate,
        });
        if (!check) {
            this.filterFailed++;
        }
        else {
            this.filterFailed = 0;
        }
        return this.filterFailed >= this.checkNumber;
    }
    // 设置是否启用优化策略
    setVipOptimize() {
        // 判断页面类型
        if (!this.enablePageType.includes(_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type)) {
            return false;
        }
        // 判断是否是会员
        if (!_Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.isPremium()) {
            return false;
        }
        // 判断 order 方式
        const order = _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.getURLSearchField(window.location.href, 'order');
        // 无排序方式
        if (!order) {
            return false;
        }
        const vipOrder = this.vipOrders.includes(order);
        // 不是按热门度排序
        if (!vipOrder) {
            return false;
        }
        // 按热门度排序
        // 判断是否启用了收藏数设置，如果是，则启用会员搜索优化
        return _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.BMKNumSwitch;
    }
}
const vipSearchOptimize = new VipSearchOptimize();



/***/ }),

/***/ "./src/ts/download/BookmarkAfterDL.ts":
/*!********************************************!*\
  !*** ./src/ts/download/BookmarkAfterDL.ts ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   BookmarkAfterDL: () => (/* binding */ BookmarkAfterDL)
/* harmony export */ });
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");






// 当文件下载成功后，收藏这个作品
class BookmarkAfterDL {
    constructor(tipEl) {
        // 储存接收到的 id，用于防止对一个作品重复添加收藏
        // 其实重复添加收藏没什么影响，和只添加一次没区别。为了不浪费网络请求，还是尽量不要重复添加。
        this.savedIds = [];
        this.successCount = 0;
        this.tipEl = document.createElement('span');
        // 如果之前的下载已完成，那么当下一次开始下载时（也就是新的下载，而不是暂停后继续的下载），则重置状态
        this.delayReset = false;
        this.showCompleteTip = true;
        if (tipEl) {
            this.tipEl = tipEl;
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.tipEl);
        }
        this.bindEvents();
    }
    // 可选传入一个元素，显示收藏的数量和总数
    bindEvents() {
        // 当有文件下载完成时，提取 id 进行收藏
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.downloadSuccess, (ev) => {
            const successData = ev.detail.data;
            this.send(Number.parseInt(successData.id));
        });
        // 当有文件跳过下载时，如果是重复的下载，也进行收藏
        // 因为重复的下载，本意还是要下载的，只是之前下载过了。所以进行收藏。
        // 其他跳过下载的原因，则是本意就是不下载，所以不收藏。
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.skipDownload, (ev) => {
            const skipData = ev.detail.data;
            if (skipData.reason === 'duplicate') {
                this.send(Number.parseInt(skipData.id));
            }
        });
        // 当开始新的抓取时重置状态和提示
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.crawlStart, (ev) => {
            this.reset();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.downloadComplete, () => {
            this.delayReset = true;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.downloadStart, () => {
            if (this.delayReset) {
                this.reset();
                this.delayReset = false;
            }
        });
    }
    showProgress() {
        if (this.savedIds.length === 0) {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.tipEl, '');
            return;
        }
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.tipEl, '_已收藏带参数', `${this.successCount}/${this.savedIds.length}`);
        if (this.successCount === this.savedIds.length && this.showCompleteTip) {
            // 当全部收藏完成时，只显示一次提示。否则会显示多次
            this.showCompleteTip = false;
            _Log__WEBPACK_IMPORTED_MODULE_5__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_收藏作品完毕'));
        }
    }
    reset() {
        this.showCompleteTip = true;
        this.savedIds = [];
        this.successCount = 0;
        this.tipEl.classList.remove('red');
        this.tipEl.classList.add('green');
        this.showProgress();
    }
    // 接收作品 id，开始收藏
    send(id) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.bmkAfterDL) {
            return;
        }
        if (typeof id !== 'number') {
            id = Number.parseInt(id);
        }
        // 检查这个 id 是否已经添加了
        if (this.savedIds.includes(id)) {
            return;
        }
        this.addBookmark(id);
    }
    // 给所有作品添加收藏（之前收藏过的，新 tag 将覆盖旧 tag）
    async addBookmark(id) {
        return new Promise(async (resolve, reject) => {
            this.savedIds.push(id);
            this.showProgress();
            // 从 store 里查找这个作品的数据
            const dataSource = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta.length > 0 ? _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta : _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result;
            const data = dataSource.find((val) => val.idNum === id);
            if (data === undefined) {
                _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(`Not find ${id} in result`);
                return resolve();
            }
            // 当抓取结果很少时，不使用慢速收藏
            await _Bookmark__WEBPACK_IMPORTED_MODULE_4__.bookmark.add(id.toString(), data.type !== 3 ? 'illusts' : 'novels', data.tags, undefined, undefined, _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length > 24);
            this.successCount++;
            this.showProgress();
            resolve();
        });
    }
}



/***/ }),

/***/ "./src/ts/download/CheckWarningMessage.ts":
/*!************************************************!*\
  !*** ./src/ts/download/CheckWarningMessage.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");




/**当下载数量达到一定数值时，检查当前用户是否被 pixiv 警告 */
class CheckWarningMessage {
    constructor() {
        /**已下载（成功保存到硬盘上）的文件数量
         *
         * 这个数字不会重置，除非当前标签页被关闭
         */
        this.downloaded = 0;
        /**每当保存数量增加了指定数量时，进行一次检查 */
        this.unitNumber = 100;
        /**上次检查时的下载数量 */
        this.lastCheckDownloaded = 0;
        this.bindEvents();
    }
    bindEvents() {
        // 当有文件保存成功后，计算已下载文件的数量（不会计算跳过的文件）
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadSuccess, () => {
            this.addDownloaded();
        });
    }
    async addDownloaded() {
        this.downloaded++;
        if (this.downloaded >= this.lastCheckDownloaded + this.unitNumber) {
            this.lastCheckDownloaded = this.downloaded;
            const result = await this.check();
            if (result) {
                _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_过度访问警告警告') + '<br>' + _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已暂停'));
                return _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('requestPauseDownload');
            }
        }
    }
    async check() {
        return new Promise(async (resolve, reject) => {
            const data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getLatestMessage(3);
            if (data.error) {
                console.error(data.message);
                return resolve(false);
            }
            if (data.body.total === 0) {
                return resolve(false);
            }
            for (const msgData of data.body.message_threads) {
                if (msgData.is_official === true &&
                    msgData.thread_name === 'pixiv事務局') {
                    if (msgData.latest_content.includes('policies.pixiv.net') &&
                        msgData.latest_content.includes('14')) {
                        // 如果找到了官方账号发送的警告消息，则判断时间
                        const now = new Date().getTime();
                        const msgTime = Number.parseInt(msgData.modified_at + '000');
                        // 如果这是 1 小时内的消息，则视为有效的警告消息
                        // 如果警告消息的时间过去比较久了，则不再显示提示消息，否则就会无限提示了
                        // 在进行大量下载时，pixiv 的警告消息可能会延迟几十分钟发送
                        if (now - msgTime < 60 * 60 * 1000) {
                            return resolve(true);
                        }
                    }
                }
            }
            return resolve(false);
        });
    }
}
new CheckWarningMessage();


/***/ }),

/***/ "./src/ts/download/Download.ts":
/*!*************************************!*\
  !*** ./src/ts/download/Download.ts ***!
  \*************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Download: () => (/* binding */ Download)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _ConvertUgoira_ConvertUgoira__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../ConvertUgoira/ConvertUgoira */ "./src/ts/ConvertUgoira/ConvertUgoira.ts");
/* harmony import */ var _ProgressBar__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./ProgressBar */ "./src/ts/download/ProgressBar.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _DownloadRecord__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./DownloadRecord */ "./src/ts/download/DownloadRecord.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _MakeNovelFile__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./MakeNovelFile */ "./src/ts/download/MakeNovelFile.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ./DownloadNovelEmbeddedImage */ "./src/ts/download/DownloadNovelEmbeddedImage.ts");
/* harmony import */ var _DownloadNovelCover__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./DownloadNovelCover */ "./src/ts/download/DownloadNovelCover.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _DownloadStates__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ./DownloadStates */ "./src/ts/download/DownloadStates.ts");
// 下载文件，然后发送给浏览器进行保存



















// 处理下载队列里的任务
// 不显示在进度条上的下载任务，不在这里处理
class Download {
    constructor(progressBarIndex, data, downloadStatesIndex) {
        this.retry = 0; // 重试次数
        this.lastRequestTime = 0; // 最后一次发起请求的时间戳
        this.retryInterval = []; // 保存每次到达重试环节时，距离上一次请求的时间差
        this.sizeChecked = false; // 是否对文件体积进行了检查
        this.skip = false; // 这个下载是否应该被跳过。如果这个文件不符合某些过滤条件就应该跳过它
        this.error = false; // 在下载过程中是否出现了无法解决的错误
        this.progressBarIndex = progressBarIndex;
        this.downloadStatesIndex = downloadStatesIndex;
        this.beforeDownload(data);
    }
    get cancel() {
        return this.skip || this.error || !_store_States__WEBPACK_IMPORTED_MODULE_13__.states.downloading;
    }
    // 跳过下载这个文件。可以传入用于提示的文本
    skipDownload(data, msg) {
        this.skip = true;
        if (msg) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(msg);
        }
        if (_store_States__WEBPACK_IMPORTED_MODULE_13__.states.downloading) {
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('skipDownload', data);
        }
    }
    // 在开始下载前进行检查
    async beforeDownload(arg) {
        // 检查是否是重复文件
        const duplicate = await _DownloadRecord__WEBPACK_IMPORTED_MODULE_7__.downloadRecord.checkDeduplication(arg.result);
        if (duplicate) {
            return this.skipDownload({
                id: arg.id,
                reason: 'duplicate',
            }, _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_跳过下载因为重复文件', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.id, arg.result.type !== 3)));
        }
        // 如果是动图，再次检查是否排除了动图
        // 因为有时候用户在抓取时没有排除动图，但是在下载时排除了动图。所以下载时需要再次检查
        if (arg.result.type === 2 && !_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.downType2) {
            return this.skipDownload({
                id: arg.id,
                reason: 'excludedType',
            });
        }
        // 检查宽高条件和宽高比
        if ((_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.setWHSwitch || _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.ratioSwitch) &&
            arg.result.type !== 3) {
            // 默认使用当前作品中第一张图片的宽高
            let wh = {
                width: arg.result.fullWidth,
                height: arg.result.fullHeight,
            };
            // 如果不是第一张图片，则加载图片以获取宽高
            if (arg.result.index > 0) {
                // 始终获取原图的尺寸
                wh = await _utils_Utils__WEBPACK_IMPORTED_MODULE_10__.Utils.getImageSize(arg.result.original);
            }
            // 如果获取宽高失败，图片会被视为通过宽高检查
            if (wh.width === 0 || wh.height === 0) {
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_获取图片的宽高时出现错误') +
                    _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.id));
                // 图片加载失败可能是请求超时，或者图片不存在。这里无法获取到具体原因，所以不直接返回。
                // 如果是 404 错误，在 download 方法中可以处理这个问题
                // 如果是请求超时，则有可能错误的通过了这个图片
            }
            const result = await _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check(wh);
            if (!result) {
                return this.skipDownload({
                    id: arg.id,
                    reason: 'widthHeight',
                }, _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_不保存图片因为宽高', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.id)));
            }
        }
        this.download(arg);
    }
    // 设置进度条信息
    setProgressBar(name, loaded, total) {
        _ProgressBar__WEBPACK_IMPORTED_MODULE_5__.progressBar.setProgress(this.progressBarIndex, {
            name,
            loaded,
            total,
        });
    }
    // 当重试达到最大次数时
    afterReTryMax(status, fileId) {
        const errorMsg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_作品id无法下载带状态码', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(fileId), status.toString());
        // 404, 500 错误，跳过，不会再尝试下载这个文件（因为没有触发 downloadError 事件，所以不会重试下载）
        if (status === 404 || status === 500) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(errorMsg);
            return this.skipDownload({
                id: fileId,
                reason: status.toString(),
            });
        }
        // 状态码为 0，可能是系统磁盘空间不足导致的错误，也可能是代理软件导致的网络错误
        // 超时也会返回状态码 0
        if (status === 0) {
            // 判断是否是磁盘空间不足。特征是每次重试之间的间隔时间比较短。
            // 如果是超时，那么等待时间会比较长，可能超过 20 秒
            const timeLimit = 10000; // 如果从发起请求到进入重试的时间间隔小于这个值，则视为磁盘空间不足的情况
            const result = this.retryInterval.filter((val) => val <= timeLimit);
            // 在全部的 10 次请求中，如果有 9 次小于 10 秒，就有可能是磁盘空间不足。
            if (result.length > 9) {
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(errorMsg);
                const tip = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_状态码为0的错误提示');
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(tip);
                _MsgBox__WEBPACK_IMPORTED_MODULE_12__.msgBox.error(tip);
                return _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('requestPauseDownload');
            }
        }
        // 其他状态码，暂时跳过这个任务，但最后还是会尝试重新下载它
        this.error = true;
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadError', fileId);
    }
    // 下载文件
    async download(arg) {
        // 获取文件名
        const _fileName = _FileName__WEBPACK_IMPORTED_MODULE_3__.fileName.createFileName(arg.result);
        // 重设当前下载栏的信息
        this.setProgressBar(_fileName, 0, 0);
        // 下载文件
        let url;
        if (arg.result.type === 3) {
            // 生成小说的文件
            if (arg.result.novelMeta) {
                if (arg.result.novelMeta?.coverUrl) {
                    _DownloadNovelCover__WEBPACK_IMPORTED_MODULE_16__.downloadNovelCover.download(arg.result.novelMeta.coverUrl, _fileName, 'downloadNovel');
                }
                let blob = await _MakeNovelFile__WEBPACK_IMPORTED_MODULE_9__.MakeNovelFile.make(arg.result.novelMeta);
                url = URL.createObjectURL(blob);
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.novelSaveAs === 'txt') {
                    await _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_15__.downloadNovelEmbeddedImage.TXT(arg.result.novelMeta.content, arg.result.novelMeta.embeddedImages, _fileName);
                }
            }
            else {
                throw new Error('Not found novelMeta');
            }
        }
        else {
            // 对于图像作品，如果设置了图片尺寸就使用指定的 url，否则使用原图 url
            url = arg.result[_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.imageSize] || arg.result.original;
        }
        let xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'blob';
        // 显示下载进度
        xhr.addEventListener('progress', async (event) => {
            // 检查体积设置
            if (!this.sizeChecked) {
                this.sizeChecked = true;
                const result = await _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check({ size: event.total });
                if (!result) {
                    // 当因为体积问题跳过下载时，可能这个下载进度还是 0 或者很少，所以这里直接把进度条拉满
                    this.setProgressBar(_fileName, 1, 1);
                    this.skipDownload({
                        id: arg.id,
                        reason: 'size',
                    }, _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_不保存图片因为体积', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.id)));
                }
            }
            if (this.cancel) {
                xhr.abort();
                xhr = null;
                return;
            }
            this.setProgressBar(_fileName, event.loaded, event.total);
        });
        // 文件加载完毕，或者加载出错
        xhr.addEventListener('loadend', async () => {
            if (this.cancel) {
                xhr = null;
                return;
            }
            let file = xhr.response; // 要下载的文件
            // 状态码错误，进入重试流程
            if (xhr.status !== 200) {
                // 正常下载完毕的状态码是 200
                // 储存重试的时间戳等信息
                this.retryInterval.push(new Date().getTime() - this.lastRequestTime);
                _ProgressBar__WEBPACK_IMPORTED_MODULE_5__.progressBar.errorColor(this.progressBarIndex, true);
                this.retry++;
                if (this.retry >= _Config__WEBPACK_IMPORTED_MODULE_11__.Config.retryMax) {
                    // 重试达到最大次数
                    this.afterReTryMax(xhr.status, arg.id);
                }
                else {
                    // 开始重试
                    return this.download(arg);
                }
            }
            else {
                // 状态码正常
                _ProgressBar__WEBPACK_IMPORTED_MODULE_5__.progressBar.errorColor(this.progressBarIndex, false);
                // 需要转换动图的情况
                const convertExt = ['webm', 'gif', 'png'];
                const ext = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.ugoiraSaveAs;
                if (convertExt.includes(ext) &&
                    arg.result.ugoiraInfo &&
                    _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.imageSize !== 'thumb') {
                    // 当下载图片的方形缩略图时，不转换动图，因为此时下载的是作品的静态缩略图，无法进行转换
                    try {
                        if (ext === 'webm') {
                            file = await _ConvertUgoira_ConvertUgoira__WEBPACK_IMPORTED_MODULE_4__.convertUgoira.webm(file, arg.result.ugoiraInfo, arg.result.idNum);
                        }
                        if (ext === 'gif') {
                            file = await _ConvertUgoira_ConvertUgoira__WEBPACK_IMPORTED_MODULE_4__.convertUgoira.gif(file, arg.result.ugoiraInfo, arg.result.idNum);
                        }
                        if (ext === 'png') {
                            file = await _ConvertUgoira_ConvertUgoira__WEBPACK_IMPORTED_MODULE_4__.convertUgoira.apng(file, arg.result.ugoiraInfo, arg.result.idNum);
                        }
                    }
                    catch (error) {
                        const msg = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_动图转换失败的提示', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.result.idNum));
                        // 因为会重试所以不在日志上显示
                        // log.error(msg, 1)
                        console.error(msg);
                        this.error = true;
                        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadError', arg.id);
                    }
                }
            }
            if (this.cancel) {
                return;
            }
            // 生成下载链接
            const blobUrl = URL.createObjectURL(file);
            // 对插画、漫画进行颜色检查
            // 在这里进行检查的主要原因：抓取时只会检查单图作品的颜色，不会检查多图作品的颜色。所以多图作品需要在这里进行检查。
            // 另一个原因：如果抓取时没有设置图片的颜色条件，下载时才设置颜色条件，那么就必须在这里进行检查。
            if (arg.result.type === 0 || arg.result.type === 1) {
                const result = await _filter_Filter__WEBPACK_IMPORTED_MODULE_6__.filter.check({
                    mini: blobUrl,
                });
                if (!result) {
                    return this.skipDownload({
                        id: arg.id,
                        reason: 'color',
                    }, _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_不保存图片因为颜色', _Tools__WEBPACK_IMPORTED_MODULE_14__.Tools.createWorkLink(arg.id)));
                }
            }
            // 向浏览器发送下载任务
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings.setFileDownloadOrder) {
                await this.waitPreviousFileDownload();
            }
            this.browserDownload(blobUrl, _fileName, arg.id, arg.taskBatch);
            xhr = null;
            file = null;
        });
        this.lastRequestTime = new Date().getTime();
        // 没有设置 timeout，默认值是 0，不会超时
        xhr.send();
    }
    // 等待上一个文件下载成功之后（浏览器将文件保存到硬盘上），再保存这个文件。这是为了保证文件的保存顺序不会错乱
    waitPreviousFileDownload() {
        return new Promise(async (resolve) => {
            if (this.downloadStatesIndex === 0) {
                return resolve(true);
            }
            if (_DownloadStates__WEBPACK_IMPORTED_MODULE_18__.downloadStates.states[this.downloadStatesIndex - 1] === 1) {
                return resolve(true);
            }
            else {
                return resolve(new Promise((resolve) => {
                    _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_17__.setTimeoutWorker.set(() => {
                        resolve(this.waitPreviousFileDownload());
                    }, 50);
                }));
            }
        });
    }
    // 向浏览器发送下载任务
    browserDownload(blobUrl, fileName, id, taskBatch) {
        // 如果任务已停止，不会向浏览器发送下载任务
        if (this.cancel) {
            // 释放 bloburl
            URL.revokeObjectURL(blobUrl);
            return;
        }
        const sendData = {
            msg: 'save_work_file',
            fileUrl: blobUrl,
            fileName: fileName,
            id,
            taskBatch,
        };
        try {
            chrome.runtime.sendMessage(sendData);
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('sendBrowserDownload');
        }
        catch (error) {
            let msg = `${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_发生错误原因')}<br>{}${_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_请刷新页面')}`;
            if (error.message.includes('Extension context invalidated')) {
                msg = msg.replace('{}', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_扩展程序已更新'));
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(msg);
                _MsgBox__WEBPACK_IMPORTED_MODULE_12__.msgBox.error(msg);
                return;
            }
            console.error(error);
            msg = msg.replace('{}', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_未知错误'));
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.error(msg);
            _MsgBox__WEBPACK_IMPORTED_MODULE_12__.msgBox.error(msg);
        }
    }
}



/***/ }),

/***/ "./src/ts/download/DownloadControl.ts":
/*!********************************************!*\
  !*** ./src/ts/download/DownloadControl.ts ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _download_Download__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../download/Download */ "./src/ts/download/Download.ts");
/* harmony import */ var _ProgressBar__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./ProgressBar */ "./src/ts/download/ProgressBar.ts");
/* harmony import */ var _DownloadStates__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./DownloadStates */ "./src/ts/download/DownloadStates.ts");
/* harmony import */ var _ShowDownloadStates__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./ShowDownloadStates */ "./src/ts/download/ShowDownloadStates.ts");
/* harmony import */ var _ShowSkipCount__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./ShowSkipCount */ "./src/ts/download/ShowSkipCount.ts");
/* harmony import */ var _ShowConvertCount__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ./ShowConvertCount */ "./src/ts/download/ShowConvertCount.ts");
/* harmony import */ var _BookmarkAfterDL__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./BookmarkAfterDL */ "./src/ts/download/BookmarkAfterDL.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _CheckWarningMessage__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! ./CheckWarningMessage */ "./src/ts/download/CheckWarningMessage.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");
// 下载控制






















class DownloadControl {
    constructor() {
        this.wrapper = document.createElement('div');
        /**在插槽里添加的操作抓取结果的按钮 */
        this.resultBtns = {
            exportCSV: document.createElement('button'),
            exportJSON: document.createElement('button'),
            importJSON: document.createElement('button'),
        };
        this.thread = 5; // 同时下载的线程数的默认值
        // 这里默认设置为 5，是因为国内一些用户的下载速度比较慢，所以不应该同时下载很多文件。
        // 最大值由 Config.downloadThreadMax 定义
        this.taskBatch = 0; // 标记任务批次，每次重新下载时改变它的值，传递给后台使其知道这是一次新的下载
        this.taskList = {}; // 下载任务列表，使用下载的文件的 id 做 key，保存下载栏编号和它在下载状态列表中的索引
        this.errorIdList = []; // 有任务下载失败时，保存 id
        this.downloaded = 0; // 已下载的任务数量
        this.stop = false; // 是否已经停止下载
        this.pause = false; // 是否已经暂停下载
        this.crawlIdListTimer = undefined;
        this.checkDownloadTimeoutTimer = undefined;
        this.msgFlag = 'uuidTip';
        this.createResultBtns();
        this.createDownloadArea();
        this.bindEvents();
        const statusTipWrap = this.wrapper.querySelector('.down_status');
        new _ShowDownloadStates__WEBPACK_IMPORTED_MODULE_10__.ShowDownloadStates(statusTipWrap);
        const skipTipWrap = this.wrapper.querySelector('.skip_tip');
        new _ShowSkipCount__WEBPACK_IMPORTED_MODULE_11__.ShowSkipCount(skipTipWrap);
        const convertTipWrap = this.wrapper.querySelector('.convert_tip');
        new _ShowConvertCount__WEBPACK_IMPORTED_MODULE_12__.ShowConvertCount(convertTipWrap);
        // 只在 p 站内启用下载后收藏的功能
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_17__.Utils.isPixiv()) {
            const bmkAfterDLTipWrap = this.wrapper.querySelector('.bmkAfterDL_tip');
            new _BookmarkAfterDL__WEBPACK_IMPORTED_MODULE_13__.BookmarkAfterDL(bmkAfterDLTipWrap);
        }
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.hideResultBtns();
            this.hideDownloadArea();
            this.reset();
        });
        for (const ev of [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resultChange,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resume,
        ]) {
            window.addEventListener(ev, (ev) => {
                // 当恢复了未完成的抓取数据时，将下载状态设置为暂停
                this.pause = ev.type === 'resume';
                // 让开始下载的方法进入任务队列，以便让监听上述事件的其他部分的代码先执行完毕
                window.setTimeout(() => {
                    this.readyDownload();
                }, 0);
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.skipDownload, (ev) => {
            // 跳过下载的文件不会触发 downloadSuccess 事件
            const data = ev.detail.data;
            this.downloadOrSkipAFile(data);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadError, (ev) => {
            const id = ev.detail.data;
            this.downloadError(id);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.requestPauseDownload, (ev) => {
            // 请求暂停下载
            this.pauseDownload();
        });
        // 如果下载器让浏览器保存文件到本地，但是之后没有收到回应（不知道文件是否有成功保存），这会导致下载进度卡住
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.sendBrowserDownload, () => {
            window.clearTimeout(this.checkDownloadTimeoutTimer);
            this.checkDownloadTimeoutTimer = window.setTimeout(() => {
                const msg = _Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_可能发生了错误请刷新页面重试');
                _MsgBox__WEBPACK_IMPORTED_MODULE_19__.msgBox.once('mayError', msg, 'warning');
                _Log__WEBPACK_IMPORTED_MODULE_3__.log.warning(msg);
            }, 5000);
        });
        const clearDownloadTimeoutTimerList = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadError,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadSuccess,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart,
        ];
        clearDownloadTimeoutTimerList.forEach((evt) => {
            window.addEventListener(evt, () => {
                window.clearTimeout(this.checkDownloadTimeoutTimer);
            });
        });
        // 监听浏览器返回的消息
        chrome.runtime.onMessage.addListener((msg) => {
            if (!this.taskBatch) {
                return;
            }
            // UUID 的情况
            if (msg.data?.uuid) {
                _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_uuid'));
                _MsgBox__WEBPACK_IMPORTED_MODULE_19__.msgBox.once(this.msgFlag, _Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_uuid'), 'error');
            }
            // 文件下载成功
            if (msg.msg === 'downloaded') {
                // 释放 BLOBURL
                URL.revokeObjectURL(msg.data.url);
                // 发送下载成功的事件
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadSuccess', msg.data);
                this.downloadOrSkipAFile(msg.data);
            }
            else if (msg.msg === 'download_err') {
                // 浏览器把文件保存到本地失败
                // 用户操作导致下载取消的情况，跳过这个文件，不再重试保存它。触发条件如：
                // 用户在浏览器弹出“另存为”对话框时取消保存
                // 用户让 IDM 转接这个下载时
                if (msg.err === 'USER_CANCELED') {
                    _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_user_canceled_tip', _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.createWorkLink(msg.data.id), msg.err || 'unknown'));
                    this.downloadOrSkipAFile(msg.data);
                    return;
                }
                // 其他原因，下载器会重试保存这个文件
                _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_save_file_failed_tip', _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.createWorkLink(msg.data.id), msg.err || 'unknown'));
                if (msg.err === 'FILE_FAILED') {
                    _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_FILE_FAILED_tip'));
                }
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('saveFileError');
                // 重新下载这个文件
                // 但并不确定能否如预期一样重新下载这个文件
                this.saveFileError(msg.data);
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete, () => {
            // 如果有等待中的下载任务，则开始下载等待中的任务
            if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.waitingIdList.length === 0) {
                _Toast__WEBPACK_IMPORTED_MODULE_16__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_下载完毕2'), {
                    position: 'center',
                });
                // 通知后台清除保存的此标签页的 idList
                chrome.runtime.sendMessage({
                    msg: 'clearDownloadsTempData',
                });
            }
            else {
                window.clearTimeout(this.crawlIdListTimer);
                this.crawlIdListTimer = window.setTimeout(() => {
                    _store_States__WEBPACK_IMPORTED_MODULE_14__.states.quickCrawl = true; // 下载等待的任务时，不显示下载器面板
                    const idList = _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.waitingIdList;
                    _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.waitingIdList = [];
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', idList);
                }, 0);
            }
        });
    }
    createDownloadArea() {
        const html = `<div class="download_area">
    <div class="centerWrap_btns">
    <button class="startDownload" type="button" style="background:${_Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgBlue};" data-xztext="_开始下载"></button>
    <button class="pauseDownload" type="button" style="background:${_Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgYellow};" data-xztext="_暂停下载"></button>
    <button class="stopDownload" type="button" style="background:${_Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgRed};" data-xztext="_停止下载"></button>
    <button class="copyUrl" type="button" style="background:${_Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgGreen};" data-xztext="_复制url"></button>
    </div>
    <div class="download_status_text_wrap">
    <span data-xztext="_当前状态"></span>
    <span class="down_status" data-xztext="_未开始下载"></span>
    <span class="skip_tip warn"></span>
    <span class="convert_tip warn"></span>
    <span class="bmkAfterDL_tip green"></span>
    </div>
    </div>`;
        this.wrapper = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.useSlot('downloadArea', html);
        _Lang__WEBPACK_IMPORTED_MODULE_4__.lang.register(this.wrapper);
        this.wrapper
            .querySelector('.startDownload')
            .addEventListener('click', () => {
            this.startDownload();
        });
        this.wrapper
            .querySelector('.pauseDownload')
            .addEventListener('click', () => {
            this.pauseDownload();
        });
        this.wrapper
            .querySelector('.stopDownload')
            .addEventListener('click', () => {
            this.stopDownload();
        });
        this.wrapper.querySelector('.copyUrl').addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('showURLs');
        });
    }
    createResultBtns() {
        // 只在 pixiv 上添加这些按钮
        if (_utils_Utils__WEBPACK_IMPORTED_MODULE_17__.Utils.isPixiv()) {
            // 导入抓取结果
            this.resultBtns.importJSON = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.addBtn('exportResult', _Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgGreen, '_导入抓取结果');
            // 导入抓取结果的按钮始终显示，因为它需要始终可用。
            // 导出抓取结果的按钮只有在可以准备下载时才显示
            this.resultBtns.importJSON.addEventListener('click', () => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('importResult');
            }, false);
            // 导出抓取结果
            this.resultBtns.exportJSON = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.addBtn('exportResult', _Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgGreen, '_导出抓取结果');
            this.resultBtns.exportJSON.style.display = 'none';
            this.resultBtns.exportJSON.addEventListener('click', () => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('exportResult');
            }, false);
            // 导出 csv
            this.resultBtns.exportCSV = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.addBtn('exportResult', _Colors__WEBPACK_IMPORTED_MODULE_5__.Colors.bgGreen, '_导出csv');
            this.resultBtns.exportCSV.style.display = 'none';
            this.resultBtns.exportCSV.addEventListener('click', () => {
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('exportCSV');
            }, false);
            this.resultBtns.exportCSV.addEventListener('mouseenter', () => {
                _ShowHelp__WEBPACK_IMPORTED_MODULE_21__.showHelp.show('tipCSV', _Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_导出CSV文件的提示'));
            }, false);
        }
    }
    // 抓取完毕之后，已经可以开始下载时，显示必要的信息，并决定是否立即开始下载
    readyDownload() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.busy || _store_States__WEBPACK_IMPORTED_MODULE_14__.states.mergeNovel) {
            return;
        }
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length === 0) {
            return _ProgressBar__WEBPACK_IMPORTED_MODULE_8__.progressBar.reset(0);
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.downloadUgoiraFirst) {
            _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.resultMeta.sort(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.sortUgoiraFirst);
            _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.sort(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.sortUgoiraFirst);
        }
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('readyDownload');
        this.showResultBtns();
        this.showDownloadArea();
        this.setDownloaded();
        this.setDownloadThread();
        // 在插画漫画搜索页面里，如果启用了“预览搜索页面的筛选结果”
        if (_PageType__WEBPACK_IMPORTED_MODULE_18__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_18__.pageType.list.ArtworkSearch &&
            _setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.previewResult) {
            // “预览搜索页面的筛选结果”会阻止自动开始下载。但是一些情况例外
            // 允许快速抓取发起的下载请求自动开始下载
            // 允许由抓取标签列表功能发起的下载请求自动开始下载
            if (!_store_States__WEBPACK_IMPORTED_MODULE_14__.states.quickCrawl && !_store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList) {
                return;
            }
        }
        // 自动开始下载的情况
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.autoStartDownload ||
            _store_States__WEBPACK_IMPORTED_MODULE_14__.states.quickCrawl ||
            _store_States__WEBPACK_IMPORTED_MODULE_14__.states.crawlTagList) {
            this.startDownload();
        }
    }
    // 开始下载
    startDownload() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.busy) {
            return _Toast__WEBPACK_IMPORTED_MODULE_16__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_当前任务尚未完成'));
        }
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length === 0) {
            return _Toast__WEBPACK_IMPORTED_MODULE_16__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_没有可用的抓取结果'));
        }
        if (this.pause) {
            // 从上次中断的位置继续下载
            // 把“使用中”的下载状态重置为“未使用”
            _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.resume();
        }
        else {
            // 如果之前没有暂停任务，也没有进入恢复模式，则重新下载
            // 初始化下载状态列表
            _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.init();
        }
        this.reset();
        _MsgBox__WEBPACK_IMPORTED_MODULE_19__.msgBox.resetOnce(this.msgFlag);
        this.setDownloaded();
        this.taskBatch = new Date().getTime(); // 修改本批下载任务的标记
        this.setDownloadThread();
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadStart');
        // 建立并发下载线程
        for (let i = 0; i < this.thread; i++) {
            this.createDownload(i);
        }
        _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_正在下载中'));
        if (_Config__WEBPACK_IMPORTED_MODULE_15__.Config.mobile) {
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_Kiwi浏览器可能不能建立文件夹的bug'));
        }
    }
    // 暂停下载
    pauseDownload() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length === 0) {
            return;
        }
        // 停止的优先级高于暂停。点击停止可以取消暂停状态，但点击暂停不能取消停止状态
        if (this.stop === true) {
            return;
        }
        if (this.pause === false) {
            // 如果正在下载中
            if (_store_States__WEBPACK_IMPORTED_MODULE_14__.states.busy) {
                this.pause = true;
                _Log__WEBPACK_IMPORTED_MODULE_3__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_已暂停'), 2);
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadPause');
            }
            else {
                // 不在下载中的话不允许启用暂停功能
                return;
            }
        }
    }
    // 停止下载
    stopDownload() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length === 0 || this.stop) {
            return;
        }
        this.stop = true;
        _Log__WEBPACK_IMPORTED_MODULE_3__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_已停止'), 2);
        this.pause = false;
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadStop');
    }
    downloadError(id) {
        this.errorIdList.push(id);
        // 是否继续下载
        const task = this.taskList[id];
        const no = task.progressBarIndex;
        if (this.checkContinueDownload()) {
            this.createDownload(no);
        }
        else {
            this.checkCompleteWithError();
        }
    }
    setDownloaded() {
        this.downloaded = _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.downloadedCount();
        const text = `${this.downloaded} / ${_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length}`;
        _Log__WEBPACK_IMPORTED_MODULE_3__.log.log(text, 2, false);
        // 设置总下载进度条
        _ProgressBar__WEBPACK_IMPORTED_MODULE_8__.progressBar.setTotalProgress(this.downloaded);
        _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.remainingDownload = _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length - this.downloaded;
        // 所有文件正常下载完毕（跳过下载的文件也算正常下载）
        if (this.downloaded === _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length) {
            _Log__WEBPACK_IMPORTED_MODULE_3__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_下载完毕'), 2);
            window.setTimeout(() => {
                // 延后触发下载完成的事件。因为下载完成事件是由上游事件（跳过下载，或下载成功事件）派生的，如果这里不延迟触发，可能导致其他模块先接收到下载完成事件，后接收到上游事件。
                _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('downloadComplete');
            }, 0);
            this.reset();
        }
        this.checkCompleteWithError();
    }
    // 设置下载线程数量
    setDownloadThread() {
        const setThread = _setting_Settings__WEBPACK_IMPORTED_MODULE_6__.settings.downloadThread;
        if (setThread < 1 ||
            setThread > _Config__WEBPACK_IMPORTED_MODULE_15__.Config.downloadThreadMax ||
            isNaN(setThread)) {
            // 如果数值非法，则重设为默认值
            this.thread = _Config__WEBPACK_IMPORTED_MODULE_15__.Config.downloadThreadMax;
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_6__.setSetting)('downloadThread', _Config__WEBPACK_IMPORTED_MODULE_15__.Config.downloadThreadMax);
        }
        else {
            this.thread = setThread; // 设置为用户输入的值
        }
        // 如果剩余任务数量少于下载线程数
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length - this.downloaded < this.thread) {
            this.thread = _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length - this.downloaded;
        }
        // 重设下载进度条
        _ProgressBar__WEBPACK_IMPORTED_MODULE_8__.progressBar.reset(this.thread, this.downloaded);
    }
    saveFileError(data) {
        if (this.pause || this.stop) {
            return false;
        }
        const task = this.taskList[data.id];
        // 复位这个任务的状态
        _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.setState(task.index, -1);
        // 建立下载任务，再次下载它
        this.createDownload(task.progressBarIndex);
    }
    downloadOrSkipAFile(data) {
        const task = this.taskList[data.id];
        // 更改这个任务状态为“已完成”
        _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.setState(task.index, 1);
        // 统计已下载数量
        this.setDownloaded();
        // 是否继续下载
        const no = task.progressBarIndex;
        if (this.checkContinueDownload()) {
            this.createDownload(no);
        }
    }
    // 当一个文件下载成功或失败之后，检查是否还有后续下载任务
    checkContinueDownload() {
        // 如果没有全部下载完毕
        if (this.downloaded < _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length) {
            // 如果任务已停止
            if (this.pause || this.stop) {
                return false;
            }
            // 如果已完成的数量 加上 线程中未完成的数量，仍然没有达到文件总数，继续添加任务
            if (this.downloaded + this.thread - 1 < _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length) {
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }
    // 查找需要进行下载的作品，建立下载
    createDownload(progressBarIndex) {
        const index = _DownloadStates__WEBPACK_IMPORTED_MODULE_9__.downloadStates.getFirstDownloadItem();
        if (index === undefined) {
            // 当已经没有需要下载的作品时，检查是否带着错误完成了下载
            // 如果下载过程中没有出错，就不会执行到这个分支
            return this.checkCompleteWithError();
        }
        else {
            const workData = _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result[index];
            const argument = {
                id: workData.id,
                result: workData,
                index: index,
                progressBarIndex: progressBarIndex,
                taskBatch: this.taskBatch,
            };
            // 保存任务信息
            this.taskList[workData.id] = {
                index,
                progressBarIndex: progressBarIndex,
            };
            // 建立下载
            new _download_Download__WEBPACK_IMPORTED_MODULE_7__.Download(progressBarIndex, argument, index);
        }
    }
    // 在有下载出错的情况下，是否已经完成了下载
    checkCompleteWithError() {
        if (this.errorIdList.length > 0 &&
            this.downloaded + this.errorIdList.length === _store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length) {
            // 进入暂停状态，一定时间后自动开始下载，重试下载出错的文件
            this.pauseDownload();
            setTimeout(() => {
                this.startDownload();
            }, 2000);
        }
    }
    reset() {
        this.pause = false;
        this.stop = false;
        this.errorIdList = [];
        this.downloaded = 0;
    }
    showDownloadArea() {
        this.wrapper.style.display = 'block';
    }
    hideDownloadArea() {
        this.wrapper.style.display = 'none';
    }
    showResultBtns() {
        this.resultBtns.exportJSON.style.display = 'flex';
        this.resultBtns.exportCSV.style.display = 'flex';
    }
    hideResultBtns() {
        this.resultBtns.exportJSON.style.display = 'none';
        this.resultBtns.exportCSV.style.display = 'none';
    }
}
new DownloadControl();


/***/ }),

/***/ "./src/ts/download/DownloadNovelCover.ts":
/*!***********************************************!*\
  !*** ./src/ts/download/DownloadNovelCover.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   downloadNovelCover: () => (/* binding */ downloadNovelCover)
/* harmony export */ });
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");


class DownloadNovelCover {
    /**下载小说的封面图片
     *
     * 默认是正常下载小说的情况，可以设置为合并系列小说的情况
     */
    async download(coverURL, novelName, action = 'downloadNovel') {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.downloadNovelCoverImage || !coverURL) {
            return;
        }
        const url = await this.getCoverBolbURL(coverURL);
        let coverName = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.replaceSuffix(novelName, coverURL);
        // 合并系列小说时，文件直接保存在下载目录里，封面图片也保存在下载目录里
        // 所以要替换掉封面图路径里的斜线
        if (action === 'mergeNovel') {
            coverName = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.replaceUnsafeStr(coverName);
        }
        this.sendDownload(url, coverName);
    }
    // 生成封面图片的 Blob URL
    async getCoverBolbURL(coverURL) {
        return new Promise(async (resolve, reject) => {
            const res = await fetch(coverURL, {
                method: 'get',
                credentials: 'same-origin',
            });
            const blob = await res.blob();
            const url = URL.createObjectURL(blob);
            return resolve(url);
        });
    }
    sendDownload(url, name) {
        chrome.runtime.sendMessage({
            msg: 'save_novel_cover_file',
            fileUrl: url,
            fileName: name,
        });
    }
}
const downloadNovelCover = new DownloadNovelCover();



/***/ }),

/***/ "./src/ts/download/DownloadNovelEmbeddedImage.ts":
/*!*******************************************************!*\
  !*** ./src/ts/download/DownloadNovelEmbeddedImage.ts ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   downloadNovelEmbeddedImage: () => (/* binding */ downloadNovelEmbeddedImage)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");



/**下载小说里的内嵌图片 */
class DownloadNovelEmbeddedImage {
    // 小说保存为 txt 时，直接下载小说里的内嵌图片。因为 txt 无法存储图像，只能单独保存
    /**下载小说为 txt 时
     *
     * 默认是正常下载小说的情况，可以设置为合并系列小说的情况
     */
    async TXT(content, embeddedImages, novelName, action = 'downloadNovel') {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.downloadNovelEmbeddedImage) {
            return;
        }
        const idList = await this.getIdList(content, embeddedImages);
        // 保存为 TXT 格式时，每加载完一个图片，就立即保存这个图片
        for (let idData of idList) {
            // 如果 url 是 null，则不会保存这个图片
            if (idData.url) {
                idData = await this.getImageBolbURL(idData);
                let imageName = _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceSuffix(novelName, idData.url);
                // 在文件名末尾加上内嵌图片的 id 和序号
                const array = imageName.split('.');
                const addString = `-${idData.id}${idData.p === 0 ? '' : '-' + idData.p}`;
                array[array.length - 2] = array[array.length - 2] + addString;
                imageName = array.join('.');
                // 合并系列小说时，文件直接保存在下载目录里，内嵌图片也保存在下载目录里
                // 所以要替换掉内嵌图片路径里的斜线
                if (action === 'mergeNovel') {
                    imageName = _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(imageName);
                }
                this.sendDownload(idData.blobURL, imageName);
            }
        }
    }
    /**下载小说为 EPUB 时，替换内嵌图片标记，把图片用 img 标签保存到正文里 */
    async EPUB(content, embeddedImages) {
        return new Promise(async (resolve) => {
            if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.downloadNovelEmbeddedImage) {
                return resolve(content);
            }
            const idList = await this.getIdList(content, embeddedImages);
            for (let idData of idList) {
                if (idData.url) {
                    idData = await this.getImageBolbURL(idData);
                    const dataURL = await this.getImageDataURL(idData);
                    const html = `<img src="${dataURL}" />`;
                    content = content.replaceAll(idData.flag, html);
                }
                else {
                    // 如果 url 是 null，则修改标记，做出提示
                    content = content.replaceAll(idData.flag, ` ${idData.flag} url is null`);
                }
            }
            return resolve(content);
        });
    }
    // 获取正文里上传的图片 id 和引用的图片 id
    async getIdList(content, embeddedImages) {
        return new Promise(async (resolve) => {
            const idList = [];
            // 获取上传的图片数据
            if (embeddedImages) {
                for (const [id, url] of Object.entries(embeddedImages)) {
                    idList.push({
                        id,
                        p: 0,
                        type: 'upload',
                        url,
                        flag: `[uploadedimage:${id}]`,
                    });
                }
            }
            // 获取引用的图片数据
            const reg = /\[pixivimage:(.+?)\]/g;
            let test;
            while ((test = reg.exec(content))) {
                if (test && test.length === 2) {
                    // 99381250
                    // 一个图像作品可能有多个被引用的图片，如
                    // 99760571-1
                    // 99760571-130
                    const idInfo = test[1].split('-');
                    idList.push({
                        id: idInfo[0],
                        p: idInfo[1] ? parseInt(idInfo[1]) : 0,
                        type: 'pixiv',
                        url: '',
                        flag: `[pixivimage:${test[1]}]`,
                    });
                }
            }
            // 引用的图片此时没有 URL
            // 统计引用的图像作品的 id （不重复），然后获取每个 id 的数据
            const artworkIDs = new Set();
            idList.forEach((data) => {
                if (data.type === 'pixiv') {
                    artworkIDs.add(data.id);
                }
            });
            for (const id of Array.from(artworkIDs)) {
                try {
                    // 尝试获取原图作品数据，提取 URL
                    const workData = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(id);
                    const p0URL = workData.body.urls.original;
                    for (const idData of idList) {
                        if (idData.id === id) {
                            // 如果 p 为 0 则表示未指定图片序号，也就是第一张图片
                            if (idData.p === 0) {
                                idData.url = p0URL;
                            }
                            else {
                                // 如果指定了图片序号，则从第一张图片的 URL 生成指定图片的 URL
                                idData.url = p0URL.replace('p0.', `p${idData.p - 1}.`);
                            }
                        }
                    }
                }
                catch (error) {
                    // 原图作品可能被删除了，404
                    console.log(error);
                    continue;
                }
            }
            // 返回数据时，删除没有 url 的数据
            const result = idList.filter((data) => data.url !== '');
            return resolve(result);
        });
    }
    async getImageBolbURL(idData) {
        return new Promise(async (resolve) => {
            if (idData.url) {
                const res = await fetch(idData.url);
                const blob = await res.blob();
                idData.blobURL = URL.createObjectURL(blob);
            }
            resolve(idData);
        });
    }
    async getImageDataURL(data) {
        return new Promise(async (resolve) => {
            const img = await _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.loadImg(data.blobURL);
            const canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            const con = canvas.getContext('2d');
            con.drawImage(img, 0, 0, img.width, img.height);
            const suffix = _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.getSuffix(data.url);
            // 如果原图是 png 格式，就转换成 png 格式的数据，否则转换为 jpeg 格式
            if (suffix === 'png') {
                const ImgDataURL = canvas.toDataURL();
                return resolve(ImgDataURL);
            }
            else {
                const ImgDataURL = canvas.toDataURL('image/jpeg', 0.95);
                return resolve(ImgDataURL);
            }
        });
    }
    sendDownload(url, name) {
        chrome.runtime.sendMessage({
            msg: 'save_novel_embedded_image',
            fileUrl: url,
            fileName: name,
        });
    }
}
const downloadNovelEmbeddedImage = new DownloadNovelEmbeddedImage();



/***/ }),

/***/ "./src/ts/download/DownloadOnClickBookmark.ts":
/*!****************************************************!*\
  !*** ./src/ts/download/DownloadOnClickBookmark.ts ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   downloadOnClickBookmark: () => (/* binding */ downloadOnClickBookmark)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _WorkToolBar__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../WorkToolBar */ "./src/ts/WorkToolBar.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../ArtworkThumbnail */ "./src/ts/ArtworkThumbnail.ts");
/* harmony import */ var _NovelThumbnail__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../NovelThumbnail */ "./src/ts/NovelThumbnail.ts");











// 点击作品的收藏按钮时，下载这个作品
class DownloadOnClickBookmark {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        // 在作品缩略图上点击收藏按钮时，下载这个作品
        _ArtworkThumbnail__WEBPACK_IMPORTED_MODULE_9__.artworkThumbnail.onClickBookmarkBtn((el, id) => {
            if (!id) {
                id = _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.findWorkIdFromElement(el, 'illusts');
            }
            this.send(id);
        });
        _NovelThumbnail__WEBPACK_IMPORTED_MODULE_10__.novelThumbnail.onClickBookmarkBtn((el, id) => {
            if (!id || id === '0') {
                id = _Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.findWorkIdFromElement(el, 'novels');
                console.log(id);
            }
            this.send(id, 'novels');
        });
        // 在作品页面里点击收藏按钮时，下载这个作品
        _WorkToolBar__WEBPACK_IMPORTED_MODULE_6__.workToolBar.register((toolbar, pixivBMKDiv, likeBtn) => {
            pixivBMKDiv.addEventListener('click', () => {
                if (_PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.list.Artwork) {
                    this.send(_Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getIllustId(window.location.href));
                }
                if (_PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_7__.pageType.list.Novel) {
                    this.send(_Tools__WEBPACK_IMPORTED_MODULE_8__.Tools.getNovelId(window.location.href), 'novels');
                }
            });
        });
    }
    /**发送作品的 id 和类型，抓取并下载这个作品
     *
     * @type 默认值是 'illusts'
     */
    send(id, type = 'illusts') {
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.downloadOnClickBookmark) {
            _store_States__WEBPACK_IMPORTED_MODULE_2__.states.quickCrawl = true;
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlIdList', [
                {
                    id,
                    type,
                },
            ]);
            _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_5__.lang.transl('_已发送下载请求'), {
                bgColor: _Colors__WEBPACK_IMPORTED_MODULE_4__.Colors.bgBlue,
            });
        }
    }
}
const downloadOnClickBookmark = new DownloadOnClickBookmark();



/***/ }),

/***/ "./src/ts/download/DownloadOnClickLike.ts":
/*!************************************************!*\
  !*** ./src/ts/download/DownloadOnClickLike.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _WorkToolBar__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../WorkToolBar */ "./src/ts/WorkToolBar.ts");









// 在作品页面里点赞时，下载这个作品
class DownloadOnClickLike {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        _WorkToolBar__WEBPACK_IMPORTED_MODULE_8__.workToolBar.register((toolbar, pixivBMKDiv, likeBtn) => {
            likeBtn.addEventListener('click', () => {
                if (_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.list.Artwork) {
                    this.send(_Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getIllustId(window.location.href));
                }
                if (_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.list.Novel) {
                    this.send(_Tools__WEBPACK_IMPORTED_MODULE_7__.Tools.getNovelId(window.location.href), 'novels');
                }
            });
        });
    }
    /**发送作品的 id 和类型，抓取并下载这个作品
     *
     * @type 默认值是 'illusts'
     */
    send(id, type = 'illusts') {
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downloadOnClickLike) {
            _store_States__WEBPACK_IMPORTED_MODULE_5__.states.quickCrawl = true;
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('crawlIdList', [
                {
                    id,
                    type,
                },
            ]);
            _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已发送下载请求'), {
                bgColor: _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgBlue,
            });
        }
    }
}
new DownloadOnClickLike();


/***/ }),

/***/ "./src/ts/download/DownloadRecord.ts":
/*!*******************************************!*\
  !*** ./src/ts/download/DownloadRecord.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   downloadRecord: () => (/* binding */ downloadRecord)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/IndexedDB */ "./src/ts/utils/IndexedDB.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/SecretSignal */ "./src/ts/utils/SecretSignal.ts");











// 保存下载记录，用来判断重复下载的文件
class DownloadRecord {
    constructor() {
        this.DBName = 'DLRecord';
        this.DBVer = 1;
        this.storeNameList = [
            'record1',
            'record2',
            'record3',
            'record4',
            'record5',
            'record6',
            'record7',
            'record8',
            'record9',
        ]; // 表名的列表
        this.existedIdList = []; // 检查文件是否重复时，会查询数据库。查询到的数据的 id 会保存到这个列表里。当向数据库添加记录时，可以先查询这个列表，如果已经有过记录就改为 put 而不是 add，因为添加主键重复的数据会报错
        // 从图片 url 里取出日期字符串的正则表达式
        this.dateRegExp = /img\/(.*)\//;
        this.IDB = new _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_4__.IndexedDB();
        this.init();
    }
    async init() {
        await this.initDB();
        this.bindEvents();
    }
    // 初始化数据库，获取数据库对象
    async initDB() {
        // 在升级事件里创建表和索引
        const onUpdate = (db) => {
            for (const name of this.storeNameList) {
                if (!db.objectStoreNames.contains(name)) {
                    const store = db.createObjectStore(name, { keyPath: 'id' });
                    store.createIndex('id', 'id', { unique: true });
                }
            }
        };
        return new Promise(async (resolve, reject) => {
            resolve(await this.IDB.open(this.DBName, this.DBVer, onUpdate));
        });
    }
    bindEvents() {
        // 当有文件下载完成时，存储这个任务的记录
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadSuccess, (ev) => {
            const successData = ev.detail.data;
            const result = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.findResult(successData.id);
            result && this.addRecord(result);
        });
        // 导入含有 id 列表的 txt 文件
        _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_10__.secretSignal.register('recordtxt', () => {
            this.importRecordFromTxt();
        });
        // 导入下载记录的按钮
        {
            const btn = document.querySelector('#importDownloadRecord');
            if (btn) {
                btn.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('importDownloadRecord');
                });
            }
        }
        // 监听导入下载记录的事件
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.importDownloadRecord, () => {
            this.importRecordFromJSON();
        });
        // 导出下载记录的按钮
        {
            const btn = document.querySelector('#exportDownloadRecord');
            if (btn) {
                btn.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('exportDownloadRecord');
                });
            }
        }
        // 监听导出下载记录的事件
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.exportDownloadRecord, () => {
            this.exportRecord();
        });
        // 清空下载记录的按钮
        {
            const btn = document.querySelector('#clearDownloadRecord');
            if (btn) {
                btn.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('clearDownloadRecord');
                });
            }
        }
        // 监听清空下载记录的事件
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clearDownloadRecord, () => {
            this.clearRecords();
            this.existedIdList = [];
        });
    }
    // 当要查找或存储一个 id 时，返回它所对应的 storeName
    getStoreName(id) {
        const firstNum = parseInt(id[0]);
        return this.storeNameList[firstNum - 1];
    }
    // 生成一个下载记录
    createRecord(data) {
        let result = undefined;
        if (typeof data === 'string') {
            result = _store_Store__WEBPACK_IMPORTED_MODULE_5__.store.findResult(data);
        }
        else {
            result = data;
        }
        if (result === undefined) {
            throw new Error('createRecord failed');
        }
        return {
            id: result.id,
            n: _FileName__WEBPACK_IMPORTED_MODULE_6__.fileName.createFileName(result),
            d: this.getDateString(result),
        };
    }
    /**返回作品的修改日期字符串 */
    getDateString(result) {
        // 图像作品不使用 uploadDate，这是历史遗留原因，因为以前下载器的内部数据里没有 uploadDate 数据
        // 而是从文件 URL 里取出日期字符串。例如
        // 'https://i.pximg.net/img-original/img/2021/10/11/00/00/06/93364702_p0.png'
        // 返回
        // '2021/10/11/00/00/06'
        // 为了保持向后兼容，这里不做修改
        if (result.type !== 3) {
            return result.original.match(this.dateRegExp)[1];
        }
        else {
            // 小说作品使用 uploadDate，返回值如
            // '2021-09-03T14:31:03+00:00'
            return result.uploadDate;
        }
    }
    // 添加一条下载记录
    async addRecord(result) {
        const storeName = this.getStoreName(result.id);
        const record = this.createRecord(result);
        if (this.existedIdList.includes(result.id)) {
            this.IDB.put(storeName, record);
        }
        else {
            // 先查询有没有这个记录
            const result = await this.IDB.get(storeName, record.id);
            this.IDB[result ? 'put' : 'add'](storeName, record);
        }
    }
    /** 检查一个作品是否是重复下载
     *
     * 返回值 true 表示重复，false 表示不重复
     */
    async checkDeduplication(result) {
        if (!_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.isPixiv()) {
            return false;
        }
        return new Promise(async (resolve, reject) => {
            // 如果未启用去重，直接返回不重复
            if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.deduplication) {
                return resolve(false);
            }
            // 在数据库进行查找
            const storeName = this.getStoreName(result.id);
            const data = (await this.IDB.get(storeName, result.id));
            if (data === null) {
                return resolve(false);
            }
            // 有记录，说明这个文件下载过
            this.existedIdList.push(data.id);
            // 首先检查日期字符串是否发生了变化
            // 如果日期字符串变化了，则不视为重复文件
            if (data.d !== undefined && data.d !== this.getDateString(result)) {
                return resolve(false);
            }
            // 如果之前的下载记录里没有日期，说明是早期的下载记录，那么就不检查日期
            // 同时，更新这个作品的下载记录，为其添加日期
            if (data.d === undefined) {
                this.addRecord(result);
            }
            // 如果日期字符串没有变化，再根据策略进行判断
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.dupliStrategy === 'loose') {
                // 如果是宽松策略（不比较文件名）
                return resolve(true);
            }
            else {
                // 如果是严格策略（考虑文件名）
                const name = _FileName__WEBPACK_IMPORTED_MODULE_6__.fileName.createFileName(result);
                return resolve(name === data.n);
            }
        });
    }
    // 清空下载记录
    async clearRecords() {
        if (window.confirm(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('确定要清除下载记录吗')) === false) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_清除下载记录'));
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_清除下载记录'));
        let total = this.storeNameList.length;
        let num = 0;
        for (const name of this.storeNameList) {
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_任务进度')} ${num}/${total}`);
            num++;
            await this.IDB.clear(name);
        }
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_任务进度')} ${num}/${total}`);
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_下载记录已清除'));
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_下载记录已清除'));
    }
    // 导出下载记录
    async exportRecord() {
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导出下载记录'));
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.show(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导出下载记录'));
        let total = this.storeNameList.length;
        let num = 0;
        let record = [];
        for (const name of this.storeNameList) {
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_任务进度')} ${num}/${total}`);
            num++;
            const r = (await this.IDB.getAll(name));
            record = record.concat(r);
        }
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_任务进度')} ${num}/${total}`);
        if (record.length === 0) {
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
            _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
            return;
        }
        const resultList = await _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.json2BlobSafe(record);
        for (const result of resultList) {
            _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.downloadFile(result.url, `record-total ${result.total}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.replaceUnsafeStr(new Date().toLocaleString())}.json`);
        }
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导出成功');
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.success(msg);
    }
    // 导入下载记录
    async importRecord(record) {
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入下载记录'));
        // 显示导入进度
        let stored = 0;
        let total = record.length;
        if (total > 10000) {
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_数据较多需要花费一些时间'));
        }
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${stored}/${total}`, 1, false);
        console.time('importRecord');
        // 依次处理每个存储库
        for (let index = 0; index < this.storeNameList.length; index++) {
            // 提取出要存入这个存储库的数据
            const data = [];
            for (const r of record) {
                if (parseInt(r.id[0]) - 1 === index) {
                    data.push(r);
                }
            }
            if (data.length === 0) {
                continue;
            }
            // 添加数据
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_待处理')} ${data.length}`);
            try {
                // console.time('restoreRecord' + (index + 1))
                await this.IDB.batchAddData(this.storeNameList[index], data, 'id');
                // console.timeEnd('restoreRecord' + (index + 1))
                stored += data.length;
                _Log__WEBPACK_IMPORTED_MODULE_2__.log.log(`${stored}/${total}`, 1, false);
            }
            catch (error) {
                const errorMsg = error?.target?.error;
                const tip = errorMsg ? errorMsg : error;
                _Log__WEBPACK_IMPORTED_MODULE_2__.log.error(tip);
                _MsgBox__WEBPACK_IMPORTED_MODULE_9__.msgBox.error(tip);
            }
        }
        console.timeEnd('importRecord');
        if (stored < total) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入成功'));
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入成功'));
        _MsgBox__WEBPACK_IMPORTED_MODULE_9__.msgBox.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入成功'), {
            title: _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入下载记录'),
        });
        // 时间参考：导入 100000 条下载记录，花费的时间在 30 秒以内。但偶尔会有例外，中途像卡住了一样，很久没动，最后花了两分钟多的时间。
    }
    // 从 json 文件导入
    async importRecordFromJSON() {
        const record = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.loadJSONFile().catch((err) => {
            _MsgBox__WEBPACK_IMPORTED_MODULE_9__.msgBox.error(err);
            return;
        }));
        if (!record) {
            return;
        }
        // 判断格式是否符合要求
        if (Array.isArray(record) === false ||
            record[0].id === undefined ||
            record[0].n === undefined) {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_9__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_格式错误'));
        }
        this.importRecord(record);
    }
    // 从 txt 文件导入
    // 每行一个文件 id（带序号），以换行分割
    async importRecordFromTxt() {
        const file = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.selectFile('.txt'))[0];
        const text = await file.text();
        // 以换行分割
        let split = '\r\n';
        if (!text.includes(split)) {
            split = '\n';
        }
        const arr = text.split(split);
        // 把每一行视为一个 id，进行导入
        const record = [];
        for (const str of arr) {
            if (str) {
                record.push({
                    id: str,
                    n: str,
                });
            }
        }
        this.importRecord(record);
    }
}
const downloadRecord = new DownloadRecord();



/***/ }),

/***/ "./src/ts/download/DownloadStates.ts":
/*!*******************************************!*\
  !*** ./src/ts/download/DownloadStates.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   downloadStates: () => (/* binding */ downloadStates)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");


// 下载状态列表
class DownloadStates {
    constructor() {
        this.states = [];
        this.bindEvents();
    }
    bindEvents() {
        // 初始化下载状态
        const evs = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resultChange];
        for (const ev of evs) {
            window.addEventListener(ev, () => {
                this.init();
            });
        }
    }
    // 创建新的状态列表
    init() {
        this.states = new Array(_store_Store__WEBPACK_IMPORTED_MODULE_1__.store.result.length).fill(-1);
    }
    // 统计下载完成的数量
    downloadedCount() {
        let count = 0;
        const length = this.states.length;
        for (let i = 0; i < length; i++) {
            if (this.states[i] === 1) {
                count++;
            }
        }
        return count;
    }
    // 接受传入的状态数据
    // 目前只有在恢复下载的时候使用
    replace(states) {
        this.states = states;
    }
    // 恢复之前的下载任务
    // 这会把之前的“下载中”标记复位到“未开始下载”，以便再次下载
    resume() {
        const length = this.states.length;
        for (let i = 0; i < length; i++) {
            if (this.states[i] === 0) {
                this.setState(i, -1);
            }
        }
    }
    // 获取第一个“未开始下载”标记的索引
    getFirstDownloadItem() {
        const length = this.states.length;
        for (let i = 0; i < length; i++) {
            if (this.states[i] === -1) {
                this.setState(i, 0);
                return i;
            }
        }
        return undefined;
    }
    // 设置已下载列表中的标记
    setState(index, value) {
        this.states[index] = value;
    }
    clear() {
        this.states = [];
    }
}
const downloadStates = new DownloadStates();



/***/ }),

/***/ "./src/ts/download/ExportLST.ts":
/*!**************************************!*\
  !*** ./src/ts/download/ExportLST.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");






// 输出 lst 文件
class ExportLST {
    constructor() {
        this.separate = '?/'; // 分隔符
        this.CRLF = '\r\n'; // 换行符
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener('keydown', (ev) => {
            if (ev.altKey && ev.code === 'KeyL') {
                this.createLst();
            }
        }, false);
    }
    createLst() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_1__.store.result.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_没有可用的抓取结果'));
            return;
        }
        const array = [];
        for (const data of _store_Store__WEBPACK_IMPORTED_MODULE_1__.store.result) {
            array.push(data.original + this.separate + _FileName__WEBPACK_IMPORTED_MODULE_2__.fileName.createFileName(data));
        }
        const result = array.join(this.CRLF);
        const blob = new Blob([result]);
        const url = URL.createObjectURL(blob);
        const name = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.getPageTitle() + '.lst';
        _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.downloadFile(url, name);
    }
}
new ExportLST();


/***/ }),

/***/ "./src/ts/download/ExportResult.ts":
/*!*****************************************!*\
  !*** ./src/ts/download/ExportResult.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");







class ExportResult {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.exportResult, () => {
            this.output();
        });
    }
    async output() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_没有可用的抓取结果'));
            return;
        }
        const resultList = await _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.json2BlobSafe(_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result);
        for (const result of resultList) {
            _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.downloadFile(result.url, `result-total ${result.total}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.replaceUnsafeStr(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getPageTitle())}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.replaceUnsafeStr(_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.crawlCompleteTime.toLocaleString())}.json`);
        }
        const msg = _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_导出成功');
        _Log__WEBPACK_IMPORTED_MODULE_6__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(msg);
    }
}
new ExportResult();


/***/ }),

/***/ "./src/ts/download/ExportResult2CSV.ts":
/*!*********************************************!*\
  !*** ./src/ts/download/ExportResult2CSV.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _utils_CreateCSV__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/CreateCSV */ "./src/ts/utils/CreateCSV.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");










// name 这个字段在 csv 里的标题
// index 这个字段在数据里的索引名
// 导出抓取结果为 csv 文件
class ExportResult2CSV {
    constructor() {
        // 定义要保存的字段
        this.fieldCfg = [
            {
                name: 'id',
                index: 'idNum',
            },
            {
                name: 'tags',
                index: 'tags',
            },
            {
                name: 'tags_transl',
                index: 'tagsTranslOnly',
            },
            {
                name: 'user',
                index: 'user',
            },
            {
                name: 'userId',
                index: 'userId',
            },
            {
                name: 'title',
                index: 'title',
            },
            {
                name: 'description',
                index: 'description',
            },
            {
                name: 'type',
                index: 'type',
            },
            {
                name: 'page',
                index: 'pageCount',
            },
            {
                name: 'bookmark',
                index: 'bmk',
            },
            {
                name: 'bookmarked',
                index: 'bookmarked',
            },
            {
                name: 'likeCount',
                index: 'likeCount',
            },
            {
                name: 'viewCount',
                index: 'viewCount',
            },
            {
                name: 'commentCount',
                index: 'commentCount',
            },
            {
                name: 'width',
                index: 'fullWidth',
            },
            {
                name: 'height',
                index: 'fullHeight',
            },
            {
                name: 'xRestrict',
                index: 'xRestrict',
            },
            {
                name: 'AI',
                index: 'aiType',
            },
            {
                name: 'date',
                index: 'date',
            },
            {
                name: 'original',
                index: 'original',
            },
            {
                name: 'thumb',
                index: 'thumb',
            },
            // fileName 字段的 index 属性可以随便写，因为没有影响。
            {
                name: 'fileName',
                index: 'title',
            },
        ];
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.exportCSV, () => {
            this.beforeCreate();
        });
    }
    beforeCreate() {
        // 如果没有数据则不执行
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有数据可供使用'));
            return;
        }
        // 使用 result 而不使用 resultMeta。主要是因为断点续传时只会恢复 result，不会恢复 resultMeta，所以 result 最可靠。考虑如下情况：
        // 1：刷新页面后，断点续传恢复了保存的数据，此时只有 result 里有数据，resultMeta 没有数据。
        // 2: 如果在页面 A 进行了下载，resultMeta 保存的是页面 A 的数据。此时进入页面 B，恢复了 B 页面保存的任务，此时 resultMeta 里还是页面 A 的数据。
        // 所以还是使用 result 比较可靠，不易出问题。
        this.create(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.result);
    }
    create(data) {
        const body = []; // 内容数组
        // 标题数组
        const head = [];
        for (const field of this.fieldCfg) {
            head.push(field.name);
        }
        body.push(head);
        // 循环每个作品的数据
        for (const d of data) {
            // 每个作品只导出一条记录
            // 如果是多图作品里第一张图之后的图片数据，则跳过
            const index = d.index ?? _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getResultIndex(d);
            if (index > 0) {
                continue;
            }
            const bodyItem = []; // 储存这个作品生成的所有字段
            // 生成每个字段的结果
            for (const field of this.fieldCfg) {
                if (field.name === 'fileName') {
                    bodyItem.push(_FileName__WEBPACK_IMPORTED_MODULE_5__.fileName.createFileName(d));
                }
                else {
                    let result = d[field.index] ?? '';
                    // 对于某些字段，将其内容特殊化处理
                    if (field.name === 'type') {
                        result = _Config__WEBPACK_IMPORTED_MODULE_3__.Config.worksTypeName[result];
                    }
                    if (field.name === 'bookmarked') {
                        result = result ? 'Yes' : 'No';
                    }
                    if (field.name === 'xRestrict') {
                        result = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getXRestrictText(result) || '';
                    }
                    if (field.name === 'AI') {
                        result = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getAITypeText(d.aiType || 0);
                    }
                    bodyItem.push(result);
                }
            }
            // 把这个作品的数据添加到内容数组里
            body.push(bodyItem);
        }
        const csv = _utils_CreateCSV__WEBPACK_IMPORTED_MODULE_6__.createCSV.create(body);
        const csvURL = URL.createObjectURL(csv);
        // 设置文件名
        let csvName = `result-total ${body.length - 1}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_8__.Utils.replaceUnsafeStr(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getPageTitle())}-${_utils_Utils__WEBPACK_IMPORTED_MODULE_8__.Utils.replaceUnsafeStr(_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.crawlCompleteTime.toLocaleString())}.csv`;
        _utils_Utils__WEBPACK_IMPORTED_MODULE_8__.Utils.downloadFile(csvURL, csvName);
        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导出成功'));
        _ShowHelp__WEBPACK_IMPORTED_MODULE_9__.showHelp.show('tipCSV', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_导出CSV文件的提示'));
    }
}
new ExportResult2CSV();


/***/ }),

/***/ "./src/ts/download/ImportResult.ts":
/*!*****************************************!*\
  !*** ./src/ts/download/ImportResult.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");








class ImportResult {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.importResult, () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_3__.states.busy) {
                _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前任务尚未完成'));
                return;
            }
            this.import();
        });
    }
    async import() {
        const loadedJSON = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.loadJSONFile().catch((err) => {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(err);
        }));
        if (!loadedJSON) {
            return;
        }
        // 要求是数组并且要有内容
        if (!Array.isArray(loadedJSON) || !loadedJSON.length || !loadedJSON[0]) {
            return _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_格式错误'));
        }
        // 检查是否含有必须的字段（只检查了一部分）
        const keys = Object.keys(loadedJSON[0]);
        const need = ['idNum', 'id', 'original', 'type', 'ext'];
        for (const field of need) {
            if (!keys.includes(field)) {
                return _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_格式错误'));
            }
        }
        // 根据过滤选项，过滤导入的结果
        const temp = [];
        for (const result of loadedJSON) {
            const check = await _filter_Filter__WEBPACK_IMPORTED_MODULE_7__.filter.check({
                aiType: result.aiType,
                id: result.idNum,
                workType: result.type,
                pageCount: result.pageCount,
                tags: result.tagsWithTransl,
                bookmarkCount: result.bmk,
                bookmarkData: result.bookmarked,
                width: result.pageCount === 1 ? result.fullWidth : 0,
                height: result.pageCount === 1 ? result.fullHeight : 0,
                createDate: result.date,
                userId: result.userId,
                xRestrict: result.xRestrict,
            });
            if (check) {
                temp.push(result);
            }
        }
        // 如果没有符合过滤条件的结果
        if (temp.length === 0) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有符合条件的结果'));
            return;
        }
        // 恢复数据
        // 通过 store.addResult 添加数据，可以应用多图作品设置，对导入的结果进行调整
        _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.reset();
        for (const r of temp) {
            _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.addResult(r);
        }
        // 发送通知
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('crawlComplete');
        _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_导入成功'));
    }
}
new ImportResult();


/***/ }),

/***/ "./src/ts/download/MakeNovelFile.ts":
/*!******************************************!*\
  !*** ./src/ts/download/MakeNovelFile.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   MakeNovelFile: () => (/* binding */ MakeNovelFile)
/* harmony export */ });
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./DownloadNovelEmbeddedImage */ "./src/ts/download/DownloadNovelEmbeddedImage.ts");




class MakeNovelFile {
    static async make(data, type = _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.novelSaveAs) {
        if (type === 'txt') {
            return this.makeTXT(data, _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.saveNovelMeta);
        }
        return this.makeEPUB(data, _setting_Settings__WEBPACK_IMPORTED_MODULE_0__.settings.saveNovelMeta);
    }
    static makeTXT(data, saveMeta = true) {
        let content = saveMeta ? data.meta + data.content : data.content;
        // 替换换行标签，移除 html 标签
        content = content.replace(/<br \/>/g, '\n').replace(/<\/?.+?>/g, '');
        return new Blob([content], {
            type: 'text/plain',
        });
    }
    static makeEPUB(data, saveMeta = true) {
        return new Promise(async (resolve, reject) => {
            let content = saveMeta ? data.meta + data.content : data.content;
            content = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.replaceEPUBText(content);
            // 添加小说里内嵌的图片。这部分必须放在 replaceEPUBText 后面，否则 <img> 标签的左尖括号会被转义
            content = await _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_3__.downloadNovelEmbeddedImage.EPUB(content, data.embeddedImages);
            // epub 内部会使用标题 title 建立一个文件夹，把一些文件存放进去，所以要替换掉标题的特殊字符。特殊字符会导致这个文件夹名被截断，结果就是这个 epub 文件无法被解析。
            const userName = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.replaceEPUBText(_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(data.userName));
            const title = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.replaceEPUBText(_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(data.title));
            new EpubMaker()
                .withTemplate('idpf-wasteland')
                .withAuthor(userName)
                .withModificationDate(new Date(data.createDate))
                .withRights({
                description: _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.replaceEPUBText(data.description),
                license: '',
            })
                .withAttributionUrl(`https://www.pixiv.net/novel/show.php?id=${data.id}`)
                .withCover(data.coverUrl, {
                license: '',
                attributionUrl: '',
            })
                .withTitle(title)
                .withSection(new EpubMaker.Section('chapter', null, {
                title: title,
                content: content,
            }, true, true))
                .makeEpub()
                .then((blob) => {
                resolve(blob);
            });
        });
    }
}



/***/ }),

/***/ "./src/ts/download/MergeNovel.ts":
/*!***************************************!*\
  !*** ./src/ts/download/MergeNovel.ts ***!
  \***************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _download_DownloadNovelCover__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../download/DownloadNovelCover */ "./src/ts/download/DownloadNovelCover.ts");
/* harmony import */ var _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./DownloadNovelEmbeddedImage */ "./src/ts/download/DownloadNovelEmbeddedImage.ts");









class MergeNovel {
    constructor() {
        this.CRLF = '\n'; // pixiv 小说的换行符
        /**在文件开头添加的元数据 */
        this.meta = '';
        this.init();
    }
    init() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete, () => {
            window.setTimeout(() => {
                if (_store_States__WEBPACK_IMPORTED_MODULE_3__.states.mergeNovel) {
                    this.merge();
                }
            }, 0);
        });
    }
    async merge() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta.length === 0 ||
            _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta[0].novelMeta === null) {
            _store_States__WEBPACK_IMPORTED_MODULE_3__.states.mergeNovel = false;
            return;
        }
        // 因为结果里的小说顺序可能是乱的，所以需要按照小说的序号对结果进行排序
        const allResult = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta.sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.sortByProperty('seriesOrder', 'asc'));
        const firstResult = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.resultMeta[0];
        // 汇总小说数据
        const allNovelData = [];
        for (const result of allResult) {
            allNovelData.push({
                no: result.seriesOrder,
                title: _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(result.title),
                content: result.novelMeta.content,
                embeddedImages: result.novelMeta.embeddedImages,
            });
        }
        // 生成 meta 文本
        this.meta = '';
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.saveNovelMeta) {
            const metaArray = [];
            // 系列标题
            metaArray.push(firstResult.seriesTitle);
            // 作者
            metaArray.push(firstResult.user);
            // 网址链接
            const link = `https://www.pixiv.net/novel/series/${firstResult.seriesId}`;
            metaArray.push(link + this.CRLF.repeat(2));
            // 设定资料
            if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.novelSeriesGlossary) {
                metaArray.push(_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.novelSeriesGlossary);
            }
            this.meta = metaArray.join(this.CRLF.repeat(2));
        }
        // 生成小说文件并下载
        let file = null;
        const novelName = `${firstResult.seriesTitle}-tags_${firstResult.tags}-user_${firstResult.user}-seriesId_${firstResult.seriesId}.${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.novelSaveAs}`;
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.novelSaveAs === 'txt') {
            file = await this.makeTXT(allNovelData);
            // 保存为 txt 格式时，在这里下载小说内嵌的图片
            for (const result of allResult) {
                await _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_8__.downloadNovelEmbeddedImage.TXT(result.novelMeta.content, result.novelMeta.embeddedImages, novelName, 'mergeNovel');
            }
        }
        else {
            file = await this.makeEPUB(allNovelData, firstResult);
        }
        const url = URL.createObjectURL(file);
        _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.downloadFile(url, _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(novelName));
        _store_States__WEBPACK_IMPORTED_MODULE_3__.states.mergeNovel = false;
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('downloadComplete');
        // 保存第一个小说的封面图片
        // 实际上系列的封面不一定是第一个小说的封面，这里用第一个小说的封面凑合一下
        if (firstResult.novelMeta?.coverUrl) {
            _download_DownloadNovelCover__WEBPACK_IMPORTED_MODULE_7__.downloadNovelCover.download(firstResult.novelMeta.coverUrl, novelName, 'mergeNovel');
        }
        _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.reset();
    }
    async makeTXT(novelDataArray) {
        return new Promise(async (resolve, reject) => {
            const result = [];
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.saveNovelMeta) {
                result.push(this.meta);
            }
            for (const data of novelDataArray) {
                // 添加章节名
                result.push(`${this.chapterNo(data.no)} ${data.title}`);
                // 在章节名与正文之间添加换行
                result.push(this.CRLF.repeat(2));
                // 添加正文
                // 替换换行标签，移除 html 标签
                result.push(data.content.replace(/<br \/>/g, this.CRLF).replace(/<\/?.+?>/g, ''));
                // 在正文结尾添加换行标记，使得不同章节之间区分开来
                result.push(this.CRLF.repeat(4));
            }
            const blob = new Blob(result, {
                type: 'text/plain',
            });
            return resolve(blob);
        });
    }
    makeEPUB(novelDataArray, firstResult) {
        return new Promise(async (resolve, reject) => {
            // 添加一些元数据
            let epubData = new EpubMaker()
                .withTemplate('idpf-wasteland')
                .withAuthor(_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(firstResult.novelMeta.userName))
                .withModificationDate(new Date(firstResult.novelMeta.createDate))
                .withRights({
                description: firstResult.novelMeta.description,
                license: '',
            })
                .withAttributionUrl(`https://www.pixiv.net/novel/show.php?id=${firstResult.novelMeta.id}`)
                .withCover(firstResult.novelMeta.coverUrl, {
                license: '',
                attributionUrl: '',
            })
                .withTitle(_utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.replaceUnsafeStr(firstResult.seriesTitle));
            // 下面注释的伪代码是用于创建二级目录用的。目前 pixiv 的小说只需要一层目录就够了，所以这里的代码未被使用
            // const Section = new EpubMaker.Section(...........)
            // for (const data of novelDataArray) {
            //   Section.withSubSection(
            //     new EpubMaker.Section(...........)
            //   )
            // }
            // epubData = epubData.withSection(Section)
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.saveNovelMeta) {
                epubData.withSection(new EpubMaker.Section('chapter', 0, {
                    title: _Lang__WEBPACK_IMPORTED_MODULE_5__.lang.transl('_设定资料'),
                    content: _Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.replaceEPUBText(this.meta),
                }, true, true));
            }
            // 为每一篇小说创建一个章节
            for (const data of novelDataArray) {
                let content = _Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.replaceEPUBText(data.content);
                // 添加小说里内嵌的图片。这部分必须放在 replaceEPUBText 后面，否则 <img> 标签的左尖括号会被转义
                content = await _DownloadNovelEmbeddedImage__WEBPACK_IMPORTED_MODULE_8__.downloadNovelEmbeddedImage.EPUB(content, data.embeddedImages);
                // 创建 epub 文件时不需要在标题和正文后面添加换行符
                epubData.withSection(new EpubMaker.Section('chapter', data.no, {
                    title: `${this.chapterNo(data.no)} ${data.title}`,
                    content: content,
                }, true, true)
                // 倒数第二个参数是 includeInToc，必须为 true，否则某些小说阅读软件无法读取章节信息
                // includeInToc 的作用是在 .ncx 文件和 nav.xhtml 文件里添加导航信息
                );
            }
            epubData.makeEpub().then((blob) => {
                resolve(blob);
            });
        });
    }
    // 在每个小说的开头加上章节编号
    // 在 TXT 格式的小说里添加章节编号，可以使小说阅读软件能够识别章节，以及显示章节导航，提高阅读体验
    // 对于 EPUB 格式的小说，由于其内部自带分章结构，所以并不依赖这里的章节编号
    chapterNo(number) {
        // 如果是中文用户，返回“第N章”。这样最容易被国内的小说阅读软件识别出来
        if (_Lang__WEBPACK_IMPORTED_MODULE_5__.lang.type === 'zh-cn' || _Lang__WEBPACK_IMPORTED_MODULE_5__.lang.type === 'zh-tw' || _Lang__WEBPACK_IMPORTED_MODULE_5__.lang.type === 'ja') {
            return `第${number}章`;
        }
        else {
            // 对于其他地区，返回 `Chapter N`。但是由于我没有使用过国外的小说阅读软件，所以并不清楚是否能够起到分章作用
            return `Chapter ${number}`;
        }
        // 我还尝试过使用 #1 这样的编号，但是这种方式并不可靠，有的小说可以分章有的小说不可以
    }
}
new MergeNovel();


/***/ }),

/***/ "./src/ts/download/ProgressBar.ts":
/*!****************************************!*\
  !*** ./src/ts/download/ProgressBar.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   progressBar: () => (/* binding */ progressBar)
/* harmony export */ });
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
// 下载进度条




// 进度条
class ProgressBar {
    constructor() {
        this.wrapHTML = `
  <div class="progressBarWrap">
  <div class="total">
  <span class="text" data-xztext="_下载进度"></span>
  <div class="right1">
  <div class="progressBar progressBar1">
  <div class="progress progress1"></div>
  </div>
  <div class="totalNumberWrap">
  <span class="downloaded">0</span>
  /
  <span class="imgNum totalNumber">0</span>
  </div>
  </div>
  </div>

  <ul class="progressBarList"></ul>
  </div>
  `;
        this.barHTML = `<li class="downloadBar">
  <div class="progressBar progressBar2">
  <div class="progress progress2"></div>
  </div>
  <div class="progressTip progressTip2">
  <span class="fileName"></span>
  <span class="loadedWrap">
  <span class="loaded"></span>
  </span>
  </div>
  </li>`;
        this.allProgressBar = [];
        this.KB = 1024;
        this.MB = 1024 * 1024;
        this.createElements();
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.wrap);
        this.bindEvents();
    }
    createElements() {
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.useSlot('progressBar', this.wrapHTML);
        this.downloadedEl = this.wrap.querySelector('.downloaded');
        this.progressColorEl = this.wrap.querySelector('.progress1');
        this.listWrap = this.wrap.querySelector('.progressBarList');
        this.totalNumberEl = this.wrap.querySelector('.totalNumber');
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_3__.EVT.list.crawlStart, () => {
            this.hide();
        });
    }
    // 重设所有进度
    reset(progressBarNum, downloaded = 0) {
        if (progressBarNum === 0) {
            // 如果进度条数量为 0（抓取结果为空），则隐藏进度条区域
            return this.hide();
        }
        // 重置总进度条
        this.setTotalProgress(downloaded);
        this.totalNumberEl.textContent = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length.toString();
        // 重置子进度条
        this.listWrap.innerHTML = this.barHTML.repeat(progressBarNum);
        this.show();
        // 保存子进度条上需要使用到的元素
        const allProgressBar = this.listWrap.querySelectorAll('.downloadBar');
        this.allProgressBar = [];
        for (const bar of allProgressBar) {
            const data = {
                name: bar.querySelector('.fileName'),
                loaded: bar.querySelector('.loaded'),
                progress: bar.querySelector('.progress'),
            };
            this.allProgressBar.push(data);
        }
    }
    // 设置总进度条的进度
    setTotalProgress(downloaded) {
        this.downloadedEl.textContent = downloaded.toString();
        const progress = (downloaded / _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length) * 100;
        this.progressColorEl.style.width = progress + '%';
    }
    // 设置子进度条的进度
    setProgress(index, data) {
        const bar = this.allProgressBar[index];
        if (!bar) {
            console.error(index, this.allProgressBar);
            return;
        }
        bar.name.textContent = data.name;
        bar.name.setAttribute('title', data.name);
        let text = '';
        if (data.total >= this.MB) {
            // 如果判断条件加上： || data.total === 0
            // 则文件未下载时显示的默认单位会是 MiB
            // 使用 MiB 作为单位
            text = `${(data.loaded / this.MB).toFixed(1)}/${(data.total / this.MB).toFixed(1)} MiB`;
        }
        else {
            // 使用 KiB 作为单位
            text = `${Math.floor(data.loaded / this.KB)}/${Math.floor(data.total / this.KB)} KiB`;
        }
        bar.loaded.textContent = text;
        const progress = data.loaded / data.total || 0; // 若结果为 NaN 则设为 0
        bar.progress.style.width = progress * 100 + '%';
    }
    // 让某个子进度条显示警告色
    errorColor(index, show) {
        const bar = this.allProgressBar[index];
        if (!bar) {
            console.error(index, this.allProgressBar);
            return;
        }
        bar.name.classList[show ? 'add' : 'remove']('downloadError');
    }
    show() {
        this.wrap.style.display = 'block';
    }
    hide() {
        this.wrap.style.display = 'none';
    }
}
const progressBar = new ProgressBar();



/***/ }),

/***/ "./src/ts/download/Resume.ts":
/*!***********************************!*\
  !*** ./src/ts/download/Resume.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _DownloadStates__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./DownloadStates */ "./src/ts/download/DownloadStates.ts");
/* harmony import */ var _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/IndexedDB */ "./src/ts/utils/IndexedDB.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");









// 断点续传。恢复未完成的下载
class Resume {
    constructor() {
        this.DBName = 'PBD';
        this.DBVer = 3;
        this.metaName = 'taskMeta'; // 下载任务元数据的表名
        this.dataName = 'taskData'; // 下载任务数据的表名
        this.statesName = 'taskStates'; // 下载状态列表的表名
        this.part = []; // 储存每个分段里的数据的数量
        this.try = 0; // 任务结果是分批储存的，记录每批失败了几次。根据失败次数减少每批的数量
        // 尝试存储抓取结果时，单次存储的数量不能超过这个数字。因为超过这个数字可能会碰到单次存储的上限
        // 由于每个结果的体积可能不同，所以这只是一个预估值
        // 这有助于减少尝试次数。因为存储的思路是存储失败时改为上次数量的 1/2。例如有 100 w 个结果，存储算法会依次尝试存入 100 w、50 w、25 w、12.5 w 以此类推，直到最后有一次能成功存储一批数据。这样的话就进行了 4 次尝试才成功存入一批数据。但通过直接指定一批数据的大小为 onceMax，理想情况下可以只尝试一次就成功存入一批数据。
        // 非理想情况下，即这个数量的结果已经超过了单次存储上限（目前推测这可能会在大量抓取小说、动图时出现；如果抓取的作品大部分是插画、漫画，这个数量的结果应该不可能超出存储上限），那么这不会减少尝试数量，但因为每次尝试存储的数量不会超过这个数字，这依然有助于减少每次尝试时的资源占用、耗费时间。
        this.onceMax = 150000;
        this.putStatesTime = 1000; // 每隔指定时间存储一次最新的下载状态
        this.needPutStates = false; // 指示是否需要更新存储的下载状态
        this.IDB = new _utils_IndexedDB__WEBPACK_IMPORTED_MODULE_6__.IndexedDB();
        this.init();
    }
    async init() {
        if (!_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.isPixiv()) {
            return;
        }
        await this.initDB();
        this.bindEvents();
        if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.settingInitialized) {
            this.restoreData();
        }
        this.regularPutStates();
        this.clearExired();
    }
    // 初始化数据库，获取数据库对象
    async initDB() {
        // 在升级事件里创建表和索引
        const onUpdate = (db) => {
            if (!db.objectStoreNames.contains(this.metaName)) {
                const metaStore = db.createObjectStore(this.metaName, {
                    keyPath: 'id',
                });
                metaStore.createIndex('id', 'id', { unique: true });
                metaStore.createIndex('url', 'url', { unique: true });
            }
            if (!db.objectStoreNames.contains(this.dataName)) {
                const dataStore = db.createObjectStore(this.dataName, {
                    keyPath: 'id',
                });
                dataStore.createIndex('id', 'id', { unique: true });
            }
            if (!db.objectStoreNames.contains(this.statesName)) {
                const statesStore = db.createObjectStore(this.statesName, {
                    keyPath: 'id',
                });
                statesStore.createIndex('id', 'id', { unique: true });
            }
        };
        // 打开数据库
        return new Promise(async (resolve, reject) => {
            resolve(await this.IDB.open(this.DBName, this.DBVer, onUpdate));
        });
    }
    bindEvents() {
        // 切换页面时，重新检查恢复数据
        const restoreEvt = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitch, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingInitialized];
        restoreEvt.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.restoreData();
            });
        });
        // 抓取完成时，保存这次任务的数据
        const evs = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resultChange];
        for (const ev of evs) {
            window.addEventListener(ev, async () => {
                this.saveData();
            });
        }
        // 当有文件下载完成或者跳过下载时，更新下载状态
        const saveEv = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadSuccess, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.skipDownload];
        saveEv.forEach((val) => {
            window.addEventListener(val, () => {
                this.needPutStates = true;
            });
        });
        // 任务下载完毕时，以及停止任务时，清除这次任务的数据
        const clearDataEv = [_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete, _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop];
        for (const ev of clearDataEv) {
            window.addEventListener(ev, async () => {
                this.clearData();
            });
        }
        // 清空已保存的抓取结果
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.clearSavedCrawl, () => {
            this.clearSavedCrawl();
        });
    }
    // 恢复未完成任务的数据
    async restoreData() {
        // 如果下载器在抓取或者在下载，则不恢复数据
        if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
            return;
        }
        // 1 获取任务的元数据
        const meta = (await this.IDB.get(this.metaName, this.getURL(), 'url'));
        if (!meta) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正在恢复抓取结果'));
        this.taskId = meta.id;
        // 2 恢复抓取结果
        // 生成每批数据的 id 列表
        const dataIdList = this.createIdList(meta.id, meta.part);
        // 读取全部数据并恢复
        const promiseList = [];
        for (const id of dataIdList) {
            promiseList.push(this.IDB.get(this.dataName, id));
        }
        await Promise.all(promiseList).then((res) => {
            // 恢复数据时不适合使用 store.addResult，因为那样会被多图作品设置影响，可能导致恢复的数据和之前下载时不一致
            // 所以这里直接替换整个 store.result
            _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result = [];
            const r = res;
            for (const taskData of r) {
                for (const data of taskData.data) {
                    _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.push(data);
                }
            }
            _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.resetDownloadCount();
        });
        // 3 恢复下载状态
        const data = (await this.IDB.get(this.statesName, this.taskId));
        if (data) {
            _DownloadStates__WEBPACK_IMPORTED_MODULE_5__.downloadStates.replace(data.states);
        }
        _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.crawlCompleteTime = meta.date;
        _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.URLWhenCrawlStart = meta.URLWhenCrawlStart || '';
        // 恢复模式就绪
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已恢复抓取结果'), 2);
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('resume');
    }
    async saveData() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.mergeNovel) {
            return;
        }
        // 首先检查这个网址下是否已经存在数据，如果有数据，则清除之前的数据，保持每个网址只有一份数据
        const taskData = (await this.IDB.get(this.metaName, this.getURL(), 'url'));
        if (taskData) {
            await this.IDB.delete(this.metaName, taskData.id);
            await this.IDB.delete(this.statesName, taskData.id);
        }
        // 保存本次任务的数据
        // 如果此时本次任务已经完成，就不进行保存了
        if (_DownloadStates__WEBPACK_IMPORTED_MODULE_5__.downloadStates.downloadedCount() === _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.length) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_正在保存抓取结果'));
        this.taskId = new Date().getTime();
        this.part = [];
        await this.saveTaskData();
        // 保存 meta 数据
        const metaData = {
            id: this.taskId,
            url: this.getURL(),
            URLWhenCrawlStart: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.URLWhenCrawlStart,
            part: this.part.length,
            date: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.crawlCompleteTime,
        };
        this.IDB.add(this.metaName, metaData);
        // 保存 states 数据
        const statesData = {
            id: this.taskId,
            states: _DownloadStates__WEBPACK_IMPORTED_MODULE_5__.downloadStates.states,
        };
        this.IDB.add(this.statesName, statesData);
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已保存抓取结果'), 2);
    }
    // 存储抓取结果
    async saveTaskData() {
        return new Promise(async (resolve, reject) => {
            // 每一批任务的第一次执行会尝试保存所有剩余数据(0.5 的 0 次幂是 1)
            // 如果出错了，则每次执行会尝试保存上一次数据量的一半，直到这次存储成功
            // 之后继续进行下一批任务（如果有）
            let tryNum = Math.floor(_store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.length * Math.pow(0.5, this.try));
            // 如果这批尝试数据大于指定数量，则设置为指定数量
            tryNum > this.onceMax && (tryNum = this.onceMax);
            let data = {
                id: this.numAppendNum(this.taskId, this.part.length),
                data: _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.slice(this.getPartTotal(), this.getPartTotal() + tryNum),
            };
            try {
                // 当成功存储了一批数据时
                await this.IDB.add(this.dataName, data);
                this.part.push(data.data.length); // 记录这一次保存的结果数量
                this.try = 0; // 重置已尝试次数
                // 任务数据全部添加完毕
                if (this.getPartTotal() >= _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.length) {
                    resolve(true);
                }
                else {
                    // 任务数据没有添加完毕，继续添加
                    resolve(this.saveTaskData());
                }
            }
            catch (error) {
                // 当存储失败时
                console.error(error);
                if (error.target && error.target.error && error.target.error.message) {
                    const msg = error.target.error.message;
                    if (msg.includes('too large')) {
                        // 体积超大
                        // 尝试次数 + 1 ，进行下一次尝试
                        this.try++;
                        resolve(this.saveTaskData());
                    }
                    else {
                        // 未知错误，不再进行尝试
                        this.try = 0;
                        _Log__WEBPACK_IMPORTED_MODULE_1__.log.error('IndexedDB: ' + msg);
                        reject(error);
                    }
                }
            }
        });
    }
    // 定时 put 下载状态
    async regularPutStates() {
        window.setInterval(() => {
            if (this.needPutStates) {
                const statesData = {
                    id: this.taskId,
                    states: _DownloadStates__WEBPACK_IMPORTED_MODULE_5__.downloadStates.states,
                };
                this.needPutStates = false;
                // 如果此时本次任务已经完成，就不进行保存了
                if (_DownloadStates__WEBPACK_IMPORTED_MODULE_5__.downloadStates.downloadedCount() === _store_Store__WEBPACK_IMPORTED_MODULE_3__.store.result.length) {
                    return;
                }
                this.IDB.put(this.statesName, statesData);
            }
        }, this.putStatesTime);
    }
    async clearData() {
        if (!this.taskId) {
            return;
        }
        const meta = (await this.IDB.get(this.metaName, this.taskId));
        if (!meta) {
            return;
        }
        this.IDB.delete(this.metaName, this.taskId);
        this.IDB.delete(this.statesName, this.taskId);
        const dataIdList = this.createIdList(this.taskId, meta.part);
        for (const id of dataIdList) {
            this.IDB.delete(this.dataName, id);
        }
    }
    // 清除过期的数据
    async clearExired() {
        // 数据的过期时间，设置为 30 天。30*24*60*60*1000
        const expiryTime = 2592000000;
        // 每隔一天检查一次数据是否过期
        const nowTime = new Date().getTime();
        let lastCheckTime = 0;
        const storeName = 'lastCheckExired';
        const data = localStorage.getItem(storeName);
        if (data === null) {
            localStorage.setItem(storeName, lastCheckTime.toString());
        }
        else {
            lastCheckTime = Number.parseInt(data);
        }
        if (nowTime - lastCheckTime < 86400000) {
            return;
        }
        localStorage.setItem(storeName, nowTime.toString());
        // 检查数据是否过期
        const callback = (item) => {
            if (item) {
                const data = item.value;
                if (nowTime - data.id > expiryTime) {
                    this.IDB.delete(this.metaName, data.id);
                    this.IDB.delete(this.statesName, data.id);
                    const dataIdList = this.createIdList(data.id, data.part);
                    for (const id of dataIdList) {
                        this.IDB.delete(this.dataName, id);
                    }
                }
                item.continue();
            }
        };
        this.IDB.openCursor(this.metaName, callback);
    }
    // 计算 part 数组里的数字之和
    getPartTotal() {
        if (this.part.length === 0) {
            return 0;
        }
        return this.part.reduce((prev, curr) => {
            return prev + curr;
        });
    }
    // 处理本页面的 url
    getURL() {
        return window.location.href.split('#')[0];
    }
    // 在数字后面追加数字
    // 用于在 task id  后面追加序号数字(part)
    numAppendNum(id, num) {
        return parseInt(id.toString() + num);
    }
    // 根据 taskMeta 里的 id 和 part 数量，生成 taskData 里对应的数据的 id 列表
    createIdList(taskid, part) {
        // part 记录数据分成了几部分，所以是从 1 开始的，而不是从 0 开始
        // 生成的 id 的结尾是从 0 开始增加的
        const arr = [];
        let start = 0;
        while (start < part) {
            arr.push(this.numAppendNum(taskid, start));
            start++;
        }
        return arr;
    }
    // 清空已保存的抓取结果
    async clearSavedCrawl() {
        await Promise.all([
            this.IDB.clear(this.metaName),
            this.IDB.clear(this.dataName),
            this.IDB.clear(this.statesName),
        ]);
        _Toast__WEBPACK_IMPORTED_MODULE_8__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_数据清除完毕'));
    }
}
new Resume();


/***/ }),

/***/ "./src/ts/download/SaveWorkMeta.ts":
/*!*****************************************!*\
  !*** ./src/ts/download/SaveWorkMeta.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");






// 为每个作品创建一个 txt 文件，保存这个作品的元数据
class SaveWorkMeta {
    constructor() {
        // 保存已经下载了元数据的作品的 id
        this.savedIds = [];
        this.CRLF = '\n'; // txt 文件中使用的换行符
        this.bindEvents();
    }
    bindEvents() {
        // 当有作品文件下载成功时，保存其元数据
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadSuccess, (ev) => {
            const successData = ev.detail.data;
            this.saveMeta(Number.parseInt(successData.id));
        });
        // 当开始新的抓取时，清空保存的 id 列表
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.savedIds = [];
        });
    }
    // 添加一项元数据
    // 在 name 和 value 后面添加换行符
    addMeta(name, value) {
        return `${name}${this.CRLF}${value}${this.CRLF.repeat(2)}`;
    }
    getWorkURL(data) {
        return `https://www.pixiv.net/${data.type === 3 ? 'n' : 'i'}/${data.idNum}`;
    }
    joinTags(tags) {
        const format = tags.map((tag) => '#' + tag);
        return format.join(this.CRLF);
    }
    // 根据作品类型判断是否需要保存它的元数据
    checkNeedSave(type) {
        switch (type) {
            case 0:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType0;
            case 1:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType1;
            case 2:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType2;
            case 3:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType3;
            default:
                return false;
        }
    }
    saveMeta(id) {
        // 如果所有类型的作品都不需要保存元数据
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType0 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType1 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType2 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.saveMetaType3) {
            return;
        }
        if (this.savedIds.includes(id)) {
            return;
        }
        // 查找这个作品的数据
        const dataSource = _store_Store__WEBPACK_IMPORTED_MODULE_1__.store.resultMeta.length > 0 ? _store_Store__WEBPACK_IMPORTED_MODULE_1__.store.resultMeta : _store_Store__WEBPACK_IMPORTED_MODULE_1__.store.result;
        const data = dataSource.find((val) => val.idNum === id);
        if (data === undefined) {
            console.error(`Not find ${id} in result`);
            return;
        }
        if (this.checkNeedSave(data.type) === false) {
            return;
        }
        // 添加文件内容
        const fileContent = [];
        fileContent.push(this.addMeta('ID', data.idNum.toString()));
        fileContent.push(this.addMeta('URL', this.getWorkURL(data)));
        if (data.type !== 3) {
            fileContent.push(this.addMeta('Original', data.original));
        }
        fileContent.push(this.addMeta('Thumbnail', data.thumb));
        fileContent.push(this.addMeta('xRestrict', _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getXRestrictText(data.xRestrict)));
        fileContent.push(this.addMeta('AI', _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.getAITypeText(data.aiType || 0)));
        fileContent.push(this.addMeta('User', data.user));
        fileContent.push(this.addMeta('UserID', data.userId));
        fileContent.push(this.addMeta('Title', data.title));
        fileContent.push(this.addMeta('Description', _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.htmlToText(data.description)));
        fileContent.push(this.addMeta('Tags', this.joinTags(data.tags)));
        if (data.type !== 3) {
            fileContent.push(this.addMeta('Size', `${data.fullWidth} x ${data.fullHeight}`));
        }
        fileContent.push(this.addMeta('Bookmark', data.bmk.toString()));
        fileContent.push(this.addMeta('Date', data.date));
        // 生成文件
        const blob = new Blob(fileContent, {
            type: 'text/plain',
        });
        // 生成文件名
        // 元数据文件需要和它对应的图片/小说文件的路径相同，文件名相似，这样它们才能在资源管理器里排在一起，便于查看
        // 生成这个数据的路径和文件名
        const _fileName = _FileName__WEBPACK_IMPORTED_MODULE_2__.fileName.createFileName(data);
        // 取出后缀名之前的部分
        const index = _fileName.lastIndexOf('.');
        let part1 = _fileName.substring(0, index);
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.zeroPadding) {
            // 把 id 字符串换成数字 id，这是为了去除 id 后面可能存在的序号，如 p0
            // 但如果用户启用了在序号前面填充 0，则不替换 id，因为文件名里的 id 后面可能带多个 0，如 p000，用 idNum 去替换的话替换不了后面两个 0
            part1 = part1.replace(data.id, data.idNum.toString());
        }
        // 拼接出元数据文件的文件名
        const metaFileName = `${part1}-meta.txt`;
        // 发送下载请求
        // 因为我偷懒，所以后台不会返回下载状态，默认为下载成功
        chrome.runtime.sendMessage({
            msg: 'save_description_file',
            fileUrl: URL.createObjectURL(blob),
            fileName: metaFileName,
        });
        this.savedIds.push(id);
    }
}
new SaveWorkMeta();


/***/ }),

/***/ "./src/ts/download/ShowConvertCount.ts":
/*!*********************************************!*\
  !*** ./src/ts/download/ShowConvertCount.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ShowConvertCount: () => (/* binding */ ShowConvertCount)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");


// 显示正在转换的文件数量
class ShowConvertCount {
    constructor(el) {
        this.el = el;
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.register(this.el);
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.convertChange, (ev) => {
            const count = ev.detail.data;
            if (count > 0) {
                _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.el, '_转换任务提示', count.toString());
            }
            else {
                this.el.textContent = '';
                _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.el, '');
            }
        });
    }
}



/***/ }),

/***/ "./src/ts/download/ShowDownloadStates.ts":
/*!***********************************************!*\
  !*** ./src/ts/download/ShowDownloadStates.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ShowDownloadStates: () => (/* binding */ ShowDownloadStates)
/* harmony export */ });
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");



// 显示下载状态
class ShowDownloadStates {
    constructor(el) {
        this.el = el;
        this.bindEvents();
    }
    bindEvents() {
        for (const ev of [
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resultChange,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resume,
        ]) {
            window.addEventListener(ev, () => {
                this.setText('_未开始下载');
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadStart, () => {
            this.setText('_正在下载中');
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadPause, () => {
            this.setText('_已暂停', '#f00');
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadStop, () => {
            this.setText('_已停止', '#f00');
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadComplete, () => {
            this.setText('_下载完毕', _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.textSuccess);
        });
    }
    setText(textFlag, color = _Colors__WEBPACK_IMPORTED_MODULE_0__.Colors.bgBlue) {
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.el, textFlag);
        this.el.style.color = color;
    }
}



/***/ }),

/***/ "./src/ts/download/ShowRemainingDownloadOnTitle.ts":
/*!*********************************************************!*\
  !*** ./src/ts/download/ShowRemainingDownloadOnTitle.ts ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");



// 在网页标题上显示剩余下载数量
class ShowRemainingDownloadOnTitle {
    constructor() {
        // 保存缓存的字符串，后面会需要在标题中查找缓存的字符串
        this.str = this.createStr();
        this.bindEvents();
    }
    bindEvents() {
        window.setInterval(() => {
            this.show();
        }, 500);
        const removeStrEvents = [
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.crawlStart,
        ];
        for (const evt of removeStrEvents) {
            window.addEventListener(evt, () => {
                this.removeStr();
            });
        }
    }
    // 生成新的字符串
    createStr() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.remainingDownload > 0) {
            return ` ${_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.remainingDownload} `;
        }
        return '';
    }
    // 检查标题中是否有下载状态的 flag
    // 如果没有，就不会显示剩余数量
    checkStatusFlag() {
        return document.title.indexOf(']') > 0;
    }
    show() {
        if (!_store_States__WEBPACK_IMPORTED_MODULE_1__.states.downloading || !this.checkStatusFlag()) {
            return;
        }
        // 先移除旧的字符串，然后添加新的字符串
        const title = this.removeStr(document.title);
        this.str = this.createStr();
        if (!this.str || !title) {
            return;
        }
        document.title = title.replace(']', ']' + this.str);
    }
    removeStr(titleStr) {
        if (!this.str) {
            return;
        }
        if (titleStr) {
            return titleStr.replace(this.str, '');
        }
        else {
            document.title = document.title.replace(this.str, '');
        }
    }
}
new ShowRemainingDownloadOnTitle();


/***/ }),

/***/ "./src/ts/download/ShowSkipCount.ts":
/*!******************************************!*\
  !*** ./src/ts/download/ShowSkipCount.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   ShowSkipCount: () => (/* binding */ ShowSkipCount)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");


// 显示跳过下载的文件数量
class ShowSkipCount {
    constructor(el) {
        this.count = 0; // 跳过下载的数量
        this.el = el;
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.register(this.el);
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.reset();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop, () => {
            // 重置计数但不清空提示文字，因为用户还需要看
            this.count = 0;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.skipDownload, () => {
            this.addCount();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStart, () => {
            if (this.count === 0) {
                this.reset();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resultChange, () => {
            this.reset();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete, () => {
            // 重置计数但不清空提示文字，因为用户还需要看
            this.count = 0;
        });
    }
    addCount() {
        this.count++;
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.el, '_已跳过n个文件', this.count.toString());
    }
    reset() {
        this.count = 0;
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.el, '');
    }
}



/***/ }),

/***/ "./src/ts/download/ShowStatusOnTitle.ts":
/*!**********************************************!*\
  !*** ./src/ts/download/ShowStatusOnTitle.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");





/**
↑ 抓取中
→ 等待下一步操作（搜索页）
▶ 可以开始下载
↓ 下载中
║ 下载暂停
■ 下载停止
✓ 下载完毕
☑ 下载完毕，并且此时的页面不是开始抓取时的页面（页面网址发生了变化）
*/
var Flags;
(function (Flags) {
    Flags["crawling"] = "\u2191";
    Flags["waiting"] = "\u2192";
    Flags["readyDownload"] = "\u25B6";
    Flags["downloading"] = "\u2193";
    Flags["paused"] = "\u2551";
    Flags["stopped"] = "\u25A0";
    Flags["completed"] = "\u2713";
    Flags["completedAndPageURLChange"] = "\u2611";
    Flags["space"] = " ";
})(Flags || (Flags = {}));
// 把下载器运行中的状态添加到页面标题前面
class ShowStatusOnTitle {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlStart, () => {
            this.set(Flags.crawling);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.worksUpdate, () => {
            this.set(Flags.waiting);
        });
        for (const ev of [
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resultChange,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.resume,
        ]) {
            window.addEventListener(ev, () => {
                this.set(Flags.readyDownload);
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadStart, () => {
            this.set(Flags.downloading);
        });
        // 切换了页面之后，标题可能会被 pixiv 修改，这样就没有标记了
        // 在这里监听页面切换的事件，如果切换后下载器仍在下载中，则重新添加标记
        // 回调函数应该在 pixiv 修改标题之后执行
        // 但是 pageSwitch 触发时，标题尚未被 pixiv 修改。pixiv 是在 pageSwitch 之后修改标题的。
        // 所以我使用定时器来检查标题是否被 pixiv 修改了
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitch, () => {
            window.clearInterval(this.pageSwitchTimer);
            if (!_store_States__WEBPACK_IMPORTED_MODULE_2__.states.downloading) {
                return;
            }
            const nowTitle = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getPageTitle();
            this.pageSwitchTimer = window.setInterval(() => {
                if (_Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getPageTitle() !== nowTitle) {
                    this.set(Flags.downloading);
                    window.clearInterval(this.pageSwitchTimer);
                }
            }, 500);
        });
        // 切换同类型页面时，如果切换之前已经有了正常下载完成的标记，则将其修改为另一个标记
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitchedTypeNotChange, () => {
            if (this.includeFlag(Flags.completed) ||
                this.includeFlag(Flags.completedAndPageURLChange)) {
                this.setCompleteFlag();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadComplete, () => {
            this.setCompleteFlag();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadPause, () => {
            this.set(Flags.paused);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.downloadStop, () => {
            this.set(Flags.stopped);
        });
        const resetEvents = [_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlEmpty, _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.stopCrawl];
        resetEvents.forEach((event) => {
            window.addEventListener(event, () => {
                this.reset();
            });
        });
    }
    // 检查标题里是否含有标记
    includeFlag(flag) {
        if (!flag) {
            // 没有传递标记，则检查所有标记
            for (const value of Object.values(Flags)) {
                const str = `[${value}]`;
                if (document.title.includes(str)) {
                    return true;
                }
            }
        }
        else {
            // 否则检查指定标记
            const str = `[${flag}]`;
            return document.title.includes(str);
        }
        return false;
    }
    // 重设 title
    reset() {
        window.clearInterval(this.flashingTimer);
        const metaTagPage = [
            _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.list.Artwork,
            _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.list.UserHome,
            _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.list.Novel,
        ];
        // 从 og:title 标签获取标题。og:title 标签是最早更新标题的。但不确定是否在所有页面上都可以直接使用 og:title 标签的内容，所以这里只在部分页面上使用
        if (metaTagPage.includes(_PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type)) {
            const ogTitle = document.querySelector('meta[property="og:title"]');
            if (ogTitle) {
                document.title = ogTitle.content;
                return;
            }
        }
        // 去掉 title 里的标记
        const index = document.title.indexOf(']');
        document.title = document.title.substring(index + 1);
    }
    // 在标题上显示指定标记
    set(flag) {
        const str = `[${flag}]`;
        // 如果 title 里没有标记，就添加标记
        if (!this.includeFlag()) {
            document.title = `${str} ${document.title}`;
        }
        else {
            // 如果已经有标记了，则替换为新当前传入的标记
            document.title = document.title.replace(/\[.?\]/, str);
        }
        // 可以开始下载，或者等待下一步操作，进行闪烁提醒
        if (flag === Flags.readyDownload || flag === Flags.waiting) {
            this.flashing(flag);
        }
        else {
            window.clearInterval(this.flashingTimer);
        }
    }
    setCompleteFlag() {
        let flag = Flags.completed;
        if (_store_Store__WEBPACK_IMPORTED_MODULE_4__.store.URLWhenCrawlStart !== '' &&
            window.location.href !== _store_Store__WEBPACK_IMPORTED_MODULE_4__.store.URLWhenCrawlStart) {
            flag = Flags.completedAndPageURLChange;
        }
        this.set(flag);
    }
    // 闪烁提醒，把给定的标记替换成空白，来回切换
    flashing(flag) {
        window.clearInterval(this.flashingTimer);
        const str = `[${flag}]`;
        const whiteSpace = `[${Flags.space}]`;
        this.flashingTimer = window.setInterval(() => {
            if (this.includeFlag(flag)) {
                // 如果含有标记，就替换成空白
                document.title = document.title.replace(str, whiteSpace);
            }
            else {
                if (this.includeFlag(Flags.space)) {
                    // 如果含有空白，就替换成标记
                    document.title = document.title.replace(whiteSpace, str);
                }
                else {
                    // 如果都没有，一般是页面切换了，标题被重置了，取消闪烁
                    window.clearInterval(this.flashingTimer);
                }
            }
        }, 500);
    }
}
new ShowStatusOnTitle();


/***/ }),

/***/ "./src/ts/download/ShowTotalResultOnTitle.ts":
/*!***************************************************!*\
  !*** ./src/ts/download/ShowTotalResultOnTitle.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");



// 抓取阶段，在网页标题上显示抓取到的结果数量
class ShowTotalResultOnTitle {
    constructor() {
        this.enable = false;
        // 保存缓存的字符串，后面会需要在标题中查找缓存的字符串
        this.str = this.createStr();
        this.bindEvents();
    }
    bindEvents() {
        const enableEvts = [_EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.crawlStart, _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.resultChange];
        enableEvts.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.removeStr();
                this.enable = true;
            });
        });
        const disableEvts = [
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadStart,
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.downloadStop,
        ];
        disableEvts.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.removeStr();
                this.enable = false;
            });
        });
        window.setInterval(() => {
            this.enable && this.show();
        }, 500);
    }
    // 生成新的字符串
    createStr() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length > 0) {
            return ` ${_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length} `;
        }
        return '';
    }
    // 检查标题中是否有下载状态的 flag
    // 如果没有，就不会显示剩余数量
    checkStatusFlag() {
        return document.title.indexOf(']') > 0;
    }
    show() {
        if (_store_States__WEBPACK_IMPORTED_MODULE_1__.states.downloading || !this.checkStatusFlag()) {
            return;
        }
        // 先移除旧的字符串，然后添加新的字符串
        const title = this.removeStr(document.title);
        this.str = this.createStr();
        if (!this.str || !title) {
            return;
        }
        document.title = title.replace(']', ']' + this.str);
    }
    // 如果传入字符串，则不直接修改 document.title，以提高性能
    removeStr(titleStr) {
        if (!this.str) {
            return;
        }
        if (titleStr) {
            return titleStr.replace(this.str, '');
        }
        else {
            document.title = document.title.replace(this.str, '');
        }
    }
}
new ShowTotalResultOnTitle();


/***/ }),

/***/ "./src/ts/filter/BlackandWhiteImage.ts":
/*!*********************************************!*\
  !*** ./src/ts/filter/BlackandWhiteImage.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   blackAndWhiteImage: () => (/* binding */ blackAndWhiteImage)
/* harmony export */ });
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");

// 检查图片是否是黑白图片
// 获取图片中 rgb 三色的平均值，如果很接近就判断为黑白图片
// 这是一个不太可靠的方法，在少数情况下，彩色图片可能会被误判为黑白图片
class BlackAndWhiteImage {
    constructor() {
        this.latitude = 1; // 宽容度
    }
    async check(imgUrl) {
        // 加载图片
        let img;
        try {
            img = await this.loadImg(imgUrl);
        }
        catch (error) {
            // loadImg 失败时返回的 reject 会在这里被捕获
            // 直接把这个图片视为彩色图片
            return false;
        }
        const imgData = this.getImageData(img);
        // 把图片的像素分为 4 份，依次检查它们的色彩
        const pixel = img.width * img.height;
        const part = 4;
        // 计算每一份有多少字节。由于像素数量可能不是 4 的整数倍，所以向下舍入
        let eachLength = Math.floor(pixel / part) * 4;
        let times = 0;
        while (times < part) {
            const start = times * eachLength;
            times++;
            const end = times * eachLength;
            const bool = this.getResult(imgData, start, end);
            // 如果某一部分是彩色图片，就直接返回结果，把整个图片视为彩色图片
            // 如果这一部分是黑白图片，则继续检查下一部分
            if (!bool) {
                return false;
            }
        }
        // 因为彩色图片会短路返回，所以执行到这里意味着所有部分都是黑白图片
        return true;
    }
    async loadImg(url) {
        return new Promise(async (resolve, reject) => {
            // 如果传递的是 blobURL 就直接使用
            if (url.startsWith('blob')) {
                resolve(_utils_Utils__WEBPACK_IMPORTED_MODULE_0__.Utils.loadImg(url));
            }
            else {
                // 不是 blobURL 的话先获取图片
                const res = await fetch(url).catch((error) => {
                    // fetch 加载图片可能会失败 TypeError: Failed to fetch
                    console.log(`Load image error! url: ${url}`);
                });
                // 如果 fetch 加载图片失败，res 会是 undefined
                if (!res) {
                    return reject();
                }
                const blob = await res.blob();
                const blobURL = URL.createObjectURL(blob);
                resolve(_utils_Utils__WEBPACK_IMPORTED_MODULE_0__.Utils.loadImg(blobURL));
            }
        });
    }
    getImageData(img) {
        const width = img.width;
        const height = img.height;
        const canvas = document.createElement('canvas');
        canvas.width = width;
        canvas.height = height;
        const con = canvas.getContext('2d');
        con.drawImage(img, 0, 0);
        const imageData = con.getImageData(0, 0, width, height);
        return imageData.data;
    }
    /**计算 r g b 三种颜色的平均值，判断是否是黑白图片
     *
     * 返回值 true 为黑白图片，false 为彩色图片
     */
    getResult(imgData, start, end) {
        // 把 R G B 值分别相加
        let r = 0;
        let g = 0;
        let b = 0;
        const totalLength = end - start;
        while (start < end) {
            r += imgData[start];
            g += imgData[start + 1];
            b += imgData[start + 2];
            start = start + 4;
        }
        // 求平均值，并取整
        const pixel = totalLength / 4;
        r = Math.round(r / pixel);
        g = Math.round(g / pixel);
        b = Math.round(b / pixel);
        // 如果 rgb 值相同则是黑白图片
        if (r === g && g === b) {
            return true;
        }
        else {
            // 如果 rgb 值不相同，则根据宽容度判断是否为黑白图片
            // 因为获取 rgb 的结果时，进行了四舍五入，即使 rgb 非常接近，也可能会相差 1，所以我设置了一个宽容度
            const max = Math.max(r, g, b); // 取出 rgb 中的最大值
            const min = max - this.latitude; // 允许的最小值
            // 如果 rgb 三个数值与最小的数值相比，差距都在宽容度之内，则视为黑白图片
            return [r, g, b].every((number) => {
                return number >= min;
            });
        }
    }
}
const blackAndWhiteImage = new BlackAndWhiteImage();



/***/ }),

/***/ "./src/ts/filter/BlockTagsForSpecificUser.ts":
/*!***************************************************!*\
  !*** ./src/ts/filter/BlockTagsForSpecificUser.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   blockTagsForSpecificUser: () => (/* binding */ blockTagsForSpecificUser)
/* harmony export */ });
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");









// 针对特定用户屏蔽 tag
class BlockTagsForSpecificUser {
    constructor() {
        this.rules = [];
        this._addWrapShow = false;
        this.wrapHTML = `
  <div class="blockTagsForSpecificUserWrap">

    <div class="controlBar">
      <span class="total">0</span>
      <button type="button" class="textButton expand" data-xztext="_收起"></button>
      <button type="button" class="textButton showAdd" data-xztext="_添加"></button>
    </div>

    <div class="addWrap">
      <div class="settingItem addInputWrap" >
        <div class="inputItem uid">
          <span class="label uidLabel" data-xztext="_用户id"></span>
          <input type="text" class="setinput_style1 blue addUidInput" data-xzplaceholder="_必须是数字" />
        </div>

        <div class="inputItem tags">
          <span class="label tagsLabel">Tags</span>
          <input type="text" class="setinput_style1 blue addTagsInput" data-xzplaceholder="_tag用逗号分割" />
        </div>

        <div class="btns">
          <button type="button" class="textButton add" data-xztitle="_添加">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-wanchengqueding"></use>
            </svg>
          </button>

          
          <button type="button" class="textButton cancel" data-xztitle="_取消">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-guanbiquxiao"></use>
            </svg>
          </button>
        </div>
      </div>
    </div>

    <div class="listWrap">
    </div>
  </div>
  `;
        this.createWrap();
        _Theme__WEBPACK_IMPORTED_MODULE_6__.theme.register(this.wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.wrap);
        this.bindEvents();
    }
    set addWrapShow(val) {
        this._addWrapShow = val;
        this.addWrap.style.display = val ? 'block' : 'none';
        if (!val) {
            this.addInputUid.value = '';
            this.addInputTags.value = '';
        }
    }
    get addWrapShow() {
        return this._addWrapShow;
    }
    // 创建列表外部的容器，静态html
    createWrap() {
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.useSlot('blockTagsForSpecificUser', this.wrapHTML);
        this.expandBtn = this.wrap.querySelector('.expand');
        this.showAddBtn = this.wrap.querySelector('.showAdd');
        this.totalSpan = this.wrap.querySelector('.total');
        this.addWrap = this.wrap.querySelector('.addWrap');
        this.addInputUid = this.wrap.querySelector('.addUidInput');
        this.addInputTags = this.wrap.querySelector('.addTagsInput');
        this.addBtn = this.wrap.querySelector('.add');
        this.cancelBtn = this.wrap.querySelector('.cancel');
        this.listWrap = this.wrap.querySelector('.listWrap');
        // 展开/折叠
        this.expandBtn.addEventListener('click', () => {
            (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserShowList', !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUserShowList);
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUserShowList &&
                this.rules.length === 0) {
                _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有数据可供使用'));
            }
        });
        // 切换显示添加区域
        this.showAddBtn.addEventListener('click', () => {
            this.addWrapShow = !this.addWrapShow;
            if (this.addWrapShow) {
                this.addInputUid.focus();
            }
        });
        // 添加规则的按钮
        this.addBtn.addEventListener('click', () => {
            this.addRule(this.addInputUid.value, this.addInputTags.value);
        });
        // 取消添加的按钮
        this.cancelBtn.addEventListener('click', () => {
            this.addWrapShow = false;
        });
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'blockTagsForSpecificUserShowList') {
                this.showListWrap();
            }
            if (data.name === 'blockTagsForSpecificUser') {
                this.createAllList();
            }
            if (data.name === 'blockTagsForSpecificUserList') {
                this.createAllList();
            }
        });
    }
    showListWrap() {
        const show = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUserShowList;
        this.listWrap.style.display = show ? 'block' : 'none';
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.expandBtn, show ? '_收起' : '_展开');
    }
    // 根据规则动态创建 html
    createAllList() {
        this.rules = [..._setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUserList];
        this.wrap.style.display = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUser
            ? 'block'
            : 'none';
        this.totalSpan.textContent = this.rules.length.toString();
        this.listWrap.innerHTML = '';
        for (const data of this.rules) {
            this.createList(data);
        }
    }
    // 创建规则对应的元素，并绑定事件
    createList(data) {
        const { uid, user, tags } = data;
        const html = `
    <div class="settingItem" data-key="${uid}">
      <div class="inputItem uid">
        <span class="label uidLabel" data-xztext="_用户id"></span>
        <input type="text" class="setinput_style1 blue" data-uidInput="${uid}" value="${uid}" />
      </div>

      <div class="inputItem tags">
        <span class="label tagsLabel">Tags</span>
        <input type="text" class="setinput_style1 blue" data-tagsInput="${uid}" value="${tags.toString()}" />
      </div>

      <div class="btns">

        <button type="button" class="textButton" data-updateRule="${uid}" data-xztitle="_更新">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-gengxin"></use>
          </svg>
        </button>

        <button type="button" class="textButton" data-deleteRule="${uid}" data-xztitle="_删除">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-shanchu1"></use>
          </svg>
        </button>
      </div>
    </div>`;
        // 倒序显示，早添加的处于底部，晚添加的处于顶部
        this.listWrap.insertAdjacentHTML('afterbegin', html);
        const uidLabel = this.listWrap.querySelector('.uidLabel');
        if (user) {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(uidLabel, '');
            uidLabel.textContent = user;
        }
        else {
            this.updateUserName(data);
        }
        const updateRule = this.listWrap.querySelector(`button[data-updateRule='${uid}']`);
        const deleteRule = this.listWrap.querySelector(`button[data-deleteRule='${uid}']`);
        const uidInput = this.listWrap.querySelector(`input[data-uidInput='${uid}']`);
        const tagsInput = this.listWrap.querySelector(`input[data-tagsInput='${uid}']`);
        [uidInput, tagsInput].forEach((el) => {
            el?.addEventListener('change', () => {
                if (el.value) {
                    this.updateRule(uid, uidInput.value, tagsInput.value, false);
                }
            });
        });
        // 更新按钮
        updateRule?.addEventListener('click', () => {
            this.updateRule(uid, uidInput.value, tagsInput.value);
        });
        // 删除按钮
        deleteRule?.addEventListener('click', () => {
            this.deleteRule(uid);
        });
    }
    // 如果某个规则没有用户名，就获取用户名储存起来
    async updateUserName(data) {
        const profile = await _API__WEBPACK_IMPORTED_MODULE_5__.API.getUserProfile(data.uid.toString()).catch((err) => {
            console.log(err);
        });
        if (profile && profile.body.name) {
            const name = profile.body.name;
            const index = this.findIndex(data.uid);
            if (index > -1) {
                this.rules[index].user = name;
                (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserList', [...this.rules]);
                // 显示到页面上
                const listElement = this.listWrap.querySelector(`.settingItem[data-key='${data.uid}']`);
                if (listElement) {
                    const label = listElement.querySelector('.uidLabel');
                    label && (label.textContent = name);
                }
            }
        }
    }
    // 检查用户输入的值
    checkValue(uidInput, tagsInput) {
        const tags = _utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.string2array(tagsInput);
        if (!uidInput || !tagsInput || tags.length === 0) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_8__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_必填项不能为空'));
            return false;
        }
        const uid = Number.parseInt(uidInput);
        if (!uid || isNaN(uid)) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_8__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_用户ID必须是数字'));
            return false;
        }
        return {
            uid,
            tags,
        };
    }
    findIndex(uid) {
        return this.rules.findIndex((rule) => rule.uid === uid);
    }
    // 添加规则
    addRule(uidInput, tagsInput) {
        const check = this.checkValue(uidInput, tagsInput);
        if (!check) {
            return;
        }
        const { uid, tags } = check;
        // 查找这个用户是否已经被添加过，如果添加过，则改为更新，而不是添加新规则
        const index = this.findIndex(uid);
        if (index > -1) {
            // 把两次的 tag 合并起来
            const joinTags = this.rules[index].tags.concat(tags);
            return this.updateRule(uid, uid.toString(), joinTags.toString());
        }
        this.addWrapShow = false;
        this.rules.push(check);
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserList', [...this.rules]);
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserShowList', true);
        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_添加成功'));
    }
    // 更新规则
    // tip 表示是否用消息框进行提示。当用户点击了更新按钮时应该显示提示；输入内容变化导致的自动更新可以不显示提示
    updateRule(oldUid, uidInput, tagsInput, tip = true) {
        const check = this.checkValue(uidInput, tagsInput);
        if (!check) {
            return;
        }
        const listElement = this.listWrap.querySelector(`.settingItem[data-key='${oldUid}']`);
        listElement?.remove();
        const index = this.findIndex(oldUid);
        this.rules[index] = check;
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserList', [...this.rules]);
        if (tip) {
            _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_更新成功'));
        }
        this.addWrapShow = false;
    }
    // 删除规则
    deleteRule(uid) {
        const index = this.findIndex(uid);
        this.rules.splice(index, 1);
        (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('blockTagsForSpecificUserList', [...this.rules]);
        const listElement = this.listWrap.querySelector(`.settingItem[data-key='${uid}']`);
        listElement?.remove();
    }
    // 如果找到了符合的记录，则返回 true
    check(uid, tags) {
        if (typeof uid === 'string') {
            uid = Number.parseInt(uid);
        }
        // 查找有无记录
        const index = this.findIndex(uid);
        if (index === -1) {
            return false;
        }
        // 如果有记录则判断是否有相同的 tag，有任意一个就返回
        const rule = this.rules[index];
        const tagsString = tags.toString().toLowerCase();
        for (const tag of rule.tags) {
            if (tagsString.includes(tag.toLowerCase())) {
                return true;
            }
        }
        // 没有相同的 tag
        return false;
    }
}
const blockTagsForSpecificUser = new BlockTagsForSpecificUser();



/***/ }),

/***/ "./src/ts/filter/Filter.ts":
/*!*********************************!*\
  !*** ./src/ts/filter/Filter.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   filter: () => (/* binding */ filter)
/* harmony export */ });
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _BlackandWhiteImage__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./BlackandWhiteImage */ "./src/ts/filter/BlackandWhiteImage.ts");
/* harmony import */ var _Mute__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./Mute */ "./src/ts/filter/Mute.ts");
/* harmony import */ var _BlockTagsForSpecificUser__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./BlockTagsForSpecificUser */ "./src/ts/filter/BlockTagsForSpecificUser.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _WorkPublishTime__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./WorkPublishTime */ "./src/ts/filter/WorkPublishTime.ts");










// 检查作品是否符合过滤条件
class Filter {
    constructor() {
        // 检查收藏数要求
        this.oneDayTime = 24 * 60 * 60 * 1000; // 一天的毫秒数
        this.minimumTime = 4 * 60 * 60 * 1000; // 检查日均收藏数量时，要求作品发表之后经过的时间大于这个值。因为发表之后经过时间很短的作品，其日均收藏数量非常不可靠，所以对于小于这个值的作品不进行日均收藏数量的检查。
        // 检查文件体积
        this.MiB = 1024 * 1024;
        this.bindEvents();
    }
    // 对启用了的过滤选项输出提示
    showTip() {
        this.getDownType();
        this.getDownTypeByAge();
        this.getAIWorkType();
        this.getDownTypeByImgCount();
        this.getDownTypeByColor();
        this.getDownTypeByBmked();
        this.getMultiImageWorkImageLimit();
        this.getBMKNum();
        this.getSetWh();
        this.getRatio();
        this.getIdRange();
        this.getPostDate();
        this.getIncludeTag();
        this.getExcludeTag();
        if (_store_States__WEBPACK_IMPORTED_MODULE_3__.states.debut) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_抓取首次登场的作品Title'));
        }
        this.getBlockList();
        this.getSize();
    }
    /**检查作品是否符合过滤器的要求，返回值 false 表示作品不符合要求，true 表示符合要求 */
    // 注意：这是一个异步函数，所以要使用 await 获取检查结果
    // 想要检查哪些数据就传递哪些数据，不需要传递 FilterOption 的所有选项
    // 每个过滤器函数里都必须检查参数为 undefined 的情况
    // 每个过滤器函数必须返回一个 boolean 值，true 表示保留这个作品，false 表示排除这个作品
    async check(option) {
        // 检查下载的作品类型设置
        if (!this.checkDownType(option.workType)) {
            return false;
        }
        if (!this.checkDownTypeByAge(option.xRestrict)) {
            return false;
        }
        if (!this.checkAIWorkType(option.aiType)) {
            return false;
        }
        // 检查单图、多图的下载
        if (!this.checkPageCount(option.workType, option.pageCount)) {
            return false;
        }
        // 检查单图、多图的下载
        if (!this.checkMultiImageWorkImageLimit(option.workType, option.pageCount)) {
            return false;
        }
        // 检查收藏和未收藏的要求
        if (!this.checkDownTypeByBmked(option.bookmarkData)) {
            return false;
        }
        // 检查收藏数要求
        if (!this.checkBMK(option.bookmarkCount, option.createDate)) {
            return false;
        }
        // 检查要排除的 tag
        if (!this.checkExcludeTag(option.tags)) {
            return false;
        }
        // 检查必须包含的 tag
        if (!this.checkIncludeTag(option.tags)) {
            return false;
        }
        // 检查宽高设置
        if (!this.checkWidthHeight(option.width, option.height)) {
            return false;
        }
        // 检查宽高比设置
        if (!this.checkRatio(option.width, option.height)) {
            return false;
        }
        // 检查 id 范围设置
        if (!this.checkIdRange(option.id)) {
            return false;
        }
        // 检查用户在 Pixiv 的屏蔽设定
        if (!(await this.checkMuteUser(option.userId))) {
            return false;
        }
        if (!(await this.checkMuteTag(option.tags))) {
            return false;
        }
        // 检查用户阻止名单
        if (!this.checkBlockList(option.userId)) {
            return false;
        }
        // 检查针对特定用户屏蔽的 tags
        if (!this.checkBlockTagsForSpecificUser(option.userId, option.tags)) {
            return false;
        }
        // 检查投稿时间设置
        if (!this.checkPostDate(option.createDate)) {
            return false;
        }
        // 检查投稿时间设置
        if (!this.checkIdPublishTime(option.id, option.workTypeString)) {
            return false;
        }
        // 检查首次登场设置
        if (!this.checkDebut(option.yes_rank)) {
            return false;
        }
        // 检查文件体积设置
        if (!this.checkSize(option.size)) {
            return false;
        }
        // 检查黑白图片
        // 这一步需要加载图片，需要较长的时间，较多的资源占用，所以放到最后检查
        if (!(await this.checkBlackWhite(option.mini))) {
            return false;
        }
        return true;
    }
    // ---------------- get ----------------
    // 提示下载的作品类型设置
    getDownType() {
        // 如果全部排除则取消任务
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType0 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType1 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType2 &&
            !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType3) {
            this.showWarning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除了所有作品类型'));
        }
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType0 && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_插画'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType1 && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_漫画'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType2 && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_动图'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType3 && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_小说'));
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    getDownTypeByAge() {
        // 如果全部排除则取消任务
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downAllAges && !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18 && !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18G) {
            this.showWarning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除了所有作品类型'));
        }
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downAllAges && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_全年龄'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18 && tips.push('R-18');
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18G && tips.push('R-18G');
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    getAIWorkType() {
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.AIGenerated && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_AI生成'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notAIGenerated && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_非AI生成'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.UnknownAI && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_未知') + '(AI)');
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    getDownTypeByImgCount() {
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downSingleImg && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_单图作品'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downMultiImg && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_多图作品'));
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    // 提示图像颜色设置
    getDownTypeByColor() {
        // 如果全部排除则取消任务
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downColorImg && !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBlackWhiteImg) {
            this.showWarning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除了所有作品类型'));
        }
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downColorImg && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_彩色图片'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBlackWhiteImg && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_黑白图片'));
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    // 提示下载收藏和未收藏作品的设置
    getDownTypeByBmked() {
        // 如果全部排除则取消任务
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downNotBookmarked && !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBookmarked) {
            this.showWarning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除了所有作品类型'));
        }
        const tips = [];
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downNotBookmarked && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_未收藏'));
        !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBookmarked && tips.push(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_已收藏'));
        if (tips.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_排除作品类型') + tips.join(', '));
        }
    }
    // 提示多图作品的图片数量限制
    getMultiImageWorkImageLimit() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimitSwitch) {
            return;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimit > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_多图作品的图片数量上限') +
                '：' +
                _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimit);
        }
    }
    // 提示必须包含的tag
    getIncludeTag() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTagSwitch) {
            return;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTag.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_设置了必须tag之后的提示') + _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTag.toString());
        }
    }
    // 提示要排除的tag
    getExcludeTag() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTagSwitch) {
            return;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTag.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_设置了排除tag之后的提示') + _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTag.toString());
        }
    }
    // 提示宽高设置
    getSetWh() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWHSwitch) {
            return;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidth || _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setHeight) {
            const andOr = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidthAndOr
                .replace('|', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_或者'))
                .replace('&', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_并且'));
            const text = `${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_宽度')} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.widthHeightLimit} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidth} ${andOr} ${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_高度')} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.widthHeightLimit} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setHeight}`;
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(text);
        }
    }
    // 提示输入的收藏数
    getBMKNum() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumSwitch) {
            return;
        }
        const min = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumMin;
        const max = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumMax;
        const average = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumAverage;
        if (min >= 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_收藏数大于') + min);
        }
        if (max >= 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_收藏数小于') + max);
        }
        if (average >= 0 && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumAverageSwitch) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(`${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_日均收藏数量')} >= ${average}`);
        }
    }
    // 提示宽高比设置
    getRatio() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.ratioSwitch) {
            return;
        }
        switch (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.ratio) {
            case 'square':
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_设置了宽高比之后的提示', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_正方形')));
                break;
            case 'horizontal':
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_设置了宽高比之后的提示', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_横图')));
                break;
            case 'vertical':
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_设置了宽高比之后的提示', _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_竖图')));
                break;
            case 'userSet':
                _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_宽高比') +
                    ` ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatioLimit} ` +
                    _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatio);
                break;
        }
    }
    // 提示 id 范围设置
    getIdRange() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRangeSwitch) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(`id ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRange} ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRangeInput}`);
    }
    // 提示投稿时间设置
    getPostDate() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDate) {
            return;
        }
        if (isNaN(_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart) || isNaN(_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart)) {
            const msg = _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_日期时间格式错误');
            this.showWarning(msg);
        }
        else {
            const start = new Date(_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart).toLocaleString();
            const end = new Date(_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd).toLocaleString();
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(`${_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_时间范围')}: ${start} - ${end}`);
        }
    }
    // 提示文件体积设置
    getSize() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeSwitch) {
            return;
        }
        _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(`Size: ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeMin}MiB - ${_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeMax}MiB`);
    }
    getBlockList() {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userBlockList) {
            return;
        }
        for (const uid of _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockList) {
            if (isNaN(Number.parseInt(uid))) {
                return this.showWarning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_用户ID必须是数字'));
            }
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockList.length > 0) {
            _Log__WEBPACK_IMPORTED_MODULE_1__.log.warning(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_用户阻止名单') + ': ' + _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockList.toString());
        }
    }
    // ---------------- check ----------------
    // 检查下载的作品类型设置
    checkDownType(workType) {
        switch (workType) {
            case -1:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType0 || _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType1 || _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType2;
            case 0:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType0;
            case 1:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType1;
            case 2:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType2;
            case 3:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downType3;
            default:
                return true;
        }
    }
    checkDownTypeByAge(xRestrict) {
        switch (xRestrict) {
            case 0:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downAllAges;
            case 1:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18;
            case 2:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downR18G;
            default:
                return true;
        }
    }
    checkAIWorkType(aiType) {
        switch (aiType) {
            case 0:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.UnknownAI;
            case 1:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notAIGenerated;
            case 2:
                return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.AIGenerated;
            default:
                return true;
        }
    }
    // 检查多图作品的图片数量限制
    checkMultiImageWorkImageLimit(workType, pageCount) {
        // 此过滤条件只检查插画和漫画，只对多图作品生效。如果图片数量小于 2 则不检查
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimitSwitch ||
            _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimit < 1 ||
            pageCount === undefined ||
            pageCount < 2 ||
            (workType !== 0 && workType !== 1)) {
            return true;
        }
        return pageCount <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.multiImageWorkImageLimit;
    }
    // 依据图片数量，检查下载的作品类型
    checkPageCount(workType, pageCount) {
        if (workType === undefined || pageCount === undefined) {
            return true;
        }
        // 将动图视为单图
        if (workType === 2) {
            pageCount = 1;
        }
        if (pageCount === 1) {
            return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downSingleImg;
        }
        if (pageCount > 1) {
            return _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downMultiImg;
        }
        return false;
    }
    // 检查过滤黑白图像设置
    async checkBlackWhite(imgUrl) {
        // 如果没有图片网址，或者没有排除任何一个选项，则不检查
        if (!imgUrl || (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downColorImg && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBlackWhiteImg)) {
            return true;
        }
        // result 为 true，表示它是黑白图片，false 是彩色图片
        const result = await _BlackandWhiteImage__WEBPACK_IMPORTED_MODULE_5__.blackAndWhiteImage.check(imgUrl);
        return ((result && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBlackWhiteImg) ||
            (!result && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downColorImg));
    }
    // 检查作品是否符合已收藏、未收藏作品的设置
    checkDownTypeByBmked(bookmarked) {
        // 如果没有参数，或者都没有排除
        if (bookmarked === undefined) {
            return true;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downNotBookmarked && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBookmarked) {
            return true;
        }
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downNotBookmarked && _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBookmarked) {
            // 只下载已收藏
            return !!bookmarked;
        }
        else if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downNotBookmarked && !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.downBookmarked) {
            // 只下载未收藏
            return !bookmarked;
        }
        return false;
    }
    checkBMK(bmk, date) {
        if (bmk === undefined || !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumSwitch) {
            return true;
        }
        // 检查收藏数量是否达到设置的最大值、最小值范围
        const checkNumber = bmk >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumMin && bmk <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumMax;
        // 如果没有设置检查日均收藏，就直接返回收藏数量的检查结果
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumAverageSwitch || date === undefined) {
            return checkNumber;
        }
        // 检查日均收藏
        const createTime = new Date(date).getTime();
        const nowTime = new Date().getTime();
        // 如果作品发表时间太短（小于 4 小时）
        if (nowTime - createTime < this.minimumTime) {
            // 如果 4 小时里的收藏数量已经达到要求，则保留这个作品
            // 如果 4 小时里的收藏数量没有达到要求，则不检查继续它的日均收藏数量，返回收藏数量的检查结果
            return bmk >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumAverage ? true : checkNumber;
        }
        const day = (nowTime - createTime) / this.oneDayTime; // 计算作品发表以来的天数
        const average = bmk / day;
        const checkAverage = average >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.BMKNumAverage;
        // 返回结果。收藏数量和日均收藏并不互斥，两者只要有一个满足条件就会保留这个作品
        return checkNumber || checkAverage;
    }
    // 检查作品是否符合包含 tag 的条件。返回值表示是否保留这个作品。
    checkIncludeTag(tags) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTagSwitch ||
            _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTag.length === 0 ||
            tags === undefined) {
            return true;
        }
        let result = false;
        // 把设置的包含的 tag 转换成小写，生成数组
        const needTags = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTag.map((val) => {
            return val.toLowerCase();
        });
        // 如果设置了必须的 tag
        if (needTags.length > 0) {
            // 把处理的 tag 变成小写，并且去重
            // 如果不区分大小写的话，Fate/grandorder 和 Fate/GrandOrder 会被算作符合两个 tag，所以用 Set 结构去重。测试 id 51811780
            const workTags = new Set();
            for (const tag of tags) {
                workTags.add(tag.toLowerCase());
            }
            // 全部包含
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.needTagMode === 'all') {
                let tagNeedMatched = 0;
                for (const tag of workTags) {
                    for (const need of needTags) {
                        if (tag === need) {
                            tagNeedMatched++;
                            break;
                        }
                    }
                }
                // 如果全部匹配
                if (tagNeedMatched >= needTags.length) {
                    result = true;
                }
            }
            else {
                // 包含任意一个
                for (const tag of workTags.values()) {
                    if (needTags.includes(tag)) {
                        result = true;
                        break;
                    }
                }
            }
        }
        else {
            result = true;
        }
        return result;
    }
    // 检查作品是否符合排除 tag 的条件, 只要作品包含其中一个就排除。返回值表示是否保留这个作品。
    checkExcludeTag(tags) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTagSwitch ||
            _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTag.length === 0 ||
            tags === undefined) {
            return true;
        }
        const notNeedTags = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.notNeedTag.map((str) => str.toLowerCase());
        for (const tag of tags) {
            for (const notNeed of notNeedTags) {
                // 部分匹配
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.tagMatchMode === 'partial') {
                    if (tag.toLowerCase().includes(notNeed)) {
                        // 如果检查到了排除的 tag，进行复查
                        // 使用空格对 tag 进行分词，尝试提高准确率
                        // 例如：用户本意是排除腐向作品（bl），但是如果作品的 tag 是 Strike the Blood 或者 Blue Poison 都会导致作品被排除。这是错误的。
                        // 所以在有分词的情况下，应当对分词进行全等匹配以提高准确度
                        const words = tag.split(' ');
                        if (words.length > 1) {
                            // 如果 tag 有空格，依次使用每个分词进行全词匹配。如果有任一一个 tag 被匹配到则排除这个作品
                            if (words.some((word) => word.toLowerCase() === notNeed)) {
                                return false;
                            }
                        }
                        else {
                            // 如果 tag 没有空格，直接返回结果
                            return false;
                        }
                    }
                }
                else {
                    // 全词匹配
                    if (tag.toLowerCase() === notNeed) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    // 检查作品是否符合过滤宽高的条件
    checkWidthHeight(width, height) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWHSwitch ||
            width === undefined ||
            height === undefined ||
            width === 0 ||
            height === 0) {
            return true;
        }
        const setWidth = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidth;
        const setHeight = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setHeight;
        // 未设置宽高，或者设置的宽高都不合法
        if (setWidth === 0 && setHeight === 0) {
            return true;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.widthHeightLimit === '>=') {
            // 大于等于
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidthAndOr === '&') {
                return width >= setWidth && height >= setHeight;
            }
            else {
                return width >= setWidth || height >= setHeight;
            }
        }
        else if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.widthHeightLimit === '<=') {
            // 小于等于
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidthAndOr === '&') {
                return width <= setWidth && height <= setHeight;
            }
            else {
                return width <= setWidth || height <= setHeight;
            }
        }
        else {
            // 精确等于
            if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.setWidthAndOr === '&') {
                return width === setWidth && height === setHeight;
            }
            else {
                return width === setWidth || height === setHeight;
            }
        }
    }
    // 检查作品是否符合宽高比条件
    checkRatio(width, height) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.ratioSwitch ||
            width === undefined ||
            height === undefined ||
            width === 0 ||
            height === 0) {
            return true;
        }
        switch (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.ratio) {
            case 'square':
                return width === height;
            case 'horizontal':
                return width / height > 1;
            case 'vertical':
                return width / height < 1;
            case 'userSet':
                switch (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatioLimit) {
                    case '>=':
                        return width / height >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatio;
                    case '=':
                        return width / height === _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatio;
                    case '<=':
                        return width / height <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userRatio;
                }
        }
    }
    // 检查 id 范围设置
    checkIdRange(id) {
        if (id === undefined || !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRangeSwitch) {
            return true;
        }
        const setId = _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRangeInput;
        let nowId;
        if (typeof id !== 'number') {
            nowId = parseInt(id);
        }
        else {
            nowId = id;
        }
        if (_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.idRange === '>') {
            return nowId > setId;
        }
        else {
            return nowId < setId;
        }
    }
    // 检查投稿时间设置
    checkPostDate(date) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDate || date === undefined) {
            return true;
        }
        const time = new Date(date).getTime();
        return time >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart && time <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd;
    }
    checkIdPublishTime(id, type) {
        if (id === undefined || !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDate || !type) {
            return true;
        }
        const _id = Number.parseInt(id);
        const _type = type === 'novels' ? 'novels' : 'illusts';
        const range = _WorkPublishTime__WEBPACK_IMPORTED_MODULE_9__.workPublishTime.getTimeRange(_id, _type);
        // console.log(new Date(range[0]).toLocaleString())
        // console.log(new Date(range[1]).toLocaleString())
        // 如果返回的数据中的开始时间大于用户设置的结束时间，则检查不通过
        // 如果返回的数据中的结束时间小于用户设置的开始时间，则检查不通过
        if (range[0] > _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd || range[1] < _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart) {
            return false;
        }
        // 如果两条记录的时间差大于用户设置的时间差，此时的数据不可采信。将其通过
        if (range[1] - range[0] >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd - _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart) {
            return true;
        }
        // 如果两条记录的时间范围与用户设置的时间范围只有部分重叠，此时的数据不可采信。将其通过
        if (range[0] < _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart &&
            range[1] > _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart &&
            range[1] < _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd) {
            return true;
        }
        if (range[0] > _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart &&
            range[0] < _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd &&
            range[1] > _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd) {
            return true;
        }
        // 达到这里的数据是可信的，不会发生误判
        return (range[0] >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateStart && range[1] <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.postDateEnd);
    }
    // 检查首次登场设置
    // yes_rank 是昨日排名，如果为 0，则此作品是“首次登场”的作品
    checkDebut(yes_rank) {
        if (!_store_States__WEBPACK_IMPORTED_MODULE_3__.states.debut || yes_rank === undefined) {
            return true;
        }
        return yes_rank === 0;
    }
    checkBlockList(userId) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.userBlockList || userId === undefined) {
            return true;
        }
        // 如果阻止名单里有这个用户 id，则返回 false 表示阻止这个作品
        return !_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockList.includes(userId);
    }
    checkSize(size) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeSwitch || size === undefined) {
            return true;
        }
        return (size >= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeMin * this.MiB && size <= _setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.sizeMax * this.MiB);
    }
    async checkMuteUser(userId) {
        if (userId === undefined) {
            return true;
        }
        return !(await _Mute__WEBPACK_IMPORTED_MODULE_6__.mute.checkUser(userId));
    }
    async checkMuteTag(tags) {
        if (tags === undefined) {
            return true;
        }
        // 一旦检查到某个 tag 存在于 mute 列表里，就排除这个作品
        for (const tag of tags) {
            if (await _Mute__WEBPACK_IMPORTED_MODULE_6__.mute.checkTag(tag)) {
                return false;
            }
        }
        return true;
        // return !(tags.some((mute.checkTag.bind(mute))))
    }
    checkBlockTagsForSpecificUser(userId, tags) {
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.blockTagsForSpecificUser ||
            userId === undefined ||
            tags === undefined) {
            return true;
        }
        // 对结果取反
        return !_BlockTagsForSpecificUser__WEBPACK_IMPORTED_MODULE_7__.blockTagsForSpecificUser.check(userId, tags);
    }
    // 如果设置项的值不合法，则显示提示
    showWarning(msg) {
        _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.fire('wrongSetting');
        _MsgBox__WEBPACK_IMPORTED_MODULE_8__.msgBox.error(msg);
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.crawlStart, () => {
            this.showTip();
        });
    }
}
const filter = new Filter();



/***/ }),

/***/ "./src/ts/filter/Mute.ts":
/*!*******************************!*\
  !*** ./src/ts/filter/Mute.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   mute: () => (/* binding */ mute)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");




// 获取用户在 Pixiv 里屏蔽的用户和/或 tag，进行过滤
class Mute {
    constructor() {
        this.userList = [];
        this.tagList = [];
        this.got = false; // 是否获取过用户设置
    }
    /**检查传入的 user id 是否包含在用户屏蔽设置里 */
    async checkUser(id) {
        // 检查时，要求至少进行过一次获取用户设置的操作
        if (!this.got) {
            await this.getMuteSettings();
        }
        if (typeof id === 'number') {
            id = id.toString();
        }
        return this.userList.includes(id);
    }
    /**检查传入的 tag 是否包含在用户屏蔽设置里 */
    async checkTag(tag) {
        if (!this.got) {
            await this.getMuteSettings();
        }
        return this.tagList.includes(tag);
    }
    // 此模块不会在初始化时获取设置，这是为了避免增加一次无谓的网络请求
    // 当执行此模块的 check 方法时，如果没有获取过设置，则此模块会主动获取一次设置
    // 其他模块也可以在必要的时候（如开始抓取时）直接执行此方法，预先获取设置，为后面的流程做准备
    async getMuteSettings() {
        this.userList = [];
        this.tagList = [];
        return new Promise(async (resolve, reject) => {
            if (_Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.checkUserLogin() === false) {
                return resolve(401);
            }
            try {
                const response = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getMuteSettings();
                const items = response.body.mute_items;
                for (const item of items) {
                    // 如果这个屏蔽项未启用，则不保存
                    if (item.enabled === false) {
                        continue;
                    }
                    if (item.type === 'user') {
                        this.userList.push(item.value);
                    }
                    if (item.type === 'tag') {
                        this.tagList.push(item.value);
                    }
                }
                if (this.userList.length > 0 || this.tagList.length > 0) {
                    let msg = _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_屏蔽设定') + ': <br>';
                    if (this.userList.length > 0) {
                        msg += `Users: ${this.userList.join(',')}<br>`;
                    }
                    if (this.tagList.length > 0) {
                        msg += `Tags: ${this.tagList.join(',')}<br>`;
                    }
                    _Log__WEBPACK_IMPORTED_MODULE_2__.log.warning(msg);
                }
                this.got = true;
                return resolve(items);
            }
            catch (error) {
                // 当请求出错时，视为获取完成。不抛出 reject，否则会导致抓取中止
                this.got = true;
                if (error.status === 401) {
                    console.error('get mute settings error ' + _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_作品页状态码401'));
                }
                return resolve(error.status);
            }
        });
    }
}
const mute = new Mute();



/***/ }),

/***/ "./src/ts/filter/WorkPublishTime.ts":
/*!******************************************!*\
  !*** ./src/ts/filter/WorkPublishTime.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   workPublishTime: () => (/* binding */ workPublishTime)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../utils/SecretSignal */ "./src/ts/utils/SecretSignal.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_workPublishTimeIllusts__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../store/workPublishTimeIllusts */ "./src/ts/store/workPublishTimeIllusts.ts");
/* harmony import */ var _store_WorkPublishTimeNovels__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/WorkPublishTimeNovels */ "./src/ts/store/WorkPublishTimeNovels.ts");





// 获取指定 id 的发布时间范围
class WorkPublishTime {
    constructor() {
        // 数据源是二维数组，里面的每一项都是一个由作品 id 和作品发布时间组成的子数组。如：
        // [[20, 1189343647000], [10000, 1190285376000], [20006, 1190613767000]]
        /**每隔 10000 个作品采集一次数据 */
        this.gap = 10000;
        this.illustsLength = 0;
        this.novelsLength = 0;
        this.illustsLength = _store_workPublishTimeIllusts__WEBPACK_IMPORTED_MODULE_3__.illustsData.length;
        this.novelsLength = _store_WorkPublishTimeNovels__WEBPACK_IMPORTED_MODULE_4__.novelData.length;
        this.bindEvents();
    }
    /**获取作品的发布时间范围。
     *
     * 返回值是一个包含 2 个数字的数组，第一个数字是开始时间，第二个数字是结束时间。 */
    getTimeRange(id, type = 'illusts') {
        const data = type === 'illusts' ? _store_workPublishTimeIllusts__WEBPACK_IMPORTED_MODULE_3__.illustsData : _store_WorkPublishTimeNovels__WEBPACK_IMPORTED_MODULE_4__.novelData;
        const length = type === 'illusts' ? this.illustsLength : this.novelsLength;
        const index = Math.floor(id / this.gap);
        // 如果传入的 id 匹配到最后一条记录，则将结束时间设置为现在
        if (index >= length - 1) {
            return [data[length - 1][1], new Date().getTime()];
        }
        // 如果传入的 id 匹配到第一条记录，则直接返回数据
        if (index === 0) {
            return [data[0][1], data[1][1]];
        }
        const record = data[index];
        // 如果有与传入 id 相匹配的记录，则判断这个记录的 id 与传入的 id 哪个大
        // 如果记录的 id 等于传入的 id，则直接返回其时间戳
        if (record[0] === id) {
            return [record[1], record[1]];
        }
        else if (record[0] < id) {
            // 如果记录的 id 小于传入的 id，则此记录的时间作为开始时间，下一条记录的时间作为结束时间
            // 此时必然有下一条记录，因为前面已经处理了没有下一条记录的情况
            return [record[1], data[index + 1][1]];
        }
        else {
            // 如果记录的 id 大于传入的 id，则此记录的时间作为结束时间，上一条记录的时间作为开始时间
            // 此时必然有上一条记录，因为前面已经处理了没有上一条记录的情况
            return [data[index - 1][1], record[1]];
        }
    }
    bindEvents() {
        _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_1__.secretSignal.register('ppdtask1', () => {
            // 上次记录到 113760000
            this.crawlData(113370000, 113769980);
        });
        _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_1__.secretSignal.register('ppdtask2', () => {
            // 上次记录到 21100000
            this.crawlData(21010000, 21103388, 'novels');
        });
    }
    async crawlData(start, end, type = 'illusts') {
        console.log('start crawl time data');
        const result = [];
        const min_illust = 20; // 最早的插画作品
        const min_novel = 129; // 最早的小说作品
        let id = start;
        if (type === 'illusts' && start < min_illust) {
            id = min_illust;
        }
        if (type === 'novels' && start < min_novel) {
            id = min_novel;
        }
        while (id < end) {
            const data = await this.crawlWork(id, type);
            result.push(data);
            // 使用下一个接近 10000 倍数的 id 进行下一次抓取
            id = (Math.floor(data[0] / this.gap) + 1) * this.gap;
        }
        console.log(result);
        console.log('crawl time data complete');
        const resultList = await _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.json2BlobSafe(result);
        for (const result of resultList) {
            _utils_Utils__WEBPACK_IMPORTED_MODULE_2__.Utils.downloadFile(result.url, `workPublishTime-${type}-${start}-${end}.json`);
        }
        return result;
    }
    // 获取指定作品的发布时间
    // 如果抓取出错（如 404 错误），则顺延到下一个作品 id 重试抓取
    async crawlWork(id, type = 'illusts') {
        return new Promise(async (resolve) => {
            // 为了避免出现 429 错误，每次抓取之间设置了间隔时间
            window.setTimeout(async () => {
                try {
                    const data = await _API__WEBPACK_IMPORTED_MODULE_0__.API[type === 'illusts' ? 'getArtworkData' : 'getNovelData'](id.toString());
                    if (data.error === false) {
                        const dateStr = data.body.createDate;
                        if (!dateStr) {
                            return resolve(this.crawlWork(++id, type));
                        }
                        const time = new Date(dateStr).getTime();
                        return resolve([id, time]);
                    }
                    else {
                        return resolve(this.crawlWork(++id, type));
                    }
                }
                catch (error) {
                    return resolve(this.crawlWork(++id, type));
                }
            }, 1700);
        });
    }
}
const workPublishTime = new WorkPublishTime();



/***/ }),

/***/ "./src/ts/output/OutputPanel.ts":
/*!**************************************!*\
  !*** ./src/ts/output/OutputPanel.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _CopyToClipboard__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../CopyToClipboard */ "./src/ts/CopyToClipboard.ts");









// 输出面板
class OutputPanel {
    constructor() {
        this.addOutPutPanel();
        _Theme__WEBPACK_IMPORTED_MODULE_5__.theme.register(this.outputPanel);
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.register(this.outputPanel);
        this.bindEvents();
    }
    bindEvents() {
        this.closeBtn.addEventListener('click', () => {
            this.close();
        });
        this.outputPanel.addEventListener('click', (e) => {
            const ev = e || window.event;
            ev.stopPropagation();
        });
        document.addEventListener('click', () => {
            if (this.outputPanel.style.display !== 'none') {
                this.close();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.closeCenterPanel, () => {
            this.close();
        });
        // 复制输出内容
        this.copyBtn.addEventListener('click', () => {
            const text = this.outputContent.innerText.replaceAll('\n\n', '\n');
            _CopyToClipboard__WEBPACK_IMPORTED_MODULE_8__.CopyToClipboard.setClipboard(text);
            window.setTimeout(() => {
                this.close();
            }, 100);
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.output, (ev) => {
            this.output(ev.detail.data);
        });
    }
    addOutPutPanel() {
        const html = `
    <div class="outputWrap">
    <div class="outputClose" data-xztitle="_关闭">×</div>
    <div class="outputTitle" data-xztext="_输出信息"></div>
    <div class="outputContent beautify_scrollbar"></div>
    <div class="outputFooter">
    <button class="outputCopy" data-xztext="_复制"></button>
    </div>
    </div>
    `;
        document.body.insertAdjacentHTML('beforebegin', html);
        this.outputPanel = document.querySelector('.outputWrap');
        if (_Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile) {
            this.outputPanel.classList.add('mobile');
        }
        this.outputTitle = this.outputPanel.querySelector('.outputTitle');
        this.outputContent = this.outputPanel.querySelector('.outputContent');
        this.copyBtn = this.outputPanel.querySelector('.outputCopy');
        this.closeBtn = this.outputPanel.querySelector('.outputClose');
    }
    // 输出内容
    output(data) {
        if (!data.content) {
            return _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
        }
        if (_store_Store__WEBPACK_IMPORTED_MODULE_2__.store.result.length < _Config__WEBPACK_IMPORTED_MODULE_4__.Config.outputMax) {
            this.copyBtn.disabled = false;
            _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.outputTitle, data.title);
            this.outputContent.innerHTML = data.content;
            this.outputPanel.style.display = 'block';
        }
        else {
            // 如果结果较多，则不直接输出，改为保存 txt 文件
            const file = new Blob([data.content], {
                type: 'text/plain',
            });
            const url = URL.createObjectURL(file);
            const fileName = `Output-${new Date().toLocaleString()}.txt`;
            _utils_Utils__WEBPACK_IMPORTED_MODULE_3__.Utils.downloadFile(url, fileName);
            this.copyBtn.disabled = true;
            _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.warning(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_输出内容太多已经为你保存到文件'));
        }
    }
    // 关闭输出面板
    close() {
        this.outputPanel.style.display = 'none';
        this.outputContent.innerHTML = '';
        _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.outputTitle, '_输出信息');
    }
}
new OutputPanel();


/***/ }),

/***/ "./src/ts/output/PreviewFileName.ts":
/*!******************************************!*\
  !*** ./src/ts/output/PreviewFileName.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _FileName__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../FileName */ "./src/ts/FileName.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");








// 预览文件名
class PreviewFileName {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.previewFileName, () => {
            this.previewFileName();
        });
        const prevBtn = _Tools__WEBPACK_IMPORTED_MODULE_6__.Tools.addBtn('namingBtns', _Colors__WEBPACK_IMPORTED_MODULE_7__.Colors.bgGreen, '_预览文件名');
        prevBtn.addEventListener('click', () => {
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('previewFileName');
        }, false);
        prevBtn.style.display = 'none';
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.crawlStart, () => {
            prevBtn.style.display = 'none';
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.readyDownload, () => {
            prevBtn.style.display = 'flex';
        });
    }
    previewFileName() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length === 0) {
            return _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_没有可用的抓取结果'));
        }
        // 使用数组储存和拼接字符串，提高性能
        const resultArr = [];
        let result = '';
        const length = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length;
        if (length < _Config__WEBPACK_IMPORTED_MODULE_4__.Config.outputMax) {
            // 输出结果，添加 html 标签
            for (let i = 0; i < length; i++) {
                const data = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result[i];
                // 生成文件名，并为文件名添加颜色显示
                // 只有当文件数量少于限制值时才添加颜色。这是因为添加颜色会导致生成的 HTML 元素数量增多，渲染和复制时的资源占用增多
                const part = _FileName__WEBPACK_IMPORTED_MODULE_2__.fileName.createFileName(data).split('/');
                const length = part.length;
                for (let i = 0; i < length; i++) {
                    const str = part[i];
                    if (i < length - 1) {
                        // 如果不是最后一项，说明是文件夹名，添加特定的颜色
                        part[i] = `<span class="colorFolder">${str}</span>`;
                    }
                    else {
                        // 最后一项是文件名，设置为黑色
                        part[i] = `<span class="color000">${str}</span>`;
                    }
                }
                const fullNameHtml = part.join('/');
                if (data.type !== 3) {
                    // 对于图片作品，在文件名前面显示默认文件名
                    // 默认文件名有两种选择，一种是使用 url 里的文件名，一种是使用 data.id。这里使用前者，方便用户用其他下载软件下载后，复制输出的内容制作重命名脚本
                    const defaultName = data.original.replace(/.*\//, '');
                    const nowResult = `<p class="result"><span class="color999">${defaultName}</span>: ${fullNameHtml}</p>`;
                    resultArr.push(nowResult);
                }
                else {
                    // 小说作品不显示原文件名（因为没有此数据）
                    const nowResult = `<p class="result">${fullNameHtml}</p>`;
                    resultArr.push(nowResult);
                }
            }
            result = resultArr.join('');
        }
        else {
            // 不生成 html 标签，只生成纯文本，保存为 txt 文件
            for (let i = 0; i < length; i++) {
                const data = _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result[i];
                const fullName = _FileName__WEBPACK_IMPORTED_MODULE_2__.fileName.createFileName(data);
                if (data.type !== 3) {
                    // 图片作品，在文件名前面显示文件 url 里的文件名
                    let defaultName = data.original.replace(/.*\//, '');
                    resultArr.push(`${defaultName}: ${fullName}`);
                }
                else {
                    // 小说作品不显示原文件名（因为没有此数据）
                    resultArr.push(fullName);
                }
            }
            result = resultArr.join('\n');
        }
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('output', {
            content: result,
            title: '_预览文件名',
        });
    }
}
new PreviewFileName();


/***/ }),

/***/ "./src/ts/output/ShowURLs.ts":
/*!***********************************!*\
  !*** ./src/ts/output/ShowURLs.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");






// 显示 url
class ShowURLs {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.showURLs, () => {
            this.showURLs();
        });
    }
    showURLs() {
        const urls = [];
        const size = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.imageSize;
        for (const data of _store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result) {
            // 只输出图片文件的 url
            // 小说文件没有固定的 url 所以不输出
            if (data.type !== 3) {
                urls.push(data[size]);
            }
        }
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length === 0 || urls.length === 0) {
            return _Toast__WEBPACK_IMPORTED_MODULE_4__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_没有可用的抓取结果'));
        }
        let result = '';
        if (_store_Store__WEBPACK_IMPORTED_MODULE_0__.store.result.length < _Config__WEBPACK_IMPORTED_MODULE_5__.Config.outputMax) {
            result = urls.join('<br>');
        }
        else {
            result = urls.join('\n');
        }
        _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.fire('output', {
            content: result,
            title: '_复制url',
        });
    }
}
new ShowURLs();


/***/ }),

/***/ "./src/ts/pageFunciton/BookmarkAllWorks.ts":
/*!*************************************************!*\
  !*** ./src/ts/pageFunciton/BookmarkAllWorks.ts ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   BookmarkAllWorks: () => (/* binding */ BookmarkAllWorks)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../SetTimeoutWorker */ "./src/ts/SetTimeoutWorker.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");










class BookmarkAllWorks {
    constructor(tipWrap) {
        this.idList = [];
        this.bookmarKData = [];
        this.tipWrap = document.createElement('button');
        if (tipWrap) {
            this.tipWrap = tipWrap;
        }
    }
    // 传递 workList，这是作品列表元素的合集。代码会尝试分析每个作品元素中的超链接，提取出作品 id
    // 如果传递的作品是本页面上的作品，可以省略 type。代码会根据页面 url 判断是图片还是小说。
    // 如果传递的作品不是本页面上的，为防止误判，需要显式传递 type
    sendWorkList(list, type) {
        this.reset();
        type =
            type ??
                (window.location.pathname.includes('/novel') ? 'novels' : 'illusts');
        const regExp = type === 'illusts' ? /\/artworks\/(\d*)/ : /\?id=(\d*)/;
        for (const el of list) {
            const a = el.querySelector('a');
            if (a) {
                // "https://www.pixiv.net/artworks/82618568"
                // "https://www.pixiv.net/novel/show.php?id=12350618"
                const test = regExp.exec(a.href);
                if (test && test.length > 1) {
                    this.idList.push({
                        type,
                        id: test[1],
                    });
                }
            }
        }
        this.startBookmark();
    }
    // 直接传递 id 列表
    sendIdList(list) {
        this.reset();
        this.idList = list;
        this.startBookmark();
    }
    reset() {
        this.idList = [];
        this.bookmarKData = [];
    }
    // 启动收藏流程
    async startBookmark() {
        if (this.idList.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有数据可供使用'));
            return;
        }
        this.tipWrap.textContent = `Checking`;
        this.tipWrap.setAttribute('disabled', 'disabled');
        await this.getTagData();
        await this.addBookmarkAll();
        this.complete();
    }
    // 获取每个作品的 tag 数据
    async getTagData() {
        return new Promise(async (resolve, reject) => {
            for (const id of this.idList) {
                this.tipWrap.textContent = `Get data ${this.bookmarKData.length} / ${this.idList.length}`;
                try {
                    // 如果作品数量大于一定数量，则启用慢速抓取，以免在获取作品数据时发生 429 错误
                    await new Promise(async (res) => {
                        _SetTimeoutWorker__WEBPACK_IMPORTED_MODULE_8__.setTimeoutWorker.set(async () => {
                            let data;
                            if (id.type === 'novels') {
                                data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getNovelData(id.id);
                            }
                            else {
                                data = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(id.id);
                            }
                            this.bookmarKData.push({
                                type: id.type,
                                id: data.body.id,
                                tags: _Tools__WEBPACK_IMPORTED_MODULE_5__.Tools.extractTags(data),
                                restrict: false,
                            });
                            res(id);
                        }, this.idList.length >= 120 ? _Config__WEBPACK_IMPORTED_MODULE_9__.Config.slowCrawlDealy : 0);
                    });
                }
                catch (error) {
                    // 捕获错误，主要是为了处理 429 错误。但是现在只会提示，没有重试。因为需要过 3 分钟之后才能重试成功，等待时间太长了
                    const e = error;
                    let msg = '';
                    if (e.status) {
                        msg = `${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_发生错误原因')}${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_错误代码')}${e.status}. ${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_请稍后重试')}`;
                    }
                    else {
                        msg = `${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_发生错误原因')}${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_未知错误')}${_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_请稍后重试')}`;
                    }
                    // 显示提示，并中止执行
                    _Log__WEBPACK_IMPORTED_MODULE_6__.log.error(msg);
                    _MsgBox__WEBPACK_IMPORTED_MODULE_7__.msgBox.error(msg);
                    this.tipWrap.textContent = `× Error`;
                    this.tipWrap.removeAttribute('disabled');
                    _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.fire('bookmarkModeEnd');
                    return reject();
                }
            }
            resolve();
        });
    }
    // 给所有作品添加收藏（之前收藏过的，新 tag 将覆盖旧 tag）
    async addBookmarkAll() {
        return new Promise(async (resolve) => {
            let index = 0;
            for (const data of this.bookmarKData) {
                this.tipWrap.textContent = `Add bookmark ${index} / ${this.bookmarKData.length}`;
                await _Bookmark__WEBPACK_IMPORTED_MODULE_4__.bookmark.add(data.id, data.type, data.tags, undefined, undefined, true);
                index++;
            }
            resolve();
        });
    }
    complete() {
        this.tipWrap.textContent = `✓ Complete`;
        this.tipWrap.removeAttribute('disabled');
        _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_收藏作品完毕'));
        _EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.fire('bookmarkModeEnd');
    }
}



/***/ }),

/***/ "./src/ts/pageFunciton/BookmarksAddTag.ts":
/*!************************************************!*\
  !*** ./src/ts/pageFunciton/BookmarksAddTag.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   BookmarksAddTag: () => (/* binding */ BookmarksAddTag)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");





// 给收藏页面里的未分类作品批量添加 tag
class BookmarksAddTag {
    constructor(btn) {
        this.type = 'illusts'; // 页面是图片还是小说
        this.addTagList = []; // 需要添加 tag 的作品的数据
        this.addIndex = 0; // 添加 tag 时的计数
        this.once = 100; // 一次请求多少个作品的数据
        this.btn = btn;
        this.bindEvents();
    }
    bindEvents() {
        this.btn.addEventListener('click', () => {
            // 每次点击重置状态
            this.addTagList = [];
            this.addIndex = 0;
            this.btn.setAttribute('disabled', 'disabled');
            this.btn.textContent = `Checking`;
            if (window.location.pathname.includes('/novel')) {
                this.type = 'novels';
            }
            this.readyAddTag();
        });
    }
    // 准备添加 tag。loop 表示这是第几轮循环
    async readyAddTag(loop = 0) {
        const offset = loop * this.once; // 一次请求只能获取一部分，所以可能有多次请求，要计算偏移量
        let errorFlag = false;
        // 发起请求
        const [showData, hideData] = await Promise.all([
            _API__WEBPACK_IMPORTED_MODULE_0__.API.getBookmarkData(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getUserId(), this.type, '未分類', offset, false),
            _API__WEBPACK_IMPORTED_MODULE_0__.API.getBookmarkData(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getUserId(), this.type, '未分類', offset, true),
        ]).catch((error) => {
            if (error.status && error.status === 403) {
                this.btn.textContent = `× Permission denied`;
            }
            errorFlag = true;
            return [];
        });
        if (errorFlag) {
            return;
        }
        // 保存有用的数据
        for (const data of [showData, hideData]) {
            const works = data.body.works;
            // 如果作品的 bookmarkData 为假说明没有实际数据，可能是在获取别人的收藏数据。
            if (works.length > 0 && works[0].bookmarkData) {
                works.forEach((work) => {
                    // 如果该作品没有标签，则不添加它
                    // 这个作品也许确实没有标签，但绝大多数情况下，这表示这个作品已经被删除而不存在了（404）
                    if (work.tags.length > 0) {
                        this.addTagList.push({
                            id: work.id,
                            tags: work.tags,
                            restrict: work.bookmarkData.private,
                        });
                    }
                });
            }
        }
        // 已删除或无法访问的作品不会出现在请求结果里。本来一次请求 100 个，但返回的结果有可能会比 100 个少，甚至极端情况下是 0。所以实际获取到的作品可能比  total 数量少，这是正常的。
        // 判断是否请求了所有未分类的作品数据
        const total = offset + this.once;
        if (total >= showData.body.total && total >= hideData.body.total) {
            if (this.addTagList.length === 0) {
                // 如果结果为空，不需要处理
                this.btn.textContent = `✓ No need`;
                this.btn.removeAttribute('disabled');
                return;
            }
            else {
                // 开始添加 tag
                this.addTag();
            }
        }
        else {
            // 需要继续获取
            this.readyAddTag(++loop);
        }
    }
    // 给未分类作品添加 tag
    async addTag() {
        const item = this.addTagList[this.addIndex];
        await _Bookmark__WEBPACK_IMPORTED_MODULE_3__.bookmark.add(item.id, this.type, item.tags, true, item.restrict, true);
        if (this.addIndex < this.addTagList.length - 1) {
            this.addIndex++;
            this.btn.textContent = `${this.addIndex} / ${this.addTagList.length}`;
            // 继续添加下一个
            return this.addTag();
        }
        else {
            // 添加完成
            this.btn.textContent = `✓ Complete`;
            this.btn.removeAttribute('disabled');
            _Toast__WEBPACK_IMPORTED_MODULE_2__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_4__.lang.transl('_收藏作品完毕'));
        }
    }
}



/***/ }),

/***/ "./src/ts/pageFunciton/DeleteWorks.ts":
/*!********************************************!*\
  !*** ./src/ts/pageFunciton/DeleteWorks.ts ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   DeleteWorks: () => (/* binding */ DeleteWorks)
/* harmony export */ });
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Colors__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Colors */ "./src/ts/Colors.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _store_Store__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../store/Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
// 删除页面上的作品










class DeleteWorks {
    constructor(worksSelectors) {
        this.worksSelector = ''; // 选择页面上所有作品的选择器
        this.multipleSelector = ''; // 多图作品特有的元素的标识
        this.ugoiraSelector = ''; // 动图作品特有的元素的标识
        this.delMode = false; // 是否处于手动删除作品状态
        this.delBtn = document.createElement('button');
        this.iconId = 'deleteWorkEl';
        this.left = 0;
        this.top = 0;
        this.half = 12;
        this.deleteWorkCallback = () => { }; // 保存手动删除作品的回调函数，因为可能会多次绑定手动删除事件，所以需要保存传入的 callback 备用
        this.worksSelector = worksSelectors;
        this.icon = this.createDeleteIcon();
        this.bindEvents();
    }
    createDeleteIcon() {
        const el = document.createElement('div');
        el.id = this.iconId;
        document.body.appendChild(el);
        return el;
    }
    updateDeleteIcon() {
        if (!this.icon) {
            return;
        }
        this.icon.style.display = this.delMode ? 'block' : 'none';
        // 如果指示图标处于隐藏状态，就不会更新其坐标。这样可以优化性能
        if (this.delMode) {
            this.icon.style.left = this.left - this.half + 'px';
            this.icon.style.top = this.top - this.half + 'px';
        }
    }
    bindEvents() {
        // 作品列表更新后，需要重新给作品绑定手动删除事件
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.worksUpdate, () => {
            if (this.delMode) {
                this.bindDeleteEvent();
            }
        });
        // 切换页面时，退出手动删除模式
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.list.pageSwitch, () => {
            if (this.delMode) {
                this.toggleDeleteMode();
            }
        });
        // 鼠标移动时保存鼠标的坐标
        window.addEventListener('mousemove', (ev) => {
            this.moveEvent(ev);
        }, true);
    }
    // 监听鼠标移动
    moveEvent(ev) {
        this.left = ev.x;
        this.top = ev.y;
        this.updateDeleteIcon();
    }
    // 清除多图作品的按钮
    addClearMultipleBtn(selector, callback = () => { }) {
        this.multipleSelector = selector;
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgRed, '_清除多图作品').addEventListener('click', () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
                _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前任务尚未完成'));
                return;
            }
            if (_store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta.length === 0) {
                _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有可用的抓取结果'));
                return;
            }
            this.clearMultiple();
            callback();
        }, false);
    }
    // 清除动图作品的按钮
    addClearUgoiraBtn(selector, callback = () => { }) {
        this.ugoiraSelector = selector;
        _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgRed, '_清除动图作品').addEventListener('click', () => {
            if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
                _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前任务尚未完成'));
                return;
            }
            if (_store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta.length === 0) {
                _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有可用的抓取结果'));
                return;
            }
            this.ClearUgoira();
            callback();
        }, false);
    }
    // 手动删除作品的按钮
    addManuallyDeleteBtn(callback = () => { }) {
        this.deleteWorkCallback = callback;
        this.delBtn = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.addBtn('crawlBtns', _Colors__WEBPACK_IMPORTED_MODULE_2__.Colors.bgRed, '_手动删除作品', '_手动删除作品Title');
        this.delBtn.addEventListener('click', () => {
            this.toggleDeleteMode();
        });
    }
    // 切换删除模式
    toggleDeleteMode() {
        if (_store_Store__WEBPACK_IMPORTED_MODULE_8__.store.resultMeta.length === 0) {
            _Toast__WEBPACK_IMPORTED_MODULE_9__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_没有可用的抓取结果'));
            return;
        }
        this.delMode = !this.delMode;
        this.bindDeleteEvent();
        this.updateDeleteIcon();
        if (this.delMode) {
            _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.delBtn, '_退出手动删除');
            window.setTimeout(() => {
                _EVT__WEBPACK_IMPORTED_MODULE_5__.EVT.fire('closeCenterPanel');
            }, 100);
        }
        else {
            _Lang__WEBPACK_IMPORTED_MODULE_1__.lang.updateText(this.delBtn, '_手动删除作品');
        }
    }
    // 清除多图作品
    clearMultiple() {
        const allPicArea = document.querySelectorAll(this.worksSelector);
        allPicArea.forEach((el) => {
            if (el.querySelector(this.multipleSelector)) {
                el.remove();
            }
        });
        this.showWorksCount();
    }
    // 清除动图作品
    ClearUgoira() {
        const allPicArea = document.querySelectorAll(this.worksSelector);
        allPicArea.forEach((el) => {
            if (el.querySelector(this.ugoiraSelector)) {
                el.remove();
            }
        });
        this.showWorksCount();
    }
    // 给作品绑定手动删除事件
    // 删除作品后，回调函数可以接收到被删除的元素
    bindDeleteEvent() {
        const listElement = document.querySelectorAll(this.worksSelector);
        listElement.forEach((el) => {
            el.onclick = (ev) => {
                if (this.delMode) {
                    ev.preventDefault();
                    if (_store_States__WEBPACK_IMPORTED_MODULE_4__.states.busy) {
                        _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_当前任务尚未完成'));
                        return;
                    }
                    const target = ev.currentTarget;
                    target.remove();
                    this.showWorksCount();
                    this.deleteWorkCallback(target);
                }
            };
        });
    }
    // 显示调整后，列表里的作品数量
    showWorksCount() {
        const selector = this.worksSelector;
        _Log__WEBPACK_IMPORTED_MODULE_0__.log.success(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_调整完毕', _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.getVisibleEl(selector).length.toString()), 2, false);
    }
}



/***/ }),

/***/ "./src/ts/pageFunciton/DestroyManager.ts":
/*!***********************************************!*\
  !*** ./src/ts/pageFunciton/DestroyManager.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   destroyManager: () => (/* binding */ destroyManager)
/* harmony export */ });
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");


// 管理所有页面的销毁事件
// 页面把自己的 destory 函数注册到这个类里，当页面类型变化时会自动执行对应
class DestroyManager {
    constructor() {
        this.list = new Map();
        this.lastType = _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type;
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitchedTypeChange, () => {
            const fun = this.list.get(this.lastType);
            fun && fun();
            this.lastType = _PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type;
        });
    }
    // 接收 destory 函数，并关联到对应的页面类型
    register(fun) {
        this.list.set(_PageType__WEBPACK_IMPORTED_MODULE_0__.pageType.type, fun);
    }
}
const destroyManager = new DestroyManager();



/***/ }),

/***/ "./src/ts/pageFunciton/DisplayThumbnailListOnMultiImageWorkPage.ts":
/*!*************************************************************************!*\
  !*** ./src/ts/pageFunciton/DisplayThumbnailListOnMultiImageWorkPage.ts ***!
  \*************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _ImageViewer__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../ImageViewer */ "./src/ts/ImageViewer.ts");






class DisplayThumbnailListOnMultiImageWorkPage {
    constructor() {
        this.ID = 'viewerWarpper';
        this.insertTarget = 'main figcaption';
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.pageSwitch, () => {
            this.init();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'displayThumbnailListOnMultiImageWorkPage') {
                data.value ? this.init() : this.remove();
            }
        });
    }
    init() {
        this.remove();
        if (!_setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.displayThumbnailListOnMultiImageWorkPage) {
            return;
        }
        if (_PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Artwork &&
            _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_2__.pageType.list.Unlisted) {
            return;
        }
        window.setTimeout(() => {
            this.display();
        }, 0);
    }
    remove() {
        // 删除之前创建的元素，因为切换页面时它不会被自动清理
        document.querySelector(`#${this.ID}`)?.remove();
        window.clearTimeout(this.waitTimer);
    }
    async display() {
        // 等待要插入的目标元素生成
        const target = document.querySelector(this.insertTarget);
        if (!target) {
            this.waitTimer = window.setTimeout(() => {
                this.display();
            }, 300);
            return;
        }
        // 把缩略图列表添加到页面上
        const viewer = new _ImageViewer__WEBPACK_IMPORTED_MODULE_5__.ImageViewer({
            workId: _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getIllustId(),
            imageNumber: 2,
        });
        const wrap = await viewer.init();
        if (wrap) {
            wrap.id = this.ID;
            _Theme__WEBPACK_IMPORTED_MODULE_0__.theme.register(wrap);
            wrap.style.display = 'block';
            target.insertAdjacentElement('beforebegin', wrap);
        }
    }
}
new DisplayThumbnailListOnMultiImageWorkPage();


/***/ }),

/***/ "./src/ts/pageFunciton/FastScreen.ts":
/*!*******************************************!*\
  !*** ./src/ts/pageFunciton/FastScreen.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   FastScreen: () => (/* binding */ FastScreen)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");





// 在搜索页面按收藏数快速筛选
class FastScreen {
    constructor() {
        this.fastScreenArea = document.createElement('div');
        this.tagList = [
            '100users入り',
            '500users入り',
            '1000users入り',
            '5000users入り',
            '10000users入り',
            '20000users入り',
            '30000users入り',
            '50000users入り',
            '100000users入り',
        ];
        this.create();
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'showFastSearchArea') {
                this.setDisplay();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitchedTypeChange, () => {
            this.destroy();
        });
    }
    // 添加快速筛选功能
    create() {
        // 判断插入点的元素有没有加载出来
        const selector = _Config__WEBPACK_IMPORTED_MODULE_4__.Config.mobile ? '.search-header' : '#root>div';
        const target = document.querySelector(selector);
        if (!target) {
            setTimeout(() => {
                this.create();
            }, 300);
            return;
        }
        this.fastScreenArea.className = 'fastScreenArea';
        this.tagList.forEach((tag) => {
            const btn = document.createElement('button');
            btn.innerText = tag;
            btn.onclick = () => {
                this.openFastScreenLink(tag);
            };
            this.fastScreenArea.appendChild(btn);
        });
        _Theme__WEBPACK_IMPORTED_MODULE_2__.theme.register(this.fastScreenArea);
        target.insertAdjacentElement('afterend', this.fastScreenArea);
        this.setDisplay();
    }
    // 设置是否显示快速筛选区域
    setDisplay() {
        this.fastScreenArea.style.display = _setting_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showFastSearchArea
            ? 'block'
            : 'none';
    }
    // 打开快速筛选链接
    openFastScreenLink(fastTag) {
        // 拼接 tag。因为搜索页面可以无刷新切换搜索的 tag，所以需要动态获取当前 tag
        const nowTag = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getTagFromURL();
        const firstTag = nowTag.split(' ')[0];
        const fullTag = firstTag + ' ' + fastTag;
        // 用组合后的 tag 替换掉当前网址里的 tag
        let newURL = location.href.replace(encodeURIComponent(nowTag), encodeURIComponent(fullTag));
        // 如果 url 路径的 tags/ 后面没有子路径，代表是在“顶部”分类。
        // “顶部”分类里始终是严格的搜索模式，即使添加 s_mode=s_tag 也无效，这经常会导致搜索结果为 0。所以如果分类是“顶部”，就自动修改为“插画·漫画”分类以获取更多搜索结果。
        // “顶部”分类的 url 示例
        // https://www.pixiv.net/tags/%E9%9B%AA%E8%8A%B1%E3%83%A9%E3%83%9F%E3%82%A3%2010000users%E5%85%A5%E3%82%8A?s_mode=s_tag
        const str = new URL(newURL).pathname.split('tags/')[1];
        if (str.includes('/') === false) {
            // 在 tag 后面添加“插画·漫画”分类的路径
            newURL = newURL.replace(str, str + '/artworks');
        }
        // 设置宽松的搜索模式 s_mode=s_tag
        const u = new URL(newURL);
        u.searchParams.set('s_mode', 's_tag');
        location.href = u.toString();
    }
    destroy() {
        this.fastScreenArea.remove();
    }
}



/***/ }),

/***/ "./src/ts/pageFunciton/QuickBookmark.ts":
/*!**********************************************!*\
  !*** ./src/ts/pageFunciton/QuickBookmark.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Token__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Token */ "./src/ts/Token.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Bookmark__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Bookmark */ "./src/ts/Bookmark.ts");
/* harmony import */ var _WorkToolBar__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../WorkToolBar */ "./src/ts/WorkToolBar.ts");
/* harmony import */ var _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../download/DownloadOnClickBookmark */ "./src/ts/download/DownloadOnClickBookmark.ts");
/* harmony import */ var _ShowHelp__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../ShowHelp */ "./src/ts/ShowHelp.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
// 作品页面内的快速收藏功能










class QuickBookmark {
    constructor() {
        this.isNovel = false;
        this.btn = document.createElement('a'); // 快速收藏按钮
        this.btnId = 'quickBookmarkEl'; // 快速收藏按钮的 id
        this.redClass = 'bookmarkedColor'; // 收藏后的红色的颜色值
        _WorkToolBar__WEBPACK_IMPORTED_MODULE_6__.workToolBar.register((toolbar, pixivBMKDiv, likeBtn) => {
            this.init(toolbar, pixivBMKDiv, likeBtn);
        });
    }
    async init(toolbar, pixivBMKDiv, likeBtn) {
        // 没有 token 就不能进行收藏
        if (!_Token__WEBPACK_IMPORTED_MODULE_3__.token.token) {
            return;
        }
        if (_PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Artwork &&
            _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.type !== _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Novel) {
            return;
        }
        this.isNovel = _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.type === _PageType__WEBPACK_IMPORTED_MODULE_4__.pageType.list.Novel;
        // 删除可能存在的旧的快速收藏按钮
        const oldBtn = toolbar.querySelector('#' + this.btnId);
        if (oldBtn) {
            oldBtn.remove();
        }
        // 判断这个作品是否收藏过了
        this.workData = await this.getWorkData();
        this.isBookmarked = !!this.workData.body.bookmarkData;
        // 监听心形收藏按钮从未收藏到收藏的变化
        if (!this.isBookmarked) {
            if (!_Config__WEBPACK_IMPORTED_MODULE_9__.Config.mobile) {
                // 桌面端
                // 没有收藏时，心形按钮的第一个子元素是 button。收藏之后，button 被移除，然后添加一个 a 标签
                this.ob = new MutationObserver((mutations) => {
                    for (const change of mutations) {
                        if (change.type === 'childList') {
                            const added = change.addedNodes;
                            if (added.length > 0 && added[0].nodeName === 'A') {
                                this.isBookmarked = true;
                                this.redQuickBookmarkBtn();
                            }
                        }
                    }
                });
                this.ob.observe(pixivBMKDiv, {
                    childList: true,
                });
            }
            else {
                // 移动端
                // 点击心形按钮收藏作品后，不会添加 a 标签，也不会跳转到编辑收藏的页面，仅仅会改变 path 的 fill 颜色。
                const path = pixivBMKDiv.querySelector('path');
                if (!path) {
                    return;
                }
                this.ob = new MutationObserver((mutations) => {
                    if (path.getAttribute('fill') === '#FF4060') {
                        this.isBookmarked = true;
                        this.redQuickBookmarkBtn();
                    }
                    else {
                        this.isBookmarked = false;
                        this.resetQuickBookmarkBtn();
                    }
                });
                this.ob.observe(path, {
                    attributes: true,
                    attributeFilter: ['fill'],
                });
            }
        }
        // 添加快速收藏按钮
        this.btn = this.createBtn();
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.btn);
        toolbar.insertBefore(this.btn, toolbar.childNodes[3]);
        if (this.isBookmarked) {
            this.redQuickBookmarkBtn();
        }
        else {
            this.btn.addEventListener('click', () => {
                // 添加收藏
                this.addBookmark(pixivBMKDiv, likeBtn);
                // 下载这个作品
                this.sendDownload();
                _ShowHelp__WEBPACK_IMPORTED_MODULE_8__.showHelp.show('tipBookmarkButton', _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_下载器的收藏按钮默认会添加作品的标签'));
            });
        }
        // 使用快捷键 Ctrl + B 点击快速收藏按钮
        window.addEventListener('keydown', (ev) => {
            if (ev.code === 'KeyB' && ev.ctrlKey) {
                this.btn && this.btn.click();
            }
        });
    }
    sendDownload() {
        if (_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.isArtworkData(this.workData)) {
            _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_7__.downloadOnClickBookmark.send(this.workData.body.illustId);
        }
        else {
            _download_DownloadOnClickBookmark__WEBPACK_IMPORTED_MODULE_7__.downloadOnClickBookmark.send(this.workData.body.id, 'novels');
        }
    }
    //　创建快速收藏按钮
    createBtn() {
        const btn = document.createElement('a');
        btn.id = this.btnId;
        btn.textContent = '✩';
        btn.href = 'javascript:void(0)';
        btn.dataset.xztitle = '_快速收藏';
        return btn;
    }
    async getWorkData() {
        // 这里不能从缓存的数据中获取作品数据，因为作品的收藏状态可能已经发生了变化
        if (this.isNovel) {
            return await _API__WEBPACK_IMPORTED_MODULE_0__.API.getNovelData(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getNovelId());
        }
        else {
            return await _API__WEBPACK_IMPORTED_MODULE_0__.API.getArtworkData(_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getIllustId());
        }
    }
    async addBookmark(pixivBMKDiv, likeBtn) {
        const type = this.isNovel ? 'novels' : 'illusts';
        const id = this.isNovel ? _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getNovelId() : _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getIllustId();
        // 移动端不自动点赞和设置点赞按钮的颜色，因为切换作品后元素没有重新生成，样式会依旧存在
        if (!_Config__WEBPACK_IMPORTED_MODULE_9__.Config.mobile) {
            this.like(type, id, likeBtn);
        }
        if (this.isBookmarked) {
            return;
        }
        // 先模拟点击 Pixiv 原本的收藏按钮，这样可以显示推荐作品
        // 这会发送一次 Pixiv 原生的收藏请求
        this.clickPixivBMKBtn(pixivBMKDiv);
        // 然后再由下载器发送收藏请求
        // 因为下载器的收藏按钮具有添加标签、非公开收藏等功能，所以要在后面执行，覆盖掉 Pixiv 原生收藏的效果
        window.setTimeout(async () => {
            const res = await _Bookmark__WEBPACK_IMPORTED_MODULE_5__.bookmark.add(id, type, _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.extractTags(this.workData));
            if (res !== 429) {
                // 收藏成功之后
                this.isBookmarked = true;
                this.redQuickBookmarkBtn();
            }
        }, 100);
    }
    // 点赞这个作品
    like(type, id, likeBtn) {
        _API__WEBPACK_IMPORTED_MODULE_0__.API.addLike(id, type, _Token__WEBPACK_IMPORTED_MODULE_3__.token.token);
        likeBtn.style.color = '#0096fa';
    }
    getEditBookmarkLink() {
        if (this.isNovel) {
            return `/novel/bookmark_add.php?id=${_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getNovelId()}`;
        }
        else {
            return `/bookmark_add.php?type=illust&illust_id=${_Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.getIllustId()}`;
        }
    }
    // 如果这个作品已收藏，则改变快速收藏按钮
    redQuickBookmarkBtn() {
        this.btn.classList.add(this.redClass);
        this.btn.href = this.getEditBookmarkLink();
    }
    // 如果这个作品从已收藏变成未收藏，则改变快速收藏按钮
    resetQuickBookmarkBtn() {
        this.btn.classList.remove(this.redClass);
        this.btn.href = 'javascript:void(0)';
    }
    clickPixivBMKBtn(pixivBMKDiv) {
        if (_Config__WEBPACK_IMPORTED_MODULE_9__.Config.mobile) {
            pixivBMKDiv && pixivBMKDiv.click();
        }
        else {
            const btn = pixivBMKDiv.querySelector('button');
            btn && btn.click();
        }
        // 取消监听心形收藏按钮的变化
        this.ob && this.ob.disconnect();
    }
}
new QuickBookmark();


/***/ }),

/***/ "./src/ts/pageFunciton/SaveAvatarIcon.ts":
/*!***********************************************!*\
  !*** ./src/ts/pageFunciton/SaveAvatarIcon.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_imageToIcon__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/imageToIcon */ "./src/ts/utils/imageToIcon.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");








// 保存用户头像为图标
class SaveAvatarIcon {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.saveAvatarIcon, () => {
            this.saveAvatarIcon();
        });
    }
    async saveAvatarIcon() {
        const userId = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getUserId();
        const userProfile = await _API__WEBPACK_IMPORTED_MODULE_1__.API.getUserProfile(userId);
        const bigImg = userProfile.body.imageBig; // imageBig 并不是头像原图，而是裁剪成 170 px 的尺寸
        const fullSizeImg = bigImg.replace('_170', ''); // 去掉 170 标记，获取头像图片的原图
        // 生成 ico 文件
        // 尺寸固定为 256，因为尺寸更小的图标如 128，在 windows 资源管理器里会被缩小到 48 显示
        const blob = await _utils_imageToIcon__WEBPACK_IMPORTED_MODULE_5__.img2ico.convert({
            size: [256],
            source: fullSizeImg,
            shape: 'fillet',
            bleed: true,
        });
        // 直接保存到下载文件夹
        const url = URL.createObjectURL(blob);
        const name = `${userProfile.body.name}_${userId}_icon.ico`;
        _utils_Utils__WEBPACK_IMPORTED_MODULE_6__.Utils.downloadFile(url, name);
        const msg = '✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_保存用户头像为图标');
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_7__.toast.success(msg);
        _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.fire('closeCenterPanel');
    }
}
new SaveAvatarIcon();


/***/ }),

/***/ "./src/ts/pageFunciton/SaveAvatarImage.ts":
/*!************************************************!*\
  !*** ./src/ts/pageFunciton/SaveAvatarImage.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");







// 保存用户头像
class SaveAvatarImage {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.saveAvatarImage, () => {
            this.saveAvatarImage();
        });
    }
    async saveAvatarImage() {
        const userId = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getUserId();
        const userProfile = await _API__WEBPACK_IMPORTED_MODULE_1__.API.getUserProfile(userId);
        const imageURL = userProfile.body.imageBig;
        // 提取图片的后缀名
        const arr = imageURL.split('.');
        const ext = arr[arr.length - 1];
        // imageBig 并不是头像原图，而是裁剪成 170 px 的尺寸
        // 如果是 gif 格式，则不生成其大图 url，因为生成的大图是静态图。不知道 gif 头像是否有大图，以及其 url 是什么样的
        // 如果是其他格式，则去掉 170 标记，获取头像图片的原图
        const fullSizeImgURL = ext === 'gif' ? imageURL : imageURL.replace('_170', '');
        // 加载文件
        const img = await fetch(fullSizeImgURL);
        const blob = await img.blob();
        // 直接保存到下载文件夹
        const url = URL.createObjectURL(blob);
        const name = `${userProfile.body.name}_${userId}_avatar.${ext}`;
        _utils_Utils__WEBPACK_IMPORTED_MODULE_5__.Utils.downloadFile(url, name);
        const msg = '✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_保存用户头像');
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(msg);
        _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.fire('closeCenterPanel');
    }
}
new SaveAvatarImage();


/***/ }),

/***/ "./src/ts/pageFunciton/SaveUserCover.ts":
/*!**********************************************!*\
  !*** ./src/ts/pageFunciton/SaveUserCover.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");







// 保存用户封面图片
class SaveUserCover {
    constructor() {
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.list.saveUserCover, () => {
            this.saveUserCover();
        });
    }
    async saveUserCover() {
        const userId = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getUserId();
        const userProfile = await _API__WEBPACK_IMPORTED_MODULE_1__.API.getUserProfile(userId);
        const bgData = userProfile.body.background;
        if (bgData === null) {
            return _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_没有数据可供使用'));
        }
        const bgUrl = bgData.url;
        if (!bgUrl) {
            return _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.error(_Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_没有数据可供使用'));
        }
        // 加载文件
        const img = await fetch(bgUrl);
        const blob = await img.blob();
        // 提取后缀名
        const arr = bgUrl.split('.');
        const ext = arr[arr.length - 1];
        // 直接保存到下载文件夹
        const url = URL.createObjectURL(blob);
        const name = `${userProfile.body.name}_${userId}_cover.${ext}`;
        _utils_Utils__WEBPACK_IMPORTED_MODULE_5__.Utils.downloadFile(url, name);
        const msg = '✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_0__.lang.transl('_保存用户封面');
        _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(msg);
        _EVT__WEBPACK_IMPORTED_MODULE_4__.EVT.fire('closeCenterPanel');
    }
}
new SaveUserCover();


/***/ }),

/***/ "./src/ts/setting/ConvertOldSettings.ts":
/*!**********************************************!*\
  !*** ./src/ts/setting/ConvertOldSettings.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   convertOldSettings: () => (/* binding */ convertOldSettings)
/* harmony export */ });
// 为了兼容以前的版本的设置，把旧的设置值转换为新版本的设置值
class ConvertOldSettings {
    constructor() {
        // 旧设置和新设置的对应关系
        // 为了集中管理，便于使用，写到了一个对象里
        this.data = {
            ratio: {
                '0': 'square',
                '1': 'horizontal',
                '2': 'vertical',
                '3': 'userSet',
            },
            idRange: {
                '1': '>',
                '2': '<',
            },
            widthTag: {
                '1': 'yes',
                '-1': 'no',
            },
            restrict: {
                '1': 'yes',
                '-1': 'no',
            },
            userSetLang: {
                '-1': 'auto',
                '0': 'zh-cn',
                '1': 'ja',
                '2': 'en',
                '3': 'zh-tw',
                '4': 'ko',
            },
        };
    }
    // 传递需要转换的设置的键值
    convert(key, value) {
        const map = this.data[key];
        // 如果这是一个可以转换的设置
        if (map) {
            // 如果传递的值是旧的设置值，则能够获取到新的设置值
            // 如果传递的值已经是新的设置值，则获取到的是 undefined ，此时不需要转换
            const newValue = map[value];
            if (newValue !== undefined) {
                return newValue;
            }
        }
        return value;
    }
}
const convertOldSettings = new ConvertOldSettings();



/***/ }),

/***/ "./src/ts/setting/DoNotDownloadLastFewImages.ts":
/*!******************************************************!*\
  !*** ./src/ts/setting/DoNotDownloadLastFewImages.ts ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");









// 针对某些用户,不下载他们的多图作品的最后几张图片
class DoNotDownloadLastFewImages {
    constructor() {
        this.slotName = 'DoNotDownloadLastFewImagesSlot';
        this._addWrapShow = false;
        this.wrapHTML = `
  <span class="DoNotDownloadLastFewImagesWarp">

    <span class="controlBar">
    <span class="total">0</span>
      <button type="button" class="textButton expand" data-xztext="_收起"></button>
      <button type="button" class="textButton showAdd" data-xztext="_添加"></button>
    </span>

    <div class="addWrap">
      <div class="settingItem addInputWrap" >
        <div class="inputItem uid">
          <span class="label uidLabel" data-xztext="_用户id"></span>
          <input type="text" class="setinput_style1 blue addUidInput" data-xzplaceholder="_必须是数字" />
        </div>

        <div class="inputItem value">
          <span class="label nameLabel" data-xztext="_不下载最后几张图片"></span>
          <input type="text" class="has_tip setinput_style1 blue addValueInput" data-xztip="_提示0表示不生效" />
        </div>

        <div class="btns">
          <button type="button" class="textButton add" data-xztitle="_添加">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-wanchengqueding"></use>
            </svg>
          </button>

          
          <button type="button" class="textButton cancel" data-xztitle="_取消">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-guanbiquxiao"></use>
            </svg>
          </button>
        </div>
      </div>
    </div>

    <div class="listWrap">
    </div>
  </span>
  `;
        this.createWrap();
        _Theme__WEBPACK_IMPORTED_MODULE_5__.theme.register(this.wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.register(this.wrap);
        this.bindEvents();
    }
    set addWrapShow(val) {
        this._addWrapShow = val;
        if (val) {
            this.addWrap.style.display = 'block';
            this.addInputUid.focus();
        }
        else {
            this.addWrap.style.display = 'none';
            this.addInputUid.value = '';
            this.addValueInput.value = '';
        }
    }
    get addWrapShow() {
        return this._addWrapShow;
    }
    // 创建列表外部的容器，静态 html
    createWrap() {
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.useSlot(this.slotName, this.wrapHTML);
        this.expandBtn = this.wrap.querySelector('.expand');
        this.showAddBtn = this.wrap.querySelector('.showAdd');
        this.totalSpan = this.wrap.querySelector('.total');
        this.addWrap = this.wrap.querySelector('.addWrap');
        this.addInputUid = this.wrap.querySelector('.addUidInput');
        this.addValueInput = this.wrap.querySelector('.addValueInput');
        this.addBtn = this.wrap.querySelector('.add');
        this.cancelBtn = this.wrap.querySelector('.cancel');
        this.listWrap = this.wrap.querySelector('.listWrap');
        // 展开/折叠按钮
        this.expandBtn.addEventListener('click', () => {
            (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('DoNotDownloadLastFewImagesShow', !_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesShow);
        });
        // 切换显示添加规则的区域
        this.showAddBtn.addEventListener('click', () => {
            this.addWrapShow = !this.addWrapShow;
        });
        // 添加规则的按钮
        this.addBtn.addEventListener('click', () => {
            this.addRule(this.addInputUid.value, this.addValueInput.value);
        });
        // 取消添加的按钮
        this.cancelBtn.addEventListener('click', () => {
            this.addWrapShow = false;
        });
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_2__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'DoNotDownloadLastFewImagesShow') {
                this.showListWrap();
            }
            if (data.name === 'DoNotDownloadLastFewImagesList') {
                this.createAllList();
            }
        });
    }
    showListWrap() {
        const show = _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesShow;
        this.listWrap.style.display = show ? 'flex' : 'none';
        _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.updateText(this.expandBtn, show ? '_收起' : '_展开');
    }
    // 根据规则动态创建 html
    createAllList() {
        this.totalSpan.textContent =
            _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.length.toString();
        this.listWrap.innerHTML = '';
        const df = document.createDocumentFragment();
        for (const { uid, user, value, } of _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList) {
            df.append(this.createOneList(uid, user, value));
        }
        this.listWrap.append(df);
    }
    // 创建规则对应的元素，并绑定事件
    createOneList(uid, user, value) {
        const html = `
      <div class="inputItem user">
        <span>${user}</span>
      </div>

      <div class="inputItem uid">
        <input type="text" class="setinput_style1 blue" data-uidInput="${uid}" value="${uid}" />
      </div>

      <div class="inputItem value">
        <input type="text" class="has_tip setinput_style1 blue" data-valueInput="${uid}" value="${value}" data-xztip="_提示0表示不生效" />
      </div>

      <div class="btns">
        <button type="button" class="textButton" data-updateRule="${uid}" data-xztitle="_更新">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-gengxin"></use>
          </svg>
        </button>

        <button type="button" class="textButton" data-deleteRule="${uid}" data-xztitle="_删除">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-shanchu1"></use>
          </svg>
        </button>
    </div>`;
        const element = document.createElement('div');
        element.classList.add('settingItem');
        element.dataset.key = uid.toString();
        element.innerHTML = html;
        const updateRule = element.querySelector(`button[data-updateRule='${uid}']`);
        const deleteRule = element.querySelector(`button[data-deleteRule='${uid}']`);
        const uidInput = element.querySelector(`input[data-uidInput='${uid}']`);
        const valueInput = element.querySelector(`input[data-valueInput='${uid}']`);
        [uidInput, valueInput].forEach((el) => {
            el?.addEventListener('change', () => {
                if (el.value) {
                    this.updateRule(uid, uidInput.value, valueInput.value, false);
                }
            });
        });
        // 更新规则
        updateRule?.addEventListener('click', () => {
            this.updateRule(uid, uidInput.value, valueInput.value);
        });
        // 删除规则
        deleteRule?.addEventListener('click', () => {
            this.deleteRule(uid);
        });
        return element;
    }
    // 检查用户输入的值
    checkValue(uidInput, value) {
        if (!uidInput || !value) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_7__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_必填项不能为空'));
            return false;
        }
        const uid = Number.parseInt(uidInput);
        if (!uid || isNaN(uid)) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_7__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_用户ID必须是数字'));
            return false;
        }
        // value 允许为 0
        const val = Number.parseInt(value);
        if (isNaN(val) || val < 0) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_7__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_不下载最后几张图片') + ' ' + _Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_必须是数字'));
            return false;
        }
        return {
            uid,
            val,
        };
    }
    async getUserName(uid) {
        return new Promise(async (resolve) => {
            const profile = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getUserProfile(uid.toString()).catch((err) => {
                console.log(err);
                _Log__WEBPACK_IMPORTED_MODULE_8__.log.error(`ERROR: userID ${uid}, status ${err.status}<br><a href="https://www.pixiv.net/users/${uid}" target="_blank">https://www.pixiv.net/users/${uid}</a>`);
            });
            if (profile && profile.body.name) {
                return resolve(profile.body.name);
            }
            return resolve('');
        });
    }
    // 添加规则
    async addRule(uid, value) {
        const check = this.checkValue(uid, value);
        if (!check) {
            return;
        }
        let old = _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.find((item) => item.uid === check.uid);
        if (old) {
            old.value = check.val;
        }
        else {
            const user = await this.getUserName(check.uid);
            const data = {
                uid: check.uid,
                user: user,
                value: check.val,
            };
            _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.push(data);
        }
        (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('DoNotDownloadLastFewImagesList', _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList);
        this.addWrapShow = false;
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_添加成功'));
    }
    // 更新规则
    // tip 表示是否用显示操作成功的提示。当用户点击了更新按钮时应该显示提示；输入内容变化导致的自动更新可以不显示提示
    async updateRule(oldUid, uid, value, tip = true) {
        const check = this.checkValue(uid, value);
        if (!check) {
            return;
        }
        let old = _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.find((item) => item.uid === oldUid);
        if (old) {
            // 更新时如果 uid 未改变，依然会获取用户名，因为用户名可能更新了
            const user = await this.getUserName(check.uid);
            old.uid = check.uid;
            old.user = user;
            old.value = check.val;
        }
        else {
            return;
        }
        (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('DoNotDownloadLastFewImagesList', _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList);
        this.addWrapShow = false;
        if (tip) {
            _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_3__.lang.transl('_更新成功'));
        }
    }
    // 删除规则
    deleteRule(uid) {
        let index = _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.findIndex((item) => item.uid === uid);
        if (index > -1) {
            _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList.splice(index, 1);
        }
        else {
            return;
        }
        (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('DoNotDownloadLastFewImagesList', _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.DoNotDownloadLastFewImagesList);
        this.removeListElement(uid);
    }
    removeListElement(uid) {
        const listElement = this.listWrap.querySelector(`.settingItem[data-key='${uid}']`);
        listElement?.remove();
    }
}
new DoNotDownloadLastFewImages();


/***/ }),

/***/ "./src/ts/setting/Form.ts":
/*!********************************!*\
  !*** ./src/ts/setting/Form.ts ***!
  \********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _FormHTML__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./FormHTML */ "./src/ts/setting/FormHTML.ts");
/* harmony import */ var _SaveNamingRule__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./SaveNamingRule */ "./src/ts/setting/SaveNamingRule.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _FormSettings__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./FormSettings */ "./src/ts/setting/FormSettings.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _setting_Options__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../setting/Options */ "./src/ts/setting/Options.ts");










// 设置表单
class Form {
    constructor() {
        /**所有的美化表单元素 */
        // 每个美化的 input 控件后面必定有一个 span 元素
        // label 和 子选项区域则不一定有
        this.allBeautifyInput = [];
        /**一些固定格式的帮助元素 */
        this.tips = [];
        this.form = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.useSlot('form', _FormHTML__WEBPACK_IMPORTED_MODULE_3__.formHtml);
        _Theme__WEBPACK_IMPORTED_MODULE_5__.theme.register(this.form);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.form);
        this.getElements();
        const allOptions = this.form.querySelectorAll('.option');
        _setting_Options__WEBPACK_IMPORTED_MODULE_9__.options.init(allOptions);
        new _SaveNamingRule__WEBPACK_IMPORTED_MODULE_4__.SaveNamingRule(this.form.userSetName);
        new _FormSettings__WEBPACK_IMPORTED_MODULE_6__.FormSettings(this.form);
        this.bindEvents();
    }
    getElements() {
        // 获取所有的美化控件和它们对应的 span 元素
        const allCheckBox = this.form.querySelectorAll('input[type="checkbox"]');
        const allRadio = this.form.querySelectorAll('input[type="radio"]');
        const checkboxAndRadio = [allCheckBox, allRadio];
        for (const arr of checkboxAndRadio) {
            arr.forEach((input) => {
                let subOption = null;
                if (input.classList.contains('checkbox_switch')) {
                    subOption = this.form.querySelector(`.subOptionWrap[data-show="${input.name}"]`);
                }
                this.allBeautifyInput.push({
                    input: input,
                    span: input.nextElementSibling,
                    label: this.form.querySelector(`label[for="${input.id}"]`),
                    subOption: subOption,
                });
            });
        }
        // 获取所有在表单上直接显示的提示元素
        for (const item of this.tips) {
            const wrap = this.form.querySelector('#' + item.wrapID);
            if (wrap) {
                item.wrap = wrap;
            }
        }
    }
    bindEvents() {
        // 为美化的表单控件绑定事件
        for (const item of this.allBeautifyInput) {
            const { input, span } = item;
            // 点击美化元素时，点击真实的 input 控件
            span.addEventListener('click', () => {
                input.click();
            });
            // 当美化元素获得焦点，并且用户按下了回车或空格键时，点击真实的 input 控件
            span.addEventListener('keydown', (event) => {
                if ((event.code === 'Enter' || event.code === 'Space') &&
                    event.target === span) {
                    event.stopPropagation();
                    event.preventDefault();
                    input.click();
                }
            });
        }
        // 设置变化或者重置时，重新设置美化状态
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.debounce(() => {
            this.initFormBeautify();
            this.showTips();
        }, 50));
        // 用户点击“我知道了”按钮之后不再显示对应的提示
        for (const item of this.tips) {
            if (item.wrap) {
                const btn = item.wrap.querySelector('button');
                btn.addEventListener('click', () => {
                    (0,_setting_Settings__WEBPACK_IMPORTED_MODULE_8__.setSetting)(item.settingName, false);
                });
            }
        }
        // 选择背景图片
        {
            const el = this.form.querySelector('#selectBG');
            if (el) {
                el.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('selectBG');
                });
            }
        }
        // 清除背景图片
        {
            const el = this.form.querySelector('#clearBG');
            if (el) {
                el.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('clearBG');
                });
            }
        }
        // 重置设置
        {
            const el = this.form.querySelector('#resetSettings');
            if (el) {
                el.addEventListener('click', () => {
                    const result = window.confirm(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_是否重置设置'));
                    if (result) {
                        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('resetSettings');
                    }
                });
            }
        }
        // 导出设置
        {
            const el = this.form.querySelector('#exportSettings');
            if (el) {
                el.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('exportSettings');
                });
            }
        }
        // 导入设置
        {
            const el = this.form.querySelector('#importSettings');
            if (el) {
                el.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('importSettings');
                });
            }
        }
        // 重新显示帮助
        {
            const el = this.form.querySelector('#resetHelpTip');
            if (el) {
                el.addEventListener('click', () => {
                    _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('resetHelpTip');
                });
            }
        }
        // 显示命名字段提示
        this.form
            .querySelector('.showFileNameTip')
            .addEventListener('click', () => _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.toggleEl(document.querySelector('.fileNameTip')));
        // 显示日期格式提示
        this.form
            .querySelector('.showDateTip')
            .addEventListener('click', () => _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.toggleEl(document.querySelector('.dateFormatTip')));
        // 显示标签分隔提示
        this.form
            .querySelector('.showTagsSeparatorTip')
            .addEventListener('click', () => _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.toggleEl(document.querySelector('.tagsSeparatorTip')));
        // 输入框获得焦点时自动选择文本（文件名输入框例外）
        const centerInputs = this.form.querySelectorAll('input[type=text]');
        for (const el of centerInputs) {
            if (el.name !== 'userSetName') {
                el.addEventListener('focus', function () {
                    this.select();
                });
            }
        }
        // 把下拉框的选择项插入到文本框里
        const from = this.form.fileNameSelect;
        const to = this.form.userSetName;
        from.addEventListener('change', () => {
            if (from.value !== 'default') {
                // 把选择项插入到光标位置,并设置新的光标位置
                const position = to.selectionStart;
                to.value =
                    to.value.substring(0, position) +
                        from.value +
                        to.value.substring(position);
                to.selectionStart = position + from.value.length;
                to.selectionEnd = position + from.value.length;
                to.focus();
            }
        });
    }
    // 设置表单里的美化元素的状态
    initFormBeautify() {
        for (const item of this.allBeautifyInput) {
            const { input, span, label, subOption } = item;
            // 重设 label 的高亮状态
            if (label) {
                const method = input.checked ? 'add' : 'remove';
                label.classList[method]('active');
            }
            // 重设子选项区域的显示/隐藏状态
            if (subOption) {
                subOption.style.display = input.checked ? 'inline' : 'none';
            }
        }
    }
    // 是否显示提示
    showTips() {
        for (const item of this.tips) {
            if (!_utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.isPixiv()) {
                item.wrap.style.display = 'none';
            }
            else {
                item.wrap.style.display = _setting_Settings__WEBPACK_IMPORTED_MODULE_8__.settings[item.settingName] ? 'block' : 'none';
            }
        }
    }
}
new Form();


/***/ }),

/***/ "./src/ts/setting/FormHTML.ts":
/*!************************************!*\
  !*** ./src/ts/setting/FormHTML.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   formHtml: () => (/* binding */ formHtml)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");

// 已使用的最大编号是 87
const formHtml = `<form class="settingForm">
  <div class="tabsContnet">
    <p class="option" data-no="1">
    <span class="setWantPageWrap has_tip" data-xztip="_抓取多少页面">
    <span class="settingNameStyle1"><span class="setWantPageTip1" data-xztext="_抓取多少页面"></span><span class="gray1"> ? </span></span>
    <input type="text" name="setWantPage" class="setinput_style1 blue setWantPage"
    value = '-1'>&nbsp;
    <span class="setWantPageTip2 gray1" data-xztext="_数字提示1"></span>
    </span>
    </p>

    <p class="option" data-no="2">
    <span class="settingNameStyle1">
    <span data-xztext="_作品类型"></span>
    </span>
    <input type="checkbox" name="downType0" id="setWorkType0" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0" aria-labelledby="setWorkType0"></span>
    <label for="setWorkType0" data-xztext="_插画"></label>
    <input type="checkbox" name="downType1" id="setWorkType1" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0" data-xztitle="_漫画"></span>
    <label for="setWorkType1" data-xztext="_漫画"></label>
    <input type="checkbox" name="downType2" id="setWorkType2" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setWorkType2" data-xztext="_动图"></label>
    <input type="checkbox" name="downType3" id="setWorkType3" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setWorkType3" data-xztext="_小说"></label>
    </p>

    <p class="option" data-no="44">
    <span class="settingNameStyle1">
    <span data-xztext="_年龄限制"></span>
    </span>
    <input type="checkbox" name="downAllAges" id="downAllAges" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="downAllAges" data-xztext="_全年龄"></label>
    <input type="checkbox" name="downR18" id="downR18" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="downR18"> R-18</label>
    <input type="checkbox" name="downR18G" id="downR18G" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="downR18G"> R-18G</label>
    </p>
    
    <p class="option" data-no="81">
    <span class="settingNameStyle1">
    <span data-xztext="_AI作品"></span>
    </span>
    <input type="checkbox" name="AIGenerated" id="AIGenerated" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="AIGenerated" data-xztext="_AI生成"></label>
    <input type="checkbox" name="notAIGenerated" id="notAIGenerated" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="notAIGenerated" data-xztext="_非AI生成"></label>
    <input type="checkbox" name="UnknownAI" id="UnknownAI" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="UnknownAI" data-xztext="_未知" class="has_tip" data-xztip="_AI未知作品的说明"></label>
    </p>

    <p class="option" data-no="6">
    <span class="settingNameStyle1">
    <span data-xztext="_收藏状态"></span>
    </span>
    <input type="checkbox" name="downNotBookmarked" id="setDownNotBookmarked" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownNotBookmarked" data-xztext="_未收藏"></label>
    <input type="checkbox" name="downBookmarked" id="setDownBookmarked" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownBookmarked" data-xztext="_已收藏"></label>
    </p>

    <p class="option" data-no="23">
    <span class="settingNameStyle1">
    <span data-xztext="_图片色彩"></span>
    </span>
    <input type="checkbox" name="downColorImg" id="setDownColorImg" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownColorImg" data-xztext="_彩色图片"></label>
    <input type="checkbox" name="downBlackWhiteImg" id="setDownBlackWhiteImg" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownBlackWhiteImg" data-xztext="_黑白图片"></label>
    </p>

    <p class="option" data-no="21">
    <span class="settingNameStyle1">
    <span data-xztext="_图片数量"></span>
    </span>
    <input type="checkbox" name="downSingleImg" id="setDownSingleImg" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownSingleImg" data-xztext="_单图作品"></label>
    <input type="checkbox" name="downMultiImg" id="setDownMultiImg" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setDownMultiImg" data-xztext="_多图作品"></label>
    </p>

    <p class="option" data-no="51">
    <span class="has_tip settingNameStyle1" data-xztip="_显示高级设置说明">
    <span data-xztext="_显示高级设置"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="showAdvancedSettings" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="3">
    <span class="has_tip settingNameStyle1" data-xztip="_必须大于0">
    <span data-xztext="_多图作品只下载前几张图片"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="firstFewImagesSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="firstFewImagesSwitch">
    <input type="text" name="firstFewImages" class="setinput_style1 blue" value="1">
    </span>
    </p>

    <p class="option" data-no="47">
    <span class="has_tip settingNameStyle1" data-xztip="_超出此限制的多图作品不会被下载">
    <span data-xztext="_多图作品的图片数量上限"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="multiImageWorkImageLimitSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="multiImageWorkImageLimitSwitch">
    &lt;=&nbsp;
    <input type="text" name="multiImageWorkImageLimit" class="setinput_style1 blue" value="1">
    </span>
    </p>

    <p class="option" data-no="5">
    <span class="has_tip settingNameStyle1" data-xztip="_设置收藏数量的提示">
    <span data-xztext="_收藏数量"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="BMKNumSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="BMKNumSwitch">
    &gt;=&nbsp;
    <input type="text" name="BMKNumMin" class="setinput_style1 blue bmkNum" value="0">
    &lt;=&nbsp;
    <input type="text" name="BMKNumMax" class="setinput_style1 blue bmkNum" value="${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.BookmarkCountLimit}">
&nbsp;<span data-xztext="_或者"></span>
    <span class="has_tip settingNameStyle1" data-xztip="_日均收藏数量的提示">
    <span data-xztext="_日均收藏数量"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="BMKNumAverageSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="BMKNumAverageSwitch">
      <input type="text" name="BMKNumAverage" class="setinput_style1 blue bmkNum" value="600">
    </span>
    </span>
    </p>

    <p class="option" data-no="7">
    <span class="has_tip settingNameStyle1" data-xztip="_筛选宽高的提示文字">
    <span data-xztext="_图片的宽高"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="setWHSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="setWHSwitch">

    <input type="radio" name="widthHeightLimit" id="widthHeightLimit1" class="need_beautify radio" value=">=" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="widthHeightLimit1">&gt;=</label>

    <input type="radio" name="widthHeightLimit" id="widthHeightLimit2" class="need_beautify radio" value="=">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="widthHeightLimit2">=</label>
    
    <input type="radio" name="widthHeightLimit" id="widthHeightLimit3" class="need_beautify radio" value="<=">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="widthHeightLimit3">&lt;=</label>

    <span data-xztext="_宽度"></span>
    <input type="text" name="setWidth" class="setinput_style1 blue" value="0">
    <input type="radio" name="setWidthAndOr" id="setWidth_AndOr1" class="need_beautify radio" value="&" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="setWidth_AndOr1" data-xztext="_并且"></label>
    <input type="radio" name="setWidthAndOr" id="setWidth_AndOr2" class="need_beautify radio" value="|">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="setWidth_AndOr2" data-xztext="_或者"></label>
    <span data-xztext="_高度"></span>
    <input type="text" name="setHeight" class="setinput_style1 blue" value="0">
    </span>
    </p>

    <p class="option" data-no="8">
    <span class="has_tip settingNameStyle1" data-xztip="_设置宽高比例Title">
    <span data-xztext="_图片的宽高比例"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="ratioSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="ratioSwitch">
    <input type="radio" name="ratio" id="ratio1" class="need_beautify radio" value="horizontal">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ratio1" data-xztext="_横图"></label>

    <input type="radio" name="ratio" id="ratio2" class="need_beautify radio" value="vertical">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ratio2" data-xztext="_竖图"></label>
    
    <input type="radio" name="ratio" id="ratio0" class="need_beautify radio" value="square">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ratio0" data-xztext="_正方形"></label>

    <input type="radio" name="ratio" id="ratio3" class="need_beautify radio" value="userSet">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ratio3" data-xztext="_宽高比"></label>

    <!-- 这里使用了一个不可见的开关 userSetChecked，用来根据 radio 的值来控制子选项的显示或隐藏 -->
    <input type="checkbox" name="userSetChecked" class="need_beautify checkbox_switch" style="display:none;">
    <span class="beautify_switch" tabindex="0" style="display:none;"></span>

    <span class="subOptionWrap" data-show="userSetChecked">
    <input type="radio" name="userRatioLimit" id="userRatioLimit1" class="need_beautify radio" value=">=" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userRatioLimit1">&gt;=</label>

    <input type="radio" name="userRatioLimit" id="userRatioLimit2" class="need_beautify radio" value="=">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userRatioLimit2">=</label>
    
    <input type="radio" name="userRatioLimit" id="userRatioLimit3" class="need_beautify radio" value="<=">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userRatioLimit3">&lt;=</label>

    <input type="text" name="userRatio" class="setinput_style1 blue" value="1.4">
    </span>

    </span>
    </p>

    <p class="option" data-no="9">
    <span class="has_tip settingNameStyle1" data-xztip="_设置id范围提示">
    <span data-xztext="_id范围"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="idRangeSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="idRangeSwitch">
    <input type="radio" name="idRange" id="idRange1" class="need_beautify radio" value=">" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="idRange1">&gt;</label>
    <input type="radio" name="idRange" id="idRange2" class="need_beautify radio" value="<">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="idRange2">&lt;</label>
    <input type="text" name="idRangeInput" class="setinput_style1 w100 blue" value="" placeholder="100000000">
    </span>
    </p>

    <p class="option" data-no="10">
    <span class="has_tip settingNameStyle1" data-xztip="_设置投稿时间提示">
    <span data-xztext="_投稿时间"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="postDate" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="postDate">
    <input type="datetime-local" name="postDateStart" placeholder="yyyy-MM-dd HH:mm" class="setinput_style1 postDate blue" value="">
    &nbsp;-&nbsp;
    <input type="datetime-local" name="postDateEnd" placeholder="yyyy-MM-dd HH:mm" class="setinput_style1 postDate blue" value="">
    </span>
    </p>

    <p class="option" data-no="11">
    <span class="has_tip settingNameStyle1" data-xztip="_必须tag的提示文字">
    <span data-xztext="_必须含有tag"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="needTagSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="needTagSwitch">
    <input type="radio" name="needTagMode" id="needTagMode1" class="need_beautify radio" value="all" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="needTagMode1" data-xztext="_全部"></label>
    <input type="radio" name="needTagMode" id="needTagMode2" class="need_beautify radio" value="one">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="needTagMode2" data-xztext="_任一"></label>
    <input type="text" name="needTag" class="setinput_style1 blue setinput_tag" placeholder="tag1,tag2,tag3">
    </span>
    </p>

    <p class="option" data-no="12">
    <span class="has_tip settingNameStyle1" data-xztip="_排除tag的提示文字">
    <span data-xztext="_不能含有tag"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="notNeedTagSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="notNeedTagSwitch">
    <span class="gray1" data-xztext="_任一"></span>&nbsp;
    <input type="radio" id="tagMatchMode1" class="need_beautify radio" name="tagMatchMode" value="partial" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="tagMatchMode1" data-xztext="_部分一致"></label>
    <input type="radio" id="tagMatchMode2" class="need_beautify radio" name="tagMatchMode" value="whole" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="tagMatchMode2" data-xztext="_完全一致"></label>
    <br>
    <textarea class="centerPanelTextArea beautify_scrollbar" name="notNeedTag" rows="1" placeholder="tag1,tag2,tag3"></textarea>
    </span>
    </p>

    <div class="centerWrap_btns">
      <slot data-name="stopCrawl"></slot>
      <slot data-name="crawlBtns"></slot>
      <slot data-name="selectWorkBtns"></slot>
    </div>
  </div>
  <div class="tabsContnet">
  <p class="option" data-no="13">
    <span class="settingNameStyle1" data-xztext="_命名规则"></span>
    <input type="text" name="userSetName" class="setinput_style1 blue fileNameRule" value="{page_title}/{id}">
    &nbsp;
    <select name="fileNameSelect" class="beautify_scrollbar">
      <option value="default">…</option>
      <option value="{id}">{id}</option>
      <option value="{user}">{user}</option>
      <option value="{user_id}">{user_id}</option>
      <option value="{title}">{title}</option>
      <option value="{page_title}">{page_title}</option>
      <option value="{tags}">{tags}</option>
      <option value="{tags_translate}">{tags_translate}</option>
      <option value="{tags_transl_only}">{tags_transl_only}</option>
      <option value="{page_tag}">{page_tag}</option>
      <option value="{type}">{type}</option>
      <option value="{AI}">{AI}</option>
      <option value="{like}">{like}</option>
      <option value="{bmk}">{bmk}</option>
      <option value="{bmk_1000}">{bmk_1000}</option>
      <option value="{bmk_id}">{bmk_id}</option>
      <option value="{view}">{view}</option>
      <option value="{rank}">{rank}</option>
      <option value="{date}">{date}</option>
      <option value="{upload_date}">{upload_date}</option>
      <option value="{task_date}">{task_date}</option>
      <option value="{px}">{px}</option>
      <option value="{series_title}">{series_title}</option>
      <option value="{series_order}">{series_order}</option>
      <option value="{series_id}">{series_id}</option>
      <option value="{id_num}">{id_num}</option>
      <option value="{p_num}">{p_num}</option>
      </select>
    &nbsp;
    <slot data-name="saveNamingRule"></slot>
    <button class="showFileNameTip textButton" type="button" data-xztext="_提示2"></button>
    </p>
    <p class="fileNameTip tip">
    <span data-xztext="_设置文件夹名的提示"></span>
    <span>{user}<span class="key">/</span>{id}</span>
    <br>
    <span data-xztext="_命名标记提醒"></span>
    <br>
    * <span data-xztext="_有些标记并不总是可用的提醒"></span>
    <br>
    <span class="blue">{id}</span>
    <span data-xztext="_命名标记id"></span>
    <br>
    <span class="blue">{user}</span>
    <span data-xztext="_命名标记user"></span>
    <br>
    <span class="blue">{user_id}</span>
    <span data-xztext="_用户id"></span>
    <br>
    <span class="blue">{title}</span>
    <span data-xztext="_命名标记title"></span>
    <br>
    <span class="blue">{tags}</span>
    <span data-xztext="_命名标记tags"></span>
    <br>
    <span class="blue">{tags_translate}</span>
    <span data-xztext="_命名标记tags_trans"></span>
    <br>
    <span class="blue">{tags_transl_only}</span>
    <span data-xztext="_命名标记tags_transl_only"></span>
    <br>
    <span class="blue">{page_title}</span>
    <span data-xztext="_文件夹标记PTitle"></span>
    <br>
    * <span class="blue">{page_tag}</span>
    <span data-xztext="_文件夹标记PTag"></span>
    <br>
    <span class="blue">{type}</span>
    <span data-xztext="_命名标记type"></span>
    <br>
    * <span class="blue">{AI}</span>
    <span data-xztext="_命名标记AI"></span>
    <br>
    <span class="blue">{like}</span>
    <span data-xztext="_命名标记like"></span>
    <br>
    <span class="blue">{bmk}</span>
    <span data-xztext="_命名标记bmk"></span>
    <br>
    <span class="blue">{bmk_1000}</span>
    <span data-xztext="_命名标记bmk_1000"></span>
    <br>
    <span class="blue">{bmk_id}</span>
    <span data-xztext="_命名标记bmk_id"></span>
    <br>
    <span class="blue">{view}</span>
    <span data-xztext="_命名标记view"></span>
    <br>
    * <span class="blue">{rank}</span>
    <span data-xztext="_命名标记rank"></span>
    <br>
    <span class="blue">{date}</span>
    <span data-xztext="_命名标记date"></span>
    <br>
    <span class="blue">{upload_date}</span>
    <span data-xztext="_命名标记upload_date"></span>
    <br>
    <span class="blue">{task_date}</span>
    <span data-xztext="_命名标记taskDate"></span>
    <br>
    <span class="blue">{px}</span>
    <span data-xztext="_命名标记px"></span>
    <br>
    * <span class="blue">{series_title}</span>
    <span data-xztext="_命名标记seriesTitle"></span>
    <br>
    * <span class="blue">{series_order}</span>
    <span data-xztext="_命名标记seriesOrder"></span>
    <br>
    * <span class="blue">{series_id}</span>
    <span data-xztext="_命名标记seriesId"></span>
    <br>
    <span class="blue">{id_num}</span>
    <span data-xztext="_命名标记id_num"></span>
    <br>
    <span class="blue">{p_num}</span>
    <span data-xztext="_命名标记p_num"></span>
    </p>

    <p class="option" data-no="50">
    <span class="settingNameStyle1" data-xztext="_在不同的页面类型中使用不同的命名规则"></span>
    <input type="checkbox" name="setNameRuleForEachPageType" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>
    
    <p class="option" data-no="64">
    <span class="settingNameStyle1" data-xztext="_只有一个抓取结果时不建立文件夹"></span>
    <input type="checkbox" name="notFolderWhenOneFile" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>
    
    <p class="option" data-no="38">
    <span class="settingNameStyle1" data-xztext="_把r18作品存入指定的文件夹里"></span>
    <input type="checkbox" name="r18Folder" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="r18Folder">
    <span data-xztext="_目录名"></span>
    <input type="text" name="r18FolderName" class="setinput_style1 blue" style="width:150px;min-width: 150px;" value="[R-18&R-18G]">
    </span>
    </p>

    <p class="option" data-no="16">
    <span class="settingNameStyle1"">
    <span data-xztext="_下载线程"></span>
    </span>
    <input type="text" name="downloadThread" class="has_tip setinput_style1 blue" data-xztip="_下载线程的说明" value="5">
    </p>

    <p class="option" data-no="17">
    <span class="has_tip settingNameStyle1" data-xztip="_自动开始下载的提示">
    <span data-xztext="_自动开始下载"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="autoStartDownload" id="setQuietDownload" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="33">
    <span class="has_tip settingNameStyle1" data-xztip="_下载之后收藏作品的提示">
    <span data-xztext="_下载之后收藏作品"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="bmkAfterDL" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="52">
    <span class="settingNameStyle1" data-xztext="_下载完成后显示通知"></span>
    <input type="checkbox" name="showNotificationAfterDownloadComplete" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <div class="centerWrap_btns">
    <slot data-name="exportResult"></slot>
    <slot data-name="namingBtns"></slot>
    </div>

    <slot data-name="downloadArea"></slot>
    <slot data-name="progressBar"></slot>
  </div>
  
  <div class="tabsContnet">

    <div class="centerWrap_btns">
      <slot data-name="otherBtns"></slot>
    </div>

    <p class="option" data-no="57">
    <span class="has_tip settingNameStyle1" data-xztip="_显示高级设置说明">
    <span data-xztext="_显示高级设置"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="showAdvancedSettings" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option settingCategoryName" data-no="59">
      <span data-xztext="_抓取"></span>
    </p>

    <p class="option" data-no="75">
    <span class="has_tip settingNameStyle1" data-xztip="_减慢抓取速度的说明">
    <span data-xztext="_减慢抓取速度"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="slowCrawl" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    
    <span class="subOptionWrap" data-show="slowCrawl">
    <span data-xztext="_当作品数量大于"></span>
    <input type="text" name="slowCrawlOnWorksNumber" class="setinput_style1 blue" value="100" style="width:60px;min-width: 60px;">
    </span>
    </p>

    <p class="option" data-no="69">
    <span class="settingNameStyle1" data-xztext="_不抓取多图作品的最后一张图片"></span>
    <input type="checkbox" name="doNotDownloadLastImageOfMultiImageWork" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>
    
    <p class="option" data-no="79">
    <span class="settingNameStyle1">
    <span data-xztext="_特定用户的多图作品不下载最后几张图片"></span>
    <span class="gray1"> ? </span>
    </span>
    <slot data-name="DoNotDownloadLastFewImagesSlot"></slot>
    </p>

    <p class="option" data-no="35">
    <span class="has_tip settingNameStyle1" data-xztip="_用户阻止名单的说明">
    <span data-xztext="_用户阻止名单"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="userBlockList" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="userBlockList">
    <input type="text" name="blockList" class="has_tip setinput_style1 blue setinput_tag" data-xztip="_用户阻止名单的说明" data-xzplaceholder="_用户ID必须是数字">
    </span>
    </p>

    <p class="option" data-no="39">
    <span class="settingNameStyle1" data-xztext="_针对特定用户屏蔽tag"></span>
    <input type="checkbox" name="blockTagsForSpecificUser" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="blockTagsForSpecificUser">
    <slot data-name="blockTagsForSpecificUser"></slot>
    </span>
    </p>
    
    <p class="option" data-no="74">
    <span class="has_tip settingNameStyle1" data-xztext="_定时抓取的间隔时间" data-xztip="_定时抓取说明"></span>
    <input type="text" name="timedCrawlInterval" class="setinput_style1 blue" value="120">
    <span class="settingNameStyle1" data-xztext="_分钟"></span>
    </span>
    </p>
    
    <p class="option" data-no="54">
    <span class="settingNameStyle1" data-xztext="_自动导出抓取结果"></span>
    <input type="checkbox" name="autoExportResult" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="autoExportResult">
    <span data-xztext="_抓取结果"></span>
    <span>&gt;</span>
    <input type="text" name="autoExportResultNumber" class="setinput_style1 blue" value="1" style="width:30px;min-width: 30px;">
    <span>&nbsp;</span>
    <span class="settingNameStyle1" data-xztext="_文件格式"> </span>
    <input type="checkbox" name="autoExportResultCSV" id="autoExportResultCSV" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="autoExportResultCSV"> CSV </label>
    <input type="checkbox" name="autoExportResultJSON" id="autoExportResultJSON" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="autoExportResultJSON"> JSON </label>
    </span>
    </p>

    <p class="option settingCategoryName" data-no="65">
      <span data-xztext="_命名"></span>
    </p>

    <p class="option" data-no="42">
    <span class="settingNameStyle1" data-xztext="_根据作品类型自动建立文件夹"></span>
    <input type="checkbox" name="createFolderByType" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="createFolderByType">
    <input type="checkbox" name="createFolderByTypeIllust" id="createFolderByTypeIllust" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="createFolderByTypeIllust" class="has_tip" data-tip="${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.worksTypeName[0]}">
    <span data-xztext="_插画"></span></label>
    <input type="checkbox" name="createFolderByTypeManga" id="createFolderByTypeManga" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="createFolderByTypeManga" class="has_tip" data-tip="${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.worksTypeName[1]}">
    <span data-xztext="_漫画"></span></label>
    <input type="checkbox" name="createFolderByTypeUgoira" id="createFolderByTypeUgoira" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="createFolderByTypeUgoira" class="has_tip" data-tip="${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.worksTypeName[2]}">
    <span data-xztext="_动图"></span></label>
    <input type="checkbox" name="createFolderByTypeNovel" id="createFolderByTypeNovel" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="createFolderByTypeNovel" class="has_tip" data-tip="${_Config__WEBPACK_IMPORTED_MODULE_0__.Config.worksTypeName[3]}">
    <span data-xztext="_小说"></span></label>
    </p>

    <p class="option" data-no="19">
    <span class="settingNameStyle1" data-xztext="_为作品建立单独的文件夹"></span>
    <input type="checkbox" name="workDir" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="workDir">
    <label for="workDirFileNumber" data-xztext="_文件数量大于"></label>
    <input type="text" name="workDirFileNumber" id="workDirFileNumber" class="setinput_style1 blue" value="1" style="width:30px;min-width: 30px;">
    <span>&nbsp;</span>
    <label for="workDirNameRule" data-xztext="_目录名"></label>
    <input class="has_tip setinput_style1 blue" type="text"  data-xztip="_为作品建立单独的文件夹说明" name="workDirNameRule" id="workDirNameRule" value="{id_num}">
    </span>
    </p>

    <p class="option" data-no="43">
    <span class="has_tip settingNameStyle1" data-xztip="_使用匹配的tag建立文件夹的说明">
    <span data-xztext="_使用第一个匹配的tag建立文件夹"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="createFolderByTag" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="createFolderByTag">
    <span class="gray1" data-xztext="_tag用逗号分割"></span>
    <br>
    <textarea class="centerPanelTextArea beautify_scrollbar" name="createFolderTagList" rows="1" placeholder="tag1,tag2,tag3"></textarea>
    </span>
    </p>

    <p class="option" data-no="80">
    <span class="settingNameStyle1" data-xztext="_如果作品含有某些标签则对这个作品使用另一种命名规则"></span>
    <input type="checkbox" name="UseDifferentNameRuleIfWorkHasTagSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="UseDifferentNameRuleIfWorkHasTagSwitch">
    <slot data-name="UseDifferentNameRuleIfWorkHasTagSlot"></slot>
    </span>
    </p>

    <p class="option" data-no="22">
    <span class="has_tip settingNameStyle1" data-xztip="_第一张图不带序号说明">
    <span data-xztext="_第一张图不带序号"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="noSerialNo" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="noSerialNo">
      <input type="checkbox" name="noSerialNoForSingleImg" id="setNoSerialNoForSingleImg" class="need_beautify checkbox_common" checked>
      <span class="beautify_checkbox" tabindex="0"></span>
      <label for="setNoSerialNoForSingleImg" data-xztext="_单图作品"></label>
      <input type="checkbox" name="noSerialNoForMultiImg" id="setNoSerialNoForMultiImg" class="need_beautify checkbox_common" checked>
      <span class="beautify_checkbox" tabindex="0"></span>
      <label for="setNoSerialNoForMultiImg" data-xztext="_多图作品"></label>
    </span>
    </p>
    
    <p class="option" data-no="46">
    <span class="has_tip settingNameStyle1" data-xztip="_在序号前面填充0的说明">
    <span data-xztext="_在序号前面填充0"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="zeroPadding" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="zeroPadding">
    <span data-xztext="_序号总长度"></span>
    <input type="text" name="zeroPaddingLength" class="setinput_style1 blue" value="3" style="width:30px;min-width: 30px;">
    </span>
    </p>

    <p class="option" data-no="14">
    <span class="has_tip settingNameStyle1" data-xztip="_添加字段名称提示">
    <span data-xztext="_添加命名标记前缀"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="tagNameToFileName" id="setTagNameToFileName" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="29">
    <span class="settingNameStyle1" data-xztext="_文件名长度限制"></span>
    <input type="checkbox" name="fileNameLengthLimitSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="fileNameLengthLimitSwitch">
    <input type="text" name="fileNameLengthLimit" class="setinput_style1 blue" value="200">
    </span>
    </p>

    <p class="option" data-no="83">
    <span class="settingNameStyle1" data-xztext="_标签分隔符号"></span>
    <input type="text" name="tagsSeparator" class="setinput_style1 blue" value=",">
    <button type="button" class="gray1 textButton showTagsSeparatorTip" data-xztext="_提示"></button>
    </p>
    <p class="tagsSeparatorTip tip" style="display:none">
    <span data-xztext="_标签分隔符号提示"></span>
    </p>
    
    <p class="option" data-no="67">
    <span class="has_tip settingNameStyle1" data-xztip="_移除用户名中的at和后续字符的说明">
    <span data-xztext="_移除用户名中的at和后续字符"></span>
    <span class="gray1"> ? </span>
    </span>
    <input type="checkbox" name="removeAtFromUsername" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="66">
    <span class="has_tip settingNameStyle1" data-xztip="_自定义用户名的说明">
    <span data-xztext="_自定义用户名"></span>
    <span class="gray1"> ? </span>
    </span>
    <slot data-name="setUserNameSlot"></slot>
    </p>

    <p class="option settingCategoryName" data-no="58">
      <span data-xztext="_下载"></span>
    </p>
    
    <p class="option" data-no="76">
    <span class="settingNameStyle1">
    <span data-xztext="_点击收藏按钮时下载作品"></span>
    </span>
    <input type="checkbox" name="downloadOnClickBookmark" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="77">
    <span class="settingNameStyle1">
    <span data-xztext="_点击点赞按钮时下载作品"></span>
    </span>
    <input type="checkbox" name="downloadOnClickLike" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="4">
    <span class="has_tip settingNameStyle1" data-xztip="_动图保存格式title">
    <span data-xztext="_动图保存格式"></span>
    <span class="gray1"> ? </span></span>
    <input type="radio" name="ugoiraSaveAs" id="ugoiraSaveAs1" class="need_beautify radio" value="webm" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ugoiraSaveAs1" data-xztext="_webmVideo"></label>
    <input type="radio" name="ugoiraSaveAs" id="ugoiraSaveAs3" class="need_beautify radio" value="gif"> 
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ugoiraSaveAs3" data-xztext="_gif"></label>
    <input type="radio" name="ugoiraSaveAs" id="ugoiraSaveAs4" class="need_beautify radio" value="png"> 
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ugoiraSaveAs4" class="has_tip" data-xztip="_无损" data-xztext="_apng"></label>
    <input type="radio" name="ugoiraSaveAs" id="ugoiraSaveAs2" class="need_beautify radio" value="zip"> 
    <span class="beautify_radio" tabindex="0"></span>
    <label for="ugoiraSaveAs2" data-xztext="_zipFile"></label>
    </p>

    <p class="option" data-no="24">
    <span class="settingNameStyle1" data-xztext="_同时转换多少个动图"></span>
    <input type="text" name="convertUgoiraThread" class="has_tip setinput_style1 blue" data-xztip="_同时转换多少个动图警告" value="1">
    </span>
    </p>

    <p class="option" data-no="26">
    <span class="settingNameStyle1" data-xztext="_小说保存格式"></span>
    <input type="radio" name="novelSaveAs" id="novelSaveAs1" class="need_beautify radio" value="txt" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="novelSaveAs1"> TXT </label>
    <input type="radio" name="novelSaveAs" id="novelSaveAs2" class="need_beautify radio" value="epub"> 
    <span class="beautify_radio" tabindex="0"></span>
    <label for="novelSaveAs2"> EPUB </label>
    </p>
    
    <p class="option" data-no="27">
    <span class="has_tip settingNameStyle1" data-xztip="_在小说里保存元数据提示">
    <span data-xztext="_在小说里保存元数据"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="saveNovelMeta" class="need_beautify checkbox_switch" >
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="70">
    <span class="settingNameStyle1" data-xztext="_下载小说的封面图片"></span>
    <input type="checkbox" name="downloadNovelCoverImage" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="72">
    <span class="settingNameStyle1" data-xztext="_下载小说里的内嵌图片"></span>
    <input type="checkbox" name="downloadNovelEmbeddedImage" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="49">
    <span class="has_tip settingNameStyle1" data-xztip="_保存作品的元数据说明">
    <span data-xztext="_保存作品的元数据"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="saveMetaType0" id="setSaveMetaType0" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setSaveMetaType0" data-xztext="_插画"></label>
    <input type="checkbox" name="saveMetaType1" id="setSaveMetaType1" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setSaveMetaType1" data-xztext="_漫画"></label>
    <input type="checkbox" name="saveMetaType2" id="setSaveMetaType2" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setSaveMetaType2" data-xztext="_动图"></label>
    <input type="checkbox" name="saveMetaType3" id="setSaveMetaType3" class="need_beautify checkbox_common">
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="setSaveMetaType3" data-xztext="_小说"></label>
    </p>

    <p class="option" data-no="30">
    <span class="settingNameStyle1" data-xztext="_图片尺寸"></span>
    <input type="radio" name="imageSize" id="imageSize1" class="need_beautify radio" value="original" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="imageSize1" data-xztext="_原图"></label>
    <input type="radio" name="imageSize" id="imageSize2" class="need_beautify radio" value="regular">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="imageSize2" data-xztext="_普通"></label>
    <span class="gray1">(1200px)</span>
    <input type="radio" name="imageSize" id="imageSize3" class="need_beautify radio" value="small">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="imageSize3" data-xztext="_小图"></label>
    <span class="gray1">(540px)</span>
    <input type="radio" name="imageSize" id="imageSize4" class="need_beautify radio" value="thumb">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="imageSize4" data-xztext="_方形缩略图"></label>
    <span class="gray1">(250px)</span>
    </p>

    <p class="option" data-no="25">
    <span class="has_tip settingNameStyle1" data-xztip="_不符合要求的文件不会被保存">
    <span data-xztext="_文件体积限制"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="sizeSwitch" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="sizeSwitch">
    <input type="text" name="sizeMin" class="setinput_style1 blue" value="0">MiB
    &nbsp;-&nbsp;
    <input type="text" name="sizeMax" class="setinput_style1 blue" value="100">MiB
    </span>
    </p>

    <p class="option" data-no="82">
    <span class="settingNameStyle1" data-xztext="_文件下载顺序"></span>
    <input type="checkbox" name="setFileDownloadOrder" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="setFileDownloadOrder">
    
    <span class="settingNameStyle1" data-xztext="_排序依据"></span>
    <input type="radio" name="downloadOrderSortBy" id="downloadOrderSortBy1" class="need_beautify radio" value="ID" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="downloadOrderSortBy1" data-xztext="_作品ID"></label>
    <input type="radio" name="downloadOrderSortBy" id="downloadOrderSortBy2" class="need_beautify radio" value="bookmarkCount">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="downloadOrderSortBy2" data-xztext="_收藏数量2"></label>
    <input type="radio" name="downloadOrderSortBy" id="downloadOrderSortBy3" class="need_beautify radio" value="bookmarkID">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="downloadOrderSortBy3" data-xztext="_收藏时间"></label>

    <span class="verticalSplit"></span>

    <input type="radio" name="downloadOrder" id="downloadOrder1" class="need_beautify radio" value="desc" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="downloadOrder1" data-xztext="_降序"></label>
    <input type="radio" name="downloadOrder" id="downloadOrder2" class="need_beautify radio" value="asc">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="downloadOrder2" data-xztext="_升序"></label>

    </span>
    </p>

    <p class="option" data-no="28">
    <span class="has_tip settingNameStyle1" data-xztip="_不下载重复文件的提示">
    <span data-xztext="_不下载重复文件"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="deduplication" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    <span class="subOptionWrap" data-show="deduplication">
    &nbsp; <span data-xztext="_策略"></span>
    <input type="radio" name="dupliStrategy" id="dupliStrategy1" class="need_beautify radio" value="strict" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label class="has_tip" for="dupliStrategy1" data-xztip="_严格模式说明" data-xztext="_严格"></label>
    <input type="radio" name="dupliStrategy" id="dupliStrategy2" class="need_beautify radio" value="loose">
    <span class="beautify_radio" tabindex="0"></span>
    <label class="has_tip" for="dupliStrategy2" data-xztip="_宽松模式说明" data-xztext="_宽松"></label>
    <button class="textButton gray1" type="button" id="exportDownloadRecord" data-xztext="_导出"></button>
    <button class="textButton gray1" type="button" id="importDownloadRecord" data-xztext="_导入"></button>
    <button class="textButton gray1" type="button" id="clearDownloadRecord" data-xztext="_清除"></button>
    </span>
    </p>

    <p class="option" data-no="73">
    <span class="settingNameStyle1" data-xztext="_隐藏浏览器底部的下载栏"></span>
    <input type="checkbox" name="hiddenBrowserDownloadBar" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="85">
    <span class="settingNameStyle1" data-xztext="_导出ID列表"></span>
    <input type="checkbox" name="exportIDList" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option settingCategoryName" data-no="60">
      <span data-xztext="_增强"></span>
    </p>
    
    <p class="option" data-no="84">
    <span class="has_tip settingNameStyle1" data-xztip="_高亮关注的用户的说明">
    <span data-xztext="_高亮关注的用户"></span>
    <span class="gray1"> ? </span>
    </span>
    <input type="checkbox" name="highlightFollowingUsers" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>
    
    <p class="option" data-no="87">
    <span class="has_tip settingNameStyle1" data-xztip="_预览作品的详细信息的说明">
    <span data-xztext="_预览作品的详细信息"></span>
    <span class="gray1"> ? </span>
    </span>
    <input type="checkbox" name="PreviewWorkDetailInfo" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="PreviewWorkDetailInfo">

    <span data-xztext="_显示区域宽度"></span>&nbsp;
    <input type="text" name="PreviewDetailInfoWidth" class="setinput_style1 blue" value="500" style="width:40px;min-width: 40px;">
    <span>&nbsp;px</span>

    </span>
    </p>

    <p class="option" data-no="68">
    <span class="settingNameStyle1" data-xztext="_显示更大的缩略图"></span>
    <input type="checkbox" name="showLargerThumbnails" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="showLargerThumbnails">
    <label for="doubleWidthThumb" data-xztext="_横图占用二倍宽度"></label>
    <input type="checkbox" name="doubleWidthThumb" id="doubleWidthThumb" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </span>
    </p>
    
    <p class="option" data-no="63">
    <span class="settingNameStyle1" data-xztext="_替换方形缩略图以显示图片比例"></span>
    <input type="checkbox" name="replaceSquareThumb" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="55">
    <span class="settingNameStyle1 has_tip" data-xztip="_快捷键AltP">
    <span data-xztext="_预览作品"></span>
    </span>
    <input type="checkbox" name="PreviewWork" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="PreviewWork">

    <label for="wheelScrollSwitchImageOnPreviewWork" class="has_tip" data-xztext="_使用鼠标滚轮切换作品里的图片" data-xztip="_这可能会阻止页面滚动"></label>
    <input type="checkbox" name="wheelScrollSwitchImageOnPreviewWork" id="wheelScrollSwitchImageOnPreviewWork" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="verticalSplit"></span>

    <span data-xztext="_等待时间"></span>&nbsp;
    <input type="text" name="previewWorkWait" class="setinput_style1 blue" value="400" style="width:40px;min-width: 40px;">
    <span>&nbsp;ms</span>

    <span class="verticalSplit"></span>

    <label for="showPreviewWorkTip" data-xztext="_显示摘要信息"></label>
    <input type="checkbox" name="showPreviewWorkTip" id="showPreviewWorkTip" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="verticalSplit"></span>

    <span class="settingNameStyle1" data-xztext="_图片尺寸2"></span>
    <input type="radio" name="prevWorkSize" id="prevWorkSize1" class="need_beautify radio" value="original">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="prevWorkSize1" data-xztext="_原图"></label>
    <input type="radio" name="prevWorkSize" id="prevWorkSize2" class="need_beautify radio" value="regular" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="prevWorkSize2" data-xztext="_普通"></label>
    </span>
    </p>

    <p class="option" data-no="71">
    <span class="settingNameStyle1" data-xztext="_预览动图"></span>
    <input type="checkbox" name="previewUgoira" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="62">
    <span class="settingNameStyle1" data-xztext="_长按右键显示大图"></span>
    <input type="checkbox" name="showOriginImage" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="showOriginImage">
    
    <span class="settingNameStyle1" data-xztext="_图片尺寸2"></span>
    <input type="radio" name="showOriginImageSize" id="showOriginImageSize1" class="need_beautify radio" value="original">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="showOriginImageSize1" data-xztext="_原图"></label>
    <input type="radio" name="showOriginImageSize" id="showOriginImageSize2" class="need_beautify radio" value="regular" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="showOriginImageSize2" data-xztext="_普通"></label>
    </p>

    <p class="option" data-no="40">
    <span class="settingNameStyle1" data-xztext="_在作品缩略图上显示放大按钮"></span>
    <input type="checkbox" name="magnifier" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="magnifier">

    <span class="settingNameStyle1" data-xztext="_位置"> </span>
    <input type="radio" name="magnifierPosition" id="magnifierPosition1" class="need_beautify radio" value="left">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="magnifierPosition1" data-xztext="_左"></label>
    <input type="radio" name="magnifierPosition" id="magnifierPosition2" class="need_beautify radio" value="right" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="magnifierPosition2" data-xztext="_右"></label>

    <span class="verticalSplit"></span>

    <span class="settingNameStyle1" data-xztext="_图片尺寸2"></span>
    <input type="radio" name="magnifierSize" id="magnifierSize1" class="need_beautify radio" value="original">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="magnifierSize1" data-xztext="_原图"></label>
    <input type="radio" name="magnifierSize" id="magnifierSize2" class="need_beautify radio" value="regular" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="magnifierSize2" data-xztext="_普通"></label>

    </span>
    </p>

    <p class="option" data-no="56">
    <span class="settingNameStyle1" data-xztext="_在作品缩略图上显示下载按钮"></span>
    <input type="checkbox" name="showDownloadBtnOnThumb" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="86">
    <span class="settingNameStyle1" data-xztext="_在多图作品页面里显示缩略图列表"></span>
    <input type="checkbox" name="displayThumbnailListOnMultiImageWorkPage" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="48">
    <span class="settingNameStyle1" data-xztext="_在搜索页面添加快捷搜索区域"></span>
    <input type="checkbox" name="showFastSearchArea" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="18">
    <span class="has_tip settingNameStyle1" data-xztip="_预览搜索结果说明">
    <span data-xztext="_预览搜索结果"></span>
    <span class="gray1"> ? </span></span>
    <input type="checkbox" name="previewResult" class="need_beautify checkbox_switch" checked>
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="previewResult">
    <span class="settingNameStyle1" data-xztext="_上限"> </span>
    <input type="text" name="previewResultLimit" class="setinput_style1 blue" value="1000" style="width:80px;min-width: 80px;">
    </span>
    </p>

    <p class="option" data-no="34">
    <span class="settingNameStyle1" data-xztext="_收藏设置"></span>
    
    <input type="radio" name="widthTag" id="widthTag1" class="need_beautify radio" value="yes" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="widthTag1" data-xztext="_添加tag"></label>
    <input type="radio" name="widthTag" id="widthTag2" class="need_beautify radio" value="no">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="widthTag2" data-xztext="_不添加tag"></label>

    <span class="verticalSplit"></span>
    
    <input type="radio" name="restrict" id="restrict1" class="need_beautify radio" value="no" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="restrict1" data-xztext="_公开"></label>
    <input type="radio" name="restrict" id="restrict2" class="need_beautify radio" value="yes">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="restrict2" data-xztext="_不公开"></label>
    </p>

    <p class="option settingCategoryName" data-no="61">
      <span data-xztext="_其他"></span>
    </p>

    <p class="option" data-no="31">
    <span class="settingNameStyle1" data-xztext="_日期格式"></span>
    <input type="text" name="dateFormat" class="setinput_style1 blue" style="width:250px;" value="YYYY-MM-DD">
    <button type="button" class="gray1 textButton showDateTip" data-xztext="_提示"></button>
    </p>
    <p class="dateFormatTip tip" style="display:none">
    <span data-xztext="_日期格式提示"></span>
    <br>
    <span class="blue">YYYY</span> <span>2021</span>
    <br>
    <span class="blue">YY</span> <span>21</span>
    <br>
    <span class="blue">MM</span> <span>04</span>
    <br>
    <span class="blue">MMM</span> <span>Apr</span>
    <br>
    <span class="blue">MMMM</span> <span>April</span>
    <br>
    <span class="blue">DD</span> <span>30</span>
    <br>
    <span class="blue">hh</span> <span>06</span>
    <br>
    <span class="blue">mm</span> <span>40</span>
    <br>
    <span class="blue">ss</span> <span>08</span>
    <br>
    </p>

    <p class="option" data-no="78">
    <span class="settingNameStyle1" data-xztext="_导出日志"></span>
    <input type="checkbox" name="exportLog" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="exportLog">

    <span class="settingNameStyle1" data-xztext="_导出时机"> </span>
    <input type="radio" name="exportLogTiming" id="exportLogTiming1" class="need_beautify radio" value="crawlComplete">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="exportLogTiming1" data-xztext="_抓取完毕2"></label>
    <input type="radio" name="exportLogTiming" id="exportLogTiming2" class="need_beautify radio" value="downloadComplete" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="exportLogTiming2" data-xztext="_下载完毕2"></label>

    <span class="verticalSplit"></span>

    <span class="settingNameStyle1" data-xztext="_日志类型"></span>
    <input type="checkbox" name="exportLogNormal" id="exportLogNormal" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="exportLogNormal" data-xztext="_正常"></label>
    <input type="checkbox" name="exportLogError" id="exportLogError" class="need_beautify checkbox_common" checked>
    <span class="beautify_checkbox" tabindex="0"></span>
    <label for="exportLogError" data-xztext="_错误"></label>

    <span class="verticalSplit"></span>

    <span data-xztext="_排除关键字"></span>&nbsp;
    <input type="text" name="exportLogExclude" class="setinput_style1 blue setinput_tag">

    </span>
    </p>

    <p class="option" data-no="36">
    <span class="settingNameStyle1" data-xztext="_颜色主题"></span>
    <input type="radio" name="theme" id="theme1" class="need_beautify radio" value="auto" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="theme1" data-xztext="_自动检测"></label>
    <input type="radio" name="theme" id="theme2" class="need_beautify radio" value="white">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="theme2">White</label>
    <input type="radio" name="theme" id="theme3" class="need_beautify radio" value="dark">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="theme3">Dark</label>
    </p>

    <p class="option" data-no="41">
    <span class="settingNameStyle1" data-xztext="_背景图片"> </span>
    <input type="checkbox" name="bgDisplay" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>

    <span class="subOptionWrap" data-show="bgDisplay">

    <button class="textButton gray1" type="button" id="selectBG" data-xztext="_选择文件"></button>
    <button class="textButton gray1" type="button" id="clearBG" data-xztext="_清除"></button>
    
    &nbsp;
    <span data-xztext="_对齐方式"></span>&nbsp;
    <input type="radio" name="bgPositionY" id="bgPosition1" class="need_beautify radio" value="center" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="bgPosition1" data-xztext="_居中"></label>
    <input type="radio" name="bgPositionY" id="bgPosition2" class="need_beautify radio" value="top">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="bgPosition2" data-xztext="_顶部"></label>
    <span data-xztext="_不透明度"></span>&nbsp;
    <input name="bgOpacity" type="range" />
    </span>
    </p>
    
    <p class="option" data-no="45">
    <span class="settingNameStyle1" data-xztext="_选项卡切换方式"></span>
    <input type="radio" name="switchTabBar" id="switchTabBar1" class="need_beautify radio" value="over" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="switchTabBar1" data-xztext="_鼠标经过"></label>
    <input type="radio" name="switchTabBar" id="switchTabBar2" class="need_beautify radio" value="click">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="switchTabBar2" data-xztext="_鼠标点击"></label>
    </p>

    <p class="option" data-no="53">
    <span class="settingNameStyle1" data-xztext="_高亮显示关键字"></span>
    <input type="checkbox" name="boldKeywords" class="need_beautify checkbox_switch">
    <span class="beautify_switch" tabindex="0"></span>
    </p>

    <p class="option" data-no="32">
    <span class="settingNameStyle1"><span class="key">Language</span></span>
    <input type="radio" name="userSetLang" id="userSetLang1" class="need_beautify radio" value="auto" checked>
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang1" data-xztext="_自动检测"></label>
    <input type="radio" name="userSetLang" id="userSetLang2" class="need_beautify radio" value="zh-cn">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang2">简体中文</label>
    <input type="radio" name="userSetLang" id="userSetLang3" class="need_beautify radio" value="zh-tw">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang3">繁體中文</label>
    <input type="radio" name="userSetLang" id="userSetLang4" class="need_beautify radio" value="ja">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang4">日本語</label>
    <input type="radio" name="userSetLang" id="userSetLang5" class="need_beautify radio" value="en">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang5">English</label>
    <input type="radio" name="userSetLang" id="userSetLang6" class="need_beautify radio" value="ko">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang6">한국어</label>
    <input type="radio" name="userSetLang" id="userSetLang7" class="need_beautify radio" value="ru">
    <span class="beautify_radio" tabindex="0"></span>
    <label for="userSetLang7">Русский</label>
    </p>

    <p class="option" data-no="37">
    <span class="settingNameStyle1" data-xztext="_管理设置"></span>
    <button class="textButton gray1" type="button" id="exportSettings" data-xztext="_导出设置"></button>
    <button class="textButton gray1" type="button" id="importSettings" data-xztext="_导入设置"></button>
    <button class="textButton gray1" type="button" id="resetSettings" data-xztext="_重置设置"></button>
    <button class="textButton gray1" type="button" id="resetHelpTip" data-xztext="_重新显示帮助"></button>
    </p>
  </div>
</form>`;


/***/ }),

/***/ "./src/ts/setting/FormSettings.ts":
/*!****************************************!*\
  !*** ./src/ts/setting/FormSettings.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   FormSettings: () => (/* binding */ FormSettings)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_DateFormat__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/DateFormat */ "./src/ts/utils/DateFormat.ts");
/* harmony import */ var _NameRuleManager__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./NameRuleManager */ "./src/ts/setting/NameRuleManager.ts");





class FormSettings {
    constructor(form) {
        // 没有填写 userSetName 字段，因为这个字段由 nameRuleManager 管理
        this.inputFileds = {
            checkbox: [
                'downType0',
                'downType1',
                'downType2',
                'downType3',
                'downAllAges',
                'downR18',
                'downR18G',
                'downSingleImg',
                'downMultiImg',
                'downColorImg',
                'downBlackWhiteImg',
                'downNotBookmarked',
                'downBookmarked',
                'firstFewImagesSwitch',
                'multiImageWorkImageLimitSwitch',
                'saveNovelMeta',
                'BMKNumSwitch',
                'BMKNumAverageSwitch',
                'setWHSwitch',
                'ratioSwitch',
                'userSetChecked',
                'postDate',
                'idRangeSwitch',
                'needTagSwitch',
                'notNeedTagSwitch',
                'magnifier',
                'tagNameToFileName',
                'noSerialNo',
                'createFolderByTag',
                'workDir',
                'r18Folder',
                'sizeSwitch',
                'autoStartDownload',
                'previewResult',
                'deduplication',
                'fileNameLengthLimitSwitch',
                'bmkAfterDL',
                'userBlockList',
                'blockTagsForSpecificUser',
                'bgDisplay',
                'createFolderByType',
                'createFolderByTypeIllust',
                'createFolderByTypeManga',
                'createFolderByTypeUgoira',
                'createFolderByTypeNovel',
                'zeroPadding',
                'showFastSearchArea',
                'saveMetaType0',
                'saveMetaType1',
                'saveMetaType2',
                'saveMetaType3',
                'setNameRuleForEachPageType',
                'showAdvancedSettings',
                'showNotificationAfterDownloadComplete',
                'boldKeywords',
                'autoExportResult',
                'autoExportResultCSV',
                'autoExportResultJSON',
                'PreviewWork',
                'showDownloadBtnOnThumb',
                'showOriginImage',
                'replaceSquareThumb',
                'notFolderWhenOneFile',
                'noSerialNoForSingleImg',
                'noSerialNoForMultiImg',
                'removeAtFromUsername',
                'showPreviewWorkTip',
                'showLargerThumbnails',
                'doubleWidthThumb',
                'wheelScrollSwitchImageOnPreviewWork',
                'doNotDownloadLastImageOfMultiImageWork',
                'downloadNovelCoverImage',
                'downloadNovelEmbeddedImage',
                'previewUgoira',
                'hiddenBrowserDownloadBar',
                'slowCrawl',
                'downloadOnClickBookmark',
                'downloadOnClickLike',
                'exportLog',
                'exportLogNormal',
                'exportLogError',
                'UseDifferentNameRuleIfWorkHasTagSwitch',
                'AIGenerated',
                'notAIGenerated',
                'UnknownAI',
                'setFileDownloadOrder',
                'highlightFollowingUsers',
                'exportIDList',
                'displayThumbnailListOnMultiImageWorkPage',
                'PreviewWorkDetailInfo',
            ],
            text: [
                'setWantPage',
                'firstFewImages',
                'multiImageWorkImageLimit',
                'convertUgoiraThread',
                'BMKNumMin',
                'BMKNumMax',
                'BMKNumAverage',
                'setWidth',
                'setHeight',
                'userRatio',
                'idRangeInput',
                'needTag',
                'notNeedTag',
                'workDirFileNumber',
                'r18FolderName',
                'sizeMin',
                'sizeMax',
                'downloadThread',
                'fileNameLengthLimit',
                'dateFormat',
                'tagsSeparator',
                'blockList',
                'bgOpacity',
                'zeroPaddingLength',
                'workDirNameRule',
                'autoExportResultNumber',
                'previewWorkWait',
                'previewResultLimit',
                'timedCrawlInterval',
                'slowCrawlOnWorksNumber',
                'exportLogExclude',
                'PreviewDetailInfoWidth',
            ],
            radio: [
                'ugoiraSaveAs',
                'novelSaveAs',
                'widthHeightLimit',
                'userRatioLimit',
                'setWidthAndOr',
                'ratio',
                'idRange',
                'magnifierSize',
                'magnifierPosition',
                'dupliStrategy',
                'imageSize',
                'userSetLang',
                'restrict',
                'widthTag',
                'needTagMode',
                'theme',
                'bgPositionY',
                'switchTabBar',
                'tagMatchMode',
                'prevWorkSize',
                'showOriginImageSize',
                'exportLogTiming',
                'downloadOrder',
                'downloadOrderSortBy',
            ],
            textarea: ['createFolderTagList'],
            datetime: ['postDateStart', 'postDateEnd'],
        };
        this.restoreTimer = 0;
        this.form = form;
        _NameRuleManager__WEBPACK_IMPORTED_MODULE_4__.nameRuleManager.registerInput(this.form.userSetName);
        this.bindEvents();
        this.restoreFormSettings();
        this.ListenChange();
    }
    bindEvents() {
        // 页面切换时，从设置里恢复当前页面的页数/个数
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitchedTypeChange, () => {
            this.restoreWantPage();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, () => {
            window.clearTimeout(this.restoreTimer);
            this.restoreTimer = window.setTimeout(() => {
                this.restoreFormSettings();
            }, 0);
        });
    }
    // 监听所有输入选项的变化
    // 该函数可执行一次，否则事件会重复绑定
    ListenChange() {
        for (const name of this.inputFileds.text) {
            // 对于某些特定输入框，不使用通用的事件处理函数
            if (name === 'setWantPage') {
                continue;
            }
            this.saveTextInput(name);
        }
        // setWantPage 变化时，保存到 wantPageArr
        this.form.setWantPage.addEventListener('change', () => {
            const temp = Array.from(_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.wantPageArr);
            temp[_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type] = Number.parseInt(this.form.setWantPage.value);
            (0,_Settings__WEBPACK_IMPORTED_MODULE_2__.setSetting)('wantPageArr', temp);
        });
        for (const name of this.inputFileds.textarea) {
            this.saveTextInput(name);
        }
        for (const name of this.inputFileds.datetime) {
            this.saveTextInput(name);
        }
        for (const name of this.inputFileds.radio) {
            this.saveRadio(name);
        }
        for (const name of this.inputFileds.checkbox) {
            this.saveCheckBox(name);
        }
    }
    // 读取设置，恢复到表单里
    restoreFormSettings() {
        for (const name of this.inputFileds.text) {
            // setWantPage 需要从 wantPageArr 恢复
            if (name === 'setWantPage') {
                this.restoreWantPage();
                continue;
            }
            this.restoreString(name);
        }
        for (const name of this.inputFileds.radio) {
            this.restoreString(name);
        }
        for (const name of this.inputFileds.textarea) {
            this.restoreString(name);
        }
        for (const name of this.inputFileds.checkbox) {
            this.restoreBoolean(name);
        }
        for (const name of this.inputFileds.datetime) {
            this.restoreDate(name);
        }
    }
    // ---------------------
    // 处理输入框： change 时保存 value
    saveTextInput(name) {
        const el = this.form[name];
        el.addEventListener('change', () => {
            (0,_Settings__WEBPACK_IMPORTED_MODULE_2__.setSetting)(name, el.value);
        });
    }
    // 处理复选框： click 时保存 checked
    saveCheckBox(name) {
        // 由于表单里存在两个 showAdvancedSettings 设置，会获取到 NodeListOf<HTMLInputElement>
        // 其他设置只有一个，是 HTMLInputElement
        const el = this.form[name];
        let elArray = [];
        if (el.length !== undefined) {
            elArray = Array.from(el);
        }
        else {
            elArray.push(el);
        }
        elArray.forEach((el) => {
            el.addEventListener('click', () => {
                (0,_Settings__WEBPACK_IMPORTED_MODULE_2__.setSetting)(name, el.checked);
            });
        });
    }
    // 处理单选框： click 时保存 value
    saveRadio(name) {
        const radios = this.form[name];
        for (const radio of radios) {
            radio.addEventListener('click', () => {
                (0,_Settings__WEBPACK_IMPORTED_MODULE_2__.setSetting)(name, radio.value);
            });
        }
    }
    // 恢复值为 Boolean 的设置项
    restoreBoolean(name) {
        if (_Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name] !== undefined) {
            // 由于表单里存在两个 showAdvancedSettings 设置，会获取到 NodeListOf<HTMLInputElement>
            // 其他设置只有一个，是 HTMLInputElement
            const el = this.form[name];
            let elArray = [];
            if (el.length !== undefined) {
                elArray = Array.from(el);
            }
            else {
                elArray.push(el);
            }
            elArray.forEach((el) => {
                el.checked = _Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name];
            });
        }
    }
    // 恢复值为 string 的设置项
    restoreString(name) {
        if (_Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name] !== undefined) {
            this.form[name].value = _Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name].toString();
        }
    }
    // 恢复日期、时间设置项
    restoreDate(name) {
        if (_Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name] !== undefined) {
            // 把时间戳转换成 input 使用的字符串
            const date = _Settings__WEBPACK_IMPORTED_MODULE_2__.settings[name];
            this.form[name].value = _utils_DateFormat__WEBPACK_IMPORTED_MODULE_3__.DateFormat.format(date, 'YYYY-MM-DDThh:mm');
        }
    }
    // 从设置里恢复当前页面的页数/个数
    restoreWantPage() {
        const want = _Settings__WEBPACK_IMPORTED_MODULE_2__.settings.wantPageArr[_PageType__WEBPACK_IMPORTED_MODULE_1__.pageType.type];
        if (want !== undefined) {
            this.form.setWantPage.value = want.toString();
        }
    }
}



/***/ }),

/***/ "./src/ts/setting/InvisibleSettings.ts":
/*!*********************************************!*\
  !*** ./src/ts/setting/InvisibleSettings.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../utils/SecretSignal */ "./src/ts/utils/SecretSignal.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");




// 管理不可见的设置。通过预设的按键，切换其开关状态
class InvisibleSettings {
    constructor() {
        // ppdss: Powerful Pixiv Downloader Secret Settings
        this.cfg = {
            createFolderBySl: ['ppdss1', 'switchsl', 'kaiguansl'],
            downloadUgoiraFirst: ['ppdss2', 'dlugoirafirst', 'qw111'],
        };
        this.register();
    }
    register() {
        for (const [name, codes] of Object.entries(this.cfg)) {
            for (const code of codes) {
                _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_1__.secretSignal.register(code, () => {
                    this.onChange(name);
                });
            }
        }
    }
    onChange(name) {
        const nowValue = _Settings__WEBPACK_IMPORTED_MODULE_0__.settings[name];
        if (typeof nowValue !== 'boolean') {
            return;
        }
        // 如果查找到了符合的记录，则反转这个设置的值
        const newValue = !_Settings__WEBPACK_IMPORTED_MODULE_0__.settings[name];
        (0,_Settings__WEBPACK_IMPORTED_MODULE_0__.setSetting)(name, newValue);
        // 显示提示信息
        if (_Settings__WEBPACK_IMPORTED_MODULE_0__.settings[name]) {
            const msg = name + ' On';
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.success(msg);
            _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.success(msg);
        }
        else {
            const msg = name + ' Off';
            _Log__WEBPACK_IMPORTED_MODULE_2__.log.warning(msg);
            _Toast__WEBPACK_IMPORTED_MODULE_3__.toast.warning(msg);
        }
    }
}
new InvisibleSettings();


/***/ }),

/***/ "./src/ts/setting/NameRuleManager.ts":
/*!*******************************************!*\
  !*** ./src/ts/setting/NameRuleManager.ts ***!
  \*******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   nameRuleManager: () => (/* binding */ nameRuleManager)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _PageType__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../PageType */ "./src/ts/PageType.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");






// 管理命名规则
// 在实际使用中，作为 settings.userSetName 的代理
// 其他类必须使用 nameRuleManager.rule 存取器来存取命名规则
class NameRuleManager {
    constructor() {
        // 所有页面通用的命名规则
        this.generalRule = '{page_title}/{id}';
        // 命名规则输入框的集合
        this.inputList = [];
        this.bindEvents();
    }
    bindEvents() {
        const evts = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingInitialized,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resetSettingsEnd,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.pageSwitchedTypeChange,
        ];
        evts.forEach((evt) => {
            window.addEventListener(evt, () => {
                this.setInputValue();
            });
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            // 当用户开启这个开关时，设置当前页面类型的命名规则
            if (data.name === 'setNameRuleForEachPageType' && data.value) {
                if (_Settings__WEBPACK_IMPORTED_MODULE_5__.settings.nameRuleForEachPageType[_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type] !==
                    _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.userSetName) {
                    this.setInputValue();
                }
            }
        });
    }
    saveCurrentPageRule(rule) {
        _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.nameRuleForEachPageType[_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type] = rule;
        (0,_Settings__WEBPACK_IMPORTED_MODULE_5__.setSetting)('nameRuleForEachPageType', _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.nameRuleForEachPageType);
    }
    get rule() {
        if (_Settings__WEBPACK_IMPORTED_MODULE_5__.settings.setNameRuleForEachPageType) {
            let rule = _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.nameRuleForEachPageType[_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type];
            if (rule === undefined) {
                rule = this.generalRule;
                this.saveCurrentPageRule(rule);
            }
            return rule;
        }
        else {
            return _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.userSetName;
        }
    }
    set rule(str) {
        // 检查传递的命名规则的合法性
        // 为了防止文件名重复，命名规则里一定要包含 {id} 或者 {id_num}{p_num}
        const check = str.includes('{id}') ||
            (str.includes('{id_num}') && str.includes('{p_num}'));
        if (!check) {
            window.setTimeout(() => {
                _MsgBox__WEBPACK_IMPORTED_MODULE_2__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_1__.lang.transl('_命名规则一定要包含id'));
            }, 300);
        }
        else {
            // 替换特殊字符
            str = this.handleUserSetName(str) || this.generalRule;
            (0,_Settings__WEBPACK_IMPORTED_MODULE_5__.setSetting)('userSetName', str);
            if (_Settings__WEBPACK_IMPORTED_MODULE_5__.settings.setNameRuleForEachPageType) {
                this.saveCurrentPageRule(str);
            }
            this.setInputValue();
        }
    }
    // 注册命名规则输入框
    registerInput(input) {
        this.inputList.push(input);
        this.setInputValue();
        // 保存事件被触发之前的值
        let lastValue = input.value;
        // 给输入框绑定事件
        const evList = ['change', 'focus'];
        // change 事件只对用户手动输入有效
        // 当用户从下拉框添加一个命名标记时，不会触发 change 事件，需要监听 focus 事件
        evList.forEach((evName) => {
            input.addEventListener(evName, () => {
                // 当事件触发时，比较输入框的值是否与事件触发之前发生了变化
                // 如果值没有变化，就什么都不做
                // 对于 change 事件来说，值必然发生了变化，但是 focus 就不一定了
                // 试想：用户修改命名规则为非法的规则，例如输入 111，触发 change 事件之后下载器会提示命名规则非法
                // 然后用户点击输入框（focus 事件）想要修改规则，此时值没有变化，就不应该执行后续代码。如果依然执行后续代码，那么每当用户点击输入框，下载器就会马上显示提示，这导致用户根本没办法在输入框里修改命名规则
                if (input.value === lastValue) {
                    return;
                }
                lastValue = input.value;
                if (_Settings__WEBPACK_IMPORTED_MODULE_5__.settings.nameRuleForEachPageType[_PageType__WEBPACK_IMPORTED_MODULE_3__.pageType.type] !== input.value) {
                    this.rule = input.value;
                }
            });
        });
    }
    // 设置输入框的值为当前命名规则
    setInputValue() {
        // 如果 settings.nameRuleForEachPageType 里面没有当前页面的 key，值就是 undefined，需要设置为默认值
        const rule = this.rule;
        this.inputList.forEach((input) => {
            input.value = rule;
        });
        if (rule !== _Settings__WEBPACK_IMPORTED_MODULE_5__.settings.userSetName) {
            (0,_Settings__WEBPACK_IMPORTED_MODULE_5__.setSetting)('userSetName', rule);
        }
    }
    // 处理用命名规则的非法字符和非法规则
    // 这里不必处理得非常详尽，因为在生成文件名时，还会对结果进行处理
    // 测试用例：在作品页面内设置下面的命名规则，下载器会自动进行更正
    // /{page_tag}/|/{user}////<//{rank}/{px}/{sl}/{page_tag}///{id}-{user}-{user_id}""-?{tags_transl_only}////
    handleUserSetName(str) {
        // 替换命名规则里可能存在的非法字符
        str = _utils_Utils__WEBPACK_IMPORTED_MODULE_4__.Utils.replaceUnsafeStr(str);
        // replaceUnsafeStr 会把斜线 / 替换成全角的斜线 ／，这里再替换回来，否则就不能建立文件夹了
        str = str.replace(/／/g, '/');
        // 处理连续的 /
        str = str.replace(/\/{2,100}/g, '/');
        // 如果命名规则头部或者尾部是 / 则去掉
        if (str.startsWith('/')) {
            str = str.replace('/', '');
        }
        if (str.endsWith('/')) {
            str = str.substring(0, str.length - 1);
        }
        return str;
    }
}
const nameRuleManager = new NameRuleManager();



/***/ }),

/***/ "./src/ts/setting/Options.ts":
/*!***********************************!*\
  !*** ./src/ts/setting/Options.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   options: () => (/* binding */ options)
/* harmony export */ });
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");




// 控制每个设置的隐藏、显示
// 设置页数/个数的提示文本
class Options {
    constructor() {
        // 保持显示的选项的 id
        this.whiteList = [
            1, 2, 4, 13, 17, 32, 44, 50, 51, 57, 64,
        ];
        // 某些页面类型需要隐藏某些选项。当调用 hideOption 方法时，把选项 id 保存起来
        // 优先级高于 whiteList
        this.hiddenList = [];
    }
    init(allOption) {
        this.allOption = allOption;
        // 获取“页数/个数”设置的元素
        const wantPageOption = this.getOption(1);
        this.wantPageEls = {
            wrap: wantPageOption.querySelector('.setWantPageWrap'),
            text: wantPageOption.querySelector('.setWantPageTip1'),
            rangTip: wantPageOption.querySelector('.setWantPageTip2'),
            input: wantPageOption.querySelector('.setWantPage'),
        };
        this.handleShowAdvancedSettings();
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'showAdvancedSettings') {
                this.handleShowAdvancedSettings();
            }
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingInitialized, () => {
            this.alwaysHideSomeOption();
        });
        const list = [
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitchedTypeNotChange,
            _EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.pageSwitchedTypeChange,
        ];
        list.forEach((ev) => {
            window.addEventListener(ev, () => {
                this.hiddenList = [];
                window.setTimeout(() => {
                    this.handleShowAdvancedSettings();
                    this.alwaysHideSomeOption();
                });
            });
        });
    }
    // 总是隐藏某些设置
    alwaysHideSomeOption() {
        this.hideOption([79, 80]);
        // 在移动端某些设置不会生效，所以隐藏它们
        // 主要是和作品缩略图相关的一些设置
        if (_Config__WEBPACK_IMPORTED_MODULE_0__.Config.mobile) {
            this.hideOption([18, 68, 55, 71, 62, 40]);
        }
    }
    handleShowAdvancedSettings() {
        for (const option of this.allOption) {
            if (option.dataset.no === undefined) {
                continue;
            }
            const no = Number.parseInt(option.dataset.no);
            // 如果需要隐藏高级设置
            if (!_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.showAdvancedSettings) {
                // 如果在白名单中，并且当前页面不需要隐藏它，那么它就是显示的
                if (this.whiteList.includes(no) && !this.hiddenList.includes(no)) {
                    this.showOption([no]);
                }
                // 如果没有在白名单中，或者当前页面需要隐藏它，就隐藏它
                if (!this.whiteList.includes(no) || this.hiddenList.includes(no)) {
                    option.style.display = 'none';
                }
            }
            else {
                // 如果需要显示高级设置，那么只隐藏当前页面需要隐藏的选项
                if (this.hiddenList.includes(no)) {
                    option.style.display = 'none';
                }
                else {
                    this.showOption([no]);
                }
            }
        }
    }
    // 使用编号获取指定选项的元素
    getOption(no) {
        for (const option of this.allOption) {
            if (option.dataset.no === no.toString()) {
                return option;
            }
        }
        throw `Not found this option: ${no}`;
    }
    // 显示或隐藏指定的选项
    setOptionDisplay(no, display) {
        for (const number of no) {
            this.getOption(number).style.display = display;
        }
    }
    // 显示所有选项
    // 在切换不同页面时使用
    showAllOption() {
        for (const el of this.allOption) {
            el.style.display = 'block';
        }
    }
    // 隐藏指定的选项。参数是数组，传递设置项的编号。
    // 注意：由于这个方法会修改 hiddenList，所以它是有副作用的
    // 这个方法只应该在其他类里面使用，在这个类里不要直接调用它
    hideOption(no) {
        this.hiddenList = no;
        this.setOptionDisplay(no, 'none');
    }
    // 显示指定的选项。因为页面无刷新加载，所以一些选项被隐藏后，可能需要再次显示
    showOption(no) {
        this.setOptionDisplay(no, 'block');
    }
    // 设置 “抓取多少作品/页面” 选项的提示和预设值
    setWantPageTip(arg) {
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.wantPageEls.text, arg.text);
        this.wantPageEls.wrap.dataset.xztip = arg.tip;
        this.wantPageEls.wrap.dataset.tip = _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl(arg.tip);
        // rangTip 可能需要翻译
        if (arg.rangTip.startsWith('_')) {
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.wantPageEls.rangTip, arg.rangTip);
        }
        else {
            // 也可能直接传递了字符串，不需要翻译
            _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.wantPageEls.rangTip, '');
            this.wantPageEls.rangTip.textContent = arg.rangTip;
        }
    }
}
const options = new Options();



/***/ }),

/***/ "./src/ts/setting/SaveNamingRule.ts":
/*!******************************************!*\
  !*** ./src/ts/setting/SaveNamingRule.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   SaveNamingRule: () => (/* binding */ SaveNamingRule)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _NameRuleManager__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./NameRuleManager */ "./src/ts/setting/NameRuleManager.ts");







// 保存和加载命名规则列表
class SaveNamingRule {
    constructor(ruleInput) {
        this.limit = 20; // 最大保存数量
        this._show = false; // 是否显示列表
        this.html = `
  <div class="saveNamingRuleWrap">
  <button class="nameSave textButton has_tip" type="button" data-xztip="_保存命名规则提示" data-xztext="_保存"></button>
  <button class="nameLoad textButton" type="button" data-xztext="_加载"></button>
  <ul class="namingRuleList"></ul>
  </div>`;
        this.ruleInput = ruleInput;
        _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.clearSlot('saveNamingRule');
        const wrap = _Tools__WEBPACK_IMPORTED_MODULE_1__.Tools.useSlot('saveNamingRule', this.html);
        _Theme__WEBPACK_IMPORTED_MODULE_3__.theme.register(wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(wrap);
        this.saveBtn = wrap.querySelector('button.nameSave');
        this.loadBtn = wrap.querySelector('button.nameLoad');
        this.listWrap = wrap.querySelector('ul.namingRuleList');
        this.createList();
        this.bindEvents();
    }
    set show(boolean) {
        this._show = boolean;
        boolean ? this.showListWrap() : this.hideListWrap();
    }
    get show() {
        return this._show;
    }
    bindEvents() {
        this.saveBtn.addEventListener('click', () => {
            this.add(this.ruleInput.value);
        });
        this.loadBtn.addEventListener('click', () => {
            this.show = !this.show;
        });
        this.listWrap.addEventListener('mouseleave', () => {
            this.show = false;
        });
        // 设置发生变化时重新创建列表
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'namingRuleList') {
                this.createList();
            }
        });
    }
    add(rule) {
        if (_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList.length === this.limit) {
            this.delete(0);
        }
        // 如果这个规则已存在，不会重复添加它
        if (!_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList.includes(rule)) {
            const list = Array.from(_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList);
            list.push(rule);
            (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('namingRuleList', list);
        }
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_已保存命名规则'));
    }
    delete(index) {
        const list = Array.from(_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList);
        list.splice(index, 1);
        (0,_Settings__WEBPACK_IMPORTED_MODULE_4__.setSetting)('namingRuleList', list);
    }
    select(rule) {
        this.ruleInput.value = rule;
        _NameRuleManager__WEBPACK_IMPORTED_MODULE_6__.nameRuleManager.rule = rule;
    }
    createList() {
        const htmlArr = [];
        for (let i = 0; i < _Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList.length; i++) {
            const html = `<li>
      <span class="rule">${_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList[i]}</span>
      <button class="delete textButton" type="button" data-index="${i}">×</button>
    </li>`;
            htmlArr.push(html);
        }
        if (_Settings__WEBPACK_IMPORTED_MODULE_4__.settings.namingRuleList.length === 0) {
            htmlArr.push(`<li><i>&nbsp;&nbsp;&nbsp;&nbsp;no data</i></li>`);
        }
        this.listWrap.innerHTML = htmlArr.join('');
        const ruleEls = this.listWrap.querySelectorAll('.rule');
        for (const el of ruleEls) {
            el.addEventListener('click', () => {
                this.select(el.textContent);
                this.show = false;
            });
        }
        const deleteEls = this.listWrap.querySelectorAll('.delete');
        for (const el of deleteEls) {
            el.addEventListener('click', () => {
                const index = parseInt(el.dataset.index);
                this.delete(index);
            });
        }
    }
    showListWrap() {
        this.listWrap.style.display = 'block';
    }
    hideListWrap() {
        this.listWrap.style.display = 'none';
    }
}



/***/ }),

/***/ "./src/ts/setting/Settings.ts":
/*!************************************!*\
  !*** ./src/ts/setting/Settings.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   setSetting: () => (/* binding */ setSetting),
/* harmony export */   settings: () => (/* binding */ settings)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");
/* harmony import */ var _ConvertOldSettings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ConvertOldSettings */ "./src/ts/setting/ConvertOldSettings.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _Config__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Config */ "./src/ts/Config.ts");
/* harmony import */ var _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/SecretSignal */ "./src/ts/utils/SecretSignal.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
// settings 保存了下载器的所有设置项
// 获取设置项的值：
// settings[name]
// 修改设置项的值：
// setSetting(name, value)
// 本模块会触发 3 个事件：
// EVT.list.settingChange
// 当任意一个设置项被赋值时触发（不会区分值是否发生了变化）。这是最常用的事件。
// 事件的参数里会传递这个设置项的名称和值，可以通过 ev.detail.data 获取，格式如：
// {name: string, value: any}
// 如果某个模块要监听特定的设置项，应该使用参数的 name 来判断触发事件的设置项是否是自己需要的设置项
// 如果不依赖于特定设置项，则应该考虑使用节流或者防抖来限制事件的回调函数的执行频率，防止造成性能问题
// EVT.list.settingInitialized
// 当设置初始化完毕（以及恢复本地储存的设置）之后触发。这个事件在生命周期里只会触发一次。
// 过程中，每个设置项都会触发一次 settingChange 事件
// EVT.list.resetSettingsEnd
// 重置设置之后触发
// 导入设置之后触发
// 过程中，每个设置项都会触发一次 settingChange 事件
// 如果打开了多个标签页，每个页面的 settings 数据是相互独立的，在一个页面里修改设置不会影响另一个页面里的设置。
// 但是持久化保存的数据只有一份：最后一次的设置变化是在哪个页面发生的，就保存哪个页面的 settings 数据。
// 所以当页面刷新时，或者打开新的页面时，会加载设置最后一次发生变化的页面里的 settings 数据








class Settings {
    constructor() {
        // 默认设置
        this.defaultSettings = {
            setWantPage: -1,
            wantPageArr: [
                -1, -1, -1, -1, -1, 1000, -1, 500, -1, 1000, 100, -1, 100, -1, -1, 1000,
                100, 100, 100, 100, -1,
            ],
            firstFewImagesSwitch: false,
            firstFewImages: 1,
            multiImageWorkImageLimitSwitch: false,
            multiImageWorkImageLimit: 10,
            downType0: true,
            downType1: true,
            downType2: true,
            downType3: true,
            downAllAges: true,
            downR18: true,
            downR18G: true,
            downSingleImg: true,
            downMultiImg: true,
            downColorImg: true,
            downBlackWhiteImg: true,
            downNotBookmarked: true,
            downBookmarked: true,
            ugoiraSaveAs: 'webm',
            convertUgoiraThread: 1,
            needTag: [],
            notNeedTag: [],
            autoStartDownload: true,
            downloadThread: 5,
            userSetName: '{page_title}/{id}',
            namingRuleList: [],
            tagNameToFileName: false,
            workDir: false,
            workDirFileNumber: 1,
            workDirNameRule: '{id_num}',
            showOptions: true,
            postDate: false,
            postDateStart: 946684800000,
            postDateEnd: 4102444800000,
            previewResult: true,
            previewResultLimit: 3000,
            BMKNumSwitch: false,
            BMKNumMin: 0,
            BMKNumMax: _Config__WEBPACK_IMPORTED_MODULE_4__.Config.BookmarkCountLimit,
            BMKNumAverageSwitch: false,
            BMKNumAverage: 600,
            setWHSwitch: false,
            widthHeightLimit: '>=',
            setWidthAndOr: '&',
            setWidth: 0,
            setHeight: 0,
            ratioSwitch: false,
            ratio: 'horizontal',
            userSetChecked: false,
            userRatio: 1.4,
            userRatioLimit: '>=',
            idRangeSwitch: false,
            idRangeInput: 0,
            idRange: '>',
            needTagSwitch: false,
            notNeedTagSwitch: false,
            noSerialNo: false,
            filterBlackWhite: false,
            sizeSwitch: false,
            sizeMin: 0,
            sizeMax: 100,
            novelSaveAs: 'txt',
            saveNovelMeta: false,
            deduplication: false,
            dupliStrategy: 'loose',
            fileNameLengthLimitSwitch: false,
            tagsSeparator: ',',
            fileNameLengthLimit: 200,
            imageSize: 'original',
            dateFormat: 'YYYY-MM-DD',
            userSetLang: 'auto',
            bmkAfterDL: false,
            widthTag: 'yes',
            restrict: 'no',
            widthTagBoolean: true,
            restrictBoolean: false,
            userBlockList: false,
            blockList: [],
            theme: 'auto',
            needTagMode: 'all',
            r18Folder: false,
            r18FolderName: '[R-18&R-18G]',
            blockTagsForSpecificUser: false,
            blockTagsForSpecificUserShowList: true,
            blockTagsForSpecificUserList: [],
            magnifier: true,
            magnifierSize: 'original',
            magnifierPosition: 'right',
            bgDisplay: false,
            bgOpacity: 60,
            bgPositionY: 'center',
            createFolderByType: false,
            createFolderByTypeIllust: false,
            createFolderByTypeManga: false,
            createFolderByTypeUgoira: false,
            createFolderByTypeNovel: false,
            createFolderByTag: false,
            createFolderTagList: [],
            createFolderBySl: false,
            downloadUgoiraFirst: false,
            switchTabBar: 'over',
            zeroPadding: false,
            zeroPaddingLength: 3,
            tagMatchMode: 'whole',
            showFastSearchArea: true,
            saveMetaType0: false,
            saveMetaType1: false,
            saveMetaType2: false,
            saveMetaType3: false,
            setNameRuleForEachPageType: false,
            nameRuleForEachPageType: {
                '-1': '{page_title}/{id}',
                '0': '{page_title}/{id}',
                '1': '{page_title}/{id}',
                '2': '{user}/{id}',
                '3': '{page_title}/{id}',
                '4': '{page_title}/{id}',
                '5': '{page_tag}/{id}',
                '6': '{page_title}/{id}',
                '7': '{page_title}/{rank}-{id}',
                '8': '{page_title}/{id}',
                '9': '{page_title}/{id}',
                '10': '{page_title}/{id}',
                '11': '{page_title}/{id}',
                '12': '{page_title}/{id}',
                '13': '{page_title}/{id}-{title}',
                '14': '{user}/{series_title}/{series_order} {title} {id}',
                '15': '{page_tag}/{id}-{title}',
                '16': '{page_title}/{rank}-{id}-{title}',
                '17': '{page_title}/{id}-{title}',
                '18': '{page_title}/{id}-{title}',
                '19': '{user}/{series_title}/{series_order} {title} {id}',
                '20': '{page_title}/{id}',
                '21': '{page_title}/{id}-{title}',
            },
            showAdvancedSettings: false,
            showNotificationAfterDownloadComplete: false,
            boldKeywords: true,
            autoExportResult: false,
            autoExportResultCSV: true,
            autoExportResultJSON: false,
            autoExportResultNumber: 1,
            PreviewWork: true,
            showDownloadBtnOnThumb: true,
            prevWorkSize: 'regular',
            previewWorkWait: 400,
            showPreviewWorkTip: true,
            showOriginImage: true,
            showOriginImageSize: 'original',
            tipHowToUse: true,
            whatIsNewFlag: 'xuejian&saber',
            replaceSquareThumb: true,
            notFolderWhenOneFile: false,
            noSerialNoForSingleImg: true,
            noSerialNoForMultiImg: true,
            setUserNameShow: true,
            setUserNameList: {},
            removeAtFromUsername: false,
            showLargerThumbnails: false,
            doubleWidthThumb: true,
            wheelScrollSwitchImageOnPreviewWork: true,
            doNotDownloadLastImageOfMultiImageWork: false,
            downloadNovelCoverImage: true,
            downloadNovelEmbeddedImage: true,
            previewUgoira: true,
            hiddenBrowserDownloadBar: false,
            tipPressDToQuickDownload: true,
            timedCrawlInterval: 120,
            slowCrawl: true,
            slowCrawlOnWorksNumber: 100,
            downloadOnClickBookmark: false,
            downloadOnClickLike: false,
            exportLog: false,
            exportLogTiming: 'downloadComplete',
            exportLogNormal: false,
            exportLogError: true,
            exportLogExclude: ['404', '429', '500'],
            DoNotDownloadLastFewImagesShow: false,
            DoNotDownloadLastFewImagesList: [],
            UseDifferentNameRuleIfWorkHasTagSwitch: false,
            UseDifferentNameRuleIfWorkHasTagShow: true,
            UseDifferentNameRuleIfWorkHasTagList: [],
            AIGenerated: true,
            notAIGenerated: true,
            UnknownAI: true,
            setFileDownloadOrder: false,
            downloadOrder: 'desc',
            downloadOrderSortBy: 'ID',
            tipAltXToShowControlPanel: true,
            tipAltSToSelectWork: true,
            tipAltQToQuickDownload: true,
            tipBookmarkButton: true,
            highlightFollowingUsers: true,
            tipCSV: true,
            exportIDList: false,
            tipExportFollowingUserList: true,
            displayThumbnailListOnMultiImageWorkPage: true,
            tipBookmarkManage: true,
            requestSponsorshipTime: 0,
            PreviewWorkDetailInfo: false,
            PreviewDetailInfoWidth: 400,
        };
        this.allSettingKeys = Object.keys(this.defaultSettings);
        // 值为浮点数的选项
        this.floatNumberKey = ['userRatio', 'sizeMin', 'sizeMax'];
        // 值为整数的选项不必单独列出
        // 值为数字数组的选项
        this.numberArrayKeys = ['wantPageArr'];
        // 值为字符串数组的选项
        this.stringArrayKeys = [
            'namingRuleList',
            'blockList',
            'needTag',
            'notNeedTag',
            'createFolderTagList',
            'exportLogExclude',
        ];
        // 以默认设置作为初始设置
        this.settings = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.deepCopy(this.defaultSettings);
        this.store = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.debounce(() => {
            // chrome.storage.local 的储存上限是 5 MiB（5242880 Byte）
            chrome.storage.local.set({
                [_Config__WEBPACK_IMPORTED_MODULE_4__.Config.settingStoreName]: this.settings,
            });
        }, 50);
        this.restore();
        this.bindEvents();
    }
    bindEvents() {
        // 当设置发生变化时进行本地存储
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, () => {
            this.store();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resetSettings, () => {
            this.reset();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.exportSettings, () => {
            this.exportSettings();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.importSettings, () => {
            this.importSettings();
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resetHelpTip, () => {
            this.resetHelpTip();
        });
        // 切换只选择动图/选择全部作品类型
        const codes = ['onlyugoira', 'qw222'];
        for (const code of codes) {
            _utils_SecretSignal__WEBPACK_IMPORTED_MODULE_5__.secretSignal.register(code, () => {
                // 如果只有动图被选中，则选择全部作品类型
                // 反之，只选择动图
                if (this.settings.downType2 &&
                    !this.settings.downType0 &&
                    !this.settings.downType1 &&
                    !this.settings.downType3) {
                    this.settings.downType0 = true;
                    this.settings.downType1 = true;
                    this.settings.downType3 = true;
                    // 多次修改只触发一次改变事件，提高效率
                    this.setSetting('downType0', true);
                    _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.warning('onlyUgoira off');
                }
                else {
                    this.settings.downType0 = false;
                    this.settings.downType1 = false;
                    this.settings.downType2 = true;
                    this.settings.downType3 = false;
                    this.setSetting('downType2', true);
                    _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success('onlyUgoira on');
                }
            });
        }
    }
    // 读取恢复设置
    restore() {
        let restoreData = this.defaultSettings;
        // 首先从 chrome.storage 获取配置（从 11.5.0 版本开始）
        chrome.storage.local.get(_Config__WEBPACK_IMPORTED_MODULE_4__.Config.settingStoreName, (result) => {
            if (result[_Config__WEBPACK_IMPORTED_MODULE_4__.Config.settingStoreName]) {
                restoreData = result[_Config__WEBPACK_IMPORTED_MODULE_4__.Config.settingStoreName];
            }
            else {
                // 如无数据则尝试从 localStorage 获取配置，因为旧版本的配置储存在 localStorage 中
                const savedSettings = localStorage.getItem(_Config__WEBPACK_IMPORTED_MODULE_4__.Config.settingStoreName);
                if (savedSettings) {
                    restoreData = JSON.parse(savedSettings);
                }
            }
            this.assignSettings(restoreData);
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('settingInitialized');
        });
    }
    // 接收整个设置项，通过循环将其更新到 settings 上
    // 循环设置而不是整个替换的原因：
    // 1. 进行类型转换，如某些设置项是 number ，但是数据来源里是 string，setSetting 可以把它们转换到正确的类型
    // 2. 某些选项在旧版本里没有，所以不能用旧的设置整个覆盖
    assignSettings(data) {
        const origin = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.deepCopy(data);
        for (const [key, value] of Object.entries(origin)) {
            this.setSetting(key, value);
        }
    }
    exportSettings() {
        const blob = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.json2Blob(this.settings);
        const url = URL.createObjectURL(blob);
        _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.downloadFile(url, _Config__WEBPACK_IMPORTED_MODULE_4__.Config.appName + ` Settings.json`);
        URL.revokeObjectURL(url);
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_导出成功'));
    }
    async importSettings() {
        const loadedJSON = (await _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.loadJSONFile().catch((err) => {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(err);
        }));
        if (!loadedJSON) {
            return;
        }
        // 检查是否存在设置里的属性
        if (loadedJSON.downloadThread === undefined) {
            return _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_格式错误'));
        }
        // 开始恢复导入的设置
        this.reset(loadedJSON);
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_导入成功'));
    }
    // 有些帮助信息是只显示一次的，这里可以让它们再次显示
    // 主要是通过 showHelp.show 显示的帮助
    resetHelpTip() {
        this.setSetting('tipHowToUse', true);
        this.setSetting('tipAltXToShowControlPanel', true);
        this.setSetting('tipPressDToQuickDownload', true);
        this.setSetting('tipAltSToSelectWork', true);
        this.setSetting('tipPressDToQuickDownload', true);
        this.setSetting('tipAltQToQuickDownload', true);
        this.setSetting('tipBookmarkButton', true);
        this.setSetting('tipCSV', true);
        this.setSetting('tipExportFollowingUserList', true);
        this.setSetting('tipBookmarkManage', true);
        _Toast__WEBPACK_IMPORTED_MODULE_6__.toast.success('✓ ' + _Lang__WEBPACK_IMPORTED_MODULE_7__.lang.transl('_重新显示帮助'));
    }
    // 重置设置 或者 导入设置
    // 可选参数：传递一份设置数据，用于从配置文件导入，恢复设置
    reset(data) {
        this.assignSettings(data ? data : this.defaultSettings);
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('resetSettingsEnd');
    }
    tipError(key) {
        _MsgBox__WEBPACK_IMPORTED_MODULE_3__.msgBox.error(`${key}: Invalid value`);
    }
    // 更改设置项
    // 其他模块应该通过这个方法更改设置
    // 这里面有一些类型转换的代码，主要目的：
    // 1. 兼容旧版本的设置。读取旧版本的设置时，将其转换成新版本的设置。例如某个设置在旧版本里是 string 类型，值为 'a,b,c'。新版本里是 string[] 类型，这里会自动将其转换成 ['a','b','c']
    // 2. 减少额外操作。例如某个设置的类型为 string[]，其他模块可以传入 string 类型的值如 'a,b,c'，而不必先把它转换成 string[]
    setSetting(key, value) {
        if (!this.allSettingKeys.includes(key)) {
            return;
        }
        const keyType = typeof this.defaultSettings[key];
        const valueType = typeof value;
        // 把旧的设置值转换为新的设置值。需要转换的值都是 string 类型
        if (valueType === 'string') {
            value = _ConvertOldSettings__WEBPACK_IMPORTED_MODULE_2__.convertOldSettings.convert(key, value);
        }
        // 将传入的值转换成选项对应的类型
        if (keyType === 'string' && valueType !== 'string') {
            value = value.toString();
        }
        if (keyType === 'number' && valueType !== 'number') {
            // 时间是需要特殊处理的 number 类型
            if (key === 'postDateStart' || key == 'postDateEnd') {
                if (valueType === 'string') {
                    if (value === '') {
                        // 如果日期是空字符串，则替换为默认值
                        value = this.defaultSettings[key];
                    }
                    else {
                        // 把日期字符串转换成时间戳
                        const date = new Date(value);
                        value = date.getTime();
                    }
                }
            }
            else {
                // 处理普通的 number 类型
                if (this.floatNumberKey.includes(key)) {
                    value = Number.parseFloat(value);
                }
                else {
                    value = Number.parseInt(value);
                }
            }
            if (isNaN(value)) {
                return this.tipError(key);
            }
        }
        if (keyType === 'boolean' && valueType !== 'boolean') {
            value = !!value;
        }
        // 处理数组类型的值
        if (Array.isArray(this.defaultSettings[key])) {
            if (this.stringArrayKeys.includes(key)) {
                // 字符串转换成 string[]
                if (valueType === 'string') {
                    value = _utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.string2array(value);
                }
            }
            if (this.numberArrayKeys.includes(key)) {
                // 把数组转换成 number[]
                if (Array.isArray(value)) {
                    value = value.map((val) => {
                        if (typeof val !== 'number') {
                            return Number(val);
                        }
                        else {
                            return val;
                        }
                    });
                }
                else {
                    return;
                }
            }
        }
        // 对于一些不合法的值，重置为默认值
        if (key === 'firstFewImages' && value < 1) {
            value = this.defaultSettings[key];
        }
        if (key === 'fileNameLengthLimit' && value < 1) {
            value = this.defaultSettings[key];
        }
        if (key === 'setWidthAndOr' && value === '') {
            value = this.defaultSettings[key];
        }
        if (key === 'previewResultLimit' && value < 0) {
            value = 999999;
        }
        if (key === 'workDirNameRule') {
            value = value.replace('{id}', '{id_num}');
        }
        // 更改设置
        ;
        this.settings[key] = value;
        // 当修改某些设置时，顺便修改以来它的设置
        if (key === 'widthTag') {
            this.settings.widthTagBoolean = value === 'yes';
        }
        if (key === 'restrict') {
            this.settings.restrictBoolean = value === 'yes';
        }
        if (key === 'ratio') {
            this.settings.userSetChecked = value === 'userSet';
        }
        // 触发设置变化的事件
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('settingChange', { name: key, value: value });
    }
}
const self = new Settings();
const settings = self.settings;
const setSetting = self.setSetting.bind(self);



/***/ }),

/***/ "./src/ts/setting/UseDifferentNameRuleIfWorkHasTag.ts":
/*!************************************************************!*\
  !*** ./src/ts/setting/UseDifferentNameRuleIfWorkHasTag.ts ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _Toast__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Toast */ "./src/ts/Toast.ts");
/* harmony import */ var _MsgBox__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../MsgBox */ "./src/ts/MsgBox.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");








// 如果作品含有某个标签，则对这个作品使用另一种命名规则
class UseDifferentNameRuleIfWorkHasTag {
    constructor() {
        this.slotName = 'UseDifferentNameRuleIfWorkHasTagSlot';
        this._addWrapShow = false;
        this.wrapHTML = `
  <div class="UseDifferentNameRuleIfWorkHasTagWarp">

    <span class="controlBar">
    <span class="total">0</span>
      <button type="button" class="textButton expand" data-xztext="_收起"></button>
      <button type="button" class="textButton showAdd" data-xztext="_添加"></button>
    </span>

    <div class="addWrap">
      <div class="settingItem addInputWrap" >
        <div class="inputItem tags">
          <span class="label uidLabel">Tags</span>
          <input type="text" class="setinput_style1 blue addTagsInput" data-xzplaceholder="_tag用逗号分割" />
        </div>

        <div class="inputItem rule">
          <span class="label nameLabel" data-xztext="_命名规则2"></span>
          <input type="text" class="setinput_style1 blue addRuleInput" />
        </div>

        <div class="btns">
          <button type="button" class="textButton add" data-xztitle="_添加">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-wanchengqueding"></use>
            </svg>
          </button>

          
          <button type="button" class="textButton cancel" data-xztitle="_取消">
            <svg class="icon" aria-hidden="true">
              <use xlink:href="#icon-guanbiquxiao"></use>
            </svg>
          </button>
        </div>
      </div>
    </div>

    <div class="listWrap">
    </div>
  </div>
  `;
        this.createWrap();
        _Theme__WEBPACK_IMPORTED_MODULE_4__.theme.register(this.wrap);
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.register(this.wrap);
        this.bindEvents();
    }
    set addWrapShow(val) {
        this._addWrapShow = val;
        if (val) {
            this.addWrap.style.display = 'block';
            this.addTagsInput.focus();
        }
        else {
            this.addWrap.style.display = 'none';
            this.addTagsInput.value = '';
            this.addRuleInput.value = '';
        }
    }
    get addWrapShow() {
        return this._addWrapShow;
    }
    // 创建列表外部的容器，静态 html
    createWrap() {
        this.wrap = _Tools__WEBPACK_IMPORTED_MODULE_0__.Tools.useSlot(this.slotName, this.wrapHTML);
        this.expandBtn = this.wrap.querySelector('.expand');
        this.showAddBtn = this.wrap.querySelector('.showAdd');
        this.totalSpan = this.wrap.querySelector('.total');
        this.addWrap = this.wrap.querySelector('.addWrap');
        this.addTagsInput = this.wrap.querySelector('.addTagsInput');
        this.addRuleInput = this.wrap.querySelector('.addRuleInput');
        this.addBtn = this.wrap.querySelector('.add');
        this.cancelBtn = this.wrap.querySelector('.cancel');
        this.listWrap = this.wrap.querySelector('.listWrap');
        // 展开/折叠按钮
        this.expandBtn.addEventListener('click', () => {
            (0,_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('UseDifferentNameRuleIfWorkHasTagShow', !_Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagShow);
        });
        // 切换显示添加规则的区域
        this.showAddBtn.addEventListener('click', () => {
            this.addWrapShow = !this.addWrapShow;
        });
        // 添加规则的按钮
        this.addBtn.addEventListener('click', () => {
            this.addRule(this.addTagsInput.value, this.addRuleInput.value);
        });
        // 取消添加的按钮
        this.cancelBtn.addEventListener('click', () => {
            this.addWrapShow = false;
        });
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_1__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            if (data.name === 'UseDifferentNameRuleIfWorkHasTagShow') {
                this.showListWrap();
            }
            if (data.name === 'UseDifferentNameRuleIfWorkHasTagList') {
                this.createAllList();
            }
        });
    }
    showListWrap() {
        const show = _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagShow;
        this.listWrap.style.display = show ? 'flex' : 'none';
        _Lang__WEBPACK_IMPORTED_MODULE_2__.lang.updateText(this.expandBtn, show ? '_收起' : '_展开');
    }
    // 根据规则动态创建 html
    createAllList() {
        this.totalSpan.textContent =
            _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.length.toString();
        this.listWrap.innerHTML = '';
        const df = document.createDocumentFragment();
        for (const { id, tags, rule, } of _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList) {
            df.append(this.createOneList(id, tags, rule));
        }
        this.listWrap.append(df);
    }
    // 创建规则对应的元素，并绑定事件
    createOneList(id, tags, rule) {
        const html = `
      <div class="inputItem id">
        <span>${id}</span>
      </div>

      <div class="inputItem tags">
        <input type="text" class="setinput_style1 blue" data-tagsInput="${id}" value="${tags}" />
      </div>

      <div class="inputItem rule">
        <input type="text" class="has_tip setinput_style1 blue" data-ruleInput="${id}" value="${rule}" />
      </div>

      <div class="btns">
        <button type="button" class="textButton" data-updateRule="${id}" data-xztitle="_更新">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-gengxin"></use>
          </svg>
        </button>

        <button type="button" class="textButton" data-deleteRule="${id}" data-xztitle="_删除">
          <svg class="icon" aria-hidden="true">
            <use xlink:href="#icon-shanchu1"></use>
          </svg>
        </button>
    </div>`;
        const element = document.createElement('div');
        element.classList.add('settingItem');
        element.dataset.key = id.toString();
        element.innerHTML = html;
        const updateRule = element.querySelector(`button[data-updateRule='${id}']`);
        const deleteRule = element.querySelector(`button[data-deleteRule='${id}']`);
        const tagsInput = element.querySelector(`input[data-tagsInput='${id}']`);
        const ruleInput = element.querySelector(`input[data-ruleInput='${id}']`);
        [tagsInput, ruleInput].forEach((el) => {
            el?.addEventListener('change', () => {
                if (el.value) {
                    this.updateRule(id, tagsInput.value, ruleInput.value, false);
                }
            });
        });
        // 更新规则
        updateRule?.addEventListener('click', () => {
            this.updateRule(id, tagsInput.value, ruleInput.value);
        });
        // 删除规则
        deleteRule?.addEventListener('click', () => {
            this.deleteRule(id);
        });
        return element;
    }
    // 检查用户输入的值
    checkValue(tagsInput, rule) {
        if (!tagsInput || !rule) {
            _MsgBox__WEBPACK_IMPORTED_MODULE_6__.msgBox.error(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_必填项不能为空'));
            return false;
        }
        return {
            tags: _utils_Utils__WEBPACK_IMPORTED_MODULE_7__.Utils.string2array(tagsInput),
            rule,
        };
    }
    // 添加规则
    async addRule(tagsInput, rule) {
        const check = this.checkValue(tagsInput, rule);
        if (!check) {
            return;
        }
        const idList = _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.map((item) => item.id);
        const id = idList.length === 0 ? 0 : Math.max(...idList) + 1;
        const data = {
            id: id,
            tags: check.tags,
            rule: rule,
        };
        _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.push(data);
        (0,_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('UseDifferentNameRuleIfWorkHasTagList', _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        console.log(..._Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        this.addWrapShow = false;
        _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_添加成功'));
    }
    // 更新规则
    // tip 表示是否用显示操作成功的提示。当用户点击了更新按钮时应该显示提示；输入内容变化导致的自动更新可以不显示提示
    async updateRule(id, tagsInput, rule, tip = true) {
        const check = this.checkValue(tagsInput, rule);
        if (!check) {
            return;
        }
        let old = _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.find((item) => item.id === id);
        if (old) {
            old.tags = check.tags;
            old.rule = rule;
        }
        else {
            return;
        }
        (0,_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('UseDifferentNameRuleIfWorkHasTagList', _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        console.log(..._Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        this.addWrapShow = false;
        if (tip) {
            _Toast__WEBPACK_IMPORTED_MODULE_5__.toast.success(_Lang__WEBPACK_IMPORTED_MODULE_2__.lang.transl('_更新成功'));
        }
    }
    // 删除规则
    deleteRule(id) {
        let index = _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.findIndex((item) => item.id === id);
        if (index > -1) {
            _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList.splice(index, 1);
        }
        else {
            return;
        }
        (0,_Settings__WEBPACK_IMPORTED_MODULE_3__.setSetting)('UseDifferentNameRuleIfWorkHasTagList', _Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        console.log(..._Settings__WEBPACK_IMPORTED_MODULE_3__.settings.UseDifferentNameRuleIfWorkHasTagList);
        this.removeListElement(id);
    }
    removeListElement(id) {
        const listElement = this.listWrap.querySelector(`.settingItem[data-key='${id}']`);
        listElement?.remove();
    }
}
new UseDifferentNameRuleIfWorkHasTag();


/***/ }),

/***/ "./src/ts/store/CacheWorkData.ts":
/*!***************************************!*\
  !*** ./src/ts/store/CacheWorkData.ts ***!
  \***************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   cacheWorkData: () => (/* binding */ cacheWorkData)
/* harmony export */ });
// 本程序有多个模块需要在抓取流程之外获取作品数据
// 为了避免重复发起请求，以及解决浏览器有时候不读取缓存的问题，所以在这里缓存一些作品数据
// 即使下载器获取过某个作品的数据，但是以后再次请求时，浏览器也有可能不会读取缓存，而是重新发起请求。
class CacheWorkData {
    constructor() {
        this.cache = [];
        // 一个图像作品的数据大约是 5 KB
        this.max = 20;
    }
    set(data) {
        if (this.has(data.body.id)) {
            return;
        }
        if (this.cache.length >= this.max) {
            this.cache.shift();
        }
        this.cache.push(data);
    }
    get(id) {
        return this.cache.find((val) => val.body.id === id);
    }
    has(id) {
        return this.cache.some((val) => val.body.id === id);
    }
}
const cacheWorkData = new CacheWorkData();



/***/ }),

/***/ "./src/ts/store/IdListWithPageNo.ts":
/*!******************************************!*\
  !*** ./src/ts/store/IdListWithPageNo.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   idListWithPageNo: () => (/* binding */ idListWithPageNo)
/* harmony export */ });
/* harmony import */ var _Store__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _utils_Utils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../utils/Utils */ "./src/ts/utils/Utils.ts");


// 这是为了解决抓取多个列表页面时，获得的 id 数据顺序混乱的问题
// 这个类会保留每个 id 所处的页码。抓取完成后可以把这些 id 按页码顺序排列，保证 id 的顺序和在页码里的顺序一致
class IdListWithPageNo {
    constructor() {
        // 存储 id 列表，按 pageId 不同分别存储
        this.allList = {};
    }
    // 添加一条记录
    add(pageId, idData, page) {
        if (this.allList[pageId] === undefined) {
            this.allList[pageId] = [];
        }
        this.allList[pageId].push({
            id: idData.id,
            type: idData.type,
            page: page,
        });
    }
    // 清空记录
    clear(pageId) {
        if (this.allList[pageId]) {
            delete this.allList[pageId];
        }
    }
    // 排序
    sort(pageId) {
        if (this.allList[pageId]) {
            this.allList[pageId].sort(_utils_Utils__WEBPACK_IMPORTED_MODULE_1__.Utils.sortByProperty('page', 'asc'));
        }
    }
    // 转储到 store.idList 里
    // 自动排序
    // 转储之后自动清空
    store(pageId) {
        if (this.allList[pageId]) {
            this.sort(pageId);
            for (const data of this.allList[pageId]) {
                _Store__WEBPACK_IMPORTED_MODULE_0__.store.idList.push({
                    id: data.id,
                    type: data.type,
                });
            }
            this.clear(pageId);
        }
    }
    // 如果没有值，返回的就是 undefined
    get(pageId) {
        return this.allList[pageId];
    }
}
const idListWithPageNo = new IdListWithPageNo();



/***/ }),

/***/ "./src/ts/store/SaveArtworkData.ts":
/*!*****************************************!*\
  !*** ./src/ts/store/SaveArtworkData.ts ***!
  \*****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   saveArtworkData: () => (/* binding */ saveArtworkData)
/* harmony export */ });
/* harmony import */ var _API__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../API */ "./src/ts/API.ts");
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Store__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");
/* harmony import */ var _Log__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../Log */ "./src/ts/Log.ts");






// 保存图片作品的数据
class SaveArtworkData {
    async save(data) {
        // 获取需要检查的信息
        const body = data.body;
        const fullWidth = body.width; // 原图宽度
        const fullHeight = body.height; // 原图高度
        const bmk = body.bookmarkCount; // 收藏数
        const tags = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.extractTags(data); // tag 列表
        const tagsWithTransl = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.extractTags(data, 'both'); // 保存 tag 列表，附带翻译后的 tag
        const tagsTranslOnly = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.extractTags(data, 'transl'); // 保存翻译后的 tag 列表
        const aiMarkString = _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.getAIGeneratedMark(body.aiType);
        if (aiMarkString) {
            tags.unshift(aiMarkString);
            tagsWithTransl.unshift(aiMarkString);
            tagsTranslOnly.unshift(aiMarkString);
        }
        const filterOpt = {
            aiType: body.aiType,
            createDate: body.createDate,
            id: body.id,
            workType: body.illustType,
            tags: tagsWithTransl,
            pageCount: body.pageCount,
            bookmarkCount: bmk,
            bookmarkData: body.bookmarkData,
            width: body.pageCount === 1 ? fullWidth : 0,
            height: body.pageCount === 1 ? fullHeight : 0,
            mini: body.pageCount === 1 ? body.urls.mini : undefined,
            userId: body.userId,
            xRestrict: body.xRestrict,
        };
        // 对于多图作品，其宽高和颜色不在这里进行检查。也就是只会在下载时检查。
        // 这是因为在多图作品里，第一张图片的宽高和颜色不能代表剩余的图片。
        // 检查通过
        if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_1__.filter.check(filterOpt)) {
            const idNum = parseInt(body.id);
            const title = body.title; // 作品标题
            const userId = body.userId; // 用户id
            const user = body.userName; // 用户名
            const pageCount = body.pageCount;
            const bookmarked = !!body.bookmarkData;
            // 保存作品在排行榜上的编号
            const rankData = _Store__WEBPACK_IMPORTED_MODULE_3__.store.getRankList(body.id);
            const rank = rankData ? rankData : null;
            const seriesTitle = body.seriesNavData ? body.seriesNavData.title : '';
            const seriesOrder = body.seriesNavData ? body.seriesNavData.order : null;
            // 储存作品信息
            if (body.illustType === 0 || body.illustType === 1) {
                // 插画或漫画
                const imgUrl = body.urls.original; // 作品的原图 URL
                if (imgUrl === null) {
                    _Log__WEBPACK_IMPORTED_MODULE_5__.log.error(`${_Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.createWorkLink(body.id, true)} URLs are null`);
                    return;
                }
                const tempExt = imgUrl.split('.');
                const ext = tempExt[tempExt.length - 1];
                // 添加作品信息
                _Store__WEBPACK_IMPORTED_MODULE_3__.store.addResult({
                    aiType: body.aiType,
                    id: body.id,
                    idNum: idNum,
                    // 对于插画和漫画的缩略图，当一个作品包含多个图片文件时，需要转换缩略图 url
                    thumb: body.pageCount > 1
                        ? _Tools__WEBPACK_IMPORTED_MODULE_4__.Tools.convertArtworkThumbURL(body.urls.thumb, 0)
                        : body.urls.thumb,
                    pageCount: pageCount,
                    original: imgUrl,
                    regular: body.urls.regular,
                    small: body.urls.small,
                    title: title,
                    description: body.description,
                    tags: tags,
                    tagsWithTransl: tagsWithTransl,
                    tagsTranslOnly: tagsTranslOnly,
                    user: user,
                    userId: userId,
                    fullWidth: fullWidth,
                    fullHeight: fullHeight,
                    ext: ext,
                    bmk: bmk,
                    bmkId: body.bookmarkData ? body.bookmarkData.id : '',
                    bookmarked: bookmarked,
                    date: body.createDate,
                    uploadDate: body.uploadDate,
                    type: body.illustType,
                    rank: rank,
                    seriesTitle: seriesTitle,
                    seriesOrder: seriesOrder,
                    seriesId: body.seriesNavData ? body.seriesNavData.seriesId : null,
                    viewCount: body.viewCount,
                    likeCount: body.likeCount,
                    commentCount: body.commentCount,
                    xRestrict: body.xRestrict,
                    sl: body.sl,
                });
            }
            else if (body.illustType === 2) {
                // 动图
                // 获取动图的信息
                const meta = await _API__WEBPACK_IMPORTED_MODULE_0__.API.getUgoiraMeta(body.id);
                // 动图帧延迟数据
                const ugoiraInfo = {
                    frames: meta.body.frames,
                    mime_type: meta.body.mime_type,
                };
                // 当下载图片的方形缩略图时，它的后缀名从 url 中提取。
                // 此时不应该把它的后缀名设置为动图的保存格式，因为缩略图无法转换成动图
                let ext = _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.ugoiraSaveAs;
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.imageSize === 'thumb') {
                    const tempExt = body.urls.thumb.split('.');
                    ext = tempExt[tempExt.length - 1];
                }
                _Store__WEBPACK_IMPORTED_MODULE_3__.store.addResult({
                    aiType: body.aiType,
                    id: body.id,
                    idNum: idNum,
                    thumb: body.urls.thumb,
                    pageCount: pageCount,
                    original: meta.body.originalSrc,
                    regular: meta.body.src,
                    small: meta.body.src,
                    title: title,
                    description: body.description,
                    tags: tags,
                    tagsWithTransl: tagsWithTransl,
                    tagsTranslOnly: tagsTranslOnly,
                    user: user,
                    userId: userId,
                    fullWidth: fullWidth,
                    fullHeight: fullHeight,
                    ext: ext,
                    bmk: bmk,
                    bmkId: body.bookmarkData ? body.bookmarkData.id : '',
                    bookmarked: bookmarked,
                    date: body.createDate,
                    uploadDate: body.uploadDate,
                    type: body.illustType,
                    rank: rank,
                    ugoiraInfo: ugoiraInfo,
                    seriesTitle: seriesTitle,
                    seriesOrder: seriesOrder,
                    viewCount: body.viewCount,
                    likeCount: body.likeCount,
                    commentCount: body.commentCount,
                    xRestrict: body.xRestrict,
                    sl: body.sl,
                });
            }
        }
    }
}
const saveArtworkData = new SaveArtworkData();



/***/ }),

/***/ "./src/ts/store/SaveNovelData.ts":
/*!***************************************!*\
  !*** ./src/ts/store/SaveNovelData.ts ***!
  \***************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   saveNovelData: () => (/* binding */ saveNovelData)
/* harmony export */ });
/* harmony import */ var _filter_Filter__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../filter/Filter */ "./src/ts/filter/Filter.ts");
/* harmony import */ var _Store__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Store */ "./src/ts/store/Store.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");




// 保存单个小说作品的数据
class SaveNovelData {
    async save(data) {
        // 小说没有 illustType 属性， 把小说的 illustType 设置为 3，这是为了方便检查
        const illustType = 3;
        // 获取需要检查的信息
        const body = data.body;
        const bmk = body.bookmarkCount; // 收藏数
        const tags = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.extractTags(data); // tag 列表
        // 小说的标签没有进行翻译，所以没有翻译后的标签
        const aiMarkString = _Tools__WEBPACK_IMPORTED_MODULE_3__.Tools.getAIGeneratedMark(body.aiType);
        if (aiMarkString) {
            tags.unshift(aiMarkString);
        }
        const filterOpt = {
            aiType: body.aiType,
            createDate: body.createDate,
            id: body.id,
            workType: illustType,
            tags: tags,
            bookmarkCount: bmk,
            bookmarkData: body.bookmarkData,
            userId: body.userId,
            xRestrict: body.xRestrict,
        };
        // 检查通过
        if (await _filter_Filter__WEBPACK_IMPORTED_MODULE_0__.filter.check(filterOpt)) {
            const id = body.id;
            const idNum = parseInt(id);
            const title = body.title;
            const userId = body.userId;
            const user = body.userName;
            const bookmarked = !!body.bookmarkData;
            // 保存作品在排行榜上的编号
            const rankData = _Store__WEBPACK_IMPORTED_MODULE_1__.store.getRankList(id);
            const rank = rankData ? rankData : null;
            // 系列标题和序号
            const seriesTitle = body.seriesNavData ? body.seriesNavData.title : '';
            const seriesOrder = body.seriesNavData ? body.seriesNavData.order : null;
            // 保存小说的一些元数据
            let meta = '';
            let metaArr = [];
            const pageUrl = `https://www.pixiv.net/novel/show.php?id=${id}`;
            const tagsA = [];
            for (const tag of tags) {
                tagsA.push('#' + tag);
            }
            metaArr.push(title, user, pageUrl, body.description, tagsA.join('\n'));
            meta = metaArr.join('\n\n') + '\n\n\n';
            // 提取嵌入的图片资源
            let embeddedImages = null;
            if (body.textEmbeddedImages) {
                embeddedImages = {};
                for (const [id, value] of Object.entries(body.textEmbeddedImages)) {
                    embeddedImages[id] = value.urls.original;
                }
            }
            // 添加作品信息
            _Store__WEBPACK_IMPORTED_MODULE_1__.store.addResult({
                aiType: body.aiType,
                id: id,
                idNum: idNum,
                thumb: body.coverUrl || undefined,
                title: title,
                description: body.description,
                tags: tags,
                tagsWithTransl: tags,
                tagsTranslOnly: tags,
                user: user,
                userId: userId,
                // 这里的 ext 并不重要，下载时会根据 novelSaveAs 设置自动生成对应的数据
                ext: _setting_Settings__WEBPACK_IMPORTED_MODULE_2__.settings.novelSaveAs,
                bmk: bmk,
                bmkId: body.bookmarkData ? body.bookmarkData.id : '',
                bookmarked: bookmarked,
                date: body.createDate,
                uploadDate: body.uploadDate,
                type: illustType,
                rank: rank,
                seriesTitle: seriesTitle,
                seriesOrder: seriesOrder,
                seriesId: body.seriesNavData ? body.seriesNavData.seriesId : null,
                viewCount: body.viewCount,
                likeCount: body.likeCount,
                commentCount: body.commentCount,
                novelMeta: {
                    id: body.id,
                    title: body.title,
                    content: this.replaceFlag(body.content),
                    description: body.description,
                    coverUrl: body.coverUrl,
                    createDate: body.createDate,
                    userName: body.userName,
                    embeddedImages: embeddedImages,
                    meta: meta,
                },
                xRestrict: body.xRestrict,
            });
        }
    }
    // '[[jumpuri:予約ページ>https://www.amazon.co.jp/dp/4758092486]]'
    // 替换成
    // '予約ページ（https://www.amazon.co.jp/dp/4758092486）'
    replaceJumpuri(str) {
        let reg = /\[\[jumpuri:(.*?)>(.*?)\]\]/g;
        let temp;
        while ((temp = reg.exec(str))) {
            str = str.replace(temp[0], `${temp[1].trim()}（${temp[2].trim()}）`);
            reg.lastIndex = 0;
        }
        return str;
    }
    // > '[[rb:莉莉丝 > Lilith]]'
    // 替换成
    // '莉莉丝（Lilith）'
    replaceRb(str) {
        let reg = /\[\[rb:(.*?)>(.*?)\]\]/g;
        let temp;
        while ((temp = reg.exec(str))) {
            str = str.replace(temp[0], `${temp[1].trim()}（${temp[2].trim()}）`);
            reg.lastIndex = 0;
        }
        return str;
    }
    // > '[chapter:标题]'
    // 替换成
    // '标题'
    replaceChapter(str) {
        const reg = /\[chapter:(.*?)\]/g;
        let temp;
        while ((temp = reg.exec(str))) {
            str = str.replace(temp[0], temp[1]);
            reg.lastIndex = 0;
        }
        return str;
    }
    // 对小说里的一些标记进行替换
    replaceFlag(str) {
        str = str.replace(/\[newpage\]/g, '');
        str = this.replaceJumpuri(str);
        str = str.replace(/\[jump:.*?\]/g, '');
        str = this.replaceRb(str);
        str = this.replaceChapter(str);
        return str;
    }
}
const saveNovelData = new SaveNovelData();



/***/ }),

/***/ "./src/ts/store/States.ts":
/*!********************************!*\
  !*** ./src/ts/store/States.ts ***!
  \********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   states: () => (/* binding */ states)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");

// 储存下载器内部产生的、会变化的状态
// 这里的状态不需要持久化保存
// 状态的值通常只由单一的模块修改
class States {
    constructor() {
        /**指示 settings 是否初始化完毕 */
        this.settingInitialized = false;
        /**表示下载器是否处于繁忙状态
         *
         * 繁忙：下载器正在抓取作品，或者正在下载文件，或者正在批量添加收藏
         */
        this.busy = false;
        /**快速下载标记
         *
         * 快速下载模式中不会显示下载面板，并且总是会自动开始下载
         *
         * 启动快速下载时设为 true，下载完成或中止时复位到 false
         */
        this.quickCrawl = false;
        /**指示是否应该停止抓取 */
        this.stopCrawl = false;
        /**在排行榜抓取时，是否只抓取“首次登场”的作品 */
        // 修改者：InitRankingArtworkPage 模块修改这个状态
        this.debut = false;
        /**收藏模式的标记 */
        // 开始批量收藏时设为 true，收藏完成之后复位到 false
        this.bookmarkMode = false;
        /**合并系列小说时使用的标记 */
        this.mergeNovel = false;
        /**抓取标签列表时使用的标记 */
        this.crawlTagList = false;
        /**是否处于手动选择作品状态 */
        this.selectWork = false;
        /**是否处于下载中 */
        this.downloading = false;
        /**是否应用慢速抓取模式 */
        // 由 InitPageBase 修改它的值
        this.slowCrawlMode = false;
        this.bindEvents();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingInitialized, () => {
            this.settingInitialized = true;
        });
        const idle = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.stopCrawl,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.bookmarkModeEnd,
        ];
        idle.forEach((type) => {
            window.addEventListener(type, () => {
                this.busy = false;
            });
        });
        const busy = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStart,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.bookmarkModeStart,
        ];
        busy.forEach((type) => {
            window.addEventListener(type, () => {
                this.busy = true;
            });
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.bookmarkModeStart, () => {
            this.bookmarkMode = true;
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.bookmarkModeEnd, () => {
            this.bookmarkMode = false;
        });
        // 下载完成，或者下载中止时，复位快速下载类状态
        const resetQuickState = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlEmpty,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadCancel,
        ];
        for (const ev of resetQuickState) {
            window.addEventListener(ev, () => {
                this.quickCrawl = false;
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStart, () => {
            this.downloading = true;
        });
        const downloadIdle = [
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadPause,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop,
            _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadComplete,
        ];
        for (const ev of downloadIdle) {
            window.addEventListener(ev, () => {
                this.downloading = false;
            });
        }
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.settingChange, (ev) => {
            const data = ev.detail.data;
            // 当用户关闭设置里的慢速抓取时，在这里把慢速抓取模式的标记设为 false
            // 但是当用户开启慢速抓取时，不应该在这里把标记设为 true
            if (data.name === 'slowCrawl' && data.value === false) {
                this.slowCrawlMode = false;
            }
        });
    }
}
const states = new States();



/***/ }),

/***/ "./src/ts/store/Store.ts":
/*!*******************************!*\
  !*** ./src/ts/store/Store.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   store: () => (/* binding */ store)
/* harmony export */ });
/* harmony import */ var _EVT__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../EVT */ "./src/ts/EVT.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _Tools__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../Tools */ "./src/ts/Tools.ts");



// 生成抓取结果
class Store {
    constructor() {
        this.loggedUserID = '';
        this.idList = []; // 储存从列表中抓取到的作品的 id
        this.waitingIdList = []; // 下载器尚未完成本次下载时，如果有新的下载请求，则添加到这里，下载完成后再处理
        this.resultMeta = []; // 储存抓取结果的元数据。
        // 当用于图片作品时，它可以根据每个作品需要下载多少张，生成每一张图片的信息
        /**系列小说的设定资料 */
        this.novelSeriesGlossary = '';
        this.artworkIDList = []; // 储存抓取到的图片作品的 id 列表，用来避免重复添加
        this.novelIDList = []; // 储存抓取到的小说作品的 id 列表，用来避免重复添加
        this.result = []; // 储存抓取结果
        /**记录从每个作品里下载多少个文件 */
        this.downloadCount = {};
        this.remainingDownload = 0; // 剩余多少个等待下载和保存的文件
        this.rankList = {}; // 储存作品在排行榜中的排名
        this.tag = ''; // 开始抓取时，储存页面此时的 tag
        this.title = ''; // 开始抓取时，储存页面此时的 title
        this.URLWhenCrawlStart = ''; // 开始抓取时，储存页面此时的 URL
        this.crawlCompleteTime = new Date();
        /**只下载作品里的一部分图片 */
        this.downloadOnlyPart = {};
        this.fileDataDefault = {
            aiType: 0,
            idNum: 0,
            id: '',
            original: '',
            thumb: '',
            regular: '',
            small: '',
            title: '',
            description: '',
            pageCount: 1,
            index: 0,
            tags: [],
            tagsWithTransl: [],
            tagsTranslOnly: [],
            user: '',
            userId: '',
            fullWidth: 0,
            fullHeight: 0,
            ext: '',
            bmk: 0,
            bookmarked: false,
            bmkId: '',
            date: '',
            uploadDate: '',
            type: 0,
            rank: null,
            ugoiraInfo: null,
            seriesTitle: null,
            seriesOrder: null,
            seriesId: null,
            novelMeta: null,
            likeCount: 0,
            viewCount: 0,
            commentCount: 0,
            xRestrict: 0,
            sl: null,
        };
        this.loggedUserID = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getLoggedUserID();
        this.bindEvents();
    }
    // 恢复未完成的下载之后，生成 downloadCount 数据
    // 因为保存的任务数据里没有 downloadCount，并且恢复数据时也没有生成 downloadCount
    resetDownloadCount() {
        this.downloadCount = {};
        for (const r of this.result) {
            this.downloadCount[r.idNum] = (this.downloadCount[r.idNum] || 0) + 1;
        }
    }
    setDownloadOnlyPart(workID, indexList) {
        if (this.downloadOnlyPart[workID]) {
            this.downloadOnlyPart[workID] = Array.from(new Set(this.downloadOnlyPart[workID].concat(indexList)));
        }
        else {
            this.downloadOnlyPart[workID] = indexList;
        }
    }
    // 添加每个作品的信息。只需要传递有值的属性
    // 如果一个作品有多张图片，只需要传递第一张图片的数据。后面的数据会根据设置自动生成
    addResult(data) {
        // 检查该作品 id 是否已存在，已存在则不添加
        const useList = data.type === 3 ? this.novelIDList : this.artworkIDList;
        if (data.idNum !== undefined) {
            if (useList.includes(data.idNum)) {
                return;
            }
            useList.push(data.idNum);
        }
        // 添加该作品的元数据
        const workData = Object.assign({}, this.fileDataDefault, data);
        // 注意：由于 Object.assign 不是深拷贝，所以不可以修改 result 的引用类型数据，否则会影响到源对象
        // 可以修改基础类型的数据
        if (workData.type === 0 || workData.type === 1) {
            workData.id = workData.idNum + `_p0`;
        }
        else {
            workData.id = workData.idNum.toString();
        }
        this.resultMeta.push(workData);
        _EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.fire('addResult', workData);
        // 保存这个作品里每个文件的数据
        if (workData.type === 2 || workData.type === 3) {
            // 动图和小说作品直接添加
            this.result.push(workData);
            this.downloadCount[workData.idNum] = 1;
        }
        else {
            // 插画和漫画
            // 储存需要下载的图片的索引
            let fileIndexList = [];
            // 只下载部分图片
            if (this.downloadOnlyPart[workData.idNum]) {
                fileIndexList = this.downloadOnlyPart[workData.idNum];
                delete this.downloadOnlyPart[workData.idNum];
            }
            else {
                // 下载全部图片
                let total = workData.pageCount;
                // 如果下载全部图片，则检查一些过滤器
                // 只下载部分图片时，用户已经手动指定了要下载的图片，所以不要检查这些过滤器
                // 多图作品只下载前几张图片
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.firstFewImagesSwitch) {
                    total = Math.min(workData.pageCount, _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.firstFewImages);
                }
                // 不抓取多图作品的最后一张图片
                if (_setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.doNotDownloadLastImageOfMultiImageWork &&
                    workData.pageCount > 1) {
                    total = Math.min(total, workData.pageCount - 1);
                }
                // 特定用户的多图作品不下载最后几张图片
                if (workData.pageCount > 1) {
                    const removeLastFew = _setting_Settings__WEBPACK_IMPORTED_MODULE_1__.settings.DoNotDownloadLastFewImagesList.find((item) => item.uid === Number.parseInt(workData.userId));
                    if (removeLastFew && removeLastFew.value > 0) {
                        let number = workData.pageCount - removeLastFew.value;
                        if (number < 1) {
                            // 用户设置的值有可能把这个作品的图片全部排除了，此时只跳过最后一张
                            number = workData.pageCount - 1;
                        }
                        total = Math.min(total, number);
                    }
                }
                for (let i = 0; i < total; i++) {
                    fileIndexList.push(i);
                }
            }
            this.downloadCount[workData.idNum] = fileIndexList.length;
            // 生成每个图片的数据
            const p0 = 'p0';
            for (const i of fileIndexList) {
                const fileData = Object.assign({}, workData);
                const pi = 'p' + i;
                fileData.index = i;
                fileData.id = fileData.id.replace(p0, pi);
                fileData.original = fileData.original.replace(p0, pi);
                fileData.regular = fileData.regular.replace(p0, pi);
                fileData.small = fileData.small.replace(p0, pi);
                fileData.thumb = fileData.thumb.replace(p0, pi);
                this.result.push(fileData);
            }
        }
    }
    getRankList(index) {
        return this.rankList[index];
    }
    setRankList(id, rank) {
        this.rankList[id] = rank;
    }
    findResult(id) {
        for (const result of this.result) {
            if (result.id === id) {
                return result;
            }
        }
    }
    reset() {
        this.resultMeta = [];
        this.artworkIDList = [];
        this.novelIDList = [];
        this.result = [];
        this.idList = [];
        this.waitingIdList = [];
        this.rankList = {};
        this.remainingDownload = 0;
        this.novelSeriesGlossary = '';
        this.tag = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getTagFromURL();
        this.title = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getPageTitle();
    }
    bindEvents() {
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.crawlStart, () => {
            this.URLWhenCrawlStart = window.location.href;
            this.reset();
        });
        // 停止下载时，清空等待下载的任务
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.downloadStop, () => {
            this.waitingIdList = [];
        });
        window.addEventListener(_EVT__WEBPACK_IMPORTED_MODULE_0__.EVT.list.resume, () => {
            this.tag = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getTagFromURL();
            this.title = _Tools__WEBPACK_IMPORTED_MODULE_2__.Tools.getPageTitle();
        });
    }
}
const store = new Store();



/***/ }),

/***/ "./src/ts/store/WorkPublishTimeNovels.ts":
/*!***********************************************!*\
  !*** ./src/ts/store/WorkPublishTimeNovels.ts ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   novelData: () => (/* binding */ novelData)
/* harmony export */ });
const novelData = [
    [129, 1280384594000],
    [10007, 1280558885000],
    [20001, 1281099833000],
    [30006, 1282016726000],
    [40000, 1283058990000],
    [50003, 1284130800000],
    [60002, 1285248459000],
    [70000, 1286383797000],
    [80000, 1287587350000],
    [90003, 1288702643000],
    [100000, 1289750009000],
    [110000, 1290800661000],
    [120000, 1291911048000],
    [130000, 1293023102000],
    [140001, 1293990763000],
    [150004, 1294921696000],
    [160004, 1295784678000],
    [170001, 1296580286000],
    [180000, 1297357894000],
    [190002, 1298051246000],
    [200000, 1298800698000],
    [210000, 1299544965000],
    [220000, 1300512363000],
    [230000, 1301213084000],
    [240000, 1301836904000],
    [250000, 1302561861000],
    [260000, 1303376035000],
    [270002, 1304084248000],
    [280000, 1304695785000],
    [290000, 1305338427000],
    [300000, 1305942926000],
    [310002, 1306521475000],
    [320000, 1307103716000],
    [330000, 1307628374000],
    [340000, 1308152212000],
    [350001, 1308671940000],
    [360003, 1309193210000],
    [370000, 1309707590000],
    [380001, 1310224382000],
    [390002, 1310744385000],
    [400000, 1311240590000],
    [410000, 1311714592000],
    [420003, 1312224244000],
    [430001, 1312731339000],
    [440001, 1313296075000],
    [450003, 1313764835000],
    [460000, 1314249696000],
    [470003, 1314711329000],
    [480000, 1315145953000],
    [490007, 1315659922000],
    [500000, 1316129991000],
    [510003, 1316556988000],
    [520000, 1316999883000],
    [530004, 1317484198000],
    [540001, 1317970536000],
    [550003, 1318383439000],
    [560001, 1318844138000],
    [570001, 1319298372000],
    [580000, 1319786020000],
    [590001, 1320157532000],
    [600000, 1320589354000],
    [610003, 1321033402000],
    [620000, 1321519466000],
    [630002, 1321925666000],
    [640000, 1322323144000],
    [650001, 1322778841000],
    [660000, 1323187376000],
    [670000, 1323619422000],
    [680002, 1324113471000],
    [690005, 1324542346000],
    [700000, 1324863292000],
    [710000, 1325265330000],
    [720001, 1325664232000],
    [730000, 1326030447000],
    [740000, 1326447371000],
    [750000, 1326813089000],
    [760000, 1327222222000],
    [770000, 1327616533000],
    [780001, 1327978661000],
    [790006, 1328363509000],
    [800000, 1328708326000],
    [810002, 1329049070000],
    [820000, 1329313827000],
    [830000, 1329661069000],
    [840000, 1330044197000],
    [850002, 1330399536000],
    [860000, 1330760171000],
    [870000, 1331091059000],
    [880001, 1331430688000],
    [890000, 1331740424000],
    [900000, 1332056343000],
    [910000, 1332342034000],
    [920004, 1332657590000],
    [930000, 1332950185000],
    [940001, 1333253995000],
    [950001, 1333531155000],
    [960003, 1333851677000],
    [970001, 1334204901000],
    [980001, 1334509654000],
    [990000, 1334915605000],
    [1000004, 1335194889000],
    [1010001, 1335577999000],
    [1020001, 1335865453000],
    [1030000, 1336148344000],
    [1040000, 1336430393000],
    [1050000, 1336801524000],
    [1060001, 1337093451000],
    [1070000, 1337435442000],
    [1080003, 1337748152000],
    [1090000, 1338048813000],
    [1100000, 1338386829000],
    [1110006, 1338706652000],
    [1120001, 1338995770000],
    [1130003, 1339304913000],
    [1140000, 1339594732000],
    [1150005, 1339900812000],
    [1160000, 1340181428000],
    [1170000, 1340467013000],
    [1180000, 1340798549000],
    [1190004, 1341081318000],
    [1200000, 1341378294000],
    [1210000, 1341627003000],
    [1220000, 1341838882000],
    [1230002, 1342107912000],
    [1240001, 1342366761000],
    [1250000, 1342624522000],
    [1260000, 1342893053000],
    [1270001, 1343141451000],
    [1280000, 1343411297000],
    [1290000, 1343663085000],
    [1300001, 1343920259000],
    [1310000, 1344169879000],
    [1320002, 1344409293000],
    [1330000, 1344659199000],
    [1340000, 1344896869000],
    [1350000, 1345124715000],
    [1360009, 1345359194000],
    [1370002, 1345599272000],
    [1380000, 1345835088000],
    [1390000, 1346075877000],
    [1400001, 1346330394000],
    [1410001, 1346550028000],
    [1420000, 1346779027000],
    [1430000, 1347060633000],
    [1440001, 1347279773000],
    [1450001, 1347538265000],
    [1460005, 1347781791000],
    [1470001, 1347973096000],
    [1480002, 1348233789000],
    [1490000, 1348421630000],
    [1500001, 1348700844000],
    [1510001, 1348937413000],
    [1520000, 1349157850000],
    [1530001, 1349372979000],
    [1540002, 1349606131000],
    [1550001, 1349795097000],
    [1560001, 1350052229000],
    [1570002, 1350269341000],
    [1580001, 1350532237000],
    [1590000, 1350758415000],
    [1600000, 1350997858000],
    [1610006, 1351261071000],
    [1620003, 1351479480000],
    [1630002, 1351693412000],
    [1640000, 1351949249000],
    [1650005, 1352189853000],
    [1660000, 1352449647000],
    [1670002, 1352637939000],
    [1680001, 1352884190000],
    [1690000, 1353154000000],
    [1700000, 1353402782000],
    [1710000, 1353638092000],
    [1720001, 1353848294000],
    [1730000, 1354118175000],
    [1740003, 1354378863000],
    [1750002, 1354631450000],
    [1760000, 1354938336000],
    [1770001, 1355153123000],
    [1780000, 1355444255000],
    [1790003, 1355670988000],
    [1800000, 1355934280000],
    [1810000, 1356190313000],
    [1820001, 1356371790000],
    [1830001, 1356616228000],
    [1840003, 1356868743000],
    [1850002, 1357070395000],
    [1860000, 1357313538000],
    [1870000, 1357548863000],
    [1880001, 1357824009000],
    [1890000, 1358074605000],
    [1900000, 1358265865000],
    [1910000, 1358569040000],
    [1920003, 1358779320000],
    [1930000, 1359041117000],
    [1940001, 1359285099000],
    [1950002, 1359536718000],
    [1960000, 1359745109000],
    [1970002, 1359966846000],
    [1980000, 1360210328000],
    [1990001, 1360429921000],
    [2000000, 1360642383000],
    [2010000, 1360844563000],
    [2020000, 1361026805000],
    [2030005, 1361263382000],
    [2040000, 1361519812000],
    [2050000, 1361713982000],
    [2060001, 1361973749000],
    [2070000, 1362225095000],
    [2080000, 1362416157000],
    [2090000, 1362671650000],
    [2100001, 1362903499000],
    [2110004, 1363125268000],
    [2120000, 1363354484000],
    [2130000, 1363575696000],
    [2140000, 1363788816000],
    [2150000, 1364031500000],
    [2160000, 1364224743000],
    [2170003, 1364468664000],
    [2180001, 1364685260000],
    [2190000, 1364872459000],
    [2200001, 1365085949000],
    [2210001, 1365305116000],
    [2220000, 1365521609000],
    [2230000, 1365821669000],
    [2240001, 1366035468000],
    [2250001, 1366338753000],
    [2260002, 1366554728000],
    [2270000, 1366817986000],
    [2280001, 1367079795000],
    [2290001, 1367301796000],
    [2300000, 1367535148000],
    [2310000, 1367755047000],
    [2320001, 1367938482000],
    [2330002, 1368198077000],
    [2340002, 1368402443000],
    [2350002, 1368671885000],
    [2360000, 1368908138000],
    [2370000, 1369143679000],
    [2380000, 1369399230000],
    [2390002, 1369586696000],
    [2400004, 1369887355000],
    [2410000, 1370104099000],
    [2420002, 1370341310000],
    [2430001, 1370603557000],
    [2440001, 1370788687000],
    [2450000, 1371038666000],
    [2460002, 1371286007000],
    [2470000, 1371481119000],
    [2480000, 1371732988000],
    [2490002, 1371958259000],
    [2500000, 1372170531000],
    [2510000, 1372430704000],
    [2520001, 1372617024000],
    [2530004, 1372871380000],
    [2540001, 1373120348000],
    [2550000, 1373293117000],
    [2560000, 1373552201000],
    [2570001, 1373799086000],
    [2580000, 1373990399000],
    [2590001, 1374251028000],
    [2600000, 1374478513000],
    [2610000, 1374695793000],
    [2620001, 1374935403000],
    [2630001, 1375151019000],
    [2640000, 1375366140000],
    [2650003, 1375582844000],
    [2660006, 1375789458000],
    [2670000, 1375982223000],
    [2680001, 1376219077000],
    [2690000, 1376419869000],
    [2700000, 1376641349000],
    [2710002, 1376830708000],
    [2720001, 1377025884000],
    [2730003, 1377257395000],
    [2740002, 1377439480000],
    [2750000, 1377658821000],
    [2760000, 1377871532000],
    [2770000, 1378042294000],
    [2780000, 1378279211000],
    [2790004, 1378512151000],
    [2800000, 1378707972000],
    [2810000, 1378919770000],
    [2820001, 1379167185000],
    [2830000, 1379338634000],
    [2840000, 1379588368000],
    [2850000, 1379791566000],
    [2860002, 1379998199000],
    [2870001, 1380244352000],
    [2880001, 1380458162000],
    [2890000, 1380709854000],
    [2900001, 1380955990000],
    [2910000, 1381149483000],
    [2920000, 1381395761000],
    [2930000, 1381600314000],
    [2940005, 1381806653000],
    [2950001, 1382032915000],
    [2960000, 1382266138000],
    [2970000, 1382497296000],
    [2980001, 1382722022000],
    [2990002, 1382958224000],
    [3000001, 1383198845000],
    [3010001, 1383402564000],
    [3020000, 1383581008000],
    [3030002, 1383836419000],
    [3040001, 1384069244000],
    [3050000, 1384263193000],
    [3060003, 1384524757000],
    [3070001, 1384737837000],
    [3080000, 1384996638000],
    [3090003, 1385212146000],
    [3100003, 1385431019000],
    [3110000, 1385699731000],
    [3120003, 1385900847000],
    [3130000, 1386153660000],
    [3140002, 1386408440000],
    [3150000, 1386604510000],
    [3160003, 1386865240000],
    [3170000, 1387110539000],
    [3180000, 1387367749000],
    [3190000, 1387604327000],
    [3200000, 1387802332000],
    [3210000, 1387970572000],
    [3220007, 1388160975000],
    [3230000, 1388398804000],
    [3240000, 1388570893000],
    [3250002, 1388768138000],
    [3260001, 1388976072000],
    [3270002, 1389194106000],
    [3280001, 1389440923000],
    [3290006, 1389592662000],
    [3300000, 1389790343000],
    [3310000, 1390038223000],
    [3320001, 1390228417000],
    [3330003, 1390483721000],
    [3340000, 1390708173000],
    [3350001, 1390916206000],
    [3360000, 1391147130000],
    [3370000, 1391328790000],
    [3380002, 1391519054000],
    [3390001, 1391742834000],
    [3400000, 1391928566000],
    [3410000, 1392108927000],
    [3420001, 1392300500000],
    [3430000, 1392440337000],
    [3440000, 1392581355000],
    [3450000, 1392818545000],
    [3460000, 1393053469000],
    [3470003, 1393240576000],
    [3480000, 1393469026000],
    [3490004, 1393675973000],
    [3500000, 1393852227000],
    [3510001, 1394081671000],
    [3520001, 1394285166000],
    [3530001, 1394462801000],
    [3540002, 1394697374000],
    [3550000, 1394880049000],
    [3560002, 1395050621000],
    [3570000, 1395246872000],
    [3580003, 1395474465000],
    [3590000, 1395659536000],
    [3600001, 1395846777000],
    [3610003, 1396065846000],
    [3620000, 1396244134000],
    [3630001, 1396417542000],
    [3640000, 1396614221000],
    [3650001, 1396791326000],
    [3660000, 1397022134000],
    [3670000, 1397274214000],
    [3680003, 1397465091000],
    [3690003, 1397723422000],
    [3700000, 1397926553000],
    [3710001, 1398151228000],
    [3720005, 1398391812000],
    [3730000, 1398595954000],
    [3740000, 1398778603000],
    [3750000, 1398968016000],
    [3760000, 1399202230000],
    [3770001, 1399372076000],
    [3780000, 1399569729000],
    [3790000, 1399790469000],
    [3800000, 1399992962000],
    [3810000, 1400246876000],
    [3820000, 1400425983000],
    [3830000, 1400679476000],
    [3840002, 1400911387000],
    [3850000, 1401098715000],
    [3860001, 1401339903000],
    [3870000, 1401551332000],
    [3880001, 1401759755000],
    [3890001, 1401983592000],
    [3900002, 1402204857000],
    [3910000, 1402398206000],
    [3920001, 1402640662000],
    [3930000, 1402831875000],
    [3940001, 1403027766000],
    [3950001, 1403273218000],
    [3960001, 1403448681000],
    [3970001, 1403689984000],
    [3980003, 1403925632000],
    [3990001, 1404107465000],
    [4000000, 1404318974000],
    [4010001, 1404563643000],
    [4020003, 1404739221000],
    [4030003, 1404955522000],
    [4040000, 1405172711000],
    [4050001, 1405356399000],
    [4060004, 1405613850000],
    [4070000, 1405857345000],
    [4080000, 1406040627000],
    [4090000, 1406291871000],
    [4100002, 1406484212000],
    [4110000, 1406727405000],
    [4120000, 1406945016000],
    [4130001, 1407141824000],
    [4140010, 1407340131000],
    [4150000, 1407562661000],
    [4160000, 1407734796000],
    [4170001, 1407935639000],
    [4180000, 1408125359000],
    [4190000, 1408330818000],
    [4200001, 1408532218000],
    [4210000, 1408722044000],
    [4220000, 1408901567000],
    [4230001, 1409137675000],
    [4240001, 1409330881000],
    [4250005, 1409500868000],
    [4260002, 1409744703000],
    [4270002, 1409967629000],
    [4280003, 1410150372000],
    [4290000, 1410357905000],
    [4300000, 1410596035000],
    [4310000, 1410775372000],
    [4320000, 1410968927000],
    [4330000, 1411212383000],
    [4340001, 1411396391000],
    [4350000, 1411601413000],
    [4360000, 1411826286000],
    [4370002, 1412006421000],
    [4380000, 1412254595000],
    [4390001, 1412446854000],
    [4400000, 1412610825000],
    [4410002, 1412853440000],
    [4420000, 1413040935000],
    [4430002, 1413210796000],
    [4440001, 1413449251000],
    [4450003, 1413674476000],
    [4460002, 1413895354000],
    [4470002, 1414148442000],
    [4480003, 1414334560000],
    [4490000, 1414587176000],
    [4500000, 1414768048000],
    [4510002, 1414948293000],
    [4520000, 1415180576000],
    [4530000, 1415427122000],
    [4540000, 1415620756000],
    [4550000, 1415833945000],
    [4560002, 1416071344000],
    [4570004, 1416308460000],
    [4580002, 1416572690000],
    [4590000, 1416760063000],
    [4600002, 1417003566000],
    [4610002, 1417259849000],
    [4620001, 1417447473000],
    [4630000, 1417714518000],
    [4640000, 1417956598000],
    [4650000, 1418216048000],
    [4660000, 1418475241000],
    [4670000, 1418710339000],
    [4680004, 1418970521000],
    [4690002, 1419169105000],
    [4700005, 1419358497000],
    [4710000, 1419526254000],
    [4720000, 1419772928000],
    [4730000, 1419974551000],
    [4740000, 1420161657000],
    [4750001, 1420365381000],
    [4760002, 1420562341000],
    [4770000, 1420815773000],
    [4780001, 1421031408000],
    [4790001, 1421246156000],
    [4800000, 1421501308000],
    [4810000, 1421686861000],
    [4820000, 1421939002000],
    [4830000, 1422169526000],
    [4840000, 1422371808000],
    [4850000, 1422623205000],
    [4860000, 1422797374000],
    [4870000, 1423017739000],
    [4880000, 1423234605000],
    [4890000, 1423410499000],
    [4900000, 1423644484000],
    [4910001, 1423837444000],
    [4920000, 1423970388000],
    [4930001, 1424165273000],
    [4940001, 1424364687000],
    [4950002, 1424582000000],
    [4960001, 1424768846000],
    [4970001, 1424966114000],
    [4980000, 1425171871000],
    [4990000, 1425349395000],
    [5000000, 1425559592000],
    [5010001, 1425741575000],
    [5020003, 1425913279000],
    [5030003, 1426126824000],
    [5040001, 1426327854000],
    [5050000, 1426495511000],
    [5060001, 1426693282000],
    [5070005, 1426922090000],
    [5080000, 1427098056000],
    [5090000, 1427293125000],
    [5100000, 1427514626000],
    [5110000, 1427695785000],
    [5120002, 1427877197000],
    [5130002, 1428060146000],
    [5140001, 1428234288000],
    [5150000, 1428419973000],
    [5160000, 1428671937000],
    [5170002, 1428847629000],
    [5180001, 1429094087000],
    [5190000, 1429341649000],
    [5200003, 1429525955000],
    [5210000, 1429765217000],
    [5220000, 1429975534000],
    [5230001, 1430176975000],
    [5240002, 1430391697000],
    [5250000, 1430580945000],
    [5260000, 1430763266000],
    [5270001, 1430924583000],
    [5280000, 1431154114000],
    [5290000, 1431323463000],
    [5300002, 1431536550000],
    [5310000, 1431780262000],
    [5320000, 1431956743000],
    [5330000, 1432204529000],
    [5340001, 1432398271000],
    [5350006, 1432606153000],
    [5360001, 1432841545000],
    [5370000, 1433063471000],
    [5380003, 1433258662000],
    [5390000, 1433508887000],
    [5400000, 1433685193000],
    [5410001, 1433916258000],
    [5420000, 1434135276000],
    [5430001, 1434333869000],
    [5440001, 1434559422000],
    [5450000, 1434804693000],
    [5460001, 1434979921000],
    [5470000, 1435217857000],
    [5480001, 1435416104000],
    [5490000, 1435602027000],
    [5500001, 1435840850000],
    [5510000, 1436030904000],
    [5520003, 1436227024000],
    [5530000, 1436442515000],
    [5540003, 1436663422000],
    [5550000, 1436874223000],
    [5560001, 1437120062000],
    [5570000, 1437316067000],
    [5580000, 1437494808000],
    [5590000, 1437746039000],
    [5600002, 1437929627000],
    [5610001, 1438176113000],
    [5620000, 1438388870000],
    [5630000, 1438589507000],
    [5640000, 1438796233000],
    [5650000, 1439030571000],
    [5660004, 1439214144000],
    [5670001, 1439431273000],
    [5680000, 1439642569000],
    [5690001, 1439824201000],
    [5700005, 1440058580000],
    [5710000, 1440256699000],
    [5720000, 1440475809000],
    [5730002, 1440689040000],
    [5740000, 1440914751000],
    [5750003, 1441110359000],
    [5760000, 1441364816000],
    [5770000, 1441549697000],
    [5780006, 1441797710000],
    [5790001, 1442037713000],
    [5800002, 1442233888000],
    [5810000, 1442481291000],
    [5820003, 1442705357000],
    [5830000, 1442908303000],
    [5840000, 1443086697000],
    [5850003, 1443282432000],
    [5860000, 1443459310000],
    [5870006, 1443707232000],
    [5880001, 1443928257000],
    [5890001, 1444139874000],
    [5900001, 1444396252000],
    [5910001, 1444577096000],
    [5920000, 1444792260000],
    [5930002, 1445029691000],
    [5940001, 1445245605000],
    [5950000, 1445504572000],
    [5960000, 1445731426000],
    [5970002, 1445953442000],
    [5980000, 1446211281000],
    [5990002, 1446383992000],
    [6000000, 1446580875000],
    [6010003, 1446842203000],
    [6020001, 1447044347000],
    [6030000, 1447253873000],
    [6040000, 1447505627000],
    [6050000, 1447691164000],
    [6060001, 1447947945000],
    [6070005, 1448186860000],
    [6080000, 1448365784000],
    [6090000, 1448624170000],
    [6100002, 1448810246000],
    [6110000, 1449066515000],
    [6120000, 1449317021000],
    [6130002, 1449507406000],
    [6140001, 1449770690000],
    [6150000, 1450002546000],
    [6160000, 1450231069000],
    [6170001, 1450484327000],
    [6180001, 1450692134000],
    [6190000, 1450882802000],
    [6200001, 1451053403000],
    [6210000, 1451234149000],
    [6220003, 1451471784000],
    [6230000, 1451644655000],
    [6240000, 1451713332000],
    [6250000, 1451847328000],
    [6260005, 1452076808000],
    [6270000, 1452299253000],
    [6280000, 1452496401000],
    [6290000, 1452698011000],
    [6300000, 1452951233000],
    [6310000, 1453142080000],
    [6320001, 1453393135000],
    [6330000, 1453616322000],
    [6340001, 1453812318000],
    [6350000, 1454055730000],
    [6360003, 1454238250000],
    [6370000, 1454427033000],
    [6380000, 1454661000000],
    [6390000, 1454842409000],
    [6400000, 1455041335000],
    [6410000, 1455272058000],
    [6420000, 1455429130000],
    [6430000, 1455565696000],
    [6440000, 1455807564000],
    [6450000, 1456029749000],
    [6460000, 1456225146000],
    [6470000, 1456470713000],
    [6480000, 1456663870000],
    [6490003, 1456892356000],
    [6500001, 1457110676000],
    [6510002, 1457312408000],
    [6520000, 1457531526000],
    [6530000, 1457759895000],
    [6540001, 1457945527000],
    [6550000, 1458146778000],
    [6560000, 1458393176000],
    [6570001, 1458570750000],
    [6580000, 1458809559000],
    [6590000, 1459016231000],
    [6600000, 1459238945000],
    [6610000, 1459438183000],
    [6620004, 1459646525000],
    [6630000, 1459854652000],
    [6640001, 1460090525000],
    [6650000, 1460290792000],
    [6660000, 1460548807000],
    [6670000, 1460814501000],
    [6680000, 1461034745000],
    [6690000, 1461309246000],
    [6700004, 1461504246000],
    [6710000, 1461752821000],
    [6720003, 1461973135000],
    [6730001, 1462174879000],
    [6740001, 1462370008000],
    [6750001, 1462548340000],
    [6760000, 1462720908000],
    [6770002, 1462975121000],
    [6780000, 1463232933000],
    [6790007, 1463433933000],
    [6800002, 1463714821000],
    [6810000, 1463920668000],
    [6820000, 1464128839000],
    [6830004, 1464385492000],
    [6840000, 1464599226000],
    [6850001, 1464861239000],
    [6860002, 1465075280000],
    [6870000, 1465297401000],
    [6880001, 1465549772000],
    [6890000, 1465736580000],
    [6900000, 1465964679000],
    [6910001, 1466179197000],
    [6920000, 1466354374000],
    [6930001, 1466603732000],
    [6940002, 1466845739000],
    [6950000, 1467033992000],
    [6960000, 1467284783000],
    [6970001, 1467480681000],
    [6980000, 1467714202000],
    [6990002, 1467914778000],
    [7000001, 1468145606000],
    [7010000, 1468384915000],
    [7020000, 1468639043000],
    [7030001, 1468840463000],
    [7040002, 1469087617000],
    [7050000, 1469337012000],
    [7060002, 1469582162000],
    [7070002, 1469845430000],
    [7080002, 1470060235000],
    [7090000, 1470312128000],
    [7100000, 1470548609000],
    [7110000, 1470754953000],
    [7120000, 1470989043000],
    [7130001, 1471190531000],
    [7140000, 1471414854000],
    [7150004, 1471624400000],
    [7160000, 1471854212000],
    [7170000, 1472088374000],
    [7180002, 1472310616000],
    [7190001, 1472520312000],
    [7200000, 1472738227000],
    [7210002, 1472965256000],
    [7220001, 1473171102000],
    [7230000, 1473424874000],
    [7240001, 1473609515000],
    [7250000, 1473863715000],
    [7260001, 1474116184000],
    [7270003, 1474295985000],
    [7280000, 1474535039000],
    [7290000, 1474737285000],
    [7300000, 1474975696000],
    [7310000, 1475234975000],
    [7320000, 1475421025000],
    [7330001, 1475671020000],
    [7340001, 1475911455000],
    [7350005, 1476103986000],
    [7360002, 1476361587000],
    [7370000, 1476599211000],
    [7380001, 1476815745000],
    [7390000, 1477099543000],
    [7400000, 1477316178000],
    [7410001, 1477579038000],
    [7420003, 1477818419000],
    [7430002, 1478011264000],
    [7440000, 1478264690000],
    [7450002, 1478480338000],
    [7460006, 1478769027000],
    [7470001, 1478994247000],
    [7480000, 1479219274000],
    [7490000, 1479486058000],
    [7500002, 1479730140000],
    [7510003, 1479958570000],
    [7520003, 1480222232000],
    [7530001, 1480471579000],
    [7540000, 1480749927000],
    [7550000, 1480952230000],
    [7560000, 1481224069000],
    [7570001, 1481463117000],
    [7580000, 1481722387000],
    [7590002, 1481985058000],
    [7600001, 1482227418000],
    [7610001, 1482478601000],
    [7620000, 1482645299000],
    [7630003, 1482841839000],
    [7640001, 1483074183000],
    [7650000, 1483258759000],
    [7660004, 1483452663000],
    [7670000, 1483674720000],
    [7680002, 1483879318000],
    [7690000, 1484064960000],
    [7700000, 1484322340000],
    [7710000, 1484542900000],
    [7720000, 1484812130000],
    [7730001, 1485026320000],
    [7740000, 1485257233000],
    [7750006, 1485510790000],
    [7760000, 1485696491000],
    [7770000, 1485932317000],
    [7780000, 1486164673000],
    [7790001, 1486362226000],
    [7800000, 1486599658000],
    [7810000, 1486822612000],
    [7820001, 1487000522000],
    [7830000, 1487174999000],
    [7840000, 1487419341000],
    [7850000, 1487602867000],
    [7860000, 1487855018000],
    [7870000, 1488086339000],
    [7880000, 1488292733000],
    [7890001, 1488543208000],
    [7900000, 1488728156000],
    [7910001, 1488979685000],
    [7920001, 1489220616000],
    [7930000, 1489412693000],
    [7940003, 1489631325000],
    [7950001, 1489848873000],
    [7960000, 1490044638000],
    [7970002, 1490279983000],
    [7980001, 1490516089000],
    [7990001, 1490719990000],
    [8000000, 1490963479000],
    [8010000, 1491137084000],
    [8020002, 1491375868000],
    [8030000, 1491612912000],
    [8040001, 1491803454000],
    [8050001, 1492077857000],
    [8060002, 1492324560000],
    [8070003, 1492561737000],
    [8080002, 1492838030000],
    [8090002, 1493043924000],
    [8100001, 1493307660000],
    [8110002, 1493543878000],
    [8120000, 1493769888000],
    [8130004, 1493982506000],
    [8140000, 1494142350000],
    [8150001, 1494318814000],
    [8160000, 1494568833000],
    [8170000, 1494762677000],
    [8180002, 1495006227000],
    [8190000, 1495250700000],
    [8200000, 1495443298000],
    [8210002, 1495646825000],
    [8220011, 1495894082000],
    [8230000, 1496082134000],
    [8240000, 1496326810000],
    [8250000, 1496555088000],
    [8260000, 1496756385000],
    [8270000, 1497007056000],
    [8280001, 1497183503000],
    [8290002, 1497397397000],
    [8300003, 1497629360000],
    [8310000, 1497796088000],
    [8320000, 1498023980000],
    [8330001, 1498235560000],
    [8340001, 1498396394000],
    [8350000, 1498613161000],
    [8360001, 1498837786000],
    [8370000, 1499015731000],
    [8380001, 1499265524000],
    [8390002, 1499481547000],
    [8400001, 1499686732000],
    [8410005, 1499932800000],
    [8420001, 1500169609000],
    [8430000, 1500374791000],
    [8440006, 1500617656000],
    [8450000, 1500814972000],
    [8460003, 1501039768000],
    [8470000, 1501277295000],
    [8480001, 1501477676000],
    [8490000, 1501697581000],
    [8500002, 1501932379000],
    [8510000, 1502117905000],
    [8520002, 1502331968000],
    [8530000, 1502540146000],
    [8540000, 1502722627000],
    [8550001, 1502898202000],
    [8560004, 1503134037000],
    [8570003, 1503323268000],
    [8580002, 1503521778000],
    [8590002, 1503744984000],
    [8600000, 1503928192000],
    [8610000, 1504170146000],
    [8620000, 1504371768000],
    [8630002, 1504602602000],
    [8640004, 1504820812000],
    [8650000, 1505027878000],
    [8660001, 1505226480000],
    [8670000, 1505482045000],
    [8680000, 1505658395000],
    [8690000, 1505834586000],
    [8700001, 1506089466000],
    [8710000, 1506264067000],
    [8720000, 1506500571000],
    [8730002, 1506728331000],
    [8740000, 1506914687000],
    [8750000, 1507128072000],
    [8760000, 1507345605000],
    [8770000, 1507518334000],
    [8780000, 1507708428000],
    [8790001, 1507940275000],
    [8800002, 1508138851000],
    [8810000, 1508387594000],
    [8820000, 1508606774000],
    [8830000, 1508827234000],
    [8840001, 1509091766000],
    [8850004, 1509287015000],
    [8860000, 1509490049000],
    [8870000, 1509723653000],
    [8880000, 1509932817000],
    [8890002, 1510207133000],
    [8900004, 1510424299000],
    [8910000, 1510667493000],
    [8920001, 1510940639000],
    [8930000, 1511186220000],
    [8940000, 1511451946000],
    [8950000, 1511704396000],
    [8960000, 1511990454000],
    [8970002, 1512274717000],
    [8980000, 1512537099000],
    [8990000, 1512820550000],
    [9000001, 1513072801000],
    [9010001, 1513357525000],
    [9020000, 1513609923000],
    [9030000, 1513915128000],
    [9040000, 1514118493000],
    [9050002, 1514301843000],
    [9060000, 1514555378000],
    [9070000, 1514732301000],
    [9080001, 1514962790000],
    [9090001, 1515172312000],
    [9100002, 1515397618000],
    [9110000, 1515639717000],
    [9120002, 1515896492000],
    [9130001, 1516144113000],
    [9140000, 1516418097000],
    [9150001, 1516626163000],
    [9160001, 1516883176000],
    [9170001, 1517111440000],
    [9180000, 1517325014000],
    [9190000, 1517578594000],
    [9200001, 1517757784000],
    [9210001, 1518010671000],
    [9220000, 1518256808000],
    [9230005, 1518430798000],
    [9240001, 1518605600000],
    [9250000, 1518794113000],
    [9260001, 1518995823000],
    [9270000, 1519225213000],
    [9280003, 1519458758000],
    [9290001, 1519651282000],
    [9300001, 1519890063000],
    [9310000, 1520090182000],
    [9320000, 1520286849000],
    [9330001, 1520520452000],
    [9340000, 1520744294000],
    [9350002, 1520943868000],
    [9360001, 1521147293000],
    [9370002, 1521362086000],
    [9380000, 1521558925000],
    [9390001, 1521769563000],
    [9400001, 1521973588000],
    [9410000, 1522173272000],
    [9420000, 1522410936000],
    [9430000, 1522582869000],
    [9440000, 1522765509000],
    [9450000, 1523004539000],
    [9460000, 1523190018000],
    [9470000, 1523431728000],
    [9480000, 1523685971000],
    [9490002, 1523878396000],
    [9500002, 1524128292000],
    [9510004, 1524330609000],
    [9520004, 1524533838000],
    [9530000, 1524759003000],
    [9540000, 1524987989000],
    [9550001, 1525164608000],
    [9560002, 1525357365000],
    [9570003, 1525528111000],
    [9580002, 1525674935000],
    [9590000, 1525878167000],
    [9600000, 1526120272000],
    [9610000, 1526290836000],
    [9620000, 1526490579000],
    [9630000, 1526727803000],
    [9640000, 1526907002000],
    [9650002, 1527128556000],
    [9660000, 1527340038000],
    [9670001, 1527517920000],
    [9680002, 1527765780000],
    [9690002, 1527957463000],
    [9700000, 1528151574000],
    [9710000, 1528379137000],
    [9720000, 1528580383000],
    [9730001, 1528772398000],
    [9740001, 1528987967000],
    [9750000, 1529204175000],
    [9760000, 1529406774000],
    [9770001, 1529609683000],
    [9780000, 1529819612000],
    [9790000, 1530014717000],
    [9800000, 1530245547000],
    [9810002, 1530432261000],
    [9820001, 1530629380000],
    [9830000, 1530874223000],
    [9840000, 1531041319000],
    [9850000, 1531235299000],
    [9860001, 1531486297000],
    [9870000, 1531670196000],
    [9880001, 1531889979000],
    [9890001, 1532135177000],
    [9900000, 1532338734000],
    [9910000, 1532578434000],
    [9920001, 1532787506000],
    [9930000, 1532975363000],
    [9940002, 1533210580000],
    [9950002, 1533417619000],
    [9960000, 1533624914000],
    [9970001, 1533822046000],
    [9980000, 1534037713000],
    [9990000, 1534228523000],
    [10000000, 1534413149000],
    [10010001, 1534602898000],
    [10020000, 1534783503000],
    [10030000, 1535021938000],
    [10040001, 1535212003000],
    [10050000, 1535428390000],
    [10060003, 1535641964000],
    [10070001, 1535858219000],
    [10080001, 1536054602000],
    [10090000, 1536289206000],
    [10100000, 1536480473000],
    [10110000, 1536676444000],
    [10120000, 1536926627000],
    [10130002, 1537109324000],
    [10140002, 1537291222000],
    [10150000, 1537542002000],
    [10160001, 1537742745000],
    [10170000, 1537951046000],
    [10180001, 1538191192000],
    [10190000, 1538335437000],
    [10200000, 1538573831000],
    [10210000, 1538809742000],
    [10220001, 1538988459000],
    [10230001, 1539181117000],
    [10240000, 1539420771000],
    [10250000, 1539603667000],
    [10260000, 1539849223000],
    [10270001, 1540053876000],
    [10280002, 1540283911000],
    [10290000, 1540536175000],
    [10300001, 1540729543000],
    [10310000, 1540922544000],
    [10320000, 1541154968000],
    [10330000, 1541337866000],
    [10340001, 1541577224000],
    [10350000, 1541802693000],
    [10360000, 1541963072000],
    [10370000, 1542202693000],
    [10380000, 1542444218000],
    [10390000, 1542627147000],
    [10400000, 1542839314000],
    [10410000, 1543047446000],
    [10420001, 1543235621000],
    [10430000, 1543484894000],
    [10440000, 1543685077000],
    [10450000, 1543913221000],
    [10460000, 1544159482000],
    [10470003, 1544358248000],
    [10480004, 1544606237000],
    [10490002, 1544855404000],
    [10500000, 1545051752000],
    [10510000, 1545300827000],
    [10520005, 1545521938000],
    [10530000, 1545661181000],
    [10540000, 1545834409000],
    [10550000, 1546063345000],
    [10560000, 1546244119000],
    [10570000, 1546421525000],
    [10580000, 1546614293000],
    [10590000, 1546787040000],
    [10600000, 1547033210000],
    [10610000, 1547273443000],
    [10620006, 1547455616000],
    [10630000, 1547657448000],
    [10640000, 1547904630000],
    [10650001, 1548082511000],
    [10660000, 1548328452000],
    [10670000, 1548524147000],
    [10680000, 1548734159000],
    [10690000, 1548948502000],
    [10700000, 1549167894000],
    [10710000, 1549362890000],
    [10720000, 1549582063000],
    [10730001, 1549784961000],
    [10740000, 1549939861000],
    [10750001, 1550125895000],
    [10760001, 1550285616000],
    [10770000, 1550431490000],
    [10780004, 1550660401000],
    [10790000, 1550851727000],
    [10800000, 1551023024000],
    [10810000, 1551259433000],
    [10820002, 1551458705000],
    [10830001, 1551629225000],
    [10840002, 1551870373000],
    [10850001, 1552064671000],
    [10860000, 1552234630000],
    [10870002, 1552471261000],
    [10880000, 1552662209000],
    [10890000, 1552836681000],
    [10900002, 1553069704000],
    [10910000, 1553257072000],
    [10920000, 1553422366000],
    [10930000, 1553606730000],
    [10940001, 1553802181000],
    [10950000, 1553998231000],
    [10960000, 1554134490000],
    [10970002, 1554360751000],
    [10980000, 1554554642000],
    [10990000, 1554735834000],
    [11000000, 1554991576000],
    [11010001, 1555218769000],
    [11020000, 1555420366000],
    [11030000, 1555672640000],
    [11040001, 1555849807000],
    [11050001, 1556084207000],
    [11060000, 1556299720000],
    [11070000, 1556504671000],
    [11080001, 1556632398000],
    [11090000, 1556782396000],
    [11100002, 1556955999000],
    [11110000, 1557106544000],
    [11120000, 1557296907000],
    [11130000, 1557514210000],
    [11140000, 1557684014000],
    [11150001, 1557925678000],
    [11160001, 1558170113000],
    [11170000, 1558354050000],
    [11180001, 1558597755000],
    [11190000, 1558792328000],
    [11200001, 1558975623000],
    [11210000, 1559221084000],
    [11220009, 1559435114000],
    [11230001, 1559636873000],
    [11240000, 1559873390000],
    [11250001, 1560067885000],
    [11260002, 1560253598000],
    [11270000, 1560479047000],
    [11280000, 1560672273000],
    [11290000, 1560863233000],
    [11300003, 1561102493000],
    [11310000, 1561287305000],
    [11320000, 1561489991000],
    [11330000, 1561731006000],
    [11340000, 1561903335000],
    [11350000, 1562136969000],
    [11360001, 1562344828000],
    [11370000, 1562510422000],
    [11380000, 1562736164000],
    [11390000, 1562948007000],
    [11400000, 1563125661000],
    [11410001, 1563328806000],
    [11420000, 1563546056000],
    [11430000, 1563724029000],
    [11440003, 1563963216000],
    [11450001, 1564167040000],
    [11460001, 1564335184000],
    [11470001, 1564568121000],
    [11480000, 1564760362000],
    [11490005, 1564962205000],
    [11500000, 1565179291000],
    [11510000, 1565367541000],
    [11520001, 1565572757000],
    [11530000, 1565747468000],
    [11540000, 1565914332000],
    [11550002, 1566091110000],
    [11560000, 1566252215000],
    [11570001, 1566459388000],
    [11580001, 1566648694000],
    [11590000, 1566824198000],
    [11600001, 1567013191000],
    [11610000, 1567228025000],
    [11620001, 1567393806000],
    [11630001, 1567606734000],
    [11640000, 1567835845000],
    [11650001, 1568012646000],
    [11660000, 1568212823000],
    [11670000, 1568435781000],
    [11680003, 1568605425000],
    [11690002, 1568787569000],
    [11700002, 1568989756000],
    [11710000, 1569163036000],
    [11720003, 1569334820000],
    [11730000, 1569574593000],
    [11740000, 1569752468000],
    [11750000, 1569935750000],
    [11760000, 1570150966000],
    [11770000, 1570332151000],
    [11780000, 1570509507000],
    [11790003, 1570715608000],
    [11800003, 1570888809000],
    [11810000, 1571034246000],
    [11820001, 1571216151000],
    [11830000, 1571419097000],
    [11840000, 1571584069000],
    [11850001, 1571757789000],
    [11860001, 1571993843000],
    [11870001, 1572169847000],
    [11880002, 1572357033000],
    [11890000, 1572533610000],
    [11900000, 1572715665000],
    [11910000, 1572878846000],
    [11920000, 1573112560000],
    [11930000, 1573308611000],
    [11940000, 1573474898000],
    [11950000, 1573661969000],
    [11960000, 1573890077000],
    [11970001, 1574075552000],
    [11980000, 1574297595000],
    [11990004, 1574507058000],
    [12000000, 1574681239000],
    [12010000, 1574895129000],
    [12020000, 1575113068000],
    [12030004, 1575288871000],
    [12040000, 1575515481000],
    [12050000, 1575722999000],
    [12060000, 1575900754000],
    [12070000, 1576138200000],
    [12080001, 1576334837000],
    [12090001, 1576511372000],
    [12100000, 1576752334000],
    [12110000, 1576943163000],
    [12120000, 1577116967000],
    [12130001, 1577281370000],
    [12140000, 1577461695000],
    [12150000, 1577646401000],
    [12160000, 1577796064000],
    [12170001, 1577957614000],
    [12180001, 1578127602000],
    [12190001, 1578279558000],
    [12200000, 1578477602000],
    [12210000, 1578666504000],
    [12220000, 1578839347000],
    [12230000, 1579008316000],
    [12240000, 1579227285000],
    [12250002, 1579415678000],
    [12260003, 1579607375000],
    [12270000, 1579805432000],
    [12280000, 1580008380000],
    [12290002, 1580191030000],
    [12300000, 1580389984000],
    [12310001, 1580566786000],
    [12320003, 1580730297000],
    [12330000, 1580915887000],
    [12340001, 1581129374000],
    [12350001, 1581261750000],
    [12360000, 1581430178000],
    [12370000, 1581606681000],
    [12380000, 1581751510000],
    [12390001, 1581868909000],
    [12400000, 1582069408000],
    [12410001, 1582272103000],
    [12420003, 1582429352000],
    [12430000, 1582554250000],
    [12440000, 1582738718000],
    [12450004, 1582951392000],
    [12460000, 1583070874000],
    [12470002, 1583235146000],
    [12480001, 1583402022000],
    [12490003, 1583559433000],
    [12500000, 1583673315000],
    [12510000, 1583825446000],
    [12520002, 1583980204000],
    [12530001, 1584135315000],
    [12540000, 1584263921000],
    [12550000, 1584416516000],
    [12560001, 1584590400000],
    [12570001, 1584744451000],
    [12580000, 1584885405000],
    [12590000, 1585058908000],
    [12600002, 1585236381000],
    [12610000, 1585399742000],
    [12620000, 1585517492000],
    [12630001, 1585669259000],
    [12640000, 1585828423000],
    [12650000, 1585989472000],
    [12660003, 1586098998000],
    [12670001, 1586269270000],
    [12680000, 1586438019000],
    [12690001, 1586591555000],
    [12700000, 1586695382000],
    [12710003, 1586836653000],
    [12720001, 1586971750000],
    [12730000, 1587131198000],
    [12740003, 1587258788000],
    [12750000, 1587373867000],
    [12760000, 1587488314000],
    [12770000, 1587641662000],
    [12780000, 1587781045000],
    [12790000, 1587893479000],
    [12800000, 1588002355000],
    [12810001, 1588148020000],
    [12820000, 1588261459000],
    [12830000, 1588405175000],
    [12840000, 1588508532000],
    [12850000, 1588603233000],
    [12860002, 1588689904000],
    [12870001, 1588780404000],
    [12880000, 1588926173000],
    [12890001, 1589031515000],
    [12900004, 1589112738000],
    [12910000, 1589198403000],
    [12920000, 1589338200000],
    [12930000, 1589463502000],
    [12940000, 1589580481000],
    [12950000, 1589690404000],
    [12960003, 1589796092000],
    [12970000, 1589904983000],
    [12980000, 1590056583000],
    [12990001, 1590174208000],
    [13000000, 1590293031000],
    [13010000, 1590402693000],
    [13020000, 1590542113000],
    [13030000, 1590673782000],
    [13040000, 1590817020000],
    [13050000, 1590921816000],
    [13060000, 1591050960000],
    [13070003, 1591197055000],
    [13080001, 1591357890000],
    [13090001, 1591464609000],
    [13100000, 1591588831000],
    [13110001, 1591714925000],
    [13120001, 1591874091000],
    [13130000, 1592018956000],
    [13140000, 1592127066000],
    [13150003, 1592239701000],
    [13160001, 1592403491000],
    [13170000, 1592562322000],
    [13180000, 1592673515000],
    [13190002, 1592805607000],
    [13200003, 1592944390000],
    [13210000, 1593102620000],
    [13220000, 1593261761000],
    [13230000, 1593358849000],
    [13240000, 1593523575000],
    [13250001, 1593691848000],
    [13260001, 1593849928000],
    [13270000, 1593953075000],
    [13280002, 1594098141000],
    [13290000, 1594221705000],
    [13300000, 1594391310000],
    [13310001, 1594528320000],
    [13320000, 1594646658000],
    [13330000, 1594810055000],
    [13340000, 1594974088000],
    [13350000, 1595086699000],
    [13360002, 1595219750000],
    [13370000, 1595373630000],
    [13380004, 1595514554000],
    [13390000, 1595646365000],
    [13400000, 1595752459000],
    [13410003, 1595863524000],
    [13420003, 1596029751000],
    [13430000, 1596194279000],
    [13440000, 1596306883000],
    [13450001, 1596441247000],
    [13460002, 1596593681000],
    [13470002, 1596734690000],
    [13480000, 1596886682000],
    [13490001, 1596987319000],
    [13500000, 1597123137000],
    [13510001, 1597244082000],
    [13520000, 1597387488000],
    [13530000, 1597496781000],
    [13540000, 1597589882000],
    [13550001, 1597746996000],
    [13560000, 1597895740000],
    [13570002, 1598026651000],
    [13580000, 1598162365000],
    [13590002, 1598279240000],
    [13600000, 1598443997000],
    [13610000, 1598608599000],
    [13620003, 1598726042000],
    [13630002, 1598859157000],
    [13640001, 1598978674000],
    [13650001, 1599140606000],
    [13660000, 1599296267000],
    [13670000, 1599398967000],
    [13680001, 1599550784000],
    [13690000, 1599701636000],
    [13700000, 1599841328000],
    [13710000, 1599975477000],
    [13720001, 1600089113000],
    [13730000, 1600252766000],
    [13740001, 1600416357000],
    [13750001, 1600532583000],
    [13760002, 1600665853000],
    [13770000, 1600769092000],
    [13780000, 1600891369000],
    [13790004, 1601046381000],
    [13800000, 1601178160000],
    [13810000, 1601294975000],
    [13820002, 1601457229000],
    [13830001, 1601618680000],
    [13840000, 1601742168000],
    [13850001, 1601888032000],
    [13860001, 1602053063000],
    [13870001, 1602229323000],
    [13880001, 1602343999000],
    [13890000, 1602487521000],
    [13900000, 1602647496000],
    [13910000, 1602824568000],
    [13920000, 1602952732000],
    [13930000, 1603099598000],
    [13940001, 1603269016000],
    [13950001, 1603431520000],
    [13960000, 1603553517000],
    [13970000, 1603683799000],
    [13980001, 1603825799000],
    [13990000, 1603990293000],
    [14000000, 1604141441000],
    [14010000, 1604239538000],
    [14020002, 1604392582000],
    [14030001, 1604547728000],
    [14040000, 1604710282000],
    [14050002, 1604831865000],
    [14060001, 1604978794000],
    [14070003, 1605111829000],
    [14080000, 1605281377000],
    [14090000, 1605426595000],
    [14100000, 1605544081000],
    [14110001, 1605710566000],
    [14120000, 1605878130000],
    [14130000, 1606009270000],
    [14140000, 1606110664000],
    [14150000, 1606228059000],
    [14160000, 1606395766000],
    [14170000, 1606550198000],
    [14180000, 1606655608000],
    [14190005, 1606811213000],
    [14200001, 1606977488000],
    [14210000, 1607132673000],
    [14220002, 1607254867000],
    [14230000, 1607408548000],
    [14240000, 1607578193000],
    [14250000, 1607731193000],
    [14260000, 1607855866000],
    [14270001, 1608015495000],
    [14280000, 1608188196000],
    [14290000, 1608344622000],
    [14300001, 1608467026000],
    [14310000, 1608631924000],
    [14320001, 1608787087000],
    [14330001, 1608900065000],
    [14340004, 1609032182000],
    [14350001, 1609157888000],
    [14360002, 1609289135000],
    [14370000, 1609392601000],
    [14380004, 1609477161000],
    [14390000, 1609591619000],
    [14400000, 1609687852000],
    [14410000, 1609831223000],
    [14420001, 1609957660000],
    [14430000, 1610118005000],
    [14440000, 1610261298000],
    [14450000, 1610366620000],
    [14460000, 1610524924000],
    [14470000, 1610684778000],
    [14480001, 1610809189000],
    [14490000, 1610930924000],
    [14500000, 1611076353000],
    [14510000, 1611244136000],
    [14520000, 1611399289000],
    [14530001, 1611494532000],
    [14540000, 1611645230000],
    [14550001, 1611779802000],
    [14560000, 1611929597000],
    [14570000, 1612053001000],
    [14580000, 1612159145000],
    [14590000, 1612284409000],
    [14600002, 1612445935000],
    [14610002, 1612594878000],
    [14620000, 1612696360000],
    [14630000, 1612822111000],
    [14640002, 1612968761000],
    [14650000, 1613108310000],
    [14660000, 1613224941000],
    [14670000, 1613306449000],
    [14680003, 1613406134000],
    [14690002, 1613567319000],
    [14700001, 1613724799000],
    [14710001, 1613833209000],
    [14720002, 1613937109000],
    [14730000, 1614073989000],
    [14740000, 1614203006000],
    [14750000, 1614348178000],
    [14760000, 1614475529000],
    [14770000, 1614583382000],
    [14780001, 1614704431000],
    [14790001, 1614865547000],
    [14800000, 1615013503000],
    [14810000, 1615118546000],
    [14820001, 1615260353000],
    [14830000, 1615392435000],
    [14840001, 1615554134000],
    [14850000, 1615656736000],
    [14860000, 1615777688000],
    [14870000, 1615909277000],
    [14880000, 1616074253000],
    [14890000, 1616225241000],
    [14900000, 1616329884000],
    [14910001, 1616479813000],
    [14920001, 1616639805000],
    [14930000, 1616776543000],
    [14940000, 1616919064000],
    [14950001, 1617033405000],
    [14960000, 1617192850000],
    [14970000, 1617324478000],
    [14980000, 1617457126000],
    [14990001, 1617560471000],
    [15000000, 1617721110000],
    [15010000, 1617890905000],
    [15020001, 1618051846000],
    [15030000, 1618154216000],
    [15040001, 1618325640000],
    [15050000, 1618498930000],
    [15060000, 1618661401000],
    [15070000, 1618759507000],
    [15080000, 1618927949000],
    [15090000, 1619101646000],
    [15100003, 1619267671000],
    [15110001, 1619366377000],
    [15120000, 1619539865000],
    [15130001, 1619698940000],
    [15140000, 1619841193000],
    [15150000, 1619952445000],
    [15160001, 1620053070000],
    [15170000, 1620147984000],
    [15180001, 1620264427000],
    [15190000, 1620408103000],
    [15200001, 1620542441000],
    [15210000, 1620656347000],
    [15220002, 1620827018000],
    [15230002, 1620996871000],
    [15240001, 1621125933000],
    [15250005, 1621247716000],
    [15260000, 1621414260000],
    [15270000, 1621576564000],
    [15280004, 1621693448000],
    [15290001, 1621784663000],
    [15300001, 1621944874000],
    [15310000, 1622111419000],
    [15320000, 1622262907000],
    [15330001, 1622368908000],
    [15340001, 1622490000000],
    [15350000, 1622646046000],
    [15360000, 1622811317000],
    [15370000, 1622934001000],
    [15380000, 1623050332000],
    [15390002, 1623204454000],
    [15400001, 1623345615000],
    [15410000, 1623499760000],
    [15420000, 1623596380000],
    [15430000, 1623754940000],
    [15440000, 1623917063000],
    [15450006, 1624062216000],
    [15460000, 1624173519000],
    [15470000, 1624285944000],
    [15480001, 1624446884000],
    [15490001, 1624611602000],
    [15500001, 1624721981000],
    [15510000, 1624848973000],
    [15520000, 1624987994000],
    [15530000, 1625149123000],
    [15540001, 1625305857000],
    [15550000, 1625407992000],
    [15560001, 1625570458000],
    [15570000, 1625708289000],
    [15580001, 1625846024000],
    [15590000, 1625983462000],
    [15600000, 1626102253000],
    [15610000, 1626268263000],
    [15620001, 1626435982000],
    [15630000, 1626571357000],
    [15640000, 1626697635000],
    [15650000, 1626859260000],
    [15660001, 1626984401000],
    [15670001, 1627121694000],
    [15680000, 1627222279000],
    [15690000, 1627377515000],
    [15700001, 1627534685000],
    [15710002, 1627667668000],
    [15720000, 1627799205000],
    [15730001, 1627916409000],
    [15740002, 1628077710000],
    [15750001, 1628239296000],
    [15760000, 1628352705000],
    [15770000, 1628478255000],
    [15780001, 1628594784000],
    [15790001, 1628735344000],
    [15800001, 1628858188000],
    [15810000, 1628957693000],
    [15820000, 1629072598000],
    [15830000, 1629203998000],
    [15840000, 1629345639000],
    [15850000, 1629469386000],
    [15860002, 1629589331000],
    [15870000, 1629705588000],
    [15880000, 1629823753000],
    [15890001, 1629980009000],
    [15900001, 1630116572000],
    [15910000, 1630221709000],
    [15920000, 1630329553000],
    [15930000, 1630470444000],
    [15940001, 1630596817000],
    [15950000, 1630742907000],
    [15960000, 1630841882000],
    [15970000, 1630949694000],
    [15980000, 1631107604000],
    [15990001, 1631258037000],
    [16000000, 1631368076000],
    [16010001, 1631459740000],
    [16020000, 1631616392000],
    [16030000, 1631759130000],
    [16040000, 1631890364000],
    [16050002, 1632006000000],
    [16060000, 1632113580000],
    [16070000, 1632224017000],
    [16080000, 1632357039000],
    [16090000, 1632479964000],
    [16100002, 1632584915000],
    [16110000, 1632693601000],
    [16120001, 1632837146000],
    [16130001, 1632992269000],
    [16140000, 1633103302000],
    [16150000, 1633236036000],
    [16160000, 1633351627000],
    [16170000, 1633510429000],
    [16180001, 1633670463000],
    [16190000, 1633787504000],
    [16200000, 1633879002000],
    [16210000, 1634036463000],
    [16220000, 1634190311000],
    [16230000, 1634316909000],
    [16240001, 1634449090000],
    [16250001, 1634564297000],
    [16260000, 1634731148000],
    [16270000, 1634894514000],
    [16280001, 1635006911000],
    [16290000, 1635146973000],
    [16300000, 1635296235000],
    [16310001, 1635438801000],
    [16320000, 1635596200000],
    [16330000, 1635683945000],
    [16340001, 1635813965000],
    [16350000, 1635944185000],
    [16360001, 1636104318000],
    [16370000, 1636216125000],
    [16380001, 1636349708000],
    [16390000, 1636485778000],
    [16400000, 1636633126000],
    [16410000, 1636775436000],
    [16420000, 1636887831000],
    [16430000, 1637035157000],
    [16440000, 1637202106000],
    [16450000, 1637344438000],
    [16460000, 1637480688000],
    [16470001, 1637592913000],
    [16480002, 1637739009000],
    [16490000, 1637904942000],
    [16500000, 1638026842000],
    [16510000, 1638162888000],
    [16520000, 1638292148000],
    [16530000, 1638458030000],
    [16540002, 1638615167000],
    [16550000, 1638714393000],
    [16560000, 1638876409000],
    [16570000, 1639040028000],
    [16580000, 1639187217000],
    [16590000, 1639307662000],
    [16600000, 1639454477000],
    [16610000, 1639627185000],
    [16620000, 1639779710000],
    [16630000, 1639907263000],
    [16640003, 1640056572000],
    [16650000, 1640195488000],
    [16660000, 1640347206000],
    [16670002, 1640441258000],
    [16680000, 1640558659000],
    [16690000, 1640704773000],
    [16700000, 1640851001000],
    [16710000, 1640944416000],
    [16720000, 1641034407000],
    [16730001, 1641145162000],
    [16740001, 1641287228000],
    [16750001, 1641432930000],
    [16760000, 1641569488000],
    [16770000, 1641714711000],
    [16780000, 1641819617000],
    [16790000, 1641976092000],
    [16800002, 1642135341000],
    [16810000, 1642258816000],
    [16820000, 1642376854000],
    [16830000, 1642522447000],
    [16840000, 1642692211000],
    [16850001, 1642848145000],
    [16860001, 1642946066000],
    [16870006, 1643104274000],
    [16880000, 1643256755000],
    [16890000, 1643383270000],
    [16900001, 1643513558000],
    [16910000, 1643624822000],
    [16920000, 1643761593000],
    [16930000, 1643897518000],
    [16940001, 1644046689000],
    [16950000, 1644148988000],
    [16960000, 1644292691000],
    [16970000, 1644424029000],
    [16980001, 1644571875000],
    [16990000, 1644672859000],
    [17000000, 1644761982000],
    [17010001, 1644848140000],
    [17020000, 1644994685000],
    [17030000, 1645143682000],
    [17040000, 1645269253000],
    [17050001, 1645357332000],
    [17060001, 1645465591000],
    [17070000, 1645604057000],
    [17080001, 1645725342000],
    [17090000, 1645877056000],
    [17100000, 1645972181000],
    [17110000, 1646124672000],
    [17120000, 1646276532000],
    [17130000, 1646407396000],
    [17140000, 1646535918000],
    [17150000, 1646650916000],
    [17160001, 1646804197000],
    [17170001, 1646946167000],
    [17180000, 1647084906000],
    [17190000, 1647182489000],
    [17200000, 1647314276000],
    [17210000, 1647447683000],
    [17220000, 1647607726000],
    [17230002, 1647721896000],
    [17240000, 1647838182000],
    [17250000, 1647954518000],
    [17260001, 1648112167000],
    [17270000, 1648244365000],
    [17280000, 1648363832000],
    [17290002, 1648479018000],
    [17300000, 1648637593000],
    [17310000, 1648744721000],
    [17320000, 1648882125000],
    [17330001, 1648989613000],
    [17340001, 1649121083000],
    [17350002, 1649258940000],
    [17360000, 1649424449000],
    [17370000, 1649558056000],
    [17380002, 1649679221000],
    [17390000, 1649848546000],
    [17400000, 1650014854000],
    [17410001, 1650127780000],
    [17420000, 1650265614000],
    [17430000, 1650415187000],
    [17440000, 1650565548000],
    [17450000, 1650718660000],
    [17460000, 1650817714000],
    [17470000, 1650980883000],
    [17480001, 1651147321000],
    [17490000, 1651256656000],
    [17500000, 1651380792000],
    [17510000, 1651497322000],
    [17520000, 1651636671000],
    [17530000, 1651742579000],
    [17540001, 1651852590000],
    [17550000, 1651980986000],
    [17560000, 1652097812000],
    [17570000, 1652258339000],
    [17580000, 1652418951000],
    [17590000, 1652537092000],
    [17600000, 1652648521000],
    [17610000, 1652798828000],
    [17620000, 1652966576000],
    [17630000, 1653120396000],
    [17640000, 1653224595000],
    [17650000, 1653366436000],
    [17660000, 1653503365000],
    [17670000, 1653662025000],
    [17680002, 1653797175000],
    [17690001, 1653916799000],
    [17700000, 1654083042000],
    [17710000, 1654250603000],
    [17720001, 1654366796000],
    [17730000, 1654500942000],
    [17740000, 1654659950000],
    [17750000, 1654795307000],
    [17760001, 1654949442000],
    [17770000, 1655049498000],
    [17780000, 1655209018000],
    [17790000, 1655371763000],
    [17800000, 1655523056000],
    [17810000, 1655635213000],
    [17820000, 1655777166000],
    [17830001, 1655925251000],
    [17840000, 1656082802000],
    [17850000, 1656223265000],
    [17860001, 1656347789000],
    [17870000, 1656519562000],
    [17880000, 1656686470000],
    [17890001, 1656823962000],
    [17900000, 1656943371000],
    [17910000, 1657110378000],
    [17920000, 1657261432000],
    [17930000, 1657389177000],
    [17940000, 1657537359000],
    [17950001, 1657703351000],
    [17960001, 1657862126000],
    [17970000, 1657982775000],
    [17980000, 1658107869000],
    [17990000, 1658231351000],
    [18000000, 1658390432000],
    [18010000, 1658529832000],
    [18020000, 1658657110000],
    [18030000, 1658786587000],
    [18040000, 1658931410000],
    [18050000, 1659090621000],
    [18060002, 1659210315000],
    [18070000, 1659342601000],
    [18080000, 1659490247000],
    [18090000, 1659623847000],
    [18100000, 1659771081000],
    [18110000, 1659878153000],
    [18120000, 1660020634000],
    [18130000, 1660135471000],
    [18140000, 1660271287000],
    [18150000, 1660392085000],
    [18160000, 1660493452000],
    [18170001, 1660634551000],
    [18180001, 1660751722000],
    [18190001, 1660907787000],
    [18200000, 1661015264000],
    [18210001, 1661154293000],
    [18220000, 1661296680000],
    [18230000, 1661441105000],
    [18240000, 1661594422000],
    [18250000, 1661697793000],
    [18260000, 1661860267000],
    [18270000, 1662006107000],
    [18280000, 1662153346000],
    [18290000, 1662284110000],
    [18300000, 1662417637000],
    [18310001, 1662564937000],
    [18320000, 1662732210000],
    [18330000, 1662864886000],
    [18340000, 1662989323000],
    [18350000, 1663157286000],
    [18360001, 1663325776000],
    [18370001, 1663457990000],
    [18380001, 1663569154000],
    [18390000, 1663685381000],
    [18400000, 1663854279000],
    [18410000, 1663986345000],
    [18420002, 1664098955000],
    [18430003, 1664233443000],
    [18440000, 1664381791000],
    [18450000, 1664545903000],
    [18460001, 1664672366000],
    [18470000, 1664797315000],
    [18480000, 1664963327000],
    [18490001, 1665129129000],
    [18500000, 1665249079000],
    [18510000, 1665378101000],
    [18520000, 1665498402000],
    [18530000, 1665666274000],
    [18540000, 1665822906000],
    [18550000, 1665928917000],
    [18560000, 1666091693000],
    [18570000, 1666258710000],
    [18580000, 1666413011000],
    [18590000, 1666525787000],
    [18600000, 1666683378000],
    [18610001, 1666856198000],
    [18620000, 1667012932000],
    [18630001, 1667130397000],
    [18640000, 1667227842000],
    [18650000, 1667391437000],
    [18660000, 1667537375000],
    [18670000, 1667659115000],
    [18680000, 1667778232000],
    [18690000, 1667928642000],
    [18700000, 1668092558000],
    [18710000, 1668243412000],
    [18720000, 1668347270000],
    [18730000, 1668511156000],
    [18740000, 1668681913000],
    [18750000, 1668835345000],
    [18760000, 1668948613000],
    [18770000, 1669107023000],
    [18780001, 1669217357000],
    [18790000, 1669385287000],
    [18800000, 1669521446000],
    [18810000, 1669644488000],
    [18820000, 1669806609000],
    [18830000, 1669969123000],
    [18840000, 1670085906000],
    [18850001, 1670230236000],
    [18860000, 1670388111000],
    [18870000, 1670553568000],
    [18880000, 1670680140000],
    [18890001, 1670803201000],
    [18900000, 1670955818000],
    [18910000, 1671119233000],
    [18920000, 1671281912000],
    [18930000, 1671409195000],
    [18940000, 1671567402000],
    [18950000, 1671721208000],
    [18960000, 1671861312000],
    [18970000, 1671954612000],
    [18980000, 1672061020000],
    [18990001, 1672222393000],
    [19000000, 1672353531000],
    [19010000, 1672467100000],
    [19020000, 1672553764000],
    [19030000, 1672671491000],
    [19040000, 1672808932000],
    [19050000, 1672932463000],
    [19060000, 1673087550000],
    [19070000, 1673193657000],
    [19080000, 1673334059000],
    [19090000, 1673495777000],
    [19100000, 1673631095000],
    [19110000, 1673768998000],
    [19120000, 1673885155000],
    [19130000, 1674054006000],
    [19140000, 1674222465000],
    [19150000, 1674353315000],
    [19160000, 1674473730000],
    [19170001, 1674632979000],
    [19180000, 1674785242000],
    [19190000, 1674912941000],
    [19200000, 1675017226000],
    [19210000, 1675171526000],
    [19220000, 1675332469000],
    [19230000, 1675444750000],
    [19240000, 1675578816000],
    [19250000, 1675692430000],
    [19260000, 1675852157000],
    [19270000, 1675999035000],
    [19280000, 1676117630000],
    [19290000, 1676213750000],
    [19300000, 1676339631000],
    [19310001, 1676450914000],
    [19320000, 1676596485000],
    [19330000, 1676726295000],
    [19340000, 1676824763000],
    [19350000, 1676985136000],
    [19360000, 1677125230000],
    [19370001, 1677247234000],
    [19380000, 1677373760000],
    [19390000, 1677495165000],
    [19400000, 1677633838000],
    [19410000, 1677769224000],
    [19420000, 1677919084000],
    [19430000, 1678019683000],
    [19440001, 1678155656000],
    [19450001, 1678288614000],
    [19460000, 1678449353000],
    [19470000, 1678577300000],
    [19480000, 1678704079000],
    [19490001, 1678828261000],
    [19500000, 1678977066000],
    [19510000, 1679123089000],
    [19520000, 1679232561000],
    [19530001, 1679366629000],
    [19540000, 1679492483000],
    [19550000, 1679650444000],
    [19560000, 1679756408000],
    [19570000, 1679865472000],
    [19580000, 1680010450000],
    [19590000, 1680165108000],
    [19600000, 1680274893000],
    [19610000, 1680386401000],
    [19620000, 1680512613000],
    [19630001, 1680642045000],
    [19640001, 1680788039000],
    [19650000, 1680928729000],
    [19660000, 1681038067000],
    [19670000, 1681176951000],
    [19680000, 1681327167000],
    [19690000, 1681483914000],
    [19700000, 1681613428000],
    [19710000, 1681733363000],
    [19720000, 1681901262000],
    [19730000, 1682061980000],
    [19740000, 1682176670000],
    [19750000, 1682309707000],
    [19760001, 1682466416000],
    [19770000, 1682617585000],
    [19780001, 1682768367000],
    [19790000, 1682866680000],
    [19800000, 1683019738000],
    [19810000, 1683136858000],
    [19820000, 1683268994000],
    [19830000, 1683372201000],
    [19840001, 1683462031000],
    [19850001, 1683603948000],
    [19860000, 1683730620000],
    [19870000, 1683898327000],
    [19880001, 1684024153000],
    [19890000, 1684148452000],
    [19900000, 1684313296000],
    [19910000, 1684469994000],
    [19920002, 1684590680000],
    [19930000, 1684699592000],
    [19940001, 1684851189000],
    [19950000, 1685017223000],
    [19960001, 1685169953000],
    [19970000, 1685278382000],
    [19980000, 1685433556000],
    [19990000, 1685564102000],
    [20000000, 1685716288000],
    [20010003, 1685847463000],
    [20020000, 1685964724000],
    [20030000, 1686119859000],
    [20040000, 1686275819000],
    [20050000, 1686401217000],
    [20060001, 1686496334000],
    [20070001, 1686654781000],
    [20080000, 1686814224000],
    [20090000, 1686958417000],
    [20100000, 1687078105000],
    [20110000, 1687190663000],
    [20120001, 1687355764000],
    [20130000, 1687523522000],
    [20140000, 1687645127000],
    [20150000, 1687774959000],
    [20160000, 1687936267000],
    [20170000, 1688078953000],
    [20180002, 1688208028000],
    [20190000, 1688306045000],
    [20200000, 1688453116000],
    [20210000, 1688598008000],
    [20220000, 1688737374000],
    [20230000, 1688867724000],
    [20240000, 1688988884000],
    [20250001, 1689149214000],
    [20260000, 1689311524000],
    [20270000, 1689433202000],
    [20280000, 1689564424000],
    [20290000, 1689687736000],
    [20300000, 1689856471000],
    [20310000, 1690007766000],
    [20320000, 1690119474000],
    [20330000, 1690271740000],
    [20340000, 1690412997000],
    [20350000, 1690556408000],
    [20360000, 1690695983000],
    [20370000, 1690813374000],
    [20380000, 1690966802000],
    [20390001, 1691109438000],
    [20400000, 1691241419000],
    [20410000, 1691360804000],
    [20420000, 1691498119000],
    [20430001, 1691642753000],
    [20440000, 1691761055000],
    [20450001, 1691887415000],
    [20460000, 1692010504000],
    [20470000, 1692112959000],
    [20480000, 1692255348000],
    [20490000, 1692372791000],
    [20500000, 1692508140000],
    [20510000, 1692624146000],
    [20520000, 1692765923000],
    [20530000, 1692888700000],
    [20540000, 1693038437000],
    [20550000, 1693143069000],
    [20560001, 1693284776000],
    [20570000, 1693409238000],
    [20580000, 1693558804000],
    [20590000, 1693667983000],
    [20600001, 1693801909000],
    [20610000, 1693950411000],
    [20620000, 1694100047000],
    [20630001, 1694257179000],
    [20640000, 1694357698000],
    [20650001, 1694519970000],
    [20660000, 1694683520000],
    [20670000, 1694831389000],
    [20680002, 1694954414000],
    [20690000, 1695056275000],
    [20700001, 1695217826000],
    [20710000, 1695374863000],
    [20720000, 1695482728000],
    [20730000, 1695622806000],
    [20740000, 1695771949000],
    [20750000, 1695917849000],
    [20760000, 1696066366000],
    [20770000, 1696167557000],
    [20780002, 1696333609000],
    [20790001, 1696498978000],
    [20800000, 1696647600000],
    [20810000, 1696769069000],
    [20820000, 1696880989000],
    [20830000, 1697036612000],
    [20840000, 1697205385000],
    [20850000, 1697351378000],
    [20860000, 1697475553000],
    [20870001, 1697647826000],
    [20880000, 1697814223000],
    [20890000, 1697964038000],
    [20900000, 1698114540000],
    [20910000, 1698284373000],
    [20920000, 1698442869000],
    [20930000, 1698575354000],
    [20940000, 1698701279000],
    [20950000, 1698838714000],
    [20960000, 1698999234000],
    [20970001, 1699114399000],
    [20980000, 1699258766000],
    [20990000, 1699425433000],
    [21000000, 1699598512000],
    [21010001, 1699714856000],
    [21020000, 1699857336000],
    [21030002, 1700020188000],
    [21040000, 1700195973000],
    [21050000, 1700322845000],
    [21060000, 1700475217000],
    [21070000, 1700647081000],
    [21080000, 1700787860000],
    [21090000, 1700923757000],
    [21100000, 1701068828000],
];


/***/ }),

/***/ "./src/ts/store/workPublishTimeIllusts.ts":
/*!************************************************!*\
  !*** ./src/ts/store/workPublishTimeIllusts.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   illustsData: () => (/* binding */ illustsData)
/* harmony export */ });
const illustsData = [
    [20, 1189343647000],
    [10000, 1190285376000],
    [20006, 1190613767000],
    [30001, 1190901997000],
    [40003, 1191179021000],
    [50004, 1191500729000],
    [60000, 1191759823000],
    [70000, 1192017141000],
    [80000, 1192296354000],
    [90000, 1192548870000],
    [100003, 1192843804000],
    [110000, 1193153408000],
    [120000, 1193485993000],
    [130000, 1193765339000],
    [140003, 1194150173000],
    [150001, 1194467241000],
    [160003, 1194796339000],
    [170000, 1195152402000],
    [180003, 1195485536000],
    [190002, 1195857370000],
    [200000, 1196184465000],
    [210004, 1196522277000],
    [220000, 1196828559000],
    [230000, 1197146134000],
    [240016, 1197455278000],
    [250000, 1197763170000],
    [260000, 1198103280000],
    [270001, 1198395166000],
    [280002, 1198674253000],
    [290000, 1198977560000],
    [300000, 1199330503000],
    [310002, 1199612897000],
    [320004, 1199885562000],
    [330000, 1200159374000],
    [340000, 1200450945000],
    [350005, 1200756520000],
    [360000, 1201025513000],
    [370004, 1201333829000],
    [380000, 1201596101000],
    [390006, 1201873948000],
    [400004, 1202135836000],
    [410003, 1202447870000],
    [420007, 1202718864000],
    [430003, 1202989278000],
    [440000, 1203245126000],
    [450000, 1203510072000],
    [460002, 1203775430000],
    [470001, 1204038291000],
    [480003, 1204299199000],
    [490000, 1204552308000],
    [500002, 1204820237000],
    [510004, 1205065613000],
    [520000, 1205311670000],
    [530009, 1205550410000],
    [540001, 1205766458000],
    [550000, 1205954720000],
    [560001, 1206140135000],
    [570001, 1206322998000],
    [580000, 1206511254000],
    [590005, 1206701143000],
    [600000, 1206881009000],
    [610000, 1207061727000],
    [620000, 1207243644000],
    [630002, 1207434510000],
    [640001, 1207637232000],
    [650001, 1207836760000],
    [660005, 1208020490000],
    [670001, 1208205586000],
    [680002, 1208438014000],
    [690001, 1208621430000],
    [700004, 1208802131000],
    [710007, 1209032196000],
    [720000, 1209218825000],
    [730002, 1209397610000],
    [740003, 1209565351000],
    [750000, 1209741709000],
    [760002, 1209914404000],
    [770000, 1210074595000],
    [780000, 1210252468000],
    [790001, 1210423656000],
    [800005, 1210580882000],
    [810005, 1210756954000],
    [820000, 1210937741000],
    [830000, 1211087983000],
    [840004, 1211236844000],
    [850000, 1211411767000],
    [860003, 1211592338000],
    [870000, 1211724268000],
    [880001, 1211896787000],
    [890000, 1212071744000],
    [900000, 1212240265000],
    [910003, 1212397535000],
    [920001, 1212570167000],
    [930000, 1212743643000],
    [940003, 1212888537000],
    [950001, 1213021904000],
    [960000, 1213191145000],
    [970010, 1213356614000],
    [980000, 1213490592000],
    [990000, 1213625981000],
    [1000000, 1213792360000],
    [1010001, 1213948014000],
    [1020002, 1214066142000],
    [1030000, 1214206483000],
    [1040004, 1214328257000],
    [1050004, 1214488555000],
    [1060000, 1214636988000],
    [1070003, 1214748702000],
    [1080011, 1214904492000],
    [1090000, 1215036636000],
    [1100011, 1215186680000],
    [1110006, 1215322803000],
    [1120006, 1215438170000],
    [1130000, 1215590433000],
    [1140001, 1215708492000],
    [1150000, 1215865334000],
    [1160001, 1215963472000],
    [1170001, 1216117693000],
    [1180001, 1216241875000],
    [1190003, 1216390773000],
    [1200003, 1216522697000],
    [1210010, 1216637029000],
    [1220004, 1216746895000],
    [1230000, 1216894616000],
    [1240001, 1217006818000],
    [1250013, 1217139907000],
    [1260003, 1217251726000],
    [1270003, 1217366045000],
    [1280005, 1217498788000],
    [1290001, 1217605113000],
    [1300002, 1217732126000],
    [1310000, 1217852250000],
    [1320001, 1217956806000],
    [1330000, 1218098038000],
    [1340000, 1218211024000],
    [1350000, 1218344320000],
    [1360003, 1218460214000],
    [1370003, 1218580323000],
    [1380000, 1218713348000],
    [1390001, 1218835149000],
    [1400000, 1218978021000],
    [1410004, 1219086217000],
    [1420005, 1219223973000],
    [1430000, 1219331134000],
    [1440000, 1219461421000],
    [1450000, 1219574536000],
    [1460003, 1219676680000],
    [1470001, 1219797775000],
    [1480000, 1219916216000],
    [1490007, 1220021891000],
    [1500004, 1220118737000],
    [1510007, 1220235399000],
    [1520005, 1220360467000],
    [1530000, 1220465705000],
    [1540005, 1220616937000],
    [1550002, 1220711361000],
    [1560000, 1220798947000],
    [1570000, 1220900621000],
    [1580000, 1221030344000],
    [1590003, 1221138309000],
    [1600001, 1221236989000],
    [1610000, 1221344315000],
    [1620000, 1221452031000],
    [1630009, 1221553447000],
    [1640000, 1221658718000],
    [1650003, 1221757535000],
    [1660006, 1221879285000],
    [1670004, 1221973316000],
    [1680002, 1222068582000],
    [1690002, 1222162793000],
    [1700004, 1222263169000],
    [1710000, 1222365270000],
    [1720000, 1222493935000],
    [1730006, 1222589500000],
    [1740000, 1222690957000],
    [1750006, 1222787660000],
    [1760002, 1222894188000],
    [1770003, 1223028337000],
    [1780000, 1223124745000],
    [1790000, 1223210324000],
    [1800000, 1223305149000],
    [1810000, 1223408306000],
    [1820004, 1223541292000],
    [1830001, 1223645742000],
    [1840003, 1223737145000],
    [1850004, 1223827774000],
    [1860001, 1223913165000],
    [1870001, 1224038216000],
    [1880000, 1224153575000],
    [1890002, 1224253802000],
    [1900005, 1224343693000],
    [1910000, 1224426875000],
    [1920000, 1224524581000],
    [1930000, 1224662433000],
    [1940001, 1224764799000],
    [1950002, 1224865468000],
    [1960008, 1224960411000],
    [1970006, 1225042106000],
    [1980000, 1225168238000],
    [1990000, 1225279658000],
    [2000002, 1225377623000],
    [2010002, 1225468910000],
    [2020002, 1225561524000],
    [2030001, 1225653134000],
    [2040000, 1225734805000],
    [2050002, 1225861022000],
    [2060005, 1225972185000],
    [2070002, 1226068225000],
    [2080002, 1226156629000],
    [2090000, 1226238861000],
    [2100002, 1226332222000],
    [2110001, 1226430234000],
    [2120005, 1226553119000],
    [2130000, 1226661352000],
    [2140000, 1226749829000],
    [2150005, 1226829597000],
    [2160003, 1226921698000],
    [2170001, 1227016838000],
    [2180004, 1227112662000],
    [2190003, 1227258648000],
    [2200001, 1227347281000],
    [2210021, 1227434020000],
    [2220001, 1227514291000],
    [2230004, 1227602927000],
    [2240003, 1227701091000],
    [2250000, 1227797903000],
    [2260001, 1227892579000],
    [2270000, 1227979672000],
    [2280000, 1228058900000],
    [2290000, 1228175237000],
    [2300008, 1228292564000],
    [2310003, 1228393475000],
    [2320002, 1228488496000],
    [2330001, 1228575829000],
    [2340001, 1228656923000],
    [2350005, 1228749831000],
    [2360001, 1228841515000],
    [2370000, 1228945993000],
    [2380009, 1229065515000],
    [2390001, 1229153952000],
    [2400001, 1229227924000],
    [2410003, 1229280540000],
    [2420002, 1229375501000],
    [2430001, 1229477555000],
    [2440005, 1229576902000],
    [2450000, 1229673103000],
    [2460000, 1229761464000],
    [2470002, 1229843101000],
    [2480006, 1229923737000],
    [2490004, 1230006968000],
    [2500004, 1230059526000],
    [2510001, 1230137905000],
    [2520001, 1230219194000],
    [2530001, 1230306089000],
    [2540001, 1230392672000],
    [2550001, 1230480029000],
    [2560002, 1230569639000],
    [2570000, 1230659339000],
    [2580001, 1230737516000],
    [2590003, 1230811927000],
    [2600002, 1230898077000],
    [2610002, 1230980712000],
    [2620001, 1231057356000],
    [2630000, 1231132141000],
    [2640011, 1231205232000],
    [2650004, 1231263657000],
    [2660001, 1231344684000],
    [2670002, 1231429477000],
    [2680003, 1231515785000],
    [2690000, 1231597519000],
    [2700002, 1231675854000],
    [2710000, 1231747570000],
    [2720000, 1231814599000],
    [2730001, 1231910705000],
    [2740000, 1231993040000],
    [2750000, 1232072385000],
    [2760000, 1232152007000],
    [2770000, 1232211499000],
    [2780002, 1232283039000],
    [2790000, 1232364267000],
    [2800003, 1232447068000],
    [2810016, 1232530355000],
    [2820000, 1232613279000],
    [2830003, 1232711358000],
    [2840005, 1232782422000],
    [2850001, 1232848213000],
    [2860000, 1232898493000],
    [2870002, 1232983769000],
    [2880002, 1233069479000],
    [2890000, 1233154231000],
    [2900004, 1233239012000],
    [2910002, 1233322130000],
    [2920002, 1233393929000],
    [2930000, 1233456592000],
    [2940000, 1233504254000],
    [2950000, 1233587016000],
    [2960005, 1233668117000],
    [2970000, 1233748455000],
    [2980004, 1233827735000],
    [2990000, 1233906015000],
    [3000000, 1233973497000],
    [3010000, 1234025920000],
    [3020000, 1234097404000],
    [3030000, 1234174857000],
    [3040000, 1234250595000],
    [3050012, 1234308966000],
    [3060001, 1234362854000],
    [3070000, 1234441241000],
    [3080001, 1234517324000],
    [3090003, 1234567842000],
    [3100005, 1234620065000],
    [3110007, 1234682828000],
    [3120001, 1234723962000],
    [3130004, 1234799784000],
    [3140000, 1234881157000],
    [3150000, 1234963132000],
    [3160000, 1235043967000],
    [3170003, 1235121229000],
    [3180000, 1235186214000],
    [3190000, 1235234695000],
    [3200001, 1235304366000],
    [3210000, 1235374231000],
    [3220002, 1235438963000],
    [3230001, 1235497393000],
    [3240000, 1235576230000],
    [3250001, 1235657222000],
    [3260003, 1235736421000],
    [3270000, 1235805204000],
    [3280000, 1235860055000],
    [3290000, 1235919464000],
    [3300000, 1235999012000],
    [3310000, 1236077174000],
    [3320000, 1236150033000],
    [3330002, 1236213654000],
    [3340001, 1236276161000],
    [3350001, 1236355989000],
    [3360002, 1236430601000],
    [3370003, 1236498230000],
    [3380001, 1236547557000],
    [3390006, 1236613645000],
    [3400001, 1236691737000],
    [3410001, 1236767006000],
    [3420000, 1236836892000],
    [3430001, 1236888047000],
    [3440008, 1236958235000],
    [3450008, 1237029038000],
    [3460001, 1237092055000],
    [3470005, 1237134446000],
    [3480010, 1237212359000],
    [3490002, 1237289939000],
    [3500002, 1237359461000],
    [3510004, 1237428257000],
    [3520000, 1237479684000],
    [3530003, 1237551651000],
    [3540007, 1237616270000],
    [3550002, 1237660604000],
    [3560001, 1237726693000],
    [3570006, 1237795071000],
    [3580000, 1237847938000],
    [3590000, 1237909900000],
    [3600001, 1237985785000],
    [3610007, 1238053132000],
    [3620003, 1238102589000],
    [3630001, 1238168800000],
    [3640002, 1238242579000],
    [3650000, 1238309057000],
    [3660000, 1238349121000],
    [3670004, 1238422770000],
    [3680002, 1238494104000],
    [3690001, 1238548283000],
    [3700001, 1238598887000],
    [3710003, 1238669626000],
    [3720001, 1238731713000],
    [3730001, 1238777676000],
    [3740000, 1238848576000],
    [3750001, 1238911413000],
    [3760002, 1238949271000],
    [3770000, 1239025032000],
    [3780003, 1239097177000],
    [3790010, 1239166551000],
    [3800001, 1239221210000],
    [3810000, 1239297378000],
    [3820002, 1239380016000],
    [3830002, 1239453905000],
    [3840000, 1239515903000],
    [3850002, 1239553333000],
    [3860000, 1239634973000],
    [3870003, 1239717244000],
    [3880001, 1239804843000],
    [3890000, 1239886748000],
    [3900000, 1239968879000],
    [3910000, 1240036921000],
    [3920000, 1240080529000],
    [3930003, 1240144719000],
    [3940003, 1240222544000],
    [3950000, 1240296272000],
    [3960001, 1240357930000],
    [3970000, 1240423383000],
    [3980001, 1240503305000],
    [3990000, 1240585260000],
    [4000000, 1240655799000],
    [4010001, 1240712027000],
    [4020000, 1240756100000],
    [4030002, 1240836439000],
    [4040004, 1240916533000],
    [4050002, 1240978980000],
    [4060012, 1241019328000],
    [4070000, 1241098459000],
    [4080001, 1241172464000],
    [4090010, 1241232896000],
    [4100000, 1241279884000],
    [4110001, 1241352170000],
    [4120001, 1241415995000],
    [4130000, 1241459609000],
    [4140013, 1241524607000],
    [4150003, 1241579188000],
    [4160001, 1241618637000],
    [4170003, 1241693840000],
    [4180000, 1241754350000],
    [4190001, 1241802244000],
    [4200004, 1241869051000],
    [4210001, 1241903352000],
    [4220007, 1241957470000],
    [4230000, 1242020819000],
    [4240001, 1242064990000],
    [4250000, 1242142670000],
    [4260000, 1242222455000],
    [4270003, 1242301994000],
    [4280001, 1242375986000],
    [4290000, 1242429235000],
    [4300000, 1242484029000],
    [4310001, 1242545688000],
    [4320003, 1242577020000],
    [4330000, 1242655220000],
    [4340000, 1242731764000],
    [4350002, 1242798205000],
    [4360000, 1242842152000],
    [4370005, 1242916566000],
    [4380000, 1242990862000],
    [4390003, 1243043132000],
    [4400003, 1243087966000],
    [4410000, 1243142065000],
    [4420004, 1243173603000],
    [4430003, 1243245111000],
    [4440000, 1243295775000],
    [4450001, 1243352896000],
    [4460000, 1243428716000],
    [4470000, 1243499239000],
    [4480000, 1243546633000],
    [4490008, 1243611829000],
    [4500000, 1243680471000],
    [4510007, 1243726638000],
    [4520000, 1243773896000],
    [4530000, 1243840248000],
    [4540001, 1243880872000],
    [4550001, 1243955623000],
    [4560003, 1244033430000],
    [4570000, 1244106466000],
    [4580000, 1244161088000],
    [4590000, 1244217575000],
    [4600012, 1244283613000],
    [4610000, 1244317138000],
    [4620001, 1244374605000],
    [4630000, 1244426298000],
    [4640003, 1244475731000],
    [4650014, 1244552008000],
    [4660002, 1244619636000],
    [4670003, 1244661268000],
    [4680000, 1244733110000],
    [4690002, 1244810784000],
    [4700003, 1244869895000],
    [4710005, 1244908052000],
    [4720001, 1244966932000],
    [4730000, 1244996596000],
    [4740000, 1245073495000],
    [4750005, 1245148333000],
    [4760003, 1245211674000],
    [4770002, 1245257650000],
    [4780002, 1245335654000],
    [4790001, 1245411977000],
    [4800003, 1245470179000],
    [4810001, 1245512460000],
    [4820004, 1245575170000],
    [4830001, 1245608036000],
    [4840000, 1245682116000],
    [4850001, 1245758893000],
    [4860000, 1245830517000],
    [4870001, 1245878690000],
    [4880000, 1245947115000],
    [4890001, 1246026562000],
    [4900002, 1246093663000],
    [4910002, 1246141590000],
    [4920000, 1246194756000],
    [4930006, 1246269185000],
    [4940000, 1246330995000],
    [4950003, 1246380594000],
    [4960004, 1246456810000],
    [4970001, 1246532999000],
    [4980000, 1246600659000],
    [4990001, 1246646313000],
    [5000000, 1246712162000],
    [5010000, 1246764226000],
    [5020002, 1246802551000],
    [5030000, 1246874584000],
    [5040002, 1246921322000],
    [5050000, 1246978642000],
    [5060001, 1247053952000],
    [5070003, 1247122990000],
    [5080000, 1247163071000],
    [5090012, 1247237814000],
    [5100001, 1247303799000],
    [5110000, 1247340850000],
    [5120006, 1247400292000],
    [5130000, 1247464875000],
    [5140001, 1247505727000],
    [5150000, 1247581477000],
    [5160000, 1247657605000],
    [5170000, 1247734497000],
    [5180003, 1247779280000],
    [5190011, 1247844270000],
    [5200001, 1247912117000],
    [5210000, 1247959876000],
    [5220003, 1248010424000],
    [5230001, 1248067662000],
    [5240002, 1248102071000],
    [5250009, 1248171495000],
    [5260001, 1248215258000],
    [5270005, 1248272067000],
    [5280003, 1248338387000],
    [5290015, 1248372446000],
    [5300001, 1248441311000],
    [5310000, 1248499026000],
    [5320000, 1248537062000],
    [5330001, 1248597583000],
    [5340000, 1248630173000],
    [5350000, 1248699529000],
    [5360001, 1248758099000],
    [5370001, 1248796495000],
    [5380001, 1248865288000],
    [5390008, 1248910616000],
    [5400006, 1248962620000],
    [5410005, 1249022010000],
    [5420000, 1249055536000],
    [5430008, 1249119772000],
    [5440000, 1249159590000],
    [5450008, 1249215395000],
    [5460000, 1249267500000],
    [5470001, 1249314199000],
    [5480002, 1249381222000],
    [5490009, 1249425570000],
    [5500000, 1249481249000],
    [5510000, 1249544055000],
    [5520001, 1249580138000],
    [5530002, 1249648156000],
    [5540006, 1249702201000],
    [5550000, 1249745436000],
    [5560006, 1249808811000],
    [5570006, 1249843110000],
    [5580011, 1249908731000],
    [5590001, 1249962535000],
    [5600000, 1250004013000],
    [5610000, 1250067638000],
    [5620000, 1250105377000],
    [5630010, 1250170869000],
    [5640001, 1250234408000],
    [5650000, 1250283303000],
    [5660000, 1250348991000],
    [5670002, 1250417131000],
    [5680002, 1250470126000],
    [5690003, 1250518788000],
    [5700000, 1250583103000],
    [5710000, 1250615540000],
    [5720000, 1250683273000],
    [5730001, 1250737779000],
    [5740001, 1250779234000],
    [5750000, 1250841660000],
    [5760004, 1250876242000],
    [5770002, 1250942945000],
    [5780000, 1250994139000],
    [5790000, 1251035549000],
    [5800012, 1251097978000],
    [5810000, 1251131781000],
    [5820001, 1251200871000],
    [5830000, 1251252353000],
    [5840002, 1251298144000],
    [5850004, 1251362895000],
    [5860002, 1251393581000],
    [5870000, 1251458864000],
    [5880004, 1251501747000],
    [5890000, 1251552074000],
    [5900000, 1251603656000],
    [5910003, 1251640979000],
    [5920001, 1251701011000],
    [5930001, 1251731077000],
    [5940004, 1251799106000],
    [5950002, 1251833410000],
    [5960000, 1251900933000],
    [5970002, 1251969785000],
    [5980001, 1252003787000],
    [5990001, 1252073509000],
    [6000009, 1252134109000],
    [6010001, 1252167556000],
    [6020008, 1252226737000],
    [6030006, 1252253877000],
    [6040005, 1252324357000],
    [6050004, 1252377437000],
    [6060001, 1252422863000],
    [6070000, 1252491703000],
    [6080000, 1252518066000],
    [6090000, 1252590024000],
    [6100000, 1252657336000],
    [6110000, 1252689755000],
    [6120000, 1252752938000],
    [6130001, 1252787625000],
    [6140000, 1252842284000],
    [6150001, 1252887499000],
    [6160004, 1252940963000],
    [6170000, 1253014662000],
    [6180000, 1253072324000],
    [6190003, 1253117275000],
    [6200000, 1253192710000],
    [6210007, 1253259594000],
    [6220004, 1253293456000],
    [6230002, 1253359261000],
    [6240001, 1253408051000],
    [6250000, 1253454244000],
    [6260000, 1253510197000],
    [6270005, 1253545430000],
    [6280003, 1253603112000],
    [6290000, 1253633675000],
    [6300000, 1253690076000],
    [6310004, 1253718754000],
    [6320000, 1253787672000],
    [6330009, 1253827693000],
    [6340003, 1253890698000],
    [6350000, 1253949685000],
    [6360003, 1253981010000],
    [6370003, 1254038599000],
    [6380000, 1254065917000],
    [6390007, 1254138466000],
    [6400002, 1254161381000],
    [6410002, 1254232715000],
    [6420005, 1254297948000],
    [6430001, 1254328569000],
    [6440003, 1254402664000],
    [6450000, 1254470858000],
    [6460000, 1254505532000],
    [6470005, 1254569748000],
    [6480004, 1254607798000],
    [6490004, 1254657512000],
    [6500001, 1254711690000],
    [6510000, 1254757243000],
    [6520003, 1254832152000],
    [6530002, 1254899396000],
    [6540001, 1254936211000],
    [6550000, 1255002705000],
    [6560005, 1255054676000],
    [6570001, 1255103185000],
    [6580004, 1255187603000],
    [6590004, 1255241732000],
    [6600006, 1255275381000],
    [6610006, 1255335923000],
    [6620004, 1255365040000],
    [6630000, 1255437371000],
    [6640004, 1255503196000],
    [6650000, 1255542374000],
    [6660003, 1255616844000],
    [6670006, 1255692389000],
    [6680002, 1255747536000],
    [6690000, 1255789048000],
    [6700007, 1255845043000],
    [6710001, 1255876623000],
    [6720006, 1255947630000],
    [6730001, 1255999240000],
    [6740001, 1256052059000],
    [6750001, 1256127357000],
    [6760000, 1256192620000],
    [6770001, 1256230979000],
    [6780004, 1256305273000],
    [6790000, 1256366761000],
    [6800002, 1256399088000],
    [6810001, 1256457917000],
    [6820014, 1256481202000],
    [6830000, 1256530568000],
    [6840003, 1256569253000],
    [6850000, 1256640930000],
    [6860001, 1256686001000],
    [6870001, 1256740999000],
    [6880001, 1256810974000],
    [6890001, 1256846841000],
    [6900006, 1256909975000],
    [6910008, 1256956991000],
    [6920001, 1256993093000],
    [6930002, 1257021797000],
    [6940007, 1257074334000],
    [6950006, 1257112755000],
    [6960010, 1257171176000],
    [6970029, 1257229635000],
    [6980001, 1257260628000],
    [6990009, 1257333480000],
    [7000004, 1257389136000],
    [7010001, 1257435146000],
    [7020003, 1257509404000],
    [7030000, 1257562969000],
    [7040011, 1257602634000],
    [7050001, 1257654208000],
    [7060007, 1257686818000],
    [7070003, 1257747290000],
    [7080000, 1257781719000],
    [7090002, 1257854092000],
    [7100000, 1257906922000],
    [7110000, 1257949717000],
    [7120010, 1258017940000],
    [7130000, 1258052980000],
    [7140003, 1258121655000],
    [7150004, 1258179932000],
    [7160005, 1258212028000],
    [7170018, 1258268171000],
    [7180007, 1258297347000],
    [7190003, 1258369922000],
    [7200000, 1258423598000],
    [7210001, 1258471779000],
    [7220000, 1258547279000],
    [7230006, 1258613975000],
    [7240009, 1258652603000],
    [7250000, 1258726763000],
    [7260001, 1258788982000],
    [7270001, 1258820885000],
    [7280001, 1258881797000],
    [7290021, 1258901428000],
    [7300003, 1258941371000],
    [7310003, 1258978847000],
    [7320000, 1258997635000],
    [7330003, 1259068970000],
    [7340002, 1259137031000],
    [7350004, 1259172407000],
    [7360000, 1259245313000],
    [7370001, 1259318445000],
    [7380001, 1259366162000],
    [7390001, 1259413895000],
    [7400001, 1259463754000],
    [7410003, 1259500536000],
    [7420006, 1259566058000],
    [7430004, 1259600101000],
    [7440000, 1259674892000],
    [7450006, 1259746273000],
    [7460007, 1259794215000],
    [7470003, 1259853521000],
    [7480001, 1259928386000],
    [7490002, 1259983081000],
    [7500006, 1260022442000],
    [7510000, 1260071171000],
    [7520000, 1260106690000],
    [7530000, 1260172447000],
    [7540000, 1260208351000],
    [7550009, 1260281963000],
    [7560001, 1260355097000],
    [7570000, 1260411385000],
    [7580002, 1260459860000],
    [7590001, 1260534424000],
    [7600000, 1260587470000],
    [7610007, 1260627276000],
    [7620004, 1260684093000],
    [7630004, 1260715167000],
    [7640004, 1260784681000],
    [7650004, 1260830330000],
    [7660002, 1260889337000],
    [7670001, 1260964691000],
    [7680001, 1261029793000],
    [7690002, 1261071760000],
    [7700003, 1261146596000],
    [7710009, 1261212831000],
    [7720001, 1261245754000],
    [7730011, 1261307347000],
    [7740001, 1261355745000],
    [7750000, 1261408395000],
    [7760010, 1261482358000],
    [7770010, 1261537178000],
    [7780002, 1261576719000],
    [7790002, 1261610556000],
    [7800002, 1261659130000],
    [7810001, 1261699801000],
    [7820004, 1261746610000],
    [7830000, 1261793852000],
    [7840001, 1261837044000],
    [7850001, 1261893306000],
    [7860000, 1261927165000],
    [7870000, 1261988405000],
    [7880006, 1262022689000],
    [7890001, 1262089362000],
    [7900000, 1262147767000],
    [7910004, 1262187188000],
    [7920000, 1262246668000],
    [7930000, 1262272485000],
    [7940000, 1262310429000],
    [7950000, 1262345982000],
    [7960000, 1262374368000],
    [7970001, 1262435699000],
    [7980006, 1262482125000],
    [7990006, 1262523783000],
    [8000000, 1262571603000],
    [8010002, 1262611322000],
    [8020002, 1262662269000],
    [8030000, 1262699160000],
    [8040000, 1262751940000],
    [8050001, 1262787880000],
    [8060003, 1262847522000],
    [8070004, 1262879016000],
    [8080003, 1262948453000],
    [8090000, 1263005525000],
    [8100001, 1263043440000],
    [8110004, 1263090083000],
    [8120001, 1263129819000],
    [8130001, 1263178927000],
    [8140000, 1263213746000],
    [8150000, 1263264406000],
    [8160001, 1263308752000],
    [8170001, 1263381109000],
    [8180000, 1263433234000],
    [8190004, 1263482236000],
    [8200001, 1263552778000],
    [8210000, 1263582347000],
    [8220000, 1263641955000],
    [8230000, 1263668432000],
    [8240000, 1263721345000],
    [8250002, 1263743461000],
    [8260003, 1263812466000],
    [8270001, 1263840381000],
    [8280000, 1263909812000],
    [8290002, 1263975879000],
    [8300002, 1264004629000],
    [8310008, 1264073609000],
    [8320001, 1264112814000],
    [8330000, 1264170205000],
    [8340003, 1264225027000],
    [8350000, 1264256849000],
    [8360004, 1264306527000],
    [8370004, 1264337256000],
    [8380000, 1264381896000],
    [8390002, 1264429503000],
    [8400000, 1264495753000],
    [8410000, 1264525760000],
    [8420000, 1264595385000],
    [8430002, 1264646313000],
    [8440000, 1264688257000],
    [8450004, 1264751892000],
    [8460000, 1264781077000],
    [8470002, 1264839031000],
    [8480003, 1264864949000],
    [8490001, 1264912159000],
    [8500001, 1264940198000],
    [8510002, 1264963055000],
    [8520005, 1265026244000],
    [8530003, 1265055913000],
    [8540004, 1265115694000],
    [8550006, 1265158385000],
    [8560001, 1265203504000],
    [8570001, 1265251639000],
    [8580003, 1265292623000],
    [8590005, 1265351842000],
    [8600000, 1265384653000],
    [8610001, 1265441412000],
    [8620000, 1265468298000],
    [8630000, 1265516784000],
    [8640001, 1265545204000],
    [8650001, 1265574034000],
    [8660001, 1265633483000],
    [8670000, 1265675793000],
    [8680002, 1265721819000],
    [8690004, 1265769855000],
    [8700000, 1265809934000],
    [8710003, 1265851086000],
    [8720010, 1265885485000],
    [8730000, 1265904395000],
    [8740000, 1265966419000],
    [8750000, 1265991793000],
    [8760005, 1266046654000],
    [8770002, 1266072240000],
    [8780000, 1266109367000],
    [8790000, 1266142840000],
    [8800000, 1266162438000],
    [8810001, 1266223290000],
    [8820004, 1266248215000],
    [8830000, 1266311115000],
    [8840000, 1266336776000],
    [8850002, 1266404590000],
    [8860003, 1266431080000],
    [8870011, 1266495239000],
    [8880000, 1266532193000],
    [8890004, 1266586323000],
    [8900006, 1266634787000],
    [8910000, 1266671132000],
    [8920003, 1266704552000],
    [8930000, 1266748887000],
    [8940001, 1266771304000],
    [8950007, 1266834419000],
    [8960000, 1266853031000],
    [8970000, 1266914766000],
    [8980000, 1266941561000],
    [8990002, 1267006421000],
    [9000002, 1267033072000],
    [9010000, 1267099263000],
    [9020000, 1267136814000],
    [9030001, 1267189340000],
    [9040001, 1267227765000],
    [9050002, 1267271033000],
    [9060001, 1267295095000],
    [9070001, 1267349581000],
    [9080000, 1267372556000],
    [9090001, 1267437953000],
    [9100001, 1267464018000],
    [9110000, 1267529940000],
    [9120002, 1267557994000],
    [9130000, 1267619150000],
    [9140001, 1267649393000],
    [9150000, 1267707477000],
    [9160001, 1267756145000],
    [9170002, 1267797588000],
    [9180003, 1267845140000],
    [9190000, 1267880161000],
    [9200000, 1267911888000],
    [9210000, 1267956602000],
    [9220000, 1267977687000],
    [9230001, 1268041678000],
    [9240001, 1268067085000],
    [9250003, 1268128511000],
    [9260000, 1268150764000],
    [9270004, 1268213052000],
    [9280000, 1268238021000],
    [9290010, 1268301488000],
    [9300001, 1268326858000],
    [9310000, 1268390142000],
    [9320000, 1268414486000],
    [9330001, 1268469126000],
    [9340000, 1268493356000],
    [9350001, 1268544135000],
    [9360001, 1268572846000],
    [9370002, 1268619924000],
    [9380007, 1268658826000],
    [9390000, 1268701926000],
    [9400000, 1268743976000],
    [9410007, 1268782970000],
    [9420003, 1268828664000],
    [9430003, 1268860371000],
    [9440002, 1268914210000],
    [9450002, 1268947479000],
    [9460005, 1269001003000],
    [9470002, 1269031061000],
    [9480000, 1269082722000],
    [9490002, 1269106998000],
    [9500001, 1269162960000],
    [9510000, 1269187333000],
    [9520000, 1269242743000],
    [9530002, 1269268090000],
    [9540002, 1269321764000],
    [9550000, 1269352784000],
    [9560000, 1269403387000],
    [9570002, 1269436975000],
    [9580005, 1269480995000],
    [9590004, 1269519219000],
    [9600000, 1269544500000],
    [9610002, 1269600989000],
    [9620002, 1269624459000],
    [9630002, 1269680285000],
    [9640006, 1269704898000],
    [9650002, 1269757853000],
    [9660003, 1269785364000],
    [9670007, 1269835201000],
    [9680001, 1269868455000],
    [9690001, 1269910981000],
    [9700002, 1269950640000],
    [9710004, 1269974845000],
    [9720000, 1270030676000],
    [9730002, 1270050327000],
    [9740002, 1270097462000],
    [9750008, 1270124323000],
    [9760010, 1270142744000],
    [9770005, 1270196324000],
    [9780007, 1270220578000],
    [9790001, 1270268030000],
    [9800000, 1270299979000],
    [9810000, 1270327188000],
    [9820004, 1270374286000],
    [9830001, 1270395746000],
    [9840007, 1270450706000],
    [9850002, 1270477065000],
    [9860000, 1270522915000],
    [9870005, 1270558305000],
    [9880002, 1270587924000],
    [9890003, 1270641211000],
    [9900009, 1270667244000],
    [9910002, 1270729503000],
    [9920000, 1270772739000],
    [9930002, 1270820871000],
    [9940000, 1270867332000],
    [9950000, 1270902222000],
    [9960000, 1270923509000],
    [9970002, 1270973617000],
    [9980005, 1270995483000],
    [9990001, 1271052458000],
    [10000000, 1271084467000],
    [10010000, 1271152586000],
    [10020001, 1271178520000],
    [10030002, 1271249106000],
    [10040004, 1271300905000],
    [10050001, 1271342704000],
    [10060001, 1271411103000],
    [10070000, 1271438670000],
    [10080000, 1271497617000],
    [10090000, 1271520730000],
    [10100006, 1271572090000],
    [10110001, 1271596744000],
    [10120000, 1271640791000],
    [10130001, 1271686480000],
    [10140000, 1271748797000],
    [10150000, 1271776989000],
    [10160002, 1271844770000],
    [10170001, 1271870174000],
    [10180000, 1271939973000],
    [10190000, 1271982449000],
    [10200002, 1272031889000],
    [10210002, 1272079184000],
    [10220000, 1272113379000],
    [10230001, 1272138602000],
    [10240005, 1272186107000],
    [10250008, 1272207045000],
    [10260000, 1272269454000],
    [10270000, 1272295567000],
    [10280004, 1272362958000],
    [10290002, 1272386402000],
    [10300003, 1272455984000],
    [10310001, 1272480451000],
    [10320003, 1272533421000],
    [10330002, 1272555900000],
    [10340001, 1272619333000],
    [10350001, 1272643694000],
    [10360000, 1272699431000],
    [10370000, 1272726810000],
    [10380000, 1272778479000],
    [10390002, 1272809410000],
    [10400003, 1272858602000],
    [10410000, 1272892018000],
    [10420001, 1272930483000],
    [10430000, 1272971101000],
    [10440000, 1272990692000],
    [10450004, 1273039646000],
    [10460004, 1273063064000],
    [10470003, 1273082413000],
    [10480012, 1273148702000],
    [10490001, 1273178432000],
    [10500000, 1273237856000],
    [10510003, 1273272118000],
    [10520000, 1273314937000],
    [10530003, 1273335371000],
    [10540003, 1273383275000],
    [10550000, 1273407737000],
    [10560003, 1273427935000],
    [10570001, 1273493380000],
    [10580000, 1273518887000],
    [10590001, 1273583380000],
    [10600000, 1273629537000],
    [10610003, 1273673975000],
    [10620002, 1273736864000],
    [10630008, 1273765536000],
    [10640001, 1273832963000],
    [10650001, 1273856472000],
    [10660000, 1273913403000],
    [10670003, 1273938190000],
    [10680003, 1273987415000],
    [10690002, 1274013883000],
    [10700001, 1274041442000],
    [10710001, 1274101755000],
    [10720001, 1274148034000],
    [10730000, 1274191649000],
    [10740001, 1274251918000],
    [10750000, 1274281421000],
    [10760001, 1274344952000],
    [10770001, 1274370852000],
    [10780000, 1274437849000],
    [10790001, 1274461341000],
    [10800003, 1274516434000],
    [10810000, 1274540863000],
    [10820000, 1274585874000],
    [10830001, 1274613400000],
    [10840000, 1274630996000],
    [10850002, 1274696077000],
    [10860002, 1274718536000],
    [10870000, 1274786118000],
    [10880005, 1274810518000],
    [10890000, 1274877703000],
    [10900005, 1274924038000],
    [10910000, 1274969786000],
    [10920003, 1275033999000],
    [10930001, 1275062164000],
    [10940002, 1275117814000],
    [10950000, 1275143781000],
    [10960001, 1275188364000],
    [10970000, 1275218494000],
    [10980000, 1275236993000],
    [10990001, 1275304384000],
    [11000001, 1275326405000],
    [11010008, 1275394258000],
    [11020002, 1275426312000],
    [11030000, 1275484933000],
    [11040001, 1275533742000],
    [11050001, 1275575272000],
    [11060005, 1275635913000],
    [11070002, 1275664460000],
    [11080000, 1275714355000],
    [11090004, 1275744055000],
    [11100000, 1275769810000],
    [11110000, 1275814398000],
    [11120011, 1275834332000],
    [11130002, 1275887874000],
    [11140002, 1275920205000],
    [11150000, 1275975608000],
    [11160000, 1276007756000],
    [11170001, 1276065720000],
    [11180000, 1276094358000],
    [11190000, 1276153093000],
    [11200003, 1276182302000],
    [11210003, 1276247378000],
    [11220003, 1276272594000],
    [11230001, 1276326893000],
    [11240005, 1276352916000],
    [11250002, 1276396637000],
    [11260002, 1276427108000],
    [11270002, 1276443222000],
    [11280002, 1276506906000],
    [11290001, 1276530201000],
    [11300004, 1276597379000],
    [11310000, 1276620884000],
    [11320004, 1276688782000],
    [11330000, 1276714922000],
    [11340000, 1276780694000],
    [11350003, 1276831842000],
    [11360001, 1276871333000],
    [11370004, 1276918283000],
    [11380006, 1276951279000],
    [11390000, 1276973002000],
    [11400005, 1277023272000],
    [11410005, 1277044636000],
    [11420007, 1277100970000],
    [11430001, 1277132150000],
    [11440000, 1277193416000],
    [11450005, 1277220644000],
    [11460000, 1277286195000],
    [11470000, 1277310437000],
    [11480007, 1277378868000],
    [11490001, 1277405797000],
    [11500000, 1277471329000],
    [11510005, 1277510838000],
    [11520014, 1277552590000],
    [11530000, 1277574577000],
    [11540002, 1277626363000],
    [11550000, 1277650324000],
    [11560003, 1277711981000],
    [11570002, 1277739469000],
    [11580003, 1277805672000],
    [11590002, 1277833631000],
    [11600007, 1277901407000],
    [11610000, 1277943642000],
    [11620004, 1277992741000],
    [11630006, 1278052857000],
    [11640004, 1278085215000],
    [11650000, 1278141043000],
    [11660000, 1278168247000],
    [11670001, 1278211901000],
    [11680005, 1278242882000],
    [11690000, 1278263525000],
    [11700000, 1278331018000],
    [11710001, 1278363283000],
    [11720001, 1278422573000],
    [11730001, 1278467910000],
    [11740002, 1278508847000],
    [11750001, 1278546502000],
    [11760002, 1278596862000],
    [11770055, 1278654118000],
    [11780003, 1278686331000],
    [11790002, 1278735703000],
    [11800000, 1278768480000],
    [11810000, 1278807401000],
    [11820006, 1278844282000],
    [11830002, 1278863674000],
    [11840002, 1278927490000],
    [11850000, 1278949803000],
    [11860004, 1279015329000],
    [11870004, 1279037966000],
    [11880006, 1279102801000],
    [11890001, 1279125228000],
    [11900000, 1279191755000],
    [11910003, 1279216455000],
    [11920000, 1279282608000],
    [11930000, 1279320656000],
    [11940000, 1279367086000],
    [11950005, 1279391585000],
    [11960001, 1279444783000],
    [11970000, 1279467624000],
    [11980002, 1279516298000],
    [11990000, 1279544139000],
    [12000006, 1279573067000],
    [12010002, 1279632740000],
    [12020000, 1279674263000],
    [12030001, 1279714985000],
    [12040004, 1279739983000],
    [12050001, 1279797478000],
    [12060002, 1279819893000],
    [12070000, 1279880710000],
    [12080001, 1279903895000],
    [12090000, 1279959100000],
    [12100000, 1279984330000],
    [12110008, 1280031762000],
    [12120000, 1280061767000],
    [12130001, 1280097529000],
    [12140002, 1280146450000],
    [12150004, 1280175516000],
    [12160000, 1280231047000],
    [12170003, 1280253885000],
    [12180004, 1280312366000],
    [12190010, 1280334261000],
    [12200002, 1280392960000],
    [12210000, 1280416789000],
    [12220001, 1280470620000],
    [12230002, 1280500335000],
    [12240000, 1280546599000],
    [12250001, 1280580530000],
    [12260000, 1280605572000],
    [12270000, 1280653774000],
    [12280002, 1280675302000],
    [12290000, 1280725084000],
    [12300000, 1280756249000],
    [12310000, 1280793725000],
    [12320002, 1280837490000],
    [12330002, 1280862689000],
    [12340001, 1280920068000],
    [12350001, 1280942280000],
    [12360002, 1280999471000],
    [12370001, 1281023696000],
    [12380000, 1281079371000],
    [12390006, 1281107837000],
    [12400001, 1281157949000],
    [12410008, 1281189784000],
    [12420004, 1281231583000],
    [12430001, 1281267381000],
    [12440012, 1281287383000],
    [12450005, 1281343947000],
    [12460000, 1281367264000],
    [12470000, 1281417336000],
    [12480000, 1281447426000],
    [12490000, 1281485017000],
    [12500002, 1281525838000],
    [12510000, 1281547664000],
    [12520000, 1281601916000],
    [12530003, 1281626599000],
    [12540000, 1281681792000],
    [12550000, 1281712170000],
    [12560000, 1281768918000],
    [12570000, 1281799924000],
    [12580005, 1281857189000],
    [12590008, 1281885144000],
    [12600000, 1281939245000],
    [12610001, 1281967862000],
    [12620000, 1282015601000],
    [12630000, 1282050647000],
    [12640000, 1282086682000],
    [12650007, 1282132325000],
    [12660001, 1282154655000],
    [12670000, 1282213268000],
    [12680003, 1282235398000],
    [12690005, 1282292642000],
    [12700001, 1282318186000],
    [12710000, 1282369235000],
    [12720002, 1282399052000],
    [12730001, 1282440554000],
    [12740000, 1282477003000],
    [12750000, 1282498068000],
    [12760003, 1282556916000],
    [12770001, 1282579108000],
    [12780007, 1282634809000],
    [12790002, 1282662019000],
    [12800001, 1282713987000],
    [12810000, 1282745037000],
    [12820000, 1282790864000],
    [12830002, 1282828562000],
    [12840002, 1282866685000],
    [12850005, 1282911120000],
    [12860002, 1282935089000],
    [12870000, 1282988256000],
    [12880000, 1283011329000],
    [12890004, 1283061649000],
    [12900010, 1283088208000],
    [12910002, 1283123926000],
    [12920000, 1283169509000],
    [12930000, 1283188995000],
    [12940001, 1283245495000],
    [12950000, 1283266662000],
    [12960001, 1283321664000],
    [12970000, 1283352413000],
    [12980001, 1283411926000],
    [12990003, 1283440104000],
    [13000000, 1283500853000],
    [13010001, 1283527719000],
    [13020004, 1283579691000],
    [13030004, 1283608092000],
    [13040004, 1283643707000],
    [13050000, 1283681660000],
    [13060000, 1283701069000],
    [13070004, 1283763448000],
    [13080010, 1283787320000],
    [13090001, 1283851111000],
    [13100002, 1283874086000],
    [13110003, 1283937995000],
    [13120002, 1283960387000],
    [13130000, 1284021968000],
    [13140001, 1284044422000],
    [13150001, 1284103338000],
    [13160000, 1284131400000],
    [13170000, 1284181635000],
    [13180003, 1284210864000],
    [13190000, 1284238395000],
    [13200004, 1284281470000],
    [13210003, 1284302032000],
    [13220003, 1284356532000],
    [13230000, 1284386961000],
    [13240000, 1284438215000],
    [13250004, 1284472990000],
    [13260003, 1284523883000],
    [13270000, 1284559545000],
    [13280001, 1284613657000],
    [13290003, 1284647063000],
    [13300000, 1284703307000],
    [13310002, 1284734091000],
    [13320002, 1284780594000],
    [13330004, 1284816503000],
    [13340000, 1284859349000],
    [13350001, 1284898520000],
    [13360000, 1284922896000],
    [13370002, 1284973746000],
    [13380000, 1284995064000],
    [13390004, 1285058627000],
    [13400002, 1285084289000],
    [13410005, 1285151263000],
    [13420000, 1285174192000],
    [13430000, 1285227503000],
    [13440004, 1285250843000],
    [13450007, 1285301710000],
    [13460000, 1285338172000],
    [13470007, 1285386754000],
    [13480002, 1285418930000],
    [13490002, 1285443125000],
    [13500001, 1285491869000],
    [13510006, 1285512382000],
    [13520000, 1285572669000],
    [13530003, 1285599963000],
    [13540002, 1285663189000],
    [13550000, 1285688428000],
    [13560001, 1285753766000],
    [13570000, 1285777318000],
    [13580005, 1285845224000],
    [13590002, 1285866408000],
    [13600001, 1285932869000],
    [13610003, 1285955477000],
    [13620000, 1286011278000],
    [13630001, 1286033520000],
    [13640001, 1286079742000],
    [13650001, 1286106928000],
    [13660002, 1286124128000],
    [13670005, 1286189352000],
    [13680002, 1286210184000],
    [13690000, 1286277190000],
    [13700001, 1286298612000],
    [13710002, 1286366512000],
    [13720001, 1286391268000],
    [13730002, 1286455082000],
    [13740007, 1286486830000],
    [13750001, 1286543733000],
    [13760001, 1286582060000],
    [13770000, 1286622918000],
    [13780003, 1286642450000],
    [13790000, 1286692726000],
    [13800001, 1286718021000],
    [13810003, 1286752215000],
    [13820000, 1286791489000],
    [13830001, 1286810989000],
    [13840001, 1286875998000],
    [13850003, 1286899374000],
    [13860000, 1286965646000],
    [13870000, 1286989108000],
    [13880001, 1287056928000],
    [13890003, 1287079973000],
    [13900000, 1287145497000],
    [13910002, 1287172085000],
    [13920000, 1287225384000],
    [13930004, 1287245969000],
    [13940003, 1287296215000],
    [13950003, 1287320263000],
    [13960000, 1287346883000],
    [13970011, 1287406934000],
    [13980000, 1287449201000],
    [13990002, 1287496124000],
    [14000000, 1287550315000],
    [14010002, 1287585133000],
    [14020002, 1287645957000],
    [14030000, 1287674130000],
    [14040003, 1287738931000],
    [14050002, 1287763516000],
    [14060000, 1287818586000],
    [14070005, 1287844055000],
    [14080003, 1287886605000],
    [14090001, 1287917308000],
    [14100000, 1287934346000],
    [14110007, 1287997009000],
    [14120003, 1288019639000],
    [14130002, 1288084105000],
    [14140000, 1288107911000],
    [14150000, 1288175810000],
    [14160003, 1288199607000],
    [14170001, 1288268739000],
    [14180000, 1288301838000],
    [14190002, 1288357841000],
    [14200005, 1288383988000],
    [14210000, 1288430900000],
    [14220000, 1288450015000],
    [14230001, 1288460894000],
    [14240001, 1288503897000],
    [14250002, 1288524444000],
    [14260002, 1288536417000],
    [14270001, 1288559551000],
    [14280004, 1288614494000],
    [14290000, 1288636790000],
    [14300001, 1288701069000],
    [14310000, 1288724423000],
    [14320002, 1288776052000],
    [14330000, 1288796401000],
    [14340003, 1288857588000],
    [14350006, 1288883585000],
    [14360004, 1288949700000],
    [14370001, 1288973472000],
    [14380001, 1289028320000],
    [14390005, 1289053182000],
    [14400004, 1289092512000],
    [14410000, 1289124567000],
    [14420002, 1289142469000],
    [14430000, 1289200698000],
    [14440000, 1289226741000],
    [14450000, 1289286036000],
    [14460001, 1289314520000],
    [14470002, 1289376224000],
    [14480002, 1289401742000],
    [14490001, 1289460865000],
    [14500001, 1289484283000],
    [14510000, 1289523233000],
    [14520001, 1289569043000],
    [14530002, 1289607472000],
    [14540001, 1289646752000],
    [14550004, 1289665668000],
    [14560005, 1289714543000],
    [14570002, 1289738263000],
    [14580001, 1289759036000],
    [14590001, 1289822837000],
    [14600006, 1289844897000],
    [14610004, 1289910783000],
    [14620000, 1289940668000],
    [14630004, 1289999394000],
    [14640000, 1290044327000],
    [14650001, 1290088675000],
    [14660010, 1290146173000],
    [14670003, 1290178715000],
    [14680006, 1290229305000],
    [14690001, 1290258858000],
    [14700000, 1290283021000],
    [14710001, 1290330017000],
    [14720001, 1290351368000],
    [14730000, 1290407282000],
    [14740002, 1290435028000],
    [14750000, 1290464600000],
    [14760004, 1290505101000],
    [14770005, 1290524323000],
    [14780002, 1290585156000],
    [14790002, 1290611533000],
    [14800002, 1290675115000],
    [14810005, 1290699536000],
    [14820003, 1290765982000],
    [14830002, 1290789167000],
    [14840001, 1290843946000],
    [14850000, 1290869019000],
    [14860000, 1290913104000],
    [14870000, 1290943562000],
    [14880003, 1290963089000],
    [14890000, 1291030901000],
    [14900000, 1291054197000],
    [14910000, 1291121135000],
    [14920001, 1291167073000],
    [14930001, 1291212813000],
    [14940001, 1291277195000],
    [14950003, 1291305231000],
    [14960001, 1291373249000],
    [14970000, 1291398478000],
    [14980003, 1291454733000],
    [14990006, 1291477691000],
    [15000018, 1291527486000],
    [15010002, 1291553361000],
    [15020002, 1291586032000],
    [15030000, 1291642446000],
    [15040001, 1291699202000],
    [15050004, 1291733347000],
    [15060010, 1291798100000],
    [15070001, 1291824339000],
    [15080000, 1291893540000],
    [15090000, 1291922099000],
    [15100005, 1291986770000],
    [15110000, 1292032722000],
    [15120005, 1292069186000],
    [15130001, 1292090384000],
    [15140000, 1292141815000],
    [15150000, 1292164081000],
    [15160001, 1292223938000],
    [15170002, 1292251943000],
    [15180009, 1292315000000],
    [15190001, 1292339700000],
    [15200000, 1292399890000],
    [15210000, 1292423274000],
    [15220007, 1292475878000],
    [15230002, 1292508400000],
    [15240003, 1292562759000],
    [15250003, 1292596779000],
    [15260000, 1292646528000],
    [15270004, 1292677534000],
    [15280001, 1292703551000],
    [15290000, 1292750853000],
    [15300001, 1292770943000],
    [15310005, 1292830092000],
    [15320003, 1292856886000],
    [15330003, 1292916490000],
    [15340000, 1292943635000],
    [15350000, 1293001746000],
    [15360003, 1293029691000],
    [15370002, 1293077329000],
    [15380000, 1293107925000],
    [15390000, 1293124460000],
    [15400012, 1293177148000],
    [15410000, 1293200478000],
    [15420002, 1293223235000],
    [15430000, 1293268036000],
    [15440002, 1293288596000],
    [15450000, 1293330169000],
    [15460004, 1293364717000],
    [15470003, 1293384306000],
    [15480004, 1293441672000],
    [15490001, 1293465151000],
    [15500002, 1293518526000],
    [15510001, 1293546831000],
    [15520002, 1293597431000],
    [15530005, 1293630221000],
    [15540002, 1293675124000],
    [15550000, 1293712423000],
    [15560002, 1293736274000],
    [15570000, 1293785654000],
    [15580000, 1293807104000],
    [15590000, 1293813108000],
    [15600004, 1293848040000],
    [15610006, 1293872314000],
    [15620010, 1293890945000],
    [15630001, 1293917668000],
    [15640001, 1293964548000],
    [15650000, 1293984667000],
    [15660000, 1294034714000],
    [15670000, 1294059820000],
    [15680002, 1294081042000],
    [15690008, 1294132088000],
    [15700004, 1294153388000],
    [15710001, 1294198562000],
    [15720001, 1294229511000],
    [15730000, 1294247497000],
    [15740000, 1294301349000],
    [15750001, 1294325182000],
    [15760002, 1294376467000],
    [15770000, 1294407648000],
    [15780002, 1294444452000],
    [15790003, 1294485613000],
    [15800003, 1294506185000],
    [15810001, 1294559429000],
    [15820000, 1294583260000],
    [15830001, 1294623456000],
    [15840000, 1294655809000],
    [15850001, 1294673991000],
    [15860000, 1294737035000],
    [15870000, 1294760300000],
    [15880003, 1294826572000],
    [15890003, 1294849889000],
    [15900000, 1294917508000],
    [15910001, 1294941223000],
    [15920002, 1295007709000],
    [15930001, 1295029623000],
    [15940002, 1295083114000],
    [15950000, 1295105042000],
    [15960008, 1295150096000],
    [15970000, 1295176582000],
    [15980003, 1295193532000],
    [15990008, 1295257398000],
    [16000000, 1295279291000],
    [16010001, 1295344325000],
    [16020001, 1295365579000],
    [16030000, 1295430007000],
    [16040000, 1295451532000],
    [16050000, 1295515049000],
    [16060001, 1295537558000],
    [16070000, 1295601077000],
    [16080006, 1295625363000],
    [16090003, 1295677743000],
    [16100002, 1295703430000],
    [16110005, 1295729619000],
    [16120000, 1295773021000],
    [16130001, 1295792062000],
    [16140002, 1295841907000],
    [16150000, 1295876478000],
    [16160003, 1295920129000],
    [16170001, 1295961719000],
    [16180003, 1296005962000],
    [16190001, 1296048001000],
    [16200000, 1296084003000],
    [16210001, 1296132386000],
    [16220000, 1296154256000],
    [16230002, 1296214941000],
    [16240001, 1296233843000],
    [16250000, 1296285251000],
    [16260001, 1296308274000],
    [16270000, 1296328631000],
    [16280000, 1296373529000],
    [16290003, 1296393394000],
    [16300004, 1296412941000],
    [16310000, 1296473201000],
    [16320003, 1296490245000],
    [16330006, 1296550062000],
    [16340000, 1296571947000],
    [16350000, 1296623456000],
    [16360000, 1296652965000],
    [16370000, 1296675824000],
    [16380002, 1296731668000],
    [16390000, 1296748000000],
    [16400000, 1296806467000],
    [16410000, 1296830277000],
    [16420000, 1296870503000],
    [16430000, 1296902174000],
    [16440000, 1296919854000],
    [16450000, 1296962274000],
    [16460001, 1296987424000],
    [16470002, 1297003969000],
    [16480000, 1297047879000],
    [16490006, 1297082200000],
    [16500001, 1297099131000],
    [16510003, 1297158628000],
    [16520001, 1297177734000],
    [16530004, 1297231023000],
    [16540001, 1297258924000],
    [16550000, 1297293429000],
    [16560000, 1297339287000],
    [16570000, 1297356967000],
    [16580000, 1297406230000],
    [16590005, 1297428558000],
    [16600000, 1297444000000],
    [16610005, 1297492058000],
    [16620000, 1297515324000],
    [16630004, 1297532087000],
    [16640002, 1297580468000],
    [16650000, 1297602605000],
    [16660000, 1297615573000],
    [16670000, 1297667071000],
    [16680001, 1297687168000],
    [16690001, 1297697982000],
    [16700001, 1297749017000],
    [16710000, 1297776910000],
    [16720000, 1297805417000],
    [16730000, 1297857224000],
    [16740004, 1297876015000],
    [16750002, 1297935653000],
    [16760000, 1297956063000],
    [16770000, 1298009599000],
    [16780000, 1298036761000],
    [16790001, 1298065707000],
    [16800000, 1298109530000],
    [16810002, 1298128885000],
    [16820000, 1298170973000],
    [16830000, 1298198906000],
    [16840000, 1298215638000],
    [16850003, 1298271332000],
    [16860003, 1298297297000],
    [16870001, 1298334574000],
    [16880000, 1298374612000],
    [16890000, 1298387834000],
    [16900006, 1298440832000],
    [16910001, 1298468928000],
    [16920002, 1298506815000],
    [16930003, 1298550268000],
    [16940000, 1298572618000],
    [16950001, 1298631374000],
    [16960001, 1298650858000],
    [16970003, 1298702149000],
    [16980000, 1298727161000],
    [16990001, 1298750306000],
    [17000000, 1298796449000],
    [17010000, 1298816402000],
    [17020001, 1298861035000],
    [17030001, 1298897501000],
    [17040000, 1298918040000],
    [17050001, 1298976561000],
    [17060002, 1298995413000],
    [17070000, 1299054977000],
    [17080002, 1299077821000],
    [17090002, 1299127707000],
    [17100000, 1299157842000],
    [17110002, 1299180217000],
    [17120003, 1299238067000],
    [17130000, 1299257087000],
    [17140006, 1299308923000],
    [17150000, 1299332822000],
    [17160002, 1299356148000],
    [17170000, 1299399969000],
    [17180001, 1299419595000],
    [17190003, 1299451732000],
    [17200000, 1299497993000],
    [17210006, 1299515396000],
    [17220000, 1299573968000],
    [17230000, 1299595987000],
    [17240001, 1299641799000],
    [17250002, 1299672911000],
    [17260000, 1299688220000],
    [17270004, 1299744534000],
    [17280000, 1299767015000],
    [17290000, 1299810783000],
    [17300003, 1299862908000],
    [17310003, 1299930844000],
    [17320000, 1299984696000],
    [17330001, 1300017855000],
    [17340009, 1300066730000],
    [17350001, 1300107195000],
    [17360002, 1300158110000],
    [17370001, 1300194119000],
    [17380001, 1300244735000],
    [17390002, 1300279598000],
    [17400002, 1300319831000],
    [17410000, 1300361932000],
    [17420000, 1300384138000],
    [17430000, 1300440756000],
    [17440000, 1300461723000],
    [17450000, 1300509897000],
    [17460003, 1300536055000],
    [17470009, 1300554024000],
    [17480004, 1300601614000],
    [17490002, 1300623320000],
    [17500002, 1300638827000],
    [17510002, 1300684104000],
    [17520001, 1300705166000],
    [17530000, 1300719958000],
    [17540002, 1300762970000],
    [17550001, 1300793717000],
    [17560001, 1300809983000],
    [17570006, 1300860379000],
    [17580002, 1300885414000],
    [17590001, 1300905210000],
    [17600002, 1300957348000],
    [17610000, 1300978270000],
    [17620004, 1301022368000],
    [17630000, 1301051509000],
    [17640002, 1301068408000],
    [17650005, 1301115630000],
    [17660002, 1301141633000],
    [17670000, 1301159101000],
    [17680000, 1301207711000],
    [17690001, 1301230061000],
    [17700000, 1301247769000],
    [17710001, 1301300097000],
    [17720004, 1301321694000],
    [17730004, 1301359814000],
    [17740000, 1301394002000],
    [17750002, 1301412230000],
    [17760001, 1301461723000],
    [17770000, 1301488134000],
    [17780000, 1301505666000],
    [17790009, 1301556016000],
    [17800002, 1301578213000],
    [17810001, 1301593721000],
    [17820002, 1301640723000],
    [17830001, 1301662323000],
    [17840002, 1301677376000],
    [17850001, 1301725555000],
    [17860000, 1301749268000],
    [17870000, 1301766216000],
    [17880000, 1301813367000],
    [17890005, 1301834761000],
    [17900010, 1301849283000],
    [17910000, 1301900908000],
    [17920000, 1301924610000],
    [17930001, 1301957678000],
    [17940002, 1301998758000],
    [17950001, 1302017959000],
    [17960002, 1302072029000],
    [17970000, 1302097165000],
    [17980001, 1302133258000],
    [17990000, 1302175976000],
    [18000000, 1302197316000],
    [18010003, 1302258790000],
    [18020004, 1302278230000],
    [18030010, 1302327468000],
    [18040001, 1302352435000],
    [18050005, 1302368737000],
    [18060000, 1302416328000],
    [18070000, 1302439072000],
    [18080000, 1302457025000],
    [18090000, 1302521270000],
    [18100001, 1302541649000],
    [18110000, 1302607823000],
    [18120001, 1302629764000],
    [18130006, 1302695816000],
    [18140000, 1302717159000],
    [18150002, 1302782840000],
    [18160001, 1302806997000],
    [18170000, 1302870683000],
    [18180001, 1302894611000],
    [18190002, 1302944980000],
    [18200004, 1302965974000],
    [18210001, 1303006426000],
    [18220000, 1303034427000],
    [18230001, 1303051920000],
    [18240003, 1303108733000],
    [18250000, 1303136821000],
    [18260001, 1303195064000],
    [18270000, 1303224047000],
    [18280001, 1303282325000],
    [18290003, 1303309822000],
    [18300000, 1303368067000],
    [18310004, 1303396790000],
    [18320000, 1303453454000],
    [18330000, 1303482659000],
    [18340001, 1303525887000],
    [18350003, 1303555922000],
    [18360005, 1303572740000],
    [18370001, 1303614719000],
    [18380009, 1303640637000],
    [18390003, 1303656640000],
    [18400001, 1303712501000],
    [18410001, 1303740814000],
    [18420000, 1303793239000],
    [18430000, 1303826988000],
    [18440003, 1303878971000],
    [18450001, 1303912501000],
    [18460000, 1303952901000],
    [18470001, 1303996909000],
    [18480001, 1304026701000],
    [18490004, 1304070588000],
    [18500000, 1304090317000],
    [18510001, 1304134618000],
    [18520003, 1304163399000],
    [18530000, 1304179558000],
    [18540000, 1304224315000],
    [18550003, 1304248335000],
    [18560000, 1304264348000],
    [18570003, 1304318720000],
    [18580005, 1304344670000],
    [18590000, 1304380431000],
    [18600000, 1304418142000],
    [18610000, 1304436977000],
    [18620000, 1304483107000],
    [18630000, 1304510844000],
    [18640001, 1304525913000],
    [18650002, 1304571627000],
    [18660001, 1304594862000],
    [18670000, 1304612428000],
    [18680000, 1304670820000],
    [18690000, 1304692839000],
    [18700001, 1304730933000],
    [18710001, 1304762137000],
    [18720001, 1304779328000],
    [18730005, 1304805791000],
    [18740000, 1304842467000],
    [18750001, 1304861184000],
    [18760001, 1304880764000],
    [18770003, 1304942590000],
    [18780005, 1304959461000],
    [18790000, 1305024086000],
    [18800000, 1305042511000],
    [18810000, 1305105612000],
    [18820000, 1305126851000],
    [18830002, 1305186649000],
    [18840002, 1305211233000],
    [18850001, 1305263143000],
    [18860002, 1305296090000],
    [18870007, 1305334791000],
    [18880002, 1305369203000],
    [18890000, 1305386728000],
    [18900002, 1305427616000],
    [18910001, 1305454232000],
    [18920000, 1305470965000],
    [18930002, 1305522320000],
    [18940001, 1305553956000],
    [18950001, 1305596355000],
    [18960001, 1305638411000],
    [18970001, 1305673304000],
    [18980002, 1305723866000],
    [18990001, 1305757275000],
    [19000000, 1305809290000],
    [19010009, 1305837813000],
    [19020007, 1305894834000],
    [19030001, 1305913206000],
    [19040002, 1305965220000],
    [19050002, 1305986717000],
    [19060004, 1306009914000],
    [19070000, 1306049956000],
    [19080003, 1306069433000],
    [19090002, 1306087127000],
    [19100000, 1306150069000],
    [19110000, 1306166258000],
    [19120003, 1306228843000],
    [19130003, 1306250294000],
    [19140006, 1306311779000],
    [19150002, 1306335760000],
    [19160002, 1306393436000],
    [19170000, 1306420009000],
    [19180000, 1306473281000],
    [19190000, 1306504375000],
    [19200002, 1306530639000],
    [19210000, 1306573710000],
    [19220003, 1306593144000],
    [19230000, 1306621510000],
    [19240003, 1306655709000],
    [19250004, 1306673753000],
    [19260001, 1306688234000],
    [19270001, 1306749515000],
    [19280001, 1306768188000],
    [19290000, 1306826654000],
    [19300000, 1306852185000],
    [19310001, 1306898547000],
    [19320000, 1306935589000],
    [19330003, 1306973637000],
    [19340005, 1307019857000],
    [19350002, 1307045441000],
    [19360000, 1307103660000],
    [19370000, 1307119838000],
    [19380000, 1307169540000],
    [19390003, 1307193572000],
    [19400001, 1307210358000],
    [19410001, 1307255861000],
    [19420003, 1307276768000],
    [19430000, 1307290443000],
    [19440000, 1307350716000],
    [19450001, 1307371274000],
    [19460000, 1307422771000],
    [19470001, 1307455172000],
    [19480001, 1307499891000],
    [19490000, 1307539315000],
    [19500000, 1307567911000],
    [19510004, 1307622498000],
    [19520000, 1307640381000],
    [19530001, 1307704413000],
    [19540001, 1307722120000],
    [19550002, 1307768243000],
    [19560000, 1307794803000],
    [19570002, 1307811258000],
    [19580003, 1307857708000],
    [19590005, 1307879918000],
    [19600001, 1307895029000],
    [19610000, 1307958216000],
    [19620002, 1307978222000],
    [19630002, 1308040399000],
    [19640000, 1308063792000],
    [19650003, 1308124029000],
    [19660002, 1308150027000],
    [19670001, 1308210699000],
    [19680003, 1308236169000],
    [19690000, 1308296022000],
    [19700000, 1308322119000],
    [19710001, 1308363044000],
    [19720002, 1308395885000],
    [19730004, 1308413453000],
    [19740002, 1308458162000],
    [19750010, 1308481668000],
    [19760006, 1308496706000],
    [19770008, 1308555450000],
    [19780001, 1308580470000],
    [19790001, 1308634952000],
    [19800000, 1308666162000],
    [19810003, 1308718375000],
    [19820000, 1308752129000],
    [19830000, 1308803871000],
    [19840000, 1308838040000],
    [19850001, 1308887980000],
    [19860001, 1308923907000],
    [19870002, 1308962490000],
    [19880002, 1308998955000],
    [19890003, 1309017577000],
    [19900012, 1309064025000],
    [19910001, 1309089449000],
    [19920000, 1309106589000],
    [19930000, 1309172893000],
    [19940004, 1309193070000],
    [19950000, 1309260287000],
    [19960001, 1309282649000],
    [19970001, 1309349322000],
    [19980001, 1309378576000],
    [19990002, 1309438414000],
    [20000000, 1309470273000],
    [20010000, 1309524618000],
    [20020000, 1309552158000],
    [20030000, 1309600178000],
    [20040003, 1309620373000],
    [20050002, 1309663215000],
    [20060000, 1309690686000],
    [20070000, 1309707046000],
    [20080001, 1309766699000],
    [20090000, 1309790656000],
    [20100001, 1309846681000],
    [20110000, 1309876281000],
    [20120004, 1309932570000],
    [20130000, 1309961691000],
    [20140000, 1310003497000],
    [20150000, 1310041811000],
    [20160000, 1310056900000],
    [20170000, 1310120484000],
    [20180001, 1310141027000],
    [20190001, 1310192843000],
    [20200001, 1310218573000],
    [20210003, 1310240759000],
    [20220001, 1310284832000],
    [20230000, 1310305227000],
    [20240000, 1310341152000],
    [20250000, 1310388758000],
    [20260001, 1310418431000],
    [20270000, 1310475992000],
    [20280001, 1310508762000],
    [20290003, 1310561348000],
    [20300000, 1310591204000],
    [20310004, 1310646798000],
    [20320001, 1310670408000],
    [20330007, 1310731839000],
    [20340002, 1310755076000],
    [20350002, 1310806937000],
    [20360001, 1310829167000],
    [20370000, 1310873205000],
    [20380004, 1310902877000],
    [20390001, 1310920468000],
    [20400001, 1310968321000],
    [20410006, 1310991474000],
    [20420000, 1311006875000],
    [20430000, 1311064100000],
    [20440000, 1311084772000],
    [20450001, 1311123782000],
    [20460002, 1311162399000],
    [20470001, 1311178561000],
    [20480000, 1311235088000],
    [20490000, 1311258344000],
    [20500000, 1311303174000],
    [20510002, 1311339356000],
    [20520000, 1311362905000],
    [20530000, 1311413763000],
    [20540001, 1311434039000],
    [20550000, 1311475736000],
    [20560001, 1311505255000],
    [20570001, 1311522648000],
    [20580000, 1311579354000],
    [20590002, 1311604273000],
    [20600000, 1311653390000],
    [20610001, 1311687194000],
    [20620004, 1311730278000],
    [20630001, 1311770214000],
    [20640001, 1311795655000],
    [20650002, 1311850920000],
    [20660000, 1311871766000],
    [20670001, 1311929902000],
    [20680002, 1311953147000],
    [20690002, 1312003428000],
    [20700000, 1312032538000],
    [20710000, 1312059365000],
    [20720004, 1312104991000],
    [20730002, 1312124466000],
    [20740000, 1312170787000],
    [20750000, 1312202579000],
    [20760004, 1312222639000],
    [20770000, 1312277576000],
    [20780003, 1312297129000],
    [20790000, 1312343645000],
    [20800001, 1312375698000],
    [20810005, 1312396079000],
    [20820000, 1312451540000],
    [20830000, 1312472965000],
    [20840007, 1312526698000],
    [20850001, 1312553734000],
    [20860002, 1312594022000],
    [20870000, 1312629862000],
    [20880000, 1312649133000],
    [20890001, 1312698590000],
    [20900003, 1312724121000],
    [20910006, 1312751963000],
    [20920006, 1312800742000],
    [20930002, 1312820645000],
    [20940007, 1312875899000],
    [20950000, 1312900379000],
    [20960001, 1312941650000],
    [20970000, 1312978307000],
    [20980003, 1312998067000],
    [20990000, 1313051765000],
    [21000001, 1313076106000],
    [21010000, 1313130407000],
    [21020001, 1313160975000],
    [21030001, 1313215417000],
    [21040004, 1313247433000],
    [21050005, 1313301467000],
    [21060001, 1313332019000],
    [21070003, 1313381642000],
    [21080000, 1313414672000],
    [21090000, 1313455024000],
    [21100003, 1313495199000],
    [21110002, 1313516792000],
    [21120001, 1313572991000],
    [21130001, 1313595683000],
    [21140004, 1313650708000],
    [21150000, 1313677682000],
    [21160000, 1313726375000],
    [21170003, 1313758608000],
    [21180000, 1313781396000],
    [21190001, 1313833429000],
    [21200001, 1313855451000],
    [21210003, 1313906661000],
    [21220000, 1313933611000],
    [21230005, 1313974048000],
    [21240002, 1314015010000],
    [21250000, 1314035676000],
    [21260007, 1314093605000],
    [21270000, 1314115440000],
    [21280000, 1314172198000],
    [21290001, 1314197976000],
    [21300002, 1314250182000],
    [21310000, 1314280626000],
    [21320002, 1314322037000],
    [21330000, 1314362035000],
    [21340000, 1314386898000],
    [21350000, 1314439830000],
    [21360001, 1314461821000],
    [21370001, 1314512215000],
    [21380000, 1314538371000],
    [21390000, 1314575380000],
    [21400000, 1314620028000],
    [21410001, 1314642285000],
    [21420001, 1314701761000],
    [21430001, 1314721520000],
    [21440000, 1314777562000],
    [21450000, 1314800426000],
    [21460000, 1314846685000],
    [21470001, 1314884232000],
    [21480001, 1314930814000],
    [21490000, 1314969958000],
    [21500000, 1314999773000],
    [21510003, 1315042331000],
    [21520000, 1315062498000],
    [21530002, 1315104703000],
    [21540006, 1315134496000],
    [21550000, 1315151760000],
    [21560003, 1315214710000],
    [21570004, 1315237490000],
    [21580002, 1315303317000],
    [21590000, 1315326783000],
    [21600001, 1315394334000],
    [21610000, 1315417250000],
    [21620007, 1315484272000],
    [21630005, 1315514205000],
    [21640002, 1315572280000],
    [21650001, 1315597738000],
    [21660000, 1315650322000],
    [21670009, 1315671921000],
    [21680001, 1315722958000],
    [21690001, 1315747606000],
    [21700000, 1315786265000],
    [21710002, 1315831829000],
    [21720000, 1315864633000],
    [21730000, 1315919301000],
    [21740001, 1315965116000],
    [21750000, 1316008394000],
    [21760003, 1316062939000],
    [21770000, 1316097353000],
    [21780013, 1316159041000],
    [21790000, 1316186261000],
    [21800005, 1316239287000],
    [21810001, 1316267691000],
    [21820003, 1316306735000],
    [21830010, 1316343785000],
    [21840001, 1316363104000],
    [21850001, 1316414441000],
    [21860003, 1316438203000],
    [21870001, 1316470394000],
    [21880001, 1316522778000],
    [21890000, 1316554809000],
    [21900000, 1316600632000],
    [21910000, 1316619796000],
    [21920009, 1316684574000],
    [21930001, 1316707729000],
    [21940001, 1316763644000],
    [21950001, 1316788791000],
    [21960000, 1316835026000],
    [21970004, 1316867190000],
    [21980000, 1316889287000],
    [21990000, 1316940627000],
    [22000000, 1316961644000],
    [22010001, 1317021110000],
    [22020004, 1317049342000],
    [22030000, 1317116618000],
    [22040000, 1317141016000],
    [22050001, 1317209989000],
    [22060000, 1317236089000],
    [22070001, 1317301682000],
    [22080000, 1317349434000],
    [22090002, 1317390868000],
    [22100004, 1317432575000],
    [22110003, 1317469757000],
    [22120000, 1317488614000],
    [22130007, 1317539969000],
    [22140002, 1317562919000],
    [22150004, 1317609637000],
    [22160000, 1317650002000],
    [22170000, 1317704968000],
    [22180000, 1317738990000],
    [22190003, 1317801343000],
    [22200000, 1317827808000],
    [22210000, 1317895808000],
    [22220003, 1317919863000],
    [22230002, 1317988722000],
    [22240000, 1318016522000],
    [22250001, 1318071385000],
    [22260001, 1318093300000],
    [22270003, 1318147216000],
    [22280000, 1318170584000],
    [22290000, 1318208160000],
    [22300003, 1318240420000],
    [22310000, 1318258161000],
    [22320003, 1318316469000],
    [22330003, 1318344717000],
    [22340001, 1318409556000],
    [22350000, 1318435205000],
    [22360001, 1318504263000],
    [22370000, 1318529232000],
    [22380002, 1318595828000],
    [22390000, 1318618338000],
    [22400000, 1318671167000],
    [22410002, 1318691515000],
    [22420001, 1318735377000],
    [22430000, 1318764254000],
    [22440003, 1318781325000],
    [22450005, 1318848420000],
    [22460001, 1318869603000],
    [22470000, 1318938778000],
    [22480001, 1318967790000],
    [22490004, 1319030449000],
    [22500004, 1319079994000],
    [22510002, 1319119857000],
    [22520003, 1319179130000],
    [22530000, 1319208961000],
    [22540001, 1319255965000],
    [22550003, 1319286551000],
    [22560000, 1319305301000],
    [22570001, 1319356571000],
    [22580000, 1319378199000],
    [22590003, 1319426643000],
    [22600001, 1319463672000],
    [22610000, 1319512754000],
    [22620004, 1319551356000],
    [22630000, 1319610153000],
    [22640005, 1319641157000],
    [22650000, 1319707349000],
    [22660000, 1319731547000],
    [22670001, 1319800581000],
    [22680000, 1319819486000],
    [22690003, 1319873970000],
    [22700001, 1319897967000],
    [22710001, 1319935963000],
    [22720004, 1319968209000],
    [22730003, 1319985235000],
    [22740001, 1320011852000],
    [22750000, 1320058950000],
    [22760000, 1320071362000],
    [22770003, 1320098160000],
    [22780001, 1320152372000],
    [22790001, 1320190601000],
    [22800001, 1320241044000],
    [22810001, 1320284413000],
    [22820004, 1320319340000],
    [22830001, 1320337268000],
    [22840003, 1320404415000],
    [22850002, 1320425018000],
    [22860001, 1320480947000],
    [22870000, 1320503617000],
    [22880005, 1320543865000],
    [22890002, 1320574159000],
    [22900002, 1320591657000],
    [22910000, 1320651757000],
    [22920003, 1320677541000],
    [22930000, 1320741302000],
    [22940003, 1320765723000],
    [22950001, 1320831753000],
    [22960006, 1320855105000],
    [22970001, 1320924795000],
    [22980002, 1320944792000],
    [22990001, 1321008281000],
    [23000000, 1321023750000],
    [23010002, 1321070842000],
    [23020003, 1321103718000],
    [23030000, 1321129017000],
    [23040000, 1321177273000],
    [23050002, 1321196946000],
    [23060001, 1321261553000],
    [23070000, 1321285909000],
    [23080000, 1321355699000],
    [23090000, 1321383060000],
    [23100005, 1321449832000],
    [23110002, 1321506266000],
    [23120003, 1321541632000],
    [23130002, 1321607482000],
    [23140001, 1321632446000],
    [23150000, 1321687922000],
    [23160008, 1321712956000],
    [23170003, 1321755137000],
    [23180004, 1321786716000],
    [23190002, 1321804967000],
    [23200000, 1321873109000],
    [23210001, 1321895437000],
    [23220003, 1321963844000],
    [23230002, 1321984715000],
    [23240001, 1322039864000],
    [23250001, 1322060908000],
    [23260000, 1322127226000],
    [23270002, 1322153048000],
    [23280001, 1322222793000],
    [23290001, 1322254119000],
    [23300000, 1322305405000],
    [23310000, 1322327316000],
    [23320000, 1322381472000],
    [23330000, 1322404249000],
    [23340002, 1322465114000],
    [23350000, 1322493113000],
    [23360006, 1322560101000],
    [23370001, 1322584337000],
    [23380000, 1322653117000],
    [23390001, 1322680475000],
    [23400002, 1322745694000],
    [23410000, 1322805019000],
    [23420001, 1322838363000],
    [23430004, 1322893389000],
    [23440000, 1322920931000],
    [23450001, 1322957348000],
    [23460001, 1322993838000],
    [23470000, 1323013257000],
    [23480002, 1323079810000],
    [23490000, 1323104038000],
    [23500001, 1323173670000],
    [23510000, 1323209081000],
    [23520000, 1323264634000],
    [23530000, 1323321279000],
    [23540000, 1323355034000],
    [23550000, 1323418051000],
    [23560000, 1323446018000],
    [23570002, 1323502786000],
    [23580003, 1323529242000],
    [23590001, 1323577529000],
    [23600001, 1323605404000],
    [23610002, 1323626059000],
    [23620000, 1323692297000],
    [23630002, 1323722658000],
    [23640000, 1323782683000],
    [23650002, 1323837289000],
    [23660002, 1323873320000],
    [23670000, 1323936718000],
    [23680000, 1323963767000],
    [23690000, 1324032527000],
    [23700002, 1324058055000],
    [23710002, 1324116644000],
    [23720000, 1324140121000],
    [23730002, 1324193994000],
    [23740000, 1324217210000],
    [23750013, 1324272754000],
    [23760005, 1324304982000],
    [23770001, 1324365660000],
    [23780000, 1324394436000],
    [23790000, 1324459838000],
    [23800000, 1324484711000],
    [23810000, 1324550120000],
    [23820000, 1324573423000],
    [23830002, 1324631388000],
    [23840000, 1324653579000],
    [23850006, 1324696103000],
    [23860002, 1324726178000],
    [23870000, 1324741818000],
    [23880005, 1324784124000],
    [23890000, 1324810794000],
    [23900002, 1324825733000],
    [23910008, 1324876708000],
    [23920002, 1324907837000],
    [23930004, 1324949590000],
    [23940001, 1324988433000],
    [23950000, 1325009155000],
    [23960001, 1325065524000],
    [23970003, 1325088851000],
    [23980000, 1325144565000],
    [23990000, 1325171450000],
    [24000001, 1325225873000],
    [24010000, 1325255398000],
    [24020000, 1325299260000],
    [24030001, 1325329713000],
    [24040000, 1325343849000],
    [24050000, 1325354156000],
    [24060002, 1325391700000],
    [24070005, 1325414182000],
    [24080002, 1325431130000],
    [24090005, 1325478851000],
    [24100000, 1325508947000],
    [24110003, 1325530378000],
    [24120001, 1325581779000],
    [24130000, 1325602300000],
    [24140003, 1325647072000],
    [24150003, 1325677503000],
    [24160000, 1325695447000],
    [24170000, 1325751276000],
    [24180003, 1325775153000],
    [24190000, 1325826025000],
    [24200004, 1325856193000],
    [24210000, 1325881515000],
    [24220004, 1325930803000],
    [24230005, 1325951349000],
    [24240002, 1326004206000],
    [24250002, 1326030421000],
    [24260000, 1326066557000],
    [24270000, 1326104441000],
    [24280000, 1326122937000],
    [24290000, 1326185192000],
    [24300004, 1326209696000],
    [24310000, 1326276388000],
    [24320006, 1326301033000],
    [24330000, 1326370510000],
    [24340004, 1326401119000],
    [24350000, 1326460332000],
    [24360000, 1326497622000],
    [24370001, 1326539733000],
    [24380003, 1326558635000],
    [24390005, 1326609165000],
    [24400001, 1326632524000],
    [24410000, 1326656873000],
    [24420001, 1326718293000],
    [24430003, 1326756418000],
    [24440000, 1326807702000],
    [24450000, 1326863425000],
    [24460001, 1326897026000],
    [24470000, 1326958084000],
    [24480003, 1326985898000],
    [24490003, 1327050172000],
    [24500000, 1327074420000],
    [24510007, 1327128099000],
    [24520000, 1327153359000],
    [24530001, 1327179534000],
    [24540000, 1327223273000],
    [24550001, 1327242616000],
    [24560000, 1327296026000],
    [24570000, 1327327396000],
    [24580001, 1327380190000],
    [24590004, 1327414612000],
    [24600003, 1327472400000],
    [24610001, 1327502381000],
    [24620000, 1327564547000],
    [24630000, 1327591471000],
    [24640000, 1327656324000],
    [24650000, 1327679517000],
    [24660001, 1327732168000],
    [24670000, 1327757534000],
    [24680000, 1327786496000],
    [24690001, 1327827888000],
    [24700003, 1327847355000],
    [24710000, 1327901011000],
    [24720002, 1327931464000],
    [24730000, 1327978534000],
    [24740003, 1328015394000],
    [24750002, 1328045492000],
    [24760003, 1328097938000],
    [24770005, 1328118306000],
    [24780001, 1328180280000],
    [24790001, 1328199589000],
    [24800000, 1328260480000],
    [24810000, 1328281188000],
    [24820003, 1328326341000],
    [24830000, 1328356163000],
    [24840001, 1328373411000],
    [24850005, 1328421344000],
    [24860000, 1328444479000],
    [24870000, 1328460248000],
    [24880003, 1328518733000],
    [24890003, 1328539867000],
    [24900002, 1328589557000],
    [24910000, 1328620608000],
    [24920000, 1328648168000],
    [24930000, 1328702024000],
    [24940000, 1328721346000],
    [24950006, 1328783479000],
    [24960000, 1328803437000],
    [24970000, 1328863429000],
    [24980001, 1328886008000],
    [24990000, 1328929723000],
    [25000000, 1328960415000],
    [25010002, 1328977344000],
    [25020000, 1329025731000],
    [25030001, 1329049980000],
    [25040000, 1329068690000],
    [25050004, 1329130478000],
    [25060001, 1329146399000],
    [25070000, 1329183000000],
    [25080002, 1329215844000],
    [25090000, 1329228949000],
    [25100001, 1329244207000],
    [25110000, 1329302631000],
    [25120001, 1329321708000],
    [25130000, 1329381029000],
    [25140000, 1329404840000],
    [25150000, 1329462877000],
    [25160000, 1329489644000],
    [25170003, 1329534862000],
    [25180003, 1329566487000],
    [25190004, 1329585135000],
    [25200000, 1329633782000],
    [25210001, 1329656239000],
    [25220000, 1329678393000],
    [25230003, 1329737720000],
    [25240000, 1329757682000],
    [25250001, 1329819619000],
    [25260006, 1329839336000],
    [25270001, 1329895309000],
    [25280001, 1329916969000],
    [25290000, 1329931156000],
    [25300002, 1329990158000],
    [25310001, 1330010883000],
    [25320002, 1330069055000],
    [25330005, 1330093987000],
    [25340002, 1330135679000],
    [25350001, 1330167880000],
    [25360000, 1330185900000],
    [25370005, 1330233020000],
    [25380000, 1330257575000],
    [25390000, 1330275288000],
    [25400006, 1330337069000],
    [25410001, 1330358238000],
    [25420000, 1330419710000],
    [25430000, 1330442947000],
    [25440000, 1330501132000],
    [25450002, 1330524884000],
    [25460003, 1330571420000],
    [25470000, 1330607826000],
    [25480000, 1330646728000],
    [25490000, 1330690283000],
    [25500000, 1330707682000],
    [25510000, 1330757961000],
    [25520002, 1330781287000],
    [25530000, 1330801141000],
    [25540000, 1330848222000],
    [25550002, 1330868611000],
    [25560000, 1330896046000],
    [25570001, 1330946186000],
    [25580002, 1330963906000],
    [25590000, 1331022524000],
    [25600002, 1331045469000],
    [25610000, 1331096539000],
    [25620000, 1331126122000],
    [25630000, 1331153408000],
    [25640001, 1331206395000],
    [25650002, 1331224551000],
    [25660000, 1331280542000],
    [25670000, 1331301283000],
    [25680000, 1331320141000],
    [25690001, 1331368532000],
    [25700000, 1331389470000],
    [25710000, 1331423623000],
    [25720001, 1331460346000],
    [25730001, 1331477367000],
    [25740002, 1331525812000],
    [25750000, 1331557421000],
    [25760002, 1331583780000],
    [25770003, 1331637365000],
    [25780002, 1331655489000],
    [25790000, 1331711754000],
    [25800002, 1331734318000],
    [25810000, 1331772872000],
    [25820004, 1331812945000],
    [25830000, 1331831627000],
    [25840002, 1331889754000],
    [25850005, 1331910420000],
    [25860000, 1331953728000],
    [25870005, 1331982874000],
    [25880001, 1331999511000],
    [25890001, 1332046691000],
    [25900002, 1332071972000],
    [25910008, 1332089058000],
    [25920000, 1332147695000],
    [25930000, 1332169521000],
    [25940000, 1332215782000],
    [25950001, 1332244364000],
    [25960000, 1332261210000],
    [25970001, 1332318531000],
    [25980001, 1332341176000],
    [25990001, 1332391355000],
    [26000000, 1332421461000],
    [26010001, 1332448273000],
    [26020001, 1332498487000],
    [26030003, 1332517971000],
    [26040003, 1332568170000],
    [26050005, 1332593995000],
    [26060000, 1332612085000],
    [26070001, 1332661599000],
    [26080000, 1332682793000],
    [26090000, 1332715829000],
    [26100001, 1332759035000],
    [26110000, 1332777419000],
    [26120001, 1332831588000],
    [26130000, 1332856823000],
    [26140001, 1332899875000],
    [26150003, 1332935043000],
    [26160005, 1332953658000],
    [26170000, 1333009094000],
    [26180002, 1333032588000],
    [26190002, 1333081253000],
    [26200000, 1333110673000],
    [26210005, 1333131212000],
    [26220001, 1333180480000],
    [26230003, 1333202751000],
    [26240002, 1333216958000],
    [26250003, 1333261703000],
    [26260001, 1333283092000],
    [26270002, 1333297313000],
    [26280003, 1333350689000],
    [26290000, 1333374776000],
    [26300003, 1333414324000],
    [26310000, 1333448080000],
    [26320001, 1333465609000],
    [26330002, 1333513846000],
    [26340000, 1333543161000],
    [26350000, 1333565419000],
    [26360000, 1333620535000],
    [26370016, 1333641602000],
    [26380000, 1333698076000],
    [26390003, 1333722924000],
    [26400000, 1333767843000],
    [26410001, 1333799891000],
    [26420000, 1333819060000],
    [26430001, 1333869652000],
    [26440004, 1333891179000],
    [26450008, 1333926182000],
    [26460000, 1333975402000],
    [26470000, 1334009892000],
    [26480001, 1334063752000],
    [26490000, 1334104979000],
    [26500001, 1334150649000],
    [26510003, 1334197725000],
    [26520006, 1334239625000],
    [26530002, 1334300793000],
    [26540002, 1334329381000],
    [26550000, 1334378884000],
    [26560004, 1334407100000],
    [26570003, 1334425369000],
    [26580001, 1334474696000],
    [26590000, 1334496269000],
    [26600000, 1334531353000],
    [26610000, 1334582684000],
    [26620000, 1334632119000],
    [26630000, 1334671685000],
    [26640003, 1334732050000],
    [26650003, 1334761449000],
    [26660000, 1334828133000],
    [26670003, 1334851309000],
    [26680000, 1334918853000],
    [26690002, 1334940306000],
    [26700001, 1334994930000],
    [26710001, 1335018257000],
    [26720000, 1335058796000],
    [26730002, 1335087232000],
    [26740000, 1335104499000],
    [26750003, 1335157043000],
    [26760002, 1335189740000],
    [26770000, 1335245891000],
    [26780000, 1335278145000],
    [26790000, 1335339602000],
    [26800003, 1335366431000],
    [26810000, 1335431515000],
    [26820000, 1335454402000],
    [26830000, 1335519587000],
    [26840000, 1335541861000],
    [26850001, 1335594938000],
    [26860000, 1335620982000],
    [26870001, 1335654336000],
    [26880000, 1335693317000],
    [26890000, 1335712127000],
    [26900000, 1335752226000],
    [26910002, 1335780274000],
    [26920001, 1335797031000],
    [26930002, 1335840978000],
    [26940001, 1335876413000],
    [26950004, 1335894469000],
    [26960010, 1335953947000],
    [26970002, 1335972896000],
    [26980002, 1336020059000],
    [26990000, 1336047444000],
    [27000004, 1336063912000],
    [27010000, 1336112956000],
    [27020000, 1336137514000],
    [27030000, 1336153320000],
    [27040002, 1336202325000],
    [27050001, 1336224411000],
    [27060000, 1336241566000],
    [27070003, 1336288292000],
    [27080002, 1336308446000],
    [27090002, 1336324327000],
    [27100000, 1336390568000],
    [27110005, 1336408761000],
    [27120004, 1336476729000],
    [27130001, 1336496528000],
    [27140000, 1336563051000],
    [27150003, 1336582429000],
    [27160002, 1336650098000],
    [27170000, 1336671286000],
    [27180001, 1336738136000],
    [27190008, 1336762649000],
    [27200001, 1336816258000],
    [27210000, 1336836124000],
    [27220001, 1336881015000],
    [27230000, 1336907644000],
    [27240001, 1336923102000],
    [27250000, 1336984269000],
    [27260001, 1337006982000],
    [27270000, 1337064011000],
    [27280003, 1337092299000],
    [27290000, 1337148869000],
    [27300001, 1337178811000],
    [27310000, 1337236012000],
    [27320001, 1337265681000],
    [27330002, 1337325114000],
    [27340000, 1337352785000],
    [27350000, 1337400123000],
    [27360001, 1337430078000],
    [27370001, 1337446930000],
    [27380000, 1337494574000],
    [27390000, 1337516826000],
    [27400000, 1337533460000],
    [27410003, 1337597039000],
    [27420000, 1337616114000],
    [27430001, 1337681477000],
    [27440000, 1337702199000],
    [27450003, 1337766827000],
    [27460001, 1337785580000],
    [27470001, 1337843145000],
    [27480000, 1337870282000],
    [27490002, 1337926687000],
    [27500006, 1337956060000],
    [27510001, 1338000450000],
    [27520002, 1338032806000],
    [27530003, 1338049617000],
    [27540001, 1338098559000],
    [27550000, 1338121850000],
    [27560001, 1338141443000],
    [27570002, 1338206892000],
    [27580003, 1338229454000],
    [27590000, 1338293963000],
    [27600005, 1338315960000],
    [27610005, 1338380867000],
    [27620000, 1338404423000],
    [27630000, 1338467376000],
    [27640002, 1338486580000],
    [27650000, 1338549472000],
    [27660001, 1338566618000],
    [27670000, 1338615989000],
    [27680000, 1338642024000],
    [27690003, 1338660669000],
    [27700001, 1338707484000],
    [27710004, 1338728007000],
    [27720000, 1338744173000],
    [27730002, 1338808696000],
    [27740000, 1338826837000],
    [27750002, 1338893483000],
    [27760003, 1338912375000],
    [27770000, 1338976276000],
    [27780000, 1338997211000],
    [27790000, 1339061938000],
    [27800005, 1339083281000],
    [27810001, 1339148802000],
    [27820001, 1339169278000],
    [27830003, 1339217169000],
    [27840004, 1339243775000],
    [27850000, 1339257949000],
    [27860001, 1339303042000],
    [27870003, 1339327478000],
    [27880001, 1339342682000],
    [27890001, 1339406105000],
    [27900002, 1339428305000],
    [27910001, 1339492536000],
    [27920000, 1339514526000],
    [27930000, 1339577898000],
    [27940000, 1339600802000],
    [27950001, 1339666418000],
    [27960000, 1339688855000],
    [27970001, 1339754701000],
    [27980000, 1339775979000],
    [27990002, 1339827471000],
    [28000001, 1339851999000],
    [28010000, 1339869886000],
    [28020003, 1339917808000],
    [28030000, 1339938744000],
    [28040001, 1339956022000],
    [28050000, 1340019221000],
    [28060000, 1340035977000],
    [28070001, 1340096245000],
    [28080001, 1340116922000],
    [28090006, 1340172198000],
    [28100000, 1340201271000],
    [28110000, 1340251067000],
    [28120000, 1340286408000],
    [28130005, 1340331666000],
    [28140001, 1340372317000],
    [28150001, 1340405911000],
    [28160007, 1340447121000],
    [28170001, 1340467578000],
    [28180005, 1340515988000],
    [28190000, 1340540304000],
    [28200000, 1340557451000],
    [28210001, 1340622994000],
    [28220000, 1340641537000],
    [28230002, 1340710729000],
    [28240002, 1340740385000],
    [28250002, 1340802755000],
    [28260001, 1340855684000],
    [28270001, 1340893485000],
    [28280000, 1340956017000],
    [28290001, 1340983084000],
    [28300006, 1341034045000],
    [28310002, 1341062165000],
    [28320002, 1341083841000],
    [28330003, 1341130422000],
    [28340002, 1341150295000],
    [28350001, 1341191796000],
    [28360003, 1341235505000],
    [28370002, 1341282818000],
    [28380002, 1341322738000],
    [28390004, 1341368707000],
    [28400002, 1341409081000],
    [28410002, 1341459614000],
    [28420000, 1341497096000],
    [28430000, 1341552647000],
    [28440001, 1341585199000],
    [28450003, 1341622267000],
    [28460000, 1341655958000],
    [28470002, 1341672783000],
    [28480004, 1341710315000],
    [28490009, 1341741193000],
    [28500000, 1341759278000],
    [28510001, 1341816710000],
    [28520002, 1341844218000],
    [28530012, 1341901072000],
    [28540000, 1341930582000],
    [28550001, 1341988592000],
    [28560004, 1342017454000],
    [28570000, 1342075828000],
    [28580001, 1342103967000],
    [28590000, 1342161081000],
    [28600003, 1342190253000],
    [28610001, 1342235717000],
    [28620001, 1342268239000],
    [28630001, 1342286166000],
    [28640010, 1342337530000],
    [28650001, 1342360857000],
    [28660003, 1342396987000],
    [28670000, 1342431172000],
    [28680003, 1342450217000],
    [28690000, 1342508733000],
    [28700000, 1342536379000],
    [28710001, 1342596428000],
    [28720002, 1342623394000],
    [28730002, 1342684054000],
    [28740000, 1342709478000],
    [28750000, 1342765943000],
    [28760006, 1342795449000],
    [28770000, 1342842784000],
    [28780001, 1342873829000],
    [28790000, 1342892175000],
    [28800000, 1342941622000],
    [28810000, 1342963500000],
    [28820001, 1342994775000],
    [28830003, 1343043964000],
    [28840001, 1343063586000],
    [28850006, 1343123005000],
    [28860001, 1343144148000],
    [28870002, 1343199642000],
    [28880003, 1343226067000],
    [28890000, 1343274080000],
    [28900001, 1343307927000],
    [28910004, 1343345931000],
    [28920000, 1343389753000],
    [28930005, 1343409584000],
    [28940001, 1343463465000],
    [28950000, 1343487606000],
    [28960005, 1343533031000],
    [28970002, 1343561406000],
    [28980000, 1343578531000],
    [28990001, 1343635911000],
    [29000001, 1343659416000],
    [29010000, 1343708503000],
    [29020001, 1343739760000],
    [29030000, 1343759280000],
    [29040002, 1343813698000],
    [29050000, 1343833676000],
    [29060003, 1343881064000],
    [29070000, 1343911244000],
    [29080000, 1343928684000],
    [29090003, 1343983187000],
    [29100001, 1344005495000],
    [29110001, 1344049127000],
    [29120000, 1344080978000],
    [29130003, 1344100077000],
    [29140000, 1344149355000],
    [29150001, 1344172653000],
    [29160001, 1344192347000],
    [29170000, 1344245752000],
    [29180000, 1344265213000],
    [29190004, 1344310025000],
    [29200000, 1344341702000],
    [29210000, 1344357427000],
    [29220000, 1344408602000],
    [29230005, 1344432797000],
    [29240000, 1344449877000],
    [29250004, 1344499896000],
    [29260003, 1344522133000],
    [29270000, 1344558985000],
    [29280000, 1344597380000],
    [29290000, 1344617865000],
    [29300000, 1344672749000],
    [29310000, 1344697892000],
    [29320002, 1344750005000],
    [29330001, 1344778553000],
    [29340000, 1344819126000],
    [29350002, 1344857294000],
    [29360000, 1344877526000],
    [29370005, 1344931746000],
    [29380004, 1344955704000],
    [29390002, 1344999794000],
    [29400003, 1345031254000],
    [29410001, 1345048813000],
    [29420002, 1345102616000],
    [29430001, 1345126849000],
    [29440002, 1345169851000],
    [29450000, 1345205034000],
    [29460000, 1345224004000],
    [29470001, 1345275931000],
    [29480001, 1345300341000],
    [29490003, 1345342277000],
    [29500000, 1345374875000],
    [29510000, 1345392844000],
    [29520002, 1345449113000],
    [29530000, 1345473512000],
    [29540001, 1345521340000],
    [29550002, 1345553473000],
    [29560000, 1345576673000],
    [29570000, 1345631832000],
    [29580000, 1345652011000],
    [29590009, 1345707406000],
    [29600004, 1345731272000],
    [29610003, 1345774886000],
    [29620005, 1345810760000],
    [29630001, 1345831478000],
    [29640002, 1345883956000],
    [29650000, 1345907135000],
    [29660000, 1345952305000],
    [29670001, 1345981178000],
    [29680000, 1345998619000],
    [29690000, 1346056048000],
    [29700001, 1346079034000],
    [29710002, 1346128089000],
    [29720003, 1346159030000],
    [29730009, 1346185433000],
    [29740005, 1346238445000],
    [29750001, 1346257670000],
    [29760000, 1346316933000],
    [29770000, 1346338800000],
    [29780005, 1346374925000],
    [29790011, 1346409902000],
    [29800001, 1346425111000],
    [29810000, 1346463735000],
    [29820002, 1346496963000],
    [29830001, 1346515159000],
    [29840003, 1346562643000],
    [29850001, 1346587804000],
    [29860004, 1346605220000],
    [29870002, 1346668826000],
    [29880000, 1346688952000],
    [29890004, 1346755226000],
    [29900002, 1346777475000],
    [29910000, 1346843623000],
    [29920003, 1346865832000],
    [29930005, 1346931902000],
    [29940001, 1346953696000],
    [29950000, 1347018301000],
    [29960000, 1347039567000],
    [29970000, 1347092333000],
    [29980002, 1347115246000],
    [29990001, 1347153602000],
    [30000002, 1347183917000],
    [30010000, 1347201084000],
    [30020000, 1347249061000],
    [30030003, 1347284093000],
    [30040001, 1347322819000],
    [30050000, 1347367541000],
    [30060000, 1347391244000],
    [30070002, 1347452321000],
    [30080000, 1347475900000],
    [30090000, 1347537862000],
    [30100002, 1347559116000],
    [30110002, 1347623392000],
    [30120002, 1347642254000],
    [30130001, 1347695632000],
    [30140002, 1347719027000],
    [30150005, 1347760228000],
    [30160005, 1347792525000],
    [30170000, 1347810529000],
    [30180003, 1347858045000],
    [30190000, 1347882687000],
    [30200005, 1347898127000],
    [30210001, 1347961234000],
    [30220000, 1347982321000],
    [30230000, 1348046296000],
    [30240009, 1348068337000],
    [30250002, 1348133170000],
    [30260000, 1348155473000],
    [30270002, 1348221215000],
    [30280000, 1348242526000],
    [30290000, 1348295069000],
    [30300000, 1348320609000],
    [30310002, 1348344515000],
    [30320004, 1348388630000],
    [30330002, 1348407709000],
    [30340000, 1348445478000],
    [30350001, 1348491432000],
    [30360002, 1348526881000],
    [30370003, 1348578382000],
    [30380006, 1348619988000],
    [30390004, 1348666157000],
    [30400001, 1348717374000],
    [30410000, 1348754288000],
    [30420001, 1348810599000],
    [30430000, 1348842518000],
    [30440001, 1348887881000],
    [30450004, 1348920258000],
    [30460001, 1348937804000],
    [30470006, 1348984917000],
    [30480001, 1349005416000],
    [30490000, 1349017720000],
    [30500001, 1349067895000],
    [30510001, 1349098051000],
    [30520000, 1349128966000],
    [30530001, 1349180813000],
    [30540000, 1349200374000],
    [30550001, 1349262605000],
    [30560004, 1349279737000],
    [30570000, 1349341873000],
    [30580001, 1349362683000],
    [30590004, 1349418047000],
    [30600003, 1349446021000],
    [30610002, 1349478102000],
    [30620000, 1349516772000],
    [30630002, 1349536086000],
    [30640001, 1349582802000],
    [30650005, 1349612047000],
    [30660000, 1349630540000],
    [30670000, 1349681778000],
    [30680001, 1349702931000],
    [30690005, 1349729020000],
    [30700000, 1349784465000],
    [30710000, 1349800542000],
    [30720005, 1349864590000],
    [30730002, 1349883197000],
    [30740007, 1349948520000],
    [30750000, 1349970840000],
    [30760001, 1350037993000],
    [30770000, 1350059990000],
    [30780006, 1350114426000],
    [30790001, 1350137646000],
    [30800000, 1350176862000],
    [30810002, 1350207937000],
    [30820001, 1350225907000],
    [30830004, 1350283400000],
    [30840000, 1350311590000],
    [30850000, 1350373544000],
    [30860000, 1350400104000],
    [30870000, 1350464861000],
    [30880001, 1350488021000],
    [30890002, 1350554413000],
    [30900000, 1350576170000],
    [30910002, 1350641788000],
    [30920001, 1350662648000],
    [30930000, 1350716905000],
    [30940004, 1350741541000],
    [30950000, 1350779641000],
    [30960001, 1350813264000],
    [30970000, 1350830966000],
    [30980004, 1350891403000],
    [30990001, 1350916993000],
    [31000000, 1350976045000],
    [31010001, 1351003096000],
    [31020001, 1351065860000],
    [31030002, 1351091655000],
    [31040002, 1351158698000],
    [31050001, 1351181008000],
    [31060001, 1351248541000],
    [31070000, 1351270528000],
    [31080000, 1351326009000],
    [31090001, 1351348327000],
    [31100000, 1351391151000],
    [31110001, 1351419833000],
    [31120000, 1351436406000],
    [31130002, 1351494007000],
    [31140000, 1351520318000],
    [31150001, 1351571494000],
    [31160000, 1351603993000],
    [31170001, 1351616879000],
    [31180001, 1351672701000],
    [31190004, 1351688915000],
    [31200002, 1351697284000],
    [31210001, 1351755761000],
    [31220000, 1351781594000],
    [31230001, 1351844251000],
    [31240000, 1351868361000],
    [31250000, 1351915604000],
    [31260002, 1351945557000],
    [31270000, 1351963493000],
    [31280001, 1352014755000],
    [31290000, 1352035437000],
    [31300001, 1352071439000],
    [31310002, 1352119911000],
    [31320000, 1352157772000],
    [31330000, 1352207601000],
    [31340000, 1352252867000],
    [31350001, 1352295460000],
    [31360003, 1352343034000],
    [31370001, 1352382698000],
    [31380000, 1352436799000],
    [31390001, 1352472324000],
    [31400000, 1352520774000],
    [31410000, 1352551559000],
    [31420003, 1352568588000],
    [31430001, 1352613659000],
    [31440001, 1352633614000],
    [31450005, 1352645308000],
    [31460008, 1352691678000],
    [31470001, 1352727836000],
    [31480000, 1352780469000],
    [31490000, 1352816844000],
    [31500003, 1352880609000],
    [31510001, 1352907694000],
    [31520001, 1352977785000],
    [31530002, 1353005580000],
    [31540000, 1353072123000],
    [31550001, 1353121500000],
    [31560003, 1353155121000],
    [31570001, 1353175532000],
    [31580000, 1353226769000],
    [31590001, 1353248098000],
    [31600002, 1353306006000],
    [31610002, 1353337613000],
    [31620001, 1353405329000],
    [31630003, 1353426802000],
    [31640003, 1353492674000],
    [31650001, 1353513109000],
    [31660002, 1353579990000],
    [31670000, 1353600377000],
    [31680001, 1353655160000],
    [31690000, 1353680325000],
    [31700000, 1353724383000],
    [31710001, 1353759163000],
    [31720002, 1353779334000],
    [31730001, 1353830614000],
    [31740000, 1353852339000],
    [31750001, 1353907684000],
    [31760001, 1353941076000],
    [31770000, 1354006974000],
    [31780001, 1354032634000],
    [31790002, 1354102566000],
    [31800000, 1354126363000],
    [31810000, 1354193232000],
    [31820003, 1354241685000],
    [31830000, 1354284395000],
    [31840009, 1354331929000],
    [31850000, 1354364870000],
    [31860001, 1354387817000],
    [31870001, 1354438127000],
    [31880001, 1354459257000],
    [31890002, 1354519928000],
    [31900002, 1354547493000],
    [31910000, 1354615038000],
    [31920001, 1354639209000],
    [31930000, 1354708282000],
    [31940000, 1354745934000],
    [31950001, 1354800989000],
    [31960001, 1354860621000],
    [31970001, 1354892723000],
    [31980000, 1354946804000],
    [31990000, 1354973952000],
    [32000000, 1355011859000],
    [32010004, 1355047009000],
    [32020000, 1355065908000],
    [32030000, 1355130015000],
    [32040001, 1355154706000],
    [32050002, 1355224028000],
    [32060000, 1355249813000],
    [32070000, 1355315780000],
    [32080000, 1355362193000],
    [32090000, 1355407121000],
    [32100000, 1355469050000],
    [32110000, 1355498162000],
    [32120002, 1355553225000],
    [32130000, 1355579716000],
    [32140000, 1355619367000],
    [32150000, 1355653645000],
    [32160000, 1355672606000],
    [32170000, 1355737957000],
    [32180000, 1355760599000],
    [32190000, 1355826598000],
    [32200000, 1355849812000],
    [32210003, 1355916669000],
    [32220001, 1355935338000],
    [32230000, 1355999504000],
    [32240000, 1356019847000],
    [32250000, 1356084601000],
    [32260002, 1356106676000],
    [32270005, 1356162133000],
    [32280001, 1356186797000],
    [32290004, 1356230230000],
    [32300001, 1356262480000],
    [32310002, 1356278457000],
    [32320003, 1356322817000],
    [32330004, 1356347954000],
    [32340000, 1356361566000],
    [32350000, 1356398407000],
    [32360000, 1356432951000],
    [32370002, 1356448195000],
    [32380000, 1356496994000],
    [32390009, 1356528420000],
    [32400001, 1356560459000],
    [32410001, 1356606089000],
    [32420003, 1356625487000],
    [32430005, 1356679025000],
    [32440001, 1356704600000],
    [32450002, 1356747556000],
    [32460001, 1356782493000],
    [32470002, 1356804577000],
    [32480005, 1356857553000],
    [32490000, 1356879703000],
    [32500000, 1356921717000],
    [32510002, 1356949912000],
    [32520001, 1356965991000],
    [32530000, 1356971788000],
    [32540000, 1357005717000],
    [32550001, 1357028963000],
    [32560008, 1357047482000],
    [32570000, 1357065837000],
    [32580000, 1357116945000],
    [32590002, 1357138715000],
    [32600001, 1357183055000],
    [32610000, 1357212910000],
    [32620006, 1357230509000],
    [32630001, 1357284201000],
    [32640000, 1357308794000],
    [32650010, 1357346583000],
    [32660000, 1357382482000],
    [32670004, 1357401362000],
    [32680001, 1357451550000],
    [32690000, 1357476017000],
    [32700000, 1357494075000],
    [32710003, 1357552561000],
    [32720000, 1357573049000],
    [32730002, 1357636115000],
    [32740001, 1357658985000],
    [32750000, 1357722615000],
    [32760001, 1357746111000],
    [32770002, 1357812581000],
    [32780001, 1357835793000],
    [32790003, 1357903657000],
    [32800002, 1357926407000],
    [32810000, 1357982969000],
    [32820005, 1358004671000],
    [32830003, 1358054726000],
    [32840001, 1358080998000],
    [32850000, 1358100823000],
    [32860001, 1358149594000],
    [32870001, 1358169325000],
    [32880003, 1358199284000],
    [32890000, 1358255481000],
    [32900000, 1358306245000],
    [32910001, 1358345546000],
    [32920000, 1358407151000],
    [32930000, 1358436073000],
    [32940001, 1358502920000],
    [32950000, 1358526289000],
    [32960001, 1358582688000],
    [32970001, 1358606161000],
    [32980002, 1358649874000],
    [32990001, 1358678827000],
    [33000005, 1358695809000],
    [33010004, 1358759419000],
    [33020001, 1358782613000],
    [33030002, 1358847606000],
    [33040001, 1358870521000],
    [33050001, 1358936101000],
    [33060000, 1358958980000],
    [33070001, 1359025425000],
    [33080001, 1359048058000],
    [33090003, 1359113713000],
    [33100002, 1359136635000],
    [33110001, 1359190541000],
    [33120000, 1359212401000],
    [33130002, 1359257152000],
    [33140001, 1359285220000],
    [33150003, 1359301502000],
    [33160000, 1359364171000],
    [33170002, 1359386594000],
    [33180003, 1359450774000],
    [33190001, 1359473743000],
    [33200001, 1359535857000],
    [33210003, 1359558011000],
    [33220000, 1359608548000],
    [33230000, 1359638083000],
    [33240001, 1359659753000],
    [33250001, 1359717631000],
    [33260000, 1359736486000],
    [33270000, 1359787846000],
    [33280000, 1359810823000],
    [33290005, 1359828486000],
    [33300002, 1359875996000],
    [33310004, 1359895908000],
    [33320002, 1359912237000],
    [33330000, 1359972666000],
    [33340003, 1359991723000],
    [33350000, 1360051414000],
    [33360000, 1360075785000],
    [33370000, 1360129999000],
    [33380004, 1360158180000],
    [33390002, 1360202106000],
    [33400000, 1360240975000],
    [33410000, 1360266923000],
    [33420002, 1360324787000],
    [33430000, 1360345066000],
    [33440001, 1360398665000],
    [33450000, 1360420991000],
    [33460000, 1360463006000],
    [33470000, 1360495216000],
    [33480000, 1360512536000],
    [33490000, 1360562472000],
    [33500002, 1360586674000],
    [33510002, 1360607159000],
    [33520004, 1360668303000],
    [33530001, 1360689941000],
    [33540000, 1360752747000],
    [33550000, 1360769416000],
    [33560000, 1360811670000],
    [33570000, 1360840391000],
    [33580000, 1360852612000],
    [33590000, 1360879818000],
    [33600000, 1360929139000],
    [33610001, 1360947954000],
    [33620000, 1361001599000],
    [33630002, 1361024996000],
    [33640001, 1361066180000],
    [33650000, 1361096699000],
    [33660002, 1361114495000],
    [33670001, 1361172685000],
    [33680000, 1361198026000],
    [33690004, 1361249539000],
    [33700001, 1361281162000],
    [33710000, 1361325351000],
    [33720001, 1361365549000],
    [33730000, 1361406340000],
    [33740003, 1361451235000],
    [33750002, 1361481535000],
    [33760003, 1361531344000],
    [33770001, 1361546410000],
    [33780000, 1361594376000],
    [33790003, 1361623143000],
    [33800001, 1361643129000],
    [33810002, 1361692896000],
    [33820003, 1361714116000],
    [33830002, 1361759930000],
    [33840000, 1361798429000],
    [33850001, 1361848052000],
    [33860000, 1361885684000],
    [33870001, 1361934021000],
    [33880000, 1361970745000],
    [33890001, 1362013181000],
    [33900001, 1362056052000],
    [33910000, 1362083062000],
    [33920001, 1362138442000],
    [33930000, 1362158543000],
    [33940002, 1362211924000],
    [33950000, 1362233946000],
    [33960003, 1362268771000],
    [33970000, 1362302007000],
    [33980005, 1362320066000],
    [33990000, 1362368927000],
    [34000001, 1362404229000],
    [34010001, 1362451090000],
    [34020001, 1362489362000],
    [34030001, 1362533721000],
    [34040000, 1362573912000],
    [34050000, 1362608923000],
    [34060000, 1362657771000],
    [34070000, 1362680849000],
    [34080000, 1362740206000],
    [34090001, 1362758897000],
    [34100000, 1362807733000],
    [34110001, 1362831210000],
    [34120000, 1362843477000],
    [34130001, 1362883265000],
    [34140000, 1362908291000],
    [34150000, 1362925095000],
    [34160000, 1362966745000],
    [34170000, 1363003879000],
    [34180000, 1363022619000],
    [34190001, 1363082209000],
    [34200003, 1363102242000],
    [34210001, 1363158027000],
    [34220006, 1363183022000],
    [34230000, 1363214377000],
    [34240000, 1363259970000],
    [34250003, 1363275997000],
    [34260003, 1363332164000],
    [34270001, 1363356684000],
    [34280000, 1363392926000],
    [34290000, 1363426876000],
    [34300000, 1363446114000],
    [34310004, 1363489717000],
    [34320004, 1363517532000],
    [34330000, 1363533647000],
    [34340000, 1363588591000],
    [34350001, 1363613184000],
    [34360004, 1363646755000],
    [34370000, 1363693573000],
    [34380000, 1363712786000],
    [34390000, 1363764801000],
    [34400004, 1363787051000],
    [34410006, 1363823001000],
    [34420000, 1363866102000],
    [34430000, 1363886655000],
    [34440002, 1363945291000],
    [34450006, 1363966020000],
    [34460000, 1364014561000],
    [34470001, 1364041700000],
    [34480001, 1364060263000],
    [34490002, 1364110667000],
    [34500002, 1364132150000],
    [34510000, 1364158715000],
    [34520002, 1364207776000],
    [34530000, 1364227606000],
    [34540000, 1364282648000],
    [34550001, 1364307643000],
    [34560005, 1364354029000],
    [34570000, 1364386780000],
    [34580001, 1364406700000],
    [34590000, 1364460535000],
    [34600002, 1364482170000],
    [34610000, 1364530121000],
    [34620000, 1364560514000],
    [34630002, 1364580436000],
    [34640003, 1364631885000],
    [34650002, 1364654125000],
    [34660000, 1364695210000],
    [34670001, 1364724468000],
    [34680000, 1364741581000],
    [34690004, 1364774365000],
    [34700001, 1364812449000],
    [34710000, 1364829362000],
    [34720004, 1364879934000],
    [34730002, 1364907299000],
    [34740001, 1364927146000],
    [34750001, 1364980183000],
    [34760003, 1365000659000],
    [34770001, 1365045313000],
    [34780000, 1365076504000],
    [34790003, 1365094373000],
    [34800002, 1365149517000],
    [34810000, 1365172056000],
    [34820000, 1365213235000],
    [34830004, 1365242665000],
    [34840000, 1365259978000],
    [34850000, 1365298274000],
    [34860003, 1365327124000],
    [34870003, 1365345164000],
    [34880002, 1365396764000],
    [34890001, 1365429032000],
    [34900003, 1365480402000],
    [34910002, 1365515898000],
    [34920000, 1365571064000],
    [34930001, 1365604805000],
    [34940003, 1365667660000],
    [34950000, 1365694831000],
    [34960000, 1365763981000],
    [34970002, 1365788159000],
    [34980003, 1365844846000],
    [34990002, 1365866098000],
    [35000000, 1365912680000],
    [35010000, 1365938418000],
    [35020009, 1365954749000],
    [35030002, 1366020003000],
    [35040003, 1366041754000],
    [35050001, 1366111126000],
    [35060002, 1366136702000],
    [35070001, 1366202459000],
    [35080006, 1366248627000],
    [35090000, 1366297213000],
    [35100000, 1366362163000],
    [35110001, 1366386116000],
    [35120000, 1366440216000],
    [35130002, 1366464803000],
    [35140002, 1366496101000],
    [35150001, 1366533569000],
    [35160002, 1366552354000],
    [35170000, 1366599837000],
    [35180003, 1366638967000],
    [35190001, 1366696124000],
    [35200005, 1366727915000],
    [35210001, 1366789728000],
    [35220000, 1366814996000],
    [35230001, 1366877875000],
    [35240000, 1366902753000],
    [35250004, 1366967102000],
    [35260002, 1366989933000],
    [35270003, 1367042683000],
    [35280000, 1367069376000],
    [35290001, 1367097857000],
    [35300001, 1367140787000],
    [35310002, 1367160512000],
    [35320000, 1367203401000],
    [35330000, 1367231060000],
    [35340004, 1367247611000],
    [35350000, 1367298029000],
    [35360000, 1367326900000],
    [35370000, 1367340896000],
    [35380001, 1367401377000],
    [35390000, 1367420920000],
    [35400001, 1367474131000],
    [35410001, 1367503508000],
    [35420002, 1367547259000],
    [35430003, 1367582146000],
    [35440000, 1367600557000],
    [35450000, 1367653215000],
    [35460003, 1367676318000],
    [35470003, 1367702380000],
    [35480002, 1367743839000],
    [35490003, 1367763454000],
    [35500002, 1367799672000],
    [35510000, 1367831949000],
    [35520007, 1367849485000],
    [35530000, 1367897825000],
    [35540001, 1367935595000],
    [35550001, 1367994106000],
    [35560001, 1368024203000],
    [35570006, 1368086266000],
    [35580000, 1368111893000],
    [35590000, 1368173676000],
    [35600001, 1368198005000],
    [35610001, 1368243823000],
    [35620002, 1368272106000],
    [35630000, 1368288754000],
    [35640000, 1368334385000],
    [35650002, 1368358082000],
    [35660002, 1368372869000],
    [35670000, 1368435347000],
    [35680000, 1368457871000],
    [35690001, 1368520421000],
    [35700000, 1368544432000],
    [35710002, 1368609152000],
    [35720000, 1368632844000],
    [35730001, 1368699096000],
    [35740000, 1368720560000],
    [35750002, 1368788434000],
    [35760000, 1368809626000],
    [35770004, 1368864506000],
    [35780000, 1368887525000],
    [35790002, 1368928451000],
    [35800006, 1368957247000],
    [35810000, 1368974492000],
    [35820000, 1369027081000],
    [35830000, 1369058682000],
    [35840001, 1369108781000],
    [35850000, 1369144659000],
    [35860000, 1369196731000],
    [35870000, 1369231420000],
    [35880000, 1369284640000],
    [35890000, 1369317701000],
    [35900000, 1369365115000],
    [35910000, 1369403469000],
    [35920002, 1369445821000],
    [35930000, 1369480312000],
    [35940001, 1369498786000],
    [35950000, 1369547395000],
    [35960005, 1369570839000],
    [35970003, 1369588480000],
    [35980000, 1369654650000],
    [35990004, 1369677301000],
    [36000003, 1369744096000],
    [36010001, 1369773124000],
    [36020000, 1369832322000],
    [36030000, 1369892728000],
    [36040000, 1369923001000],
    [36050003, 1369976757000],
    [36060001, 1370008172000],
    [36070000, 1370034071000],
    [36080002, 1370078311000],
    [36090004, 1370097755000],
    [36100000, 1370137880000],
    [36110001, 1370165367000],
    [36120000, 1370182475000],
    [36130002, 1370227262000],
    [36140000, 1370266045000],
    [36150000, 1370305401000],
    [36160001, 1370352160000],
    [36170004, 1370397133000],
    [36180000, 1370439727000],
    [36190003, 1370485761000],
    [36200001, 1370525228000],
    [36210000, 1370569914000],
    [36220004, 1370611516000],
    [36230009, 1370648909000],
    [36240001, 1370686818000],
    [36250000, 1370704462000],
    [36260003, 1370745990000],
    [36270000, 1370773068000],
    [36280001, 1370789150000],
    [36290000, 1370838426000],
    [36300001, 1370871222000],
    [36310000, 1370913442000],
    [36320001, 1370956581000],
    [36330002, 1370997170000],
    [36340001, 1371041329000],
    [36350000, 1371066729000],
    [36360003, 1371126696000],
    [36370000, 1371154621000],
    [36380001, 1371213332000],
    [36390002, 1371236016000],
    [36400000, 1371287069000],
    [36410000, 1371308204000],
    [36420000, 1371349710000],
    [36430002, 1371378306000],
    [36440000, 1371394756000],
    [36450001, 1371450733000],
    [36460000, 1371479351000],
    [36470001, 1371530182000],
    [36480000, 1371563379000],
    [36490002, 1371606919000],
    [36500002, 1371647601000],
    [36510001, 1371688831000],
    [36520002, 1371733114000],
    [36530000, 1371767609000],
    [36540000, 1371818399000],
    [36550001, 1371842736000],
    [36560002, 1371892516000],
    [36570002, 1371913541000],
    [36580000, 1371957173000],
    [36590006, 1371984267000],
    [36600001, 1372000119000],
    [36610000, 1372059277000],
    [36620000, 1372085763000],
    [36630000, 1372147271000],
    [36640002, 1372172906000],
    [36650000, 1372233724000],
    [36660002, 1372258397000],
    [36670003, 1372316050000],
    [36680000, 1372344310000],
    [36690003, 1372403092000],
    [36700000, 1372430704000],
    [36710003, 1372478206000],
    [36720001, 1372508088000],
    [36730001, 1372526097000],
    [36740003, 1372574807000],
    [36750000, 1372597137000],
    [36760000, 1372615663000],
    [36770000, 1372678627000],
    [36780004, 1372700829000],
    [36790002, 1372766402000],
    [36800001, 1372792926000],
    [36810000, 1372852918000],
    [36820003, 1372874800000],
    [36830003, 1372937528000],
    [36840000, 1372958129000],
    [36850001, 1373022700000],
    [36860000, 1373043138000],
    [36870000, 1373097021000],
    [36880006, 1373120418000],
    [36890005, 1373144993000],
    [36900000, 1373183895000],
    [36910000, 1373202657000],
    [36920000, 1373218741000],
    [36930000, 1373281954000],
    [36940000, 1373301604000],
    [36950000, 1373367314000],
    [36960000, 1373388217000],
    [36970005, 1373453035000],
    [36980000, 1373474500000],
    [36990000, 1373540055000],
    [37000001, 1373563115000],
    [37010000, 1373628900000],
    [37020000, 1373650645000],
    [37030002, 1373704437000],
    [37040000, 1373727114000],
    [37050006, 1373771545000],
    [37060005, 1373802717000],
    [37070000, 1373821369000],
    [37080001, 1373870908000],
    [37090000, 1373892934000],
    [37100000, 1373911492000],
    [37110003, 1373974641000],
    [37120000, 1373997077000],
    [37130001, 1374061966000],
    [37140002, 1374087619000],
    [37150001, 1374148800000],
    [37160008, 1374174217000],
    [37170000, 1374234991000],
    [37180001, 1374255688000],
    [37190001, 1374309357000],
    [37200002, 1374331853000],
    [37210001, 1374375180000],
    [37220000, 1374404348000],
    [37230002, 1374420626000],
    [37240000, 1374477526000],
    [37250000, 1374502909000],
    [37260001, 1374552567000],
    [37270000, 1374585225000],
    [37280003, 1374626401000],
    [37290000, 1374667952000],
    [37300001, 1374693759000],
    [37310001, 1374750480000],
    [37320000, 1374772297000],
    [37330002, 1374831692000],
    [37340000, 1374854036000],
    [37350001, 1374908418000],
    [37360004, 1374935059000],
    [37370001, 1374979887000],
    [37380000, 1375011527000],
    [37390002, 1375029597000],
    [37400002, 1375087939000],
    [37410000, 1375109998000],
    [37420001, 1375161830000],
    [37430001, 1375191923000],
    [37440000, 1375232946000],
    [37450000, 1375272094000],
    [37460001, 1375287532000],
    [37470000, 1375341379000],
    [37480000, 1375365927000],
    [37490000, 1375404444000],
    [37500003, 1375443179000],
    [37510001, 1375461256000],
    [37520002, 1375513602000],
    [37530002, 1375538515000],
    [37540002, 1375575106000],
    [37550004, 1375610354000],
    [37560000, 1375628282000],
    [37570002, 1375675631000],
    [37580000, 1375707251000],
    [37590001, 1375727391000],
    [37600001, 1375781193000],
    [37610002, 1375801243000],
    [37620001, 1375848211000],
    [37630000, 1375879256000],
    [37640005, 1375896679000],
    [37650000, 1375950116000],
    [37660002, 1375972012000],
    [37670001, 1376008998000],
    [37680001, 1376045664000],
    [37690006, 1376064238000],
    [37700002, 1376117367000],
    [37710001, 1376144696000],
    [37720005, 1376191150000],
    [37730000, 1376223380000],
    [37740001, 1376249258000],
    [37750001, 1376302974000],
    [37760000, 1376324962000],
    [37770003, 1376379604000],
    [37780001, 1376404591000],
    [37790001, 1376451395000],
    [37800001, 1376483821000],
    [37810000, 1376505856000],
    [37820002, 1376557229000],
    [37830000, 1376578917000],
    [37840000, 1376625963000],
    [37850001, 1376656561000],
    [37860001, 1376677251000],
    [37870002, 1376728731000],
    [37880000, 1376751974000],
    [37890004, 1376800729000],
    [37900000, 1376829027000],
    [37910000, 1376849565000],
    [37920000, 1376905760000],
    [37930000, 1376925891000],
    [37940000, 1376979900000],
    [37950000, 1377007345000],
    [37960000, 1377052827000],
    [37970001, 1377088208000],
    [37980000, 1377110956000],
    [37990006, 1377166181000],
    [38000001, 1377186057000],
    [38010000, 1377239944000],
    [38020002, 1377266798000],
    [38030000, 1377307287000],
    [38040001, 1377341505000],
    [38050003, 1377361200000],
    [38060000, 1377412670000],
    [38070005, 1377437016000],
    [38080003, 1377474053000],
    [38090003, 1377518447000],
    [38100002, 1377540322000],
    [38110001, 1377600153000],
    [38120001, 1377621114000],
    [38130000, 1377681632000],
    [38140001, 1377704485000],
    [38150000, 1377763375000],
    [38160003, 1377788184000],
    [38170002, 1377842353000],
    [38180005, 1377871027000],
    [38190000, 1377900233000],
    [38200002, 1377939649000],
    [38210001, 1377958601000],
    [38220003, 1377986125000],
    [38230001, 1378025318000],
    [38240002, 1378044932000],
    [38250000, 1378095298000],
    [38260000, 1378129838000],
    [38270000, 1378181543000],
    [38280001, 1378217043000],
    [38290000, 1378270350000],
    [38300000, 1378303716000],
    [38310001, 1378360455000],
    [38320001, 1378392908000],
    [38330000, 1378456687000],
    [38340001, 1378481391000],
    [38350000, 1378535921000],
    [38360001, 1378562601000],
    [38370002, 1378601699000],
    [38380000, 1378634402000],
    [38390000, 1378652499000],
    [38400000, 1378710554000],
    [38410000, 1378737840000],
    [38420003, 1378797520000],
    [38430001, 1378825252000],
    [38440001, 1378888982000],
    [38450002, 1378913608000],
    [38460000, 1378979577000],
    [38470000, 1379002251000],
    [38480001, 1379069153000],
    [38490000, 1379090230000],
    [38500000, 1379146714000],
    [38510001, 1379170837000],
    [38520001, 1379219093000],
    [38530001, 1379246161000],
    [38540000, 1379264109000],
    [38550000, 1379312723000],
    [38560000, 1379335204000],
    [38570000, 1379356798000],
    [38580001, 1379421630000],
    [38590000, 1379465000000],
    [38600002, 1379512079000],
    [38610000, 1379566247000],
    [38620004, 1379600141000],
    [38630002, 1379656975000],
    [38640002, 1379688418000],
    [38650003, 1379740660000],
    [38660001, 1379770479000],
    [38670001, 1379815814000],
    [38680004, 1379850237000],
    [38690001, 1379870995000],
    [38700002, 1379925107000],
    [38710000, 1379947170000],
    [38720001, 1380011108000],
    [38730002, 1380036696000],
    [38740000, 1380106543000],
    [38750001, 1380137694000],
    [38760000, 1380202580000],
    [38770001, 1380268313000],
    [38780000, 1380297975000],
    [38790001, 1380358738000],
    [38800000, 1380382146000],
    [38810000, 1380435524000],
    [38820001, 1380460498000],
    [38830002, 1380504622000],
    [38840001, 1380548581000],
    [38850003, 1380599636000],
    [38860001, 1380636128000],
    [38870000, 1380695813000],
    [38880000, 1380726816000],
    [38890000, 1380793282000],
    [38900002, 1380817153000],
    [38910005, 1380886252000],
    [38920002, 1380911028000],
    [38930001, 1380966632000],
    [38940004, 1380987605000],
    [38950000, 1381036757000],
    [38960001, 1381062521000],
    [38970001, 1381083578000],
    [38980000, 1381147999000],
    [38990002, 1381183596000],
    [39000000, 1381238718000],
    [39010000, 1381289418000],
    [39020001, 1381327969000],
    [39030001, 1381378325000],
    [39040001, 1381414484000],
    [39050002, 1381473072000],
    [39060001, 1381503610000],
    [39070000, 1381557108000],
    [39080001, 1381587652000],
    [39090002, 1381638859000],
    [39100005, 1381670136000],
    [39110001, 1381713463000],
    [39120002, 1381750602000],
    [39130000, 1381771266000],
    [39140000, 1381840441000],
    [39150004, 1381886453000],
    [39160000, 1381928937000],
    [39170001, 1381987485000],
    [39180001, 1382023386000],
    [39190000, 1382094081000],
    [39200000, 1382118886000],
    [39210002, 1382176669000],
    [39220000, 1382198409000],
    [39230000, 1382251062000],
    [39240000, 1382275083000],
    [39250000, 1382319937000],
    [39260000, 1382363621000],
    [39270003, 1382424605000],
    [39280000, 1382454868000],
    [39290002, 1382522396000],
    [39300000, 1382545319000],
    [39310000, 1382614356000],
    [39320001, 1382636773000],
    [39330000, 1382702066000],
    [39340001, 1382723317000],
    [39350001, 1382777459000],
    [39360000, 1382800070000],
    [39370003, 1382851982000],
    [39380000, 1382879757000],
    [39390004, 1382933535000],
    [39400001, 1382971415000],
    [39410000, 1383039852000],
    [39420000, 1383064852000],
    [39430003, 1383134247000],
    [39440000, 1383150130000],
    [39450003, 1383209088000],
    [39460004, 1383226803000],
    [39470001, 1383239461000],
    [39480002, 1383305804000],
    [39490000, 1383330874000],
    [39500003, 1383389307000],
    [39510002, 1383410824000],
    [39520000, 1383465663000],
    [39530001, 1383488952000],
    [39540004, 1383536354000],
    [39550001, 1383566404000],
    [39560000, 1383587943000],
    [39570001, 1383655387000],
    [39580000, 1383709273000],
    [39590001, 1383749270000],
    [39600000, 1383817042000],
    [39610004, 1383844566000],
    [39620000, 1383914206000],
    [39630002, 1383958908000],
    [39640009, 1383999261000],
    [39650000, 1384025481000],
    [39660000, 1384074693000],
    [39670001, 1384094814000],
    [39680002, 1384144502000],
    [39690000, 1384176297000],
    [39700000, 1384206383000],
    [39710000, 1384264473000],
    [39720000, 1384332176000],
    [39730000, 1384362320000],
    [39740000, 1384434643000],
    [39750002, 1384497066000],
    [39760001, 1384530859000],
    [39770001, 1384591769000],
    [39780000, 1384616741000],
    [39790005, 1384670466000],
    [39800001, 1384695642000],
    [39810003, 1384750781000],
    [39820000, 1384787697000],
    [39830002, 1384861205000],
    [39840001, 1384917103000],
    [39850000, 1384959607000],
    [39860000, 1385027424000],
    [39870001, 1385053575000],
    [39880001, 1385122298000],
    [39890001, 1385152257000],
    [39900001, 1385205088000],
    [39910001, 1385227283000],
    [39920001, 1385282720000],
    [39930000, 1385304834000],
    [39940006, 1385370821000],
    [39950002, 1385397805000],
    [39960001, 1385469875000],
    [39970002, 1385532270000],
    [39980002, 1385566841000],
    [39990003, 1385639730000],
    [40000001, 1385692970000],
    [40010000, 1385736566000],
    [40020004, 1385795632000],
    [40030000, 1385823366000],
    [40040004, 1385872369000],
    [40050002, 1385901651000],
    [40060000, 1385946562000],
    [40070001, 1385995108000],
    [40080000, 1386063717000],
    [40090003, 1386093291000],
    [40100004, 1386163639000],
    [40110001, 1386226926000],
    [40120001, 1386258746000],
    [40130000, 1386331000000],
    [40140001, 1386375148000],
    [40150000, 1386418743000],
    [40160001, 1386444120000],
    [40170000, 1386494429000],
    [40180000, 1386515214000],
    [40190000, 1386583554000],
    [40200002, 1386610726000],
    [40210001, 1386680661000],
    [40220000, 1386740668000],
    [40230000, 1386775701000],
    [40240001, 1386847675000],
    [40250002, 1386895894000],
    [40260002, 1386944786000],
    [40270006, 1387000670000],
    [40280000, 1387031121000],
    [40290000, 1387082024000],
    [40300001, 1387111458000],
    [40310000, 1387155881000],
    [40320002, 1387203263000],
    [40330000, 1387269619000],
    [40340001, 1387298063000],
    [40350000, 1387367915000],
    [40360000, 1387414556000],
    [40370000, 1387461528000],
    [40380000, 1387517028000],
    [40390000, 1387551023000],
    [40400002, 1387605448000],
    [40410001, 1387635538000],
    [40420002, 1387682554000],
    [40430001, 1387715737000],
    [40440001, 1387743208000],
    [40450000, 1387793441000],
    [40460000, 1387812170000],
    [40470002, 1387863887000],
    [40480000, 1387891948000],
    [40490001, 1387906185000],
    [40500004, 1387960133000],
    [40510000, 1387980328000],
    [40520000, 1388012402000],
    [40530000, 1388059806000],
    [40540002, 1388080796000],
    [40550001, 1388139588000],
    [40560000, 1388160731000],
    [40570001, 1388213456000],
    [40580001, 1388240301000],
    [40590000, 1388286861000],
    [40600001, 1388321884000],
    [40610002, 1388366148000],
    [40620000, 1388406437000],
    [40630001, 1388432055000],
    [40640003, 1388479513000],
    [40650000, 1388500080000],
    [40660000, 1388506206000],
    [40670000, 1388541772000],
    [40680000, 1388566722000],
    [40690000, 1388586411000],
    [40700002, 1388631705000],
    [40710000, 1388666584000],
    [40720000, 1388697954000],
    [40730004, 1388746402000],
    [40740002, 1388767513000],
    [40750000, 1388821955000],
    [40760000, 1388845971000],
    [40770000, 1388891736000],
    [40780002, 1388920808000],
    [40790000, 1388938832000],
    [40800000, 1388999657000],
    [40810001, 1389023069000],
    [40820000, 1389086558000],
    [40830004, 1389111542000],
    [40840002, 1389179422000],
    [40850001, 1389204596000],
    [40860002, 1389271476000],
    [40870001, 1389314701000],
    [40880001, 1389362164000],
    [40890005, 1389408217000],
    [40900003, 1389443723000],
    [40910000, 1389466321000],
    [40920000, 1389520043000],
    [40930000, 1389541663000],
    [40940001, 1389595713000],
    [40950000, 1389619982000],
    [40960001, 1389670949000],
    [40970000, 1389710777000],
    [40980000, 1389777793000],
    [40990003, 1389805633000],
    [41000001, 1389876411000],
    [41010004, 1389930176000],
    [41020000, 1389969392000],
    [41030003, 1390022259000],
    [41040000, 1390052441000],
    [41050000, 1390095627000],
    [41060001, 1390129212000],
    [41070000, 1390148564000],
    [41080002, 1390218360000],
    [41090000, 1390253841000],
    [41100000, 1390310266000],
    [41110002, 1390365394000],
    [41120000, 1390400848000],
    [41130000, 1390462727000],
    [41140000, 1390490469000],
    [41150006, 1390557209000],
    [41160000, 1390581485000],
    [41170000, 1390639346000],
    [41180000, 1390662146000],
    [41190001, 1390711554000],
    [41200000, 1390738921000],
    [41210000, 1390761786000],
    [41220001, 1390825685000],
    [41230004, 1390864879000],
    [41240001, 1390914926000],
    [41250000, 1390967166000],
    [41260001, 1391003550000],
    [41270000, 1391057312000],
    [41280001, 1391090640000],
    [41290000, 1391133271000],
    [41300004, 1391173543000],
    [41310001, 1391196402000],
    [41320000, 1391248927000],
    [41330004, 1391268984000],
    [41340002, 1391318292000],
    [41350003, 1391343049000],
    [41360000, 1391358890000],
    [41370006, 1391420322000],
    [41380002, 1391440394000],
    [41390000, 1391500984000],
    [41400001, 1391526066000],
    [41410003, 1391586012000],
    [41420001, 1391612173000],
    [41430000, 1391671991000],
    [41440000, 1391698434000],
    [41450000, 1391757471000],
    [41460000, 1391784929000],
    [41470000, 1391834616000],
    [41480000, 1391862063000],
    [41490003, 1391881559000],
    [41500000, 1391932582000],
    [41510000, 1391953765000],
    [41520004, 1391998284000],
    [41530002, 1392037552000],
    [41540000, 1392066530000],
    [41550000, 1392112644000],
    [41560000, 1392131840000],
    [41570001, 1392194076000],
    [41580000, 1392219316000],
    [41590000, 1392282853000],
    [41600000, 1392304606000],
    [41610005, 1392348357000],
    [41620001, 1392376215000],
    [41630000, 1392389474000],
    [41640001, 1392426804000],
    [41650000, 1392460251000],
    [41660002, 1392478667000],
    [41670001, 1392526199000],
    [41680001, 1392552314000],
    [41690001, 1392570369000],
    [41700000, 1392633722000],
    [41710000, 1392655918000],
    [41720000, 1392721170000],
    [41730000, 1392744219000],
    [41740002, 1392810166000],
    [41750000, 1392839290000],
    [41760000, 1392899792000],
    [41770001, 1392946266000],
    [41780003, 1392989834000],
    [41790000, 1393032132000],
    [41800001, 1393067824000],
    [41810001, 1393084402000],
    [41820001, 1393135710000],
    [41830000, 1393161351000],
    [41840002, 1393203897000],
    [41850002, 1393248383000],
    [41860001, 1393291912000],
    [41870001, 1393335014000],
    [41880000, 1393388020000],
    [41890001, 1393424619000],
    [41900004, 1393484298000],
    [41910001, 1393513270000],
    [41920000, 1393575080000],
    [41930004, 1393599724000],
    [41940000, 1393649411000],
    [41950001, 1393678485000],
    [41960000, 1393701470000],
    [41970000, 1393749770000],
    [41980000, 1393770253000],
    [41990000, 1393815455000],
    [42000000, 1393852628000],
    [42010003, 1393897936000],
    [42020000, 1393940871000],
    [42030000, 1393995575000],
    [42040000, 1394028460000],
    [42050003, 1394084274000],
    [42060000, 1394116297000],
    [42070006, 1394174826000],
    [42080000, 1394203514000],
    [42090000, 1394254421000],
    [42100001, 1394284010000],
    [42110001, 1394306295000],
    [42120004, 1394353495000],
    [42130000, 1394372476000],
    [42140000, 1394403256000],
    [42150003, 1394454978000],
    [42160002, 1394492825000],
    [42170001, 1394542229000],
    [42180000, 1394584146000],
    [42190000, 1394628598000],
    [42200001, 1394665824000],
    [42210000, 1394712644000],
    [42220000, 1394732736000],
    [42230006, 1394793084000],
    [42240000, 1394811767000],
    [42250000, 1394861234000],
    [42260001, 1394888921000],
    [42270000, 1394916941000],
    [42280002, 1394963854000],
    [42290000, 1394983698000],
    [42300005, 1395045978000],
    [42310003, 1395070383000],
    [42320002, 1395132113000],
    [42330003, 1395157270000],
    [42340000, 1395220938000],
    [42350002, 1395245872000],
    [42360000, 1395308429000],
    [42370003, 1395331913000],
    [42380000, 1395390043000],
    [42390000, 1395414897000],
    [42400001, 1395466421000],
    [42410002, 1395495043000],
    [42420002, 1395535189000],
    [42430001, 1395571843000],
    [42440003, 1395591434000],
    [42450001, 1395653812000],
    [42460000, 1395677629000],
    [42470000, 1395738654000],
    [42480002, 1395762934000],
    [42490000, 1395822734000],
    [42500002, 1395847945000],
    [42510000, 1395906367000],
    [42520001, 1395933159000],
    [42530002, 1395991085000],
    [42540002, 1396018980000],
    [42550001, 1396071207000],
    [42560001, 1396100789000],
    [42570002, 1396138699000],
    [42580000, 1396175404000],
    [42590001, 1396194254000],
    [42600002, 1396252141000],
    [42610000, 1396277944000],
    [42620000, 1396323943000],
    [42630002, 1396357555000],
    [42640000, 1396391700000],
    [42650000, 1396440208000],
    [42660002, 1396469027000],
    [42670000, 1396524679000],
    [42680001, 1396546938000],
    [42690001, 1396606462000],
    [42700001, 1396627908000],
    [42710002, 1396682080000],
    [42720000, 1396707226000],
    [42730001, 1396750142000],
    [42740003, 1396782763000],
    [42750000, 1396800959000],
    [42760001, 1396863535000],
    [42770000, 1396887145000],
    [42780001, 1396954829000],
    [42790001, 1396982837000],
    [42800001, 1397049398000],
    [42810002, 1397110653000],
    [42820000, 1397143579000],
    [42830000, 1397215545000],
    [42840002, 1397254675000],
    [42850002, 1397301720000],
    [42860000, 1397322967000],
    [42870002, 1397375930000],
    [42880000, 1397397773000],
    [42890001, 1397455706000],
    [42900000, 1397488856000],
    [42910000, 1397560697000],
    [42920001, 1397599593000],
    [42930005, 1397657542000],
    [42940000, 1397724358000],
    [42950000, 1397752975000],
    [42960000, 1397823929000],
    [42970004, 1397861939000],
    [42980000, 1397906037000],
    [42990001, 1397925324000],
    [43000001, 1397977859000],
    [43010002, 1398000649000],
    [43020001, 1398051368000],
    [43030002, 1398090478000],
    [43040000, 1398155348000],
    [43050000, 1398182383000],
    [43060000, 1398253653000],
    [43070000, 1398292539000],
    [43080002, 1398347725000],
    [43090000, 1398411627000],
    [43100001, 1398440983000],
    [43110000, 1398499356000],
    [43120000, 1398524216000],
    [43130000, 1398571734000],
    [43140000, 1398600068000],
    [43150000, 1398618732000],
    [43160008, 1398685865000],
    [43170000, 1398709556000],
    [43180003, 1398763821000],
    [43190002, 1398783609000],
    [43200001, 1398842780000],
    [43210000, 1398868275000],
    [43220000, 1398917399000],
    [43230000, 1398951267000],
    [43240001, 1398985439000],
    [43250002, 1399033139000],
    [43260000, 1399053659000],
    [43270002, 1399106899000],
    [43280000, 1399129212000],
    [43290001, 1399181866000],
    [43300000, 1399212177000],
    [43310004, 1399255726000],
    [43320000, 1399290772000],
    [43330001, 1399309105000],
    [43340003, 1399363810000],
    [43350000, 1399385332000],
    [43360002, 1399442840000],
    [43370000, 1399475201000],
    [43380000, 1399544901000],
    [43390001, 1399570022000],
    [43400000, 1399638792000],
    [43410001, 1399670418000],
    [43420001, 1399718529000],
    [43430002, 1399738317000],
    [43440002, 1399790375000],
    [43450000, 1399814799000],
    [43460002, 1399864598000],
    [43470001, 1399905263000],
    [43480000, 1399971974000],
    [43490003, 1399999898000],
    [43500000, 1400070205000],
    [43510000, 1400115414000],
    [43520000, 1400163520000],
    [43530003, 1400228908000],
    [43540000, 1400255949000],
    [43550001, 1400313912000],
    [43560000, 1400339124000],
    [43570000, 1400388580000],
    [43580004, 1400416359000],
    [43590002, 1400444855000],
    [43600005, 1400506605000],
    [43610001, 1400569839000],
    [43620000, 1400601150000],
    [43630001, 1400672517000],
    [43640000, 1400723348000],
    [43650003, 1400769717000],
    [43660000, 1400837345000],
    [43670003, 1400862090000],
    [43680003, 1400921009000],
    [43690002, 1400945026000],
    [43700000, 1400996183000],
    [43710000, 1401022371000],
    [43720000, 1401063071000],
    [43730002, 1401112830000],
    [43740000, 1401177721000],
    [43750000, 1401208122000],
    [43760000, 1401280984000],
    [43770000, 1401333522000],
    [43780001, 1401375608000],
    [43790001, 1401444365000],
    [43800001, 1401471127000],
    [43810003, 1401530344000],
    [43820001, 1401550961000],
    [43830000, 1401600993000],
    [43840000, 1401626966000],
    [43850000, 1401657217000],
    [43860008, 1401713968000],
    [43870003, 1401767777000],
    [43880001, 1401807602000],
    [43890002, 1401876480000],
    [43900000, 1401904346000],
    [43910001, 1401973528000],
    [43920001, 1402025979000],
    [43930000, 1402063985000],
    [43940000, 1402112952000],
    [43950000, 1402145459000],
    [43960001, 1402172306000],
    [43970001, 1402218554000],
    [43980000, 1402238671000],
    [43990000, 1402291402000],
    [44000001, 1402324160000],
    [44010000, 1402384520000],
    [44020002, 1402413601000],
    [44030000, 1402483256000],
    [44040000, 1402509616000],
    [44050000, 1402578519000],
    [44060000, 1402623458000],
    [44070002, 1402667904000],
    [44080001, 1402715545000],
    [44090000, 1402750894000],
    [44100000, 1402785867000],
    [44110000, 1402826595000],
    [44120000, 1402846052000],
    [44130000, 1402913327000],
    [44140000, 1402938297000],
    [44150000, 1403006985000],
    [44160001, 1403044298000],
    [44170002, 1403097803000],
    [44180000, 1403150518000],
    [44190000, 1403188706000],
    [44200000, 1403253194000],
    [44210002, 1403279667000],
    [44220001, 1403336437000],
    [44230002, 1403361689000],
    [44240001, 1403406798000],
    [44250004, 1403437621000],
    [44260000, 1403458355000],
    [44270001, 1403526600000],
    [44280000, 1403575405000],
    [44290000, 1403619591000],
    [44300006, 1403682903000],
    [44310002, 1403706840000],
    [44320003, 1403759025000],
    [44330002, 1403791871000],
    [44340000, 1403845932000],
    [44350000, 1403878913000],
    [44360000, 1403926002000],
    [44370000, 1403958052000],
    [44380004, 1403978310000],
    [44390002, 1404027816000],
    [44400000, 1404049867000],
    [44410002, 1404092202000],
    [44420000, 1404134229000],
    [44430000, 1404164270000],
    [44440001, 1404218810000],
    [44450000, 1404257765000],
    [44460001, 1404307517000],
    [44470000, 1404361188000],
    [44480000, 1404396968000],
    [44490000, 1404450979000],
    [44500000, 1404484370000],
    [44510000, 1404531329000],
    [44520002, 1404563707000],
    [44530000, 1404586575000],
    [44540003, 1404635596000],
    [44550000, 1404656947000],
    [44560001, 1404700582000],
    [44570000, 1404737750000],
    [44580001, 1404759751000],
    [44590000, 1404823038000],
    [44600002, 1404871988000],
    [44610000, 1404914244000],
    [44620000, 1404970300000],
    [44630000, 1405002069000],
    [44640002, 1405060766000],
    [44650000, 1405091407000],
    [44660000, 1405146732000],
    [44670001, 1405174715000],
    [44680001, 1405220483000],
    [44690002, 1405251314000],
    [44700000, 1405270285000],
    [44710000, 1405337178000],
    [44720000, 1405361842000],
    [44730002, 1405427494000],
    [44740003, 1405475240000],
    [44750004, 1405519456000],
    [44760003, 1405582910000],
    [44770000, 1405612234000],
    [44780003, 1405680048000],
    [44790003, 1405709128000],
    [44800000, 1405766040000],
    [44810000, 1405789241000],
    [44820001, 1405845253000],
    [44830002, 1405869189000],
    [44840001, 1405922372000],
    [44850002, 1405949188000],
    [44860000, 1405992548000],
    [44870000, 1406035142000],
    [44880001, 1406086433000],
    [44890001, 1406124315000],
    [44900002, 1406182689000],
    [44910005, 1406214574000],
    [44920000, 1406278021000],
    [44930000, 1406305823000],
    [44940001, 1406365818000],
    [44950000, 1406392124000],
    [44960001, 1406448520000],
    [44970001, 1406473115000],
    [44980002, 1406534631000],
    [44990000, 1406562421000],
    [45000000, 1406627631000],
    [45010000, 1406653768000],
    [45020000, 1406718289000],
    [45030000, 1406745105000],
    [45040002, 1406806551000],
    [45050007, 1406826261000],
    [45060000, 1406886698000],
    [45070000, 1406909284000],
    [45080002, 1406964417000],
    [45090003, 1406990559000],
    [45100001, 1407037498000],
    [45110000, 1407068214000],
    [45120000, 1407091073000],
    [45130000, 1407150920000],
    [45140000, 1407174945000],
    [45150001, 1407236279000],
    [45160000, 1407260335000],
    [45170004, 1407322261000],
    [45180000, 1407346097000],
    [45190001, 1407407515000],
    [45200002, 1407428521000],
    [45210000, 1407487560000],
    [45220002, 1407510877000],
    [45230001, 1407563375000],
    [45240001, 1407591542000],
    [45250000, 1407630080000],
    [45260002, 1407664218000],
    [45270001, 1407683121000],
    [45280000, 1407735019000],
    [45290001, 1407765570000],
    [45300001, 1407812110000],
    [45310000, 1407847709000],
    [45320000, 1407873477000],
    [45330000, 1407926302000],
    [45340000, 1407947772000],
    [45350002, 1408002387000],
    [45360000, 1408027778000],
    [45370001, 1408077726000],
    [45380000, 1408111355000],
    [45390002, 1408162182000],
    [45400000, 1408197659000],
    [45410000, 1408248522000],
    [45420004, 1408280781000],
    [45430000, 1408324617000],
    [45440001, 1408366777000],
    [45450002, 1408408751000],
    [45460005, 1408452134000],
    [45470002, 1408488705000],
    [45480000, 1408537718000],
    [45490000, 1408577858000],
    [45500001, 1408624745000],
    [45510000, 1408661595000],
    [45520000, 1408709941000],
    [45530001, 1408739295000],
    [45540002, 1408790292000],
    [45550001, 1408813387000],
    [45560000, 1408869060000],
    [45570000, 1408892386000],
    [45580003, 1408950609000],
    [45590001, 1408978478000],
    [45600001, 1409038069000],
    [45610000, 1409066062000],
    [45620006, 1409128203000],
    [45630000, 1409153928000],
    [45640000, 1409216945000],
    [45650000, 1409241599000],
    [45660002, 1409305363000],
    [45670002, 1409329768000],
    [45680001, 1409387563000],
    [45690001, 1409411227000],
    [45700002, 1409456447000],
    [45710001, 1409485571000],
    [45720001, 1409501028000],
    [45730010, 1409567310000],
    [45740000, 1409591429000],
    [45750000, 1409660281000],
    [45760001, 1409706661000],
    [45770000, 1409753397000],
    [45780001, 1409817467000],
    [45790000, 1409846233000],
    [45800001, 1409916309000],
    [45810001, 1409955523000],
    [45820001, 1410004195000],
    [45830002, 1410027341000],
    [45840001, 1410080086000],
    [45850000, 1410101234000],
    [45860003, 1410159791000],
    [45870002, 1410188406000],
    [45880000, 1410251730000],
    [45890000, 1410277481000],
    [45900002, 1410346533000],
    [45910000, 1410373358000],
    [45920001, 1410441075000],
    [45930000, 1410494820000],
    [45940000, 1410533064000],
    [45950000, 1410586687000],
    [45960002, 1410617477000],
    [45970001, 1410666060000],
    [45980000, 1410700170000],
    [45990001, 1410742254000],
    [46000003, 1410780600000],
    [46010001, 1410800885000],
    [46020000, 1410871404000],
    [46030001, 1410925985000],
    [46040000, 1410965180000],
    [46050002, 1411035539000],
    [46060000, 1411068445000],
    [46070002, 1411132729000],
    [46080000, 1411179286000],
    [46090001, 1411217966000],
    [46100001, 1411260613000],
    [46110002, 1411299116000],
    [46120001, 1411324250000],
    [46130002, 1411392713000],
    [46140002, 1411444634000],
    [46150004, 1411477767000],
    [46160001, 1411533742000],
    [46170002, 1411571943000],
    [46180002, 1411644554000],
    [46190000, 1411694810000],
    [46200002, 1411742253000],
    [46210003, 1411796970000],
    [46220001, 1411827559000],
    [46230000, 1411878229000],
    [46240000, 1411909469000],
    [46250001, 1411958548000],
    [46260000, 1412001347000],
    [46270000, 1412069171000],
    [46280000, 1412092418000],
    [46290000, 1412159293000],
    [46300000, 1412182471000],
    [46310004, 1412251678000],
    [46320001, 1412292704000],
    [46330001, 1412343931000],
    [46340000, 1412390517000],
    [46350000, 1412426684000],
    [46360003, 1412459043000],
    [46370001, 1412502758000],
    [46380000, 1412522089000],
    [46390000, 1412579126000],
    [46400001, 1412606211000],
    [46410000, 1412670515000],
    [46420000, 1412697641000],
    [46430000, 1412769724000],
    [46440000, 1412805139000],
    [46450003, 1412861657000],
    [46460001, 1412900611000],
    [46470000, 1412946324000],
    [46480000, 1412966082000],
    [46490000, 1413021523000],
    [46500000, 1413045739000],
    [46510004, 1413105184000],
    [46520002, 1413132385000],
    [46530001, 1413189711000],
    [46540000, 1413212103000],
    [46550001, 1413279134000],
    [46560000, 1413308431000],
    [46570002, 1413380899000],
    [46580000, 1413448537000],
    [46590000, 1413481017000],
    [46600002, 1413553814000],
    [46610001, 1413610622000],
    [46620000, 1413643873000],
    [46630000, 1413699682000],
    [46640002, 1413727940000],
    [46650000, 1413796851000],
    [46660000, 1413829473000],
    [46670000, 1413900925000],
    [46680000, 1413972104000],
    [46690001, 1414020575000],
    [46700006, 1414077321000],
    [46710001, 1414153803000],
    [46720000, 1414209098000],
    [46730000, 1414246272000],
    [46740001, 1414300019000],
    [46750000, 1414330741000],
    [46760000, 1414395376000],
    [46770001, 1414429636000],
    [46780003, 1414504384000],
    [46790001, 1414577244000],
    [46800001, 1414622603000],
    [46810000, 1414678523000],
    [46820000, 1414722059000],
    [46830001, 1414757909000],
    [46840001, 1414769504000],
    [46850000, 1414823220000],
    [46860000, 1414852020000],
    [46870001, 1414903508000],
    [46880000, 1414934932000],
    [46890002, 1414980757000],
    [46900002, 1415017619000],
    [46910000, 1415062708000],
    [46920000, 1415112999000],
    [46930000, 1415187739000],
    [46940002, 1415248447000],
    [46950003, 1415289061000],
    [46960000, 1415364291000],
    [46970001, 1415417408000],
    [46980003, 1415454956000],
    [46990000, 1415507307000],
    [47000000, 1415539055000],
    [47010000, 1415588639000],
    [47020000, 1415631586000],
    [47030002, 1415696295000],
    [47040002, 1415718900000],
    [47050002, 1415789004000],
    [47060000, 1415839179000],
    [47070003, 1415891607000],
    [47080000, 1415967746000],
    [47090005, 1416022074000],
    [47100000, 1416060637000],
    [47110001, 1416115908000],
    [47120001, 1416146173000],
    [47130000, 1416208876000],
    [47140001, 1416242085000],
    [47150000, 1416317801000],
    [47160000, 1416390921000],
    [47170000, 1416448604000],
    [47180000, 1416497398000],
    [47190000, 1416574038000],
    [47200001, 1416629021000],
    [47210003, 1416666230000],
    [47220001, 1416721789000],
    [47230001, 1416753783000],
    [47240000, 1416813201000],
    [47250000, 1416842192000],
    [47260002, 1416918200000],
    [47270000, 1416984962000],
    [47280002, 1417023611000],
    [47290000, 1417098065000],
    [47300000, 1417172998000],
    [47310000, 1417218646000],
    [47320006, 1417265182000],
    [47330000, 1417309885000],
    [47340001, 1417348922000],
    [47350001, 1417374842000],
    [47360001, 1417442238000],
    [47370000, 1417514169000],
    [47380001, 1417573663000],
    [47390001, 1417621738000],
    [47400000, 1417698110000],
    [47410001, 1417763923000],
    [47420000, 1417799488000],
    [47430000, 1417865705000],
    [47440002, 1417903772000],
    [47450000, 1417951702000],
    [47460002, 1417989019000],
    [47470002, 1418050247000],
    [47480000, 1418125512000],
    [47490000, 1418188809000],
    [47500001, 1418230565000],
    [47510003, 1418304837000],
    [47520000, 1418374861000],
    [47530001, 1418410134000],
    [47540001, 1418472019000],
    [47550000, 1418516238000],
    [47560001, 1418557982000],
    [47570001, 1418602760000],
    [47580000, 1418655482000],
    [47590000, 1418728714000],
    [47600002, 1418786472000],
    [47610000, 1418830833000],
    [47620000, 1418906555000],
    [47630001, 1418974005000],
    [47640003, 1419006790000],
    [47650000, 1419070128000],
    [47660000, 1419094682000],
    [47670002, 1419153085000],
    [47680003, 1419175796000],
    [47690000, 1419244695000],
    [47700000, 1419274769000],
    [47710000, 1419333862000],
    [47720000, 1419354340000],
    [47730001, 1419415711000],
    [47740000, 1419433879000],
    [47750007, 1419479734000],
    [47760000, 1419511462000],
    [47770001, 1419528679000],
    [47780001, 1419591419000],
    [47790000, 1419614182000],
    [47800003, 1419670281000],
    [47810001, 1419695043000],
    [47820003, 1419753778000],
    [47830000, 1419782289000],
    [47840000, 1419844011000],
    [47850001, 1419874897000],
    [47860000, 1419938290000],
    [47870000, 1419964946000],
    [47880001, 1420015097000],
    [47890000, 1420036028000],
    [47900000, 1420042361000],
    [47910000, 1420078699000],
    [47920001, 1420103980000],
    [47930000, 1420123777000],
    [47940000, 1420175074000],
    [47950000, 1420206280000],
    [47960001, 1420253354000],
    [47970000, 1420288344000],
    [47980000, 1420321580000],
    [47990000, 1420368200000],
    [48000001, 1420388763000],
    [48010000, 1420453511000],
    [48020004, 1420478404000],
    [48030000, 1420544239000],
    [48040000, 1420579475000],
    [48050001, 1420636400000],
    [48060002, 1420694718000],
    [48070001, 1420731031000],
    [48080000, 1420801573000],
    [48090005, 1420828904000],
    [48100000, 1420887623000],
    [48110005, 1420911824000],
    [48120000, 1420970320000],
    [48130001, 1420995201000],
    [48140001, 1421053286000],
    [48150003, 1421076820000],
    [48160001, 1421150311000],
    [48170002, 1421208087000],
    [48180002, 1421249944000],
    [48190001, 1421323732000],
    [48200001, 1421380565000],
    [48210000, 1421421390000],
    [48220001, 1421480469000],
    [48230000, 1421507793000],
    [48240001, 1421562583000],
    [48250000, 1421589740000],
    [48260000, 1421647746000],
    [48270001, 1421681904000],
    [48280000, 1421752922000],
    [48290002, 1421796949000],
    [48300002, 1421848870000],
    [48310000, 1421910136000],
    [48320000, 1421940971000],
    [48330000, 1422009927000],
    [48340004, 1422037304000],
    [48350000, 1422095668000],
    [48360001, 1422119397000],
    [48370000, 1422177393000],
    [48380000, 1422200867000],
    [48390000, 1422271145000],
    [48400000, 1422314467000],
    [48410000, 1422367980000],
    [48420000, 1422433504000],
    [48430004, 1422463457000],
    [48440000, 1422534768000],
    [48450000, 1422587272000],
    [48460003, 1422627741000],
    [48470000, 1422677639000],
    [48480003, 1422710491000],
    [48490003, 1422746403000],
    [48500000, 1422788824000],
    [48510000, 1422811475000],
    [48520002, 1422879052000],
    [48530005, 1422919644000],
    [48540000, 1422969688000],
    [48550000, 1423021714000],
    [48560000, 1423059994000],
    [48570000, 1423124083000],
    [48580000, 1423151799000],
    [48590001, 1423219438000],
    [48600001, 1423248799000],
    [48610001, 1423307183000],
    [48620003, 1423331652000],
    [48630000, 1423387909000],
    [48640000, 1423410111000],
    [48650000, 1423475783000],
    [48660000, 1423504414000],
    [48670000, 1423571959000],
    [48680001, 1423617558000],
    [48690000, 1423656664000],
    [48700000, 1423695965000],
    [48710002, 1423747660000],
    [48720002, 1423801076000],
    [48730004, 1423838902000],
    [48740006, 1423880064000],
    [48750001, 1423912035000],
    [48760000, 1423927165000],
    [48770000, 1423976919000],
    [48780001, 1424005623000],
    [48790004, 1424052534000],
    [48800001, 1424095460000],
    [48810002, 1424156267000],
    [48820000, 1424187104000],
    [48830004, 1424253921000],
    [48840002, 1424281276000],
    [48850000, 1424350047000],
    [48860000, 1424405036000],
    [48870000, 1424445222000],
    [48880001, 1424506523000],
    [48890000, 1424533717000],
    [48900002, 1424588830000],
    [48910003, 1424612370000],
    [48920000, 1424657127000],
    [48930000, 1424700853000],
    [48940005, 1424764099000],
    [48950001, 1424794919000],
    [48960000, 1424863985000],
    [48970004, 1424907331000],
    [48980000, 1424958445000],
    [48990000, 1425020098000],
    [49000000, 1425052228000],
    [49010000, 1425113573000],
    [49020002, 1425137788000],
    [49030000, 1425192625000],
    [49040000, 1425218915000],
    [49050000, 1425277345000],
    [49060000, 1425310019000],
    [49070001, 1425376397000],
    [49080000, 1425401283000],
    [49090000, 1425470717000],
    [49100001, 1425522821000],
    [49110002, 1425565908000],
    [49120000, 1425631623000],
    [49130001, 1425661246000],
    [49140001, 1425721796000],
    [49150000, 1425746273000],
    [49160000, 1425803701000],
    [49170000, 1425826933000],
    [49180000, 1425888991000],
    [49190000, 1425914570000],
    [49200001, 1425980130000],
    [49210005, 1426009556000],
    [49220001, 1426078336000],
    [49230000, 1426135941000],
    [49240003, 1426172068000],
    [49250000, 1426237030000],
    [49260000, 1426264348000],
    [49270002, 1426321389000],
    [49280002, 1426346221000],
    [49290002, 1426401154000],
    [49300000, 1426429467000],
    [49310001, 1426492115000],
    [49320000, 1426522044000],
    [49330000, 1426593267000],
    [49340001, 1426648563000],
    [49350000, 1426690127000],
    [49360000, 1426758029000],
    [49370002, 1426788601000],
    [49380008, 1426855140000],
    [49390001, 1426902362000],
    [49400002, 1426943270000],
    [49410000, 1426990368000],
    [49420000, 1427027854000],
    [49430000, 1427076951000],
    [49440002, 1427121382000],
    [49450003, 1427188723000],
    [49460000, 1427221714000],
    [49470001, 1427288345000],
    [49480001, 1427346664000],
    [49490000, 1427382958000],
    [49500000, 1427449110000],
    [49510000, 1427478001000],
    [49520001, 1427538782000],
    [49530000, 1427565781000],
    [49540001, 1427621536000],
    [49550002, 1427644856000],
    [49560003, 1427710766000],
    [49570001, 1427741876000],
    [49580000, 1427804380000],
    [49590000, 1427835668000],
    [49600002, 1427889902000],
    [49610005, 1427922598000],
    [49620000, 1427980018000],
    [49630000, 1428033261000],
    [49640001, 1428070688000],
    [49650004, 1428122102000],
    [49660001, 1428155752000],
    [49670004, 1428198715000],
    [49680001, 1428233407000],
    [49690000, 1428253898000],
    [49700002, 1428317973000],
    [49710001, 1428348567000],
    [49720001, 1428412177000],
    [49730000, 1428469600000],
    [49740002, 1428506182000],
    [49750004, 1428578339000],
    [49760002, 1428621049000],
    [49770001, 1428675507000],
    [49780000, 1428730122000],
    [49790003, 1428762157000],
    [49800000, 1428810192000],
    [49810000, 1428841777000],
    [49820000, 1428880711000],
    [49830001, 1428934950000],
    [49840000, 1429006335000],
    [49850000, 1429054292000],
    [49860001, 1429109898000],
    [49870000, 1429183117000],
    [49880000, 1429236717000],
    [49890000, 1429283590000],
    [49900001, 1429345715000],
    [49910000, 1429372438000],
    [49920001, 1429427262000],
    [49930000, 1429452183000],
    [49940000, 1429511794000],
    [49950000, 1429545046000],
    [49960001, 1429620425000],
    [49970001, 1429683143000],
    [49980000, 1429720850000],
    [49990000, 1429795605000],
    [50000000, 1429863864000],
    [50010000, 1429898787000],
    [50020000, 1429961496000],
    [50030000, 1429996983000],
    [50040004, 1430045105000],
    [50050000, 1430067028000],
    [50060003, 1430139388000],
    [50070002, 1430200615000],
    [50080002, 1430235720000],
    [50090001, 1430296784000],
    [50100000, 1430321228000],
    [50110000, 1430389982000],
    [50120001, 1430412736000],
    [50130002, 1430479345000],
    [50140000, 1430501860000],
    [50150000, 1430557445000],
    [50160001, 1430581431000],
    [50170001, 1430636674000],
    [50180000, 1430664979000],
    [50190006, 1430719047000],
    [50200001, 1430750240000],
    [50210000, 1430802866000],
    [50220000, 1430835446000],
    [50230000, 1430886375000],
    [50240000, 1430917406000],
    [50250001, 1430969831000],
    [50260000, 1431011510000],
    [50270001, 1431083973000],
    [50280003, 1431123849000],
    [50290002, 1431172739000],
    [50300000, 1431204867000],
    [50310001, 1431252706000],
    [50320000, 1431275326000],
    [50330001, 1431349340000],
    [50340000, 1431415400000],
    [50350002, 1431450112000],
    [50360000, 1431524579000],
    [50370004, 1431595757000],
    [50380000, 1431629249000],
    [50390001, 1431698689000],
    [50400001, 1431753573000],
    [50410000, 1431786571000],
    [50420000, 1431837465000],
    [50430000, 1431868109000],
    [50440003, 1431924122000],
    [50450000, 1431963346000],
    [50460001, 1432038066000],
    [50470002, 1432100070000],
    [50480000, 1432137840000],
    [50490001, 1432212978000],
    [50500001, 1432280391000],
    [50510000, 1432314638000],
    [50520000, 1432378486000],
    [50530000, 1432405540000],
    [50540000, 1432460533000],
    [50550001, 1432483593000],
    [50560000, 1432556836000],
    [50570001, 1432620778000],
    [50580000, 1432659550000],
    [50590001, 1432733347000],
    [50600000, 1432803486000],
    [50610004, 1432845996000],
    [50620001, 1432910338000],
    [50630000, 1432969407000],
    [50640000, 1432999743000],
    [50650000, 1433054954000],
    [50660001, 1433081521000],
    [50670000, 1433139525000],
    [50680005, 1433174212000],
    [50690003, 1433247766000],
    [50700007, 1433304560000],
    [50710000, 1433344640000],
    [50720000, 1433417598000],
    [50730002, 1433465723000],
    [50740000, 1433515819000],
    [50750000, 1433573481000],
    [50760000, 1433603104000],
    [50770000, 1433656399000],
    [50780000, 1433684476000],
    [50790006, 1433742245000],
    [50800000, 1433776632000],
    [50810002, 1433846603000],
    [50820000, 1433874860000],
    [50830002, 1433943909000],
    [50840000, 1434010217000],
    [50850001, 1434045170000],
    [50860002, 1434115301000],
    [50870000, 1434165997000],
    [50880000, 1434203013000],
    [50890000, 1434250611000],
    [50900000, 1434284671000],
    [50910000, 1434328565000],
    [50920000, 1434378765000],
    [50930001, 1434449589000],
    [50940000, 1434495986000],
    [50950002, 1434551913000],
    [50960000, 1434622524000],
    [50970001, 1434660867000],
    [50980001, 1434723356000],
    [50990000, 1434781138000],
    [51000000, 1434812712000],
    [51010005, 1434865927000],
    [51020003, 1434894231000],
    [51030000, 1434949765000],
    [51040001, 1434986243000],
    [51050001, 1435058040000],
    [51060000, 1435104020000],
    [51070004, 1435156327000],
    [51080001, 1435224543000],
    [51090000, 1435255929000],
    [51100000, 1435323878000],
    [51110003, 1435371108000],
    [51120002, 1435410415000],
    [51130000, 1435457064000],
    [51140000, 1435494630000],
    [51150001, 1435548043000],
    [51160000, 1435590927000],
    [51170000, 1435661743000],
    [51180000, 1435693160000],
    [51190000, 1435757210000],
    [51200001, 1435820137000],
    [51210000, 1435852363000],
    [51220001, 1435924090000],
    [51230000, 1435963538000],
    [51240000, 1436011041000],
    [51250000, 1436041964000],
    [51260001, 1436092245000],
    [51270002, 1436113783000],
    [51280000, 1436184639000],
    [51290001, 1436222209000],
    [51300002, 1436274448000],
    [51310000, 1436321505000],
    [51320000, 1436367233000],
    [51330000, 1436438434000],
    [51340000, 1436482573000],
    [51350000, 1436539264000],
    [51360001, 1436600774000],
    [51370005, 1436630589000],
    [51380000, 1436689104000],
    [51390000, 1436714870000],
    [51400000, 1436787425000],
    [51410000, 1436842693000],
    [51420000, 1436886365000],
    [51430000, 1436959083000],
    [51440001, 1437015088000],
    [51450000, 1437059007000],
    [51460002, 1437127790000],
    [51470001, 1437161200000],
    [51480000, 1437221108000],
    [51490000, 1437266583000],
    [51500000, 1437310382000],
    [51510001, 1437353953000],
    [51520001, 1437393709000],
    [51530001, 1437429645000],
    [51540002, 1437486190000],
    [51550009, 1437545501000],
    [51560000, 1437579146000],
    [51570000, 1437647564000],
    [51580001, 1437688568000],
    [51590000, 1437746118000],
    [51600006, 1437804136000],
    [51610001, 1437838006000],
    [51620001, 1437896582000],
    [51630000, 1437924161000],
    [51640004, 1437993698000],
    [51650001, 1438041123000],
    [51660000, 1438092330000],
    [51670000, 1438155491000],
    [51680000, 1438188475000],
    [51690001, 1438258316000],
    [51700004, 1438310555000],
    [51710001, 1438352766000],
    [51720000, 1438404349000],
    [51730000, 1438439607000],
    [51740000, 1438492706000],
    [51750000, 1438523901000],
    [51760000, 1438576479000],
    [51770000, 1438613331000],
    [51780001, 1438676812000],
    [51790000, 1438707698000],
    [51800001, 1438775013000],
    [51810000, 1438822665000],
    [51820000, 1438868881000],
    [51830000, 1438925089000],
    [51840001, 1438959605000],
    [51850003, 1439015686000],
    [51860000, 1439046332000],
    [51870001, 1439101072000],
    [51880001, 1439129645000],
    [51890000, 1439180053000],
    [51900001, 1439215210000],
    [51910001, 1439268110000],
    [51920001, 1439303260000],
    [51930002, 1439357219000],
    [51940000, 1439389903000],
    [51950001, 1439439328000],
    [51960000, 1439474031000],
    [51970000, 1439522995000],
    [51980002, 1439562464000],
    [51990001, 1439619122000],
    [52000003, 1439652589000],
    [52010002, 1439713106000],
    [52020000, 1439737809000],
    [52030001, 1439796813000],
    [52040002, 1439825293000],
    [52050002, 1439887490000],
    [52060002, 1439915547000],
    [52070001, 1439981728000],
    [52080000, 1440007097000],
    [52090000, 1440071411000],
    [52100000, 1440105179000],
    [52110002, 1440161382000],
    [52120002, 1440204370000],
    [52130002, 1440247573000],
    [52140000, 1440293559000],
    [52150002, 1440333019000],
    [52160002, 1440378395000],
    [52170001, 1440424211000],
    [52180003, 1440482841000],
    [52190000, 1440515889000],
    [52200000, 1440582877000],
    [52210003, 1440612864000],
    [52220000, 1440679254000],
    [52230003, 1440726969000],
    [52240002, 1440770502000],
    [52250001, 1440819055000],
    [52260001, 1440855205000],
    [52270001, 1440904479000],
    [52280001, 1440940469000],
    [52290003, 1440986585000],
    [52300001, 1441027528000],
    [52310000, 1441071842000],
    [52320000, 1441118194000],
    [52330000, 1441188677000],
    [52340002, 1441231428000],
    [52350000, 1441290113000],
    [52360000, 1441359877000],
    [52370001, 1441396714000],
    [52380000, 1441456240000],
    [52390001, 1441503699000],
    [52400000, 1441542484000],
    [52410000, 1441591959000],
    [52420002, 1441638157000],
    [52430000, 1441710469000],
    [52440000, 1441757685000],
    [52450000, 1441808428000],
    [52460001, 1441877759000],
    [52470001, 1441925058000],
    [52480001, 1441983737000],
    [52490008, 1442044638000],
    [52500000, 1442073356000],
    [52510000, 1442131880000],
    [52520001, 1442157561000],
    [52530002, 1442230417000],
    [52540002, 1442285982000],
    [52550000, 1442331304000],
    [52560006, 1442406457000],
    [52570000, 1442469129000],
    [52580003, 1442507932000],
    [52590000, 1442582714000],
    [52600000, 1442640190000],
    [52610000, 1442675304000],
    [52620000, 1442734874000],
    [52630001, 1442765771000],
    [52640000, 1442832097000],
    [52650000, 1442874140000],
    [52660000, 1442925918000],
    [52670001, 1442973989000],
    [52680000, 1443013257000],
    [52690001, 1443073808000],
    [52700001, 1443112150000],
    [52710000, 1443187348000],
    [52720001, 1443245159000],
    [52730000, 1443279080000],
    [52740000, 1443331711000],
    [52750003, 1443360896000],
    [52760000, 1443416366000],
    [52770003, 1443454110000],
    [52780000, 1443526258000],
    [52790000, 1443573159000],
    [52800000, 1443623335000],
    [52810000, 1443687223000],
    [52820000, 1443715413000],
    [52830000, 1443785028000],
    [52840000, 1443812701000],
    [52850000, 1443868318000],
    [52860001, 1443892966000],
    [52870002, 1443951923000],
    [52880001, 1443977319000],
    [52890002, 1444050020000],
    [52900001, 1444112972000],
    [52910000, 1444150582000],
    [52920001, 1444224436000],
    [52930000, 1444293770000],
    [52940000, 1444328660000],
    [52950001, 1444398014000],
    [52960001, 1444448428000],
    [52970001, 1444485550000],
    [52980001, 1444537079000],
    [52990000, 1444571696000],
    [53000000, 1444624072000],
    [53010005, 1444658586000],
    [53020001, 1444728690000],
    [53030000, 1444776865000],
    [53040001, 1444834638000],
    [53050000, 1444910842000],
    [53060001, 1444974041000],
    [53070000, 1445012878000],
    [53080000, 1445077854000],
    [53090000, 1445107072000],
    [53100000, 1445164056000],
    [53110001, 1445193150000],
    [53120000, 1445264139000],
    [53130000, 1445339154000],
    [53140000, 1445396667000],
    [53150003, 1445441692000],
    [53160000, 1445518494000],
    [53170000, 1445588117000],
    [53180002, 1445628093000],
    [53190001, 1445689759000],
    [53200001, 1445738163000],
    [53210000, 1445775998000],
    [53220001, 1445833271000],
    [53230000, 1445876063000],
    [53240002, 1445952191000],
    [53250001, 1446023732000],
    [53260001, 1446071058000],
    [53270003, 1446130056000],
    [53280000, 1446200786000],
    [53290000, 1446222908000],
    [53300000, 1446276180000],
    [53310002, 1446298208000],
    [53320002, 1446314767000],
    [53330007, 1446368379000],
    [53340001, 1446391392000],
    [53350000, 1446463645000],
    [53360000, 1446511544000],
    [53370000, 1446554904000],
    [53380000, 1446613562000],
    [53390000, 1446652845000],
    [53400001, 1446728534000],
    [53410001, 1446798097000],
    [53420000, 1446835848000],
    [53430000, 1446897512000],
    [53440000, 1446935447000],
    [53450000, 1446979710000],
    [53460000, 1447002529000],
    [53470000, 1447074958000],
    [53480003, 1447138089000],
    [53490002, 1447171463000],
    [53500000, 1447241068000],
    [53510000, 1447263850000],
    [53520000, 1447334523000],
    [53530001, 1447400956000],
    [53540000, 1447434309000],
    [53550001, 1447495192000],
    [53560001, 1447520520000],
    [53570001, 1447579045000],
    [53580002, 1447602584000],
    [53590001, 1447677054000],
    [53600000, 1447734384000],
    [53610001, 1447774550000],
    [53620000, 1447848786000],
    [53630001, 1447909667000],
    [53640000, 1447950831000],
    [53650000, 1448025195000],
    [53660000, 1448081028000],
    [53670000, 1448117623000],
    [53680002, 1448173146000],
    [53690000, 1448202318000],
    [53700002, 1448256072000],
    [53710000, 1448286558000],
    [53720008, 1448350876000],
    [53730002, 1448385838000],
    [53740003, 1448459361000],
    [53750001, 1448529328000],
    [53760001, 1448571384000],
    [53770000, 1448635733000],
    [53780001, 1448695133000],
    [53790000, 1448725329000],
    [53800001, 1448783477000],
    [53810000, 1448809518000],
    [53820000, 1448882576000],
    [53830000, 1448935147000],
    [53840001, 1448982556000],
    [53850000, 1449056968000],
    [53860001, 1449120058000],
    [53870000, 1449160498000],
    [53880000, 1449235420000],
    [53890001, 1449294310000],
    [53900002, 1449328023000],
    [53910000, 1449385484000],
    [53920000, 1449413366000],
    [53930000, 1449486304000],
    [53940000, 1449545121000],
    [53950000, 1449590438000],
    [53960000, 1449666749000],
    [53970000, 1449737401000],
    [53980000, 1449782727000],
    [53990003, 1449845097000],
    [54000005, 1449906786000],
    [54010002, 1449935748000],
    [54020000, 1449994184000],
    [54030000, 1450019154000],
    [54040000, 1450089997000],
    [54050000, 1450138364000],
    [54060000, 1450190996000],
    [54070002, 1450263073000],
    [54080000, 1450319232000],
    [54090001, 1450366580000],
    [54100000, 1450442905000],
    [54110001, 1450501438000],
    [54120002, 1450537128000],
    [54130000, 1450594818000],
    [54140000, 1450622901000],
    [54150000, 1450692686000],
    [54160002, 1450728829000],
    [54170001, 1450792823000],
    [54180000, 1450849584000],
    [54190000, 1450880944000],
    [54200002, 1450930324000],
    [54210002, 1450964468000],
    [54220001, 1450990001000],
    [54230001, 1451040270000],
    [54240001, 1451058045000],
    [54250000, 1451113602000],
    [54260000, 1451142117000],
    [54270001, 1451193442000],
    [54280001, 1451222549000],
    [54290001, 1451262047000],
    [54300000, 1451306768000],
    [54310001, 1451353931000],
    [54320001, 1451397388000],
    [54330000, 1451455072000],
    [54340002, 1451488299000],
    [54350000, 1451541348000],
    [54360000, 1451568664000],
    [54370003, 1451578081000],
    [54380002, 1451583548000],
    [54390046, 1451584646000],
    [54400004, 1451585608000],
    [54410135, 1451586483000],
    [54420052, 1451587666000],
    [54430122, 1451589379000],
    [54440001, 1451591925000],
    [54450001, 1451625209000],
    [54460004, 1451641989000],
    [54470000, 1451655523000],
    [54480004, 1451675422000],
    [54490001, 1451730868000],
    [54500000, 1451755391000],
    [54510000, 1451810931000],
    [54520003, 1451834335000],
    [54530000, 1451893722000],
    [54540001, 1451921581000],
    [54550000, 1451981684000],
    [54560000, 1452009596000],
    [54570003, 1452069136000],
    [54580000, 1452096975000],
    [54590000, 1452165734000],
    [54600000, 1452206246000],
    [54610001, 1452262161000],
    [54620000, 1452316918000],
    [54630001, 1452349574000],
    [54640001, 1452402538000],
    [54650001, 1452434481000],
    [54660000, 1452486057000],
    [54670000, 1452517947000],
    [54680000, 1452572400000],
    [54690000, 1452613850000],
    [54700001, 1452690017000],
    [54710000, 1452756792000],
    [54720000, 1452790521000],
    [54730001, 1452863830000],
    [54740000, 1452921765000],
    [54750002, 1452955817000],
    [54760000, 1453008750000],
    [54770000, 1453038460000],
    [54780000, 1453098873000],
    [54790001, 1453132481000],
    [54800002, 1453204722000],
    [54810004, 1453257872000],
    [54820000, 1453302010000],
    [54830002, 1453372832000],
    [54840001, 1453411706000],
    [54850001, 1453470798000],
    [54860007, 1453522524000],
    [54870000, 1453556436000],
    [54880002, 1453604239000],
    [54890001, 1453638389000],
    [54900000, 1453683740000],
    [54910004, 1453731139000],
    [54920001, 1453798773000],
    [54930000, 1453830095000],
    [54940000, 1453900095000],
    [54950000, 1453960642000],
    [54960002, 1453996322000],
    [54970000, 1454066614000],
    [54980000, 1454101949000],
    [54990002, 1454153860000],
    [55000000, 1454178375000],
    [55010003, 1454233202000],
    [55020000, 1454253981000],
    [55030000, 1454318469000],
    [55040000, 1454344713000],
    [55050000, 1454413707000],
    [55060000, 1454444292000],
    [55070000, 1454503062000],
    [55080004, 1454545766000],
    [55090000, 1454592044000],
    [55100000, 1454640952000],
    [55110000, 1454680957000],
    [55120000, 1454731019000],
    [55130002, 1454764695000],
    [55140000, 1454807309000],
    [55150001, 1454845415000],
    [55160002, 1454873571000],
    [55170001, 1454936377000],
    [55180007, 1454994613000],
    [55190000, 1455030538000],
    [55200008, 1455098217000],
    [55210001, 1455132058000],
    [55220001, 1455190327000],
    [55230000, 1455224267000],
    [55240002, 1455284238000],
    [55250001, 1455337972000],
    [55260000, 1455371967000],
    [55270000, 1455402129000],
    [55280001, 1455438260000],
    [55290002, 1455457052000],
    [55300000, 1455479181000],
    [55310001, 1455539860000],
    [55320007, 1455593116000],
    [55330001, 1455634142000],
    [55340000, 1455699982000],
    [55350000, 1455732305000],
    [55360000, 1455801485000],
    [55370001, 1455862899000],
    [55380001, 1455897144000],
    [55390000, 1455960059000],
    [55400003, 1455986485000],
    [55410000, 1456044639000],
    [55420000, 1456068698000],
    [55430000, 1456133038000],
    [55440002, 1456155786000],
    [55450003, 1456225718000],
    [55460000, 1456273371000],
    [55470001, 1456324697000],
    [55480000, 1456393085000],
    [55490004, 1456439002000],
    [55500002, 1456496132000],
    [55510001, 1456554911000],
    [55520001, 1456585844000],
    [55530000, 1456641519000],
    [55540001, 1456669586000],
    [55550000, 1456731043000],
    [55560000, 1456758846000],
    [55570000, 1456825629000],
    [55580000, 1456860157000],
    [55590000, 1456926706000],
    [55600000, 1456989100000],
    [55610001, 1457020517000],
    [55620005, 1457091333000],
    [55630000, 1457138155000],
    [55640002, 1457181847000],
    [55650000, 1457222400000],
    [55660001, 1457262443000],
    [55670002, 1457284199000],
    [55680000, 1457351979000],
    [55690002, 1457396717000],
    [55700000, 1457445365000],
    [55710001, 1457502688000],
    [55720002, 1457533722000],
    [55730001, 1457592521000],
    [55740005, 1457622716000],
    [55750000, 1457688390000],
    [55760002, 1457713904000],
    [55770005, 1457770918000],
    [55780002, 1457796696000],
    [55790000, 1457854875000],
    [55800002, 1457881729000],
    [55810004, 1457947485000],
    [55820000, 1457973754000],
    [55830000, 1458043691000],
    [55840003, 1458095965000],
    [55850001, 1458140031000],
    [55860000, 1458209267000],
    [55870000, 1458249917000],
    [55880000, 1458308641000],
    [55890001, 1458363092000],
    [55900000, 1458397132000],
    [55910000, 1458450051000],
    [55920001, 1458482981000],
    [55930000, 1458537698000],
    [55940001, 1458569645000],
    [55950001, 1458632632000],
    [55960002, 1458663443000],
    [55970002, 1458733391000],
    [55980000, 1458781771000],
    [55990001, 1458828772000],
    [56000001, 1458891213000],
    [56010006, 1458922690000],
    [56020001, 1458985543000],
    [56030000, 1459013112000],
    [56040001, 1459071569000],
    [56050001, 1459095916000],
    [56060000, 1459163470000],
    [56070000, 1459195288000],
    [56080000, 1459256338000],
    [56090000, 1459314017000],
    [56100000, 1459351265000],
    [56110001, 1459417661000],
    [56120000, 1459444333000],
    [56130001, 1459507802000],
    [56140000, 1459534425000],
    [56150002, 1459595264000],
    [56160002, 1459623731000],
    [56170000, 1459678424000],
    [56180001, 1459701527000],
    [56190002, 1459766357000],
    [56200000, 1459804779000],
    [56210000, 1459863795000],
    [56220000, 1459926641000],
    [56230002, 1459959385000],
    [56240001, 1460029929000],
    [56250002, 1460079251000],
    [56260000, 1460127601000],
    [56270002, 1460187551000],
    [56280000, 1460216748000],
    [56290000, 1460273543000],
    [56300001, 1460297446000],
    [56310000, 1460365122000],
    [56320000, 1460415361000],
    [56330002, 1460474858000],
    [56340001, 1460551981000],
    [56350001, 1460622390000],
    [56360002, 1460680949000],
    [56370002, 1460735958000],
    [56380001, 1460804923000],
    [56390000, 1460843792000],
    [56400003, 1460890412000],
    [56410000, 1460916070000],
    [56420000, 1460987720000],
    [56430000, 1461058187000],
    [56440000, 1461104036000],
    [56450001, 1461164452000],
    [56460001, 1461239528000],
    [56470001, 1461299694000],
    [56480000, 1461340726000],
    [56490002, 1461404556000],
    [56500000, 1461430336000],
    [56510000, 1461485723000],
    [56520001, 1461509999000],
    [56530000, 1461579857000],
    [56540001, 1461622012000],
    [56550002, 1461680516000],
    [56560003, 1461750402000],
    [56570000, 1461783666000],
    [56580001, 1461851082000],
    [56590001, 1461905468000],
    [56600000, 1461939707000],
    [56610000, 1461992012000],
    [56620000, 1462024178000],
    [56630000, 1462064072000],
    [56640001, 1462100062000],
    [56650000, 1462118558000],
    [56660000, 1462178165000],
    [56670000, 1462202537000],
    [56680002, 1462258590000],
    [56690000, 1462287637000],
    [56700000, 1462343808000],
    [56710000, 1462374018000],
    [56720000, 1462425879000],
    [56730000, 1462455521000],
    [56740000, 1462512257000],
    [56750000, 1462547701000],
    [56760000, 1462606894000],
    [56770000, 1462634736000],
    [56780000, 1462687724000],
    [56790000, 1462715398000],
    [56800001, 1462776332000],
    [56810000, 1462811630000],
    [56820000, 1462885637000],
    [56830000, 1462947935000],
    [56840001, 1462985941000],
    [56850003, 1463060847000],
    [56860002, 1463130472000],
    [56870001, 1463167081000],
    [56880000, 1463227574000],
    [56890000, 1463267664000],
    [56900000, 1463310778000],
    [56910002, 1463340423000],
    [56920000, 1463407866000],
    [56930000, 1463480118000],
    [56940001, 1463536446000],
    [56950001, 1463586244000],
    [56960000, 1463661415000],
    [56970000, 1463724491000],
    [56980000, 1463762636000],
    [56990002, 1463827553000],
    [57000000, 1463858294000],
    [57010000, 1463911420000],
    [57020000, 1463935233000],
    [57030000, 1464008061000],
    [57040000, 1464057361000],
    [57050000, 1464100893000],
    [57060000, 1464172598000],
    [57070000, 1464221271000],
    [57080000, 1464274322000],
    [57090002, 1464346599000],
    [57100000, 1464389456000],
    [57110000, 1464437876000],
    [57120001, 1464482219000],
    [57130000, 1464522044000],
    [57140001, 1464562495000],
    [57150002, 1464619102000],
    [57160000, 1464692561000],
    [57170003, 1464741541000],
    [57180000, 1464792809000],
    [57190001, 1464866840000],
    [57200000, 1464923308000],
    [57210001, 1464968228000],
    [57220002, 1465031928000],
    [57230000, 1465058901000],
    [57240000, 1465116687000],
    [57250000, 1465139412000],
    [57260000, 1465209025000],
    [57270000, 1465250979000],
    [57280002, 1465308208000],
    [57290000, 1465377991000],
    [57300002, 1465411049000],
    [57310000, 1465477312000],
    [57320000, 1465531970000],
    [57330000, 1465571088000],
    [57340002, 1465629183000],
    [57350000, 1465657956000],
    [57360001, 1465712022000],
    [57370002, 1465739702000],
    [57380003, 1465798169000],
    [57390000, 1465832711000],
    [57400002, 1465905402000],
    [57410001, 1465957217000],
    [57420000, 1466002806000],
    [57430002, 1466072473000],
    [57440000, 1466108320000],
    [57450003, 1466170932000],
    [57460000, 1466222949000],
    [57470000, 1466258707000],
    [57480002, 1466307682000],
    [57490002, 1466340883000],
    [57500001, 1466392225000],
    [57510000, 1466435119000],
    [57520000, 1466506445000],
    [57530002, 1466555125000],
    [57540001, 1466606467000],
    [57550001, 1466677968000],
    [57560000, 1466726102000],
    [57570001, 1466780549000],
    [57580000, 1466842875000],
    [57590000, 1466872426000],
    [57600003, 1466930708000],
    [57610000, 1466957296000],
    [57620001, 1467031096000],
    [57630004, 1467094259000],
    [57640001, 1467130515000],
    [57650002, 1467202266000],
    [57660000, 1467256271000],
    [57670000, 1467298424000],
    [57680001, 1467362753000],
    [57690001, 1467391224000],
    [57700001, 1467451186000],
    [57710000, 1467477064000],
    [57720001, 1467533842000],
    [57730001, 1467559523000],
    [57740000, 1467630123000],
    [57750000, 1467683320000],
    [57760001, 1467731359000],
    [57770000, 1467803600000],
    [57780000, 1467846628000],
    [57790001, 1467898220000],
    [57800002, 1467956415000],
    [57810004, 1467994201000],
    [57820000, 1468058296000],
    [57830000, 1468088599000],
    [57840000, 1468145940000],
    [57850000, 1468172938000],
    [57860000, 1468242988000],
    [57870000, 1468308685000],
    [57880001, 1468343747000],
    [57890000, 1468415336000],
    [57900000, 1468480127000],
    [57910002, 1468515175000],
    [57920000, 1468588085000],
    [57930000, 1468644842000],
    [57940000, 1468681404000],
    [57950001, 1468740473000],
    [57960000, 1468769888000],
    [57970000, 1468830116000],
    [57980000, 1468856568000],
    [57990000, 1468930009000],
    [58000000, 1468991107000],
    [58010000, 1469029824000],
    [58020000, 1469102160000],
    [58030001, 1469157377000],
    [58040002, 1469201597000],
    [58050001, 1469270090000],
    [58060000, 1469317980000],
    [58070002, 1469364513000],
    [58080003, 1469424004000],
    [58090000, 1469464510000],
    [58100001, 1469536743000],
    [58110006, 1469599305000],
    [58120000, 1469638732000],
    [58130000, 1469710329000],
    [58140003, 1469771253000],
    [58150000, 1469807649000],
    [58160000, 1469873493000],
    [58170000, 1469909390000],
    [58180000, 1469963816000],
    [58190002, 1469992257000],
    [58200000, 1470054609000],
    [58210001, 1470105154000],
    [58220001, 1470146582000],
    [58230000, 1470204408000],
    [58240001, 1470238306000],
    [58250001, 1470306237000],
    [58260000, 1470348275000],
    [58270005, 1470402711000],
    [58280000, 1470454252000],
    [58290002, 1470491835000],
    [58300000, 1470543014000],
    [58310000, 1470576227000],
    [58320004, 1470626364000],
    [58330000, 1470665253000],
    [58340000, 1470719718000],
    [58350006, 1470752430000],
    [58360000, 1470804496000],
    [58370000, 1470838465000],
    [58380001, 1470886060000],
    [58390001, 1470920134000],
    [58400001, 1470962532000],
    [58410000, 1471007534000],
    [58420001, 1471062944000],
    [58430001, 1471100546000],
    [58440000, 1471163099000],
    [58450000, 1471193115000],
    [58460000, 1471256200000],
    [58470000, 1471285401000],
    [58480001, 1471346036000],
    [58490006, 1471379846000],
    [58500000, 1471436710000],
    [58510003, 1471483845000],
    [58520000, 1471527272000],
    [58530000, 1471579773000],
    [58540002, 1471616931000],
    [58550001, 1471668133000],
    [58560001, 1471701994000],
    [58570001, 1471754111000],
    [58580004, 1471788458000],
    [58590000, 1471847283000],
    [58600000, 1471879199000],
    [58610001, 1471946052000],
    [58620000, 1471990282000],
    [58630000, 1472046646000],
    [58640002, 1472111711000],
    [58650000, 1472149702000],
    [58660005, 1472218227000],
    [58670001, 1472276534000],
    [58680001, 1472311268000],
    [58690001, 1472371312000],
    [58700004, 1472398255000],
    [58710000, 1472465816000],
    [58720000, 1472507006000],
    [58730003, 1472562516000],
    [58740002, 1472612751000],
    [58750001, 1472652473000],
    [58760002, 1472710864000],
    [58770001, 1472744177000],
    [58780000, 1472815308000],
    [58790002, 1472854851000],
    [58800001, 1472904365000],
    [58810002, 1472946637000],
    [58820003, 1472990669000],
    [58830000, 1473039462000],
    [58840001, 1473088018000],
    [58850001, 1473162209000],
    [58860001, 1473219832000],
    [58870001, 1473262856000],
    [58880001, 1473337075000],
    [58890000, 1473398787000],
    [58900004, 1473438865000],
    [58910004, 1473503559000],
    [58920000, 1473529656000],
    [58930000, 1473585805000],
    [58940000, 1473610448000],
    [58950000, 1473682349000],
    [58960000, 1473737224000],
    [58970000, 1473780525000],
    [58980001, 1473853142000],
    [58990000, 1473903889000],
    [59000002, 1473949244000],
    [59010001, 1474015896000],
    [59020000, 1474050478000],
    [59030001, 1474112215000],
    [59040000, 1474157059000],
    [59050002, 1474202612000],
    [59060001, 1474252489000],
    [59070005, 1474291094000],
    [59080001, 1474352936000],
    [59090000, 1474388153000],
    [59100000, 1474464391000],
    [59110001, 1474523991000],
    [59120000, 1474556977000],
    [59130003, 1474630670000],
    [59140000, 1474683088000],
    [59150000, 1474725202000],
    [59160005, 1474779589000],
    [59170000, 1474812561000],
    [59180000, 1474883449000],
    [59190000, 1474935867000],
    [59200000, 1474989339000],
    [59210001, 1475064332000],
    [59220000, 1475129668000],
    [59230000, 1475170305000],
    [59240002, 1475244164000],
    [59250001, 1475303419000],
    [59260001, 1475334591000],
    [59270000, 1475392316000],
    [59280005, 1475419783000],
    [59290000, 1475488696000],
    [59300000, 1475528870000],
    [59310001, 1475590468000],
    [59320000, 1475657907000],
    [59330001, 1475686232000],
    [59340000, 1475757932000],
    [59350003, 1475812066000],
    [59360004, 1475852837000],
    [59370000, 1475911406000],
    [59380001, 1475941922000],
    [59390001, 1476004293000],
    [59400000, 1476034568000],
    [59410000, 1476096636000],
    [59420000, 1476129978000],
    [59430000, 1476196619000],
    [59440000, 1476272537000],
    [59450000, 1476340363000],
    [59460004, 1476384975000],
    [59470000, 1476456661000],
    [59480003, 1476521187000],
    [59490002, 1476552341000],
    [59500000, 1476612768000],
    [59510000, 1476644723000],
    [59520002, 1476714278000],
    [59530003, 1476789295000],
    [59540004, 1476854219000],
    [59550000, 1476898255000],
    [59560002, 1476972714000],
    [59570001, 1477045122000],
    [59580000, 1477092184000],
    [59590002, 1477141991000],
    [59600001, 1477195190000],
    [59610000, 1477230360000],
    [59620000, 1477300850000],
    [59630000, 1477358100000],
    [59640000, 1477410899000],
    [59650000, 1477489171000],
    [59660000, 1477563831000],
    [59670001, 1477626136000],
    [59680000, 1477670133000],
    [59690000, 1477737118000],
    [59700003, 1477771839000],
    [59710001, 1477825224000],
    [59720000, 1477844574000],
    [59730000, 1477908183000],
    [59740001, 1477926263000],
    [59750000, 1477992314000],
    [59760000, 1478035514000],
    [59770000, 1478098882000],
    [59780000, 1478165969000],
    [59790000, 1478199515000],
    [59800003, 1478269516000],
    [59810001, 1478333879000],
    [59820000, 1478366872000],
    [59830000, 1478428778000],
    [59840001, 1478470292000],
    [59850000, 1478530563000],
    [59860001, 1478607047000],
    [59870000, 1478676582000],
    [59880001, 1478740581000],
    [59890001, 1478793768000],
    [59900001, 1478867764000],
    [59910001, 1478918808000],
    [59920002, 1478960589000],
    [59930003, 1479018936000],
    [59940000, 1479048927000],
    [59950000, 1479121186000],
    [59960000, 1479182550000],
    [59970005, 1479229974000],
    [59980001, 1479307323000],
    [59990000, 1479384685000],
    [60000000, 1479455423000],
    [60010000, 1479514144000],
    [60020001, 1479564619000],
    [60030000, 1479623969000],
    [60040000, 1479654902000],
    [60050000, 1479732185000],
    [60060000, 1479804841000],
    [60070000, 1479857950000],
    [60080000, 1479907841000],
    [60090000, 1479980658000],
    [60100000, 1480047221000],
    [60110002, 1480096515000],
    [60120002, 1480164784000],
    [60130004, 1480221877000],
    [60140007, 1480256393000],
    [60150000, 1480327810000],
    [60160001, 1480389639000],
    [60170000, 1480437775000],
    [60180000, 1480515550000],
    [60190001, 1480590152000],
    [60200001, 1480656554000],
    [60210001, 1480701685000],
    [60220000, 1480769066000],
    [60230001, 1480823736000],
    [60240000, 1480859801000],
    [60250000, 1480928761000],
    [60260001, 1480986080000],
    [60270000, 1481040525000],
    [60280000, 1481118864000],
    [60290000, 1481195165000],
    [60300000, 1481265395000],
    [60310000, 1481317642000],
    [60320000, 1481376091000],
    [60330001, 1481434401000],
    [60340002, 1481467474000],
    [60350000, 1481542146000],
    [60360000, 1481611462000],
    [60370000, 1481658725000],
    [60380000, 1481727720000],
    [60390000, 1481805784000],
    [60400001, 1481879887000],
    [60410000, 1481937290000],
    [60420000, 1481984144000],
    [60430002, 1482044400000],
    [60440000, 1482075538000],
    [60450002, 1482151308000],
    [60460000, 1482217539000],
    [60470000, 1482260096000],
    [60480001, 1482331019000],
    [60490002, 1482403591000],
    [60500000, 1482456355000],
    [60510000, 1482500440000],
    [60520000, 1482546777000],
    [60530000, 1482582859000],
    [60540000, 1482601298000],
    [60550000, 1482652348000],
    [60560000, 1482674545000],
    [60570001, 1482721794000],
    [60580000, 1482762956000],
    [60590000, 1482820331000],
    [60600002, 1482851372000],
    [60610001, 1482910028000],
    [60620000, 1482939295000],
    [60630000, 1483000868000],
    [60640002, 1483028900000],
    [60650000, 1483091458000],
    [60660000, 1483122307000],
    [60670001, 1483175760000],
    [60680004, 1483196425000],
    [60690000, 1483210669000],
    [60700003, 1483254919000],
    [60710000, 1483280373000],
    [60720001, 1483331831000],
    [60730000, 1483365638000],
    [60740000, 1483419372000],
    [60750000, 1483451305000],
    [60760000, 1483503879000],
    [60770000, 1483538081000],
    [60780000, 1483594341000],
    [60790003, 1483628720000],
    [60800001, 1483696774000],
    [60810002, 1483726056000],
    [60820001, 1483785559000],
    [60830005, 1483815367000],
    [60840001, 1483873749000],
    [60850001, 1483903342000],
    [60860000, 1483959039000],
    [60870001, 1484000637000],
    [60880001, 1484058261000],
    [60890002, 1484130896000],
    [60900000, 1484187151000],
    [60910000, 1484235037000],
    [60920000, 1484310570000],
    [60930000, 1484369465000],
    [60940001, 1484406004000],
    [60950002, 1484465204000],
    [60960000, 1484493581000],
    [60970001, 1484568233000],
    [60980000, 1484632631000],
    [60990000, 1484673727000],
    [61000000, 1484747006000],
    [61010000, 1484818553000],
    [61020000, 1484873088000],
    [61030004, 1484924798000],
    [61040000, 1484989157000],
    [61050000, 1485019482000],
    [61060003, 1485079433000],
    [61070003, 1485106552000],
    [61080000, 1485176741000],
    [61090001, 1485243764000],
    [61100002, 1485280302000],
    [61110001, 1485351326000],
    [61120002, 1485419493000],
    [61130005, 1485452399000],
    [61140000, 1485520824000],
    [61150000, 1485569319000],
    [61160001, 1485609703000],
    [61170000, 1485661493000],
    [61180000, 1485696254000],
    [61190001, 1485756890000],
    [61200000, 1485791360000],
    [61210000, 1485864250000],
    [61220000, 1485916122000],
    [61230000, 1485961290000],
    [61240002, 1486029393000],
    [61250001, 1486060297000],
    [61260000, 1486126376000],
    [61270000, 1486176696000],
    [61280000, 1486215506000],
    [61290000, 1486266720000],
    [61300001, 1486299874000],
    [61310000, 1486353467000],
    [61320000, 1486393351000],
    [61330003, 1486463624000],
    [61340003, 1486507596000],
    [61350002, 1486561760000],
    [61360001, 1486626809000],
    [61370000, 1486657775000],
    [61380000, 1486727915000],
    [61390000, 1486773392000],
    [61400000, 1486816368000],
    [61410000, 1486865070000],
    [61420000, 1486904647000],
    [61430000, 1486965714000],
    [61440000, 1486999592000],
    [61450000, 1487058450000],
    [61460000, 1487082681000],
    [61470000, 1487141957000],
    [61480002, 1487177786000],
    [61490000, 1487249996000],
    [61500000, 1487315623000],
    [61510004, 1487351938000],
    [61520000, 1487417611000],
    [61530000, 1487464184000],
    [61540000, 1487506450000],
    [61550000, 1487558601000],
    [61560001, 1487602436000],
    [61570002, 1487673379000],
    [61580000, 1487729561000],
    [61590001, 1487773681000],
    [61600001, 1487840488000],
    [61610000, 1487886123000],
    [61620000, 1487946620000],
    [61630000, 1488009110000],
    [61640000, 1488041915000],
    [61650000, 1488104050000],
    [61660000, 1488137656000],
    [61670003, 1488203273000],
    [61680000, 1488272954000],
    [61690004, 1488306578000],
    [61700001, 1488374500000],
    [61710002, 1488439391000],
    [61720000, 1488476649000],
    [61730003, 1488546944000],
    [61740000, 1488604970000],
    [61750000, 1488640963000],
    [61760001, 1488700486000],
    [61770000, 1488727677000],
    [61780000, 1488799826000],
    [61790001, 1488857988000],
    [61800000, 1488900604000],
    [61810001, 1488975206000],
    [61820000, 1489034828000],
    [61830000, 1489072603000],
    [61840003, 1489142367000],
    [61850002, 1489193541000],
    [61860000, 1489239443000],
    [61870000, 1489294279000],
    [61880001, 1489327105000],
    [61890001, 1489392956000],
    [61900000, 1489426148000],
    [61910001, 1489495011000],
    [61920001, 1489548258000],
    [61930000, 1489589470000],
    [61940001, 1489659435000],
    [61950001, 1489705940000],
    [61960000, 1489759515000],
    [61970000, 1489814949000],
    [61980000, 1489848301000],
    [61990002, 1489899476000],
    [62000000, 1489932248000],
    [62010000, 1489989312000],
    [62020001, 1490024376000],
    [62030001, 1490098245000],
    [62040001, 1490161753000],
    [62050000, 1490201923000],
    [62060000, 1490274647000],
    [62070002, 1490341815000],
    [62080001, 1490380510000],
    [62090000, 1490445446000],
    [62100000, 1490500749000],
    [62110001, 1490536871000],
    [62120000, 1490603274000],
    [62130000, 1490642055000],
    [62140004, 1490708949000],
    [62150001, 1490777542000],
    [62160000, 1490820295000],
    [62170003, 1490881477000],
    [62180001, 1490946798000],
    [62190004, 1490976957000],
    [62200002, 1491039126000],
    [62210001, 1491067568000],
    [62220000, 1491128642000],
    [62230002, 1491159785000],
    [62240004, 1491224760000],
    [62250000, 1491283628000],
    [62260000, 1491320899000],
    [62270002, 1491393053000],
    [62280002, 1491453235000],
    [62290000, 1491494583000],
    [62300000, 1491567593000],
    [62310000, 1491623151000],
    [62320002, 1491661534000],
    [62330002, 1491716479000],
    [62340003, 1491746742000],
    [62350000, 1491815038000],
    [62360001, 1491871127000],
    [62370001, 1491925389000],
    [62380000, 1492003693000],
    [62390000, 1492082003000],
    [62400000, 1492151690000],
    [62410002, 1492198376000],
    [62420000, 1492260748000],
    [62430000, 1492312442000],
    [62440000, 1492348959000],
    [62450000, 1492413755000],
    [62460000, 1492453035000],
    [62470002, 1492526215000],
    [62480000, 1492599361000],
    [62490001, 1492660202000],
    [62500000, 1492705541000],
    [62510000, 1492781135000],
    [62520001, 1492841436000],
    [62530002, 1492874780000],
    [62540002, 1492933762000],
    [62550001, 1492960691000],
    [62560000, 1493033974000],
    [62570001, 1493093628000],
    [62580002, 1493136732000],
    [62590000, 1493211622000],
    [62600000, 1493276444000],
    [62610003, 1493312435000],
    [62620000, 1493385440000],
    [62630001, 1493438388000],
    [62640000, 1493473315000],
    [62650000, 1493522097000],
    [62660000, 1493555085000],
    [62670001, 1493586549000],
    [62680003, 1493641931000],
    [62690003, 1493681520000],
    [62700002, 1493731183000],
    [62710000, 1493778538000],
    [62720000, 1493818359000],
    [62730003, 1493872141000],
    [62740000, 1493910022000],
    [62750000, 1493968043000],
    [62760001, 1493997052000],
    [62770000, 1494057461000],
    [62780000, 1494086198000],
    [62790000, 1494144157000],
    [62800000, 1494170300000],
    [62810000, 1494245199000],
    [62820001, 1494308647000],
    [62830000, 1494351165000],
    [62840000, 1494425972000],
    [62850000, 1494501435000],
    [62860000, 1494567101000],
    [62870000, 1494610007000],
    [62880001, 1494673577000],
    [62890000, 1494716102000],
    [62900001, 1494760899000],
    [62910002, 1494795763000],
    [62920000, 1494858716000],
    [62930001, 1494933038000],
    [62940001, 1494999035000],
    [62950000, 1495045116000],
    [62960000, 1495117455000],
    [62970002, 1495192286000],
    [62980001, 1495245451000],
    [62990000, 1495288662000],
    [63000000, 1495342893000],
    [63010001, 1495375426000],
    [63020000, 1495443878000],
    [63030001, 1495488210000],
    [63040002, 1495550606000],
    [63050000, 1495621709000],
    [63060000, 1495673523000],
    [63070003, 1495725794000],
    [63080001, 1495800279000],
    [63090001, 1495855801000],
    [63100000, 1495896259000],
    [63110000, 1495956096000],
    [63120004, 1495984862000],
    [63130000, 1496058038000],
    [63140001, 1496116229000],
    [63150000, 1496159773000],
    [63160001, 1496236152000],
    [63170001, 1496303987000],
    [63180001, 1496350329000],
    [63190002, 1496414214000],
    [63200000, 1496477730000],
    [63210001, 1496509683000],
    [63220000, 1496568699000],
    [63230002, 1496593843000],
    [63240001, 1496667827000],
    [63250000, 1496730997000],
    [63260002, 1496768310000],
    [63270000, 1496841898000],
    [63280000, 1496910562000],
    [63290001, 1496945416000],
    [63300000, 1497013621000],
    [63310000, 1497063495000],
    [63320002, 1497102454000],
    [63330002, 1497153633000],
    [63340000, 1497187604000],
    [63350001, 1497249375000],
    [63360000, 1497286640000],
    [63370000, 1497359990000],
    [63380000, 1497426544000],
    [63390000, 1497463041000],
    [63400001, 1497533901000],
    [63410000, 1497600873000],
    [63420001, 1497634004000],
    [63430000, 1497695549000],
    [63440000, 1497730358000],
    [63450001, 1497783481000],
    [63460001, 1497819943000],
    [63470004, 1497881556000],
    [63480001, 1497952328000],
    [63490006, 1498004419000],
    [63500001, 1498055950000],
    [63510000, 1498127512000],
    [63520000, 1498184357000],
    [63530000, 1498232682000],
    [63540000, 1498297671000],
    [63550001, 1498328857000],
    [63560000, 1498384089000],
    [63570004, 1498408425000],
    [63580000, 1498481408000],
    [63590001, 1498547374000],
    [63600002, 1498592418000],
    [63610001, 1498659877000],
    [63620000, 1498732262000],
    [63630000, 1498789573000],
    [63640000, 1498834886000],
    [63650000, 1498893446000],
    [63660000, 1498924888000],
    [63670001, 1498984295000],
    [63680000, 1499011669000],
    [63690002, 1499084640000],
    [63700001, 1499145935000],
    [63710000, 1499184277000],
    [63720001, 1499256812000],
    [63730000, 1499319244000],
    [63740000, 1499358325000],
    [63750000, 1499427606000],
    [63760000, 1499473779000],
    [63770000, 1499518404000],
    [63780000, 1499570137000],
    [63790000, 1499606468000],
    [63800001, 1499667532000],
    [63810001, 1499705198000],
    [63820000, 1499777692000],
    [63830000, 1499842313000],
    [63840000, 1499880848000],
    [63850000, 1499952096000],
    [63860001, 1500019471000],
    [63870000, 1500060664000],
    [63880000, 1500121683000],
    [63890003, 1500175988000],
    [63900002, 1500214550000],
    [63910000, 1500271015000],
    [63920000, 1500302259000],
    [63930002, 1500369797000],
    [63940002, 1500417210000],
    [63950000, 1500474447000],
    [63960001, 1500541939000],
    [63970004, 1500591859000],
    [63980001, 1500646704000],
    [63990000, 1500702985000],
    [64000000, 1500737424000],
    [64010000, 1500797122000],
    [64020003, 1500824521000],
    [64030000, 1500894449000],
    [64040001, 1500944980000],
    [64050001, 1500993080000],
    [64060000, 1501059445000],
    [64070000, 1501096994000],
    [64080000, 1501161638000],
    [64090001, 1501220721000],
    [64100002, 1501255441000],
    [64110000, 1501317242000],
    [64120000, 1501347076000],
    [64130000, 1501406103000],
    [64140000, 1501432171000],
    [64150000, 1501497471000],
    [64160003, 1501534700000],
    [64170000, 1501589484000],
    [64180001, 1501640305000],
    [64190000, 1501682663000],
    [64200001, 1501740921000],
    [64210000, 1501774003000],
    [64220001, 1501840191000],
    [64230003, 1501876461000],
    [64240001, 1501933039000],
    [64250000, 1501975601000],
    [64260000, 1502018981000],
    [64270002, 1502054247000],
    [64280000, 1502108300000],
    [64290000, 1502147937000],
    [64300001, 1502196328000],
    [64310000, 1502239978000],
    [64320001, 1502283677000],
    [64330001, 1502327192000],
    [64340000, 1502369852000],
    [64350001, 1502414935000],
    [64360000, 1502456746000],
    [64370000, 1502507657000],
    [64380000, 1502545058000],
    [64390000, 1502599020000],
    [64400002, 1502635191000],
    [64410001, 1502694201000],
    [64420002, 1502725088000],
    [64430000, 1502784283000],
    [64440000, 1502811813000],
    [64450000, 1502873020000],
    [64460003, 1502900305000],
    [64470000, 1502965681000],
    [64480000, 1502994637000],
    [64490001, 1503057862000],
    [64500004, 1503094715000],
    [64510003, 1503141827000],
    [64520000, 1503177780000],
    [64530000, 1503229320000],
    [64540001, 1503269341000],
    [64550000, 1503322079000],
    [64560000, 1503380947000],
    [64570000, 1503417451000],
    [64580000, 1503487289000],
    [64590000, 1503541439000],
    [64600000, 1503585532000],
    [64610001, 1503651473000],
    [64620001, 1503691200000],
    [64630000, 1503749613000],
    [64640001, 1503798428000],
    [64650000, 1503837777000],
    [64660000, 1503888823000],
    [64670000, 1503931542000],
    [64680000, 1503998660000],
    [64690000, 1504037284000],
    [64700000, 1504099208000],
    [64710000, 1504145326000],
    [64720002, 1504185871000],
    [64730000, 1504235552000],
    [64740000, 1504278115000],
    [64750000, 1504338313000],
    [64760000, 1504368311000],
    [64770000, 1504426725000],
    [64780002, 1504453083000],
    [64790004, 1504525423000],
    [64800000, 1504581975000],
    [64810000, 1504625026000],
    [64820000, 1504696220000],
    [64830000, 1504746792000],
    [64840003, 1504794394000],
    [64850001, 1504862330000],
    [64860000, 1504893632000],
    [64870002, 1504950575000],
    [64880000, 1504977342000],
    [64890000, 1505032864000],
    [64900001, 1505057676000],
    [64910000, 1505128148000],
    [64920000, 1505179616000],
    [64930000, 1505227942000],
    [64940001, 1505298552000],
    [64950000, 1505350667000],
    [64960000, 1505402075000],
    [64970000, 1505475364000],
    [64980000, 1505527756000],
    [64990000, 1505567634000],
    [65000000, 1505619955000],
    [65010000, 1505653358000],
    [65020002, 1505700308000],
    [65030002, 1505739476000],
    [65040000, 1505798668000],
    [65050001, 1505841437000],
    [65060002, 1505916471000],
    [65070000, 1505990197000],
    [65080000, 1506053515000],
    [65090003, 1506098572000],
    [65100000, 1506165418000],
    [65110000, 1506213077000],
    [65120001, 1506257441000],
    [65130004, 1506317830000],
    [65140002, 1506361335000],
    [65150002, 1506434369000],
    [65160000, 1506505630000],
    [65170001, 1506559410000],
    [65180000, 1506611217000],
    [65190000, 1506685849000],
    [65200000, 1506741954000],
    [65210000, 1506781740000],
    [65220000, 1506837965000],
    [65230001, 1506868001000],
    [65240001, 1506935676000],
    [65250002, 1506967887000],
    [65260001, 1507038169000],
    [65270000, 1507105220000],
    [65280001, 1507137257000],
    [65290000, 1507208805000],
    [65300000, 1507266104000],
    [65310002, 1507303384000],
    [65320001, 1507359758000],
    [65330002, 1507389726000],
    [65340000, 1507452124000],
    [65350004, 1507486625000],
    [65360003, 1507548526000],
    [65370000, 1507595053000],
    [65380000, 1507647739000],
    [65390000, 1507724768000],
    [65400002, 1507796920000],
    [65410001, 1507852793000],
    [65420000, 1507908618000],
    [65430000, 1507976799000],
    [65440000, 1508026300000],
    [65450000, 1508071551000],
    [65460002, 1508138699000],
    [65470003, 1508194279000],
    [65480000, 1508253836000],
    [65490000, 1508329397000],
    [65500000, 1508402756000],
    [65510000, 1508461111000],
    [65520000, 1508513731000],
    [65530000, 1508580291000],
    [65540001, 1508622937000],
    [65550002, 1508672935000],
    [65560000, 1508723206000],
    [65570000, 1508771508000],
    [65580003, 1508849150000],
    [65590000, 1508921514000],
    [65600001, 1508982148000],
    [65610000, 1509035394000],
    [65620003, 1509113929000],
    [65630001, 1509181233000],
    [65640000, 1509225439000],
    [65650001, 1509277179000],
    [65660001, 1509320755000],
    [65670000, 1509375281000],
    [65680001, 1509435848000],
    [65690000, 1509461170000],
    [65700000, 1509525368000],
    [65710000, 1509572960000],
    [65720000, 1509634932000],
    [65730000, 1509702456000],
    [65740000, 1509748102000],
    [65750002, 1509802062000],
    [65760000, 1509860261000],
    [65770001, 1509894005000],
    [65780001, 1509970166000],
    [65790001, 1510043296000],
    [65800000, 1510099873000],
    [65810002, 1510155255000],
    [65820000, 1510233192000],
    [65830000, 1510307681000],
    [65840000, 1510362918000],
    [65850002, 1510406391000],
    [65860000, 1510458601000],
    [65870000, 1510496232000],
    [65880003, 1510569457000],
    [65890000, 1510633348000],
    [65900000, 1510683341000],
    [65910000, 1510758402000],
    [65920001, 1510837965000],
    [65930000, 1510913169000],
    [65940001, 1510976767000],
    [65950000, 1511019100000],
    [65960000, 1511084403000],
    [65970000, 1511132352000],
    [65980001, 1511193070000],
    [65990002, 1511274902000],
    [66000000, 1511355236000],
    [66010001, 1511424780000],
    [66020000, 1511486569000],
    [66030001, 1511548859000],
    [66040000, 1511618201000],
    [66050000, 1511682667000],
    [66060000, 1511716807000],
    [66070000, 1511794805000],
    [66080000, 1511876394000],
    [66090000, 1511953844000],
    [66100000, 1512027193000],
    [66110001, 1512082204000],
    [66120000, 1512142392000],
    [66130002, 1512212602000],
    [66140001, 1512268916000],
    [66150000, 1512310671000],
    [66160000, 1512383781000],
    [66170003, 1512452862000],
    [66180001, 1512514503000],
    [66190000, 1512576281000],
    [66200003, 1512656239000],
    [66210000, 1512735127000],
    [66220000, 1512801376000],
    [66230000, 1512841118000],
    [66240001, 1512906660000],
    [66250001, 1512970369000],
    [66260001, 1513026996000],
    [66270000, 1513093478000],
    [66280001, 1513174934000],
    [66290000, 1513255346000],
    [66300001, 1513332121000],
    [66310000, 1513398412000],
    [66320002, 1513439931000],
    [66330001, 1513506007000],
    [66340000, 1513557857000],
    [66350000, 1513615875000],
    [66360003, 1513694759000],
    [66370001, 1513773236000],
    [66380000, 1513847414000],
    [66390000, 1513911314000],
    [66400002, 1513960253000],
    [66410002, 1514028888000],
    [66420002, 1514072781000],
    [66430000, 1514113979000],
    [66440000, 1514132751000],
    [66450000, 1514194504000],
    [66460000, 1514217822000],
    [66470001, 1514287260000],
    [66480001, 1514332191000],
    [66490000, 1514381665000],
    [66500002, 1514436757000],
    [66510001, 1514473371000],
    [66520001, 1514536420000],
    [66530000, 1514571264000],
    [66540000, 1514634599000],
    [66550001, 1514681448000],
    [66560000, 1514718967000],
    [66570001, 1514733848000],
    [66580000, 1514770953000],
    [66590001, 1514805113000],
    [66600000, 1514838239000],
    [66610000, 1514896743000],
    [66620001, 1514951564000],
    [66630002, 1514989931000],
    [66640000, 1515053617000],
    [66650000, 1515088457000],
    [66660001, 1515158299000],
    [66670002, 1515218400000],
    [66680000, 1515253066000],
    [66690000, 1515315406000],
    [66700001, 1515344449000],
    [66710001, 1515405256000],
    [66720000, 1515436566000],
    [66730002, 1515506638000],
    [66740003, 1515577451000],
    [66750004, 1515636081000],
    [66760000, 1515686408000],
    [66770000, 1515763229000],
    [66780000, 1515824048000],
    [66790000, 1515859245000],
    [66800000, 1515923095000],
    [66810002, 1515962501000],
    [66820000, 1516028988000],
    [66830000, 1516105801000],
    [66840000, 1516175859000],
    [66850000, 1516226300000],
    [66860001, 1516288122000],
    [66870000, 1516364367000],
    [66880000, 1516427246000],
    [66890002, 1516462797000],
    [66900000, 1516524145000],
    [66910000, 1516553175000],
    [66920000, 1516625683000],
    [66930000, 1516689581000],
    [66940001, 1516728074000],
    [66950003, 1516801407000],
    [66960001, 1516872999000],
    [66970000, 1516930948000],
    [66980000, 1516980600000],
    [66990000, 1517045188000],
    [67000003, 1517081791000],
    [67010002, 1517139796000],
    [67020002, 1517192905000],
    [67030000, 1517242650000],
    [67040000, 1517319211000],
    [67050000, 1517392855000],
    [67060000, 1517450939000],
    [67070000, 1517499186000],
    [67080001, 1517571947000],
    [67090001, 1517626970000],
    [67100002, 1517667339000],
    [67110001, 1517726092000],
    [67120003, 1517756647000],
    [67130002, 1517828941000],
    [67140001, 1517889529000],
    [67150000, 1517934672000],
    [67160000, 1518009187000],
    [67170002, 1518080642000],
    [67180000, 1518137129000],
    [67190000, 1518189353000],
    [67200000, 1518253579000],
    [67210003, 1518285361000],
    [67220000, 1518347750000],
    [67230000, 1518393591000],
    [67240000, 1518438178000],
    [67250000, 1518496710000],
    [67260002, 1518535498000],
    [67270000, 1518594649000],
    [67280001, 1518619642000],
    [67290000, 1518680031000],
    [67300001, 1518715800000],
    [67310000, 1518787067000],
    [67320002, 1518846975000],
    [67330000, 1518882031000],
    [67340000, 1518942703000],
    [67350000, 1518969582000],
    [67360000, 1519041746000],
    [67370000, 1519099005000],
    [67380000, 1519140204000],
    [67390000, 1519211860000],
    [67400000, 1519260688000],
    [67410002, 1519306946000],
    [67420001, 1519362722000],
    [67430002, 1519399402000],
    [67440000, 1519459261000],
    [67450000, 1519488534000],
    [67460000, 1519550053000],
    [67470001, 1519582526000],
    [67480003, 1519652081000],
    [67490001, 1519721536000],
    [67500003, 1519767915000],
    [67510001, 1519826491000],
    [67520001, 1519891484000],
    [67530001, 1519927907000],
    [67540000, 1519998020000],
    [67550000, 1520056025000],
    [67560000, 1520089627000],
    [67570001, 1520148400000],
    [67580001, 1520176867000],
    [67590005, 1520247852000],
    [67600000, 1520304428000],
    [67610000, 1520349746000],
    [67620000, 1520422811000],
    [67630000, 1520486827000],
    [67640000, 1520524749000],
    [67650000, 1520595483000],
    [67660001, 1520646573000],
    [67670000, 1520690096000],
    [67680000, 1520746182000],
    [67690001, 1520779081000],
    [67700000, 1520848667000],
    [67710002, 1520899906000],
    [67720000, 1520952228000],
    [67730001, 1521021293000],
    [67740000, 1521057070000],
    [67750000, 1521120782000],
    [67760000, 1521184574000],
    [67770000, 1521218836000],
    [67780000, 1521281394000],
    [67790000, 1521316367000],
    [67800000, 1521373329000],
    [67810000, 1521423767000],
    [67820001, 1521471600000],
    [67830001, 1521542819000],
    [67840000, 1521598520000],
    [67850000, 1521638974000],
    [67860000, 1521702616000],
    [67870000, 1521742100000],
    [67880000, 1521811971000],
    [67890001, 1521872786000],
    [67900000, 1521906671000],
    [67910000, 1521969555000],
    [67920007, 1522001670000],
    [67930000, 1522070406000],
    [67940000, 1522135837000],
    [67950000, 1522173584000],
    [67960002, 1522242791000],
    [67970000, 1522306809000],
    [67980000, 1522344460000],
    [67990001, 1522412594000],
    [68000000, 1522468069000],
    [68010000, 1522506554000],
    [68020000, 1522557559000],
    [68030000, 1522590706000],
    [68040000, 1522650575000],
    [68050000, 1522685331000],
    [68060000, 1522756281000],
    [68070000, 1522813717000],
    [68080000, 1522856141000],
    [68090003, 1522928876000],
    [68100000, 1522988245000],
    [68110000, 1523029133000],
    [68120001, 1523093331000],
    [68130002, 1523125014000],
    [68140001, 1523183935000],
    [68150001, 1523227230000],
    [68160000, 1523284539000],
    [68170002, 1523358484000],
    [68180000, 1523420575000],
    [68190001, 1523466033000],
    [68200001, 1523541144000],
    [68210000, 1523615555000],
    [68220000, 1523672390000],
    [68230000, 1523713946000],
    [68240000, 1523766455000],
    [68250001, 1523798848000],
    [68260000, 1523863242000],
    [68270001, 1523901468000],
    [68280000, 1523973786000],
    [68290001, 1524047744000],
    [68300001, 1524107018000],
    [68310000, 1524154044000],
    [68320001, 1524229042000],
    [68330000, 1524286922000],
    [68340000, 1524323212000],
    [68350000, 1524381473000],
    [68360000, 1524409014000],
    [68370000, 1524478747000],
    [68380001, 1524525543000],
    [68390000, 1524580850000],
    [68400000, 1524653994000],
    [68410000, 1524709045000],
    [68420000, 1524755100000],
    [68430000, 1524826488000],
    [68440000, 1524865483000],
    [68450000, 1524916337000],
    [68460001, 1524953059000],
    [68470000, 1524999613000],
    [68480001, 1525025191000],
    [68490000, 1525080615000],
    [68500002, 1525101539000],
    [68510003, 1525162635000],
    [68520002, 1525187749000],
    [68530000, 1525245718000],
    [68540000, 1525273420000],
    [68550000, 1525327908000],
    [68560001, 1525359119000],
    [68570002, 1525416288000],
    [68580000, 1525447801000],
    [68590001, 1525506649000],
    [68600004, 1525534849000],
    [68610000, 1525591851000],
    [68620003, 1525618511000],
    [68630000, 1525643849000],
    [68640001, 1525701190000],
    [68650000, 1525767545000],
    [68660000, 1525808401000],
    [68670000, 1525876015000],
    [68680000, 1525949150000],
    [68690000, 1526007825000],
    [68700000, 1526056163000],
    [68710000, 1526122821000],
    [68720000, 1526169299000],
    [68730000, 1526210632000],
    [68740001, 1526245158000],
    [68750000, 1526306217000],
    [68760000, 1526373997000],
    [68770001, 1526422460000],
    [68780000, 1526482497000],
    [68790000, 1526556215000],
    [68800001, 1526614484000],
    [68810000, 1526660277000],
    [68820001, 1526726230000],
    [68830000, 1526766179000],
    [68840000, 1526815391000],
    [68850000, 1526860749000],
    [68860001, 1526914592000],
    [68870001, 1526986814000],
    [68880000, 1527046837000],
    [68890007, 1527089275000],
    [68900002, 1527162614000],
    [68910000, 1527220624000],
    [68920000, 1527263825000],
    [68930000, 1527328049000],
    [68940001, 1527361209000],
    [68950004, 1527416538000],
    [68960000, 1527442251000],
    [68970000, 1527513976000],
    [68980001, 1527583168000],
    [68990000, 1527634736000],
    [69000000, 1527691902000],
    [69010001, 1527766421000],
    [69020000, 1527823668000],
    [69030000, 1527868950000],
    [69040000, 1527935212000],
    [69050000, 1527978877000],
    [69060000, 1528025307000],
    [69070002, 1528064391000],
    [69080001, 1528123377000],
    [69090000, 1528198458000],
    [69100000, 1528260415000],
    [69110001, 1528302226000],
    [69120000, 1528377728000],
    [69130000, 1528447154000],
    [69140002, 1528488138000],
    [69150000, 1528545941000],
    [69160000, 1528592623000],
    [69170000, 1528632255000],
    [69180000, 1528680956000],
    [69190005, 1528729587000],
    [69200000, 1528803749000],
    [69210000, 1528862130000],
    [69220000, 1528905872000],
    [69230001, 1528981229000],
    [69240000, 1529046681000],
    [69250000, 1529088379000],
    [69260000, 1529150795000],
    [69270000, 1529200936000],
    [69280000, 1529240724000],
    [69290000, 1529300926000],
    [69300003, 1529343151000],
    [69310000, 1529416036000],
    [69320002, 1529486838000],
    [69330000, 1529539015000],
    [69340000, 1529593237000],
    [69350001, 1529667628000],
    [69360000, 1529722072000],
    [69370002, 1529763311000],
    [69380002, 1529818645000],
    [69390000, 1529851102000],
    [69400001, 1529920747000],
    [69410001, 1529975408000],
    [69420002, 1530027810000],
    [69430000, 1530103971000],
    [69440001, 1530173255000],
    [69450000, 1530224811000],
    [69460000, 1530283275000],
    [69470000, 1530342956000],
    [69480002, 1530373895000],
    [69490000, 1530432807000],
    [69500000, 1530461484000],
    [69510002, 1530536210000],
    [69520001, 1530605315000],
    [69530002, 1530658087000],
    [69540000, 1530716093000],
    [69550001, 1530790105000],
    [69560000, 1530854239000],
    [69570000, 1530895115000],
    [69580000, 1530957423000],
    [69590000, 1530988766000],
    [69600000, 1531046898000],
    [69610000, 1531091459000],
    [69620002, 1531148189000],
    [69630004, 1531223860000],
    [69640000, 1531291270000],
    [69650000, 1531340220000],
    [69660000, 1531407337000],
    [69670000, 1531481045000],
    [69680000, 1531538663000],
    [69690004, 1531581179000],
    [69700004, 1531642256000],
    [69710000, 1531674430000],
    [69720001, 1531736842000],
    [69730000, 1531780044000],
    [69740000, 1531839242000],
    [69750002, 1531912872000],
    [69760000, 1531978052000],
    [69770001, 1532024345000],
    [69780000, 1532096007000],
    [69790001, 1532160363000],
    [69800000, 1532195393000],
    [69810001, 1532255368000],
    [69820001, 1532295023000],
    [69830001, 1532355365000],
    [69840004, 1532425955000],
    [69850001, 1532481458000],
    [69860002, 1532531549000],
    [69870001, 1532604726000],
    [69880002, 1532664516000],
    [69890000, 1532707078000],
    [69900000, 1532772952000],
    [69910001, 1532811097000],
    [69920001, 1532864268000],
    [69930000, 1532911701000],
    [69940001, 1532961232000],
    [69950000, 1533028668000],
    [69960000, 1533065978000],
    [69970000, 1533128703000],
    [69980000, 1533187983000],
    [69990000, 1533224798000],
    [70000001, 1533293626000],
    [70010000, 1533341881000],
    [70020002, 1533388770000],
    [70030003, 1533441440000],
    [70040003, 1533475406000],
    [70050000, 1533530540000],
    [70060000, 1533568278000],
    [70070001, 1533634816000],
    [70080000, 1533663670000],
    [70090001, 1533727595000],
    [70100000, 1533754702000],
    [70110001, 1533815063000],
    [70120001, 1533853793000],
    [70130001, 1533907802000],
    [70140001, 1533965009000],
    [70150000, 1534003521000],
    [70160001, 1534068838000],
    [70170003, 1534116792000],
    [70180000, 1534167758000],
    [70190000, 1534226116000],
    [70200000, 1534261750000],
    [70210000, 1534324674000],
    [70220001, 1534354181000],
    [70230002, 1534417528000],
    [70240001, 1534457754000],
    [70250000, 1534510811000],
    [70260000, 1534559909000],
    [70270004, 1534599747000],
    [70280000, 1534654766000],
    [70290000, 1534689851000],
    [70300000, 1534757233000],
    [70310002, 1534799115000],
    [70320000, 1534859703000],
    [70330001, 1534925365000],
    [70340000, 1534965074000],
    [70350000, 1535029985000],
    [70360002, 1535091868000],
    [70370000, 1535129089000],
    [70380001, 1535193273000],
    [70390000, 1535240430000],
    [70400001, 1535286462000],
    [70410000, 1535341070000],
    [70420002, 1535383883000],
    [70430000, 1535457290000],
    [70440000, 1535518458000],
    [70450000, 1535560250000],
    [70460000, 1535633551000],
    [70470001, 1535689162000],
    [70480001, 1535727872000],
    [70490000, 1535789665000],
    [70500002, 1535821987000],
    [70510000, 1535882378000],
    [70520000, 1535916475000],
    [70530001, 1535983603000],
    [70540000, 1536050503000],
    [70550000, 1536088917000],
    [70560000, 1536157094000],
    [70570000, 1536231786000],
    [70580000, 1536296501000],
    [70590000, 1536340085000],
    [70600002, 1536406617000],
    [70610000, 1536455059000],
    [70620000, 1536495553000],
    [70630001, 1536547843000],
    [70640000, 1536593072000],
    [70650000, 1536668563000],
    [70660000, 1536736908000],
    [70670001, 1536780704000],
    [70680000, 1536848733000],
    [70690000, 1536921949000],
    [70700001, 1536973292000],
    [70710001, 1537017778000],
    [70720000, 1537069850000],
    [70730000, 1537107049000],
    [70740000, 1537164128000],
    [70750000, 1537196639000],
    [70760000, 1537270358000],
    [70770000, 1537331917000],
    [70780001, 1537378349000],
    [70790000, 1537452583000],
    [70800002, 1537526264000],
    [70810000, 1537581934000],
    [70820000, 1537627198000],
    [70830002, 1537688897000],
    [70840000, 1537722143000],
    [70850000, 1537780273000],
    [70860000, 1537804597000],
    [70870000, 1537872500000],
    [70880000, 1537917687000],
    [70890000, 1537971507000],
    [70900001, 1538039774000],
    [70910000, 1538089855000],
    [70920000, 1538147137000],
    [70930000, 1538210737000],
    [70940000, 1538242739000],
    [70950000, 1538297704000],
    [70960000, 1538320420000],
    [70970000, 1538386328000],
    [70980000, 1538422301000],
    [70990000, 1538489010000],
    [71000000, 1538556741000],
    [71010001, 1538597401000],
    [71020000, 1538661225000],
    [71030000, 1538728102000],
    [71040000, 1538761843000],
    [71050000, 1538820539000],
    [71060002, 1538854718000],
    [71070000, 1538911539000],
    [71080002, 1538957736000],
    [71090000, 1539001928000],
    [71100000, 1539061140000],
    [71110003, 1539102071000],
    [71120001, 1539174711000],
    [71130003, 1539239471000],
    [71140000, 1539279611000],
    [71150003, 1539352332000],
    [71160002, 1539417036000],
    [71170001, 1539450581000],
    [71180000, 1539511324000],
    [71190000, 1539540615000],
    [71200000, 1539610136000],
    [71210000, 1539681105000],
    [71220001, 1539738088000],
    [71230000, 1539788591000],
    [71240001, 1539861270000],
    [71250000, 1539922323000],
    [71260003, 1539968212000],
    [71270000, 1540035844000],
    [71280000, 1540088174000],
    [71290000, 1540127974000],
    [71300000, 1540194402000],
    [71310000, 1540241544000],
    [71320000, 1540306298000],
    [71330001, 1540382709000],
    [71340000, 1540449555000],
    [71350001, 1540496571000],
    [71360000, 1540564024000],
    [71370000, 1540627992000],
    [71380000, 1540662656000],
    [71390000, 1540724624000],
    [71400002, 1540774195000],
    [71410002, 1540825418000],
    [71420000, 1540900181000],
    [71430001, 1540942900000],
    [71440000, 1540988683000],
    [71450000, 1541013088000],
    [71460001, 1541078575000],
    [71470000, 1541149141000],
    [71480000, 1541200241000],
    [71490000, 1541251238000],
    [71500000, 1541308607000],
    [71510000, 1541341872000],
    [71520000, 1541412652000],
    [71530000, 1541472532000],
    [71540000, 1541520635000],
    [71550000, 1541596851000],
    [71560001, 1541668759000],
    [71570000, 1541726871000],
    [71580001, 1541777843000],
    [71590000, 1541845928000],
    [71600002, 1541888736000],
    [71610000, 1541934928000],
    [71620000, 1541964051000],
    [71630000, 1542031527000],
    [71640000, 1542103996000],
    [71650000, 1542165525000],
    [71660001, 1542213349000],
    [71670001, 1542288816000],
    [71680000, 1542361750000],
    [71690000, 1542418474000],
    [71700000, 1542464082000],
    [71710000, 1542521873000],
    [71720000, 1542551494000],
    [71730001, 1542621946000],
    [71740002, 1542667578000],
    [71750000, 1542722500000],
    [71760000, 1542791211000],
    [71770000, 1542824840000],
    [71780003, 1542892813000],
    [71790000, 1542943408000],
    [71800000, 1542982468000],
    [71810000, 1543043560000],
    [71820006, 1543080139000],
    [71830000, 1543142640000],
    [71840000, 1543192718000],
    [71850002, 1543246324000],
    [71860003, 1543323743000],
    [71870001, 1543396323000],
    [71880000, 1543454076000],
    [71890004, 1543506855000],
    [71900000, 1543582963000],
    [71910000, 1543644113000],
    [71920001, 1543681587000],
    [71930001, 1543744321000],
    [71940001, 1543782600000],
    [71950001, 1543847396000],
    [71960000, 1543910919000],
    [71970002, 1543949491000],
    [71980000, 1544014940000],
    [71990002, 1544077125000],
    [72000002, 1544121143000],
    [72010003, 1544190546000],
    [72020000, 1544254822000],
    [72030000, 1544292984000],
    [72040001, 1544353258000],
    [72050000, 1544397195000],
    [72060000, 1544454946000],
    [72070000, 1544530756000],
    [72080002, 1544600454000],
    [72090000, 1544654112000],
    [72100000, 1544712024000],
    [72110000, 1544784886000],
    [72120000, 1544843542000],
    [72130002, 1544886475000],
    [72140000, 1544947989000],
    [72150000, 1544978552000],
    [72160000, 1545051850000],
    [72170000, 1545120134000],
    [72180000, 1545174503000],
    [72190000, 1545232427000],
    [72200000, 1545306855000],
    [72210000, 1545371245000],
    [72220000, 1545413187000],
    [72230000, 1545477616000],
    [72240001, 1545523994000],
    [72250000, 1545568189000],
    [72260001, 1545607132000],
    [72270001, 1545648313000],
    [72280000, 1545666632000],
    [72290000, 1545723283000],
    [72300002, 1545749364000],
    [72310000, 1545808557000],
    [72320001, 1545839607000],
    [72330003, 1545905923000],
    [72340001, 1545937307000],
    [72350001, 1545997652000],
    [72360002, 1546044137000],
    [72370001, 1546089745000],
    [72380000, 1546143723000],
    [72390000, 1546181512000],
    [72400000, 1546233763000],
    [72410000, 1546262657000],
    [72420000, 1546274077000],
    [72430000, 1546317453000],
    [72440000, 1546348206000],
    [72450000, 1546396049000],
    [72460000, 1546437174000],
    [72470000, 1546493281000],
    [72480000, 1546527962000],
    [72490000, 1546592449000],
    [72500000, 1546629592000],
    [72510001, 1546688629000],
    [72520000, 1546734599000],
    [72530004, 1546775131000],
    [72540001, 1546815138000],
    [72550002, 1546870083000],
    [72560000, 1546939278000],
    [72570000, 1546987244000],
    [72580000, 1547043211000],
    [72590003, 1547110805000],
    [72600000, 1547152350000],
    [72610000, 1547214363000],
    [72620001, 1547271736000],
    [72630000, 1547305600000],
    [72640001, 1547363893000],
    [72650000, 1547394641000],
    [72660000, 1547454693000],
    [72670001, 1547482786000],
    [72680000, 1547554701000],
    [72690000, 1547614636000],
    [72700000, 1547656970000],
    [72710000, 1547729496000],
    [72720000, 1547795122000],
    [72730000, 1547836724000],
    [72740000, 1547898735000],
    [72750001, 1547945548000],
    [72760000, 1547986028000],
    [72770002, 1548033768000],
    [72780000, 1548082371000],
    [72790001, 1548152633000],
    [72800000, 1548202317000],
    [72810000, 1548253713000],
    [72820001, 1548321357000],
    [72830000, 1548360891000],
    [72840000, 1548423665000],
    [72850001, 1548480132000],
    [72860002, 1548514935000],
    [72870001, 1548574649000],
    [72880000, 1548604181000],
    [72890001, 1548677054000],
    [72900001, 1548739463000],
    [72910000, 1548781107000],
    [72920002, 1548853626000],
    [72930000, 1548919152000],
    [72940000, 1548951077000],
    [72950000, 1549019715000],
    [72960001, 1549068329000],
    [72970003, 1549111971000],
    [72980001, 1549159968000],
    [72990000, 1549196188000],
    [73000000, 1549237832000],
    [73010002, 1549289058000],
    [73020002, 1549354695000],
    [73030005, 1549392741000],
    [73040000, 1549458883000],
    [73050003, 1549520105000],
    [73060002, 1549555854000],
    [73070000, 1549627217000],
    [73080001, 1549681748000],
    [73090000, 1549720758000],
    [73100002, 1549772898000],
    [73110000, 1549807132000],
    [73120002, 1549856659000],
    [73130000, 1549890810000],
    [73140001, 1549946094000],
    [73150001, 1549986080000],
    [73160000, 1550056521000],
    [73170000, 1550084186000],
    [73180001, 1550140671000],
    [73190001, 1550159800000],
    [73200001, 1550224779000],
    [73210001, 1550260933000],
    [73220000, 1550316429000],
    [73230000, 1550353828000],
    [73240001, 1550400558000],
    [73250000, 1550425730000],
    [73260001, 1550492995000],
    [73270000, 1550546568000],
    [73280000, 1550587970000],
    [73290000, 1550653467000],
    [73300001, 1550684542000],
    [73310001, 1550751102000],
    [73320000, 1550793473000],
    [73330001, 1550841517000],
    [73340001, 1550881298000],
    [73350002, 1550922499000],
    [73360001, 1550958074000],
    [73370000, 1551006541000],
    [73380000, 1551041014000],
    [73390001, 1551099431000],
    [73400002, 1551157089000],
    [73410000, 1551196014000],
    [73420000, 1551266496000],
    [73430001, 1551314543000],
    [73440000, 1551362590000],
    [73450002, 1551420873000],
    [73460000, 1551456707000],
    [73470002, 1551518312000],
    [73480001, 1551546935000],
    [73490000, 1551601645000],
    [73500000, 1551625574000],
    [73510001, 1551692198000],
    [73520001, 1551733459000],
    [73530000, 1551794253000],
    [73540000, 1551856997000],
    [73550002, 1551892186000],
    [73560001, 1551961298000],
    [73570003, 1552017331000],
    [73580000, 1552059548000],
    [73590000, 1552120744000],
    [73600000, 1552148547000],
    [73610000, 1552206052000],
    [73620001, 1552232092000],
    [73630002, 1552299864000],
    [73640000, 1552343880000],
    [73650000, 1552398957000],
    [73660001, 1552464772000],
    [73670000, 1552499469000],
    [73680000, 1552567147000],
    [73690000, 1552619001000],
    [73700000, 1552662176000],
    [73710000, 1552721307000],
    [73720000, 1552751752000],
    [73730000, 1552808226000],
    [73740000, 1552835248000],
    [73750000, 1552902939000],
    [73760001, 1552954284000],
    [73770000, 1553006331000],
    [73780000, 1553073115000],
    [73790000, 1553113343000],
    [73800001, 1553169072000],
    [73810000, 1553216422000],
    [73820000, 1553265664000],
    [73830001, 1553325454000],
    [73840001, 1553356922000],
    [73850000, 1553416024000],
    [73860000, 1553442284000],
    [73870000, 1553511233000],
    [73880001, 1553558316000],
    [73890000, 1553609553000],
    [73900001, 1553674157000],
    [73910001, 1553708617000],
    [73920000, 1553776601000],
    [73930000, 1553833131000],
    [73940001, 1553872563000],
    [73950003, 1553931861000],
    [73960001, 1553960734000],
    [73970000, 1554014776000],
    [73980000, 1554042148000],
    [73990000, 1554093422000],
    [74000000, 1554131333000],
    [74010000, 1554199206000],
    [74020000, 1554250709000],
    [74030000, 1554302489000],
    [74040001, 1554372228000],
    [74050000, 1554423465000],
    [74060000, 1554474600000],
    [74070000, 1554534629000],
    [74080000, 1554567156000],
    [74090001, 1554626234000],
    [74100000, 1554651766000],
    [74110002, 1554721970000],
    [74120000, 1554780065000],
    [74130000, 1554824868000],
    [74140001, 1554899176000],
    [74150003, 1554964923000],
    [74160000, 1555009812000],
    [74170000, 1555078977000],
    [74180000, 1555142601000],
    [74190000, 1555177120000],
    [74200000, 1555233822000],
    [74210000, 1555258531000],
    [74220000, 1555330359000],
    [74230000, 1555394652000],
    [74240000, 1555439575000],
    [74250002, 1555509015000],
    [74260000, 1555581255000],
    [74270000, 1555633380000],
    [74280001, 1555686276000],
    [74290000, 1555749001000],
    [74300000, 1555780505000],
    [74310000, 1555837371000],
    [74320001, 1555862390000],
    [74330000, 1555933600000],
    [74340000, 1555989889000],
    [74350000, 1556034502000],
    [74360001, 1556107770000],
    [74370000, 1556167348000],
    [74380000, 1556208193000],
    [74390000, 1556281031000],
    [74400001, 1556331452000],
    [74410004, 1556370777000],
    [74420000, 1556420818000],
    [74430000, 1556458605000],
    [74440000, 1556511455000],
    [74450000, 1556546313000],
    [74460001, 1556593795000],
    [74470000, 1556624857000],
    [74480000, 1556637699000],
    [74490001, 1556688527000],
    [74500000, 1556718811000],
    [74510001, 1556765309000],
    [74520000, 1556802015000],
    [74530002, 1556849353000],
    [74540002, 1556889643000],
    [74550002, 1556939986000],
    [74560000, 1556977364000],
    [74570000, 1557023415000],
    [74580000, 1557059372000],
    [74590000, 1557096900000],
    [74600001, 1557140408000],
    [74610000, 1557172924000],
    [74620003, 1557238020000],
    [74630000, 1557309534000],
    [74640000, 1557363295000],
    [74650000, 1557414782000],
    [74660001, 1557488614000],
    [74670001, 1557541721000],
    [74680002, 1557583324000],
    [74690001, 1557636382000],
    [74700000, 1557669411000],
    [74710000, 1557733896000],
    [74720000, 1557775631000],
    [74730000, 1557842303000],
    [74740000, 1557910870000],
    [74750000, 1557960886000],
    [74760000, 1558019461000],
    [74770000, 1558092814000],
    [74780000, 1558146423000],
    [74790000, 1558188463000],
    [74800001, 1558243324000],
    [74810000, 1558274890000],
    [74820002, 1558338702000],
    [74830000, 1558378003000],
    [74840000, 1558445935000],
    [74850002, 1558515890000],
    [74860001, 1558566351000],
    [74870001, 1558622469000],
    [74880003, 1558691372000],
    [74890000, 1558739188000],
    [74900002, 1558789053000],
    [74910001, 1558841401000],
    [74920000, 1558876318000],
    [74930001, 1558932416000],
    [74940001, 1558974705000],
    [74950000, 1559048815000],
    [74960000, 1559117085000],
    [74970001, 1559163704000],
    [74980000, 1559226208000],
    [74990000, 1559296147000],
    [75000000, 1559341839000],
    [75010000, 1559391087000],
    [75020001, 1559440340000],
    [75030000, 1559478346000],
    [75040001, 1559528747000],
    [75050000, 1559574491000],
    [75060002, 1559649834000],
    [75070000, 1559711830000],
    [75080000, 1559753905000],
    [75090002, 1559824921000],
    [75100000, 1559886875000],
    [75110004, 1559923687000],
    [75120000, 1559985944000],
    [75130000, 1560016514000],
    [75140000, 1560072057000],
    [75150001, 1560095010000],
    [75160001, 1560165185000],
    [75170003, 1560218438000],
    [75180000, 1560265400000],
    [75190000, 1560336925000],
    [75200000, 1560388429000],
    [75210001, 1560438284000],
    [75220000, 1560510607000],
    [75230000, 1560564573000],
    [75240000, 1560605980000],
    [75250000, 1560656737000],
    [75260001, 1560691792000],
    [75270000, 1560751011000],
    [75280001, 1560791029000],
    [75290000, 1560862353000],
    [75300000, 1560927205000],
    [75310001, 1560964659000],
    [75320000, 1561035219000],
    [75330000, 1561095592000],
    [75340000, 1561133284000],
    [75350001, 1561193649000],
    [75360000, 1561223789000],
    [75370002, 1561281019000],
    [75380000, 1561308233000],
    [75390001, 1561379138000],
    [75400000, 1561437826000],
    [75410000, 1561481017000],
    [75420000, 1561553132000],
    [75430000, 1561616259000],
    [75440000, 1561655761000],
    [75450001, 1561726404000],
    [75460000, 1561782418000],
    [75470000, 1561819049000],
    [75480000, 1561872925000],
    [75490000, 1561903030000],
    [75500003, 1561960317000],
    [75510001, 1561996854000],
    [75520002, 1562067922000],
    [75530002, 1562124498000],
    [75540000, 1562167665000],
    [75550001, 1562240504000],
    [75560000, 1562298567000],
    [75570000, 1562341575000],
    [75580000, 1562406542000],
    [75590000, 1562437694000],
    [75600004, 1562491887000],
    [75610000, 1562514825000],
    [75620003, 1562585131000],
    [75630000, 1562636339000],
    [75640000, 1562683191000],
    [75650002, 1562753859000],
    [75660000, 1562800478000],
    [75670000, 1562854095000],
    [75680000, 1562918120000],
    [75690001, 1562950801000],
    [75700000, 1563010535000],
    [75710002, 1563041761000],
    [75720002, 1563099712000],
    [75730002, 1563130877000],
    [75740001, 1563186079000],
    [75750000, 1563218209000],
    [75760000, 1563282794000],
    [75770000, 1563344879000],
    [75780001, 1563381629000],
    [75790000, 1563453956000],
    [75800000, 1563517085000],
    [75810000, 1563553688000],
    [75820000, 1563613736000],
    [75830003, 1563643643000],
    [75840000, 1563703356000],
    [75850000, 1563734344000],
    [75860001, 1563799753000],
    [75870001, 1563858045000],
    [75880000, 1563898194000],
    [75890000, 1563968622000],
    [75900000, 1564027533000],
    [75910001, 1564069923000],
    [75920000, 1564139955000],
    [75930000, 1564192711000],
    [75940000, 1564235050000],
    [75950000, 1564290089000],
    [75960001, 1564323613000],
    [75970002, 1564388017000],
    [75980000, 1564423582000],
    [75990000, 1564490729000],
    [76000000, 1564547542000],
    [76010000, 1564586012000],
    [76020002, 1564650865000],
    [76030000, 1564685184000],
    [76040000, 1564749129000],
    [76050000, 1564798987000],
    [76060000, 1564840050000],
    [76070000, 1564892912000],
    [76080000, 1564926788000],
    [76090001, 1564986889000],
    [76100001, 1565020129000],
    [76110000, 1565086406000],
    [76120000, 1565118741000],
    [76130002, 1565180359000],
    [76140000, 1565220758000],
    [76150001, 1565267646000],
    [76160002, 1565313456000],
    [76170000, 1565358718000],
    [76180000, 1565413169000],
    [76190000, 1565450269000],
    [76200001, 1565511983000],
    [76210001, 1565546923000],
    [76220002, 1565607908000],
    [76230001, 1565656671000],
    [76240000, 1565703462000],
    [76250001, 1565759227000],
    [76260001, 1565795841000],
    [76270000, 1565855146000],
    [76280000, 1565883889000],
    [76290000, 1565946107000],
    [76300000, 1565978408000],
    [76310000, 1566039436000],
    [76320002, 1566079948000],
    [76330003, 1566125560000],
    [76340000, 1566153619000],
    [76350001, 1566216889000],
    [76360000, 1566265859000],
    [76370000, 1566310563000],
    [76380000, 1566372197000],
    [76390000, 1566403363000],
    [76400000, 1566471275000],
    [76410001, 1566509566000],
    [76420001, 1566565068000],
    [76430000, 1566611680000],
    [76440001, 1566652231000],
    [76450003, 1566702182000],
    [76460001, 1566740305000],
    [76470000, 1566799829000],
    [76480000, 1566837513000],
    [76490000, 1566908438000],
    [76500000, 1566967629000],
    [76510002, 1567007870000],
    [76520000, 1567078974000],
    [76530003, 1567135140000],
    [76540000, 1567178242000],
    [76550000, 1567236718000],
    [76560000, 1567264875000],
    [76570000, 1567320844000],
    [76580000, 1567349250000],
    [76590000, 1567414982000],
    [76600000, 1567457714000],
    [76610000, 1567518907000],
    [76620000, 1567587982000],
    [76630002, 1567637441000],
    [76640000, 1567694711000],
    [76650000, 1567766416000],
    [76660002, 1567819638000],
    [76670005, 1567864904000],
    [76680002, 1567919174000],
    [76690000, 1567951278000],
    [76700001, 1568011851000],
    [76710002, 1568047453000],
    [76720000, 1568119256000],
    [76730003, 1568183199000],
    [76740001, 1568225531000],
    [76750000, 1568295584000],
    [76760000, 1568362361000],
    [76770002, 1568402570000],
    [76780000, 1568463499000],
    [76790000, 1568513761000],
    [76800001, 1568553173000],
    [76810002, 1568604342000],
    [76820001, 1568640092000],
    [76830001, 1568699299000],
    [76840001, 1568738525000],
    [76850003, 1568811673000],
    [76860000, 1568875384000],
    [76870000, 1568917822000],
    [76880000, 1568986806000],
    [76890002, 1569046157000],
    [76900000, 1569080246000],
    [76910001, 1569139762000],
    [76920001, 1569168747000],
    [76930000, 1569226998000],
    [76940000, 1569253569000],
    [76950000, 1569325964000],
    [76960000, 1569386704000],
    [76970000, 1569431406000],
    [76980001, 1569505424000],
    [76990002, 1569576236000],
    [77000002, 1569626464000],
    [77010000, 1569675238000],
    [77020000, 1569727667000],
    [77030000, 1569762931000],
    [77040000, 1569822825000],
    [77050000, 1569858533000],
    [77060000, 1569929216000],
    [77070000, 1569980978000],
    [77080002, 1570027438000],
    [77090001, 1570095720000],
    [77100001, 1570135068000],
    [77110000, 1570195330000],
    [77120000, 1570248898000],
    [77130001, 1570284981000],
    [77140002, 1570336741000],
    [77150000, 1570366946000],
    [77160000, 1570417375000],
    [77170000, 1570459051000],
    [77180000, 1570526961000],
    [77190001, 1570565629000],
    [77200002, 1570627858000],
    [77210000, 1570687028000],
    [77220000, 1570724875000],
    [77230001, 1570796190000],
    [77240000, 1570847380000],
    [77250001, 1570883067000],
    [77260000, 1570926372000],
    [77270000, 1570963263000],
    [77280000, 1570984991000],
    [77290000, 1571041957000],
    [77300001, 1571067701000],
    [77310000, 1571139043000],
    [77320001, 1571193128000],
    [77330000, 1571239423000],
    [77340005, 1571312887000],
    [77350000, 1571369811000],
    [77360002, 1571411950000],
    [77370000, 1571473839000],
    [77380000, 1571505313000],
    [77390000, 1571563317000],
    [77400000, 1571591232000],
    [77410000, 1571660678000],
    [77420000, 1571712562000],
    [77430003, 1571750399000],
    [77440000, 1571813293000],
    [77450000, 1571851607000],
    [77460001, 1571923655000],
    [77470000, 1571992236000],
    [77480000, 1572037784000],
    [77490000, 1572092494000],
    [77500000, 1572142437000],
    [77510000, 1572180148000],
    [77520001, 1572231861000],
    [77530000, 1572275768000],
    [77540000, 1572348116000],
    [77550001, 1572405568000],
    [77560000, 1572447709000],
    [77570001, 1572501282000],
    [77580000, 1572529172000],
    [77590001, 1572568204000],
    [77600000, 1572616006000],
    [77610000, 1572672492000],
    [77620001, 1572707631000],
    [77630000, 1572765930000],
    [77640000, 1572795506000],
    [77650002, 1572855145000],
    [77660003, 1572883288000],
    [77670000, 1572955653000],
    [77680002, 1573016599000],
    [77690001, 1573058789000],
    [77700001, 1573131780000],
    [77710002, 1573197783000],
    [77720000, 1573239769000],
    [77730000, 1573299891000],
    [77740000, 1573348388000],
    [77750000, 1573387544000],
    [77760000, 1573428601000],
    [77770000, 1573478893000],
    [77780003, 1573537253000],
    [77790000, 1573576677000],
    [77800000, 1573648497000],
    [77810000, 1573711114000],
    [77820000, 1573750607000],
    [77830001, 1573822482000],
    [77840000, 1573880260000],
    [77850001, 1573917037000],
    [77860000, 1573977296000],
    [77870000, 1574006064000],
    [77880000, 1574077861000],
    [77890000, 1574136265000],
    [77900000, 1574181839000],
    [77910001, 1574255379000],
    [77920000, 1574323713000],
    [77930002, 1574374417000],
    [77940000, 1574431990000],
    [77950000, 1574490397000],
    [77960002, 1574521934000],
    [77970000, 1574578831000],
    [77980000, 1574607595000],
    [77990001, 1574675031000],
    [78000000, 1574722236000],
    [78010000, 1574778123000],
    [78020000, 1574847408000],
    [78030001, 1574893044000],
    [78040000, 1574949601000],
    [78050000, 1575015696000],
    [78060001, 1575050002000],
    [78070000, 1575112026000],
    [78080000, 1575151289000],
    [78090005, 1575198305000],
    [78100004, 1575233752000],
    [78110000, 1575293230000],
    [78120000, 1575359472000],
    [78130000, 1575396543000],
    [78140000, 1575465710000],
    [78150001, 1575533610000],
    [78160000, 1575579854000],
    [78170000, 1575640986000],
    [78180000, 1575701599000],
    [78190000, 1575733850000],
    [78200001, 1575794252000],
    [78210000, 1575823317000],
    [78220000, 1575896218000],
    [78230000, 1575964173000],
    [78240000, 1576011655000],
    [78250000, 1576073933000],
    [78260001, 1576144389000],
    [78270000, 1576200749000],
    [78280000, 1576250393000],
    [78290000, 1576315055000],
    [78300000, 1576351491000],
    [78310001, 1576406488000],
    [78320000, 1576444214000],
    [78330000, 1576504997000],
    [78340002, 1576574146000],
    [78350000, 1576621106000],
    [78360002, 1576678062000],
    [78370001, 1576746527000],
    [78380000, 1576794604000],
    [78390002, 1576851721000],
    [78400000, 1576911721000],
    [78410004, 1576943227000],
    [78420000, 1577002401000],
    [78430001, 1577028867000],
    [78440001, 1577096620000],
    [78450000, 1577123671000],
    [78460000, 1577184292000],
    [78470001, 1577203292000],
    [78480000, 1577257657000],
    [78490001, 1577283436000],
    [78500000, 1577332465000],
    [78510000, 1577369168000],
    [78520000, 1577418022000],
    [78530001, 1577454785000],
    [78540000, 1577504358000],
    [78550000, 1577541845000],
    [78560002, 1577594977000],
    [78570000, 1577628750000],
    [78580000, 1577681692000],
    [78590000, 1577714761000],
    [78600000, 1577760133000],
    [78610001, 1577789089000],
    [78620000, 1577804530000],
    [78630000, 1577821920000],
    [78640000, 1577860275000],
    [78650001, 1577885897000],
    [78660000, 1577930220000],
    [78670001, 1577968102000],
    [78680000, 1578013562000],
    [78690000, 1578054428000],
    [78700000, 1578097732000],
    [78710000, 1578138315000],
    [78720000, 1578173994000],
    [78730001, 1578218433000],
    [78740001, 1578241469000],
    [78750000, 1578305688000],
    [78760000, 1578337485000],
    [78770000, 1578398338000],
    [78780000, 1578444020000],
    [78790000, 1578491624000],
    [78800001, 1578551468000],
    [78810000, 1578583667000],
    [78820000, 1578649262000],
    [78830000, 1578679125000],
    [78840001, 1578735714000],
    [78850000, 1578763096000],
    [78860001, 1578820711000],
    [78870000, 1578847582000],
    [78880000, 1578904548000],
    [78890001, 1578930264000],
    [78900000, 1578999366000],
    [78910000, 1579045378000],
    [78920001, 1579097395000],
    [78930000, 1579163961000],
    [78940001, 1579201038000],
    [78950000, 1579265770000],
    [78960001, 1579313475000],
    [78970003, 1579352400000],
    [78980000, 1579400812000],
    [78990000, 1579436538000],
    [79000003, 1579479371000],
    [79010001, 1579529582000],
    [79020000, 1579593758000],
    [79030000, 1579628044000],
    [79040000, 1579695257000],
    [79050000, 1579748926000],
    [79060000, 1579791612000],
    [79070000, 1579858764000],
    [79080001, 1579894275000],
    [79090000, 1579950627000],
    [79100002, 1579987385000],
    [79110000, 1580034338000],
    [79120003, 1580058941000],
    [79130000, 1580125354000],
    [79140000, 1580172351000],
    [79150000, 1580219627000],
    [79160000, 1580281139000],
    [79170001, 1580313877000],
    [79180000, 1580381019000],
    [79190000, 1580418004000],
    [79200000, 1580474093000],
    [79210001, 1580516349000],
    [79220001, 1580557493000],
    [79230001, 1580589474000],
    [79240000, 1580635332000],
    [79250000, 1580655762000],
    [79260000, 1580712913000],
    [79270000, 1580742007000],
    [79280002, 1580803170000],
    [79290001, 1580833015000],
    [79300001, 1580898048000],
    [79310000, 1580930509000],
    [79320000, 1580991151000],
    [79330000, 1581040569000],
    [79340001, 1581083505000],
    [79350000, 1581133125000],
    [79360000, 1581166802000],
    [79370000, 1581209820000],
    [79380000, 1581245802000],
    [79390000, 1581268960000],
    [79400000, 1581332594000],
    [79410000, 1581365287000],
    [79420000, 1581414114000],
    [79430000, 1581436278000],
    [79440001, 1581499724000],
    [79450000, 1581527485000],
    [79460002, 1581591921000],
    [79470000, 1581611896000],
    [79480002, 1581664302000],
    [79490000, 1581686246000],
    [79500000, 1581707828000],
    [79510001, 1581755219000],
    [79520000, 1581778876000],
    [79530001, 1581827495000],
    [79540001, 1581855277000],
    [79550000, 1581885174000],
    [79560000, 1581940328000],
    [79570000, 1581979931000],
    [79580000, 1582030075000],
    [79590001, 1582079484000],
    [79600001, 1582120357000],
    [79610000, 1582175729000],
    [79620001, 1582210202000],
    [79630001, 1582270110000],
    [79640001, 1582298785000],
    [79650000, 1582350093000],
    [79660000, 1582377404000],
    [79670000, 1582413033000],
    [79680000, 1582453470000],
    [79690000, 1582476625000],
    [79700000, 1582529117000],
    [79710000, 1582554052000],
    [79720000, 1582608340000],
    [79730000, 1582642389000],
    [79740000, 1582703142000],
    [79750000, 1582733106000],
    [79760000, 1582797396000],
    [79770000, 1582831796000],
    [79780000, 1582891176000],
    [79790000, 1582932604000],
    [79800002, 1582973282000],
    [79810000, 1582993603000],
    [79820000, 1583041517000],
    [79830001, 1583066674000],
    [79840000, 1583097399000],
    [79850000, 1583147549000],
    [79860001, 1583175322000],
    [79870000, 1583230930000],
    [79880001, 1583254210000],
    [79890000, 1583312479000],
    [79900000, 1583337435000],
    [79910000, 1583395809000],
    [79920002, 1583421490000],
    [79930000, 1583478558000],
    [79940000, 1583507226000],
    [79950003, 1583559493000],
    [79960001, 1583588210000],
    [79970000, 1583630350000],
    [79980004, 1583660784000],
    [79990000, 1583680236000],
    [80000000, 1583729581000],
    [80010001, 1583760195000],
    [80020000, 1583800746000],
    [80030001, 1583841420000],
    [80040001, 1583873476000],
    [80050002, 1583924963000],
    [80060001, 1583954200000],
    [80070003, 1584013619000],
    [80080002, 1584053110000],
    [80090000, 1584098273000],
    [80100001, 1584123740000],
    [80110000, 1584172919000],
    [80120000, 1584196207000],
    [80130000, 1584236963000],
    [80140000, 1584268538000],
    [80150000, 1584288460000],
    [80160004, 1584346855000],
    [80170000, 1584374092000],
    [80180000, 1584433499000],
    [80190000, 1584460885000],
    [80200001, 1584520112000],
    [80210000, 1584547904000],
    [80220001, 1584607409000],
    [80230002, 1584633704000],
    [80240001, 1584689221000],
    [80250001, 1584717343000],
    [80260000, 1584769637000],
    [80270000, 1584799213000],
    [80280001, 1584844859000],
    [80290004, 1584876896000],
    [80300001, 1584901795000],
    [80310000, 1584960974000],
    [80320000, 1584994959000],
    [80330000, 1585050808000],
    [80340001, 1585093198000],
    [80350000, 1585140198000],
    [80360000, 1585188885000],
    [80370000, 1585229910000],
    [80380000, 1585282846000],
    [80390000, 1585318135000],
    [80400000, 1585366586000],
    [80410000, 1585398047000],
    [80420000, 1585431266000],
    [80430000, 1585471931000],
    [80440000, 1585493157000],
    [80450000, 1585546072000],
    [80460000, 1585578800000],
    [80470001, 1585632499000],
    [80480000, 1585664658000],
    [80490000, 1585710656000],
    [80500000, 1585745275000],
    [80510000, 1585784723000],
    [80520001, 1585829200000],
    [80530001, 1585869472000],
    [80540000, 1585914540000],
    [80550000, 1585951327000],
    [80560000, 1585994226000],
    [80570000, 1586016746000],
    [80580001, 1586065316000],
    [80590000, 1586090070000],
    [80600000, 1586122548000],
    [80610001, 1586171687000],
    [80620001, 1586201295000],
    [80630001, 1586256244000],
    [80640000, 1586285461000],
    [80650001, 1586340914000],
    [80660000, 1586366212000],
    [80670000, 1586423226000],
    [80680003, 1586448333000],
    [80690001, 1586505875000],
    [80700000, 1586531864000],
    [80710002, 1586583545000],
    [80720000, 1586611358000],
    [80730001, 1586649157000],
    [80740000, 1586681597000],
    [80750000, 1586701176000],
    [80760000, 1586745365000],
    [80770000, 1586780528000],
    [80780000, 1586814400000],
    [80790000, 1586861417000],
    [80800000, 1586886787000],
    [80810000, 1586941592000],
    [80820000, 1586965088000],
    [80830001, 1587020675000],
    [80840001, 1587048413000],
    [80850004, 1587098709000],
    [80860001, 1587129781000],
    [80870000, 1587171493000],
    [80880000, 1587203816000],
    [80890000, 1587223661000],
    [80900000, 1587267891000],
    [80910001, 1587293885000],
    [80920000, 1587311373000],
    [80930000, 1587362365000],
    [80940000, 1587390170000],
    [80950000, 1587434564000],
    [80960004, 1587470310000],
    [80970001, 1587500856000],
    [80980000, 1587548549000],
    [80990001, 1587571495000],
    [81000000, 1587623222000],
    [81010000, 1587650355000],
    [81020000, 1587695608000],
    [81030000, 1587729330000],
    [81040000, 1587758238000],
    [81050002, 1587802832000],
    [81060000, 1587825583000],
    [81070001, 1587868470000],
    [81080000, 1587895722000],
    [81090001, 1587914447000],
    [81100001, 1587963144000],
    [81110000, 1587991996000],
    [81120001, 1588028404000],
    [81130002, 1588069594000],
    [81140001, 1588092759000],
    [81150000, 1588142605000],
    [81160002, 1588166470000],
    [81170000, 1588206886000],
    [81180000, 1588243564000],
    [81190000, 1588264193000],
    [81200000, 1588314060000],
    [81210001, 1588339106000],
    [81220000, 1588373618000],
    [81230000, 1588407867000],
    [81240000, 1588429734000],
    [81250000, 1588469180000],
    [81260000, 1588497556000],
    [81270000, 1588517856000],
    [81280000, 1588557854000],
    [81290000, 1588585602000],
    [81300001, 1588604457000],
    [81310000, 1588639853000],
    [81320001, 1588667014000],
    [81330000, 1588686356000],
    [81340000, 1588716907000],
    [81350001, 1588752094000],
    [81360001, 1588771880000],
    [81370000, 1588806874000],
    [81380001, 1588846871000],
    [81390000, 1588868932000],
    [81400000, 1588921819000],
    [81410001, 1588947698000],
    [81420000, 1588988258000],
    [81430002, 1589018884000],
    [81440001, 1589038387000],
    [81450000, 1589081675000],
    [81460001, 1589106242000],
    [81470001, 1589122621000],
    [81480000, 1589165020000],
    [81490000, 1589198298000],
    [81500000, 1589222553000],
    [81510001, 1589274227000],
    [81520003, 1589297244000],
    [81530001, 1589348690000],
    [81540002, 1589377233000],
    [81550001, 1589419759000],
    [81560000, 1589455384000],
    [81570000, 1589476947000],
    [81580000, 1589529412000],
    [81590001, 1589553375000],
    [81600000, 1589597520000],
    [81610000, 1589626092000],
    [81620000, 1589644519000],
    [81630000, 1589687424000],
    [81640000, 1589712385000],
    [81650000, 1589729438000],
    [81660000, 1589776069000],
    [81670000, 1589805780000],
    [81680000, 1589831247000],
    [81690000, 1589878317000],
    [81700000, 1589900422000],
    [81710000, 1589946751000],
    [81720001, 1589977852000],
    [81730000, 1590004142000],
    [81740000, 1590052120000],
    [81750000, 1590074143000],
    [81760000, 1590122002000],
    [81770000, 1590152424000],
    [81780000, 1590186109000],
    [81790000, 1590223220000],
    [81800000, 1590244149000],
    [81810001, 1590280602000],
    [81820000, 1590309802000],
    [81830000, 1590328866000],
    [81840000, 1590365787000],
    [81850001, 1590404238000],
    [81860000, 1590426625000],
    [81870002, 1590481067000],
    [81880001, 1590505421000],
    [81890001, 1590554488000],
    [81900000, 1590585537000],
    [81910003, 1590627640000],
    [81920000, 1590666740000],
    [81930002, 1590696478000],
    [81940001, 1590746190000],
    [81950001, 1590768658000],
    [81960000, 1590817160000],
    [81970000, 1590843166000],
    [81980000, 1590870668000],
    [81990000, 1590910285000],
    [82000001, 1590930718000],
    [82010000, 1590956360000],
    [82020001, 1591005181000],
    [82030002, 1591026660000],
    [82040000, 1591083085000],
    [82050000, 1591110373000],
    [82060000, 1591163968000],
    [82070000, 1591193751000],
    [82080001, 1591243811000],
    [82090000, 1591277157000],
    [82100001, 1591322136000],
    [82110000, 1591359589000],
    [82120001, 1591389164000],
    [82130000, 1591431941000],
    [82140000, 1591453966000],
    [82150000, 1591494520000],
    [82160000, 1591523716000],
    [82170000, 1591542092000],
    [82180006, 1591592338000],
    [82190000, 1591624821000],
    [82200001, 1591672179000],
    [82210002, 1591708023000],
    [82220000, 1591751015000],
    [82230000, 1591791293000],
    [82240000, 1591830419000],
    [82250000, 1591875827000],
    [82260001, 1591911847000],
    [82270000, 1591961077000],
    [82280000, 1591986746000],
    [82290000, 1592034683000],
    [82300001, 1592057862000],
    [82310001, 1592099204000],
    [82320000, 1592127432000],
    [82330000, 1592145964000],
    [82340000, 1592193978000],
    [82350001, 1592228903000],
    [82360002, 1592277287000],
    [82370001, 1592314508000],
    [82380002, 1592362674000],
    [82390000, 1592400722000],
    [82400000, 1592450790000],
    [82410000, 1592487981000],
    [82420002, 1592538424000],
    [82430000, 1592574401000],
    [82440002, 1592619918000],
    [82450000, 1592653527000],
    [82460001, 1592678101000],
    [82470001, 1592723845000],
    [82480000, 1592746447000],
    [82490002, 1592786843000],
    [82500003, 1592829927000],
    [82510003, 1592872839000],
    [82520000, 1592917116000],
    [82530000, 1592963754000],
    [82540000, 1593004368000],
    [82550000, 1593053016000],
    [82560000, 1593091205000],
    [82570000, 1593140653000],
    [82580000, 1593179016000],
    [82590000, 1593226109000],
    [82600000, 1593260110000],
    [82610000, 1593292880000],
    [82620000, 1593333710000],
    [82630001, 1593355075000],
    [82640000, 1593405247000],
    [82650000, 1593439573000],
    [82660000, 1593489699000],
    [82670001, 1593523829000],
    [82680001, 1593567734000],
    [82690002, 1593608143000],
    [82700001, 1593653460000],
    [82710001, 1593695518000],
    [82720000, 1593745692000],
    [82730000, 1593783686000],
    [82740000, 1593828572000],
    [82750000, 1593862380000],
    [82760001, 1593886312000],
    [82770000, 1593934010000],
    [82780000, 1593956231000],
    [82790002, 1593998674000],
    [82800000, 1594039473000],
    [82810000, 1594075207000],
    [82820000, 1594120404000],
    [82830000, 1594141724000],
    [82840001, 1594200917000],
    [82850001, 1594226720000],
    [82860000, 1594287260000],
    [82870000, 1594314277000],
    [82880000, 1594374352000],
    [82890001, 1594400826000],
    [82900000, 1594452807000],
    [82910003, 1594479187000],
    [82920000, 1594522939000],
    [82930000, 1594552179000],
    [82940000, 1594571960000],
    [82950000, 1594630946000],
    [82960000, 1594655608000],
    [82970001, 1594714679000],
    [82980000, 1594740707000],
    [82990002, 1594799679000],
    [83000000, 1594827097000],
    [83010000, 1594886333000],
    [83020000, 1594914451000],
    [83030000, 1594975465000],
    [83040000, 1595002269000],
    [83050000, 1595054245000],
    [83060000, 1595081829000],
    [83070004, 1595124763000],
    [83080000, 1595154475000],
    [83090001, 1595173905000],
    [83100003, 1595230572000],
    [83110000, 1595257562000],
    [83120001, 1595315003000],
    [83130003, 1595343775000],
    [83140001, 1595399483000],
    [83150000, 1595429817000],
    [83160003, 1595478091000],
    [83170000, 1595508448000],
    [83180000, 1595546386000],
    [83190000, 1595584953000],
    [83200000, 1595607597000],
    [83210000, 1595657153000],
    [83220001, 1595683315000],
    [83230000, 1595717647000],
    [83240000, 1595751744000],
    [83250000, 1595772715000],
    [83260000, 1595818547000],
    [83270002, 1595855249000],
    [83280001, 1595900392000],
    [83290004, 1595940913000],
    [83300001, 1595985442000],
    [83310000, 1596026279000],
    [83320000, 1596067464000],
    [83330000, 1596112206000],
    [83340000, 1596153373000],
    [83350000, 1596196765000],
    [83360000, 1596223051000],
    [83370000, 1596269059000],
    [83380001, 1596292424000],
    [83390000, 1596334336000],
    [83400000, 1596363703000],
    [83410000, 1596381794000],
    [83420000, 1596434923000],
    [83430000, 1596464523000],
    [83440001, 1596516746000],
    [83450000, 1596550377000],
    [83460000, 1596601180000],
    [83470000, 1596635514000],
    [83480001, 1596685585000],
    [83490003, 1596720358000],
    [83500000, 1596766201000],
    [83510000, 1596803270000],
    [83520000, 1596837133000],
    [83530000, 1596878712000],
    [83540000, 1596901234000],
    [83550000, 1596949353000],
    [83560003, 1596977164000],
    [83570000, 1597008775000],
    [83580000, 1597048325000],
    [83590000, 1597071327000],
    [83600001, 1597119083000],
    [83610001, 1597151951000],
    [83620001, 1597194929000],
    [83630000, 1597232364000],
    [83640001, 1597263295000],
    [83650000, 1597310339000],
    [83660001, 1597334343000],
    [83670001, 1597386089000],
    [83680000, 1597413459000],
    [83690003, 1597454239000],
    [83700000, 1597486103000],
    [83710008, 1597506740000],
    [83720000, 1597551335000],
    [83730000, 1597577788000],
    [83740001, 1597597200000],
    [83750000, 1597653183000],
    [83760002, 1597678126000],
    [83770000, 1597734775000],
    [83780001, 1597763020000],
    [83790000, 1597818557000],
    [83800000, 1597847957000],
    [83810000, 1597900828000],
    [83820000, 1597932895000],
    [83830000, 1597985003000],
    [83840000, 1598017642000],
    [83850000, 1598061692000],
    [83860000, 1598095381000],
    [83870000, 1598120266000],
    [83880000, 1598166455000],
    [83890002, 1598189754000],
    [83900000, 1598230443000],
    [83910000, 1598272013000],
    [83920001, 1598311996000],
    [83930001, 1598356953000],
    [83940000, 1598393233000],
    [83950001, 1598441409000],
    [83960000, 1598473930000],
    [83970001, 1598526618000],
    [83980000, 1598557164000],
    [83990000, 1598611628000],
    [84000001, 1598640860000],
    [84010001, 1598691060000],
    [84020000, 1598714144000],
    [84030000, 1598758204000],
    [84040000, 1598786391000],
    [84050000, 1598802968000],
    [84060000, 1598854786000],
    [84070001, 1598880613000],
    [84080000, 1598915240000],
    [84090000, 1598960805000],
    [84100000, 1598992906000],
    [84110000, 1599044803000],
    [84120000, 1599071788000],
    [84130000, 1599129363000],
    [84140000, 1599157743000],
    [84150000, 1599215826000],
    [84160000, 1599244993000],
    [84170000, 1599294970000],
    [84180001, 1599318855000],
    [84190002, 1599365651000],
    [84200000, 1599393085000],
    [84210000, 1599414750000],
    [84220000, 1599471537000],
    [84230000, 1599495506000],
    [84240000, 1599556292000],
    [84250000, 1599582680000],
    [84260001, 1599642678000],
    [84270003, 1599667852000],
    [84280001, 1599728801000],
    [84290000, 1599756746000],
    [84300001, 1599818638000],
    [84310000, 1599844574000],
    [84320000, 1599895309000],
    [84330002, 1599920152000],
    [84340003, 1599962736000],
    [84350000, 1599992375000],
    [84360000, 1600010654000],
    [84370000, 1600067378000],
    [84380000, 1600096395000],
    [84390000, 1600154553000],
    [84400001, 1600182935000],
    [84410000, 1600243243000],
    [84420000, 1600270065000],
    [84430000, 1600333148000],
    [84440001, 1600360786000],
    [84450000, 1600422881000],
    [84460000, 1600450161000],
    [84470000, 1600503314000],
    [84480000, 1600528307000],
    [84490001, 1600577352000],
    [84500000, 1600606321000],
    [84510004, 1600644331000],
    [84520000, 1600684992000],
    [84530001, 1600709066000],
    [84540001, 1600760266000],
    [84550000, 1600784472000],
    [84560003, 1600833748000],
    [84570000, 1600869873000],
    [84580000, 1600925480000],
    [84590000, 1600958971000],
    [84600000, 1601017833000],
    [84610000, 1601047273000],
    [84620000, 1601099559000],
    [84630000, 1601126922000],
    [84640000, 1601166841000],
    [84650000, 1601199955000],
    [84660000, 1601220002000],
    [84670000, 1601279997000],
    [84680000, 1601307991000],
    [84690002, 1601371822000],
    [84700000, 1601399958000],
    [84710000, 1601460743000],
    [84720002, 1601484176000],
    [84730001, 1601543158000],
    [84740001, 1601568062000],
    [84750000, 1601628361000],
    [84760001, 1601654620000],
    [84770000, 1601706617000],
    [84780000, 1601734222000],
    [84790002, 1601777347000],
    [84800000, 1601807920000],
    [84810001, 1601826134000],
    [84820001, 1601883369000],
    [84830000, 1601910614000],
    [84840001, 1601968183000],
    [84850000, 1601997403000],
    [84860000, 1602056593000],
    [84870000, 1602083577000],
    [84880002, 1602144536000],
    [84890000, 1602172352000],
    [84900000, 1602233741000],
    [84910001, 1602259465000],
    [84920000, 1602311733000],
    [84930000, 1602337315000],
    [84940000, 1602377797000],
    [84950000, 1602411111000],
    [84960000, 1602430625000],
    [84970000, 1602489973000],
    [84980000, 1602517478000],
    [84990001, 1602578895000],
    [85000002, 1602605576000],
    [85010001, 1602668663000],
    [85020003, 1602701055000],
    [85030001, 1602761490000],
    [85040003, 1602802821000],
    [85050000, 1602851208000],
    [85060000, 1602896027000],
    [85070000, 1602933391000],
    [85080000, 1602960055000],
    [85090000, 1603008554000],
    [85100000, 1603031419000],
    [85110000, 1603084867000],
    [85120000, 1603118190000],
    [85130001, 1603176350000],
    [85140000, 1603207204000],
    [85150000, 1603269807000],
    [85160000, 1603298349000],
    [85170000, 1603361902000],
    [85180000, 1603392486000],
    [85190006, 1603451581000],
    [85200000, 1603488670000],
    [85210000, 1603534355000],
    [85220000, 1603558403000],
    [85230000, 1603607507000],
    [85240001, 1603632338000],
    [85250000, 1603674774000],
    [85260001, 1603717899000],
    [85270000, 1603768489000],
    [85280000, 1603807261000],
    [85290000, 1603863363000],
    [85300000, 1603897106000],
    [85310000, 1603958296000],
    [85320000, 1603987706000],
    [85330000, 1604049178000],
    [85340000, 1604071668000],
    [85350000, 1604113097000],
    [85360001, 1604138820000],
    [85370000, 1604154118000],
    [85380000, 1604182076000],
    [85390000, 1604217262000],
    [85400001, 1604238600000],
    [85410000, 1604283830000],
    [85420000, 1604322279000],
    [85430001, 1604365714000],
    [85440000, 1604401200000],
    [85450006, 1604426722000],
    [85460000, 1604488455000],
    [85470000, 1604522436000],
    [85480002, 1604575885000],
    [85490001, 1604614044000],
    [85500000, 1604665701000],
    [85510001, 1604707601000],
    [85520000, 1604746791000],
    [85530000, 1604771476000],
    [85540000, 1604820463000],
    [85550001, 1604843004000],
    [85560000, 1604888244000],
    [85570001, 1604928134000],
    [85580000, 1604978324000],
    [85590000, 1605017606000],
    [85600001, 1605066713000],
    [85610000, 1605101470000],
    [85620000, 1605150004000],
    [85630000, 1605189793000],
    [85640000, 1605244786000],
    [85650002, 1605278578000],
    [85660000, 1605328883000],
    [85670000, 1605360539000],
    [85680000, 1605403968000],
    [85690000, 1605435942000],
    [85700002, 1605455705000],
    [85710000, 1605517074000],
    [85720000, 1605543716000],
    [85730000, 1605605844000],
    [85740000, 1605634746000],
    [85750000, 1605696751000],
    [85760000, 1605730228000],
    [85770000, 1605786348000],
    [85780000, 1605827263000],
    [85790001, 1605875392000],
    [85800002, 1605915640000],
    [85810000, 1605954736000],
    [85820002, 1605978690000],
    [85830000, 1606029172000],
    [85840000, 1606052878000],
    [85850002, 1606092611000],
    [85860000, 1606126897000],
    [85870000, 1606148517000],
    [85880000, 1606211093000],
    [85890001, 1606239435000],
    [85900001, 1606301459000],
    [85910000, 1606332601000],
    [85920002, 1606390291000],
    [85930000, 1606426752000],
    [85940000, 1606478449000],
    [85950000, 1606514400000],
    [85960000, 1606557362000],
    [85970003, 1606578715000],
    [85980000, 1606627726000],
    [85990003, 1606653018000],
    [86000001, 1606683108000],
    [86010000, 1606735819000],
    [86020000, 1606762500000],
    [86030000, 1606818754000],
    [86040000, 1606848141000],
    [86050001, 1606906257000],
    [86060000, 1606940375000],
    [86070001, 1606996588000],
    [86080000, 1607037903000],
    [86090000, 1607084583000],
    [86100000, 1607127525000],
    [86110000, 1607165807000],
    [86120001, 1607190672000],
    [86130000, 1607240200000],
    [86140000, 1607263451000],
    [86150001, 1607309630000],
    [86160002, 1607347237000],
    [86170003, 1607396501000],
    [86180000, 1607435611000],
    [86190002, 1607490136000],
    [86200000, 1607524276000],
    [86210001, 1607581076000],
    [86220000, 1607612673000],
    [86230000, 1607673108000],
    [86240000, 1607701001000],
    [86250000, 1607754713000],
    [86260001, 1607781967000],
    [86270000, 1607825737000],
    [86280000, 1607857680000],
    [86290000, 1607880016000],
    [86300000, 1607940113000],
    [86310000, 1607969943000],
    [86320000, 1608031851000],
    [86330001, 1608073772000],
    [86340001, 1608122282000],
    [86350000, 1608172435000],
    [86360000, 1608214074000],
    [86370000, 1608271201000],
    [86380000, 1608303611000],
    [86390001, 1608357966000],
    [86400001, 1608386966000],
    [86410000, 1608432676000],
    [86420000, 1608464132000],
    [86430000, 1608491850000],
    [86440000, 1608548383000],
    [86450000, 1608584314000],
    [86460000, 1608637792000],
    [86470000, 1608680410000],
    [86480000, 1608726826000],
    [86490000, 1608763061000],
    [86500001, 1608804647000],
    [86510000, 1608822113000],
    [86520000, 1608858720000],
    [86530003, 1608891984000],
    [86540000, 1608909214000],
    [86550001, 1608956008000],
    [86560000, 1608986831000],
    [86570000, 1609024070000],
    [86580000, 1609061639000],
    [86590000, 1609083571000],
    [86600002, 1609136877000],
    [86610000, 1609166538000],
    [86620002, 1609215625000],
    [86630001, 1609246633000],
    [86640000, 1609281333000],
    [86650001, 1609318776000],
    [86660000, 1609340404000],
    [86670002, 1609378551000],
    [86680001, 1609402813000],
    [86690000, 1609419573000],
    [86700000, 1609427833000],
    [86710001, 1609438598000],
    [86720000, 1609471195000],
    [86730000, 1609492245000],
    [86740001, 1609510048000],
    [86750000, 1609542326000],
    [86760000, 1609576482000],
    [86770000, 1609597332000],
    [86780001, 1609636492000],
    [86790000, 1609666321000],
    [86800000, 1609685861000],
    [86810000, 1609730596000],
    [86820000, 1609761612000],
    [86830000, 1609785653000],
    [86840002, 1609838038000],
    [86850002, 1609860617000],
    [86860002, 1609913574000],
    [86870000, 1609942399000],
    [86880000, 1609989846000],
    [86890003, 1610024770000],
    [86900000, 1610067652000],
    [86910003, 1610108358000],
    [86920001, 1610143689000],
    [86930000, 1610185067000],
    [86940001, 1610206533000],
    [86950001, 1610253334000],
    [86960000, 1610280157000],
    [86970000, 1610301936000],
    [86980000, 1610349015000],
    [86990000, 1610372052000],
    [87000000, 1610415162000],
    [87010000, 1610455354000],
    [87020001, 1610499781000],
    [87030000, 1610542450000],
    [87040001, 1610588105000],
    [87050000, 1610628593000],
    [87060000, 1610674146000],
    [87070000, 1610714760000],
    [87080000, 1610755748000],
    [87090000, 1610792712000],
    [87100000, 1610813843000],
    [87110000, 1610860693000],
    [87120004, 1610885481000],
    [87130000, 1610905671000],
    [87140000, 1610963265000],
    [87150001, 1610987688000],
    [87160000, 1611047115000],
    [87170001, 1611073100000],
    [87180000, 1611133202000],
    [87190000, 1611158922000],
    [87200000, 1611218267000],
    [87210001, 1611244919000],
    [87220003, 1611303699000],
    [87230000, 1611328247000],
    [87240001, 1611376847000],
    [87250001, 1611404915000],
    [87260000, 1611429073000],
    [87270001, 1611473057000],
    [87280001, 1611494297000],
    [87290002, 1611532048000],
    [87300000, 1611574806000],
    [87310001, 1611603103000],
    [87320000, 1611657841000],
    [87330000, 1611683185000],
    [87340001, 1611740504000],
    [87350001, 1611764653000],
    [87360000, 1611823133000],
    [87370000, 1611847489000],
    [87380001, 1611904116000],
    [87390000, 1611931539000],
    [87400000, 1611976818000],
    [87410000, 1612007710000],
    [87420000, 1612028208000],
    [87430003, 1612073912000],
    [87440000, 1612095997000],
    [87450000, 1612114063000],
    [87460000, 1612168956000],
    [87470000, 1612192262000],
    [87480001, 1612244712000],
    [87490000, 1612273447000],
    [87500000, 1612318002000],
    [87510001, 1612355620000],
    [87520000, 1612393200000],
    [87530000, 1612439260000],
    [87540000, 1612470865000],
    [87550000, 1612521992000],
    [87560000, 1612546329000],
    [87570000, 1612597585000],
    [87580000, 1612621395000],
    [87590001, 1612663032000],
    [87600000, 1612692007000],
    [87610000, 1612710664000],
    [87620000, 1612761984000],
    [87630001, 1612792557000],
    [87640000, 1612840944000],
    [87650005, 1612876756000],
    [87660000, 1612922132000],
    [87670000, 1612961112000],
    [87680000, 1613002087000],
    [87690000, 1613038016000],
    [87700000, 1613059448000],
    [87710000, 1613115117000],
    [87720000, 1613141773000],
    [87730000, 1613187144000],
    [87740000, 1613216930000],
    [87750000, 1613234854000],
    [87760000, 1613272307000],
    [87770000, 1613293810000],
    [87780000, 1613308342000],
    [87790000, 1613323241000],
    [87800000, 1613374101000],
    [87810001, 1613398896000],
    [87820000, 1613444966000],
    [87830001, 1613480261000],
    [87840001, 1613517135000],
    [87850000, 1613561701000],
    [87860000, 1613588747000],
    [87870000, 1613643245000],
    [87880001, 1613668258000],
    [87890000, 1613726307000],
    [87900000, 1613748976000],
    [87910000, 1613799261000],
    [87920000, 1613826002000],
    [87930000, 1613857147000],
    [87940000, 1613895016000],
    [87950001, 1613915579000],
    [87960000, 1613952398000],
    [87970000, 1613991958000],
    [87980000, 1614010102000],
    [87990000, 1614057614000],
    [88000000, 1614083703000],
    [88010000, 1614108067000],
    [88020000, 1614160604000],
    [88030001, 1614182571000],
    [88040000, 1614240526000],
    [88050000, 1614266222000],
    [88060000, 1614322531000],
    [88070000, 1614350324000],
    [88080000, 1614395570000],
    [88090001, 1614425858000],
    [88100000, 1614445571000],
    [88110000, 1614490959000],
    [88120002, 1614513295000],
    [88130000, 1614528724000],
    [88140000, 1614581461000],
    [88150000, 1614607485000],
    [88160001, 1614652568000],
    [88170000, 1614687823000],
    [88180000, 1614722235000],
    [88190000, 1614769470000],
    [88200000, 1614794834000],
    [88210001, 1614852362000],
    [88220000, 1614877875000],
    [88230000, 1614935713000],
    [88240000, 1614958693000],
    [88250001, 1615008512000],
    [88260000, 1615035799000],
    [88270001, 1615069177000],
    [88280000, 1615105490000],
    [88290003, 1615125960000],
    [88300003, 1615169063000],
    [88310004, 1615207124000],
    [88320000, 1615239351000],
    [88330000, 1615285711000],
    [88340001, 1615306402000],
    [88350000, 1615362866000],
    [88360001, 1615388634000],
    [88370000, 1615444524000],
    [88380000, 1615474182000],
    [88390001, 1615527046000],
    [88400000, 1615557587000],
    [88410000, 1615601796000],
    [88420000, 1615633604000],
    [88430000, 1615653786000],
    [88440000, 1615699116000],
    [88450000, 1615722802000],
    [88460000, 1615739556000],
    [88470000, 1615795474000],
    [88480000, 1615820457000],
    [88490002, 1615875495000],
    [88500001, 1615904999000],
    [88510000, 1615956873000],
    [88520000, 1615989658000],
    [88530000, 1616039715000],
    [88540000, 1616075008000],
    [88550000, 1616123180000],
    [88560002, 1616159310000],
    [88570000, 1616196667000],
    [88580000, 1616233617000],
    [88590002, 1616254733000],
    [88600000, 1616300154000],
    [88610001, 1616325026000],
    [88620000, 1616342572000],
    [88630001, 1616397499000],
    [88640000, 1616424247000],
    [88650000, 1616477154000],
    [88660000, 1616508572000],
    [88670000, 1616558712000],
    [88680000, 1616592726000],
    [88690001, 1616637905000],
    [88700001, 1616676059000],
    [88710002, 1616716130000],
    [88720000, 1616759999000],
    [88730000, 1616795128000],
    [88740001, 1616838735000],
    [88750000, 1616860732000],
    [88760001, 1616908211000],
    [88770000, 1616933790000],
    [88780000, 1616958010000],
    [88790000, 1617011950000],
    [88800003, 1617036555000],
    [88810000, 1617093907000],
    [88820000, 1617118816000],
    [88830001, 1617174316000],
    [88840001, 1617200731000],
    [88850001, 1617243634000],
    [88860002, 1617278494000],
    [88870000, 1617309504000],
    [88880000, 1617358689000],
    [88890001, 1617383138000],
    [88900000, 1617434397000],
    [88910000, 1617459553000],
    [88920001, 1617501009000],
    [88930000, 1617529499000],
    [88940002, 1617548191000],
    [88950000, 1617595153000],
    [88960002, 1617627783000],
    [88970000, 1617672115000],
    [88980001, 1617712524000],
    [88990000, 1617752994000],
    [89000000, 1617797507000],
    [89010000, 1617836768000],
    [89020001, 1617883489000],
    [89030000, 1617921920000],
    [89040000, 1617969299000],
    [89050001, 1617999997000],
    [89060001, 1618045483000],
    [89070000, 1618068552000],
    [89080000, 1618115428000],
    [89090000, 1618142898000],
    [89100001, 1618169322000],
    [89110000, 1618225851000],
    [89120000, 1618256800000],
    [89130000, 1618312355000],
    [89140001, 1618343716000],
    [89150001, 1618399131000],
    [89160001, 1618434000000],
    [89170000, 1618487875000],
    [89180001, 1618530483000],
    [89190001, 1618576475000],
    [89200000, 1618617370000],
    [89210000, 1618654080000],
    [89220001, 1618675923000],
    [89230001, 1618722709000],
    [89240000, 1618748103000],
    [89250000, 1618773431000],
    [89260000, 1618829097000],
    [89270001, 1618855923000],
    [89280001, 1618915423000],
    [89290001, 1618945208000],
    [89300000, 1619003717000],
    [89310001, 1619038604000],
    [89320001, 1619090895000],
    [89330000, 1619125411000],
    [89340000, 1619177962000],
    [89350000, 1619211232000],
    [89360000, 1619256552000],
    [89370000, 1619278794000],
    [89380000, 1619326404000],
    [89390001, 1619352456000],
    [89400000, 1619379540000],
    [89410000, 1619435000000],
    [89420000, 1619465104000],
    [89430000, 1619522412000],
    [89440000, 1619558208000],
    [89450000, 1619610454000],
    [89460001, 1619650162000],
    [89470000, 1619690241000],
    [89480000, 1619712665000],
    [89490000, 1619768104000],
    [89500002, 1619793721000],
    [89510000, 1619838000000],
    [89520000, 1619868624000],
    [89530001, 1619891012000],
    [89540000, 1619936521000],
    [89550000, 1619961045000],
    [89560000, 1619998134000],
    [89570000, 1620035507000],
    [89580000, 1620056966000],
    [89590000, 1620104538000],
    [89600001, 1620132106000],
    [89610001, 1620156501000],
    [89620000, 1620197445000],
    [89630002, 1620218762000],
    [89640000, 1620246460000],
    [89650000, 1620296840000],
    [89660000, 1620321447000],
    [89670000, 1620378694000],
    [89680001, 1620402803000],
    [89690000, 1620453849000],
    [89700000, 1620480136000],
    [89710000, 1620515600000],
    [89720000, 1620548748000],
    [89730002, 1620568797000],
    [89740000, 1620606311000],
    [89750001, 1620647872000],
    [89760000, 1620676706000],
    [89770001, 1620730642000],
    [89780000, 1620757879000],
    [89790001, 1620815341000],
    [89800000, 1620840939000],
    [89810000, 1620900235000],
    [89820000, 1620924966000],
    [89830000, 1620983564000],
    [89840000, 1621006065000],
    [89850001, 1621053339000],
    [89860001, 1621081841000],
    [89870000, 1621109974000],
    [89880000, 1621150548000],
    [89890000, 1621171336000],
    [89900000, 1621208665000],
    [89910000, 1621252583000],
    [89920000, 1621286110000],
    [89930000, 1621335985000],
    [89940001, 1621365022000],
    [89950000, 1621419685000],
    [89960001, 1621445054000],
    [89970000, 1621503348000],
    [89980002, 1621528309000],
    [89990001, 1621586938000],
    [90000000, 1621611201000],
    [90010000, 1621660634000],
    [90020000, 1621688662000],
    [90030000, 1621721040000],
    [90040000, 1621756835000],
    [90050000, 1621776870000],
    [90060000, 1621812602000],
    [90070000, 1621855438000],
    [90080004, 1621881435000],
    [90090000, 1621937529000],
    [90100000, 1621963889000],
    [90110001, 1622022538000],
    [90120000, 1622047836000],
    [90130000, 1622107620000],
    [90140000, 1622134471000],
    [90150001, 1622193679000],
    [90160001, 1622217099000],
    [90170000, 1622266721000],
    [90180000, 1622293836000],
    [90190000, 1622325877000],
    [90200000, 1622361667000],
    [90210000, 1622381815000],
    [90220000, 1622420125000],
    [90230000, 1622461211000],
    [90240000, 1622482229000],
    [90250000, 1622538039000],
    [90260000, 1622559740000],
    [90270000, 1622614058000],
    [90280002, 1622642923000],
    [90290000, 1622691764000],
    [90300000, 1622727037000],
    [90310001, 1622773389000],
    [90320001, 1622809521000],
    [90330000, 1622843167000],
    [90340000, 1622884240000],
    [90350000, 1622905227000],
    [90360000, 1622946676000],
    [90370000, 1622973786000],
    [90380000, 1622991614000],
    [90390000, 1623042022000],
    [90400000, 1623073658000],
    [90410000, 1623119778000],
    [90420000, 1623157012000],
    [90430000, 1623195060000],
    [90440001, 1623239172000],
    [90450000, 1623272548000],
    [90460000, 1623323818000],
    [90470000, 1623356437000],
    [90480000, 1623408801000],
    [90490000, 1623434687000],
    [90500000, 1623484497000],
    [90510000, 1623508098000],
    [90520000, 1623551323000],
    [90530000, 1623579777000],
    [90540000, 1623597466000],
    [90550000, 1623652098000],
    [90560000, 1623680595000],
    [90570000, 1623732073000],
    [90580000, 1623765415000],
    [90590000, 1623815281000],
    [90600000, 1623849449000],
    [90610000, 1623894287000],
    [90620000, 1623933712000],
    [90630002, 1623975946000],
    [90640000, 1624017229000],
    [90650000, 1624048744000],
    [90660000, 1624092010000],
    [90670000, 1624114410000],
    [90680000, 1624155450000],
    [90690000, 1624183428000],
    [90700000, 1624201282000],
    [90710000, 1624250418000],
    [90720000, 1624283420000],
    [90730000, 1624329178000],
    [90740001, 1624366823000],
    [90750001, 1624409521000],
    [90760000, 1624451215000],
    [90770000, 1624488127000],
    [90780001, 1624534770000],
    [90790000, 1624565483000],
    [90800000, 1624617743000],
    [90810001, 1624642581000],
    [90820000, 1624693006000],
    [90830001, 1624717159000],
    [90840001, 1624759122000],
    [90850000, 1624788393000],
    [90860001, 1624807262000],
    [90870002, 1624862147000],
    [90880000, 1624890846000],
    [90890000, 1624941410000],
    [90900000, 1624975119000],
    [90910003, 1625023200000],
    [90920000, 1625057873000],
    [90930002, 1625093378000],
    [90940000, 1625137864000],
    [90950001, 1625167064000],
    [90960000, 1625221678000],
    [90970000, 1625247018000],
    [90980000, 1625297083000],
    [90990000, 1625321661000],
    [91000000, 1625361446000],
    [91010000, 1625391835000],
    [91020000, 1625410814000],
    [91030001, 1625460847000],
    [91040000, 1625492417000],
    [91050002, 1625538794000],
    [91060000, 1625576331000],
    [91070000, 1625612008000],
    [91080000, 1625655607000],
    [91090000, 1625676564000],
    [91100000, 1625734139000],
    [91110000, 1625758730000],
    [91120001, 1625815829000],
    [91130000, 1625841974000],
    [91140000, 1625886925000],
    [91150000, 1625918380000],
    [91160000, 1625944463000],
    [91170000, 1625988355000],
    [91180001, 1626010374000],
    [91190000, 1626048401000],
    [91200000, 1626090518000],
    [91210001, 1626120952000],
    [91220000, 1626173755000],
    [91230000, 1626201993000],
    [91240000, 1626257370000],
    [91250000, 1626282016000],
    [91260001, 1626339404000],
    [91270000, 1626364995000],
    [91280001, 1626422902000],
    [91290001, 1626448420000],
    [91300000, 1626497465000],
    [91310000, 1626526559000],
    [91320000, 1626562870000],
    [91330001, 1626598863000],
    [91340000, 1626619253000],
    [91350000, 1626667572000],
    [91360000, 1626702033000],
    [91370000, 1626748789000],
    [91380000, 1626786000000],
    [91390000, 1626832490000],
    [91400000, 1626871207000],
    [91410000, 1626910348000],
    [91420000, 1626950240000],
    [91430001, 1626975304000],
    [91440001, 1627025779000],
    [91450000, 1627052434000],
    [91460000, 1627098880000],
    [91470000, 1627129161000],
    [91480001, 1627159365000],
    [91490001, 1627200772000],
    [91500001, 1627222086000],
    [91510000, 1627266953000],
    [91520002, 1627303365000],
    [91530000, 1627344228000],
    [91540000, 1627386109000],
    [91550002, 1627419996000],
    [91560000, 1627469441000],
    [91570002, 1627496681000],
    [91580002, 1627552012000],
    [91590000, 1627577244000],
    [91600000, 1627633974000],
    [91610002, 1627658630000],
    [91620003, 1627707670000],
    [91630000, 1627735328000],
    [91640000, 1627758052000],
    [91650001, 1627800425000],
    [91660000, 1627822804000],
    [91670000, 1627854019000],
    [91680000, 1627899489000],
    [91690000, 1627922287000],
    [91700000, 1627977531000],
    [91710000, 1628002930000],
    [91720000, 1628055932000],
    [91730000, 1628086281000],
    [91740000, 1628135358000],
    [91750000, 1628169378000],
    [91760000, 1628214699000],
    [91770000, 1628252181000],
    [91780001, 1628284290000],
    [91790001, 1628325896000],
    [91800001, 1628348457000],
    [91810001, 1628390478000],
    [91820000, 1628419243000],
    [91830002, 1628438044000],
    [91840000, 1628482975000],
    [91850000, 1628509353000],
    [91860002, 1628530158000],
    [91870000, 1628580157000],
    [91880001, 1628605566000],
    [91890002, 1628650436000],
    [91900000, 1628684140000],
    [91910000, 1628717443000],
    [91920002, 1628760853000],
    [91930000, 1628783284000],
    [91940000, 1628833352000],
    [91950000, 1628859509000],
    [91960000, 1628888945000],
    [91970000, 1628929003000],
    [91980002, 1628949933000],
    [91990001, 1628976385000],
    [92000000, 1629013891000],
    [92010000, 1629034080000],
    [92020001, 1629050568000],
    [92030000, 1629098917000],
    [92040001, 1629122464000],
    [92050001, 1629165939000],
    [92060000, 1629201251000],
    [92070000, 1629228686000],
    [92080000, 1629279438000],
    [92090003, 1629301654000],
    [92100000, 1629354528000],
    [92110000, 1629382820000],
    [92120001, 1629428380000],
    [92130000, 1629461783000],
    [92140000, 1629489334000],
    [92150000, 1629533407000],
    [92160000, 1629554732000],
    [92170000, 1629591443000],
    [92180000, 1629622576000],
    [92190000, 1629642144000],
    [92200000, 1629684467000],
    [92210000, 1629720044000],
    [92220000, 1629747693000],
    [92230000, 1629798593000],
    [92240000, 1629820858000],
    [92250002, 1629875567000],
    [92260000, 1629901711000],
    [92270000, 1629949310000],
    [92280000, 1629982572000],
    [92290000, 1630018215000],
    [92300001, 1630060374000],
    [92310000, 1630081934000],
    [92320000, 1630129786000],
    [92330001, 1630155300000],
    [92340000, 1630183059000],
    [92350000, 1630221839000],
    [92360000, 1630242352000],
    [92370000, 1630271314000],
    [92380001, 1630317518000],
    [92390000, 1630337159000],
    [92400000, 1630382926000],
    [92410000, 1630412215000],
    [92420001, 1630429095000],
    [92430001, 1630483322000],
    [92440000, 1630508357000],
    [92450000, 1630560280000],
    [92460000, 1630590093000],
    [92470000, 1630636193000],
    [92480000, 1630671179000],
    [92490001, 1630700747000],
    [92500000, 1630744157000],
    [92510000, 1630765812000],
    [92520001, 1630806856000],
    [92530000, 1630835440000],
    [92540001, 1630853116000],
    [92550000, 1630898590000],
    [92560000, 1630932842000],
    [92570000, 1630970533000],
    [92580000, 1631014453000],
    [92590000, 1631043850000],
    [92600000, 1631095907000],
    [92610000, 1631119701000],
    [92620000, 1631176254000],
    [92630000, 1631199949000],
    [92640002, 1631253216000],
    [92650002, 1631282400000],
    [92660001, 1631324551000],
    [92670002, 1631357277000],
    [92680000, 1631377229000],
    [92690000, 1631422503000],
    [92700003, 1631446220000],
    [92710000, 1631462798000],
    [92720000, 1631516450000],
    [92730000, 1631542583000],
    [92740003, 1631591632000],
    [92750000, 1631625667000],
    [92760000, 1631669372000],
    [92770000, 1631708250000],
    [92780000, 1631746635000],
    [92790000, 1631792358000],
    [92800000, 1631826201000],
    [92810000, 1631874789000],
    [92820003, 1631896937000],
    [92830003, 1631946719000],
    [92840001, 1631971337000],
    [92850000, 1632003550000],
    [92860002, 1632040968000],
    [92870000, 1632062151000],
    [92880000, 1632104085000],
    [92890000, 1632134232000],
    [92900000, 1632153650000],
    [92910000, 1632207629000],
    [92920000, 1632233716000],
    [92930001, 1632281347000],
    [92940003, 1632316006000],
    [92950002, 1632354953000],
    [92960000, 1632390604000],
    [92970000, 1632411632000],
    [92980002, 1632468523000],
    [92990001, 1632494582000],
    [93000000, 1632539184000],
    [93010001, 1632569325000],
    [93020000, 1632591106000],
    [93030000, 1632637348000],
    [93040002, 1632659517000],
    [93050000, 1632686353000],
    [93060002, 1632738377000],
    [93070000, 1632762448000],
    [93080000, 1632820353000],
    [93090000, 1632844137000],
    [93100000, 1632902683000],
    [93110000, 1632928133000],
    [93120000, 1632983492000],
    [93130000, 1633011118000],
    [93140000, 1633055783000],
    [93150000, 1633090148000],
    [93160000, 1633117454000],
    [93170001, 1633162248000],
    [93180000, 1633185803000],
    [93190001, 1633227999000],
    [93200000, 1633257628000],
    [93210001, 1633276861000],
    [93220001, 1633333310000],
    [93230001, 1633359566000],
    [93240001, 1633412563000],
    [93250001, 1633444232000],
    [93260000, 1633498298000],
    [93270001, 1633530121000],
    [93280000, 1633582979000],
    [93290001, 1633615633000],
    [93300000, 1633668364000],
    [93310000, 1633701509000],
    [93320000, 1633745574000],
    [93330000, 1633778453000],
    [93340000, 1633799743000],
    [93350000, 1633846044000],
    [93360000, 1633869822000],
    [93370000, 1633898119000],
    [93380000, 1633948902000],
    [93390002, 1633973431000],
    [93400000, 1634032282000],
    [93410000, 1634057765000],
    [93420000, 1634117857000],
    [93430000, 1634144756000],
    [93440000, 1634205236000],
    [93450000, 1634234014000],
    [93460001, 1634293036000],
    [93470000, 1634318727000],
    [93480002, 1634368869000],
    [93490000, 1634394483000],
    [93500001, 1634437527000],
    [93510001, 1634467278000],
    [93520000, 1634486578000],
    [93530000, 1634546192000],
    [93540001, 1634572876000],
    [93550000, 1634634855000],
    [93560000, 1634662513000],
    [93570000, 1634724092000],
    [93580000, 1634750955000],
    [93590000, 1634812596000],
    [93600000, 1634848910000],
    [93610001, 1634902063000],
    [93620000, 1634932691000],
    [93630001, 1634980370000],
    [93640001, 1635003828000],
    [93650001, 1635051305000],
    [93660000, 1635077962000],
    [93670000, 1635106708000],
    [93680000, 1635159684000],
    [93690000, 1635188542000],
    [93700000, 1635245335000],
    [93710003, 1635273356000],
    [93720000, 1635331885000],
    [93730000, 1635363989000],
    [93740000, 1635419650000],
    [93750002, 1635456306000],
    [93760000, 1635506037000],
    [93770000, 1635534789000],
    [93780000, 1635583147000],
    [93790000, 1635604701000],
    [93800001, 1635632402000],
    [93810000, 1635661010000],
    [93820000, 1635678590000],
    [93830000, 1635690570000],
    [93840000, 1635717518000],
    [93850000, 1635761107000],
    [93860000, 1635783455000],
    [93870000, 1635843752000],
    [93880000, 1635869652000],
    [93890000, 1635922335000],
    [93900002, 1635948667000],
    [93910000, 1636001431000],
    [93920001, 1636035436000],
    [93930001, 1636087879000],
    [93940000, 1636121598000],
    [93950004, 1636168844000],
    [93960002, 1636201173000],
    [93970000, 1636235282000],
    [93980000, 1636274954000],
    [93990000, 1636296417000],
    [94000000, 1636347998000],
    [94010000, 1636380724000],
    [94020000, 1636432758000],
    [94030000, 1636467490000],
    [94040000, 1636524000000],
    [94050000, 1636556411000],
    [94060000, 1636609572000],
    [94070000, 1636639397000],
    [94080001, 1636688884000],
    [94090000, 1636724837000],
    [94100001, 1636769763000],
    [94110000, 1636804397000],
    [94120000, 1636832830000],
    [94130000, 1636877299000],
    [94140001, 1636898962000],
    [94150000, 1636945784000],
    [94160001, 1636983624000],
    [94170000, 1637035689000],
    [94180001, 1637071919000],
    [94190000, 1637126743000],
    [94200000, 1637160488000],
    [94210000, 1637217623000],
    [94220002, 1637248047000],
    [94230000, 1637307501000],
    [94240003, 1637335174000],
    [94250000, 1637387507000],
    [94260001, 1637415849000],
    [94270000, 1637457796000],
    [94280003, 1637490222000],
    [94290000, 1637510450000],
    [94300000, 1637568614000],
    [94310001, 1637594427000],
    [94320000, 1637645400000],
    [94330000, 1637673692000],
    [94340000, 1637717882000],
    [94350000, 1637758190000],
    [94360000, 1637804370000],
    [94370000, 1637845662000],
    [94380000, 1637895340000],
    [94390000, 1637932735000],
    [94400000, 1637975870000],
    [94410000, 1638010801000],
    [94420000, 1638034566000],
    [94430000, 1638082864000],
    [94440000, 1638106561000],
    [94450000, 1638150124000],
    [94460001, 1638190313000],
    [94470001, 1638236140000],
    [94480000, 1638275362000],
    [94490000, 1638312529000],
    [94500001, 1638359064000],
    [94510000, 1638396225000],
    [94520000, 1638446401000],
    [94530000, 1638487812000],
    [94540001, 1638534056000],
    [94550000, 1638573040000],
    [94560001, 1638611797000],
    [94570001, 1638633654000],
    [94580001, 1638681819000],
    [94590000, 1638707684000],
    [94600002, 1638739779000],
    [94610000, 1638791053000],
    [94620000, 1638827797000],
    [94630000, 1638877512000],
    [94640001, 1638916136000],
    [94650000, 1638964380000],
    [94660000, 1639002600000],
    [94670000, 1639051564000],
    [94680000, 1639092606000],
    [94690000, 1639138004000],
    [94700000, 1639175470000],
    [94710000, 1639216802000],
    [94720000, 1639240239000],
    [94730000, 1639289730000],
    [94740000, 1639314497000],
    [94750000, 1639355956000],
    [94760000, 1639399376000],
    [94770000, 1639446657000],
    [94780000, 1639488341000],
    [94790000, 1639540890000],
    [94800000, 1639578070000],
    [94810000, 1639634825000],
    [94820000, 1639667014000],
    [94830000, 1639727096000],
    [94840000, 1639754857000],
    [94850002, 1639807985000],
    [94860000, 1639836570000],
    [94870000, 1639881936000],
    [94880000, 1639913328000],
    [94890001, 1639939256000],
    [94900001, 1639997861000],
    [94910000, 1640032136000],
    [94920000, 1640086745000],
    [94930001, 1640126944000],
    [94940000, 1640174567000],
    [94950000, 1640216396000],
    [94960000, 1640262124000],
    [94970000, 1640293978000],
    [94980000, 1640337791000],
    [94990000, 1640355810000],
    [95000000, 1640383148000],
    [95010000, 1640417193000],
    [95020000, 1640437332000],
    [95030000, 1640457665000],
    [95040000, 1640502272000],
    [95050000, 1640525742000],
    [95060000, 1640568505000],
    [95070000, 1640607220000],
    [95080000, 1640643662000],
    [95090000, 1640688477000],
    [95100000, 1640713771000],
    [95110001, 1640765263000],
    [95120000, 1640789455000],
    [95130000, 1640836018000],
    [95140000, 1640865820000],
    [95150000, 1640886986000],
    [95160000, 1640929078000],
    [95170000, 1640948539000],
    [95180000, 1640962119000],
    [95190000, 1640967773000],
    [95200000, 1640999289000],
    [95210001, 1641021726000],
    [95220000, 1641041414000],
    [95230000, 1641062154000],
    [95240001, 1641107276000],
    [95250000, 1641130900000],
    [95260000, 1641169453000],
    [95270000, 1641202441000],
    [95280000, 1641222169000],
    [95290001, 1641268706000],
    [95300000, 1641299401000],
    [95310000, 1641331812000],
    [95320000, 1641378569000],
    [95330000, 1641402623000],
    [95340000, 1641459753000],
    [95350001, 1641484334000],
    [95360000, 1641541402000],
    [95370000, 1641567555000],
    [95380000, 1641614938000],
    [95390000, 1641645081000],
    [95400001, 1641678389000],
    [95410000, 1641718133000],
    [95420000, 1641740063000],
    [95430000, 1641783577000],
    [95440000, 1641813730000],
    [95450000, 1641836631000],
    [95460000, 1641893792000],
    [95470000, 1641919136000],
    [95480000, 1641979082000],
    [95490000, 1642005692000],
    [95500000, 1642066270000],
    [95510001, 1642094064000],
    [95520000, 1642152651000],
    [95530001, 1642178810000],
    [95540000, 1642231351000],
    [95550000, 1642256244000],
    [95560000, 1642301175000],
    [95570000, 1642330779000],
    [95580000, 1642350152000],
    [95590001, 1642407721000],
    [95600000, 1642433495000],
    [95610000, 1642491125000],
    [95620003, 1642518675000],
    [95630000, 1642577539000],
    [95640000, 1642605531000],
    [95650000, 1642662484000],
    [95660002, 1642691138000],
    [95670000, 1642748927000],
    [95680000, 1642777004000],
    [95690001, 1642824011000],
    [95700000, 1642853326000],
    [95710000, 1642879158000],
    [95720001, 1642923104000],
    [95730000, 1642944769000],
    [95740001, 1642984481000],
    [95750000, 1643025451000],
    [95760001, 1643061600000],
    [95770002, 1643108753000],
    [95780000, 1643139339000],
    [95790000, 1643193042000],
    [95800000, 1643219513000],
    [95810000, 1643276236000],
    [95820000, 1643301999000],
    [95830000, 1643359596000],
    [95840000, 1643384088000],
    [95850000, 1643436377000],
    [95860000, 1643463390000],
    [95870000, 1643503825000],
    [95880001, 1643535017000],
    [95890000, 1643554532000],
    [95900000, 1643602080000],
    [95910000, 1643634055000],
    [95920000, 1643669363000],
    [95930000, 1643713379000],
    [95940000, 1643739115000],
    [95950000, 1643793651000],
    [95960000, 1643816717000],
    [95970000, 1643870775000],
    [95980000, 1643897062000],
    [95990000, 1643943154000],
    [96000000, 1643978568000],
    [96010000, 1644014001000],
    [96020000, 1644052331000],
    [96030000, 1644073609000],
    [96040000, 1644117712000],
    [96050000, 1644144613000],
    [96060000, 1644163045000],
    [96070004, 1644217418000],
    [96080001, 1644245022000],
    [96090001, 1644294817000],
    [96100002, 1644327396000],
    [96110000, 1644373211000],
    [96120000, 1644410656000],
    [96130000, 1644452618000],
    [96140000, 1644492703000],
    [96150001, 1644522089000],
    [96160000, 1644568767000],
    [96170000, 1644591604000],
    [96180000, 1644636121000],
    [96190001, 1644665864000],
    [96200000, 1644687566000],
    [96210000, 1644732574000],
    [96220000, 1644754873000],
    [96230001, 1644769034000],
    [96240000, 1644810107000],
    [96250000, 1644835977000],
    [96260000, 1644849447000],
    [96270000, 1644884644000],
    [96280000, 1644923709000],
    [96290000, 1644948749000],
    [96300000, 1645004525000],
    [96310000, 1645027867000],
    [96320000, 1645085715000],
    [96330000, 1645111159000],
    [96340001, 1645166953000],
    [96350000, 1645194497000],
    [96360000, 1645238464000],
    [96370000, 1645270349000],
    [96380000, 1645291965000],
    [96390003, 1645338654000],
    [96400000, 1645362057000],
    [96410000, 1645396567000],
    [96420000, 1645441201000],
    [96430000, 1645465905000],
    [96440003, 1645518199000],
    [96450000, 1645538687000],
    [96460000, 1645578555000],
    [96470000, 1645612446000],
    [96480000, 1645634807000],
    [96490000, 1645691338000],
    [96500000, 1645716653000],
    [96510000, 1645776440000],
    [96520000, 1645801874000],
    [96530000, 1645852693000],
    [96540000, 1645882000000],
    [96550000, 1645922691000],
    [96560001, 1645954326000],
    [96570001, 1645974002000],
    [96580000, 1646021264000],
    [96590000, 1646053246000],
    [96600000, 1646089480000],
    [96610001, 1646132910000],
    [96620000, 1646160063000],
    [96630000, 1646215743000],
    [96640000, 1646240060000],
    [96650001, 1646298000000],
    [96660000, 1646321417000],
    [96670000, 1646379141000],
    [96680000, 1646405726000],
    [96690000, 1646452784000],
    [96700000, 1646481415000],
    [96710000, 1646506562000],
    [96720000, 1646551308000],
    [96730000, 1646573095000],
    [96740000, 1646613282000],
    [96750000, 1646653562000],
    [96760000, 1646688519000],
    [96770000, 1646737225000],
    [96780000, 1646761941000],
    [96790001, 1646815449000],
    [96800000, 1646837455000],
    [96810003, 1646889463000],
    [96820000, 1646920634000],
    [96830000, 1646970593000],
    [96840000, 1647004780000],
    [96850000, 1647047991000],
    [96860001, 1647081744000],
    [96870000, 1647104597000],
    [96880000, 1647151938000],
    [96890000, 1647175394000],
    [96900001, 1647201204000],
    [96910000, 1647250064000],
    [96920000, 1647270916000],
    [96930000, 1647323937000],
    [96940000, 1647352851000],
    [96950001, 1647400831000],
    [96960001, 1647435932000],
    [96970001, 1647482574000],
    [96980001, 1647520201000],
    [96990000, 1647559457000],
    [97000000, 1647601584000],
    [97010000, 1647625416000],
    [97020000, 1647675245000],
    [97030000, 1647699013000],
    [97040001, 1647738565000],
    [97050000, 1647770012000],
    [97060000, 1647788815000],
    [97070000, 1647832918000],
    [97080000, 1647862795000],
    [97090001, 1647885819000],
    [97100000, 1647941829000],
    [97110000, 1647967953000],
    [97120000, 1648025460000],
    [97130000, 1648050847000],
    [97140000, 1648108091000],
    [97150000, 1648134375000],
    [97160000, 1648187483000],
    [97170000, 1648217127000],
    [97180000, 1648260677000],
    [97190000, 1648292570000],
    [97200000, 1648313035000],
    [97210000, 1648360179000],
    [97220001, 1648385292000],
    [97230002, 1648416624000],
    [97240000, 1648464289000],
    [97250000, 1648490405000],
    [97260000, 1648545108000],
    [97270000, 1648570375000],
    [97280000, 1648627532000],
    [97290000, 1648653425000],
    [97300000, 1648706876000],
    [97310000, 1648735286000],
    [97320000, 1648774530000],
    [97330000, 1648811160000],
    [97340000, 1648832900000],
    [97350000, 1648883188000],
    [97360000, 1648909261000],
    [97370000, 1648951531000],
    [97380000, 1648981759000],
    [97390000, 1649000134000],
    [97400000, 1649048805000],
    [97410001, 1649079946000],
    [97420000, 1649124773000],
    [97430001, 1649162281000],
    [97440000, 1649202085000],
    [97450001, 1649245271000],
    [97460000, 1649279910000],
    [97470000, 1649329956000],
    [97480000, 1649362356000],
    [97490000, 1649415174000],
    [97500000, 1649443697000],
    [97510001, 1649492288000],
    [97520000, 1649516238000],
    [97530001, 1649559814000],
    [97540000, 1649588261000],
    [97550001, 1649607526000],
    [97560000, 1649665207000],
    [97570000, 1649691288000],
    [97580000, 1649749484000],
    [97590000, 1649777794000],
    [97600000, 1649837193000],
    [97610000, 1649864455000],
    [97620000, 1649924301000],
    [97630000, 1649951118000],
    [97640001, 1650010638000],
    [97650000, 1650036067000],
    [97660000, 1650085732000],
    [97670001, 1650114550000],
    [97680000, 1650149599000],
    [97690000, 1650184582000],
    [97700001, 1650204445000],
    [97710000, 1650246943000],
    [97720000, 1650285061000],
    [97730000, 1650324640000],
    [97740000, 1650369926000],
    [97750000, 1650406505000],
    [97760001, 1650454683000],
    [97770000, 1650488690000],
    [97780000, 1650539666000],
    [97790000, 1650572092000],
    [97800000, 1650623950000],
    [97810000, 1650649218000],
    [97820000, 1650698937000],
    [97830000, 1650723060000],
    [97840000, 1650763419000],
    [97850001, 1650793105000],
    [97860000, 1650811744000],
    [97870000, 1650859436000],
    [97880000, 1650892827000],
    [97890000, 1650937001000],
    [97900000, 1650976372000],
    [97910000, 1651012201000],
    [97920000, 1651058881000],
    [97930000, 1651085134000],
    [97940000, 1651140494000],
    [97950000, 1651164143000],
    [97960000, 1651214464000],
    [97970000, 1651238558000],
    [97980000, 1651274894000],
    [97990000, 1651309740000],
    [98000000, 1651329556000],
    [98010000, 1651367050000],
    [98020000, 1651395706000],
    [98030001, 1651414572000],
    [98040000, 1651453238000],
    [98050000, 1651489385000],
    [98060000, 1651511038000],
    [98070000, 1651560061000],
    [98080001, 1651585717000],
    [98090000, 1651626209000],
    [98100000, 1651660631000],
    [98110001, 1651680777000],
    [98120000, 1651727064000],
    [98130000, 1651752712000],
    [98140000, 1651777533000],
    [98150000, 1651829455000],
    [98160000, 1651850625000],
    [98170000, 1651897753000],
    [98180000, 1651925450000],
    [98190000, 1651949752000],
    [98200000, 1651991741000],
    [98210000, 1652013784000],
    [98220000, 1652040136000],
    [98230000, 1652091772000],
    [98240000, 1652115170000],
    [98250000, 1652171957000],
    [98260000, 1652195468000],
    [98270001, 1652250334000],
    [98280000, 1652280651000],
    [98290001, 1652333857000],
    [98300000, 1652365069000],
    [98310000, 1652416463000],
    [98320000, 1652449293000],
    [98330000, 1652490305000],
    [98340002, 1652524153000],
    [98350001, 1652544334000],
    [98360001, 1652590508000],
    [98370000, 1652615956000],
    [98380002, 1652636691000],
    [98390000, 1652692865000],
    [98400000, 1652716070000],
    [98410000, 1652774155000],
    [98420001, 1652800199000],
    [98430002, 1652856185000],
    [98440000, 1652885172000],
    [98450000, 1652937760000],
    [98460000, 1652970116000],
    [98470000, 1653019210000],
    [98480000, 1653052985000],
    [98490001, 1653093221000],
    [98500000, 1653127987000],
    [98510000, 1653148724000],
    [98520001, 1653193876000],
    [98530000, 1653219729000],
    [98540000, 1653239894000],
    [98550000, 1653295318000],
    [98560000, 1653318459000],
    [98570000, 1653371956000],
    [98580000, 1653401929000],
    [98590000, 1653449762000],
    [98600000, 1653484814000],
    [98610000, 1653529521000],
    [98620000, 1653568713000],
    [98630001, 1653607977000],
    [98640000, 1653650480000],
    [98650000, 1653674013000],
    [98660000, 1653722738000],
    [98670001, 1653747137000],
    [98680000, 1653787374000],
    [98690000, 1653817935000],
    [98700000, 1653836604000],
    [98710000, 1653888350000],
    [98720000, 1653920114000],
    [98730000, 1653969242000],
    [98740000, 1654003806000],
    [98750000, 1654045210000],
    [98760001, 1654084663000],
    [98770000, 1654118935000],
    [98780000, 1654169178000],
    [98790000, 1654201643000],
    [98800000, 1654253088000],
    [98810000, 1654278868000],
    [98820001, 1654329163000],
    [98830000, 1654354013000],
    [98840000, 1654397808000],
    [98850001, 1654426920000],
    [98860000, 1654445286000],
    [98870000, 1654500301000],
    [98880002, 1654526820000],
    [98890000, 1654579790000],
    [98900000, 1654611580000],
    [98910000, 1654664420000],
    [98920000, 1654697588000],
    [98930000, 1654749487000],
    [98940000, 1654783568000],
    [98950000, 1654833489000],
    [98960001, 1654867663000],
    [98970000, 1654908892000],
    [98980001, 1654944139000],
    [98990000, 1654965503000],
    [99000000, 1655011751000],
    [99010001, 1655036623000],
    [99020000, 1655062050000],
    [99030000, 1655115730000],
    [99040000, 1655140802000],
    [99050001, 1655200238000],
    [99060000, 1655227418000],
    [99070000, 1655286100000],
    [99080000, 1655311322000],
    [99090000, 1655371756000],
    [99100000, 1655397488000],
    [99110000, 1655456409000],
    [99120001, 1655481377000],
    [99130001, 1655530998000],
    [99140000, 1655558478000],
    [99150000, 1655595815000],
    [99160001, 1655629094000],
    [99170000, 1655649877000],
    [99180000, 1655697169000],
    [99190002, 1655732133000],
    [99200001, 1655777542000],
    [99210000, 1655815828000],
    [99220000, 1655858280000],
    [99230000, 1655900428000],
    [99240000, 1655939022000],
    [99250000, 1655985533000],
    [99260000, 1656021877000],
    [99270000, 1656068560000],
    [99280000, 1656092055000],
    [99290001, 1656141419000],
    [99300001, 1656166335000],
    [99310000, 1656207842000],
    [99320000, 1656239403000],
    [99330000, 1656258623000],
    [99340000, 1656314463000],
    [99350000, 1656342153000],
    [99360000, 1656396899000],
    [99370000, 1656427915000],
    [99380000, 1656482185000],
    [99390000, 1656512903000],
    [99400001, 1656562839000],
    [99410000, 1656595579000],
    [99420000, 1656633247000],
    [99430000, 1656675331000],
    [99440000, 1656705852000],
    [99450000, 1656751022000],
    [99460000, 1656774792000],
    [99470000, 1656819776000],
    [99480000, 1656848027000],
    [99490002, 1656869398000],
    [99500000, 1656925839000],
    [99510000, 1656951106000],
    [99520000, 1657009297000],
    [99530000, 1657035775000],
    [99540000, 1657093341000],
    [99550000, 1657120434000],
    [99560000, 1657174517000],
    [99570000, 1657202489000],
    [99580000, 1657249291000],
    [99590000, 1657288695000],
    [99600000, 1657335917000],
    [99610000, 1657368370000],
    [99620001, 1657397214000],
    [99630001, 1657439756000],
    [99640000, 1657461763000],
    [99650000, 1657506983000],
    [99660000, 1657543658000],
    [99670000, 1657583872000],
    [99680000, 1657627368000],
    [99690000, 1657663740000],
    [99700002, 1657710358000],
    [99710000, 1657740367000],
    [99720000, 1657793730000],
    [99730000, 1657819234000],
    [99740000, 1657874904000],
    [99750000, 1657897814000],
    [99760000, 1657947118000],
    [99770000, 1657975961000],
    [99780000, 1658009309000],
    [99790000, 1658047456000],
    [99800000, 1658068837000],
    [99810000, 1658111280000],
    [99820001, 1658140909000],
    [99830000, 1658160879000],
    [99840000, 1658217236000],
    [99850000, 1658242527000],
    [99860001, 1658291248000],
    [99870001, 1658323415000],
    [99880001, 1658366167000],
    [99890000, 1658404801000],
    [99900000, 1658430178000],
    [99910000, 1658480339000],
    [99920000, 1658502571000],
    [99930000, 1658546742000],
    [99940000, 1658576671000],
    [99950000, 1658599585000],
    [99960000, 1658645517000],
    [99970000, 1658668180000],
    [99980000, 1658701239000],
    [99990001, 1658746444000],
    [100000000, 1658770550000],
    [100010001, 1658825057000],
    [100020000, 1658849795000],
    [100030000, 1658902252000],
    [100040000, 1658931216000],
    [100050002, 1658977024000],
    [100060000, 1659011336000],
    [100070000, 1659048276000],
    [100080000, 1659091189000],
    [100090001, 1659114637000],
    [100100000, 1659161710000],
    [100110000, 1659188166000],
    [100120001, 1659225744000],
    [100130000, 1659257431000],
    [100140000, 1659277374000],
    [100150001, 1659315360000],
    [100160000, 1659352775000],
    [100170001, 1659376747000],
    [100180000, 1659428206000],
    [100190000, 1659452356000],
    [100200000, 1659499984000],
    [100210002, 1659532025000],
    [100220000, 1659571779000],
    [100230000, 1659610850000],
    [100240000, 1659635757000],
    [100250002, 1659689460000],
    [100260000, 1659711827000],
    [100270000, 1659757449000],
    [100280000, 1659787156000],
    [100290000, 1659810033000],
    [100300002, 1659854775000],
    [100310000, 1659877201000],
    [100320001, 1659904955000],
    [100330000, 1659950666000],
    [100340000, 1659971854000],
    [100350001, 1660020731000],
    [100360000, 1660051015000],
    [100370000, 1660088507000],
    [100380000, 1660128077000],
    [100390000, 1660150743000],
    [100400000, 1660199251000],
    [100410000, 1660224708000],
    [100420000, 1660263125000],
    [100430000, 1660299268000],
    [100440000, 1660319860000],
    [100450000, 1660368097000],
    [100460000, 1660395113000],
    [100470000, 1660428835000],
    [100480002, 1660467041000],
    [100490000, 1660488418000],
    [100500001, 1660532076000],
    [100510000, 1660563664000],
    [100520000, 1660586072000],
    [100530000, 1660636094000],
    [100540000, 1660660143000],
    [100550001, 1660705709000],
    [100560000, 1660738438000],
    [100570000, 1660769636000],
    [100580000, 1660816247000],
    [100590000, 1660838488000],
    [100600000, 1660890601000],
    [100610000, 1660916383000],
    [100620000, 1660954023000],
    [100630000, 1660988123000],
    [100640000, 1661008313000],
    [100650000, 1661051552000],
    [100660000, 1661078751000],
    [100670000, 1661096020000],
    [100680001, 1661145736000],
    [100690000, 1661175462000],
    [100700000, 1661215588000],
    [100710000, 1661254370000],
    [100720000, 1661280952000],
    [100730001, 1661333728000],
    [100740001, 1661358099000],
    [100750000, 1661414497000],
    [100760000, 1661440179000],
    [100770000, 1661492341000],
    [100780000, 1661521055000],
    [100790000, 1661561197000],
    [100800000, 1661595564000],
    [100810000, 1661617049000],
    [100820001, 1661662917000],
    [100830000, 1661689252000],
    [100840000, 1661718103000],
    [100850000, 1661768784000],
    [100860000, 1661793527000],
    [100870000, 1661849515000],
    [100880001, 1661872195000],
    [100890000, 1661917529000],
    [100900000, 1661948051000],
    [100910000, 1661969237000],
    [100920000, 1662023950000],
    [100930000, 1662046758000],
    [100940001, 1662101858000],
    [100950000, 1662128942000],
    [100960000, 1662173387000],
    [100970000, 1662204890000],
    [100980000, 1662228844000],
    [100990000, 1662273369000],
    [101000001, 1662296585000],
    [101010000, 1662330600000],
    [101020000, 1662375501000],
    [101030000, 1662399311000],
    [101040000, 1662454532000],
    [101050000, 1662478221000],
    [101060000, 1662534277000],
    [101070000, 1662562041000],
    [101080001, 1662615000000],
    [101090000, 1662646251000],
    [101100000, 1662695101000],
    [101110000, 1662728898000],
    [101120000, 1662767763000],
    [101130000, 1662804411000],
    [101140000, 1662826306000],
    [101150000, 1662873053000],
    [101160000, 1662898991000],
    [101170000, 1662921420000],
    [101180000, 1662975008000],
    [101190000, 1662998412000],
    [101200001, 1663055550000],
    [101210000, 1663081475000],
    [101220001, 1663137351000],
    [101230000, 1663167043000],
    [101240000, 1663220786000],
    [101250000, 1663252123000],
    [101260001, 1663303174000],
    [101270000, 1663335605000],
    [101280000, 1663377786000],
    [101290000, 1663411636000],
    [101300000, 1663434514000],
    [101310000, 1663481292000],
    [101320000, 1663506006000],
    [101330000, 1663538154000],
    [101340000, 1663575100000],
    [101350000, 1663596656000],
    [101360001, 1663642801000],
    [101370000, 1663679427000],
    [101380001, 1663723527000],
    [101390001, 1663764847000],
    [101400000, 1663809476000],
    [101410000, 1663850607000],
    [101420000, 1663890675000],
    [101430000, 1663928623000],
    [101440000, 1663950553000],
    [101450000, 1663999393000],
    [101460000, 1664026202000],
    [101470000, 1664062966000],
    [101480000, 1664096841000],
    [101490000, 1664117420000],
    [101500000, 1664166014000],
    [101510001, 1664199412000],
    [101520000, 1664247441000],
    [101530000, 1664284337000],
    [101540000, 1664330061000],
    [101550000, 1664369209000],
    [101560000, 1664413801000],
    [101570000, 1664454981000],
    [101580000, 1664497469000],
    [101590000, 1664538598000],
    [101600000, 1664563614000],
    [101610001, 1664610820000],
    [101620001, 1664634225000],
    [101630000, 1664676093000],
    [101640000, 1664706511000],
    [101650000, 1664726088000],
    [101660000, 1664781204000],
    [101670001, 1664808690000],
    [101680001, 1664860503000],
    [101690000, 1664893068000],
    [101700000, 1664942458000],
    [101710000, 1664977095000],
    [101720000, 1665023401000],
    [101730000, 1665060915000],
    [101740000, 1665104252000],
    [101750000, 1665143938000],
    [101760000, 1665175158000],
    [101770000, 1665218600000],
    [101780000, 1665241251000],
    [101790000, 1665284588000],
    [101800000, 1665312938000],
    [101810000, 1665330821000],
    [101820000, 1665372149000],
    [101830000, 1665398820000],
    [101840000, 1665416394000],
    [101850001, 1665465350000],
    [101860000, 1665494515000],
    [101870000, 1665530852000],
    [101880000, 1665571548000],
    [101890000, 1665593214000],
    [101900000, 1665646911000],
    [101910001, 1665670737000],
    [101920000, 1665712000000],
    [101930001, 1665746769000],
    [101940000, 1665767338000],
    [101950001, 1665810534000],
    [101960001, 1665835542000],
    [101970000, 1665854271000],
    [101980000, 1665894802000],
    [101990000, 1665917567000],
    [102000000, 1665933751000],
    [102010000, 1665979225000],
    [102020001, 1666010201000],
    [102030000, 1666035288000],
    [102040000, 1666084139000],
    [102050001, 1666105768000],
    [102060000, 1666151876000],
    [102070000, 1666183433000],
    [102080000, 1666216827000],
    [102090000, 1666261199000],
    [102100000, 1666282384000],
    [102110000, 1666334679000],
    [102120000, 1666359814000],
    [102130000, 1666396801000],
    [102140000, 1666428164000],
    [102150000, 1666448425000],
    [102160000, 1666485189000],
    [102170000, 1666514196000],
    [102180001, 1666532789000],
    [102190001, 1666565854000],
    [102200000, 1666605896000],
    [102210001, 1666626596000],
    [102220000, 1666675350000],
    [102230000, 1666704277000],
    [102240000, 1666741236000],
    [102250000, 1666781943000],
    [102260000, 1666803294000],
    [102270000, 1666855732000],
    [102280000, 1666881175000],
    [102290000, 1666926797000],
    [102300000, 1666959138000],
    [102310001, 1666985187000],
    [102320000, 1667026497000],
    [102330000, 1667049301000],
    [102340000, 1667074375000],
    [102350000, 1667110331000],
    [102360000, 1667130422000],
    [102370000, 1667143380000],
    [102380000, 1667177656000],
    [102390001, 1667206886000],
    [102400000, 1667221135000],
    [102410000, 1667232357000],
    [102420000, 1667272246000],
    [102430000, 1667301644000],
    [102440000, 1667320439000],
    [102450000, 1667367598000],
    [102460000, 1667394858000],
    [102470000, 1667427666000],
    [102480000, 1667464915000],
    [102490000, 1667485160000],
    [102500000, 1667524862000],
    [102510000, 1667560354000],
    [102520000, 1667580817000],
    [102530000, 1667624234000],
    [102540000, 1667649179000],
    [102550001, 1667668772000],
    [102560000, 1667710692000],
    [102570000, 1667734400000],
    [102580000, 1667750824000],
    [102590000, 1667797645000],
    [102600000, 1667825493000],
    [102610000, 1667852133000],
    [102620000, 1667897579000],
    [102630000, 1667919378000],
    [102640000, 1667962893000],
    [102650000, 1667995549000],
    [102660000, 1668020437000],
    [102670004, 1668068451000],
    [102680000, 1668090936000],
    [102690000, 1668129462000],
    [102700000, 1668162955000],
    [102710002, 1668179889000],
    [102720000, 1668219563000],
    [102730000, 1668247564000],
    [102740001, 1668265922000],
    [102750000, 1668303741000],
    [102760000, 1668329693000],
    [102770000, 1668346874000],
    [102780000, 1668375663000],
    [102790000, 1668418003000],
    [102800000, 1668438783000],
    [102810000, 1668484194000],
    [102820000, 1668514548000],
    [102830000, 1668539261000],
    [102840000, 1668587302000],
    [102850000, 1668609630000],
    [102860001, 1668653622000],
    [102870000, 1668686822000],
    [102880000, 1668713232000],
    [102890000, 1668756421000],
    [102900001, 1668779395000],
    [102910000, 1668811400000],
    [102920001, 1668844024000],
    [102930002, 1668865313000],
    [102940000, 1668896876000],
    [102950000, 1668928452000],
    [102960000, 1668948443000],
    [102970000, 1668971136000],
    [102980000, 1669018484000],
    [102990000, 1669041914000],
    [103000001, 1669085862000],
    [103010000, 1669119022000],
    [103020000, 1669143271000],
    [103030001, 1669186087000],
    [103040000, 1669207952000],
    [103050000, 1669233127000],
    [103060001, 1669279725000],
    [103070000, 1669301857000],
    [103080000, 1669347356000],
    [103090000, 1669378612000],
    [103100000, 1669404460000],
    [103110000, 1669445020000],
    [103120000, 1669467819000],
    [103130000, 1669492431000],
    [103140001, 1669530622000],
    [103150000, 1669551525000],
    [103160000, 1669570387000],
    [103170000, 1669620905000],
    [103180000, 1669644820000],
    [103190000, 1669685233000],
    [103200000, 1669720198000],
    [103210000, 1669740091000],
    [103220001, 1669787993000],
    [103230000, 1669813297000],
    [103240000, 1669837950000],
    [103250000, 1669883072000],
    [103260000, 1669905232000],
    [103270001, 1669948516000],
    [103280000, 1669981254000],
    [103290000, 1670003254000],
    [103300000, 1670048941000],
    [103310000, 1670072034000],
    [103320000, 1670093871000],
    [103330000, 1670133300000],
    [103340000, 1670154660000],
    [103350000, 1670172240000],
    [103360001, 1670220480000],
    [103370000, 1670245740000],
    [103380000, 1670277480000],
    [103390000, 1670319000000],
    [103400000, 1670339280000],
    [103410000, 1670384520000],
    [103420000, 1670416800000],
    [103430000, 1670448720000],
    [103440002, 1670492880000],
    [103450000, 1670513040000],
    [103460000, 1670560200000],
    [103470000, 1670589120000],
    [103480000, 1670613240000],
    [103490000, 1670655000000],
    [103500000, 1670677320000],
    [103510000, 1670706120000],
    [103520000, 1670741520000],
    [103530002, 1670762040000],
    [103540000, 1670783700000],
    [103550000, 1670833980000],
    [103560000, 1670856420000],
    [103570000, 1670900760000],
    [103580000, 1670934000000],
    [103590000, 1670962320000],
    [103600000, 1671009780000],
    [103610000, 1671030900000],
    [103620000, 1671080700000],
    [103630000, 1671109800000],
    [103640000, 1671144180000],
    [103650000, 1671185160000],
    [103660000, 1671204600000],
    [103670000, 1671247980000],
    [103680000, 1671274620000],
    [103690000, 1671292800000],
    [103700000, 1671336000000],
    [103710000, 1671361620000],
    [103720000, 1671379740000],
    [103730001, 1671430260000],
    [103740000, 1671457140000],
    [103750000, 1671496200000],
    [103760000, 1671534780000],
    [103770000, 1671558060000],
    [103780001, 1671608340000],
    [103790000, 1671632100000],
    [103800000, 1671673020000],
    [103810000, 1671708240000],
    [103820000, 1671729660000],
    [103830000, 1671779400000],
    [103840000, 1671802980000],
    [103850000, 1671830220000],
    [103860000, 1671863040000],
    [103870000, 1671882180000],
    [103880000, 1671894900000],
    [103890000, 1671923820000],
    [103900000, 1671949680000],
    [103910002, 1671967260000],
    [103920000, 1671979860000],
    [103930000, 1672016040000],
    [103940000, 1672049280000],
    [103950000, 1672068720000],
    [103960001, 1672113420000],
    [103970000, 1672143420000],
    [103980000, 1672170420000],
    [103990001, 1672215780000],
    [104000002, 1672237620000],
    [104010000, 1672277580000],
    [104020000, 1672308120000],
    [104030000, 1672327620000],
    [104040001, 1672369020000],
    [104050000, 1672396500000],
    [104060002, 1672414380000],
    [104070000, 1672451820000],
    [104080001, 1672474140000],
    [104090000, 1672489980000],
    [104100000, 1672499040000],
    [104110000, 1672506000000],
    [104120000, 1672535640000],
    [104130000, 1672556220000],
    [104140000, 1672573980000],
    [104150000, 1672587900000],
    [104160000, 1672627680000],
    [104170000, 1672652520000],
    [104180000, 1672671420000],
    [104190001, 1672707180000],
    [104200000, 1672735260000],
    [104210000, 1672753800000],
    [104220000, 1672785120000],
    [104230000, 1672820400000],
    [104240000, 1672840680000],
    [104250001, 1672873500000],
    [104260001, 1672910940000],
    [104270000, 1672930620000],
    [104280000, 1672973160000],
    [104290000, 1673003400000],
    [104300000, 1673021340000],
    [104310000, 1673064360000],
    [104320000, 1673089800000],
    [104330000, 1673107200000],
    [104340000, 1673147580000],
    [104350000, 1673173320000],
    [104360000, 1673190720000],
    [104370003, 1673229120000],
    [104380001, 1673256120000],
    [104390000, 1673274420000],
    [104400000, 1673311620000],
    [104410000, 1673348880000],
    [104420001, 1673368980000],
    [104430000, 1673420820000],
    [104440000, 1673445180000],
    [104450000, 1673485080000],
    [104460000, 1673522160000],
    [104470000, 1673545560000],
    [104480000, 1673596860000],
    [104490002, 1673619300000],
    [104500000, 1673657220000],
    [104510000, 1673688540000],
    [104520000, 1673708040000],
    [104530000, 1673745120000],
    [104540000, 1673773260000],
    [104550000, 1673791200000],
    [104560000, 1673825220000],
    [104570000, 1673864820000],
    [104580000, 1673885040000],
    [104590000, 1673934000000],
    [104600000, 1673962200000],
    [104610000, 1673999460000],
    [104620000, 1674039420000],
    [104630000, 1674061140000],
    [104640000, 1674113160000],
    [104650000, 1674138060000],
    [104660000, 1674181200000],
    [104670000, 1674214680000],
    [104680000, 1674236760000],
    [104690000, 1674280920000],
    [104700000, 1674304680000],
    [104710000, 1674326040000],
    [104720001, 1674367080000],
    [104730000, 1674389700000],
    [104740000, 1674409380000],
    [104750000, 1674459780000],
    [104760000, 1674483000000],
    [104770000, 1674523860000],
    [104780000, 1674559440000],
    [104790000, 1674583860000],
    [104800000, 1674634020000],
    [104810000, 1674656880000],
    [104820000, 1674698640000],
    [104830000, 1674733980000],
    [104840000, 1674756420000],
    [104850000, 1674807060000],
    [104860000, 1674828660000],
    [104870000, 1674865320000],
    [104880000, 1674896700000],
    [104890000, 1674916080000],
    [104900000, 1674951780000],
    [104910000, 1674981120000],
    [104920000, 1674999360000],
    [104930000, 1675029120000],
    [104940000, 1675071180000],
    [104950000, 1675090860000],
    [104960000, 1675134780000],
    [104970000, 1675165980000],
    [104980000, 1675184040000],
    [104990000, 1675233960000],
    [105000000, 1675257900000],
    [105010000, 1675291800000],
    [105020000, 1675331520000],
    [105030000, 1675350300000],
    [105040000, 1675392780000],
    [105050000, 1675420440000],
    [105060000, 1675436520000],
    [105070000, 1675474320000],
    [105080000, 1675503360000],
    [105090000, 1675522200000],
    [105100000, 1675558380000],
    [105110000, 1675585200000],
    [105120002, 1675602960000],
    [105130000, 1675627260000],
    [105140000, 1675672260000],
    [105150000, 1675693560000],
    [105160000, 1675731060000],
    [105170000, 1675766760000],
    [105180000, 1675785300000],
    [105190000, 1675833960000],
    [105200000, 1675860600000],
    [105210000, 1675885440000],
    [105220000, 1675932240000],
    [105230000, 1675953420000],
    [105240000, 1675992780000],
    [105250002, 1676024520000],
    [105260000, 1676041800000],
    [105270002, 1676079780000],
    [105280000, 1676107800000],
    [105290002, 1676126580000],
    [105300000, 1676160780000],
    [105310000, 1676188440000],
    [105320000, 1676206560000],
    [105330000, 1676225340000],
    [105340000, 1676272800000],
    [105350000, 1676294880000],
    [105360000, 1676311260000],
    [105370000, 1676348580000],
    [105380001, 1676371620000],
    [105390000, 1676384640000],
    [105400000, 1676409600000],
    [105410000, 1676449440000],
    [105420000, 1676470320000],
    [105430000, 1676508780000],
    [105440000, 1676545200000],
    [105450000, 1676564940000],
    [105460000, 1676612940000],
    [105470000, 1676639400000],
    [105480000, 1676666880000],
    [105490000, 1676704980000],
    [105500000, 1676726100000],
    [105510000, 1676749800000],
    [105520000, 1676785860000],
    [105530000, 1676807280000],
    [105540000, 1676822100000],
    [105550000, 1676867460000],
    [105560000, 1676895900000],
    [105570000, 1676917800000],
    [105580000, 1676964120000],
    [105590000, 1676987460000],
    [105600000, 1677019320000],
    [105610000, 1677056400000],
    [105620000, 1677074880000],
    [105630000, 1677107340000],
    [105640000, 1677140760000],
    [105650000, 1677160500000],
    [105660001, 1677190980000],
    [105670001, 1677230340000],
    [105680000, 1677250140000],
    [105690000, 1677286440000],
    [105700000, 1677315960000],
    [105710000, 1677334920000],
    [105720000, 1677367080000],
    [105730000, 1677396720000],
    [105740000, 1677415680000],
    [105750000, 1677433800000],
    [105760000, 1677480480000],
    [105770000, 1677503760000],
    [105780000, 1677530640000],
    [105790000, 1677572460000],
    [105800000, 1677592500000],
    [105810000, 1677622380000],
    [105820000, 1677661200000],
    [105830000, 1677680520000],
    [105840000, 1677717300000],
    [105850000, 1677752820000],
    [105860000, 1677771060000],
    [105870001, 1677815100000],
    [105880000, 1677843900000],
    [105890000, 1677860640000],
    [105900000, 1677902400000],
    [105910000, 1677927600000],
    [105920000, 1677944280000],
    [105930000, 1677982920000],
    [105940000, 1678007760000],
    [105950000, 1678024380000],
    [105960000, 1678053060000],
    [105970000, 1678092960000],
    [105980001, 1678112340000],
    [105990001, 1678147500000],
    [106000000, 1678183140000],
    [106010000, 1678201560000],
    [106020000, 1678243800000],
    [106030000, 1678274820000],
    [106040000, 1678291920000],
    [106050000, 1678335600000],
    [106060000, 1678362480000],
    [106070000, 1678378860000],
    [106080000, 1678424700000],
    [106090000, 1678450320000],
    [106100000, 1678468860000],
    [106110003, 1678509420000],
    [106120000, 1678534200000],
    [106130000, 1678551480000],
    [106140000, 1678589880000],
    [106150000, 1678612680000],
    [106160000, 1678628940000],
    [106170000, 1678657140000],
    [106180001, 1678696680000],
    [106190000, 1678716480000],
    [106200000, 1678748040000],
    [106210002, 1678784760000],
    [106220000, 1678803420000],
    [106230001, 1678837920000],
    [106240000, 1678873860000],
    [106250000, 1678892280000],
    [106260000, 1678932720000],
    [106270000, 1678964280000],
    [106280000, 1678981860000],
    [106290000, 1679026080000],
    [106300000, 1679053320000],
    [106310000, 1679069340000],
    [106320000, 1679109120000],
    [106330000, 1679134200000],
    [106340000, 1679151540000],
    [106350001, 1679186760000],
    [106360001, 1679214420000],
    [106370000, 1679232180000],
    [106380000, 1679252820000],
    [106390000, 1679296680000],
    [106400000, 1679318520000],
    [106410000, 1679342340000],
    [106420001, 1679380200000],
    [106430002, 1679401020000],
    [106440000, 1679418060000],
    [106450000, 1679463300000],
    [106460000, 1679488440000],
    [106470000, 1679508780000],
    [106480000, 1679554320000],
    [106490000, 1679577120000],
    [106500000, 1679600640000],
    [106510000, 1679643120000],
    [106520000, 1679663400000],
    [106530000, 1679683500000],
    [106540002, 1679720880000],
    [106550000, 1679743920000],
    [106560000, 1679759700000],
    [106570000, 1679797680000],
    [106580001, 1679821260000],
    [106590000, 1679837700000],
    [106600000, 1679859600000],
    [106610001, 1679901840000],
    [106620000, 1679923920000],
    [106630000, 1679951700000],
    [106640000, 1679991780000],
    [106650002, 1680012240000],
    [106660000, 1680045000000],
    [106670000, 1680081360000],
    [106680000, 1680100500000],
    [106690000, 1680135900000],
    [106700000, 1680170400000],
    [106710000, 1680188520000],
    [106720000, 1680226920000],
    [106730000, 1680257820000],
    [106740000, 1680274800000],
    [106750000, 1680304680000],
    [106760000, 1680332820000],
    [106770000, 1680352440000],
    [106780001, 1680368580000],
    [106790000, 1680406200000],
    [106800001, 1680429600000],
    [106810000, 1680445200000],
    [106820000, 1680474780000],
    [106830000, 1680512220000],
    [106840001, 1680530820000],
    [106850001, 1680562320000],
    [106860000, 1680598080000],
    [106870000, 1680617100000],
    [106880000, 1680649320000],
    [106890000, 1680684540000],
    [106900000, 1680702720000],
    [106910000, 1680732360000],
    [106920000, 1680770640000],
    [106930000, 1680789660000],
    [106940000, 1680820620000],
    [106950000, 1680856980000],
    [106960000, 1680876000000],
    [106970000, 1680904440000],
    [106980000, 1680936780000],
    [106990000, 1680956820000],
    [107000000, 1680973380000],
    [107010000, 1681010520000],
    [107020002, 1681032780000],
    [107030000, 1681047960000],
    [107040001, 1681070400000],
    [107050000, 1681111800000],
    [107060000, 1681132800000],
    [107070000, 1681158360000],
    [107080000, 1681200660000],
    [107090000, 1681221060000],
    [107100000, 1681251480000],
    [107110000, 1681290420000],
    [107120000, 1681309740000],
    [107130000, 1681345620000],
    [107140000, 1681381920000],
    [107150000, 1681399140000],
    [107160000, 1681441740000],
    [107170000, 1681471620000],
    [107180002, 1681488720000],
    [107190000, 1681527900000],
    [107200000, 1681553220000],
    [107210000, 1681570560000],
    [107220000, 1681603440000],
    [107230002, 1681629840000],
    [107240000, 1681647480000],
    [107250001, 1681661760000],
    [107260000, 1681704120000],
    [107270000, 1681732020000],
    [107280000, 1681748280000],
    [107290000, 1681792260000],
    [107300000, 1681819620000],
    [107310001, 1681837560000],
    [107320000, 1681883220000],
    [107330000, 1681908120000],
    [107340000, 1681928760000],
    [107350000, 1681974480000],
    [107360003, 1681996560000],
    [107370000, 1682024220000],
    [107380000, 1682065620000],
    [107390000, 1682084640000],
    [107400000, 1682111160000],
    [107410000, 1682145360000],
    [107420001, 1682165520000],
    [107430001, 1682181480000],
    [107440000, 1682219040000],
    [107450000, 1682241480000],
    [107460000, 1682256660000],
    [107470000, 1682277480000],
    [107480000, 1682321460000],
    [107490000, 1682341980000],
    [107500000, 1682365200000],
    [107510000, 1682407980000],
    [107520000, 1682428140000],
    [107530000, 1682453040000],
    [107540001, 1682495580000],
    [107550002, 1682515980000],
    [107560000, 1682545200000],
    [107570000, 1682586780000],
    [107580000, 1682605860000],
    [107590000, 1682641680000],
    [107600000, 1682676360000],
    [107610000, 1682693640000],
    [107620000, 1682727780000],
    [107630000, 1682755500000],
    [107640000, 1682773200000],
    [107650000, 1682790720000],
    [107660000, 1682826420000],
    [107670000, 1682850000000],
    [107680000, 1682863320000],
    [107690000, 1682884560000],
    [107700000, 1682922240000],
    [107710000, 1682942820000],
    [107720000, 1682957040000],
    [107730000, 1682996400000],
    [107740000, 1683022680000],
    [107750000, 1683039120000],
    [107760000, 1683071520000],
    [107770000, 1683101220000],
    [107780000, 1683120000000],
    [107790000, 1683143520000],
    [107800000, 1683179400000],
    [107810001, 1683201840000],
    [107820001, 1683218160000],
    [107830000, 1683255300000],
    [107840000, 1683279240000],
    [107850001, 1683295740000],
    [107860000, 1683320580000],
    [107870000, 1683352800000],
    [107880000, 1683373380000],
    [107890000, 1683388440000],
    [107900000, 1683424500000],
    [107910000, 1683446340000],
    [107920000, 1683462840000],
    [107930000, 1683478140000],
    [107940000, 1683523260000],
    [107950000, 1683549960000],
    [107960000, 1683569880000],
    [107970001, 1683615660000],
    [107980000, 1683638220000],
    [107990000, 1683654120000],
    [108000000, 1683699000000],
    [108010000, 1683723660000],
    [108020000, 1683747360000],
    [108030000, 1683792000000],
    [108040000, 1683813480000],
    [108050000, 1683848640000],
    [108060001, 1683886500000],
    [108070000, 1683905160000],
    [108080000, 1683945900000],
    [108090000, 1683973200000],
    [108100000, 1683990780000],
    [108110000, 1684027440000],
    [108120000, 1684053240000],
    [108130000, 1684070460000],
    [108140000, 1684093260000],
    [108150000, 1684137600000],
    [108160000, 1684159380000],
    [108170000, 1684195680000],
    [108180000, 1684233900000],
    [108190000, 1684252800000],
    [108200000, 1684299720000],
    [108210000, 1684327980000],
    [108220000, 1684357200000],
    [108230002, 1684399740000],
    [108240000, 1684421280000],
    [108250000, 1684463040000],
    [108260000, 1684494900000],
    [108270000, 1684513680000],
    [108280000, 1684555020000],
    [108290001, 1684580760000],
    [108300000, 1684598400000],
    [108310000, 1684637580000],
    [108320000, 1684661940000],
    [108330000, 1684678980000],
    [108340000, 1684712700000],
    [108350000, 1684749840000],
    [108360000, 1684768500000],
    [108370000, 1684811640000],
    [108380000, 1684843200000],
    [108390000, 1684864320000],
    [108400000, 1684911720000],
    [108410000, 1684935720000],
    [108420001, 1684969800000],
    [108430003, 1685009100000],
    [108440000, 1685028480000],
    [108450001, 1685073960000],
    [108460000, 1685102820000],
    [108470000, 1685123760000],
    [108480000, 1685164620000],
    [108490000, 1685188800000],
    [108500000, 1685208300000],
    [108510000, 1685248020000],
    [108520000, 1685271240000],
    [108530001, 1685286360000],
    [108540000, 1685326740000],
    [108550000, 1685358480000],
    [108560000, 1685376480000],
    [108570000, 1685421420000],
    [108580001, 1685449800000],
    [108590000, 1685474580000],
    [108600000, 1685519820000],
    [108610000, 1685540700000],
    [108620000, 1685570040000],
    [108630001, 1685610000000],
    [108640000, 1685630400000],
    [108650000, 1685668560000],
    [108660000, 1685701020000],
    [108670000, 1685719140000],
    [108680000, 1685759160000],
    [108690000, 1685785800000],
    [108700000, 1685803920000],
    [108710000, 1685838180000],
    [108720000, 1685865720000],
    [108730001, 1685883720000],
    [108740001, 1685905860000],
    [108750000, 1685950980000],
    [108760000, 1685972520000],
    [108770000, 1686003060000],
    [108780000, 1686043020000],
    [108790000, 1686063060000],
    [108800000, 1686103740000],
    [108810001, 1686136800000],
    [108820000, 1686156120000],
    [108830000, 1686203460000],
    [108840001, 1686229380000],
    [108850000, 1686259620000],
    [108860002, 1686300720000],
    [108870000, 1686320040000],
    [108880000, 1686353760000],
    [108890000, 1686385260000],
    [108900000, 1686404640000],
    [108910000, 1686434340000],
    [108920000, 1686465900000],
    [108930001, 1686485700000],
    [108940000, 1686503160000],
    [108950000, 1686548640000],
    [108960000, 1686574020000],
    [108970000, 1686598500000],
    [108980000, 1686643500000],
    [108990000, 1686665940000],
    [109000000, 1686704040000],
    [109010001, 1686739320000],
    [109020000, 1686758040000],
    [109030000, 1686805320000],
    [109040000, 1686833400000],
    [109050000, 1686859560000],
    [109060000, 1686903900000],
    [109070000, 1686924660000],
    [109080000, 1686956940000],
    [109090000, 1686988800000],
    [109100000, 1687008840000],
    [109110000, 1687035960000],
    [109120000, 1687068600000],
    [109130000, 1687087860000],
    [109140000, 1687102200000],
    [109150000, 1687143420000],
    [109160000, 1687173180000],
    [109170000, 1687190400000],
    [109180000, 1687234980000],
    [109190000, 1687263180000],
    [109200000, 1687284420000],
    [109210000, 1687330080000],
    [109220000, 1687353060000],
    [109230000, 1687382640000],
    [109240000, 1687423140000],
    [109250000, 1687442400000],
    [109260001, 1687474080000],
    [109270000, 1687511760000],
    [109280000, 1687530360000],
    [109290001, 1687562880000],
    [109300000, 1687593060000],
    [109310000, 1687612500000],
    [109320000, 1687634820000],
    [109330000, 1687671000000],
    [109340001, 1687692600000],
    [109350000, 1687707420000],
    [109360000, 1687749240000],
    [109370001, 1687779900000],
    [109380001, 1687799580000],
    [109390000, 1687846200000],
    [109400000, 1687871760000],
    [109410000, 1687903200000],
    [109420000, 1687943220000],
    [109430000, 1687963260000],
    [109440000, 1688003160000],
    [109450000, 1688036280000],
    [109460000, 1688054820000],
    [109470000, 1688097420000],
    [109480000, 1688125380000],
    [109490000, 1688141160000],
    [109500000, 1688180400000],
    [109510000, 1688205480000],
    [109520000, 1688223120000],
    [109530000, 1688253120000],
    [109540000, 1688273940000],
    [109550000, 1688290980000],
    [109560000, 1688304720000],
    [109570001, 1688323260000],
    [109580000, 1688364780000],
    [109590000, 1688387280000],
    [109600000, 1688405580000],
    [109610000, 1688449320000],
    [109620000, 1688473860000],
    [109630001, 1688494080000],
    [109640000, 1688536800000],
    [109650001, 1688561100000],
    [109660000, 1688584560000],
    [109670000, 1688628540000],
    [109680000, 1688651100000],
    [109690000, 1688680200000],
    [109700000, 1688718480000],
    [109710000, 1688736840000],
    [109720000, 1688764440000],
    [109730000, 1688798940000],
    [109740000, 1688820240000],
    [109750000, 1688840940000],
    [109760000, 1688878380000],
    [109770000, 1688900400000],
    [109780000, 1688915880000],
    [109790000, 1688956680000],
    [109800000, 1688987760000],
    [109810000, 1689006420000],
    [109820001, 1689051540000],
    [109830000, 1689079500000],
    [109840001, 1689106560000],
    [109850000, 1689150600000],
    [109860001, 1689171960000],
    [109870001, 1689209880000],
    [109880000, 1689244800000],
    [109890000, 1689264120000],
    [109900000, 1689309480000],
    [109910000, 1689337560000],
    [109920000, 1689358200000],
    [109930000, 1689398340000],
    [109940000, 1689422700000],
    [109950000, 1689441540000],
    [109960000, 1689480960000],
    [109970000, 1689505500000],
    [109980000, 1689521820000],
    [109990000, 1689560700000],
    [110000000, 1689586440000],
    [110010000, 1689604020000],
    [110020000, 1689639480000],
    [110030000, 1689676320000],
    [110040000, 1689695520000],
    [110050000, 1689741720000],
    [110060000, 1689770400000],
    [110070000, 1689797460000],
    [110080000, 1689840600000],
    [110090000, 1689862500000],
    [110100000, 1689898920000],
    [110110000, 1689934260000],
    [110120000, 1689952680000],
    [110130000, 1689993300000],
    [110140000, 1690020120000],
    [110150000, 1690038720000],
    [110160000, 1690076040000],
    [110170000, 1690102860000],
    [110180000, 1690120800000],
    [110190001, 1690153080000],
    [110200001, 1690190220000],
    [110210000, 1690209420000],
    [110220000, 1690246920000],
    [110230000, 1690281120000],
    [110240000, 1690300080000],
    [110250000, 1690343880000],
    [110260000, 1690372800000],
    [110270000, 1690393500000],
    [110280000, 1690438260000],
    [110290000, 1690462800000],
    [110300000, 1690491420000],
    [110310000, 1690531860000],
    [110320000, 1690552560000],
    [110330000, 1690583820000],
    [110340000, 1690615680000],
    [110350000, 1690636680000],
    [110360000, 1690662060000],
    [110370000, 1690696680000],
    [110380000, 1690718400000],
    [110390000, 1690735020000],
    [110400000, 1690778640000],
    [110410001, 1690805640000],
    [110420000, 1690823580000],
    [110430000, 1690867680000],
    [110440000, 1690893060000],
    [110450000, 1690913940000],
    [110460000, 1690957080000],
    [110470000, 1690980480000],
    [110480001, 1691003760000],
    [110490000, 1691047740000],
    [110500000, 1691070060000],
    [110510000, 1691102940000],
    [110520000, 1691139480000],
    [110530000, 1691158860000],
    [110540000, 1691194080000],
    [110550000, 1691224200000],
    [110560000, 1691244240000],
    [110570000, 1691276220000],
    [110580000, 1691305260000],
    [110590000, 1691324880000],
    [110600000, 1691342400000],
    [110610000, 1691385300000],
    [110620000, 1691410680000],
    [110630000, 1691429760000],
    [110640000, 1691474400000],
    [110650000, 1691498040000],
    [110660000, 1691519820000],
    [110670000, 1691563620000],
    [110680000, 1691586780000],
    [110690000, 1691612940000],
    [110700000, 1691653440000],
    [110710000, 1691674620000],
    [110720002, 1691702460000],
    [110730000, 1691737080000],
    [110740000, 1691758260000],
    [110750000, 1691782980000],
    [110760000, 1691821380000],
    [110770000, 1691845260000],
    [110780000, 1691873280000],
    [110790000, 1691909640000],
    [110800001, 1691931420000],
    [110810000, 1691954460000],
    [110820001, 1691995560000],
    [110830000, 1692017520000],
    [110840000, 1692041220000],
    [110850001, 1692078600000],
    [110860000, 1692100740000],
    [110870001, 1692117240000],
    [110880000, 1692158820000],
    [110890000, 1692185640000],
    [110900000, 1692203760000],
    [110910000, 1692247560000],
    [110920000, 1692274380000],
    [110930000, 1692294960000],
    [110940000, 1692339600000],
    [110950000, 1692363000000],
    [110960002, 1692385260000],
    [110970000, 1692423120000],
    [110980000, 1692446940000],
    [110990000, 1692465420000],
    [111000000, 1692505140000],
    [111010000, 1692529200000],
    [111020000, 1692545340000],
    [111030000, 1692586800000],
    [111040000, 1692616500000],
    [111050000, 1692633840000],
    [111060000, 1692677940000],
    [111070000, 1692706320000],
    [111080000, 1692728040000],
    [111090000, 1692774600000],
    [111100000, 1692797880000],
    [111110000, 1692829320000],
    [111120000, 1692868920000],
    [111130000, 1692889260000],
    [111140000, 1692931440000],
    [111150001, 1692962400000],
    [111160000, 1692981120000],
    [111170000, 1693022520000],
    [111180000, 1693048920000],
    [111190000, 1693066740000],
    [111200000, 1693105920000],
    [111210000, 1693130760000],
    [111220000, 1693146780000],
    [111230000, 1693179900000],
    [111240000, 1693216800000],
    [111250000, 1693236180000],
    [111260000, 1693280100000],
    [111270000, 1693310820000],
    [111280000, 1693332840000],
    [111290001, 1693380180000],
    [111300000, 1693403160000],
    [111310001, 1693428480000],
    [111320000, 1693468260000],
    [111330001, 1693487760000],
    [111340001, 1693510320000],
    [111350003, 1693555260000],
    [111360000, 1693576320000],
    [111370000, 1693607700000],
    [111380000, 1693640520000],
    [111390000, 1693661700000],
    [111400000, 1693689600000],
    [111410000, 1693722780000],
    [111420000, 1693743000000],
    [111430000, 1693762200000],
    [111440000, 1693810800000],
    [111450000, 1693834920000],
    [111460000, 1693869840000],
    [111470000, 1693908840000],
    [111480000, 1693928340000],
    [111490000, 1693974900000],
    [111500000, 1694004240000],
    [111510001, 1694033880000],
    [111520000, 1694077200000],
    [111530000, 1694098500000],
    [111540000, 1694140200000],
    [111550000, 1694171580000],
    [111560000, 1694190360000],
    [111570000, 1694231460000],
    [111580000, 1694257800000],
    [111590000, 1694274960000],
    [111600000, 1694314080000],
    [111610000, 1694338920000],
    [111620000, 1694356740000],
    [111630001, 1694395140000],
    [111640001, 1694430660000],
    [111650001, 1694450520000],
    [111660000, 1694498580000],
    [111670000, 1694524620000],
    [111680000, 1694557200000],
    [111690000, 1694599680000],
    [111700000, 1694619840000],
    [111710000, 1694667060000],
    [111720000, 1694696520000],
    [111730000, 1694729400000],
    [111740000, 1694770620000],
    [111750002, 1694790660000],
    [111760000, 1694830860000],
    [111770000, 1694860260000],
    [111780000, 1694879100000],
    [111790000, 1694919600000],
    [111800000, 1694946540000],
    [111810000, 1694963340000],
    [111820000, 1695001260000],
    [111830000, 1695029700000],
    [111840000, 1695048060000],
    [111850000, 1695086280000],
    [111860000, 1695122160000],
    [111870001, 1695141840000],
    [111880000, 1695189780000],
    [111890000, 1695215700000],
    [111900000, 1695247200000],
    [111910000, 1695287820000],
    [111920001, 1695308880000],
    [111930000, 1695352920000],
    [111940001, 1695384180000],
    [111950000, 1695404580000],
    [111960000, 1695445500000],
    [111970000, 1695469980000],
    [111980000, 1695487140000],
    [111990000, 1695526380000],
    [112000001, 1695552120000],
    [112010001, 1695568200000],
    [112020000, 1695611100000],
    [112030000, 1695643200000],
    [112040000, 1695667740000],
    [112050000, 1695716580000],
    [112060000, 1695739440000],
    [112070000, 1695781680000],
    [112080003, 1695816000000],
    [112090000, 1695842340000],
    [112100000, 1695891420000],
    [112110000, 1695913620000],
    [112120000, 1695959760000],
    [112130000, 1695989580000],
    [112140000, 1696011480000],
    [112150000, 1696053000000],
    [112160000, 1696076160000],
    [112170000, 1696093620000],
    [112180000, 1696132920000],
    [112190000, 1696157040000],
    [112200000, 1696172880000],
    [112210000, 1696214400000],
    [112220000, 1696246920000],
    [112230000, 1696269060000],
    [112240000, 1696318140000],
    [112250000, 1696341900000],
    [112260000, 1696381200000],
    [112270000, 1696417440000],
    [112280000, 1696436820000],
    [112290001, 1696485180000],
    [112300000, 1696511700000],
    [112310000, 1696544580000],
    [112320000, 1696585620000],
    [112330000, 1696605060000],
    [112340000, 1696645920000],
    [112350000, 1696674900000],
    [112360000, 1696694280000],
    [112370000, 1696735020000],
    [112380000, 1696762020000],
    [112390000, 1696779660000],
    [112400000, 1696819440000],
    [112410001, 1696845480000],
    [112420000, 1696863600000],
    [112430001, 1696902060000],
    [112440000, 1696936260000],
    [112450000, 1696956000000],
    [112460000, 1697004720000],
    [112470000, 1697031000000],
    [112480000, 1697066040000],
    [112490000, 1697105940000],
    [112500000, 1697126460000],
    [112510001, 1697174580000],
    [112520000, 1697201940000],
    [112530000, 1697231340000],
    [112540000, 1697268600000],
    [112550001, 1697290320000],
    [112560000, 1697320800000],
    [112570000, 1697353620000],
    [112580000, 1697373720000],
    [112590000, 1697394720000],
    [112600000, 1697443920000],
    [112610001, 1697466600000],
    [112620000, 1697508000000],
    [112630001, 1697543160000],
    [112640000, 1697568480000],
    [112650000, 1697616420000],
    [112660000, 1697639460000],
    [112670000, 1697680920000],
    [112680000, 1697714760000],
    [112690000, 1697737800000],
    [112700000, 1697785200000],
    [112710000, 1697808960000],
    [112720000, 1697841600000],
    [112730000, 1697876220000],
    [112740000, 1697896680000],
    [112750000, 1697930100000],
    [112760000, 1697962140000],
    [112770000, 1697981640000],
    [112780000, 1698013560000],
    [112790000, 1698055200000],
    [112800000, 1698074880000],
    [112810000, 1698120480000],
    [112820000, 1698151260000],
    [112830000, 1698182640000],
    [112840000, 1698225240000],
    [112850000, 1698246300000],
    [112860000, 1698291720000],
    [112870000, 1698323100000],
    [112880000, 1698353100000],
    [112890000, 1698397200000],
    [112900000, 1698418320000],
    [112910000, 1698456120000],
    [112920001, 1698487200000],
    [112930000, 1698505980000],
    [112940000, 1698545760000],
    [112950000, 1698572460000],
    [112960000, 1698590160000],
    [112970000, 1698627840000],
    [112980000, 1698662640000],
    [112990000, 1698678960000],
    [113000000, 1698713040000],
    [113010000, 1698742740000],
    [113020000, 1698756120000],
    [113030000, 1698767160000],
    [113040000, 1698803280000],
    [113050000, 1698834840000],
    [113060000, 1698853620000],
    [113070000, 1698897900000],
    [113080000, 1698928440000],
    [113090000, 1698954900000],
    [113100000, 1698994800000],
    [113110000, 1699017180000],
    [113120000, 1699047600000],
    [113130000, 1699083720000],
    [113140000, 1699104600000],
    [113150000, 1699132740000],
    [113160000, 1699167180000],
    [113170000, 1699187340000],
    [113180000, 1699207560000],
    [113190000, 1699256220000],
    [113200000, 1699279260000],
    [113210000, 1699317360000],
    [113220000, 1699354800000],
    [113230000, 1699374960000],
    [113240000, 1699425120000],
    [113250000, 1699451580000],
    [113260000, 1699491300000],
    [113270000, 1699528980000],
    [113280000, 1699553880000],
    [113290000, 1699604160000],
    [113300000, 1699626360000],
    [113310000, 1699660680000],
    [113320000, 1699692180000],
    [113330000, 1699711260000],
    [113340000, 1699743780000],
    [113350000, 1699775640000],
    [113360001, 1699795200000],
    [113370002, 1699822980000],
    [113380000, 1699867980000],
    [113390000, 1699889100000],
    [113400000, 1699937820000],
    [113410002, 1699968180000],
    [113420000, 1700005080000],
    [113430000, 1700045400000],
    [113440000, 1700067660000],
    [113450000, 1700119380000],
    [113460000, 1700143860000],
    [113470000, 1700185740000],
    [113480000, 1700219880000],
    [113490000, 1700240460000],
    [113500000, 1700283780000],
    [113510000, 1700308380000],
    [113520000, 1700327100000],
    [113530000, 1700368320000],
    [113540001, 1700392500000],
    [113550000, 1700408820000],
    [113560000, 1700455380000],
    [113570000, 1700484180000],
    [113580000, 1700511420000],
    [113590000, 1700557200000],
    [113600000, 1700578380000],
    [113610000, 1700619420000],
    [113620000, 1700652180000],
    [113630000, 1700672280000],
    [113640000, 1700716260000],
    [113650000, 1700742000000],
    [113660000, 1700766360000],
    [113670000, 1700814060000],
    [113680000, 1700836200000],
    [113690000, 1700875140000],
    [113700000, 1700905620000],
    [113710001, 1700924580000],
    [113720001, 1700963580000],
    [113730000, 1700990220000],
    [113740000, 1701008100000],
    [113750000, 1701043740000],
    [113760000, 1701081480000],
];


/***/ }),

/***/ "./src/ts/utils/CreateCSV.ts":
/*!***********************************!*\
  !*** ./src/ts/utils/CreateCSV.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   createCSV: () => (/* binding */ createCSV)
/* harmony export */ });
// 生成 csv 文件
// csv 文件结构参考 https://www.jianshu.com/p/54b3afc06126
const example = [
    ['titleA', 'titleB', 'titleC'],
    ['a1', 'b1', 'c1'],
    [[1, 2, 3], false, 456],
    [undefined, 'b,b,b', 'c c c'],
];
// 每一项数据可以是任何类型（any）。如果它不是 String，它会被自动转换为 String。
// 自动转换的结果可能不符合你的预期。如果你要完全控制输出的内容，你应该自己把内容全部转换成字符串，再传递到这个类里。
// 这个类会自动处理需要添加双引号的情况，所以你不用自己添加双引号。
// 如果某个数据是 Array，它不应该包含：1. 嵌套的数组；2. Symbol 类型的数据。
class CreateCSV {
    constructor() {
        this.separate = ','; // 分隔符
        this.CRLF = '\r\n'; // 换行符
        this.reg = / |\"|,|\n/; // 判断字符串是否需要添加双引号
        this.utf8BOM = this.UTF8BOM();
    }
    // 需要双引号包裹的情况：含有逗号、换行符、空格、双引号。
    // 全角符号不需要添加双引号。
    // 格式化每一行的数据
    format(arr) {
        const rowData = [];
        // 格式化这一行里每一列的数据
        for (let i = 0; i < arr.length; i++) {
            let colData = '';
            const origin = arr[i];
            const type = typeof origin;
            // 把每一列的数据转换成字符串
            switch (type) {
                case 'string':
                    colData = origin;
                    break;
                case 'bigint':
                case 'number':
                case 'boolean':
                case 'function':
                    colData = origin.toString();
                    break;
                case 'undefined':
                    colData = '';
                    break;
                case 'symbol':
                    colData = origin.description || '';
                    break;
                case 'object':
                    if (origin === null) {
                        colData = '';
                    }
                    else if (Array.isArray(origin)) {
                        colData = origin.join(this.separate);
                    }
                    break;
                default:
                    throw new Error('Unkown data type');
            }
            // 值原本就有的双引号，要替换成两个双引号
            colData = colData.replace(/\"/g, '""');
            // 用双引号包裹这个值
            if (this.reg.test(colData)) {
                colData = this.addQuotation(colData);
            }
            rowData.push(colData);
        }
        return rowData;
    }
    create(data) {
        // 储存结果。每行的结果合并为一个字符串
        const result = [];
        // 在顶部添加 utf8BOM
        result.push(this.utf8BOM);
        // 添加每一行的数据和换行符
        for (const row of data) {
            result.push(this.format(row).join(this.separate));
            result.push(this.CRLF);
        }
        const csvBlob = new Blob(result, {
            type: 'text/csv',
        });
        return csvBlob;
    }
    UTF8BOM() {
        const buff = new ArrayBuffer(3);
        const data = new DataView(buff);
        data.setInt8(0, 0xef);
        data.setInt8(1, 0xbb);
        data.setInt8(2, 0xbf);
        return buff;
    }
    // 在字符串的两端添加双引号
    addQuotation(str) {
        return '"' + str + '"';
    }
}
const createCSV = new CreateCSV();
// createCSV.create(example)



/***/ }),

/***/ "./src/ts/utils/DateFormat.ts":
/*!************************************!*\
  !*** ./src/ts/utils/DateFormat.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   DateFormat: () => (/* binding */ DateFormat)
/* harmony export */ });
// 格式化日期（和时间）
class DateFormat {
    // format 参数可以由以下格式组合：
    /*
    YYYY
    YY
    MM
    MMM
    MMMM
    DD
    hh
    mm
    ss
    */
    // 区分大小写；可以添加空格或其他符号；不要使用上面未包含的格式。
    // 参考资料：
    // https://www.w3.org/TR/NOTE-datetime
    // https://en.wikipedia.org/wiki/Date_format_by_country
    static format(date, format = 'YYYY-MM-DD') {
        // 生成年、月、日、时、分、秒
        const _date = new Date(date);
        const YYYY = _date.getFullYear().toString();
        const YY = YYYY.substring(YYYY.length - 2, YYYY.length);
        const MM = (_date.getMonth() + 1).toString().padStart(2, '0');
        const MMM = this.months[_date.getMonth()];
        const MMMM = this.Months[_date.getMonth()];
        const DD = _date.getDate().toString().padStart(2, '0');
        const hh = _date.getHours().toString().padStart(2, '0');
        const mm = _date.getMinutes().toString().padStart(2, '0');
        const ss = _date.getSeconds().toString().padStart(2, '0');
        // 对格式字符串进行替换
        let r = format;
        r = r.replace('YYYY', YYYY);
        r = r.replace('YY', YY);
        r = r.replace('MMMM', MMMM);
        r = r.replace('MMM', MMM);
        r = r.replace('MM', MM);
        r = r.replace('DD', DD);
        r = r.replace('hh', hh);
        r = r.replace('mm', mm);
        r = r.replace('ss', ss);
        return r;
    }
}
DateFormat.months = [
    'Jan',
    'Feb',
    'Mar',
    'Apr',
    'May',
    'Jun',
    'Jul',
    'Aug',
    'Sept',
    'Oct',
    'Nov',
    'Dec',
];
DateFormat.Months = [
    'January',
    'February',
    'March',
    'April',
    'May',
    'June',
    'July',
    'August',
    'September',
    'October',
    'November',
    'December',
];



/***/ }),

/***/ "./src/ts/utils/IndexedDB.ts":
/*!***********************************!*\
  !*** ./src/ts/utils/IndexedDB.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   IndexedDB: () => (/* binding */ IndexedDB)
/* harmony export */ });
// 封装操作 IndexedDB 的一些公共方法，仅满足本程序使用，并不完善
class IndexedDB {
    async open(DBName, DBVer, onUpgrade) {
        return new Promise((resolve, reject) => {
            const request = indexedDB.open(DBName, DBVer);
            request.onupgradeneeded = (ev) => {
                if (onUpgrade) {
                    onUpgrade(request.result);
                }
            };
            request.onsuccess = (ev) => {
                this.db = request.result;
                resolve(request.result);
            };
            request.onerror = (ev) => {
                console.error('open indexDB failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async add(storeNames, data) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readwrite')
                .objectStore(storeNames)
                .add(data);
            r.onsuccess = (ev) => {
                resolve(ev);
            };
            r.onerror = (ev) => {
                console.error(`add failed in ${storeNames}`, data);
                console.trace();
                reject(ev);
            };
        });
    }
    async put(storeNames, data) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readwrite')
                .objectStore(storeNames)
                .put(data);
            r.onsuccess = (ev) => {
                resolve(ev);
            };
            r.onerror = (ev) => {
                console.error('put failed');
                console.trace();
                reject(ev);
            };
        });
    }
    // 向一个存储库中批量添加数据
    async batchAddData(storeName, dataList, key) {
        return new Promise(async (resolve, reject) => {
            if (dataList.length === 0) {
                resolve();
            }
            // 获取已存在的 key
            let existedKeys = (await this.getAllKeys(storeName));
            // 使用事务
            const tr = this.db?.transaction(storeName, 'readwrite');
            if (!tr) {
                throw new Error(`transaction ${storeName} is undefined`);
            }
            const store = tr.objectStore(storeName);
            tr.oncomplete = () => {
                resolve();
            };
            tr.onerror = (err) => {
                console.error(err);
                console.trace();
                reject(err);
            };
            for (const data of dataList) {
                await insert(data);
            }
            async function insert(data) {
                return new Promise((resolve, reject) => {
                    // 如果 key 已存在，则使用 put
                    const type = existedKeys.includes(data[key])
                        ? 'put'
                        : 'add';
                    const request = store[type](data);
                    request.onsuccess = () => {
                        resolve();
                    };
                    request.onerror = (err) => {
                        reject(err);
                    };
                });
            }
        });
    }
    // 如果没有找到对应的记录，则返回 null
    async get(storeNames, key, index) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const store = this.db
                .transaction(storeNames, 'readonly')
                .objectStore(storeNames);
            let r;
            if (index !== undefined) {
                const i = store.index(index);
                r = i.get(key);
            }
            else {
                r = store.get(key);
            }
            r.onsuccess = (ev) => {
                const data = r.result;
                resolve(data ? data : null);
            };
            r.onerror = (ev) => {
                console.error('get failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async getAll(storeNames) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readwrite')
                .objectStore(storeNames)
                .getAll();
            r.onsuccess = (ev) => {
                const data = r.result;
                if (data) {
                    resolve(data);
                }
                resolve(data ? data : null);
            };
            r.onerror = (ev) => {
                console.error('getAll failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async getAllKeys(storeNames) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readonly')
                .objectStore(storeNames)
                .getAllKeys();
            r.onsuccess = (ev) => {
                const data = r.result;
                resolve(data ? data : null);
            };
            r.onerror = (ev) => {
                console.error('getAllKeys failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async delete(storeNames, key) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readwrite')
                .objectStore(storeNames)
                .delete(key);
            r.onsuccess = (ev) => {
                resolve(ev);
            };
            r.onerror = (ev) => {
                console.error('delete failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async clear(storeNames) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames, 'readwrite')
                .objectStore(storeNames)
                .clear();
            r.onsuccess = (ev) => {
                resolve();
            };
            r.onerror = (ev) => {
                console.error('clear failed');
                console.trace();
                reject(ev);
            };
        });
    }
    async openCursor(storeNames, CB) {
        return new Promise((resolve, reject) => {
            if (this.db === undefined) {
                reject('Database is not defined');
                return;
            }
            const r = this.db
                .transaction(storeNames)
                .objectStore(storeNames)
                .openCursor();
            r.onsuccess = (ev) => {
                CB(r.result);
                resolve(); // 这个 resolve 会在 cb 执行一次之后就触发
            };
            r.onerror = (ev) => {
                console.error('openCursor failed');
                console.trace();
                reject(ev);
            };
        });
    }
}



/***/ }),

/***/ "./src/ts/utils/SecretSignal.ts":
/*!**************************************!*\
  !*** ./src/ts/utils/SecretSignal.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   secretSignal: () => (/* binding */ secretSignal)
/* harmony export */ });
// 可以用字母和数字组成隐藏口令，注册到这个模块，当用户输入隐藏口令时执行回调函数
class SecretSignal {
    constructor() {
        this.list = [];
        // 允许的按键：字母键，数字键，小键盘数字键
        // "KeyX"
        // "Digit9"
        // "Numpad1"
        this.codePrefix = ['Key', 'Digit', 'Numpad'];
        this.input = '';
        this.bindEvents();
    }
    register(code, cb) {
        this.list.push({
            code,
            cb,
        });
    }
    bindEvents() {
        window.addEventListener('keydown', (ev) => {
            // 不保存控制按键，不保存输入状态中的按键
            if (!ev.code ||
                ev.altKey ||
                ev.ctrlKey ||
                ev.metaKey ||
                ev.shiftKey ||
                ev.isComposing) {
                return;
            }
            // 保存字母和数字的按键
            for (const prefix of this.codePrefix) {
                if (ev.code.startsWith(prefix) &&
                    ev.code.length === prefix.length + 1) {
                    // 如果字符串长度超过限制，则移除前面的一部分字符
                    if (this.input.length > 99) {
                        this.input = this.input.slice(80);
                    }
                    const key = ev.code[ev.code.length - 1].toLowerCase();
                    this.input += key;
                    this.check();
                }
            }
        });
    }
    check() {
        for (const item of this.list) {
            if (this.input.endsWith(item.code)) {
                item.cb();
            }
        }
    }
}
const secretSignal = new SecretSignal();



/***/ }),

/***/ "./src/ts/utils/Utils.ts":
/*!*******************************!*\
  !*** ./src/ts/utils/Utils.ts ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   Utils: () => (/* binding */ Utils)
/* harmony export */ });
class Utils {
    // reg 预先创建，而不是运行时创建，因为运行时重复创建太多次了
    // 用正则去掉不安全的字符
    static replaceUnsafeStr(str) {
        str = str.replace(this.unsafeStr, '');
        // 把一些特殊字符替换成全角字符
        for (let index = 0; index < this.fullWidthDict.length; index++) {
            const rule = this.fullWidthDict[index];
            const reg = new RegExp(rule[0], 'g');
            str = str.replace(reg, rule[1]);
        }
        return str;
    }
    /** 检查并处理 Windows 保留文件名。
     * 如果不传递可选参数，则将其替换为空字符串。
     * 如果传递了可选参数，则在其后添加传递的可选参数的值 */
    static handleWindowsReservedName(str, addStr) {
        for (const name of this.windowsReservedNames) {
            if (str.toUpperCase() === name) {
                return addStr ? str + addStr : '';
            }
            if (str.toUpperCase().startsWith(name + '.')) {
                return str.replace(/\./g, '．');
            }
        }
        return str;
    }
    // 对象深拷贝
    static deepCopy(data) {
        if (data === null || typeof data !== 'object') {
            return data;
        }
        const result = (Array.isArray(data) ? [] : {});
        for (const [key, value] of Object.entries(data)) {
            result[key] =
                data === null || typeof data !== 'object' ? value : this.deepCopy(value);
        }
        return result;
    }
    // 字符串分割成数组
    static string2array(str) {
        str = str.replace(/\n/g, ''); // textarea 的值可能会存在换行符
        const temp = str.trim().split(',');
        const result = [];
        for (const str of temp) {
            if (str !== '') {
                result.push(str.trim());
            }
        }
        return result;
    }
    // 依据对象某个属性的值（视为数字）来排序对象数组。默认降序排列
    static sortByProperty(key, order = 'desc') {
        return function (a, b) {
            // 排序的内容有时可能是字符串，需要转换成数字排序
            // 有些空字符串或者特殊字符可能转换后是 NaN，将其替换为 0
            const value1 = (typeof a[key] === 'number' ? a[key] : parseFloat(a[key])) || 0;
            const value2 = (typeof b[key] === 'number' ? b[key] : parseFloat(b[key])) || 0;
            if (value2 < value1) {
                return order === 'desc' ? -1 : 1;
            }
            else if (value2 > value1) {
                return order === 'desc' ? 1 : -1;
            }
            else {
                return 0;
            }
        };
    }
    // 创建 input 元素选择 json 文件
    static async loadJSONFile() {
        return new Promise((resolve, reject) => {
            const i = document.createElement('input');
            i.setAttribute('type', 'file');
            i.setAttribute('accept', 'application/json');
            i.onchange = () => {
                if (i.files && i.files.length > 0) {
                    // 读取文件内容
                    const file = new FileReader();
                    file.readAsText(i.files[0]);
                    file.onload = () => {
                        const str = file.result;
                        let result;
                        try {
                            result = JSON.parse(str);
                            // if((result as any).constructor !== Object){
                            // 允许是对象 {} 或者数组 []
                            if (result === null || typeof result !== 'object') {
                                const msg = 'Data is not an object!';
                                return reject(new Error(msg));
                            }
                            return resolve(result);
                        }
                        catch (error) {
                            const msg = 'JSON parse error!';
                            return reject(new Error(msg));
                        }
                    };
                }
            };
            i.click();
        });
    }
    // 创建 input 元素选择文件
    static async selectFile(accept) {
        return new Promise((resolve, reject) => {
            const i = document.createElement('input');
            i.setAttribute('type', 'file');
            if (accept) {
                i.setAttribute('accept', accept);
            }
            i.onchange = () => {
                if (i.files && i.files.length > 0) {
                    return resolve(i.files);
                }
                else {
                    return reject();
                }
            };
            i.click();
        });
    }
    // 通过创建 a 标签来下载文件
    static downloadFile(url, fileName) {
        const a = document.createElement('a');
        a.href = url;
        a.download = fileName;
        a.click();
        if (url.startsWith('blob')) {
            URL.revokeObjectURL(url);
        }
    }
    // 判断当前页面是否属于 pixiv.net
    static isPixiv() {
        return window.location.host.endsWith('.pixiv.net');
    }
    // 从 url 中获取指定的查询字段的值
    // 注意：返回值经过 encodeURIComponent 编码！
    static getURLSearchField(url, query) {
        const result = new URL(url).searchParams.get(query);
        if (result !== null) {
            return encodeURIComponent(result);
        }
        else {
            return '';
        }
    }
    /**获取 URL path 中，某个路径名称后面的字符串。适用于符合 RESTful API 风格的路径
     *
     * 注意：传入的是 path，而不是整个 URL
     */
    // 例如：
    // https://www.pixiv.net/users/27482064/following/%E9%83%A8%E5%88%86%E5%96%9C%E6%AC%A2
    // 查询 'users' 返回 '27482064'
    // 因为 location.pathname 传入的字符串是浏览器自动编码过的，所以返回的字符串也是编码过的
    static getURLPathField(path, query) {
        const array = path.split('/');
        const index = array.findIndex((str) => str === query);
        if (index === -1) {
            return '';
        }
        return array[index + 1] || '';
    }
    // 获取指定元素里，可见的结果
    static getVisibleEl(selector) {
        const list = document.querySelectorAll(selector);
        return Array.from(list).filter((el) => {
            return el.style.display !== 'none';
        });
    }
    // 删除 DOM 元素，或者 DOM 元素列表
    static removeEl(el) {
        if (!el) {
            return;
        }
        if (Reflect.has(el, 'length')) {
            // 如果有 length 属性则循环删除。
            ;
            el.forEach((el) => {
                if (el.parentNode) {
                    el.parentNode.removeChild(el);
                }
            });
        }
        else {
            // 没有 length 属性的直接删除（querySelector 的返回值是 HTMLElement）
            const parent = el.parentNode;
            if (parent) {
                parent.removeChild(el);
            }
        }
    }
    // 切换 DOM 元素的可见性
    // 第二个参数设置显示时的 display，默认是 block，如果要设置为其他类型，则需要指定第二个参数
    static toggleEl(el, showDisplay = 'block') {
        el.style.display = el.style.display === showDisplay ? 'none' : showDisplay;
    }
    // 动态添加 css 样式
    static addStyle(css) {
        const e = document.createElement('style');
        e.innerHTML = css;
        document.body.append(e);
    }
    // 加载一个图片，当 onload 事件发生之后返回 img 元素
    static async loadImg(url) {
        return new Promise((resolve, reject) => {
            const img = new Image();
            img.src = url;
            img.onload = function () {
                resolve(img);
            };
            img.onerror = () => {
                reject(new Error(`Load image error! url: ${url}`));
            };
        });
    }
    // 加载图片并在获取到其宽高之后立即返回宽高数值。不需要等待图片加载完毕
    // 请求出错时，返回值的宽高都是 0
    static async getImageSize(url) {
        return new Promise((resolve) => {
            let timer = 0;
            const img = new Image();
            // 在 Chrome 中图片请求的超时时间是 30 秒
            // 如果请求超时，则直接返回
            img.onerror = () => {
                window.clearInterval(timer);
                return resolve({
                    width: 0,
                    height: 0,
                });
            };
            img.src = url;
            timer = window.setInterval(() => {
                if (img.naturalWidth > 0) {
                    window.clearInterval(timer);
                    const wh = {
                        width: img.naturalWidth,
                        height: img.naturalHeight,
                    };
                    img.src = '';
                    return resolve(wh);
                }
            }, 50);
        });
    }
    /**JSON 转换成 Blob 对象。如果数据量可能比较大，则不应该使用这个方法 */
    static json2Blob(data) {
        const str = JSON.stringify(data, null, 2);
        const blob = new Blob([str], { type: 'application/json' });
        return blob;
    }
    /**把 JSON 转换成 Blob 对象。可以处理更大的数据量，并且导出的文件体积不会超过 500 MB */
    static async json2BlobSafe(data) {
        return new Promise((resolve) => {
            // 限制单个文件的体积上限为 500 MB
            const fileByteLengthLimit = 524288000;
            const result = [];
            // 在这个数组里储存数组字面量
            let JSONStringArray = [];
            const length = data.length;
            let index = 0;
            let total = 0;
            let bytelength = 0;
            let startNewFile = true;
            const textEncode = new TextEncoder();
            while (index < length) {
                // 添加数组的开始符号
                if (startNewFile) {
                    startNewFile = false;
                    JSONStringArray.push('[');
                    bytelength = bytelength + 1;
                }
                // 循环添加每一项数据
                const string = JSON.stringify(data[index]);
                JSONStringArray.push(string);
                JSONStringArray.push(',');
                bytelength = bytelength + textEncode.encode(string).length + 1;
                index++;
                total++;
                // 分割文件
                if (index === length || bytelength >= fileByteLengthLimit) {
                    // 删除最后一个分隔符，否则会导致格式错误
                    JSONStringArray.pop();
                    // 添加数组的结束符号
                    JSONStringArray.push(']');
                    // 生成文件数据
                    const blob = new Blob(JSONStringArray, { type: 'application/json' });
                    const url = URL.createObjectURL(blob);
                    result.push({
                        url,
                        total,
                    });
                    // 重置变量
                    startNewFile = true;
                    bytelength = 0;
                    total = 0;
                    JSONStringArray = [];
                }
            }
            return resolve(result);
        });
    }
    /**防抖 */
    static debounce(func, wait) {
        // 默认的定时器 id 不能使用有意义的数字，否则 clearTimeout 可能会错误的清除其他定时器
        let timer = undefined;
        const context = this;
        return function () {
            const args = arguments;
            window.clearTimeout(timer);
            timer = window.setTimeout(func.bind(context, ...args), wait);
        };
    }
    /**节流 */
    static throttle(func, delay) {
        let time = 0;
        const context = this;
        return function () {
            const args = arguments;
            const now = new Date().getTime();
            if (now - time >= delay) {
                time = now;
                return func.apply(context, args);
            }
        };
    }
    /**用 URL 里的后缀名替换 originName 的后缀名
     *
     * 例如传入参数 123.txt, https://.../123.jpg
     *
     * 返回 123.jpg
     */
    static replaceSuffix(originName, url) {
        const nameArray = originName.split('.');
        const urlArray = url.split('.');
        nameArray[nameArray.length - 1] = urlArray[urlArray.length - 1];
        return nameArray.join('.');
    }
    /**获取后缀名 */
    static getSuffix(name) {
        const nameArray = name.split('.');
        return nameArray[nameArray.length - 1];
    }
    /**替换换行标签，移除 html 标签 */
    static htmlToText(str) {
        return str.replace(/<br \/>/g, '\n').replace(/<\/?.+?>/g, '');
    }
}
// 不安全的字符，这里多数是控制字符，需要替换掉
Utils.unsafeStr = new RegExp(/[\u0000\u0001-\u001f\u007f-\u009f\u00ad\u0600-\u0605\u061c\u06dd\u070f\u08e2\u180e\u200b-\u200f\u202a-\u202e\u2060-\u2064\u2066-\u206f\ufdd0-\ufdef\ufeff\ufff9-\ufffb\ufffe\uffff]/g);
// 一些需要替换成全角字符的符号，左边是正则表达式的字符
Utils.fullWidthDict = [
    ['\\\\', '＼'],
    ['/', '／'],
    [':', '：'],
    ['\\?', '？'],
    ['"', '＂'],
    ['<', '＜'],
    ['>', '＞'],
    ['\\*', '＊'],
    ['\\|', '｜'],
    ['~', '～'],
];
/** Windows 保留文件名，不可单独作为文件名，不区分大小写 */
Utils.windowsReservedNames = [
    'CON',
    'PRN',
    'AUX',
    'NUL',
    'COM1',
    'LPT1',
    'LPT2',
    'LPT3',
    'COM2',
    'COM3',
    'COM4',
];



/***/ }),

/***/ "./src/ts/utils/imageToIcon.ts":
/*!*************************************!*\
  !*** ./src/ts/utils/imageToIcon.ts ***!
  \*************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   img2ico: () => (/* binding */ img2ico)
/* harmony export */ });
// 把图片转换成 icon 文件
// icon 文件结构 https://www.cnblogs.com/cswuyg/p/3603707.html
// 输入选项
// source 图片的 url，或者一个图片文件（如果使用了图片 url，请注意跨域策略的影响）
// size 尺寸，可以同时使用多个尺寸。你也可以使用自定义尺寸。
// shape 指定图标的形状。square 正方形，circle 圆形，fillet 带有圆角的正方形
// bleed 留白，仅当形状是圆角正方形时生效，可以使图片周围有一些留白。
// 输出
// 转换成功后，返回 icon 文件的 Blob 对象
// 生成的 icon 总是正方形（长和宽相等）。如果图片的长度和宽度不相等，则会以窄边作为基准，从窄边开始裁剪出一个正方形
// 生成的 icon 可以包含多种尺寸的图标。图标都是 32 位 png 图像。
class ImageToIcon {
    async convert(opt) {
        return new Promise(async (resolve, reject) => {
            // 加载图片
            const img = await this.loadImage(opt.source);
            // 生成各尺寸的 png 图像的数据
            const pngDataArray = await this.createPngBuffer(img, opt.size, opt.shape, opt.bleed);
            // 创建 ico 文件
            const blob = this.createIcon(pngDataArray);
            resolve(blob);
        });
    }
    async convertImageURL(source) {
        return new Promise(async (resolve, reject) => {
            if (typeof source === 'string') {
                // 请求图片，并为其生成 BlobURL，解决图片跨域导致 canvas 污染的问题
                const res = await fetch(source, {
                    method: 'get',
                    credentials: 'same-origin',
                });
                const blob = await res.blob();
                resolve(URL.createObjectURL(blob));
            }
            else if (source instanceof File) {
                resolve(URL.createObjectURL(source));
            }
            else {
                reject('Unrecognized opt.source');
            }
        });
    }
    async loadImage(source) {
        return new Promise(async (resolve, reject) => {
            let imgURL = await this.convertImageURL(source);
            const i = document.createElement('img');
            i.src = imgURL;
            i.onload = function () {
                resolve(i);
            };
        });
    }
    async createPngBuffer(img, size = [16, 48, 96, 256], shape = 'square', bleed = true) {
        return new Promise(async (resolve, reject) => {
            const buffer = [];
            let length = size.length;
            while (length > 0) {
                const sizeNumber = size[size.length - length];
                const canvas = this.createCanvas(sizeNumber, img);
                // 绘制图像
                this.drawImage(canvas, img, shape, bleed);
                // 把图像转换为 png 图像
                const pngBlob = await this.getPngBlob(canvas);
                // 获取 png 图像的 buffer
                const buf = await pngBlob.arrayBuffer();
                buffer.push({
                    size: sizeNumber,
                    buffer: buf,
                });
                length--;
                if (length === 0) {
                    resolve(buffer);
                }
            }
        });
    }
    createCanvas(size, img) {
        const c = document.createElement('canvas');
        c.width = size;
        c.height = size;
        return c;
    }
    drawImage(canvas, img, shape, bleed = true) {
        const ctx = canvas.getContext('2d');
        if (!ctx) {
            console.error('draw error: ctx is null');
            return;
        }
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        // 计算图像被绘制的宽高。比较短的一边占满画布，比较长的一边则根据比例计算绘制的部分
        let dw = 0;
        let dh = 0;
        // 竖图
        if (img.naturalWidth < img.naturalHeight) {
            dw = canvas.width;
            dh = (dw / img.naturalWidth) * img.naturalHeight;
        }
        else {
            // 横图
            dh = canvas.height;
            dw = (dh / img.naturalHeight) * img.naturalWidth;
        }
        // 绘制方形
        if (shape === 'square') {
            ctx.drawImage(img, 0, 0, dw, dh);
        }
        // 绘制圆形
        if (shape === 'circle') {
            let circle = {
                x: canvas.width / 2,
                y: canvas.width / 2,
                r: canvas.width / 2,
            };
            ctx.save();
            ctx.beginPath();
            ctx.arc(circle.x, circle.y, circle.r, 0, Math.PI * 2, false);
            ctx.clip();
            ctx.drawImage(img, 0, 0, dw, dh);
            ctx.restore();
        }
        // 绘制圆角矩形
        if (shape === 'fillet') {
            let x = 0;
            let y = 0;
            // 当图标尺寸大于 16 时，设置留白距离
            if (bleed && canvas.width > 16) {
                let num = 10 / 256; // 规定留白的比例，即尺寸为 256 时四周留白均为 10 px
                x = Math.ceil(num * canvas.width);
                y = Math.ceil(num * canvas.width);
            }
            // 去掉留白后，最后要保存的图片区域的宽高
            const w = canvas.width - x * 2;
            const h = canvas.height - y * 2;
            // 圆角的半径，设置为保留区域宽高的 1/8
            const r = Math.floor(w / 8);
            ctx.beginPath();
            ctx.moveTo(x + r, y);
            ctx.arcTo(x + w, y, x + w, y + h, r);
            ctx.arcTo(x + w, y + h, x, y + h, r);
            ctx.arcTo(x, y + h, x, y, r);
            ctx.arcTo(x, y, x + w, y, r);
            ctx.closePath();
            ctx.clip();
            ctx.drawImage(img, 0, 0, dw, dh);
        }
        return ctx;
    }
    async getPngBlob(canvas) {
        return new Promise((resolve, reject) => {
            canvas.toBlob((blob) => {
                if (!blob) {
                    reject('blob is null');
                }
                else {
                    resolve(blob);
                }
            });
        });
    }
    createIcon(pngData) {
        const fileData = [];
        const fileHeadSize = 6;
        // icon 文件头
        const fileHead = new ArrayBuffer(fileHeadSize);
        const v1 = new DataView(fileHead);
        v1.setInt16(0, 0, true); // idReserved
        v1.setInt16(2, 1, true); // idType
        v1.setInt16(4, pngData.length, true); // idCount
        fileData.push(fileHead);
        // 添加 icon 文件入口
        const entrySize = 16;
        const entryTotalSize = entrySize * pngData.length;
        let fileOffset = fileHeadSize + entryTotalSize;
        let fileLength = 0;
        for (const d of pngData) {
            fileOffset += fileLength;
            const entry = new ArrayBuffer(entrySize);
            const v2 = new DataView(entry);
            v2.setInt8(0, d.size); // Width, in pixels, of the image
            v2.setInt8(1, d.size); // Height, in pixels, of the image
            v2.setInt8(2, 0); // Number of colors in image (0 if >=8bpp)
            v2.setInt8(3, 0); // Reserved ( must be 0)
            v2.setInt16(4, 1, true); // Color Planes
            v2.setInt16(6, 32, true); // Bits per pixel
            v2.setInt32(8, d.buffer.byteLength, true); // How many bytes in this resource?
            v2.setInt32(12, fileOffset, true); // Where in the file is this image?
            fileData.push(entry);
            fileLength = d.buffer.byteLength;
        }
        // 添加 png 数据
        for (const d of pngData) {
            fileData.push(d.buffer);
        }
        // 生成 blob 对象
        return new Blob(fileData, {
            type: 'image/vnd.microsoft.icon',
        });
    }
}
const img2ico = new ImageToIcon();



/***/ })

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
/******/ 	/* webpack/runtime/compat get default export */
/******/ 	(() => {
/******/ 		// getDefaultExport function for compatibility with non-harmony modules
/******/ 		__webpack_require__.n = (module) => {
/******/ 			var getter = module && module.__esModule ?
/******/ 				() => (module['default']) :
/******/ 				() => (module);
/******/ 			__webpack_require__.d(getter, { a: getter });
/******/ 			return getter;
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/define property getters */
/******/ 	(() => {
/******/ 		// define getter functions for harmony exports
/******/ 		__webpack_require__.d = (exports, definition) => {
/******/ 			for(var key in definition) {
/******/ 				if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 					Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 				}
/******/ 			}
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/hasOwnProperty shorthand */
/******/ 	(() => {
/******/ 		__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/make namespace object */
/******/ 	(() => {
/******/ 		// define __esModule on exports
/******/ 		__webpack_require__.r = (exports) => {
/******/ 			if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 				Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 			}
/******/ 			Object.defineProperty(exports, '__esModule', { value: true });
/******/ 		};
/******/ 	})();
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it need to be isolated against other modules in the chunk.
(() => {
/*!***************************!*\
  !*** ./src/ts/content.ts ***!
  \***************************/
__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _Lang__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Lang */ "./src/ts/Lang.ts");
/* harmony import */ var _Theme__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Theme */ "./src/ts/Theme.ts");
/* harmony import */ var _store_States__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./store/States */ "./src/ts/store/States.ts");
/* harmony import */ var _setting_Settings__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./setting/Settings */ "./src/ts/setting/Settings.ts");
/* harmony import */ var _setting_InvisibleSettings__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./setting/InvisibleSettings */ "./src/ts/setting/InvisibleSettings.ts");
/* harmony import */ var _ListenPageSwitch__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./ListenPageSwitch */ "./src/ts/ListenPageSwitch.ts");
/* harmony import */ var _CenterPanel__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./CenterPanel */ "./src/ts/CenterPanel.ts");
/* harmony import */ var _setting_Form__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./setting/Form */ "./src/ts/setting/Form.ts");
/* harmony import */ var _setting_DoNotDownloadLastFewImages__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./setting/DoNotDownloadLastFewImages */ "./src/ts/setting/DoNotDownloadLastFewImages.ts");
/* harmony import */ var _setting_UseDifferentNameRuleIfWorkHasTag__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./setting/UseDifferentNameRuleIfWorkHasTag */ "./src/ts/setting/UseDifferentNameRuleIfWorkHasTag.ts");
/* harmony import */ var _ReplaceSquareThumb__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./ReplaceSquareThumb */ "./src/ts/ReplaceSquareThumb.ts");
/* harmony import */ var _InitPage__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./InitPage */ "./src/ts/InitPage.ts");
/* harmony import */ var _crawlMixedPage_QuickCrawl__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ./crawlMixedPage/QuickCrawl */ "./src/ts/crawlMixedPage/QuickCrawl.ts");
/* harmony import */ var _download_DownloadControl__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./download/DownloadControl */ "./src/ts/download/DownloadControl.ts");
/* harmony import */ var _download_Resume__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ./download/Resume */ "./src/ts/download/Resume.ts");
/* harmony import */ var _Tip__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ./Tip */ "./src/ts/Tip.ts");
/* harmony import */ var _Tip__WEBPACK_IMPORTED_MODULE_15___default = /*#__PURE__*/__webpack_require__.n(_Tip__WEBPACK_IMPORTED_MODULE_15__);
/* harmony import */ var _PreviewWork__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./PreviewWork */ "./src/ts/PreviewWork.ts");
/* harmony import */ var _ShowLargerThumbnails__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ./ShowLargerThumbnails */ "./src/ts/ShowLargerThumbnails.ts");
/* harmony import */ var _DoubleWidthThumb__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ./DoubleWidthThumb */ "./src/ts/DoubleWidthThumb.ts");
/* harmony import */ var _ShowZoomBtnOnThumb__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! ./ShowZoomBtnOnThumb */ "./src/ts/ShowZoomBtnOnThumb.ts");
/* harmony import */ var _ShowDownloadBtnOnThumb__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! ./ShowDownloadBtnOnThumb */ "./src/ts/ShowDownloadBtnOnThumb.ts");
/* harmony import */ var _output_OutputPanel__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! ./output/OutputPanel */ "./src/ts/output/OutputPanel.ts");
/* harmony import */ var _output_PreviewFileName__WEBPACK_IMPORTED_MODULE_22__ = __webpack_require__(/*! ./output/PreviewFileName */ "./src/ts/output/PreviewFileName.ts");
/* harmony import */ var _output_ShowURLs__WEBPACK_IMPORTED_MODULE_23__ = __webpack_require__(/*! ./output/ShowURLs */ "./src/ts/output/ShowURLs.ts");
/* harmony import */ var _download_ExportResult2CSV__WEBPACK_IMPORTED_MODULE_24__ = __webpack_require__(/*! ./download/ExportResult2CSV */ "./src/ts/download/ExportResult2CSV.ts");
/* harmony import */ var _download_ExportResult__WEBPACK_IMPORTED_MODULE_25__ = __webpack_require__(/*! ./download/ExportResult */ "./src/ts/download/ExportResult.ts");
/* harmony import */ var _download_ImportResult__WEBPACK_IMPORTED_MODULE_26__ = __webpack_require__(/*! ./download/ImportResult */ "./src/ts/download/ImportResult.ts");
/* harmony import */ var _download_ExportLST__WEBPACK_IMPORTED_MODULE_27__ = __webpack_require__(/*! ./download/ExportLST */ "./src/ts/download/ExportLST.ts");
/* harmony import */ var _download_MergeNovel__WEBPACK_IMPORTED_MODULE_28__ = __webpack_require__(/*! ./download/MergeNovel */ "./src/ts/download/MergeNovel.ts");
/* harmony import */ var _download_SaveWorkMeta__WEBPACK_IMPORTED_MODULE_29__ = __webpack_require__(/*! ./download/SaveWorkMeta */ "./src/ts/download/SaveWorkMeta.ts");
/* harmony import */ var _download_ShowStatusOnTitle__WEBPACK_IMPORTED_MODULE_30__ = __webpack_require__(/*! ./download/ShowStatusOnTitle */ "./src/ts/download/ShowStatusOnTitle.ts");
/* harmony import */ var _download_ShowTotalResultOnTitle__WEBPACK_IMPORTED_MODULE_31__ = __webpack_require__(/*! ./download/ShowTotalResultOnTitle */ "./src/ts/download/ShowTotalResultOnTitle.ts");
/* harmony import */ var _download_ShowRemainingDownloadOnTitle__WEBPACK_IMPORTED_MODULE_32__ = __webpack_require__(/*! ./download/ShowRemainingDownloadOnTitle */ "./src/ts/download/ShowRemainingDownloadOnTitle.ts");
/* harmony import */ var _download_DownloadOnClickLike__WEBPACK_IMPORTED_MODULE_33__ = __webpack_require__(/*! ./download/DownloadOnClickLike */ "./src/ts/download/DownloadOnClickLike.ts");
/* harmony import */ var _HighlightFollowingUsers__WEBPACK_IMPORTED_MODULE_34__ = __webpack_require__(/*! ./HighlightFollowingUsers */ "./src/ts/HighlightFollowingUsers.ts");
/* harmony import */ var _ShowWhatIsNew__WEBPACK_IMPORTED_MODULE_35__ = __webpack_require__(/*! ./ShowWhatIsNew */ "./src/ts/ShowWhatIsNew.ts");
/* harmony import */ var _CheckUnsupportBrowser__WEBPACK_IMPORTED_MODULE_36__ = __webpack_require__(/*! ./CheckUnsupportBrowser */ "./src/ts/CheckUnsupportBrowser.ts");
/* harmony import */ var _ShowNotification__WEBPACK_IMPORTED_MODULE_37__ = __webpack_require__(/*! ./ShowNotification */ "./src/ts/ShowNotification.ts");
/* harmony import */ var _HiddenBrowserDownloadBar__WEBPACK_IMPORTED_MODULE_38__ = __webpack_require__(/*! ./HiddenBrowserDownloadBar */ "./src/ts/HiddenBrowserDownloadBar.ts");
/* harmony import */ var _RequestSponsorship__WEBPACK_IMPORTED_MODULE_39__ = __webpack_require__(/*! ./RequestSponsorship */ "./src/ts/RequestSponsorship.ts");
/*
 * project: Powerful Pixiv Downloader
 * author:  xuejianxianzun; 雪见仙尊
 * license: GPL-3.0-or-later; http://www.gnu.org/licenses/gpl-3.0.txt
 * Github： https://github.com/xuejianxianzun/PixivBatchDownloader
 * Releases: https://github.com/xuejianxianzun/PixivBatchDownloader/releases
 * Wiki:    https://xuejianxianzun.github.io/PBDWiki
 * Website: https://pixiv.download/
 * E-mail:  xuejianxianzun@gmail.com
 */



































// import './CheckNewVersion'






})();

/******/ })()
;
//# sourceMappingURL=content.js.map