import { Config } from './Config'

// 储存下载器使用的多语言文本
// 在属性名前面加上下划线，和文本内容做出区别
// {} 是占位符
// <br> 是换行

const langText = {
  _只下载已收藏: [
    '只下载已收藏',
    '只下載已收藏',
    'Download only bookmarked works',
    'ブックマークのみをダウンロードする',
    '북마크된 작품만 다운로드',
    'Загружайте только работы, сохраненные в закладках',
  ],
  _下载作品类型: [
    '下载作品类型',
    '下載作品類型',
    'Download work type',
    'ダウンロード作品の種類',
    '다운로드할 작품 유형',
    'Типы контента для загрузки',
  ],
  _作品类型: [
    '作品<span class="key">类型</span>',
    '作品<span class="key">類型</span>',
    '<span class="key">Type</span> of work',
    '作品の<span class="key">種類</span>',
    '작품 <span class="key">유형</span>',
    'Тип <span class="key">работы</span>',
  ],
  _不能含有tag: [
    '<span class="key">不能</span>含有标签',
    '<span class="key">不能</span>含有標籤',
    '<span class="key">Exclude</span> tag',
    'タグを除外する',
    '<span class="key">제외</span> 태그',
    '<span class="key">Исключить</span> ярлык',
  ],
  _排除tag的提示文字: [
    '您可在下载前设置要排除的标签，这样在下载时将不会下载含有这些标签的作品。不区分大小写；如需排除多个标签，请使用英文逗号分隔。请注意，要排除的标签的优先级大于要包含的标签的优先级。',
    '可在下載前設定要排除的標籤，下載時將排除含有這些標籤的作品，不區分大小寫；如需排除多個標籤，請使用半形逗號（,）分隔。請注意，要排除的標籤優先於要包含的標籤。',
    'Before downloading, you can set the tag you want to exclude. Not case sensitive; If you need to set multiple tags, you can use comma (,) separated. The excluded tag takes precedence over the included tag',
    'ダウンロード前に、除外するタグを設定できます。大文字と小文字を区別しない；複数のタグを設定する必要がある場合は、「,」で区切ってください。除外されたタグは、必要なタグよりも優先されます',
    '다운로드하기 전에 제외해야 하는 태그를 설정할 수 있습니다. 대소문자를 구분하지 않습니다. 여러 태그를 설정해야 하는 경우 쉼표(,)로 구분합니다. 제외된 태그가 포함된 태그보다 우선합니다.',
    'Перед загрузкой можно задать тег, который необходимо исключить. Не чувствителен к регистру; Если вам нужно задать несколько тегов, вы можете использовать разделение запятыми (,). Исключенный тег имеет приоритет над включенным тегом',
  ],
  _设置了排除tag之后的提示: [
    '排除标签：',
    '排除標籤：',
    'Excludes tag: ',
    '以下のタグを除外：',
    '제외 태그: ',
    'Исключающий тег: ',
  ],
  _必须含有tag: [
    '<span class="key">必须</span>含有标签',
    '<span class="key">必須</span>含有標籤',
    '<span class="key">Include</span> tag',
    '<span class="key">必要な</span>タグ',
    '<span class="key">포함</span> 태그',
    '<span class="key">Включать</span> ярлык',
  ],
  _必须tag的提示文字: [
    '您可在下载前设置作品里必须包含的标签，不区分大小写；如需包含多个标签，请使用英文逗号分隔。',
    '可在下載前設定作品裡必須包含的標籤，不區分大小寫；如需包含多個標籤，請使用半形逗號（,）分隔。',
    'Before downloading, you can set the tag that must be included. Not case sensitive; If you need to set multiple tags, you can use comma (,) separated.',
    'ダウンロードする前に、必要なタグを設定することができます。大文字と小文字を区別しない；複数のタグを設定する必要がある場合は、「,」で区切ってください。',
    '다운로드하기 전에 포함해야 하는 태그를 설정할 수 있습니다. 대소문자를 구분하지 않습니다. 여러 태그를 설정해야 하는 경우 쉼표(,)로 구분합니다.',
    'Перед загрузкой можно задать тег, который должен быть включен. Не чувствителен к регистру; Если вам нужно задать несколько тегов, вы можете использовать разделение запятыми (,).',
  ],
  _设置了必须tag之后的提示: [
    '包含标签：',
    '包含標籤：',
    'Include tag: ',
    '以下の タグ を含める：',
    '포함 태그: ',
    'Включающий тег: ',
  ],
  _图片的宽高比例: [
    '图片的宽高<span class="key">比例</span>',
    '圖片的寬高<span class="key">比例</span>',
    'Aspect <span class="key">ratio</span>',
    '画像の縦横比',
    '<span class="key">종횡비</span>',
    'Сотношение <span class="key">сторон</span>',
  ],
  _设置宽高比例Title: [
    '设置宽高比例，也可以手动输入宽高比',
    '設定寬高比，也可以手動輸入寬高比。',
    'Set the aspect ratio, or manually type the aspect ratio',
    '縦横比を設定する、手動で縦横比を入力することもできる',
    '종횡비를 설정하거나, 값을 수동으로 입력할 수 있습니다.',
    'Установите соотношение сторон или введите соотношение сторон вручную',
  ],
  _不限制: [
    '不限制',
    '不限制',
    'not limited',
    '無制限',
    '제한 없음',
    'Без лимитов',
  ],
  _横图: ['横图', '橫圖', 'Horizontal', '横長', '가로', 'Горизонтальный'],
  _竖图: ['竖图', '豎圖', 'Vertical', '縦長', '세로', 'Вертикальный'],
  _正方形: ['正方形', '正方形', 'Square', '正方形', '정사각형', 'Квадрат(1:1)'],
  _宽高比: ['宽高比', '寬高比', 'Ratio', '縦横比', '종횡비 ', 'Соотношение'],
  _设置了宽高比之后的提示: [
    '宽高比：{}',
    '寬高比：{}',
    'Aspect ratio: {}',
    '縦横比：{}',
    '종횡비: {}',
    'Соотношение сторон: {}',
  ],
  _宽高比必须是数字: [
    '宽高比必须是数字',
    '寬高比必須是數字',
    'The aspect ratio must be a number',
    '縦横比は数値でなければなりません',
    '종횡비는 숫자여야 합니다',
    'Соотношение сторон должно быть числом',
  ],
  _图片的宽高: [
    '图片的<span class="key">宽高</span>',
    '圖片的<span class="key">寬高</span>',
    '<span class="key">width</span> and height',
    '画像の幅と高さ',
    '<span class="key">너비</span> 그리고 높이',
    '<span class="key">Ширина</span> и высота',
  ],
  _筛选宽高的提示文字: [
    '请输入最小宽度和最小高度，不会下载不符合要求的图片。',
    '請輸入最小寬度和最小高度，只會下載符合要求的圖片。',
    'Please type the minimum width and minimum height. Will not download images that do not meet the requirements',
    '最小幅と最小高さを入力してください。要件を満たしていない画像はダウンロードされません。',
    '최소 너비와 최소 높이를 입력해주세요, 요구 사항을 충족하지 않는 이미지는 다운로드하지 않습니다.',
    'Введите минимальную ширину и минимальную высоту. Не соответствующие требованиям изображения, загружаться не будут',
  ],
  _本次输入的数值无效: [
    '本次输入的数值无效',
    '本次輸入的數值無效',
    'Invalid input',
    '無効な入力',
    '잘못된 입력',
    'Недопустимый ввод',
  ],
  _宽度: ['宽度', '寬度', 'Width', '幅', '너비', 'Ширина'],
  _或者: [' 或者 ', ' 或是 ', ' or ', ' または ', '또는', 'или'],
  _并且: [' 并且 ', ' 並且 ', ' and ', ' そして ', '그리고', 'и'],
  _高度: ['高度', '高度', 'height', '高さ', '높이', 'высота'],
  _抓取多少作品: [
    '抓取<span class="key">多少</span>作品',
    '擷取<span class="key">多少</span>作品',
    'How <span class="key">many</span> works to crawl',
    'クロールする作品の数',
    '긁어올 작품 <span class="key">수</span>',
    'Какое <span class="key">колличество</span> работ сканировать',
  ],
  _抓取多少页面: [
    '抓取<span class="key">多少</span>页面',
    '擷取<span class="key">多少</span>頁面',
    'How <span class="key">many</span> pages to crawl',
    'クロールするページ数',
    '긁어올 페이지 <span class="key">수</span>',
    'Какое <span class="key">колличество</span> страниц сканироватьь',
  ],
  _收藏数量: [
    '<span class="key">收藏</span>数量',
    '<span class="key">收藏</span>數量',
    'Number of <span class="key">bookmarks</span>',
    'ブックマークの数',
    '<span class="key">북마크</span> 수',
    'Колличество <span class="key">закладок</span>',
  ],
  _设置收藏数量的提示: [
    '如果作品的收藏数小于设置的数字，作品不会被下载。',
    '只會下載設定收藏數範圍內的作品。',
    'If the number of bookmarks of the work is less than the set number, the work will not be downloaded.',
    '作品のブックマークされた数が設定された数字よりも少ない場合、作品はダウンロードされません。',
    '작품의 북마크 수가 설정된 값보다 적을 경우 작품은 다운로드되지 않습니다.',
    'Если количество закладок произведения меньше заданного, произведение не будет загружено',
  ],
  _筛选收藏数的提示文字: [
    '请输入一个数字，如果作品的收藏数小于这个数字，作品不会被下载。',
    '請輸入數字，只會下載設定收藏數範圍內的作品。',
    'Please type a number. If the number of bookmarks of the work is less than this number, the work will not be downloaded.',
    '数字を入力してください。 作品のブックマークされた数がこの数字より少ない場合、作品はダウンロードされません。',
    '숫자를 입력해주세요. 작품의 북마크 수가 이 수보다 적을 경우 작품은 다운로드되지 않습니다.',
    'Пожалуйста, введите число. Если количество закладок произведения меньше этого числа, произведение не будет загружено.',
  ],
  _收藏数大于: [
    '收藏数 >= ',
    '收藏數 >= ',
    'Number of bookmarks >= ',
    'ブックマークの数 >= ',
    '북마크 수 >= ',
    'Количество закладок >= ',
  ],
  _收藏数小于: [
    '收藏数 <= ',
    '收藏數 <= ',
    'Number of bookmarks <= ',
    'ブックマークの数 <= ',
    '북마크 수 <=',
    'Количество закладок <= ',
  ],
  _本次任务已全部完成: [
    '本次任务已全部完成。',
    '本次工作已全部完成。',
    'This task has been completed.',
    'この作業は完了しました。',
    '이 작업은 완료되었습니다.',
    'Эта задача была выполнена.',
  ],
  _本次任务条件: [
    '本次任务条件: ',
    '本次工作條件：',
    'This task condition: ',
    'この作業の条件：',
    '이 작업 조건: ',
    'Это условие задачи: ',
  ],
  _参数不合法本次操作已取消: [
    '参数不合法，本次操作已取消。',
    '參數不合法，本次動作已取消。',
    'Parameter is not legal, this operation has been canceled.',
    'パラメータは有効ではありません。この操作はキャンセルされました。',
    '매개변수가 잘못되었습니다, 이 작업은 취소됩니다.',
    'Параметр не является разрешенным, операция отменена.',
  ],
  _本次操作已取消: [
    '本次操作已取消',
    '本次動作已取消',
    'This operation has been canceled',
    'この操作はキャンセルされました',
    '이 작업이 취소되었습니다.',
    'Эта операция была отменена',
  ],
  _向下获取所有作品: [
    '向下获取所有作品',
    '向下取得所有作品',
    'download all the work from this page.',
    'このページからすべての作品をダウンロードする。',
    '모든 작품 다운로드',
    'загрузить все работы с этой страницы',
  ],
  _从本页开始下载提示: [
    '从当前页面开始下载。<br>如果要限制下载的页数，请输入从 1 开始的数字。<br>1 为仅下载本页，-1 为下载所有页面。',
    '從當前頁面開始下載。<br>如果要限制下載的頁數，請輸入從 1 開始的數字。<br>1 為僅下載本頁，-1 為下載所有頁面。',
    'Download from the current page.<br>If you want to set the number of pages to download, type a number starting at 1. <br>1 is to download only this page, -1 to download all pages.',
    '現在のページからダウンロードしてください。<br>ダウンロードするページを設定する場合は、1から始まる数字を入力してください。<br>1 は現在のページのみをダウンロードする，すべてのページをダウンロードするには -1。',
    '현재 페이지에서 다운로드합니다.<br>다운로드할 페이지 수를 설정하려면 1로 시작하는 숫자를 입력해주세요.<br>1은 이 페이지만 다운로드합니다, -1은 모든 페이지를 다운로드합니다.',
    'Загрузка с текущей страницы.<br>Если вы хотите задать количество страниц для загрузки, введите число, начиная с 1. <br>1 - это загрузка только этой страницы, -1 для загрузки всех страниц.',
  ],
  _下载所有页面: [
    '下载所有页面',
    '下載所有頁面',
    'download all pages',
    'すべてのページをダウンロードする',
    '모든 페이지 다운로드',
    'загрузить все страницы',
  ],
  _下载x个相关作品: [
    '下载 {} 个相关作品',
    '下載 {} 個相關作品',
    'download {} related works.',
    '関連作品 {} 枚をダウンロードする。',
    '관련 작품 {}개를 다운로드',
    'скачать {} связанные работы',
  ],
  _下载所有相关作品: [
    '下载所有相关作品',
    '下載所有相關作品',
    'download all related works.',
    '関連作品をすべてダウンロードする。',
    '모든 관련 작품 다운로드',
    'скачать все соответствующие работы.',
  ],
  _下载推荐作品: [
    '下载推荐作品',
    '下載推薦作品',
    'download recommended works',
    'おすすめ作品をダウンロードする',
    '추천 작품 다운로드',
    'скачать рекомендуемые работы',
  ],
  _下载排行榜前x个作品: [
    '下载排行榜前 {} 个作品',
    '下載排行榜前 {} 個作品',
    'download the top {} works in the ranking list',
    'ランク前 {} 位の作品をダウンロードする。',
    '랭킹 목록 상위 {}개의 작품 다운로드',
    'загрузить лучшие {} работы в рейтинговом списке',
  ],
  _输入超过了最大值: [
    '您输入的数字超过了最大值',
    '輸入的數字超出最大值',
    'The number you set exceeds the maximum',
    '入力した番号が最大値を超えています',
    '설정하신 수가 최대값을 초과합니다',
    'Заданное вами число превышает максимальное',
  ],
  _从本页开始下载x页: [
    '从本页开始下载 {} 页',
    '從本頁開始下載 {} 頁',
    'download {} pages from this page',
    'このページから {} ページをダウンロードする',
    '이 페이지부터 {} 페이지 다운로드',
    'Начать загрузку с страниц этой {} страницы',
  ],
  _从本页开始下载x个: [
    '从本页开始下载 {} 个作品',
    '從本頁開始下載 {} 個作品',
    'Download {} works from this page.',
    'このページから {} 枚の作品をダウンロード。',
    '이 페이지부터 {}개의 작품 다운로드',
    'Загрузить {} работы с этой страницы.',
  ],
  _任务开始: [
    '任务开始',
    '工作開始',
    'Task starts',
    '作業が開始されます',
    '작업 시작',
    'Задание начинается',
  ],
  _排除作品类型: [
    '排除作品类型：',
    '排除作品類型：',
    'Excludes these types of works: ',
    'これらのタイプの作品を除外：',
    '제외된 작품 유형: ',
    'Исключает эти виды работ: ',
  ],
  _多图作品: [
    '多图作品',
    '多圖作品',
    'Multi-image works',
    '複数画像作品',
    '여러 이미지 작품',
    'Работа с несколькими изображениями',
  ],
  _多图下载设置: [
    '多图下载设置',
    '多圖下載設定',
    'Download multi-image works',
    '複数画像設定',
    '여러 이미지 작품 다운로드',
    'Загрузить работы с несколькими изображениями',
  ],
  _不下载: ['不下载', '不下載', 'No', '必要なし', '아니요', 'Нет'],
  _全部下载: ['全部下载', '全部下載', 'Yes', '全部ダウンロード', '네', 'Да'],
  _插画: [
    '插画',
    '插畫',
    'Illustrations',
    'イラスト',
    '일러스트',
    'Иллюстрации',
  ],
  _漫画: ['漫画', '漫畫', 'Manga', '漫画', '만화', 'Манга'],
  _动图: [
    '动图',
    '動圖',
    'Ugoira',
    'うごイラ',
    '움직이는 일러스트',
    'Ugoira(гиф)',
  ],
  _动图保存格式: [
    '<span class="key">动图</span>保存格式',
    '<span class="key">動圖</span>儲存格式',
    'Save the <span class="key">ugoira</span> work as',
    'うごイラの保存タイプ',
    '<span class="key">움직이는 일러스트</span> 작품 저장 형식',
    'Сохранить <span class="key">Ugoira</span> как',
  ],
  _动图保存格式title: [
    '下载动图时，可以把它转换成视频文件',
    '下載動圖時，可轉換為影片檔。',
    'When you download a ugoira work, you can convert it to a video file.',
    'うごイラをダウンロードするとき、動画に変換することができます。',
    '움직이는 일러스트 작품을 다운로드 하면, 동영상 파일로 변환할 수 있습니다.',
    'Когда вы скачиваете произведение ugoira, вы можете конвертировать его в видеофайл.',
  ],
  _webmVideo: [
    'WebM 视频',
    '影片（WebM）',
    'WebM video',
    'WebM ビデオ',
    'WebM 동영상',
    'WebM видео',
  ],
  _gif: [
    'GIF 图片',
    '圖片（GIF）',
    'GIF image',
    'GIF 画像',
    'GIF 이미지',
    'GIF изображение',
  ],
  _apng: [
    'APNG 图片',
    '圖片（APNG）',
    'APNG image',
    'APNG 画像',
    'APNG 이미지',
    'APNG изображение',
  ],
  _zipFile: [
    'Zip 文件',
    '壓縮檔（Zip）',
    'Zip file',
    'ZIP ファイル',
    'Zip 파일',
    'Zip файл',
  ],
  _当前作品个数: [
    '当前有 {} 个作品',
    '目前有 {} 個作品',
    'There are now {} works',
    '今は　{}　枚の作品があります',
    '현재 {}개의 작품이 있습니다',
    'В настоящее время существует {} работ',
  ],
  _当前有x个用户: [
    '当前有 {} 个用户',
    '目前有 {} 個使用者',
    'There are currently {} users',
    '現在 {} 人のユーザーがいます',
    '현재 {}명의 유저가 있습니다',
    'В настоящее время существует {} пользователей',
  ],
  _已抓取x个用户: [
    '已抓取 {} 个用户',
    '已擷取 {} 個使用者',
    'crawled {} users',
    'クロールされた {} ユーザー',
    '{}명의 유저를 긁어왔습니다',
    'Сканированные {} пользователи',
  ],
  _排行榜进度: [
    '已抓取本页面第{}部分',
    '已擷取本頁面第 {} 部分',
    'Part {} of this page has been crawled',
    'このページの第　{}　部がクロールされました',
    '이 페이지의 {} 부분을 긁어왔습니다',
    'Часть {} этой страницы была просмотрена',
  ],
  _新作品进度: [
    '已抓取本页面 {} 个作品',
    '已擷取本頁面 {} 個作品',
    'This page has been crawled {} works',
    'このページの {} つの作品をクロールしました',
    '이 페이지의 {}개의 작품을 긁어왔습니다',
    'На этой странице было просканированно {} работ',
  ],
  _抓取多少个作品: [
    '抓取本页面 {} 个作品',
    '擷取本頁面 {} 個作品',
    'Crawl this page {} works',
    'このページの {} つの作品をクロールします',
    '이 페이지의 {}개의 작품 긁어오기',
    'Сканировать на этой странице {} работ',
  ],
  _相关作品抓取完毕: [
    '相关作品抓取完毕。包含有{}个作品，开始获取作品信息。',
    '相關作品擷取完畢。包含有 {} 個作品，開始取得作品資訊。',
    'The related works have been crawled. Contains {} works and starts getting information about the work.',
    '関連作品はクロールされました。 {} 作品を含み、その作品に関する情報の取得を開始します。',
    '관련 작품 긁어오기 완료, {}개의 작품이 포함되어 있으며, 작품 정보 가져오기를 시작합니다',
    'Связанные работы были просканированы. Содержит {} работ и начинает получать информацию о работе(ах).',
  ],
  _排行榜任务完成: [
    '本页面抓取完毕。<br>当前有{}个作品，开始获取作品信息。',
    '本頁面擷取完畢。<br>目前有 {} 個作品，開始取得作品資訊。',
    'This page is crawled and now has {} works.<br> Start getting the works for more information.',
    'このページのクロール終了。<br>{}枚の作品があります。 作品情報の取得を開始します。',
    '이 페이지 긁어오기 완료되었습니다<br>현재 {}개의 작품이 있으며, 작품 정보 가져오기를 시작합니다',
    'Эта страница просмотрена и имеет {} работ.<br> Начинаю получать работы для получения дополнительной информации.',
  ],
  _开始获取作品信息: [
    '开始获取作品信息',
    '開始取得作品資訊',
    'Start getting work data',
    '作品情報の取得を開始します',
    '작품 정보 가져오기 시작',
    'Начинаю получать данные о работе',
  ],
  _列表页抓取进度: [
    '已抓取列表页 {} 个页面',
    '已擷取清單頁 {} 個頁面',
    'Has acquired {} list pages',
    '{} のリストページを取得しました',
    '{}개의 목록 페이지를 획득하였습니다',
    'Получено {} страниц списка',
  ],
  _列表页抓取进度2: [
    '正在抓取列表页 {}/{}',
    '正在抓取列表頁 {}/{}',
    'crawling list page {}/{}',
    'リストページの取得 {}/{}',
    '목록 페이지 긁어오는 중 {}/{}',
    'Вытаскивание списка страниц {}/{}',
  ],
  _列表页抓取完成: [
    '列表页面抓取完成',
    '清單頁面擷取完成',
    'The list page is crawled',
    'リストページがクロールされ',
    '목록 페이지 긁어오기 완료',
    'Список страниц просканирован',
  ],
  _抓取结果为零: [
    '抓取完毕，但没有找到符合筛选条件的作品。<br>请检查“抓取”相关的设置。',
    '擷取完畢，但沒有找到符合篩選條件的作品。<br>請檢查“抓取”相關的設定。',
    'Crawl complete but did not find works that match the filter criteria.<br>Please check the settings related to Crawl.',
    'クロールは終了しましたが、フィルタ条件に一致する作品が見つかりませんでした。<br>クロールに関する設定を確認してください。',
    '긁어오기가 완료되었지만 필터 조건과 일치하는 작품을 찾지 못했습니다.<br>크롤링 관련 설정을 확인하세요.',
    'Вытаскивание завершено, но не найдены работы, соответствующие критериям фильтра.<br>Пожалуйста, проверьте настройки, связанные со сканированием.',
  ],
  _当前任务尚未完成: [
    '当前任务尚未完成',
    '目前工作尚未完成',
    'The current task has not yet been completed',
    '現在の作業はまだ完了していません',
    '현재 작업이 아직 완료되지 않았습니다',
    'Текущее задание еще не выполнено',
  ],
  _当前任务尚未完成2: [
    '当前任务尚未完成，请等待完成后再下载。',
    '目前工作尚未完成，請等待完成後再下載。',
    'The current task has not yet been completed',
    '現在の作業はまだ完了していません、完了するまでお待ちください',
    '현재 작업이 아직 완료되지 않았습니다, 완료될 때까지 기다려주세요.',
    'Текущее задание еще не выполнено',
  ],
  _列表抓取完成开始获取作品页: [
    '当前列表中有{}张作品，开始获取作品信息',
    '目前清單中有 {} 張作品，開始取得作品資訊',
    'Now has {} works. Start getting the works for more information.',
    '{} 枚の作品があります。 作品情報の取得を開始します。',
    '현재 {}개의 작품이 있습니다, 작품 정보 가져오기를 시작합니다',
    'Сейчас в работе {} работ. Начните получать работы для получения дополнительной информации.',
  ],
  _开始获取作品页面: [
    '开始获取作品页面',
    '開始取得作品頁面',
    'Start getting the works page',
    '作品ページの取得を開始する',
    '작품 페이지 가져오기 시작',
    'Начинаю получать страницу с работами',
  ],
  _无权访问: [
    '无权访问 {}，跳过该作品。',
    '沒有權限存取 {}，跳過該作品。',
    'No access {}, skip.',
    '{} のアクセス権限がありません、作品を無視する。',
    '{}에 접근 권한이 없습니다, 이 작업을 건너뜁니다.',
    'Нет доступа {}, пропуск.',
  ],
  _作品页状态码0: [
    '请求的 URL 不可访问 (0)',
    '要求的 URL 無法存取 (0)',
    'The requested URL is not accessible (0)',
    '要求された URL にアクセスできません (0)',
    '요청한 URL에 접근할 수 없습니다 (0)',
    'Запрашиваемый URL недоступен (0)',
  ],
  _作品页状态码400: [
    '该作品已被删除 (400)',
    '該作品已被刪除 (400)',
    'The work has been deleted (400)',
    '作品は削除されました (400)',
    '이 작품은 삭제되었습니다 (400)',
    'Работа была удалена (400)',
  ],
  _作品页状态码401: [
    '请您登录 Pixiv 账号然后重试。(401)',
    '請您登入 Pixiv 帳號後重試。(401)',
    'Please log in to your Pixiv account and try again. (401)',
    'Pixiv アカウントにログインして、もう一度お試しください。(401)',
    'Pixiv 계정에 로그인 후 다시 시도해주세요. (401)',
    'Пожалуйста, войдите в свою учетную запись Pixiv и попробуйте еще раз. (401)',
  ],
  _作品页状态码403: [
    '无权访问请求的 URL (403)',
    '沒有權限存取要求的 URL (403)',
    'Have no access to the requested URL (403)',
    'リクエストされた URL にアクセスできない (403)',
    '요청한 URL에 접근 권한이 없습니다 (403)',
    'Нет доступа к запрашиваемому URL (403)',
  ],
  _作品页状态码404: [
    '404 not found',
    '404 not found',
    '404 not found',
    '404 not found',
    '404 not found',
    '404 not found',
  ],
  _作品页状态码429: [
    '错误代码：429（请求数量过多）。下载器会重新抓取它。',
    '錯誤程式碼：429（請求數量過多）。下載器会重新抓取它。',
    'Error code: 429 (Too many requests). The downloader will re-crawl it.',
    'エラー コード: 429 (要求が多すぎます)。ダウンローダーはそれを再クロールします。',
    '오류 코드: 429(요청이 너무 많음). 다운로더가 다시 크롤링합니다.',
    'Код ошибки: 429 (Слишком много запросов). Загрузчик будет повторять вытаскивание.',
  ],
  _错误代码: [
    '错误代码：',
    '錯誤程式碼：',
    'Error code: ',
    'エラー コード: ',
    '오류 코드: ',
    'Код ошибки: ',
  ],
  _作品页状态码500: [
    'Pixiv 拒绝返回数据 (500)。下载器会重新抓取它。',
    'Pixiv 拒絕返回資料 (500)。下載器会重新抓取它。',
    'Pixiv refuses to return data (500). The downloader will re-crawl it.',
    'ピクシブはデータの返却を拒否します (500)。ダウンローダーはそれを再クロールします。',
    'pixiv는 데이터 반환을 거부합니다 (500). 다운로더가 다시 크롤링합니다.',
    'Pixiv отказывается возвращать данные (500). Загрузчик будет повторять вытаскивание.',
  ],
  _正在抓取: [
    '正在抓取，请等待……',
    '擷取中，請稍後……',
    'Getting, please wait...',
    'クロール中、しばらくお待ちください...',
    '얻어오는 중, 잠시만 기다려주세요...',
    'Получение, пожалуйста, подождите...',
  ],
  _获取全部书签作品: [
    '获取全部书签作品，时间可能比较长，请耐心等待。',
    '取得全部書籤作品，時間可能比較長，請耐心等待。',
    'Get all bookmarked works, the time may be longer, please wait.',
    'ブックマークしたすべての作品を取得すると、時間がかかることがあります。お待ちください。',
    '북마크된 작품을 모두 가져오는 것은 시간이 오래 걸릴 수 있으니 기다려주세요.',
    'Получить все работы из закладок, это может занять время, пожалуйста, подождите',
  ],
  _抓取图片网址遇到中断: [
    '当前任务已中断!',
    '目前工作已中斷！',
    'The current task has been interrupted.',
    '現在の作業が中断されました。',
    '현재 작업이 중단되었습니다!',
    'Текущая задача была прервана.',
  ],
  _关闭: ['关闭', '關閉', 'close', 'クローズ', '닫기', 'закрыть'],
  _输出信息: [
    '输出信息',
    '輸出資訊',
    'Output information',
    '出力情報',
    '출력 정보',
    'Выходная информация',
  ],
  _复制: ['复制', '複製', 'Copy', 'コピー', '복사', 'Копировать'],
  _已复制到剪贴板: [
    '已复制到剪贴板，可直接粘贴',
    '已複製至剪貼簿，可直接貼上',
    'Has been copied to the clipboard',
    'クリップボードにコピーされました',
    '클립보드에 복사되었습니다.',
    'Скопировано в буфер обмена',
  ],
  _下载设置: [
    '下载设置',
    '下載設定',
    'Download settings',
    'ダウンロード設定',
    '다운로드 설정',
    'Настройки загрузки',
  ],
  _收起展开设置项: [
    '收起/展开设置项',
    '摺疊/展開設定項目',
    'Collapse/expand settings',
    '設定の折りたたみ/展開',
    '설정 축소/확장',
    'Свернуть/развернуть настройки',
  ],
  _github: [
    'Github 页面，欢迎 star',
    'Github 頁面，歡迎 star',
    'Github page, if you like, please star it',
    'Github のページ、star をクリックしてください',
    'Github, 유용하셨다면 Star를 주세요.',
    'Страница на Github, если вам нравится, пожалуйста, поставьте звезду',
  ],
  _wiki: ['使用手册', 'Wiki', 'Wiki', 'マニュアル', '위키', 'Вики'],
  _快捷键ALTX显示隐藏控制面板: [
    '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">X</span> 显示或隐藏控制面板。',
    '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">X</span> 顯示或隱藏控制面板。',
    'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">X</span> to show or hide the control panel.',
    'ショートカット キー <span class="blue">Alt</span> + <span class="blue">X</span> を使用して、コントロール パネルを表示または非表示にできます。',
    '단축키 <span class="blue">Alt</span> + <span class="blue">X</span>를 사용하여 제어판을 표시하거나 숨길 수 있습니다.',
    'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">X</span>, чтобы отобразить или скрыть панель управления.',
  ],
  _隐藏控制面板: [
    '隐藏控制面板（Alt + X）',
    '隱藏控制面板（Alt + X）',
    'hide control panel (Alt + X)',
    'コントロールパネルを隠す（Alt + X）',
    '제어판 숨기기 (Alt + X)',
    'скрыть панель управления (Alt + X)',
  ],
  _显示控制面板: [
    '显示控制面板 (Alt + X)',
    '顯示控制面板 (Alt + X)',
    'Show control panel (Alt + X)',
    'コントロールパネルを表示 (Alt + X)',
    '제어판 표시 (Alt + X)',
    'показать панель управления (Alt + X)',
  ],
  _共抓取到n个文件: [
    '共抓取到 {} 个文件',
    '共擷取到 {} 個檔案',
    'Crawl a total of {} files',
    '合計 {} つのファイルがあります',
    '총 {}개의 파일을 긁어왔습니다',
    'Всего просканированно {} файлов',
  ],
  _共抓取到n个作品: [
    '共抓取到 {} 个作品',
    '共擷取到 {} 個作品',
    'Crawl a total of {} works',
    '合計 {} つの作品があります',
    '총 {}개의 작품을 긁어왔습니다',
    'Всего просканированно {} работ',
  ],
  _命名规则: [
    '<span class="key">命名</span>规则',
    '<span class="key">命名</span>規則',
    '<span class="key">Naming</span> rule',
    '<span class="key">命名</span>規則',
    '<span class="key">명명</span> 규칙',
    '<span class="key">Правила</span> названий',
  ],
  _命名规则2: [
    '命名规则',
    '命名規則',
    'Naming rule',
    '命名規則',
    '명명 규칙',
    'Правила названий',
  ],
  _设置文件夹名的提示: [
    `可以使用 '<span class="key">/</span>' 建立文件夹。示例：`,
    `可以使用斜線（<span class="key">/</span>）建立資料夾。範例：`,
    `You can create a directory with '<span class="key">/</span>'. Example：`,
    `フォルダーは '<span class="key">/</span>' で作成できます。例：`,
    `'<span class="key">/</span>'을 사용하여 디렉토리를 생성할 수 있습니다.<br>예:`,
    `Вы можете создать каталог с помощью '<span class="key">/</span>'. Пример:`,
  ],
  _添加命名标记前缀: [
    '添加命名标记<span class="key">前缀</span>',
    '加入命名標記<span class="key">前綴</span>',
    'Add named tag <span class="key">prefix</span>',
    '前にタグの名前を追加',
    '명명된 태그 추가 <span class="key">접두사</span>',
    'Добавить именованный тег <span class="key">префикс</span>',
  ],
  _添加字段名称提示: [
    `例如，在用户名前面添加“user_”标记`,
    '例如，在使用者名稱前面加入「user_」標記。',
    `For example, add the 'user_' tag in front of the username`,
    'たとえば、ユーザー名の前に 「user_」タグを追加します。',
    `예: 유저명 앞에 'user_' 태그 추가`,
    `Например, добавьте тег 'user_' перед именем пользователя`,
  ],
  _命名标记id: [
    '默认文件名，如 <span class="blue">44920385_p0</span>',
    '預設檔案名稱，例如：<span class="blue">44920385_p0</span>。',
    'Default file name, for example <span class="blue">44920385_p0</span>',
    'デフォルトのファイル名，例 <span class="blue">44920385_p0</span>',
    '기본 파일명. 예: <span class="blue">44920385_p0</span>',
    'Имя файла по умолчанию, например <span class="blue">44920385_p0</span>',
  ],
  _命名标记title: [
    '作品标题',
    '作品標題',
    'Works title',
    '作品のタイトル',
    '작품 제목',
    'Название работ',
  ],
  _命名标记tags: [
    '作品的标签列表',
    '作品的標籤清單',
    'The tags of the work',
    '作品のタグ',
    '작품 태그',
    'Теги работ',
  ],
  _命名标记user: [
    '用户名字',
    '使用者名稱',
    'User name',
    'ユーザー名',
    '유저명',
    'Никнейм юзера',
  ],
  _用户id: [
    '用户 ID（数字）',
    '使用者 ID（數字）',
    'User ID (Number)',
    'ユーザー ID (Number)',
    '유저 ID (숫자)',
    'ID Юзера (Число)',
  ],
  _命名标记px: [
    '宽度和高度。例如：<span class="blue">600x900</span>',
    '寬度和高度。例如：<span class="blue">600x900</span>',
    'Width and height, e.g. <span class="blue">600x900</span>',
    '幅と高さ。例：<span class="blue">600x900</span>',
    '너비와 높이. 예: <span class="blue">600x900</span>',
    'Ширина и высота, напр. <span class="blue">600x900</span>',
  ],
  _命名标记bmk: [
    'Bookmark count，作品的收藏数。把它放在最前面可以让文件按收藏数排序。',
    'Bookmark count，作品的收藏數。將它放在最前面可以讓檔案依收藏數排序。',
    'Bookmark count, bookmarks number of works.',
    'Bookmark count，作品のボックマークの数、前に追加することでボックマーク数で并べることができます。',
    '북마크 수. 맨 앞에 두면 북마크 수별로 문서를 정렬할 수 있습니다.',
    'Количество закладок, количество произведений в закладках',
  ],
  _命名标记bmk_id: [
    'Bookmark Id。你收藏的每一个作品都会有一个 Bookmark Id。收藏的时间越晚，Bookmark Id 就越大。当你下载你的收藏时，可以使用 {bmk_id} 作为排序依据。',
    'Bookmark Id。你收藏的每一個作品都會有一個 Bookmark Id。收藏的時間越晚，Bookmark Id 就越大。當你下載你的收藏時，可以使用 {bmk_id} 作為排序依據。',
    'Bookmark Id. Every work in your bookmarks will have a Bookmark Id. The later the bookmark is added, the larger the Bookmark Id. When you download your bookmarks, you can use {bmk_id} as a sorting basis.',
    'ブックマークID。 ブックマーク内のすべての作品にはブックマークIDがあります。 ブックマークを後で追加すると、ブックマークIDが大きくなります。 ブックマークをダウンロードするときは、{bmk_id}を並べ替えの基準として使用できます。',
    '북마크 ID. 당신이 북마크하고 있는 작품마다 북마크 ID가 있습니다. 북마크 시간이 늦어질수록 북마크 ID는 커집니다. 북마크를 다운로드할때 {bmk_id}를 기준으로 정렬할 수 있습니다.',
    'Bookmark Id. Каждая работа в ваших закладках будет иметь идентификатор закладки. Чем позже добавлена закладка, тем больше Id закладки. Когда вы загружаете закладки, вы можете использовать {bmk_id} в качестве основы для сортировки.',
  ],
  _命名标记bmk_1000: [
    '作品收藏数的简化显示。例如：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
    '作品收藏數的簡化顯示。例如：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
    'Simplified number of bookmark, e.g. <span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
    '作品のボックマークの数の簡略表示。例：<span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ……',
    '단순화된 북마크 수. 예: <span class="blue">0+</span>, <span class="blue">1000+</span>, <span class="blue">2000+</span>, <span class="blue">3000+</span> ……',
    'Упрощенное количество закладок, напр. <span class="blue">0+</span>、<span class="blue">1000+</span>、<span class="blue">2000+</span>、<span class="blue">3000+</span> ......',
  ],
  _命名标记like: [
    'Like count，作品的点赞数。',
    'Like count，作品的點讚數。',
    'Like count.',
    'Like count。',
    '좋아요 수',
    'Колличество лайков',
  ],
  _命名标记view: [
    'View count，作品的浏览量。',
    'View count，作品的瀏覽量。',
    'View count.',
    'View count。',
    '조회수',
    'Колличество просмотров',
  ],
  _命名标记id_num: [
    '数字 ID，如 <span class="blue">44920385</span>',
    '數字 ID，例如：<span class="blue">44920385</span>。',
    'Number ID, for example <span class="blue">44920385</span>',
    '<span class="blue">44920385</span> などの番号 ID',
    '숫자 ID. 예: <span class="blue">44920385</span>',
    'Идентификатор номера, например <span class="blue">44920385</span>',
  ],
  _命名标记p_num: [
    '图片在作品内的序号，如 <span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span> …… 每个作品都会重新计数。',
    '圖片在作品內的序號，例如：<span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span>……每個作品都將重新計數。',
    'The serial number of the image in the work, such as <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> ... Each work will be recounted.',
    '<span class="blue">0</span>、<span class="blue">1</span>、<span class="blue">2</span> など、作品の画像のシリアル番号。各ピースは再集計されます。',
    '작품 안에 있는 번호. 예: <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> …… 작품마다 다시 세어봅니다.',
    'Порядковый номер изображения в работе, например, <span class="blue">0</span>, <span class="blue">1</span>, <span class="blue">2</span> .... Каждое произведение будет пересказано',
  ],
  _命名标记tags_trans: [
    '作品的标签列表，附带翻译后的标签（如果有）',
    '作品的標籤清單，包含翻譯後的標籤（如果有的話）。',
    'The tags of the work, with the translated tag (if any)',
    '作品のタグリスト、翻訳付きタグ(あれば)',
    '작품 태그, 번역된 태그 (있다면)',
    'Теги произведения, с тегом перевода (если есть)',
  ],
  _命名标记tags_transl_only: [
    '翻译后的标签列表',
    '譯後的標籤清單。',
    'Translated tags',
    '翻訳后のタグリスト',
    '번역된 태그',
    'Теги перевода',
  ],
  _命名标记date: [
    '作品的创建时间。如 <span class="blue">2019-08-29</span>。',
    '作品的建立時間。例如：<span class="blue">2019-08-29</span>。',
    'The time the creation of the work. Such as <span class="blue">2019-08-29</span>',
    '作品の作成時間。例 <span class="blue">2019-08-29</span>',
    '작품 생성 날짜. 예: <span class="blue">2019-08-29</span>',
    'Время создания произведения. Например, <span class="blue">2019-08-29</span>',
  ],
  _命名标记upload_date: [
    '作品内容最后一次被修改的时间。如 <span class="blue">2019-08-30</span>。',
    '作品內容最後一次被修改的時間。如 <span class="blue">2019-08-30</span>。',
    'The time when the content of the work was last modified. Such as <span class="blue">2019-08-30</span>.',
    '作品の内容が最後に変更された時刻。例 <span class="blue">2019-08-30</span>',
    '저작물의 내용이 마지막으로 수정된 시간입니다. 예: <span class="blue">2019-08-30</span>',
    'Время, когда содержание работы было изменено в последний раз. Например, <span class="blue">2019-08-30</span>.',
  ],
  _命名标记rank: [
    '作品在排行榜中的排名。如 <span class="blue">#1</span>、<span class="blue">#2</span> …… 只能在排行榜页面中使用。',
    '作品在排行榜中的排名。例如：<span class="blue">#1</span>、<span class="blue">#2</span>……只能在排行榜頁面中使用。',
    'The ranking of the work in the ranking pages. Such as <span class="blue">#1</span>, <span class="blue">#2</span> ... Can only be used in ranking pages.',
    '作品のランキング。例え　<span class="blue">#1</span>、<span class="blue">#2</span> …… ランキングページのみで使用できます。',
    '작품의 랭킹. 예: <span class="blue">#1</span>, <span class="blue">#2</span> …… 랭킹 페이지에서만 사용할 수 있습니다.',
    'Рейтинг работы на страницах рейтинга. Например, <span class="blue">#1</span>, <span class="blue">#2</span> ... Может использоваться только на страницах ранжирования.',
  ],
  _命名标记type: [
    '作品类型，分为：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    '作品類型，分為：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    'The type of work, divided into：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    '作品分類は：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    '작품 유형: <span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
    'Тип работы, разделенный на：<span class="blue">Illustration</span>, <span class="blue">Manga</span>, <span class="blue">Ugoira</span>, <span class="blue">Novel</span>',
  ],
  _命名标记AI: [
    '如果作品是由 AI 生成的，则输出 <span class="blue">AI</span>',
    '如果作品是由 AI 生成的，則輸出 <span class="blue">AI</span>',
    'If the work is generated by AI, output <span class="blue">AI</span>',
    '作品がAIで生成された場合、<span class="blue">AI</span>を出力',
    '작업이 AI로 생성된 경우 <span class="blue">AI</span> 출력',
    'Если работа создана с помощью ИИ, выведите <span class="blue">AI</span>',
  ],
  _命名标记提醒: [
    '为了防止文件名重复，命名规则里一定要包含 {id} 或者 {id_num}{p_num}。<br>您可以使用多个标记；建议在不同标记之间添加分割用的字符。示例：{id}-{user_id}',
    '為了防止檔名重複，命名規則裡一定要包含 {id} 或者 {id_num}{p_num}。<br>您可以使用多個標記；建議在不同標記之間加入分隔用的字元。範例：{id}-{user_id}',
    'To prevent duplicate file names, {id} or {id_num}{p_num} must be included in the naming rules.<br>You can use multiple tags, and you can add a separate character between different tags. Example: {id}-{user_id}',
    'ファイル名の重複を防ぐために、命名規則には {id} または {id_num}{p_num} を含める必要があります。<br>複数のタグを使用することができます；異なるタグ間の分割のために文字を追加することをお勧めします。例：{id}-{user_id}',
    '파일명이 중복되지 않도록, 명명 규칙에는 {id} 또는 {id_num}{p_num}이 포함되어야 합니다.<br>여러 태그를 사용할 수 있습니다. 서로 다른 태그 사이에 구분자를 넣는 것을 권장합니다. 예: {id}-{user}',
    'Чтобы предотвратить дублирование имен файлов, {id} или {id_num}{p_num} должны быть включены в правила именования.<br>Вы можете использовать несколько тегов, и вы можете добавить отдельный символ между различными тегами. Пример: {id}-{user_id}',
  ],
  _有些标记并不总是可用的提醒: [
    '有些标记并不总是可用，有时它们可能什么都不输出。',
    '有些標記並不總是可用，有時它們可能什麼都不輸出。',
    'Some tags are not always available, and sometimes they may output nothing.',
    '一部のタグは常に使用できるとは限らず、何も出力しない場合もあります。',
    '일부 태그는 항상 사용할 수 있는 것은 아니며 때로는 아무 것도 출력하지 않을 수도 있습니다.',
    'Некоторые теги не всегда доступны, а иногда могут ничего не выводить.',
  ],
  _命名规则一定要包含id: [
    '为了防止文件名重复，命名规则里一定要包含 {id} 或者 {id_num}{p_num}',
    '為了防止檔名重複，命名規則裡一定要包含 {id} 或者 {id_num}{p_num}。',
    'To prevent duplicate file names, {id} or {id_num}{p_num} must be included in the naming rules.',
    'ファイル名の重複を防ぐために、命名規則には {id} または {id_num}{p_num} を含める必要があります。',
    '파일명이 중복되지 않도록, 명명 규칙에는 {id} 또는 {id_num}{p_num}이 포함되어야 합니다.',
    'Чтобы предотвратить дублирование имен файлов, {id} или {id_num}{p_num} должны быть включены в правила именования.',
  ],
  _文件夹标记PTag: [
    '如果页面里的作品属于同一个标签，则输出这个标签。',
    '如果頁面裡的作品屬於同一個標籤，則輸出這個標籤。',
    'If the works on the page belong to the same tag, then output this tag.',
    'ページ上の作品が同じタグに属している場合は、このタグを出力します。',
    '페이지의 작품이 같은 태그에 속하는 경우 이 태그를 출력합니다.',
    'Если работы на странице относятся к одному и тому же тегу, то выводить этот тег.',
  ],
  _命名标记seriesTitle: [
    '系列标题，只在系列页面中可用（小说系列、漫画系列）。',
    '系列標題，只在系列頁面中可用（小說系列、漫畫系列）。',
    'Series title, only available in series pages (Novel series, Manga series).',
    'シリーズタイトル，シリーズページのみ（小説連載、漫画連載）。',
    '시리즈 제목, 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
    'Название серии, доступно только на страницах серий (серия романов, серия манги).',
  ],
  _命名标记seriesOrder: [
    '作品在系列中的序号，如 <span class="blue">#1</span> <span class="blue">#2</span>。只在系列页面中可用（小说系列、漫画系列）。',
    '作品在系列中的編號，如 <span class="blue">#1</span> <span class="blue">#2</span>。只在系列頁面中可用（小說系列、漫畫系列）。',
    'The number of the work in the series, such as <span class="blue">#1</span> <span class="blue">#2</span>. only available in series pages (Novel series, Manga series).',
    'シリーズの中の作品の番号，例え <span class="blue">#1</span> <span class="blue">#2</span>。シリーズページのみ（小説連載、漫画連載）。',
    '시리즈 내 작품 번호. 예: <span class="blue">#1</span> <span class="blue">#2</span>. 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
    'Номер работы в серии, например, <span class="blue">#1</span> <span class="blue">#2</span>. доступны только на страницах серий (серия романов, серия манги).',
  ],
  _命名标记seriesId: [
    '系列 ID，只在系列页面中可用（小说系列、漫画系列）。',
    '系列 ID，只在系列頁面中可用（小說系列、漫畫系列）。',
    'Series ID, only available in series pages (Novel series, Manga series).',
    'シリーズ ID，シリーズページのみ（小説連載、漫画連載）。',
    '시리즈 ID, 시리즈 페이지에서만 사용 가능(소설 시리즈, 만화 시리즈).',
    'Идентификатор серии, доступен только на страницах серий (серия романов, серия манги).',
  ],
  _文件夹标记PTitle: [
    '页面标题',
    '頁面標題',
    'Page title',
    'ページタイトル',
    '페이지 제목',
    'Заголовок страницы',
  ],
  _预览文件名: [
    '预览文件名',
    '預覽檔案名稱',
    'Preview file name',
    'ファイル名',
    '파일명 미리보기',
    'Имя файла предварительного просмотра',
  ],
  _下载线程: [
    '下载<span class="key">线程</span>',
    '下載<span class="key">執行緒</span>',
    'Download <span class="key">thread</span>',
    '同時ダウンロード数',
    '다운로드 <span class="key">쓰레드</span>',
    'Кол-во <span class="key">потоков</span> на загрузку',
  ],
  _下载线程的说明: [
    `你可以输入 1-${Config.downloadThreadMax} 之间的数字，设置同时下载的数量`,
    `你可以輸入 1-${Config.downloadThreadMax} 之間的數字，設定同時下載的數量。`,
    `You can type a number between 1-${Config.downloadThreadMax} to set the number of concurrent downloads`,
    `同時ダウンロード数を設定、1-${Config.downloadThreadMax} の数値を入力してください`,
    `1-${Config.downloadThreadMax} 사이의 숫자를 입력하여 동시 다운로드 수를 설정할 수 있습니다.`,
    `Вы можете ввести число между 1-${Config.downloadThreadMax} , чтобы установить количество одновременных загрузок`,
  ],
  _开始下载: [
    '开始下载',
    '開始下載',
    'Start download',
    '開始',
    '다운로드 시작',
    'Начать загрузку',
  ],
  _暂停下载: [
    '暂停下载',
    '暫停下載',
    'Pause download',
    '一時停止',
    '다운로드 일시중지',
    'Приостановить загрузку',
  ],
  _停止下载: [
    '停止下载',
    '停止下載',
    'Stop download',
    '停止',
    '다운로드 정지',
    'Остановить загрузку',
  ],
  _复制url: [
    '复制 URL',
    '複製下載網址',
    'Copy URLs',
    'URL をコピー',
    'URL 복사',
    'Копировать URL',
  ],
  _当前状态: [
    '当前状态 ',
    '目前狀態：',
    'State ',
    '現在の状態 ',
    '현재 상태',
    'Текущее состояние',
  ],
  _未开始下载: [
    '未开始下载',
    '未開始下載',
    'Not yet started downloading',
    'まだダウンロードを開始していません',
    '아직 다운로드를 시작하지 않았습니다.',
    'Загрузка еще не началась',
  ],
  _下载进度: [
    '下载进度',
    '下載進度',
    'Total progress',
    '概要',
    '다운로드 진행률',
    'Полный прогресс',
  ],
  _任务进度: [
    '任务进度',
    '任務進度',
    'Task progress',
    'タスクの進行状況',
    '작업 진행',
    'прогресс',
  ],
  _常见问题: ['常见问题', '常見問題', 'Help', 'よくある質問', '도움말', 'help'],
  _uuid: [
    '如果下载后的文件名异常，请禁用其他有下载功能的浏览器扩展。<br>例如：Chrono 下载管理器、free Download Manager、Image Downloader、DownThemAll! 等。',
    '如果下載後的檔案名稱異常，請停用其他有下載功能的瀏覽器擴充功能。<br>例如：Chrono 下载管理器、free Download Manager、Image Downloader、DownThemAll! 等。',
    'If the file name after downloading is abnormal, disable other browser extensions that have download capabilities.<br>For example: Chrono Download Manager, free Download Manager, Image Downloader, DownThemAll! and more.',
    'ダウンロード後のファイル名が異常な場合は、ダウンロード機能を持つ他のブラウザ拡張機能を無効にしてください。<br>例：Chrono Download Manager, free Download Manager, Image Downloader, DownThemAll! など。',
    '다운로드 후 파일명이 이상할 경우 다운로드 기능이 있는 다른 브라우저 확장 프로그램을 비활성화해주세요.예: Chrono Download Manager, free Download Manager、Image Downloader、DownThemAll! 등.',
    'Если имя файла после загрузки ненормальное, отключите другие расширения браузера, которые имеют возможность загрузки.<br> Например: Chrono Download Manager, бесплатный менеджер загрузок, загрузчик изображений, DownThemAll! и многое другое.',
  ],
  _常见问题说明: [
    '下载的文件保存在浏览器的下载目录里。<br><br>建议在浏览器的下载设置中关闭“下载前询问每个文件的保存位置”。<br><br>如果下载后的文件名异常，请禁用其他有下载功能的浏览器扩展。<br><br>如果你的浏览器在启动时停止响应，你可以清除浏览器的下载记录。<br><br>如果你使用 V2ray、Clash 等代理软件，可以确认一下 Pixiv 的图片域名（i.pximg.net）是否走了代理，如果没走代理就在代理规则里添加这个域名。<br><br>如果你需要一个梯子（机场）,可以试试这个机场：<a href="https://v3.xiaoy666.top/#/register?code=KEA3xTT4" title="农家有风小院" target="_blank">农家有风小院</a>，价格实惠，网络稳定。先购买订阅，然后在仪表盘复制订阅链接使用。<br><br>下载器 QQ 群：499873152<br><br>在 Wiki 查看常见问题：<br><a href="https://xuejianxianzun.github.io/PBDWiki/#/zh-cn/常见问题" target="_blank">https://xuejianxianzun.github.io/PBDWiki/#/zh-cn/常见问题</a><br><br>中文教程视频：<br><a href="https://www.youtube.com/playlist?list=PLO2Mj4AiZzWEpN6x_lAG8mzeNyJzd478d" target="_blank">https://www.youtube.com/playlist?list=PLO2Mj4AiZzWEpN6x_lAG8mzeNyJzd478d</a>',
    '下載的檔案儲存在瀏覽器的下載目錄裡。<br><br>請不要在瀏覽器的下載選項裡選取「下載每個檔案前先詢問儲存位置」。<br><br>如果下載後的檔名異常，請停用其他有下載功能的瀏覽器擴充功能。<br><br>如果你的瀏覽器在啟動時停止響應，你可以清除瀏覽器的下載記錄。',
    'The downloaded file is saved in the browsers download directory. <br><br>It is recommended to turn off "Ask where to save each file before downloading" in the browser`s download settings.<br><br>If the file name after downloading is abnormal, disable other browser extensions that have download capabilities.<br><br>If your browser stops responding at startup, you can clear your browser`s download history.',
    'ダウンロードしたファイルは、ブラウザのダウンロードディレクトリに保存されます。<br><br>ブラウザのダウンロード設定で 「 ダウンロード前に各ファイルの保存場所を確認する 」 をオフにすることをお勧めします。<br><br>ダウンロード後のファイル名が異常な場合は、ダウンロード機能を持つ他のブラウザ拡張機能を無効にしてください。<br><br>起動時にブラウザーが応答しなくなった場合は、ブラウザーのダウンロード履歴を消去できます。',
    '다운로드한 파일은 브라우저의 다운로드 디렉토리에 저장됩니다.<br><br>브라우저의 다운로드 설정에서 "다운로드 전에 각 파일의 저장 위치 확인"을 끄는 것이 좋습니다.<br><br>다운로드 후 파일명이 이상할 경우 다운로드 기능이 있는 다른 브라우저 확장 프로그램을 비활성화해주세요.<br><br>시작 시 브라우저가 응답하지 않으면 브라우저의 다운로드 기록을 지울 수 있습니다.',
    'Загруженный файл сохраняется в каталоге загрузки браузеров. <br><br>Рекомендуется отключить "Спрашивать, куда сохранять каждый файл перед загрузкой" в настройках загрузки браузера.<br><br>Если имя файла после загрузки является ненормальным, отключите другие расширения браузера, которые имеют возможности загрузки.<br><br>Если ваш браузер перестает отвечать на запросы при запуске, вы можете очистить историю загрузок вашего браузера.',
  ],
  _正在下载中: [
    '正在下载中',
    '正在下載',
    'Downloading',
    'ダウンロード中',
    '다운로드 중',
    'Загрузка',
  ],
  _下载完毕: [
    '✓ 下载完毕',
    '✓ 下載完畢',
    '✓ Download complete',
    '✓ ダウンロードが完了しました',
    '✓ 다운로드 완료',
    '✓ Загрузка завершена',
  ],
  _下载完毕2: [
    '下载完毕',
    '下載完畢',
    'Download complete',
    'ダウンロードが完了しました',
    '다운로드 완료',
    'Загрузка завершена',
  ],
  _已暂停: [
    '下载已暂停',
    '下載已暫停',
    'Download is paused',
    'ダウンロードは一時停止中です',
    '다운로드 일시중지',
    'Загрузка приостановлена',
  ],
  _已停止: [
    '下载已停止',
    '下載已停止',
    'Download stopped',
    'ダウンロードが停止しました',
    '다운로드 정지',
    'Загрузка остановлена',
  ],
  _已下载: [
    '已下载',
    '已下載',
    'downloaded',
    'downloaded',
    '다운로드됨',
    'загруженно',
  ],
  _抓取完毕: [
    '抓取完毕！',
    '擷取完畢！',
    'Crawl complete!',
    'クロールが終了しました！',
    '긁어오기 완료!',
    'Вытаскивание завершено!',
  ],
  _抓取完毕2: [
    '抓取完毕',
    '擷取完畢',
    'Crawl complete',
    'クロールが終了しました',
    '긁어오기 완료',
    'Вытаскивание завершено',
  ],
  _快速下载本页: [
    '快速下载本页作品 (Alt + Q)',
    '快速下載本頁作品 (Alt + Q)',
    'Download this work quickly (Alt + Q)',
    'この作品をすばやくダウンロードする (Alt + Q)',
    '작품 빠른 다운로드 (Alt + Q)',
    'Быстро загрузить эту работу (Alt + Q)',
  ],
  _快捷键ALTQ快速下载本页作品: [
    '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">Q</span> 快速下载本页作品。',
    '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">Q</span> 快速下載本頁作品。',
    'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">Q</span> to quickly download works on this page.',
    'ショートカット キー <span class="blue">Alt</span> + <span class="blue">Q</span> を使用して、このページの作品をすばやくダウンロードできます。',
    '단축키 <span class="blue">Alt</span> + <span class="blue">Q</span>를 사용하여 이 페이지에서 작품을 빠르게 다운로드할 수 있습니다.',
    'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">Q</span> для быстрой загрузки работ на этой странице.',
  ],
  _抓取此作品: [
    '抓取此作品',
    '抓取此作品',
    'Crawl this work',
    'この作品をクロールする',
    '이 작품을 크롤링',
    'Просканировать эту работу',
  ],
  _从本页开始抓取new: [
    '从本页开始抓取新作品',
    '從本頁開始擷取新作品',
    'Crawl the new works from this page',
    'このページから新しい作品を入手する',
    '이 페이지부터 새 작품 긁어오기',
    'Просканировать новые работы с этой страницы',
  ],
  _从本页开始抓取old: [
    '从本页开始抓取旧作品',
    '從本頁開始擷取舊作品',
    'Crawl the old works from this page',
    'このページから古い作品を入手する',
    '이 페이지부터 오래된 작품 긁어오기',
    'Просканировать старые работы с этой страницы',
  ],
  _抓取推荐作品: [
    '抓取推荐作品',
    '擷取推薦作品',
    'Crawl the recommend works',
    '推奨作品をダウンロードする',
    '추천 작품 긁어오기',
    'Просканировать рекомендуемые работы',
  ],
  _抓取推荐作品Title: [
    '抓取页面底部的的推荐作品',
    '擷取頁面底部的推薦作品。',
    'Crawl the recommended works at the bottom of the page',
    'ページの下部で推奨作品をクロールします',
    '페이지 하단 추천 작품 긁어오기',
    'Просканировать рекомендованные работы внизу страницы',
  ],
  _抓取相关作品: [
    '抓取相关作品',
    '擷取相關作品',
    'Crawl the related works',
    '関連作品をダウンロードする',
    '관련 작품 긁어오기',
    'Просканировать похожие работы',
  ],
  _调整完毕: [
    '调整完毕，当前有{}个作品。',
    '調整完畢，目前有 {} 個作品。',
    'The adjustment is complete and now has {} works.',
    '調整が完了し、今、{} の作品があります。',
    '조정이 완료되어, 현재 {}개의 작품이 있습니다.',
    'Настройка завершена и теперь имеет {} работ',
  ],
  _抓取当前作品: [
    '抓取当前作品',
    '擷取目前作品',
    'Crawl the current work',
    '現在の作品をクロールする',
    '현재 작품 긁어오기',
    'Просканировать текущую работу',
  ],
  _抓取当前作品Title: [
    '抓取当前列表里的所有作品',
    '擷取目前清單裡的所有作品',
    'Crawl all the works in the current list',
    '現在のリスト内のすべての作品をクロールする',
    '현재 목록에 있는 모든 작품 긁어오기',
    'Просканировать все работы в текущем списке',
  ],
  _清除多图作品: [
    '清除多图作品',
    '清除多圖作品',
    'Remove multi-image works',
    '複数画像をクリア',
    '여러 이미지 작품 지우기',
    'Удалить работы с несколькими изображениями',
  ],
  _清除动图作品: [
    '清除动图作品',
    '清除動圖作品',
    'Remove ugoira work',
    'うごイラ作品を削除する',
    '움직이는 일러스트 작품 지우기',
    'Убрать Ugoira(gif) работы',
  ],
  _手动删除作品: [
    '手动删除作品',
    '手動刪除作品',
    'Manually delete the work',
    '作品を手動で削除する',
    '수동으로 작품 지우기',
    'Вручную удалить работу',
  ],
  _手动删除作品Title: [
    '可以在下载前手动删除不需要的作品',
    '可以在下載前手動刪除不需要的作品，點擊作品刪除。',
    'You can manually delete unwanted work before downloading',
    'ダウンロードする前に不要な作品を手動で削除することができます',
    '다운로드를 원하지 않는 작품을 수동으로 지울 수 있습니다.',
    'Вы можете вручную удалить нежелательные работы перед загрузкой',
  ],
  _退出手动删除: [
    '退出手动删除',
    '結束手動刪除',
    'Exit manually delete',
    '削除モードを終了する',
    '수동 지우기 종료',
    'Выйти из ручного удаления',
  ],
  _抓取本页作品: [
    '抓取本页作品',
    '擷取本頁作品',
    'Crawl this page works',
    'このページをクロールする',
    '이 페이지의 작품 긁어오기',
    'Просканировать работы с этой страницы',
  ],
  _抓取本页作品Title: [
    '抓取本页列表中的所有作品',
    '擷取本頁清單中的所有作品',
    'Crawl this page works',
    'このページの全ての作品をクロールする',
    '이 페이지의 모든 작품 긁어오기',
    'Просканировать работы с этой страницы',
  ],
  _抓取本排行榜作品: [
    '抓取本排行榜作品',
    '擷取本排行榜作品',
    'Crawl the works in this list',
    'このリストの作品をクロールする',
    '이 목록의 작품 긁어오기',
    'Просканировать работы из этого списка',
  ],
  _抓取本排行榜作品Title: [
    '抓取本排行榜的所有作品，包括现在尚未加载出来的。',
    '擷取本排行榜的所有作品，包括現在尚未載入出來的。',
    'Crawl all of the works in this list, including those that are not yet loaded.',
    'まだ読み込まれていないものを含めて、このリストの作品をダウンロードする',
    '아직 불러오지 않은 작품을 포함하여, 이 목록의 모든 작품을 긁어옵니다.',
    'Просмотреть все работы в этом списке, включая те, которые еще не загружены.',
  ],
  _抓取首次登场的作品: [
    '抓取首次登场作品',
    '擷取首次登場作品',
    'Crawl the debut works',
    '初登場作品をダウンロードする',
    '데뷔작 긁어오기',
    'Просканировать по дебютные работы',
  ],
  _抓取首次登场的作品Title: [
    '只下载首次登场的作品',
    '只下載首次登場的作品',
    'Download only debut works',
    '初登場作品のみダウンロードします',
    '데뷔작만 다운로드',
    'Скачать только дебютные работы',
  ],
  _抓取该页面的图片: [
    '抓取该页面的图片',
    '擷取該頁面的圖片',
    'Crawl the image of the page',
    'ページの画像をクロールする',
    '페이지의 이미지 긁어오기',
    'Просканировать по изображение страницы',
  ],
  _抓取相似图片: [
    '抓取相似图片',
    '擷取相似圖片',
    'Crawl similar works',
    '類似の作品をクロールする',
    '비슷한 작품 긁어오기',
    'Просканировать похожие работы',
  ],
  _想要获取多少个作品: [
    '您想要获取多少个作品？',
    '想要取得多少個作品？',
    'How many works do you want to download?',
    'いくつの作品をダウンロードしたいですか？',
    '몇 개의 작품을 다운로드하시겠습니까?',
    'Сколько работ вы хотите загрузить?',
  ],
  _数字提示1: [
    '-1, 或者大于 0',
    '-1，或是大於 0',
    '-1, or greater than 0',
    '-1、または 0 より大きい',
    '-1, 또는 0보다 크게',
    '-1, или больше 0',
  ],
  _下载大家的新作品: [
    '下载大家的新作品',
    '下載大家的新作品',
    'Download everyone`s new work',
    'みんなの新作をダウンロードする',
    '모두의 새 작품 다운로드',
    'Вседа загружать новые работы',
  ],
  _屏蔽设定: [
    '屏蔽設定',
    '封鎖設定',
    'Mute settings',
    'ミュート設定',
    '차단 설정',
    'Настройки защиты',
  ],
  _举报: ['举报', '回報', 'Report', '報告', '신고', 'Отчет'],
  _输入id进行抓取: [
    '输入 ID 进行抓取',
    '輸入 ID 進行擷取',
    'Type ID to crawl',
    'IDを入力してダウンロードする',
    '유형 ID 긁어오기',
    'Введите ID для вытаскивания',
  ],
  _输入id进行抓取的提示文字: [
    '请输入作品 id。如果有多个 id，则以换行分割（即每行一个id）。',
    '請輸入作品 id。如果有多個 id，則以換行分隔（即每行一個 id）。',
    'Please type the illustration id. If there is more than one id, one id per line.',
    'イラストレーターIDを入力してください。 複数の id がある場合は、1 行に 1 つの id を付けます。',
    '일러스트 작품 ID를 입력해주세요. 여러 개의 ID가 있으면 줄을 바꾸어주세요 (한 줄에 한 개의 ID).',
    'Пожалуйста, введите идентификатор иллюстрации. Если идентификаторов несколько, то по одному идентификатору на строку.',
  ],
  _输入的ID视为图像ID: [
    '因为这个标签页展示的是图像，所以输入的 ID 会被视为图像作品的 ID。',
    '因為這個標籤頁展示的是圖片，所以輸入的 ID 會被視為圖片作品的 ID。',
    'Since this tab displays images, the ID entered will be considered the ID of the image work.',
    'このタブは画像を表示するため、入力したIDが画像作品のIDとなります。',
    '이 탭에는 이미지가 표시되므로 입력한 ID가 해당 이미지 작품의 ID로 간주됩니다.',
    'Поскольку на этой вкладке отображаются изображения, введенный идентификатор будет считаться идентификатором работы с изображением.',
  ],
  _输入的ID视为小说ID: [
    '因为这个标签页展示的是小说，所以输入的 ID 会被视为小说作品的 ID。',
    '因為這個標籤頁展示的是小說，所以輸入的 ID 會被視為小說作品的 ID。',
    'Since this tab displays novels, the ID entered will be treated as the ID of the novel work.',
    'このタブは小説を表示するため、入力したIDは小説作品のIDとして扱われます。',
    '이 탭에는 소설이 표시되므로 입력한 ID는 소설 작품의 ID로 처리됩니다.',
    'Поскольку на этой вкладке отображаются романы, введенный идентификатор будет рассматриваться как идентификатор романа.',
  ],
  _开始抓取: [
    '开始抓取',
    '開始擷取',
    'Start crawl',
    'クロールを開始する',
    '긁어오기 시작',
    'Начать вытаскивание',
  ],
  _给未分类作品添加添加tag: [
    '给未分类的作品添加标签',
    '幫未分類的作品加入標籤',
    'Add tag to uncategorized work',
    '未分類の作品にタグを追加',
    '분류되지 않은 작품에 태그 추가',
    'Добавить метку к неклассифицированной работе',
  ],
  _id不合法: [
    'id不合法',
    'id 不合法',
    'id is illegal',
    'id が不正な',
    '올바르지 않은 ID',
    'Это ID неверно',
  ],
  _快速收藏: [
    '快速收藏 (Ctrl + B)',
    '快速收藏 (Ctrl + B)',
    'Quick bookmarks (Ctrl + B)',
    'クイックブックマーク (Ctrl + B)',
    '빠른 북마크 (Ctrl + B)',
    'Быстрые закладки (Ctrl + B)',
  ],
  _启用: ['启用', '啟用', 'Enable', '有効にする', '활성화', 'Включить'],
  _自动开始下载: [
    '<span class="key">自动</span>开始下载',
    '<span class="key">自動</span>開始下載',
    'Download starts <span class="key">automatically</span>',
    'ダウンロードは自動的に開始されます',
    '<span class="key">자동으로</span> 다운로드 시작',
    'Загрузка начинается <span class="key">автоматически</span>',
  ],
  _自动开始下载的提示: [
    '当“开始下载”状态可用时，自动开始下载，不需要点击下载按钮。',
    '當可下載時自動開始下載，不需要點選下載按鈕。',
    'When the &quot;Start Download &quot; status is available, the download starts automatically and no need to click the download button.',
    '「ダウンロードを開始する」ステータスが利用可能になると、ダウンロードは自動的に開始され、ダウンロードボタンをクリックする必要はありません。',
    '"다운로드 시작" 상태가 활성화되면, 다운로드가 자동으로 시작되고 다운로드 시작 버튼을 클릭할 필요가 없게 됩니다.',
    'При активации этого тумблера загрузка начнется автоматически, без необходимости нажимать кнопку загрузки',
  ],
  _转换任务提示: [
    '正在转换 {} 个文件',
    '正在轉換 {} 個檔案',
    'Converting {} files',
    '{} ファイルの変換',
    '{}개의 파일을 변환하는 중',
    'Преобразование {} файлов',
  ],
  _最近更新: [
    '最近更新',
    '最近更新',
    'What`s new',
    '最近更新する',
    '새로운 기능',
    'Что нового',
  ],
  _确定: ['确定', '確定', 'Ok', '確定', '확인', 'Ок'],
  _file404: [
    '404 错误：文件 {} 不存在。',
    '404 錯誤：檔案 {} 不存在。',
    '404 error: File {} does not exist.',
    '404 エラー：ファイル {} は存在しません。',
    '404 오류: 파일 {}이 존재하지 않습니다.',
    '404 ошибка: Файл {} не существует.',
  ],
  _文件下载失败: [
    '文件 {} 下载失败',
    '檔案 {} 下載失敗',
    'File {} download failed',
    'ファイル {} のダウンロードを失敗しました',
    '파일 {} 다운로드 실패',
    'Загрузка файла {} не удалась',
  ],
  _是否重置设置: [
    '是否重置设置？',
    '確定要重設設定嗎？',
    'Do you want to reset the settings?',
    '設定をリセットしますか？',
    '설정을 초기화하시겠습니까?',
    'Вы хотите сбросить настройки?',
  ],
  _newver: [
    '有新版本可用',
    '有新版本可更新',
    'A new version is available',
    '新しいバージョンがあります',
    '새 버전이 있습니다',
    'Доступна новая версия',
  ],
  _id范围: [
    '<span class="key">ID</span> 范围',
    '<span class="key">ID</span> 範圍',
    '<span class="key">ID</span> range',
    '<span class="key">ID</span> 範囲',
    '<span class="key">ID</span> 범위',
    '<span class="key">ID</span> диапазон',
  ],
  _设置id范围提示: [
    '您可以输入一个作品 id，抓取比它新或者比它旧的作品',
    '可以輸入一個作品 id，擷取比它新或者比它舊的作品。',
    'You can type a work id and crawl works that are newer or older than it',
    '1 つの作品 id を入力することで、それより新しいあるいは古い作品をクロールことができます',
    '작품 ID를 입력하여, 그보다 새로운 혹은 오래된 작품을 긁어올 수 있습니다.',
    'Вы можете ввести идентификатор работы и просмотреть работы, которые новее или старше его',
  ],
  _大于: ['大于', '大於', 'Bigger than', 'より大きい', '보다 큼', 'Больше чем'],
  _小于: ['小于', '小於', 'Less than', 'より小さい', '보다 작음', 'Меньше чем'],
  _投稿时间: [
    '投稿<span class="key">时间</span>',
    '投稿<span class="key">時間</span>',
    'Posting <span class="key">date</span>',
    '投稿日時',
    '게시 <span class="key">날짜</span>',
    '<span class="key">Дата</span> публикации',
  ],
  _设置投稿时间提示: [
    '您可以下载指定时间内发布的作品',
    '可以下載指定時間內發布的作品。',
    'You can download works posted in a specified period of time',
    '指定された時間内に配信された作品をダウンロードすることができます',
    '지정된 기간 내에 게시된 작품을 다운로드할 수 있습니다.',
    'Вы можете загружать работы, размещенные за определенный период времени',
  ],
  _时间范围: [
    '时间范围',
    '時間範圍',
    'Time range',
    '時間範囲',
    '시간 범위',
    'Диапазон времени',
  ],
  _必须大于0: [
    '必须大于 0',
    '必須大於 0',
    'must be greater than 0',
    '0 より大きくなければなりません',
    '0보다 커야합니다',
    'должно быть больше 0',
  ],
  _开始筛选: [
    '开始筛选',
    '開始篩選',
    'Start screening',
    'スクリーニング開始',
    '선별 시작',
    'Начать скрининг',
  ],
  _开始筛选Title: [
    '按照设置来筛选当前标签里的作品。',
    '按照設定來篩選目前標籤裡的作品。',
    'Screen the works in the current tag.',
    '現在のタグにある作品を設定によってスクリーニングする',
    '설정에 따라 현재 태그 내 작품을 선별합니다.',
    'Отобразить работы в с текущим тегом',
  ],
  _在结果中筛选: [
    '在结果中筛选',
    '在結果中篩選',
    'Screen in results',
    '結果の中からスクリーニング',
    '결과 중에서 선별',
    'Экран результатов',
  ],
  _在结果中筛选说明: [
    '您可以改变设置，并在结果中再次筛选。',
    '可以變更設定，並在結果中再次篩選。',
    'You can change the settings and screen again in the results.',
    '設定を変えて、結果の中で再びスクリーニングすることができます。',
    '설정을 변경하고, 결과를 다시 선별할 수 있습니다',
    'Вы можете изменить настройки и снова просмотреть результаты',
  ],
  _抓取筛选结果: [
    '抓取筛选结果',
    '擷取篩選結果',
    'Crawl the screening results',
    'スクリーニングの結果をクロールする',
    '선별 결과 긁어오기',
    'Просканировать результаты скрининга',
  ],
  _尚未开始筛选: [
    '尚未开始筛选',
    '尚未開始篩選',
    'Screening has not started',
    'まだスクリーニングを開始していない',
    '선별이 시작되지 않았습니다',
    'Скрининг не начался',
  ],
  _没有数据可供使用: [
    '没有数据可供使用',
    '沒有資料可供使用',
    'No data is available.',
    '使用可能なデータはない',
    '사용 가능한 데이터가 없습니다',
    'Нет данных',
  ],
  _预览搜索结果: [
    '<span class="key">预览</span>搜索页面的筛选结果',
    '<span class="key">預覽</span>搜尋頁面的篩選結果',
    '<span class="key">Preview</span> filter results on search page',
    '検索ページのフィルタ結果をプレビューします',
    '<span class="key">미리보기</span> 검색 페이지 필터 결과',
    '<span class="key">Предварительный просмотр</span> результатов фильтрации на странице поиска',
  ],
  _预览搜索结果说明: [
    '下载器可以把符合条件的作品显示在当前页面上。如果抓取结果太多导致页面崩溃，请关闭这个功能。<br>启用预览功能时，下载器不会自动开始下载。',
    '下載器可以將符合條件的作品顯示在目前頁面上。如果擷取結果太多導致頁面當掉，請關閉這個功能。<br>啟用預覽功能時，下載器不會自動開始下載。',
    'The downloader can display the qualified works on the current page. If too many crawling results cause the page to crash, turn off this feature.<br>When the preview feature is enabled, the downloader does not start downloading automatically.',
    'ローダは、該当する作品を現在のページに表示することができます。クロール結果が多すぎてページが崩れる場合は、この機能をオフにしてください。<br>プレビュー機能を有効にすると、ダウンロードは自動的に開始されません。',
    '다운로더는 조건에 맞는 작품을 현재 페이지에 표시할 수 있습니다. 긁어오기 결과가 너무 많아서 페이지가 충돌하면 이 기능을 꺼주세요.<br> 미리보기를 사용하면 다운로드가 자동으로 시작되지 않습니다.',
    'Загрузчик может отображать подходящие работы на текущей странице. Пожалуйста, отключите эту функцию, если слишком большое количество результатов просмотра приводит к сбою страницы.<br>Загрузчик не начинает автоматическую загрузку, если включена функция предварительного просмотра.',
  ],
  _目录名使用: [
    '目录名使用：',
    '資料夾名稱使用：',
    'Name: ',
    'ディレクトリ名の使用：',
    '이름: ',
    'Имя: ',
  ],
  _目录名: ['目录名', '資料夾名稱', 'Name', 'ディレクトリ名', '이름', 'Имя'],
  _启用快速收藏: [
    '启用快速收藏',
    '開啟快速收藏',
    'Enable quick bookmark',
    'クイックボックマークを有効にする',
    '빠른 북마크 활성화',
    'Включить быструю закладку',
  ],
  _启用快速收藏说明: [
    '当你点击下载器添加的收藏按钮(☆)，把作品添加到书签时，自动添加这个作品的标签。',
    '當點選下載器新增的收藏按鈕（☆），將作品加入書籤時，自動新增這個作品的標籤。',
    'When you click the favorite button (☆) added by the downloader to bookmark a work, the tag of the work is automatically added.',
    'ダウンローダーに追加されたボックマークボタン「☆」をクリックして、作品をブックマークに追加すると、自動的に作品のタグが追加されます。',
    '다운로더에 추가된 북마크 버튼(☆)을 클릭하여 북마크에 작품을 추가하면 자동으로 이 작품의 태그가 추가됩니다.',
    'Когда вы нажимаете на кнопку Закладка (☆), добавленную загрузчиком, чтобы добавить произведение в закладки, автоматически добавляется тег для этого произведения',
  ],
  _新增设置项: [
    '新增设置项',
    '新增設定項目',
    'Added setting items',
    '新たな機能を追加されました',
    '새로운 설정 항목 추가',
    'Добавить новый элемент настройки',
  ],
  _新增功能: [
    '新增功能',
    '新增功能',
    'New feature',
    '新機能',
    '새로운 기능',
    'Новая фича',
  ],
  _抓取: ['抓取', '擷取', 'Crawl', 'クロール', '긁어오기', 'Сканирование'],
  _下载: ['下载', '下載', 'Download', 'ダウンロード', '다운로드', 'Скачивание'],
  _其他: ['其他', '其他', 'Other', 'その他', '그 외', 'Другие настройки'],
  _更多: ['更多', '更多', 'More', 'もっと', '더보기', 'Больше'],
  _第一张图不带序号: [
    '第一张图不带<span class="key">序号</span>',
    '第一張圖片不包含<span class="key">序號</span>',
    'The first image without a <span class="key">serial number</span>',
    '最初のイメージの番号を削除',
    '<span class="key">일련번호</span>가 없는 첫 번째 이미지',
    'Первое изображение без <span class="key">серийного номера</span>',
  ],
  _第一张图不带序号说明: [
    '去掉每个作品第一张图的序号。例如 80036479_p0 变成 80036479',
    '去掉每個作品第一張圖的序號。例如：80036479_p0 變成 80036479。',
    'Remove the serial number of the first image of each work. For example 80036479_p0 becomes 80036479.',
    '作品ごとの最初のイメージの番号を削除します。例えば 80036479_p0 は 80036479 になります。',
    '작품마다 첫 번째 이미지의 일련번호를 지웁니다.<br>예: 80036479_p0은 80036479가 됩니다.',
    'Удалите серийный номер с первой фотографии каждой работы. Например, 80036479_p0 становится 80036479',
  ],
  _最小值: ['最小值', '最小值', 'Minimum', '最小値', '최소', 'Минимум'],
  _最大值: ['最大值', '最大值', 'Maximum', '最大値', '최대', 'Максимум'],
  _单图作品: [
    '单图作品',
    '單圖作品',
    'Single image works',
    'シングルイメージ作品',
    '단일 이미지 작품',
    'Работа с одним изображением',
  ],
  _彩色图片: [
    '彩色图片',
    '彩色圖片',
    'Color images',
    'カラーイメージ',
    '컬러 이미지',
    'Цветная картинки',
  ],
  _黑白图片: [
    '黑白图片',
    '黑白圖片',
    'Black and white images',
    '白黒イメージ',
    '흑백 이미지',
    'Черно-белые изображения',
  ],
  _不保存图片因为颜色: [
    '{} 没有被保存，因为它的颜色不符合设定。',
    '{} 並未儲存，因為它的色彩不符合設定。',
    '{} was not saved because its colors do not match the settings.',
    '{} は色が設定に合わないため、保存されていません。',
    '{} 색상이 설정과 일치하지 않아, 저장되지 않았습니다.',
    '{} не был(и) сохранен(ы), потому что его цвета не соответствуют настройкам.',
  ],
  _同时转换多少个动图: [
    '同时<span class="key">转换</span>多少个动图',
    '同時<span class="key">轉換</span>多少個動圖',
    'How many animations are <span class="key">converted</span> at the same time',
    '同時変換のうごイラの上限',
    '동시에 <span class="key">변환할</span> 움직이는 일러스트 수',
    'Сколько анимаций <span class="key">преобразуется</span> одновременно',
  ],
  _同时转换多少个动图警告: [
    '同时转换多个动图会增加资源占用。',
    '同時轉換多個動圖會增加資源占用。',
    'Converting multiple animations at the same time will increase resource consumption. ',
    '複数の動画を同時に変換すると、リソースの占有が増加します。',
    '여러 움직이는 일러스트를 동시에 변환하면 리소스가 더 많이 사용됩니다.',
    'Одновременное преобразование нескольких анимаций увеличит потребление ресурсов. ',
  ],
  _提示: ['提示', '提示', 'Tip', 'ヒント', '팁', 'Совет'],
  _提示2: ['提示', '提示', 'Tip', '？', '팁', 'Совет'],
  _fanboxDownloader: [
    'Fanbox 下载器',
    'Fanbox 下載器',
    'Fanbox Downloader',
    'Fanbox ダウンロード',
    'Fanbox 다운로더',
    'Fanbox загрузчик',
  ],
  _不保存图片因为体积: [
    '{} 没有被保存，因为它的体积不符合设定。',
    '{} 並未儲存，因為它的大小不符合設定。',
    '{} was not saved because its size do not match the settings.',
    '{} はファイルサイズが設定に合わないため、保存されていません。',
    '{} 크기가 설정에 맞지 않아, 저장되지 않았습니다.',
    '{} не был(и) сохранен(ы), потому что его размер не соответствует настройкам.',
  ],
  _文件体积限制: [
    '文件<span class="key">体积</span>限制',
    '檔案<span class="key">體積</span>限制',
    'File <span class="key">size</span> limit',
    'ファイルサイズ制限',
    '파일 <span class="key">크기</span> 제한',
    'Ограничение <span class="key">размера</span> файла',
  ],
  _不符合要求的文件不会被保存: [
    '不符合要求的文件不会被保存。',
    '不會儲存不符合要求的檔案。',
    'Files that do not meet the requirements will not be saved.',
    '設定 に合わないファイルは保存されません。',
    '요구 사항을 충족하지 않는 파일은 저장되지 않습니다.',
    'Файлы, не соответствующие требованиям, не будут сохранены',
  ],
  _小说: ['小说', '小說', 'Novel', '小説', '소설', 'Новеллы'],
  _抓取系列小说: [
    '抓取系列小说',
    '擷取系列小說',
    'Crawl series of novels',
    '小説のシリーズをクロールする',
    '시리즈 소설 긁어오기',
    'Просканировать серию новелл',
  ],
  _合并系列小说: [
    '合并系列小说',
    '合併系列小說',
    'Merge series of novels',
    'シリーズ小説の統合',
    '시리즈 소설 합치기',
    'Объединить серию новелл',
  ],
  _小说保存格式: [
    '<span class="key">小说</span>保存格式',
    '<span class="key">小說</span>儲存格式',
    'Save the <span class="key">novel</span> as',
    '<span class="key">小説</span>の保存形式',
    '<span class="key">소설</span>저장 형식',
    'Сохранить <span class="key">новеллу</span> как',
  ],
  _在小说里保存元数据: [
    '在小说里保存<span class="key">元数据</span>',
    '在小說裡儲存<span class="key">元資料</span>',
    'Save <span class="key">metadata</span> in the novel',
    '小説の中にメタデータを保存する',
    '소설 내 <span class="key">메타데이터</span> 저장',
    'Сохранить <span class="key">метаданные</span> новеллы',
  ],
  _在小说里保存元数据提示: [
    '把作者、网址等信息保存到小说里',
    '將作者、網址等資訊儲存到小說裡',
    'Save the author, URL and other information in the file',
    '作者やURLなどの情報をファイルの中に保存します。',
    '작가, URL, 그 외 정보를 소설 내에 저장합니다.',
    'Сохранить автора, URL и другую информацию в файле',
  ],
  _收藏本页面的所有作品: [
    '收藏本页面的所有作品',
    '收藏本頁面的所有作品',
    'Bookmark all works on this page',
    'この頁の全ての作品をブックマークに追加します',
    '이 페이지의 북마크된 모든 작품 다운로드',
    'Перенести в закладки все работы на этой странице',
  ],
  _输出内容太多已经为你保存到文件: [
    '因为输出内容太多，已经为您保存到文件。',
    '因為輸出內容太多，已經為你儲存到檔案。',
    'Because the output is too much, it has been saved to a file.',
    '出力内容が多いため、txt ファイルに保存しました。',
    '출력 내용이 너무 많아, 파일로 저장했습니다.',
    'Так как выход слишком большой, он был сохранен в файл',
  ],
  _不下载重复文件: [
    '不下载<span class="key">重复</span>文件',
    '不下載<span class="key">重複</span>檔案',
    'Don`t download <span class="key">duplicate</span> files',
    '重複ファイルをダウンロードしない',
    '<span class="key">중복</span>파일 다운로드하지 않기',
    'Не загружать <span class="key">дубликаты</span> файлов',
  ],
  _不下载重复文件的提示: [
    '下载器会保存自己的下载记录，以避免下载重复的文件。<br>你可以清除浏览器的下载记录，这不会影响下载器的下载记录。<br>当你清除 Cookie 及其他网站数据时，下载器的记录也会被清除。',
    '下載器會儲存自己的下載紀錄，以避免下載重複的檔案。<br>你可以清除瀏覽器的下載記錄，這不會影響下載器的下載記錄。<br>當你清除 Cookie 及其他網站資料時，下載器的記錄也會被清除。',
    `The downloader will save its download record to avoid downloading duplicate files.<br>You can clear the browser's download history, which will not affect the downloader's download record.<br>When you clear cookies and other site data, the downloader's records will also be cleared.`,
    'ダウンローダーは独自のダウンロード履歴を保存して、重複ファイルのダウンロードを回避する。<br>ブラウザのダウンロード履歴をクリアできますが、ダウンローダのダウンロード記録には影響しません。<br>cookie と他のサイトデータを削除すると、ダウンローダーの記録も削除されます。',
    '다운로더가 중복되는 파일을 다운로드하지 않도록 자신의 다운로드 기록을 저장합니다.<br>브라우저의 다운로드 기록을 지울 수 있으며 이는 다운로더의 다운로드 기록에 영향을 미치지 않습니다.<br>쿠키와 다른 사이트 데이터를 지울 때 다운로드 기록도 삭제됩니다.',
    'Загрузчик хранит собственную историю загрузок, чтобы избежать загрузки дубликатов файлов.<br> Вы можете очистить историю загрузок вашего браузера, и это не повлияет на историю загрузок загрузчика.<br> Когда вы очищаете cookies и другие данные веб-сайта, история загрузчика также будет очищена.<br>',
  ],
  _策略: [
    '策略：',
    '策略：',
    'Strategy:',
    'フィルター：',
    '전략:',
    'Стратегия',
  ],
  _严格: ['严格', '嚴格', 'Strict', '厳格', '엄격하게', 'Строгий'],
  _宽松: ['宽松', '寬鬆', 'Loose', '緩い', '느슨하게', 'Свободный'],
  _严格模式说明: [
    '判断条件：作品的 id、上传日期、文件名',
    '判斷條件：作品的 id、上傳日期、檔名',
    'Judgment conditions: id, upload date, file name of the work',
    '審査条件：作品のID、アップロード日、ファイル名',
    '판정 조건: 작품 ID, 업로드 날짜, 파일명',
    'Условия оценки: идентификатор, дата загрузки, имя файла работы',
  ],
  _宽松模式说明: [
    '判断条件：作品的 id、上传日期',
    '判斷條件：作品的 id、上傳日期',
    'Judgment conditions: id, upload date of the work',
    '審査条件：作品のID、アップロード日',
    '판정 조건: 작품 ID, 업로드 날짜',
    'Условия оценки: идентификатор, дата загрузки работы',
  ],
  _清除下载记录: [
    '清除下载记录',
    '清除下載記錄',
    'Clear download record',
    'ダウンロード記録をクリア',
    '다운로드 기록 지우기',
    'Очистить запись загрузки',
  ],
  确定要清除下载记录吗: [
    '确定要清除下载记录吗？',
    '確定要清除下載記錄嗎？',
    'Are you sure you want to clear download record?',
    'ダウンロード記録を消去してもよろしいですか?',
    '다운로드 기록을 지우시겠습니까?',
    'Вы уверены, что хотите очистить запись загрузки?',
  ],
  _下载记录已清除: [
    '下载记录已清除',
    '已清除下載紀錄',
    'Download record has been cleared',
    'ダウンロード履歴がクリアされました',
    '다운로드 기록이 비워졌습니다',
    'Запись загрузок была очищена',
  ],
  _跳过下载因为重复文件: [
    '检测到文件 {} 已经下载过，跳过此次下载',
    '偵測到檔案 {} 已經下載過，跳過此次下載。',
    'Skip downloading duplicate files {}',
    '重複ファイル {} をスキップ',
    '파일 {}이(가) 이미 다운로드되어 있어, 다운로드를 건너뜁니다',
    'Пропустить загрузку дубликатов файлов {}',
  ],
  _保存用户头像为图标: [
    '保存用户头像为图标',
    '將使用者頭貼另存為圖示檔案',
    'Save user avatar as icon',
    'プロフィール画像をアイコンとして保存',
    '아이콘으로 유저 프로필 이미지 저장',
    'Сохранить аватар пользователя как иконку',
  ],
  _保存用户头像为图标说明: [
    '把用户头像保存为 ico 文件，可以手动设置成文件夹的图标。',
    '將使用者頭貼儲存為 ico 檔案，可以手動設定成資料夾圖示。',
    'Save user avatar as icon',
    'ユーザーのプロフィール画像を ico ファイルとして保存して、フォルダーアイコンとして設定できます。',
    '유저 프로필 이미지를 ico 파일로 저장하면, 디렉토리 아이콘으로 수동 설정할 수 있습니다.',
    'Сохранить аватар пользователя как иконку',
  ],
  _正在保存抓取结果: [
    '正在保存抓取结果',
    '正在儲存擷取結果',
    'Saving crawl results',
    'クロール結果を保存しています',
    '긁어오기 결과 저장 중',
    'Сохранение результатов вытаскивания',
  ],
  _已保存抓取结果: [
    '已保存抓取结果',
    '已儲存擷取結果',
    'Crawl results saved',
    'クロール結果を保存しました',
    '긁어오기 결과가 저장되었습니다',
    'Сохранение результатов вытаскивания',
  ],
  _正在恢复抓取结果: [
    '正在恢复抓取结果',
    '正在還原擷取結果',
    'Restoring crawl results',
    'クロール結果を再開しています',
    '긁어오기 결과 복구 중',
    'Восстановление результатов вытаскивания',
  ],
  _已恢复抓取结果: [
    '已恢复抓取结果',
    '已還原擷取結果',
    'Crawl results resumed',
    'クロール結果を再開しました',
    '긁어오기 결과가 복구되었습니다',
    'Результаты вытаскивания восстановлены',
  ],
  _清空已保存的抓取结果: [
    '清空已保存的抓取结果',
    '清除已儲存的擷取結果',
    'Clear saved crawl results',
    'セーブしたクロール結果をクリアします',
    '저장된 긁어오기 결과 비우기',
    'Очистить сохраненные результаты вытаскивания',
  ],
  _数据清除完毕: [
    '数据清除完毕',
    '資料清除完畢',
    'Data cleared',
    'クリアされたデータ',
    '데이터가 비워졌습니다',
    'Данные очищены',
  ],
  _已跳过n个文件: [
    '已跳过 {} 个文件',
    '已跳過 {} 個檔案',
    '{} files skipped',
    '{} つのファイルをスキップしました',
    '{}개의 파일을 건너뛰었습니다',
    '{} файл(ов) пропущены',
  ],
  _不保存图片因为宽高: [
    '{} 没有被保存，因为它的宽高不符合设定。',
    '{} 並未儲存，因為它的寬高不符合設定。',
    '{} was not saved because its width and height do not match the settings.',
    '{} は幅と高さが設定に合わないため、保存されていません。',
    '{} 너비와 높이가 설정에 맞지 않아, 저장되지 않았습니다.',
    '{} не был(и) сохранен, потому что его ширина и высота не соответствуют настройкам.',
  ],
  _保存: ['保存', '儲存', 'Save', '保存', '저장', 'Сохранить'],
  _加载: ['加载', '載入', 'Load', 'ロード', '불러오기', 'Загрузить'],
  _保存命名规则提示: [
    '保存命名规则，最多 20 个',
    '儲存命名規則，最多 20 個',
    'Save naming rule, up to 20',
    '命名規則を保存します。最大 20 個まで',
    '명명 규칙 저장, 최대 20개',
    'Сохранить правило именования, до 20',
  ],
  _已保存命名规则: [
    '已保存命名规则',
    '已儲存命名規則',
    'Naming rule saved',
    '命名規則を保存しました',
    '명명 규칙이 저장되었습니다.',
    'Правило наименования сохранено',
  ],
  _命名: ['命名', '命名', 'Name', '命名', '이름', 'Имя'],
  _无损: ['无损', '無損', 'Lossless', 'ロスレス', '무손실', 'Без потерь'],
  _文件名长度限制: [
    '文件名<span class="key">长度</span>限制',
    '檔案名稱<span class="key">長度</span>限制',
    'File name <span class="key">length</span> limit',
    'ファイル名の長さ制限',
    '파일명 <span class="key">길이</span> 제한',
    'Лимит <span class="key">длины</span> имени файла',
  ],
  _标签分隔符号: [
    '标签<span class="key">分隔</span>符号',
    '標簽<span class="key">分隔</span>符號',
    'Tag <span class="key">separation</span> symbol',
    'タグ <span class="key">セパレーション</span>シンボル',
    '태그 <span class="key">분리</span> 기호',
    'Тег <span class="key">символ разделения</span>',
  ],
  _标签分隔符号提示: [
    '只会影响这些命名标记的结果：<span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{tags_transl_only}</span>。<br>推荐符号<span class="blue"> , # ^ & _</span>',
    '只會影響這些命名標記的結果：<span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{tags_transl_only}</span>。<br>推薦符號<span class="blue"> , # ^ & _</span>',
    'Only affects results for these named tags: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only}</span>. <br>Recommended symbols <span class="blue"> , # ^ & _</span>',
    '次の名前付きタグの結果にのみ影響します: <span class="blue">{tags}</span>、<span class="blue">{tags_translate}</span>、<span class="blue">{ tags_transl_only }</スパン>。 <br>推奨記号 <span class="blue"> , # ^ & _</span>。',
    '이러한 명명된 태그의 결과에만 영향을 미칩니다: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only }</스팬>. <br>권장 기호 <span class="blue"> , # ^ & _</span>',
    'Влияет только на результаты для следующих именованных тегов: <span class="blue">{tags}</span>, <span class="blue">{tags_translate}</span>, <span class="blue">{ tags_transl_only </промежуток>. <br>Рекомендуемые символы <span class="blue"> , # ^ & _</span>',
  ],
  _导出csv: [
    '导出 CSV 文件',
    '匯出 CSV 檔',
    'Export CSV file',
    'CSV ファイルをエクスポート',
    'CSV 파일 내보내기',
    'Экспорт в файл CSV',
  ],
  _导出抓取结果: [
    '导出抓取结果',
    '匯出擷取結果',
    'Export results',
    'クロール結果をエクスポート',
    '결과 내보내기',
    'Экспорт результатов',
  ],
  _导入抓取结果: [
    '导入抓取结果',
    '匯入擷取結果',
    'Import results',
    'クロール結果をインポート',
    '결과 불러오기',
    'Импорт результатов',
  ],
  _导入成功: [
    '导入成功',
    '匯入成功',
    'Import successfully',
    'インポート成功',
    '가져오기 성공',
    'Импорт успешен',
  ],
  _导出成功: [
    '导出成功',
    '匯出成功',
    'Export successfully',
    'エクスポート成功',
    '내보내기 성공',
    'Импорт успешен',
  ],
  _图片尺寸: [
    '图片<span class="key">尺寸</span>',
    '圖片<span class="key">尺寸</span>',
    'Image <span class="key">size</span>',
    '画像<span class="key">サイズ</span>',
    '이미지 <span class="key">크기</span>',
    '<span class="key">Размер</span> изображения',
  ],
  _图片尺寸2: [
    '图片尺寸',
    '圖片尺寸',
    'Image size',
    '画像サイズ',
    '이미지 크기',
    'Размер изображения',
  ],
  _原图: ['原图', '原圖', 'Original', 'Original', '원본', 'Оригинал'],
  _普通: ['普通', '普通', 'Regular', 'Regular', '레귤러', 'Обычный'],
  _小图: ['小图', '小圖', 'Small', 'Small', '스몰', 'Маленький'],
  _方形缩略图: [
    '方形缩略图',
    '方形縮圖',
    'Square thumbnail',
    'Square thumbnail',
    '정사각형 썸네일',
    'Квадратная миниатюра',
  ],
  _导出: ['导出', '匯出', 'Export', 'エクスポート', '내보내기', 'Экспорт'],
  _导入: ['导入', '匯入', 'Import', 'インポート', '불러오기', 'Импорт'],
  _清除: ['清除', '清除', 'Clear', 'クリア', '비우기', 'Очистить'],
  _导入下载记录: [
    '导入下载记录',
    '匯入下載紀錄',
    'Import download record',
    'ダウンロード記録をインポート',
    '다운로드 기록 불러오기',
    'Импорт записи загрузки',
  ],
  _导出下载记录: [
    '导出下载记录',
    '匯出下載紀錄',
    'Export download record',
    'ダウンロード記録のエクスポート',
    '다운로드 기록 내보내기',
    'Экспорт записи загрузки',
  ],
  _数据较多需要花费一些时间: [
    '数据较多，需要花费一些时间',
    '資料較多，需要花費一些時間',
    'A lot of data, it will take some time',
    'データ量が多いので少し時間がかかります',
    '데이터가 많아 시간이 좀 걸립니다',
    'Много данных, это займет некоторое время',
  ],
  _完成: ['完成', '完成', 'Completed', '完了', '완료됨', 'Готово'],
  _日期格式: [
    '日期和时间<span class="key">格式</span>',
    '日期和時間<span class="key">格式</span>',
    'Date and time <span class="key">format</span>',
    '日付と時刻の書式',
    '날짜 및 시간 <span class="key">형식</span>',
    '<span class="key">Формат</span> даты и времени',
  ],
  _日期格式提示: [
    '你可以使用以下标记来设置日期和时间格式。这会影响命名规则里的 {date} 和 {upload_date} 和 {task_date}。<br>对于时间如 2021-04-30T06:40:08',
    '你可以使用以下標記來設定日期和時間格式。這會影響命名規則裡的 {date} 和 {upload_date} 和 {task_date}。<br>對於資料如：2021-04-30T06:40:08。',
    'You can use the following notation to set the date and time format. This will affect {date} and {upload_date} and {task_date} in the naming rules. <br>For time such as 2021-04-30T06:40:08',
    '以下のタグを使用して日時と時刻の書式を設定することができます。 これは命名規則の {date} と {upload_date} と {task_date} に影響します。 <br> 例：2021-04-30T06:40:08',
    '다음 표기법을 사용하여 날짜 및 시간 형식을 설정할 수 있습니다.<br>이것은 명명 규칙에 있는 {date}와 {upload_date}와 {task_date}에 영향을 미칩니다.<br>예: 2021-04-30T06:40:08',
    'Для установки формата даты и времени можно использовать следующую нотацию. Это повлияет на {date} и {upload_date} и {task_date} в правилах именования. <br>Для времени, например, 2021-04-30T06:40:08',
  ],
  _命名标记taskDate: [
    '本次任务抓取完成时的时间。例如：<span class="blue">2020-10-21</span>。',
    '本次工作擷取完成時的時間。例如：<span class="blue">2020-10-21</span>。',
    'The time when the task was crawl completed. For example: <span class="blue">2020-10-21</span>',
    'この作業のクロールが完了した時刻です。 例：<span class="blue">2020-10-21</span>',
    '긁어오기 작업을 완료한 날짜입니다. 예: <span class="blue">2020-10-21</span>',
    'Время, когда задание было выполнено. Например: <span class="blue">2020-10-21</span>',
  ],
  _自动检测: [
    '自动检测',
    '自動偵測',
    'Auto',
    '自動検出',
    '자동',
    'Авто детект',
  ],
  _公开: ['公开', '公開', 'Public', '公開', '공개', 'Публичный'],
  _不公开: ['不公开', '非公開', 'Private', '非公開', '비공개', 'Приватный'],
  _已收藏: [
    '已收藏',
    '已收藏',
    'Bookmarked',
    'ブックマークした',
    '북마크됨',
    'В закладках',
  ],
  _已收藏带参数: [
    '已收藏 {}',
    '已收藏 {}',
    'Bookmarked {}',
    'ブックマークした {}',
    '북마크된 {}',
    'В закладках {}',
  ],
  _未收藏: [
    '未收藏',
    '未收藏',
    'Not bookmarked',
    'ブックマークされていない',
    '북마크되지 않음',
    'Не в закладках',
  ],
  _下载之后收藏作品: [
    '下载之后<span class="key">收藏</span>作品',
    '下載之後<span class="key">收藏</span>作品',
    '<span class="key">Bookmark</span> works after downloading',
    'ダウンロードした作品をブックマークする',
    '다운로드 후 작품 <span class="key">북마크</span>',
    '<span class="key">Закладка</span> работает после загрузки',
  ],
  _下载之后收藏作品的提示: [
    '下载文件之后，自动收藏这个作品。',
    '下載檔案之後，自動收藏這個作品。',
    'After you download a file, the downloader will automatically bookmark the work.',
    'ダウンロード後、作品は自動的にブックマークされます。',
    '파일을 다운로드하면, 자동으로 작품을 북마크합니다.',
    'После загрузки файла загрузчик автоматически делает закладку',
  ],
  _收藏设置: [
    '下载器的<span class="key">收藏</span>按钮 (✩)',
    '下載器的<span class="key">收藏</span>按鈕 (✩)',
    `Downloader's <span class="key">bookmark</span> button (✩)`,
    'ダウンローダーの<span class="key">ブックマーク</span>ボタン (✩)',
    '다운로더의 <span class="key">북마크</span> 버튼 (☆)',
    `Кнопка <span class="key">закладок</span> загрузчика (✩)`,
  ],
  _下载器的收藏按钮默认会添加作品的标签: [
    '点击 <span class="blue">✩</span> 按钮时，下载器会收藏这个作品并且附带它的标签。',
    '點選 <span class="blue">✩</span> 按鈕時，下載器會收藏這個作品並且附帶它的標籤。',
    'When the <span class="blue">✩</span> button is clicked, the downloader bookmarks this work and attaches its tag.',
    '<span class="blue">✩</span> ボタンをクリックすると、ダウンローダはこの作品をブックマークし、タグを付けます。',
    '<span class="blue">✩</span> 버튼을 클릭하면 다운로더는 이 작품을 북마크하고 태그를 붙입니다.',
    'При нажатии кнопки <span class="blue">✩</span> загрузчик добавляет эту работу в закладки и прикрепляет свой тег.',
  ],
  _添加tag: [
    '添加标签',
    '加入標籤',
    'Add tag',
    'タグを追加',
    '태그 추가',
    'Добавить тег',
  ],
  _不添加tag: [
    '不添加标签',
    '不加入標籤',
    "Don't add tag",
    'タグなし',
    '태그 추가하지 않기',
    'Не добавлять тег',
  ],
  _用户阻止名单: [
    '用户<span class="key">阻止</span>名单',
    '使用者<span class="key">阻止</span>名單',
    'User <span class="key">block</span> list',
    'ユーザーブロックリスト',
    '유저 <span class="key">차단</span> 목록',
    '<span class="key">Блок</span> списка пользователей ',
  ],
  _用户阻止名单的说明: [
    '不下载这些用户的作品。需要输入用户 id。如果有多个用户 id，使用英文逗号,分割。',
    '不下載這些使用者的作品。需要輸入使用者 id。若有多個使用者 id，使用半形逗號（,）分隔。',
    'The works of these users will not be downloaded. Need to type the user ID. If there are multiple user ID, use comma (,) separated.',
    'これらのユーザーの作品はダウンロードしません。ユーザー ID が必要です。複数のユーザ ID は "," で区切ってください。',
    '이 유저들의 작품은 다운로드되지 않습니다. 유저 ID를 입력해야 합니다.<br>여러 유저 ID가 있는 경우 쉼표(,)로 구분합니다.',
    'Работы этих пользователей не будут загружаться. Необходимо ввести идентификатор пользователя. Если имеется несколько идентификаторов пользователя, используйте разделение запятыми (,).',
  ],
  _全部: ['全部', '全部', 'All', '全部', '전부', 'Все'],
  _任一: ['任一', '任一', 'One', '何れか', '하나만', 'Один'],
  _颜色主题: [
    '颜色<span class="key">主题</span>',
    '色彩<span class="key">主題</span>',
    'Color <span class="key">theme</span>',
    'カラーテーマ',
    '<class key="key">테마</span>',
    'Цветовая <span class="key">тема</span>',
  ],
  _管理设置: [
    '管理<span class="key">设置</span>',
    '管理<span class="key">設定</span>',
    'Manage <span class="key">settings</span>',
    '<span class="key">設定</span>の管理',
    '<span class="key">설정</span> 관리',
    'Изменение <span class="key">настроек</span>',
  ],
  _导出设置: [
    '导出设置',
    '匯出設定',
    'Export settings',
    'エクスポート設定',
    '내보내기',
    'Настройки экспорта',
  ],
  _导入设置: [
    '导入设置',
    '匯入設定',
    'Import settings',
    'インポート設定',
    '불러오기',
    'Настройки импорта',
  ],
  _重置设置: [
    '重置设置',
    '重設設定',
    'Reset settings',
    'リセット設定',
    '설정 초기화',
    'Сброс настроек',
  ],
  _日均收藏数量: [
    '日均收藏数量',
    '日均收藏數量',
    'Average number of daily bookmarks',
    '1 日の平均ブックマーク数',
    '일일 평균 북마크 수',
    'Среднее количество ежедневных закладок',
  ],
  _日均收藏数量的提示: [
    '你可以设置作品的平均每日收藏数量。满足条件的作品会被下载。',
    '您可以設定作品的平均每日收藏數量。滿足條件的作品會被下載。',
    'You can set the average daily bookmarks number of works. Works that meet the conditions will be downloaded.',
    '作品の 1 日の平均ブックマーク数を設定することができます。条件を満した作品はダウンロードされます。',
    '작품의 일일 평균 북마크 수를 설정할 수 있습니다. 조건을 만족한 작품은 다운로드됩니다.',
    'Вы можете установить среднесуточное количество закладок в работах. Работы, удовлетворяющие условиям, будут загружены.',
  ],
  _导出关注列表CSV: [
    '导出关注的用户列表（CSV）',
    '匯出關注的使用者列表（CSV）',
    'Export Followed Users List (CSV)',
    'フォローされているユーザーのリストをエクスポートする（CSV）',
    '팔로우한 사용자 목록 내보내기 (CSV)',
    'Экспорт списка отслеживаемых пользователей (CSV)',
  ],
  _导出关注列表: [
    '导出关注的用户列表（JSON）',
    '匯出關注的使用者列表（JSON）',
    'Export Followed Users List (JSON)',
    'フォローされているユーザーのリストをエクスポートする（JSON）',
    '팔로우한 사용자 목록 내보내기 (JSON)',
    'Экспорт списка отслеживаемых пользователей (JSON)',
  ],
  _批量关注用户: [
    '批量关注用户（JSON）',
    '批次關注使用者（JSON）',
    'Follow users in batches (JSON)',
    'ユーザーをバッチでフォローする（JSON）',
    '일괄적으로 사용자 팔로우 (JSON)',
    'Подписывайтесь на пользователей пакетами (JSON)',
  ],
  _导入导出关注用户列表的说明: [
    '在你或其他用户的 Following 页面里，你可以导出关注的用户列表，也可以导入列表来批量关注用户。<br>当你有多个帐户时，可以使用这个方法同步你关注的用户列表。你也可以复制其他用户的关注用户列表。',
    '在你或其他使用者的 Following 頁面裡，你可以匯出關注的使用者列表，也可以匯入列表來批次關注使用者。<br>當你有多個帳戶時，可以使用這個方法同步你關注的使用者列表。你也可以複製其他使用者的關注使用者列表。',
    "On the Following page of you or other users, you can export the list of followed users, or import the list to follow users in batches. <br>When you have multiple accounts, you can use this method to synchronize the list of users you follow. You can also copy another user's followed user list.",
    '自分または他のユーザーの [フォロー中] ページで、フォローしているユーザーのリストをエクスポートしたり、ユーザーをフォローするリストをバッチでインポートしたりできます。 <br>複数のアカウントをお持ちの場合、この方法を使用して、フォローしているユーザーのリストを同期できます。 別のユーザーのフォローしているユーザー リストをコピーすることもできます。',
    '본인 또는 다른 사용자의 팔로잉 페이지에서 팔로우한 사용자 목록을 내보내거나 목록을 가져와 사용자를 일괄적으로 팔로우할 수 있습니다. <br>계정이 여러 개인 경우 이 방법을 사용하여 팔로우하는 사용자 목록을 동기화할 수 있습니다. 다른 사용자의 팔로우된 사용자 목록을 복사할 수도 있습니다.',
    'На странице «Отслеживание» вас или других пользователей вы можете экспортировать список отслеживаемых пользователей или импортировать список для подписки на пользователей в пакетном режиме. <br>Если у вас несколько учетных записей, вы можете использовать этот метод для синхронизации списка пользователей, на которых вы подписаны. Вы также можете скопировать список отслеживаемых пользователей другого пользователя.',
  ],
  _手动选择作品: [
    '手动选择作品',
    '手動選擇作品',
    'Manually select',
    '手動で作品を選ぶ',
    '수동 선택',
    'Ручной выбор',
  ],
  _快捷键ALTS手动选择作品: [
    '你可以使用快捷键 <span class="blue">Alt</span> + <span class="blue">S</span> 开始或暂停手动选择作品。<br>选择完毕之后，打开下载器面板，点击“抓取选择的作品”。',
    '你可以使用快捷鍵 <span class="blue">Alt</span> + <span class="blue">S</span> 開始或暫停手動選擇作品。<br>選擇完畢之後，開啟下載器面板，點選“抓取選擇的作品”。',
    'You can use the shortcut keys <span class="blue">Alt</span> + <span class="blue">S</span> to start or pause manual selection of works.<br>After selecting, open the downloader panel and click "Crawl selected works".',
    'ショートカット キー <span class="blue">Alt</span> + <span class="blue">S</span> を使用して、作品の手動選択を開始または一時停止できます。<br>選択後、ダウンローダパネルを開いて「選ばれた作品をクロール」をクリック。',
    '바로 가기 키 <span class="blue">Alt</span> + <span class="blue">S</span>를 사용하여 작품 수동 선택을 시작하거나 일시 중지할 수 있습니다.<br>선택한 후 다운로더 패널을 열고 "선택된 작품 긁어오기"를 클릭합니다.',
    'Вы можете использовать сочетания клавиш <span class="blue">Alt</span> + <span class="blue">S</span>, чтобы начать или приостановить ручной выбор произведений.<br>После выбора откройте панель загрузчика и нажмите «Стащить выбранные работы».',
  ],
  _抓取选择的作品: [
    '抓取选择的作品',
    '擷取選擇的作品',
    'Crawl selected works',
    '選ばれた作品をクロール',
    '선택된 작품 긁어오기',
    'Стащить выбранные работы',
  ],
  _抓取选择的作品2: [
    '抓取选择的作品 {}',
    '擷取選擇的作品 {}',
    'Crawl selected works {}',
    '選ばれた作品をクロール {}',
    '선택된 작품 긁어오기 {}',
    'Стащить выбранные работы',
  ],
  _清空选择的作品: [
    '清空选择的作品',
    '清空選擇的作品',
    'Clear selected works',
    '選んだ作品をクリアします',
    '선택된 작품 비우기',
    'Очистить выбранные работы',
  ],
  _暂停选择: [
    '暂停选择',
    '暫停選擇',
    'Pause select',
    '選択を一時停止',
    '선택 일시중지',
    'Остановить выбора',
  ],
  _继续选择: [
    '继续选择',
    '繼續選擇',
    'Continue select',
    '選択を続ける',
    '선택 이어하기',
    'Продолжить выбор',
  ],
  _离开页面前提示选择的作品未抓取: [
    '选择的作品尚未抓取。现在离开此页面会导致你选择的作品被清空。',
    '選擇的作品尚未擷取。現在離開此頁面會導致您選擇的作品被清空。',
    'The selected work has not been crawled. Leaving this page now will cause your selected work to be cleared.',
    '選ばれた作品はまだクロールしていません。今このページを離れると、選ばれた作品がクリアされます。',
    '선택된 작품을 아직 긁어오지 않았습니다. 지금 현재 페이지를 떠나면 선택된 작품이 비워집니다.',
    'Выбранная работа не была стащена. Если вы покинете эту страницу, выбранная вами работа будет очищена.',
  ],
  _排除了所有作品类型: [
    '排除了所有作品类型',
    '排除了所有作品類型',
    'Excluded all work types',
    'すべての作品種類を除外しました',
    '모든 작품 유형 제외',
    'Исключены все типы работ',
  ],
  _为作品建立单独的文件夹: [
    '为<span class="key">每个</span>作品建立单独的文件夹',
    '為<span class="key">每個</span>作品建立單獨的資料夾',
    'Create a separate folder for <span class="key">each</span> work',
    '作品ごとに別フォルダを作成',
    '작품마다 <span class="key">별도</span>의 디렉토리 생성',
    'Создайте отдельную папку для <span class="key">каждой</span> работы',
  ],
  _为作品建立单独的文件夹说明: [
    '这里应该使用 {id_num} 代替 {id}',
    '這裡應該使用 {id_num} 代替 {id}',
    'Here {id_num} should be used instead of {id}',
    'ここでは、{id} の代わりに {id_num} を使用する必要があります',
    '여기서는 {id}대신 {id_num}을 사용해야 합니다',
    'Здесь {id_num} следует использовать вместо {id}',
  ],
  _文件数量大于: [
    '文件数量 >',
    '檔案數量 >',
    'Number of files >',
    'ファイル数 >',
    '파일 수 >',
    'Количество файлов >',
  ],
  _保存用户头像: [
    '保存用户头像',
    '儲存使用者頭貼',
    'Save user avatar',
    'ユーザーアイコンの保存',
    '유저 프로필 이미지 저장',
    'Сохранить аватар пользователя',
  ],
  _保存用户封面: [
    '保存用户封面',
    '儲存使用者封面',
    'Save user cover',
    'ユーザーカバーの保存',
    '유저 커버 저장',
    'Сохранить обложку пользователя',
  ],
  _待处理: [
    '待处理',
    '待處理',
    'Pending',
    '処理待ち',
    '처리 대기',
    'В ожидании',
  ],
  _超出最大页码: [
    '超出最大页码：',
    '超出最大頁碼：',
    'Maximum page number exceeded:',
    '最大ページ数を超えました：',
    '최대 페이지 번호 초과:',
    'Превышен максимальный номер страницы:',
  ],
  _针对特定用户屏蔽tag: [
    '针对特定用户屏蔽<span class="key">标签</span>',
    '針對特定使用者排除<span class="key">標籤</span>',
    'Block <span class="key">tags</span> for specific users',
    '特定のユーザーに対して<span class="key">タグ</span>をブロック',
    '특정 유저에 대한 차단 <span class="key">태그</span>',
    'Блокировать <span class="key">теги</span> для определенных пользователей',
  ],
  _展开收起: [
    '展开/收起',
    '展開/摺疊',
    'Expand/Collapse',
    '展開/折りたたみ',
    '확장/축소',
    'Развернуть/Свернуть',
  ],
  _展开: ['展开', '展開', 'Expand', '展開', '확장', 'Развернуть'],
  _收起: ['收起', '摺疊', 'Collapse', '折りたたみ', '축소', 'Свернуть'],
  _把r18作品存入指定的文件夹里: [
    '把 <span class="key">R-18(G)</span> 作品存入指定的文件夹里',
    '把 <span class="key">R-18(G)</span> 作品存入指定的資料夾裡',
    'Save the <span class="key">R-18(G)</span> works in the designated folder',
    '<span class="key">R-18(G)</span> の作品を指定のフォルダに入れる',
    '<span class="key">R-18(G)</span> 작품을 지정된 디렉토리에 저장',
    'Сохраните <span class="key">R-18(G)</span> работы в указанной папке',
  ],
  _必填项不能为空: [
    '必填项不能为空',
    '必填項不能為空',
    'Required fields cannot be empty',
    '必須フィールドが入力されていません',
    '필수 입력 항목은 비워둘 수 없습니다',
    'Обязательные поля не могут быть пустыми',
  ],
  _用户ID必须是数字: [
    '用户 ID 必须是数字',
    '使用者 ID 必須是數字',
    'User ID must be a number',
    'ユーザー ID は数字です',
    '유저 ID는 숫자만 허용합니다',
    'Идентификатор пользователя должен быть числом',
  ],
  _必须是数字: [
    '必须是数字',
    '必須是數字',
    'Number',
    '数字でなければなりません',
    '숫자만 허용',
    'Число',
  ],
  _tag用逗号分割: [
    '多个标签使用英文逗号,分割',
    '多個標籤使用半形逗號（,）分隔',
    'Multiple tags use comma (,) split',
    '複数のタグはカンマ「,」で区切ってください',
    '여러 태그는 쉼표(,)로 구분합니다.',
    'Для нескольких тегов используется разделение запятой (,)',
  ],
  _添加: ['添加', '新增', 'Add', '追加', '추가', 'Добавить'],
  _取消: ['取消', '取消', 'Cancel', 'キャンセル', '취소', 'Отмена'],
  _更新: ['更新', '更新', 'Update', '更新', '업데이트', 'Обновить'],
  _删除: ['删除', '刪除', 'Delete', '削除', '제거', 'Удалить'],
  _添加成功: [
    '添加成功',
    '新增成功',
    'Added successfully',
    '追加されました',
    '성공적으로 추가되었습니다.',
    'Добавлено успешно',
  ],
  _更新成功: [
    '更新成功',
    '更新成功',
    'update completed',
    '更新成功',
    '업데이트에 성공하였습니다.',
    'обновление завершено',
  ],
  _在作品缩略图上显示放大按钮: [
    '在作品缩略图上显示<span class="key">放大</span>按钮',
    '在作品縮圖上顯示<span class="key">放大</span>按鈕',
    'Show <span class="key">zoom</span> button on thumbnail',
    '作品のサムネイルに<span class="key">拡大</span>ボタンを表示',
    '썸네일에 <span class="key">확대</span> 버튼 표시',
    'Показать кнопку <span class="key">увеличить</span> на миниатюре',
  ],
  _在作品缩略图上显示下载按钮: [
    '在作品缩略图上显示<span class="key">下载</span>按钮',
    '在作品縮圖上顯示<span class="key">下載</span>按鈕',
    'Show <span class="key">download</span> button on thumbnail',
    '作品のサムネイルに<span class="key">ダウンロード</span>ボタンを表示',
    '썸네일에 <span class="key">다운로드</span> 버튼 표시',
    'Показать кнопку <span class="key">загрузить</span> на миниатюре',
  ],
  _已发送下载请求: [
    '已发送下载请求',
    '已傳送下載請求',
    'Download request sent',
    'ダウンロードリクエストを送信しました',
    '다운로드 요청 전송',
    'Запрос на скачивание отправлен',
  ],
  _HowToUse: [
    '点击页面右侧的蓝色按钮可以打开下载器面板。<br><br>下载的文件保存在浏览器的下载目录里。<br><br>建议您在浏览器的下载设置中关闭“下载前询问每个文件的保存位置”。<br><br>下载器默认开启了一些增强功能，这可能导致 Pixiv 页面样式改变。你可以在下载器的“更多”标签页中开启或关闭这些功能。',
    '點選頁面右側的藍色按鈕可以開啟下載器面板。<br><br>下載的檔案儲存在瀏覽器的下載目錄裡。<br><br>請不要在瀏覽器的下載選項裡選取「下載每個檔案前先詢問儲存位置」。<br><br>下載器預設開啟了一些增強功能，這可能導致 Pixiv 頁面樣式改變。你可以在下載器的“更多”標籤頁中開啟或關閉這些功能。',
    'Click the blue button on the right side of the page to open the downloader panel.<br><br>The downloaded file is saved in the browser`s download directory. <br><br>It is recommended to turn off "Ask where to save each file before downloading" in the browser`s download settings.<br><br>The downloader has some enhancements turned on by default, which may cause changes in the style of Pixiv pages. You can turn these features on or off in the "More" tab of the downloader.',
    'ページ右側の青いボタンをクリックすると、ダウンローダーパネルが開きます。<br><br>ダウンロードしたファイルは、ブラウザのダウンロードディレクトリに保存されます。<br><br>ブラウザのダウンロード設定で 「 ダウンロード前に各ファイルの保存場所を確認する 」 をオフにすることをお勧めします。<br><br>ダウンローダーにはデフォルトでいくつかの機能拡張が有効になっており、これにより Pixiv ページのスタイルが変更される可能性があります。 これらの機能は、ダウンローダーの「その他」タブでオンまたはオフにできます。',
    '페이지 오른쪽에 있는 파란색 버튼을 클릭하면 다운로드 패널이 열립니다.<br><br>다운로드한 파일은 브라우저의 다운로드 디렉토리에 저장됩니다.<br><br>브라우저의 다운로드 설정에서 "다운로드 전에 각 파일의 저장 위치 확인"을 끄는 것이 좋습니다.<br><br>다운로더에는 기본적으로 몇 가지 향상된 기능이 켜져 있으며 이로 인해 Pixiv 페이지 스타일이 변경될 수 있습니다. 다운로더의 "더 보기" 탭에서 이러한 기능을 켜거나 끌 수 있습니다.',
    'Нажмите синюю кнопку в правой части страницы, чтобы открыть панель загрузчика.<br><br>Загруженный файл сохраняется в каталоге загрузки браузера. <br><br>Рекомендуется отключить "Спрашивать, куда сохранять каждый файл перед загрузкой" в настройках загрузки браузера.<br><br>В загрузчике по умолчанию включены некоторые улучшения, которые могут привести к изменению стиля страниц Pixiv. Вы можете включить или отключить эти функции на вкладке «Дополнительно» загрузчика.',
  ],
  _我知道了: ['我知道了', '我知道了', 'OK', '分かりました', '확인', 'Ок'],
  _背景图片: [
    '<span class="key">背景</span>图片',
    '<span class="key">背景</span>圖片',
    '<span class="key">Background</span> image',
    '<span class="key">背景</span>画像',
    '<span class="key">배경</span> 이미지',
    '<span class="key">Фоновое</span> изображение',
  ],
  _选择文件: [
    '选择文件',
    '選擇檔案',
    'Select a file',
    'ファイルを選択',
    '파일 선택',
    'Выберите файл',
  ],
  _不透明度: [
    '不透明度',
    '不透明度',
    'Opacity',
    '不透明度',
    '투명도',
    'Непрозрачность',
  ],
  _对齐方式: [
    '对齐方式',
    '對齊方式',
    'Alignment',
    '揃え方式',
    '정렬',
    'Выравнивание',
  ],
  _顶部: ['顶部', '頂部', 'top', '上揃え', '상단', 'топ'],
  _居中: ['居中', '居中', 'center', '中央揃え', '중앙', 'центр'],
  _根据作品类型自动建立文件夹: [
    '根据作品<span class="key">类型</span>自动建立文件夹',
    '根據作品<span class="key">類型</span>自動建立資料夾',
    'Create folders based on the <span class="key">type</span> of work',
    '作品種類に応じてフォルダを自動作成',
    '작품 <span class="key">유형</span>에 따라 자동으로 디렉토리 생성',
    'Создание папок на основе <span class="key">вида</span> работы',
  ],
  _使用第一个匹配的tag建立文件夹: [
    '使用第一个匹配的<span class="key">标签</span>建立文件夹',
    '使用第一個符合的<span class="key">標籤</span>建立資料夾',
    'Create a folder with the first matched <span class="key">tag</span>',
    '最初の一致するタグにフォルダを作成',
    '첫 번째 일치하는 <span class="key">태그</span>로 디렉토리 생성',
    'Создать папку с первым совпавшим <span class="key">тегом</span>',
  ],
  _使用匹配的tag建立文件夹的说明: [
    '如果作品的标签列表里含有用户设置的标签，就会使用这个标签建立文件夹（仅限第一个）',
    '如果作品的標籤列表裡含有使用者設定的標籤，就會使用這個標籤建立資料夾（僅限第一個）',
    'If the tag list of the work contains a tag set by the user, this tag will be used to create a folder (only the first one)',
    '作品のタグリストにユーザーが設定したタグが含まれている場合、そのタグを使用してフォルダが作成されます。(最初の1つだけ)',
    '작품의 태그에 유저가 설정한 태그가 포함되어 있다면, 태그를 사용하여 디렉토리를 생성합니다. (첫 번째 태그만 해당)',
    'Если в списке тегов работы есть тег, заданный пользователем, этот тег будет использован для создания папки (только первой)',
  ],
  _全年龄: [
    '全年龄',
    '全年齡',
    'All ages',
    '全年齢',
    '전체 연령',
    'Все возраста',
  ],
  _没有符合条件的结果: [
    '没有符合条件的结果',
    '沒有符合條件的結果',
    'There are no eligible results',
    '対象となる結果はありません',
    '조건에 부합하는 결과가 없습니다',
    'Нет результатов, отвечающих требованиям',
  ],
  _收藏: ['收藏', '收藏', 'Bookmark', 'ブックマーク', '북마크', 'Закладка'],
  _已加入收藏: [
    '已加入收藏',
    '已加入收藏',
    'Bookmarked',
    'ブックマークした',
    '북마크됨',
    'В закладках',
  ],
  _全屏查看: [
    '全屏',
    '全螢幕',
    'Full screen view',
    '全画面表示',
    '전체 화면 보기',
    'Просмотр на весь экран',
  ],
  _抓取id区间: [
    '抓取 ID 区间',
    '擷取 ID 區間',
    'Crawl ID range',
    'ID 範囲をクロール',
    'ID 범위 긁어오기',
    'Стащить диапазон идентификаторов',
  ],
  _抓取id区间说明: [
    '你可以设置一个作品 ID 范围，抓取此范围内的所有作品（包含开始和结束的 id）。<br>注意：如果一次任务中产生的抓取结果数量太多，可能会导致页面崩溃。<br>如果你需要抓取很多 ID，请考虑拆分成多个任务。我建议每批抓取的 ID 数量不要超过 100,000 个。',
    '你可以設定一個作品 ID 範圍，擷取此範圍內的所有作品（包含開始和結束的 id）。<br>注意：如果一次任務中產生的擷取結果數量太多，可能會導致頁面崩潰。<br>如果你需要抓取很多 ID，請考慮拆分成多個任務。我建議每批抓取的 ID 數量不要超過 100,000 個。',
    'You can set a range of work ID and grab all works in this range (including the begin and end ID). <br>Note: If the number of crawling results in a task is too much, it may cause the page to crash.<br>If you need to crawl a lot of IDs, consider splitting it into multiple tasks. I recommend crawling no more than 100,000 IDs per batch.',
    '作品 ID の範囲を設定し、その範囲内のすべての作品をクロールすることができます。「開始 ID と終了 id を含む」<br>注意：1 つのタスクであまりにも多くのクロール結果を生成すると、ページがクラッシュする可能性があります。<br>多数の ID をクロールする必要がある場合は、複数のタスクに分割することを検討してください。 バッチごとにクロールする ID は 100,000 未満にすることをお勧めします。',
    '작품 ID 범위를 설정할 수 있습니다. 이 범위 내의 모든 작품 (시작과 끝 ID 포함).<br>참고: 작업의 긁어오기 결과가 너무 많으면 페이지가 충돌할 수 있습니다.<br>많은 ID를 크롤링해야 하는 경우 이를 여러 작업으로 분할하는 것이 좋습니다. 배치당 100,000개 이하의 ID를 크롤링하는 것이 좋습니다.',
    'Вы можете задать диапазон идентификаторов работ и захватить все работы в этом диапазоне (включая идентификаторы начала и конца). <br>Примечание: Если в задании слишком большое количество результатов стаскивания, это может привести к сбою страницы.<br>Если вам нужно сканировать большое количество идентификаторов, рассмотрите возможность разделения этого процесса на несколько задач. Я рекомендую сканировать не более 100 000 идентификаторов за пакет.',
  ],
  _抓取id区间起点: [
    '请输入开始的 ID: ',
    '請輸入開始的 ID: ',
    'Please type in the beginning ID: ',
    '開始 ID を入力してください: ',
    '시작 ID를 입력해주세요: ',
    'Пожалуйста, введите начальный идентификатор: ',
  ],
  _抓取id区间终点: [
    '请输入结束的 ID: ',
    '請輸入結束的 ID: ',
    'Please type  in the ending ID: ',
    '終了 ID を入力してください: ',
    '끝 ID를 입력해주세요: ',
    'Пожалуйста, введите конечный идентификатор: ',
  ],
  _选项卡切换方式: [
    '<span class="key">选项卡</span>切换方式',
    '<span class="key">頁籤</span>切換方式',
    'How to switch <span class="key">tabs</span>',
    'タブ切り替え方式',
    '<span class="key">탭</span> 전환 방식',
    'Как переключать <span class="key">вкладки</span>',
  ],
  _鼠标经过: [
    '鼠标经过',
    '滑鼠經過',
    'Mouse over',
    'マウスオーバー',
    '마우스 올리기',
    'Наведите мышь',
  ],
  _鼠标点击: [
    '鼠标点击',
    '滑鼠點選',
    'Mouse click',
    'マウスクリック',
    '마우스 클릭',
    'Кликнуть мышкой',
  ],
  _在序号前面填充0: [
    '在序号前面<span class="key">填充 0</span>',
    '在序號前面<span class="key">填充 0</span>',
    '<span class="key">Add 0</span> in front of the serial number',
    'シリアル番号の前に 0 を記入',
    '일련번호 앞 <span class="key">0 채우기</span>',
    '<span class="key">Добавьте 0</span> перед серийным номером',
  ],
  _在序号前面填充0的说明: [
    '这可以解决一些软件不能正确的按照文件名来排序文件的问题。',
    '這可以解決一些軟體不能正確的按照檔名來排序檔案的問題。',
    'This can solve the problem that some software cannot correctly sort files by file name.',
    'これにより、一部のソフトウェアがファイルをファイル名で正しくソートできないという問題を解決できます。',
    '이것은 일부 소프트웨어가 파일 이름별로 파일을 올바르게 정렬할 수 없는 문제를 해결할 수 있습니다.',
    'Это может решить проблему того, что некоторые программы не могут правильно сортировать файлы по имени файла.',
  ],
  _序号总长度: [
    '序号总长度',
    '序號總長度',
    'Total length of serial number',
    'シリアル番号の全長',
    '일련번호 전체 길이',
    'Общая длина серийного номера',
  ],
  _完全一致: [
    '完全一致',
    '完全一致',
    'Perfect match',
    '完全一致',
    '완전 일치',
    'Идеальное совпадение',
  ],
  _部分一致: [
    '部分一致',
    '部分一致',
    'Partial match',
    '部分一致',
    '부분 일치',
    'Частичное совпадение',
  ],
  _位置: ['位置', '位置', 'Position', '位置', '위치', 'Позиция'],
  _左: ['左', '左', 'Left', '左', '왼쪽', 'Слева'],
  _右: ['右', '右', 'Right', '右', '오른쪽', 'Справа'],
  _多图作品只下载前几张图片: [
    '多图作品只下载<span class="key">前几张</span>图片',
    '多圖作品只下載<span class="key">前幾張</span>圖片',
    'Multi-image works only download the <span class="key">first few</span> images',
    'マルチ作品は最初の何枚の画像のみをダウンロードする',
    '여러 이미지 작품은 <span class="key">처음 몇 개</span>의 이미지만 다운로드합니까',
    'При работе с несколькими изображениями загружаются только <span class="key">первые несколько</span> изображений',
  ],
  _多图作品的图片数量上限: [
    '多图作品的图片<span class="key">数量</span>上限',
    '多圖作品的圖片<span class="key">數量</span>上限',
    '<span class="key">Maximum number</span> of images for multi-image works',
    'マルチ作品の最大画像数',
    '여러 이미지 작품의 <span class="key">최대 수</span>',
    '<span class="key">Максимальное количество</span> изображений для работ с несколькими изображениями',
  ],
  _超出此限制的多图作品不会被下载: [
    '超出此限制的多图作品不会被下载',
    '不會下載超出此限制的多圖作品',
    'Multi-image works exceeding this limit will not be downloaded',
    'この制限を超えたマルチ作品はダウンロードされません',
    '이 제한을 초과하는 여러 이미지 작품은 다운로드되지 않습니다.',
    'Работы с несколькими изображениями, превышающие этот лимит, не будут загружены',
  ],
  _在搜索页面添加快捷搜索区域: [
    '在搜索页面添加快捷<span class="key">搜索</span>区域',
    '在搜尋頁面新增快速<span class="key">搜尋</span>區域',
    'Add a quick <span class="key">search</span> area on the search page',
    '検索ページにクイック検索領域を追加します',
    '검색 페이지에 빠른 <span class="key">검색</span> 영역 추가',
    'Добавить область быстрого <span class="key">поиска</span> на странице поиска',
  ],
  _保存作品的元数据: [
    '保存作品的<span class="key">元数据</span>',
    '儲存作品的<span class="key">元資料</span>',
    'Save the <span class="key">metadata</span> of the work',
    '作品のメタデータを保存する',
    '작품 <span class="key">메타데이터</span> 저장',
    'Сохранить <span class="key">метаданные</span> работы',
  ],
  _保存作品的元数据说明: [
    '为每个作品建立一个 txt 文件保存它的元数据',
    '為每個作品建立一個 txt 檔案儲存它的元資料',
    'Create a txt file for each work to save its metadata',
    '作品ごとに txt ファイルを作成して、メタデータを保存します',
    '각 작품에 대한 메타데이터를 저장하는 txt 파일 생성',
    'Создайте txt-файл для каждой работы, чтобы сохранить ее метаданные',
  ],
  _在不同的页面类型中使用不同的命名规则: [
    '在不同的页面类型中使用<span class="key">不同</span>的命名规则',
    '在不同的頁面類型中使用<span class="key">不同</span>的命名規則',
    'Use <span class="key">different</span> naming rules in different page types',
    'ページの種類によって異なる命名規則を使用',
    '페이지 유형에 따라 <span class="key">다른</span> 명명 규칙 사용',
    'Использовать <span class="key">различные</span> правила именования в разных типах страниц',
  ],
  _显示高级设置: [
    '显示<span class="key">高级</span>设置',
    '顯示<span class="key">進階</span>設定',
    'Show <span class="key">advanced</span> settings',
    '詳細設定を表示する',
    '<span class="key">고급</span> 설정 보기',
    'Показать <span class="key">расширенные</span> настройки',
  ],
  _显示高级设置说明: [
    '被隐藏的设置仍然会发挥作用',
    '被隱藏的設定仍然會發揮作用',
    'Hidden settings will still work',
    '隠していた設定がそのまま機能する',
    '숨겨진 설정은 계속 작동합니다.',
    'Скрытые настройки будут работать',
  ],
  _状态码为0的错误提示: [
    '下载时发生错误，状态码为 0，请求未成功。可能的原因：<br><br>1. 系统磁盘的剩余空间可能不足（通常是 C 盘）（建议剩余空间大于 4GB）。请尝试清理系统磁盘空间，然后重新启动浏览器，继续未完成的下载。<br><br>2. 网络错误。可能是网络代理导致的问题。如果你使用 Nginx 或者 Apache 反代理访问 pixiv，请换成梯子。<br><br>3. 可以尝试重启浏览器，或者禁用此扩展然后重新启用，并刷新这个标签页。',
    '下載時發生錯誤，狀態碼為 0，請求未成功。可能的原因：<br><br>1. 系統磁碟的剩餘空間可能不足（通常是 C 盤）（建議剩餘空間大於 4GB）。請嘗試清理系統磁碟空間，然後重新啟動瀏覽器，繼續未完成的下載。<br><br>2. 網路錯誤。可能是網路代理導致的問題。<br><br>3. 可以嘗試重啟瀏覽器，或者禁用此擴充套件然後重新啟用，並重新整理這個標籤頁。',
    'An error occurred while downloading, the status code is 0, and the request was unsuccessful. Possible reasons: <br><br>1. The remaining space of the system disk may be insufficient (usually C drive)(it is recommended that the remaining space be greater than 4GB). Please try to clear the system disk space, and then restart the browser to continue the unfinished download. <br><br>2. Network error. It may be a problem caused by a network proxy.<br><br>3. You can try to restart the browser, or disable and re-enable the extension, and refresh the tab.',
    'ダウンロード中にエラーが発生し、ステータスコードは0で、リクエストは失敗しました。 考えられる理由：<br> <br> 1。 システムディスクの残りのスペースが不足している可能性があります(通常はCドライブ)（残りのスペースは4GBを超えることをお勧めします）。 システムのディスク領域をクリアしてから、ブラウザを再起動して、未完了のダウンロードを続行してください。 <br> <br> 2。 ネットワークエラー。 ネットワークプロキシが原因の問題である可能性があります。<br><br>3. ブラウザを再起動するか、拡張機能を無効にしてから再度有効にして、タブを更新してみてください。',
    '다운로드 중 오류가 발생했으며, 상태 코드가 0이고 요청에 실패했습니다. 가능한 원인: <br><br>1. 시스템 디스크의 남은 공간이 부족할 수 있습니다(보통 C드라이브)(남은 공간은 4GB보다 큰 것이 좋습니다). 시스템 디스크 공간을 비운 다음 브라우저를 다시 시작하여 완료되지 않은 다운로드를 계속해주세요. <br><br>2. 네트워크 오류. 네트워크 프록시로 인한 문제일 수 있습니다.<br><br>3. 브라우저를 다시 시작하거나 확장 프로그램을 비활성화했다가 다시 활성화하고 탭을 새로 고칠 수 있습니다.',
    'Во время загрузки произошла ошибка, код состояния равен 0, и запрос был выполнен неудачно. Возможные причины: <br><br>1. Оставшегося места на системном диске может быть недостаточно (обычно это диск C) (рекомендуется, чтобы оставшееся место было больше 4 ГБ). Пожалуйста, попробуйте освободить место на системном диске, а затем перезапустите браузер, чтобы продолжить незаконченную загрузку. <br><br>2. Ошибка сети. Это может быть проблема, вызванная сетевым прокси-сервером.<br><br>3. Вы можете попробовать перезапустить браузер или отключить и снова включить расширение и обновить вкладку.',
  ],
  _下载完成后显示通知: [
    '下载完成后显示<span class="key">通知</span>',
    '下載完成後顯示<span class="key">通知</span>',
    'Show <span class="key">notification</span> after download is complete',
    'ダウンロードが完了した後に通知を表示する',
    '다운로드가 완료되면 <span class="key">알림</span> 표시',
    'Показать <span class="key">уведомление</span> после завершения загрузки',
  ],
  _高亮显示关键字: [
    '<span class="key">高亮</span>显示关键字',
    '<span class="key">標明</span>顯示關鍵字',
    '<span class="key">Highlight</span> keywords',
    'キーワードを強調表示',
    '<span class="key">강조</span> 키워드 표시',
    '<span class="key">Выделить</span> ключевые слова',
  ],
  _抓取标签列表: [
    '抓取标签列表',
    '擷取標籤列表',
    'Crawl a list of tags',
    'タグのリストをクロール',
    '태그 긁어오기',
    'Сканировать список тегов',
  ],
  _抓取标签列表的输入框提示: [
    '请输入你要抓取的标签列表。多个标签之间使用换行分割',
    '請輸入你要擷取的標籤列表。多個標籤之間使用換行分隔',
    'Please type the list of tags you want to crawl. Use line breaks between multiple tags',
    'クロールしたいタグのリストを入力してください。 複数のタグを改行で分割',
    '긁어올 태그를 입력해주세요. 여러 태그는 줄 바꿈 사용',
    'Пожалуйста, введите список тегов, которые вы хотите просмотреть. Используйте разрывы строк между несколькими тегами',
  ],
  _抓取标签列表的文件夹提示: [
    '在抓取标签列表时，你可以使用 {page_tag} 或者 {page_title} 标记获取当前抓取的标签，并用来建立文件夹。例如：{page_tag}/{id}',
    '在擷取標籤列表時，你可以使用 {page_tag} 或者 {page_title} 標記獲取目前擷取的標籤，並用來建立資料夾。例如：{page_tag}/{id}',
    'When crawling the tag list, you can use {page_tag} or {page_title} tags to get the tags currently crawled and use them to create folders. For example: {page_tag}/{id}',
    'タグリストをクロールする時に、 {page_tag} や {page_title}を使用すると、現在クロールされているタグを取得し、それらを使ってフォルダを作成することができます。例：{page_tag}/{id}',
    '태그를 긁어올 때 {page_tag} 또는 {page_title} 태그를 사용하여, 긁어온 태그로 디렉토리를 생성할 수 있습니다. 예: {page_tag}/{id}',
    'При сканировании списка тегов вы можете использовать теги {page_tag} или {page_title}, чтобы получить теги, которые в данный момент просматриваются, и использовать их для создания папок. Например: {page_tag}/{id}',
  ],
  _停止抓取标签列表: [
    '停止抓取标签列表',
    '停止擷取標籤列表',
    'Stop crawling the list of tags',
    'タグリストのクロールを停止',
    '태그 긁어오기 정지',
    'Прекратить сканирование списка тегов',
  ],
  _等待下载的标签: [
    '等待下载的标签',
    '等待下載的標籤',
    'Tags waiting to be downloaded',
    'ダウンロード待ちのタグ',
    '다운로드 대기 중인 태그',
    'Теги, ожидающие загрузки',
  ],
  _你确定要停止抓取吗: [
    '你确定要停止抓取吗？',
    '確定要停止擷取嗎？',
    'Are you sure you want to stop crawling?',
    '本当にクロールをやめたいのか',
    '긁어오기를 중지하시겠습니까?',
    'Ты уверен, что хочешь перестать сканировать?',
  ],
  _只能在搜索页面使用: [
    '只能在搜索页面使用',
    '只能在搜尋頁面使用',
    'Can only be used on the search page',
    '検索ページでのみ使用できます',
    '검색 페이지에서만 사용 가능',
    'Можно использовать только на странице поиска',
  ],
  _自动导出抓取结果: [
    '自动<span class="key">导出</span>抓取结果',
    '自動<span class="key">匯出</span>擷取結果',
    'Automatically <span class="key">export</span> crawl results',
    'クロール結果の自動エクスポート',
    '자동으로 긁어오기 결과 <span class="key">내보내기</span>',
    'Автоматически <span class="key">экспортировать</span> результаты сканирования',
  ],
  _抓取结果: [
    '抓取结果',
    '擷取結果',
    'Crawl results',
    'クロール結果',
    '긁어오기 결과',
    'Сканировать результаты',
  ],
  _文件格式: [
    '文件格式',
    '檔案格式',
    'File format',
    'ファイル形式',
    '파일 형식',
    'Формат файла',
  ],
  _格式: ['格式', '格式', 'Format', '形式', '형식', 'Формат'],
  _预览作品: [
    '<span class="key">预览</span>作品',
    '<span class="key">預覽</span>作品',
    '<span class="key">Preview</span> works',
    '作品のプレビュー',
    '작품 <span class="key">미리보기</span>',
    '<span class="key">Превью</span> работает',
  ],
  _点击鼠标左键可以关闭预览图: [
    '点击鼠标左键可以关闭预览图',
    '點選滑鼠左鍵可以關閉預覽圖',
    'Click the left mouse button to close the preview',
    'マウスの左クリックでプレビュー画像を閉じる',
    '마우스 왼쪽 버튼을 클릭하면 미리보기를 닫습니다',
    'Нажмите левую кнопку мыши, чтобы закрыть предварительный просмотр',
  ],
  _尺寸: ['尺寸', '尺寸', 'Size', 'サイズ', '크기', 'Размер'],
  _允许鼠标停留在预览图上: [
    '允许鼠标停留在预览图上',
    '允許滑鼠停留在預覽圖上',
    'Allow the mouse to stay on the preview image',
    'プレビュー画像の上にマウスを置くことができます',
    '마우스가 미리보기 이미지 위에서 유지되도록 허용',
    'Разрешить мыши оставаться на изображении предварительного просмотра',
  ],
  _点击预览图时下载作品: [
    '点击预览图时下载作品',
    '點選預覽圖時下載作品',
    'Download the work when you click on the preview',
    'プレビュー画像をクリックするとその作品がダウンロードされます',
    '미리보기 이미지를 클릭하면 작품 다운로드',
    'Загружать работу при нажатии на предварительный просмотр',
  ],
  _转换动图时页面被隐藏的提示: [
    '这个标签页正在转换动图。如果这个标签页被隐藏了，转换速度可能会变慢。',
    '這個標籤頁正在轉換動圖。如果這個標籤頁被隱藏了，轉換速度可能會變慢。',
    'This tab page is converting ugoira. If this tab page is hidden, the conversion speed may slow down.',
    'このタブページはうごイラを変換しています。 このタブを非表示にすると、変換速度が低下する場合があります。',
    '이 탭은 움직이는 일러스트를 변환하는 중입니다.이 탭이 숨겨지면 변환 속도가 느려질 수 있습니다.',
    'Эта страница вкладки преобразует ugoira. Если эта страница вкладки скрыта, скорость конвертации может замедлиться.',
  ],
  _原始尺寸: [
    '原始尺寸',
    '原始尺寸',
    'Original size',
    'オリジナルサイズ',
    '원본 크기',
    'Оригинальный размер',
  ],
  _增强: ['增强', '增強', 'Enhance', '強化機能', '향상', 'Улучшение'],
  _长按右键显示大图: [
    '在缩略图上长按鼠标右键时显示<span class="key">大图</span>',
    '在縮圖上長按滑鼠右鍵時顯示<span class="key">大圖</span>',
    'Long press the right mouse button on the thumbnail to display the <span class="key">large image</span>',
    'サムネイルでマウスの右ボタンを長押しすると、大きな画像が表示されます。',
    '썸네일을 마우스 오른쪽 버튼으로 클릭했을 때 <span class="key">큰 이미지</span> 표시',
    'Длительное нажатие правой кнопки мыши на миниатюре для отображения <span class="key">большого изображения</span>',
  ],
  _鼠标滚轮切换图片: [
    '预览多图作品时，可以使用鼠标滚轮切换图片。',
    '預覽多圖作品時，可以使用滑鼠滾輪切換圖片。',
    'When previewing multi-image works, you can use the mouse wheel to switch images.',
    '複数画像をプレビューする際に、マウスホイールを使って画像を切り替えることができます。',
    '여러 이미지 작품을 미리 볼 때, 마우스 휠을 사용하여 이미지를 전환할 수 있습니다.',
    'При предварительном просмотре работ с несколькими изображениями можно использовать колесико мыши для переключения изображений',
  ],
  _whatisnew: [
    `修复因为 Pixiv 的变化而导致的抓取失败的问题。`,
    `修復因為 Pixiv 的變化而導致的抓取失敗的問題。`,
    `Fixed crawl failures due to Pixiv changes.`,
    `Pixiv の変更によるクロールの失敗を修正しました。`,
    `Pixiv 변경으로 인한 크롤링 실패를 수정했습니다.`,
    'Исправлены сбои в сканировании из-за изменений в Pixiv',
  ],
  _等待时间: [
    '等待时间',
    '等待時間',
    'Waiting time',
    '待ち時間',
    '대기 시간',
    'Время ожидания',
  ],
  _格式错误: [
    '格式错误',
    '格式錯誤',
    'Format error',
    'フォーマットエラー',
    '형식 오류',
    'Ошибка форматантирования',
  ],
  _下载数量错误: [
    '下载的页数（作品）数量设置错误',
    '下載的頁數（作品）數量設定錯誤',
    'The number of downloaded pages (works) is set incorrectly',
    '下载页数（作品）设置不正确',
    '다운로드할 페이지 수(작품) 설정 오류',
    'Количество загруженных страниц (работ) установлено неверно',
  ],
  _默认下载多页: [
    '开始抓取, 如有多页，默认会下载全部。',
    '開始擷取，如有多頁，預設會下載全部。',
    'Start crawl, if there are multiple pages, the default will be downloaded.',
    'クロールを開始する、複数のページがある場合、デフォルトですべてをダウンロードされます。',
    '긁어오기를 시작합니다. 여러 페이지가 있으면 기본적으로 모두 다운로드됩니다.',
    'Начать сканирование, если есть несколько страниц, все будут загружены по умолчанию.',
  ],
  _赞助我: [
    '赞助我',
    '贊助我',
    'Sponsor me',
    '支援する',
    '후원하기',
    'Поддержать меня',
  ],
  _赞助方式提示: [
    `非常感谢您的支持！<br>
    您可以在 Patreon 上赞助我：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a><br>
    中国大陆用户可以在“爱发电”上赞助我：<br>
    <a href="https://afdian.net/@xuejianxianzun" target="_blank">https://afdian.net/@xuejianxianzun</a><br>
    也可以扫描二维码：<br>
    <a href="https://github.com/xuejianxianzun/PixivBatchDownloader#%E6%94%AF%E6%8C%81%E5%92%8C%E6%8D%90%E5%8A%A9" target="_blank">在 Github 上查看二维码</a> 或者加入 QQ 群 499873152，在群文件里查看二维码。
    `,
    `非常感謝您的支持！<br>
    您可以在 Patreon 上贊助我：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a><br>
    中國大陸使用者可以在“愛發電”上贊助我：<br>
    <a href="https://afdian.net/@xuejianxianzun" target="_blank">https://afdian.net/@xuejianxianzun</a><br>
    也可以掃描行動條碼（QR Code）：<br>
    <a href="https://github.com/xuejianxianzun/PixivBatchDownloader#%E6%94%AF%E6%8C%81%E5%92%8C%E6%8D%90%E5%8A%A9" target="_blank">在 Github 上檢視行動條碼</a>。
    `,
    `Thank you very much for your support!<br>
    You can sponsor me on Patreon: <br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
    `ご支援してくださった皆様、本当にありがとうございました。<br>
    ご支援してくださった方は、以下の Patreon で：<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank"> https://www.patreon.com/xuejianxianzun </a>
    `,
    `지원해주셔서 정말 감사합니다!<br>
    Patreon에서 저를 후원해주세요<br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
    `Большое вам спасибо за вашу поддержку!<br>
    Вы можете спонсировать меня на Patreon: <br>
    <a href="https://www.patreon.com/xuejianxianzun" target="_blank">https://www.patreon.com/xuejianxianzun</a>
    `,
  ],
  _替换方形缩略图以显示图片比例: [
    '替换方形<span class="key">缩略图</span>以显示图片比例',
    '替換方形<span class="key">縮圖</span>以顯示圖片比例',
    'Replace square <span class="key">thumbnails</span> to show image ratio',
    '正方形のサムネイルを置き換えて、画像のスケールを表示。',
    '이미지 종횡비를 표시하기 위해 정사각형 <span class="key">썸네일</span> 교체',
    'Замените квадратные <span class="key">миниатюры</span>, чтобы показать соотношение сторон изображения',
  ],
  _只有一个抓取结果时不建立文件夹: [
    '只有一个抓取结果时<span class="key">不建立</span>文件夹',
    '只有一個擷取結果時<span class="key">不建立</span>資料夾',
    '<span class="key">Do not create</span> a folder when there is only one crawl result',
    'クロール結果が１つのみの場合、フォルダを作成しない',
    '긁어오기 결과가 하나일 때 디렉토리 <span class="key">생성하지 않기</span>',
    'Когда есть только один результат сканирования, <span class="key">никакая</span> папка не создается',
  ],
  _搜索页面页数限制: [
    '由于 pixiv 的限制，下载器最多只能抓取到第 {} 页。',
    '由於 pixiv 的限制，下載器最多只能擷取到第 {} 頁。',
    'Due to the limitation of pixiv, the downloader can only crawl up to the {}th page.',
    'pixiv の制限により、ダウンローダーは {} ページ目までしかクロールできません。',
    'pixiv 제한으로 인해 최대 {} 페이지까지만 다운로드 받을 수 있습니다.',
    'Из-за ограничений pixiv загрузчик может сканировать только до {}-й страницы',
  ],
  _获取图片的宽高时出现错误: [
    '获取图片的宽高时出现错误：',
    '獲取圖片的寬高時出現錯誤：',
    'An error occurred while getting the width and height of the image:',
    '画像の幅と高さの取得中にエラーが発生しました：',
    '이미지의 너비를 가져오는 도중 오류가 발생했습니다:',
    'Произошла ошибка при получении ширины и высоты изображения:',
  ],
  _上限: ['上限', '上限', 'Upper limit', '上限', '상한', 'Верхний предел'],
  _预览搜索结果的数量达到上限的提示: [
    '预览搜索结果的数量已经达到上限，剩余的结果不会显示。',
    '預覽搜尋結果的數量已經達到上限，剩餘的結果不會顯示。',
    'The number of preview search results has reached the upper limit, and the remaining results will not be displayed.',
    'プレビュー検索結果の数が上限に達し、残りの結果は表示されません。',
    '미리보기 검색 결과 수가 상한에 도달하여, 남은 결과는 표시되지 않습니다.',
    'Количество результатов предварительного поиска достигло верхнего предела, и оставшиеся результаты не будут отображаться.',
  ],
  _新增命名标记: [
    '新增命名标记',
    '新增命名標記',
    'Add named tag',
    '名前付きタグを追加',
    '명명된 태그 추가',
    'Добавить именованный тег',
  ],
  _自定义用户名: [
    '自定义<span class="key">用户名</span>',
    '自訂<span class="key">使用者名稱</span>',
    'Customize <span class="key">username</span>',
    'カスタムユーザー名',
    '사용자 정의 <span class="key">유저명</span>',
    'Настроить <span class="key">имя пользователя</span>',
  ],
  _自定义用户名的说明: [
    `有些用户可能会改名，如果你想使用他原来的名字，你可以在这里手动设置他的名字。<br>
    你也可以为用户设置别名。<br>
    当你在命名规则中使用 {user} 标记时，下载器会优先使用你设置的名字。`,
    `有些使用者可能會改名，如果你想使用他原來的名字，你可以在這裡手動設定他的名字。<br>
    你也可以為使用者設定別名。<br>
    當你在命名規則中使用 {user} 標記時，下載器會優先使用你設定的名字。`,
    `Some users may change their name. If you want to use his original name, you can manually set his name here. <br>
    You can also set aliases for users. <br>
    When you use the {user} tag in the naming rule, the downloader will give priority to the name you set.`,
    `ユーザーによっては名前を変更する場合があります。元の名前を使いたい場合は、ここで名前を手動で設定することができます。<br>
    また、ユーザーの別名を設定することも可能です。<br>
    命名規則で {user} タグを使用すると、ダウンローダーは設定された名前を優先的に使用します。`,
    `일부 유저는 이름을 바꿀 수 있습니다. 만약 당신이 그의 원래 이름을 사용하고 싶다면, 당신은 여기에서 그의 이름을 수동으로 설정할 수 있습니다.<br>
    사용자의 별칭을 설정할 수도 있습니다. <br>
    명명 규칙에 {user} 태그를 사용할 때 다운로드더가 사용자 정의 유저명을 우선시합니다.`,
    `Некоторые пользователи могут изменить свое имя. Если вы хотите использовать его оригинальное имя, вы можете вручную задать его имя здесь. <br>
    Вы также можете задать псевдонимы для пользователей. <br>
    Когда вы используете тег {user} в правиле именования, загрузчик будет отдавать приоритет имени, которое вы задали.`,
  ],
  _移除用户名中的at和后续字符: [
    '移除用户名中的 <span class="key">@</span> 和后续字符',
    '移除使用者名稱中的 <span class="key">@</span> 和後續字元',
    'Remove <span class="key">@</span> and subsequent characters in username',
    'ユーザー名から <span class="key">@</span> 以降の文字を削除する',
    '유저명에서 @와 후속 문자 제거',
    'Удалить <span class="key">@</span> и последующие символы в имени пользователя',
  ],
  _移除用户名中的at和后续字符的说明: [
    '例如：Anmi@画集発売中 → Anmi',
    '例如：Anmi@画集発売中 → Anmi',
    'For example：Anmi@画集発売中 → Anmi',
    '例：Anmi@画集発売中 → Anmi',
    '예: Anmi@画集発売中 → Anmi',
    'Например: Anmi@画集発売中 → Anmi',
  ],
  _抓取被限制时返回空结果的提示: [
    'Pixiv 返回了空数据。下载器已暂停抓取，并且会在等待几分钟后继续抓取。(429)<br>这说明您的账号被 Pixiv 限制访问了，等待几分钟即可恢复正常。<br>您可以启用“减慢抓取速度”功能来减少 429 问题出现的概率。',
    'Pixiv 返回了空資料。下載器已暫停抓取，並且會在等待幾分鐘後繼續抓取。(429)<br>這說明您的賬號被 Pixiv 限制訪問了，等待幾分鐘即可恢復正常。<br>您可以啟用“減慢抓取速度”功能來減少 429 問題出現的機率。',
    'Pixiv returned empty data. The downloader has paused crawling and will resume crawling after a few minutes. (429)<br>This means that your account has been restricted by Pixiv, please wait for a few minutes for it to return to normal.<br>You can reduce the chances of 429 issues by enabling the "Slow down crawl" feature.',
    'Pixivが空のデータを返しました。 ダウンローダーはクロールを一時停止し、数分後にクロールを再開します。(429)<br>これは、あなたのアカウントが Pixiv によって制限されていることを意味します。通常の状態に戻るまで数分お待ちください。<br>"クロールを遅くする" 機能を有効にすると、429 の問題が発生する可能性を減らすことができます。',
    'Pixiv가 빈 데이터를 반환했습니다. 다운로더가 긁어오기를 일시 중지하고 몇 분 동안 기다린 후 긁어오기를 계속합니다. (429)<br>이것은 귀하의 계정이 Pixiv에 의해 제한되었음을 의미합니다. 정상으로 돌아갈 때까지 몇 분 정도 기다리십시오.<br>"천천히 크롤링" 기능을 활성화하면 429 문제 발생 가능성을 줄일 수 있습니다.',
    'Pixiv вернул пустые данные. Загрузчик приостановил загрузку и возобновит ее через несколько минут. (429)<br>Это означает, что ваша учетная запись была ограничена Pixiv, подождите несколько минут, пока она вернется в нормальное состояние.<br>Вы можете снизить вероятность возникновения ошибок 429, включив функцию «Замедлить сканирование».',
  ],
  _搜索模式: [
    '搜索模式',
    '搜尋模式',
    'Search mode',
    '検索モード',
    '검색 모드',
    'Режим поиска',
  ],
  _标签部分一致: [
    '标签（部分一致）',
    '標籤（部分一致）',
    'Tags (partial match)',
    'タグ（部分一致）',
    '태그 (부분 일치)',
    'Теги (частичное совпадение)',
  ],
  _标签完全一致: [
    '标签（完全一致）',
    '標籤（完全一致）',
    'Tags (perfect match)',
    'タグ（完全一致）',
    '태그 (완전 일치)',
    'Теги (идеальное совпадение)',
  ],
  _标题说明文字: [
    '标题、说明文字',
    '標題、說明文字',
    'Title, Caption',
    'タイトル・キャプション',
    '제목, 설명',
    'Название, Подпись',
  ],
  _正文: ['正文', '本文', 'Text', '本文', '본문', 'Текст'],
  _标签标题说明文字: [
    '标签、标题、说明文字',
    '標籤、標題、說明文字',
    'Tags, Titles, Captions',
    'タグ・タイトル・キャプション',
    '태그, 제목, 설명',
    'Теги, Заголовки, Подписи',
  ],
  _save_file_failed_tip: [
    `{} 保存失败，code：{}。下载器将会重试下载这个文件。`,
    `{} 儲存失敗，code：{}。下載器將會重試下載這個檔案。`,
    `{} save failed, code: {}. The downloader will retry to download the file.`,
    `{} 保存に失敗しました。code：{}。ダウンローダーはファイルのダウンロードを再試行します。`,
    `{} 저장에 실패했습니다. 코드: {}. 다운로드더가 파일 다운로드를 다시 시도합니다.`,
    `{} сохранение не удалось, код: {}. Загрузчик повторит попытку загрузить файл.`,
  ],
  _user_canceled_tip: [
    `{} 未保存，code：{}。`,
    `{} 未儲存，code：{}。`,
    `{} not saved, code: {}.`,
    `{} 保存されていません。code：{}。`,
    `{} 저장되지 않음, 코드: {}.`,
    `{} не сохранено, код: {}.`,
  ],
  _FILE_FAILED_tip: [
    '可能是文件名太长，或是其他原因导致文件保存失败。你可以尝试启用高级设置里的“文件名长度限制”。',
    '可能是檔名太長，或是其他原因導致檔案儲存失敗。你可以嘗試啟用高階設定裡的“檔案名稱長度限制”。',
    'Maybe the file name is too long, or other reasons cause the file to fail to save. You can try enabling "File name length limit" in advanced settings.',
    'ファイル名が長すぎるか、他の理由でファイルの保存に失敗した可能性があります。 詳細設定で「ファイル名の長さ制限」を有効にしてみてください。',
    '파일명이 너무 길거나 다른 이유로 저장에 실패한 것 같습니다. 고급 설정에서 "파일명 길이 제한"을 사용하도록 설정할 수 있습니다.',
    'Возможно, имя файла слишком длинное, или по другим причинам файл не удается сохранить. Вы можете попробовать включить "Ограничение длины имени файла". в расширенных настройках.',
  ],
  _显示摘要信息: [
    '显示摘要信息',
    '顯示摘要資訊',
    'Show summary',
    '要約情報を表示する',
    '요약 정보 표시',
    'Показать сводку',
  ],
  _显示更大的缩略图: [
    '显示<span class="key">更大</span>的缩略图',
    '顯示<span class="key">更大</span>的縮圖',
    'show <span class="key">larger</span> thumbnails',
    '大きなサムネイルを表示する',
    '<span class="key">더 큰</span> 썸네일 표시',
    'Показывать <span class="key">большие</span> миниатюры',
  ],
  _横图占用二倍宽度: [
    '横图占用二倍宽度',
    '橫圖佔用二倍寬度',
    'Horizontal image takes up double the width',
    '水平方向の画像は幅の2倍を占めます',
    '가로 그림은 두 배의 너비를 차지',
    'Горизонтальное изображение занимает вдвое большую ширину',
  ],
  _该功能默认开启: [
    '这个功能默认启用。',
    '這個功能預設啟用。',
    'This feature is enabled by default.',
    'この機能はデフォルトで有効になっています。',
    '이 기능은 기본적으로 활성화됩니다.',
    'Эта функция включена по умолчанию.',
  ],
  _默认未启用: [
    '默认未启用。',
    '預設未啟用。',
    'It is disabled by default.',
    'デフォルトでは有効になっていません。',
    '기본값이 비활성화되어 있습니다.',
    'По умолчанию не работает.',
  ],
  _你可以在更多选项卡的xx分类里找到它: [
    '你可以在“更多”选项卡 → “{}”分类里找到它。（需要先启用“显示高级设置”）',
    '你可以在“更多”選項卡 → “{}”分類裡找到它。（需要先啟用“顯示進階設定”）',
    'You can find it in the "More" tab → "{}" category. ("Show advanced settings" needs to be enabled first)',
    '[もっと]タブ→[{}]カテゴリにあります。（最初に「詳細設定を表示」を有効にする必要があります）',
    '"더보기" 탭 → "{}" 카테고리에서 찾을 수 있습니다. ("고급 설정 보기"를 먼저 활성화해야 합니다.)',
    'Вы можете найти его в разделе "Еще". вкладка → "{}" категория. ("Показать расширенные настройки" необходимо сначала включить)',
  ],
  _你可以在xx选项卡里找到它: [
    '你可以在“{}”选项卡里找到它。（需要先启用“显示高级设置”）',
    '你可以在“{}”選項卡裡找到它。（需要先啟用“顯示進階設定”）',
    'You can find it in the "{}" tab. ("Show advanced settings" needs to be enabled first)',
    '「{}」タブにあります。（最初に「詳細設定を表示」を有効にする必要があります）',
    '"{}" 탭에서 찾을 수 있습니다. ("고급 설정 보기"를 먼저 활성화해야 합니다.)',
    'Вы можете найти его на вкладке "{}". ("Показать расширенные настройки" необходимо сначала включить)',
  ],
  _使用鼠标滚轮切换作品里的图片: [
    '使用鼠标滚轮切换多图作品里的图片',
    '使用滑鼠滾輪切換多圖作品裡的圖片',
    'Use the mouse wheel to switch images in multi-image works',
    'マウスホイールを使用して、マルチイメージ作品のイメージを切り替えます',
    '마우스 휠을 사용하여 여러 이미지 작품에서 이미지 전환',
    'Используйте колесико мыши для переключения изображений в работах с несколькими изображениями',
  ],
  _这可能会阻止页面滚动: [
    '这可能会阻止页面滚动',
    '這可能會阻止頁面滾動',
    'This might stop the page from scrolling',
    'ページのスクロールを妨げる可能性があります',
    '이 기능은 페이지를 스크롤하지 못하게 할 수 있습니다.',
    'Это может остановить прокрутку страницы',
  ],
  _动图转换失败的提示: [
    '动图转换失败，id：{}',
    '動圖轉換失敗，id：{}',
    'Ugoira(animation) conversion failed, id: {}',
    'うごイラの変換に失敗しました、id：{}',
    '움직이는 일러스트 변환에 실패했습니다, ID: {}',
    'Не удалось преобразовать Ugoira(анимацию), идентификатор: {}',
  ],
  _动图不能转换为WEBM视频的提示: [
    '作品 ID {} 不能转换为 WEBM 视频，因为它的某一帧延迟大于 32767 毫秒。下载器会把它转换为 GIF 图像。',
    '作品 ID {} 不能轉換為 WEBM 影片，因為它的某一幀延遲大於 32767 毫秒。下載器會把它轉換為 GIF 影象。',
    'Work ID {} cannot be converted to WEBM video because it has a frame duration greater than 32767 ms. The downloader will convert it into a GIF image.',
    'ワークid {} は、32767ミリ秒以上のフレーム長を持つため、webm動画に変換できません。ダウンローダはそれをgif画像に変換します。',
    '작업 ID {}의 프레임 지속 시간이 32767 ms보다 크기 때문에 WEBM 비디오로 변환할 수 없습니다.다운로더가 GIF 이미지로 변환해 줍니다.',
    'Рабочий ID {} не может быть преобразован в WEBM видео, потому что он имеет длительность кадров более 32767 мс. Загрузчик преобразует его в изображение GIF.',
  ],
  _作品id无法下载带状态码: [
    '{} 无法下载，状态码：{}',
    '{} 無法下載，狀態碼：{}',
    '{} failed to download, status code: {}',
    '{} ダウンロードに失敗しました、ステータスコード：{}',
    '{} 다운로드할 수 없습니다, 상태 코드: {}',
    '{} не удалось загрузить, код состояния: {}',
  ],
  _作品总数为0: [
    '作品总数为 0，Pixiv 可能拒绝了此次抓取。请稍后重试。',
    '作品總數為 0，Pixiv 可能拒絕了此次抓取。請稍後重試。',
    'The total number of works is 0, Pixiv may have refused this crawl. Please try again later.',
    '作品の総数は 0 です。 Pixivがこのクロールを拒否した可能性があります。 後でもう一度やり直してください。',
    '총 작품 수가 0개입니다, Pixiv가 이번 긁어오기를 거부한 것으로 보입니다. 잠시 후에 다시 시도해주세요.',
    'Общее количество работ равно 0, возможно, Pixiv блокирует сканирование. Пожалуйста, повторите попытку позже.',
  ],
  _快捷键AltP: [
    '快捷键 Alt + P',
    '快捷鍵 Alt + P',
    'Hot key: Alt + P',
    'ホットキー Alt + P',
    '단축키: Alt + P',
    'Горячая клавиша: Alt + P',
  ],
  _优化预览作品功能: [
    '优化“预览作品”功能',
    '最佳化“預覽作品”功能',
    'Optimize the "Preview Works" function',
    '「作品のプレビュー」機能を最適化する',
    '"작품 미리보기" 기능 최적화',
    'Оптимизация "Предварительного просмотра работ" функция',
  ],
  _设定资料: [
    '设定资料',
    '設定資料',
    'Reference materials',
    '設定資料',
    '설정 자료',
    'Справочные материалы',
  ],
  _年龄限制: [
    '<span class="key">年龄</span>限制',
    '<span class="key">年齡</span>限制',
    '<span class="key">Age</span> limit',
    '<span class="key">年齢</span>制限',
    '<span class="key">연령</span> 제한',
    '<span class="key">Возраст</span> ограничение',
  ],
  _收藏状态: [
    '<span class="key">收藏</span>状态',
    '<span class="key">收藏</span>狀態',
    '<span class="key">Bookmark</span> status',
    '<span class="key">ブックマーク</span>ステータス',
    '<span class="key">북마크</span> 상태',
    'Статус <span class="key">закладки</span> ',
  ],
  _图片色彩: [
    '图片<span class="key">色彩</span>',
    '圖片<span class="key">色彩</span>',
    'Image <span class="key">color</span>',
    '画像の<span class="key">色</span>',
    '이미지 <span class="key">색채</span>',
    '<span class="key">Цвет</span> изображения',
  ],
  _图片数量: [
    '图片<span class="key">数量</span>',
    '圖片<span class="key">數量</span>',
    '<span class="key">Number</span> of images',
    '画像の<span class="key">数</span>',
    '이미지 <span class="key">수</span>',
    '<span class="key">Количество</span> изображений',
  ],
  _不抓取多图作品的最后一张图片: [
    '不抓取多图作品的<span class="key">最后一张</span>图片',
    '不抓取多圖作品的<span class="key">最後一張</span>圖片',
    'Do not crawl the <span class="key">last image</span> of multi-image works',
    'マルチ画像作品の<span class="key">最後の画像</span>をつかまないでください',
    '여러 이미지의 <span class="key">마지막 이미지</span> 긁어오지 않기',
    'Не сканировать по <span class="key">последние изображения</span> в много картинных работах',
  ],
  _下载小说的封面图片: [
    '下载小说的<span class="key">封面</span>图片',
    '下載小說的<span class="key">封面</span>圖片',
    'Download the <span class="key">cover</span> image of the novel',
    '小説の<span class="key">表紙画像</span>をダウンロード',
    '소설 <span class="key">커버</span> 이미지 다운로드',
    'Скачать изображение <span class="key">обложки</span> новеллы',
  ],
  _预览动图: [
    '<span class="key">预览</span>动图',
    '<span class="key">預覽</span>動圖',
    '<span class="key">Preview</span> Ugoira',
    'うごイラのプレビュー',
    '움직이는 일러스트 <span class="key">미리보기</span>',
    '<span class="key">Превью</span> Ugoira(анимации)',
  ],
  _过度访问警告警告: [
    '下载器检测到你可能收到了 pixiv 的警告消息，这通常是因为过度下载导致的。<br><strong>当你再次被警告时，你会被 Pixiv 封号。</strong><br>我建议你减少下载数量，或者使用新的账号进行下载。',
    '下載器檢測到你可能收到了 pixiv 的警告訊息，這通常是因為過度下載導致的。<br><strong>當你再次被警告時，你會被 Pixiv 封號。</strong><br>我建議你減少下載數量，或者使用新的賬號進行下載。',
    'The downloader has detected that you may have received a warning message from pixiv, usually due to excessive downloads.<br><strong>When you are warned again, you will be banned from Pixiv. </strong><br>I suggest you reduce your downloads, or use a new account to download.',
    'ダウンロードが多すぎるため、pixivから警告メッセージが届いた可能性があることをダウンローダーが検出しました。<br><strong>再度警告を受けた場合、Pixivから追放されます。 </strong><br>ダウンロード数を減らすか、新しいアカウントを使用してダウンロードすることをお勧めします。',
    '다운로더는 일반적으로 과도한 다운로드로 인해 pixiv에서 경고 메시지를 수신했을 수 있음을 감지했습니다.<br><strong>다시 경고를 받으면 Pixiv에서 차단됩니다. </strong><br>다운로드를 줄이거나 새 계정을 사용하여 다운로드하는 것이 좋습니다.',
    'Программа загрузки обнаружила, что вы могли получить предупреждающее сообщение от pixiv, обычно из-за чрезмерной загрузки.<br><strong>Когда вы снова получите предупреждение, вы будете заблокированы в Pixiv. </strong><br>Я предлагаю вам сократить количество загрузок или использовать новую учетную запись для загрузки.',
  ],
  _下载小说里的内嵌图片: [
    '下载小说里的<span class="key">内嵌</span>图片',
    '下載小說裡的<span class="key">內嵌</span>圖片',
    'Download <span class="key">embedded</span> images in novels',
    '小説に埋め込まれた画像をダウンロードする',
    '소설에서 <span class="key">인라인</span> 이미지 다운로드',
    'Загрузка <span class="key">вложенных</span> изображений в новеллах',
  ],
  _其他优化: [
    '其他优化',
    '其他最佳化',
    'Other optimizations',
    'その他の最適化',
    '기타 최적화',
    'Другие оптимизации',
  ],
  _隐藏浏览器底部的下载栏: [
    '隐藏浏览器底部的<span class="key">下载栏</span>',
    '隱藏瀏覽器底部的<span class="key">下載欄</span>',
    `Hide the <span class="key">download bar</span> at the bottom of the browser`,
    'ブラウザの下部にあるダウンロードバーを非表示にします',
    '브라우저 하단의 <span class="key">다운로드 바</span> 숨기기',
    `Скрыть <span class="key">панель загрузки</span> в нижней части браузера`,
  ],
  _没有可用的抓取结果: [
    '没有可用的抓取结果',
    '沒有可用的抓取結果',
    'No crawl results available',
    'クロール結果がありません',
    '사용 가능한 크롤링 결과가 없습니다.',
    'Результаты сканирования недоступны',
  ],
  _预览作品时按快捷键可以下载这个作品: [
    '预览作品时，按快捷键 <span class="blue">D</span> 可以下载这个作品。<br>按快捷键 <span class="blue">C</span> 仅下载当前显示的这张图片。',
    '預覽作品時，按快捷鍵 <span class="blue">D</span> 可以下載這個作品。<br>按快捷鍵 <span class="blue">C</span> 僅下載當前顯示的這張圖片。',
    'When previewing a work, press the shortcut key <span class="blue">D</span> to download the work.<br>Press the shortcut key <span class="blue">C</span> to download only the currently displayed image.',
    '作品をプレビューしているときに、ショートカット キー <span class="blue">D</span> を押すと、作品をダウンロードできます。<br>ショートカット キー <span class="blue">C</span> を押して、現在表示されている画像のみをダウンロードします。',
    '이미지를 미리 보는 동안 바로 가기 <span class="blue">D</span>를 눌러 다운로드하세요.<br>현재 표시된 이미지만 다운로드하려면 단축키 <span class="blue">C</span>를 누르십시오.',
    'При предварительном просмотре произведения нажмите клавишу <span class="blue">D</span>, чтобы загрузить произведение.<br>Нажмите клавишу быстрого доступа <span class="blue">C</span>, чтобы загрузить только отображаемое в данный момент изображение.',
  ],
  _预览作品时按快捷键C仅下载当前图片: [
    '预览作品时，按快捷键 <span class="blue">C</span> 仅下载当前显示的这张图片。',
    '預覽作品時，按快捷鍵 <span class="blue">C</span> 僅下載當前顯示的這張圖片。',
    'When previewing a work, press the shortcut key <span class="blue">C</span> to download only the currently displayed image.',
    '作品のプレビュー中に、ショートカット キー <span class="blue">C</span> を押すと、現在表示されている画像のみをダウンロードできます。',
    '작품 미리보기 시 단축키 <span class="blue">C</span>를 누르면 현재 표시된 이미지만 다운로드 됩니다.',
    'При предварительном просмотре работы нажмите клавишу быстрого доступа <span class="blue">C</span>, чтобы загрузить только отображаемое в данный момент изображение.',
  ],
  _定时抓取: [
    '定时抓取',
    '定時抓取',
    'Timed crawl',
    '時限クロール',
    '시간 제한 크롤링',
    'Сканирование по таймеру',
  ],
  _定时抓取说明: [
    '每隔一定时间，自动开始抓取和下载。',
    '每隔一定時間，自動開始抓取和下載。',
    'Automatically start crawling and downloading at regular intervals.',
    '定期的にクロールとダウンロードを自動的に開始します。',
    '정기적으로 자동으로 크롤링 및 다운로드를 시작합니다.',
    'Автоматически запускать сканирование и загрузку через регулярные промежутки времени',
  ],
  _定时抓取已启动的提示: [
    '定时抓取已启动，间隔时间：{} 分钟。<br>如果你想修改间隔时间，可以在“更多”选项卡里修改设置：定时抓取的间隔时间。',
    '定時抓取已啟動，間隔時間：{} 分鐘。<br>如果你想修改間隔時間，可以在“更多”選項卡里修改設定：定時抓取的間隔時間。',
    'Timed crawl started, interval: {} minutes.<br>If you want to modify the interval time, you can modify the settings in the "More" tab: The interval time of timed crawl.',
    '時限クロールが開始されました。間隔: {} 分。<br>間隔時間を変更したい場合は、[詳細] タブの設定 (時間指定クロールの間隔時間) を変更できます。',
    '시간 제한 크롤링이 시작되었습니다. 간격: {}분. <br>간격 시간을 수정하려면 "자세히" 탭에서 예약된 크롤링 간격 설정을 수정할 수 있습니다.',
    'Таймер сканирования запущен, интервал: {} минут.<br>Если вы хотите изменить интервал времени, вы можете изменить настройки на вкладке «Дополнительно»: Интервальное время сканирования с таймером.',
  ],
  _定时抓取的推荐用法: [
    '推荐用法：增量抓取新作品。例如在关注的用户的新作品页面里，设置抓取页数为 2，然后启动定时抓取。这样下载器可以自动下载新作品。<br>建议启用“不下载重复文件”功能，以避免下载重复的文件。',
    '推薦用法：增量抓取新作品。例如在關注的使用者的新作品頁面裡，設定抓取頁數為 2，然後啟動定時抓取。這樣下載器可以自動下載新作品。<br>建議啟用“不下載重複檔案”功能，以避免下載重複的檔案。',
    'Recommended usage: Fetch new work incrementally. For example, in the new work page of the user you follow, set the number of crawled pages to 2, and then start timing crawling. This way the downloader can automatically download new works.<br>It is recommended to enable the "Do not download duplicate files" feature to avoid downloading duplicate files.',
    '推奨される使用法: 新しい作業を段階的にフェッチします。 たとえば、フォローしているユーザーの新しい作品ページで、クロールされたページの数を 2 に設定し、クロールのタイミングを開始します。 このようにして、ダウンローダーは新しい作品を自動的にダウンロードできます。<br>重複ファイルのダウンロードを避けるために、「重複ファイルをダウンロードしない」機能を有効にすることをお勧めします。',
    '권장 사용법: 새 작업을 점진적으로 가져옵니다. 예를 들어 팔로우하는 사용자의 새 작업 페이지에서 크롤링 페이지 수를 2로 설정한 다음 타이밍 크롤링을 시작합니다. 이렇게 하면 다운로더가 자동으로 새 작품을 다운로드할 수 있습니다.<br>중복 파일 다운로드를 방지하기 위해 "중복 파일 다운로드 금지" 기능을 활성화하는 것이 좋습니다.',
    'Рекомендуемое использование: получать новую работу постепенно. Например, на новой рабочей странице пользователя, за которым вы следите, установите количество просканированных страниц равным 2, а затем запустите сканирование по времени. Таким образом, загрузчик может автоматически загружать новые работы.<br>Рекомендуется включить функцию "Не загружать дубликаты файлов", чтобы избежать загрузки дубликатов файлов.',
  ],
  _定时抓取已启动的提示2: [
    '在定时抓取时，将这个标签页静置即可。不要改变这个标签页的 URL，否则抓取结果可能不符合预期。<br><br>如果这个扩展程序自动更新了，那么这个页面将不能正常下载文件（需要刷新页面来恢复正常）。 如果你想长期执行定时抓取任务，建议安装下载器的离线版本，以免因为自动更新而导致问题。<br>你可以在这里下载离线安装包：<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
    '在定時抓取時，將這個標籤頁靜置即可。不要改變這個標籤頁的 URL，否則抓取結果可能不符合預期。<br><br>如果這個擴充套件程式自動更新了，那麼這個頁面將不能正常下載檔案（需要重新整理頁面來恢復正常）。 如果你想長期執行定時抓取任務，建議安裝下載器的離線版本，以免因為自動更新而導致問題。<br>你可以在這裡下載離線安裝包：<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
    'During timed crawling, just leave this tab alone. Do not change the URL of this tab, or the crawl results may not be as expected.<br><br>If the extension is automatically updated, the page will not be able to download files normally (refresh the page to restore normal). If you want to perform scheduled crawling tasks for a long time, it is recommended to install the offline version of the downloader to avoid problems caused by automatic updates.<br>You can download the offline installation package here: <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
    '時限クロール中は、このタブをそのままにしておきます。 このタブの URL は変更しないでください。変更すると、クロール結果が期待どおりにならない可能性があります。<br><br>拡張機能が自動的に更新されると、ページはファイルを正常にダウンロードできなくなります (ページを更新して正常に戻します)。 スケジュールされたクロール タスクを長時間実行する場合は、自動更新による問題を回避するために、ダウンローダのオフライン バージョンをインストールすることをお勧めします。<br>オフライン インストール パッケージは、次の場所からダウンロードできます。<a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
    '시간 제한 크롤링 중에는 이 탭을 그대로 두십시오. 이 탭의 URL을 변경하지 마십시오. 그렇지 않으면 크롤링 결과가 예상과 다를 수 있습니다.<br><br>확장자가 자동으로 업데이트되면 페이지에서 파일을 정상적으로 다운로드할 수 없습니다(페이지를 새로고침하여 정상으로 복원). 예약된 크롤링 작업을 장기간 수행하려면 자동 업데이트로 인한 문제를 방지하기 위해 다운로더의 오프라인 버전을 설치하는 것이 좋습니다.<br>여기에서 오프라인 설치 패키지를 다운로드할 수 있습니다. <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Releases page</a>',
    'Во время сканирования по времени просто оставьте эту вкладку в покое. Не меняйте URL-адрес этой вкладки, иначе результаты сканирования могут отличаться от ожидаемых.<br><br>Если расширение автоматически обновляется, страница не сможет загружать файлы в обычном режиме (обновите страницу, чтобы восстановить нормальный режим). Если вы хотите выполнять запланированные задачи обхода в течение длительного времени, рекомендуется установить автономную версию загрузчика, чтобы избежать проблем, вызванных автоматическими обновлениями.<br>Вы можете скачать автономный установочный пакет здесь: <a href="https://github.com/xuejianxianzun/PixivBatchDownloader/releases" target="_blank">Страница релизов</a>',
  ],
  _定时抓取的间隔时间: [
    '<span class="key">定时</span>抓取的间隔时间',
    '<span class="key">定時</span>抓取的間隔時間',
    'The interval time of <span class="key">timed crawl</span>',
    '<span class="key">時間指定</span>クロールの間隔時間',
    '정기 크롤링 간격 시간',
    'Интервальное время сканирования с таймером',
  ],
  _定时抓取的间隔时间2: [
    '定时抓取的间隔时间',
    '定時抓取的間隔時間',
    'The interval time of timed crawl',
    '時間指定クロールの間隔時間',
    '정기 크롤링 간격 시간',
    'Интервальное время сканирования с таймером',
  ],
  _分钟: ['分钟', '分鐘', 'Minute', '分', '분', 'Минут'],
  _定时抓取的时间超过最大值: [
    '定时抓取的间隔时间超过最大值：',
    '定時抓取的間隔時間超過最大值：',
    'The interval of timed crawl exceeds the maximum value: ',
    '時間指定クロールの間隔が最大値を超えています: ',
    '시간 지정 크롤링 간격이 최대값을 초과합니다: ',
    'Интервал сканирования по таймеру превышает максимальное значение: ',
  ],
  _定时抓取的时间最小值: [
    '定时抓取的间隔时间最小值为 1 分钟。',
    '定時抓取的間隔時間最小值為 1 分鐘。',
    'The minimum interval for timed crawls is 1 minute.',
    '時間指定クロールの最小間隔は 1 分です。',
    '시간 지정 크롤링의 최소 간격은 1분입니다.',
    'Минимальный интервал для сканирования по таймеру составляет 1 минуту.',
  ],
  _取消定时抓取: [
    '取消定时抓取',
    '取消定時抓取',
    'Cancel timed crawl',
    '時間指定クロールをキャンセル',
    '시간 지정 크롤링 취소',
    'Отменить сканирование по таймеру',
  ],
  _已取消定时抓取: [
    '已取消定时抓取',
    '已取消定時抓取',
    'Timed crawl canceled',
    '時間指定クロールがキャンセルされました',
    '예약된 크롤링이 취소되었습니다.',
    'Сканирование по таймеру отменено',
  ],
  _因为URL变化取消定时抓取任务: [
    '因为 URL 变化，定时抓取任务已被取消。',
    '因為 URL 變化，定時抓取任務已被取消。',
    'The timed crawl task has been canceled due to URL changes.',
    'URL が変更されたため、時間指定クロール タスクがキャンセルされました。',
    'URL 변경으로 인해 시간이 지정된 크롤링 작업이 취소되었습니다.',
    'Задание на сканирование по времени было отменено из-за изменений URL.',
  ],
  _开始定时抓取: [
    '开始定时抓取',
    '開始定時抓取',
    'Start timed crawling',
    '時間指定クロールを開始する',
    '시간 지정 크롤링 시작',
    'Начать сканирование по таймеру',
  ],
  _等待下一次定时抓取: [
    '等待下一次定时抓取',
    '等待下一次定時抓取',
    'Wait for the next timed crawl',
    '次回の時限クロールを待つ',
    '다음 시간 크롤링을 기다립니다.',
    'Подождите следующего сканирования с таймером',
  ],
  _当前时间: [
    '当前时间：',
    '當前時間：',
    'Current time: ',
    '現在の時刻：',
    '현재 시간: ',
    'Текущее время: ',
  ],
  _仅在部分页面中可用: [
    '仅在部分页面中可用。',
    '僅在部分頁面中可用。',
    'Only available on some pages.',
    '一部のページのみ利用可能です。',
    '일부 페이지에서만 사용할 수 있습니다.',
    'Доступно только на некоторых страницах',
  ],
  _发生错误原因: [
    '发生错误，原因：',
    '發生錯誤，原因：',
    'An error occurred due to:',
    '次の理由でエラーが発生しました:',
    '다음으로 인해 오류가 발생했습니다.',
    'Произошла ошибка по причине:',
  ],
  _扩展程序已更新: [
    '扩展程序已更新。',
    '擴充套件程式已更新。',
    'The extension has been updated.',
    '拡張機能が更新されました。',
    '확장이 업데이트되었습니다.',
    'Расширение было обновлено.',
  ],
  _未知错误: [
    '未知错误。',
    '未知錯誤。',
    'unknown mistake.',
    '未知の間違い。',
    '알 수 없는 실수.',
    'неизвестная ошибка',
  ],
  _请刷新页面: [
    '请刷新页面。',
    '請重新整理頁面。',
    'Please refresh the page.',
    'ページを更新してください。',
    '페이지를 새로고침하세요.',
    'Пожалуйста, обновите страницу.',
  ],
  _减慢抓取速度: [
    '<span class="key">减慢</span>抓取速度',
    '<span class="key">減慢</span>抓取速度',
    '<span class="key">Slow down</span> crawl',
    'クロールを<span class="key">遅くする</span>',
    '<span class="key">천천히</span> 크롤링',
    '<span class="key">Замедлить</span> сканирование',
  ],
  _减慢抓取速度的说明: [
    '减慢抓取速度可以避免在抓取时被 Pixiv 临时限制。但是会增加抓取时间。',
    '減慢抓取速度可以避免在抓取時被 Pixiv 臨時限制。但是會增加抓取時間。',
    'Slow down the crawl to avoid being temporarily restricted by Pixiv while crawling. But this will increase the crawl time.',
    'クロール中にPixivによって一時的に制限されないように、クロールを遅くします。 ただし、これによりクロール時間が長くなります。',
    '크롤링하는 동안 Pixiv에 의해 일시적으로 제한되지 않도록 크롤링 속도를 늦춥니다. 그러나 이것은 크롤링 시간을 증가시킵니다.',
    'Замедлить сканирование, чтобы избежать временного ограничения Pixiv во время сканирования. Это увеличит время сканирования.',
  ],
  _作品数量: [
    '作品数量',
    '作品數量',
    'Number of works',
    '作品数',
    '작품 수',
    'Количество работ',
  ],
  _当作品数量大于: [
    '当作品数量 >',
    '當作品數量 >',
    'When the number of works >',
    '作品数 >',
    '작품 수 >',
    'При количестве работ >',
  ],
  _慢速抓取: [
    '慢速抓取，以避免触发 429 限制',
    '慢速抓取，以避免觸發 429 限制',
    'Crawl slowly to avoid triggering 429 throttling',
    '429 スロットリングのトリガーを避けるためにゆっくりとクロールします',
    '429 스로틀링 트리거를 방지하기 위해 천천히 크롤링',
    'Медленно сканируйте, чтобы не спровоцировать 429 троттлинг.',
  ],
  _慢速执行以避免引起429错误: [
    '慢速执行，以避免引起 429 错误',
    '慢速執行，以避免引起 429 錯誤',
    'Execute slowly to avoid causing 429 errors',
    '429 エラーの発生を避けるためにゆっくり実行してください',
    '429 오류가 발생하지 않도록 천천히 실행하십시오.',
    'Выполняйте медленно, чтобы избежать ошибок 429',
  ],
  _点击收藏按钮时下载作品: [
    '点击<span class="key">收藏</span>按钮时下载作品',
    '點選<span class="key">收藏</span>按鈕時下載作品',
    'Download a work when you click the <span class="key">bookmark</span> button',
    '<span class="key">ブックマーク</span>ボタンをクリックすると作品をダウンロード',
    '<span class="key">북마크</span> 버튼 클릭 시 작품 다운로드',
    'Загрузка произведения при нажатии кнопки <span class="key">закладка</span>',
  ],
  _点击点赞按钮时下载作品: [
    '点击<span class="key">点赞</span>按钮时下载作品',
    '點選<span class="key">點贊</span>按鈕時下載作品',
    'Download a work when you click the <span class="key">like</span> button',
    '<span class="key">いいね</span> ボタンをクリックすると作品がダウンロードされます',
    '<span class="key">좋아요</span> 버튼 클릭 시 작품 다운로드',
    'Загрузка произведения при нажатии на кнопку <span class="key">лайк</span>',
  ],
  _优化性能和用户体验: [
    '优化性能和用户体验。',
    '最佳化效能和使用者體驗。',
    'Optimize performance and user experience.',
    'パフォーマンスとユーザー エクスペリエンスを最適化します。',
    '성능과 사용자 경험을 최적화합니다.',
    'Оптимизация производительности и пользовательского опыта',
  ],
  _修复bug: [
    '修复 bug',
    '修復 bug',
    'fix bugs',
    'バグを修正',
    '버그 수정',
    'Баг фикс',
  ],
  _修复已知问题: [
    '修复已知问题',
    '修復已知問題',
    'fix known issues',
    '既知の問題を修正する',
    '알려진 문제 수정',
    'исправить известные проблемы',
  ],
  _不支持的浏览器: [
    '你的浏览器不能正常使用这个扩展程序，主要原因可能是浏览器内核版本太低，或者存在兼容性问题。<br>建议您更换成最新版本的 Chrome 或 Edge 浏览器。',
    '你的瀏覽器不能正常使用這個擴充套件程式，主要原因可能是瀏覽器核心版本太低，或者存在相容性問題。<br>建議您更換成最新版本的 Chrome 或 Edge 瀏覽器。',
    'Your browser cannot use this extension properly. The main reason may be that the browser kernel version is too low, or there is a compatibility problem. <br>We recommend that you switch to the latest version of Chrome or Edge.',
    'お使いのブラウザでは、この拡張機能を正しく使用できません。 主な理由としては、ブラウザのカーネル バージョンが低すぎるか、互換性の問題がある可能性があります。 <br>最新バージョンの Chrome または Edge に切り替えることをお勧めします。',
    '브라우저에서 이 확장 프로그램을 제대로 사용할 수 없습니다. 주된 이유는 브라우저 커널 버전이 너무 낮거나 호환성 문제가 있기 때문일 수 있습니다. <br>최신 버전의 Chrome 또는 Edge로 전환하는 것이 좋습니다.',
    'Ваш браузер не может правильно использовать это расширение. Основной причиной может быть слишком низкая версия ядра браузера или проблема совместимости. <br>Мы рекомендуем вам перейти на последнюю версию Chrome или Edge.',
  ],
  _日期时间格式错误: [
    '日期时间格式错误',
    '日期時間格式錯誤',
    'wrong datetime format',
    '間違った日時形式',
    '잘못된 날짜/시간 형식',
    'неправильный формат даты',
  ],
  _添加了对此页面类型的支持: [
    '添加了对此页面类型的支持：',
    '添加了对此页面类型的支持：',
    'Added support for this page type:',
    '次のページ タイプのサポートが追加されました：',
    '이 페이지 유형에 대한 지원이 추가되었습니다：',
    'Добавлена поддержка этого типа страницы:',
  ],
  _仅可由链接浏览: [
    '仅可由链接浏览',
    '僅可由連結瀏覽',
    'URL restricted',
    'URL限定公開',
    'URL 한정 공개',
    'URL ограничен',
  ],
  _添加了俄语翻译: [
    '添加了俄语翻译',
    '添加了俄語翻譯',
    'Added Russian translation',
    'ロシア語の翻訳を追加',
    '러시아어 번역 추가',
    'Добавлен русский перевод',
  ],
  _移除本页面中所有作品的标签: [
    '移除本页面中所有作品的标签',
    '移除本頁面中所有作品的標籤',
    'Remove tags from all works on this page',
    'このページのすべての作品からタグを削除します',
    '이 페이지의 모든 작품에서 태그 제거',
    'Удалить теги со всех работ на этой странице',
  ],
  _它们会变成未分类状态: [
    '它们会变成未分类状态',
    '它們會變成未分類狀態',
    'They become uncategorized',
    'それらは未分類になります',
    '분류되지 않습니다',
    'Они становятся некатегоризированными',
  ],
  _取消收藏本页面的所有作品: [
    '取消收藏本页面中的所有作品',
    '取消收藏本頁面中的所有作品',
    'Unbookmark all works on this page',
    'このページのすべての作品のブックマークを解除',
    '이 페이지의 모든 작품에 대한 북마크 해제',
    'Удалить из избранного все работы на этой странице',
  ],
  _取消收藏所有已被删除的作品: [
    '取消收藏所有已被删除的作品',
    '取消收藏所有已被刪除的作品',
    'Unbookmark all deleted works',
    '削除した作品をすべてブックマーク解除する',
    '삭제된 모든 작품 북마크 해제',
    'Снять закладку со всех удаленных работ',
  ],
  _取消收藏所有已被删除的作品的使用说明: [
    '在你的收藏页面里，切换到下载器的“更多”标签页可以看到该功能按钮。',
    '在你的收藏頁面裡，切換到下載器的“更多”標籤頁可以看到該功能按鈕。',
    'In your bookmark page, switch to the "More" tab of the downloader to see this function button.',
    'ブックマーク ページで、ダウンローダーの「その他」タブに切り替えると、この機能ボタンが表示されます。',
    '북마크 페이지에서 다운로더의 "더보기" 탭으로 전환하면 이 기능 버튼을 볼 수 있습니다.',
    'На странице закладок перейдите на вкладку «Дополнительно» загрузчика, чтобы увидеть эту функциональную кнопку.',
  ],
  _取消收藏作品: [
    '取消收藏作品',
    '取消收藏作品',
    'Unbookmark works',
    '作品のブックマークを解除',
    '작품 북마크 해제',
    'Снять закладку с работ',
  ],
  _收藏页面里的按钮: [
    '当你在自己的收藏页面时，可以在“更多”选项卡里看到这个按钮。',
    '當你在自己的收藏頁面時，可以在“更多”選項卡里看到這個按鈕。',
    `You can see this button in the "More" tab when you're on your bookmarks page.`,
    'このボタンは、ブックマーク ページの [もっと] タブに表示されます。',
    '북마크 페이지에 있을 때 "더보기" 탭에서 이 버튼을 볼 수 있습니다.',
    'Вы можете увидеть эту кнопку на вкладке «Больше», когда находитесь на странице закладок.',
  ],
  _收藏任务尚未完成请等待: [
    '收藏作品的任务尚未全部完成，请等待',
    '收藏作品的任務尚未全部完成，請等待',
    'The task of bookmarking works has not been completed yet, please wait.',
    '作品のブックマーク作業がまだ完了していませんので、しばらくお待ちください',
    '작품을 북마크하는 작업이 아직 완료되지 않았습니다. 잠시만 기다려 주세요',
    'Задание работ по закладке еще не выполнено, пожалуйста, подождите',
  ],
  _收藏作品完毕: [
    '收藏作品完毕',
    '收藏作品完畢',
    'Bookmark works finished',
    'ブックマーク作業終了',
    '북마크 작업 완료',
    'Работа над закладками завершена',
  ],
  _添加收藏失败: [
    '添加收藏失败',
    '新增收藏失敗',
    'Failed to add bookmark',
    'ブックマークを追加できませんでした',
    '북마크 추가 실패',
    'Не удалось добавить закладку',
  ],
  _下载器会在几分钟后重试: [
    '下载器会在几分钟后重试。',
    '下載器會在幾分鐘後重試。',
    'The downloader will try again in a few minutes.',
    'ダウンローダーは数分後に再試行します。',
    '다운로더는 몇 분 후에 다시 시도합니다.',
    'Загрузчик повторит попытку через несколько минут.',
  ],
  _重试收藏: [
    '重试收藏',
    '重試收藏',
    'Retry bookmark',
    'ブックマークを再試行',
    '북마크 다시 시도',
    'Повторить закладку',
  ],
  _剩余xx个: [
    '剩余 {} 个。',
    '剩餘 {} 個。',
    '{} remaining.',
    '{} 残り。',
    '{} 남음.',
    '{} осталось.',
  ],
  _重试收藏成功: [
    '重试收藏成功。',
    '重試收藏成功。',
    'Retry bookmark successfully.',
    'ブックマークを再試行します。',
    '북마크를 다시 시도하십시오.',
    'Повторите попытку закладки.',
  ],
  _出现错误请稍后重试: [
    '出现错误，请稍后重试。',
    '出現錯誤，請稍後重試。',
    'An error occurred, please try again later.',
    'エラーが発生しました。しばらくしてからもう一度お試しください。',
    '오류가 발생했습니다. 잠시 후 다시 시도 해주세요.',
    'Произошла ошибка. Пожалуйста, повторите попытку позже.',
  ],
  _请稍后重试: [
    '请稍后重试。',
    '請稍後重試。',
    'Please try again later.',
    '後でもう一度やり直してください。',
    '잠시 후에 다시 시도해주세요.',
    'Пожалуйста, повторите попытку позже.',
  ],
  _确定要离开吗: [
    '确定要离开吗？',
    '確定要離開嗎？',
    'Are you sure you want to leave?',
    '退会してもよろしいですか？',
    '떠나시겠습니까?',
    'Вы уверены, что хотите оставить?',
  ],
  _yandex浏览器的警告: [
    `如果你在 Yandex 浏览器（Android）上使用 Powerful Pixiv Downloader，请换成 Kiwi 浏览器。<br>
    因为下载器在最近将会升级到 Manifest version 3，但是 Yandex 浏览器不支持  Manifest version 3， 所以它不能使用新版本的下载器。`,
    `如果你在 Yandex 瀏覽器（Android）上使用 Powerful Pixiv Downloader，請換成 Kiwi 瀏覽器。<br>
    因為下載器在最近將會升級到 Manifest version 3，但是 Yandex 瀏覽器不支援  Manifest version 3， 所以它不能使用新版本的下載器。`,
    `If you are using Powerful Pixiv Downloader on Yandex browser（Android）, please switch to Kiwi browser. <br>
    Because the downloader will be upgraded to Manifest version 3 in the near future, but Yandex browser does not support Manifest version 3, so it cannot use the new version of the downloader.`,
    `Yandex（Android） ブラウザで強力な Pixiv Downloader を使用している場合は、Kiwi ブラウザに切り替えてください。 <br>
    ダウンローダは近いうちにマニフェスト バージョン 3 にアップグレードされますが、Yandex ブラウザはマニフェスト バージョン 3 をサポートしていないため、新しいバージョンのダウンローダを使用することはできません。`,
    `Yandex（Android） 브라우저에서 강력한 Pixiv Downloader를 사용하는 경우 Kiwi 브라우저로 전환하십시오. <br>
    다운로더는 가까운 시일 내에 Manifest 버전 3으로 업그레이드되지만 Yandex 브라우저는 Manifest 버전 3을 지원하지 않으므로 새 버전의 다운로더를 사용할 수 없습니다.`,
    `Если вы используете Powerful Pixiv Downloader в браузере Yandex（Android）, перейдите на браузер Kiwi. <br>
    Потому что в ближайшее время загрузчик будет обновлен до Манифеста версии 3, но Yandex браузер не поддерживает Манифест версии 3, поэтому он не может использовать новую версию загрузчика.`,
  ],
  _导出日志: [
    '导出<span class="key">日志</span>',
    '匯出<span class="key">日誌</span>',
    'Export <span class="key">log</span>',
    '<span class="key">ログ</span>のエクスポート',
    '내보내기 로그',
    'Экспорт <span class="key">журнала</span>',
  ],
  _导出日志成功: [
    '✓ 导出日志',
    '✓ 匯出日誌',
    '✓ Export log',
    '✓ ログのエクスポート',
    '✓ 내보내기 로그',
    '✓ Экспорт журнала',
  ],
  _导出时机: [
    '导出时机',
    '匯出時機',
    'Export timing',
    'エクスポートのタイミング',
    '내보내기 타이밍',
    'Время экспорта',
  ],
  _日志类型: [
    '日志类型',
    '日誌型別',
    'Log type',
    'ログの種類',
    '로그 유형',
    'Тип журнала',
  ],
  _正常: ['正常', '正常', 'Normal', '普通', '정상', 'Обычный'],
  _错误: ['错误', '錯誤', 'Error', 'エラー', '오류', 'Ошибка'],
  _排除关键字: [
    '排除关键字',
    '排除關鍵字',
    'Exclude keywords',
    'キーワードを除外',
    '키워드 제외',
    'Исключить ключевые слова',
  ],
  _Chrome108版本转换WebM失败的问题: [
    '从 Chrome 108 版本开始，浏览器的一些变化导致下载器转换 WebM 视频失败。<br>现已修复转换功能。',
    '從 Chrome 108 版本開始，瀏覽器的一些變化導致下載器轉換 WebM 影片失敗。<br>現已修復轉換功能。',
    'Starting with Chrome version 108, some changes in the browser caused the downloader to fail to convert WebM videos. <br>The conversion function is now fixed.',
    'Chrome バージョン 108 以降、ブラウザーの一部の変更により、ダウンローダーが WebM ビデオの変換に失敗しました。 <br>変換機能を修正しました。',
    'Chrome 버전 108부터 브라우저의 일부 변경으로 인해 다운로더가 WebM 비디오를 변환하지 못했습니다. <br>변환 기능이 수정되었습니다.',
    'Начиная с Chrome версии 108, некоторые изменения в браузере приводили к тому, что загрузчик не мог конвертировать видео WebM. <br>Функция преобразования теперь исправлена.',
  ],
  _特定用户的多图作品不下载最后几张图片: [
    '特定用户的多图作品不下载<span class="key">最后几张</span>图片',
    '特定使用者的多圖作品不下載<span class="key">最後幾張</span>圖片',
    `Don't download the <span class="key">last few</span> images for specific user's multi-image works`,
    '特定のユーザーのマルチイメージ作品の最後のいくつかのイメージをダウンロードしないでください',
    '특정 사용자의 다중 이미지 작품에 대한 마지막 몇 개의 이미지를 다운로드하지 마십시오.',
    'Не загружайте несколько последних изображений для работы с несколькими изображениями конкретного пользователя.',
  ],
  _不下载最后几张图片: [
    '不下载最后几张图片',
    '不下載最後幾張圖片',
    'Do not download the last few images',
    '最後の数枚の画像をダウンロードしない',
    '마지막 몇 개의 이미지를 다운로드하지 마십시오',
    'Не загружайте последние несколько изображений',
  ],
  _提示0表示不生效: [
    '0 表示不生效',
    '0 表示不生效',
    '0 means no effect',
    '0 は影響なしを意味します',
    '0은 영향이 없음을 의미합니다.',
    '0 означает отсутствие эффекта',
  ],
  _如果作品含有某些标签则对这个作品使用另一种命名规则: [
    '如果作品含有某些<span class="key">特定标签</span>，则对这个作品使用另一种命名规则',
    '如果作品含有某些<span class="key">特定標籤</span>，則對這個作品使用另一種命名規則',
    'Use a different naming rule for the work if it has certain tags',
    '特定のタグがある場合は、作品に別の命名規則を使用する',
    '특정 태그가 있는 경우 작업에 다른 명명 규칙을 사용하십시오.',
    'Используйте другое правило именования для работы, если она имеет определенные теги',
  ],
  _升级到manifest_v3的提示: [
    '下载器已升级到 Manifest V3。<br>如果你在下载时遇到问题，请打开扩展管理页面，重新加载本扩展。',
    '下載器已升級到 Manifest V3。<br>如果你在下載時遇到問題，請開啟擴充套件管理頁面，重新載入本擴充套件。',
    'Downloader has been upgraded to Manifest V3. <br>If you encounter problems when downloading, please open the extension management page and reload this extension.',
    'Downloader が Manifest V3 にアップグレードされました。 <br>ダウンロード中に問題が発生した場合は、拡張機能の管理ページを開いて、この拡張機能をリロードしてください。',
    '다운로더가 Manifest V3로 업그레이드되었습니다. <br>다운로드 시 문제가 발생하면 확장 프로그램 관리 페이지를 열고 이 확장 프로그램을 새로고침하세요.',
    'Загрузчик обновлен до версии Manifest V3. <br>Если у вас возникли проблемы при загрузке, откройте страницу управления расширением и перезагрузите это расширение.',
  ],
  _AI作品: [
    '<span class="key">AI</span> 作品',
    '<span class="key">AI</span> 作品',
    '<span class="key">AI</span> works',
    '<span class="key">AI</span>が働く',
    '<span class="key">AI</span> 작동',
    '<span class="key">ИИ</span> работает',
  ],
  _AI生成: [
    'AI 生成',
    'AI 生成',
    'AI-generated',
    'AI 生成',
    'AI 생성',
    'сгенерированный ИИ',
  ],
  _非AI生成: [
    '非 AI 生成',
    '非 AI 生成',
    'Not AI-generated',
    'AI生成ではない',
    'AI 생성 아님',
    'Не сгенерировано ИИ',
  ],
  _未知: [
    '未知',
    '未知',
    'Unknown',
    '知らない',
    '알려지지 않은',
    'Неизвестный',
  ],
  _AI未知作品的说明: [
    '早期作品没有标记，无法判断',
    '早期作品沒有標記，無法判斷',
    'Early works are not marked and cannot be judged',
    '初期の作品は採点せず、審査不可',
    '초기 작품은 표시되지 않으며 평가할 수 없습니다.',
    'Ранние работы не отмечены и не могут быть оценены',
  ],
  _用户可以选择是否下载AI生成的作品: [
    '用户可以选择是否下载由 AI 生成的作品。',
    '使用者可以選擇是否下載由 AI 生成的作品。',
    'Users can choose whether to download AI-generated works.',
    'ユーザーは、AI によって生成された作品をダウンロードするかどうかを選択できます。',
    '사용자는 AI가 생성한 작품을 다운로드할지 여부를 선택할 수 있습니다.',
    'Пользователи могут выбирать, загружать ли работы, созданные ИИ.',
  ],
  _文件下载顺序: [
    '文件下载<span class="key">顺序</span>',
    '檔案下載<span class="key">順序</span>',
    'File download <span class="key">order</span>',
    'ファイルのダウンロード<span class="key">順序</span>',
    '파일 다운로드 순서',
    'Порядок загрузки файлов',
  ],
  _降序: [
    '降序',
    '降序',
    'Descending',
    '降順',
    '내림차순',
    'в порядке убывания',
  ],
  _升序: [
    '升序',
    '升序',
    'Ascending',
    '昇順',
    '오름차순',
    'возрастающий порядок',
  ],
  _排序依据: [
    '排序依据',
    '排序依據',
    'Sort by',
    'ソート基準',
    '정렬 기준',
    'Сортировать по',
  ],
  _作品ID: [
    '作品 ID',
    '作品 ID',
    'Work ID',
    '作品ID',
    'ID 아이디',
    'РРабочий идентификатор',
  ],
  _收藏时间: [
    '收藏时间',
    '收藏時間',
    'Bookmark time',
    'ブックマーク時間',
    '북마크 시간',
    'время сбора',
  ],
  _收藏数量2: [
    '收藏数量',
    '收藏數量',
    'Number of bookmarks',
    'ブックマークの数',
    '북마크 수',
    'Колличество закладок',
  ],
  _重新显示帮助: [
    '重新显示帮助',
    '重新顯示幫助',
    'Redisplay help',
    'ヘルプを再表示',
    '도움말 다시 표시',
    'Повторно отобразить справку',
  ],
  _自定义标签分隔符号的提示: [
    '现在你可以自定义文件名中使用的标签分隔符号，以替换默认的 <span class="blue">,</span>。',
    '現在你可以自定義檔名中使用的標籤分隔符號，以替換預設的 <span class="blue">,</span>。',
    'You can now customize the tag separator used in filenames to replace the default <span class="blue">,</span>',
    'ファイル名で使用されるタグ区切りをカスタマイズして、デフォルトの <span class="blue">,</span> を置き換えることができるようになりました',
    '이제 파일 이름에 사용되는 태그 구분 기호를 사용자 지정하여 기본 <span class="blue">,</span>',
    'Теперь вы можете настроить разделитель тегов, используемый в именах файлов, чтобы заменить используемый по умолчанию <span class="blue">,</span>',
  ],
  _高亮关注的用户: [
    '<span class="key">高亮</span>关注的用户',
    '<span class="key">高亮</span>關注的使用者',
    '<span class="key">Highlight</span> following users',
    'フォローしているユーザーを強調表示する',
    '다음 사용자 <span class="key">강조표시</span>',
    '<span class="key">Выделить</span> следующих пользователей',
  ],
  _高亮关注的用户的说明: [
    '你关注（Following）的用户的名字会具有黄色背景，或者显示为黄色。<br>这便于你确认自己是否关注了某个用户。',
    '你關注（Following）的使用者的名字會具有黃色背景，或者顯示為黃色。<br>這便於你確認自己是否關注了某個使用者。',
    'The names of users you are following will have a yellow background, or be displayed in yellow. <br>This is convenient for you to confirm whether you follow a certain user.',
    'フォローしているユーザーの名前は背景が黄色、または黄色で表示されます。 <br>特定のユーザーをフォローしているかどうかを確認するのに便利です。',
    '팔로우하는 사용자의 이름은 노란색 배경으로 표시되거나 노란색으로 표시됩니다. <br>특정 사용자를 팔로우하고 있는지 확인할 때 편리합니다.',
    'Имена пользователей, на которых вы подписаны, будут иметь желтый фон или отображаться желтым цветом. <br>Это удобно для вас, чтобы подтвердить, подписаны ли вы на определенного пользователя',
  ],
  _正在加载关注用户列表: [
    '正在加载关注用户列表',
    '正在載入關注使用者列表',
    'Loading list of followed users',
    'フォローしているユーザーのリストを読み込み中',
    '팔로우한 사용자 목록 로드 중',
    'Загрузка списка отслеживаемых пользователей',
  ],
  _Kiwi浏览器可能不能建立文件夹的bug: [
    '如果你使用的是 Kiwi 浏览器，它可能不会建立文件夹。这是 Kiwi 浏览器的 bug。',
    '如果你使用的是 Kiwi 瀏覽器，它可能不會建立資料夾。這是 Kiwi 瀏覽器的 bug。',
    'If you are using the Kiwi browser, it may not create the folder. This is a bug in the Kiwi browser',
    'Kiwi ブラウザを使用している場合、フォルダが作成されない場合があります。 これは Kiwi ブラウザのバグです。',
    '키위 브라우저를 사용하는 경우 폴더가 생성되지 않을 수 있습니다. 이것은 Kiwi 브라우저의 버그입니다.',
    'Если вы используете браузер Kiwi, он может не создать папку. Это баг браузера Киви.',
  ],
  _优化移动设备上的用户体验: [
    '优化移动设备上的用户体验。',
    '最佳化移動裝置上的使用者體驗。',
    'Optimize user experience on mobile devices.',
    'モバイルデバイスでのユーザーエクスペリエンスを最適化します。',
    '모바일 장치에서 사용자 경험을 최적화합니다.',
    'Оптимизируйте взаимодействие с пользователем на мобильных устройствах.',
  ],
  _导出CSV文件的提示: [
    'CSV 文件的可读性更好，但它不能用于恢复（导入）抓取结果。',
    'CSV 檔案的可讀性更好，但它不能用於恢復（匯入）抓取結果。',
    'A CSV file is more readable, but it cannot be used to restore (import) crawl results.',
    'CSV ファイルは読みやすいですが、クロール結果の復元 (インポート) には使用できません。',
    'CSV 파일은 더 읽기 쉽지만 크롤링 결과를 복원(가져오기)하는 데 사용할 수 없습니다.',
    'CSV-файл более удобен для чтения, но его нельзя использовать для восстановления (импорта) результатов сканирования.',
  ],
  _批量收藏作品时减慢速度: [
    '批量收藏作品时减慢速度，以减少 429 错误发生的概率',
    '批量收藏作品时减慢速度，以减少 429 错误发生的概率',
    'Slow down when batch bookmarking works to reduce chance of 429 errors',
    'バッチブックマークが機能すると、429 エラーの可能性を減らすために速度が低下します。',
    '429 오류 가능성을 줄이기 위해 일괄 북마크가 작동할 때 속도를 늦춥니다.',
    'Замедлите работу, когда пакетная закладка работает, чтобы уменьшить вероятность ошибки 429',
  ],
  _停止抓取: [
    '停止抓取',
    '停止擷取',
    'Stop crawling',
    'クロールをやめる',
    '크롤링 중지',
    'Остановить сканирование',
  ],
  _已停止抓取: [
    '已停止抓取',
    '已停止擷取',
    'Crawl stopped',
    'クロールを停止しました',
    '크롤링 중지됨',
    'Сканирование остановлено',
  ],
  _导入ID列表: [
    '导入 ID 列表',
    '匯入 ID 列表',
    'Import ID list',
    'インポートIDリスト',
    'ID 목록 가져오기',
    'Список идентификаторов импорта',
  ],
  _导出ID列表: [
    '获取 ID 列表完毕后导出列表，并停止抓取',
    '獲取 ID 列表完畢後匯出列表，並停止抓取',
    'Export the ID list after fetching it, and stop crawling',
    'IDリストを取得後にエクスポートし、クロールを停止します',
    'ID 목록을 가져온 후 내보내기 및 크롤링 중지',
    'Экспортируйте список идентификаторов после его извлечения и остановите сканирование.',
  ],
  _导入的用户ID数量: [
    '导入的用户 ID 数量：',
    '匯入的使用者 ID 數量：',
    'Number of user IDs imported: ',
    'インポートされたユーザー ID の数: ',
    '가져온 사용자 ID 수: ',
    'Количество импортированных идентификаторов пользователей:',
  ],
  _任务已中止: [
    '任务已中止',
    '任務已中止',
    'task aborted',
    'タスクが中止されました',
    '작업이 중단됨',
    'задача прервана',
  ],
  _新增的关注用户达到每日限制: [
    '新增的关注用户数量达到 {}， 下载器已中止任务，以免你的账号被 Pixiv 限制。<br>建议明天再执行此任务。',
    '新增的關注使用者數量達到 {}， 下載器已中止任務，以免你的賬號被 Pixiv 限制。<br>建議明天再執行此任務。',
    'The number of newly added followers has reached {}, the downloader has stopped the task to prevent your account from being restricted by Pixiv. <br>It is recommended to perform this task again tomorrow.',
    '新しく追加されたフォロワーの数が {} に達しました。あなたのアカウントが Pixiv によって制限されるのを防ぐために、ダウンローダーはタスクを停止しました。 <br>このタスクは明日もう一度実行することをお勧めします。',
    '새로 추가된 팔로워 수가 {}에 도달했습니다. 다운로더가 작업을 중지하여 Pixiv에서 귀하의 계정을 제한하지 않도록 했습니다. <br>내일 이 작업을 다시 수행하는 것이 좋습니다.',
    'Количество новых подписчиков достигло {}, загрузчик остановил задачу, чтобы предотвратить ограничение вашей учетной записи Pixiv. <br>Рекомендуется повторить это задание завтра.',
  ],
  _没有找到关注按钮的提示: [
    '跳过关注用户 {} 因为没有找到关注按钮。你可以手动关注此用户。再次执行此任务有可能解决此问题。',
    '跳過關注使用者 {} 因為沒有找到關注按鈕。你可以手動關注此使用者。再次執行此任務有可能解決此問題。',
    'Skip following user {} because no follow button was found. You can follow this user manually. Performing this task again may resolve the issue.',
    'フォロー ボタンが見つからなかったため、ユーザー {} のフォローをスキップします。このユーザーを手動でフォローできます。 このタスクを再度実行すると、問題が解決される可能性があります。',
    '팔로우 버튼을 찾을 수 없으므로 사용자 {} 팔로우를 건너뜁니다. 이 사용자를 수동으로 팔로우할 수 있습니다. 이 작업을 다시 수행하면 문제가 해결될 수 있습니다.',
    'Пропустить подписку на пользователя {}, поскольку кнопка подписки не найдена. Вы можете подписаться на этого пользователя вручную. Повторное выполнение этой задачи может решить проблему.',
  ],
  _你的账号已经被Pixiv限制: [
    '你的账号已经被 Pixiv 限制',
    '你的賬號已經被 Pixiv 限制',
    'Your account has been restricted by Pixiv',
    'あなたのアカウントはPixivによって制限されています',
    '귀하의 계정은 Pixiv에 의해 제한되었습니다.',
    'Ваша учетная запись была ограничена Pixiv',
  ],
  _模拟用户点击: [
    '下载器发送的 API 返回 400 错误（需要 recaptcha enterprise score token），切换到模拟用户点击的方式，这会使用较多的硬件资源。',
    '下載器傳送的 API 返回 400 錯誤（需要 recaptcha enterprise score token），切換到模擬使用者點選的方式，這會使用較多的硬體資源。',
    'The API sent by the downloader returns a 400 error (recaptcha enterprise score token is required), and switches to the method of simulating user clicks, which will use more hardware resources.',
    'ダウンローダーによって送信された API は 400 エラー (recaptcha enterprise score token が必要です) を返し、より多くのハードウェア リソースを使用するユーザーのクリックをシミュレートする方法に切り替わります。',
    '다운로더가 보낸 API는 400 오류(recaptcha enterprise score token 필요)를 반환하고 더 많은 하드웨어 리소스를 사용하는 사용자 클릭 시뮬레이션 방법으로 전환합니다.',
    'API, отправленный загрузчиком, возвращает ошибку 400 (требуется recaptcha enterprise score token) и переключается на метод имитации пользовательских кликов, который будет использовать больше аппаратных ресурсов.',
  ],
  _提示可以重新执行批量关注任务: [
    '如果该标签页失去响应，或者关注的用户有遗漏，请关闭标签页，再重新打开，重新执行此任务。',
    '如果該標籤頁失去響應，或者關注的使用者有遺漏，請關閉標籤頁，再重新開啟，重新執行此任務。',
    'If the tab becomes unresponsive, or if you miss a follower, close the tab, reopen it, and redo the task.',
    'タブが応答しなくなった場合、またはフォロワーを見逃した場合は、タブを閉じて再度開き、タスクをやり直してください。',
    '탭이 응답하지 않거나 팔로어를 놓친 경우 탭을 닫았다가 다시 열고 작업을 다시 실행하십시오.',
    'Если вкладка перестает отвечать на запросы или вы пропустили подписчика, закройте вкладку, снова откройте ее и повторите задачу.',
  ],
  _新增x个: [
    '新增 {} 个',
    '新增 {} 個',
    'Added {}',
    '追加した {}',
    '추가됨 {}',
    'Добавлен {}',
  ],
  _优化批量关注用户的功能: [
    '优化批量关注用户的功能',
    '最佳化批次關注使用者的功能',
    'Optimize the function of following users in batches',
    'ユーザーの一括フォロー機能を最適化',
    '사용자 일괄 팔로우 기능 최적화',
    'Оптимизируйте функцию подписки на пользователей в пакетном режиме.',
  ],
  _修复了显示更大的缩略图失效的问题: [
    '修复了“显示更大的缩略图”失效的问题',
    '修復了“顯示更大的縮圖”失效的問題',
    'Fixed an issue where "Show Larger Thumbnails" didn\'t work',
    '「大きなサムネイルを表示」が機能しない問題を修正しました',
    '"큰 축소판 보기"가 작동하지 않는 문제를 수정했습니다.',
    'Исправлена ​​ошибка, из-за которой не работал параметр «Показать увеличенные эскизы».',
  ],
  _可能发生了错误请刷新页面重试: [
    '可能发生了错误。<br>如果下载进度卡住，请刷新页面重试，或者重启浏览器。',
    '可能發生了錯誤。<br>如果下載進度卡住，請重新整理頁面重試，或者重啟瀏覽器。',
    'An error may have occurred. <br>If the download progress is stuck, please refresh the page and try again, or restart the browser.',
    'エラーが発生した可能性があります。 <br>ダウンロードの進行が進まない場合は、ページを更新して再試行するか、ブラウザを再起動してください。',
    '오류가 발생했을 수 있습니다. <br>다운로드 진행이 중단되면 페이지를 새로 고친 후 다시 시도하거나 브라우저를 다시 시작하세요.',
    'Возможно, произошла ошибка. <br>Если процесс загрузки завис, обновите страницу и повторите попытку или перезапустите браузер.',
  ],
  _在多图作品页面里显示缩略图列表: [
    '在多图作品页面里显示<span class="key">缩略图</span>列表',
    '在多圖作品頁面裡顯示<span class="key">縮圖</span>列表',
    'In the multi-image work page, display the <span class="key">thumbnail</span> list',
    '複数画像作品ページでサムネイル一覧を表示',
    '멀티 이미지 작품 페이지에서 썸네일 목록을 표시',
    'На рабочей странице с несколькими изображениями отобразите список эскизов',
  ],
  _提交: ['提交', '提交', 'Submit', '提出する', '제출하다', 'Подавать'],
  _已导出被删除的作品的ID列表: [
    '已导出被删除的作品的 ID 列表',
    '已匯出被刪除的作品的 ID 列表',
    'List of IDs of deleted works exported',
    'エクスポートされた削除作品のID一覧',
    '내보낸 삭제된 작품의 ID 목록',
    'Список идентификаторов удаленных работ экспортирован',
  ],
  _在收藏页面里提示有辅助功能可用: [
    '在你的收藏页面里，下载器的“更多”标签页里有一些功能可以帮助管理你的收藏。',
    '在你的收藏頁面裡，下載器的“更多”標籤頁裡有一些功能可以幫助管理你的收藏。',
    `On your bookmarks page, the Downloader's "More" tab has some features to help you manage your bookmarks.`,
    'ブックマーク ページのダウンローダーの [その他] タブには、ブックマークの管理に役立つ機能がいくつかあります。',
    '북마크 페이지에서 다운로더의 "더보기" 탭에는 북마크를 관리하는 데 도움이 되는 몇 가지 기능이 있습니다.',
    'На странице закладок на вкладке «Дополнительно» Downloader есть некоторые функции, которые помогут вам управлять своими закладками.',
  ],
  _预览作品的详细信息: [
    '预览作品的<span class="key">详细</span>信息',
    '預覽作品時的<span class="key">詳細</span>資料',
    'Preview the <span class="key">details</span> of the work',
    'プレビュー作品の詳細です',
    '작품의 상세한 정보를 미리보다',
    'Подробности предварительного показа',
  ],
  _预览作品的详细信息的说明: [
    '鼠标放在作品缩略图上即可查看作品数据',
    '滑鼠放在作品縮圖上即可檢視作品資料',
    'Mouse over the thumbnail of the work to view the work data',
    '作品のサムネイルにマウスをかざすだけで作品データを見ることができます',
    '마우스를 작품 썸네일 위에 놓으면 작품 데이터를 볼 수 있다',
    'Данные о работе можно увидеть с помощью мыши на сокращённом графике',
  ],
  _显示区域宽度: [
    '显示区域宽度',
    '顯示區域寬度',
    'Display area width',
    '表示領域幅です',
    '영역 너비 보이기',
    'Покажи ширину зоны',
  ],
  _写入剪贴板失败: [
    '写入剪贴板失败',
    '寫入剪貼簿失敗',
    'Writing to clipboard failed',
    'クリップボードへの書き込みに失敗しました',
    '클립보드에 쓰지 못했습니다.',
    'Запись в буфер обмена не удалась',
  ],
}

export { langText }
